/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: fvr_if.h,v $
 * $Revision: #3 $
 *
 *---------------------------------------------------------------------------*/

/** @file fvr_if.h
 *  FVR driver - FVR Interface Header
 */

#ifndef _TLV_FVR_IF_H
#define _TLV_FVR_IF_H

#include "tlv_if.h"
#include "dmx_mm_if.h"
#include "fvr_if.h"

#define TLV_FVR_GBL_COUNT                       (4)

typedef struct
{
    UINT16 u2TargetPktCount;
    BOOL fgAllocateBuffer;
    UPTR u4BufStart;
    USIZE u4BufSize;
    UPTR u4Wp;
    UPTR u4Rp;

    BOOL fgSetAlign;
    UINT32 u4BufAlign;

    PFN_FVR_GBL_NOTIFY pfnNotifyFunc;
    void* pvNotifyTag;
   
    UINT32 u4TimeToNotify;
} TLV_FVR_GBL_T;

//------------------------------------------------------------
// Macro
//------------------------------------------------------------
#define GET_BYTE(arg, n)                ((UINT8*)&(arg))[n]

EXTERN BOOL TLV_FVR_Start(UINT8 u1Idx);
EXTERN BOOL TLV_FVR_Stop(UINT8 u1Idx);

EXTERN BOOL TLV_FVR_SetGBL(UINT8 u1Idx, UINT32 u4Flags, const FVR_GBL_T * prGBL);
EXTERN BOOL TLV_FVR_GetGBL(UINT8 u1Idx, UINT32 u4Flags, FVR_GBL_T *prGBL);
EXTERN UINT8 TLV_FVR_GetAvailableVirIpFilterIdx(UINT8 u1TlvIdx);
EXTERN BOOL TLV_FVR_FreeVirIpFilterIdx(UINT8 u1VirFilterIdx);
EXTERN BOOL TLV_FVR_SetIpFilter(UINT8 u1FilterIdx,  const DMX_TLV_IP_FILTER_T *prIpFilter);
EXTERN BOOL TLV_FVR_SetPid(UINT8 u1Pidx, UINT32 u4Flags, const FVR_PID_T* prPid);
EXTERN BOOL TLV_FVR_GetPid(UINT8 u1Pidx, UINT32 u4Flags, FVR_PID_T *prPid);
EXTERN BOOL TLV_FVR_FreePid(UINT8 u1Pidx);
EXTERN BOOL TLV_FVR_SetVideoTypeByPidx(UINT8 u1Pidx, FVR_VIDEO_TYPE_T eVideoType);
EXTERN BOOL TLV_FVR_SetAesKey(UINT8 u1Idx, BOOL fgEven, UINT8 *pu1Key);
EXTERN BOOL TLV_FVR_SetAesEvenOddIV(UINT8 u1Idx, BOOL fgEven, UINT32 *pu4IV);

EXTERN BOOL TLV_FVR_SetNtpIpFilter(UINT8 u1TsIdx,  UINT8 u1Pidx, const DMX_TLV_IP_FILTER_T* prIpFilter);

EXTERN BOOL TLV_FVR_SwapSetMptPattern(UINT8 u1Pidx, UINT8* pu1Pattern, UINT16 u2Len);
EXTERN BOOL TLV_FVR_SwapFreePattern(UINT8 u1Pidx);

EXTERN BOOL TLV_FVR_SwapSetPattern(UINT8 u1Pidx, UINT8* pu1Pattern, UINT16 u2Len);

EXTERN BOOL TLV_FVR_SwapSetPattern(UINT8 u1Pidx, UINT8* pu1Pattern, UINT16 u2Len);

EXTERN BOOL TLV_FVR_GetBufferInfo(UINT8 u1Idx, UPTR *pu4BufStart, UPTR *pu4BufEnd, UPTR *pu4Rp, UPTR *pu4Wp, UINT32 *pu4DataSize);
EXTERN BOOL TLV_FVR_UpdateReadPointer(UINT8 u1Idx, UPTR u4Rp);
EXTERN BOOL TLV_FVR_UpdateWritePointer(UINT8 u1Idx, UPTR u4Rp);
EXTERN BOOL TLV_FVR_UnlockBuffer(UINT8 u1Idx, UPTR u4Addr, USIZE u4Size);
EXTERN BOOL TLV_FVR_FlushBuffer(UINT8 u1Idx);


#endif  // FVR_IF_H

