/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: chi.yu $
 * $Date: 2015/08/07 $
 * $RCSfile: b2r_drvif.h,v $
 * $Revision: #4 $
 *
 *---------------------------------------------------------------------------*/

/** @file vdp_if.h
 *  Video plane interface, including init, enable,
 *  mode selection and region desciption.
 */

#ifndef B2R_DRVIF_H
#define B2R_DRVIF_H

#include "x_typedef.h"
#include "stc_drvif.h"
#include "fbm_drvif.h"
#include "b2r_if.h"
#include "vdp_if.h"
#include "mute_if.h"

#define _B2R_SetDPEarlyDisp(o,value)      \
    do \
    { \
        UINT32 i; \
        VDP_CFG_T *ptDP; \
        for (i = 0; i < o->u4VideoPanelNs; i++) \
        { \
            ptDP = (VDP_CFG_T *)o->avVideoPanel[i]; \
            if (ptDP) ptDP->ucEarlyDisp = value; \
        } \
    }while (0); 

#define LOG_REL(FLAG, REL_LVL, DBG_LVL, MSG...)     \
    if (FLAG) { LOG(REL_LVL, MSG); } else { LOG(DBG_LVL, MSG); }

#define _B2R_GetDP(o)                   ((VDP_CFG_T *)(o)->avVideoPanel[0])

#define VDP2B2RID(ucVdpId)              (_prVdpCfg[ucVdpId]->ucB2rId) 

#define B2R_MUTEX_LOCK(id)              _B2R_VsyncMutexLock(id)
#define B2R_MUTEX_UNLOCK(id)            _B2R_VsyncMutexUnlock(id)

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
#define PD_32_FLAG_SEQ                  (4)
// after field trail (ch48_f49_0530_signal84_V_Shake)
#define PD_32_FLAG_LOOP_SIZE            (12)                        // 0.5 second
#define PD_32_FLAG_LEAVE_LOOP_SIZE      (6)                         // 0.25 second
#define VDP_STC_CLOCK                   (90000)                     // 90 KHz
#define VDP_STC_CLOCK_16M               (0x1000000)                 // 16M

// 0~(VDEC_MAX_INST - 1) for MPEG
// VDEC_MAX_INST for non-MPEG main(such as sw decoder)
// VDEC_MAX_INST + 1 for non-MPEG sub(such as sw decoder)
#define NON_HW_ES_ID_MAIN        ((VDEC_MAX_INST) + 0)
#define NON_HW_ES_ID_SUB         ((VDEC_MAX_INST) + 1)
#define MAX_ES_NS                ((VDEC_MAX_INST) + 2)

#define B2R_MAX_INPORT_NS           1    ///< Maximum number of input port for each video plane.

// Trick Play at 20081001
#define VDP_PLAY_NORMAL 0 ///< Normal Play
#define VDP_PLAY_TRICK 1 ///< Trick Play
#define VDP_PLAY_STEP_FORWARD 2 ///< Step Forward
#define VDP_PLAY_SLOW_FORWARD 3 ///< Slow Forward

// B2R Video plane mode.
#define VDP_MODE_NORMAL             0    ///< normal mode, video out
#define VDP_MODE_BG                 1    ///< background mode, mute
#define VDP_MODE_QV                 2    ///< obsolete
#define VDP_MODE_DEINT              3    ///< obsolete
#define VDP_MODE_UNKNOWN            4

// Maximum number of B2R module
#define B2R_NS                      4// capy have 4 B2R HW/ husky and boar have 2
#define B2R_1                       0
#define B2R_2                       1
#define B2R_3                       2//// New B2R HD
#define B2R_4                       3//// New B2R HD

#define B2R_INVALID                 0xFF

// Video plane status
#define VDP_STATUS_NOSIGNAL         0
#define VDP_STATUS_NOSUPPORT        1
#define VDP_STATUS_UNKNOWN          2
#define VDP_STATUS_STABLE           3

// Aspect Ratio
#define VDP_UNIT_ASPECT_RATIO       0x10000
#define VDP_4_3_ASPECT_RATIO        ((B2R_UNIT_ASPECT_RATIO * 4) / 3)
#define VDP_16_9_ASPECT_RATIO       ((B2R_UNIT_ASPECT_RATIO * 16) / 9)
#define VDP_221_1_ASPECT_RATIO      ((B2R_UNIT_ASPECT_RATIO * 221) / 100)

// B2R Mode
#define VDP_B2R_MODE_UNKNOWN        0
//i
#define VDP_B2R_MODE_480I_60        1
#define VDP_B2R_MODE_480I_120       2

#define VDP_B2R_MODE_576I_50        3
#define VDP_B2R_MODE_576I_100       4

#define VDP_B2R_MODE_1080I_48		5
#define VDP_B2R_MODE_1080I_50       6
#define VDP_B2R_MODE_1080I_60       7
#define VDP_B2R_MODE_1080I_100      8
#define VDP_B2R_MODE_1080I_120      9


//p
#define VDP_B2R_MODE_480P_24        10
#define VDP_B2R_MODE_480P_30        11
#define VDP_B2R_MODE_480P_48        12
#define VDP_B2R_MODE_480P_60        13
#define VDP_B2R_MODE_480P_120       14

#define VDP_B2R_MODE_576P_25        15
#define VDP_B2R_MODE_576P_50        16
#define VDP_B2R_MODE_576P_100       17

#define VDP_B2R_MODE_720P_24        18
#define VDP_B2R_MODE_720P_25        19
#define VDP_B2R_MODE_720P_30        20
#define VDP_B2R_MODE_720P_48        21
#define VDP_B2R_MODE_720P_50        22
#define VDP_B2R_MODE_720P_60        23
#define VDP_B2R_MODE_720P_100       24
#define VDP_B2R_MODE_720P_120       25

#define VDP_B2R_MODE_1080P_24       26
#define VDP_B2R_MODE_1080P_25       27
#define VDP_B2R_MODE_1080P_30       28
#define VDP_B2R_MODE_1080P_48       29
#define VDP_B2R_MODE_1080P_50       30
#define VDP_B2R_MODE_1080P_60       31
#define VDP_B2R_MODE_1080P_120      32


//non standard
#define VDP_B2R_MODE_480P_1440W_24       33
#define VDP_B2R_MODE_480P_1440W_60       34

#define VDP_B2R_MODE_1440P_1280W_24       35
#define VDP_B2R_MODE_1440P_1280W_25       36
#define VDP_B2R_MODE_1440P_1280W_30       37
#define VDP_B2R_MODE_1440P_1280W_50       38
#define VDP_B2R_MODE_1440P_1280W_60       39

#define VDP_B2R_MODE_720P_1080W_24		40  //33~37 add for CR DTV00351047 when mm video is 1024*768(tab)and 1080*540(sbs) => v_total too large to do MJC 3D in mt5396
#define VDP_B2R_MODE_720P_1080W_25		41
#define VDP_B2R_MODE_720P_1080W_30		42
#define VDP_B2R_MODE_720P_1080W_50		43
#define VDP_B2R_MODE_720P_1080W_60		44

#define VDP_B2R_MODE_1080P_ULTRAW_24       45
#define VDP_B2R_MODE_1080P_ULTRAW_25       46
#define VDP_B2R_MODE_1080P_ULTRAW_30       47
#define VDP_B2R_MODE_1080P_ULTRAW_50       48

#define VDP_B2R_MODE_480P_12		49
#define VDP_B2R_MODE_480P_15		50
#define VDP_B2R_MODE_480P_20		51

#define VDP_B2R_MODE_576P_12		52
#define VDP_B2R_MODE_576P_15		53
#define VDP_B2R_MODE_576P_20		54
#define VDP_B2R_MODE_576P_60		55

#define VDP_B2R_MODE_720P_12		56
#define VDP_B2R_MODE_720P_15		57
#define VDP_B2R_MODE_720P_20		58

#define VDP_B2R_MODE_1080P_12		59
#define VDP_B2R_MODE_1080P_15		60
#define VDP_B2R_MODE_1080P_20		61

#define VDP_B2R_MODE_MAX            70

// Video Call Back Message Type
#define VDP_MSG_NOTIFY              0
#define VDP_MSG_CC_CB               1
#define VDP_MSG_AFD_CB              2
#define VDP_MSG_UNMUTE_CB           3
#define VDP_MSG_MM_CB               4
#define VDP_MSG_LIPSYNC_CB          5
#define VDP_MSG_MM_STEP_FIN_CB      6
#define VDP_MSG_MM_SEEK_FIN_CB      7
#define VDP_MSG_REPEAT_DROP_CB      8
#define VDP_MSG_LIPSYNC_OK_TO_NG_CB 9
#define VDP_MSG_LIPSYNC_NG_TO_OK_CB 10
#define VDP_MSG_GET_PIC_TYPE_CB     11
#define VDP_MSG_PLAY_DONE_CB		12
#define VDP_MSG_RES_CHG_CB			13
#define VDP_MSG_TRIG_AUD_TO_PLAY    14
#define VDP_MSG_MM_COMM_CB          15
#define VDP_MSG_MM_PTS_CB           16


//B2R clock mode
#define VDP_B2R_CLK_UNKNOWN         0
#define VDP_B2R_CLK_27_000          1   // 27.000 MHz: for 23.97Hz, 29.97Hz, 59.94Hz, 25Hz, 50Hz
#define VDP_B2R_CLK_27_027          2   // 27.027 MHz: for 24Hz, 30Hz, 60Hz
#define VDP_B2R_CLK_74_176          3   // 75.176 MHz: for 23.97Hz, 29.97Hz, 59.94Hz
#define VDP_B2R_CLK_74_250          4   // 75.250 MHz: for 24Hz, 25Hz, 30Hz, 50Hz, 60Hz
#define VDP_B2R_CLK_148_352         5   // 148.352 MHz: for 23.97Hz, 29.97Hz, 59.94Hz
#define VDP_B2R_CLK_148_500         6   // 148.500 MHz: for 24Hz, 25Hz, 30Hz, 50Hz, 60Hz
#define VDP_B2R_CLK_54_000          7
#define VDP_B2R_CLK_54_054          8
#define VDP_B2R_CLK_10_368			9
#define VDP_B2R_CLK_21_996			10
#define VDP_B2R_CLK_37_125			11
#define VDP_B2R_CLK_08_100			12
#define VDP_B2R_CLK_10_800          13
#define VDP_B2R_CLK_32_400          14
#define VDP_B2R_CLK_297_000         15   // 297000 MHZ for 4K2K 30FPS
#define VDP_B2R_CLK_064_800         16




// Video plane status
#define VDP_B2R_SOURCE_CHG          0
#define VDP_B2R_OUTPUT_CHG          1
#define VDP_B2R_START_PLAY          2
#define VDP_B2R_RESOLUTION_NFY      3
#define VDP_B2R_NO_SIGNAL           4
 

// Video Plane COMMAND (Max 256)
#define VDP_CMD_RST                 0
#define VDP_CMD_SET_ENABLE          1
#define VDP_CMD_SET_MODE            2
#define VDP_CMD_SET_INPUT           3
#define VDP_CMD_SET_OUTPUT          4
#define VDP_CMD_SET_SRCR            5
#define VDP_CMD_SET_OUTR            6
#define VDP_CMD_SET_BG              7
#define VDP_CMD_SET_SRCSIZE         8
#define VDP_CMD_MAX                 9

//B2R Enable Status
#define B2R_ENABLE                  1
#define B2R_DISABLE                 0

#undef  _INFO_HEADER
#undef  _API_HEADER
#undef  _ERROR_HEADER
#define _INFO_HEADER        "{B2R}Info:"
#define _API_HEADER         "{B2R}API: "
#define _ERROR_HEADER       "{B2R}Err: "

#define B2R_8BIT_MASK(x)            ((x) & 0xFF)
#define B2R_16BIT_MASK(x)           ((x) & 0xFFFF)


/*  B2R Notify Conditions for MM*/
typedef enum
{
    VDP_COND_ERROR = -1,
    VDP_COND_EOS,
    VDP_COND_SYNC_POINT,
    VDP_COND_RANGE_AB_DONE,
    VDP_COND_CHAP_DONE = 5
}  VDP_COND_T;

typedef enum
{
	ADDR_MODE_64_32,
	ADDR_MODE_16_32,
	ADDR_MODE_RASTER
}ADDR_MODE_T;

// Video Call Back Message Q
typedef struct
{
    UINT32 u4Type;
    UINT32 u4Arg1;
    UINT32 u4Arg2;
    UINT32 u4Arg3;
} VDP_CB_MSG_T;

/**
 *  PTS Reach Notify Function Type
 */
typedef void (*PTS_CB_FUNC)(UINT32 u4VdpId, UINT32 u4Pts, UINT32 u4Arg);

/**
 *  AFD/Aspect Ratio Change Notify Function Type
 */
typedef void (*AFD_CFG_CB_FUNC)(UINT32 u4VdpId, UINT32 u4Afd, FBM_ASPECT_RATIO_T* rAspectRatio);

/**
 *  Video Play Done Notify Function Type
 */
typedef void (*PLAYDONE_CFG_CB_FUNC)(UINT32 u4VdpId, UINT32 u4Arg1, UINT32 u4Arg2);

/**
 *  UnMute Status Change Notify Function Type
 */
typedef void (*UNMUTE_CFG_CB_FUNC)(UINT32 u4VdpId, BOOL fgUnMute);


/**
 *  Video Plane Notify Function Type for MM
 */
typedef void (*VDP_MM_CB_FUNC)(UINT32 u4VdpId, VDP_COND_T eCond, BOOL fgFastForward, UINT32 u4AttachedSrcId);


/**
 *  Video Plane Lip Sync Notify
 */
typedef void (*VDP_LIPSYNC_CB_FUNC)(void);

typedef void (*B2R_CB_COMM_FUNC)(UINT32 u4VdpId, UINT64 u4Data, UPTR u4FuncPtr, UINT64 u4Tag);

#define VDP_SEEK_STEP_OK         0
#define VDP_SEEK_STEP_NO_DATA    1
/**
 *  Video Plane Step Forward Finish Notify
 */
typedef void (*VDP_STEP_FIN_CB_FUNC)(UINT32 u4VdpId, UINT32 u4Pts, UINT32 u4Ret, UINT32 u4AttachedSrcId);


/**
 *  Video Plane Seek Finish Notify
 */
typedef void (*VDP_SEEK_FIN_CB_FUNC)(UINT32 u4AttachedSrcId, UINT32 u4VdpId, UINT32 u4Pts, UINT32 u4Ret);


/**
 *  Video Plane Repeat/Drop Notify
 */
typedef void (*VDP_REPEAT_DROP_CB_FUNC)(UINT32 u4VdpId, UINT32 u4RepeatDrop, UCHAR ucEsId);

typedef void (*VDP_RES_CHG_CB_FUNC)(UINT32 u4VdpId);



//from vdp_hal.c
typedef struct
{
    UCHAR ucTrickPlay; /* Trick Play at 20081001 */
    UCHAR ucStepForward; /* Step Forward at 20081030 */
    UCHAR ucSlowForward; /* Slow Forward at 20090506 */
    UINT32 u4StepCnt;
} B2R_INFO_T;

/** Brief of VDP_STC_T
 */
typedef struct
{
    BOOL fgEnable;
    BOOL fgDiv10;
    UINT32 u4Speed;
    UINT32 u4VSyncPeriod;
    UINT32 u4B2rStcVal;
} B2R_STC_T;

enum
{
	MIRROR_ON,
	MIRROR_OFF,
	INVALID_MIRROR_STATUS
};

typedef enum
{
	PATH_DEFAULT = 0,
	PATH_BYPASS,
	PATH_DUOMUX,
	PATH_COMBINE
}B2R_CONFIG_PATH;

typedef struct _VDP_FREEZE_PRM_T
{
    UINT32 u4OnOff;
    UINT32 u4UnFreeze;
} VDP_FREEZE_PRM_T;

//integrate local variable 
typedef struct 
{
    B2R_INFO_T rB2rData;
    B2R_STC_T rB2rStc;
    UCHAR ucMirrorStatus;
    UINT32 u4RepeatCntForPushTrickMode;
    UINT32 u4ElapseCntForPushTrickMode;
    BOOL fgModeChangeSent;
    UINT32 u4LastWidth;
    UINT32 u4LastHeight;
    UINT32 u4OriSrcWidth;
    UINT32 u4OriSrcHeight;
    BOOL fgRepeatFrame;
    BOOL fgDropFrame;
    //UINT32 u4PreviousResizeWidth;
    //UINT32 u4PreviousResizeHeight;
    //UINT32 u4PreviousResizeWidth2;
    //UINT32 u4PreviousResizeHeight2;
    UINT32 u4DriftCount;
    UINT32 u4MaxDriftCount;
    UINT32 u4ValidCount;
    BOOL fgB2RProfile;
    UCHAR ucRemainTimeToSendSeekDone;
    BOOL fgTimeShfitThumbStartPlay;
    UINT32 u4TimeShfitChangeFrmCnt;
    BOOL fgNPTVStable;
    UINT32 u4WaitNPTVStableCount;
    UINT8 u1PlayMode;
    UINT16 u2VsyncNotStableCount;
    UCHAR ucFrameRateUnkownCount;
    BOOL fgHandleSeqChg;
    BOOL fgEnableWaitSTC;
    FBM_ASPECT_RATIO_T rFbmAspectRatio;
    UINT32        u4ReturnValue;

    BOOL  fgDropFb;
    BOOL  fgGstPlayBack;
    BOOL  fgPauseMM ;
    BOOL  fg2x;
    BOOL  fgTunnel;
    BOOL  fgTimeShift;
    BOOL  fgFirstFrame;
    BOOL  fgPendingForSTC;
    BOOL  fgPendingChgFrmMsg;
    BOOL  fgRstStc;
    BOOL  fgAvSyncSkipBufReady;
    VDP_B2R_CHG_FRAME_MSG_T rPendingChgFrmMsg;
    VDP_TRIG_AUD_PRM_T rVdpTrigAud;
    STC_SPEED_TYPE_T eSpeed;
    VDP_SEEK_PRM_T rVdpSeek;
    INT32 i4TimeShiftSpeed;
    

    // LipSync Call Back
    UINT32 u4CbSyncEn;
    UINT32 u4CbSyncThrsd;
    UINT32 u4CbSyncChkFrmCnt;
    UINT32 u4CbSyncFrmCnt;
    UINT32 u4CbSyncDone;

    // PTS Call Back
    UINT32 u4CbPtsEn;
    UINT32 u4CbPts;
    UINT32 u4CbArg;
    UINT32 u4CbCntDown;
    UINT32 u4CbLastPts;
    UINT32 u4SeamlessMode;

    //--for AB repeat
    VDP_AB_INTO_T rAB;

    // for sequence change speedup
    BOOL fgSeqChgSpeedup;

    // for double clock
    BOOL fgDoubleClock;

    VDP_LOCK_PIC_INFO_T arLockBuf[VDP_MAX_LOCK_BUF_NS]; 
    BOOL fgEnableVideoLog;
    BOOL fgEnableGstLog;
    BOOL fgGameMode;
    BOOL fgOmxEos;
    BOOL fgSkipMode;
    BOOL fgDoubleFrmMode;
    BOOL fg4096Crop;
    FBM_DOLBY_TYPE_T eDvType;
    UINT32 u4ResolutionW;
    UINT32 u4ResolutionH;
    UCHAR  ucLastFrmRate;
    BOOL   fgFixMode;
#ifdef CC_SUPPORT_PHOTO8K
    UINT32 u4Playmode;
    UINT32 u4PhotoStateChg;
    UINT32 u4PhotoState;
    UINT32 u4PhotoHRegion;
    UINT32 u4PhotoVRegion;
#endif

    B2R_CONFIG_PATH ePathSel;
} B2R_VAR_T;

typedef struct
{
    //hal part, must be in the first position
    VOID* pvData;
    VOID  (*IsrCb)(VOID *, VOID *);
    VOID* hB2r;

    UCHAR ucB2rId;

    //todo
    UINT32 u4State; //b2r thread State
    UINT32 u4HWSt;  //b2r thread State
    HANDLE_T hMsgQ;
    HANDLE_T hNotifyMsgQ;
    BOOL fgEntry;

    //output panel
    VOID* avVideoPanel[4];
    UINT32 u4VideoPanelNs;

    //sub structure
    B2R_PRM_T *ptB2rPrm;
    B2R_VAR_T *ptB2rVar;
} B2R_OBJECT_T;

///-----------------------------------------------------------------------------
/// Public functions
///-----------------------------------------------------------------------------
B2R_OBJECT_T * _B2R_GetObj(UCHAR ucB2rId);
VOID _B2R_ObjectInit(VOID);

void _VDP_StatusNotify(UCHAR ucVdpId, UINT32 u4Status);

void _VDP_EventNofify(UCHAR ucVdpId, UINT32 u4Arg1, UINT32 u4Arg2, UINT32 u4Arg3);

VOID _B2R_SetTimeShiftMode(UCHAR ucB2rId,BOOL fgEnable);

VOID _B2R_SetPauseMM(UCHAR ucB2rId, BOOL fgPauseMM);

STC_SPEED_TYPE_T _B2R_GetFrcStcSpeed(UCHAR ucB2rId);

UCHAR _B2R_GetWaitSTC(UCHAR ucB2rId);

VOID _B2R_SetABRepeat(UCHAR ucB2rId, UCHAR ucSet);

BOOL _B2R_TriggerAudReceiveEx(UCHAR ucB2rId, BOOL fgEnable, UINT64 u4Pts);

VOID _B2R_SetEnableLog(UCHAR ucB2rId,BOOL fgEnableGstLog, BOOL fgEnableVideoLog);

VOID _B2R_SetAVsync(UCHAR ucVdpId,BOOL fgSkipAvsync);

BOOL _B2R_GetEnableGstLog(UCHAR ucB2rId);
//from vdp_hal.h
void _B2R_SetTrickMode(UCHAR ucB2rId, UCHAR ucTrick);
UCHAR _B2R_GetTrickMode(UCHAR ucB2rId);
void _B2R_SetStepForward(UCHAR ucB2rId, UCHAR ucSet);
UCHAR _B2R_GetStepForward(UCHAR ucB2rId);
void _B2R_SetSlowForward(UCHAR ucB2rId, UCHAR ucSet);
UCHAR _B2R_GetSlowForward(UCHAR ucB2rId);
void _B2R_UpdateStc(B2R_OBJECT_T *this);
void _B2R_StcSpeed(UCHAR ucB2rId,UINT32 u4Speed, BOOL fgDiv10);
void _B2R_StcPeriod(UCHAR ucB2rId,UINT32 u4VSyncPeriod);
void _B2R_EnB2rUpdateStc(UCHAR ucB2rId,BOOL fgEnable);
BOOL _B2R_GetUpdateStc(UCHAR ucB2rId);
void _B2R_SetB2rStc(UCHAR ucB2rId,UINT32 u4B2rStc);
UINT32 _B2R_GetB2rStc(UCHAR ucB2rId);
void _B2R_SetMirrorStatus(UCHAR ucB2rId, BOOL fgOn);
/* whether Update STC when VDP start output or not */
BOOL _B2R_SetPendingForStc(UCHAR ucB2rId, BOOL fgPending);
/* VDP Seek done notification */
VOID _B2R_SetSeek(UCHAR ucB2rId, UCHAR ucSet, UINT32 u4Pts);
VOID _B2R_SetSeekMode(UCHAR ucB2rId, UCHAR ucSet, UINT32 u4Pts);
/* VDP Set Wait STC while starts */
VOID _B2R_SetWaitSTC(UCHAR ucVdpId, UCHAR ucSet);
VOID _B2R_SetTimeShiftSpeed(UCHAR ucB2rId,UINT32 u4SpeedValue);
BOOL _B2R_GetTimeShfitStartPlay(UCHAR ucB2rId);
INT32 _B2R_GetPtsStcDelta(UCHAR ucB2rId);
VOID _B2R_SetTunnel2X(UCHAR ucB2rId, BOOL fg2x);
void _B2R_StatusTest (UCHAR ucB2rId,UINT32 u4AppendId, UINT32 u4All);
void _B2R_DisStatusTest (UCHAR ucB2rId,UINT32 u4AppendId, UINT32 u4All);
void _B2R_BlkPatTest (UCHAR ucB2rId,UINT32 u4AppendId, UINT32 u4All);
void _B2R_GetYCStart(UCHAR ucB2rId,UINT32 *pu4YStart, UINT32 *pu4CStart, UINT32 *pu4LineSize, UINT32 *pu4VActive);
void _B2R_SetBlack(UCHAR ucB2rId, BOOL fgBlack);
void _B2R_ForceBg(UCHAR ucB2rId, UCHAR ucForce);
EXTERN void B2R_SetB2rUpdateStc(UINT8 ucVdpId,BOOL fgEnable);
EXTERN BOOL B2R_GetB2rUpdateStc(UINT8 ucVdpId);
VOID _B2R_SetDropFrame(UCHAR ucB2rId);
VOID _B2R_SetRepeatFrame(UCHAR ucB2rId);
UINT32 _B2R_SetABInfo(UCHAR ucB2rId, VDP_AB_INTO_T *prAB);
VOID _B2R_FrcSetLipSyncCb(UCHAR ucVdpId, UCHAR ucPort, UINT32 u4Thrsd, UINT32 u4FrmCnt);
UINT32 _B2R_CalculateVsyncNs(B2R_OBJECT_T* this);
UINT32 _B2R_PtsCbProc(B2R_OBJECT_T * this, UINT32 u4Status);
void _B2R_PostNotify(B2R_OBJECT_T *this, UINT32 u4Type, UINT32 u4Arg2, UINT32 u4Arg3);
UINT32 _B2R_GetContainerType(B2R_OBJECT_T *this);
VOID _B2R_PreparePendingForNextVsync(B2R_OBJECT_T* this);
UINT32 _B2R_SetPathMute(UCHAR ucB2rId, MUTE_MODULE_ID eModule, UINT32 u4UnMuteDelay, BOOL bReplaceByLongerDelay);
void _B2R_GetSeek(B2R_OBJECT_T* this, UCHAR* pucSeekMode, UINT32* pu4Pts, UCHAR* pucNotifyTrigger);
UINT32 _B2R_GetDispingPts(B2R_OBJECT_T* this);
UINT32 _B2R_GetEsmQCnt(void);
void B2R_EnalbleSmartPcr(UCHAR ucB2rId, BOOL fgEn);
BOOL _VDP_RTNeedSuperGameMode(UCHAR ucVdpId);
UINT32 _B2R_GetContentSourceType(B2R_OBJECT_T *this);
VOID _B2R_NPTVSeamlessStatus(B2R_OBJECT_T* this, UCHAR ucMode);
UINT8 _B2R_GetVdpIds(UINT8 u1B2rId, UINT8* pu1VdpIds);

UINT8 _B2R_GetVdpId(UINT8 u1VdpId);

void VDP_SetRenderRegion(UINT8 ucVdpId, VDP_REGION_T *prRendRegion);
/**
 *  Vsync callback
 */
typedef VOID (*VDP_VSYNC_CB_FUNC)(UCHAR ucVdpId, VOID* pv_tag);
BOOL VDP_VsyncCBRegister(UCHAR ucVdpId, VDP_VSYNC_CB_FUNC VDP_VsyncCB, VOID* pv_tag);

BOOL B2R_OneFbChgTrigger(UCHAR ucVdpId);

void _B2R_EventNotify(UCHAR ucVdpId, UINT32 u4Arg1, UINT32 u4Arg2, UINT32 u4Arg3);

UINT32 _B2R_GetLogMask(B2R_OBJECT_T* this);

VOID _B2R_SetTunnelMode(UCHAR ucB2rId, BOOL fgTunnel);

UINT32 FrameRateConvert(UCHAR ucFrmRatCod,UINT32 u4TimeScale);

BOOL _B2R_IsPathMute(B2R_OBJECT_T *this);
VOID _B2R_EnableCrc(UCHAR ucB2rId,BOOL fgCrcEnable);
VOID _B2R_SetCNMode(UCHAR ucB2rId, BOOL fgCN);

#endif    // B2R_DRVIF_H

