/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

 /*-----------------------------------------------------------------------------
  * @file HDMI interface header file
  * $Author$ xiongfeng.wen@mediatek.com
  * $Date  $ 2019.08.09
  * $RCSfile: drv_hdmi_if.h,v $
  * $Revision$
  *
  *---------------------------------------------------------------------------*/
#ifndef _DRV_HDMI2_IF_H_
#define _DRV_HDMI2_IF_H_

#include "drv_hdmi2.h"
#include "drv_hdmi21.h"

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------
#define HDMI_3D_NO_VSI_FRAME_COUNT 10
#define NO_HDR_MAX_FRAME_COUNT 5
#ifdef CC_SUPPORT_DOLBY_VISION
#define HDMI_DOLBY_EDR_NO_METADATA_THRES 6
#define HDMI_DOLBY_EDR_NO_FREEZE_THRES 20
#define HDMI_DOLBY_EDR_NO_VSI_FRAME_COUNT 3
#define DOLBY_UNFREEZE_COUNT_MAX 4


#ifdef HDMI_SUPPORT_DOLBY_VISION_NEW
typedef UINT8 (*u1pfDVInforProcessCB)(E_HDMI_SWITCH_NUM eHdmiPort, void *peDolbyInfor, E_HDMI_DOLBY_HDR_STATE *pDolbyHdrState);
void fpRegisterDVInforProcessCB(u1pfDVInforProcessCB fpDolbyVisionInforCB);
void vDolbyVisionSetHdrState(E_HDMI_SWITCH_NUM eHdmiPort, E_HDMI_DOLBY_HDR_STATE eDolbyHdrState);
typedef UINT8 (*u1pfDVEMPInforProcessCB)(E_HDMI_SWITCH_NUM eHdmiPort,
                void *pDVEMPInfor, UINT16 u2MDSize, E_HDMI_DOLBY_HDR_STATE *pDolbyHdrState, void *pEmpInfor);
void fpRegisterDVEMPInforProcessCB(u1pfDVEMPInforProcessCB fpDVEMPInforCB);
#endif

/**
 * @brief        Check dolby HDR exist
 * @param        u1Path---- Video path number
 * @retval       TRUE dolby HDR exist, FALSE for not exist
*/
extern BOOL fgHDMI2DolbyHDRExist(UINT8 u1Path);
#endif

extern UINT8 bHDMI2GetHdmiMode(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2GetInfoFrameToIF(E_HDMI_SWITCH_NUM eHdmiPort,HdmiRxInfo_t *ptInfo,UINT8 u1Type);

extern UINT8 bHDMI2ScanInfo(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2AFD(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2RgbInput(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2AspectRatio(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2DeepColorStatus(E_HDMI_SWITCH_NUM eHdmiPort);

extern BOOL fgHDMI2IsPCM(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 u1HDMI2GetAudioSamplingFreq(void);

extern UINT8 bHDMI2AudBitWidth(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI HDR EOTF value
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI HDR EOTF value
*/
extern UINT8 u1HDMI2GetHdrEOTF(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI HDR metadata
 * @param        eHdmiPort---- HDMI port number
 * @param        *prHdrMetadata---- Pointer to HDMI_HDR_METADATA
*/
extern void vHDMI2GetHdrMetaData(E_HDMI_SWITCH_NUM eHdmiPort,HDMI_HDR_METADATA *prHdrMetadata);
/**
 * @brief        Transfer the color info number to Enum HdmiRxClrSpc_t
 * @param        u1ColorInfo ---- color info metadata
 * @retval       Enum value for color info, see HdmiRxClrSpc_t
*/
extern HdmiRxClrSpc_t eHDMI2TransColorInfo(UINT8 u1ColorInfo);
/**
 * @brief        clear hdmi avi info
 * @param        eHdmiPort---- HDMI Port Number
 * @retval       Enum value for color info, see HdmiRxClrSpc_t
*/
extern void vHDMIClearAviInfo(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI Allm Config
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI Allm config
*/
extern UINT8 u1HDMI2GetAllmConfig(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Set HDMI Allm Config
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI Allm config
*/
extern UINT8 u1HDMI2SetAllmConfig(E_HDMI_SWITCH_NUM eHdmiPort, BOOL u1EnableFlag, BOOL u1ALLM_en);
/**
 * @brief        Get HDMI color space
 * @param        eHdmiPort---- HDMI port number
 * @retval       See Enum HdmiRxClrSpc_t
*/
extern HdmiRxClrSpc_t eHDMI2GetHdmiClrSpc(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT32 u4HDMI2GetNoVsiTimeInMs(E_HDMI_SWITCH_NUM eHdmiPort);
extern void vHDMI2SetVsiReceiveTime(E_HDMI_SWITCH_NUM eHdmiPort);
extern UINT32 u4HDMI2GetNoDolbyVsiTimeInMs(E_HDMI_SWITCH_NUM eHdmiPort);
extern void vHDMI2SetDolbyVsiReceiveTime(E_HDMI_SWITCH_NUM eHdmiPort);
extern UINT32 u4HDMI2GetNoDolbyEMPTimeInMs(E_HDMI_SWITCH_NUM eHdmiPort);
extern void vHDMI2SetDolbyEMPReceiveTime(E_HDMI_SWITCH_NUM eHdmiPort);
extern UINT32 u4HDMI2GetHDR10PlusNoVsiTimeInMs(E_HDMI_SWITCH_NUM eHdmiPort);
extern void vHDMI2SetHDR10PlusVsiReceiveTime(E_HDMI_SWITCH_NUM eHdmiPort);
extern UINT32 u4HDMI2GetNoHFVsiTimeInMs(E_HDMI_SWITCH_NUM eHdmiPort);
extern void vHDMI2SetHFVsiReceiveTime(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2GetAudChannelNum(E_HDMI_SWITCH_NUM eHdmiPort);

extern BOOL fgHDMI2Get420Mode(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2422Input(E_HDMI_SWITCH_NUM eHdmiPort);

extern UINT8 bHDMI2RgbRange(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get audio status
 * @retval       1 for audio NG, 0 for audio OK
*/
extern UINT8 bHDMI2AudStatus(void);
/**
 * @brief        Check dsd audio exist
 * @retval       TRUE for dsd audio exist, FALSE for dsd audio not exist
*/
extern BOOL fgHDMI2AudDsdOn(void);
/**
 * @brief        Check hbr audio exist
 * @retval       TRUE for hbr audio exist, FALSE for hbr audio not exist
*/
extern BOOL fgHDMI2AudHbrOn(void);
/**
 * @brief        Get audio PCW mode
 * @retval       TRUE for HW mode, FALSE for SW mode
*/
extern BOOL fgHDMI2AudioPCWMode(void);

/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        value1----
 * @param        value2----
 * @param        bSDINOVR----
*/
extern void vHDMI2AudioSigmaDeltaSetting(UINT32 value1, UINT8 value2, UINT8 bSDINOVR);
/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        value1----
 * @param        value2----
 * @param        target----
 * @param        bSDINOVR----
*/
extern void vHDMI2AudioSigmaDeltaFineTuneSetting(UINT32 value1, UINT8 value2, UINT32 target, UINT8 bSDINOVR);
/**
 * @brief        Get audio channel status
 * @param        ePort---- HDMI port number
 * @param        pfgCopyright:---- Pointer to copy right info
 * @param        pu1CategoryCode:---- Pointer to category code info
 * @param        p1WordLength:---- Pointer to word length info
 * @param        pfgIsPcm:---- Pointer to pcm info
*/
extern void vHDMI2AudChannelStatus(E_HDMI_SWITCH_NUM ePort, BOOL *pfgCopyright, UINT8 *pu1CategoryCode,
                           UINT8 *p1WordLength, BOOL *pfgIsPcm);
/**
 * @brief        Check HDMI resolution stable
 * @param        ePort---- HDMI port number
 * @retval       1 for HDMI resolution stable, 0 for unstable
*/
extern UINT8 u1IsHDMI2Stable(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get bit rate
 * @param        ePort---- HDMI port number
 * @retval       Bit rate
*/
extern UINT32 u4HDMI2GetBitRate(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        Monitor VRR metadata exists or not
 * @param        ePort---- HDMI port number
 * @retval       N/A
*/
extern void vHDMI2NoVrrMonitor(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        set time when update vrr metadata
 * @param        ePort---- HDMI port number
 * @retval       N/A
*/
extern void vHDMI2SetVrrReceiveTime(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        Monitor HDR infoframes
 * @param        ePort---- HDMI port number
 * @retval       N/A
*/
extern void vHDMI2NoHdrMonitor(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        set time when read hdr infoframes
 * @param        ePort---- HDMI port number
 * @retval       N/A
*/
extern void vHDMI2SetHdrReceiveTime(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        get video frame rate in millisecond
 * @param        ePort---- HDMI port number
 * @retval       frame rate
*/
extern UINT32 u4HDMI2GetFramePeriodInMs(E_HDMI_SWITCH_NUM ePort);

/**
 * @brief        Get HDMI ITC flag
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI ITC flag
*/
extern UINT8 bHDMI2ITCFlag(E_HDMI_SWITCH_NUM eHdmiPort);

/**
 * @brief        Get HDMI ITC content
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI ITC content
*/
extern UINT8 bHDMI2ITCContent(E_HDMI_SWITCH_NUM eHdmiPort);

#ifdef HDMI21_SUPPORT_VRR
extern void _vHDMI2ParseVrrEMPPkt1Info(const UINT8 *data, HDMI_VRR_Info* vrr_info);
extern UINT8 bHDMI2ParseFreesyncSPDInfo(const UINT8 *data, HDMI_DRR_Fs_Info* drr_info);
extern UINT8 bHDMI2ParseGyncSPDInfo(E_HDMI_SWITCH_NUM eHdmiPort, const UINT8 *data);
extern void vHDMI2ResetVrrInfo(E_HDMI_SWITCH_NUM ePort);
extern UINT8 u1HDMI2GetFs2Hdr(E_HDMI_SWITCH_NUM ePort);
extern UINT8 u1HDMI2GetFs2Active(E_HDMI_SWITCH_NUM ePort);
extern UINT8 u1HDMI2GetFs2NativeColor(E_HDMI_SWITCH_NUM ePort);
#endif
#endif
