/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: tobey.chang $
 * $Date: 2015/06/16 $
 * $RCSfile: loader_if.h,v $
 * $Revision: #2 $
 *
 *---------------------------------------------------------------------------*/

/** @file loader_if.h
 *  Loader library interface function header file
 *  This is loader library interface function header file.
 *  Here are all interface functions for loader_main.c and loader_cmd.c.
 */


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#ifndef LOADER_DRVIF_H
#define LOADER_DRVIF_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_typedef.h"
#include "eeprom_if.h"
#include "loader_imghdr.h"
#include "x_dlm.h"

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------

/** LDR_DATA_T is loader data structure among functions.
 */
typedef struct _LdrData
{
    IMAGE_UPGRADE_HEADER_T  rLdrHdr;    ///< Image header information.
    DTVCFG_T    rDtvCfg;        ///< EEPROM boot info data structure
    UINT32      u4WakeupReason; ///< System wakeup reason info
    UINT32      u4PowerDown;    ///< Systam enter standby mode (IRRX, VGA, Keypad...)
    UINT32      u4LdrFlags;     ///< System flags for previous behavior.
    UINT32      u4ImageSrc;     ///< System image source.
    UINT32      u4ExternalStatus;    ///< System enable loader standby at AC on.
    UINT32      u4UsePowerBtnCount;     //<  Use Power button State to set standby
    UINT32      u4UsePrevPowerState;    //<  Use previous power state to set standby
    UINT32      u4UseVGAStatus;         //<  Use VGA status to enter standby mode when power key trigger.
    #ifdef CC_LDR_ATOMIC_MODE_SUPPORT
    UINT32      u4LdrAtomicMode;        //<  Use Ldr Atomic mode for P-mode/F-mode/migraiotn-mode
    #endif
} LDR_DATA_T;

///< defines for LDR_DATA_T.u4LdrFlags
#define LDRFLAG_UPGRADE_DONE        (1U << 0)
#define LDRFLAG_BOOTLOGO_DISABLE    (1U << 1)
#define LDRFLAG_BEBOOT_FAIL         (1U << 2)

///< defines for LDR_DATA_T.u4ImageSrc
#define IMAGESRC_NORFLASH           (1U << 0)
#define IMAGESRC_SMALLNAND          (1U << 1)
#define IMAGESRC_BIGNAND            (1U << 2)
#define IMAGESRC_USBFILE            (1U << 3)

typedef struct _BELFrameConfig
{
    UINT8     u1FrameType;  //0: IFRAME, 1: FFRAME, 2: DFRAME others skip
    UINT8     u1BootState;
    UINT8     u1FrmNo;
    UINT32    u4HKey;
    UINT8     u1TotalFrmNo;
    UINT8     u1TotalPktNo; //max is 7 packets in 1 frame
    UINT32    u4FKey;
    UINT8     u1PktContainer[7];
} BE_FRAME_CONFIG_T;

#ifdef CC_UPGRADING_DYNAMIC_LOGO_SUPPORT
typedef struct _DYNAMIC_LOGO_INFO_T
{
	UINT32 u4RegionList;
	UINT32 u4Region[2];
	UINT32 u4BmpAddr[4];
	UINT32 u4DstAddr[4];
	UINT32 u4RgnHeaderAddr[2];
	UINT32 u4BmpBarBGAddr;
	UINT32 u4BmpBarFullAddr;
	UINT32 u4BmpBarDstAddr;
} DYNAMIC_LOGO_INFO_T;

typedef enum
{
  LDR_UPGRADE_TOTAL_TAG,
  LDR_UPGRADE_PROCESS_TAG,
  LDR_UPGRADE_PROCESS_IDX,
  LDR_UPGRADE_MAX
} BAR_STATUS_CMD_T;

#define DYNAMIC_DST_BUFFER_SIZE (0x200000)
#define DYNAMIC_RGN_HEADER_SIZE (0x1000)
#endif

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/** LDR_IsEnterCli(): Loader check escape from execution to cli environment
 *  This function is to check whether uart has a ESC key in a period, then
 *  return the result.
 *
 *  @param  u4BootWaitTime  cli wait time.
 *  @retval 0               No ESC, continue to execute.
 *  @retval otherwise       break into cli mode.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_IsEnterCli(UINT32 u4BootWaitTime);

//-----------------------------------------------------------------------------
/** LDR_GetLdrData(): get the loader data structure pointer.
 *
 *  @param  no parameter
 *  @retval LDR_DATA_T pointer. the pointer to loader data structure.
 */
//-----------------------------------------------------------------------------
EXTERN LDR_DATA_T *LDR_GetLdrData(VOID);


//-----------------------------------------------------------------------------
/** LDR_EEPWriteEnable(): Loader enable/disable EEPROM write access.
 *  This function enable/disable WP Pin of EEPROM
 *  @param  fgState       The write access state
 *  @retval                   No return value.
 */
//-----------------------------------------------------------------------------
EXTERN void LDR_EEPWriteEnable(BOOL fgState);

//-----------------------------------------------------------------------------
/** LDR_MTDEncryptInit(): Loader initialize MTD encryption

 *  @param  no parameter
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN void LDR_MTDEncryptInit(void);

//-----------------------------------------------------------------------------
/** LDR_PreInit(): Loader initialize some setting at the beginning.
 *  This function is going to setup the basic setting for basic operations such
 *  as DRAM priority and USB initialization settings.

 *  @param  no parameter
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_PreInit(VOID);

//-----------------------------------------------------------------------------
/** LDR_EnvInit(): Loader initialize the basic setting for loader environment.
 *  This function is going to setup the basic setting for basic operations such
 *  as 8032, eeprom power, and PDWNC initialization settings.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_EnvInit(LDR_DATA_T *prLdrData);

//-----------------------------------------------------------------------------
/** LDR_CheckUpgrade(): check it's necessary to upgrade and do it if need.
 *  This function is going to check whether there is a upgrade need at loader
 *  boot up.  If it's necessary, it will continue to go to upgrade.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN INT32 LDR_FlashWrite(DLM_ACCESS_T, UCHAR*, INT32*, UINT32, UINT32);
EXTERN UINT32 LDR_CheckUpgrade(LDR_DATA_T *prLdrData);
EXTERN UINT32 LDR_ForceUpgrade(LDR_DATA_T *prLdrData);

#ifdef MTKBT_FAST_WOBLE
//-----------------------------------------------------------------------------
/* LDR_BtProbe(): load FW patch to flash and enable WoBLE mode for boot without flash
*/
//-----------------------------------------------------------------------------
EXTERN void LDR_BtProbe(void);
//-----------------------------------------------------------------------------

/* LDR_CheckBTReady(void): enumerate BT device
*/
EXTERN UINT32 LDR_CheckBTReady(void);
//-----------------------------------------------------------------------------
#endif

//-----------------------------------------------------------------------------
/** LDR_IsGoBootup(): Decide whether go to standby or continue boot up.
 *  This function is going to check wakeup reason and eeprom status (or some
 *  external uP) to decide it should enter standby or contineu boot.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Enter standby.
 *  @retval otherwise       Continue boot.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_IsGoBootup(LDR_DATA_T *prLdrData);

//-----------------------------------------------------------------------------
/** LDR_EnableCEC(): Decide whether enable CEC or not
 *
 *  @param  fgCecEnable  Enable or disable CEC or not.
 *  @retval void
 */
//-----------------------------------------------------------------------------
EXTERN void LDR_EnableCEC(BOOL fgCecEnable);

//-----------------------------------------------------------------------------
/** LDR_EnterStandby(): Enter standby mode after calling.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_EnterStandby(LDR_DATA_T *prLdrData);

//-----------------------------------------------------------------------------
/** LDR_UpSetup(): Do necessary setup settings for image boot up.
 *  This function is going to setup boot settings, such as GPIO_Init().
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_UpSetup(LDR_DATA_T *prLdrData);

//-----------------------------------------------------------------------------
/** LDR_BENotifySetup(): Do necessary setup settings for BEIC data transferring
 *  This function is going to setup boot settings, such as GPIO_Init() and register call back
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------

EXTERN UINT32 LDR_BENotifySetup(LDR_DATA_T *prLdrData);
EXTERN UINT32 LDR_BEBootVerify(LDR_DATA_T *prLdrData);
EXTERN UINT32 LDR_BEPatternDisplay(UINT32 u4PartitionIdx, UINT32 u4Offset);
EXTERN BOOL LDR_BECheckDDRI(VOID);
EXTERN UINT32 LDR_BEUpgReboot(LDR_DATA_T *prLdrData);
//-----------------------------------------------------------------------------
/** LDR_PanelLogo(): display logo on panel.
 *  This function is going to find out the logo location and call lvds/pmx/osd
 *  low level driver to display logo on panel.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_PanelLogo(LDR_DATA_T *prLdrData);

EXTERN BOOL LDR_IsLogoDisplay(LDR_DATA_T *prLdrData);
//-----------------------------------------------------------------------------
/** LDR_LogoDisplay(): To display logo with panel index.
 *  This function is going to display logo at boot loader.
 *
 *  @param  u4BmpAddr       The bmp data pointer.
 *  @param  u4Width         The width info of the bmp data.
 *  @param  u4Height        The height info of the bmp data.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_LogoDisplay(LDR_DATA_T *prLdrData, UINT32 u4Idx);

//-----------------------------------------------------------------------------
/** LDR_ImageDecompress(): decompress image to dram on the offset of entry.
 *  This function is going to search the target device and find out the compressed
 *  image. Then copy to dram (if necessary) and decompress the data from dram
 *  to dram by LZHS hardware.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_ImageDecompress(LDR_DATA_T *prLdrData);

//-----------------------------------------------------------------------------
/** LDR_ImageJump(): leave loader and jump to image binary in dram.
 *
 *  @param  prLdrData       The loader global data structure pointer.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_ImageJump(LDR_DATA_T *prLdrData);


//-----------------------------------------------------------------------------
/** LDR_FlashCopy(): copy flash data from system flash cueernt sys_build used.
 *  it will automatically deside copy from NOR or NAND
 *
 *  @param  pu1Buf          destination memory buffer.
 *  @param  u4Offset        flash offset.
 *  @param  u4Size          copy size.
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_FlashCopy(UINT8* pu1Buf, UINT32 u4FlashOffset, UINT32 u4Size);

EXTERN void LDR_SetEtherMAC(void);

//-----------------------------------------------------------------------------
/** LDR_DramParamsStore(): Try to store DRAM parameters if any
 *
 *  @retval 0               Success.
 *  @retval otherwise       Failed.
 */
//-----------------------------------------------------------------------------
EXTERN UINT32 LDR_DramParamsStore(void);

#ifdef CC_UPGRADING_DYNAMIC_LOGO_SUPPORT
EXTERN UINT32 LDR_DynamicLogoInit(LDR_DATA_T *prLdrData, BOOL fgMode);
EXTERN UINT32 LDR_DynamicLogoDisplay(LDR_DATA_T *prLdrData, UINT32 u4Idx);
EXTERN UINT32 LDR_PatternBarSetStatus(BAR_STATUS_CMD_T LdrCmd, UINT32 u4Val);
EXTERN INT32 LDR_PatternBarSetPercentage(INT32 i4Val);
#endif

#endif /* LOADER_DRVIF_H */

