/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


#ifndef MPV_DRVIF_H
#define MPV_DRVIF_H

#if 0
#ifndef NDEBUG
#define MPV_VIRTUAL_CC
#endif
#endif

#include "mpv_if.h"
LINT_EXT_HEADER_BEGIN
#include "x_os.h"
LINT_EXT_HEADER_END


//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

#ifndef VLD_ID
#define VLD_ID
typedef enum{
    VLD_Core0 = 0,
    VLD_Core1,
    VLD_Core2,
    VLD_LAE0,
    VLD_LAE1,
    VLD_IMG,
    VLD_MAX,
}ENUM_VLD_ID;
#endif

// VLD ID
#ifndef VLD0 //Remove VLD0 VLD1 later.
#define VLD0   VLD_Core0
#define VLD1   VLD_Core1
#endif

//ES ID
#ifndef ES0
#define ES0     0  // in the case, other don't expect we could decode have more than on es
#define ES1     1
#define ES2     2
#define ES3     3
#endif

//Maximum VLD number
#define MPV_MAX_VLD            2
#define MPV_MAX_ES         4

// Picture Coding Type
#define MPV_I_TYPE            1
#define MPV_P_TYPE            2
#define MPV_B_TYPE            3
/*
//Decode Mode
#define MPV_CTRL_MODE_MASK    0xF
#define MPV_IDLE_MODE         0x0
#define MPV_PLAY_MODE         0x1
#define MPV_STOP_MODE         0x2
#define MPV_AVSYNC_MODE       0x3
#define MPV_HALFREF_MODE      0x4
#define MPV_PLAY_I_FRAME_MODE 0x5
#define MPV_DEBLOCK_MODE      0x6
*/
//Event notify
#define MPV_DEC_ERR            1
#define MPV_DEC_TIMEOUT        2
#define MPV_SEQH_NOI        3
#define MPV_SD_HD            4
#define MPV_FIELD            5
#define MPV_PAN_SCAN        6

//Start code
#define MPV_SEQ_HDR         0x1b3
#define MPV_GOP_HDR         0x1b8
#define MPV_PIC_HDR         0x100
#define MPV_NO_HDR          0x0

#define MPV_DBK_LEVEL_MAX   29

#define MPV_SEMI_FLAG_SAVE          (0x1 << 0)
#define MPV_SEMI_FLAG_IGNORE_ERR    (0x1 << 1)
typedef struct _MPV_VDEC_LOGO_CTRL_FIELD_T
{
    /* 8 bits */
    UCHAR ucDefault   : 1;
    UCHAR ucEnable    : 1;
    UCHAR ucSelect    : 5;
    UCHAR ucReserved  : 1;
} MPV_VDEC_LOGO_CTRL_FIELD_T;

typedef union _MPV_VDEC_LOGO_CTRL_UNION_T
{
LINT_SUPPRESS_BRACE(960)

    UCHAR                        aucReg[1];
    MPV_VDEC_LOGO_CTRL_FIELD_T   rField;
} MPV_VDEC_LOGO_CTRL_UNION_T;


//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------


//ES Info in ESMQ

typedef struct
{
    BOOL fgSeqHdr;
    BOOL fgGop;
    BOOL fgDtsValid;      //DTS is valid
    UCHAR ucMpvId;
    UCHAR ucEsId;
    UCHAR ucPicType;
    UINT32 u4FifoStart;
    UINT32 u4FifoEnd;
    UINT32 u4VldReadPtr;
    UINT64 u4PTS;
    UINT32 u4DTS;
} MPV_PES_INFO_T;

typedef void (*PFN_MPV_EVENT_NOTIFY)(
    UINT32 u4Arg1,
    UINT32 u4Arg2,
    UPTR u4Arg3,
    UINT32 u4Arg4
);


typedef struct //_CC_INFO_T
{
    UINT32   u4BufRptr;
    UINT32   u4Size;
    PTS_T    u8Pts;
    BOOL     fgIsScte;
    BOOL     fgBottomFirst;
} CC_INFO_T;


typedef struct //_CC_BUF_INFO_T
{
    UINT32        u4Rptr;
    UINT32        u4Wptr;

}CC_BUF_INFO_T;


typedef enum
{
    MPEG_CC_DATA,
    MPEG_CC_TS130_DATA
} MPEG_CC_DATA_TYPE;

typedef struct //_MPV_CC_T
{
    BOOL     fgPlayCc;
    BOOL     fgCcIPTmpIsScte;
    BOOL     fgCcPreRef;
    BOOL     fgCcBottomFirst;
    HANDLE_T hCcQueue;
    UCHAR*   pucCcBuf;
    UCHAR*   pucCcTmpBuf;
    UCHAR*   pucCcAtscIPTmpBuf;
    UCHAR*   pucCcScteIPTmpBuf;
    UINT32   u4CcBufSize;
    UINT32   u4AtscIPTmpSize;
    UINT64   u4AtscIPTmpPts;
    UINT32   u4ScteIPTmpSize;
    UINT64   u4ScteIPTmpPts;
    UINT32   u4IPFrameNum;
    UINT32   fgTmpBottomFirst;
    UINT32   u4TmpSize;
    void*    pvCcNfyTag;
    PFN_CC_NFY_FCT pfCcNfyFct;
    CC_INFO_T rCcInfo;
    CC_BUF_INFO_T rCcBufInfo;

    /*mpeg2 new added*/
    MPEG_CC_DATA_TYPE eCcDataType;
    UINT32   u4AtscIdentifier;
    BOOL     fgAtsc;
    UCHAR    ucUsrDataTp;
    BOOL     fgScte;
    UINT32   u4CcLen;

} MPV_CC_T;

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

EXTERN void _MPV_CreateDecSema(UCHAR ucMpvId);
EXTERN void _MPV_CleanRRDbk(UCHAR ucEsId);
EXTERN void _MPV_SetDbk(UCHAR ucEsId);
//EXTERN void _MPV_SetRR(UCHAR ucEsId);
EXTERN void* _MpvGetHdlr(UCHAR ucEsId);
EXTERN void* _MPV_GetExtInfo(void);
EXTERN void* _MPV_GetMpvInfo(void);

EXTERN void MPV_CleanRRDbk(UCHAR ucEsId);
EXTERN UINT32 MPV_GetDecMode(UCHAR ucEsId);
EXTERN UCHAR MPV_CallBackEsm(const MPV_PES_INFO_T *prPesInfo);
EXTERN void MPV_FlushEsmQ(UCHAR ucEsId, BOOL fgAbortDecode);
EXTERN void MPV_SetEC(UCHAR ucEsId, UCHAR ucECMethod, UINT32 u4ErrThrsd,
    BOOL fgDetectRefLost);
EXTERN void MPV_RegEventNotifyFunc(PFN_MPV_EVENT_NOTIFY pfFunc);
EXTERN void MPV_EventNotify(UINT32 u4Arg1, UINT32 u4Arg2, UINT32 u4Arg3, UINT32 u4Arg4);
EXTERN BOOL MPV_IsDisplay(UCHAR ucEsId);
EXTERN void MPV_UpdateVldWptr(UCHAR ucEsId, UINT32 u4VldWptr);
//EXTERN void MPV_SetHalfRef(UCHAR ucEsId, UINT32 u4HRefMode, BOOL fgFilterY, BOOL fgFilterC, BOOL fgBit5, BOOL fgBit4, BOOL fgBit3);
EXTERN void MPV_SetMcMode(UCHAR ucEsId, BOOL fgMcPipe, BOOL fgMcFastAddr, BOOL fgMcNewFsm);
EXTERN void MPV_SetDbkRegion(UCHAR ucEsId, UINT32 u4XStart, UINT32 u4XEnd,  UINT32 u4YStart, UINT32 u4YEnd);

EXTERN void MPV_SetDBkThrsd(UCHAR ucEsId, BOOL fgDisable, UINT32 u4QpAvgThrsd, UINT32 u4DbkRatioThrsd,
                                    UINT32 u4QpAvgThrsdP, UINT32 u4DbkRatioThrsdP, UINT32 u4MvThrsd);

EXTERN void MPV_SetDbkPara1(UCHAR ucEsId, UINT32 u4P1, UINT32 u4P2,UINT32 u4P3,UINT32 u4P4, UINT32 u4P5,
                                                UINT32 u4P6, UINT32 u4P7, UINT32 u4P8,UINT32 u4P9,UINT32 u4P10);
EXTERN void MPV_SetDbkPara2(UCHAR ucEsId, UINT32 u4P1, UINT32 u4P2,UINT32 u4P3,UINT32 u4P4, UINT32 u4P5,
                                                UINT32 u4P6, UINT32 u4P7, UINT32 u4P8,UINT32 u4P9,UINT32 u4P10);
EXTERN void MPV_SetEcSetting(UCHAR ucEsId, UINT32 u4Mode);
EXTERN void MPV_LockVld(UCHAR ucMpvId);
EXTERN void MPV_UnlockVld(UCHAR ucMpvId);
//EXTERN BOOL MPV_ResetSearch(UCHAR ucMpvId, UINT32 u4FifoStart, UINT32 u4FifoEnd, UINT32 u4SearchStart ,UINT32 u4SearchEnd);
EXTERN UINT32 MPV_SearchStartCode(UCHAR ucMpvId, UCHAR ucEsId, UINT32* pu4Addr);
//EXTERN void MPV_QueryStatus(void);

EXTERN BOOL MPV_ShowLogo(VOID);

EXTERN void _MPV_SetSyncStc(UCHAR ucEsId);

#ifdef MPV_VIRTUAL_CC
EXTERN void MPV_PlayVirtualCc(UCHAR ucEsId, BOOL fgEnable);
#endif

EXTERN void MPV_CountPicNs(UINT32 u4StartPts, UINT32 u4EndPts);
EXTERN UINT32 MPV_GetPicNs(void);

#ifdef MPV_SEMI_HOST
EXTERN void _MPV_SetSaveNo(UINT32 u4StartSaveNo, UINT32 u4EndSaveNo);
EXTERN void _MPV_SetFilePath(UINT32 u4Flag, char *szFilePath);
EXTERN void _MPV_CompareGold(UCHAR ucEsId, UCHAR ucFbgId, UCHAR ucFbId);
#endif

#ifdef MPV_VIRTUAL_PRS
EXTERN void MPV_VirtualPrsParse(UCHAR ucMvpId, UINT32 u4Addr, UINT32 u4PicNs, UINT32 u4FrmNs);
EXTERN void MPV_VirtualPrsStart(UINT32 u4Delay);
EXTERN void MPV_VirtualPrsStop(void);
#endif

#ifdef MPV_STATISTICS
EXTERN void _MPV_ResetTime(void);
EXTERN void _MPV_PrintTime(UCHAR ucMpvId);
#endif

struct _VDEC_DECODER_HANDLE_T_;


// decode
EXTERN void _MPV_StopDecES(UCHAR ucEsId, struct _VDEC_DECODER_HANDLE_T_ *prHandle);

// mpv_if
EXTERN void _MPV_ClearEsInfo(UCHAR ucEsId);
EXTERN void _MPV_ClearMpvCc(UCHAR ucEsId);
EXTERN void* _MPV_GetMpvCc(UCHAR ucEsId);
EXTERN void _MPV_SkipPic(UCHAR ucMpvId, UCHAR ucEsId);
//EXTERN void MPV_SetRR(UCHAR ucEsId, UINT32 u4Ratio);
EXTERN BOOL _MPV_PutFb2DispQ(UCHAR ucEsId, UCHAR ucMpvId, UCHAR ucFbId);
EXTERN BOOL MPV_EsFlwCtrl(UCHAR ucEsId, UCHAR* pucMpvId, INT8 i1EsBufStatus, UCHAR ucSkipType, struct _VDEC_DECODER_HANDLE_T_ *prHandle);
EXTERN BOOL _MPV_EsFlwCtrl(UCHAR ucEsId, UCHAR* pucMpvId, INT8 i1EsBufStatus, UCHAR ucSkipMode);

EXTERN BOOL _MPV_WaitRandomAP(const void* prPesInfo);

EXTERN UINT64 _MPV_CalcPts(UCHAR ucEsId, UCHAR ucFbId);
EXTERN UINT64 _MPV_CalcPtsBase(UCHAR ucEsId, VOID* pvrFbmPicHdr);
EXTERN VOID _MPV_SetIsr(UCHAR ucEsId, UCHAR ucMpvId);
EXTERN void _MPV_VldSetVFifo(UCHAR ucMpvId, UINT32 u4Sa, UINT32 u4Ea);
EXTERN BOOL _MPV_VldInitBarrelShifter(UCHAR ucMpvId, UINT32 u4Ptr);
EXTERN UINT32 _MPV_VParserGetHdr(UCHAR ucMpvId, UCHAR ucEsId,
                                        UINT32* pu4Addr, UCHAR* pu4PicType);
#endif
