/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: shuo.ding $
 * $Date: 2015/08/04 $
 * $RCSfile: pmx_if.h,v $
 * $Revision: #2 $
 *
 *---------------------------------------------------------------------------*/

/** @file pmx_if.h
 *  plane mixer(pmx) public functions
 */

#ifndef PMX_IF_H
#define PMX_IF_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_lint.h"
LINT_EXT_HEADER_BEGIN
#include "x_typedef.h"
LINT_EXT_HEADER_END


//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------

/** maximum number of plane mixer */
#define PMX_MAX_NS                  1
/** pmx id 1 */
#define PMX_1                       0

/** pmx set value fail */
#define PMX_SET_ERROR               0
/** pmx set value ok */
#define PMX_SET_OK                  1

#define PMX_MAIN 0 // VDP1
#define PMX_PIP  1 // VDP2
#define PMX_OSD1 2 // VDP3, alpha-bit plane
#define PMX_OSD2 3 // VDP4
#define PMX_OSD3 4 // VDP5


//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

/** initialize pmx
 */
EXTERN void PMX_Init(void);

/** get plane z-order
 *  @param  ucPlaneId plane id:{PMX_VDP_M, PMX_VDP_P, PMX_OSD_1, PMX_OSD_2}
 *  @param  ucPlaneOrder order:{PLA_MXR_Z_ORDER_1/2/3/4}
 *  @retval PMX_SET_OK success
 */
EXTERN UINT32 PMX_GetPlaneOrder(UCHAR ucPlaneId, UCHAR *pucPlaneOrder);

/** set plane z-order
 *  @param  ucPlaneId plane id:{PMX_VDP_M, PMX_VDP_P, PMX_OSD_1, PMX_OSD_2}
 *  @param  ucPlaneOrder order:{PLA_MXR_Z_ORDER_1/2/3/4}
 *  @retval PMX_SET_OK success
 */
EXTERN UINT32 PMX_SetPlaneOrder(UCHAR ucPlaneId, UCHAR ucPlaneOrder);

/** set pmx background color
 *  @param  u4BgColor background color(0xrrggbb)
 *  @param  fgInVsync update it in next vsync or immediately
 *  @retval PMX_SET_OK success
 */
EXTERN UINT32 PMX_SetBg(UINT32 u4BgColor, BOOL fgInVsync);

#ifdef CC_FOR_POC_DEMO
/** set pmx blanking color
 *  @param  u4BlkColor blanking color(0xrrggbb)
 *  @retval PMX_SET_OK success
 */
EXTERN UINT32 PMX_SetBlk(UINT32 u4BgColor);
#endif

/** setup alpha value for video plane
 *  @param  ucPlaneId video plane id:{PMX_VDP_M, PMX_VDP_P}
 *  @param  ucValue alpha value(0~255)
 *  @retval PMX_SET_OK success
 */
EXTERN UINT32 PMX_SetAlpha(UCHAR ucPlaneId, UCHAR ucValue);

/** _PMX_Set_ALPHA_MIX_GRP1_0
 *  @param  fgEnable   ALPHA_MIX_GRP1_0:MODE
 */
EXTERN void PMX_Set_ALPHA_MIX_GRP1_0(BOOL fgEnable);

EXTERN void PMX_Set_ALPHA_MIX_GRP12_0(BOOL fgEnable);

EXTERN UINT32 PMX_SetPlaneOrderArray(const UINT32 *pu4PlaneOrder);
EXTERN UINT32 PMX_GetPlaneOrderArray(UINT32 *pu4PlaneOrder);
EXTERN UINT32 PMX_SetMixAlpha(UCHAR ucType, const UINT32 *pu4Alpha);

#endif    // PMX_IF_H

