/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: vdp_if.h,v $
 * $Revision: #2 $
 *
 *---------------------------------------------------------------------------*/

/** @file vdp_if.h
 *  Video plane interface, including init, enable,
 *  mode selection and region desciption.
 */

#ifndef VDP_IF_H
#define VDP_IF_H

#include "x_typedef.h"
#include "stc_drvif.h"
#include "feature.h"

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------

#define VDP_MAX_NS              	2    	///< Maximum number of video plane.
#define VDP_NS                      2


// Value of parameter ucVdpId.
#define VDP_1                       0    	///< VDP1
#define VDP_2                       1    	///< VDP2
#define VDP_3                       2    	///< OSD1, obsolete
#define VDP_4                       3    	///< OSD2, obsolete
#define VDP_5                       4    	///< OSD3, obsolete

#define VDP_BYPASS                  0    	///< VDP bypass path for 4k2k, HDMI/B2R
#define VDP_PQPATH                  1    	///< VDP pq path for normal video
#define VDP_GFXOUT                  2    	///< VDP gfx out path for 4k2k
#define OUTPUT_PLANE_NS             3

#define VDP_MAX_INPORT_NS           1    	///< Maximum number of input port for each video plane.


// Video plane configuration return value.
#define VDP_SET_ERROR               0    	///< return value
#define VDP_SET_OK                  1		///< return value

// Trick Play at 20081001
#define VDP_PLAY_NORMAL 0 ///< Normal Play
#define VDP_PLAY_TRICK 1 ///< Trick Play
#define VDP_PLAY_STEP_FORWARD 2 ///< Step Forward

// Lip Sync OK/NG status change notification
#define VDP_LIPSYNC_REG_NG_TO_OK 0
#define VDP_LIPSYNC_REG_OK_TO_NG 1

#define VERIFY_NULL(ptr)                                    \
          do {                                              \
            if (ptr == NULL) { return VDP_SET_ERROR; }      \
          } while (0)

#define VDP_CHECK_CB_FUNC_VERIFY(func, crc)     ((UINT32)func == ~((UINT32)crc))

#define VDP_MAX_REGION_WIDTH 10000
#define VDP_MAX_REGION_HEIGHT 10000

// Video plane mode.
#define VDP_MODE_NORMAL             0		///< normal mode, video out
#define VDP_MODE_BG                 1		///< background mode, mute
#define VDP_MODE_QV                 2		///< obsolete
#define VDP_MODE_DEINT              3		///< obsolete
#define VDP_MODE_UNKNOWN            4

// Fixed PreScaling Down
#ifndef CC_VDP_FIXED_PRESCALER
#define CC_VDP_FIXED_PRESCALER       1
#endif

/**
 *  Video plane region (source/output).
 */
typedef struct
{
    UINT32 u4X;        	///< start position in x direction (u4X/1000 = start pixel/total pixel)
    UINT32 u4Y;        	///< start position in y direction (u4Y/1000 = start pixel/total pixel)
    UINT32 u4Width;  	///< width in percentage (u4Width/1000 = region width/total width)
    UINT32 u4Height; 	///< height in percentage (u4Width/1000 = region height/total width)
} VDP_REGION_T;

/** 3D Mode chage
*/
typedef struct
{
	UINT32 u4SrcWidth;				///< new source width
	UINT32 u4SrcHeight; 			///< new source height
	UINT32 u4InOutFrameRate; 			///< new frame rate
	UINT32 u1Output3DFormat; 			///< new scan mode
	UINT32  u1Input3DFormat;
	UINT32  u1ColorFormat;
	UINT32 u1ColorSpace;
}VDP_3D_MODE_CHG_INFO_T;


/**
 *  Video plane overscan region.
 */
typedef struct
{
    UINT32 u4Top;      	///<overscan percentage on top side (u4Top%)
    UINT32 u4Bottom;	///<overscan percentage on top side (u4Bottom%)
    UINT32 u4Left;     	///<overscan percentage on top side (u4Left%)
    UINT32 u4Right;   	///<overscan percentage on top side (u4Right%)
} VDP_OVERSCAN_REGION_T;

/**
 *  Video source change information.
 */
typedef struct
{
    UINT32 u4SrcWidth;              ///< new source width
    UINT32 u4SrcHeight;  	        ///< new source height
    UINT32 u4OutWidth; 	        ///< new output width
    UINT32 u4OutHeight; 	        ///< new output height  
    UINT32 u4FrameRate;            ///< new frame rate
    UINT32 u4Interlace;             ///< new scan mode
} VDP_SOURCE_INFO_T;

/* Thumbnail Color mode */
typedef enum
{
    VDP_COLORMODE_AYUV_CLUT2     = 0,
    VDP_COLORMODE_AYUV_CLUT4     = 1,
    VDP_COLORMODE_AYUV_CLUT8     = 2,
    VDP_COLORMODE_UYVY_16        = 3,
    VDP_COLORMODE_YUYV_16        = 4,
    VDP_COLORMODE_AYUV_D8888     = 5,
    VDP_COLORMODE_ARGB_CLUT2     = 6,
    VDP_COLORMODE_ARGB_CLUT4     = 7,
    VDP_COLORMODE_ARGB_CLUT8     = 8,
    VDP_COLORMODE_RGB_D565       = 9,
    VDP_COLORMODE_ARGB_D1555     = 10,
    VDP_COLORMODE_ARGB_D4444     = 11,
    VDP_COLORMODE_ARGB_D8888     = 12,
    VDP_COLORMODE_YUV_420_BLK    = 13,
    VDP_COLORMODE_YUV_420_RS     = 14,
    VDP_COLORMODE_YUV_422_BLK    = 15,
    VDP_COLORMODE_YUV_422_RS     = 16,
    VDP_COLORMODE_YUV_444_BLK    = 17,
    VDP_COLORMODE_YUV_444_RS     = 18
}VDP_COLORMODE_T;


/* Thumbnail Info */
typedef struct
{
    UCHAR*                    pucCanvasBuffer;
    VDP_COLORMODE_T           eCanvasColormode;
    UINT32                    u4CanvasWidth;     /* in pixels */
    UINT32                    u4CanvasHeight;    /* in pixels */
    UINT32                    u4CanvasPitch;     /* in bytes */

    UINT32                    u4ThumbnailX;      /* in pixels */
    UINT32                    u4ThumbnailY;      /* in pixels */
    UINT32                    u4ThumbnailWidth;  /* in pixels */
    UINT32                    u4ThumbnailHeight; /* in pixels */
}VDP_THUMBNAIL_INFO_T;

/* Thumbnail Info only w/h*/
typedef struct
{
    UINT32                    u4ThumbnailX;      /* in pixels */
    UINT32                    u4ThumbnailY;      /* in pixels */
    UINT32                    u4ThumbnailWidth;  /* in pixels */
    UINT32                    u4ThumbnailHeight; /* in pixels */
}VDP_THUMBNAIL_T;

/**
 *  Source Change Notify Function Type
 */
typedef UPTR (*PFN_VDP_SRC_CHG_NOTIFY)(UCHAR u4VdpId, VDP_SOURCE_INFO_T rVdpSrcInfo);

typedef UPTR (*PFN_VDP_UNMUTE_NOTIFY)(UINT32 u4VdpId, BOOL fgUnMute);

/**
 *  Source/Output Range Change Notify Function Type
 */

typedef UPTR (*PFN_VDP_RANGE_CHG_NOTIFY)(UCHAR u4VdpId, VDP_REGION_T rVdpRegionInfo);

/**
 *  Scaler display mode support Notify Function Type
 */
typedef UPTR (*PFN_VDP_DISP_SUPPORT_NOTIFY)(UCHAR ucVdpId, UCHAR ucStatus);

/**
*/
#ifndef CC_S_PLATFORM
typedef UPTR (*PFN_VDP_3D_Mode_CHG_NOTIFY)(UCHAR u4VdpId, VDP_3D_MODE_CHG_INFO_T rMode);
#else
typedef UPTR (*PFN_VDP_3D_MODE_CHG_NOTIFY)(UINT8 u13dMode);
#endif
#ifdef CC_SUPPORT_DOLBY_VISION
typedef UPTR (*PFN_VDP_DOVIHDR_CHGED_NOTIFY)(UINT8 u1IsDoviHDR);
#endif

typedef UPTR (*PFN_VDP_MEMC_MODE_CHGED_NOTIFY)(UINT8 u1IsByp);

typedef UPTR (*PFN_VDP_VRR_STATUS_CHGED_NOTIFY)(UINT8 u1Path,UINT8 u1IsVRR);


/**
 *  Scaler display mode support Notify Function Type
 */
typedef UPTR (*PFN_VDP_NAVI_TAG_NOTIFY)(UINT32 u4VdpId, UINT32 u4NaviTag);


/**
 *  Video plane call back function type id.
 */
typedef enum
{
    VDP_CB_FUNC_SRC_CHG_IND,        ///< Source Change Notify
    VDP_CB_FUNC_DISP_SUPPORT_IND,   //Scaler display mode not support notify
    VDP_CB_FUNC_DRIVER_SRC_RANGE_CHG_IND,  //driver source change notify
    VDP_CB_FUNC_DRIVER_OUT_RANGE_CHG_IND,  //driver output change notify
    VDP_CB_FUNC_PTS_IND,            ///< PTS Reach Notify
    VDP_CB_FUNC_AFD_IND,            ///< AFD/Aspect Ratio Change Notify
    VDP_CB_FUNC_UNMUTE_IND,         ///< UnMute Status Change Notify
    VDP_CB_FUNC_LIPSYNC_IND,        ///< Lip Sync Norify     
    VDP_CB_FUNC_MM_COND_IND,        ///< Vdp Condition Notify for MM
    VDP_CB_FUNC_MM_STEP_FIN_IND,        ///< Vdp Step Forward finish for MM
    VDP_CB_FUNC_MM_SEEK_FIN_IND,        ///< Vdp Seek finish for MM
    VDP_CB_FUNC_REPEAT_DROP_IND,        ///< Vdp repeat or drop
    VDP_CB_FUNC_LIPSYNC_OK_TO_NG_IND,   //< Vdp Lip Sync OK to NG notify
    VDP_CB_FUNC_LIPSYNC_NG_TO_OK_IND,   //< Vdp Lip Sync NG to OK notify        
    VDP_CB_FUNC_SRC_CHG_IND_2,
    VDP_CB_FUNC_AFD_IND_2,
    VDP_CB_FUNC_PLAY_DONE_IND,      ///< Vdp Play Done Notify
    VDP_CB_FUNC_Get_PIC_TYPE,
    VDP_CB_FUNC_VSS_FIN_IND,        ///< Vdp VSS finish notify
    VDP_CB_FUNC_TDNAVI_IND,         //TDNAVI Deteciton notify
	VDP_CB_FUNC_RES_CHG_IND,
	VDP_CB_FUNC_BIT_RATE_CHG_IND,
	VDP_CB_FUNC_VSS_420_IND,
	VDP_CB_FUNC_3D_MODE_CHG_IND,
	VDP_CB_FUNC_AFTER_3D_MODE_CHGED_IND,
	VDP_CB_FUNC_DOVI_HDR_CHGED_IND,
	VDO_CB_FUCN_MEMC_MODE_CHG_IND,
    VDO_CB_FUCN_VRR_STATUS_CHG_IND,
    VDP_CB_FUNC_NS
} VDP_CB_FUNC_ENUM_T;

/** Brief of VDP_NONLINEAR_PRM_T
 *  Video plane nonlinear scaler factors.
 */
typedef struct
{
    UINT32 u4Start;
    UINT32 u4End;
    UINT32 u4Slope;
    UINT32 u4MiddlePoint;
} VDP_NONLINEAR_PRM_T;

/** Brief of VDP_NONLINEAR_PRM_T
 *  Video plane nonlinear scaler factors.
 */
typedef struct
{
    //UCHAR ucUpDown;             // Linear region up scale or down scale   
    UINT32 u4SrcMiddleRatio;         // Source linear scaling region width in ratio from 0~10000
    UINT32 u4OutMiddleRatio;         // Output linear scaling region width in ratio from 0~10000
} VDP_NEW_NONLINEAR_PRM_T;

/** Brief of VDP_NEW_NONLINEAR_REG_T
 *  Video plane nonlinear scaler factors.
 */
typedef struct
{
    UINT32 u4StartFactor;   // Non-linear up scaling start factor
    UINT32 u4StepFactor;    // Non-linear up scaling step factor
    UINT32 u4MiddlePoint;   // Non-linear up scaling middle point
    //UINT8  u1Neg;           // Negative non-linear up scaling 
    UINT8  u1Step;          // Non-linear up scaling step width for each increment
    UINT32 u4StopPoint;     // Non-linear up scaling stop point
} VDP_NEW_NONLINEAR_REG_T;

/** Brief of VDP_AUTO_NONLINEAR_PRM_T
 *  Video plane nonlinear scaler factors in auto mode.
 */
typedef struct
{
    UINT32 u4Adaptive;
    UINT32 u4Slope;
    UINT32 u4EndFactor;
    UINT8 u1Mode;  
} VDP_AUTO_NONLINEAR_PRM_T;

/**
 * Video plane call back function pool
 */
typedef struct
{
    UPTR au4CbFunc[VDP_CB_FUNC_NS];
    UPTR au4CbFuncCRC[VDP_CB_FUNC_NS];
} VDP_CB_FUNC_T;

#if 1 //for capture logo
typedef struct
{
	UINT8 dram_from;  
	UINT8 u1ColorMode;
	UINT8 u1BitRes;

	//UINT8 HBlockNum;
	//UINT8 BlockSize;
	UINT8 BlockMode;
	UINT8 FrameNum;
	UINT32 YMsbAddr;
	UINT32 YLsbAddr;
	UINT32 UMsbAddr;
	UINT32 ULsbAddr;
	UINT32 VMsbAddr;
	UINT32 VLsbAddr;
	UINT32 PictureWidth;
	UINT32 PictureHeight;
	UINT32 DramPitch;

	BOOL   bDramSnakeMapping;
	UINT8  u1DramFlipMirror; // 1 : means pscan write dram flip/mirror  ; 0 : means scaler read flip mirror
	UINT8  u1CfgFlipMirror;  // 0 : means normal ; 1 : means mirror ; 2 : means flip ; 3 : means flip & mirror 
}SCALER_INFO_T;

typedef struct{
	UINT8  dram_from;
	UINT8  u1BlockMode;
	UINT8  u1OutMode;
	UINT8  u1BitRes;
	UINT8  u1ColorMode;
	UINT8  u1AFBottom;
	UINT8  u1AFYUVIndex;
	UINT16 u2Width;
	UINT16 u2Height;
    UINT16 u2DramPitch;
	
	UINT8  u1YFrameNum; //1W1R:Y=2  1W4R:Y=5 FLIP_MIRROR_1W1R:Y=3 FLIP_MIRROR_CBOB_1W4R:Y=6
	UINT8  u1CFrameNUm; //1W1R:C=2  1W4R:C=5 FLIP_MIRROR_1W1R:C=3 FLIP_MIRROR_CBOB_1W4R:C=5
	UINT8  u1DramFlipMirror; // 1 : means pscan write dram flip/mirror  ; 0 : means scaler read flip mirror
	UINT8  u1CfgFlipMirror;  // 0 : means normal ; 1 : means mirror ; 2 : means flip ; 3 : means flip & mirror 
	
	UINT32 u4YStartMSBAddr;
	UINT32 u4YStartLSBAddr;
	UINT32 u4UStartMSBAddr;
	UINT32 u4UStartLSBAddr;
	UINT32 u4VStartMSBAddr;
	UINT32 u4VStartLSBAddr;

	 BOOL   bDramSnakeMapping; 
}DI_DISP_INFO_T;

typedef struct _B2R_DISP_INFO_T
{
	UINT8 dram_from;

    UINT8  u1OutMode;          // 0 Progress mode;1 interlace mode
    UINT8  u1BlockMode;        // 0:64x32 ; 1:16x32 ; others: raster mode

	UINT32 u4Pitch;            // Line Size of buffer
    UINT32 u4YStart;           // Y address
    UINT32 u4CStart;           // C address
    UINT32 u4With;             // active  width
    UINT32 u4Height;           // active  height
} B2R_DISP_INFO_T;

typedef union
{
    SCALER_INFO_T scaler_inf;
    B2R_DISP_INFO_T B2R_inf;
    DI_DISP_INFO_T DI_inf;
}DRAM_INFO_T;

#endif

// Video Image Module Color Mode.
#define VDP_IMAGE_420_MODE          (1 << 0)		///< 420 mode
#define VDP_IMAGE_422_MODE          (1 << 1)		///< 422 mode
#define VDP_IMAGE_444_MODE          (1 << 2)		///< 424 mode

///-----------------------------------------------------------------------------
/// Public functions
///-----------------------------------------------------------------------------

/**
 * VDP init function.
 *
 * @param void
 * @return void
 */
EXTERN void VDP_Init(void);

/**
 * VDP stop function.
 *
 * @param void
 * @return void
 */
EXTERN void VDP_Stop(void);

/**
 * VDP reset function.
 *
 * @param ucVdpId specify the video plane id.
 * @return void
 */
EXTERN void VDP_Reset(UCHAR ucVdpId);

/**
 * VDP query function, show status in CLI with debug level 1.
 *
 * @param void
 * @return void
 */
EXTERN void VDP_QueryStatus(void);

/**
 * VDP query function, show DTV status in CLI with debug level 1.
 *
 * @param void
 * @return void
 */
EXTERN void VDP_PrintDtvInfo(void);

/**
 * VDP set video plane enable.
 *
 * @param ucVdpId specify the video plane id.
 * @param ucEnable value 0:disable, 1:enable.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetEnable(UCHAR ucVdpId, UCHAR ucEnable);

/**
 * VDP set video plane mode.
 *
 * @param ucVdpId specify the video plane id.
 * @param ucMode VDP_MODE_NORMAL or VDP_MODE_BG.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetMode(UCHAR ucVdpId, UCHAR ucMode);

/**
 * VDP set video plane with freeze feature.
 *
 * @param ucVdpId specify the video plane id.
 * @param ucFreeze value 0:disable, 1:freeze feature enable.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetFreeze(UCHAR ucVdpId, UCHAR ucFreeze);
EXTERN void VDP_SetFreezeEx(UCHAR ucVdpId, UCHAR ucFreeze);

/**
 * VDP query if nonlinear is supported.
 *
 * @param ucVdpId specify the video plane id.
 * @return TRUE/FALSE
 */
EXTERN BOOL VDP_IsSupportNonlinear(UCHAR ucVdpId);


/**
 * VDP query if overscan is supported.
 *
 * @param ucVdpId specify the video plane id.
 * @return TRUE/FALSE
 */
EXTERN BOOL VDP_IsSupportOverscan(UCHAR ucVdpId);
EXTERN BOOL VDP_IsSupportAspectRatio(UCHAR ucVdpId);

/**
 * VDP set overscan region.
 *
 * @param ucVdpId specify the video plane id.
 * @param rRegion overscan percentage setting.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetOverScan(UCHAR ucVdpId, VDP_OVERSCAN_REGION_T rRegion);

/**
 * VDP set source region.
 *
 * @param ucVdpId specify the video plane id.
 * @param ucSrcFullRegion value 1:full region and ignore parameter rSrcRegion.
 * @param rSrcRegion source region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetSrcRegion(UCHAR ucVdpId, UCHAR ucSrcFullRegion, VDP_REGION_T rSrcRegion);

/**
 * VDP set output region.
 *
 * @param ucVdpId specify the video plane id.
 * @param ucOutFullRegion value 1:full region and ignore parameter rOutRegion.
 * @param rOutRegion output region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetOutRegion(UCHAR ucVdpId, UCHAR ucOutFullRegion, VDP_REGION_T rOutRegion);

/**
 * VDP set dolby source region.
 *
 * @param ucVdpId specify the video plane id.
 * @param rSrcRegion source region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetDolbySrcRegion(UCHAR ucVdpId, VDP_REGION_T rDolbySrcRegion);

/**
 * VDP set dolby output region.
 *
 * @param ucVdpId specify the video plane id.
 * @param rOutRegion output region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetDolbyOutRegion(UCHAR ucVdpId, VDP_REGION_T rDolbyOutRegion);

/**
 * VDP set background color.
 *
 * @param ucVdpId specify the video plane id.
 * @param u4BgColor specify the color in RGB color mode, 1 byte per color component.(e.g. Red: 0xFF0000).
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetBg(UCHAR ucVdpId, UINT32 u4BgColor);

/**
 * VDP query video plane enable/disable status.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucEnable value 0:disable, 1:enable.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetEnable(UCHAR ucVdpId, UCHAR* pucEnable);

/**
 * VDP query video plane mode.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucMode VDP_MODE_NORMAL or VDP_MODE_BG.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetMode(UCHAR ucVdpId, UCHAR* pucMode);

/**
 * VDP query video plane with freeze feature.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucFreeze value 0:disable, 1:freeze feature enable.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetFreeze(UCHAR ucVdpId, UCHAR* pucFreeze);

/**
 * VDP query video plane with freeze feature.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucFreezable value 0: can't freeze, 1:freeze feature is available.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetFreezable(UCHAR ucVdpId, UCHAR* pucFreezable);

/**
 * VDP query overscan region.
 *
 * @param ucVdpId specify the video plane id.
 * @param prRegion overscan percentage setting.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetOverScan(UCHAR ucVdpId, VDP_OVERSCAN_REGION_T* prRegion);

/**
 * VDP query source region.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucSrcFullRegion value 1:full region and ignore parameter rSrcRegion.
 * @param prSrcRegion source region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetSrcRegion(UCHAR ucVdpId, UCHAR* pucSrcFullRegion, VDP_REGION_T* prSrcRegion);

/**
 * VDP query output region.
 *
 * @param ucVdpId specify the video plane id.
 * @param pucOutFullRegion value 1:full region and ignore parameter rOutRegion.
 * @param prOutRegion output region description.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetOutRegion(UCHAR ucVdpId, UCHAR* pucOutFullRegion, VDP_REGION_T* prOutRegion);

EXTERN UINT32 _VDP_GetOutRegion(UCHAR ucVdpId, VDP_REGION_T* prOutRegion);

/**
 * VDP query background color.
 *
 * @param ucVdpId specify the video plane id.
 * @param pu4BgColor the color in RGB color mode, 1 byte per color component.(e.g. Red: 0xFF0000).
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_GetBg(UCHAR ucVdpId, UINT32* pu4BgColor);

/**
 * Video plane call back function register
 *
 * @param eFuncType specify the call back function type.
 * @param u4FuncPtr specify the call back function pointer.
 * @param u4Arg1 specify argument #1.
 * @param u4Arg2 specify argument #2.
 * @return void
 */
EXTERN void VDP_RegCbFunc(VDP_CB_FUNC_ENUM_T eFuncType, UPTR u4FuncPtr, UINT32 u4Arg1, UINT32 u4Arg2);

/**
 * VDP set scaler to pixel based or percentage based.
 *
 * @param u4OnOff = 0 refer to scaler API change to percentage based. (Default)
 * @param u4OnOff = 1 refer to scaler API change to pixel based.
 * @return void
 */
EXTERN void VDP_SetScalerPixelBased(UINT32 u4OnOff);


/**
 * VDP Get scaler to pixel based or percentage based.
 *
 * @param void
 * @return 1 refer to scaler API change to pixel based.
 */
EXTERN UINT32 VDP_GetScalerPixelBased(void);


/**
 * VDP set SCPOS porch.
 *
 * @param u4VdpId specify the video plane id.
 * @param bPorchType SV_VPORCH_CURRENT/SV_HPORCH_CURRENT
 * @param wValue porch value of 1000-based.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetScposPorch(UINT32 u4VdpId, UINT8 bPorchType, UINT16 wValue);
/**
 * VDP get VDP_RTNeedSuperGameMode or not (WFD only)
 *
 * @param u4VdpId specify the video plane id.
 * @return TRUE or FALSE
 */
EXTERN BOOL VDP_RTNeedSuperGameMode(UCHAR ucVdpId);

/**
 * VDP set game mode on/off
 *
 * @param u4VdpId specify the video plane id.
 * @param u1OnOff enable/disable game mode.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetGameMode(UINT32 u4VdpId, UINT8 u1OnOff);

//mtk01140 20071010 3D game mode
/**
 * VDP set 3D game mode on/off
 *
 * @param u4VdpId specify the video plane id.
 * @param u1OnOff enable/disable 3D game mode.
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_Set3DGameMode(UINT32 u4VdpId, UINT8 u1OnOff);


/**
 * Get delay time of PQ
 *
 * @return delay time in 90000
 */
EXTERN UINT32 VDP_GetDelayTime(void);

/**
 * VDP set scaler DRAM 
 *
 * @param u4VdpId specify the video plane id.
 * @param u4X x offset to fill DRAM data
 * @param u4Y y offset to fill DRAM data
 * @param u4Width pixel width to fill DRAM data
 * @param u4Height pixel height to fill DRAM data
 * @param u4GBRYCbCr pixel data to fill DRAM
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetScalerDRAM(UINT32 u4VdpId, UINT32 u4X, UINT32 u4Y, UINT32 u4Width, UINT32 u4Height, UINT32 u4GBRYCbCr);
EXTERN UINT32 VDP_SetScalerPattern(UINT32 u4VdpId, UINT32 u4OnOff);


EXTERN UINT8 VDP_GetDIFilm32Enable(UCHAR ucVdpId);

EXTERN UINT32 VDP_SetNoOverScan(UCHAR ucVdpId, UCHAR ucNoOverScan);

EXTERN UINT32 VDP_LBoxDetectMode(UCHAR ucVdpId, UINT32 u4DetectPeriod, UINT32 Ratio);

EXTERN void VDP_GetActiveImageArea(UINT16 *x, UINT16 *y, UINT16 *w, UINT16 *h);


#define BLANK_FRAMERATE_CHANGED  (1)
#define BLANK_RESOLUTION_CHANGED (1<<1)

EXTERN void VDP_SetBlankedReason(UINT32 u4BlankedReason, BOOL fgOnOff);
EXTERN void VDP_SetPlanesBlank(UINT32 u4BlankedReason);
EXTERN void VDP_SetPlanesUnblank(void);

EXTERN UINT32 VDP_GetMinSrcRegionHeight(void);
EXTERN UINT32 VDP_GetMinSrcRegionWidth(void);
#if SUPPORT_DOT_TO_DOT_PATTERN
EXTERN void VDP_DotToDotPattern(UINT8 ucOnOff,UINT32 u4RGBColor);
#endif

/**
 * VDP set force 2D mode
 * @param  ucVdpId    
 * @param  ucMode    0: DRAM mode, 1: Scaler display mode, 2: n/a, 3: pscan display mode, 4: mjc display mode
 * @return VDP_SET_ERROR or VDP_SET_OK
 */
EXTERN UINT32 VDP_SetDispmode(UCHAR ucVdpId, UCHAR ucMode);
EXTERN UINT32 VDP_GetDispmode(UCHAR ucVdpId);
EXTERN UINT32 VDP_SetScaler10BitMode(UINT32 u4VdpId, UINT32 u4Mode);
EXTERN UINT32 VDP_GetScaler10BitMode(UINT32 u4VdpId);

EXTERN UINT32 VDP_UpdateCmdDelay(UINT32 u4VdpId);   // for MW usage
EXTERN UINT32 _VDP_UpdateCmdDelay(UINT32 u4VdpId);  // for internal used without mutex protect
EXTERN UINT32 VDP_GetCmdDelay(void);
EXTERN UINT32 VDP_GetPanelType(void);
#if 1 //for capture log
EXTERN void VDP_GetDramInf(UINT8 bPath,DRAM_INFO_T* inf);
#endif
EXTERN UINT32 VDP_SetDownBWMode(UINT8 u1OnOff);
EXTERN UINT32 VDP_IsSupportDotByDot(void);
EXTERN UINT32 VDP_CheckScalerDisplayMode(void);
EXTERN void VDP_SetAirPlayerPlay(BOOL bIsAirPlayerPlay);
EXTERN BOOL VDP_GetAirPlayerPlay(void);

#endif    // VDP_IF_H

