/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: vdp_prescale.h,v $
 * $Revision: #3 $
 *
 *---------------------------------------------------------------------------*/

/*****************************************************************************
*  Notes: 
*****************************************************************************/
#ifndef VDP_PRESCALE_H
#define VDP_PRESCALE_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "imgrz_if.h"

//-----------------------------------------------------------------------------
// Configurations
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
typedef struct _VDP_PRESCALE_TG_BUF_T
{
    UINT32 u4Pts; // input
    UINT32 u4YTgBase;
    UINT32 u4CTgBase;
} VDP_PRESCALE_TG_BUF_T;

typedef struct _VDP_PRESCALE_TG_INFO_T
{
    UINT32 u4YTgW;
    UINT32 u4YTgH;
    UINT32 u4YTgBufLen;
    BOOL fgRasterOrder;
} VDP_PRESCALE_TG_INFO_T;

typedef struct _VDP_PRESCALE_RZFIN_CB_T
{
    UINT32 u4YTgBase;
    UINT32 u4CTgBase;
    UINT32 u4PTS;
} VDP_PRESCALE_RZFIN_CB_T;

typedef BOOL (*PFN_PRESCALE_GETTGBUF_CB)
(
    VOID *pvTag,
    VDP_PRESCALE_TG_BUF_T* prBuf
);

typedef BOOL (*PFN_PRESCALE_RZFIN_CB)
(
    VOID *pvTag,
    VDP_PRESCALE_RZFIN_CB_T* prCbInfo
);

typedef UCHAR (*PFN_GET_GETRZBUF) (
    UCHAR ucFbgId, UCHAR ucSrcFbId, UCHAR ucTaskId, UINT32 u4Delay, UCHAR* pucRetValue
);

typedef void (*PFN_GET_GETRZBUFADDR) (
    UCHAR ucFbgId, UCHAR ucFbId, UPTR *pu4AddrY, UPTR *pu4AddrC
);
typedef void (*PFN_SET_SETRZBUFSTATUS) (
    UCHAR ucFbgId, UCHAR ucFbId, UCHAR ucFbStatus
);
typedef BOOL (*PFN_SET_SETRZBUFFLAG) (
    UCHAR ucFbgId, UCHAR ucFbId, UCHAR ucSetType, UINT32 u4Flag
);


//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

#define VDP_PRESCALE_RZ_ID_DEFAULT E_SECND_RESIZER
#define VDP_PRESCALE_RZ_ID_UNKNOWN 0xFF

//-----------------------------------------------------------------------------
/** Brief of _VDP_Prescale_Init.
 */
//-----------------------------------------------------------------------------
UCHAR _VDP_Prescale_Init(UCHAR ucRzIdMask);

void _VDP_Prescale_Send(UCHAR ucRzId, UCHAR ucFbgId, UCHAR ucFbId, UCHAR ucTaskId,UINT32 u4PTS);

void _VDP_Prescale_Flush(UCHAR ucRzId, BOOL fgIsWait);

void _VDP_Prescale_Close(UCHAR ucRzId);

BOOL _VDP_Prescale_RegRzFinCb(UCHAR ucRzId, PFN_PRESCALE_RZFIN_CB pfnRzFinCb, VOID *pvTag);

BOOL _VDP_Prescale_RegGetTgBufCb(UCHAR ucRzId, 
    PFN_PRESCALE_GETTGBUF_CB pfnGetTgBufCb, VDP_PRESCALE_TG_INFO_T *prTgBufInfo, VOID *pvTag);

UCHAR _VDP_GetRzId(UCHAR ucRzIdMask,UCHAR ucTaskId);

void _VDP_SetPfn_RzGetFB(UCHAR ucRzId,PFN_GET_GETRZBUF pfnGetRzFb);
void _VDP_SetPfn_RzGetFBAddr(UCHAR ucRzId,PFN_GET_GETRZBUFADDR pfnGetRzFbAddr);
void _VDP_SetPfn_RzSetFBStatus(UCHAR ucRzId,PFN_SET_SETRZBUFSTATUS pfnSetRzFbStatus);
void _VDP_Prescale_OutputInfo(UCHAR u4FbgId,FBM_SEQ_HDR_T* prFbmSeqHdr,FBM_PIC_POSTPROCESS_OUTPUT_T *pOutParam);
void _VDP_SetPfn_RzSetFBFlag(UCHAR ucRzId,PFN_SET_SETRZBUFFLAG pfnSetRzFbFlag);

#endif


