/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/

#ifndef X_HAL_ARM_H
#define X_HAL_ARM_H

#include "x_typedef.h"
#include "x_lint.h"

#define LDR_ALIGN(value,align) (((value)+(align)-1) & ~((typeof(value))(align)-1))

//-----------------------------------------------------------------------------
/** Specify DMA data direction
 *  used in HalDmaBufferStart() / HalDmaBufferDone()
 */
//-----------------------------------------------------------------------------
enum HalDmaDirection {
    HAL_DMA_TO_DEVICE,
    HAL_DMA_FROM_DEVICE,
    HAL_DMA_BIDIRECTION,
    HAL_DMA_NONE,
};

//===========================================================================
// IRQ/FIQ related functions

externC void HalEnableIRQ(void);
externC void HalDisableIRQ(void);
externC void HalEnableFIQ(void);
externC void HalDisableFIQ(void);

#ifndef __KERNEL__
externC UINT32 HalCriticalStart(void);
externC UINT32 HalCriticalSemiStart(void);
externC void HalCriticalEnd(UINT32 u4Flags);
#endif

//===========================================================================
// Cache related functions

externC BOOL HalIsICacheEnabled(void);
externC BOOL HalIsDCacheEnabled(void);
externC UINT32 HalGetICacheSize(void);
externC UINT32 HalGetDCacheSize(void);
externC UINT32 HalGetICacheLineSize(void);
externC UINT32 HalGetDCacheLineSize(void);
externC void HalEnableICache(void);
externC void HalDisableICache(void);
externC void HalEnableDCache(void);
externC void HalDisableDCache(void);
externC void HalEnableCaches(void);
externC void HalDisableCaches(void);
externC void HalDisableL1Caches(void);
externC void HalInvalidateAllCaches(void);
externC void HalInvalidateICache(void);
externC void HalInvalidateICacheSingleLine(UPTR u4Addr);
externC void HalInvalidateDCache(void);
externC void HalInvalidateDCacheSingleLine(UPTR u4Addr);
externC void HalFlushDCacheSingleLine(UPTR u4Addr);
externC void HalFlushInvalidateDCacheSingleLine(UPTR u4Addr);
externC void HalInvalidateDCacheMultipleLine(UPTR u4Addr, USIZE u4Size);
externC void HalFlushDCacheMultipleLine(UPTR u4Addr, USIZE u4Size);
externC void HalFlushInvalidateDCacheMultipleLine(UPTR u4Addr, USIZE u4Size);
externC void HalFlushDCache(void);
externC void HalFlushInvalidateDCache(void);
externC void HalupFlushInvalidateDCache(void);
externC void HalFlushWriteBuffer(void);
externC void HalInstructionMemoryBarrier(void);
externC void HalIOMMUInvalidateDCacheMultipleLine(UPTR u4VirAddr, UPTR u4PhyAddr, USIZE u4Size);
externC void HalIOMMUFlushDCacheMultipleLine(UPTR u4VirAddr, UPTR u4PhyAddr, USIZE u4Size);
externC void HalIOMMUFlushInvalidateDCacheMultipleLine(UPTR u4VirAddr, UPTR u4PhyAddr, USIZE u4Size);

extern UINT32 g_u4Dcachelinesize;

//-----------------------------------------------------------------------------
/** Called when DMA on a buffer is about to start.
 *
 *  This function will do proper cache operation.
 *  CPU should not access DMA buffer after this function call.
 *
 *  @param pBuffer:  Virtual address of the buffer
 *  @param size: Size of the buffer.
 *  @param direction: the direction of DMA
 */
//-----------------------------------------------------------------------------
#ifndef __GNUC__
externC void HalDmaBufferStart(void *pBuffer, USIZE size, enum HalDmaDirection direction);
#else
static inline void HalDmaBufferStart(void *pBuffer, USIZE size, enum HalDmaDirection direction)
{
    switch (direction)
    {
        case HAL_DMA_TO_DEVICE:
            HalFlushDCacheMultipleLine((UPTR)pBuffer, size);
            break;

        case HAL_DMA_FROM_DEVICE:
            HalInvalidateDCacheMultipleLine((UPTR)pBuffer, size);
            break;

        case HAL_DMA_BIDIRECTION:
            HalFlushInvalidateDCacheMultipleLine((UPTR)pBuffer, size);
            break;

        default:
            break;
    }
}

static inline void HalIOMMUDmaBufferStart(void *pVirBuffer, void * pPhyBuffer, USIZE size, enum HalDmaDirection direction)
{
    switch (direction)
    {
        case HAL_DMA_TO_DEVICE:
            HalIOMMUFlushDCacheMultipleLine((UPTR)pVirBuffer, (UPTR)pPhyBuffer, size);
            break;

        case HAL_DMA_FROM_DEVICE:
            HalIOMMUInvalidateDCacheMultipleLine((UPTR)pVirBuffer, (UPTR)pPhyBuffer, size);
            break;

        case HAL_DMA_BIDIRECTION:
            HalIOMMUFlushInvalidateDCacheMultipleLine((UPTR)pVirBuffer, (UPTR)pPhyBuffer, size);
            break;

        default:
            break;
    }
}
#endif

//-----------------------------------------------------------------------------
/**  Called when DMA on a buffer is done.
 *
 *  Device DMA should not access DMA buffer after this function call.
 *
 *  NOTE:
 *     The parameter should match that of BSP_DmaBufferStart
 *
 *  @param pBuffer: Virtual address of the buffer
 *  @param size: Size of the buffer.
 *  @param direction: the direction of DMA
 */
//-----------------------------------------------------------------------------
#ifndef __GNUC__
externC void HalDmaBufferDone(void *pBuffer, USIZE size, enum HalDmaDirection direction);
#else
static inline void HalDmaBufferDone(void *pBuffer, USIZE size, enum HalDmaDirection direction)
{
    switch (direction)
    {
        case HAL_DMA_FROM_DEVICE:
        case HAL_DMA_BIDIRECTION:
            HalInvalidateDCacheMultipleLine((UPTR)pBuffer, size);
            break;

        case HAL_DMA_TO_DEVICE:
        default:
            break;
    }
}

static inline void HalIOMMUDmaBufferDone(void *pVirBuffer, void *pPhyBuffer, USIZE size, enum HalDmaDirection direction)
{
    switch (direction)
    {
        case HAL_DMA_FROM_DEVICE:
        case HAL_DMA_BIDIRECTION:
            HalIOMMUInvalidateDCacheMultipleLine((UPTR)pVirBuffer, (UPTR)pPhyBuffer, size);
            break;

        case HAL_DMA_TO_DEVICE:
        default:
            break;
    }
}
#endif

//===========================================================================
// MMU related functions

externC BOOL HalIsMMUEnabled(void);
externC void HalEnableMMU(void);
externC void HalDisableMMU(void);
externC void HalInvalidateTLB(void);
externC BOOL HalInitMMU(UINT32 u4Addr);
externC void HalSet1MSectionTable(UINT32 u4PhyAddr, UINT32 u4VirtAdr);
externC void HalSet4KPageTable(UINT32 u4PhyAddr, UINT32 u4VirtAdr, UINT32* pu4PageTableAddress);
externC UINT32 HalGetMMUTableAddress(void);
externC UINT32 HalCpuGetPhyPgd(void);


//===========================================================================
// TCM mode

externC void HalSetInstTCMReg(UINT32 u4BaseAddr, UINT32 u4Size, BOOL fgEnable);
externC void HalSetDataTCMReg(UINT32 u4BaseAddr, UINT32 u4Size, BOOL fgEnable);


//===========================================================================
// Sleep mode

externC void HalWaitForInterrupt(void);

//===========================================================================
// Endian related

UINT32 BYTESWAP32(UINT32 u4Value);

// LINT_SUPPRESS_NEXT_EXPRESSION(129)
// INLINE UINT16 BYTESWAP16(UINT16 u2Value);

// LINT_SUPPRESS_NEXT_EXPRESSION(129)
// INLINE UINT16 BYTESWAP16(UINT16 u2Value)
// {
// 	return (UINT16)((u2Value >> 8) | (u2Value << 8));
// }

// HTONL: Convert an UINT32 from host byte order to network byte order
// NTOHL: Convert an UINT32 from network byte order to host byte order
// HTONS: Convert an UINT16 from host byte order to network byte order
// NTOHS: Convert an UINT16 from network byte order to host byte order
#ifdef __KERNEL__
#ifdef __BIG_ENDIAN

#define BIG_ENDIAN
#define HTONL(x)	(x)
#define NTOHL(x)	(x)
#define HTONS(x)	(x)
#define NTOHS(x)	(x)

#else	// __BIG_ENDIAN

#define LITTLE_ENDIAN
#define HTONL(x)	BYTESWAP32((x))
#define NTOHL(x)	BYTESWAP32((x))
#define HTONS(x)	((UINT16)((((UINT16)(x)) >> 8) | (((UINT16)(x)) << 8)))
#define NTOHS(x)	((UINT16)((((UINT16)(x)) >> 8) | (((UINT16)(x)) << 8)))

#endif	// __BIG_ENDIAN

#endif

#endif  // X_HAL_ARM_H
