/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date  $
 * $RCSfile: drv_hdmi.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/
#ifndef _DRV_EDID_H_
#define _DRV_EDID_H_

#define HDMI_EDID_MUX_SEL
#define HDMI_EDID_SRAM_NUMBER 2

#define HDMIEDID_SIZE_512 512
#define HDMIEDID_SIZE_256 256
#define HDMIEDID_SIZE_DEFAULT 256

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
#define HDMIEDID_DATA_BLOCK_START_INDEX 4
#define HDMIEDID_AUDIO_DATA_BLOCK_TAG_CODE 1
#define HDMIEDID_AUDIO_DATA_BLOCK_SIZE 3
#define HDMIEDID_AUDIO_ATMOS_BIT_MASK 0x1
#define HDMIEDID_CHECKSUM_INDEX 127
#define HDMIEDID_INVALID_DATA_INDEX 127
#define HDMIEDID_INVALID_AUDIO_TYPE_INDEX 128 //if audio type cannot be found, return this macro.

#define HDMIEDID_DATA_BLOCK_TAG_CODE_MASK_VALUE(_a_) (((_a_)&0xE0)>>5)
#define HDMIEDID_DATA_BLOCK_LENGTH_MASK_VALUE(_a_) ((_a_)&0x1F)
#define HDMIEDID_AUDIO_FMT_TYPE_MASK_VALUE(_a_) (((_a_)&0x78)>>3)
#define HDMIEDID_AUDIO_ATMOS_BIT_MASK_VALUE(_a_) ((_a_)&0x1)

typedef enum
{
    HDMIEDID_AUDIO_LPCM_FMT_TYPE  = 0x1,
    HDMIEDID_AUDIO_AC3_FMT_TYPE  = 0x2,
    HDMIEDID_AUDIO_DTS_FMT_TYPE  = 0x7,
    HDMIEDID_AUDIO_EAC3_FMT_TYPE = 0xA,
    HDMIEDID_AUDIO_MAT_FMT_TYPE  = 0xC,
} EDID_AUDIO_TYPE_T;

typedef enum
{
    HDMI2_EDID_TYPE_STANDARD = 0,
    HDMI2_EDID_TYPE_ENHANCE,
    HDMI2_EDID_TYPE_HDMI21,
    HDMI2_EDID_TYPE_MAX,
} HDMI2_EDID_TYPE_T;

typedef enum {
    HDMI_EDID20_TYPE_STANDARD,//no HF_VSDB
    HDMI_EDID20_TYPE_ENHANCE,//has HF_VSDB
    MHL_EDID
}HDMI_EDID_CHECK_MODE_T;

extern UINT8* pu1DrvLoadHdmiEdid1(void);
extern UINT8* pu1DrvLoadHdmiEdid2(void);
extern void vHDMIEDIDWriteEDID1(UINT8 *pu1Edid);
extern void vHDMIEDIDWriteEDID2(UINT8 *pu1Edid);
extern UINT8 u1HDMIEDIDCalCheckSum(UINT8 *pu1EdidBlockx);
extern BOOL fgGetEdidData(UINT8 *pu1EdidBlock,UINT8 u1Index,UINT8 *pu1Data);
extern BOOL fgSetEdidData(UINT8 *pu1EdidBlock,UINT8 u1Index,UINT8 u1Data);
extern UINT8 u1HDMIEDIDGetDetailTimingStartIndex(UINT8 *pu1EdidBlock1);
extern UINT8 u1HDMIEDIDGetDataBlockIndex(UINT8 *pu1EdidBlock1,UINT8 u1TagCode);
extern UINT8 u1HDMIEDIDGetAudioDataBlockIndex(UINT8 *pu1EdidBlock1,UINT8 u1FmtType);
extern void vEDID_AddShortAudioDescriptor(UINT8 *pu1EdidBlock1, EDID_AUDIO_TYPE_T eAudioType, BOOL fgAtmosOn);
extern void vEDID_DeleteShortAudioDescriptor(UINT8 *pu1EdidBlock1, EDID_AUDIO_TYPE_T eAudioType);
extern void vHDMIDisablePortEdid(UINT8 u1Port, BOOL fgDisable);
extern void vHDMI_INTERNAL_EDID_Init(void);
#ifndef CC_MTK_LOADER
extern UINT8 u1HDMIEDIDCheckValid(HDMI_EDID_CHECK_MODE_T eEdidMode);
extern BOOL fgHDMIEDIDReplaceSad(BOOL fgSet,UINT8 u1FormatNum,UINT8 *pu1AudioDataBlock);
extern BOOL fgHDMIEDIDAtmosHandle(BOOL fgSet,UINT8 u1FormatNum,UINT8 *pu1AudioDataBlock);
extern void vHDMIEDIDIndexChangeAllPort(void);
extern void vHDMIEDIDIndexChange(UINT8 u1Port);
#endif
extern void vHDMIEDIDWriteToSram(UINT8 u1Port, UINT8 *pu1Edid, UINT16 u2EdidSize);
#endif

