/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date  $
 * $RCSfile: drv_hdmi.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/
#ifndef _DRV_HDMICOM_H_
#define _DRV_HDMICOM_H_

#include "typedef.h"
#include "x_timer.h"
#include "drv_mhlcom.h"

#define HDMI2_COM_INVALID_FRAME_RATE    1 //Set 1 as invalid frame rate for calculation

#define CCIR_SEL_PORT_1                 3
#define CCIR_SEL_PORT_2                 0
#define CCIR_SEL_PORT_3                 1
#define CCIR_SEL_PORT_4                 2

typedef enum
{
    HDMI2ANA_DELAY_SAOSC,
    HDMI2ANA_DELAY_EQLOCK_HALF,
    HDMI2ANA_DELAY_EQLOCK_FULL,
    HDMI2ANA_DELAY_EQINCREASE,
    HDMI2ANA_DELAY_MAX
}HDMI2ANA_DELAY_ITEM;

typedef struct _HDMI2ANA_LEQ_ITEM
{
    BOOL fgLeak;
    BOOL fgAvg;
    BOOL fgLeak_Increase;
    BOOL fgLeak_Decrease;
    BOOL fgLEQ_Reduce;
}HDMI2ANA_LEQ_ITEM;

typedef struct _HDMI2ANA_CONFIG
{
    UINT16 au2DelayRatio[HDMI2ANA_DELAY_MAX]; //ratio*10%
    HDMI2ANA_LEQ_ITEM tLEQItem;
    UINT8 u1SAOSC_LFSEL;
    UINT8 u1EQOSC_LFSEL;
    BOOL fgSupport3GHDMI20;
    BOOL fgForce40xMode;
}HDMI2ANA_CONFIG;

/**
 * @brief        Set analog SAOSC and EQOSC low-pass filter value
 * @param        u1SAOSC_LFSelValue---- SAOSC low-pass filter value
 * @param        u1EQOSC_LFSelValue---- EQOSC low-pass filter value
*/
extern void vHDMI2ComAnaLFSelValue_Set(UINT8 u1SAOSC_LFSelValue,UINT8 u1EQOSC_LFSelValue);

/**
 * @brief        Support 3G HDMI2.0. Debug method for DTG5334 output such timing
 * @param        fgEnable---- TRUE for enable support, FALSE for disable
*/
extern void vHDMI2ComAnaSupport3GHDMI20(BOOL fgEnable);
/**
 * @brief        Force analog work at 40x mode. Debug method for no SCDC case
 * @param        fgEnable---- TRUE for force 40x mode, FALSE for auto-detect from SCDC
*/
extern void vHDMI2ComAnaForce40xMode(BOOL fgEnable);
/**
 * @brief        Set hdmi digital clock from xtal
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSetHdmiClkFromXtal(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Close hdmi digital clock from xtal
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComCloseHdmiClkFromXtal(E_HDMI_SWITCH_NUM eHdmiPort);

/**
 * @brief        Set hdmi digital clock on
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSetHdmiClkOn(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Set hdmi digital clock off
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSetHdmiClkOff(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Select MHL2 digital clock
*/
extern void vHDMI2ComSetMhl2Clk(void);
/**
 * @brief        Select MHL3 digital clock
*/
extern void vHDMI2ComSetMhl3Clk(void);
/**
 * @brief        Get TMDS sample clock value, and store it in static variable _u4TMDSSampleClock
*/
extern void vHDMI2ComGetTMDSSampleClock(void);
/**
 * @brief        Enable/Disable HDCP ddc
 * @param        eHdmiPort---- HDMI port number
 * @param        u1En---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComSetHdcpDdcEn(E_HDMI_SWITCH_NUM eHdmiPort,UINT8 u1En);
/**
 * @brief        Select HDCP for MHL or HDMI
 * @param        u1Mhl---- TRUE for select MHL, FALSE for select HDMI
*/
extern void vHDMI2MHLComHdcpSel(UINT8 u1Mhl);
/**
 * @brief        Select CCIR0 clock source port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSetCcir0Clk(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Select CCIR1 clock source port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSetCcir1Clk(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Select W2D video clock&data source port
 * @param        eHdmiPort---- HDMI port number
 * @param        u1W2DIdx---- W2D index(0,1)
*/
extern void vHDMI2ComSelW2DVideo(E_HDMI_SWITCH_NUM eHdmiPort,UINT8 u1W2DIdx,BOOL bBypassMode);
/**
 * @brief        Select main(CCIR0) video data source port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSelMainVideo(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Select audio data source port/APLL2 control port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSelAudio(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Select sub(CCIR1) video data source port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComSelSubVideo(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Check audio PLL control type
 * @param        u2HdmiBase---- HDMI port number
 * @retval       TRUE for APLL control, FALSE for DPLL control
*/
extern BOOL fgHDMI2ComGetAudioPllCtrl(UINT16 u2HdmiBase);
/**
 * @brief        Check realtime HDCP2x under decrypting or not
 * @param        u2HdmiBase---- HDMI port number
 * @retval       TRUE for under decrypting, FALSE for not
*/
extern BOOL fgHDMI2ComIsHdcp2xDecyptOn(UINT16 u2HdmiBase);
/**
 * @brief        Select digital working at HDMI(not MHL)
*/
extern void vHDMI2ComSetHdmiMode(void);
/**
 * @brief        Select digital working at MHL2 normal mode
*/
extern void vHDMI2ComSetMHL2NorMode(void);
/**
 * @brief        Select digital working at MHL2 PP mode
*/
extern void vHDMI2ComSetMHL2PpMode(void);
/**
 * @brief        Select digital working at MHL3 normal mode
*/
extern void vHDMI2ComSetMHL3NorMode(void);
/**
 * @brief        Select digital working at MHL3 PP mode
*/
extern void vHDMI2ComSetMHL3PpMode(void);
/**
 * @brief        Get HDMI H total value
 * @param        u2HdmiBase---- Base address by port
 * @retval       H total value
*/
extern UINT32 dwHDMI2ComHTotal(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI V total even value
 * @param        u2HdmiBase---- Base address by port
 * @retval       V total even value
*/
extern UINT32 dwHDMI2ComVTotalEven(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI V total odd value
 * @param        u2HdmiBase---- Base address by port
 * @retval       V total odd value
*/
extern UINT32 dwHDMI2ComVTotalOdd(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI V total value. For progressive signal, equal to V total even. For interlaced signal, equal to V total even + V total odd
 * @param        u2HdmiBase---- Base address by port
 * @retval       V total value
*/
extern UINT32 dwHDMI2ComVTotal(UINT16 u2HdmiBase);
extern UINT32 dwHDMI2ComVTotalRealTime(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI H active value
 * @param        u2HdmiBase---- Base address by port
 * @retval       H active value
*/
extern UINT32 dwHDMI2ComHActive(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI V active value
 * @param        u2HdmiBase---- Base address by port
 * @retval       V active value
*/
extern UINT32 dwHDMI2ComVActive(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI frame rate
 * @param        u2HdmiBase---- Base address by port
 * @retval       Frame rate
*/
extern UINT32 dwHDMI2ComFrameRate(UINT16 u2HdmiBase);
extern UINT32 dwHDMI2ComFrameRateRealTime(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI pixel clock
 * @param        u2HdmiBase---- Base address by port
 * @retval       Pixel clock in Hz
*/
extern UINT32 dwHDMI2ComPixelClk(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI H-sync clock
 * @param        u2HdmiBase---- Base address by port
 * @retval       H-sync clock
*/
extern UINT32 dwHDMI2ComHClk(UINT16 u2HdmiBase);
/**
 * @brief        Check H resolution stable or not
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for stable, FALSE for unstable
*/
extern BOOL fgHDMI2ComGetHresStb(UINT16 u2HdmiBase);
/**
 * @brief        Chekc V resolution stable or not
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for stable, FALSE for unstable
*/
extern BOOL fgHDMI2ComGetVresStd(UINT16 u2HdmiBase);

/**
 * @brief        before check vde stable need to reset interrupt
 * @param        u2HdmiBase---- Base address by port
 * @retval       None
*/
extern void vHDMI2ComRstVdeStableInt(UINT16 u2HdmiBase);
/**
 * @brief        Chekc V resolution stable or not
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for stable, FALSE for unstable
*/
extern BOOL fgHDMI2ComGetVresStd(UINT16 u2HdmiBase);
/**
 * @brief        Check HDMI analog CRC
                 Assume progressive signal if get only one CRC value
                 Assume interlaced signal if get two CRC values
                 Assume signal unstable(CRC fail) if get more than two CRC values
 * @param        u2HdmiBase---- Base address by port
 * @param        u2Times---- Check times
 * @retval       Don't care
*/
extern BOOL fgHDMI2ComAnaCRC(UINT16 u2HdmiBase,UINT16 u2Times);
/**
 * @brief        Get realtime CRC once. Usually use it to compare with Golden CRC Value
 * @param        u2HdmiBase---- Base address by port
 * @param        *pu4Crc---- Pointer to CRC value
 * @retval       Don't care
*/
extern BOOL fgHDMI2ComGetSingleCRC(UINT16 u2HdmiBase,UINT32 *pu4Crc);
/**
 * @brief        Check HDMI video CRC
                 Assume progressive signal if get only one CRC value
                 Assume interlaced signal if get two CRC values
                 Assume signal unstable(CRC fail) if get more than two CRC values
 * @param        u2HdmiBase---- Base address by port
 * @param        u2Times---- Check times
 * @retval       Don't care
*/
extern BOOL fgHDMI2ComCRC(UINT16 u2HdmiBase,UINT16 u2Times);
/**
 * @brief        Check HDMI interlaced or progressive
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for interlaced,FALSE for progressive
*/
extern BOOL fgHDMI2ComGetInterlaced(UINT16 u2HdmiBase);
/**
 * @brief        Check HDMI AV-mute
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for AV-mute, FALSE for not
*/
extern BOOL fgHDMI2ComGetAvMute(UINT16 u2HdmiBase);
/**
 * @brief        Clear HDMI AVMute status
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for clear AV mute, 0 for not
*/
extern void vHDMI2ComClrAvMute(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Enable/Disable HDMI audio mute
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable mute, 0 for disable mute
*/
extern void vHDMI2ComAudMuteSet(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Enable/Disable HDMI video mute
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable mute, 0 for disable mute
*/
extern void vHDMI2ComVideMuteSet(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Set HDMI black picture(mute) data
                 The black picture data depends on the color space
 * @param        u2HdmiBase---- Base address by port
 * @param        u1IsRgb---- 1 for RGB, 0 for YCbCr
*/
extern void vHDMI2ComVideoBlackSet(UINT16 u2HdmiBase,UINT8 u1IsRgb);
/**
 * @brief        Set HDMI blanking data
                 The blanking data depends on the color space
 * @param        u2HdmiBase---- Base address by port
 * @param        u1IsRgb---- 1 for RGB, 0 for YCbCr
*/
extern void vHDMI2ComVideoBlankingSet(UINT16 u2HdmiBase, UINT8 u1IsRgb);
/**
 * @brief        Enable/Disable HDMI AAC
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable, 0 for disable
*/
extern void vHDMI2ComAacEn(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Get HDMI mode status
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for HDMI mode, FALSE for DVI mode
*/
extern BOOL fgHDMI2ComGetHdmiMode(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI Tx scrambling en/dis status
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for scrambling enabled, FALSE for disabled
*/
extern BOOL fgHDMI2ComGetScramblingBySCDC(UINT16 u2HdmiBase);
/**
 * @brief        Get TMDS clk ratio by scdc
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for 1/40 clk ratio, FALSE for 1/10 clk ratio
*/
extern BOOL fgHDMI2ComGetTmdsClkRatio(UINT16 u2HdmiBase);
/**
 * @brief        Get TMDS clk status
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE clk active, FALSE for no clk
*/
extern BOOL fgHDMI2ComGetCKDT(UINT16 u2HdmiBase);
/**
 * @brief        Get SCDT status
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for sync active, FALSE for no SCDT
*/
extern BOOL fgHDMI2ComGetSCDT(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI CTS value
 * @param        u2HdmiBase---- Base address by port
 * @retval       CTS value
*/
extern UINT32 dwHDMI2ComGetCTS(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI N value
 * @param        u2HdmiBase---- Base address by port
 * @retval       N value
*/
extern UINT32 dwHDMI2ComGetN(UINT16 u2HdmiBase);
/**
 * @brief        Get HDMI FS value
 * @param        u2HdmiBase---- Base address by port
 * @retval       FS value
*/
extern UINT8 bHDMI2ComGetFs(UINT16 u2HdmiBase);
/**
 * @brief        Get TMDS clk frequency, cal from pixel frequency, only avaliable at stable signal
 * @param        u2HdmiBase---- Base address by port
 * @retval       TMDS clk in Hz
*/
extern UINT32 bHDMI2ComGetTMDSClkFreq(UINT16 u2HdmiBase);
/**
 * @brief        Calculate the audio sample rate from N/CTS/TMDS clock
 * @param        u2HdmiBase---- Base address by port
 * @retval       Audio sample rate
*/
extern UINT8 bHDMI2AudSampleRateCal(UINT16 u2HdmiBase);
/**
 * @brief        Set software program N value
 * @param        u2HdmiBase---- Base address by port
 * @param        u4N---- N value
*/
extern void vHDMI2ComAudSetN(UINT16 u2HdmiBase,UINT32 u4N);
/**
 * @brief        Set software program CTS value
 * @param        u2HdmiBase---- Base address by port
 * @param        u4CTS---- CTS value
*/
extern void vHDMI2ComAudSetCTS(UINT16 u2HdmiBase,UINT32 u4CTS);
/**
 * @brief        Set software program FS value
 * @param        u2HdmiBase---- Base address by port
 * @param        u4Fs---- FS value
*/
extern void vHDMI2ComAudSetFs(UINT16 u2HdmiBase,UINT8 u4Fs);
/**
 * @brief        Get audio channel status
 * @param        *HdmiRx---- Pointer to HDMI rx object struct, the audio channel status is the member HdmiRx->_AudStatus.chlStatus, see HdmiRxAudioChannelStatus_t
*/
extern void vHDMI2ComGetChannelStatus(UINT16 u2HdmiBase, HdmiRxAudioChannelStatus_t *pChlStatus);
/**
 * @brief        Get audio layout status
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for up to 8 channels, FALSE for 2 channels
*/
extern BOOL fgHDMI2ComGetAudioLayout(UINT16 u2HdmiBase);
/**
 * @brief        Check HDMI audio FIFO status
 * @param        u2HdmiBase---- Base address by port
 * @retval       Audio FIFO overrun/underrun/normal
*/
extern UINT8 bHDMI2ComGetAudFifo(UINT16 u2HdmiBase);
/**
 * @brief        Select HDMI audio APLL or DPLL status
 * @param        u2HdmiBase---- Base address by port
 * @param        u1On---- 1 for APLL, 0 for DPLL
*/
extern void vHDMI2ComAudStatusSel(UINT16 u2HdmiBase,UINT8 u1On);
/**
 * @brief        Get deep color  status
 * @param        u2HdmiBase---- Base address by port
 * @retval       Enum value for deep color, see HdmiRxBitDepth_t
*/
extern HdmiRxBitDepth_t bHDMI2ComGetDeepSta(UINT16 u2HdmiBase);
/**
 * @brief        Get input TMDS clock in KHz
 * @param        u2HdmiBase---- Base address by port
 * @retval       TMDS clock in KHz
*/
extern UINT32 u4HDMI2ComGetRefClkKhz(UINT16 u2HdmiBase);
/**
 * @brief        Get infoframe data
 * @param        *ptRxInfo---- Pointer to HdmiRxInfo_t struct
 * @param        u2HdmiBase---- Base address by port
 * @param        Type---- Infoframe type
*/
extern void vHDMI2ComGetInfoframe(HdmiRxInfo_t *ptRxInfo,UINT16 u2HdmiBase,UINT8 Type);
/**
 * @brief        Parse video info from AVI infoframe
 * @param        u2HdmiBase---- Base address by port
 * @retval       Video info struct, see HdmiVideoInfo_t
*/
extern HdmiVideoInfo_t bHDMI2ComAviParser(UINT16 u2HdmiBase);
/**
 * @brief        set DDC enable or disable
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable,0 for disable
*/
extern void vHDMI2ComSetHdcpDDC(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Clear SCDC status
 * @param        ePort---- HDMI port number
*/
extern void vHDMI2ComClearScdcStatus(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set Scrambling mode enable or disable by sw,used for some 1.4 timing
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable,0 for disable
*/
extern void vHDMI2ComSetScrambling(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Set 420 enable or disable,used for 4k 420 timing
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable 420 mode, 0 for disable
*/
extern void vHDMI2ComSet420(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Do SW reset
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComSwRst(UINT16 u2HdmiBase);
/**
 * @brief        Set 444 to 422 enable or disable
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable,0 for disable
*/
extern void vHDMI2ComSet444To422(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Set dither mode
 * @param        u2HdmiBase----    Base address by port
 * @param        u1DitherMode----  12bit -> 10bit = 0
 *                                         12bit -> 8bit   = 1
 *                                         10bit -> 8bit   = 2
 *                                         dither off        = 3
*/
extern void vHDMI2ComSetDither(UINT16 u2HdmiBase,UINT8 u1DitherMode);
/**
 * @brief        Set bypass video path in HDMI digital IP
 *                 The video path in HDMI digital IP can only handle 12bit or lower deep color
 *                 For 16bit mode we don't expect to cut, should bypass the video path
 * @param        u2HdmiBase----  Base address by port
 * @param        u1En----  Enable or disable
*/
extern void vHDMI2ComBypassVideoPath(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Config HDMI I2S
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComAudCfg(UINT16 u2HdmiBase);
/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        u2HdmiBase---- Base address by port
 * @param        value1----
 * @param        value2----
 * @param        bSDINOVR----
*/
extern void vHDMI2ComAudioSigmaDeltaSetting(UINT16 u2HdmiBase,UINT32 value1, UINT8 value2, UINT8 bSDINOVR);
/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        u2HdmiBase---- Base address by port
 * @param        value1----
 * @param        value2----
 * @param        target----
 * @param        bSDINOVR----
*/
extern void vHDMI2ComAudioSigmaDeltaFineTuneSetting(UINT16 u2HdmiBase,UINT32 value1, UINT8 value2, UINT32 target, UINT8 bSDINOVR);
/**
 * @brief        Select HDMI APLL2 control with hardware mode or software mode
 * @param        u2HdmiBase---- Base address by port
 * @param        bAcrHwModeSel---- 1 for hardware mode, 0 for software mode
*/
extern void vHDMI2ComAcrHwSwModeSel(UINT16 u2HdmiBase, BOOL bAcrHwModeSel);
/**
 * @brief        Check HDMI audio PCW(APLL2 control) hardware mode or software mode
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for hardware mode, FALSE for software mode
*/
extern BOOL fgHDMI20ComAudioPCWMode(UINT16 u2HdmiBase);
/**
 * @brief        Select audio PLL control with APLL or DPLL
 * @param        u2HdmiBase---- Base address by port
 * @param        u1Pll---- 1 for APLL, 0 for DPLL
*/
extern void vHDMI2ComAudPllSel(UINT16 u2HdmiBase,UINT8 u1Pll);
/**
 * @brief        Get HDCP1X status
 * @param        u2HdmiBase---- Base address by port
 * @retval       HDCP1X stauts, see bit map in HdmiRxHdcp1Status_t
*/
extern UINT8 bHDMI2ComGetHdcp1Status(UINT16 u2HdmiBase);
/**
 * @brief        Enable/Disable HDCP2X re-authenticate
 * @param        u2HdmiBase---- Base address by port
 * @param        bEn---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComHDCP2ReAuthSet(UINT16 u2HdmiBase,BOOL bEn);
/**
 * @brief        Enable/Disable HDCP2X repeater mode
 * @param        u2HdmiBase---- Base address by port
 * @param        bEn---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComSetHDCP2RepeaterMode(UINT16 u2HdmiBase,BOOL bEn);
/**
 * @brief        Set HDCP2X repeater parameter
 * @param        u2HdmiBase---- Base address by port
 * @param        bIsHdcp2DownStream----
 * @param        u1DeviceDepth----
 * @param        u1DeviceCount----
 * @param        *pReceiverID----
*/
extern void vHDMI2ComSetHDCP2RepeaterPara(UINT16 u2HdmiBase,BOOL bIsHdcp2DownStream,UINT8 u1DeviceDepth,UINT8 u1DeviceCount,UINT8 *pReceiverID);
/**
 * @brief        Get HDCP error count
 * @param        u2HdmiBase---- Base address by port
 * @retval       HDCP error count
*/
extern UINT16 u2HDMI2ComGetHdcpError(UINT16 u2HdmiBase);
/**
 * @brief        Enable/Disable HDCP interrupts to CPU
 * @param        u2HdmiBase---- Base address by port
 * @param        fgEnable---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComEnHDCP2Int(UINT16 u2HdmiBase,BOOL fgEnable);
/**
 * @brief        HDCP2X init
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComHDCP2Init(UINT16 u2HdmiBase);
/**
 * @brief        Set HDCP2X PUPD
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComHDCP2PUPD(UINT16 u2HdmiBase);
/**
 * @brief        Toggle HDMI digital phy reset -> release
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComDigPhyRst1(UINT16 u2HdmiBase);
/**
 * @brief        Reset/Release HDMI digital phy
 * @param        u2HdmiBase---- Base address by port
 * @param        u1en---- 0 for reset, 1 for release
*/
extern void vHDMI2ComDigPhyRst(UINT16 u2HdmiBase,UINT8 u1en);
/**
 * @brief        Reset HDMI audio ACR(APLL)
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComAcrRst(UINT16 u2HdmiBase);
/**
 * @brief        Reset HDMI audio ACR(DPLL)
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComDAcrRst(UINT16 u2HdmiBase);
/**
 * @brief        Reset HDMI audio FIFO
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComFifoRst(UINT16 u2HdmiBase);
/**
 * @brief        Toggle HDMI digital deep phy reset -> release
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComDeepRst1(UINT16 u2HdmiBase);
/**
 * @brief        Reset/Release HDMI digital deep phy
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for reset, 0 for release
*/
extern void vHDMI2ComDeepRst(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Toggle HDMI HDCP1X reset -> release
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComHdcp1Rst(UINT16 u2HdmiBase);
/**
 * @brief        Toggle HDMI HDCP2X reset -> release
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComHdcp2Rst(UINT16 u2HdmiBase);
/**
 * @brief        HDMI RX digital init
 * @param        u2HdmiBase---- Base address by port
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2ComRxInit(UINT16 u2HdmiBase,E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Enable/Disable 420 to 444 transfer
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable, 0 for disable
*/
extern void vHDMI2ComSet420To444(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Enable/Disable 420 to 442 transfer
 * @param        u2HdmiBase---- Base address by port
 * @param        u1En---- 1 for enable, 0 for disable
*/
extern void vHDMI2ComSet420To422(UINT16 u2HdmiBase, UINT8 u1En);
/**
 * @brief     set 422 to 444 enable or disable,used for 4k 420 timing
 * @param    Base address by port, enable or disable
 * @retval    none
 **/
extern void _vHDMI2ComSet422To444(UINT16 u2HdmiBase,UINT8 u1En);
/**
 * @brief        Print video info parsed from AVI
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComGetVideoInfoByAVI(UINT16 u2HdmiBase);
/**
 * @brief        Print video status
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComGetVideoStatus(E_HDMI_SWITCH_NUM eHdmiPort,UINT16 u2HdmiBase);
/**
 * @brief        Print audio status
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComGetAudioStatus(UINT16 u2HdmiBase);
/**
 * @brief        Check HDMI 420 mode or not
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for 420 mode, FALSE for not
*/
extern BOOL fgHDMI2ComGet420Mode(UINT16 u2HdmiBase);
/**
 * @brief        Check AVI header valid or not
 * @param        u2HdmiBase---- Base address by port
 * @retval       TRUE for valid, FALSE for invalid
*/
extern BOOL fgHDMI2ComIsAVIHeaderValid(UINT16 u2HdmiBase);
/**
 * @brief        Select clock for 420 mode
 * @param        u2HdmiBase---- Base address by port
 * @param        eHdmiPort---- HDMI port number
 * @param        u1En---- 1 for select 420 mode clock, 0 for not
*/
extern void vHDMI2Com420ClkSel(UINT16 u2HdmiBase, E_HDMI_SWITCH_NUM eHdmiPort,UINT8 u1En);
/**
 * @brief        Clear HDMI interrupt
 * @param        u2HdmiBase---- Base address by port
*/
extern void vHDMI2ComClrInt(UINT16 u2HdmiBase);
/**
 * @brief        Select HDMI FS software/hardware mode
 * @param        u2HdmiBase---- Base address by port
 * @param        fgHwSwSel---- TRUE for select software mode, FALSE for select hardware mode
 * @param        u1FsVal---- FS value for software mode
*/
extern void vHDMI2ComFsSel(UINT16 u2HdmiBase,BOOL fgHwSwSel,UINT8 u1FsVal);
/**
 * @brief        Set HDMI color space converter
 * @param        u2HdmiBase---- Base address by port
 * @param        fgEnable---- TRUE for enable color space converter, FALSE for disable
 * @param        fgInRgb---- TRUE for input RGB, FALSE for input YCbCr
 * @param        fgOutRgb---- TRUE for output RGB, FALSE for output YCbCr
 * @param        u1InStd---- Input colormetry
 * @param        u1OutStd---- Output colormetry
*/
extern void vHDMI2ComCSCSet(UINT16 u2HdmiBase,BOOL fgEnable,BOOL fgInRgb,BOOL fgOutRgb,UINT8 u1InStd,UINT8 u1OutStd);
/**
 * @brief        Enable/Disable YC422 extern bit mode
                 The ext bit mode will extern the Y and C data in CH0 to CH1 Y and CH2 C for 12bit data.
                 If it is disabled, Y and C data in CH0 will be discarded, therefore, CH1 Y and CH2 C only have 8 bit data.
 * @param        u2HdmiBase---- Base address by port
 * @param        fgEnable---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComSetExtBitMode(UINT16 u2HdmiBase,BOOL fgEnable);
/**
 * @brief        Enable/Disable Dolby EDR related
 * @param        u1En---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2ComDolbyEdrEn(UINT8 u1En);
/**
 * @brief        Get Dolby EDR metadata
 * @param        *prEdrMetadata---- Pointer to Dolby EDR metadata struct, see HDMI_DOLBY_EDR_METADATA_T
*/
extern void vHDMI2ComGetDolbyEdrMetadata(HDMI_DOLBY_EDR_METADATA_T *prEdrMetadata);
/**
 * @brief        Enable Dolby EDR read vsif use new register
 * @param       ePort---- HDMI port number
*/
extern void vHDMI2ComDolbyEdrVsiEn(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get Signal H Back Porch
 * @param        u2HdmiBase---- Base address by port
*/
extern UINT32 dwHDMI20ComHBPCount(UINT16 u2HdmiBase);
/**
 * @brief        Get Signal V Back Porch
 * @param        u2HdmiBase---- Base address by port
*/
extern UINT32 dwHDMI20ComVBPCount(UINT16 u2HdmiBase);
/**
 * @brief        Get Signal H sync width
 * @param        u2HdmiBase---- Base address by port
 * @param        u1High---- H sync is high or low
*/
extern UINT32 dwHDMI20ComHsyncWidth(UINT16 u2HdmiBase);
extern void vHDMI2DigFixColorDepth(HdmiRxBitDepth_t tdeepSel,E_HDMI_SWITCH_NUM ePort);
extern UINT32 u4HDMI2ComAnaGetCRCValue(UINT16 u2HdmiBase);
extern BOOL fgHDMI2ComAnaGetCRCFAIL(UINT16 u2HdmiBase);
extern BOOL fgHDMI2ComAnaGetCRCReady(UINT16 u2HdmiBase);
extern UINT32 u4HDMI2ComDigGetCRCValue(UINT16 u2HdmiBase);
extern BOOL fgHDMI2ComDigGetCRCFAIL(UINT16 u2HdmiBase);
extern BOOL fgHDMI2ComDigGetCRCReady(UINT16 u2HdmiBase);

/**
 * @brief        Get Signal V sync width
 * @param        u2HdmiBase---- Base address by port
 * @param        u1High---- V sync is high or low
*/
extern UINT32 dwHDMI20ComVsyncWidth(UINT16 u2HdmiBase);

/**
 * @brief        Clear Dolby EDR interrupt
 * @param        Mask of interrupt to clear
*/
extern void vHDMI2ComDolbyEdrClrInt(UINT32 u4IntMask);
/**
 * @brief        Reset ePort's EDID DDC
 */
extern void vHDMI2ComEDIDResetDDC(E_HDMI_SWITCH_NUM ePort);

extern void vHDMI2ComHDR10PlusVsiEn(E_HDMI_SWITCH_NUM ePort);

extern void vHDMI2ComHDR10PlusVsiClrInt(UINT16 u2HdmiBase);

extern void vHDMI2ComGetHDR10PlusVsiMetadata(UINT16 u2HdmiBase, HdmiRxInfo_t *prVsiMetadata);

extern UINT32 dwHDMI2ComGetBits(HDMI_HDR_EMP_MD_T *prHdmi_DyMdPKT, UINT8 u1Bits);

extern void vHDMI2ComDynamicMetaDataEn(E_HDMI_SWITCH_NUM ePort, UINT8 u1En);

extern void vHDMI2ComDynamicMetaDataClrInt(UINT16 u2HdmiBase, UINT8 u1Fifo);

extern void vHDMI2ComGetDynamicMetadata(UINT16 u2HdmiBase, UINT8 u1Len, HDMI_HDR_EMP_MD_T *prDynamicMetadata);

extern void vHDMI2ComDpClkSetting(E_HDMI_SWITCH_NUM ePort, BOOL fgDpClrClk, BOOL fgDpClrClk2x, BOOL fgIsFRLMode);

extern void vHDMI2ComMutePathSel(BOOL fgIsFRLMode);

extern void _vHDMI2SetVrrIntMask(E_HDMI_SWITCH_NUM ePort);

extern void _vHDMI2SetVrrTGenSWMode(E_HDMI_SWITCH_NUM ePort, UINT8 u1En);

extern UINT8 _vHDMI2GetVrrTGenSWMode(E_HDMI_SWITCH_NUM ePort);

extern void _vHDMI2SetVrr420SWMode(E_HDMI_SWITCH_NUM ePort, UINT8 u1En);

/*u1En = 0 : HW MODE control   1: Force vrr mode 3: Force vrr disable ;*/
/*VRR_EN_CLR > VRR_EN_SET>HW_CONTROL MODE ;*/
extern void _vHDMI2SetVrrModeEn(E_HDMI_SWITCH_NUM ePort, UINT8 u1En);

extern void _vHDMI2SetVrr420Mode(E_HDMI_SWITCH_NUM ePort, UINT8 u1En);

extern void _vHDMI2SetVrrHtiming(E_HDMI_SWITCH_NUM ePort, UINT16 u1Hde, UINT16 u1Hs, UINT16 u1HBP);

extern void _vHDMI2SetVrrVtiming(E_HDMI_SWITCH_NUM ePort, UINT16 u1Vde, UINT16 u1Vs, UINT16 u1VBP);
extern void _vHDMI2ClrVrrPkt1(E_HDMI_SWITCH_NUM ePort);
extern UINT8 u1HDMI2GetVrrMode(E_HDMI_SWITCH_NUM ePort);
extern void vHDMI2ComIntEn(UINT16 u2BaseAddr, BOOL fgEnable);
extern void vHDMI2ComSetHdmiModeSwOverWrite(UINT16 u2BaseAddr, UINT8 value);
extern void vHDMI2ComClkGenEnable(UINT8 u1Port, UINT8 bMode);
extern void vHDMI2ComHVresIntEn(UINT16 u2BaseAddr, BOOL fgEnable);
#endif
