/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: drv_lbox.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file drv_lbox.h
 *  LBox driver interface
 */


#ifndef _DRV_LBOX_H_
#define _DRV_LBOX_H_

#include "x_typedef.h"

#define LBOX_ONOFF_AP  (1<<0)
#define LBOX_ONOFF_ADAPTIVE_LUMA (1<<1)

typedef enum
{
    LBOX_IDX_UP  = 0,
    LBOX_IDX_DOWN,
    #if SUPPORT_HORIZONTAL_LBOX
    LBOX_IDX_LEFT,
    LBOX_IDX_RIGHT,
    #endif
    LBOX_IDX_MAX
}LBOX_DIR_INDEX_T;

/**
 * Tunning on/off Letter Box Detection.
 *
 * @param bOnOff Letter Box Detection on/off control.
 * @param bFixed Fix letter box final results or not for debug purpose
 * @return void
 */
void LboxSetOnOff(UINT8 u1Module, UINT8 bOnOff, UINT8 bFixed);


/**
 * Query letter box detection results
 *
 * @param pu4LBoxX pointer for output of left-side letter box x position 
 * @param pu4LBoxY pointer for output of up-side letter box y position 
 * @param pu4Width pointer for output width of letter box
 * @param pu4Height pointer for output height of letter box
 * @param pu4HValid pointer for output to indicate horizontal letter box is valid or not
 * @param pu4VValid pointer for output to indicate verticall letter box is valid or not
 * @return SV_SUCCESS/SV_FAIL
 */
 UINT8 LboxQueryRegion(UINT32 *pu4LBoxX, UINT32 *pu4LBoxY, UINT32 *pu4Width, UINT32 *pu4Height, UINT32 *pu4HValid, UINT32 *pu4VValid);

/**
 * Create letter box detection thread and processing loop.
 *
 * @param void
 * @return void
 */
 void LboxDetectionInit(void);

 UINT8 u1LBoxCalculateDispRegion(UINT32 *pu4LBoxXOut, UINT32 *pu4LBoxYOut, UINT32 *pu4WidthOut, UINT32 *pu4HeightOut);

/**
 * Notify letter box that the input source timing  or  source regionis changed
 *
 * @return void
 */
extern void LboxSourceChange(void);

/**
 * Get caption status, 
 *@param u4DirectIdx : LBOX_IDX_UP/LBOX_IDX_DOWN
 * @return SV_TRUE(Caption mode) , SV_FALSE(Non-caption mode)
 */
extern UINT8 u1LboxGetCaptionMonitorResult(UINT8 u4DirectIdx);

#endif
