/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date  $
 * $RCSfile: drv_hdmi.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/
#ifndef _DRV_MHLCOM_H_
#define _DRV_MHLCOM_H_

#include "typedef.h"
#include "x_timer.h"
#include "drv_hdmi2.h"

#define MHL_MAX_BUFFER_SIZE 16
#define MHL_SCRATCHPAD_SIZE MHL_MAX_BUFFER_SIZE
#define MHL_MAX_WRITE_BURST MHL_MAX_BUFFER_SIZE

//write 1 to trigger peer command including Get_State,get_vendor_id,set_hpd,clr_hpd,set_cap_id,get_cap_id,get_vs_abort,get_ddc_abort,get_msc_abort
#define RX_CBUS_MSC_PEER_CMD                            0x01
//write 1 to trigger msc_msg command
#define RX_CBUS_MSC_MSC_MSG_CMD                         0x02
//write 1 to trigger read_devcap command
#define RX_CBUS_MSC_READ_DEVCAP_CMD                     0x04
//write 1 to trigger write_state/set_state command
#define RX_CBUS_MSC_WRITE_STATE_CMD                     0x08
//write 1 to trigger write_burst command
#define RX_CBUS_MSC_WRITE_BURST_CMD                     0x10

enum
{
    MSC_WRITE_STAT        =0x60,
    MSC_SET_INT           =0x60,
    MSC_READ_DEVCAP       =0x61,
    MSC_GET_STATE         =0x62,
    MSC_GET_VENDOR_ID     =0x63,
    MSC_SET_HPD           =0x64,
    MSC_CLR_HPD           =0x65,
    MSC_MSG               =0x68,
    MSC_GET_SC1_ERRORCODE =0x69,
    MSC_GET_DDC_ERRORCODE =0x6A,
    MSC_GET_MSC_ERRORCODE =0x6B,
    MSC_WRITE_BURST       =0x6c,
    MSC_GET_SC3ERRORCODE  =0x6D,
};
#define MHL_REG_RCHANGE_INT_OFFSET    0x20
#define MHL_REG_BIT_DCAP_CHG          0x01
#define MHL_REG_BIT_DSCR_CHG          0x02
#define MHL_REG_BIT_REQ_WRT           0x04
#define MHL_REG_BIT_GRT_WRT           0x08
#define MHL_REG_BIT_3D_REQ            0x10

#define MHL_REG_DCHANGE_INT_OFFSET    0x21
#define MHL_REG_BIT_EDID_CHG          0x02

#define MHL_REG_CONNECTED_RDY_OFFSET  0x30
#define MHL_REG_MHL_STAT_0_DCAP_RDY      0x01
#define MHL_REG_MHL_STAT_0_POW_STAT      0x04
#define MHL_REG_MHL_STAT_0_PLIM_STAT     0x20

#define MHL_REG_LINK_MODE_OFFSET      0x31
#define MHL_REG_MHL_CLK_MODE_PACKED_PIXEL 0x02
#define MHL_REG_MHL_CLK_MODE_NORMAL       0x03
#define MHL_REG_MHL_STAT_1_PATH_EN       0x08
#define MHL_REG_MHL_STAT_1_MUTED         0x10

#define MHL_REG_VERSION_STAT_OFFSET   0x32

#define MHL_REG_SCRATCHPAD_OFFSET 0x40

typedef enum
{
    RX_CBUS_MODE_MHL_CABLE_NO_CONNECT,
    RX_CBUS_MODE_oCBUS_NO_CONNECTION,
    RX_CBUS_MODE_oCBUS_UNKNOWN,
    RX_CBUS_MODE_oCBUS_MHL_1_2,
}MhlRxCbusMode_t;

typedef enum
{
    CBUS_REQ_IDLE=0,
    CBUS_REQ_PENDING,
    CBUS_REQ_WAITING,
}MHL_RX_CBUS_REQ_t;

typedef enum
{
    MON_CK_TMDS_INPUT_CLK,
    MON_CK_MAIN_PLL_OUT,
    MON_CK_MAIN_PLL_REF_OUT,
    MON_CK_MAIN_PLL_FBK_OUT,
    MON_CK_DEEP_PLL_OUT,
    MON_CK_DEEP_2XPLL_OUT,
    MON_CK_MAIN_OUT_TO_DIG,
}MON_CK_SRC;

typedef enum
{
    CBUS_NO_PENDING_EVENT = 0x00,
    CBUS_READ_DCAP = 0x01,
    CBUS_3D_REQ = 0x02,
    CBUS_GRT_WRT = 0x04,
    CBUS_REAT_REQ = 0x08,
    CBUS_REQ_WRT = 0x10,
}
MHL_RX_CBUS_EVENT_t;

typedef enum
{
    MHL_RX_CBUS_IMPEDANCE_UP         =1,
    MHL_RX_CBUS_IMPEDANCE_DOWN_1K,
    MHL_RX_CBUS_IMPEDANCE_DOWN_100K,
    MHL_RX_CBUS_IMPEDANCE_HIZ,
    MHL_RX_CBUS_HPD
}MHL_RX_CBUS_IMPEDANCE_t;

typedef enum
{
    MHL_RX_CBUS_MSC_CMD_NULL = 0x00,

    MHL_RX_CBUS_MSC_CMD_INTERRUPT_DCAP_CHG = 0x10,
    MHL_RX_CBUS_MSC_CMD_INTERRUPT_DSCR_CHG,
    MHL_RX_CBUS_MSC_CMD_INTERRUPT_REQ_WRT,
    MHL_RX_CBUS_MSC_CMD_INTERRUPT_GRT_WRT,
    MHL_RX_CBUS_MSC_CMD_INTERRUPT_3D_REQ,
    MHL_RX_CBUS_MSC_CMD_INTERRUPT_EDID_CHG,

    MHL_RX_CBUS_MSC_CMD_STATUS_DCAP_RDY,
    MHL_RX_CBUS_MSC_CMD_STATUS_LINK_MODE,
    MHL_RX_CBUS_MSC_CMD_STATUS_VERSION_STAT,
    MHL_RX_CBUS_MSC_CMD_STATUS_LINK_STATUS,

    MHL_RX_CBUS_MSC_CMD_SCRATCHPAD_WRITE_BURST,

    MHL_RX_CBUS_MSC_CMD_MSGE,
    MHL_RX_CBUS_MSC_CMD_MSG_RCP,
    MHL_RX_CBUS_MSC_CMD_MSG_RCPK,
    MHL_RX_CBUS_MSC_CMD_MSG_RCPE,

    MHL_RX_CBUS_MSC_CMD_MSG_RAP,
    MHL_RX_CBUS_MSC_CMD_MSG_RAPK,

    MHL_RX_CBUS_MSC_CMD_MSG_UCP,
    MHL_RX_CBUS_MSC_CMD_MSG_UCPK,
    MHL_RX_CBUS_MSC_CMD_MSG_UCPE,

    MHL_RX_CBUS_MSC_CMD_SET_HPD,
}MHL_RX_CBUS_MSC_CMD_t;

typedef struct
{
    MHL_RX_CBUS_MSC_CMD_t eSentCommand;
    UINT8 u1Data[16];
    BOOL bSent;
    UINT8* pFailureReason;
}MHL_RX_CBUS_CALLBACK_PARAMETERS_t;

typedef VOID (*pWRTIEBURSTCALLBACK)(UINT8 u1Val);
typedef struct
{
    UINT8 u1GeneralWriteBurst[MHL_MAX_WRITE_BURST][MHL_SCRATCHPAD_SIZE];
    pWRTIEBURSTCALLBACK pWriteBurstCallBack[MHL_MAX_WRITE_BURST];
    UINT8 u1WriteBurstLength[MHL_MAX_WRITE_BURST];
    UINT8 u1ReadPointer;
    UINT8 u1WritePointer;
    BOOL bBusy;
}MHL_RX_CBUS_WRITE_BURST_BUFFER_t;

typedef void (* pMHL3CommandReport)(MHL_RX_CBUS_CALLBACK_PARAMETERS_t* pCbusCallbackParameter);
typedef struct
{
    UINT8 u1ReqStatus;
    MHL_RX_CBUS_MSC_CMD_t CbusMscCommand;
    UINT8 u1Command;
    UINT8 u1OffsetData;
    UINT8 u1Length;
    UINT8 u1MsgData[MHL_MAX_BUFFER_SIZE];
    UINT8 u1MsgData2[MHL_MAX_BUFFER_SIZE];
    UINT32 u4TimeOutCnt;
    UINT8 u1RetryCnt;
    void (* pMHL3DataRetHandler)(UINT8); //callback
    pMHL3CommandReport pCommandReport ;//callback
}MHL_RX_CBUS_OUT_QUEUE_t;

typedef struct
{
    UINT8 u1ReqStatus;
    UINT8 u1Command;
    UINT8 u1MsgData;
    UINT32 u4TimeOutCnt;
}MHL_RX_CBUS_IN_QUEUE_t;
extern void vMHL3ComForceCbusConnect(BOOL bEn);

extern void vMHL2ComAnaMainPLLCfg(UINT16 u2HdmiAnaOfst,MhlTmdsClkRate_t eTmdsClk,BOOL fg24bit);
extern void vMHL2ComAnaDeepPLLCfg(UINT16 u2HdmiAnaOfst,MhlTmdsClkRate_t eTmdsClk,BOOL fg24bit);
extern void vMHL3ComAnaMainPLLEn(UINT16 u2HdmiAnaOfst,BOOL fgEnable);
extern UINT32 MHL3ComMonitorClk(MON_CK_SRC eClk);
extern void vMHL2ComAnaSetting(UINT16 u2HdmiAnaOfst,MhlTmdsClkRate_t eTmdsClk,BOOL fg24Bit);
extern void vMHL3ComSetEx1K(BOOL bEn);
extern void vMHL3ComSetCbusImpedanceUp(void);
extern void vMHL3ComSetCbusImpedanceHiZ(void);
extern void vMHL3ComSetCbusImpedance1K(void);
extern void vMHL3ComSetPdwncCbusImp100K(void);
extern void vMHL3ComSetCbusImpedance100K(void);
extern void vMHL3ComSetCbusImpedanceHwMode(void);
extern void vMHL3ComOcbusEn(BOOL bEn);
extern void vMHL3ComCbusRiscRst(void);
extern void vMHL3ComCbusClkRst(void);
extern void vMHL3ComSetCbusIntMask(UINT8 u1En);
extern UINT8 u1MHL3ComGetCbusInt(void);
extern UINT8 u1MHL3ComGetCbusInt1(void);
extern void vMHL3ComClrCbusInt(void);
extern UINT32 u1MHL3ComGetCbusInt0Stat(void);
extern UINT32 u1MHL3ComGetCbusInt0Mask(void);
extern void vMHL3ComClrCbusInt0Stat(UINT32 u4Value);
extern UINT32 u4MHL3ComGetCbusMscFailStat(void);
extern void vMHL3ComClrCbusMscFailStat(UINT32 u4Value);
extern UINT32 u4MHL3ComGetCbusDdcFailStat(void);
extern void vMHL3ComClrCbusDdcFailStat(UINT32 u4Value);
extern UINT32 u1MHL3ComGetMhlInt0Stat(void);
extern void vMHL3ComClrMhlInt0Stat(UINT32 u4Value);
extern BOOL fgMHL3ComGetCdSense(void);
extern BOOL fgMHL3ComGetCbusConnect(void);
extern void vMHL3ComSetEcbusCtrl(void);
extern void vMHL3ComDevCapInit(UINT8 *pDevCap,UINT8 u1Len);
extern UINT8 u1MHL3ComGetCbusReqMscData(void);
extern UINT8 u1MHL3ComGetCbusResMscOffset(void);
extern UINT8 u1MHL3ComGetCbusResMscData(void);
extern UINT8 u1MHL3ComGetCbusRcvMscData(void);
extern void vMHL3ComGetDevStat(UINT8 *pData,UINT8 u1Len);
extern void vMHL3ComGetCbusScratchPadData(UINT8 *pData,UINT8 u1Len);
extern BOOL fgMHL3ComCbusWriteCommand(MHL_RX_CBUS_OUT_QUEUE_t *pReq);
extern void vMHL2ComClkPowerOn(void);
extern void vMHL3ComCbusInit(void);
extern void vMHL3ComCbusInit(void);
extern void vMHL3ComCbusHwMode(void);
#endif
