/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: drv_scaler.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

#ifndef _DRV_SCALER_PSC_H_
#define _DRV_SCALER_PSC_H_

#include "hw_scpos.h"
#include "panel.h"
#include "drv_display.h"

#define PSC_WRITE_FLD(reg,value,fld) vIO32WriteFldAlign(reg, value, fld)
#define PSC_READ_FLD(reg,fld) IO32ReadFldAlign(reg, fld)

//*****************************************************************************
//  drv_scaler_psc.c
//*****************************************************************************
void vPscSwInit(void);
void vPscHwInit(void);
void vPscPreInit(void);
void vPscFireModeChanged(void);
void vPscClearModeChanged(void);
UINT8 fgPscIsModeChanged(void);
void vPscSetImportOff(UINT8 bOnOff);
void vPSCDumpResInfo(void);
void vPscPrintCurrentScale(void);
void vPsc4k1k(UINT8 bOnOff);

BOOL bPscIsNeedSetData(void);
void vPscSetNeedSetDataFlag(UINT8 bFlag);
void vPscUpdateReg_SETDATA(void);

/* API for PSC frame track */
void vPscConfigFrameTrack(UINT32 u4Step);
void vPscSetFrameTrackOnOff(UINT32 u4OnOff);

void vPscNormProc(void);


//*****************************************************************************
//  drv_scaler_psc_hw.c
//*****************************************************************************

// functions for time gen.
void vPscSetPanelWidth(UINT32 u4Width);
void vPscSetPanelHeight(UINT32 u4Height);
void vPscSetPanelHTotal(UINT32 u4HTotal);
void vPscSetPanelVTotal(UINT32 u4VTotal);
UINT32 u4PscGetPanelWidth(void);
UINT32 u4PscGetPanelHeight(void);
UINT32 u4PscGetPanelHTotal(void);
UINT32 u4PscGetPanelVTotal(void);
UINT32 u4PscGetOutputHActiveStatus(void);
UINT32 u4PscGetOutputVActiveStatus(void);
UINT32 u4PscGetOutputHTotalStatus(void);
UINT32 u4PscGetOutputVTotalStatus(void);
UINT32 vPscGetInputHactive(void);
UINT32 vPscGetInputHtotal(void);
UINT32 vPscGetInputVactive(void);
UINT32 vPscGetInputVtotal(void);
UINT16 u2PscGetPanelVCNTStatus(void);
void vPscSetPanelHSyncFp(UINT32 u4FrontPorch);
void vPscSetPanelVSyncFp(UINT32 u4FrontPorch);
void vPscSetPanelHSyncBp(UINT32 u4BackPorch);
void vPscSetPanelVSyncBp(UINT32 u4BackPorch);
UINT32 u4PscGetPanelHSyncFp(void);
UINT32 u4PscGetPanelVSyncFp(void);
UINT32 u4PscGetPanelHSyncBp(void);
UINT32 u4PscGetPanelVSyncBp(void);
UINT8 u1PscGetInDispR(void);
UINT8 u1PscGetOutDispR(void);
void vPscSetData(UINT8 bOnOff);
void vPscTimegenImportEnable(UINT8 bOnOff);

//functions for register update
void vPscSetResImportEnable(UINT8 bOnOff);
void vPscSetResToggle(UINT8 bOnOff);
void vPscSetInputWidthHeight(UINT32 u4Width, UINT32 u4Height);

void vPscSetHInitPhase(UINT32 u4InitSign, UINT32 u4InitIntg, UINT32 u4InitFrac);
void vPscSetHDeltaPhase(UINT32 u4DeltaIntg, UINT32 u4DeltaFrac);
void vPscSetVInitPhase(UINT32 u4InitSign, UINT32 u4InitIntg, UINT32 u4InitFrac);
void vPscSetVDeltaPhase(UINT32 u4DeltaIntg, UINT32 u4DeltaFrac);
void vPscSetVScalerEnable(UINT8 bOnOff);
void vPscSetHmskStartOfst(UINT32 u4HStart);
void vPscSetVmskStartOfst(UINT32 u4VStart);
void vPscSetOutputWidthHeight(UINT32 u4Width, UINT32 u4Height);

//functions for in/ out pattern gen.
void vPscSetInputPatternGenEnable(UINT8 bOnOff);
void vPscSetInputPatternGenBoundaryEnable(UINT8 bOnOff);
void vPscSetInputPatternGenBoundary(UINT32 u4Hboundary,UINT32 u4Vboundary);
void vPscSetOutputPatternGenEnable(UINT8 bOnOff);
void vPscSetOutputPatternGenBoundaryEnable(UINT8 bOnOff);
void vPscSetOutputPatternGenBoundary(UINT32 u4Hboundary,UINT32 u4Vboundary);

//functions for frame track
void vPscSelectFrameTrackVsyncInEdge(UINT8 bOnOff);
void vPscSetDddsTrackFrameTurboSpeed(UINT32 u4Speed);
void vPscSetFrameTrackStep(UINT32 u4Step);
void vPscSetFrameTrackLockWin(UINT32 u4LockWin);
void vPscSetFrameTrackLockTolerateLineNum(UINT32 u4LineNum);
void vPscSetFrameTrackTurboRegion(UINT32 u4Region);
void vPscSetFrameTrackTargetDistance(UINT32 u4Distance);
void vPscSetDddsFrameTrackEnable(UINT8 bOnOff);
UINT32 vPscGetDddsFrameTrackEnable(void);
void vPscMaskInputFrame(UINT32 u4Frame);
void vPscMaskOutputFrame(UINT32 u4Frame);
void vPscSelectOsdVsync(UINT8 bOnOff);
UINT8 u1PscGetOsdVsyncSel(void);
void vPscSetInvOsdHVSync(UINT8 bInvHSync, UINT8 bInvVSync);

void vPscSetOsdHsyncDly(UINT16 u2OsdHsyncDly);   
void vPscSetOsdVsyncDly(UINT16 u2OsdVsyncDly);  
void vPscTGSlaveSyncMode(UINT8 u1OnOff );
void vPscTGSlaveMode(UINT8 u1OnOff );
void vPscTGEnable( UINT8 u1OnOf );
void vPscSetCoreSel(UINT8 u1Sel1, UINT8 u1Sel2);
void v1PscSetRiscChannelSel(UINT8 u1Mode, UINT8 u1Sel);
void vPscSetMEMCOutputWidth(UINT16 u4Width);
void vPscSetDispCropInputWidthHeight(UINT32 u4Width, UINT32 u4Height);
void vPscSetDispCropROIWidthHeight(UINT32 u4Width, UINT32 u4Height);
void vPscSetDispCropOfst(UINT32 u4StartX, UINT32 u4StartY);
void vPscSetDispCropEnable(UINT8 bOnOff);
void vPscSetDispCropForceCommit(UINT8 bOnOff);
void vDrvPscCropDelay(UINT32 cropDelay);
void vPscBypassPathEnable(UINT8 bOnOff);
void vPscSetFrameTrackCrossVsync(UINT8 u1OnOff);

UINT8 vPscTGIsSlaveMode(void);
UINT8 vPscTGIsSlaveSyncMode(void);

UINT32 vPscGetFrameTrackTargetDistance(void);
UINT32 vPscGetFrameTrackLockTolerateLineNum(void);
UINT32 vPscGetFrameTrackLockWin(void);
UINT32 vPscGetFrameTrackLockOkStatus(void);
UINT32 u4PscGetInOutVDistance(void);

UINT32 vPscCalPscTrkLine(void);
UINT32 vPscCalPscRefOfst(void);
UINT32 vPscGetFrameTrackStep(void);
UINT32 u4PscGetVRROutVTotal(void);



#endif
