/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: drv_scpos.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *
 *
 * Author:
 * -------
 *
 *
 * Last changed:
 * -------------
 * $Author: brianpc.huang $
 *
 * $Modtime: 04/05/28 9:33a $
 *
 * $Revision: #1 $
*************************************************************************/

#ifndef _FRAMETRACK_DRVIF_H_
#define _FRAMETRACK_DRVIF_H_

#include "typedef.h"

EXTERN UINT32 u4ForceFrameTrackCtrl;
EXTERN UINT32 u4ForceFrameTrackMode;

typedef enum
{
    SCPIP_FRAME_TRACK_DISABLE =0,
    SCPIP_FRAME_TRACK_LOCK =1,
    SCPIP_FRAME_TRACK_NOT_STALBE =2,
}E_FRAME_TRACK_STAUS;

#define SCPIP_FRAME_TRACK_LOCK_RANGE    2
#define SCPIP_FRAME_TRACK_LOSS_LOCK_RANGE    5
#define SCPIP_FRAME_TRACK_STEP_SMALL 1
#define SCPIP_FRAME_TRACK_STEP_LARGE 20

#define SCPIP_FRAME_TRACK_MJC_LOCK_RANGE    60

#define SCPIP_FRAME_TRACK_AUTO 0
#define SCPIP_FRAME_TRACK_ON_CLI ((UINT8)1 << 1)
#define SCPIP_FRAME_TRACK_OFF_CLI ((UINT8)1 << 2)
#define SCPIP_FRAME_TRACK_OFF_DS_PATTERN ((UINT8)1 << 3)

#define FRAME_TRACK_MODE_DDDS_BY_MJC      (1 << 0)
#define FRAME_TRACK_MODE_DDDS_BY_FNTSCALER   (1 << 1)
#define FRAME_TRACK_MODE_DDDS_BY_PSTSCALER   (1 << 2)
#define FRAME_TRACK_MODE_VTOTAL_BY_MJC    (1 << 3)
#define FRAME_TRACK_MODE_VTOTAL_BY_FNTSCALER (1 << 4)
#define FRAME_TRACK_MODE_VTOTAL_BY_PSTSCALER (1 << 5)
#define FRAME_TRACK_MODE_SW_DDDS          (1 << 6)
#define FRAME_TRACK_MODE_DDDS_BY_REF      (1 << 7)
#define FRAME_TRACK_MODE_VTOTAL_BY_REF      (1 << 8)

#define FRAME_TRACK_MODE_AUTO             (1 << 10)
#define FRAME_TRACK_MODE_CLI_DEBUG        (1 << 16)


#define DDDS_FT_OFF (0)
#define DDDS_FT_FROM_SCALER (1)
#define DDDS_FT_FROM_MJC (2)

#define FSC_FRAME_TRACK_MASK              (FRAME_TRACK_MODE_DDDS_BY_FNTSCALER | FRAME_TRACK_MODE_VTOTAL_BY_FNTSCALER)  
    #define IS_FSC_FRAME_TRACK(mode)      (mode & FSC_FRAME_TRACK_MASK)      
#define PSC_FRAME_TRACK_MASK              (FRAME_TRACK_MODE_DDDS_BY_PSTSCALER | FRAME_TRACK_MODE_VTOTAL_BY_PSTSCALER)
    #define IS_PSC_FRAME_TRACK(mode)      (mode & PSC_FRAME_TRACK_MASK) 
#define REF_FRAME_TRACK_MASK              (FRAME_TRACK_MODE_DDDS_BY_REF | FRAME_TRACK_MODE_VTOTAL_BY_REF)
    #define IS_REF_FRAME_TRACK(mode)      (mode & REF_FRAME_TRACK_MASK)

//Frame Track functions for input/output V sync alignment
EXTERN void vDrvFrameTrackProc(void);
void vFrameTrackDDDSConfig(UINT32 u4Step1, UINT32 u4Step2);

EXTERN void vFrameTrackConfig(UINT32 u4Step);
EXTERN void vScpipSwitchFrameTrackOnOff(UINT32 u4FscOnOff, UINT32 u4PscOnOff, UINT32 u4RefOnOff);

EXTERN void vDumpFrameTrackStatus(void);
BOOL fgFrameTrackIsFlgSet(UINT32 u4Flag);
BOOL fgFrameTrackIsDDDSModeInMJCOrPSC(void);

#if  defined(CC_MT5890)
void vDrvSetFrameTrackIsMJC(UINT8 u1FttMJC);
#endif

void vDrvSetFrameTrackMode(UINT8 u1FTModule, UINT8 u1FTMode);
EXTERN void vScpipFrameTrackStep(UINT8 bStep);

void vFrameTrackSetFlg(UINT32 u4Flag);
void vFrameTrackClrFlg(UINT32 u4Flag);

void vFrameTrackInit(void);

BOOL fgFrameTrackIsRefTrk(void);

void vFrameTrackDynMode(void);


#endif //#ifndef _FRAMETRACK_DRVIF_H_
