/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_dsmcc_eng.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains DSM-CC engine specific definitions, which
 *         are exported.
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_DSMCC_ENG_H_
#define _C_DSMCC_ENG_H_


/*---------------------------------------------------------------------------
                    include files
---------------------------------------------------------------------------*/

#include "c_handle.h"
#include "c_common.h"
#include "c_tm.h"
#include "u_dsmcc_eng.h"


/*---------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 --------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

/* DSMCC monitoring API's. */
/*------------------------------------------------------------------*/
/*! @brief  Monitor for a top-level download table
 *  @param  [in]  h_root      - Handle to root table object to attach DSI or DII to
 *  @param  [in]  t_pid       - PID of DSI or DII
 *  @param  [in]  e_src_type  - Source type
 *  @param  [in]  pv_src_info - Source information
 *  @param  [in]  pv_nfy_tag  - Tag to attach to this DSI request
 *  @param  [in]  pf_nfy      - Notification function to call
 *  @param  [out] ph_mon      - Pointer to hold handle to monitor
 *  @param  [out] pe_cond     - Pointer to variable to hold table condition
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_INV_HANDLE      - h_root is not a valid handle
 *  @retval TMR_INV_HANDLE_TYPE - h_root is not a handle to a root
 *  @retval TMR_INV_ARG         - ph_mon_top is NULL
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_mon_top_by_pid (HANDLE_T       h_root,
                                     MPEG_2_PID_T   t_pid,
                                     TM_SRC_TYPE_T  e_src_type,
                                     VOID*          pv_src_info,
                                     VOID*          pv_nfy_tag,
                                     x_tm_nfy_fct   pf_nfy,
                                     HANDLE_T*      ph_mon,
                                     TM_COND_T*     pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  Get the download level from a monitor handle
 *  @param  [in]  h_mon            - Handle to a monitor
 *  @param  [out] pe_download_type - Pointer to variable to hold download level
 *  @param  [out] pe_cond          - Pointer to variable to hold table condition
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_INV_HANDLE      - h_mon is not a valid handle
 *  @retval TMR_INV_HANDLE_TYPE - h_mon is not a handle to a monitor
 *  @retval TMR_INV_ARG         - pe_download_level is NULL
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_get_download_level (HANDLE_T                h_mon,
                                         DSMCC_DOWNLOAD_TYPE_T*  pe_download_type,
                                         TM_COND_T*              pe_cond);

/* NEW DSMCC API'S */
/*------------------------------------------------------------------*/
/*! @brief  This API loads a DSI based on PID and TS id.
 *  @param  [in]  h_root      - Contains the handle referencing the root
 *                              object.
 *  @param  [in]  t_pid       - Contaisn the PID value.
 *  @param  [in]  ui2_ts_id   - Contains the TS id.
 *  @param  [in]  e_src_type  - Specifies the source type for loading the
 *                              requested table.
 *  @param  [in]  pv_src_info - Contains additional information describing
 *                              the source.
 *  @param  [in]  pv_nfy_tag  - Contains a notify tag value.
 *  @param  [in]  pf_nfy      - References the notify function.
 *  @param  [out] ph_dsi      - Contains a handle which references the DSI object.
 *  @param  [out] pe_cond     - Contains the condition of the DSI object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_dsi_by_pid_ts_id (HANDLE_T       h_root,
                                            MPEG_2_PID_T   t_pid,
                                            UINT16         ui2_ts_id,
                                            TM_SRC_TYPE_T  e_src_type,
                                            VOID*          pv_src_info,
                                            VOID*          pv_nfy_tag,
                                            x_tm_nfy_fct   pf_nfy,
                                            HANDLE_T*      ph_dsi,
                                            TM_COND_T*     pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API loads a DSI based on PID and TS descriptor.
 *  @param  [in]  h_root      - Contains the handle referencing the root
 *                              object.
 *  @param  [in]  t_pid       - Contaisn the PID value.
 *  @param  [in]  pt_ts_descr - References the TS descriptor.
 *  @param  [in]  e_src_type  - Specifies the source type for loading the
 *                              requested table.
 *  @param  [in]  pv_src_info - Contains additional information describing
 *                              the source.
 *  @param  [in]  pv_nfy_tag  - Contains a notify tag value.
 *  @param  [in]  pf_nfy      - References the notify function.
 *  @param  [out] ph_dsi      - Contains a handle which references the DSI object.
 *  @param  [out] pe_cond     - Contains the condition of the DSI object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_dsi_by_pid_ts_descr (HANDLE_T       h_root,
                                               MPEG_2_PID_T   t_pid,
                                               TS_DESCR_T*    pt_ts_descr,
                                               TM_SRC_TYPE_T  e_src_type,
                                               VOID*          pv_src_info,
                                               VOID*          pv_nfy_tag,
                                               x_tm_nfy_fct   pf_nfy,
                                               HANDLE_T*      ph_dsi,
                                               TM_COND_T*     pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API loads a DII based on PID, TS id ad some other ids.
 *  @param  [in]  h_root       - Contains the handle referencing the root
 *                               object.
 *  @param  [in]  t_pid        - Contaisn the PID value.
 *  @param  [in]  ui2_ts_id    - Contains the TS id.
 *  @param  [in]  pt_dsmcc_ids - Contains the DII load id's.
 *  @param  [in]  e_src_type   - Specifies the source type for loading the
 *                               requested table.
 *  @param  [in]  pv_src_info  - Contains additional information describing
 *                               the source.
 *  @param  [in]  pv_nfy_tag   - Contains a notify tag value.
 *  @param  [in]  pf_nfy       - References the notify function.
 *  @param  [out] ph_dii       - Contains a handle which references the DII object.
 *  @param  [out] pe_cond      - Contains the condition of the DII object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_dii_by_pid_ts_id_ids (HANDLE_T       h_root,
                                                MPEG_2_PID_T   t_pid,
                                                UINT16         ui2_ts_id,
                                                DSMCC_IDS_T*   pt_dsmcc_ids,
                                                TM_SRC_TYPE_T  e_src_type,
                                                VOID*          pv_src_info,
                                                VOID*          pv_nfy_tag,
                                                x_tm_nfy_fct   pf_nfy,
                                                HANDLE_T*      ph_dii,
                                                TM_COND_T*     pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API loads a DII based on PID, TS decsriptor and some
 *          other ids.
 *  @param  [in]  h_root       - Contains the handle referencing the root
 *                               object.
 *  @param  [in]  t_pid        - Contaisn the PID value
 *  @param  [in]  pt_ts_descr  - References a TS decsriptor.
 *  @param  [in]  pt_dsmcc_ids - Contains the DII load id's.
 *  @param  [in]  e_src_type   - Specifies the source type for loading the
 *                        requested table.
 *  @param  [in]  pv_src_info  - Contains additional information describing
 *                        the source.
 *  @param  [in]  pv_nfy_tag   - Contains a notify tag value.
 *  @param  [in]  pf_nfy       - References the notify function.
 *  @param  [out] ph_dii       - Contains a handle which references the DII object.
 *  @param  [out] pe_cond      - Contains the condition of the DII object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_dii_by_pid_ts_descr_ids (HANDLE_T       h_root,
                                                   MPEG_2_PID_T   t_pid,
                                                   TS_DESCR_T*    pt_ts_descr,
                                                   DSMCC_IDS_T*   pt_dsmcc_ids,
                                                   TM_SRC_TYPE_T  e_src_type,
                                                   VOID*          pv_src_info,
                                                   VOID*          pv_nfy_tag,
                                                   x_tm_nfy_fct   pf_nfy,
                                                   HANDLE_T*      ph_dii,
                                                   TM_COND_T*     pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API loads a DDB based on PID, TS id and some other ids.
 *  @param  [in]  h_root         - Contains the handle referencing the root
 *                                 object.
 *  @param  [in]  t_pid          - Contaisn the PID value.
 *  @param  [in]  ui2_ts_id      - Contains the TS id.
 *  @param  [in]  pt_dsmcc_ids   - Contains the DDB load id's.
 *  @param  [in]  b_feed_section - Indicates if whole sections shall be notified.
 *  @param  [in]  e_src_type     - Specifies the source type for loading the
 *                                 requested table.
 *  @param  [in]  pv_src_info    - Contains additional information describing
 *                                 the source.
 *  @param  [in]  pv_nfy_tag     - Contains a notify tag value.
 *  @param  [in]  pf_nfy         - References the notify function.
 *  @param  [out] ph_ddb         - Contains a handle which references the DDB object.
 *  @param  [out] pe_cond        - Contains the condition of the DDB object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_ddb_by_pid_ts_id_ids (HANDLE_T       h_root,
                                                MPEG_2_PID_T   t_pid,
                                                UINT16         ui2_ts_id,
                                                DSMCC_IDS_T*   pt_dsmcc_ids,
                                                BOOL           b_feed_section,
                                                TM_SRC_TYPE_T  e_src_type,
                                                VOID*          pv_src_info,
                                                VOID*          pv_nfy_tag,
                                                x_tm_nfy_fct   pf_nfy,
                                                HANDLE_T*      ph_ddb,
                                                TM_COND_T*     pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API loads a DDB based on PID, TS decsriptor and some
 *          other ids.
 *  @param  [in]  h_root         - Contains the handle referencing the root
 *                                 object.
 *  @param  [in]  t_pid          - Contaisn the PID value.
 *  @param  [in]  pt_ts_descr    - References a TS decsriptor.
 *  @param  [in]  pt_dsmcc_ids   - Contains the DDB load id's.
 *  @param  [in]  b_feed_section - Indicates if whole sections shall be notified.
 *  @param  [in]  e_src_type     - Specifies the source type for loading the
 *                                 requested table.
 *  @param  [in]  pv_src_info    - Contains additional information describing
 *                                 the source.
 *  @param  [in]  pv_nfy_tag     - Contains a notify tag value.
 *  @param  [in]  pf_nfy         - References the notify function.
 *  @param  [out] ph_ddb         - Contains a handle which references the DDB object.
 *  @param  [out] pe_cond        - Contains the condition of the DDB object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_ddb_by_pid_ts_descr_ids (HANDLE_T       h_root,
                                                   MPEG_2_PID_T   t_pid,
                                                   TS_DESCR_T*    pt_ts_descr,
                                                   DSMCC_IDS_T*   pt_dsmcc_ids,
                                                   BOOL           b_feed_section,
                                                   TM_SRC_TYPE_T  e_src_type,
                                                   VOID*          pv_src_info,
                                                   VOID*          pv_nfy_tag,
                                                   x_tm_nfy_fct   pf_nfy,
                                                   HANDLE_T*      ph_ddb,
                                                   TM_COND_T*     pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API loads a DDB based on a DII, the DII's module index
 *          and the block number. The DDB will be acquired from the same
 *          PID value as the DII.
 *  @param  [in]  h_dii          - References the DII.
 *  @param  [in]  ui2_module_idx - Contains the module entry within the DII.
 *  @param  [in]  ui2_block_num  - Contains the DDB block number.
 *  @param  [in]  b_feed_section - Indicates if whole sections shall be notified.
 *  @param  [in]  pv_nfy_tag     - Contains a notify tag value.
 *  @param  [in]  pf_nfy         - References the notify function.
 *  @param  [out] ph_ddb         - Contains a handle which references the DDB object.
 *  @param  [out] pe_cond        - Contains the condition of the DDB object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval      -
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_ddb_by_module_idx_block_num (HANDLE_T      h_dii,
                                                       UINT16        ui2_module_idx,
                                                       UINT16        ui2_block_num,
                                                       BOOL          b_feed_section,
                                                       VOID*         pv_nfy_tag,
                                                       x_tm_nfy_fct  pf_nfy,
                                                       HANDLE_T*     ph_ddb,
                                                       TM_COND_T*    pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API loads a range of DDB's based on a DII, the DII's
 *          module index and a block number range. The DDB will be
 *          acquired from the same PID value as the DII.
 *  @param  [in]  h_dii               - References the DII.
 *  @param  [in]  ui2_module_idx      - Contains the module entry within the DII.
 *  @param  [in]  ui2_first_block_num - Contains the first DDB block number.
 *  @param  [in]  ui2_last_block_num  - Contains the first DDB block number.
 *  @param  [in]  pv_nfy_tag          - Contains a notify tag value.
 *  @param  [in]  pf_nfy              - References the notify function.
 *  @param  [out] ph_ddb              - Contains a handle which references the DDB object.
 *  @param  [out] pe_cond             - Contains the condition of the DDB object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK
 *  @retval TMR_AEE_OUT_OF_RESOURCES - Not enough resources available.
 *  @retval TMR_AEE_NO_RIGHTS        - Routine failed.
 *  @retval TMR_NOT_INIT             - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG              - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE           - Invalid handle.
 *  @retval TMR_OUT_OF_HANDLES       - No handles available.
 *  @retval TMR_OUT_OF_RESOURCES     - Out of resources.
 *  @retval TMR_NO_SUPPORT           - No support.
 *  @retval TMR_INV_HANDLE_STATE     - Invalid handle state.
 *  @retval TMR_INV_OBJ_TYPE         - Invalid object type.
 *  @retval TMR_INV_LOAD_TYPE        - Invalid load operation.
 *  @retval TMR_INV_LOAD_INFO        - Invalid load operation data.
 *  @retval TMR_INV_FLAG_SETTING     - Invalid flag settings.
 *  @retval TMR_INV_SRC_TYPE         - Invalid source type.
 *  @retval TMR_INV_SRC_INFO         - Invalid source information.
 *  @retval TMR_SINGLE_SRC_REQUIRED  - Single source required.
 *  @retval TMR_SRC_PARTIAL_OVERLAP  - Sources partial overlap.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_load_ddb_by_module_idx_block_range (HANDLE_T      h_dii,
                                                         UINT16        ui2_module_idx,
                                                         UINT16        ui2_first_block_num,
                                                         UINT16        ui2_last_block_num,
                                                         VOID*         pv_nfy_tag,
                                                         x_tm_nfy_fct  pf_nfy,
                                                         HANDLE_T*     ph_ddb,
                                                         TM_COND_T*    pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API returns the common data of a DSI table.
 *  @param  [in] h_dsi        - Contains a handle referencing the DSI table.
 *  @param  [out] pt_dsi_info - Contains the retrieved info.
 *  @param  [out] pe_cond     - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_get_dsi_info (HANDLE_T           h_dsi,
                                   DSMCC_DSI_INFO_T*  pt_dsi_info,
                                   TM_COND_T*         pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API returns the common data of a DII table.
 *  @param  [in]  h_dii       - Contains a handle referencing the DII table.
 *  @param  [out] pt_dii_info - Contains the retrieved info.
 *  @param  [out] pe_cond     - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_get_dii_info (HANDLE_T           h_dii,
                                   DSMCC_DII_INFO_T*  pt_dii_info,
                                   TM_COND_T*         pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API returns the common data of a DII module.
 *  @param  [in]  h_dii              - h_dii
 *  @param  [in]  ui2_module_idx     - ui2_module_idx
 *  @param  [out] pt_dii_module_info - pt_dii_module_info
 *  @param  [out] pe_cond            - pe_cond
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_get_dii_module_info (HANDLE_T                  h_dii,
                                          UINT16                    ui2_idx,
                                          DSMCC_DII_MODULE_INFO_T*  pt_dii_module_info,
                                          TM_COND_T*                pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API returns the common data of a DDB table.
 *  @param  [in]  h_ddb       - Contains a handle referencing the DDB table.
 *  @param  [out] pt_ddb_info - Contains the retrieved info.
 *  @param  [out] pe_cond     - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                - Routine successful.
 *  @retval TMR_NOT_INIT          - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG           - One or more of the arguments contain invalid data.
 *  @retval TMR_INV_HANDLE        - Invalid handle.
 *  @retval TMR_NO_SUPPORT        - No support.
 *  @retval TMR_INV_HANDLE_STATE  - Invalid handle state.
 *  @retval TMR_NOT_ENOUGH_SPACE  - Not enough space.
 *  @retval TMR_INV_GET_TYPE      - Invalid et operation.
 *  @retval TMR_INV_GET_INFO      - Invalid get operation data.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_get_ddb_info (HANDLE_T           h_ddb,
                                   DSMCC_DDB_INFO_T*  pt_ddb_info,
                                   TM_COND_T*         pe_cond);


/*------------------------------------------------------------------*/
/*! @brief  This API allows a client to parse a DSI's raw private data.
 *  @param  [in]  h_dsi      - Contains a handle referencing the DSI table.
 *  @param  [in]  pv_data    - References private data which is passed to the parsing
 *                             function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced by
 *                             argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                 - data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_parse_dsi_private_data (HANDLE_T        h_dsi,
                                             VOID*           pv_data,
                                             SIZE_T          z_data_len,
                                             x_tm_parse_fct  pf_parse,
                                             TM_COND_T*      pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API allows a client to parse a DII's compatibility
 *          descriptor data.
 *  @param  [in]  h_dii      - Contains a handle referencing the DII table.
 *  @param  [in]  pv_data    - References private data which is passed to the parsing
 *                             function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced by
 *                             argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                 - data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_parse_dii_comp_descr (HANDLE_T        h_dii,
                                           VOID*           pv_data,
                                           SIZE_T          z_data_len,
                                           x_tm_parse_fct  pf_parse,
                                           TM_COND_T*      pe_cond);
/*------------------------------------------------------------------*/
/*! @brief   This API allows a client to parse a DII's raw module info
 *           data.
 *  @param  [in]  h_dii      - Contains a handle referencing the DII table.
 *  @param  [in]  ui2_idx    - Indicates the entry in the module loop.
 *  @param  [in]  pv_data    - References private data which is passed to the
 *                             parsing function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced
 *                             by argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                 - data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_parse_dii_module_info (HANDLE_T        h_dii,
                                            UINT16          ui2_idx,
                                            VOID*           pv_data,
                                            SIZE_T          z_data_len,
                                            x_tm_parse_fct  pf_parse,
                                            TM_COND_T*      pe_cond);
/*------------------------------------------------------------------*/
/*! @brief  This API allows a client to parse a DII's private data.
 *  @param  [in]  h_dii      - Contains a handle referencing the DII table.
 *  @param  [in]  pv_data    - References private data which is passed to the parsing
 *                             function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced by
 *                             argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                 - data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
INT32 c_dsmcc_parse_dii_private_data (HANDLE_T        h_dii,
                                      VOID*           pv_data,
                                      SIZE_T          z_data_len,
                                      x_tm_parse_fct  pf_parse,
                                      TM_COND_T*      pe_cond);

/*------------------------------------------------------------------*/
/*! @brief  This API allows a client to parse a DDB's raw block data.
 *  @param  [in]  h_ddb      - Contains a handle referencing the DDB table.
 *  @param  [in]  pv_data    - References private data which is passed to the parsing
 *                             function.
 *  @param  [in]  z_data_len - Contains the length of the data buffer referenced by
 *                             argument 'pv_data'.
 *  @param  [in]  pf_parse   - References the parsing function.
 *  @param  [out] pe_cond    - Contains the condition of the table object.
 *  @note
 *  @see
 *  @return Return the error code
 *  @retval TMR_OK                 - Routine successful.
 *  @retval TMR_NOT_INIT           - The Table Manager is not initialized.
 *  @retval TMR_INV_ARG            - One or more of the arguments contain invalid
 *                                 - data.
 *  @retval TMR_INV_HANDLE         - Invalid handle.
 *  @retval TMR_NO_SUPPORT         - No support.
 *  @retval TMR_INV_HANDLE_STATE   - Invalid handle state.
 *  @retval TMR_INV_PARSE_TYPE     - Invalid parse operation.
 *  @retval TMR_INV_PARSE_INFO     - Invalid parse operation data.
 *  @retval TMR_PARSE_FCT_FAILURE  - Parse function returned a failure.
 */
/*------------------------------------------------------------------*/
extern INT32 c_dsmcc_parse_ddb_block_data (HANDLE_T        h_ddb,
                                           VOID*           pv_data,
                                           SIZE_T          z_data_len,
                                           x_tm_parse_fct  pf_parse,
                                           TM_COND_T*      pe_cond);


#endif /* _C_DSMCC_ENG_H_ */

