/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_gl.h 
 *  $RCSfile: $
 *  $Revision: #2 $
 *  $Date: 2015/06/15 $
 *  $Author: cloud.chang $
 *  
 *  @par Description:
 *         This header file contains Graphics Library specific definitions,
 *         which are exported.
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_GL
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_GL_H_
#define _C_GL_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_common.h"
#include "u_sys_name.h"
#include "u_handle.h"
#include "u_fe.h"
#include "u_gl.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief  use to free any GL created handle
 *  @param  [in]    h_obj           - GL object handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_INV_HANDLE          - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_obj_free(GL_HOBJ_T h_obj);

/*------------------------------------------------------------------*/
/*! @brief  open a screen
 *  @param  [in]    ps_name         - screen name
 *  @param  [in]    pf_screen_nfy   - screen notification function
 *  @param  [in]    pv_nfy_tag      - user defined tag data
 *  @param  [out]   ph_screen       - screen handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_OUT_OF_HANDLES      - out of handles
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_NO_MEMORY           - out of memory
 *  @retval GLR_DRV_ERROR           - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_screen_open(
    const CHAR*         ps_name,
    x_gl_nfy_fct        pf_screen_nfy,
    VOID*               pv_nfy_tag,
    GL_HSCREEN_T*       ph_screen);

/*------------------------------------------------------------------*/
/*! @brief  get panel size
 *  @param  [in]    h_screen        - screen handle
 *  @param  [out]   pui4_w          - screen width
 *  @param  [out]   pui4_h          - screen height
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_INV_HANDLE          - an invalid handle is passed
 *  @retval GLR_INCOMPATIBLE_OBJ    - incompatible GL object(s) is/are passed
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_DRV_ERROR           - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_screen_get_panel_size(
    GL_HSCREEN_T        h_screen,
    UINT32              *pui4_w,
    UINT32              *pui4_h);

/*------------------------------------------------------------------*/
/*! @brief  get screen status
 *  @param  [in]    h_screen        - screen handle
 *  @param  [out]   pt_scr_status   - screen status
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ    - incompatible GL object(s) is/are passed
 *  @retval GLR_DRV_ERROR           - driver error
 *  @retval GLR_INV_HANDLE          - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_screen_get_status(
    GL_HSCREEN_T        h_screen, 
    GL_SCREEN_STATUS_T* pt_scr_status);

/*------------------------------------------------------------------*/
/*! @brief  config OSD scaler
 *  @param  [in]    h_plane         - plane handle
 *  @param  [in]    pt_pln_cfg      - OSD scaler configuration
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_DRV_ERROR           - driver error
 *  @retval GLR_UNSUPPORTED         - unsupported function
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_config(
    HANDLE_T            h_plane,
    GL_PLANE_CFG_T*     pt_pln_cfg);

/*------------------------------------------------------------------*/
/*! @brief  get GL plane size
 *  @param  [in]    h_plane         - plane handle
 *  @param  [out]   pui4_w          - width of plane
 *  @param  [out]   pui4_h          - height of plane
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_DRV_ERROR           - driver error
 *  @retval GLR_UNSUPPORTED         - unsupported function
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_get_size(
    HANDLE_T            h_plane,
    UINT32              *pui4_w,
    UINT32              *pui4_h);

/*------------------------------------------------------------------*/
/*! @brief  c_gl_plane_num
 *  @param  [out]   pui2_num        - number of OSD plane
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_num(
    UINT16*             pui2_num);

/*------------------------------------------------------------------*/
/*! @brief  enumerate the plane capability of each OSD plane
 *  @param  [in]    h_screen        - screen handle
 *  @param  [in]    ui2_idx         - plane index
 *  @param  [out]   pt_pln_caps     - plane capability
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_DRV_ERROR           - driver error
 *  @retval GLR_HDL_LIB_ERROR       - handle library error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_enum(
    GL_HSCREEN_T        h_screen, 
    UINT16              ui2_idx, 
    GL_PLANE_CAPS_T*    pt_pln_caps);

/*------------------------------------------------------------------*/
/*! @brief  open a OSD plane
 *  @param  [in]    h_screen        - screen handle
 *  @param  [in]    ui2_idx         - plane index
 *  @param  [out]   ph_plane        - plane handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                  - ok
 *  @retval GLR_NOT_INIT            - GL is not initialized
 *  @retval GLR_INV_ARG             - invalid argument is passed
 *  @retval GLR_DRV_ERROR           - driver error
 *  @retval GLR_INV_HANDLE          - an invalid handle is passed
 *  @retval GLR_OUT_OF_HANDLES      - out of handles
 *  @retval GLR_HDL_LIB_ERROR       - handle library error
 *  @retval GLR_UNLINKABLE          - this plane can not be linked
 *  @retval GLR_INCOMPATIBLE_OBJ    - incompatible GL object(s) is/are passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_open(
    GL_HSCREEN_T        h_screen, 
    UINT16              ui2_idx,
    GL_HPLANE_T*        ph_plane);

/*------------------------------------------------------------------*/
/*! @brief  link a plane to a screen
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    h_screen            - screen handle
 *  @param  [in]    b_force             - if this plane has been linked to a screen, re-link this plane to the desired screen
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_PLANE_ALREADY_LINKED    - this plane has been linked to a screen
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_link(
    GL_HPLANE_T         h_plane,
    GL_HSCREEN_T        h_screen,
    BOOL                b_forced);

/*------------------------------------------------------------------*/
/*! @brief  reorder the OSD plane
 *  @param  [in]    h_plane             - the OSD plane handle
 *  @param  [in]    e_reorder_type      - define the reorder type
 *  @param  [in]    i4_param            - new plane order
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_PLANE_NOT_LINKED        - this plane has not been linked to a screen
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_reorder(
    GL_HPLANE_T         h_plane, 
    UINT32              e_reorder_type, 
    INT32               i4_param);

/*------------------------------------------------------------------*/
/*! @brief  enable or disable the specified OSD plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    b_enalble           - enable or disable this plane
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_enable(
    GL_HPLANE_T         h_plane,
    BOOL                b_enable);

/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_clear(GL_HPLANE_T         h_plane);
extern INT32 c_gl_plane_set_color(GL_HPLANE_T h_plane,
    const GL_COLOR_T*   pt_color);

extern INT32 c_gl_plane_change_resolution(
       GL_HPLANE_T         h_plane,
       UINT32              ui4_width,
       UINT32              ui4_height);

extern INT32 c_gl_plane_change_resolution_cm(
        GL_HPLANE_T    h_plane,
        UINT32         ui4_width,
        UINT32         ui4_height,
        GL_COLORMODE_T e_cm);

/*! @brief  flip a view to a plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    h_view              - view handle
 *  @param  [in]    b_force             - if a plane has been connected to a view, force this flipping operation to be executed
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_FLIP_FAILED             - flipping operation failed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_flip(
    GL_HPLANE_T     h_plane,
    GL_HVIEW_T      h_view,
    BOOL            b_forced);

extern INT32 c_gl_surface_flip(
    GL_HSURFACE_T       h_surface,
    UINT32              ui4_flag);

extern INT32 c_gl_plane_flip_4k2k(
    GL_HPLANE_T     h_plane,
    GL_HVIEW_T      h_view,
    BOOL            b_forced);
extern INT32 c_gl_plane_set_4k2k_cropping(GL_HPLANE_T h_plane, 
                                                  BOOL        b_3d_mode,
                                                  UINT32      ui4_bottom,
                                                  UINT32      ui4_top,
                                                  UINT32      ui4_left,
                                                  UINT32      ui4_right,
                                                  BOOL        b_is_4k2k,
                                                  BOOL        b_fhd_photo);

extern INT32 c_gl_plane_change_buffer_mode(
    GL_HPLANE_T     h_plane,
    UINT8           ui1_buf_cnt);
/*------------------------------------------------------------------*/
/*! @brief  query the plane capability
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    h_screen            - screen handle
 *  @param  [out]   pt_pln_caps         - plane capability
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_get_capability(
    GL_HPLANE_T         h_plane, 
    GL_HSCREEN_T        h_screen,
    GL_PLANE_CAPS_T*    pt_pln_caps);

/*------------------------------------------------------------------*/
/*! @brief  get the linked screen of a specific plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    pf_screen_nfy       - screen status notification function
 *  @param  [in]    pv_nfy_tag          - use defined tag data
 *  @param  [out]   ph_screen           - linked screen
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_PLANE_NOT_LINKED        - this plane has been linked to a screen
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_get_linked_screen(
    GL_HPLANE_T         h_plane, 
    x_gl_nfy_fct        pf_screen_nfy,
    VOID*               pv_nfy_tag,
    GL_HSCREEN_T*       ph_screen);

extern INT32 c_gl_plane_get_resource(HANDLE_T h_plane,HANDLE_T h_on_screen_surf,UINT32 buf_width,UINT32 buf_height);
extern INT32 c_gl_plane_release_resource(HANDLE_T h_plane,HANDLE_T h_on_screen_surf);
extern INT32 c_gl_plane_set_vdp_cropping(GL_HPLANE_T h_plane, 
                                                  BOOL        b_3d_mode,
                                                  UINT32      ui4_bottom,
                                                  UINT32      ui4_top,
                                                  UINT32      ui4_left,
                                                  UINT32      ui4_right);
extern INT32 c_gl_plane_set_buffer_info(void* pv_buffer_info);
extern INT32 c_gl_surface_create_from_vdp(
    const GL_SURFACE_DESC_T*    pt_surf_desc, 
    GL_HSURFACE_T*              ph_surf);

extern INT32 c_gl_surface_vdp_strech_blend(
    GL_HSURFACE_T       h_surf_dst, 
    GL_HSURFACE_T       h_surf_src_pic, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    GL_HSURFACE_T       h_surf_src_black,
    UINT8               ui1_alpha);


/*------------------------------------------------------------------*/
/*! @brief  query the order of a specific plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [out]   pi4_order           - plane order
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_PLANE_NOT_LINKED        - this plane has been linked to a screen
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_get_order(
    GL_HPLANE_T         h_plane, 
    INT32               *pi4_order);

/*------------------------------------------------------------------*/
/*! @brief  get the status of a specific OSD plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [out]   pt_pln_sts          - plane status
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_PLANE_NOT_LINKED        - this plane has been linked to a screen
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_get_status(
    GL_HPLANE_T         h_plane, 
    GL_PLANE_STATUS_T*  pt_pln_sts);

/*------------------------------------------------------------------*/
/*! @brief  get a alpha value to a specific OSD plane
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    ui1_alpha           - alpha value to be set
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_plane_set_alpha(
    GL_HPLANE_T         h_plane,
    UINT8               ui1_alpha);



extern INT32 c_gl_plane_enable_colorkey(
    GL_HPLANE_T         h_plane,
    BOOL                b_new_enable,
    BOOL*               old_enable_fg);


/*------------------------------------------------------------------*/
/*! @brief  create a view
 *  @param  [in]    ui4_w               - width of this view
 *  @param  [in]    ui4_h               - height of this view
 *  @param  [in]    b_auto_scale        - if the size of this view is not the same as the screen, scale it or not
 *  @param  [out]   ph_view             - view handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_view_create(
    UINT32          ui4_w,
    UINT32          ui4_h,
    BOOL            b_auto_scale,
    GL_HVIEW_T*     ph_view);

/*------------------------------------------------------------------*/
/*! @brief  create a viewport
 *  @param  [in]    h_view              - view handle
 *  @param  [in]    h_surface           - surface handle
 *  @param  [in]    pt_disp_opt         - viewport configuration
 *  @param  [out]   pui4_id             - viewport ID
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_view_create_viewport(
    GL_HVIEW_T              h_view, 
    GL_HSURFACE_T           h_surface, 
    const GL_DISPLAY_OPT_T* pt_disp_opt,
    UINT32*                 pui4_id);

/*------------------------------------------------------------------*/
/*! @brief  delete a viewport
 *  @param  [in]    h_view              - view handle
 *  @param  [in]    ui4_id              - viewport ID
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_HDL_LIB_ERROR           - handle library error
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_view_delete_viewport(
    GL_HVIEW_T      h_view,
    UINT32          ui4_id);

/*------------------------------------------------------------------*/
/*! @brief  reconfigure a viewport setting
 *  @param  [in]    h_plane             - plane handle
 *  @param  [in]    h_view              - view handle
 *  @param  [in]    ui4_viewport_id     - viewport ID
 *  @param  [in]    i4_view_x           - x position of a viewport on the view
 *  @param  [in]    i4_view_y           - y position of a viewport on the view
 *  @param  [in]    ui4_width           - viewport width
 *  @param  [in]    ui4_height          - viewport height
 *  @param  [in]    ui4_surf_offset_x   - x offset on the surface for this viewport
 *  @param  [in]    ui4_surf_offset_y   - y offset on the surface for this viewport
 *  @param  [in]    fx_shrink_x         - shrinking ratio in x direction
 *  @param  [in]    fx_shrink_y         - shrinking ratio in y direction
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_view_reconfig_viewport(
    GL_HPLANE_T     h_plane,
    GL_HVIEW_T      h_view,
    UINT32          ui4_viewport_id,
    INT32           i4_view_x,
    INT32           i4_view_y,
    UINT32          ui4_width,
    UINT32          ui4_height,
    UINT32          ui4_surf_offset_x,
    UINT32          ui4_surf_offset_y,
    GL_FIX16_T      fx_shrink_x,
    GL_FIX16_T      fx_shrink_y);

/* composite graphics API */
/*------------------------------------------------------------------*/
/*! @brief  create a graphics
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_translation      - surface translation
 *  @param  [in]    pt_clip             - clipper region
 *  @param  [out]   ph_graphics         - graphics handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_create_graphics(
    GL_HSURFACE_T       h_surf,
    const GL_POINT_T*   pt_translation,
    const GL_RECT_T*    pt_clip,
    GL_HGRAPHICS_T*     ph_graphics);

/*------------------------------------------------------------------*/
/*! @brief  change the translation of a graphics
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    i4_trans_x          - translation in x direction
 *  @param  [in]    i4_trans_y          - translation in y direction
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_translate(
    GL_HGRAPHICS_T      h_graphics, 
    INT32               i4_trans_x, 
    INT32               i4_trans_y);

/*------------------------------------------------------------------*/
/*! @brief  set the clipper to a graphics
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_clip             - clipper region
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_set_clip(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_clip);
        
/*------------------------------------------------------------------*/
/*! @brief  set graphics color
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_color            - graphics color
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_set_color(
    GL_HGRAPHICS_T      h_graphics,
    const GL_COLOR_T*   pt_color);

/*------------------------------------------------------------------*/
/*! @brief  set the pen width for primitive drawing
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    ui2_width           - pen width
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_set_pen_width(
    GL_HGRAPHICS_T      h_graphics,
    UINT16              ui2_width);

/*------------------------------------------------------------------*/
/*! @brief  draw a line
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_start            - starting point
 *  @param  [in]    pt_end              - ending point
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_line(
    GL_HGRAPHICS_T      h_graphics,
    const GL_POINT_T*   pt_start, 
    const GL_POINT_T*   pt_end);

/*------------------------------------------------------------------*/
/*! @brief  draw a rectangle
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_rect(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound);
    
/*------------------------------------------------------------------*/
/*! @brief  fill a rectangle
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_rect(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound);
    
/*------------------------------------------------------------------*/
/*! @brief  draw a rounded-corner rectangle
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region
 *  @param  [in]    pt_arc              - contains the arc information for rounded corners
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_round_rect(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    const GL_SIZE_T*    pt_arc);
    
/*------------------------------------------------------------------*/
/*! @brief  fill a rounded-corner rectangle
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region
 *  @param  [in]    pt_arc              - contains the arc information for rounded corners
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_round_rect(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    const GL_SIZE_T*    pt_arc);

/*------------------------------------------------------------------*/
/*! @brief  draw a polygon
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    ui4_count           - vertex count of this polygon
 *  @param  [in]    at_vertex           - contains the vertex information for this polygon
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_polygon(
    GL_HGRAPHICS_T      h_graphics,
    UINT32              ui4_count, 
    const GL_POINT_T    at_vertex[]);
    
/*------------------------------------------------------------------*/
/*! @brief  fill a polygon
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    ui4_count           - vertex count of this polygon
 *  @param  [in]    at_vertex           - contains the vertex information for this polygon
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_polygon(
    GL_HGRAPHICS_T      h_graphics,
    UINT32              ui4_count, 
    const GL_POINT_T    at_vertex[]);

/*------------------------------------------------------------------*/
/*! @brief  draw an ellipse
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this ellipse
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_ellipse(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound);
    
/*------------------------------------------------------------------*/
/*! @brief  fill an ellipse
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this ellipse
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_ellipse(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound);

/*------------------------------------------------------------------*/
/*! @brief  draw an arc
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this arc
 *  @param  [in]    i4_deg_start        - the starting degree for this arc
 *  @param  [in]    i4_deg_end          - the ending degree for this arc
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_arc(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    INT32               i4_deg_start, 
    INT32               i4_deg_end);

/*------------------------------------------------------------------*/
/*! @brief  fill an arc
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this arc
 *  @param  [in]    i4_deg_start        - the starting degree for this arc
 *  @param  [in]    i4_deg_end          - the ending degree for this arc
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_arc(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    INT32               i4_deg_start, 
    INT32               i4_deg_end);

/*------------------------------------------------------------------*/
/*! @brief  draw a pie
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this pie
 *  @param  [in]    i4_deg_start        - the starting degree for this pie
 *  @param  [in]    i4_deg_end          - the ending degree for this pie
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_pie(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    INT32               i4_deg_start, 
    INT32               i4_deg_end);

/*------------------------------------------------------------------*/
/*! @brief  fill a pie
 *  @param  [in]    h_graphics          - graphics handle
 *  @param  [in]    pt_bound            - contains the rectangle region for this pie
 *  @param  [in]    i4_deg_start        - the starting degree for this pie
 *  @param  [in]    i4_deg_end          - the ending degree for this pie
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_fill_pie(
    GL_HGRAPHICS_T      h_graphics,
    const GL_RECT_T*    pt_bound,
    INT32               i4_deg_start, 
    INT32               i4_deg_end);

/*------------------------------------------------------------------*/
/*! @brief  perform the bit-blitting operation
 *  @param  [in]    h_graphics          - graphics handle (destination)
 *  @param  [in]    h_surf              - surface handle (source)
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the bit-blitting region
 *  @param  [in]    ui4_height          - height of the bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_bitblt(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf,
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha_assigned);

/*------------------------------------------------------------------*/
/*! @brief  perform the bit-blitting operation with pattern repetition
 *  @param  [in]    h_graphics          - graphics handle (destination)
 *  @param  [in]    h_surf              - surface handle (source)
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    ui4_dst_w           - width of the destination bit-blitting region
 *  @param  [in]    ui4_dst_h           - height of the destination bit-blitting region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source bit-blitting region
 *  @param  [in]    ui4_src_h           - height of the source bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_patblt(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_alpha_assigned);

/*------------------------------------------------------------------*/
/*! @brief  copies the content of the source surface to the target graphics with source color key specified
 *  @param  [in]    h_graphics          - graphics handle (destination)
 *  @param  [in]    h_surf              - surface handle (source)
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the bit-blitting region
 *  @param  [in]    ui4_height          - height of the bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    b_off_mask          - contains the flag to indicate if an off-mask bit-blitting should be performed. If this field is TRUE, only pixel data outside the specified color key range are copied, otherwise pixel data within the range are blitted
 *  @param  [in]    pt_color_min        - the lower bound of the specified source color key range
 *  @param  [in]    pt_color_max        - the upper bound of the specified source color key range
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_transp_blt(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha_assigned,
    BOOL                b_off_mask,
    const GL_COLOR_T*   pt_color_min,
    const GL_COLOR_T*   pt_color_max);

/*------------------------------------------------------------------*/
/*! @brief  copies the content of the source surface to the target graphics with source color key specified and pattern repetition
 *  @param  [in]    h_graphics          - graphics handle (destination)
 *  @param  [in]    h_surf              - surface handle (source)
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    ui4_dst_w           - width of the destination bit-blitting region
 *  @param  [in]    ui4_dst_h           - height of the destination bit-blitting region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source bit-blitting region
 *  @param  [in]    ui4_src_h           - height of the source bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    b_off_mask          - contains the flag to indicate if an off-mask bit-blitting should be performed. If this field is TRUE, only pixel data outside the specified color key range are copied, otherwise pixel data within the range are blitted
 *  @param  [in]    pt_color_min        - the lower bound of the specified source color key range
 *  @param  [in]    pt_color_max        - the upper bound of the specified source color key range
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_transp_patblt(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_alpha_assigned,
    BOOL                b_off_mask,
    const GL_COLOR_T*   pt_color_min,
    const GL_COLOR_T*   pt_color_max);

/*------------------------------------------------------------------*/
/*! @brief  stretch the source surface into the destination graphics
 *  @param  [in]    h_graphcis          - destination graphics handle
 *  @param  [in]    h_surf              - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    ui4_dst_w           - width of the destination region
 *  @param  [in]    ui4_dst_h           - height of the destination region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source region
 *  @param  [in]    ui4_src_h           - height of the source region
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_stretch_blt(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h);

/*------------------------------------------------------------------*/
/*! @brief  blend the source surface onto the destination graphics
 *  @param  [in]    h_graphics          - destination graphics handle
 *  @param  [in]    h_surf              - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the blending region
 *  @param  [in]    ui4_height          - height of the blending region
 *  @param  [in]    ui1_alpha           - blending level
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_blend(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha);

/*------------------------------------------------------------------*/
/*! @brief  compose the source surface onto the destination graphics
 *  @param  [in]    h_graphics          - destination graphics handle
 *  @param  [in]    h_surf              - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the composing region
 *  @param  [in]    ui4_height          - height of the composing region
 *  @param  [in]    ui1_pre_alpha       - an alpha value which would be multiplied to each pixel of the source surface
 *  @param  [in]    e_rule              - Porter-Duff composition rule
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_compose(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_pre_alpha,
    GL_COMPOSE_RULE_T   e_rule);

/*------------------------------------------------------------------*/
/*! @brief  compose the source surface onto the destination graphics with pattern repetition
 *  @param  [in]    h_graphics          - destination graphics handle
 *  @param  [in]    h_surf              - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination graphics
 *  @param  [in]    i4_dst_y            - y position on the destination graphics
 *  @param  [in]    ui4_dst_w           - width of the destination composing region
 *  @param  [in]    ui4_dst_h           - height of the destination composing region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source composing region
 *  @param  [in]    ui4_src_h           - height of the source composing region
 *  @param  [in]    ui1_pre_alpha       - an alpha value which would be multiplied to each pixel of the source surface
 *  @param  [in]    e_rule              - Porter-Duff composition rule
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_pat_compose(
    GL_HGRAPHICS_T      h_graphics,
    GL_HSURFACE_T       h_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_pre_alpha,
    GL_COMPOSE_RULE_T   e_rule);

/*------------------------------------------------------------------*/
/*! @brief  draw a string
 *  @param  [in]    h_graphics          - destination graphics handle
 *  @param  [in]    i4_x                - x position on the destination graphics
 *  @param  [in]    i4_y                - y position on the destination graphics
 *  @param  [in]    i4_bg_y             - y position of the background on the destination graphics
 *  @param  [in]    w2s_str             - pointer to the string to be drawn
 *  @param  [in]    ui2_strlen          - string length of w2s_str
 *  @param  [in]    w2s_skip            - pointer to the string which contains characters to be skipped in w2s_str
 *  @param  [in]    ui2_skiplen         - string length of w2s_skip
 *  @param  [in]    h_fnt               - font handle
 *  @param  [in]    pt_fg_clr           - foreground color
 *  @param  [in]    pt_bg_clr           - background color
 *  @param  [in]    pt_edge_clr         - edge color
 *  @param  [in]    b_fixed             - indicates if the interval between characters is fixed
 *  @param  [in]    ui2_ch_w            - when b_fixed is TRUE, this indicates what the interval is
 *  @param  [in]    ui2_bg_h            - indicates the height of the background
 *  @param  [in]    i4_bg_adj_x         - indicates the offset of background in the x direction
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 *  @retval GLR_FE_ERROR                - font library error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_draw_ustring_ex(
    GL_HGRAPHICS_T      h_graphics,
    INT32               i4_x,
    INT32               i4_y,
    INT32               i4_bg_y,
    UTF16_T*            w2s_str,
    UINT16              ui2_strlen,
    UTF16_T*            w2s_skip,
    UINT16              ui2_skiplen,
    HFONT_T             h_fnt,
    const GL_COLOR_T*   pt_fg_clr,
    const GL_COLOR_T*   pt_bg_clr,
    const GL_COLOR_T*   pt_edge_clr,
    BOOL                b_fixed,
    UINT16              ui2_ch_w,
    UINT16              ui2_bg_h,
    INT32               i4_bg_adj_x);

/*------------------------------------------------------------------*/
/*! @brief  get the GPU capability
 *  @param  [out]   pt_caps             - GPU capability
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_get_gpu_capability(
    GL_GPU_CAPS_T       *pt_caps);

/*------------------------------------------------------------------*/
/*! @brief  create a surface
 *  @param  [in]    pt_surf_desc        - surface description
 *  @param  [out]   ph_surf             - surface handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_create(
    const GL_SURFACE_DESC_T*    pt_surf_desc, 
    GL_HSURFACE_T*              ph_surf);

/*------------------------------------------------------------------*/
/*! @brief  create a surface from FBM
 *  @param  [in]    pt_surf_desc        - surface description
 *  @param  [out]   ph_surf             - surface handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_create_from_fbm(
    const GL_SURFACE_DESC_T*    pt_surf_desc, 
    GL_HSURFACE_T*              ph_surf);

extern INT32 c_gl_surface_create_from_dfb(
    const GL_SURFACE_DESC_T*    pt_surf_desc, 
    GL_HSURFACE_T*              ph_surf);


/*------------------------------------------------------------------*/
/*! @brief  lock a surface to acquire its related information
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_rect             - region to be locked. NULL should be passed for now to lock all surface region
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @param  [out]   pt_lock_info        - surface lock information
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_lock(
    GL_HSURFACE_T       h_surf, 
    const GL_RECT_T*    pt_rect, 
    UINT32              ui4_flag, 
    GL_LOCK_INFO_T*     pt_lock_info);

/*------------------------------------------------------------------*/
/*! @brief  unlock a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_rect             - region to be locked. NULL should be passed for now to lock all surface region
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_NOT_LOCKED      - this surface is not locked
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_unlock(
    GL_HSURFACE_T       h_surf, 
    const GL_RECT_T*    pt_rect);

/*------------------------------------------------------------------*/
/*! @brief  get surface information
 *  @param  [in]    h_surf              - surface handle
 *  @param  [out]   pt_surf_info        - surface information
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_get_info(
    GL_HSURFACE_T       h_surf, 
    GL_SURF_INFO_T*     pt_surf_info);

/*------------------------------------------------------------------*/
/*! @brief  perform the bit-blitting operation with additional functionality (alpha/color value/map)
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the bit-blitting region
 *  @param  [in]    ui4_height          - height of the bit-blitting region
 *  @param  [in]    pt_blt_ex           - data used for performing additional functions
 *  @param  [in]    ui4_blt_ex_opt      - indicates which operation to be performed (BLT_EX_OPT_ALPHAVAL_COLORMAP_REP, BLT_EX_OPT_ALPHAVAL_COLORMAP_MUL, BLT_EX_OPT_ALPHAMAP_COLORVAL, BLT_EX_OPT_ALPHAMAP_COLORMAP)
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_blt_ex(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    const GL_BLT_EX_T*  pt_blt_ex,
    UINT8               ui1_blt_ex_opt,
    UINT32              ui4_flag);
    
/*------------------------------------------------------------------*/
/*! @brief  perform the bit-blitting operation
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the bit-blitting region
 *  @param  [in]    ui4_height          - height of the bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_blt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha_assigned,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw a line on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_start            - starting point
 *  @param  [in]    pt_end              - ending point
 *  @param  [in]    pt_color            - line color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_line(
    GL_HSURFACE_T       h_surf,
    const GL_POINT_T*   pt_start, 
    const GL_POINT_T*   pt_end,
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw a rectangle on a surface
 *  @param  [in]    h_surface           - surface handle
 *  @param  [in]    pt_rect             - contains the rectangle region
 *  @param  [in]    pt_color            - rectangle color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - rectangle style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_rect(
    GL_HSURFACE_T       h_surf,
    const GL_RECT_T*    pt_rect,
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    GL_STYLE_T          t_style,
    UINT32              ui4_flag);


/*------------------------------------------------------------------*/
/*! @brief  query the size of a surface in bytes
 *  @param  [in]    h_surf              - surface handle
 *  @param  [out]   pui4_max_leng       - surface size in bytes
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_get_bitmap_max_leng(
    GL_HSURFACE_T       h_surf,
    UINT32*             pui4_max_leng);
    
/*------------------------------------------------------------------*/
/*! @brief  draw a rounded-corner rectangle on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_rect             - contains the rectangle region
 *  @param  [in]    pt_arc              - contains the arc information for rounded corners
 *  @param  [in]    pt_color            - rectangle color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - rectangle style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_round_rect(
    GL_HSURFACE_T           h_surf,
    const GL_RECT_T*        pt_rect,
    const GL_ARC_DESC_T*    pt_arc,
    const GL_COLOR_T*       pt_color,
    UINT16                  ui2_line_width,
    GL_STYLE_T              t_style,
    UINT32                  ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw an ellipse on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_rect             - contains the rectangle region for this ellipse
 *  @param  [in]    pt_color            - ellipse color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - ellipse style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_ellipse(
    GL_HSURFACE_T       h_surf,
    const GL_RECT_T*    pt_rect,
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    GL_STYLE_T          t_style,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw a polygon on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    ui4_count           - vertex count of this polygon
 *  @param  [in]    at_vertex           - contains the vertex information for this polygon
 *  @param  [in]    pt_color            - polygon color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - polygon style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_polygon(
    GL_HSURFACE_T       h_surf,
    UINT32              ui4_count, 
    const GL_POINT_T    at_vertex[],
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    GL_STYLE_T          t_style,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw an arc on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_rect             - contains the rectangle region for this arc
 *  @param  [in]    i4_deg_start        - the starting degree for this arc
 *  @param  [in]    i4_deg_end          - the ending degree for this arc
 *  @param  [in]    pt_color            - arc color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - arc style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_arc(
    GL_HSURFACE_T       h_surf,
    const GL_RECT_T*    pt_rect,
    INT32               i4_deg_start, 
    INT32               i4_deg_end,
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    GL_STYLE_T          t_style,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw a pie on a surface
 *  @param  [in]    h_surface           - surface handle
 *  @param  [in]    pt_rect             - contains the rectangle region for this pie
 *  @param  [in]    i4_deg_start        - the starting degree for this pie
 *  @param  [in]    i4_deg_end          - the ending degree for this pie
 *  @param  [in]    pt_color            - pie color
 *  @param  [in]    ui2_line_width      - line width
 *  @param  [in]    t_style             - pie style. It could be GL_STYLE_OUTLINED or GL_STYLE_FILLED
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_pie(
    GL_HSURFACE_T       h_surf,
    const GL_RECT_T*    pt_rect,
    INT32               i4_deg_start, 
    INT32               i4_deg_end,
    const GL_COLOR_T*   pt_color,
    UINT16              ui2_line_width,
    GL_STYLE_T          t_style,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  perform the bit-blitting operation with pattern repetition
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination bit-blitting region
 *  @param  [in]    ui4_dst_h           - height of the destination bit-blitting region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source bit-blitting region
 *  @param  [in]    ui4_src_h           - height of the source bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_patblt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_alpha_assigned,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  stretch the source surface into the destination surface with repeat-and-drop algorithm
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination region
 *  @param  [in]    ui4_dst_h           - height of the destination region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source region
 *  @param  [in]    ui4_src_h           - height of the source region
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_stretch_blt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT32              ui4_flag);

extern INT32 c_gl_surface_compose_with_stretch(
	GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_pre_alpha,
    GL_COMPOSE_RULE_T   e_rule,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  stretch the source surface into the destination surface with interpolation algorithm
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination region
 *  @param  [in]    ui4_dst_h           - height of the destination region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source region
 *  @param  [in]    ui4_src_h           - height of the source region
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_adv_stretch_blt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  stretch the source surface into the destination surface with interpolation algorithm and options
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination region
 *  @param  [in]    ui4_dst_h           - height of the destination region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source region
 *  @param  [in]    ui4_src_h           - height of the source region
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @param  [in]    e_option            - specifies this operation is for UI style data or photo related data
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_adv_stretch_blt_ex(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT32              ui4_flag,
    GL_STRETCH_OPTION   e_option);

/*------------------------------------------------------------------*/
/*! @brief  copies the content of the source surface to the target surface with source color key specified
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the bit-blitting region
 *  @param  [in]    ui4_height          - height of the bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    b_off_mask          - contains the flag to indicate if an off-mask bit-blitting should be performed. If this field is TRUE, only pixel data outside the specified color key range are copied, otherwise pixel data within the range are blitted
 *  @param  [in]    pt_color_min        - the lower bound of the specified source color key range
 *  @param  [in]    pt_color_max        - the upper bound of the specified source color key range
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_transp_blt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha_assigned,
    BOOL                b_off_mask,
    const GL_COLOR_T*   pt_color_min,
    const GL_COLOR_T*   pt_color_max,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  change a rectangle source surface to a quadrilateral destination surface with its reflection
 *  @param  [in]    h_dst_surf           - destination surface handle
 *  @param  [in]    h_src_surf            -source surface handle
 *  @param  [in]    t_src_quad           - source quadrilateral
 *  @param  [in]    t_dst_quad         - destination quadrilateral
 *  @param  [in]    t_reflect         - the destination reflection structure
 *  @param  [in]   b_alpha_comp_en          - alpha composition
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/

extern INT32 c_gl_surface_scaler_reflection(
    GL_HGRAPHICS_T              h_dst_graphics, 
    GL_HSURFACE_T               h_src_surf,
    GL_QUADRILATERAL_T     t_src_quad,
    GL_QUADRILATERAL_T     t_dst_quad,
    GL_REFLECT_T*               t_reflect,
    BOOL                    b_alpha_comp_en);

/*------------------------------------------------------------------*/
/*! @brief  rotate a source surface onto the destination surface
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the rotation region
 *  @param  [in]    ui4_height          - height of the rotation region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    ui4_rot_op          - rotation options
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_rotate_blt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha_assigned,
    UINT32              ui4_rot_op,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  copies the content of the source surface to the target surface with source color key specified and pattern repetition
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination bit-blitting region
 *  @param  [in]    ui4_dst_h           - height of the destination bit-blitting region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source bit-blitting region
 *  @param  [in]    ui4_src_h           - height of the source bit-blitting region
 *  @param  [in]    ui1_alpha_assigned  - if the source surface is without alpha channel, apply this value as a default alpha on the destination
 *  @param  [in]    b_off_mask          - contains the flag to indicate if an off-mask bit-blitting should be performed. If this field is TRUE, only pixel data outside the specified color key range are copied, otherwise pixel data within the range are blitted
 *  @param  [in]    pt_color_min        - the lower bound of the specified source color key range
 *  @param  [in]    pt_color_max        - the upper bound of the specified source color key range
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_transp_patblt(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_alpha_assigned,
    BOOL                b_off_mask,
    const GL_COLOR_T*   pt_color_min,
    const GL_COLOR_T*   pt_color_max,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  blend the source surface onto the destination surface
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the blending region
 *  @param  [in]    ui4_height          - height of the blending region
 *  @param  [in]    ui1_alpha           - blending level
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_blend(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_alpha,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  compose the source surface onto the destination surface
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_width           - width of the composing region
 *  @param  [in]    ui4_height          - height of the composing region
 *  @param  [in]    ui1_pre_alpha       - an alpha value which would be multiplied to each pixel of the source surface
 *  @param  [in]    e_rule              - Porter-Duff composition rule
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_compose(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_width,
    UINT32              ui4_height,
    UINT8               ui1_pre_alpha,
    GL_COMPOSE_RULE_T   e_rule,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  compose the source surface onto the destination surface with pattern repetition
 *  @param  [in]    h_dst_surf          - destination surface handle
 *  @param  [in]    h_src_surf          - source surface handle
 *  @param  [in]    i4_dst_x            - x position on the destination surface
 *  @param  [in]    i4_dst_y            - y position on the destination surface
 *  @param  [in]    ui4_dst_w           - width of the destination composing region
 *  @param  [in]    ui4_dst_h           - height of the destination composing region
 *  @param  [in]    i4_src_x            - x position on the source surface
 *  @param  [in]    i4_src_y            - y position on the source surface
 *  @param  [in]    ui4_src_w           - width of the source composing region
 *  @param  [in]    ui4_src_h           - height of the source composing region
 *  @param  [in]    ui1_pre_alpha       - an alpha value which would be multiplied to each pixel of the source surface
 *  @param  [in]    e_rule              - Porter-Duff composition rule
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_pat_compose(
    GL_HSURFACE_T       h_dst_surf, 
    GL_HSURFACE_T       h_src_surf, 
    INT32               i4_dst_x,
    INT32               i4_dst_y,
    UINT32              ui4_dst_w,
    UINT32              ui4_dst_h,
    INT32               i4_src_x,
    INT32               i4_src_y,
    UINT32              ui4_src_w,
    UINT32              ui4_src_h,
    UINT8               ui1_pre_alpha,
    GL_COMPOSE_RULE_T   e_rule,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  draw characters onto a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    i4_x                - x position on the destination surface
 *  @param  [in]    i4_y                - y position on the destination surface
 *  @param  [in]    aw2_chars           - an array for characters to be drawn
 *  @param  [in]    ui2_len             - length of aw2_chars
 *  @param  [in]    h_fnt               - font handle
 *  @param  [in]    pt_fg_clr           - foreground color
 *  @param  [in]    pt_bg_clr           - background color
 *  @param  [out]   pr_rc_str           - the rendering rectangle covered by drawn characters
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 *  @retval GLR_FE_ERROR                - font library error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_uchars(
    GL_HSURFACE_T       h_surf,
    INT32               i4_x,
    INT32               i4_y,
    UTF16_T             aw2_chars[],
    UINT16              ui2_len,
    HFONT_T             h_fnt,
    const GL_COLOR_T*   pt_fg_clr,
    const GL_COLOR_T*   pt_bg_clr,
    GL_RECT_T*          pt_rc_str);

/*------------------------------------------------------------------*/
/*! @brief  draw a string on a surface
 *  @param  [in]    h_surf              - destination surface handle
 *  @param  [in]    i4_x                - x position on the destination surface
 *  @param  [in]    i4_y                - y position on the destination surface
 *  @param  [in]    w2s_str             - pointer to the string to be drawn
 *  @param  [in]    ui2_strlen          - string length of w2s_str
 *  @param  [in]    h_fnt               - font handle
 *  @param  [in]    pt_fg_clr           - foreground color
 *  @param  [in]    pt_bg_clr           - background color
 *  @param  [out]   pr_rc_str           - the rendering rectangle covered by drawn characters
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 *  @retval GLR_FE_ERROR                - font library error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_ustring(
    GL_HSURFACE_T       h_surf,
    INT32               i4_x,
    INT32               i4_y,
    UTF16_T*            w2s_str,
    UINT16              ui2_strlen,
    HFONT_T             h_fnt,
    const GL_COLOR_T*   pt_fg_clr,
    const GL_COLOR_T*   pt_bg_clr,
    GL_RECT_T*          pt_rc_str);

/*------------------------------------------------------------------*/
/*! @brief  draw a string on a surface with more options
 *  @param  [in]    h_surf              - destination surface handle
 *  @param  [in]    i4_x                - x position on the destination surface
 *  @param  [in]    i4_y                - y position on the destination surface
 *  @param  [in]    i4_bg_y             - y position of the background on the destination surface
 *  @param  [in]    w2s_str             - pointer to the string to be drawn
 *  @param  [in]    ui2_strlen          - string length of w2s_str
 *  @param  [in]    w2s_skip            - pointer to the string which contains characters to be skipped in w2s_str
 *  @param  [in]    ui2_skiplen         - string length of w2s_skip
 *  @param  [in]    h_fnt               - font handle
 *  @param  [in]    pt_fg_clr           - foreground color
 *  @param  [in]    pt_bg_clr           - background color
 *  @param  [in]    pt_edge_clr         - edge color
 *  @param  [in]    b_fixed             - indicates if the interval between characters is fixed
 *  @param  [in]    ui2_ch_w            - when b_fixed is TRUE, this indicates what the interval is
 *  @param  [in]    ui2_bg_h            - indicates the height of the background
 *  @param  [in]    i4_bg_adj_x         - indicates the offset of background in the x direction
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 *  @retval GLR_FE_ERROR                - font library error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_draw_ustring_ex(
    GL_HSURFACE_T       h_surf,
    INT32               i4_x,
    INT32               i4_y,
    INT32               i4_bg_y,
    UTF16_T*            w2s_str,
    UINT16              ui2_strlen,
    UTF16_T*            w2s_skip,
    UINT16              ui2_skiplen,
    HFONT_T             h_fnt,
    const GL_COLOR_T*   pt_fg_clr,
    const GL_COLOR_T*   pt_bg_clr,
    const GL_COLOR_T*   pt_edge_clr,
    BOOL                b_fixed,
    UINT16              ui2_ch_w,
    UINT16              ui2_bg_h,
    INT32               i4_bg_adj_x);

/*------------------------------------------------------------------*/
/*! @brief  fill a surface with a specific color
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    pt_color            - color to be filled
 *  @param  [in]    ui4_flag            - synchorization flag. It could be 0, GL_SYNC, or GL_NOWAIT
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_SURFACE_LOCKED          - the associated surface is locked
 *  @retval GLR_SURFACE_BUSY            - the associated surface is busy
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_fill(
    GL_HSURFACE_T       h_surf, 
    const GL_COLOR_T*   pt_color,
    UINT32              ui4_flag);

/*------------------------------------------------------------------*/
/*! @brief  reconfig the surface dimension without reallocating memory
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    ui4_width           - new surface width
 *  @param  [in]    ui4_height          - new surface height
 *  @param  [in]    e_mode              - new color mode
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_INCOMPATIBLE_SURFACE    - input surface(s) is/are not compatible
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_reconfig(
    GL_HSURFACE_T   h_surf,
    UINT32          ui4_width,
    UINT32          ui4_height,
    GL_COLORMODE_T  e_mode);

/*------------------------------------------------------------------*/
/*! @brief  set a palette on a surface
 *  @param  [in]    h_surf              - surface handle
 *  @param  [in]    h_palette           - palette handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                          - ok
 *  @retval GLR_NOT_INIT                    - GL is not initialized
 *  @retval GLR_INV_ARG                     - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ            - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE                  - an invalid handle is passed
 *  @retval GLR_DRV_ERROR                   - driver error
 *  @retval GLR_OUT_OF_HANDLES              - out of handles
 *  @retval GLR_HDL_LIB_ERROR               - handle library error
 *  @retval GLR_INV_COLORMODE               - invalid colormode
 *  @retval GLR_SURFACE_ALREADY_ATTACHED    - surface has been attached on a view
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_set_palette(
    GL_HSURFACE_T       h_surf, 
    GL_HPALETTE_T       h_palette);

/*------------------------------------------------------------------*/
/*! @brief  set a clipper on a surface
 *  @param  [in]    h_surf                  - surface handle
 *  @param  [in]    h_clipper               - clipper handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                          - ok
 *  @retval GLR_NOT_INIT                    - GL is not initialized
 *  @retval GLR_INV_ARG                     - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ            - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE                  - an invalid handle is passed
 *  @retval GLR_OUT_OF_HANDLES              - out of handles
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_surface_set_clipper(
    GL_HSURFACE_T       h_surf, 
    GL_HCLIPPER_T       h_clipper);

/*------------------------------------------------------------------*/
/*! @brief  create a clipper
 *  @param  [in]    ui2_count           - number of clipping region associated with this clipper. Only 1 region is allowed in current implementation
 *  @param  [in]    pt_rect             - clipping region
 *  @param  [out]   ph_clipper          - clipper handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_clipper_create(
    UINT16              ui2_count, 
    const GL_RECT_T*    pt_rect, 
    GL_HCLIPPER_T*      ph_clipper);

/*------------------------------------------------------------------*/
/*! @brief  change the clipping list of a clipper
 *  @param  [in]    h_clipper           - clipper handle
 *  @param  [in]    ui2_count           - number of clipping region to be associated with thish clipper. Only 1 region is allowed in current implementation
 *  @param  [in]    pt_rect             - pointer to the clipping region list
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_clipper_set_clip_list(
    GL_HCLIPPER_T       h_clipper, 
    UINT16              ui2_count, 
    const GL_RECT_T*    pt_rect);

/*------------------------------------------------------------------*/
/*! @brief  get the clipping list of a clipper
 *  @param  [in]    h_clipper           - clipper handle
 *  @param  [out]   pui2_count          - number of clipping region associated with this clipper
 *  @param  [out]   pt_rect             - pointer to the clipping region list
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_clipper_get_clip_list(
    GL_HCLIPPER_T       h_clipper, 
    UINT16*             pui2_count, 
    GL_RECT_T*          pt_rect);

/*------------------------------------------------------------------*/
/*! @brief  create a palette
 *  @param  [in]    e_clrmode           - color mode of this palette table
 *  @param  [in]    ui2_count           - number of entries in this palette table
 *  @param  [in]    at_color            - an array of colors to be set into this palette table
 *  @param  [out]   ph_palette          - palette handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_create(
    GL_COLORMODE_T      e_clrmode, 
    UINT16              ui2_count,
    const GL_COLOR_T    at_color[], 
    GL_HPALETTE_T*      ph_palette);

/*------------------------------------------------------------------*/
/*! @brief  create a palette with a specified palette ID
 *  @param  [in]    e_clrmode           - color mode of this palette table
 *  @param  [in]    ui2_count           - number of entries in this palette table
 *  @param  [in]    at_color            - an array of colors to be set into this palette table
 *  @param  [in]    ui4_id              - palette ID
 *  @param  [out]   ph_palette          - palette handle
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 *  @retval GLR_OUT_OF_HANDLES          - out of handles
 *  @retval GLR_NO_MEMORY               - out of memory
 *  @retval GLR_INV_COLORMODE           - invalid colormode
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_create_with_id(
    GL_COLORMODE_T      e_clrmode, 
    UINT16              ui2_count,
    const GL_COLOR_T    at_color[], 
    UINT32              ui4_id,
    GL_HPALETTE_T*      ph_palette);

/*------------------------------------------------------------------*/
/*! @brief  set the palette entry
 *  @param  [in]    h_palette           - palette handle
 *  @param  [in]    ui2_base            - the base palette entry to be set
 *  @param  [in]    ui2_count           - number of palette entries from entry[ui2_base] to be set
 *  @param  [in]    at_color            - an array of colors to be set into the palette entry
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 *  @retval GLR_DRV_ERROR               - driver error
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_set_entries(
    GL_HPALETTE_T       h_palette, 
    UINT16              ui2_base, 
    UINT16              ui2_count, 
    const GL_COLOR_T    at_color[]);

/*------------------------------------------------------------------*/
/*! @brief  get palette entries from a palette table
 *  @param  [in]    h_palette           - palette handle
 *  @param  [in]    ui2_base            - the base palette entry to be got
 *  @param  [in]    ui2_count           - number of palette entries from entry[ui2_base] to be got
 *  @param  [out]   at_color            - an array to get colors from the specified palette entries
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_get_entries(
    GL_HPALETTE_T       h_palette, 
    UINT16              ui2_base, 
    UINT16              ui2_count, 
    GL_COLOR_T          at_color[]);

/*------------------------------------------------------------------*/
/*! @brief  find the closest color in a palette table for a specified color
 *  @param  [in]    h_palette           - palette handle
 *  @param  [in]    b_cmp_alpha         - take the alpha channel into consideration or not
 *  @param  [in]    pt_to_match         - the original color to match
 *  @param  [out]   pt_matched          - the matched color in the palette table
 *  @param  [out]   pui2_index          - index of pt_matched in the palette table
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_match_color(
    GL_HPALETTE_T       h_palette,
    BOOL                b_cmp_alpha,
    const GL_COLOR_T*   pt_to_match,
    GL_COLOR_T*         pt_matched,
    UINT16*             pui2_index);

/*------------------------------------------------------------------*/
/*! @brief  find the closest color in a range of a palette table for a specified color
 *  @param  [in]    h_palette           - palette handle
 *  @param  [in]    ui2_base            - the base palette entry to be queried
 *  @param  [in]    ui2_count           - number of palette entries from entry[ui2_base] to be queried
 *  @param  [in]    b_cmp_alpha         - take the alpha channel into consideration or not
 *  @param  [in]    pt_to_match         - the original color to match
 *  @param  [out]   pt_matched          - the matched color in the palette table
 *  @param  [out]   pui2_index          - index of pt_matched in the palette table
 *  @note
 *  @see
 *  @return return the error code
 *  @retval GLR_OK                      - ok
 *  @retval GLR_NOT_INIT                - GL is not initialized
 *  @retval GLR_INV_ARG                 - invalid argument is passed
 *  @retval GLR_INCOMPATIBLE_OBJ        - incompatible GL object(s) is/are passed
 *  @retval GLR_INV_HANDLE              - an invalid handle is passed
 */
/*------------------------------------------------------------------*/
extern INT32 c_gl_palette_match_color_ex(
    GL_HPALETTE_T       h_palette,
    UINT16              ui2_base,
    UINT16              ui2_count,
    BOOL                b_cmp_alpha,
    const GL_COLOR_T*   pt_to_match,
    GL_COLOR_T*         pt_matched,
    UINT16*             pui2_index);

extern INT32 c_gl_get_fbm_surface_info(
                    UINT8* pui1_cnt, 
                    UINT32* pui4_mem_size);


extern VOID c_gl_get_fbm_buffer(GL_DFB_FBM_BUFFER_T* pt_fbm_info);
extern INT32 c_gl_set_HID_language(UINT32 lang);
extern INT32 c_gl_set_HID_language_ex(UINT32 lang, UINT32 modechg);
extern INT32 c_gl_set_plane_scaler(UINT32 ui4_scaler_id, BOOL b_enable, UINT32 ui4_src_width, UINT32 ui4_src_height);


#ifdef __cplusplus
}
#endif

#endif

/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_GL*/
/*----------------------------------------------------------------------------*/
