/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_rm.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Resource Manager specific definitions,
 *         which are exported to applications.
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_RM_H_
#define _C_RM_H_

/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_RES_MNGR Resource Manager
 *  @ingroup groupMW
 *  @brief The resource manager module is the core module for driver registration and managerment
 *  %MW Application API would be used by Application, on D interface is export for AP
 *   
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_common.h"
#include "u_rm.h"


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
 /*------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 *! @brief API to sets information for a component without having to
 *              open the it first.
 *
 *
 * @param [in]      -e_type          Specifies the component type.
 * @param [in]      -ui2_id          Contains a specific component id or ANY_COMP_ID.
 * @param [in]      -b_sel_out_port  Specifies if argument "ui1_port" references an
 * @param [in]                       input or output port.
 * @param [in]      -ui1_port        Contains the input or output port number.
 * @param [in]      -e_set_type      Specifies the type of information which is
 *                                   referenced by argument pv_set_info.
 * @param [in]      -pv_set_info     Contains the set information.
 * @param [in]      -z_set_info_len  Indicates the length of the data in the buffer
 *                                   referenced by argument pv_set_info.
 *
 * @return  Return the error code.
 * @retval  RMR_OK                Routine successful.
 *
 *          RMR_INV_ARG           One or more of the arguments contain
 *
 *                                invalid data.
 *
 *          RMR_NOT_ENOUGH_SPACE  Argument pv_get_info is not big enough to
 *
 *                                hold the requested information.
 *
 *          RMR_DRV_ERROR         A driver error occured.
 *
 *          RMR_INV_PORT_NUMBER   Invalid port number specified.
 *
 *          RMR_INV_SET           Unknown get operation.
 ----------------------------------------------------------------------------*/
 /*------------------------------------------------------------------*/
extern INT32 c_rm_set_comp (DRV_TYPE_T      e_type,
                            UINT16          ui2_id,
                            BOOL            b_sel_out_port,
                            UINT8           ui1_port,
                            DRV_SET_TYPE_T  e_set_type,
                            const VOID*     pv_set_info,
                            SIZE_T          z_set_info_len);
/*------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 *! @brief API This API gets information about a component without having to
 *              open the it first.
 *
 * @param [in]      -e_type           Specifies the component type.
 * @param [in]      -ui2_id           Contains a specific component id or ANY_COMP_ID.
 * @param [in]      -b_sel_out_port   Specifies if argument "ui1_port" references an
 * @param [in]                          input or output port.
 * @param [in]      -ui1_port         Contains the input or output port number.
 * @param [in]      -e_get_type       Specifies the type of information which shall be
 *                                    returned in the buffer, referenced by argument
 * @param [in]      -pv_get_info.
 * @param [in]      -pz_get_info_len  Contains the length of the buffer which is
 *                                    referenced by argument pv_get_info.
 *
 * @param [out]     -pv_get_info      Contains the retrieved information.
 * @param [out]     -pz_get_info_len  Indicates the length of the valid data in the
 *                                    buffer referenced by argument pv_get_info.
 * @return  Return the error code.
 * @retval  RMR_OK                Routine successful.
 *
 *          RMR_INV_ARG           One or more of the arguments contain
 *
 *                                invalid data.
 *
 *          RMR_NOT_ENOUGH_SPACE  Argument pv_get_info is not big enough to
 *
 *                                hold the requested information.
 *
 *          RMR_DRV_ERROR         A driver error occured.
 *
 *          RMR_INV_PORT_NUMBER   Invalid port number specified.
 *
 *          RMR_INV_GET           Unknown get operation.
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
extern INT32 c_rm_get_comp (DRV_TYPE_T      e_type,
                            UINT16          ui2_id,
                            BOOL            b_sel_out_port,
                            UINT8           ui1_port,
                            DRV_GET_TYPE_T  e_get_type,
                            VOID*           pv_get_info,
                            SIZE_T*         pz_get_info_len);

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_RES_MNGR */
/*----------------------------------------------------------------------------*/

#endif /* _C_RM_H_ */
