/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_dlna.h 
*  $RCSfile: $
*  $Revision: #1 $
*  $Date: 2015/05/29 $
*  $Author: brianpc.huang $
*  
*  @par Description:
*         This header file contains DLNA exported constants, macros,
*         and types.
*/
/*----------------------------------------------------------------------------*/

/*!
* @defgroup    groupMW_INET_DLNA DLNA API
* @ingroup     groupMW_INET
* @brief       Provide an external API about DLNA.
* %INET_DLNA API would be used by user who wants to play media stream in local network,
*                The DLNA core stuffs like UPnP Architecture, UPnP AV, SSDP, SOAP, 
*                GENA and HTTP is hide for application engineer, those protocol 
*                is locate at middleware and application engineer should not modify it.
* 
* @see 
*
* @{
* */

#ifndef U_DLNA_H
#define U_DLNA_H

#include "u_common.h"
#include "u_dbg.h"


/* Error code */
/*----------------------------------------------------------------------------*/
/*! @enum   INET_DLNA_RET_T 
*  @brief  DLNA return type
*  @code
*  enum
*  {
*    DLNAR_SUCCESS                                       = 0,
*    DLNAR_FAIL                                          = -128,
*    DLNAR_RW_FAIL                                       = -129,               
*    DLNAR_BROWSE_FAILED                                 = -130,
*    DLNAR_DEVICE_FAILED                                 = -131,
*    DLNAR_CONNECT_SERVER_FAILED                         = -132,
*    DLNAR_SEARCH_FAILED                                 = -133,
*    DLNAR_NOT_ENOUGH_RESOURCE                           = -134,
*    DLNAR_OBJECT_NOT_EXIST                              = -135,
*    DLNAR_ALREADY_START                                 = -136,
*    DLNAR_NOT_START                                     = -137,
*    DLNAR_INVALID_ARGUMENT                              = -138,
*    DLNAR_NOT_INIT                                      = -139,  
*    DLNAR_DEVICE_SORT_CAPABILITY_FAILED                 = -140,
*    DLNAR_DEVICE_SEARCH_CAPABILITY_FAILED               = -141,
*    DLNAR_NOT_SUPPORT                                   = -142, 
*    DLNAR_CLOSED                                        = -143,
*    DLNAR_ERROR                                         = -144,
*    DLNAR_TIME_OUT                                      = -145,
*    DLNAR_OBJECT_EOF                                    = -146,
*    DLNAR_SERVER_ERROR                                  = -147,
*    DLNAR_CLIENT_ERROR                                  = -148,
*    DLNAR_CANCELLED                                     = -149
*  };
*  @endcode
*   
*  @li@c DLNAR_SUCCESS                                  - Return successful
*  @li@c DLNAR_FAIL                                     - Return fail
*  @li@c DLNAR_RW_FAIL                                  - Return R/W fail
*  @li@c DLNAR_BROWSE_FAILED                            - Return browse fail
*  @li@c DLNAR_DEVICE_FAILED                            - Return device error
*  @li@c DLNAR_CONNECT_SERVER_FAILED                    - Return connect to server fail
*  @li@c DLNAR_SEARCH_FAILED                            - Return search fail
*  @li@c DLNAR_NOT_ENOUGH_RESOURCE                      - Return memory not enough
*  @li@c DLNAR_OBJECT_NOT_EXIST                         - Return object not exist
*  @li@c DLNAR_ALREADY_START                            - Return the client already started
*  @li@c DLNAR_NOT_START                                - Return the client not started
*  @li@c DLNAR_INVALID_ARGUMENT                         - Return invalid argument
*  @li@c DLNAR_NOT_INIT                                 - Return the client not init
*  @li@c DLNAR_DEVICE_SORT_CAPABILITY_FAILED            - Return check the device sort capability fail 
*  @li@c DLNAR_DEVICE_SEARCH_CAPABILITY_FAILED          - Return check the device search capability fail 
*  @li@c DLNAR_NOT_SUPPORT                              - Return system not support
*  @li@c DLNAR_CLOSED                                   - Return the client or server closed 
*  @li@c DLNAR_ERROR                                    - Return the client or server error
*  @li@c DLNAR_TIME_OUT                                 - Return time out 
*  @li@c DLNAR_OBJECT_EOF                               - Return file EOF
*  @li@c DLNAR_SERVER_ERROR                             - Return server error
*  @li@c DLNAR_CLIENT_ERROR                             - Return client error
*  @li@c DLNAR_CANCELLED                                - Return the action cancelled
*              
*/
/*----------------------------------------------------------------------------*/
enum
{
    DLNAR_SUCCESS = 0,
    DLNAR_FAIL                                          = -128,
    DLNAR_RW_FAIL                                       = -129,               
    DLNAR_BROWSE_FAILED                                 = -130,
    DLNAR_DEVICE_FAILED                                 = -131,
    DLNAR_CONNECT_SERVER_FAILED                         = -132,
    DLNAR_SEARCH_FAILED                                 = -133,
    DLNAR_NOT_ENOUGH_RESOURCE                           = -134,
    DLNAR_OBJECT_NOT_EXIST                              = -135,
    DLNAR_ALREADY_START                                 = -136,
    DLNAR_NOT_START                                     = -137,
    DLNAR_INVALID_ARGUMENT                              = -138,
    DLNAR_NOT_INIT                                      = -139,  
    DLNAR_DEVICE_SORT_CAPABILITY_FAILED                 = -140,
    DLNAR_DEVICE_SEARCH_CAPABILITY_FAILED               = -141,
    DLNAR_NOT_SUPPORT                                   = -142, 
    DLNAR_CLOSED                                        = -143,
    DLNAR_ERROR                                         = -144,
    DLNAR_TIME_OUT                                      = -145,
    DLNAR_OBJECT_EOF                                    = -146,
    DLNAR_SERVER_ERROR                                  = -147,
    DLNAR_CLIENT_ERROR                                  = -148,
    DLNAR_CANCELLED                                     = -149
};



/* DLNA device type.  */
/*----------------------------------------------------------------------------*/
/*! @enum   DLNA_DEVICE_TYPE_T 
*  @brief  DLNA device type
*  @code
*  typedef enum {
*    DLNA_DEVICE_TYPE_START = 0, 
*    DLNA_DEVICE_TYPE_UNKNOWN,
*    DLNA_DEVICE_TYPE_DMS,
*    DLNA_DEVICE_TYPE_M_DMS,
*    DLNA_DEVICE_TYPE_DMR,
*    DLNA_DEVICE_TYPE_DMPr,
*    DLNA_DEVICE_TYPE_DMP,
*    DLNA_DEVICE_TYPE_M_DMP,
*    DLNA_DEVICE_TYPE_DMC,
*    DLNA_DEVICE_TYPE_M_DMC,
*    DLNA_DEVICE_TYPE_M_DMU,
*    DLNA_DEVICE_TYPE_M_DMD,
*    DLNA_DEVICE_TYPE_END 
*  } DLNA_DEVICE_TYPE_T;
*  @endcode
*   
*  @li@c DLNA_DEVICE_TYPE_START                 - DLNA device type start
*  @li@c DLNA_DEVICE_TYPE_UNKNOWN               - DLNA device type unknown
*  @li@c DLNA_DEVICE_TYPE_DMS                   - DLNA DMS
*  @li@c DLNA_DEVICE_TYPE_M_DMS                 - DLNA M-DMS
*  @li@c DLNA_DEVICE_TYPE_DMR                   - DLNA DMR
*  @li@c DLNA_DEVICE_TYPE_DMPr                  - DLNA DMPr
*  @li@c DLNA_DEVICE_TYPE_DMP                   - DLNA DMP
*  @li@c DLNA_DEVICE_TYPE_M_DMP                 - DLNA M-DMP
*  @li@c DLNA_DEVICE_TYPE_DMC                   - DLNA DMC
*  @li@c DLNA_DEVICE_TYPE_M_DMC                 - DLNA M-DMC
*  @li@c DLNA_DEVICE_TYPE_M_DMU                 - DLNA M-DMU
*  @li@c DLNA_DEVICE_TYPE_M_DMD                 - DLNA M-DMD
*  @li@c DLNA_DEVICE_TYPE_END                   - DLNA device type end
*              
*/
/*----------------------------------------------------------------------------*/
typedef enum {
    DLNA_DEVICE_TYPE_START = 0, /* Please keep this one the first one. */
    DLNA_DEVICE_TYPE_UNKNOWN,
    DLNA_DEVICE_TYPE_DMS,
    DLNA_DEVICE_TYPE_M_DMS,
    DLNA_DEVICE_TYPE_DMR,
    DLNA_DEVICE_TYPE_DMPr,
    DLNA_DEVICE_TYPE_DMP,
    DLNA_DEVICE_TYPE_M_DMP,
    DLNA_DEVICE_TYPE_DMC,
    DLNA_DEVICE_TYPE_M_DMC,
    DLNA_DEVICE_TYPE_M_DMU,
    DLNA_DEVICE_TYPE_M_DMD,
    DLNA_DEVICE_TYPE_END /* Please keep this one the last one. */
} DLNA_DEVICE_TYPE_T;



/* dlna device event */
/*----------------------------------------------------------------------------*/
/*! @enum   DLNA_DEVICE_EVENT_T 
*  @brief  DLNA device event
*  @code
*  typedef enum {  
*   DLNA_DEVICE_EVENT_START = 0,
*   DLNA_DEVICE_EVENT_FOUND_DEVS,
*   DLNA_DEVICE_EVENT_BYEBYE, 
*   DLNA_DEVICE_EVENT_UNAVAILABLE,
*   DLNA_DEVICE_EVENT_SUID_CHG,
*   DLNA_DEVICE_EVENT_CUIDS_CHG,
*   DLNA_DEVICE_EVENT_TIDS_CHG,
*   DLNA_DEVICE_EVENT_LAST_CHG,
*   DLNA_DEVICE_EVENT_DMR_START = 0x100,
*   DLNA_DEVICE_EVENT_DMR_STOP        
*   } DLNA_DEVICE_EVENT_T;
*  @endcode
*   
*  @li@c DLNA_DEVICE_EVENT_START                        - DLNA device event start
*  @li@c DLNA_DEVICE_EVENT_FOUND_DEVS                   - DLNA found device
*  @li@c DLNA_DEVICE_EVENT_BYEBYE                       - DLNA device bye-bye
*  @li@c DLNA_DEVICE_EVENT_UNAVAILABLE                  - DLNA device unavailable
*  @li@c DLNA_DEVICE_EVENT_SUID_CHG                     - DLNA SUID changed
*  @li@c DLNA_DEVICE_EVENT_CUIDS_CHG                    - DLNA CUIDS changed
*  @li@c DLNA_DEVICE_EVENT_TIDS_CHG                     - DLNA TIDS changed
*  @li@c DLNA_DEVICE_EVENT_LAST_CHG                     - DLNA last changed
*  @li@c DLNA_DEVICE_EVENT_DMR_START                    - DLNA DMR start
*  @li@c DLNA_DEVICE_EVENT_DMR_STOP                     - DLNA DMR stop
*/
/*----------------------------------------------------------------------------*/
typedef enum {
    DLNA_DEVICE_EVENT_START = 0,
    DLNA_DEVICE_EVENT_FOUND_DEVS,
    DLNA_DEVICE_EVENT_BYEBYE, 
    DLNA_DEVICE_EVENT_UNAVAILABLE,
    DLNA_DEVICE_EVENT_SUID_CHG,
    DLNA_DEVICE_EVENT_CUIDS_CHG,
    DLNA_DEVICE_EVENT_TIDS_CHG,
    DLNA_DEVICE_EVENT_LAST_CHG,
    DLNA_DEVICE_EVENT_DMR_START = 0x100,
    DLNA_DEVICE_EVENT_DMR_STOP        
} DLNA_DEVICE_EVENT_T;


/* seek type supported by dms */
/*----------------------------------------------------------------------------*/
/*! @enum   DLNA_SEEK_TYPE_T 
*  @brief  DLNA server seek type
*  @code
*  typedef enum {
*    DLNA_SEEK_NONSUPPORT = 0,               
*    DLNA_SEEK_FULL_BYTE = 1,                
*    DLNA_SEEK_FULL_TIME = 2,                
*    DLNA_SEEK_FULL_BYTE_AND_TIME = 3,       
*    DLNA_SEEK_LIMITED_BYTE = 4,             
*    DLNA_SEEK_LIMITED_TIME = 8,             
*    DLNA_SEEK_LIMITED_BYTE_AND_TIME = 12,   
*    DLNA_SEEK_UNKNOWN,
*  } DLNA_SEEK_TYPE_T;  
*  @endcode
*   
*  @li@c DLNA_SEEK_NONSUPPORT                          - DLNA not support seek
*  @li@c DLNA_SEEK_FULL_BYTE                           - DLNA full byte seek
*  @li@c DLNA_SEEK_FULL_TIME                           - DLNA full time seek
*  @li@c DLNA_SEEK_FULL_BYTE_AND_TIME                  - DLNA full byte and time seek
*  @li@c DLNA_SEEK_LIMITED_BYTE                        - DLNA limited byte seek
*  @li@c DLNA_SEEK_LIMITED_TIME                        - DLNA limited time seek
*  @li@c DLNA_SEEK_LIMITED_BYTE_AND_TIME               - DLNA limited byte and time seek
*  @li@c DLNA_SEEK_UNKNOWN                             - DLNA unknown
*/
/*----------------------------------------------------------------------------*/
typedef enum {
    DLNA_SEEK_NONSUPPORT = 0,            /* nonsupport seek */     
    DLNA_SEEK_FULL_BYTE = 1,                 /* full byte seek supported by dms */
    DLNA_SEEK_FULL_TIME = 2,                 /* full time seek supported by dms */
    DLNA_SEEK_FULL_BYTE_AND_TIME = 3,        /* full byte and time seek supported by dms */
    DLNA_SEEK_LIMITED_BYTE = 4,              /* full byte seek supported by dms */
    DLNA_SEEK_LIMITED_TIME = 8,              /* full time seek supported by dms */
    DLNA_SEEK_LIMITED_BYTE_AND_TIME = 12,      /* full byte and time seek supported by dms */
    DLNA_SEEK_UNKNOWN,
} DLNA_SEEK_TYPE_T;

/* protection type supported by dms */
/*----------------------------------------------------------------------------*/
/*! @enum   DLNA_DRM_TYPE_T 
*  @brief  DLNA drm type
*  @code
*  typedef enum {
*    DLNA_DRM_NONE = 0,          
*    DLNA_DRM_DTCP = 1,          
*    DLNA_DRM_WMDRM = 2,         
*    DLNA_DRM_UNKNOWN,
*  } DLNA_DRM_TYPE_T;
*  @endcode
*   
*  @li@c DLNA_DRM_NONE                - DLNA not DRM
*  @li@c DLNA_DRM_DTCP                - DLNA DRM DTCP
*  @li@c DLNA_DRM_WMDRM               - DLNA DRM WMDRM
*  @li@c DLNA_DRM_UNKNOWN             - DLNA DRM unknown
*/
/*----------------------------------------------------------------------------*/
typedef enum {
    DLNA_DRM_NONE = 0,            /* non content protection */     
    DLNA_DRM_DTCP = 1,            /* protected by dtcp */
    DLNA_DRM_WMDRM = 2,           /* protected by wmdrm  */
    DLNA_DRM_UNKNOWN,
} DLNA_DRM_TYPE_T;


/*General type */
typedef VOID * DLNA_DEVICE_T;



/* DLNA MIME type */
/*----------------------------------------------------------------------------*/
/*! @struct   DLNA_PROTOCOL_INFO_T 
*  @brief  DLNA protocol info
*  @code
*  typedef struct
*  {
*    CHAR *ps_mime;            
*    CHAR *ps_host;            
*    CHAR *ps_port;            
*    CHAR *ps_rate;            
*    CHAR *ps_channels;        
*    CHAR *ps_profile_id;      
*    CHAR *ps_content_format; 
*    CHAR *ps_operations_parameter; 
*    CHAR *ps_play_speeds;
*    CHAR *ps_conversion_indicator;
*    CHAR *ps_flags;
*  } DLNA_PROTOCOL_INFO_T;  
*  @endcode
*   
*  @li@c ps_mime                         - DLNA protocol MIME info 
*  @li@c ps_host                         - DLNA host URI
*  @li@c ps_port                         - DLNA server port
*  @li@c ps_rate                         - DLNA audio rate
*  @li@c ps_channels                     - DLNA audio channels
*  @li@c ps_profile_id                   - DLNA profile ID
*  @li@c ps_content_format               - DLNA content format 
*  @li@c ps_operations_parameter         - DLNA operation parameter
*  @li@c ps_play_speeds                  - DLNA play speeds
*  @li@c ps_conversion_indicator         - DLNA conversion indicator
*  @li@c ps_flags                        - DLNA flags
*/
/*----------------------------------------------------------------------------*/
typedef struct
{
    CHAR *ps_mime;             /* the object MIME type */
    CHAR *ps_host;             /* the ake host if media protected*/
    CHAR *ps_port;             /* the port of host */
    CHAR *ps_rate;             /* audio rate */
    CHAR *ps_channels;          /* audio channel */
    CHAR *ps_profile_id;       /* the object profile id, get value in DLNA.ORG_PN */
    CHAR *ps_content_format;   /* the object content format, if the object is CP */

    /* MM protocolInfo values: 4th Field */
    /* Operations Parameter 
    this value for HTTP, time-base seek and byte-base seek, such as '11' means supporting all, '01' means support byte-base see;  
    this value for RTP, byte-base seek and '0', such as '10' means supporting byte-seek*/
    CHAR *ps_operations_parameter; 
    /* Server-Side PlaySpeeds Parameter */
    CHAR *ps_play_speeds;
    /*Conversion Indicator Flag*/
    CHAR *ps_conversion_indicator;
    /* Flags */
    CHAR *ps_flags;
} DLNA_PROTOCOL_INFO_T;


/* Device list change notify prototype the pv_arg is DLNA_DEVICE_T */
/*------------------------------------------------------------------*/
/*! @brief  callback of device found or bye-bye
*  @param [in]t_event             - The device event
*  @param [in]pv_tag              - The callback tag
*  @param [in]pv_arg              - The callback argument, the type is DLNA_DEVICE_T
*  @note
*  @see
*  @return
*  @retval      -
*/
/*------------------------------------------------------------------*/
typedef VOID (*x_dlna_device_nfy) (DLNA_DEVICE_EVENT_T t_event, 
              VOID * pv_tag,
              VOID * pv_arg);

/* Sema macro */
#define X_DLNA_SEMA_LOCK(x)    { if (x_sema_lock(x, X_SEMA_OPTION_WAIT) != OSR_OK) DBG_ASSERT(0, DBG_MOD_ACCESS_DLNA); }
#define X_DLNA_SEMA_UNLOCK(x)       { if (x_sema_unlock(x) != OSR_OK) DBG_ASSERT(0, DBG_MOD_ACCESS_DLNA); }
#define X_DLNA_SEMA_DELETE(x)       { if (x_sema_delete(x) != OSR_OK) DBG_ASSERT(0, DBG_MOD_ACCESS_DLNA); }
#define X_DLNA_SEMA_CREATE(h, t, s) { if(x_sema_create(h, t, s) != OSR_OK) DBG_ASSERT(0, DBG_MOD_ACCESS_DLNA); }

/* Check flag */
#define X_DLNA_CHECK_FLAG(a, b)     ((a) & (b))
#define X_DLNA_SET_FLAG(a, b)       ((a) |= (b))
#define X_DLNA_UNSET_FLAG(a, b)     ((a) &= (~(b)))

#endif

/*! @} */

