/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_hbbtv.h
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/11/26 $
 *  $Author: dtvbm11 $
 *
 *  @par Description: This header file declares HBBTV exported APIs.
 */
/*----------------------------------------------------------------------------*/
#ifndef _U_OPERA_HBBTV_DVB_H_
#define _U_OPERA_HBBTV_DVB_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_handle.h"
#include "u_svl.h"
#include "u_brdcst.h"
#include "u_opera_hbbtv.h"
#include "u_opera_hbbtv_common.h"
#include "u_nwl.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/

#define OPHT_DVB_CONTEX            (HT_GROUP_OP_HBBTV + (HANDLE_TYPE_T) 1)

#define OPERA_HBBTV_MAX_ENCODING_LEN    64
#define OPERA_HBBTV_MAX_LANG_LEN        32

#define OPERA_HBBTV_NULL_SVC_CH_ID      0xFFFF
#define OPERA_HBBTV_STOP_SVC_CH_ID      0x0

#define OPERA_HBBTV_PROGRAM_MAX_NAME_LEN                     (64 )
#define OPERA_HBBTV_PROGRAM_MAX_SHORT_DESCRIPTION_LEN        (256)
#define OPERA_HBBTV_PROGRAM_MAX_EXTEND_DESCRIPTION_LEN       (1096)
#define MAX_DSD_STR_LEN                                      (32)

/*------------------------------------------------------------------*/
/*! @brief Callback AP of the tune method
 *  @param [in] e_tune_method   - 0: Subsequent tuning shall be "normal"
 *                                1: No banner, A/V stops and after is restarted, original channel info
 *                                2: No banner, A/V continues, new channel info
 *                                3: No banner, A/V continues, original channel info.
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef enum _OPERA_HBBTV_TUNE_METHOD_T
{
    OPERA_HBBTV_TUNE_ALL_NORMALLY        = 0x00,
    OPERA_HBBTV_TUNE_ALL_QUIETLY         = 0x01,
    OPERA_HBBTV_TUNE_QUIET_THEN_NORMAL   = 0x02,
    OPERA_HBBTV_TUNE_COMPLETELY_QUIET    = 0x03
} OPERA_HBBTV_TUNE_METHOD_T;

typedef struct _OPERA_HBBTV_TUNE_SVC_INFO_T
{
    UINT32                          ui4_channel_id;
    BOOL                            b_direct_channel;
    OPERA_HBBTV_TUNE_METHOD_T       e_tune_mode;
    HANDLE_T                        h_dvb_ctx;
    BOOL                            b_force_tune;
}OPERA_HBBTV_TUNE_SVC_INFO_T;

typedef enum _OPERA_HBBTV_COMP_TYPE_T
{
    OPERA_HBBTV_COMP_TYPE_UNKNOWN = 0,
    OPERA_HBBTV_COMP_TYPE_VIDEO,
    OPERA_HBBTV_COMP_TYPE_AUDIO,
    OPERA_HBBTV_COMP_TYPE_SUBTITLE
} OPERA_HBBTV_COMP_TYPE_T;

typedef struct _OPERA_HBBTV_COMP_INTO_T
{
    UINT32                    ui4_index;               /* index in array from 0 */
    STREAM_COMP_ID_T          t_comp_id;               /* stream component id */
    OPERA_HBBTV_COMP_TYPE_T   t_type;                  /* type of component */
    UINT8                     b_is_active;             /* is this component being rendered */
    UINT8                     b_is_encrypted;          /* is this component encrypted */
    UINT32                    ui4_aspect_ratio;        /* aspect ratio - fixed point with 2 decimal places */
    UINT8                     b_is_audio_description;  /* is this an audio description component */
    UINT32                    ui4_audio_channels;      /* number of channels */
    UINT8                     b_is_hearing_impaired;   /* is this a video component for the hearing impaired */
    UINT32                    ui4_pid;                 /* PID */
    UINT32                    ui4_sample_rate;         /* Sample Rate*/
    UINT16                    ui2_comp_pg_id;          /*subtitle comp page id*/
    UINT16                    ui2_anci_pg_id;          /*subtitle anci page id*/
    CHAR                      s_encoding[OPERA_HBBTV_MAX_ENCODING_LEN];     /* encoding description string */
    CHAR                      s_language[OPERA_HBBTV_MAX_LANG_LEN];/* ISO 639 language string */
    INT32                     ui4_comp_tag;
}OPERA_HBBTV_COMP_INTO_T;

typedef struct _OPERA_HBBTV_DVB_CHANGE_SVC_INFO_T
{
    UINT8                       ui1_crnt_tv_inp_src_id;
    UINT16                      ui2_crnt_svl_id;
    UINT16                      ui2_crnt_svl_rec_id;
    UINT32                      ui4_crnt_channel_id;
    UINT8                       ui1_trgt_tv_inp_src_id;
    UINT16                      ui2_trgt_svl_id;
    UINT16                      ui2_trgt_svl_rec_id;
    UINT32                      ui4_trgt_channel_id;

    OPERA_HBBTV_VIDEO_BRDCST_EVENT_T e_status;

}OPERA_HBBTV_DVB_CHANGE_SVC_INFO_T;

typedef enum _OPERA_HBBTV_DVB_SVC_TYPE
{
    OPERA_HBBTV_SVC_TYPE_TV = 0,
    OPERA_HBBTV_SVC_TYPE_RADIO = 1,
    OPERA_HBBTV_SVC_TYPE_OTHER = 2,
    OPERA_HBBTV_SVC_TYPE_ALL = 128,
    OPERA_HBBTV_SVC_TYPE_HBBTV_DATA = 256,

}OPERA_HBBTV_DVB_SVC_TYPE;
typedef enum _OPERA_HBBTV_DVB_ID_TYPE
{
    OPERA_HBBTV_ID_ANALOG = 0,
    OPERA_HBBTV_ID_DVB_C = 10,
    OPERA_HBBTV_ID_DVB_S = 11,
    OPERA_HBBTV_ID_DVB_T = 12,
    OPERA_HBBTV_ID_DVB_SI_DIRECT = 13,
    OPERA_HBBTV_ID_DVB_C2 = 14,
    OPERA_HBBTV_ID_DVB_S2 = 15,
    OPERA_HBBTV_ID_DVB_T2 = 16,
    OPERA_HBBTV_ID_ISDB_C = 20,
    OPERA_HBBTV_ID_ISDB_S = 21,
    OPERA_HBBTV_ID_ISDB_T = 22,

    OPERA_HBBTV_ID_ATSC_T   = 30,
    OPERA_HBBTV_ID_IPTV_SDS = 40,
    OPERA_HBBTV_ID_IPTV_URI = 41,
    OPERA_HBBTV_ID_UNKNOWN = 128,

}OPERA_HBBTV_DVB_ID_TYPE;

typedef struct{
    UINT16                      ui2_svl_id;
    UINT16                      ui2_on_id;
    UINT16                      ui2_nw_id;
    UINT16                      ui2_ts_id;
    UINT16                      ui2_svc_id;
    UINT32                      ui4_channel_id;
    BRDCST_MEDIUM_T             e_bcst_medium;
    UINT8                       ui1_svc_type;
    UINT8                       ui1_id_type;
    UINT16                      ui2_svl_rec_id;
    UINT16                      ui2_major_channel;
    UINT16                      ui2_terminal_channel;
    UINT8                       ui1_index;
    CHAR                        s_svc_name[MAX_DVB_SERVICE_NAME_LEN+1];
    CHAR                        ac_name[MAX_PROG_NAME_LEN+1];
    UINT8                       s_dsd[MAX_DSD_STR_LEN];
}OPERA_HBBTV_DVB_SVC_INFO_T;

typedef struct{
    UINT16   ui2_nw_id;
    CHAR     pch_url[MAX_NETWORK_AUTH_URL_LEN];  // assuming maxmum lenght of  uri is 10*1024
}OPERA_HBBTV_DVB_AUTH_URL_T;
/*------------------------------------------------------------------*/
/*! @enum OPERA_HBBTV_PROGRAM_LIST_T
 *  @brief HBBTV program list data
 *  @code
 *  typedef enum _OPERA_HBBTV_PROGRAM_LIST_T
 *  {
 *    TIME_T      t_start;
 *    UINT32      ui4_duration;
 *    CHAR        s_name[HBBTV_PROGRAM_MAX_NAME_LEN];
 *    CHAR        s_description[HBBTV_PROGRAM_MAX_DESCRIPTION_LEN];
 *  } OPERA_HBBTV_PROGRAM_LIST_T;
 *
 *  @endcode
 *  @li@c  t_start              - start time of the selected program
 *  @li@c  ui4_duration     - duration of the selected program
 *  @li@c  s_name            - name of the selected program
 *  @li@c  s_description    - description of the selected program
 */
/*------------------------------------------------------------------*/
typedef struct _OPERA_HBBTV_PROGRAM_LIST_T
{
    UINT32      ui4_channel_id;
    UINT16      ui2_event_id;
    TIME_T      t_start;
    UINT32      ui4_duration;
    CHAR        s_name[OPERA_HBBTV_PROGRAM_MAX_NAME_LEN];
    CHAR        s_short_description[OPERA_HBBTV_PROGRAM_MAX_SHORT_DESCRIPTION_LEN];
    CHAR        s_extended_description[OPERA_HBBTV_PROGRAM_MAX_EXTEND_DESCRIPTION_LEN];
}OPERA_HBBTV_PROGRAM_LIST_T;

#define OPERA_HBBTV_DVB_MSG_NONE                   ((UINT32)  0)
#define OPERA_HBBTV_DVB_MSG_OPEN                   ((UINT32)  1)
#define OPERA_HBBTV_DVB_MSG_CLOSE                  ((UINT32)  2)
#define OPERA_HBBTV_DVB_MSG_UPDATE_VID_REGION      ((UINT32)  3)
#define OPERA_HBBTV_DVB_MSG_SET_DFT_VID_REGION     ((UINT32)  4)
#define OPERA_HBBTV_DVB_MSG_SVC_CHANGING           ((UINT32)  5)
#define OPERA_HBBTV_DVB_MSG_SVC_CHANGED            ((UINT32)  6)
#define OPERA_HBBTV_DVB_MSG_SVL_UPDATE             ((UINT32)  7)
#define OPERA_HBBTV_DVB_MSG_SVL_CHANGED            ((UINT32)  8)


#endif



