/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*--------------------------------------------------------------------------*/
/*! @file u_ring_buff.h 
 * $RCSfile: u_ring_buff.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 *  @par Description:
 *         This header file contains SECT engine specific definitions, which
 *         are exported.
 *------------------------------------------------------------------------*/

#ifndef _U_RING_BUFF_H_
#define _U_RING_BUFF_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_TBL_MNGR
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
                    include files
--------------------------------------------------------------------------*/

#include "u_common.h"
#include "u_handle.h"


/*--------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 -------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @struct    RING_BUFF_T
 *  @brief        Ring Buffer structure
 
 *  @code
 * typedef struct _RING_BUFF_T
 * {
 *     SIZE_T z_slot_len;
 *     UINT16 ui2_num_slots;
 * 
 *     UINT16 ui2_head;
 *     UINT16 ui2_tail;
 * 
 *     HANDLE_T h_semaphore;
 * 
 *     UINT8    aui1_data[1];
 * }   RING_BUFF_T; 
 *  @endcode
 
 *  @li@c    z_slot_len     - slot length
 *  @li@c    ui2_num_slots  - number of slots
 *  @li@c    ui2_head       - head index
 *  @li@c    ui2_tail       - tail index
 *  @li@c    h_semaphore    - semaphore to lock the ring buffer
 *  @li@c    aui1_data      - the buffer pointer to store data
 */
/*------------------------------------------------------------------*/
typedef struct _RING_BUFF_T
{
    SIZE_T z_slot_len;
    UINT16 ui2_num_slots;

    UINT16 ui2_head;   /* 0xffff means empty */
    UINT16 ui2_tail;   /* 0xffff means full */

    HANDLE_T h_semaphore;

    UINT8    aui1_data[1];  /* this MUST be the last field!! */
}   RING_BUFF_T;

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_TBL_MNGR */
/*----------------------------------------------------------------------------*/

#endif /* _U_RING_BUFF_H_ */
