/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_wgl_group.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains public interfaces, constant and type 
 *         definitions of the Widget Library.
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_WGL_GROUP  Group Widget
 *
 *  @ingroup groupMW_WGL
 *  @brief Provides definitions,commands of the group widget.
 *
 *  %MW Application API would be used by TV Application, lot of these APIs would
 *   be wrapped from Exported API
 *
 *  @see groupMW_WGL
 *  @{ 
 */
/*----------------------------------------------------------------------------*/ 
#ifndef _U_WGL_GROUP_H_
#define _U_WGL_GROUP_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_wgl.h"
#include "u_wgl_tools.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#define GROUP_CNT_PRIVATE_START                 0    /**< group private command start id       */
/*Specific Widget Style*/
#define WGL_STL_GRP_VIRTICAL           (((UINT32)(WGL_STL_WGT_SPF_START))<<0)    /**< Indicate widgets in group is virtical layout        */
#define WGL_STL_GRP_HORIZONTAL         (((UINT32)(WGL_STL_WGT_SPF_START))<<1)    /**<  Indicate widgets in group is horicontal layout          */
#define WGL_STL_GRP_HIDE_NO_SPACE      (((UINT32)(WGL_STL_WGT_SPF_START))<<2)    /**< Indicate widget who is invisible in group will occupy no space, this style is Default style  */
#define WGL_STL_GRP_HIDE_WITH_SPACE    (((UINT32)(WGL_STL_WGT_SPF_START))<<3)    /**<  Indicate widget who is invisible in group will occupy space indicated by its rectangle      */
/*Specific Command*/
#define WGL_CMD_GRP_SET_ALIGN          WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 0)    /**< set all widgets in group's alignment ,

                                                                                                                                                              related parameters in c_wgl_do_cmd(),

                                                                                                                                                              pv_param1: UINT8* alignment value;  

                                                                                                                                                              pv_param2:NULL*/
#define WGL_CMD_GRP_GET_ALIGN          WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 1)    /**< get all widgets in group's alignment ,

                                                                                                                                                              related parameters in c_wgl_do_cmd(),

                                                                                                                                                              pv_param1: UINT8* alignment value;  

                                                                                                                                                              pv_param2:NULL*/

#define WGL_CMD_GRP_SET_CHILD_INFO     WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 2)    /**< Set child infomation to group,just gap infomation is set at moment.

                                                                                                                                                               related parameters in c_wgl_do_cmd(),

                                                                                                                                                               pv_param1: HANDLE_T h_child_widget;

                                                                                                                                                               pv_param2:UINT32 ui4_gap;gap between h_child_widget and its previous brother widget */

#define WGL_CMD_GRP_GET_CHILD_INFO     WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 3)    /**< Get child infomation from group,just gap infomation can be get at moment.

                                                                                                                                                               related parameters in c_wgl_do_cmd(),

                                                                                                                                                               pv_param1: HANDLE_T h_child_widget;

                                                                                                                                                               pv_param2:UINT32 *pui4_gap;gap between h_child_widget and its previous brother widget */
#define WGL_CMD_GRP_UPDATE_ALL         WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 4)    /**<relayout all children,

                                                                                                                                                              related parameters in c_wgl_do_cmd(),

                                                                                                                                                              pv_param1: NULL;  

                                                                                                                                                              pv_param2:NULL*/
#define WGL_CMD_GRP_UPDATE             WGL_MAKE_WIDGET_CMD(HT_WGL_WIDGET_GROUP, 5)    /**< relayout specific child widget,

                                                                                                                                                              related parameters in c_wgl_do_cmd(),

                                                                                                                                                              pv_param1: HANDLE_T h_child_widget;  

                                                                                                                                                              pv_param2:NULL*/


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/



#ifdef __cplusplus
}
#endif

#endif

/*----------------------------------------------------------------------------*/
/*! @} */   /* end of groupMW_WGL_GROUP*/
/*----------------------------------------------------------------------------*/
