/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef LOADER_IMGHDR_H
#define LOADER_IMGHDR_H

/*--------------------------------------------------------------------------*/
/* Common part of loader/packing */

/* loader version 0.2v */
#define IMAGE_SIGNATURE                 0x35333958  /* "539X" */
#define IMAGE_SIGNATURE_UPGRADE         0x35333950  /* "539P" */

#ifndef CC_LOADER_MEM_OFFSET
    #define CC_LOADER_MEM_OFFSET            (DRAM_START + 0xd00000)
#endif /* CC_LOADER_MEM_OFFSET */


// LOADER_MEM_OFFSET - PRELOADER2_MEM_OFFSET = 3M
#ifndef CC_PRELOADER2_MEM_OFFSET
#define CC_PRELOADER2_MEM_OFFSET        (DRAM_START + 0xa00000)
#endif /* CC_PRELOADER2_MEM_OFFSET */

#if defined(SUPPORT_PRELOADER_80K_SIZE) || defined(CC_MT5895) 
#define PRELOADER_MAX_SIZE              0x00014000
#elif defined(CC_MT5893)
#define PRELOADER_MAX_SIZE              0x00010000
#else
#define PRELOADER_MAX_SIZE              0x0000A000
#endif

#ifdef LOADER_USB_UPGRADE
    #define USB_STACK_SIZE              0x10000 /*64 KB*/
#else
    #define USB_STACK_SIZE              0
#endif

#ifdef CC_MTK_LOADER_MAX_SIZE
#define LOADER_MAX_SIZE                 (CC_MTK_LOADER_MAX_SIZE)
#else
#ifdef CC_LOAD_UBOOT
#ifndef CC_BOOT_NO_LZHS
#define LOADER_MAX_SIZE                 (0x00080000) /* 512 KB */
#else /* CC_BOOT_NO_LZHS */
#define LOADER_MAX_SIZE                 (0x00100000) /* 1 MB */
#endif /* CC_BOOT_NO_LZHS */
#else
    #ifndef LOADER_MAX_SIZE
        #ifndef CC_UBOOT
            #define LOADER_MAX_SIZE                 (0x00040000 + USB_STACK_SIZE) /* 256 KB */
        #else
            #define LOADER_MAX_SIZE                 (0x00030000 + USB_STACK_SIZE) /* 192 KB */
        #endif
    #endif /* LOADER_MAX_SIZE */
#endif
#endif

#define METHOD_PLAIN_TEXT               0
#define METHOD_DEXOR_TEXT               1
#define METHOD_DECRYPT_TEXT             2

#define COMPRESS_TYPE_NO_COMPRESS       0
#define COMPRESS_TYPE_GZIP              1
#define COMPRESS_TYPE_LZHS              2

#define LZHS_HEADER                     (0x10)
#define IMAGE_HEADER                    (sizeof(IMAGE_HEADER_T))

// major.minor.patch
#define VERSION_MEMBER_CHAR_LEN         10
#define VERSION_MEMBER_NUM              3
#define VERSION_CHAR_LEN                (VERSION_MEMBER_CHAR_LEN * VERSION_MEMBER_NUM)

typedef struct
{
     UINT32 u4Signature;
     UINT32 u4Size;
     UINT32 fgCompress;
     UINT32 u4FirmwareExecAddr;
     UINT32 u4Method;
     UINT32 u4Key;
     UINT32 u4FirmwareChksum;
     UINT32 u4PQDataAddr;
     UINT32 u4Reserved4;
     UINT32 u4Reserved5;
     UINT32 u4Reserved6;
     UINT32 u4Reserved7;
     UINT32 u4Reserved8;
     UINT32 u4Reserved9;
     UINT32 u4ReservedA;
     UINT32 u4ReservedB;
} IMAGE_HEADER_T;

typedef struct
{
     UINT32 u4Signature;
     UINT32 u4Size;
     UINT32 fgCompress;
     UINT32 u4FirmwareExecAddr;
     UINT32 u4Method;
     UINT32 u4Key;
     UINT32 u4FirmwareChksum;
     UINT32 u4PQDataAddr;
     UINT8 au1CustomerId[16];
     UINT8 au1ModelName[32];
     UINT8 au1Version[64];
} IMAGE_UPGRADE_HEADER_T;

/*---------------------------------------------------------------------
 * Type definitions
 *---------------------------------------------------------------------*/

typedef void (*PFN_IMAGE)(void);

#endif /* LOADER_IMGHDR_H */
