/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: nptv_vdec_if.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file nptv_vdec_if.h
 *  NPTV Video interface for decoder, including VGA, YPbpr, and DVI
 */

#ifndef _NPTV_VDEC_IF_H_
#define _NPTV_VDEC_IF_H_

#include "x_typedef.h"
#include "sv_const.h"
#include "drv_vbi_if.h"

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------



///-----------------------------------------------------------------------------
/// Public functions
///-----------------------------------------------------------------------------
/**
 * @brief vApiVideoVgaSetPhase(UINT8 bPath, UINT8 bValue) 
 * Set Vga phase
 * @param  bPath :main/sub path (SV_VP_MAIN / SV_VP_PIP)
 * @param bValue  : Phase value (0~31)
 * @retval void
 * @example vApiVideoVgaSetPhase(SV_VP_MAIN, 5);
 */
EXTERN UINT8 fgApiVideoVgaSetPhase(UINT8 bValue);

/**
 * @brief vApiVideoVgaSetClock(UINT8 bPath, UINT16 wValue) 
 * Set VGA Clock value
 * @param  bPath :main/sub path (SV_VP_MAIN / SV_VP_PIP)
 * @param wValue  : clock  value (not UI value)
 * @retval void
 * @example vApiVideoVgaSetClock(SV_VP_MAIN, 1024);
 */
EXTERN UINT8 fgApiVideoVgaSetClock(UINT16 wValue); 

/**
 * @brief fgApiVideoGeoHPosition(UINT8 bPath, UINT16 wValue) 
 * Set VGA H Position
 * @param  bPath :main/sub path (SV_VP_MAIN / SV_VP_PIP)
 * @param wValue  : H Position (not UI value) (H porch)
 * @retval void
 * @example fgApiVideoGeoHPosition(SV_VP_MAIN, 100);
 */
EXTERN UINT8 fgApiVideoGeoHPosition(UINT8 bPath, UINT16 wValue); 

/**
 * @brief fgApiVideoGeoVPosition(UINT8 bPath, UINT16 wValue) 
 * Set VGA V Position
 * @param  bPath :main/sub path (SV_VP_MAIN / SV_VP_PIP)
 * @param wValue  : V Position (not UI value) 
 * @retval void
 * @example fgApiVideoGeoVPosition(SV_VP_MAIN, 30);
 */
EXTERN UINT8 fgApiVideoGeoVPosition(UINT8 bPath, UINT16 wValue);

/**
 * @brief fgApiVideoVgaAuto(void) 
 * Trigger Auto state machine entry,Do it when path is VGA or DVI+ext AD
 * @param void
 * @retval void
 * @example vApiVideoVgaAuto(void);
 */
EXTERN UINT8 fgApiVideoVgaAuto(void);

/**
 * 	StopVGA AutoClock, AutoPhase, and AutoPosition
 *
 * @return NONE
 * @retval void
 */
EXTERN void vApiVgaAutoStop(void);

/**
 * 	VGA call-back function for vVdoMainState, vVdoPipState polling update video status to UI
 *
 *  @retval SV_VDO_UNKNOWN / SV_VDO_NOSIGNAL / SV_VDO_NOSUPPORT / SV_VDO_STABLE
 */
EXTERN UINT8 bApiVgaSigStatus(void);

/**
 * @brief fgApiVideoVgaAuto(void) 
 * Trigger Auto state machine entry,Do it when path is VGA or DVI+ext AD
 * @param void
 * @retval void
 * @example vApiVideoVgaAuto(void);
 */
EXTERN UINT8 fgApiVideoYPbPrAuto(void);

/**
 * @brief fgApiVideoAutoColor(void) 
 * Trigger Auto Color state machine (VGA or YPbPr)
 * @param void
 * @retval UINT8
 * @example fgApiVideoAutoColor(void);
 */
EXTERN UINT8 fgApiVideoAutoColor(void);

/**
 * @brief fgApiVideoCVBSAutoColor(void) 
 * Trigger CVBS Auto Color state machine 
 * @param void
 * @retval UINT8
 * @example fgApiVideoCVBSAutoColor(void);
 */
EXTERN UINT8 fgApiVideoCVBSAutoColor(void); 

/**
 * @brief wApiVideoGetVgaStdHtotal(void)
 * Get  Standard Htotal value of current VGA timing
 * @param void
 * @retval :Htotal value
 */
EXTERN UINT16 wApiVideoGetVgaStdHtotal(void);

/**
 * @brief wApiVideoGetVgaStdHPos(void)
 * Get  Standard H position value of current VGA timing
 * @param void
 * @retval :H position value
 */
EXTERN UINT16 wApiVideoGetVgaStdHPos(void);

/**
 * @brief wApiVideoGetVgaStdVPos(void)
 * Get  Standard V position value of current VGA timing
 * @param void
 * @retval :V position value
 */
EXTERN UINT16 wApiVideoGetVgaStdVPos(void);


/**
* @brief vInternalEDIDInit(void)
* Initial internal Edid 
* @param void
* @retval : void
*/
EXTERN void vInternalEDIDInit(void);



#endif // _NPTV_VDEC_IF_H_
 
