/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_df_mngr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains Download function manager related API definitions,
 *         which are exported to application layer.
 *
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_DFM_H_
#define _C_DFM_H_

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/

#include "u_df_mngr.h"
#include "u_wgl_common.h"

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_DF_MNGR
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*****************************************************************
    functions declarations
****************************************************************/
/*------------------------------------------------------------------*/
/*! @brief The function is used to open Download Function Manager instance.
 *  @param [in] ps_dfm_name  The name is used to open a DF Manager instance.
 *  @param [out] ph_dfm    If the operartion is successful, it will handle this 
 *                         DF Manager instance.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  The invalid name in argument.
 *  @retval      -DFMR_OUT_OF_RESOURCES No handle resource is available.
 *  @retval      -DFMR_FAIL It fail in current operation.
 *  @retval      -DFMR_AEE_OUT_OF_RESOURCES  No resource available in AEE.
 *  @retval      -DFMR_AEE_NO_RIGHTS  No right do this operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_open(
                    const CHAR*         ps_dfm_name,
                    HANDLE_T*           ph_dfm
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to close a Download Function Manager instance.
 *  @param [in] ph_dfm The handle for current DF Manager instance.
 *  @param [out] None.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_close(
                    HANDLE_T            ph_dfm
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to load PNG or simgle logo data by
 *         ONID, and Service ID.
 *  @param [in] h_dfm The handle for current DF Manager instance.
 *  @param [in] ui2_on_id The original network identify.
 *  @param [in] ui2_svc_id The channel will be load its logo.
 *  @param [in] ui2_scan_idx The scan index for current logo data.
 *  @param [in] pv_tag The private tag.
 *  @param [in] pt_nfy The notify function for current load process.
 *  @param [out] ph_logo If this operation is sucessful, it will be handle a logo data.
 *  @param [out] pe_cond The condition for this operation of loading data.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_OUT_OF_RESOURCES No handle resource is available.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_load_logo(
                    HANDLE_T            h_dfm,
                    UINT16              ui2_on_id,
                    UINT16              ui2_svc_id,
                    UINT16              ui2_scan_idx,
                    VOID*               pv_tag,
                    x_dfm_nfy_fct       pt_nfy,
                    HANDLE_T*           ph_logo,
                    DFM_COND_T*         pe_cond
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to get the condition by the logo handle.
 *  @param [in]  h_dfm The handle for current DF Manager instance.
 *  @param [in]  h_logo The handle to logo data.
 *  @param [out] pe_cond The condition for the logo data handle.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_get_cond(
                    HANDLE_T            h_dfm,
                    HANDLE_T            h_logo,
                    DFM_COND_T*         pe_cond
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to get all logo data type by the logo handle.
 *  @param [in]  h_dfm The handle for current DF Manager instance.
 *  @param [in]  h_logo The handle to logo data.
 *  @param [out] pui1_logo_types The all available logo data type.
 *  @param [out] pe_cond The condition for the logo data handle.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_get_logo_type(
                    HANDLE_T            h_dfm,
                    HANDLE_T            h_logo,
                    UINT8*              pui1_logo_types,/* |= */
                    DFM_COND_T*         pe_cond
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to get logo data with current logo type 
 *         by logo handle, the logo data has converted a WGL_HIMG_TPL_T.
 *  @param [in]  h_dfm The handle for current DF Manager instance.
 *  @param [in]  h_logo The handle to logo data.
 *  @param [in]  t_logo_type The logo type is used to get the relative logo data.
 *  @param [out] pt_img This is the logo type, and it is a WGL_HIMG_TPL_T instance.
 *  @param [out] pe_cond The condition for the logo data handle.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_FAIL It fail in current operation.
 *  @retval      -DFMR_OUT_OF_RESOURCES No handle resource is available.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_get_logo_data(
                    HANDLE_T            h_dfm,
                    HANDLE_T            h_logo,
                    DFM_LOGO_TYPE_T     t_logo_type,
                    WGL_HIMG_TPL_T*     pt_img,/*this is a WGL_HIMG_TPL_T, Don't free it*/
                    DFM_COND_T*         pe_cond
                    );


/*------------------------------------------------------------------*/
/*! @brief This function is used to get logo file path with current logo type 
 *         by logo handle, the logo data has converted a WGL_HIMG_TPL_T.
 *  @param [in]  h_dfm The handle for current DF Manager instance.
 *  @param [in]  h_logo The handle to logo data.
 *  @param [in]  t_logo_type The logo type is used to get the relative logo data.
 *  @param [out] pt_img_path This is the logo type, and it is a logo file path.
 *  @param [out] pe_cond The condition for the logo data handle.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_FAIL It fail in current operation.
 *  @retval      -DFMR_OUT_OF_RESOURCES No handle resource is available.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_get_logo_file_path(
                    HANDLE_T            h_dfm,
                    HANDLE_T            h_logo,
                    DFM_LOGO_TYPE_T     t_logo_type,
                    CHAR*               pt_img_path,
                    DFM_COND_T*         pe_cond
                    );

/*------------------------------------------------------------------*/
/*! @brief The function is used to free a logo data(all type) by logo handle.
 *  @param [in]  h_dfm The handle for current DF Manager instance.
 *  @param [in]  h_logo The handle to logo data.
 *  @param [out] None.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_INV_ARG  One or more of the arguments contain invalid data.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_free_logo(
                    HANDLE_T            h_dfm,
                    HANDLE_T            h_logo
                    );
/*------------------------------------------------------------------*/
/*! @brief This function is used to notify Download Function Manager with 
 *         service change message.
 *  @param [in] h_dfm The handle for current DF Manager instance.
 *  @param [in] ui2_on_id The original network identify.
 *  @param [in] ui2_svc_id The service identify of current changed.
 *  @param [in] ui2_scan_idx The scan index for current service change.
 *  @param [out] None.
 *  @return
 *  @retval      -DFMR_OK Routine successful.
 *  @retval      -DFMR_NOT_INIT It is not initialized in system start process.
 *  @retval      -DFMR_FAIL It fail in current operation.
 */
/*------------------------------------------------------------------*/
extern INT32    c_dfm_change_svc(
                    HANDLE_T            h_dfm, 
                    UINT16           ui2_onid,
                    UINT16           ui2_service_ID,
                    UINT16           ui2_scan_idx
                    );


#endif /* _C_DFM_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                           /* end of groupMW_DF_MNGR */
/*----------------------------------------------------------------------------*/
