/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_fe.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains font library specific definitions,
 *         which are exported for application.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_FONT   Font Engine
 *
 *  @ingroup groupMW
 *  @brief Provides constant,type definitions and function of the Font Engine.
 *
 *  %MW Application API would be used by TV Application, lot of these APIs would
 *   be wrapped from Exported API
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _C_FE_H_
#define _C_FE_H_


/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_fe.h"


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/
/**
    system functions (for querying ...)
 **/

/*------------------------------------------------------------------*/
/*! @brief  c_fe_add_file_font
 *
 *              Applications could call this function to add a font into system.
 *              When this function is called, font engine will read and validate
 *              the specified font file. Clients should also specify three font 
 *              sizes (Small / Medium / Large) for this font. For vector font,
 *              the font size is assumed to be 'Point Size', and for bitmap
 *              font, 'Pixel Size' instead.  If the specified font size inside
 *              a vector font is an embedded bitmap, the size is turned to be
 *              'Pixel Size' also.
 *
 *  @param [in] ps_font          Specify the Font name. User can define the font
 *                                       name. To use the real font family name is not
 *                                       necessary.
 *  @param [in]  ps_file            Specify the real font file name.
 *  @param [in]  ui1_small         Font size for 'Small Font'.
 *  @param [in]  ui1_medium     Font size for 'Medium Font'.
 *  @param [in]  ui1_large         Font size for 'Large Font'.
 *  @return Return the error code.
 *  @retval FER_OK              The font is added successfully.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 *  @retval FER_INV_FONT        The ps_file file is not a valid font file.
 *  @retval FER_NO_FONT_SIZE    Some specified font sizes are not available.
 *  @retval FER_NO_MEMORY       Out of system memory
 */
/*------------------------------------------------------------------*/
INT32 c_fe_add_file_font(
    const CHAR*             ps_font,        /* font name                     */
    const CHAR*             ps_file,        /* font file name                */
    UINT8                   ui1_small,      /* size of FE_FNT_SIZE_SMALL     */
    UINT8                   ui1_medium,     /* size of FE_FNT_SIZE_MEDIUM    */
    UINT8                   ui1_large);     /* size of FE_FNT_SIZE_LARGE     */

/*------------------------------------------------------------------*/
/*! @brief c_fe_add_mem_font
 *
 *              Applications could call this function to add a font into system.
 *              When this function is called, font engine will read and validate
 *              the specified font file. Clients should also specify three font 
 *              sizes (Small / Medium / Large) for this font. For vector font,
 *              the font size is assumed to be 'Point Size', and for bitmap
 *              font, 'Pixel Size' instead.  If the specified font size inside
 *              a vector font is an embedded bitmap, the size is turned to be
 *              'Pixel Size' also.
 *  @param [in] ps_font             Specify the Font name. User can define the font
 *                                          name. To use the real font family name is not
 *                                          necessary.
 *  @param [in] pui1_fontdata       Specify the font data.
 *  @param [in] ui4_size            Size of font data.
 *  @param [in] ui1_small           Font size for 'Small Font'.
 *  @param [in] ui1_medium          Font size for 'Medium Font'.
 *  @param [in] ui1_large           Font size for 'Large Font'.
 *  @return Return the error code.
 *  @retval FER_OK              The font is added successfully.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 *  @retval FER_INV_FONT        The ps_file file is not a valid font file.
 *  @retval FER_NO_FONT_SIZE    Some specified font sizes are not available.
 *  @retval FER_NO_MEMORY       Out of system memory
 *  @retval FER_FONT_EXIST      The same font existed.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_add_mem_font(
    const CHAR*             ps_font,        /* font name                     */
    const UINT8*            pui1_fontdata,  /* font data                     */
    UINT32                  ui4_size,       /* size of font data             */
    UINT8                   ui1_small,      /* size of FE_FNT_SIZE_SMALL     */
    UINT8                   ui1_medium,     /* size for FE_FNT_SIZE_MEDIUM   */
    UINT8                   ui1_large);     /* size for FE_FNT_SIZE_LARGE    */

/*------------------------------------------------------------------*/
/*! @brief  c_fe_add_combine_bmp_mem_font
 *              Client should specify three font files, of which each one must
 *              be a bitmap font of one and only one font size.
 *  @param [in] ps_font             Specify the Font name. User can define the font
 *                                          name. To use the real font family name is not
 *                                          necessary.
 *  @param [in] pui1_fontdata_small     Specify the data of small-sized font.
 *  @param [in] ui4_size_small          Specify the data size of small-sized font.
 *  @param [in] pui1_fontdata_medium    Specify the data of medium-sized font.
 *  @param [in] ui4_size_medium         Specify the data size of medium-sized font.
 *  @param [in] pui1_fontdata_large     Specify the data of large-sized font.
 *  @param [in] ui4_size_large          Specify the data size of large-sized font.
 *  @return Return the error code.
 *  @retval FER_OK              The font is added successfully.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 *  @retval FER_INV_FONT        The ps_file file is not a valid font file.
 *  @retval FER_NO_FONT_SIZE    Some specified font sizes are not available.
 *  @retval FER_NO_MEMORY       Out of system memory
 *  @retval FER_FONT_EXIST      The same font existed.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_add_combine_bmp_mem_font(
    const CHAR*             ps_font,                /* font name                     */
    const UINT8*            pui1_fontdata_small,    /* data of small-sized font      */
    UINT32                  ui4_size_small,         /* data size of small-sized font */
    const UINT8*            pui1_fontdata_medium,   /* data of medium-sized font      */
    UINT32                  ui4_size_medium,        /* data size of medium-sized font */
    const UINT8*            pui1_fontdata_large,    /* data of large-sized font      */
    UINT32                  ui4_size_large);        /* data size of large-sized font */

/*------------------------------------------------------------------*/
/*! @brief  c_fe_add_multiple_mem_font
 *              Applications could call this function to add ui1_data_count memory
 *              fonts into system. When this function is called, font engine will 
 *              read and validate the specified font data. Clients should also specify 
 *              three font sizes (Small / Medium / Large) for this fonts. For vector font,
 *              the font size is assumed to be 'Point Size', and for bitmap
 *              font, 'Pixel Size' instead.  If the specified font size inside
 *              a vector font is an embedded bitmap, the size is turned to be
 *              'Pixel Size' also.
 *  @param [in] ps_font          Specify the Font name. User can define the font
 *                                        name. To use the real font family name is not
 *                                       necessary.
 *  @param [in] ui1_data_count        Specify the font count.
 *  @return Return the error code.
 *  @retval FER_OK              The font is added successfully.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 *  @retval FER_INV_FONT        The ps_file file is not a valid font file.
 *  @retval FER_NO_FONT_SIZE    Some specified font sizes are not available.
 *  @retval FER_NO_MEMORY       Out of system memory
 *  @retval FER_FONT_EXIST      The same font existed.
 */
/*------------------------------------------------------------------*/

INT32 c_fe_add_multiple_mem_font(
    const CHAR*             ps_font,
    FE_INIT_DATA            at_init_data[],
    UINT8                   ui1_data_count);

/*------------------------------------------------------------------*/
/*! @brief c_fe_change_font_resource
 *             Applications could call this function to Change font.
 *  @param [in] ps_font          Specify the Font name. User can define the font
 *                                       name. To use the real font family name is not
 *                                       necessary.
 *  @param [in] ui1_font_id     Specify the font record ID. 
 *  @return Return the error code.
 *  @retval FER_OK              The font is change font successfully.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_change_font_resource(
    const CHAR              *ps_font,
    UINT8                   ui1_font_id);


INT32 c_fe_get_cur_font_resource(
	HFONT_T h_fnt,
	UINT8  *pui1_cur_font_id);


/*------------------------------------------------------------------*/
/*! @brief  c_fe_add_multiple_file_font
 *              Applications could call this function to add ui1_data_count 
 *              fonts into system. When this function is called, font engine will 
 *              read and validate the specified font file. Clients should also specify 
 *              three font sizes (Small / Medium / Large) for this fonts. For vector font,
 *              the font size is assumed to be 'Point Size', and for bitmap
 *              font, 'Pixel Size' instead.  If the specified font size inside
 *              a vector font is an embedded bitmap, the size is turned to be
 *              'Pixel Size' also.
 *  @param [in] ps_font          Specify the Font name. User can define the font
 *                                       name. To use the real font family name is not
 *                                       necessary.
 *  @param [in] at_init_file    Specify the file and size of fonts.
 *  @param [in] ui1_file_count        Specify the font count.
 *  @return Return the error code.
 *  @retval FER_OK              The font is added successfully.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_NO_FONT_FILE    The ps_file font file doesn't exist.
 *  @retval FER_INV_FONT        The ps_file file is not a valid font file.
 *  @retval FER_NO_FONT_SIZE    Some specified font sizes are not available.
 *  @retval FER_NO_MEMORY       Out of system memory
 *  @retval FER_FONT_EXIST      The same font existed.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_add_multiple_file_font(
    const CHAR*             ps_font,
    FE_INIT_FILE            at_init_file[],
    UINT8                   ui1_file_count);

/*------------------------------------------------------------------*/
/*! @brief c_fe_remove_font
 *              This function removes a font from system.
 *  @param [in] ps_font             Specify the Font name.
 *  @param [in] e_option            Specify X_FE_OPTION_FORCE to force the removal 
 *                                          of specified font; the font engine will free
 *                                          all the associated objects, handles and other
 *                                          resources; conversely, if existed any handle
 *                                          associated with this font, the font engine
 *                                          will return the error status FER_FONT_IN_USE.
 *  @return  Return the error code.
 *  @retval FER_OK              The routine was successful.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         One or more parameter(s) are invalid.
 *  @retval FER_FONT_IN_USE     One or more handle(s) are associated with this
 *                                           font.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_remove_font(
    const CHAR*             ps_font,        /* font name                     */
    FE_REMOVE_OPTION_T      e_option);

/*------------------------------------------------------------------*/
/*! @brief c_fe_has_font
 *              Applications can use this function to check if the font is in
 *              this system.
 *  @param [in] ps_font             Contains the font name.
 *  @param [in] e_style             Font style.
 *  @return Return if the system has the specify font.
 *  @retval TRUE                This system contains the font.
 *  @retval FALSE               Otherwise.
 */
/*------------------------------------------------------------------*/
BOOL c_fe_has_font(
    const CHAR*             ps_font,
    FE_FNT_STYLE            e_style);


/**
    font handle functions (create/release)
 **/
/*------------------------------------------------------------------*/
/*! @brief c_fe_create_font
 *              This function creates a font object according to the specified
 *              font face, size, style and charmap.
 *  @param [in] ps_font             Contains the font name.
 *                                          If ps_font is NULL, use system default font,
 *                                          and e_style parameter is ignored.
 *  @param [in] e_size              Contains the font size. Can be
 *                                          FE_FNT_SIZE_SMALL,
 *                                          FE_FNT_SIZE_MEDIUM or
 *                                          FE_FNT_SIZE_LARGE.
 *  @param [in] e_style             Contains the font style, and can be bit-ored.
 *                                          Style can be
 *                                          FE_FNT_STYLE_REGULAR,
 *                                          FE_FNT_STYLE_ITALIC,
 *                                          FE_FNT_STYLE_BOLD,
 *                                          FE_FNT_STYLE_UNDERLINE and
 *                                          FE_FNT_STYLE_STRIKEOUT.
 *  @param [in] e_cmap             Contains the character map encoding method.
 *                                          Can be
 *                                          FE_CMAP_ENC_MS_SYMBOL,
 *                                          FE_CMAP_ENC_UNICODE,
 *                                          FE_CMAP_ENC_SJIS,
 *                                          FE_CMAP_ENC_GB2312,
 *                                          FE_CMAP_ENC_BIG5,
 *                                          FE_CMAP_ENC_WANSUNG,
 *                                          FE_CMAP_ENC_JOHAB,
 *                                          FE_CMAP_ENC_ADOBE_STANDARD,
 *                                          FE_CMAP_ENC_ADOBE_EXPERT,
 *                                          FE_CMAP_ENC_ADOBE_CUSTOM,
 *                                          FE_CMAP_ENC_ADOBE_LATIN_1,
 *                                          FE_CMAP_ENC_OLD_LATIN_2,
 *                                          FE_CMAP_ENC_APPLE_ROMAN.
 *  @param [out] ph_fnt              References the handle of the new font object if
 *                                            the routine was successful. In all other cases,
 *                                            the value of this argument is undetermined.
 *  @note
 *  @see
 *  @return Return the error code.
 *  @retval FER_OK              The routine was successful.
 *  @retval FER_NOT_INIT        The Font Engine is not yet initialized.
 *  @retval FER_INV_ARG         Invalid arguments.
 *  @retval FER_INV_FONT        The font is not in system.
 *  @retval FER_NO_FONT_CMAP    The font character map is not supported.
 *  @retval FER_NO_FONT_STYLE   The font style is not supported by this font.
 *  @retval FER_NO_MEMORY       Out of system memory.
 *  @retval FER_OUT_OF_HANDLES  There are no more handles available.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_create_font(
    const CHAR*             ps_font,        /* Font name                     */
    FE_FNT_SIZE             e_size,         /* Size of font                  */
    FE_FNT_STYLE            e_style,        /* Italic/bold/underline/...     */
    FE_CMAP_ENCODING        e_cmap,         /* Character map identifier      */
    HFONT_T*                ph_fnt);        /* return : font hadle           */


/**
    character map specific functions
 **/

/*------------------------------------------------------------------*/
/*! @brief c_fe_has_cmap
 *              This function is used to check if the font contains the
 *              character map.
 *  @param [in] ps_font             The font name to be checked.
 *  @param [in] e_style             Font style.
 *  @param [in] e_cmap            Character Map to be checked.
 *  @return If the character map is supported by this font.
 *  @retval TRUE                The character map is supported by this font.
 *  @retval FALSE               Otherwise.
 */
/*------------------------------------------------------------------*/
BOOL c_fe_has_cmap(
    const CHAR*             ps_font,
    FE_FNT_STYLE            e_style,
    FE_CMAP_ENCODING        e_cmap);

/*------------------------------------------------------------------*/
/*! @brief c_fe_get_string_size
 *              This API calculates the pixel size of the string
 *              (when displayed on screen).
 *  @param [in] h_fnt               Contains a handle of font object.
 *  @param [in] w2s_str             The string to be evaluated.
 *  @param [in] i4_length           Maximum length to be evaluated.
 *                                          If <0, use whole w2s_str. (NULL-terminated)
 *  @param [out] pi2_width           References the width of the string if the
 *                                             routine was successful. In all other cases, the
 *                                             value of this argument is undetermined.
 *  @param [out] pi2_height          References the height of the string if the
 *                                             routine was successful. In all other cases, the
 *                                             value of this argument is undetermined.
 *  @return Return the error code.
 *  @retval FER_OK              The routine was successful.
 *  @retval FER_NOT_INIT        The Font Engine is not initialized yet.
 *  @retval FER_INV_HANDLE      The argument h_fnt is an invalid font handle.
 *  @retval FER_INV_ARG         One or more characters in the string is invalid
 *                                          or not in font.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_get_string_size(
    HFONT_T                 h_fnt,
    const UTF16_T*          w2s_str,        /* string to be evaluated        */
    INT32                   i4_length,      /* max length to be evaluated    */
    INT32*                  pi4_width,      /* width of string               */
    INT32*                  pi4_height);    /* height of string              */

/*------------------------------------------------------------------*/
/*! @brief c_fe_set_resolution
 *              This function changes the resolution (dpi).
 *  @param [in] ui4_dpi_hori        Horizontal resolution (dpi) of display.
 *  @param [in] ui4_dpi_vert        Vertical resolution (dpi) of display.
 *  @return Return the error code.
 *  @retval FER_OK              Font Engine is successfully initialized.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_set_resolution(
    UINT32                  ui4_dpi_hori,   /* horizontal resolution in dpi  */
    UINT32                  ui4_dpi_vert);  /* vertical   resolution in dpi  */

/*------------------------------------------------------------------*/
/*! @brief c_fe_set_dpi
 *              This function changes the resolution (dpi).
 *  @param [in] ui4_dpi_hori        Horizontal resolution (dpi) of display.
 *  @param [in] ui4_dpi_vert        Vertical resolution (dpi) of display.
 *  @return Return the error code.
 *  @retval FER_OK              Font Engine is successfully initialized.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_set_dpi(
    HFONT_T                 h_fnt,
    UINT32                  h_resolution,
    UINT32                  v_resolution);

/*------------------------------------------------------------------*/
/*! @brief c_fe_use_global_dpi
 *              This function get the resolution (dpi) from global or user setted.
 *  @param [in] ui4_dpi_hori        Horizontal resolution (dpi) of display.
 *  @param [in] ui4_dpi_vert        Vertical resolution (dpi) of display.
 *  @return Return the error code.
 *  @retval FER_OK              Font Engine is successfully initialized.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_use_global_dpi(HFONT_T  h_fnt);

/*------------------------------------------------------------------*/
/*! @brief  c_fe_set_rendering_offset
 *              This function set the rendering offset of a font.
 *  @param [in] ps_font             font name
 *  @param [in] ui1_font_id         font ID
 *  @param [in] i4_offset_small     offset for size SMALL
 *  @param [in] i4_offset_medium    offset for size MEDIUM
 *  @param [in] i4_offset_large     offset for size LARGE
 *  @return Return the error code.
 *  @retval FER_OK              Font Engine is successfully initialized.
 *  @retval FER_NO_FONT         No such font name or font ID is found.
 */
/*------------------------------------------------------------------*/
extern INT32 c_fe_set_rendering_offset(
    const CHAR          *ps_font,
    UINT8               ui1_font_id,
    INT32               i4_offset_small,
    INT32               i4_offset_medium,
    INT32               i4_offset_large);

/*------------------------------------------------------------------*/
/*! @brief  c_fe_set_unrecognizable_char
 *              This function sets the character to be displayed when a unrecognizable character is met
 *  @param [in] h_font              font handle
 *  @param [in] w2_ch               desired character
 *  @return Return the error code.
 *  @retval FER_OK              The routine was successful.
 *  @retval FER_INV_HANDLE      Invalid handle.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_set_unrecognizable_char(
    HFONT_T             h_font,
    CHAR_CODE           t_ch);
/*------------------------------------------------------------------*/
/*! @brief  c_fe_get_unrecognizable_char
 *              This function gets the character to be displayed when a unrecognizable character is met
 *  @param [in]  h_font              font handle
 *  @param [out] pw2_ch              pointer to the desired character
 *  @return Return the error code.
 *  @retval FER_OK              The routine was successful.
 *  @retval FER_INV_HANDLE      Invalid handle.
 */
/*------------------------------------------------------------------*/
INT32 c_fe_get_unrecognizable_char(
    HFONT_T             h_font,
    CHAR_CODE           *pt_ch);
   
/*------------------------------------------------------------------*/
/*! @brief  c_fe_cache_flush
 *              This function empty the cache memory.
 *  @return Return the error code.
 *  @retval FER_OK              The routine was successful.
 */
/*------------------------------------------------------------------*/

INT32 c_fe_cache_flush(VOID);

/*------------------------------------------------------------------*/
/*! @brief  c_fe_preload_char
 *              This function preload characters to be displayed
 *  @param [in]  h_font              font handle
 *  @param [in]  e_render_mode  font render mode
 *  @param [in]  w2s_preload_string  string for preload
 *  @param [in]  ui4_str_size  string size
 *  @param [out] pui4_str_size_loaded        size of preloaded
 *  @return Return the error code.
 *  @retval FER_OK              preload successful.
 *  @retval FER_CACHE_FULL  cache full, loaded *pui4_str_size_loaded characters.
 *  @retval other      preload error.
 */
/*------------------------------------------------------------------*/

INT32 c_fe_preload_char(
    HFONT_T                     h_fnt,
    FE_RENDER_MODE_T            e_render_mode,
    UTF16_T*                    w2s_preload_string,
    UINT32                      ui4_str_size,
    UINT32*                     pui4_str_size_loaded);

INT32	c_fe_set_no_hinting_by_font_name(
	const char*	ps_fnt_name,
	BOOL		b_no_hinting);

INT32 c_fe_set_custom_size(
    HFONT_T                 h_fnt,
    UINT8                   ui1_custom_size) ;

#endif  /* _C_FE_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_FONT*/
/*----------------------------------------------------------------------------*/
