/*----------------------------------------------------------------------------*
 * No Warranty                                                                *
 * Except as may be otherwise agreed to in writing, no warranties of any      *
 * kind, whether express or implied, are given by MTK with respect to any MTK *
 * Deliverables or any use thereof, and MTK Deliverables are provided on an   *
 * "AS IS" basis.  MTK hereby expressly disclaims all such warranties,        *
 * including any implied warranties of merchantability, non-infringement and  *
 * fitness for a particular purpose and any warranties arising out of course  *
 * of performance, course of dealing or usage of trade.  Parties further      *
 * acknowledge that Company may, either presently and/or in the future,       *
 * instruct MTK to assist it in the development and the implementation, in    *
 * accordance with Company's designs, of certain softwares relating to        *
 * Company's product(s) (the "Services").  Except as may be otherwise agreed  *
 * to in writing, no warranties of any kind, whether express or implied, are  *
 * given by MTK with respect to the Services provided, and the Services are   *
 * provided on an "AS IS" basis.  Company further acknowledges that the       *
 * Services may contain errors, that testing is important and Company is      *
 * solely responsible for fully testing the Services and/or derivatives       *
 * thereof before they are used, sublicensed or distributed.  Should there be *
 * any third party action brought against MTK, arising out of or relating to  *
 * the Services, Company agree to fully indemnify and hold MTK harmless.      *
 * If the parties mutually agree to enter into or continue a business         *
 * relationship or other arrangement, the terms and conditions set forth      *
 * hereunder shall remain effective and, unless explicitly stated otherwise,  *
 * shall prevail in the event of a conflict in the terms in any agreements    *
 * entered into between the parties.                                          *
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2004, MediaTek Inc.
 * All rights reserved.
 *
 * Unauthorized use, practice, perform, copy, distribution, reproduction,
 * or disclosure of this information in whole or in part is prohibited.
 *-----------------------------------------------------------------------------
 * $RCSfile: c_mhp.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X/DTV_X_HQ_int/DTV_X_ATSC/3 $
 * $SWAuthor: Weider Chang $
 * $MD5HEX: 72ec35f463230d3f1747ea7bd18764a5 $
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/
 
#ifndef _C_MHP_H_
#define _C_MHP_H_

#include "u_mhp.h"
#include "x_ac_mhp.h"
#include "x_mhp.h"


#ifndef __MHP_NAV_COMP_CLEAN__
#define MHP_STOP_SVC_PARAM_NONE       ((UINT32) 0x0)    /**<Operation flag, do nothing when stop MHP service   */
#define MHP_STOP_SVC_PARAM_ADJ_VDP    ((UINT32) 0x1)    /**<Operation flag, adjust vdp when stop MHP service        */

#define MHP_VIDEO_UPDATE_PARAM_NONE       ((UINT32) 0x0)    /**<Video update notification flag, N/A       */
#define MHP_VIDEO_UPDATE_PARAM_FREEZE     ((UINT32) 0x1)    /**<Video update notification flag, video froze/unfroze      */
#define MHP_VIDEO_UPDATE_PARAM_SCR_MDOE   ((UINT32) 0x2)    /**<Video update notification flag,  screen mode controled by MHEG5 or not    */
#define MHP_VIDEO_UPDATE_PARAM_IFRAME     ((UINT32) 0x4)    /**< Video update notification flag, Iframe played/stopped       */

/* Minimum section buffers */
#define MHP_MIN_SEC_BUF           (100)       /**<MHP_MIN_SECTION_BUFFERS        */

/* Section buffer, buffer size (bytes) */
#define MHP_SEC_BUF_SIZE          (4*1024)    /**<MHP_SECTION_BUFFER_SIZE        */


/*------------------------------------------------------------------*/
/*! @enum MHP_COEXIST_DISPLAY_T
 *  @brief MHP UI display mode
 *  @li@c  MHP_HIDE_GUI                                    - Hide MHP UI
 *  @li@c  MHP_OVERLAY_GUI                              - Overlay MHP UI
 *  @li@c  MHP_OVERLAY_GUI_LIGHTER                - Overlay MHP UI lighter
 *  @li@c  MHP_OVERLAY_GUI_BOTTOMMOST        - Overlay MHP UI bottommost
 */
/*------------------------------------------------------------------*/
typedef enum _MHP_COEXIST_DISPLAY_T
{
    MHP_HIDE_GUI,
    MHP_OVERLAY_GUI,
    MHP_OVERLAY_GUI_LIGHTER,
    MHP_OVERLAY_GUI_BOTTOMMOST
} MHP_COEXIST_DISPLAY_T;

typedef enum _MHP_PROFILE_ID_T
{
    MHP_PROFILE_UK,
    MHP_PROFILE_NZ,
    MHP_PROFILE_IRL,

    MHP_PROFILE_NONE
} MHP_PROFILE_ID_T;

/*------------------------------------------------------------------*/
/*! @struct MHP_STOP_SVC_PARAM_T
 *  @brief MHP stop service parameters
 *  @li@c  ui4_op_flag    - Operation flag:
 *                          MHP_STOP_SVC_PARAM_NONE
 *                          MHP_STOP_SVC_PARAM_ADJ_VDP
 *
 *  @li@c  b_adjust_vdp   - when ui4_op_flag == MHP_STOP_SVC_PARAM_ADJ_VDP, indicate if need adjust vdp
 */
/*------------------------------------------------------------------*/
typedef struct
{
    UINT32          ui4_op_flag;
    BOOL            b_adjust_vdp;
} MHP_STOP_SVC_PARAM_T;

/*------------------------------------------------------------------*/
/*! @enum MHP_NFY_COND_T
 *  @brief MHP notify condition
 *  @li@c  MHP_NFY_COND_TABLE_LOADED                       - SI table loaded well
 *  @li@c  MHP_NFY_COND_TABLE_FAILED                       - SI table loaded failed
 *  @li@c  MHP_NFY_COND_DATA_BROADCASD_ID_DESC_RETURNS     - Broadcast ID descriptor returns
 *  @li@c  MHP_NFY_COND_DATA_BROADCASD_ID_DESC_GONE        - Broadcast ID descriptor gone
 *  @li@c  MHP_NFY_COND_NB_ACTION_INITIATE_AUTOBOOT        - initiate autoboot
 *  @li@c  MHP_NFY_COND_CAROUSEL_ID_CHANGED                - Carousel ID chanaged
 *  @li@c  MHP_NFY_COND_SERVICE_GONE                       - Service gone
 *  @li@c  MHP_NFY_COND_SERVICE_RETURNS                    - Service returns
 *  @li@c  MHP_NFY_COND_SERVICE_CHANGED                    - Service changed
 */
/*------------------------------------------------------------------*/
typedef enum _MHP_NFY_COND_T
{
    MHP_NFY_COND_TABLE_LOADED,
    MHP_NFY_COND_TABLE_FAILED,    
    MHP_NFY_COND_DATA_BROADCASD_ID_DESC_RETURNS,
    MHP_NFY_COND_DATA_BROADCASD_ID_DESC_GONE,
    MHP_NFY_COND_NB_ACTION_INITIATE_AUTOBOOT,
    MHP_NFY_COND_CAROUSEL_ID_CHANGED,
    MHP_NFY_COND_SERVICE_GONE,
    MHP_NFY_COND_SERVICE_RETURNS,
    MHP_NFY_COND_SERVICE_CHANGED
} MHP_NFY_COND_T;

/*------------------------------------------------------------------*/
/*! @enum MHP_IFRAME_SATTE_T
 *  @brief Iframe states
 *  @li@c  MHP_IFRAME_PLAY  - Iframe plyed
 *  @li@c  MHP_IFRAME_STOP  - Iframe stopped
 */
/*------------------------------------------------------------------*/
typedef enum _MHP_IFRAME_STATE_T
{
    MHP_IFRAME_PLAY,
    MHP_IFRAME_STOP
} MHP_IFRAME_SATTE_T;


/*------------------------------------------------------------------*/
/*! @struct MHP_VIDEO_UPDATE_NFY_T
 *  @brief Notification when video update
 *  @li@c  ui4_nfy_flag           - Notification flag:
 *                                  MHP_VIDEO_UPDATE_PARAM_NONE    
 *                                  MHP_VIDEO_UPDATE_PARAM_FREEZE  
 *                                  MHP_VIDEO_UPDATE_PARAM_SCR_MDOE
 *                                  MHP_VIDEO_UPDATE_PARAM_IFRAME  
 *  @li@c  b_freeze               - Video froze or unfroze
 *  @li@c  e_iframe_state         - Iframe played or stopped
 *  @li@c  b_internal_scrn_mode   - Internal screen mode or not
 */
/*------------------------------------------------------------------*/
typedef struct
{
    UINT32                  ui4_nfy_flag;
    
    BOOL                    b_freeze;
    MHP_IFRAME_SATTE_T      e_iframe_state;
    BOOL                    b_internal_scrn_mode;
} MHP_VIDEO_UPDATE_NFY_T;


/*------------------------------------------------------------------*/
/*! @brief Callback to notify the SI loading status
 *  @param [in] e_cond - MHP notify condition
 *  @param [in] pv_tag - User's tag
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*mhg_eng_load_nfy_fct)(
    MHP_NFY_COND_T              e_cond,
    VOID*                       pv_tag
    );
    

/*------------------------------------------------------------------*/
/*! @brief Callback AP to tune to specified service
 *  @param [in] i4_svc_id   - Service index
 *  @param [out] pb_status -True: The specified service is available.
 *                                         False: The specified service is not available.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*mhp_tune_svc_nfy)(
    INT32                       i4_svc_id, 
    BOOL*                       pb_status
    );

/*------------------------------------------------------------------*/
/*! @brief Callback AP of the tune method
 *  @param [in] b_tune_quietly - FALSE: Subsequent tuning shall be "normal"
 *                               TRUE:  Subsequent tuning shall be "quiet"
 *  @param [out] None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*mhp_tune_method_update_nfy)(
    BOOL                        b_tune_quietly
    );

/*------------------------------------------------------------------*/
/*! @brief Callback to notify the video update info
 *  @note Internel use
 */
/*------------------------------------------------------------------*/
typedef VOID  (*mhp_video_update_nfy)(
    MHP_VIDEO_UPDATE_NFY_T*     pt_data
    );

/*------------------------------------------------------------------*/
/*! @struct MHG_SETUP_T
 *  @brief MHP setup structure
 *  @li@c  ui1_ver                                            - Version
 *  @li@c  pv_mem_rgn                                   - Pointer to the memory region
 *  @li@c  z_mem_rgn                                     - Size of the mmeory region
 *  @li@c  ui1_task_pri                                    - MHP task priority
 *  @li@c  ui4_sec_buf_num                            - Number of the section buffers
 *  @li@c  b_sub_with_gfx                               - Indicates if DVB subtitles and MHP Visibles can displayed simultaneously
 *  @li@c  s_dvp_name[10]                             - Null terminated hardware, software and manufacturer specific
 *  @li@c  pf_tune_svc_nfy                              - Callback function for tuning service
 *  @li@c  pf_tune_method_update_nfy            - Callback function for updating tune method
 */
/*------------------------------------------------------------------*/
typedef struct
{
    UINT8 ui1_ver;

    VOID*  pv_mem_rgn;
    SIZE_T z_mem_rgn;
    UINT8  ui1_task_pri;
    UINT32 ui4_sec_buf_num;
    BOOL   b_sub_with_gfx;
    CHAR   s_dvp_name[10];
    mhp_tune_svc_nfy           pf_tune_svc_nfy;
    mhp_tune_method_update_nfy pf_tune_method_update_nfy;
    mhp_video_update_nfy       pf_video_update_nfy;
    
} MHP_SETUP_T;

/* Locate to s specified service */
typedef struct 
{
    UINT16 ui2_on_id;     /* Original network ID */
    UINT16 ui2_ts_id;     /* Transport stream ID */
    UINT16 ui2_svc_id;    /* Service ID */
    INT32  i4_comp_tag;   /* Component Tag */
} MHP_DVB_LOCATOR_T;

typedef struct
{
    UINT8 ui1_ver;                  /* Version */
    
    MHP_DVB_LOCATOR_T* pt_dvb_loc;  /* DVB locator */
} MHP_START_EXEC_T;

typedef struct
{
    UINT8 ui1_ver;  /* Version */
    
    BOOL  b_clean;  /* Clean cached DSMCC data nor not */
} MHP_STOP_EXEC_T;

typedef struct
{
    UINT8 ui1_ver;  /* Version */
    
    BOOL  b_on;     /* Active or inactive */
} MHP_PAUSE_EXEC_T;


/*------------------------------------------------------------------*/
/*! @brief Callback AP to tune to specified service
 *  @param [in] i4_svc_id  - Service index
 *  @param [out] pb_status - True: The specified service is available.
 *                           False: The specified service is not available.
 */
/*------------------------------------------------------------------*/
typedef INT32 ( *mhp_tune_svc_fct )(
    INT32                      i4_svc_id,
    VOID*                      pv_tag,
    BOOL*                      pb_status
    );

#endif


BOOL c_mhp_support();
VOID c_mhp_lib_init( VOID );
VOID c_mhp_lib_deinit( VOID );

/*extern Alticast implementation APIs prototype*/
VOID c_mhp_init(
    HANDLE_T                h_canvas_mhp,
    HANDLE_T                h_canvas_sbtl,
    HANDLE_T                h_canvas_ttx,
    mhp_eng_state_nfy_fct pf_state_nfy);
        

VOID c_mhp_deinit();

VOID c_mhp_hide();

VOID c_mhp_show();
VOID c_mhp_ALA_hide( VOID );
VOID c_mhp_disable();

INT32 c_mhp_set(
    MHP_SET_TYPE_T        e_set_type,
    void*                 pv_set_info,
    SIZE_T                z_set_info_size );
    
VOID c_mhp_key_press(
    UINT32                ui4_key_code,
    BOOL                  b_key_down,
    BOOL*                 pb_processed );


/* Interface for registering SBTL notify function when mhp init */
INT32 c_mhp_register_sbtl_nfy_fct(
	mhp_sbtl_ttx_nfy_fct  pf_do_sbtl_nfy,
	VOID*                 pv_tag );

/* Interface for registering TTX notify function when mhp init */
INT32 c_mhp_register_ttx_nfy_fct(	
	mhp_sbtl_ttx_nfy_fct  pf_do_ttx_nfy,
	VOID*                 pv_tag );		
VOID c_mhp_reg_request_op_nfy(
                    mhp_request_op_nfy  pf_request_op_nfy,
                    VOID*                 pv_tag
                    );

#endif /* _C_MHP_H_ */
