/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_midxbuld.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This file exports all media index builder API and constant for 
 *         other module using.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_MUTIL_MIDXBULD Multimedia Index Builder
 *
 *  @ingroup groupMW
 *  @brief Provides external function TV Application
 *
 *  %MW Application API would be used by TV Application, lot of these APIs would
 *   be wrapped from Exported API
 *
 *  @see groupMW
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
#ifndef _C_MIDXBULD_H_
#define _C_MIDXBULD_H_


/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#ifndef __KERNEL__

#include "u_common.h"
#include "u_handle.h"
#include "u_midxbuld.h"
#include "u_mfmtrecg.h"

#else

#include "u_common.h"
#include "u_handle.h"
#include "u_midxbuld.h"
#include "u_mfmtrecg.h"

#endif
/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
 
/* This API opens a media file to do media indexing. */                        
/*------------------------------------------------------------------*/
/*! @brief          This API opens a media file to do media indexing.
 *  @param [in]     ps_path_name    specify the path string of file which will be opened in minfo.
 *  @param [in]     pf_nfy          specify the function point of notification function.
 *  @param [in]     pv_tag          specify the tag which will be send back to AP when minfo notification.
 *  @param [in]     pt_format       if the application specify the format (except MEDIA_TYPE_UNKNOWN), media info module will use this information to select the right handler to deal with the media file.
 *
 *  @param [out]    pt_format       If the application specify the unknown format (ie. MEDIA_TYPE_UNKNOWN), media info module will use format recognizer to look it up; then return it to AP.
 *  @param [out]    ph_midxbuld     The parameter will retrieve a media index builder handle.
 *
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 *  @retval      -MIDXBULDR_FMT_NOT_SUPPORT Format not support.
 *  @retval      -MIDXBULDR_OUT_OF_MEMORY Out of memory.
 *  @retval      -MIDXBULDR_OUT_OF_HANDLE Out of Handle
 *  @retval      -MIDXBULDR_FMT_NOT_SUPPORT Format nor support.
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_fopen (
                        CHAR*                    ps_path_name,  /* (I) specify the path string of file which will be opened in minfo. */ 
                        x_midxbuld_nfy_fct       pf_nfy,        /* (I) specify the function point of notification function. */ 
                        VOID*                    pv_tag,        /* (I) specify the tag which will be send back to AP when minfo notification. */ 
                        MEDIA_FORMAT_T*          pt_format,     /* (I/O) if the application specify the format (except MEDIA_TYPE_UNKNOWN), media info module will use this information to select the right handler to deal with the media file./If the application specify the unknown format (ie. MEDIA_TYPE_UNKNOWN), media info module will use format recognizer to look it up; then return it to AP. */
                        MIDXBULD_SOURCE_INFO_T*  pt_src_info,   /* (I) specify the media source information for index building */
                        MIDXBULD_KEY_INFO_T*     pt_key_info,   /* (I/O) specify the key frame information for index building & retrieve handles for key frame. */
                        HANDLE_T*                ph_midxbuld);  /* (O) The parameter will retrieve a media index builder handle. */ 


/* This API opens a media memory buffer to do media indexing. */                        
/*------------------------------------------------------------------*/
/*! @brief          This API opens a media memory buffer to do media indexing.
 *  @param [in]     pv_mem_data     specify the memory address of media data buffer.
 *  @param [in]     z_mem_leng      specify the memory length of media data buffer.
 *  @param [in]     pf_nfy          specify the function point of notification function.
 *  @param [in]     pv_tag          specify the tag which will be send back to AP when minfo notification.
 *  @param [in]     pt_format       if the application specify the format (except MEDIA_TYPE_UNKNOWN), media info module will use this information to select the right handler to deal with the media file.
 *
 *  @param [out]    pt_format       If the application specify the unknown format (ie. MEDIA_TYPE_UNKNOWN), media info module will use format recognizer to look it up; then return it to AP.
 *  @param [out]    ph_midxbuld     The parameter will retrieve a media index builder handle.
 *
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 *  @retval      -MIDXBULDR_FMT_NOT_SUPPORT Format not support.
 *  @retval      -MIDXBULDR_OUT_OF_MEMORY Out of memory.
 *  @retval      -MIDXBULDR_OUT_OF_HANDLE Out of Handle
 *  @retval      -MIDXBULDR_FMT_NOT_SUPPORT Format nor support.
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_mopen (
                        VOID*                    pv_mem_data,   /* (I) specify the memory address of media data buffer. */ 
                        SIZE_T                   z_mem_leng,    /* (I) specify the memory length of media data buffer. */ 
                        x_midxbuld_nfy_fct       pf_nfy,        /* (I) specify the function point of notification function. */ 
                        VOID*                    pv_tag,        /* (I) specify the tag which will be send back to AP when minfo notification. */ 
                        MEDIA_FORMAT_T*          pt_format,     /* (I/O) if the application specify the format (except MEDIA_TYPE_UNKNOWN), media info module will use this information to select the right handler to deal with the media file./If the application specify the unknown format (ie. MEDIA_TYPE_UNKNOWN), media info module will use format recognizer to look it up; then return it to AP. */
                        MIDXBULD_SOURCE_INFO_T*  pt_src_info,   /* (I) specify the media source information for index building */
                        MIDXBULD_KEY_INFO_T*     pt_key_info,   /* (I/O) specify the key frame information for index building & retrieve handles for key frame. */
                        HANDLE_T*                ph_midxbuld);  /* (O) The parameter will retrieve a media index builder handle. */ 

/* This API closes the media index builder handle. */
/*------------------------------------------------------------------*/
/*! @brief          This API closes the media index builder handle.
 *  @param [in]     h_midxbuld         specify the media index builder handle which will be closed.
 *  @param [out]
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_close (
                        HANDLE_T                 h_midxbuld);   /*(I) specify the media index builder handle which will be closed. */ 

/* This API starts indexing the media.This function call is asynchronous call. */
/*------------------------------------------------------------------*/
/*! @brief          specify the media index builder handle which will start indexing
 *  @param [in]     h_midxbuld      specify the media info handle which will be retrieved information.
 *  @param [out]
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_start_indexing (
                        HANDLE_T                 h_midxbuld);   /* (I) specify the media index builder handle which will start indexing*/

/* This API stops indexing the media.This function call is asynchronous call. */
/*------------------------------------------------------------------*/
/*! @brief          specify the media index builder handle which will stop indexing
 *  @param [in]     h_midxbuld      specify the media info handle which will be retrieved information.
 *  @param [out]
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_stop_indexing (
                        HANDLE_T                 h_midxbuld);   /* (I) specify the media index builder handle which will stop indexing*/


/* This API retrieves the information of media idx builder handle. It will return MINFO_NOT_READY, before parse media complete. */
/*------------------------------------------------------------------*/
/*! @brief          This API retrieves the information of media idx builder handle. It will return MINFO_NOT_READY, before parse media complete.
 *  @param [in]     h_midxbuld   specify the media idx buld handle which will be retrieved information.
 *  @param [in]     e_info_type  specify the information type of the media.
 *  @param [in]     pv_buf       specify the memory buffer which will contain the media information.
 *  @param [in]     pz_buf_leng  specify the length of the buffer.
 *
 *  @param [out]    pv_buf       specify the memory buffer which will contain the media information
 *  @param [out]    pz_buf_leng  If it is smaller than required, it will return MINFO_INSUFFICENT_MEM, and pz_buf_leng will contain required length.
 *
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 *  @retval      -MIDXBULDR_INV_ARG Invalid argument.    
 *  @retval      -MIDXBULDR_FCT_NOT_SUPPORT     Function not support.
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_get_info (
                        HANDLE_T                 h_midxbuld,    /* (I) specify the media idx builder handle which will be retrieved information. */ 
                        MIDXBULD_GET_INFO_TYPE_T e_info_type,   /* (I) specify the information type of the media. */ 
                        VOID*                    pv_buf,        /* (I/O) specify the memory buffer which will contain the media information./specify the memory buffer which will contain the media information */ 
                        SIZE_T*                  pz_buf_leng);  /* (I/O) specify the length of the buffer./If it is smaller than required, it will return MINFO_INSUFFICENT_MEM, and pz_buf_leng will contain required length. */ 

/* This API retrieves the information of media idx builder handle. It will return MINFO_NOT_READY, before parse media complete. */
/*------------------------------------------------------------------*/
/*! @brief          This API retrieves the information of media idx builder handle. It will return MINFO_NOT_READY, before parse media complete.
 *  @param [in]     h_midxbuld   specify the media idx buld handle which will be retrieved information.
 *  @param [in]     e_info_type  specify the information type of the media.
 *  @param [in]     pv_buf       specify the memory buffer which will contain the media information.
 *  @param [in]     pz_buf_leng  specify the length of the buffer.
 *
 *  @param [out]    pv_buf       specify the memory buffer which will contain the media information
 *  @param [out]    pz_buf_leng  If it is smaller than required, it will return MINFO_INSUFFICENT_MEM, and pz_buf_leng will contain required length.
 *
 *  @note
 *  @see
 *  @return        
 *  @retval      -MIDXBULDR_OK    Success. The routine was successful and the operation has been completed MIDXBULDR_OK
 *  @retval      -MIDXBULDR_INV_ARG Invalid argument.    
 *  @retval      -MIDXBULDR_FCT_NOT_SUPPORT     Function not support.
 */
/*------------------------------------------------------------------*/
extern INT32 c_midxbuld_set_info (
                        HANDLE_T                 h_midxbuld,    /* (I) specify the media idx builder handle which will be retrieved information. */ 
                        MIDXBULD_SET_INFO_TYPE_T e_info_type,   /* (I) specify the information type of the media. */ 
                        VOID*                    pv_buf,        /* (I/O) specify the memory buffer which will contain the media information./specify the memory buffer which will contain the media information */ 
                        SIZE_T*                  pz_buf_leng);  /* (I/O) specify the length of the buffer./If it is smaller than required, it will return MINFO_INSUFFICENT_MEM, and pz_buf_leng will contain required length. */ 
#ifdef __cplusplus
}
#endif

#endif /* _C_MIDXBULD_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                              /* end of groupMW_MUTIL_MIDXBULD */
/*----------------------------------------------------------------------------*/


