/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*---------------------------------------------------------------------------*/
/*! @file c_tm.h 
 * $RCSfile: c_tm.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 *  @par Description:
 *         This header file contains Table Manager specific definitions,
 *         which are exported.
 *-------------------------------------------------------------------------*/

#ifndef _C_TM_H_
#define _C_TM_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_TBL_MNGR
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                    include files
---------------------------------------------------------------------------*/

#include "u_handle.h"
#include "u_common.h"
#include "u_tm.h"

/*---------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @brief  change tm srouce function
 *
 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [in] e_src_type        - tm source type
 *  @param  [in] pv_src_info       - source info
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return		Error code
 *  @retval		TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_change_src (
                            HANDLE_T       h_obj,
                            TM_SRC_TYPE_T  e_src_type,
                            VOID*          pv_src_info,
                            TM_COND_T*     pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  delete tm object function
 *
 *  @param  [in] h_obj             - handle of tm objst
 *  
 *  @note   
 *
 *  @see
 *  @return     Error code
 *  @retval     TMR_OK   - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_delete_obj (
                            HANDLE_T  h_obj);
                            
/*----------------------------------------------------------------------------*/
/*! @brief  find tm root function
 *
 *  @param  [in] ps_name           - tm root name
 *  
 *  @note   
 *
 *  @see
 *  @return     Found or not
 *  @retval     TRUE   - found
 *  @retval     FALSE  - not found
 */
/*----------------------------------------------------------------------------*/
extern BOOL  c_tm_find_root (
                            const CHAR*  ps_name);

/*----------------------------------------------------------------------------*/
/*! @brief  tm free function

 *  @param  [in] h_obj             - handle of tm objst
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK   - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_free (
                            HANDLE_T  h_obj);

/*----------------------------------------------------------------------------*/
/*! @brief  get tm condition function

 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return		Error code
 *  @retval		TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_get_cond (
                            HANDLE_T    h_obj,
                            TM_COND_T*  pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  get tm object function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [in] e_get_type        - tm get type
 *  @param  [in] pv_get_info       - pointer of tm get info
 *  @param  [out] pv_data          - pointer of tm object data
 *  @param  [out] pz_data_len      - pointer of tm object data length
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_get_obj (
                            HANDLE_T       h_obj,
                            TM_GET_TYPE_T  e_get_type,
                            VOID*          pv_get_info,
                            VOID*          pv_data,
                            SIZE_T*        pz_data_len,
                            TM_COND_T*     pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  get tm protection function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [out] pb_protected     - pointer of boolean value to determine protected or not
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_get_protection (
                            HANDLE_T    h_obj,
                            BOOL*       pb_protected,
                            TM_COND_T*  pe_cond);
                            
/*----------------------------------------------------------------------------*/
/*! @brief  get table version function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [out] pui1_version     - table version
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_get_tbl_version (
                            HANDLE_T    h_obj,
                            UINT8*      pui1_version,
                            TM_COND_T*  pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  load tm object function

 *  @param  [in] h_parent          - handle of tm parent
 *  @param  [in] e_obj_type        - tm object type
 *  @param  [in] e_load_type       - tm load type
 *  @param  [in] pv_load_info      - pointer of load info
 *  @param  [in] e_src_type        - source type
 *  @param  [in] pv_src_info       - pointer of source info
 *  @param  [in] pv_nfy_tag        - notify tag
 *  @param  [in] pf_nfy            - notify function
 *  @param  [out] ph_obj           - pionter of tm object handle
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_load_obj (
                            HANDLE_T        h_parent,
                            HANDLE_TYPE_T   e_obj_type,
                            TM_LOAD_TYPE_T  e_load_type,
                            VOID*           pv_load_info,
                            TM_SRC_TYPE_T   e_src_type,
                            VOID*           pv_src_info,
                            VOID*           pv_nfy_tag,
                            x_tm_nfy_fct    pf_nfy,
                            HANDLE_T*       ph_obj,
                            TM_COND_T*      pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  monitior tm object function

 *  @param  [in] h_parent          - handle of tm parent
 *  @param  [in] e_obj_type        - tm object type
 *  @param  [in] e_mon_type        - tm monitor type
 *  @param  [in] pv_mon_info       - pointer of monitor info
 *  @param  [in] e_src_type        - source type
 *  @param  [in] pv_src_info       - pointer of source info
 *  @param  [in] pv_nfy_tag        - notify tag
 *  @param  [in] pf_nfy            - notify function
 *  @param  [out] ph_obj           - pionter of tm object handle
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/

extern INT32 c_tm_monitor_obj (
                            HANDLE_T       h_parent,
                            HANDLE_TYPE_T  e_obj_type,
                            TM_MON_TYPE_T  e_mon_type,
                            VOID*          pv_mon_info,
                            TM_SRC_TYPE_T  e_src_type,
                            VOID*          pv_src_info,
                            VOID*          pv_nfy_tag,
                            x_tm_nfy_fct   pf_nfy,
                            HANDLE_T*      ph_obj,
                            TM_COND_T*     pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  open tm root function

 *  @param  [in] ps_name          - root name
 *  @param  [in] e_src_type        - source type
 *  @param  [in] pv_src_info       - pointer of source info
 *  @param  [in] pv_nfy_tag        - notify tag
 *  @param  [in] pf_nfy            - notify function
 *  @param  [out] ph_root           - pionter of tm root handle
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_open_root (
                            const CHAR*    ps_name,
                            TM_SRC_TYPE_T  e_src_type,
                            VOID*          pv_src_info,
                            VOID*          pv_nfy_tag,
                            x_tm_nfy_fct   pf_nfy,
                            HANDLE_T*      ph_root,
                            TM_COND_T*     pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  tm object parse function

 *  @param  [in] h_obj             - handle of tm object
 *  @param  [in] e_parse_type      - parse type
 *  @param  [in] pv_parse_info     - pointer of parse info
 *  @param  [in] pv_data           - pointer of parse data
 *  @param  [in] z_data_len        - pointer of parse data length
 *  @param  [in] pf_parse          - pionter parse function
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_parse_obj (
                            HANDLE_T         h_obj,
                            TM_PARSE_TYPE_T  e_parse_type,
                            VOID*            pv_parse_info,
                            VOID*            pv_data,
                            SIZE_T           z_data_len,
                            x_tm_parse_fct   pf_parse,
                            TM_COND_T*       pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  tm set function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [in] e_set_type        - tm set type
 *  @param  [in] pv_set_info       - pointer of set info
 *  @param  [in] z_len             - info length
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_set_obj (
                            HANDLE_T       h_obj,
                            TM_SET_TYPE_T  e_set_type,
                            VOID*          pv_set_info,
                            SIZE_T         z_len,
                            TM_COND_T*     pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  tm set protection function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [in] b_protected       - set protection or not
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_set_protection (
                            HANDLE_T    h_obj,
                            BOOL        b_protected,
                            TM_COND_T*  pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  tm store function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [in] e_store_type      - tm store type
 *  @param  [in] pv_store_info     - tm store info
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_store_obj (
                            HANDLE_T         h_obj,
                            TM_STORE_TYPE_T  e_store_type,
                            VOID*            pv_store_info,
                            TM_COND_T*       pe_cond);


/*----------------------------------------------------------------------------*/
/*! @brief  tm refresh function

 *  @param  [in] h_obj             - handle of tm objst
 *  @param  [out] pe_cond          - pointer of tm condition
 *  
 *  @note   

 *  @see
 *  @return     Error code
 *  @retval     TMR_OK          - Success
 *  @retval     TMR_NOT_INIT    - Table Manager has not been initialized
 *  @retval     TMR_INV_HANDLE  - Invalide handle of h_obj
 */
/*----------------------------------------------------------------------------*/
extern INT32 c_tm_refresh_obj (
                        HANDLE_T    h_obj,
                        TM_COND_T*  pe_cond);

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_TBL_MNGR */
/*----------------------------------------------------------------------------*/

#endif /* _C_TM_H */
