/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_dhcpc_api.h
 * $RCSfile: $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * @par Description:
 *      dhcp client
 */
/*----------------------------------------------------------------------------*/
/*!
* @addtogroup  groupMW_INET
* @{
*/

#ifndef U_DHCPC_API_H
#define U_DHCPC_API_H

/*!
 * @brief DHCP client return value
 * @code
 * enum
 * {
 *     DHCPC_R_OK              =  0,
 *     DHCPC_R_INIT_FAILED     = -1,
 *     DHCPC_R_DEINIT_FAILED   = -2,
 *     DHCPC_R_NI_NOT_EXIST    = -3,
 *     DHCPC_R_MAP_FULL        = -4,
 *     DHCPC_R_START_FAILED    = -5,
 *     DHCPC_R_OSAI_FAILED     = -6,
 *     DHCPC_R_STOP_FAILED     = -7,
 *     DHCPC_R_NO_REQUEST_ID   = -8,
 *     DHCPC_R_RESTART_FAILED  = -9,
 *     DHCPC_R_GET_INFO_FAILED = -10,
 *     DHCPC_R_INVALID_ARGS    = -11,
 *     DHCPC_R_INTERNAL_FAILED = -12,
 *     DHCPC_R_ALREADY_INIT    = -13,
 *     DHCPC_R_NOT_INIT        = -14
 * };
 * @endcode
 *
 * @li@c DHCPC_R_OK              - OK
 * @li@c DHCPC_R_INIT_FAILED     - initialize failed
 * @li@c DHCPC_R_DEINIT_FAILED   - de-initialize failed
 * @li@c DHCPC_R_NI_NOT_EXIST    - not corresponding network interface
 * @li@c DHCPC_R_MAP_FULL        - map is full
 * @li@c DHCPC_R_START_FAILED    - DHCP client start failed
 * @li@c DHCPC_R_OSAI_FAILED     - OSAI failed
 * @li@c DHCPC_R_STOP_FAILED     - DHCP client stop failed
 * @li@c DHCPC_R_NO_REQUEST_ID   - no request id
 * @li@c DHCPC_R_RESTART_FAILED  - DHCP client restart failed
 * @li@c DHCPC_R_GET_INFO_FAILED - get DHCP information failed
 * @li@c DHCPC_R_INVALID_ARGS    - invalid arguments
 * @li@c DHCPC_R_INTERNAL_FAILED - internal error
 * @li@c DHCPC_R_ALREADY_INIT    - already initialization
 * @li@c DHCPC_R_NOT_INIT        - not initialize yet
 */
enum
{
    DHCPC_R_OK              =  0,
    DHCPC_R_INIT_FAILED     = -1,
    DHCPC_R_DEINIT_FAILED   = -2,
    DHCPC_R_NI_NOT_EXIST    = -3,
    DHCPC_R_MAP_FULL        = -4,
    DHCPC_R_START_FAILED    = -5,
    DHCPC_R_OSAI_FAILED     = -6,
    DHCPC_R_STOP_FAILED     = -7,
    DHCPC_R_NO_REQUEST_ID   = -8,
    DHCPC_R_RESTART_FAILED  = -9,
    DHCPC_R_GET_INFO_FAILED = -10,
    DHCPC_R_INVALID_ARGS    = -11,
    DHCPC_R_INTERNAL_FAILED = -12,
    DHCPC_R_ALREADY_INIT    = -13,
    DHCPC_R_NOT_INIT        = -14
};

/*!
 * @enum DLNA_DHCPC_EV_T
 * @brief DHCP client event value
 * @code
 * typedef enum
 * {
 *     DHCPC_EVENT_SUCCESS_DHCPv4     = 1,
 *     DHCPC_EVENT_FAILURE_DHCPv4     = 2,
 *     DHCPC_EVENT_NEW_ADDRESS_DHCPv4 = 3,
 *     DHCPC_EVENT_SUCCESS_LINKLOCAL  = 4,
 *     DHCPC_EVENT_FAILURE_LINKLOCAL  = 5
 * } DLNA_DHCPC_EV_T;
 * @endcode
 *
 * @li@c DHCPC_EVENT_SUCCESS_DHCPv4     - DHCP success event
 * @li@c DHCPC_EVENT_FAILURE_DHCPv4     - DHCP failed event
 * @li@c DHCPC_EVENT_NEW_ADDRESS_DHCPv4 - new address event
 * @li@c DHCPC_EVENT_SUCCESS_LINKLOCAL  - success linklocal event
 * @li@c DHCPC_EVENT_FAILURE_LINKLOCAL  - failed linklocal event
 */
typedef enum
{
    DHCPC_EVENT_SUCCESS_DHCPv4     = 1,
    DHCPC_EVENT_FAILURE_DHCPv4     = 2,
    DHCPC_EVENT_NEW_ADDRESS_DHCPv4 = 3,
    DHCPC_EVENT_SUCCESS_LINKLOCAL  = 4,
    DHCPC_EVENT_FAILURE_LINKLOCAL  = 5
} DLNA_DHCPC_EV_T;

/*!
 * @brief DHCP client callback function
 * @param in e_event   - DHCP event
 */
typedef VOID (*x_dhcpc_nfy_fct) (DLNA_DHCPC_EV_T e_event);

#endif

/*! @} */

