/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_evctx_atsc_eng.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains exported definitions and delcarations of
 *         Event Context ATSC engine.
 */
/*----------------------------------------------------------------------------*/
 
#ifndef _U_EVCTX_ATSC_ENG_H_
#define _U_EVCTX_ATSC_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"
 
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_EVCTX_ATSC_ENG EVCTX ATSC Engine
 *
 *  @ingroup groupMW_EVCTX
 *  @brief The EVCTX ATSC engine module is used to get EVCTX ATSC information.
 *
 *  %MW Application API would be used by Application, lot of these APIs would
 *   be wrapped from Exported API. Using these APIs, application could get event infromation
 *   from the exported API of EVCTX ATSC engine.
 *
 *  @see groupMW_EVCTX_ATSC_ENG
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#pragma pack(4)
/*------------------------------------------------------------------*/
/*! @enum	EVCTX_ATSC_KEY_TYPE_T
 *  @brief		Key type of specific infomation
 *
 *  @code
 *	typedef enum _EVCTX_ATSC_KEY_TYPE_T
 *	{
 *		EVCTX_ATSC_KEY_TYPE_UNKNOWN = 0,
 *		EVCTX_ATSC_KEY_TYPE_TOTAL_AVAIL_TIME,
 *		EVCTX_ATSC_KEY_TYPE_RC_INFO,
 *		EVCTX_ATSC_KEY_TYPE_TOTAL_NUMBER
 *	}EVCTX_ATSC_KEY_TYPE_T;  
 *  @endcode
 *
 *  @li@c	EVCTX_ATSC_KEY_TYPE_UNKNOWN			- Unknown key type
 *  @li@c	EVCTX_ATSC_KEY_TYPE_TOTAL_AVAIL_TIME	- Key type of total available time
 *  @li@c	EVCTX_ATSC_KEY_TYPE_TOTAL_NUMBER		- Total number of key types
 */
/*------------------------------------------------------------------*/
typedef enum _EVCTX_ATSC_KEY_TYPE_T
{
    EVCTX_ATSC_KEY_TYPE_UNKNOWN = 0,
    EVCTX_ATSC_KEY_TYPE_TOTAL_AVAIL_TIME,   /* UINT16 type. In hours. */
    EVCTX_ATSC_KEY_TYPE_RC_INFO,
    EVCTX_ATSC_KEY_TYPE_IDX_RC_INFO,
    EVCTX_ATSC_KEY_TYPE_TOTAL_NUMBER
} EVCTX_ATSC_KEY_TYPE_T;


/*------------------------------------------------------------------*/
/*! @struct	EVCTX_ATSC_KEY_INFO_T
 *  @brief		Key infomation structure
 *
 *  @code
 *	typedef struct _EVCTX_ATSC_KEY_INFO_T
 *	{
 *		EVCTX_ATSC_KEY_TYPE_T   e_atsc_key_type;
 *		VOID*                   pv_atsc_key_info;
 *	} EVCTX_ATSC_KEY_INFO_T;   
 *  @endcode
 *
 *  @li@c	e_atsc_key_type		-Key type of EVCTX
 *  @li@c	pv_atsc_key_info		-Key information of EVCTX
 */
/*------------------------------------------------------------------*/
typedef struct _EVCTX_ATSC_KEY_INFO_T
{
    EVCTX_ATSC_KEY_TYPE_T   e_atsc_key_type;
    VOID*                   pv_atsc_key_info;
} EVCTX_ATSC_KEY_INFO_T;

typedef struct _EVCTX_ATSC_KEY_INFO_EX_T
{
    EVCTX_ATSC_KEY_TYPE_T   e_atsc_key_type;
    UNION_POINTER_T                   pv_atsc_key_info;//VOID*
} EVCTX_ATSC_KEY_INFO_EX_T;
/*------------------------------------------------------------------*/
/*! @struct	EVCTX_ATSC_RATING_T
 *  @brief		Rating structure
 *
 *  @code
 *	typedef struct _EVCTX_ATSC_RATING_T
 *	{
 *		UINT16      ui2_dimension;
 *		UINT16      ui2_rating_value;
 *	} EVCTX_ATSC_RATING_T;  
 *  @endcode
 *
 *  @li@c	ui2_dimension			-Dimension of the rating
 *  @li@c	ui2_rating_value		-Value of the rating
 */
/*------------------------------------------------------------------*/

typedef struct _EVCTX_ATSC_RATING_T
{
    UINT16      ui2_dimension;
    UINT16      ui2_rating_value;
} EVCTX_ATSC_RATING_T;


/*
    pt_ratings is a pointer to an array of EVCTX_ATSC_RATING_T type data.
    ui2_rated_dimensions contains the number of elements in the pt_ratings
    array.
    
    The following is an illustration of the layout, which a real instance
    of EVCTX_ATSC_RATING_LIST_T may be.
    
    Arrows in the left side shows that the pt_ratings array is allocated
    exactly below the pt_ratings field, and followed by the rating 
    description string.
    
    Arrows in the right side shows how to calculate the total size of
    an instance of EVCTX_ATSC_RATING_LIST_T data.
    
        ui2_rating_region           <---
        ui2_rated_dimensions            | sizeof(EVCTX_ATSC_RATING_LIST_T)
------- ps_rating_description           |
| ----- pt_ratings                      |
| |  -- pt_next                     <---                   +
| |  |
| ----->  pt_ratings[0]                      <---
|    |    pt_ratings[1]                          | ui2_rated_dimensions *
|    |    pt_ratings[2]                          | sizeof(EVCTX_ATSC_RATING_T)
|    |        :                                  |
|    |    pt_ratings[ui2_rated_dimensions-1] <---          +
--------> "TV-PG-L-S"                        <---  x_strlen(pt_rating_description)+1
     |
     |
     -> ui2_rating_region   (the next region)
        ui2_rated_dimensions
        pt_rating_description
        pt_ratings
        pt_next
*/        
/*------------------------------------------------------------------*/
/*! @struct	EVCTX_ATSC_RATING_LIST_T
 *  @brief		Rating list structure
 *
 *  @code
 *	typedef struct _EVCTX_ATSC_RATING_LIST_T
 *	{
 *		UINT16                  ui2_rating_region;
 *  		UINT16                  ui2_rated_dimensions;
 *   		CHAR*                   ps_rating_descrption;
 *   		EVCTX_ATSC_RATING_T*    pt_ratings;
 *
 *   		struct _EVCTX_ATSC_RATING_LIST_T*   pt_next;
 *	} EVCTX_ATSC_RATING_T;  
 *  @endcode
 *
 *  @li@c	ui2_rating_region			-Region index 
 *  @li@c	ui2_rated_dimensions		-Total dimension number
 *  @li@c	ps_rating_descrption		-Descrption of the rating
 *  @li@c	pt_ratings				-The one of rating
 *  @li@c	pt_next					-The point to the other _EVCTX_ATSC_RATING_LIST_T
 */
/*------------------------------------------------------------------*/

typedef struct _EVCTX_ATSC_RATING_LIST_T
{
    UINT16                  ui2_rating_region;
    UINT16                  ui2_rated_dimensions;
    CHAR*                   ps_rating_descrption;
    EVCTX_ATSC_RATING_T*    pt_ratings;

    struct _EVCTX_ATSC_RATING_LIST_T*   pt_next;
} EVCTX_ATSC_RATING_LIST_T;

/* for 64bit rpc api */
#pragma pack(4)
typedef struct _EVCTX_ATSC_RATING_LIST_EX_T
{
    UINT16                  ui2_rating_region;
    UINT16                  ui2_rated_dimensions;
    UNION_POINTER_T         ps_rating_descrption;
    UNION_POINTER_T         pt_ratings;

    UNION_POINTER_T         pt_next;
} EVCTX_ATSC_RATING_LIST_EX_T;
#pragma pack()
#pragma pack()
#endif /* _U_EVCTX_ATSC_ENG_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                               /* end of groupMW_EVCTX_ATSC_ENG          */
/*----------------------------------------------------------------------------*/

