/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------*/
/*! @file u_evctx_eng.h 
 * $RCSfile: u_evctx_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/1 $
 * $SWAuthor: Justin Deng $
 * $MD5HEX: 837b3912ec66833817b2c33cd904c44f $
 *
 *  @par Description:
 *         This contains Event Context library macros, definitons, structures,
 *         and API's, which are shared with other modules, and customized engines.
 *---------------------------------------------------------------------------*/

#ifndef _U_EVCTX_ENG_H_
#define _U_EVCTX_ENG_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_EVCTX
 *  @{
 */
/*----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_evctx.h"
#include "u_dt.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @brief  evctx create object function

 *  @param  [in] h_event            - handle of event object
 *  @param  [in] pt_svc_cmd         - event command
 *  @param  [in] pt_offset          - evctx offset
 *  @param  [out] ppv_event_obj_data - pointer of evctx obj data
 *  @param  [out] pe_cond            - pointer of evctx condition
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_create_event_obj_fct) (
    HANDLE_T            h_event,
    EVCTX_COMMAND_T*    pt_svc_cmd,
    EVCTX_OFFSET_T*     pt_offset,
    VOID**              ppv_event_obj_data,
    EVCTX_COND_T*       pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  evctx delete object function

 *  @param  [in] pv_event_obj_data  - event object to delete
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_delete_event_obj_fct) (
    VOID*               pv_event_obj_data);

/*----------------------------------------------------------------------------*/
/*! @brief  evctx get object information length function

 *  @param  [in] h_event                - handle of event object
 *  @param  [in] e_key_type             - pointer event key information
 *  @param  [in] pv_key_info            - event information
 *  @param  [out] pz_event_info_len     - pointer of evctx obj length
 *  @param  [out] pe_cond               - pointer of evctx condition
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_get_event_obj_info_len_fct) (
    HANDLE_T            h_event,
    EVCTX_KEY_TYPE_T    e_key_type,
    VOID*               pv_key_info,
    SIZE_T*             pz_event_info_len,
    EVCTX_COND_T*       pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  evctx get object information function

 *  @param  [in] h_event                - handle of event object
 *  @param  [in] e_key_type             - event type to query
 *  @param  [in] pv_key_info            - pointer event key information
 *  @param  [in] pz_event_info_len      - pointer of evctx obj length
 *  @param  [out] pz_event_info_len     - pointer of evctx obj length
 *  @param  [out] pv_event_info         - pointer of evctx obj information
 *  @param  [out] pe_cond               - pointer of evctx condition
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_get_event_obj_info_fct) (
    HANDLE_T            h_event,
    EVCTX_KEY_TYPE_T    e_key_type,
    VOID*               pv_key_info,
    SIZE_T*             pz_event_info_len,
    VOID*               pv_event_info,
    EVCTX_COND_T*       pe_cond);

/*----------------------------------------------------------------------------*/
/*! @brief  evctx task function
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_engine_task_fct) ( VOID );

/*----------------------------------------------------------------------------*/
/*! @brief  evctx set function

 *  @param  [in] pt_set_cmd         - pointer to the set command
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_set_fct) (
    EVCTX_COMMAND_T*    pt_set_cmd);

/*----------------------------------------------------------------------------*/
/*! @brief  evctx get function

 *  @param  [in] pt_get_cmd         - pointer to the get command
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK           - Success
 */
/*----------------------------------------------------------------------------*/
typedef INT32 (*x_evctx_get_fct) (
    EVCTX_COMMAND_T*    pt_get_cmd);

/*----------------------------------------------------------------------------*/
/*! @brief  date time notify function

 *  @param  [in] h_event            - handle of event object
 *  @param  [in] pv_tag             - notify tag
 *  @param  [in] e_dt_cond          - DT condition
 *  @param  [in] t_delta            - delta time
 *  
 *  @note   Evctx Engine should implement such function pointer and register to evctx
 *  @see
 *  @return		VOID
 *  @retval		VOID
 */
/*----------------------------------------------------------------------------*/
typedef VOID (*x_evctx_dt_nfy_fct) (
    HANDLE_T            h_event,
    VOID*               pv_tag,
    DT_COND_T           e_dt_cond,
    TIME_T              t_delta);

/*------------------------------------------------------------------*/
/*! @struct EVCTX_ENG_FCT_TBL_T
 *  @brief  Structure for the EVCTX engine function table
 
 *  @code
 * typedef struct _EVCTX_ENG_FCT_TBL_T
 * {
 *     x_evctx_create_event_obj_fct        pf_create_event_obj;
 *     x_evctx_delete_event_obj_fct        pf_delete_event_obj;
 *     x_evctx_get_event_obj_info_len_fct  pf_get_event_obj_info_len;
 *     x_evctx_get_event_obj_info_fct      pf_get_event_obj_info;
 *     x_evctx_engine_task_fct             pf_engine_task;
 *     x_evctx_set_fct                     pf_set;
 *     x_evctx_get_fct                     pf_get;
 * } EVCTX_ENG_FCT_TBL_T;
 *  @endcode
 
 *  @li@c   pf_create_event_obj         - evctx create fucntion pointer
 *  @li@c   pf_delete_event_obj         - evctx delete fucntion pointer     
 *  @li@c   pf_get_event_obj_info_len   - evctx get info length fucntion pointer
 *  @li@c   pf_get_event_obj_info       - evctx get info fucntion pointer
 *  @li@c   pf_engine_task              - evctx task fucntion pointer
 *  @li@c   pf_set                      - evctx set fucntion pointer                  
 *  @li@c   pf_get                      - evctx get fucntion pointer                  
 */
/*------------------------------------------------------------------*/
/* Structure for the EVCTX engine function table*/
typedef struct _EVCTX_ENG_FCT_TBL_T
{
    x_evctx_create_event_obj_fct        pf_create_event_obj;
    x_evctx_delete_event_obj_fct        pf_delete_event_obj;
    x_evctx_get_event_obj_info_len_fct  pf_get_event_obj_info_len;
    x_evctx_get_event_obj_info_fct      pf_get_event_obj_info;
    x_evctx_engine_task_fct             pf_engine_task;
    x_evctx_set_fct                     pf_set;
    x_evctx_get_fct                     pf_get;
} EVCTX_ENG_FCT_TBL_T;

/*----------------------------------------------------------------------------*/
/*! @} */                                             /* end of groupMW_EVCTX */
/*----------------------------------------------------------------------------*/


#endif /* _U_EVCTX_ENG_H_ */
