/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_msctx.h 
 *  $RCSfile: $
 *  $Revision: #3 $
 *  $Date: 2016/09/22 $
 *  $Author: xuefei.zhao $
 *  
 *  @par Description:
 *         This header file contains the exported constants and type
 *         definitions of the Power Control Library.
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_MSCTX_H_
#define _U_MSCTX_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
#define MSCTX_OK                            ((INT32)     0)    
#define MSCTX_NO_ACTION                     ((INT32)    -4)    
#define MSCTX_INTERNAL_ERROR                ((INT32)    -5)    
#define MSCTX_INV_ARG                       ((INT32)    -6)    
#define MSCTX_INV_COND                      ((INT32)    -7)    
#define MSCTX_INV_HANDLE                    ((INT32)    -8)    
#define MSCTX_NOT_INIT                      ((INT32)   -10)    
#define MSCTX_NOT_MATCH                     ((INT32)   -11)    
#define MSCTX_OUT_OF_HANDLES                ((INT32)   -13)    
#define MSCTX_OUT_OF_MEM                    ((INT32)   -14)    
#define MSCTX_ERR_LOCK                      ((INT32)   -15)    
#define MSCTX_ERR_UNLOCK                    ((INT32)   -16)    
#define MSCTX_ERR_OUT_OF_RESOURCES          ((INT32)   -23)    
#define MSCTX_NOT_ENOUGH_SPACE              ((INT32)   -24)    
#define MSCTX_NOT_FOUND                     ((INT32)   -25)    
#define MSCTX_NOT_SUPPORT                   ((INT32)   -29)    


typedef enum
{
    MSCTX_COND_AVAIL                                    = 0,
    MSCTX_COND_OPENING,
    MSCTX_COND_OPENED,
    MSCTX_COND_UPDATING,
    MSCTX_COND_UPDATED,
    MSCTX_COND_CLOSED
}   MSCTX_COND_T;


typedef enum
{
    MSCTX_LTS_ID_MODE_UNKNOWN                           = 0,
    MSCTX_LTS_ID_MODE_SVC,
    MSCTX_LTS_ID_MODE_FREQUENCY
}   MSCTX_LTS_ID_MODE_T;


typedef enum
{
    MSCTX_MODE_UNKNOWN                                  = 0,
    MSCTX_MODE_SINGLE,
    MSCTX_MODE_MULTIPLE
}   MSCTX_MODE_T;


typedef enum
{
    MSCTX_REQUESTER_TYPE_UNKNOWN                        = 0,
    MSCTX_REQUESTER_TYPE_CI_14,
    MSCTX_REQUESTER_TYPE_OTHER
}   MSCTX_REQUESTER_TYPE_T;


typedef enum
{
    MSCTX_REQUESTER_EVENT_UNKNOWN                       = 0,
    MSCTX_REQUESTER_EVENT_CAM_MS_CAPA_ANNOUNCE,
    MSCTX_REQUESTER_EVENT_CAM_MS_CAPA_LOST,
    MSCTX_REQUESTER_EVENT_CAM_SWITCH_INPUT2MUX,
    MSCTX_REQUESTER_EVENT_CAM_SWITCH_INPUT2BYPASS,
    MSCTX_REQUESTER_EVENT_LAST_ONE
}   MSCTX_REQUESTER_EVENT;


typedef enum
{
    MSCTX_GET_TYPE_UNKNOWN                              = 0,
    MSCTX_GET_TYPE_LTS_ID,
}   MSCTX_GET_TYPE_T;


typedef struct _MSCTX_CI_MS_CAP_T
{
   UINT8   ui1_max_lts_num;
   UINT16  ui2_max_descrambler_num;
} MSCTX_CI_MS_CAP_T;


#endif /* _U_MSCTX_H_ */


