/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_rrctx_rrt_eng.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains exported definitions and delcarations of
 *         RRT engine of RRCTX.
 */
/*----------------------------------------------------------------------------*/
 
#ifndef _U_RRCTX_RRT_ENG_H_
#define _U_RRCTX_RRT_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_common.h"
 
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_RRCTX_RRT_ENG RRCTX RRT Engine
 *
 *  @ingroup groupMW_RRCTX
 *  @brief The RRCTX RRT engine module is used to get rating information.
 *
 *  %MW Application API would be used by Application, lot of these APIs would
 *   be wrapped from Exported API. Using these APIs, application could get rating infromation
 *   from the exported API of RRCTX RRT engine.
 *
 *  @see groupMW_RRCTX_RRT_ENG
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
/*! @enum	RRCTX_RRT_KEY_TYPE_T
 *  @brief		Key type of specific infomation
 
 *  @code
 *    typedef enum _RRCTX_RRT_KEY_TYPE_T
 *    {
 *        EVCTX_RRT_KEY_TYPE_UNKNOWN = 0,
 *        RRCTX_RRT_KEY_TYPE_RGN_NUM,
 *        RRCTX_RRT_KEY_TYPE_RGN_TEXT,
 *        RRCTX_RRT_KEY_TYPE_RGN_VER,         
 *        RRCTX_RRT_KEY_TYPE_RGN_ID,          
 *        RRCTX_RRT_KEY_TYPE_DIM_NUM,        
 *        RRCTX_RRT_KEY_TYPE_DIM_TEXT,       
 *        RRCTX_RRT_KEY_TYPE_DIM_GRAD,      
 *        RRCTX_RRT_KEY_TYPE_LVL_NUM,         
 *        RRCTX_RRT_KEY_TYPE_LVL_ABBR,       
 *        RRCTX_RRT_KEY_TYPE_LVL_TEXT,        
 *        RRCTX_RRT_KEY_TYPE_CRNT_TUNER,   
 *        RRCTX_RRT_KEY_TYPE_TOTAL_NUMBER 
 *    }RRCTX_RRT_KEY_TYPE_T; 
 *  @endcode
 
 *  @li@c    EVCTX_RRT_KEY_TYPE_UNKNOWN            - Unknown key type
 *  @li@c    RRCTX_RRT_KEY_TYPE_RGN_NUM            - Key type of total region number
 *  @li@c    RRCTX_RRT_KEY_TYPE_RGN_TEXT           - Key type of region text
 *  @li@c    RRCTX_RRT_KEY_TYPE_RGN_VER            - Key type of region version number
 *  @li@c    RRCTX_RRT_KEY_TYPE_RGN_ID             - Key type of region index
 *  @li@c    RRCTX_RRT_KEY_TYPE_DIM_NUM            - Key type of total dimension number
 *  @li@c    RRCTX_RRT_KEY_TYPE_DIM_TEXT           - Key type of dimension text
 *  @li@c    RRCTX_RRT_KEY_TYPE_DIM_GRAD           - Key type of the graduated-scale
 *  @li@c    RRCTX_RRT_KEY_TYPE_LVL_NUM            - Key type of total level number
 *  @li@c    RRCTX_RRT_KEY_TYPE_LVL_ABBR           - Key type of abbreviate level text
 *  @li@c    RRCTX_RRT_KEY_TYPE_LVL_TEXT           - Key type of level text
 *  @li@c    RRCTX_RRT_KEY_TYPE_AVAIL              - Key type of rating available
 *  @li@c    RRCTX_RRT_KEY_TYPE_CRNT_TUNER         - Key type of current tuner
 *  @li@c    RRCTX_RRT_KEY_TYPE_TOTAL_NUMBER       - Total number of key types
 */
/*------------------------------------------------------------------*/

typedef enum _RRCTX_RRT_KEY_TYPE_T
{
    RRCTX_RRT_KEY_TYPE_UNKNOWN = 0,
    RRCTX_RRT_KEY_TYPE_RGN_NUM,         /* UINT8  number  */
    RRCTX_RRT_KEY_TYPE_RGN_TEXT,        /* UTF16  string  */
    RRCTX_RRT_KEY_TYPE_RGN_VER,         /* UINT8  number  */
    RRCTX_RRT_KEY_TYPE_RGN_ID,          /* UINT8  number  */
    RRCTX_RRT_KEY_TYPE_DIM_NUM,         /* UINT8  number  */
    RRCTX_RRT_KEY_TYPE_DIM_TEXT,        /* UTF16  string  */
    RRCTX_RRT_KEY_TYPE_DIM_GRAD,        /* BOOLEAN        */
    RRCTX_RRT_KEY_TYPE_LVL_NUM,         /* UINT8  number  */
    RRCTX_RRT_KEY_TYPE_LVL_ABBR,        /* UTF16  string  */
    RRCTX_RRT_KEY_TYPE_LVL_TEXT,        /* UTF16  string  */
    RRCTX_RRT_KEY_TYPE_AVAIL,           /* BOOLEAN        */
    RRCTX_RRT_KEY_TYPE_CRNT_TUNER,      /* UTF8   string  */
    RRCTX_RRT_KEY_TYPE_TOTAL_NUMBER     /* total number of key types   */
} RRCTX_RRT_KEY_TYPE_T;


/*------------------------------------------------------------------*/
/*! @struct	RRCTX_RRT_KEY_PARAM_T
 *  @brief		Key parameters structure
 *
 *  @code
 *    typedef struct _RRCTX_RRT_KEY_PARAM_T
 *    {
 *        RRCTX_RRT_KEY_TYPE_T          e_rrt_key_type;
 *        VOID*                         pv_rrt_key_param;
 *    } RRCTX_RRT_KEY_PARAM_T;  
 *  @endcode
 *
 *  @li@c    e_rrt_key_type            -Key type of the rrctx
 *  @li@c    pv_rrt_key_param          -Parameters of key type
 */
/*------------------------------------------------------------------*/

typedef struct _RRCTX_RRT_KEY_PARAM_T
{
    RRCTX_RRT_KEY_TYPE_T   e_rrt_key_type;
    VOID*                  pv_rrt_key_param;
} RRCTX_RRT_KEY_PARAM_T;


#endif /* _U_RRCTX_RRT_ENG_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                               /* end of groupMW_RRCTX_RRT_ENG            */
/*----------------------------------------------------------------------------*/

