/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_sb_eng.h 
 *  $RCSfile: $
 *  $Revision: #2 $
 *  $Date: 2015/06/01 $
 *  $Author: cheng.liu $
 *  
 *  @par Description:
 *         This header file contains SVL Builder specific definitions,
 *         which are exported for customized engines.
 */
/*----------------------------------------------------------------------------*/
#ifndef _U_SB_ENG_H_
#define _U_SB_ENG_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_SVL_BLDR
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_sb.h"
#include "u_lnk_list.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/*
    (*x_sb_eng_scan_nfy_fct)() is the notify function, called by builder
    engine to notify SB Selector of the scan status (see the pf_nfy argument
    of the API sb_start_scan_fct() below).
    
    Based on this notification, SB Selector then notifies the applicaton of 
    the same status info by calling x_sb_scan_nfy_fct().
*/
/*------------------------------------------------------------------*/
/*! @brief (*x_sb_eng_scan_nfy_fct)() is the notify function, called by builder
 *  engine to notify SB Selector of the scan status (see the pf_nfy argument
 *  of the API sb_start_scan_fct() below).
 *  
 *  
 *  @param [in] h_builder_base      -Builder base handler
 *  @param [in] ui4_nfy_reason      -Notify reason
 *  @param [in] i4_nfy_data         -Notify data
 *  @param [in] pv_nfy_tag          -Notify tag
 *  @note Based on this notification, SB Selector then notifies the applicaton of the same status info by calling x_sb_scan_nfy_fct(). 
 *  @return      VOID
 *  @retval      -
 */
/*------------------------------------------------------------------*/
typedef VOID (*x_sb_eng_scan_nfy_fct) (
      HANDLE_T          h_builder_base,
      UINT32            ui4_nfy_reason,
      INT32             i4_nfy_data,
      VOID*             pv_nfy_tag);

/*
    Followings are function types in the function table 
    structure SB_ENG_FCT_TBL_T.
 */
/*------------------------------------------------------------------*/
/*! @brief function types in the function table,(*sb_create_builder_data_fct) type function.
 *  @param [in] h_builder_base      -The handle of builder base object.
 *  @param [in] pv_open_data        -Contains the engine open data type data.
 *  @param [out]ppv_builder_data    -Contains the engine builder data type data.
 *  @param [out]pe_builder_cond     -Contains the current builder state.
 *  @return Return the error code.
 *  @retval 
 *          SBR_OK              -Success.
 *          SBR_FILE_NOT_FOUND  -Fail. The ps_file_name was not found. 
 *          SBR_INTERNAL_ERROR  -Fail. Internal error happens.         
 *          SBR_INV_ARG         -Fail. Invalid arguments.              
 *          SBR_LIST_NOT_FOUND  -Fail. The named SVL doesn't exist.    
 *          SBR_OUT_OF_MEM      -Fail. Run out of memory.              
 */               
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_create_builder_data_fct) (
     HANDLE_T           h_builder,
     VOID*              pv_open_data,
     VOID**             ppv_builder_data,
     SB_COND_T*         pe_builder_cond);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_delete_builder_data_fct) type function.
 *  @param [in] pv_builder_data    -Contains the builder engine specific data.
 *  @return Return the error code.
 *  @retval SBR_OK              -Success.
 *          SBR_INTERNAL_ERROR  -Fail. Internal error happens.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_delete_builder_data_fct) (
     VOID*              pv_builder_data);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_match_builder_data_fct) type function.
 *  @param [in] pv_open_data        -The first operand to matched.
 *  @param [in] pv_builder_data     -Contains the builder engine specific data, in which contains the 2nd operand to be matched.
 *                              
 *  @return Return the error code.
 *  @retval         SBR_OK              -The SVL id matches.
 *                  SBR_INV_ARG         -Fail. Invalid arguments.
 *                  SBR_NOT_MATCH       -Fail. the SVL id in the pv_open_data doesn't match the SVL id in the pv_builder_data.
 *                                       
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_match_builder_data_fct) (
     const VOID*        pv_open_data,
     const VOID*        pv_builder_data);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_engine_task_fct) type function.
 *  @return
 *  @retval      SBR_OK              -Success.
 *               SBR_INTERNAL_ERROR  -Fail. Internal error happens.
 *               SBR_INV_COND        -Fail. Invalid builder condition to perform the request.
 *                                    
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_engine_task_fct) (VOID);

/*------------------------------------------------------------------*/
/*! @brief (*sb_start_scan_fct)() type function
 *  @param [in] h_builder_base  -The handle of builder base object.
 *  @param [in] pv_scan_data    -Contains the SB_ATSC_SCAN_DATA_T type data.
 *  @param [in] pv_nfy_tag      -Contains the caller private tag.
 *  @param [in] pf_nfy          -The notification function, which will be called during the scan. 
 *                          
 *  @return
 *  @retval     SBR_OK         -Success.
 *              SBR_INTERNAL_ERROR  -Fail. Internal error happens.
 *              SBR_INV_ARG         -Fail. Invalid arguments.
 *              SBR_INV_HANDLE      -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_start_scan_fct) (
     HANDLE_T               h_builder,
     VOID*                  pv_scan_data,
     VOID*                  pv_nfy_tag,
     x_sb_eng_scan_nfy_fct  pf_nfy);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_cancel_scan_fct)() type function.
 *  @param [in] h_builder_base  -The handle of builder base object.
 * 
 *  @return
 *  @retval      SBR_OK              -Success.
 *               SBR_INTERNAL_ERROR  -Fail. Internal error happens.
 *               SBR_INV_ARG         -Fail. Invalid arguments.
 *               SBR_INV_HANDLE      -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_cancel_scan_fct) (
     HANDLE_T           h_builder);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_save_lists_fct)() type function.
 *  @param [in] h_builder_base  -The handle of builder base object.
 *  @param [in] ps_file_name    -The file in which the lists are stored.
 *  
 *  @return
 *  @retval SBR_OK                   -Success.
 *          SBR_ERR_SAVE_LIST        -Fail. An error happened when saving SVL or TSL.
 *          SBR_INV_ARG              -Fail. Invalid arguments.
 *          SBR_INV_HANDLE           -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_save_lists_fct) (
     HANDLE_T           h_builder,
     const CHAR*        ps_file_name);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_load_lists_fct)() type function.
 *  @param [in] h_builder_base  -The handle of builder base object.
 *  @param [in  ps_file_name    -The file from which the lists are loaded.]
 *  
 *  @return
 *  @retval SBR_OK              -Success.
 *          SBR_ERR_LOAD_LIST   -Fail. An error happened when loading SVL or TSL. 
 *          SBR_FILE_NOT_FOUND  -Fail. The specified ps_file_name was not found.
 *          SBR_INV_ARG         -Fail. Invalid arguments.
 *          SBR_INV_HANDLE      -Fail. Invalid builder handle.
 *          SBR_LIST_NOT_FOUND  -FAIL. The SVL doesn't exist in ps_file_name.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_load_lists_fct) (
     HANDLE_T           h_builder,
     const CHAR*        ps_file_name);

/*------------------------------------------------------------------*/
/*! @brief the (*sb_clean_lists_fct)() type function.
 *  @param [in] h_builder_base  -The handle of builder base object.
 *  @param [in] ps_file_name    -The file in which the lists are stored.
 *  
 *  @return
 *  @retval SBR_OK              -Success.
 *          SBR_ERR_CLEAN_LIST  -Fail. An error happened when cleaning SVL or TSL.
 *          SBR_FILE_NOT_FOUND  -Fail. The specified ps_file_name was not found.
 *          SBR_INV_ARG         -Fail. Invalid arguments.
 *          SBR_INV_HANDLE      -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_clean_lists_fct) (
     HANDLE_T           h_builder,
     const CHAR*        ps_file_name);

/*------------------------------------------------------------------*/
/*! @brief  the (*x_sb_reg_scan_map_fct)() type function.
 *  @param [in] pv_nfy_fct      The scan map function.
 *  @param [in] pt_nfy_tag      The notify data.
 *  
 *  @return
 *  @retval SBR_OK                -Success.
 *          SBR_INV_ARG           -Operation fail.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_reg_scan_map_fct) (
     VOID*              pv_nfy_fct,
     VOID*              pt_nfy_tag);

/*------------------------------------------------------------------*/
/*! @brief the (*x_sb_get_fct)() type function.
 *  @param [in] h_builder_base  The handle of builder base object.
 *  @param [in] e_key_type      The requested type to get.
 *  @param [in] pv_key_info     The auxiliary info for the requested get type.
 *  @param [out]pz_key_data_len Contains the real length of retrieved key data if the API successfully returns.
 *  @param [out]pv_key_data     Contains the requested get data.
 *  
 *  @return
 *  @retval SBR_OK                -Success.
 *          SBR_INV_KEY_TYPE      -Invalid get operation.
 *          SBR_INV_KEY_INFO      -Invalid get operation data.
 *          SBR_INV_HANDLE        -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_get_fct) (
     HANDLE_T           h_builder,
     SB_KEY_TYPE_T      e_key_type,
     VOID*              pv_key_info,
     SIZE_T*            pz_key_data_len,
     VOID*              pv_key_data);

/*------------------------------------------------------------------*/
/*! @brief  the (*x_sb_set_fct)() type function.
 *  @param [in] h_builder_base  The handle of builder base object.
 *  @param [in] e_key_type      The requested type to set.
 *  @param [in] pv_key_info     The auxiliary info for the requested set type.
 *  @param [in] z_key_data_len  Contains length of the buffer to which pv_key_data points.
 *  @param [in] pv_key_data     Contains the requested set data.
 *  
 *  @return
 *  @retval SBR_OK                -Success.
 *          SBR_INV_KEY_TYPE      -Invalid set operation.
 *          SBR_INV_KEY_INFO      -Invalid set operation data.
 *          SBR_INV_HANDLE        -Fail. Invalid builder handle.
 */
/*------------------------------------------------------------------*/
typedef INT32 (*x_sb_set_fct) (
     HANDLE_T           h_builder,
     SB_KEY_TYPE_T      e_key_type,
     VOID*              pv_key_info,
     SIZE_T             z_key_data_len,
     VOID*              pv_key_data);

/*------------------------------------------------------------------*/
/*! @struct SB_ENG_FCT_TBL_T
 *  @brief Internal use
 *  @code
 *  typedef struct _SB_ENG_FCT_TBL_T
 *  {
 *      x_sb_create_builder_data_fct    pf_create_builder_data;
 *      x_sb_delete_builder_data_fct    pf_delete_builder_data;
 *      x_sb_match_builder_data_fct     pf_match_builder_data;
 *      x_sb_engine_task_fct            pf_engine_task;
 *      x_sb_start_scan_fct             pf_start_scan;
 *      x_sb_cancel_scan_fct            pf_cancel_scan;
 *      x_sb_save_lists_fct             pf_save_lists;
 *      x_sb_load_lists_fct             pf_load_lists;
 *      x_sb_clean_lists_fct            pf_clean_lists;
 *      x_sb_get_fct                    pf_get;
 *      x_sb_set_fct                    pf_set;
 *      x_sb_cancel_scan_fct            pf_cancel_scan_sync;
 *  } SB_ENG_FCT_TBL_T;
 *  @endcode
 *  @li@c  pf_create_builder_data                            - Create SB engine handler
 *  @li@c  pf_delete_builder_data                            - Delete SB engine handler
 *  @li@c  pf_match_builder_data                             - Check the SB engine handler is
 *  @li@c  pf_engine_task                                    - The core state machine of engine task
 *  @li@c  pf_start_scan                                     - Start scan function of each engine implementation
 *  @li@c  pf_cancel_scan                                    - Cancel scan function of each engine implementation
 *  @li@c  pf_save_lists                                     - Save channel list function of each engine implementation
 *  @li@c  pf_load_lists                                     - Load channel list function of each engine implementation
 *  @li@c  pf_clean_lists                                    - Clean channel list function of each engine implemnetation
 *  @li@c  pf_get                                            - get function of each engine implementation
 *  @li@c  pf_set                                            - set function of each engine implementation
 *  @li@c  pf_cancel_scan_sync                               - Synchronism cancel scan function of each engine implementation
 */
/*------------------------------------------------------------------*/
typedef struct _SB_ENG_FCT_TBL_T
{
    x_sb_create_builder_data_fct    pf_create_builder_data;
    x_sb_delete_builder_data_fct    pf_delete_builder_data;
    x_sb_match_builder_data_fct     pf_match_builder_data;
    x_sb_engine_task_fct            pf_engine_task;
    x_sb_start_scan_fct             pf_start_scan;
    x_sb_cancel_scan_fct            pf_cancel_scan;
    x_sb_save_lists_fct             pf_save_lists;
    x_sb_load_lists_fct             pf_load_lists;
    x_sb_clean_lists_fct            pf_clean_lists;
    x_sb_get_fct                    pf_get;
    x_sb_set_fct                    pf_set;
    /* cliu 20090706, sync cancel scan function pointer */    
    x_sb_cancel_scan_fct            pf_cancel_scan_sync;
    x_sb_reg_scan_map_fct           pf_reg_scan_map;

} SB_ENG_FCT_TBL_T;

/*
    SB_ENG_CTRL_T is the structure of nodes in the engine list.
 */ 
typedef struct _SB_ENG_CTRL_T
{
    DLIST_ENTRY_T(_SB_ENG_CTRL_T)   t_link;
    SB_ENG_FCT_TBL_T                t_sb_fct_tbl;
    CHAR                            s_name[SB_MAX_ENG_NAME_LEN+1];
} SB_ENG_CTRL_T;

typedef DLIST_T(_SB_ENG_CTRL_T)     SB_ENG_LIST_T;

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_SVL_BLDR */
/*----------------------------------------------------------------------------*/

#endif /* _U_SB_ENG_H_ */

