/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_sb_fmrdo_eng.h
 *  $Revision: #1 $
 *  $Date: 2015/06/01 $
 *  $Author: cheng.liu $
 *
 *  @par Description:
 *       This header file contains exported definitions and delcarations of
 *       SVL Builder FM radio engine.
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_SVL_BLDR_SB_FMRDO_ENG FM Radio Engine
 *  @ingroup  groupMW_SVL_BLDR
 *  @brief    Provides a unified interface for service list building of FM radio
 *  @see      groupMW_SVL_BLDR
 *
 *  @{
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_SB_FMRDO_ENG_H_
#define _U_SB_FMRDO_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_common.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

#define SB_FMRDO_ENG_NAME    "sb_fmrdo"    /**< Default name of SVL builder FM radio engine */

/*---------------------------------------------------------------------------*/
/*! @struct SB_FMRDO_OPEN_DATA_T
 *  @brief  This structure is used as the argument pv_open_data of the API
 *          c_sb_open_builder() / x_sb_open_builder() when opening an SVL
 *          Builder with FM radio engine.
 *
 *  @code
 *  typedef struct _SB_FMRDO_OPEN_DATA_T
 *  {
 *      UINT16    ui2_svl_id;
 *      CHAR*     ps_svl_name;
 *      CHAR*     ps_file_name;
 *      BOOL      b_use_orig;
 *      UINT32    ui4_first_freq;
 *      UINT32    ui4_last_freq;
 *  } SB_FMRDO_OPEN_DATA_T;
 *  @endcode
 *
 *  @li@c   ui2_svl_id   - The ID of SVL to be loaded
 *  @li@c   ps_svl_name  - The name of SVL to be loaded
 *  @li@c   ps_file_name - The file name from which the SVL will be loaded
 *  @li@c   b_use_orig   - A flag, When set to true, use the existed SVL instead
 *                         of reloading it. When set to false, overwrite the
 *                         SVL (if exists) by reloading.
 *  @li@c   ui4_first_freq - The variable to specifiy range of FM Radio frequency
 *                           (lower bound), ex.  87500000 Hz
 *  @li@c   ui4_last_freq  - The variable to specifiy range of FM Radio frequency
 *                           (upper bound), ex. 108000000 Hz
 */
/*---------------------------------------------------------------------------*/

typedef struct _SB_FMRDO_OPEN_DATA_T
{
    UINT16      ui2_svl_id;
    CHAR*       ps_svl_name;
    CHAR*       ps_file_name;
    BOOL        b_use_orig;
    UINT32      ui4_first_freq;
    UINT32      ui4_last_freq;
} SB_FMRDO_OPEN_DATA_T;

/*---------------------------------------------------------------------------*/
/*! @enum  SB_FMRDO_SCAN_TYPE_T
 *  @brief This structure enumerates all scan types of SVL builder FM radio
 *         engine
 *
 *  @code
 *  typedef enum
 *  {
 *      SB_FMRDO_SCAN_TYPE_UNKNOWN = 0,
 *      SB_FMRDO_SCAN_TYPE_FULL_MODE,
 *      SB_FMRDO_SCAN_TYPE_ONCE_UP,
 *      SB_FMRDO_SCAN_TYPE_ONCE_DOWN,
 *      SB_FMRDO_SCAN_TYPE_NUM,
 *  }   SB_FMRDO_SCAN_TYPE_T;
 *  @endcode
 *
 *  @li@c  SB_FMRDO_SCAN_TYPE_UNKNOWN   - Unknow
 *  @li@c  SB_FMRDO_SCAN_TYPE_FULL_MODE - Full mode
 *  @li@c  SB_FMRDO_SCAN_TYPE_ONCE_UP   - Scan up
 *  @li@c  SB_FMRDO_SCAN_TYPE_ONCE_DOWN - Scan down
 *  @li@c  SB_FMRDO_SCAN_TYPE_NUM       - Number of scan types
 */
/*---------------------------------------------------------------------------*/

typedef enum
{
    SB_FMRDO_SCAN_TYPE_UNKNOWN = 0,
    SB_FMRDO_SCAN_TYPE_FULL_MODE,
    SB_FMRDO_SCAN_TYPE_ONCE_UP,
    SB_FMRDO_SCAN_TYPE_ONCE_DOWN,
    SB_FMRDO_SCAN_TYPE_NUM    /*  */
} SB_FMRDO_SCAN_TYPE_T;

/*---------------------------------------------------------------------------*/
/*! @struct SB_FMRDO_SCAN_DATA_T
 *  @brief  This structure is used as the argument pv_scan_data of the API
 *          c_sb_start_scan() / x_sb_start_scan() when scaning a media.
 *  @code
 *  typedef struct _SB_FMRDO_SCAN_DATA_T
 *  {
 *      SB_FMRDO_SCAN_TYPE_T    e_scan_type;
 *      UINT32                  ui4_start_freq;
 *  } SB_FMRDO_SCAN_DATA_T;
 *  @endcode
 *
 *  @li@c   e_scan_type    - Scan type
 *  @li@c   ui4_start_freq - Start frequency of scanning
 */
/*---------------------------------------------------------------------------*/

typedef struct _SB_FMRDO_SCAN_DATA_T
{
    SB_FMRDO_SCAN_TYPE_T    e_scan_type;
    UINT32                  ui4_start_freq;
} SB_FMRDO_SCAN_DATA_T;

/*! @} */

#endif /* _U_SB_FMRDO_ENG_H_ */

