/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

 
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_MM_SBTL_HDLR
*  @{ 
*/
/*----------------------------------------------------------------------------*/
#ifndef _U_SBTL_IPCC_H_
#define _U_SBTL_IPCC_H_

#ifndef __KERNEL__

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
//#include "x_lnk_list.h"
//#include "x_handle.h"
//#include "x_sm.h"
//#include "u_wgl_common.h"
//#include "u_wgl_cc.h"
//#include "u_common.h"
#include "x_lnk_list.h"
#include "x_handle.h"
#include "x_sm.h"
#include "u_wgl_common.h"
#include "u_wgl_atsc3_cc.h"
#include "u_common.h"

/*-----------------------------------------------------------------------------
                    macros, defines, enums
-----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @enum   SBTL_IPCC_CAPTION_MODE
 *  @brief  SBTL_IPCC_CAPTION_MODE
 *  @code
 *  typedef enum _SBTL_IPCC_CAPTION_MODE
 *  {
 *      SBTL_IPCC_CAPTION_MODE_POP_ON = 0,       
 *      SBTL_IPCC_CAPTION_MODE_PAINT_ON,
 *      SBTL_IPCC_CAPTION_MODE_ROLL_UP
 *  } SBTL_IPCC_CAPTION_MODE;
 *  @endcode
 *   
 *  @li@c SBTL_IPCC_CAPTION_MODE_POP_ON                 - Pop-on
 *  @li@c SBTL_IPCC_CAPTION_MODE_PAINT_ON               - Paint-on
 *  @li@SBTL_IPCC_CAPTION_MODE_ROLL_UP                  - Roll-up
 */
/*----------------------------------------------------------------------------*/
typedef enum _SBTL_IPCC_CAPTION_MODE
{
    SBTL_IPCC_CAPTION_MODE_POP_ON = 0,
    SBTL_IPCC_CAPTION_MODE_PAINT_ON,
    SBTL_IPCC_CAPTION_MODE_ROLL_UP
} SBTL_IPCC_CAPTION_MODE;


/*----------------------------------------------------------------------------*/
/*! @enum   TTML_LENGTH_TYPE
 *  @brief  TTML_LENGTH_TYPE
 *  @code
 *  typedef enum _TTML_LENGTH_TYPE
 *  {
 *      TTML_LENGTH_TYPE_SCALAR = 0,       
 *      TTML_LENGTH_TYPE_PERCENTAGE              
 *  } TTML_LENGTH_TYPE;
 *  @endcode
 *   
 *  @li@c TTML_LENGTH_TYPE_SCALAR                   - Number units
 *  @li@c TTML_LENGTH_TYPE_PERCENTAGE               - Number "%"
 */
/*----------------------------------------------------------------------------*/
typedef enum _TTML_LENGTH_TYPE
{
    TTML_LENGTH_TYPE_SCALAR   = 0,
    TTML_LENGTH_TYPE_PERCENTAGE
}TTML_LENGTH_TYPE;


/*------------------------------------------------------------------*/
/*! @struct TTML_EXTENT_T
 *  @brief This struct contains the tts:extent.
 *  @code
 *  typedef struct _TTML_EXTENT_T
 *  {
 *      UINT8 ui1_column;
 *      UINT8 ui1_row;
 *  }TTML_EXTENT_T;
 *  @endcode
 *  @li@c  ui1_column                              -column
 *  @li@c  ui1_row                                 -row
 */
/*------------------------------------------------------------------*/
typedef struct _TTML_EXTENT_T
{
    UINT8   ui1_column;
    UINT8   ui1_row;
}TTML_EXTENT_T;


/*------------------------------------------------------------------*/
/*! @struct SBTL_IPCC_CONFIG_T
 *  @brief This struct contains the size of screen.
 *  @code
 *  typedef struct _SBTL_IPCC_CONFIG_T
 *  {
 *        UINT32  ui4_screen_width;
 *        UINT32  ui4_screen_height;
 *  }SBTL_IPCC_CONFIG_T;
 *  @endcode
 *  @li@c  ui4_screen_width             -The width of screen width
 *  @li@c  ui4_screen_height            -The width of screen height

 */
/*------------------------------------------------------------------*/
typedef struct _SBTL_IPCC_CONFIG_T
{
    UINT32  ui4_screen_width;
    UINT32  ui4_screen_height;
}SBTL_IPCC_CONFIG_T;


/*----------------------------------------------------------------------------*/
/*! @enum   SBTL_IPCC_VIDEO_TYPE
 *  @brief  SBTL_IPCC_VIDEO_TYPE
 *  @code
 *  typedef enum _SBTL_IPCC_VIDEO_TYPE
 *  {
 *      SBTL_IPCC_VIDEO_TYPE_HD = 0,       
 *      SBTL_IPCC_VIDEO_TYPE_SD
 *      
 *  } SBTL_IPCC_VIDEO_TYPE;
 *  @endcode
 *   
 *  @li@c SBTL_IPCC_VIDEO_TYPE_HD                             - HD
 *  @li@c SBTL_IPCC_VIDEO_TYPE_SD                             - SD
 */
/*----------------------------------------------------------------------------*/
typedef enum _SBTL_IPCC_VIDEO_TYPE
{
    SBTL_IPCC_VIDEO_TYPE_HD = 0,
    SBTL_IPCC_VIDEO_TYPE_SD
}SBTL_IPCC_VIDEO_TYPE;


/*----------------------------------------------------------------------------*/
/*! @enum   SBTL_IPCC_FILE_TYPE
 *  @brief  SBTL_IPCC_FILE_TYPE
 *  @code
 *  typedef enum _SBTL_IPCC_FILE_TYPE
 *  {
 *      SBTL_IPCC_FILE_TYPE_SENVU = 0,       
 *      SBTL_IPCC_FILE_TYPE_CRACKLE
 *      
 *  } SBTL_IPCC_FILE_TYPE;
 *  @endcode
 *   
 *  @li@c SBTL_IPCC_FILE_TYPE_SENVU                 - m4t type
 *  @li@c SBTL_IPCC_FILE_TYPE_CRACKLE               - Other type, except m4t
 */
/*----------------------------------------------------------------------------*/
typedef enum _SBTL_IPCC_FILE_TYPE
{
    SBTL_IPCC_FILE_TYPE_SENVU = 0,
    SBTL_IPCC_FILE_TYPE_CRACKLE
}SBTL_IPCC_FILE_TYPE;


/*----------------------------------------------------------------------------*/
/*! @enum   SBTL_IPCC_FILE_MODE
 *  @brief  SBTL_IPCC_FILE_MODE
 *  @code
 *  typedef enum _SBTL_IPCC_FILE_MODE
 *  {
 *      SBTL_IPCC_FILE_MODE_PRESERVED = 0,       
 *      SBTL_IPCC_FILE_MODE_ENHANCED
 *  } SBTL_IPCC_FILE_MODE;
 *  @endcode
 *   
 *  @li@c SBTL_IPCC_FILE_MODE_PRESERVED              - Preserved, in xml/m4t file, search {mode="Preserved"}
 *  @li@c SBTL_IPCC_FILE_MODE_ENHANCED               - Enhanced, in xml/m4t file, search {mode="Enhanced"}
 */
/*----------------------------------------------------------------------------*/
typedef enum _SBTL_IPCC_FILE_MODE
{
    SBTL_IPCC_FILE_MODE_PRESERVED = 0,
    SBTL_IPCC_FILE_MODE_ENHANCED
}SBTL_IPCC_FILE_MODE;



/*----------------------------------------------------------------------------*/
/*! @struct SBTL_IPCC_VIDEO_T
 *  @brief  This struct contains video info
 *  @code
 *  typedef struct _SBTL_IPCC_VIDEO_T
 *  {
 *      SBTL_IPCC_VIDEO_TYPE    e_video_type;
 *      UINT16                  ui2_video_width;
 *      UINT16                  ui2_video_height;
 *      SBTL_IPCC_FILE_TYPE     e_file_type;
 *      SBTL_IPCC_FILE_MODE     e_file_mode; 
 *  } SBTL_IPCC_VIDEO_T;
 *  @endcode
 *   
 *  @li@c e_video_type        - Video type
 *  @li@c ui2_video_width     - Video width
 *  @li@c ui2_video_height    - Video height
 *  @li@c e_file_type         - File type
 *  @li@c e_file_mode         - File mode
 */
/*----------------------------------------------------------------------------*/
typedef struct _SBTL_IPCC_VIDEO_T
{
    SBTL_IPCC_VIDEO_TYPE    e_video_type;
    UINT16                  ui2_video_width;
    UINT16                  ui2_video_height;
    SBTL_IPCC_FILE_TYPE     e_file_type;
    SBTL_IPCC_FILE_MODE     e_file_mode;
}SBTL_IPCC_VIDEO_T;


/*----------------------------------------------------------------------------*/
/*! @struct SBTL_IPCC_OPEN_CFG_T
 *  @brief  This struct contains configuration args
 *  @code
 *  typedef struct _SBTL_IPCC_OPEN_CFG_T
 *  {
 *      CHAR*               ps_extent;       
 *      CHAR*               ps_text;
 *      UINT8               ui1_row_count;
 *      SBTL_IPCC_VIDEO_T   t_ipcc_video; 
 *      
 *  } SBTL_IPCC_OPEN_CFG_T;
 *  @endcode
 *   
 *  @li@c ps_extent            - tts:extent
 *  @li@c ps_text              - Text
 *  @li@c ui1_row_count        - Row count of text
 *  @li@c t_ipcc_video         - Video args
 */
/*----------------------------------------------------------------------------*/
typedef struct _SBTL_IPCC_OPEN_CFG_T
{
    CHAR*               ps_extent;
    CHAR*               ps_aspect_ratio;
    UINT8               ui1_row_count;
    SBTL_IPCC_VIDEO_T   t_ipcc_video;   
        
}SBTL_IPCC_OPEN_CFG_T;



#endif /* __KERNEL__ */
#endif/* _U_SBTL_IPCC_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_STRM_MNGR_MM_SBTL_HDLR*/
/*----------------------------------------------------------------------------*/

