/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_sm_cc_isdb_hdlr.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains ISDB closed caption Stream handler specific 
 *         definitions, which are exported to applications.
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_SM_CC_ISDB_HDLR_H_
#define _U_SM_CC_ISDB_HDLR_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "u_common.h"

/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_STRM_MNGR_CCI_HDLR ISDB Closed Caption Handler
  *  @ingroup groupMW_STRM_MNGR
  *  @brief Provides external function TV Application
  *  %MW Application API would be used by TV Application, lot of these APIs would
  *   be wrapped from Exported API
  *  @see groupMW_STRM_MNGR_CCI_HDLR
  *  @{ 
  */
/*----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
#define SM_CCI_CMD_CODE_SET_OSD_HANDLE      ((SM_CMD_CODE_T)    1)    /**< set OSD canvas handle */
#define SM_CCI_CMD_CODE_SET_LANG_ID         ((SM_CMD_CODE_T)    2)    /**< set language data */
#define SM_CCI_CMD_CODE_SET_PES_DIRECT_MODE ((SM_CMD_CODE_T)    3)
#define SM_CCI_CMD_CODE_SET_PES_CB_FUNC     ((SM_CMD_CODE_T)    4)
#define SM_CCI_CMD_CODE_SET_PES_CB_TAG      ((SM_CMD_CODE_T)    5)


/* set/get type groups *******************************************************/
#define SM_GRP_CCI                          (( SM_GRP_LAST_ENTRY + 13 ) << SM_CODE_BITS)    /**< get/set command group id */
    
/* set types *****************************************************************/
#define SM_CCI_SET_TYPE_IGNORE          ((SM_SET_TYPE_T)(SM_GRP_CCI | 0))    /**< setting type is IGNORE */
    
/* get types *****************************************************************/
#define SM_CCI_GET_TYPE_IGNORE          ((SM_GET_TYPE_T)(SM_GRP_CCI | 0))    /**< getting type is IGNORE */


/*------------------------------------------------------------------*/
/*! @brief     Indicates the ISDB closed caption justify type.
  *  @code 
  *  typedef enum
  *  {
  *      SM_CCI_ATTRIBS_UNKNOWN           = 0,
  *      SM_CCI_ATTRIBS_EXTERN_SIZE,
  *      SM_CCI_ATTRIBS_EQUAL_WIDTH,
  *      SM_CCI_ATTRIBS_AUTO_LINE_FEED,
  *      SM_CCI_ATTRIBS_ROLL_UP_MODE
  *  }   SM_CCI_ATTRIBS_TYPE_T;
  *  @endcode 
  *  @li@c  SM_CCI_ATTRIBS_UNKNOWN          -The justify type is invalid.
  *  @li@c  SM_CCI_ATTRIBS_EXTERN_SIZE      -The justify type is switch of EXTERN_SIZE.
  *  @li@c  SM_CCI_ATTRIBS_EQUAL_WIDTH      -The justify type is switch of EQUAL_WIDTH.
  *  @li@c  SM_CCI_ATTRIBS_AUTO_LINE_FEED   -The justify type is switch of AUTO_LINE_FEED.
  *  @li@c  SM_CCI_ATTRIBS_ROLL_UP_MODE     -The justify type is switch of ROLL_UP_MODE.
  */  
/*------------------------------------------------------------------*/
typedef enum
{
    SM_CCI_ATTRIBS_UNKNOWN           = 0,
    SM_CCI_ATTRIBS_EXTERN_SIZE,
    SM_CCI_ATTRIBS_EQUAL_WIDTH,
    SM_CCI_ATTRIBS_AUTO_LINE_FEED,
    SM_CCI_ATTRIBS_ROLL_UP_MODE
}   SM_CCI_ATTRIBS_TYPE_T;


/*------------------------------------------------------------------*/
/*! @brief the structure of ISDB closed caption attribute information.
  *  @code 
  *  typedef struct  _SM_CCI_ATTRIBS_T
  *  {
  *      SM_CCI_ATTRIBS_TYPE_T       t_cci_attrib_type;
  *      BOOL                        fg_enable_flag;
  *  }   SM_CCI_ATTRIBS_T;
  *  @endcode 
  *  @li@c  t_cci_attrib_type           -Contains the attribute type.
  *  @li@c  fg_enable_flag              -Contains the attribute flag to indicates it is valid or not.
  */  
/*------------------------------------------------------------------*/
typedef struct  _SM_CCI_ATTRIBS_T
{
    SM_CCI_ATTRIBS_TYPE_T       t_cci_attrib_type;
    BOOL                        fg_enable_flag;
}   SM_CCI_ATTRIBS_T;


/*-----------------------------------------------------------------------------
                    data declarations, extern, static, const
----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/


#endif /* _U_SM_CC_ISDB_HDLR_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                /* end of groupMW_STRM_MNGR_CCI_HDLR */
/*----------------------------------------------------------------------------*/


