/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*! @file u_sort_search.h
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * @par Description: 
 *         This header file contains shared typedef and return codes for
 *         sorting and searching function API.
 *         
 *---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_UTIL
  *  @{  
  */
/*----------------------------------------------------------------------------*/ 

#ifndef _U_SORT_SEARCH_H_
#define _U_SORT_SEARCH_H_

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/

#include "u_common.h"

/*--------------------------------------------------------------------------
 Constants, enumerations, and macros
---------------------------------------------------------------------------*/
    
#define SSR_OK           ((INT32)   0)      /**<Routine successful*/
#define SSR_INV_ARG      ((INT32)  -1)      /**<One or more of the arguments contain invalid data.*/
#define SSR_OUT_OF_MEM   ((INT32)  -2)      /**<There is no enough memory*/
#define SSR_NOT_FOUND    ((INT32)  -3)      /**<The specified record can not be found.*/
#define SSR_FAIL         ((INT32)  -4)      /**<Routine fail*/


/*
  return code for the sorting comparison function and for searching
  comparison function.
*/
#define  RC_CMP_FCT_T  INT32                /**<Definition of the type of return code for the sorting comparision function*/

#define   RC_SMALLER  ((INT32)-1)           /**< return code of smaller*/
#define   RC_EQUAL    ((INT32)0)            /**<return code of equal*/
#define   RC_GREATER  ((INT32)1)            /**<return code of greater*/
#define   RC_ERROR    ((INT32)-100)         /**<return code of error*/

/*---------------------------------------------------------------------------
 Type definition
----------------------------------------------------------------------------*/

/* 
  Typedef for the sort compare function. This function is passed
  to the x_qsort function.
   
*/
/*------------------------------------------------------------------*/
/*! @brief  This type defines for the sort compare function. This function is passed to the x_qsort function.
*  @param [in] pv_elem_a        Indicates the first element to be compared.
*  @param [in] pv_elem_b        Indicates the second element to be compared.
*  @param [in] pv_opt_param   Reference the optional parameter.
*/
/*------------------------------------------------------------------*/
typedef INT32 (*x_sort_cmp_fct)
(
    const VOID*  pv_elem_a,
    const VOID*  pv_elem_b,
    VOID*        pv_opt_param
);

/* Typedef for the search compare function. This function is to be passed
   to the binary search function to compare the search parameters and search
   field in the element in the sorted array.
*/ 
/*------------------------------------------------------------------*/
/*! @brief  This type defines for the search compare function. This function is to be passed
*   to the binary search function to compare the search parameters and search
*   field in the element in the sorted array.
*  @param [in] pv_elem_a            Indicates the first element to be compared.
*  @param [in] pv_search_param  Indicates the parameter used to search the object array.
*  @param [in] pv_opt_param       Reference the optional parameter.
*/
/*------------------------------------------------------------------*/
typedef INT32 (*x_search_cmp_fct) 
(
    const VOID*  pv_elem_a,
    const VOID*  pv_search_param,
    VOID*        pv_opt_param
);

#endif /* _U_SORT_SEARCH_H_ */


/*----------------------------------------------------------------------------*/
/*! @} */                                               /* end of groupMW_UTIL*/
/*----------------------------------------------------------------------------*/


