/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file u_wgl_tktp_cnt_common.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains public interfaces, constant and type 
 *         definitions of the Widget Library.
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_WGL_TKTP
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

#ifndef _U_WGL_TKTP_CNT_COMMON_H_
#define _U_WGL_TKTP_CNT_COMMON_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_wgl_common.h"

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/
/* common commands */
/** 
This command sets Ticker Tape widget's content insets, which defines the margin between the boundary of the content component and the boundary of the viewport inside. 

Input arguments

pv_param1	Specifies the new content inset. This argument must be of type (WGL_INSET_T *). If this argument is NULL, the content insets will be reset as zero.

pv_param2	Reserved. It must be NULL.


Returns

WGLR_OK	The function call is successful.

WGLR_INV_ARG	The specified inset is invalid. 
*/
#define WGL_CMD_TKTP_SET_CNT_INSET                WGL_MAKE_CONTENT_CMD(WGL_CONTENT_TKTP_DEF, 100)  
/**
This command gets Ticker Tape widget's content insets, which defines the margin between the boundary of the content component and the boundary of the viewport inside. 

Input arguments

pv_param2	Reserved. It must be NULL.

Output arguments

pv_param1	Specifies the pointer to a buffer of type WGL_INSET_T to receive the content insets. Thus this argument must be of type (WGL_INSET_T *) and cannot be NULL.

Returns

WGLR_OK	The function call is successful.

WGLR_INV_ARG	The argument pv_param1 is NULL.
*/
#define WGL_CMD_TKTP_GET_CNT_INSET                WGL_MAKE_CONTENT_CMD(WGL_CONTENT_TKTP_DEF, 101)  
/**
This command sets the alignment mode of message text. Note that this attributes only takes effect when the message text is not scrolled when it is short and the style WGL_STL_TKTP_NO_SCRL_SHORT_TEXT is specified when creating the widget. 

Input arguments

pv_param1	Specifies the new text alignment mode. This argument must be of type (UINT8). 

pv_param2	Reserved. It must be NULL.

Returns

WGLR_OK	The function call is successful.

WGLR_INV_ARG	The specified alignment is invalid. 
*/
#define WGL_CMD_TKTP_SET_ALIGN                    WGL_MAKE_CONTENT_CMD(WGL_CONTENT_TKTP_DEF, 102) 
/**
This command gets the alignment mode of message text. 

Input arguments

pv_param2	Reserved. It must be NULL.

Output arguments

pv_param1	Specifies the pointer to the buffer of type UINT8 to receive the alignment mode. Thus this argument must be of type (UINT8 *) and cannot be NULL.

Returns

WGLR_OK	The function call is successful.

WGLR_INV_ARG	The argument pv_param1 is NULL.
*/
#define WGL_CMD_TKTP_GET_ALIGN                    WGL_MAKE_CONTENT_CMD(WGL_CONTENT_TKTP_DEF, 103)    

/* specific color targets */
/**   text background color target     */
#define WGL_CLR_TEXT_BK                        (WGL_CLR_WGT_SPF_START+1)    


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/


#ifdef __cplusplus
}
#endif


#endif 
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of groupMW_WGL_TKTP*/
/*----------------------------------------------------------------------------*/
