/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_atsc_si_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains ATSC-SI engine specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_ATSC_SI_H_
#define _X_ATSC_SI_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_atsc_si_eng.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

extern INT32 x_atsc_si_get_tbl_version (HANDLE_T    h_obj,
                                        UINT8*      pui1_tbl_version,
                                        TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_prot_version (HANDLE_T    h_obj,
                                         UINT8*      pui1_prot_version,
                                         TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_num_mgt_entries (HANDLE_T    h_mgt,
                                            UINT16*     pui2_num_entries,
                                            TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_mgt_entry (HANDLE_T             h_mgt,
                                      UINT16               ui2_idx,
                                      ATSC_SI_MGT_INFO_T*  pt_mgt_info,
                                      TM_COND_T*           pe_cond);

extern INT32 x_atsc_si_get_vct_ts_id (HANDLE_T    h_vct,
                                      UINT16*     pui2_ts_id,
                                      TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_num_vct_entries (HANDLE_T    h_vct,
                                            UINT16*     pui2_num_entries,
                                            TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_vct_entry (HANDLE_T             h_vct,
                                      UINT16               ui2_idx,
                                      ATSC_SI_VCT_INFO_T*  pt_cvct_info,
                                      TM_COND_T*           pe_cond);

extern INT32 x_atsc_si_get_rrt_region (HANDLE_T    h_rrt,
                                       UINT8*      pui1_region,
                                       TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_num_rrt_entries (HANDLE_T    h_rrt,
                                            UINT16*     pui2_num_entries,
                                            TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_rrt_entry (HANDLE_T             h_rrt,
                                      UINT16               ui2_idx,
                                      ATSC_SI_RRT_INFO_T*  pt_rrt_info,
                                      TM_COND_T*           pe_cond);

extern INT32 x_atsc_si_get_eit_src_id (HANDLE_T    h_eit,
                                       UINT16*     pui2_src_id,
                                       TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_num_eit_entries (HANDLE_T    h_eit,
                                            UINT16*     pui2_num_entries,
                                            TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_eit_entry (HANDLE_T             h_eit,
                                      UINT16               ui2_idx,
                                      ATSC_SI_EIT_INFO_T*  pt_eit_info,
                                      TM_COND_T*           pe_cond);

extern INT32 x_atsc_si_get_ett_etm_id (HANDLE_T    h_ett,
                                       UINT32*     pui4_etm_id,
                                       TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_txt_num_lang (HANDLE_T    h_txt,
                                         UINT16*     pui2_num_lang);

extern INT32 x_atsc_si_get_txt_info (HANDLE_T                  h_txt,
                                     UINT16                    ui2_lang_idx,
                                     ATSC_SI_TXT_LANG_INFO_T*  pt_lang_info);

extern INT32 x_atsc_si_get_txt_seg_info (HANDLE_T                 h_txt,
                                         UINT16                   ui2_lang_idx,
                                         UINT16                   ui2_seg_idx,
                                         ATSC_SI_TXT_SEG_INFO_T*  pt_text_seg_info);

extern INT32 x_atsc_si_get_txt (HANDLE_T  h_txt,
                                UINT16    ui2_lang_idx,
                                UINT16    ui2_seg_idx,
                                CHAR*     ps_txt,
                                SIZE_T*   pz_txt_len);

extern INT32 x_atsc_si_get_txt_len (HANDLE_T  h_txt,
                                    SIZE_T*   pz_txt_len);

extern INT32 x_atsc_si_get_txt_len_by_lang (HANDLE_T        h_txt,
                                            ISO_639_LANG_T  t_lang,
                                            SIZE_T*         pz_event_info_len);

extern INT32 x_atsc_si_get_txt_by_lang (HANDLE_T        h_txt,
                                        ISO_639_LANG_T  t_lang,
                                        SIZE_T*         pz_event_info_len,
                                        CHAR*           ps_txt);

extern INT32 x_atsc_si_get_eas_event_data (HANDLE_T     h_eas,
                                           EAS_EVENT_T* pt_eas_info,
                                           TM_COND_T*   pe_cond);

extern INT32 x_atsc_si_get_eas_evt_code_len (HANDLE_T   h_eas,
                                             SIZE_T*    pz_evt_code_len,
                                             TM_COND_T* pe_cond);

extern INT32 x_atsc_si_get_eas_evt_code (HANDLE_T   h_eas,
                                         CHAR*      ps_evt_code,  
                                         SIZE_T*    pz_evt_code_len,
                                         TM_COND_T* pe_cond);

extern INT32 x_atsc_si_get_eas_num_loc_entries (HANDLE_T    h_eas,
                                                UINT16*     pui2_num_entries,
                                                TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_get_eas_loc_entry (HANDLE_T          h_eas,
                                          UINT16            ui2_idx,
                                          EAS_LOC_INFO_T*   pt_location,
                                          TM_COND_T*        pe_cond);

extern INT32 x_atsc_si_get_eas_num_xcept_entries (HANDLE_T      h_eas,
                                                  UINT16*       pui2_num_entries,
                                                  TM_COND_T*    pe_cond);

extern INT32 x_atsc_si_get_eas_xcept_entry (HANDLE_T            h_eas,
                                            UINT16              ui2_idx,
                                            EAS_XCEPT_INFO_T*   pt_xcept,
                                            TM_COND_T*          pe_cond);

extern INT32 x_atsc_si_txt_loc_vct_sname (HANDLE_T   h_vct,
                                          UINT16     ui2_idx,
                                          HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_rrt_region (HANDLE_T   h_rrt,
                                           HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_rrt_dim (HANDLE_T   h_rrt,
                                        UINT16     ui2_dim_idx,
                                        HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_rrt_abrv_rating (HANDLE_T   h_rrt,
                                                UINT16     ui2_dim_idx,
                                                UINT16     ui2_val_idx,
                                                HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_rrt_rating (HANDLE_T   h_rrt,
                                           UINT16     ui2_dim_idx,
                                           UINT16     ui2_val_idx,
                                           HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_eit_title (HANDLE_T   h_eit,
                                          UINT16     ui2_idx,
                                          HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_ett_ext_text (HANDLE_T   h_ett,
                                             HANDLE_T*  ph_txt);

extern INT32 x_atsc_si_txt_loc_descr (HANDLE_T      h_obj,
                                      const UINT8*  pui1_multi_lang_txt,
                                      HANDLE_T*     ph_txt);

extern INT32 x_atsc_si_txt_loc_eas_nat_of_act (HANDLE_T     h_eas,
                                               HANDLE_T*    ph_txt);

extern INT32 x_atsc_si_txt_loc_eas_alert (HANDLE_T  h_eas,
                                          HANDLE_T* ph_txt);

extern INT32 x_atsc_si_init (const CHAR*                ps_name,
                             const ATSC_SI_CONFIG_T*    pt_config);

extern INT32 x_atsc_si_load_stt (HANDLE_T       h_root,
                                 VOID*          pv_nfy_tag,
                                 x_tm_nfy_fct   pf_nfy,
                                 HANDLE_T*      ph_stt,
                                 TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_stt_by_ts_descr (HANDLE_T       h_root,
                                             TS_DESCR_T*    pt_ts_descr,
                                             VOID*          pv_nfy_tag,
                                             x_tm_nfy_fct   pf_nfy,
                                             HANDLE_T*      ph_stt,
                                             TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_mgt_by_ts_id (HANDLE_T       h_root,
                                          UINT16         ui2_ts_id,
                                          TM_SRC_TYPE_T  e_src_type,
                                          VOID*          pv_src_info,
                                          VOID*          pv_nfy_tag,
                                          x_tm_nfy_fct   pf_nfy,
                                          HANDLE_T*      ph_mgt,
                                          TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_mgt_by_ts_descr (HANDLE_T       h_root,
                                             TS_DESCR_T*    pt_ts_descr,
                                             TM_SRC_TYPE_T  e_src_type,
                                             VOID*          pv_src_info,
                                             VOID*          pv_nfy_tag,
                                             x_tm_nfy_fct   pf_nfy,
                                             HANDLE_T*      ph_mgt,
                                             TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_mgt_by_ts_descr_wo_time_check (
                                             HANDLE_T       h_root,
                                             TS_DESCR_T*    pt_ts_descr,
                                             TM_SRC_TYPE_T  e_src_type,
                                             VOID*          pv_src_info,
                                             VOID*          pv_nfy_tag,
                                             x_tm_nfy_fct   pf_nfy,
                                             HANDLE_T*      ph_mgt,
                                             TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_mgt (HANDLE_T       h_root,
                                 TM_SRC_TYPE_T  e_src_type,
                                 VOID*          pv_src_info,
                                 VOID*          pv_nfy_tag,
                                 x_tm_nfy_fct   pf_nfy,
                                 HANDLE_T*      ph_mgt,
                                 TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_cvct (HANDLE_T       h_mgt,
                                  TM_SRC_TYPE_T  e_src_type,
                                  VOID*          pv_src_info,
                                  VOID*          pv_nfy_tag,
                                  x_tm_nfy_fct   pf_nfy,
                                  HANDLE_T*      ph_cvct,
                                  TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_tvct (HANDLE_T       h_mgt,
                                  TM_SRC_TYPE_T  e_src_type,
                                  VOID*          pv_src_info,
                                  VOID*          pv_nfy_tag,
                                  x_tm_nfy_fct   pf_nfy,
                                  HANDLE_T*      ph_tvct,
                                  TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_cvct_by_ts_id (HANDLE_T       h_mgt,
                                           TM_SRC_TYPE_T  e_src_type,
                                           VOID*          pv_src_info,
                                           VOID*          pv_nfy_tag,
                                           x_tm_nfy_fct   pf_nfy,
                                           HANDLE_T*      ph_cvct,
                                           TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_tvct_by_ts_id (HANDLE_T       h_mgt,
                                           TM_SRC_TYPE_T  e_src_type,
                                           VOID*          pv_src_info,
                                           VOID*          pv_nfy_tag,
                                           x_tm_nfy_fct   pf_nfy,
                                           HANDLE_T*      ph_tvct,
                                           TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_rrt (HANDLE_T       h_mgt,
                                 UINT8          ui1_region,
                                 TM_SRC_TYPE_T  e_src_type,
                                 VOID*          pv_src_info,
                                 VOID*          pv_nfy_tag,
                                 x_tm_nfy_fct   pf_nfy,
                                 HANDLE_T*      ph_rrt,
                                 TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_eit_by_idx (HANDLE_T       h_vct,
                                        UINT16         ui2_idx,
                                        TIME_T         t_time,
                                        TM_SRC_TYPE_T  e_src_type,
                                        VOID*          pv_src_info,
                                        VOID*          pv_nfy_tag,
                                        x_tm_nfy_fct   pf_nfy,
                                        HANDLE_T*      ph_eit,
                                        TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_eit_by_src_id (HANDLE_T       h_obj,
                                           UINT16         ui2_src_id,
                                           TIME_T         t_time,
                                           TM_SRC_TYPE_T  e_src_type,
                                           VOID*          pv_src_info,
                                           VOID*          pv_nfy_tag,
                                           x_tm_nfy_fct   pf_nfy,
                                           HANDLE_T*      ph_eit,
                                           TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_eit_0_by_idx (HANDLE_T       h_vct,
                                         UINT16         ui2_idx,
                                         TM_SRC_TYPE_T  e_src_type,
                                         VOID*          pv_src_info,
                                         VOID*          pv_nfy_tag,
                                         x_tm_nfy_fct   pf_nfy,
                                         HANDLE_T*      ph_eit,
                                         TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_eit_0_by_src_id (HANDLE_T       h_obj,
                                            UINT16         ui2_src_id,
                                            TM_SRC_TYPE_T  e_src_type,
                                            VOID*          pv_src_info,
                                            VOID*          pv_nfy_tag,
                                            x_tm_nfy_fct   pf_nfy,
                                            HANDLE_T*      ph_eit,
                                            TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_ett_by_idx (HANDLE_T       h_obj,
                                        UINT16         ui2_idx,
                                        TM_SRC_TYPE_T  e_src_type,
                                        VOID*          pv_src_info,
                                        VOID*          pv_nfy_tag,
                                        x_tm_nfy_fct   pf_nfy,
                                        HANDLE_T*      ph_ett,
                                        TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_ett_by_src_id (HANDLE_T       h_mgt,
                                           UINT16         ui2_src_id,
                                           TM_SRC_TYPE_T  e_src_type,
                                           VOID*          pv_src_info,
                                           VOID*          pv_nfy_tag,
                                           x_tm_nfy_fct   pf_nfy,
                                           HANDLE_T*      ph_ett,
                                           TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_ett_by_src_evt_id (HANDLE_T       h_mgt,
                                               UINT16         ui2_src_id,
                                               UINT16         ui2_evt_id,
                                               TIME_T         t_time,
                                               TM_SRC_TYPE_T  e_src_type,
                                               VOID*          pv_src_info,
                                               VOID*          pv_nfy_tag,
                                               x_tm_nfy_fct   pf_nfy,
                                               HANDLE_T*      ph_ett,
                                               TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_load_eas (HANDLE_T       h_root,
                                 VOID*          pv_nfy_tag,
                                 x_tm_nfy_fct   pf_nfy,
                                 HANDLE_T*      ph_eas,
                                 TM_COND_T*     pe_cond);

extern INT32 x_atsc_si_set_vct_by_ts_id (HANDLE_T    h_vct,
                                         UINT16      ui2_ts_id,
                                         TM_COND_T*  pe_cond);

extern INT32 x_atsc_si_reg_raw_mem_nfy (atsc_si_rm_nfy_fct pf_nfy);

extern INT32 x_atsc_si_raw_mem_set (ATSC_SI_RM_SET_T e_set_type);

#endif /* _X_ATSC_SI_H_ */
