/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_cdb_api.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description: 
 *         This header file contains core database related API definitions,
 *         which are exported to other Middleware components.
 *         
 *---------------------------------------------------------------------------*/

#ifndef _X_CDB_API_H_
#define _X_CDB_API_H_

/*---------------------------------------------------------------------------
                    include files
----------------------------------------------------------------------------*/
#include "x_common.h"
#include "u_cdb.h"
/*-------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

extern INT32 x_db_create
(
    UINT16            ui2_max_record,
    DB_MEM_POLICY_T   e_memory_policy,
    CHAR*             s_db_name,
    UINT8             ui1_count,
    DB_FIELD_INFO_T   at_field_info[],
    UINT8			  ui1_key_field_count,
    DB_FIELD_T        aui4_key_field_types[],
    VOID*             pv_tag,
    db_notify_fct     pf_nfy,
    HANDLE_T*         ph_db_obj
);


extern INT32 x_db_delete
(
    HANDLE_T          h_client,
    CHAR*             s_db_name
);

extern INT32 x_db_clean
(
    HANDLE_T          h_client
);

extern INT32 x_db_get_handle_by_name
(
    CHAR*             s_db_name,
    VOID*             pv_tag,
    db_notify_fct     pf_nfy,
    HANDLE_T*         ph_client
);

extern INT32 x_db_read_rec
(
    HANDLE_T          h_client,
    UINT8		      ui1_keyfld_count,
    const DB_FIELD_T  aui4_keyfld_types[],
    const VOID*       apv_keyfld_addrs[],
    const DB_SEARCH_T at_keyfld_search_opts[],
    UINT8             ui1_count,
    const DB_FIELD_T  aui4_fld_type_selected[],
    VOID*             apv_fld_value_buffer[],
    UINT16            aui2_buffer_len[],
    UINT16            ui2_record_index,
    UINT32*           pui4_db_version
);

extern INT32 x_db_update_rec
(
    HANDLE_T          h_client,
    const VOID*       apv_key_field_values[],
    UINT8             ui1_fld_count,
    const DB_FIELD_T  aui4_fld_type_selected[],
    const VOID*       apv_fld_buffer[],
    const UINT16      aui2_buffer_len[]
);

extern INT32 x_db_write_rec
(
    HANDLE_T          h_client,
    const VOID*       apv_key_field_values[],
    const VOID*       apv_field_values[],
    const UINT16      aui2_buffer_len[],
    DB_CTRL_FLAG_T    ui1_write_mode
);

extern INT32 x_db_delete_rec
(
    HANDLE_T          h_client,
    UINT8		      ui1_key_field_count,
    const DB_FIELD_T  aui4_field_types[],  
    const VOID*       apv_field_values[],     
    UINT16		      ui2_record_index 
);

extern INT32 x_db_write_lock
(
    HANDLE_T h_client
);

extern INT32 x_db_write_unlock
(
    HANDLE_T h_client
);

extern INT32 x_db_read_lock
(
    HANDLE_T h_client
);

extern INT32 x_db_read_unlock
(
    HANDLE_T h_client
);

#endif /* _X_CDB_API_H_ */
