/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_dbg_drv.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains debug driver related definitions, which
 *         are exported.
 *---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_DBG
  *  @{  
  */
/*----------------------------------------------------------------------------*/

#ifndef _X_DBG_DRV_H_
#define _X_DBG_DRV_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_dbg_drv.h"
#include "x_dbg.h"
#include "x_common.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

#define SYS_EXIT  ((UINT32) 0)
/**<This definition of SYS_EXIT*/


/*------------------------------------------------------------------*/
/*! @brief  This type defines the data receive callback function prototype.
  *  @param [in] ui1_data        Contain the input data.
  */
/*------------------------------------------------------------------*/
typedef VOID (*x_dbg_data_rec_fct) (UINT8  ui1_data);


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

/* The following API's must ONLY be implemented by the manufacturer */
/* if the debug output is not routed via a driver registered with   */
/* the Resource Manager. This means, the debug library has been     */
/* compiled with the define EXT_DBG_DRV set.                        */

/*------------------------------------------------------------------*/
/*! @brief  This API open the oputput device.
*  @param [in]   ps_output_name     References the name of output device.
*  @param [in]   e_output_type        References the type of output device.
*  @param [in]   pv_output_info       References the info to open output device.
*  @retval      -DBGR_OK                Routine successful.
*  @retval      -DBG_FAIL                 Debug fail.
*/
/*------------------------------------------------------------------*/
extern INT32 x_dbg_open_output (const CHAR*        ps_output_name,
                                DBG_OUTPUT_TYPE_T  e_output_type,
                                const VOID*        pv_output_info);
                                


/*------------------------------------------------------------------*/
/*! @brief  This API output the log information.
*  @param [in]   ps_stmt                 References the log information.
*  @retval      -DBGR_OK                Routine successful.
*  @retval      -DBG_FAIL                 Debug fail.
*/
/*------------------------------------------------------------------*/
extern VOID x_dbg_put_stmt (const CHAR* ps_stmt);


/*------------------------------------------------------------------*/
/*! @brief  This API set the data receive callback function.
*  @param [in]   pf_data_rec           References the pointer of data receive callback function.
*  @retval      -DBGR_OK                Routine successful.
*  @retval      -DBG_FAIL                 Debug fail.
*/
/*------------------------------------------------------------------*/
extern VOID x_dbg_reg_data_rec (x_dbg_data_rec_fct  pf_data_rec);

#endif /* _X_DBG_DRV_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                                 /* end of groupMW_DBG*/
/*----------------------------------------------------------------------------*/

