/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_dlna_api.h 
*  $RCSfile: $
*  $Revision: #1 $
*  $Date: 2015/05/29 $
*  $Author: brianpc.huang $
*  
*  @par Description:
*         This header file contains common DLNA exported API.
*/
/*----------------------------------------------------------------------------*/

/*!
* @addtogroup  groupMW_INET_DLNA
* @brief       Provide an external API about DLNA.
* %DLNA API would be used by user who wants to play media stream in local network  
* 
* @see 
*
* @{
* */


#ifndef X_DLNA_API_H
#define X_DLNA_API_H

#include "u_dlna.h"

/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_get_device_name(DLNA_DEVICE_T t_device, CHAR ** pps_name)
*  @brief  Gets device name which in local network 
*          - Called while who wants to know the indicated device name
*  @param  [in] t_device        - The device pointer
*  @param  [out]pps_name        - The device name
*  
*  @note   
*  @see     
*  @return Return the error code                
*  @retval DLNAR_OK          	- The routine was successful. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_get_device_name
*
* Description: Get device name
*
* Inputs:  t_device - device to check type
*
* Outputs: pps_name - the device name
*
* Returns: DLNAR_OK - get name ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_get_device_name(DLNA_DEVICE_T t_device, CHAR ** pps_name);

/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_get_device_type(DLNA_DEVICE_T t_device, DLNA_DEVICE_TYPE_T * pe_type)
*  @brief  Gets device type which in local network 
*          - Called while who wants to know the indicated device type
*  @param  [in] t_device        - The device pointer
*  @param  [out]pe_type         - The device type
*  
*  @note   
*  @see     
*  @return Return the error code                
*  @retval DLNAR_OK          	- The routine was successful. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_get_device_type
*
* Description: The function to get device type
*
* Inputs:  t_device - the object to get
*
* Outputs: pe_type - the device type
*
* Returns: DLNAR_OK - get device type ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_get_device_type(DLNA_DEVICE_T t_device, DLNA_DEVICE_TYPE_T * pe_type);


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_parse_protocol_info(CHAR * ps_orig, DLNA_PROTOCOL_INFO_T * pt_protocol)
*  @brief  Parsing protocol info string 
*          - Called while who wants to know the protocol info
*  @param  [in] ps_orig         - The string which contains protocol info
*  @param  [in][out]pt_protocol - The protocol info
*  
*  @note   
*  @see    x_dlna_free_protocol_info 
*  @return Return the error code                
*  @retval DLNAR_OK          	- The routine was successful. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_parse_protocol_info
*
* Description: The function to parse protocol string to info
*
* Inputs:  ps_orgi - the protocol info string
*
* Outputs: pt_protocol - the protocol info
*
* Returns: DLNAR_OK - get parse info ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_parse_protocol_info(CHAR * ps_orig, DLNA_PROTOCOL_INFO_T * pt_protocol);

/*----------------------------------------------------------------------------*/
/*! @fn    VOID x_dlna_free_protocol_info(DLNA_PROTOCOL_INFO_T * pt_protocol)
*  @brief  free protocol info 
*          - Called while who wants to know the protocol info
*  @param  [in] pt_protocol             - The protocol info
*  
*  @note   
*  @see    x_dlna_parse_protocol_info 
*  @return void
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_free_protocol_info
*
* Description: The function to free protocol info
*
* Inputs:  pt_protocol - the protocol info 
*
* Outputs:  
*
* Returns:
----------------------------------------------------------------------------*/
extern VOID x_dlna_free_protocol_info(DLNA_PROTOCOL_INFO_T * pt_protocol);

#endif

/*! @} */

