/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_dlna_dmr_api.h 
*  $RCSfile: $
*  $Revision: #1 $
*  $Date: 2015/05/29 $
*  $Author: brianpc.huang $
*  
*  @par Description:
*         This header file contains DLNA DMR exported API.
*/
/*----------------------------------------------------------------------------*/

/*!
* @addtogroup  groupMW_INET_DLNA
* @brief       Provide an external API about DLNA.
* %DLNA API would be used by user who wants to play media stream in local network  
* 
* @see 
*
* @{
* */


#ifndef X_DLNA_DMR_API_H
#define X_DLNA_DMR_API_H

#include "u_dlna_dmr.h"

/**
 * Utility
 */

/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_dmr_start(x_dlna_device_nfy pf_nfy, VOID * pv_tag)
*  @brief  Starts the DMR
*          - Called while who wants to start DMR
*  @param  [in]pf_nfy          - The callback while device found or left
*  @param  [in]pv_tag          - The callback tag
*  
*  @note   
*  @see    x_dlna_dmr_stop  
*  @return Return the error code                
*  @retval DLNAR_OK     	- Start dmr core ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_start
*
* Description: The API to start dmr kernel
*
* Inputs:  
*
* Outputs: -
*
* Returns: DLNAR_OK - start dmr core ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_dmr_start(x_dlna_device_nfy pf_nfy, VOID * pv_tag);


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_dmr_add_player_listener(x_dlna_dmr_player_nfy pf_nfy, VOID * pv_tag)
*  @brief  Adds the listener to the DMR
*          - Called while who wants to add a listener to DMR
*  @param  [in]pf_nfy          - The callback while player state changed
*  @param  [in]pv_tag          - The callback tag
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Add ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_add_player_listener
*
* Description: The API to add dmr player listener
*
* Inputs:  
*
* Outputs: -
*
* Returns: DLNAR_OK - start dmr core ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_dmr_add_player_listener(x_dlna_dmr_player_nfy pf_nfy, VOID * pv_tag);


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_dmr_remove_player_listener(VOID)
*  @brief  Removes the listener to the DMR
*          - Called while who wants to remove a listener to DMR
*  @param  
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Remove ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_remove_player_listener
*
* Description: The API to add dmr player listener
*
* Inputs:  
*
* Outputs: -
*
* Returns: DLNAR_OK - start dmr core ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_dmr_remove_player_listener(VOID);


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_dmr_stop(VOID)
*  @brief  Stops the DMR
*          - Called while who wants to stop DMR
*  @param  
*  
*  @note   
*  @see    x_dlna_dmr_start  
*  @return Return the error code                
*  @retval DLNAR_OK     	- stop dmr core ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_stop
*
* Description: The API to close dmr kernel
*
* Inputs:  -
*
* Outputs: -
*
* Returns: DLNAR_OK - close dmr ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_dmr_stop(VOID);


/*----------------------------------------------------------------------------*/
/*! @fn    INT32 x_dlna_dmr_set_file_handler(x_dlna_dmr_file_hdlr pf_handler)
*  @brief  Sets the file handle to the DMR
*          - Called while who wants to add a listener to DMR
*  @param  [in]pf_handler        - The file handler 
*  
*  @note   
*  @see      
*  @return Return the error code                
*  @retval DLNAR_OK     	   - Set ok. 
*/
/*----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_set_file_handler
*
* Description: The API to mount uri and protocol info to a local file path
*
* Inputs:  -
*
* Outputs: -
*
* Returns: DLNAR_OK -  ok
----------------------------------------------------------------------------*/
extern INT32 x_dlna_dmr_set_file_handler(x_dlna_dmr_file_hdlr pf_handler);

/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_getname
*
* Description: The API to get dmr friendly name from dmr.xml
*
* Inputs:  -
*
* Outputs: -
*
* Returns: DLNAR_OK -  ok
*/
extern INT32 x_dlna_dmr_getname(CHAR ** ps_dmr_name);

/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_setname
*
* Description: The API to set dmr friendly name to dmr.xml
*
* Inputs:  -
*
* Outputs: -
*
* Returns: DLNAR_OK -  ok
*/
extern INT32 x_dlna_dmr_setname(CHAR * ps_dmr_name);

/*-----------------------------------------------------------------------------
* Name: x_dlna_dmr_get_media_info
*
* Description: The API to get dmr media detail info
*
* Inputs:  -
*
* Outputs: -
*
* Returns: DLNAR_OK -  ok
*/
extern INT32 x_dlna_dmr_get_media_info(DLNA_DMR_MEDIA_INFO_T **pt_dmr_media_info);

extern INT32 x_dlna_dmr_update_interface(CHAR* pName);
#endif

/*! @} */

