/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_dsmcc_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains DSM-CC engine specific definitions, which
 *         are exported.
 *-------------------------------------------------------------------------*/

#ifndef _X_DSMCC_ENG_H_
#define _X_DSMCC_ENG_H_


/*---------------------------------------------------------------------------
                    include files
---------------------------------------------------------------------------*/

#include "x_handle_grp.h"
#include "x_common.h"
#include "x_tm.h"
#include "u_dsmcc_eng.h"


/*---------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 --------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                    functions declarations
 --------------------------------------------------------------------------*/

extern INT32 x_dsmcc_init (const CHAR*      ps_name,
                           DSMCC_CONFIG_T*  pt_config);

/* DSMCC monitoring API's. */
extern INT32 x_dsmcc_mon_top_by_pid (HANDLE_T        h_root,
                                     MPEG_2_PID_T    t_pid,
                                     TM_SRC_TYPE_T   e_src_type,
                                     VOID*           pv_src_info,
                                     VOID*           pv_nfy_tag,
                                     x_tm_nfy_fct    pf_nfy,
                                     HANDLE_T*       ph_mon,
                                     TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_get_download_level (HANDLE_T               h_mon,
                                         DSMCC_DOWNLOAD_TYPE_T* pe_download_type,
                                         TM_COND_T*             pe_cond);




/* NEW API'S */
extern INT32 x_dsmcc_load_dsi_by_pid_ts_id (HANDLE_T       h_root,
                                            MPEG_2_PID_T   t_pid,
                                            UINT16         ui2_ts_id,
                                            TM_SRC_TYPE_T  e_src_type,
                                            VOID*          pv_src_info,
                                            VOID*          pv_nfy_tag,
                                            x_tm_nfy_fct   pf_nfy,
                                            HANDLE_T*      ph_dsi,
                                            TM_COND_T*     pe_cond);
extern INT32 x_dsmcc_load_dsi_by_pid_ts_descr (HANDLE_T       h_root,
                                               MPEG_2_PID_T   t_pid,
                                               TS_DESCR_T*    pt_ts_descr,
                                               TM_SRC_TYPE_T  e_src_type,
                                               VOID*          pv_src_info,
                                               VOID*          pv_nfy_tag,
                                               x_tm_nfy_fct   pf_nfy,
                                               HANDLE_T*      ph_dsi,
                                               TM_COND_T*     pe_cond);

extern INT32 x_dsmcc_load_dii_by_pid_ts_id_ids (HANDLE_T       h_root,
                                                MPEG_2_PID_T   t_pid,
                                                UINT16         ui2_ts_id,
                                                DSMCC_IDS_T*   pt_dsmcc_ids,
                                                TM_SRC_TYPE_T  e_src_type,
                                                VOID*          pv_src_info,
                                                VOID*          pv_nfy_tag,
                                                x_tm_nfy_fct   pf_nfy,
                                                HANDLE_T*      ph_dii,
                                                TM_COND_T*     pe_cond);
extern INT32 x_dsmcc_load_dii_by_pid_ts_descr_ids (HANDLE_T       h_root,
                                                   MPEG_2_PID_T   t_pid,
                                                   TS_DESCR_T*    pt_ts_descr,
                                                   DSMCC_IDS_T*   pt_dsmcc_ids,
                                                   TM_SRC_TYPE_T  e_src_type,
                                                   VOID*          pv_src_info,
                                                   VOID*          pv_nfy_tag,
                                                   x_tm_nfy_fct   pf_nfy,
                                                   HANDLE_T*      ph_dii,
                                                   TM_COND_T*     pe_cond);


extern INT32 x_dsmcc_load_ddb_by_pid_ts_id_ids (HANDLE_T       h_root,
                                                MPEG_2_PID_T   t_pid,
                                                UINT16         ui2_ts_id,
                                                DSMCC_IDS_T*   pt_dsmcc_ids,
                                                BOOL           b_feed_section,
                                                TM_SRC_TYPE_T  e_src_type,
                                                VOID*          pv_src_info,
                                                VOID*          pv_nfy_tag,
                                                x_tm_nfy_fct   pf_nfy,
                                                HANDLE_T*      ph_ddb,
                                                TM_COND_T*     pe_cond);
extern INT32 x_dsmcc_load_ddb_by_pid_ts_descr_ids (HANDLE_T       h_root,
                                                   MPEG_2_PID_T   t_pid,
                                                   TS_DESCR_T*    pt_ts_descr,
                                                   DSMCC_IDS_T*   pt_dsmcc_ids,
                                                   BOOL           b_feed_section,
                                                   TM_SRC_TYPE_T  e_src_type,
                                                   VOID*          pv_src_info,
                                                   VOID*          pv_nfy_tag,
                                                   x_tm_nfy_fct   pf_nfy,
                                                   HANDLE_T*      ph_ddb,
                                                   TM_COND_T*     pe_cond);
extern INT32 x_dsmcc_load_ddb_by_module_idx_block_num (HANDLE_T      h_dii,
                                                       UINT16        ui2_module_idx,
                                                       UINT16        ui2_block_num,
                                                       BOOL          b_feed_section,
                                                       VOID*         pv_nfy_tag,
                                                       x_tm_nfy_fct  pf_nfy,
                                                       HANDLE_T*     ph_ddb,
                                                       TM_COND_T*    pe_cond);
extern INT32 x_dsmcc_load_ddb_by_module_id_block_range (HANDLE_T      h_dii,
                                                        UINT16        ui2_module_id,
                                                        UINT8         ui1_module_version,
                                                        UINT16        ui2_first_block_num,
                                                        UINT16        ui2_last_block_num,
                                                        VOID*         pv_nfy_tag,
                                                        x_tm_nfy_fct  pf_nfy,
                                                        HANDLE_T*     ph_ddb,
                                                        TM_COND_T*    pe_cond);
extern INT32 x_dsmcc_load_ddb_by_module_idx_block_range (HANDLE_T      h_dii,
                                                         UINT16        ui2_module_idx,
                                                         UINT16        ui2_block_first_num,
                                                         UINT16        ui2_block_last_num,
                                                         VOID*         pv_nfy_tag,
                                                         x_tm_nfy_fct  pf_nfy,
                                                         HANDLE_T*     ph_ddb,
                                                         TM_COND_T*    pe_cond);
extern INT32 x_dsmcc_load_dsmcc_desc_by_pid_ts_id_evt_id (HANDLE_T       h_root,
                                                          MPEG_2_PID_T   t_pid,
                                                          UINT16         ui2_ts_id,
                                                          UINT16         ui2_evt_id,
                                                          TM_SRC_TYPE_T  e_src_type,
                                                          VOID*          pv_src_info,
                                                          VOID*          pv_nfy_tag,
                                                          x_tm_nfy_fct   pf_nfy,
                                                          HANDLE_T*      ph_dsmcc_desc,
                                                          TM_COND_T*     pe_cond);

extern INT32 x_dsmcc_get_module_idx_by_module_id(HANDLE_T           h_dii,
                                                 UINT32             ui4_mod_id,
                                                 UINT16*            pui2_mod_idx);

extern INT32 x_dsmcc_get_dsi_info (HANDLE_T           h_dsi,
                                   DSMCC_DSI_INFO_T*  pt_dsi_info,
                                   TM_COND_T*         pe_cond);
extern INT32 x_dsmcc_get_dii_info (HANDLE_T           h_dii,
                                   DSMCC_DII_INFO_T*  pt_dii_info,
                                   TM_COND_T*         pe_cond);
extern INT32 x_dsmcc_get_dii_module_info (HANDLE_T                  h_dii,
                                          UINT16                    ui2_module_idx,
                                          DSMCC_DII_MODULE_INFO_T*  pt_dii_module_info,
                                          TM_COND_T*                pe_cond);

extern INT32 x_dsmcc_get_ddb_info (HANDLE_T           h_ddb,
                                   DSMCC_DDB_INFO_T*  pt_ddb_info,
                                   TM_COND_T*         pe_cond);

extern INT32 x_dsmcc_parse_dsi_private_data (HANDLE_T        h_dsi,
                                             VOID*           pv_data,
                                             SIZE_T          z_data_len,
                                             x_tm_parse_fct  pf_parse,
                                             TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_parse_dii_comp_descr (HANDLE_T        h_dii,
                                           VOID*           pv_data,
                                           SIZE_T          z_data_len,
                                           x_tm_parse_fct  pf_parse,
                                           TM_COND_T*      pe_cond);
extern INT32 x_dsmcc_parse_dii_module_info (HANDLE_T        h_dii,
                                            UINT16          ui2_module_idx,
                                            VOID*           pv_data,
                                            SIZE_T          z_data_len,
                                            x_tm_parse_fct  pf_parse,
                                            TM_COND_T*      pe_cond);
extern INT32 x_dsmcc_parse_dii_private_data (HANDLE_T        h_dii,
                                             VOID*           pv_data,
                                             SIZE_T          z_data_len,
                                             x_tm_parse_fct  pf_parse,
                                             TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_parse_ddb_block_data (HANDLE_T        h_ddb,
                                           VOID*           pv_data,
                                           SIZE_T          z_data_len,
                                           x_tm_parse_fct  pf_parse,
                                           TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_parse_dsmcc_desc_all_descr(HANDLE_T        h_dsmcc_desc,
                                                VOID*           pv_data,
                                                SIZE_T          z_data_len,
                                                x_tm_parse_fct  pf_parse,
                                                TM_COND_T*      pe_cond);

extern INT32 x_dsmcc_parse_dsmcc_desc_descr(HANDLE_T                  h_dsmcc_desc,
                                            TM_MAIN_BODY_DESCR_SEL_T* pt_descr,
                                            VOID*                     pv_data,
                                            SIZE_T                    z_data_len,
                                            x_tm_parse_fct            pf_parse,
                                            TM_COND_T*                pe_cond);

#endif /* _X_DSMCC_ENG_H_ */

