/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_dvb_si_eng.h,v $
 * $Revision: #2 $
 * $Date: 2015/06/19 $
 * $Author: biao.liu $
 *
 * Description: 
 *         This header file contains DVB-SI engine specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_DVB_SI_H_
#define _X_DVB_SI_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "u_dvb_si_eng.h"
#include "u_rm.h"
#include "u_secm_api.h"



/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

extern INT32 x_dvb_si_get_bq_id (
                    HANDLE_T    h_bat,
                    UINT16*     pui2_bq_id,
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_get_nw_id (
                    HANDLE_T    h_nit,
                    UINT16*     pui2_nw_id,
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_get_num_evt_entries ( 
                    HANDLE_T    h_eit, 
                    UINT16*     pui2_num_entries, 
                    TM_COND_T*  pe_cond); 

extern INT32 x_dvb_si_get_evt_entry (
                    HANDLE_T                  h_eit,
                    UINT16                    ui2_idx,
                    DVB_SI_EVT_ENTRY_INFO_T*  pt_evt_info,
                    TM_COND_T*                pe_cond);

extern INT32 x_dvb_si_get_evt_ext_cond(
                    HANDLE_T                  h_eit,
                    TM_COND_T*                pe_ext_cond,
                    TM_COND_T*                pe_cond);

extern INT32 x_dvb_si_get_num_ts_entries (
                    HANDLE_T    h_nit, 
                    UINT16*     pui2_num_entries, 
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_bat_get_num_ts_entries (
    				HANDLE_T    h_bat, 
                    UINT16*     pui2_num_entries, 
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_get_ts_entry (
                    HANDLE_T                  h_nit,
                    UINT16                    ui2_idx,
                    DVB_SI_ON_TS_ID_INFO_T*   pt_ts_info,
                    TM_COND_T*                pe_cond);

extern INT32 x_dvb_si_bat_get_ts_entry (
    				HANDLE_T                  h_bat,
                    UINT16               	  ui2_idx,
                    DVB_SI_ON_TS_ID_INFO_T*	  pt_ts_info,
                    TM_COND_T*            	  pe_cond);

extern INT32 x_dvb_si_get_num_svc_entries ( 
                    HANDLE_T    h_sdt, 
                    UINT16*     pui2_num_entries, 
                    TM_COND_T*  pe_cond); 

extern INT32 x_dvb_si_get_svc_entry (
                    HANDLE_T                  h_sdt,
                    UINT16                    ui2_idx,
                    DVB_SI_SVC_ENTRY_INFO_T*  pt_svc_info,
                    TM_COND_T*                pe_cond);

extern INT32 x_dvb_si_get_svc_ids (
                    HANDLE_T                     h_eit,
                    DVB_SI_ON_TS_SVC_ID_INFO_T*  pt_svc_info,
                    TM_COND_T*                   pe_cond);

extern INT32 x_dvb_si_get_ts_ids (
                    HANDLE_T                 h_sdt,
                    DVB_SI_ON_TS_ID_INFO_T*  pt_ts_info,
                    TM_COND_T*               pe_cond);

extern INT32 x_dvb_si_get_sched_end_time ( 
                    HANDLE_T                        h_eit_s, 
                    DVB_SI_SCHED_END_TIME_INFO_T*   pt_sched_end_time_info, 
                    TM_COND_T*                      pe_cond); 

extern INT32 x_dvb_si_get_num_app_entries (HANDLE_T    h_ait, 
                                   UINT16*     pui2_num_entries, 
                                   TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_get_app_entry (HANDLE_T                     h_ait,
                             UINT16                           ui2_idx,
                             DVB_SI_AIT_APP_ID_CTRL_CODE_T*   pt_app_info,
                             TM_COND_T*                pe_cond);



extern INT32 x_dvb_si_mon_get_num_nw_id (
                    HANDLE_T    h_mon_nit,
                    UINT16*     pui2_num_nw_id,
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_mon_get_num_oui (
                    HANDLE_T    h_mon_unt,
                    UINT16*     pui2_num_oui,
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_mon_get_nw_id (
                    HANDLE_T    h_mon_nit,
                    UINT16      ui2_idx,
                    UINT16*     pui2_nw_id,
                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_mon_get_on_ts_id ( 
                    HANDLE_T                  h_mon_sdt, 
                    UINT16                    ui2_idx, 
                    DVB_SI_ON_TS_ID_INFO_T*   pt_on_ts_id, 
                    TM_COND_T*                pe_cond); 

extern INT32 x_dvb_si_mon_get_oui ( 
                    HANDLE_T                  h_mon_unt, 
                    UINT16                    ui2_idx, 
                    UINT32*                   pui4_oui, 
                    TM_COND_T*                pe_cond); 

extern INT32 x_dvb_si_txt_loc_descr ( 
                    HANDLE_T                h_obj, 
                    const UINT8*            pui1_txt, 
                    SIZE_T                  z_txt_len,
                    CHAR*                   t_count_code,
                    HANDLE_T*               ph_txt); 

extern INT32 x_dvb_si_get_txt_len ( 
                    HANDLE_T                h_txt, 
                    DVB_SI_GET_TXT_MODE_T   e_get_mode, 
                    SIZE_T*                 pz_txt_len);

extern INT32 x_dvb_si_get_txt ( 
                    HANDLE_T                h_txt, 
                    DVB_SI_GET_TXT_MODE_T   e_get_mode, 
                    CHAR*                   ps_txt, 
                    SIZE_T*                 pz_txt_len); 

extern INT32 x_dvb_si_get_txt_style_size ( 
                    HANDLE_T                h_txt, 
                    SIZE_T*                 pz_style_str_size); 

extern INT32 x_dvb_si_get_txt_style ( 
                    HANDLE_T                h_txt, 
                    STYLE_STR_T*            pt_style_str, 
                    SIZE_T*                 pz_style_str_size); 

extern INT32 x_dvb_si_init (
                    const CHAR*       ps_name,
                    DVB_SI_CONFIG_T*  pt_config);

extern INT32 x_dvb_si_load_bat (
                    HANDLE_T      h_root,
                    UINT16        ui2_bq_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_bat,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_bat_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    UINT16        ui2_bat_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_bat,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_pf (
                    HANDLE_T      h_root,
                    UINT16        ui2_on_id,
                    UINT16        ui2_ts_id,
                    UINT16        ui2_svc_id,
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_pf,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_pf_by_idx (
                    HANDLE_T      h_sdt,
                    UINT16        ui2_idx,
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_pf,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_pf_by_svc_id (
                    HANDLE_T      h_sdt,
                    UINT16        ui2_svc_id, 
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_pf,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_s (
                    HANDLE_T      h_root,
                    UINT16        ui2_on_id,
                    UINT16        ui2_ts_id,
                    UINT16        svc_id,
                    TIME_T        t_time,
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_s,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_pf_sections (
                    HANDLE_T      h_root,
                    BOOL          b_actual_only,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_pf,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_pf_pp_sections (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    BOOL          b_actual_only,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_pf,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_s_sections (
                    HANDLE_T      h_root,
                    BOOL    b_actual_only,
                    UINT8   ui1_tbl_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_s,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_s_sections_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    BOOL    b_actual_only,
                    UINT8   ui1_tbl_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_s,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_set_eit_section_mask_clear (HANDLE_T      h_eit,
                    DVB_SI_ON_TS_SVC_ID_SEC_INFO_T*          pv_sec_info,
                    TM_COND_T*     pe_cond);

extern INT32 x_dvb_si_set_eit_section_received(HANDLE_T h_eit,
                                        SECM_SECTION_KEY_SET_T* pv_sec_info,
                                        TM_COND_T*   p4_cond
                                        );

extern INT32 x_dvb_si_set_eit_reset_all(HANDLE_T h_eit,
                                 TM_COND_T*   p4_cond
                                 );

extern INT32 x_dvb_si_set_eit_section_clear(HANDLE_T h_eit,
                                        SECM_SECTION_KEY_SET_T* pv_sec_info,
                                        TM_COND_T*   p4_cond
                                        );


extern INT32 x_dvb_si_load_eit_s_by_idx (
                    HANDLE_T      h_sdt,
                    UINT16        ui2_idx,
                    TIME_T        t_time,
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_s,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_eit_s_by_svc_id (
                    HANDLE_T      h_sdt,
                    UINT16        ui2_svc_id,
                    TIME_T        t_time,
                    BOOL          b_high_prio,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_eit_s,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_nit (
                    HANDLE_T      h_root,
                    UINT16        ui2_nw_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_nit,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_nit_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    UINT16        ui2_nw_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_nit,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_nit_actual (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_nit,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_nit_actual_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_nit,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_rst (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_rst,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt (
                    HANDLE_T      h_root,
                    UINT16        ui2_on_id,
                    UINT16        ui2_ts_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    UINT16        ui2_on_id,
                    UINT16        ui2_ts_id,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tkgs (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_tkgs,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt_by_idx (
                    HANDLE_T      h_bat_nit,
                    UINT16        ui2_idx,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt_by_ts_descr (
                    HANDLE_T      h_root,
                    TS_DESCR_T*   pt_ts_descr,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt_actual (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_sdt_actual_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tdt (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_tdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tdt_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_tdt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tot (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_tot,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tot_by_pid (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_tot,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_unt (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    UINT32        ui4_oui,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_unt,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_ait (HANDLE_T      h_root,
                         MPEG_2_PID_T   t_pid,
                         UINT16         ui2_app_type,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_ait,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_bat (
                    HANDLE_T      h_root,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_bat_mon,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_nit (
                    HANDLE_T      h_root,
                    BOOL          b_actual_ts,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_nit_mon,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_nit_by_pid (HANDLE_T      h_root,
                        MPEG_2_PID_T  t_pid,
                        BOOL          b_actual_ts,
                        VOID*         pv_nfy_tag,
                        x_tm_nfy_fct  pf_nfy,
                        HANDLE_T*     ph_mon_nit,
                        TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_sdt (
                    HANDLE_T      h_root,
                    BOOL          b_actual_ts,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_sdt_mon,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_unt (
                    HANDLE_T      h_root,
                    MPEG_2_PID_T  t_pid,
                    VOID*         pv_nfy_tag,
                    x_tm_nfy_fct  pf_nfy,
                    HANDLE_T*     ph_unt_mon,
                    TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_ait (HANDLE_T      h_root,
                        MPEG_2_PID_T  t_pid,
                        VOID*         pv_nfy_tag,
                        x_tm_nfy_fct  pf_nfy,
                        HANDLE_T*     ph_ait_mon,
                        TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_mon_get_num_app_types (HANDLE_T    h_mon_ait,
                                  UINT16*     pui2_num_app_types,
                                  TM_COND_T*  pe_cond);


extern INT32 x_dvb_si_mon_get_app_type (HANDLE_T    h_mon_ait,
                              UINT16      ui2_idx,
                              UINT16*     pui2_app_type,
                              TM_COND_T*  pe_cond);


extern INT32 x_dvb_si_load_sgt (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT16        ui2_service_list_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_sgt,
                         TM_COND_T*    pe_cond);


extern INT32 x_dvb_si_mon_sgt (HANDLE_T      h_root,
                        MPEG_2_PID_T  t_pid,
                        VOID*         pv_nfy_tag,
                        x_tm_nfy_fct  pf_nfy,
                        HANDLE_T*     ph_sgt,
                        TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_sgt_mon_get_num_service_list_id (HANDLE_T    h_mon_sgt,
                                  UINT16*     pui2_num_service_list_id,
                                  TM_COND_T*  pe_cond);


extern INT32 x_dvb_si_sgt_mon_get_service_list_id (HANDLE_T    h_mon_sgt,
                              UINT16      ui2_idx,
                              UINT16*     pui2_service_list_id,
                              TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_sgt_get_num_svc_entries (HANDLE_T    h_sgt, 
                                    UINT16*     pui2_num_entries, 
                                    TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_sgt_get_svc_entry (HANDLE_T               h_sgt,
                                  UINT16                        ui2_idx,
                                  DVB_SI_SGT_SVC_ENTRY_INFO_T*  pt_svc_info,
                                  TM_COND_T*                    pe_cond);

extern INT32 x_dvb_si_load_fnt (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT16        ui2_operator_network_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_fnt,
                         TM_COND_T*    pe_cond);


extern INT32 x_dvb_si_load_tcdw (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT8         ui1_tbl_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_tcdw,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_trdw (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT8         ui1_tbl_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_trdw,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_tmdw (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT8         ui1_tbl_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_tmdw,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_lcn (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         UINT8         ui1_tbl_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_lcn,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_pct (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_pct,
                         TM_COND_T*    pe_cond);


extern INT32 x_dvb_si_fnt_get_num_service_entries (HANDLE_T    h_fnt, 
                                   UINT16*     pui2_num_entries, 
                                   TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_fnt_get_ts_entry (HANDLE_T                h_fnt,
                                  UINT16                        ui2_idx,
                                  DVB_SI_ON_TS_ID_INFO_T*       pt_ts_info,
                                  TM_COND_T*                    pe_cond);


extern INT32 x_dvb_si_load_fst (HANDLE_T      h_root,
                                MPEG_2_PID_T  t_pid,
                                UINT16        ui2_operator_network_id,
                                VOID*         pv_nfy_tag,
                                x_tm_nfy_fct  pf_nfy,
                                HANDLE_T*     ph_fst,
                                TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_fst_get_num_service_entries (HANDLE_T    h_fst, 
                                   UINT16*     pui2_num_entries, 
                                   TM_COND_T*  pe_cond);

extern INT32 x_dvb_si_fst_get_service_entry (HANDLE_T           h_fst,
                                  UINT16                        ui2_idx,
                                  DVB_SI_FST_SVC_ENTRY_INFO_T*  pt_service_info,
                                  TM_COND_T*                    pe_cond);


extern INT32 x_dvb_si_load_cmt (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_cmt,
                         TM_COND_T*    pe_cond);

extern INT32 x_dvb_si_load_clt (HANDLE_T      h_root,
                         MPEG_2_PID_T  t_pid,
                       //    UINT8        ui1_tbl_id,
                         VOID*         pv_nfy_tag,
                         x_tm_nfy_fct  pf_nfy,
                         HANDLE_T*     ph_cmt,
                         TM_COND_T*    pe_cond);



#endif /* _X_DVB_SI_H_ */
