/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_evctx_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/3 $
 * $SWAuthor: Justin Deng $
 * $MD5HEX: 1fd90ebb03749cab35e47b87d8576a94 $
 *
 * Description:
 *         This contains Event Context library macros, definitons, structures,
 *         and API's, which are shared with other modules.
 *---------------------------------------------------------------------------*/

#ifndef _X_EVCTX_ENG_H_
#define _X_EVCTX_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_evctx_eng.h"

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_EVCTX
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @brief  register the engine to the evctx selecter
 *
 *  @param  [in] e_brdcst_type          			- broadcast type
 *  @param  [in] pt_eng_fct_tbl          			- pointer of engine function table
 *  @param  [in] ui2_max_num_event_objs        - the max number of event jobs
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 x_evctx_reg_engine (
                BRDCST_TYPE_T           e_brdcst_type,
                EVCTX_ENG_FCT_TBL_T*    pt_eng_fct_tbl,
                UINT16                  ui2_max_num_event_objs);


/*----------------------------------------------------------------------------*/
/*! @brief  lock the h_evctx_base_ctrl_list
 *
 *  @note   
 *
 *  @see
 */
/*----------------------------------------------------------------------------*/

extern VOID x_evctx_base_ctrl_list_lock (VOID);

/*----------------------------------------------------------------------------*/
/*! @brief  unlock the h_evctx_base_ctrl_list
 *
 *  @note   
 *
 *  @see
 */
/*----------------------------------------------------------------------------*/

extern VOID x_evctx_base_ctrl_list_unlock (VOID);

/*----------------------------------------------------------------------------*/
/*! @brief  active the engine task
 *
 *  @param  [in] e_brdcst_type          			- broadcast type
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_evctx_activate_eng_task (BRDCST_TYPE_T e_brdcst_type);

/*----------------------------------------------------------------------------*/
/*! @brief  get the date of engine
 *
 *  @param  [in] h_event          		- handle of evctx object
 *  @param  [out] ppv_eng_data         - date of engine
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_evctx_get_engine_data (
                HANDLE_T                h_event,
                VOID**                  ppv_eng_data);

/*----------------------------------------------------------------------------*/
/*! @brief  a notification function that will be called by the Evctx selector when the condition or 
 *		   statement of engine changed.
 *
 *  @param  [in] h_event          		- handle of evctx object
 *  @param  [in] e_evctx_cond         	- new condition of the evctx object.
 *  @param  [in] ui4_data            		- determine the value according to e_evctx_cond.
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   				- Success
 *          			EVCTXR_INTERNAL_ERROR   - Cannot send message to the selector.
 *          			EVCTXR_NOT_INIT         	- The EVCTX component is not initialized yet.
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_evctx_eng_cond_change_nfy (
                HANDLE_T                h_event,
                EVCTX_COND_T            e_evctx_cond,
                UINT32                  ui4_data);


/*----------------------------------------------------------------------------*/
/*! @brief  a notification function that will be called by the Evctx selector when the Date/Time library
 *               calls back.
 *
 *  @param  [in] h_event          	- handle of evctx object
 *  @param  [in] pf_dt_nfy         	- Notification function provided by the caller.  which called when the system clock
 *                 					   is synchronized with the current time value from STT, or TOT, or TDT.
 *  @param  [in] pv_tag            	- A tag value provided by the caller. 
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   				- Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 x_evctx_eng_reg_dt_nfy_fct (
                HANDLE_T                h_event,
                x_evctx_dt_nfy_fct      pf_dt_nfy,
                VOID*                   pv_tag);


/*----------------------------------------------------------------------------*/
/*! @brief   un-registers a notification function that will be called by the Evctx selector when the Date/Time library
 *               calls back.
 *
 *  @param  [in] h_event          	- handle of evctx object
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   				- Success
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_evctx_eng_unreg_dt_nfy_fct (HANDLE_T   h_event);


/*----------------------------------------------------------------------------*/
/*! @brief   unifies the specified country code to the same case.
 *
 *  @param  [in] pt_country_code          - ISO-3166 country code
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   				- Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 x_evctx_eng_unify_country_code (
                ISO_3166_COUNT_T*   pt_country_code);

#endif /* _X_EVCTX_ENG_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                                             /* end of groupMW_EVCTX */
/*----------------------------------------------------------------------------*/

