/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef _X_HTTPC_API_H_
#define _X_HTTPC_API_H_

#include "u_httpc_api.h"


/***************************************************************************/
/* API to init HTTP module                                                 */
/***************************************************************************/
extern VOID    x_httpc_init_module(VOID);

/***************************************************************************/
/* Basic HTTP Client APIs                                                  */
/***************************************************************************/
extern VOID    x_httpc_data_init(HTTPC_DATA_T *pt_data);
extern VOID    x_httpc_data_free(HTTPC_DATA_T *pt_data);

extern HTTPCR_CODE_T   x_httpc_new_session(HANDLE_T *ph_handle);
extern HTTPCR_CODE_T   x_httpc_free_session(HANDLE_T h_file);
extern HTTPCR_CODE_T   x_httpc_get(HANDLE_T h_file, 
                                   const CHAR *ps_url, 
                                   HTTPC_DATA_T *pt_data);


extern HTTPCR_CODE_T   x_httpc_post(HANDLE_T      h_file,
                                    const CHAR   *ps_url,
                                    VOID         *pv_body,
                                    SIZE_T        z_body_size,
                                    HTTPC_DATA_T *pt_data);


extern HTTPCR_CODE_T   x_httpc_get_async(HANDLE_T                h_file,
                                         const CHAR             *ps_url,
                                         HTTPC_DATA_T           *pt_data, 
                                         x_httpc_async_fct    pf_nfy_fct,
                                         VOID                   *pv_tag,
                                         HANDLE_T               *ph_req);
extern HTTPCR_CODE_T x_httpc_head_async(HANDLE_T                h_file,
                        const CHAR             *ps_url,
                        HTTPC_DATA_T           *pt_data, 
                        x_httpc_async_fct    pf_nfy_fct,
                        VOID                   *pv_tag,
                        HANDLE_T               *ph_req);

INT32 x_httpc_get_last_modify_info(HANDLE_T      h_file,
                                            VOID*   pv_val);

extern HTTPCR_CODE_T   x_httpc_post_async(HANDLE_T             h_file,
                                          const CHAR          *ps_url,
                                          VOID                *pv_body,
                                          SIZE_T               z_body_size,
                                          HTTPC_DATA_T        *pt_data,
                                          x_httpc_async_fct pf_nfy_fct,
                                          VOID                *pv_tag,
                                          HANDLE_T            *ph_req);

extern HTTPCR_CODE_T    x_httpc_put(HANDLE_T      h_file,
                                    const CHAR   *ps_url,
                                    VOID         *pv_body,
                                    SIZE_T        z_body_size,
                                    HTTPC_DATA_T *pt_data);

extern HTTPCR_CODE_T    x_httpc_put_async(HANDLE_T            h_file,
                                          const CHAR         *ps_url,
                                          VOID               *pv_body,
                                          SIZE_T              z_body_size,
                                          HTTPC_DATA_T       *pt_data,
                                          x_httpc_async_fct   pf_nfy_fct,
                                          VOID               *pv_tag,
                                          HANDLE_T           *ph_req);


extern HTTPCR_CODE_T   x_httpc_append_header(HANDLE_T    h_file, const CHAR *ps_hdr);

extern HTTPCR_CODE_T   x_httpc_clear_append_header(HANDLE_T h_file);

extern HTTPCR_CODE_T   x_httpc_abort_async(HANDLE_T h_req);

extern HTTPCR_CODE_T   x_httpc_set_option(HANDLE_T h_handle, HTTP_HANDLE_OPT_T e_option, VOID* pv_val);

extern HTTPCR_CODE_T   x_httpc_get_info(HANDLE_T h_handle, HTTP_HANDLE_INFO_T e_option, VOID* pv_val);

extern HTTPCR_CODE_T   x_httpc_set_progress_callback(HANDLE_T h_handle, x_httpc_progress_fct pf_progrs_cb, VOID* pv_tag);

/***************************************************************************/
/* Basic HTTP streaming API                                                */
/***************************************************************************/
extern INT32 x_httpc_fm_open(const CHAR *ps_url, 
                             UINT32 ui4_buffer_size, 
                             HANDLE_T *ph_handle);

extern INT32 x_httpc_fm_open_proxy(const CHAR *ps_url, 
                            const CHAR *ps_proxy, 
                            UINT32 ui4_buffer_size, 
                            HANDLE_T *ph_handle);

extern INT32 x_httpc_fm_close(HANDLE_T h_file);

extern INT32 x_httpc_fm_read(HANDLE_T h_file, 
                             VOID *pv_data, 
                             UINT32 ui4_count, 
                             UINT32 *pui4_read);
#if 1/* MTK: Add by Jinlong */
extern INT32 x_httpc_fm_read_break(HANDLE_T h_file);
#endif

extern INT32 x_httpc_fm_read_async(HANDLE_T        h_file,
                                   VOID            *pv_data,
                                   UINT32          ui4_count,
                                   UINT8           ui1_pri,
                                   x_fm_async_fct  pf_nfy_fct,
                                   VOID            *pv_tag,
                                   HANDLE_T        *ph_req);

extern INT32 x_httpc_fm_abort_async(HANDLE_T h_file, HANDLE_T h_req);

extern INT32 x_httpc_fm_feof(HANDLE_T h_file, BOOL* pb_eof);

extern INT32 x_httpc_fm_lseek(HANDLE_T h_file, 
                              INT64 i8_offset, 
                              UINT8 ui1_whence, 
                              UINT64 *pui8_cur_pos);

extern INT32 x_httpc_fm_get_info_by_handle(HANDLE_T h_file, FM_FILE_INFO_T *pt_info);

extern INT32 x_httpc_fm_get_rsp_hdr(HANDLE_T h_file, CHAR *pc_hdr, UINT32 *pui4_rsp_hdr_len);

extern INT32 x_httpc_fm_set_progress_callback(HANDLE_T h_file, 
                                              x_httpc_fm_progress_fct pf_progrs_cb, VOID* pv_tag);
/*dont free the return value*/
extern CHAR* x_httpc_fm_get_content_type(HANDLE_T h_file);
extern CHAR* x_httpc_fm_get_server_name(HANDLE_T h_file);

extern INT32 x_httpc_fm_open_proxy_agent(const CHAR *ps_url, 
                            const CHAR *ps_proxy, 
                            const CHAR *ps_agent, 
                            UINT32 ui4_buffer_size, 
                            HANDLE_T *ph_handle);

extern INT32 x_httpc_fm_open_proxy_agent_timeout(const CHAR *ps_url, 
                            const CHAR *ps_proxy, 
                            const CHAR *ps_agent, 
                            UINT32 ui4_time_out,
                            UINT32 ui4_buffer_size, 
                            HANDLE_T *ph_handle);

extern INT32 x_httpc_fm_open_proxy_agent_timeout_cookie(const CHAR *ps_url, 
                            const CHAR *ps_proxy, 
                            const CHAR *ps_agent, 
                            UINT32 ui4_time_out,
                            UINT32 ui4_buffer_size, 
                            HANDLE_T *ph_handle,
                            const CHAR* ps_cookie);

extern INT32 x_httpc_fm_get_speed_download(HANDLE_T h_file, 
												 DOUBLE* d_speed);

extern INT32 x_httpc_fm_set_speed_download(HANDLE_T h_file, 
 													UINT32 ui4_speed);

#endif /*_X_HTTPC_API_H_*/

