/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_lol.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X/DTV_X_HQ_int/20 $
 * $SWAuthor: Pedro Tsai $
 * $MD5HEX: aa3109d410702d35a82d86c6386592c7 $
 *
 * Description:
 *         This header file contains Large Object list related API definitions,
 *         which are exported to other Middleware components.
 *
 *---------------------------------------------------------------------------*/

#ifndef _X_LOL_H_
#define _X_LOL_H_

#include "u_lol.h"

extern INT32 x_lol_create            (const CHAR*      s_lol_name,
                                      UINT16           ui2_lol_id,
                                      LOL_MEM_POLICY_T e_mem_policy,
                                      UINT16           ui2_max_nb_rec,
                                      VOID*            pv_tag,
                                      x_lol_nfy_fct    pf_nfy,
                                      HANDLE_T*        ph_lol);

extern  INT32 x_lol_delete           (HANDLE_T    	   h_lol);

extern  INT32 x_lol_open             (UINT16           ui2_lol_id,
                                      VOID*            pv_tag,
                                      x_lol_nfy_fct    pf_nfy,
                                      HANDLE_T*        ph_lol);

extern INT32 x_lol_close             (HANDLE_T         h_lol);

                                      
extern INT32 x_lol_clean             (HANDLE_T         h_lol);

extern INT32 x_lol_add_rec           (HANDLE_T         h_lol,
                                      const LOL_REC_T* pt_lol_rec);

extern INT32 x_lol_del_rec           (HANDLE_T         h_lol,
                                      UINT16           ui2_lol_rec_id);

extern INT32 x_lol_update_rec        (HANDLE_T         h_lol,
                                      const LOL_REC_T* pt_lol_rec,
                                      BOOL             b_must_exist);

extern INT32 x_lol_qry_rec_id        (HANDLE_T         h_lol,
                                      LOL_REC_ID_QRY_T e_lol_qry_option,
                                      UINT16*          pui2_rec_id);
     
extern INT32 x_lol_lock              (HANDLE_T         h_lol);

extern INT32 x_lol_unlock            (HANDLE_T         h_lol);

extern INT32 x_lol_read_lock         (HANDLE_T         h_lol);

extern INT32 x_lol_read_unlock       (HANDLE_T         h_lol);

extern INT32 x_lol_get_rec           (HANDLE_T         h_lol,
                                      UINT16           ui2_lol_rec_id,
                                      LOL_REC_T*       pt_lol_rec,
                                      UINT32*          pui4_ver_id);

extern INT32 x_lol_get_num_rec       (HANDLE_T            h_lol,
                                      UINT16*             pui2_num_recs,
                                      UINT32*             pui4_ver_id);

extern INT32 x_lol_get_rec_by_data_tag  (HANDLE_T         h_lol,
                                         UINT32           ui4_data_tag,
                                         UINT16		   ui2_idx,
                                         LOL_REC_T*       pt_lol_rec,
                                         UINT32*          pui4_ver_id);

extern INT32 x_lol_get_num_rec_by_data_tag(HANDLE_T            h_lol,
                                           UINT32              ui4_data_tag,
                                           UINT16*             pui2_num_recs,
                                           UINT32*             pui4_ver_id);

extern INT32 x_lol_rec_init          (LOL_REC_T*        pt_lol_rec);

/********************************************
   LOL data persistent storage related APIs
*********************************************/

extern INT32 x_lol_fs_load           (HANDLE_T            h_dir,
                                      const CHAR*         ps_lol_path,
                                      UINT16              ui2_lol_id,
                                      const CHAR*	      ps_lol_name,
                                      VOID*               pv_tag,
                                      x_lol_nfy_fct       pf_nfy,
                                      HANDLE_T*           ph_lol);

extern  INT32 x_lol_fs_store         (HANDLE_T            h_dir,
                                      const CHAR*         ps_lol_path,
                                      UINT16              ui2_lol_id);
	

extern  INT32 x_lol_fs_del           (HANDLE_T            h_dir,
                                      const CHAR*         ps_lol_path,
                                      UINT16              ui2_lol_id);



extern  INT32 x_lol_fs_qry           (HANDLE_T        h_dir,
                                      const CHAR*     ps_lol_path,
                                      UINT16          ui2_idx,
                                      UINT16*         pui2_lol_id,
                                      CHAR*           ps_lol_name);

extern  INT32 x_lol_fs_sync          (HANDLE_T        h_lol,
                                      HANDLE_T        h_dir,
                                      const CHAR*     ps_lol_path,
                                      UINT16          ui2_lol_id);

extern INT32 x_lol_init_mem_dump     (LOL_MEM_DUMP_T*        pt_mem_dump);
extern INT32 x_lol_init_mem_dump_64     (LOL_MEM_DUMP_EX_T*        pt_mem_dump);


extern INT32 x_lol_free_mem_dump     (LOL_MEM_DUMP_T*        pt_mem_dump);
extern INT32 x_lol_free_mem_dump_64     (LOL_MEM_DUMP_EX_T*        pt_mem_dump);


extern INT32 x_lol_dump_to_mem       (HANDLE_T               h_lol,
                                      LOL_MEM_DUMP_T*        pt_mem_dump);

extern INT32 x_lol_dump_to_mem_64       (HANDLE_T               h_lol,
                                      LOL_MEM_DUMP_EX_T*        pt_mem_dump);


extern INT32 x_lol_load_from_mem     (HANDLE_T               h_lol,
                                      LOL_MEM_DUMP_T*        pt_mem_dump);

extern INT32 x_lol_load_from_mem_64     (HANDLE_T               h_lol,
                                      LOL_MEM_DUMP_EX_T*        pt_mem_dump);


extern INT32 x_lol_set_cursor
(
    HANDLE_T            h_lol,
    LOL_ITERATE_T       e_pos
);

extern INT32 x_lol_read_cursor
(
    HANDLE_T            h_lol,
    LOL_REC_T*          pt_lol_rec,
    UINT32*             pui4_ver_id
);

extern INT32 x_lol_sort_rec
(
    HANDLE_T			h_lol,
    UINT32*             pui4_ver_id 
);

extern INT32 x_lol_fs_store_given_file_handle
(
    HANDLE_T        h_file,
    UINT16          ui2_lol_id  
);

extern
INT32 x_lol_fs_sync_given_file_handle
(
    HANDLE_T		h_lol,	
    HANDLE_T        h_file,
    UINT16		    ui2_lol_id
);
/******************************************************
Utility function to get information about Large Object list
*****************************************************/
INT32 x_lol_get_info                 (HANDLE_T        h_lol);


#endif /* _X_LOL_H_ */


