/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_mdb.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/DTV_X_ATSC/1 $
 * $SWAuthor: Yuchien Chen $
 * $MD5HEX: 34076dbf72c1e756a741024f89fac47f $
 *
 * Description: 
 *         This file contains File Manager exported API's.
 *---------------------------------------------------------------------------*/
 
#ifndef _X_MDB_H_
#define _X_MDB_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#ifdef _WIN32_
#include "x_os.h"
#else
#include "x_os.h"
#endif
#include "x_common.h"
#include "u_mtp.h"
#include "u_mdb.h"
#include "u_dm.h"
#include "x_dm.h"

/*-----------------------------------------------------------------------------
                    external functions declarations
 ----------------------------------------------------------------------------*/  
extern INT32 x_mdb_init(VOID);

extern INT32 x_mdb_uninit(VOID);

/*******************
  Virtual Tree Define related API
 *******************/ 
extern INT32 x_mdb_start_define_tree(VOID** ppv_tree);

extern INT32 x_mdb_config_category(VOID*               pv_tree,
                                   UINT8               ui1_category_idx,
                                   UINT32              ui4_max_item,
                                   UINT32              ui4_format_num,
                                   UINT16*             aui2_formats);

extern INT32 x_mdb_add_tree_node(VOID*               pv_tree,
                                 MDB_LAYOUT_NODE_T*  pt_node, 
                                 MDB_NODE_HND_T      t_parent,
                                 MDB_NODE_HND_T*     pt_new_node);
                            
extern INT32 x_mdb_finish_define_tree(VOID*    pv_tree);

extern INT32 x_mdb_virtual_tree_free(VOID*    pv_tree);

/*******************
  MTP database builder related API
 *******************/   
extern INT32 x_mdb_open(
                        VOID*           pt_cfg,
                        HANDLE_T*       ph_mdb);

extern INT32 x_mdb_register_async(HANDLE_T              h_mdb,
                                  x_mdb_async_fct       pf_async);

extern INT32 x_mdb_deregister_async(HANDLE_T            h_mdb,
                                    x_mdb_async_fct     pf_async);

extern INT32 x_mdb_config(HANDLE_T          h_mdb,
                          MDB_CONFIG_T*     pt_cfg);

extern INT32 x_mdb_close(HANDLE_T h_mdb);

extern INT32 x_mdb_status(HANDLE_T         h_mdb,
                          MDB_STATUS_T*    pe_status);

extern INT32 x_mdb_start(HANDLE_T     h_mdb);

extern INT32 x_mdb_pause(HANDLE_T     h_mdb);

extern INT32 x_mdb_resume(HANDLE_T    h_mdb);

extern INT32 x_mdb_percentage(HANDLE_T    h_mdb,
                              UINT8*      pui1_percentage);

extern INT32 x_mdb_query(HANDLE_T           h_mdb,
                         MDB_ITEM_HND_T*    at_item, 
                         UINT32             ui4_offset,
                         UINT32             ui4_query_num,
                         UINT32*            pui4_cnt, 
                         MDB_ITEM_INFO_T*   pt_item_info_list);

extern INT32 x_mdb_obj_prop_size(HANDLE_T           h_mdb,
                                 MDB_OBJ_HND_T      t_obj, 
                                 UINT16             ui2_prop_code,
                                 UINT32*            pui4_size);

extern INT32 x_mdb_obj_prop_value(HANDLE_T          h_mdb,
                                  MDB_OBJ_HND_T     t_obj, 
                                  UINT16            ui2_prop_code,
                                  UINT16*           pui2_data_type,
                                  UINT8*            pui1_buf);

extern INT32 x_mdb_obj_detail_info(HANDLE_T                 h_mdb,
                                   MDB_OBJ_HND_T            t_obj, 
                                   MDB_ITEM_DETAIL_INFO_T*  pt_info);


extern INT32 x_mdb_device_info(HANDLE_T             h_mdb,
                               MTP_DEVICE_INFO_T**  ppt_mdb_device_info);

extern INT32 x_mdb_storage_ids(HANDLE_T     h_mdb,
                               UINT32*      pui4_storage_num,
                               UINT32**     ppui4_storage_ids);

extern INT32 x_mdb_storage_info(HANDLE_T                h_mdb,
                                UINT32                  ui4_storage_id,
                                MTP_STORAGE_INFO_T**    ppt_mdb_storage_info);

#endif /* _X_MDB_H_ */


