/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*---------------------------------------------------------------------------*
 * Description:
 *    system startup
 *       IP stack -> Switch -> NI -> Drv
 *---------------------------------------------------------------------------*/
#ifndef X_NET_NI_IF_H
#define X_NET_NI_IF_H

#include "u_net_drv_if.h"
#include "u_net_ni_if.h"

/* platform debug API */
int
_inet_get_dbg_level(VOID);

void
_inet_set_dbg_level(int level);

char *
_inet_get_dbg_prefix(VOID);

#ifdef DBG_LEVEL_MODULE
#undef DBG_LEVEL_MODULE
#endif
#define DBG_LEVEL_MODULE _inet_get_dbg_level()

#ifdef DBG_MODULE_NAME_PREFIX
#undef DBG_MODULE_NAME_PREFIX
#endif
#define DBG_MODULE_NAME_PREFIX _inet_get_dbg_prefix()


/**
 * Layer 2 switch
 */
extern L2_SWITCH_T *x_net_switch_new    (CHAR *ps_name);
extern INT32        x_net_switch_add_default_ni  (L2_SWITCH_T *pt_Switch, INT32 i4_NiHandle);
extern INT32        x_net_switch_add_ni          (L2_SWITCH_T *pt_Switch, INT32 i4_NiHandle);
extern INT32        x_net_switch_add_learn_entry (L2_SWITCH_T *pt_Switch, UINT8 *pac_Mac, INT32 i4_NiHandle);
extern INT32        x_net_swtich_find_learn_entry(L2_SWITCH_T *pt_Switch, UINT8 *pac_Mac);
extern INT32        x_net_switch_transmit_packet(L2_SWITCH_T *pt_Switch, PKT_BUFF_T *pt_packet, INT32 i4_NiHandle);

/**
 * Network interface utility
 */
extern NET_NI_T    *x_net_ni_new(CHAR *ps_name, INT32 i4_ifType);
extern INT32        x_net_ni_install_drv_entry(NET_NI_T *pt_ni, NetworkDriverEntry pf_fn);
extern INT32        x_net_ni_start(NET_NI_T *pt_ni);
extern INT32        x_net_ni_stop(NET_NI_T *pt_ni);
extern INT32        x_net_ni_install_transmit_packet_callback(NET_NI_T *pt_ni, net_ni_transmit_pkt pf_fn);
extern INT32        x_net_ni_install_receive_packet_callback(NET_NI_T *pt_ni, net_ni_receive_pkt pf_fn);
extern INT32        x_net_ni_packet_indication(NET_DRV_IF_T *pt_drv, PKT_BUFF_T *pt_pkb);
extern INT32        x_net_ni_tx_completed(NET_DRV_IF_T *pt_drv, PKT_BUFF_T *pt_pkb);
extern INT32        x_net_ni_rx_packet(NET_NI_T *pt_ni, PKT_BUFF_T *pt_buff);
extern INT32        x_net_ni_tx_packet(NET_NI_T *pt_ni, PKT_BUFF_T *pt_buff);
extern NET_NI_T    *x_net_ni_get_by_name(CHAR *ps_name);
extern VOID         x_net_ni_list_reset(VOID);
extern INT32        x_net_ni_set_mac(CHAR *psz_name, CHAR *pc_mac);
extern INT32        x_net_ni_save_mac (const CHAR *psz_name, UINT8 *pui1_mac);
extern INT32        x_net_ni_get_mac(CHAR *psz_name, CHAR *pac_mac);
extern INT32        x_net_ni_set_if_type(NET_NI_T *pt_ni, INT32 i4_type);
extern NET_NI_T    *x_net_ni_list_get(VOID);
extern INT32        x_net_ni_add_assocate_ni(NET_NI_T *pt_ni1, NET_NI_T *pt_ni2);
extern VOID         x_net_ping(INT32 i4_ip_version, UINT32 ui4_src_ip, UINT32 ui4_dest_ip, INT16 i2_try_cnt, INT32 i4_size, INT16 i2_interval);
extern VOID         x_net_ni_show_by_name(CHAR *psz_name);
extern VOID         x_net_ni_show_all(VOID);
extern VOID         x_net_ping4 (CHAR * ps_ip, INT32  i4_len, INT16  i2_wait, VOID (*notify_fn) (INT16 i2_rsp_time));
extern VOID         x_net_ni_debug(VOID);
extern INT32        x_net_ni_add_pkt_hook(CHAR *ps_niName, INT32 i4_priority, net_pkt_hook_fn fn);
extern INT32        x_net_discard_netbios_service_pkt(PKT_BUFF_T *pt_buff);

/* if information get */
extern INT32        x_net_ni_get(CHAR *ps_niName, INT32 i4_cmd, VOID *pv_param);
extern INT32        x_net_ni_set(CHAR *ps_niName, INT32 i4_cmd, VOID *pv_param);

extern INT32        x_net_ni_reg_ev_notify(CHAR *ps_ni_name, NI_DRV_EV_T e_ev, ni_ev_notify_fnct pf_notify);
extern INT32        x_net_ni_reg_ev_notify_ifname(CHAR* ps_ni_name,NI_DRV_EV_T e_ev,  ni_ev_notify_fnct_ifname pf_notify);
extern INT32        c_net_ni_reg_ev_notify_ifname(CHAR *ps_ni_name, NI_DRV_EV_T e_ev, ni_ev_notify_fnct_ifname pf_notify);

extern INT32        x_net_ni_unreg_ev_notify(CHAR *ps_ni_name, NI_DRV_EV_T e_ev, ni_ev_notify_fnct pf_notify);
extern INT32        x_net_ni_ev_send(NET_NI_T *pt_ni, NI_DRV_EV_T e_ev);

/**
 * x_net_if_join
 *
 * Add multicast IP address to interface
 *
 * @param ps_niName
 * @param ui4_ip, the IP address (network order)
 *
 * @return INT32
 * NET_OK, ok
 * NET_NI_NOT_EXIST, ni not exist
 */
extern INT32 x_net_if_join(const CHAR* ps_niName, UINT32 ui4_ip);

/**
 * x_net_if_leave
 *
 * leave a multicast group
 *
 * @param ps_niName
 * @param ui4_ip, ip address (network order)
 *
 * @return INT32
 */
extern INT32 x_net_if_leave(const CHAR* ps_niName, UINT32 ui4_ip);

/**
 * x_net_if_multi_list
 *
 * get multicast list
 *
 * @param ps_niName
 * @param paui4_ip, local buffer for multicast IP
 * @param i4_size, size of paui4_ip
 *
 * @return INT32
 * >0, the # set of multicast IP
 */
extern INT32 x_net_if_multi_list(const CHAR* ps_niName, UINT32 *paui4_ip, INT32 i4_size);

/**
 * x_net_get_mac_by_ip
 *
 * get MAC by IP address
 *
 * @param ps_ip, IP address string
 * @param puac_mac, output, need allocate by users. UCHAR mac[6]
 *
 * @return 0 OK, else failure.
 */
extern INT32 x_net_get_mac_by_ip (const CHAR *ps_ip, UCHAR *puac_mac);

#endif


