/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file c_hbbtv.h
 *  $RCSfile: $
 *  $Revision$
 *  $Date$
 *  $Author$
 *
 *  @par Description: This header file declares HBBTV exported APIs.
 */
/*----------------------------------------------------------------------------*/
#ifndef _X_OPERA_HBBTV_H_
#define _X_OPERA_HBBTV_H_

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_handle.h"
#include "u_gl.h"

#include "u_opera_hbbtv.h"

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
#define HBBTV_KEY_STATE_DOWN          ((UINT32) 0)        /* key down state */
#define HBBTV_KEY_STATE_UP            ((UINT32) 1)        /* key up state   */
#define HBBTV_KEY_STATE_REPEAT        ((UINT32) 2)        /* key repeat     */

typedef INT32   OPERA_HBBTV_BWS_MSG_TYPE_T;
#define OPERA_HBBTV_BWS_MSG_UNKNOWN          ((OPERA_HBBTV_BWS_MSG_TYPE_T)  0)
#define OPERA_HBBTV_BWS_MSG_IS_HBBTV_TYPE    ((OPERA_HBBTV_BWS_MSG_TYPE_T)  1)
#define OPERA_HBBTV_BWS_MSG_EXIT_HBBTV_TYPE  ((OPERA_HBBTV_BWS_MSG_TYPE_T)  2)
#define OPERA_HBBTV_BWS_MSG_LANG_CHG         ((OPERA_HBBTV_BWS_MSG_TYPE_T)  3)
#define OPERA_HBBTV_BWS_MSG_CLEAR_COOKIE     ((OPERA_HBBTV_BWS_MSG_TYPE_T)  4)
#define OPERA_HBBTV_BWS_MSG_EXIT_BROWSER     ((OPERA_HBBTV_BWS_MSG_TYPE_T)  5)

/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
/* Internal function declarations */
extern INT32 x_opera_hbbtv_init(VOID);

extern INT32 x_opera_hbbtv_reset(VOID);

extern INT32 x_opera_hbbtv_browser_is_alive(
    BOOL*               pb_is_alive);

extern INT32 x_opera_hbbtv_goto_url(
    const CHAR*                 s_uri );

extern INT32 x_opera_hbbtv_osd_update(
    GL_HSURFACE_T       h_surface,
    GL_RECT_T *         pt_rect );
extern INT32 x_opera_hbbtv_dvb_svl_update(
                                    HANDLE_T   h_dvb_ctx,
                                    VOID*      data,
                                    UINT32     data_length
                                    );

/* External function declarations */
extern BOOL x_opera_hbbtv_support( VOID );

extern INT32 x_opera_hbbtv_start( VOID );

extern INT32 x_opera_hbbtv_stop( VOID );

extern INT32 x_opera_hbbtv_exit_process( VOID );

extern INT32 x_opera_hbbtv_create_app(
    BOOL                        b_brdcst_related,
    const CHAR*                 s_uri );

extern INT32 x_opera_hbbtv_init_osd(
    HANDLE_T            h_canvas_hbbtv);

extern INT32 x_opera_hbbtv_show( VOID );

extern INT32 x_opera_hbbtv_hide( VOID );

extern INT32 x_opera_hbbtv_wgl_set_focus( VOID );

extern INT32 x_opera_hbbtv_osd_reg_nfy(
    x_opera_hbbtv_osd_nfy_fct   pf_nfy,
    VOID*                       pv_tag );

extern INT32 x_opera_hbbtv_osd_show_reg_nfy(
    x_opera_hbbtv_osd_show_nfy_fct  pf_nfy,
    VOID*                           pv_tag );

extern INT32 x_opera_hbbtv_key_press(
    UINT32                      ui4_key_code,
    UINT32                      ui4_msg );

extern INT32 x_opera_hbbtv_set_country(
    ISO_3166_COUNT_T            s3166_country );

extern INT32 x_opera_hbbtv_set_gui_lang (
    ISO_639_LANG_T              s639_lang );

extern INT32 x_opera_hbbtv_set_audio_desc(
    BOOL                        b_ad_enabled );

extern INT32 x_opera_hbbtv_set_default_audio_lang(
    ISO_639_LANG_T              s639_lang );

extern INT32 x_opera_hbbtv_set_default_subtitle_lang(
    ISO_639_LANG_T                  s639_lang,
    OPERA_HBBTV_SUBTITLE_ENABLE_T   e_enable );

extern INT32 x_opera_hbbtv_set_default_subtitle_attr(
    OPERA_HBBTV_SUBTITLE_ATTR_T     e_attr );

extern INT32 x_opera_hbbtv_subtitle_get_prefer (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb,
    size_t *ntotal);

extern INT32 x_opera_hbbtv_subtitle_set_prefer (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb);

extern INT32 x_opera_hbbtv_subtitle_get_list (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb,
    size_t *ntotal);

extern INT32 x_opera_hbbtv_subtitle_set_active (
    HBBTV_STREAM_SUBTITLE_T *pSubtitle,
    size_t nmemb);

extern INT32 x_opera_hbbtv_stm_get_audio_num(
    UINT16*                         pui2_num );

extern INT32 x_opera_hbbtv_stm_get_audio_lang(
    UINT16                          ui2_idx,
    ISO_639_LANG_T                  s639_lang );

extern INT32 x_opera_hbbtv_stm_get_audio(
    UINT16*                         pui2_idx );

extern INT32 x_opera_hbbtv_stm_set_audio(
    UINT16                          ui2_idx );

extern INT32 x_opera_hbbtv_set_next_audio(VOID);

extern INT32 x_opera_hbbtv_stm_set_visibility(
    BOOL                            b_visible );

extern INT32 x_opera_hbbtv_set_rating_age(
    UINT8                           ui8_rating_age );
extern INT32 x_opera_hbbtv_set_privacy(
    OPERA_HBBTV_PRIVACY_TYPE_T ui2_privacy_type,
    OPERA_HBBTV_PRIVACY_E privacy_cfg);

extern INT32 x_opera_hbbtv_device_id_cfg(OPERA_HBBTV_DEVICEID_CFG_E e_dev_id_cfg);

extern INT32 x_opera_hbbtv_set_drm_priority_array (
    OPERA_HBBTV_DRM_PRIO_ARR_T*     pt_drm_prio_arr );

extern INT32 x_opera_hbbtv_clear_cookie(VOID);

extern INT32 x_opera_hbbtv_browser_goto_url_reg_nfy(
    x_opera_hbbtv_layers_goto_url   pf_nfy,
    VOID*                           pv_tag );

extern INT32 x_opera_hbbtv_create_app_reg_nfy(
    x_opera_hbbtv_layers_create_app pf_nfy,
    VOID*                           pv_tag );

extern INT32 x_opera_hbbtv_input_keypress_reg_nfy(
    x_opera_hbbtv_layers_input_keypress pf_nfy,
    VOID*                               pv_tag );

extern INT32 x_opera_hbbtv_reg_browser_com_nfy(
        x_opera_hbbtv_browser_com_nfy_fct   pf_nfy_fct,
        VOID*                               pv_nfy_tag);

extern INT32 x_opera_hbbtv_release_res( VOID );

extern INT32 x_hbbtv_get_mode(
    UINT32*                     pui4_hbbtv_mode_mask );

extern INT32 x_hbbtv_set_mode(
    UINT32                      ui4_hbbtv_mode_mask );

extern INT32 x_hbbtv_get_mime_type_support(
    const CHAR*                 s_mime_type,
    BOOL*                       pb_supported );

extern INT32 x_hbbtv_set_app_svc_type(
    HBBTV_APP_SVC_TYPE_T              e_app_svc );

extern INT32 x_hbbtv_get_app_svc_type(
    HBBTV_APP_SVC_TYPE_T*             pe_app_svc );

extern INT32 x_hbbtv_stop_3rd_app(VOID);

extern INT32 x_hbbtv_launch_app(
    HBBTV_APP_INFO_T*               pt_hbbtv_app_info,
    x_hbbtv_app_event_nfy_fct       pf_launch_status_nfy,
    VOID*                           pv_tag );

extern INT32 x_hbbtv_kill_app(
    HBBTV_APP_KILL_INFO_T*          pt_app_kill_info);

extern INT32 x_hbbtv_get_app_type(
    HBBTV_APP_SRC_TYPE_T*           pe_app_type );

extern INT32 x_hbbtv_reg_app_event_listener(
        x_hbbtv_app_event_nfy_fct   pf_nfy_fct,
        VOID*                       pv_nfy_tag);

extern INT32 x_hbbtv_unreg_app_event_listener(
        x_hbbtv_app_event_nfy_fct   pf_nfy_fct,
        VOID*                       pv_nfy_tag);

extern INT32 x_hbbtv_app_status_nfy(
        INT32                       i4_msg_type,
        VOID*                       pv_data,
        SIZE_T                      z_data_len );
extern INT32 x_hbbtv_set_browser_status(
    HBBTV_BRWS_STATUS_E s);

extern INT32 x_hbbtv_fvp_reset(unsigned long long timesec);

#endif /* _X_OPERA_HBBTV_H_ */
