/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_rec_demux.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 *
 * Description:
 *         This header file contains Record Demux driver specific definitions,
 *         which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_REC_DEMUX_H_
#define _X_REC_DEMUX_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/* Get operations */
#define RECDEMUX_GET_TYPE_PID               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  0))
#define RECDEMUX_GET_TYPE_CTRL              (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)  1))

/* Set operations */
#define RECDEMUX_SET_TYPE_PID              ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  0)) | RM_SET_TYPE_ARG_NO_REF)
#define RECDEMUX_SET_TYPE_DEST_ID           (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  1))
#define RECDEMUX_SET_TYPE_NFY_FCT           (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  2))
#define RECDEMUX_SET_TYPE_CTRL             ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  3)) | RM_SET_TYPE_ARG_NO_REF)
#define RECDEMUX_SET_TYPE_SCRS_NFY_FCT      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  4))
#define RECDEMUX_SET_TYPE_STREAM_TYPE_MASK ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)  5)) | RM_SET_TYPE_ARG_NO_REF)

/* record demux condition */
typedef enum _RECDEMUX_COND_T
{
    RECDEMUX_COND_ERROR,
    RECDEMUX_COND_SCRS    
} RECDEMUX_COND_T;

typedef enum
{
    RECDEMUX_SCRS_UNKNOWN = 0,
    RECDEMUX_SCRS_CLEAR,
    RECDEMUX_SCRS_SCRAMBLED
}   RECDEMUX_SCRS_T;

/* Notify function */
typedef BOOL (*x_recdemux_nfy_fct) (VOID*            pv_nfy_tag,
                                    RECDEMUX_COND_T  e_nfy_cond,
                                    UINT32           ui4_data);


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/

/* Destination type and id info */
typedef struct _RECDEMUX_DEST_INFO_T
{
    DRV_TYPE_T      e_type;

    UINT16          ui2_id;
    
}   RECDEMUX_DEST_INFO_T;

/* Notify info setting */
typedef struct _RECDEMUX_NFY_INFO_T
{
    VOID*               pv_tag;

    x_recdemux_nfy_fct  pf_recdemux_nfy;
    
} RECDEMUX_NFY_INFO_T;


#endif /* _X_REC_DEMUX_H_ */
