/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_rrctx_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/DTV_X_ATSC/1 $
 * $SWAuthor: Justin Deng $
 * $MD5HEX: 9969fe0123011f2e59aede7557663436 $
 *
 * Description:
 *         This contains RRCTX library macros, definitons, structures,
 *         and API's, which are shared with other modules.
 *---------------------------------------------------------------------------*/

#ifndef _X_RRCTX_ENG_H_
#define _X_RRCTX_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_rrctx_eng.h"

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_RRCTX
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/
 /*----------------------------------------------------------------------------*/
/*! @brief  register the engine to the rrctx selecter
 *
 *  @param  [in] e_rating_type          			- rating type
 *  @param  [in] pt_eng_fct_tbl          			- pointer of engine function table
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		RRCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/
extern INT32 x_rrctx_reg_engine (
                RRCTX_RATING_TYPE_T     e_rating_type,
                RRCTX_ENG_FCT_TBL_T*    pt_eng_fct_tbl);

/*----------------------------------------------------------------------------*/
/*! @brief  lock the h_rrctx_base_ctrl_list
 *
 *  @note   
 *
 *  @see
 */
/*----------------------------------------------------------------------------*/

extern VOID x_rrctx_base_ctrl_list_lock (VOID);

/*----------------------------------------------------------------------------*/
/*! @brief  unlock the h_rrctx_base_ctrl_list
 *
 *  @note   
 *
 *  @see
 */
/*----------------------------------------------------------------------------*/

extern VOID x_rrctx_base_ctrl_list_unlock (VOID);

/*----------------------------------------------------------------------------*/
/*! @brief  active the engine task
 *
 *  @param  [in] e_rating_type          			- rating type
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		RRCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_rrctx_activate_eng_task (RRCTX_RATING_TYPE_T e_rating_type);

/*----------------------------------------------------------------------------*/
/*! @brief  get the date of engine
 *
 *  @param  [in] h_rating          		- handle of rrctx object
 *  @param  [out] ppv_eng_data         - date of engine
 *  
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		RRCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_rrctx_get_engine_data (
                HANDLE_T                h_rating,
                VOID**                  ppv_eng_data);

/*----------------------------------------------------------------------------*/
/*! @brief  a notification function that will be called by the Rrctx selector when the condition or 
 *		   statement of engine changed.
 *
 *  @param  [in] h_rating          		- handle of rrctx object
 *  @param  [in] ui4_reason         	- reason of the condition changed.
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		RRCTXR_OK   				- Success
 *          			RRCTXR_INTERNAL_ERROR   	- Cannot send message to the selector.
 *          			RRCTXR_NOT_INIT         		- The RRCTX component is not initialized yet.
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_rrctx_eng_cond_change_nfy (
                HANDLE_T                h_rating,
                UINT32                  ui4_reason);

/*----------------------------------------------------------------------------*/
/*! @brief  a notification function that will be called by the Rrctx selector when the brdcast type changed.
 *
 *  @param  [in] e_rating_type          		- rating type
 *  @param  [in] ui4_reason         		- reason of the brdcast type changed.
 *  @note   
 *
 *  @see
 *  @return		INT32
 *  @retval		RRCTXR_OK   				- Success
 *          			RRCTXR_INTERNAL_ERROR   	- Cannot send message to the selector.
 *          			RRCTXR_NOT_INIT         		- The RRCTX component is not initialized yet.
 */
/*----------------------------------------------------------------------------*/

extern INT32 x_rrctx_eng_cfg_change_brdct_nfy (
                RRCTX_RATING_TYPE_T     e_rating_type,
                UINT32                  ui4_reason);

#endif /* _X_RRCTX_ENG_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                               /* end of groupMW_RRCTX                            */
/*----------------------------------------------------------------------------*/


