/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_rrctx_rrt_eng.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision:  $
 * $SWAuthor: Clear Case Administrator $
 * $MD5HEX: 2020cb37c9f4fb6d1974080187cb506c $
 *
 * Description:
 *         This header file contains exported definitions and delcarations of
 *         RRT engine of RRCTX.
 *---------------------------------------------------------------------------*/
 
#ifndef _X_RRCTX_RRT_ENG_H_
#define _X_RRCTX_RRT_ENG_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_rrctx_rrt_eng.h"
 
/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_RRCTX
 *  @{ 
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    functions declaraions
 ----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*! @brief	init the rrt engine of rrctx.
 *
 *	@note	
 *
 *	@see
 *	@return 	INT32
 *	@retval   RRCTXR_OK					- Success
 *			RRCTXR_DUP_BRDCST_TYPE	- There's already an engine registered with the same broadcast type.
 *			RRCTXR_INTERNAL_ERROR		- Internal error happened.		  
 *			RRCTXR_INV_ARG				- Invalid argument.
 *			RRCTXR_NOT_INIT 			- The RRCTX component is not initialized yet.
 */
/*----------------------------------------------------------------------------*/

INT32 x_rrctx_rrt_eng_init (VOID);

/*----------------------------------------------------------------------------*/
/*! @brief  get rrctx info length function

 *  @param  [in] h_rating                		- handle of rrctx object
 *  @param  [in] e_rrt_key_type             	- rrctx key type
 *  @param  [in] pv_rrt_key_param            - cntains the auxiliary data to get the requested rating info. 
 *
 *  @param  [out] pz_rating_info_len    	- pointer of rrctx information length
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

INT32 x_rrctx_rrt_get_info_len (
                    HANDLE_T                h_rating,
                    RRCTX_RRT_KEY_TYPE_T    e_rrt_key_type,
                    VOID*                   pv_rrt_key_param,
                    SIZE_T*                 pz_rating_info_len);

/*----------------------------------------------------------------------------*/
/*! @brief  get rrctx information  function

 *  @param  [in] h_rating                		- handle of rrctx object
 *  @param  [in] e_rrt_key_type             	- rrctx key type
 *  @param  [in] pv_rrt_key_param            - cntains the auxiliary data to get the requested rating info. 
 *
 *  @param  [out] pz_rating_info_len    	- pointer of rrctx information length
 *  @param  [out] pv_rating_info    		- pointer of rrctx information 
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

INT32 x_rrctx_rrt_get_info (
                    HANDLE_T                h_rating,
                    RRCTX_RRT_KEY_TYPE_T    e_rrt_key_type,
                    VOID*                   pv_rrt_key_param,
                    SIZE_T*                 pz_rating_info_len,
                    VOID*                   pv_rating_info);

/*----------------------------------------------------------------------------*/
/*! @brief  reload the TM root.

 *  @param  [in] ps_tuner_name                	- tuner name
 *  @param  [in] b_force_open             	- force to reload or not
 *
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

INT32 x_rrctx_rrt_reload_tm_root(
                    const CHAR*             ps_tuner_name,
                    BOOL                    b_force_open);

/*----------------------------------------------------------------------------*/
/*! @brief  reload the MGT table.
 *
 *  
 *  @note   

 *  @see
 *  @return		INT32
 *  @retval		EVCTXR_OK   - Success
 */
/*----------------------------------------------------------------------------*/

INT32 x_rrctx_rrt_reload_mgt(VOID);

#endif /* _X_RRCTX_RRT_ENG_H_ */

/*----------------------------------------------------------------------------*/
/*! @} */                               /* end of groupMW_RRCTX                            */
/*----------------------------------------------------------------------------*/


