/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_sb.h,v $
 * $Revision: #2 $
 * $Date: 2015/06/01 $
 * $Author: cheng.liu $
 * $CCRevision: /main/DTV_X_HQ_int/11 $
 * $SWAuthor: Iolo Tsai $
 * $MD5HEX: b8a039cd49cf517913bb5080795df307 $
 *
 * Description:
 *         This header file contains SVL Builder specific
 *         definitions, which are shared with other modules.
 *---------------------------------------------------------------------------*/
#ifndef _X_SB_H_
#define _X_SB_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_SVL_BLDR
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/

#include "u_sb.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declaraions
 ----------------------------------------------------------------------------*/

BOOL x_sb_is_inited(VOID);

INT32 x_sb_open_builder(
    const CHAR*         ps_builder_name,
    VOID*               pv_open_data,
    VOID*               pv_nfy_tag,
    x_sb_nfy_fct        pf_nfy,
    HANDLE_T*           ph_builder,
    SB_COND_T*          pe_builder_cond);

INT32 x_sb_close_builder(
    HANDLE_T            h_builder);

INT32 x_sb_start_scan(
    HANDLE_T            h_builder,
    VOID*               pv_scan_data,
    VOID*               pv_nfy_tag,
    x_sb_scan_nfy_fct   pf_nfy,
    SB_COND_T*          pe_builder_cond);

INT32 x_sb_cancel_scan(
    HANDLE_T            h_builder);

INT32 x_sb_save_lists(
    HANDLE_T            h_builder,
    const CHAR*         ps_file_name);

INT32 x_sb_load_lists(
    HANDLE_T            h_builder,
    const CHAR*         ps_file_name);

INT32 x_sb_clean_lists(
    HANDLE_T            h_builder,
    const CHAR*         ps_file_name);

INT32 x_sb_get(
    HANDLE_T            h_builder,
    SB_KEY_TYPE_T       e_key_type,
    VOID*               pv_key_info,
    SIZE_T*             pz_key_data_len,
    VOID*               pv_key_data);

INT32 x_sb_set(
    HANDLE_T            h_builder,
    SB_KEY_TYPE_T       e_key_type,
    VOID*               pv_key_info,
    SIZE_T              z_key_data_len,
    VOID*               pv_key_data);

/* cliu 20090706,  Declaration*/
INT32 x_sb_cancel_scan_sync(
    HANDLE_T            h_builder);

INT32 x_sb_reg_scan_map(
    const CHAR*          ps_name,
    VOID*                pv_nfy_fct,
    VOID*                pv_data);


INT32 x_sb_get_cond(
    HANDLE_T            h_builder,
    SB_COND_T*          pe_builder_cond);

UINT16 x_sb_get_dbg_level(VOID);

BOOL x_sb_set_dbg_level(
    UINT16              ui2_level);

UINT32 x_sb_gen_channel_id(
      UINT32                     major,
      UINT32                     minor,
      UINT32                     index);

UINT32 x_sb_gen_one_part_channel_id(
      UINT32                              major,
      UINT32                              index);

UINT32 x_sb_gen_non_psip_channel_id(
      UINT32                              major,
      UINT32                              minor,
      UINT32                              index);

BOOL x_sb_is_one_part_channel_id(UINT32 channel_id);

BOOL x_sb_is_non_psip_channel_id(UINT32 channel_id);

UINT16 x_sb_get_major_channel_num(UINT32 channel_id);

UINT16 x_sb_get_minor_channel_num(UINT32 channel_id);

UINT8 x_sb_get_channel_index(UINT32 channel_id);

UINT32 x_sb_max_channel_index(VOID);

UINT16 x_sb_get_cable_last_channel(VOID);

INT32 sb_init (const THREAD_DESCR_T*   pt_sb_thread_descr);

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_SVL_BLDR */
/*----------------------------------------------------------------------------*/

#endif /* _X_SB_H_ */

