/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

 /*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_MM_SBTL_HDLR
 *  @{ 
 */
/*----------------------------------------------------------------------------*/
#ifndef _X_SBTL_IPCC_H_
#define _X_SBTL_IPCC_H_
#ifndef __KERNEL__

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
//#include "x_handle.h"
//#include "x_sm.h"
//#include "u_mm_sbtl_engine.h"
#include "u_sbtl_ipcc.h"
#include "x_handle.h"
#include "x_sm.h"
#include "u_mm_sbtl_engine.h"


/*-----------------------------------------------------------------------------
                    function declaration
-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to open ipcc
 *
 * @param [in]      h_canvas        - The handle of canvas
 * @param [in]      pv_cmd_sets     - The pointer of arg array
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK               - Routine successful.
 * @retval MMSBTLR_WGL_FAIL         - Routine Failed.
 * @retval MMSBTLR_OUT_OF_HANDLE    - Handle is invalid
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_open_fct(HANDLE_T h_canvas, VOID* pv_cmd_sets, BOOL b_atsc3);


/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to close ipcc
 *
 * @param [in]      None        
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK               - Routine successful.
 * @retval MMSBTLR_WGL_FAIL         - Routine Failed.
 *
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_close_fct(BOOL b_atsc3);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to set attributes of ipcc
 *
 * @param [in]      e_set_type          - The type of attribute
 * @param [in]      pv_set_info         - The info of attribute
 * @param [in]      z_set_info_len      - The length of attribute
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK                   - Routine successful.
 * @retval MMSBTLR_WGL_FAIL             - Routine Failed.
 * @retval MMSBTLR_INV_ARG              - Invalid argument.
 * @retval MMSBTLR_INSUFFICIENT_SIZE    - Memory is insufficient.
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_set_fct(SBTL_IPCC_SET_TYPE e_set_type, VOID* pv_set_info, SIZE_T z_set_info_len,BOOL b_atsc3);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to get attributes of ipcc
 *
 * @param [in]      e_get_type          - The type of attribute
 * @param [in]      pv_get_info         - The info of attribute
 * @param [in]      z_get_info_len      - The length of attribute
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK                   - Routine successful.
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_get_fct(SBTL_IPCC_GET_TYPE e_get_type, VOID* pv_get_info, SIZE_T z_get_info_len);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to remove window of ipcc
 *
 * @param [in]      ps_crnt_wnd         - The id of <region>
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK                   - Routine successful.
 * @retval MMSBTLR_INV_ARG              - Invalid argument.
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_remove(CHAR* ps_crnt_wnd);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to init IPCC
 *
 * @param [in]      pt_ipcc_config      - The config of IPCC
 * @param [out]     None
 * @return return the error code
 * @retval MMSBTLR_OK                   - Routine successful.
 * @retval MMSBTLR_INV_ARG              - Invalid argument.
 */
/* ----------------------------------------------------------------------------*/
extern INT32 x_sbtl_ipcc_init(SBTL_IPCC_CONFIG_T* pt_ipcc_config);




/*---------ATSC3.0 WebSocket API-----------------*/

/*------------------------------------------------------------------*/
/*! @brief  This API is the call back function for sbtl cc engine. 
 *  @param [in] ui1_enable -    The preference of cc ..
 *  @param [in] pv_tag -        The function tag which is used when the call back function is called to mach the sequence of the call back function.
 *  @note
 *  @see
 *  @return    None.
 *  @retval   -None.
 */
/*------------------------------------------------------------------*/
typedef VOID (*sbtl_ipcc_preference_callback_fct)(MM_SBTL_ATSC30_CC_PREFERNCE_T* pt_preference, VOID* pv_cc_tag);

extern INT32 x_sbtl_ipcc_ATSC3_wgt_reg_preference(sbtl_ipcc_preference_callback_fct pf_nfy, VOID* pv_tag);


#endif/* _U_SBTL_IPCC_H_ */
#endif
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_STRM_MNGR_MM_SBTL_HDLR*/
/*----------------------------------------------------------------------------*/




