/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_STRM_MNGR_MM_SBTL_HDLR
*  @{ 
*/
/*----------------------------------------------------------------------------*/
#ifndef _X_SBTL_IPCC_UTIL_H_
#define _X_SBTL_IPCC_UTIL_H_
#ifndef __KERNEL__

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "x_os.h"


#include "u_sbtl_ipcc.h"
#include "_sbtl_ipcc_common.h"


/*-----------------------------------------------------------------------------
                    function declaration
-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to divide ps_str by ps_rgx
 *
 * @param [in]      ps_rgx        - String separator
 * @param [in]      ps_str        - String which will be splited
 * @param [in]      ps_rslt       - Save the result after split
 * @param [in]      i4_len        - The length of ps_rslt, if ps_rslt is a pointer of array, then i4_len will be the length of array
 * @param [in]      ps_rest_str   - Save the rest string after splited
 * @param [out]     ps_rslt       - Save the result after split  
 * @return return the error code
 * @retval MMSBTLR_OK              - Routine successful.
 * @retval MMSBTLR_INV_ARG         - Invalid para.
 * @retval MMSBTLR_NOT_FOUND       - ps_rgx was not found in ps_str
 */
/* ----------------------------------------------------------------------------*/
INT32 x_sbtl_ipcc_str_split(CHAR* ps_rgx, CHAR* ps_str, CHAR** ps_rslt, INT32 i4_len, CHAR* ps_rest_str);



/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to convert string to float
 *
 * @param [in]      ps_src_str        - String which will be convert
 * @param [out]     pf_rslt           - The result after convert  
 * @return return the error code
 * @retval MMSBTLR_OK              - Routine successful.
 * @retval MMSBTLR_INV_ARG         - Invalid para.
 */
/* ----------------------------------------------------------------------------*/
INT32 x_sbtl_ipcc_str_to_float(CHAR* ps_src_str, FLOAT* pf_rslt);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to detect lenght unit
 *
 * @param [in]      ps_str          - String which will be detect
 * @param [out]     pe_origin_type  - The result after detect  
 * @return return the error code
 * @retval MMSBTLR_OK              - Routine successful.
 * @retval MMSBTLR_INV_ARG         - Invalid para.
 */
/* ----------------------------------------------------------------------------*/
INT32 x_sbtl_ipcc_util_detect_length_unit(CHAR* ps_str, TTML_LENGTH_UNIT_TYPE* pe_origin_type);




/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to abstract value from string
 *
 * @param [in]      ps_string          - String which will be abstract
 * @param [in]      pe_length_unit     - Length unit
 * @param [out]     ps_row             -   
 * @param [out]     ps_col             - 
 * @return return the error code
 * @retval MMSBTLR_OK              - Routine successful.
 * @retval MMSBTLR_INV_ARG         - Invalid para.
 */
/* ----------------------------------------------------------------------------*/
INT32 x_sbtl_ipcc_util_abstract_value(CHAR* ps_string, TTML_LENGTH_UNIT_TYPE* pe_length_unit, 
                                        CHAR* ps_row, CHAR* ps_col);





/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to detect the current character is whitespace or not
 *
 * @param [in]      c_char          - Current character
 * @return return the error code
 * @retval TRUE
 * @retval FALSE 
 */
/* ----------------------------------------------------------------------------*/
BOOL x_sbtl_ipcc_util_is_space (CHAR  c_char);





/*-----------------------------------------------------------------------------*/
/*! @brief  This API is used to calculate the row count of the specify text
 *
 * @param [in]      ps_text          - Current text
 * @param [out]     pi4_row_count    - Row count
 * @return return the error code
 * @retval MMSBTLR_OK              - Routine successful.
 * @retval MMSBTLR_INV_ARG         - Invalid para.
 */
/* ----------------------------------------------------------------------------*/
INT32 x_sbtl_ipcc_util_calculate_row_count(const CHAR* ps_text, INT32* pi4_row_count);






#endif
#endif/* _X_SBTL_IPCC_UTIL_H_ */
/*----------------------------------------------------------------------------*/
/*! @} */   /* end of  groupMW_STRM_MNGR_MM_SBTL_HDLR*/
/*----------------------------------------------------------------------------*/



