/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef _X_LIST_H_
#define _X_LIST_H_

#include "u_common.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef VOID*        HANDLE;

typedef struct 
{
    UINT32 ui4_unused;
} *HLIST_T;

typedef struct 
{
    UINT32 ui4_unused;
} *POSITION_T;

typedef enum _ListFlag
{
    FLAG_NOSORT_FROMTAIL,
    FLAG_NOSORT_FROMHEAD,
    FLAG_SORT_DESCENDING,
    FLAG_SORT_ASCENDING,
    FLAG_SORT_STRICT_DESCENDING,
    FLAG_SORT_STRICT_ASCENDING
} ListFlag;

typedef INT32 (*CompareTwoFunc) (HANDLE hElem1, HANDLE hElem2);
typedef BOOL (*EquivalenceFunc) (HANDLE hElem1, HANDLE hElem2);

HLIST_T      LIST_Create(ListFlag eFlag, CompareTwoFunc pFunc);
void        LIST_Destroy(HLIST_T hList);
POSITION_T  LIST_AddElem(HLIST_T hList, HANDLE hElem);
POSITION_T  LIST_Find(HLIST_T hList, HANDLE hData, EquivalenceFunc pFunc);
void        LIST_RemoveAll(HLIST_T hList);
void        LIST_RemoveAt(HLIST_T hList, POSITION_T posNode);
POSITION_T  LIST_GetHeadPosition(HLIST_T hList);
POSITION_T  LIST_GetTailPosition(HLIST_T hList);
HANDLE      LIST_GetNext(POSITION_T* pPos);
HANDLE      LIST_GetPrev(POSITION_T* pPos);
INT32       LIST_GetCount(HLIST_T hList);
BOOL        LIST_IsEmpty(HLIST_T hList);
HANDLE      LIST_GetElem(HLIST_T hList);

#ifdef __cplusplus
}
#endif

#endif
