/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_start.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X/DTV_X_HQ_int/DTV_X_ATSC/2 $
 * $SWAuthor: Allen Horng $
 * $MD5HEX: c589c0eea0e5fa86862cd59821fe9e34 $
 *
 * Description:
 *         This header file contains Middleware startup definitions, which
 *         are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_START_H_
#define _X_START_H_

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_START
 *  @{
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_dbg.h"
#include "x_start_common.h"

#include "x_dt.h"
#include "u_gl.h"
#include "u_dm.h"  
#include "x_atv_mngr.h"
#include "u_dsm.h"
#include "u_svl.h"

typedef INT32 (*x_sys_drv_post_start) (VOID *pv_data);

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

typedef struct _X_CONFIG_T
{
    GEN_CONFIG_T  t_rtos_config;         /* RTOS specific configuration parameters */
    GEN_CONFIG_T  t_drv_config;          /* Driver specific configuration parameters */
    GEN_CONFIG_T  t_custom_config;       /* Customization specific configuration parameters */
    GEN_CONFIG_T  t_appl_config;         /* Application specific configuration parameters */

    /* Debug trace buffer and output device setup */
    UINT16  ui2_dbg_num_trace_recs;

    const CHAR*  ps_dbg_output_name;

    DBG_OUTPUT_TYPE_T  e_dbg_output_type;

    const VOID*  pv_dbg_output_info;

    const CHAR*  ps_model_name;
    const CHAR*  ps_serial_number;

    /* Handle library parameters */
    UINT16  ui2_num_handles;

    /* OS library parameters */
    VOID*  pv_mem_addr;

    SIZE_T  z_mem_size;
    
    /* Resource Manager library parameters */
    UINT16  ui2_num_comps;
    THREAD_DESCR_T  t_rm_thread;
    /* System DateTime library */
    DT_CONFIG_T  t_dt_config;

    /* Download Function Manager */
    UINT16                      ui2_num_svc_objs;
    UINT16                      ui2_num_logo_objs;

    /* Device Status Manager. */
    DSM_CONFIG_T  t_dsm_config;
    /* Core Database parameters */
    UINT16  ui2_cdb_num_db;
    /* SVL init parameters */
    UINT16  ui2_svl_max_num;
    /* TSL init parameters */
    UINT16  ui2_tsl_max_num;
    /* NWL init parameters */
    UINT16  ui2_nwl_max_num;
    /* SATL init parameters */
    UINT16  ui2_satl_max_num;
    /* BSL init parameters */
    UINT16  ui2_bsl_max_num;
    /* Connection Manager parameters */
    UINT16  ui2_cm_num_handlers;
    /* IO Manager parameters */
    BOOL            b_apdemon_irrc_enabled;
    THREAD_DESCR_T  t_iom_thread;
    /* Process Manager library parameters */
    UINT16  ui2_pm_num_ca;
    UINT16  ui2_pm_num_service;
    UINT16  ui2_pm_num_comps;
    THREAD_DESCR_T  t_pm_thread;
    /* Stream Manager library parameters */
    THREAD_DESCR_T  t_sm_thread;
    /* SVL Buidler parameters */
    THREAD_DESCR_T  t_sb_thread;
    /* Event Context parameters */
    THREAD_DESCR_T  t_evctx_thread;
    /* Event Database parameters */
    UINT8           ui1_edb_max_num;
    THREAD_DESCR_T  t_edb_thread;
    THREAD_DESCR_T  t_edb_section_thread;
    SIZE_T          z_edb_mem_size;
    /* Rating Region Context parameters */
    THREAD_DESCR_T  t_rrctx_thread;
    /* Favorites List Manager init parameters */
    UINT16 ui2_flm_max_num;
    UINT16 ui2_flm_max_size;
    /* ATV manager configuration */
    X_ATV_MANAGER_CONFIG_T t_atv_mngr_cfg;
    /* Widget library parameters */
    UINT32          ui4_width;
    UINT32          ui4_height;
    GL_COLORMODE_T  e_colormode;
    BOOL            b_auto_scale_view;
    BOOL            b_no_inter_compose;
    BOOL            b_support_compose_anim;
    BOOL            b_osd_compress;
    BOOL            b_compressed_compose;
    UINT16          ui2_clut_sz;
    GL_COLOR_T*     pt_clut;
    GL_COLOR_T      t_color_key;
    UINT16          ui2_num_widgets;
    UINT8           ui1_max_floated_widget;
    UINT16          ui2_key_first_repeat_delay;
    UINT16          ui2_key_repeat_delay;
    THREAD_DESCR_T  t_wm_thread;
    /* service context parameters */
    UINT16         ui2_svctx_max_num;
    THREAD_DESCR_T t_svctx_thread;

    /* CLI parameters */
    THREAD_DESCR_T  t_cli_thread;
    
    /* Section Manager library parameters. */
    UINT16  ui2_log_filter;
    UINT16  ui2_sec_req;
    UINT16  ui2_max_req;
    THREAD_DESCR_T  t_secm_thread;
    DM_INIT_PARM_T t_dm_init;
    /* Font Engine parameters */
    UINT32  ui4_hori_dpi;
    UINT32  ui4_vert_dpi;
    
    /* VBI Filter parameters */
    THREAD_DESCR_T  t_vbif_thread;
    /* CEC Manager parameters */
    THREAD_DESCR_T  t_cecm_thread;

    x_sys_drv_post_start pt_drv_post_start_fct;

    SVL_HASHCODE_FIELD_T t_hashcode_fields;

}   X_CONFIG_T;


/*-----------------------------------------------------------------------------
                    functions declarations
 ----------------------------------------------------------------------------*/

/* The following API's must be implemented by the manufacturer. */
extern INT32 x_appl_init (GEN_CONFIG_T*  pt_config);
extern INT32 x_custom_init (GEN_CONFIG_T*  pt_config);
extern INT32 x_custom_init_boot(GEN_CONFIG_T*  pt_config);
extern INT32 x_drv_init (GEN_CONFIG_T*  pt_config);
extern INT32 x_sys_service_list_custom_init(X_CONFIG_T* pt_sys_config);


/* The following API is expported by the Middleware. */
/*------------------------------------------------------------------*/
/*! @brief This API initializes the configuration structure to default
 *              values.
 *  @param [in]     pt_config - References the configuration structure.
 *  @param [out]    pt_config - References the initialized configuration structure.
 *  @return Return the error code.
 *  @retval         INITR_OK  -     Routine successful.
 *  @retval         INITR_INV_ARG - Invalid argument.
 */
/*------------------------------------------------------------------*/
extern INT32 x_start_def_config (X_CONFIG_T*  pt_config);

/*------------------------------------------------------------------*/
/*! @brief This API is the entry point for the Middleware software
 *              initialization.
 *  @param [in]     pt_config - References the configuration structure.
 *  @param [out]     - None
 *  @return Return the error code.
 *  @retval         INITR_OK  -     Routine successful.
 *  @retval         INITR_ALREDAY_INIT - Alreday initialized.
 *  @retval         INITR_INV_ARG - Invalid argument.
 */
/*------------------------------------------------------------------*/
extern INT32 x_start (X_CONFIG_T*  pt_config);

extern INT32 x_start_phase1 (X_CONFIG_T*  pt_config);
extern INT32 x_start_phase2 (X_CONFIG_T*  pt_config);

/*----------------------------------------------------------------------------*/
/*! @} */                                          /* end of groupMW_START */
/*----------------------------------------------------------------------------*/

#endif /* _X_START_H_ */
