/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_svctx_sm_broker.h,v $
 * $Revision: #1 $
 * $Date: 2015/05/29 $
 * $Author: brianpc.huang $
 * $CCRevision: /main/DTV_X_HQ_int/DTV_X_ATSC/6 $
 * $SWAuthor: Weider Chang $
 * $MD5HEX: 29fe902dc7b0175781aa7c8566cead6f $
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/
#ifndef _X_SVCTX_SM_BROKER_H_
#define _X_SVCTX_SM_BROKER_H_

/*-----------------------------------------------------------------------------
 *  include files
 *---------------------------------------------------------------------------*/
#include "u_common.h"
#include "u_device_type.h"
#include "u_handle.h"
#include "u_svctx.h"
#include "u_sm_video_hdlr.h"
/*-----------------------------------------------------------------------------
 * structure, constants, macro definitions
 *---------------------------------------------------------------------------*/
typedef UINT32 SVCTX_SM_BROKER_CMD_CODE_T;

#define SVCTX_SM_BROKER_CMD_CODE_END                   ((SVCTX_SM_BROKER_CMD_CODE_T) 0)
#define SVCTX_SM_BROKER_CMD_CODE_DEVICE_TYPE           ((SVCTX_SM_BROKER_CMD_CODE_T) 1)
#define SVCTX_SM_BROKER_CMD_CODE_SRC_NAME              ((SVCTX_SM_BROKER_CMD_CODE_T) 2)
#define SVCTX_SM_BROKER_CMD_CODE_SVC_LST_ID            ((SVCTX_SM_BROKER_CMD_CODE_T) 3)
#define SVCTX_SM_BROKER_CMD_CODE_SVL_REC_ID            ((SVCTX_SM_BROKER_CMD_CODE_T) 4)

typedef struct
{
    SVCTX_SM_BROKER_CMD_CODE_T  e_code;

    union
    {
        DEVICE_TYPE_T   e_device_type; /* DEV_TUNER or DEV_AVC defined u_device_type.h*/
        UINT16          ui2_data;
        CHAR*           ps_text;
        VOID*           pv_data;       /* SM_VSH_VBI_NFY_T* for SVCTX_SM_BROKER_CMD_CODE_VSH_VBI_NTY */
    } u;
} SVCTX_SM_BROKER_CMD_T;

typedef enum
{
    SVCTX_SM_BROKER_COND_UNAVAILABLE = 0,
    SVCTX_SM_BROKER_COND_TRANSITION,
    SVCTX_SM_BROKER_COND_AVAILABLE
}   SVCTX_SM_BROKER_COND_T;

typedef VOID (*x_svctx_vbi_broker_nfy_fct) (
                    HANDLE_T                        h_vbi_broker,
                    SVCTX_SM_BROKER_COND_T          e_cond,
                    VOID*                           pv_tag,
                    UINT32                          ui4_data
                    );
/*-----------------------------------------------------------------------------
 * functions declarations
 *---------------------------------------------------------------------------*/
extern INT32 x_svctx_vbi_broker_open (
                    const SVCTX_SM_BROKER_CMD_T*    pt_cmd_sets,    /* in  */
                    SM_VSH_VBI_NFY_T*               pt_vbi,         /* in  */
                    x_svctx_vbi_broker_nfy_fct      pf_nfy,         /* in  */
                    VOID*                           pv_tag,         /* in  */
                    HANDLE_T*                       ph_vbi_broker,  /* out */
                    SVCTX_SM_BROKER_COND_T*         pe_cond		    /* out */
                    );
extern INT32 x_svctx_vbi_broker_close (
                    HANDLE_T                        h_vbi_broker    /* in  */
                    );
/* The difference between this API with Stream Manager's x_sm_get()
 * is the object handle. In this API, it is VBI broker handle. In Stream
 * Manager, it is video stream handle. The Details about how to use this 
 * API, please reference Stream Manager's spec for e_get_type, pv_get_info
 * and pz_get_info_size.
 */
extern INT32 x_svctx_vbi_broker_video_sm_get(
                    HANDLE_T                        h_vbi_broker,   /* in  */
                    SM_GET_TYPE_T                   e_get_type,     /* in  */
                    VOID*                           pv_get_info,    /* in/out */
                    SIZE_T*                         pz_get_info_size/* in/out */
                    );
#endif  /* _X_SVCTX_SM_BROKER_H_ */
