/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * $RCSfile: x_vid_dec.h,v $
 * $Revision: #2 $
 * $Date: 2015/06/14 $
 * $Author: hui.he $
 *
 * Description:
 *         This header file contains video decoder driver specific
 *         definitions, which are exported.
 *---------------------------------------------------------------------------*/

#ifndef _X_VID_DEC_H_
#define _X_VID_DEC_H_


/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/

#include "x_common.h"
#include "x_rm.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/


/* Get operations */
#define VID_DEC_GET_TYPE_CTRL               (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 0))
#define VID_DEC_GET_TYPE_STC                (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 1))
#define VID_DEC_GET_TYPE_RESOLUTION         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 2))
#define VID_DEC_GET_TYPE_CAPABILITY         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 3))
#define VID_DEC_GET_TYPE_PCR_ID             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 4))
#define VID_DEC_GET_TYPE_CC_DATA            (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 5))
#define VID_DEC_GET_TYPE_I_FRAME_SURFACE    (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 6))
#define VID_DEC_GET_TYPE_DEBLOCKING         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 7))
#define VID_DEC_GET_TYPE_DECODE_STATUS      (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 8))
#define VID_DEC_GET_TYPE_SPEED              (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T) 9))
#define VID_DEC_GET_TYPE_THUMBNAIL          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)10))
#define VID_DEC_GET_TYPE_ALLOC_I_FRAME_BUFF (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)11))
#define VID_DEC_GET_TYPE_STATUS             (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)12))
#define VID_DEC_GET_TYPE_PROFILE_LEVEL		(RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)13))
#define VID_DEC_GET_TYPE_DROP_FRAME_CNT     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)14))
#define VID_DEC_GET_TYPE_WFDERRINFO         (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)15))
#define VID_DEC_GET_TYPE_DELAYTIME          (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)16))
#define VID_DEC_GET_TYPE_THUMBNAIL_INFO     (RM_GET_TYPE_LAST_ENTRY + ((DRV_GET_TYPE_T)17))

/* Set operations */
#define VID_DEC_SET_TYPE_CTRL               ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 0)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_NFY_FCT            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 1))
#define VID_DEC_SET_TYPE_NFY_PTS            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 2))
#define VID_DEC_SET_TYPE_INP_MODE           (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 3))
#define VID_DEC_SET_TYPE_PLAY_CC            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 4))
#define VID_DEC_SET_TYPE_STOP_CC            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 5))
#define VID_DEC_SET_TYPE_PCR_ID             ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 6)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_CAPTURE            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 7))
#define VID_DEC_SET_TYPE_ALLOC_CC_BUFF      ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 8)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_FREE_CC_BUFF       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T) 9))
#define VID_DEC_SET_TYPE_FLUSH_CC_BUFF      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)10))
#define VID_DEC_SET_TYPE_I_FRAME_BUFF       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)11))
#define VID_DEC_SET_TYPE_ALLOC_I_FRAME_BUFF (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)12))
#define VID_DEC_SET_TYPE_FREE_I_FRAME_BUFF  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)13))
#define VID_DEC_SET_TYPE_I_FRAME_DECODE     (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)14))
#define VID_DEC_SET_TYPE_I_FRAME_NFY_FCT    (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)15))
#define VID_DEC_SET_TYPE_DEBLOCKING         (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)16))
#define VID_DEC_SET_TYPE_CODEC              ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)17)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_SPEED              ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)18)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_SEQUENCE_INFO      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)19))
#define VID_DEC_SET_TYPE_START_RENDER_INFO  (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)20))
#define VID_DEC_SET_TYPE_FLUSH              ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)21)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_SUPER_FREEZE       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)22))
#define VID_DEC_SET_TYPE_RD3D_REG_DESC      (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)23))
#define VID_DEC_SET_TYPE_MVC_EXT_DESC       (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)24))
#define VID_DEC_SET_TYPE_NOT_DISPLAY        ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)25)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_FLUSH_ES           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)26)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_VDEC_REPLAY        ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)27)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_STC_ID             ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)28)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_APP                ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)29)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_IFRAME_MODE        ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)30)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_CODEC_CHG_DONE     ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)31)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_IMGRZ_ID           ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)32)))
#define VID_DEC_SET_TYPE_RESET_WFD_INFO     ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)33)) | RM_SET_TYPE_ARG_NO_REF)
#define VID_DEC_SET_TYPE_VIDEO_TEXTURE_MODE ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)34)))
#define VID_DEC_SET_TYPE_SEAMLESS_MODE      ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)35)))
#define VID_DEC_SET_TYPE_SEAMLESS_TARGET    ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)36)))
#define VID_DEC_SET_TYPE_HDR_INFO_NFY_FCT   (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)37))
#define VID_DEC_SET_TYPE_HDR10_INFO          (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)38))
#define VID_DEC_SET_TYPE_HDR10_METADATA_INFO (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)39))
#define VID_DEC_SET_TYPE_TV_NODATA_TIME_INFO (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)40))
#define VID_DEC_SET_TYPE_DV_INFO            (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)41))
#define VID_DEC_SET_TYPE_DELTA_PTS          (RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)42))
#define VID_DEC_SET_TYPE_SECURE_PLAY_FLAG    ((RM_SET_TYPE_LAST_ENTRY + ((DRV_SET_TYPE_T)43)))

/* Registration Flags */
#define VID_DEC_REG_FLAG_NONE                     ((UINT32) 0)
#define VID_DEC_REG_FLAG_USED_NORMAL_MODE         ((UINT32) 1)
#define VID_DEC_REG_FLAG_USED_IN_QUAD_MODE        ((UINT32) 2)


/* Notify conditions */
typedef enum
{
    VID_DEC_COND_ERROR = -1,
    VID_DEC_COND_CTRL_DONE,
    VID_DEC_COND_RESOLUTION_CHG,
    VID_DEC_COND_I_FRAME_SET_BUFF_DONE,
    VID_DEC_COND_I_FRAME_DECODE_DONE,
    VID_DEC_COND_DECODE_STATUS_CHG,
    VID_DEC_COND_SPEED_CHG,
    VID_DEC_COND_THUMBNAIL_DONE,
    VID_DEC_COND_THUMBNAIL_FAIL,
    VID_DEC_COND_BIT_RATE,
    VID_DEC_COND_PLAY_DONE,   /*notify MM/AP can trick, pause*/
    VID_DEC_COND_FRAMERATE_NOT_SUPPORT,
	VID_DEC_COND_THUMBNAIL_PLAY_EOS,
	VID_DEC_COND_WFD_ERROR,
	VID_DEC_COND_HDR_CONTENT_CHG
}   VID_DEC_COND_T;

/* VID_DEC_GET_TYPE_DECODE_STATUS settings ***********************************/
typedef enum
{
    VID_DEC_DECODE_UNKNOWN = -1,
    VID_DEC_DECODE_NO_DATA,
    VID_DEC_DECODE_NORMAL,
    VID_DEC_DECODE_ERROR,
    VID_DEC_DECODE_HD_NOT_SUPPORT,
    VID_DEC_DECODE_CODEC_NOT_SUPPORT,
    VID_DEC_DECODE_BITSTREAM_NOT_SUPPORT,
    VID_DEC_DECODE_CODEC_TYPE_CHG,
    VID_DEC_DECODE_DATA_RECEIVE,
    VID_DEC_DECODE_RES_NOT_SUPPORT, /* resolution not support */
    VID_DEC_DECODE_CODEC_CHANGE,
    VID_DEC_DECODE_FORCE_STOP,
    VID_DEC_DECODE_FRAME_RATE_OVERFLOW,
    VID_DEC_DECODE_RESOURCE_CONFLICT,
    VID_DEC_DECODE_HDR10_INFO_UPDATE,
    VID_DEC_DECODE_GETSTC_UPDATE
}   VID_DEC_DECODE_STATUS_T;

/* VID_DEC_SET_TYPE_CODEC settings *******************************************/
typedef enum
{
    VID_DEC_CODEC_UNKNOWN = 0,
    VID_DEC_CODEC_MPEG_1,
    VID_DEC_CODEC_MPEG_2,
    VID_DEC_CODEC_MPEG_4,
    VID_DEC_CODEC_DIVX_311,
    VID_DEC_CODEC_DIVX_4,
    VID_DEC_CODEC_DIVX_5,
    VID_DEC_CODEC_XVID,
    VID_DEC_CODEC_WMV1,
    VID_DEC_CODEC_WMV2,
    VID_DEC_CODEC_WMV3,
    VID_DEC_CODEC_WVC1,
    VID_DEC_CODEC_H264,
    VID_DEC_CODEC_H263,
    VID_DEC_CODEC_MJPEG,
    VID_DEC_CODEC_RV8,
    VID_DEC_CODEC_RV9,
    VID_DEC_CODEC_AVS,
    VID_DEC_CODEC_NV12,
    VID_DEC_CODEC_VP8,
    VID_DEC_CODEC_VP6,
    VID_DEC_CODEC_H265,
    VID_DEC_CODEC_VP9,
    VID_DEC_CODEC_DV_H264,
    VID_DEC_CODEC_DV_H265,
    VID_DEC_CODEC_AVS2,
    VID_DEC_CODEC_SHVC,
    VID_DEC_CODEC_AV1
}   VID_DEC_CODEC_T;

/*VID_DEC_PROFILE_TYPE_T settings **********************************************/
typedef enum
{
	VID_DEC_PROFILE_UNKNOWN = 0,
	VID_DEC_PROFILE_SIMPLE,	//Simple profile
	VID_DEC_PROFILE_MAIN,	//Main profile
	VID_DEC_PROFILE_SNR_SCALABLE,	//Snr Scalable profile
	VID_DEC_PROFILE_SPATIAL_SCALABLE,		//Spatial Scalable profile
	VID_DEC_PROFILE_HIGH,	//High profile
	VID_DEC_PROFILE_MULTIVIEW,	//Multiview profile
	VID_DEC_PROFILE_4_2_2,	//4:2:2 profile
	//MPEG4
	VID_DEC_PROFILE_ADVANCED_CODING_EFFICIENCY,		//Advanced Coding Efficiency profle
	VID_DEC_PROFILE_SIMPLE_STUDIO,		//Simple Studio profile
	VID_DEC_PROFILE_CORE_STUDIO,	//Core Studio profile
	VID_DEC_PROFILE_FINE_GRANULARITY_SCALABLE,	//Fine Granularity Scalable profile
	VID_DEC_PROFILE_SIMPLE_FACE_ANIMATION,		//Simple Face Animation profile
	VID_DEC_PROFILE_SIMPLE_FBA,		//Simple FBA proflie
	VID_DEC_PROFILE_BASIC_ANIMATED_TEXTURE,	//Basic Animated Texture profile
	VID_DEC_PROFILE_SCALABLE_TEXTURE,		//Scalable Texture profile
	VID_DEC_PROFILE_ADVANCED_SCALABLE_TEXTURE,	//Advanced Scalable Texture profile
	VID_DEC_PROFILE_HYBRID,	//Hybrid profile
	VID_DEC_PROFILE_SIMPLE_SCALABLE,	//Simple Scalable profile
	VID_DEC_PROFILE_ADVANCED_SIMPLE,	//Advanced Simple profile
	VID_DEC_PROFILE_CORE,	//Core profile
	VID_DEC_PROFILE_CORE_SCALABLE,		//Core Scalable profile
	VID_DEC_PROFILE_ADVANCED_CORE,		//Advanced Core profile
	VID_DEC_PROFILE_N_BIT,		//N-Bit profile
	VID_DEC_PROFILE_REAL_TIME_SIMPLE,	//Real Time profile
	//H264
	VID_DEC_PROFILE_BASELINE,		//Baseline profile
	VID_DEC_PROFILE_EXTENDED,		//Extended profile
	VID_DEC_PROFILE_HIGH_10,		//High 10 profile
	VID_DEC_PROFILE_HIGH_4_2_2,	//High 4:2:2 profile
	VID_DEC_PROFILE_HIGH_4_4_4,	//High 4:4:4 profile
	//VC1
	VID_DEC_PROFILE_ADVANCED,	//Advanced profile
	//wmv
	VID_DEC_PROFILE_PC	//PC Profile
}	VID_DEC_PROFILE_TYPE_T;

/* VID_DEC_WFD_ERR_INFO settings ****************************************/
typedef struct _VID_DEC_WFD_ERR_INFO_T
{
    INT32 i4TimeoutCnt;
    INT32 i4LackSeqHeaderCnt;
    INT32 i4OutOfSpecCnt;
} VID_DEC_WFD_ERR_INFO_T;

/*VID_DEC_LEVEL_TYPE_T settings **********************************************/
typedef enum
{
	VID_DEC_LEVEL_UNKNOWN = 0,
	VID_DEC_LEVEL_MAIN,	//Main level
	VID_DEC_LEVEL_LOW,	//Low level
	VID_DEC_LEVEL_HIGH,	//High level
	VID_DEC_LEVEL_HIGH_1440,	//High 1440 level
	VID_DEC_LEVEL_0,	//level 0
	VID_DEC_LEVEL_1,	//level 1
	VID_DEC_LEVEL_2,	//level 2
	VID_DEC_LEVEL_3,	//level 3
	VID_DEC_LEVEL_4,	//level 4
	VID_DEC_LEVEL_5,	//level 5
	VID_DEC_LEVEL_1_B,	//level 1b
	VID_DEC_LEVEL_1_1,	//level 1.1
	VID_DEC_LEVEL_1_2,	//level 1.2
	VID_DEC_LEVEL_1_3,	//level 1.3
	VID_DEC_LEVEL_2_1,	//level 2.1
	VID_DEC_LEVEL_2_2,	//level 2.2
	VID_DEC_LEVEL_3_1,	//level 3.1
	VID_DEC_LEVEL_3_2,	//level 3.2
	VID_DEC_LEVEL_4_1,	//level 4.1
	VID_DEC_LEVEL_4_2,	//level 4.2
	VID_DEC_LEVEL_5_1,	//level 5.1
	VID_DEC_LEVEL_LL,		//level LL
	VID_DEC_LEVEL_ML,		//level ML
	VID_DEC_LEVEL_HL,		//level HL
	VID_DEC_LEVEL_L_0,	//level L0
	VID_DEC_LEVEL_L_1,	//level L1
	VID_DEC_LEVEL_L_2,	//level L2
	VID_DEC_LEVEL_L_3,	//level L3
	VID_DEC_LEVEL_L_4		//level L4
}	VID_DEC_LEVEL_TYPE_T;

/* VID_DEC_SET_TYPE_CTRL settings ********************************************/
typedef enum
{
    VID_DEC_CTRL_RESET        = 0x00,
    VID_DEC_CTRL_STOP         = 0x01,
    VID_DEC_CTRL_FREEZE       = 0x02,
    VID_DEC_CTRL_PLAY         = 0x04,
    VID_DEC_CTRL_PLAY_I_FRAME = 0x08
}   VID_DEC_CTRL_T;

/* VID_DEC_SRC_ASPECT_RATIO settings **************************************/
typedef enum
{
    VID_DEC_SRC_ASPECT_RATIO_UNKNOWN = 0,
    VID_DEC_SRC_ASPECT_RATIO_PXL_1_1,     /* PAR 1:1 */
    VID_DEC_SRC_ASPECT_RATIO_4_3,         /* DAR 4:3 */
    VID_DEC_SRC_ASPECT_RATIO_16_9,        /* DAR 16:9 */
    VID_DEC_SRC_ASPECT_RATIO_2_21_1,      /* DAR 2.21:1 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_10_11,   /* PAR 10:11 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_40_33,   /* PAR 40:33 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_16_11,   /* PAR 16:11 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_12_11,   /* PAR 12:11 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_3_2,     /* PAR 3:2 */
    VID_DEC_SRC_ASPECT_RATIO_PXL_24_11,
    VID_DEC_SRC_ASPECT_RATIO_PXL_20_11,
    VID_DEC_SRC_ASPECT_RATIO_PXL_32_11,
    VID_DEC_SRC_ASPECT_RATIO_PXL_80_33,
    VID_DEC_SRC_ASPECT_RATIO_PXL_18_11,
    VID_DEC_SRC_ASPECT_RATIO_PXL_15_11,
    VID_DEC_SRC_ASPECT_RATIO_PXL_64_33,
    VID_DEC_SRC_ASPECT_RATIO_PXL_160_99,
    VID_DEC_SRC_ASPECT_RATIO_USR,          /* user */
    VID_DEC_SRC_ASPECT_RATIO_2_1
} VID_DEC_SRC_ASPECT_RATIO_T;

/* Video source resolution descriptor */
typedef enum
{
    VID_DEC_TIMING_UNKNOWN = 0,
    VID_DEC_TIMING_VIDEO,
    VID_DEC_TIMING_GRAPHIC,
    VID_DEC_TIMING_NOT_SUPPORT
}   VID_DEC_TIMING_TYPE_T;

/*VID_DEC_TAG3D_TYPE settings **********************************************/
typedef enum
{
    VID_DEC_TAG3D_2D = 0,
    VID_DEC_TAG3D_MVC,          /* MVC = Multi-View Codec */
    VID_DEC_TAG3D_FP,           /* FP = Frame Packing */
    VID_DEC_TAG3D_FS,           /* FS = Frame Sequential */
    VID_DEC_TAG3D_TB,           /* TB = Top-and-Bottom */
    VID_DEC_TAG3D_SBS,          /* SBS = Side-by-Side */
    VID_DEC_TAG3D_REALD,
    VID_DEC_TAG3D_SENSIO,
    VID_DEC_TAG3D_LA,           /* LA = Line Alternative */
    VID_DEC_TAG3D_TTDO,         /* TTD only */
    VID_DEC_TAG3D_NOT_SUPPORT
}   VID_DEC_TAG3D_TYPE_T;

typedef enum
{
    VID_DEC_SEAMLESS_MODE_NONE,
    VID_DEC_SEAMLESS_MODE_SD,
    VID_DEC_SEAMLESS_MODE_HD,
    VID_DEC_SEAMLESS_MODE_FHD,
    VID_DEC_SEAMLESS_MODE_4K2K
}  VID_DEC_SEAMLESS_MODE_T;

typedef struct
{
    UINT32    u4HdrType;                      ///< HDR type
    UINT32    u4VdoHdrChange;                 ///< HDR type change
    UINT32    u4VdoGamma;                     ///< Gamma type
    UINT32    u4ColorSpaceTypeDef;            ///< ColorSpace type
    UINT32    u4VdoColorSpaceChange;          ///< ColorSpace type change
    UINT32    u4TV_MaxLuma;                   ///< TV_MaxLuma
    UINT32    u4TypeMatrixCoefficients;       ///< add by lihui.li for TV MatrixCoefficients type
    UINT32    u4TypeRange;                    ///< add by lihui.li for range type
}VID_DEC_HDR10_INFO_T;


typedef struct
{
    UINT16  ui2DisplayPrimaries_x[3];
    UINT16  ui2DisplayPrimaries_y[3];
    UINT16  ui2WhitePoint_x;
    UINT16  ui2WhitePoint_y;
    UINT16  ui2MaxDisplayMasteringLum;
    UINT16  ui2MinDisplayMasteringLum;
    UINT16  ui2MaxCll;
    UINT16  ui2MaxFall;
}VID_DEC_HDR10_METADATA_INFO_T;


/* VID_DEC_RESOLUTION_INFO settings ****************************************/
typedef struct _VID_DEC_RESOLUTION_INFO_T
{
    UINT16                     ui2_width;
    UINT16                     ui2_height;
    UINT16                     ui2_frame_rate;
    UINT16                     ui2_afd;
    BOOL                       b_is_progressive;
    BOOL                       b_overscan_not_applied;
    VID_DEC_SRC_ASPECT_RATIO_T e_src_asp_ratio;
    VID_DEC_TIMING_TYPE_T      e_timing_type;
    VID_DEC_TAG3D_TYPE_T       e_tag3d_type;
    UINT32                     u4_aspect_ratio_w;
    UINT32                     u4_aspect_ratio_h;
    BOOL                       b_is_display_aspect_ratio;
    BOOL                       b_is_mhl_signal;
    BOOL                       b_resized;
    BOOL                       b_orig_is_display_aspect_ratio;
    UINT16                     ui2_orig_width;
    UINT16                     ui2_orig_height;
    VID_DEC_SRC_ASPECT_RATIO_T e_orig_src_asp_ratio;
    UINT32                     u4_orig_aspect_ratio_w;
    UINT32                     u4_orig_aspect_ratio_h;
    UINT8                      c_hdr_content;
    BOOL                       b_overscan_info_present;
    BOOL                       b_overscan_appropriate;
    VID_DEC_CODEC_T            e_codec_type;
    BOOL                       b_cropping;
    UINT32                     u4_crop_left_offset;
    UINT32                     u4_crop_right_offset;
    UINT32                     u4_crop_top_offset;
    UINT32                     u4_crop_bottom_offset;
} VID_DEC_RESOLUTION_INFO_T;

/* VID_DEC_PROFILE_LEVEL_INFO settings ****************************************/
typedef struct _VID_DEC_PROFILE_LEVEL_INFO_T
{
	VID_DEC_PROFILE_TYPE_T		e_profile_type;
	VID_DEC_LEVEL_TYPE_T		e_level_type;
} VID_DEC_PROFILE_LEVEL_INFO_T;

/*VID_DEC_APP_TYPE settings **********************************************/
typedef enum
{
    VID_DEC_APP_NONE = 0,
    VID_DEC_APP_MHP
}   VID_DEC_APP_TYPE_T;

typedef struct _VID_DEC_START_RENDER_INFO_T
{
    PTS_T                     t_pts;
    BOOL                      b_auto_pause;
} VID_DEC_START_RENDER_INFO_T;


/* VID_DEC_GET_TYPE_CAPABILITY settings **************************************/
typedef enum
{
    VID_DEC_FMT_UNKNOWN = 0,
    VID_DEC_FMT_480I,
    VID_DEC_FMT_480P,
    VID_DEC_FMT_576I,
    VID_DEC_FMT_576P,
    VID_DEC_FMT_720P,
    VID_DEC_FMT_1080I,
    VID_DEC_FMT_1080P
}   VID_DEC_FMT_T;

#define VID_DEC_CAP_NONE            ((UINT32) 0)
#define VID_DEC_CAP_FMT_480I        MAKE_BIT_MASK_32(1)
#define VID_DEC_CAP_FMT_480P        MAKE_BIT_MASK_32(2)
#define VID_DEC_CAP_FMT_576I        MAKE_BIT_MASK_32(3)
#define VID_DEC_CAP_FMT_576P        MAKE_BIT_MASK_32(4)
#define VID_DEC_CAP_FMT_720P        MAKE_BIT_MASK_32(5)
#define VID_DEC_CAP_FMT_1080I       MAKE_BIT_MASK_32(6)
#define VID_DEC_CAP_FMT_1080P       MAKE_BIT_MASK_32(7)
#define VID_DEC_CAP_DEBLOCK         MAKE_BIT_MASK_32(8)

#define VID_DEC_CAP_CODEC_UNKNOWN   ((UINT32) 0)
#define VID_DEC_CAP_CODEC_MPEG_1    MAKE_BIT_MASK_32(1)
#define VID_DEC_CAP_CODEC_MPEG_2    MAKE_BIT_MASK_32(2)
#define VID_DEC_CAP_CODEC_MPEG_4    MAKE_BIT_MASK_32(3)
#define VID_DEC_CAP_CODEC_DIVX_311  MAKE_BIT_MASK_32(4)
#define VID_DEC_CAP_CODEC_DIVX_4    MAKE_BIT_MASK_32(5)
#define VID_DEC_CAP_CODEC_DIVX_5    MAKE_BIT_MASK_32(6)
#define VID_DEC_CAP_CODEC_XVID      MAKE_BIT_MASK_32(7)
#define VID_DEC_CAP_CODEC_WMV1      MAKE_BIT_MASK_32(8)
#define VID_DEC_CAP_CODEC_WMV2      MAKE_BIT_MASK_32(9)
#define VID_DEC_CAP_CODEC_WMV3      MAKE_BIT_MASK_32(10)
#define VID_DEC_CAP_CODEC_WVC1      MAKE_BIT_MASK_32(11)
#define VID_DEC_CAP_CODEC_H264      MAKE_BIT_MASK_32(12)
#define VID_DEC_CAP_CODEC_H263      MAKE_BIT_MASK_32(13)
#define VID_DEC_CAP_CODEC_MJPEG     MAKE_BIT_MASK_32(14)
#define VID_DEC_CAP_CODEC_VP6       MAKE_BIT_MASK_32(15)
#define VID_DEC_CAP_CODEC_VP8       MAKE_BIT_MASK_32(16)
#define VID_DEC_CAP_CODEC_VP9       MAKE_BIT_MASK_32(17)

#define VID_DEC_STATUS_NORMAL       ((UINT32) 0)
#define VID_DEC_STATUS_DROP         MAKE_BIT_MASK_32(1)
#define VID_DEC_STATUS_SKIP         MAKE_BIT_MASK_32(2)
#define VID_DEC_STATUS_TIMEOUT      MAKE_BIT_MASK_32(3)

/* VID_DEC_CAPABILITY_INFO *****************************************************/
typedef struct _VID_DEC_CAPABILITY_INFO_T
{
    UINT32      ui4_fmt_timing;
    UINT32      ui4_codec_type;
    BOOL        b_3d_multi_track_available;
} VID_DEC_CAPABILITY_INFO_T;

/* VID_DEC_GET_TYPE_CC_DATA **************************************************/
typedef struct _VID_DEC_CC_DATA
{
    UCHAR*     puc_cc_data;    /* IN/OUT */
    SIZE_T     z_size;         /* IN/OUT */
    PTS_T      t_pts;          /* OUT */
    BOOL       b_is_scte_cc;   /* OUT */
    BOOL       b_even_first;   /* OUT */
} VID_DEC_CC_DATA;

/* VID_DEC_SET_TYPE_NFY_FCT settings *****************************************/
typedef VOID (*x_vid_dec_nfy_fct) (
    VOID*               pv_nfy_tag,
    VID_DEC_COND_T      e_nfy_cond,
    UINT32              ui4_data_1,
    UINT32              ui4_data_2
    );

typedef struct _VID_DEC_NFY_INFO_T
{
    VOID*              pv_tag;
    x_vid_dec_nfy_fct  pf_vid_dec_nfy;
    VOID*              pv_previous_tag;          /* OUTPUT */
    x_vid_dec_nfy_fct  pf_previous_vid_dec_nfy;  /* OUTPUT */
}   VID_DEC_NFY_INFO_T;


/* VID_DEC_SET_TYPE_PLAY_CC settings *****************************************/
/* CC notify conditions */
typedef enum
{
    VID_DEC_CC_COND_ERROR = -1,
    VID_DEC_CC_COND_DATA_ARRIVAL
}   VID_DEC_CC_COND_T;

/* error codes used in VID_DEC_CC_COND_ERROR */
#define VID_DEC_CC_ERR_BUFF_OVER_FLOW       ((UINT32) 1)

typedef VOID (*x_vid_dec_cc_nfy_fct) (
    VOID*                   pv_tag,
    VID_DEC_CC_COND_T       e_cc_cond,
    VOID*                   pv_arg
    );

typedef struct _VID_DEC_CC_NFY_INFO_T
{
    VOID*                       pv_tag;
    x_vid_dec_cc_nfy_fct        pf_cc_nfy;
}   VID_DEC_CC_NFY_INFO_T;


/* VID_DEC_SET_TYPE_NFY_PTS settings *****************************************/
typedef VOID (*x_vid_dec_pts_reach_nfy_fct) (
    VOID*               pv_nfy_tag,
    PTS_T               t_pts,
    BOOL                b_out_of_sync
    );

typedef struct _VID_DEC_NFY_PTS_INFO_T
{
    PTS_T                         t_pts;
    VOID*                         pv_tag;
    x_vid_dec_pts_reach_nfy_fct   pf_vid_dec_pts_reach_nfy;
}   VID_DEC_NFY_PTS_INFO_T;


/* VID_DEC_SET_TYPE_INP_MODE settings ****************************************/
typedef enum
{
    VID_DEC_INP_MODE_NORMAL,
    VID_DEC_INP_MODE_PULL
}   VID_DEC_INP_MODE_T;

typedef INT32 (*x_vid_dec_data_acquire_fct)(
    VOID*            pv_tag,
    UCHAR**          ppc_buff,
    SIZE_T*          pz_buff_size
    );

typedef VOID (*x_vid_dec_data_release_fct)(
    VOID*            pv_tag,
    UCHAR*           pc_buff,
    SIZE_T           z_buff_size
    );

typedef struct _VID_DEC_INP_MODE_INFO_T
{
    VID_DEC_INP_MODE_T           e_inp_mode;
    VOID*                        pv_tag;                      /* pull mode only */
    x_vid_dec_data_acquire_fct   pf_vid_dec_data_acquire_nfy; /* pull mode only */
    x_vid_dec_data_release_fct   pf_vid_dec_data_release_nfy; /* pull mode only */
}   VID_DEC_INP_MODE_INFO_T;


/* VID_DEC_SET_TYPE_CAPTURE settings ****************************************/
typedef struct _VID_DEC_CAPTURE_INFO_T
{
    UCHAR*                    pc_frame_buff;
    SIZE_T                    z_buff_size;
    BOOL                      b_is_progressive;
}   VID_DEC_CAPTURE_INFO_T;


/* VID_DEC_SET_TYPE_I_FRAME_BUFF settings ***********************************/
/* VID_DEC_SET_TYPE_ALLOC_I_FRAME_BUFF **************************************/
/* VID_DEC_SET_TYPE_FREE_I_FRAME_BUFF ***************************************/
typedef struct _VID_DEC_I_FRAME_BUFF_T
{
    UCHAR*                    pc_frame_buff_sp;
    SIZE_T                    z_frame_size;
}   VID_DEC_I_FRAME_BUFF_T;

/* VID_DEC_GET_TYPE_I_FRAME_SURFACE *****************************************/
typedef struct _VID_DEC_GET_I_FRAME_SURFACE_T
{
    UCHAR*                     pc_y_buf;
    UINT32                     ui4_y_pitch;
    UCHAR*                     pc_c_buf;
    UINT32                     ui4_c_pitch;
}   VID_DEC_GET_I_FRAME_SURFACE_T;

/* VID_DEC_GET_TYPE_DEBLOCKING/VID_DEC_SET_TYPE_DEBLOCKING ******************/
typedef struct _VID_DEC_REGION_T
{
    UINT16        ui4_x;
    UINT16        ui4_y;
    UINT16        ui4_h;
    UINT16        ui4_w;
} VID_DEC_REGION_T;

typedef struct _VID_DEC_DEBLOCKING_INFO_T
{
    BOOL                       b_is_on;

    union
    {
        VID_DEC_REGION_T       t_region;
    } u;
}   VID_DEC_DEBLOCKING_INFO_T;

/* VID_DEC_SET_TYPE_SEQUENCE_INFO *******************************************/
#define VID_DEC_SRC_WIDTH_UNKNOWN                 ((UINT16) 0xFFFF)
#define VID_DEC_SRC_HEIGHT_UNKNOWN                ((UINT16) 0xFFFF)

typedef enum
{
    VID_DEC_SRC_FRAME_RATE_UNKNOWN = 0,
    VID_DEC_SRC_FRAME_RATE_23_976, /* 24000/1001 (23.976...) */
    VID_DEC_SRC_FRAME_RATE_24,
    VID_DEC_SRC_FRAME_RATE_25,
    VID_DEC_SRC_FRAME_RATE_29_97, /* 30000/1001 (29.97...) */
    VID_DEC_SRC_FRAME_RATE_30,
    VID_DEC_SRC_FRAME_RATE_50,
    VID_DEC_SRC_FRAME_RATE_59_94, /* 60000/1001 (59.94...) */
    VID_DEC_SRC_FRAME_RATE_60,
    VID_DEC_SRC_FRAME_RATE_120,
    VID_DEC_SRC_FRAME_RATE_1,
    VID_DEC_SRC_FRAME_RATE_5,
    VID_DEC_SRC_FRAME_RATE_8,
    VID_DEC_SRC_FRAME_RATE_10,
    VID_DEC_SRC_FRAME_RATE_12,
    VID_DEC_SRC_FRAME_RATE_15,
    VID_DEC_SRC_FRAME_RATE_2,
    VID_DEC_SRC_FRAME_RATE_6,
    VID_DEC_SRC_FRAME_RATE_48,
    VID_DEC_SRC_FRAME_RATE_70,
    VID_DEC_SRC_FRAME_RATE_20,
	VID_DEC_SRC_FRAME_RATE_240,
	VID_DEC_SRC_FRAME_RATE_100,
    VID_DEC_SRC_FRAME_RATE_VARIABLE
} VID_DEC_SRC_FRAME_RATE_T;

typedef enum
{
    VID_DEC_ASP_USR_TYPE_SRC = 1, /* source aspect ratio */
    VID_DEC_ASP_USR_TYPE_DISP     /* display aspect ratio */
} VID_DEC_ASP_USR_TYPE_T;

typedef struct _VID_DEC_SEQUENCE_DATA_T
{
    UINT16                     ui2_width;      /* horizontal_size_value */
    UINT16                     ui2_height;     /* vertical_size_value */
    VID_DEC_SRC_FRAME_RATE_T   e_frame_rate;   /* Frame Rate */
    VID_DEC_SRC_ASPECT_RATIO_T e_src_asp;      /* Display Aspect Ratio (DAR) */
    VID_DEC_ASP_USR_TYPE_T     e_asp_usr_type; /* source or display asp */
    UINT16                     ui2_asp_usr_h;
    UINT16                     ui2_asp_usr_w;
    UINT32                     ui4_seq_hdr;
    UINT16                     ui2_seq_hdr_sz;
    BOOL                       b_chk_framerate;
    BOOL                       b_divx_plus;
} VID_DEC_SEQUENCE_DATA_T;

typedef struct _VID_DEC_SEQUENCE_RAW_T
{
    UCHAR*                     pc_buff;
    SIZE_T                     z_buff_size;
    VID_DEC_SRC_FRAME_RATE_T   e_frame_rate;   /* Frame Rate */
} VID_DEC_SEQUENCE_RAW_T;

typedef struct _VID_DEC_SEQUENCE_INFO_T
{
    BOOL                       b_is_raw;
    BOOL                       b_is_second_raw;

    union
    {
        VID_DEC_SEQUENCE_RAW_T     t_seq_raw;
        VID_DEC_SEQUENCE_DATA_T    t_seq_data;
    } u;
/*#if SUPPORT_MULTI_INFO*/
    struct _VID_DEC_SEQUENCE_INFO_T*  pt_next_scale;
/*#endif*/
} VID_DEC_SEQUENCE_INFO_T;

typedef struct _VID_DEC_DV_INFO_T
{
    BOOL      b_single_track;
    UINT16    ui2_dv_version;
    UINT8     ui1_dv_profile;
    UINT8     ui1_dv_level;
    BOOL      b_rpu_present;
    BOOL      b_el_present;
    BOOL      b_bl_present;
    UINT16    ui2_depend_id;
} VID_DEC_DV_INFO_T;

/* VID_DEC_SET_TYPE_SUPER_FREEZE **********************************************/
typedef enum
{
    VID_DEC_SUPER_FREEZE_TYPE_MEMO = 0,
    VID_DEC_SUPER_FREEZE_TYPE_FORCE,    /* for super freeze */
    VID_DEC_SUPER_FREEZE_TYPE_ATVPVR
}   VID_DEC_SUPER_FREEZE_TYPE_T;

typedef struct _VID_DEC_SUPER_FREEZE_T
{
    VID_DEC_SUPER_FREEZE_TYPE_T e_type;
    BOOL                        b_enable;
}   VID_DEC_SUPER_FREEZE_T;


/* VID_DEC_GET_TYPE_THUMBNAIL ***********************************************/
/* color mode */
typedef enum
{
    VID_DEC_COLORMODE_AYUV_CLUT2     = 0,
    VID_DEC_COLORMODE_AYUV_CLUT4     = 1,
    VID_DEC_COLORMODE_AYUV_CLUT8     = 2,
    VID_DEC_COLORMODE_UYVY_16        = 3,
    VID_DEC_COLORMODE_YUYV_16        = 4,
    VID_DEC_COLORMODE_AYUV_D8888     = 5,
    VID_DEC_COLORMODE_ARGB_CLUT2     = 6,
    VID_DEC_COLORMODE_ARGB_CLUT4     = 7,
    VID_DEC_COLORMODE_ARGB_CLUT8     = 8,
    VID_DEC_COLORMODE_RGB_D565       = 9,
    VID_DEC_COLORMODE_ARGB_D1555     = 10,
    VID_DEC_COLORMODE_ARGB_D4444     = 11,
    VID_DEC_COLORMODE_ARGB_D8888     = 12,
    VID_DEC_COLORMODE_YUV_420_BLK    = 13,
    VID_DEC_COLORMODE_YUV_420_RS     = 14,
    VID_DEC_COLORMODE_YUV_422_BLK    = 15,
    VID_DEC_COLORMODE_YUV_422_RS     = 16,
    VID_DEC_COLORMODE_YUV_444_BLK    = 17,
    VID_DEC_COLORMODE_YUV_444_RS     = 18
} VID_DEC_COLORMODE_T;

typedef struct _VID_DEC_THUMBNAIL_INFO_T
{
    UCHAR*                    pc_canvas_buffer;
    VID_DEC_COLORMODE_T       e_canvas_colormode;
    UINT32                    ui4_canvas_width;     /* in pixels */
    UINT32                    ui4_canvas_height;    /* in pixels */
    UINT32                    ui4_canvas_pitch;     /* in bytes */

    UINT32                    ui4_thumbnail_x;      /* in pixels */
    UINT32                    ui4_thumbnail_y;      /* in pixels */
    UINT32                    ui4_thumbnail_width;  /* in pixels */
    UINT32                    ui4_thumbnail_height; /* in pixels */
}   VID_DEC_THUMBNAIL_INFO_T;

typedef struct _VID_DEC_BOOT_LOGO_SET_T
{
    BOOL    b_default;      /* enable default boot logo */
    BOOL    b_onoff;        /* on/off the vdec boot logo */
    UINT8   c_data_select;  /* select the number for boot */
    UINT32  ui4_valid;      /* valid for each vdec boot logo address */
} VID_DEC_BOOT_LOGO_SET_T;

typedef struct _VID_DEC_BOOT_LOGO_GET_T
{
    BOOL     b_default;      /* enable default boot logo */
    BOOL     b_default_exist;/* is default boot logo exist?*/
    BOOL     b_onoff;        /* on/off the vdec boot logo */
    UINT8    c_data_select;  /* select the number for boot */
    UINT8    c_slot_num;     /* slot number */
    UINT32   ui4_valid;      /* valid for each vdec boot logo address */
    UINT32   ui4_flash_addr; /* flash start address */
    UINT32   ui4_flash_size; /* flash size */
} VID_DEC_BOOT_LOGO_GET_T;

typedef struct _VID_DEC_BOOT_LOGO_ADDR_SET_T
{
    UINT8   c_data_type;       /* data type, one of VID_DEC_CODEC_T */
    UINT32  ui4_data_size;     /* data size */
    UINT32  ui4_addr;          /* the address of this vdec boot logo */
} VID_DEC_BOOT_LOGO_ADDR_SET_T;

typedef struct _VID_DEC_BOOT_LOGO_ADDR_GET_T
{
    UINT8    c_data_type;        /* data type, one of VID_DEC_CODEC_T */
    UINT32   ui4_data_size;      /* data size */
    UINT32   ui4_addr;           /* the address of this vdec boot logo */
} VID_DEC_BOOT_LOGO_ADDR_GET_T;

/* for 3D video */
typedef UINT8 VD_REALD_LAYER_TYPE_T;

#define VD_REALD_BASE_LAYER        ((VD_REALD_LAYER_TYPE_T)0x1)
#define VD_REALD_ENHANCEMENT_LAYER ((VD_REALD_LAYER_TYPE_T)0x2)


typedef UINT8 VD_REALD_CODING_METHOD_T;

#define VD_CRITICALLY_SAMPLED_METHOD        ((VD_REALD_CODING_METHOD_T)0x00)
#define VD_NON_CRITICALLY_SAMPLED_METHOD    ((VD_REALD_CODING_METHOD_T)0x80)

/* RD3D_registration descriptor */
typedef struct _VID_DEC_RD3D_REG_DESC_T
{
    VD_REALD_LAYER_TYPE_T      e_layer_type;
    VD_REALD_CODING_METHOD_T   e_coding_method;
} VID_DEC_RD3D_REG_DESC_T;

/* MVC extension descriptor */
typedef struct _VID_DEC_MVC_EXT_DESC_T
{
    UINT16  ui2_avg_bit_rate;
    UINT16  ui2_max_bit_rate;
    UINT16  ui2_view_ord_idx_min;
    UINT16  ui2_view_ord_idx_max;
    UINT8   ui1_temporal_id_start;
    UINT8   ui1_temporal_id_end;
    BOOL    b_no_sei_nal_present;
    BOOL    b_no_prefix_nal_present;
} VID_DEC_MVC_EXT_DESC_T;

/*For seamless target w/h*/
typedef struct _VID_DEC_SEAMLESS_TARGET_INFO_T
{
	UINT32  ui4_width;
	UINT32  ui4_height;
}VID_DEC_SEAMLESS_TARGET_INFO_T;
typedef struct _VID_DEC_SECURE_TYPE_T
{
	UINT32  ui4_reserved;
}VID_DEC_SECURE_TYPE_T;

#endif /* _X_VID_DEC_H_ */
