/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_wgl_cc_isdb.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 */
/*----------------------------------------------------------------------------*/
#ifndef _X_WGL_CC_ISDB_H_
#define _X_WGL_CC_ISDB_H_

/*----------------------------------------------------------------------------*/
/*! @defgroup groupMW_WGL_CCI ISDB Closed Caption Widget
  *  @ingroup groupMW_WGL
  *  @brief Provides x_api function TV Application
  *  %MW Application API would be used by TV Application, lot of these APIs would
  *   be wrapped from Exported API
  *  @see groupMW_WGL_CCI
  *  @{ 
  */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                    include files
-----------------------------------------------------------------------------*/
#include "x_wgl.h"

#include "x_wgl_cc_isdb_type.h"


/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
-----------------------------------------------------------------------------*/
#define WGL_CCI_SYS_INIT_FLAG_CLUT8             MAKE_BIT_MASK_32(0)    /**< CCI support color mode CLUT8 */
#define WGL_CCI_SYS_INIT_FLAG_ARBG4444          MAKE_BIT_MASK_32(1)    /**< CCI support color mode ARBG4444 */
#define WGL_CCI_SYS_INIT_FLAG_ARBG8888          MAKE_BIT_MASK_32(2)    /**< CCI support color mode ARBG8888 */


/*------------------------------------------------------------------*/
/*! @struct WGL_CC_ISDB_SYS_INIT_T
 *  @brief This structure contains the CCI widget initialization data
 *  @code
 *  typedef struct _WGL_CC_ISDB_SYS_INIT_T
 *  {
 *      UINT32                              ui4_flag;
 *      unsigned char*                      pt_font;
 *      UINT8                               ui1_clut_start;
 *      UINT8                               ui1_clut_end;
 *  }   WGL_CC_ISDB_SYS_INIT_T;
 *  @endcode
 *  @li@c  ui4_flag                                          - The support color mode
 *  @li@c  pt_font                                           - The font information
 *  @li@c  ui1_clut_start                                    - The first color entry index in color table
 *  @li@c  ui1_clut_end                                      - The last color entry index in color table
 */
/*------------------------------------------------------------------*/
typedef struct _WGL_CC_ISDB_SYS_INIT_T
{
    UINT32                              ui4_flag;
    unsigned char*                      pt_font;
    /* canvas palette entry range for teletext - start */
    /* valid only when WGL_TTX_SYS_INIT_FLAG_PALETTE_MODE is ON */
    UINT8                               ui1_clut_start;
    UINT8                               ui1_clut_end;
}   WGL_CC_ISDB_SYS_INIT_T;


/*-----------------------------------------------------------------------------
                    data declarations, extern, static, const
----------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------------
                    functions declarations
-----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief Initialize the ISDB Closed Caption configuration
 *  @param [in]     pt_sys_init                   - Configuration context
 *  @param [out]
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      WGLR_OK                - The routine was successful.
 *  @retval      WGLR_OUT_OF_MEMORY     - The routine was fail.
 */
/*------------------------------------------------------------------*/
INT32 x_wgl_cc_isdb_init( WGL_CC_ISDB_SYS_INIT_T* pt_sys_init );
extern INT32 x_wgl_cc_isdb_font_init( CHAR* ps_fontfile );



#endif /* _X_WGL_CC_ISDB_H_ */


/*----------------------------------------------------------------------------*/
/*! @} */                                /* end of groupMW_WGL_CCI */
/*----------------------------------------------------------------------------*/



