/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/*! @file x_wgl_sbtl.h 
 *  $RCSfile: $
 *  $Revision: #1 $
 *  $Date: 2015/05/29 $
 *  $Author: brianpc.huang $
 *  
 *  @par Description:
 *         This header file contains public interfaces, constant and type 
 *         definitions of the Widget Library.
 */
/*----------------------------------------------------------------------------*/

#ifndef _X_WGL_SBTL_H_
#define _X_WGL_SBTL_H_

/*-----------------------------------------------------------------------------
                    include files
 ----------------------------------------------------------------------------*/
#include "u_wgl_sbtl.h"

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_WGL_SBTL
*  @{  
*/
/*----------------------------------------------------------------------------*/


#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
                    macros, defines, typedefs, enums
 ----------------------------------------------------------------------------*/

#define WGL_SBTL_MAX_PALETTE_COLOR_NUM      64    /**< the subtitle palette size */


/*-----------------------------------------------------------------------------
                    structures
 ----------------------------------------------------------------------------*/
/* system init structure */
#define WGL_SBTL_SYS_INIT_FLAG_SOFTWARE_SCALER      MAKE_BIT_MASK_32(0)    /**< flag to enable software scaler */
#define WGL_SBTL_SYS_INIT_FLAG_PALETTE_MODE         MAKE_BIT_MASK_32(1)    /**< flag to make subtitle widget work in palette mode */
/*------------------------------------------------------------------*/
/*! @struct WGL_SBTL_SYS_INIT_T
 *  @brief  This structure contains the configuration data for subtitle stream widget
 *  @code
 *  typedef struct _WGL_SBTL_SYS_INIT_T
 *  {
 *      UINT32                      ui4_flag;
 *      GL_COLORMODE_T              e_clr_mode_frm_buf;
 *      UINT8                       ui1_cnvs_plt_sbtl_start;
 *      UINT8                       ui1_cnvs_plt_sbtl_end;
 *  } WGL_SBTL_SYS_INIT_T;
 *  @endcode
 *  @li@c  ui4_flag                                          - initialization flags
 *  @li@c  e_clr_mode_frm_buf                                - color mode
 *  @li@c  ui1_cnvs_plt_sbtl_start                           - start position in palette
 *  @li@c  ui1_cnvs_plt_sbtl_end                             - end position in palette
 */
/*------------------------------------------------------------------*/
typedef struct _WGL_SBTL_SYS_INIT_T
{
    UINT32                      ui4_flag;
    GL_COLORMODE_T              e_clr_mode_frm_buf;
    
    /* canvas palette entry range for subtitle - start */
    /* valid only when WGL_SBTL_SYS_INIT_FLAG_PALETTE_MODE is ON */
    UINT8                       ui1_cnvs_plt_sbtl_start;
    
    /* canvas palette entry range for subtitle - end */
    /* valid only when WGL_SBTL_SYS_INIT_FLAG_PALETTE_MODE is ON */
    /* (ui1_cnvs_plt_sbtl_end - ui1_cnvs_plt_sbtl_start + 1) must be WGL_SBTL_MAX_PALETTE_COLOR_NUM */
    UINT8                       ui1_cnvs_plt_sbtl_end;
    
} WGL_SBTL_SYS_INIT_T;

 

/*-----------------------------------------------------------------------------
                    functions
 ----------------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
/*! @brief Initialize the DVB subtitle widget configuration
 *  @param [in]     pt_sys_init                 - Configuration context
 *  @param [out]
 *  @note
 *  @see
 *  @return     Return the error code.
 *  @retval      WGLR_OK                - The routine was successful.
 *  @retval      WGLR_INV_ARG           - The routine was fail.
 */
/*------------------------------------------------------------------*/
extern INT32 x_wgl_sbtl_init(
    WGL_SBTL_SYS_INIT_T                 *pt_sys_init);


#ifdef __cplusplus
}
#endif


#endif

/*----------------------------------------------------------------------------*/
/*! @addtogroup groupMW_WGL_SBTL
*  @{  
*/
/*----------------------------------------------------------------------------*/


