/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: chksum.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   checksum register header file
 *
 * Author:
 * -------
 *   HsHuang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#ifndef _CHKSUM_H_
#define _CHKSUM_H_

#include "x_hal_5381.h"
#include "ether_com.h"

/****************************************************************************
** Config definitions
****************************************************************************/
#define HW_CHKSUM_ON_THE_FLY  0 /* apply hw to calculate checksum */
//need to set CONFIG_ETHER_HW_CHKSUM  1 in Ether_priv.h

// *********************************************************************
// checksum Registers offset
// *********************************************************************
#define CHECKSUM_REG_OFFSET     (IO_VIRT + 0x29700)

// *********************************************************************
// 1's complement checksum registers map
// *********************************************************************
#if 0
#define RW_CHKSUM_ADDR   0x00     //	UDP/TCP/IP Checksum Start Address										
  #define CHECKSUM_SADR_MASK 	 ((UINT32)0xffffFFFF)	  //Checksum Start Address (Byte Address)

#define RW_CHKSUM_LENGTH  0x04
  #define CHECKSUM_LENGTH_MASK 	 ((UINT32)0xffffFFFF)	  // Checksum Length (Byte)										

#define RW_CHKSUM_CTRL     0x08
  #define  OUT_INV		((UINT32)0x01 << 7)  // 1's complement of UDP/TCP/IP Checksum Result Data
  #define  OUT_REV		((UINT32)0x01 << 6)  //Reverse Byte of Checksum Result Data
  #define  BYTE_REV         ((UINT32)0x01 << 5)  //Reverse Byte of Checksum Input Data (Word)
  #define  DW_REV	      ((UINT32)0x01 << 4)  //Reverse Byte of Checksum Input Data (Double Word)	
  #define  INTR_EN		((UINT32)0x01 << 1) //Checksum Interrupt Enable
  #define  CHECKSUM_EN	((UINT32)0x01 << 0)  //Checksum Enable. When Checksum is finish, this bit will be clear to 0.	

#define RW_CHKSUM_RESULT     0x0C
  #define  CHECKSUM_RESULT_MASK	((UINT32)0x0FFFF)  //Checksum Result Data Mask

#define RW_CHKSUM_DMA_CTRL     0x10
  #define  CHECKSUM_DMA_IDLE_TIME_MASK	((UINT32)0xFFFF0000)  //Checksum DMA Idle Time (dram clock cycle)
  #define  CHECKSUM_DMA_ALE_NUM_MASK	((UINT32)0x0000FFFF)  //Checksum Burst Length (64 bytes). If these registers set to 0, burst length is CHECKSUM_LENGTH.
#endif
// *********************************************************************
// Macros
// *********************************************************************
#define vWriteChkSum(dAddr, dVal)  IO_WRITE32((CHECKSUM_REG_OFFSET), (dAddr), (dVal))
#define dReadChkSum(dAddr)         IO_READ32((CHECKSUM_REG_OFFSET), (dAddr))
#define SetBitChkSum(Reg, Bit)     vWriteChkSum(Reg, dReadChkSum(Reg) |   (Bit) )
#define ClrBitChkSum(Reg, Bit)     vWriteChkSum(Reg, dReadChkSum(Reg) & (~(Bit)))


/****************************************************************************
** Function prototype
****************************************************************************/
void  vChksumDeQueue(void);
void vCheckSumInit(void);
INT32 _i4EtherTxChkSumVerify(UINT32 u4Size, UINT32 u4Count, BOOL fgSingleSizeTest);
INT32 _i4EtherRxChkSumVerify(UINT32 u4Size, UINT32 u4Count, BOOL fgSingleSizeTest);
#endif // _CHKSUM_H_
