/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_cli.c $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ether CLI debug mode header file
 *
 * Author:
 * -------
 *   Hs Huang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime: 
 *
 * $Revision:
****************************************************************************/

#ifndef X_ETHER_CLI_H
#define X_ETHER_CLI_H

/****************************************************************************
** Constant definitions
****************************************************************************/
typedef enum 
{
    OID_CLI_OID_START        	            = 0,
    OID_CLI_IF_INDEX                        = 1,        /* param : INT32 */
    OID_CLI_IF_DESCR                        = 2,        /* param : CHAR[256] */
    OID_CLI_IF_TYPE                         = 3,        /* param : UINT32 */
    OID_CLI_IF_MTU                          = 4,        /* param : UINT32 */
    OID_CLI_IF_SPEED                        = 5,        /* param : UINT32 */
    OID_CLI_IF_PHYADDRESS                   = 6,        /* param : MAC_ADDRESS_T */
    OID_CLI_IF_MULTICASTADDRESSLIST         = 7,        /* param : NET_MAC_ADDRESS_LIST_T */
    OID_CLI_IF_MULTICASTADDRESS             = 8,        /* param : MAC_ADDRESS_T */
    OID_CLI_IF_ADMIN_STATUS                 = 9,        /* param : UINT32 */
    OID_CLI_IF_OPERSTATUS                   = 10,       /* param : UINT32 */
    OID_CLI_IF_LASTCHANGE                   = 11,       /* param : UINT32 */
    OID_CLI_IF_NAME                         = 12,       /* param : CHAR[256] */
    OID_CLI_IF_PROMISCUOUS_MODE             = 13,       /* param : UINT32 */
    /* the following info will record in net_dev */
    OID_CLI_IF_IN_OCTETS                    = 14,       /* param : UINT64 */
    OID_CLI_IF_IN_UCASTPKT                  = 15,       /* param : UINT64 */
    OID_CLI_IF_IN_DISCARDS                  = 16,       /* param : UINT64 */
    OID_CLI_IF_IN_ERRORS                    = 17,       /* param : UINT64 */
    OID_CLI_IF_IN_UNKNOWN_PROTOS            = 18,       /* param : UINT64 */
    OID_CLI_IF_OUT_OCTETS                   = 19,       /* param : UINT64 */
    OID_CLI_IF_OUT_UCASTPKTS                = 20,       /* param : UINT64 */
    OID_CLI_IF_OUT_DISCARDS                 = 21,       /* param : UINT64 */
    OID_CLI_IF_OUT_ERRORS                   = 22,       /* param : UINT64 */
    OID_CLI_IF_IN_MULTICAST_PKTS            = 23,       /* param : UINT64 */
    OID_CLI_IF_IN_BROADCAST_PKTS            = 24,       /* param : UINT64 */
    OID_CLI_IF_OUT_MULTICAST_PKTS           = 25,       /* param : UINT64 */
    OID_CLI_IF_OUT_BROADCAST_PKTS           = 26,       /* param : UINT64 */
    OID_CLI_IF_IP_ADDRESS                   = 27,       /* param : UINT32, IP address (get only) */
    OID_CLI_IF_MULTICASTADDRESS_DEL         = 28,       /* remove multicast addr */
    OID_CLI_IF_CONNECT_STATUS               = 29,       /* param : UINT32 */
    OID_CLI_IF_NIC_STATUS                   = 30,       /* param : UINT32 */
    OID_CLI_IF_CURRNET_SPEED_INFO           = 31,       /* param : UINT32 */
    /* -------------------------------------------------------------- */
    OID_CLI_ETHER_OID_START                 = 40,       
    OID_CLI_ETHER_ALIGNMENT_ERRORS          = 41,       /* param : UINT32 */
    OID_CLI_ETHER_FCSERRORS                 = 42,       /* param : UINT32 */
    OID_CLI_ETHER_SINGLECOLLISIONFRAMES     = 43,       /* param : UINT32 */
    OID_CLI_ETHER_MULTIPLECOLLISIONFRAMES   = 44,       /* param : UINT32 */
    OID_CLI_ETHER_DEFERREDTRANSMISSIONS     = 45,       /* param : UINT32 */
    OID_CLI_ETHER_LATECOLLISIONS            = 46,       /* param : UINT32 */
    OID_CLI_ETHER_EXCESSIVECOLLISIONS       = 47,       /* param : UINT32 */
    OID_CLI_ETHER_INTERNALMACTRANSMITERRORS = 48,       /* param : UINT32 */
    OID_CLI_ETHER_FRAMETOOLONGS             = 49,       /* param : UINT32 */
    OID_CLI_ETHER_INTERNALMACRECEIVEERRORS  = 50,       /* param : UINT32 */
    OID_CLI_ETHER_SYMBOLERRORS              = 51,       /* param : UINT32 */
    OID_CLI_ETHER_DUPLEXSTATUS              = 52,       /* param : UINT32 */
    /* -------------------------------------------------------------- */
	OID_CLI_ETHER_CLI_MAX
}   ETHER_CLI_OID_T;

/****************************************************************************
** Structure and Type definitions
****************************************************************************/

/****************************************************************************
** Export API
****************************************************************************/

/****************************************************************************
** External Network API
****************************************************************************/
INT32 _i4EtherInit(void);
INT32 _i4EtherDeInit(void);
INT32 _i4EtherStart(void);
INT32 _i4EtherStop(void);
INT32 _i4EtherReadPhy(UINT32 u4RegAddr,UINT32 u4Len);
INT32 _i4EtherWritePhy(UINT32 u4RegAddr, UINT32 u4RegData);
INT32 _i4EtherCL45Read(UINT32 u4RegAddr,UINT32 u4Len);
INT32 _i4EtherCL45Write(UINT32 u4RegAddr, UINT32 u4RegData);
INT32 _i4EtherPhyANTest(void);
INT32 _i4EtherSetLoopBack(INT32 i4Enable);
INT32 _i4EtherSendFrame(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4EtherOidSet(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4EtherOidGet(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4CliEthMacAddrSet(void *pvArgv);
INT32 _i4CliEthMacAddrGet(void *pvArgv);
INT32 _i4EtherReadRegister(UINT32 u4Offset, UINT32 u4Len);
INT32 _i4EtherWriteRegister(UINT32 u4Offset, UINT32 u4Value);
INT32 _i4EtherGetBits(UINT32 u4Offset, UINT32 u4MSB, UINT32 u4LSB);
INT32 _i4EtherSetBits(UINT32 u4Offset, UINT32 u4MSB, UINT32 u4LSB, UINT32 u4Value);

INT32 _i4EtherDMAAutoPollingTest(INT32 i4Argc, const CHAR ** szArgv);

INT32 _i4EtherAdjPhySignal(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4EtherGetPhySignalSetting(INT32 i4Argc, const CHAR ** szArgv);
//INT32 i4EtherIndicateFrame(NET_DRV_IF_T *prDev, PKT_BUFF_T *prPktBuf);

INT32 i4ChkSumTestTaskCtrl(BOOL TxEnable, UINT32 TxCkSumSize, BOOL RxEnable, UINT32 RxCkSumSize);

#if CONFIG_CHKSUM
INT32 _i4CheckSumInit(void);
INT32 _i4EtherTxChkSumTest(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4EtherRxChkSumTest(INT32 i4Argc, const CHAR ** szArgv);
INT32 _i4EtherTxRxChkSumTest(INT32 i4Argc, const CHAR ** szArgv);
#endif

#endif /* X_ETHER_CLI_H */
