/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_drv.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ethernet driver header file
 *
 * Author:
 * -------
 *   HsHuang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#ifndef _ETHER_DRV_H_
#define _ETHER_DRV_H_

#include "ether_priv.h"
#include "x_hal_5381.h"

/****************************************************************************
** Constant definitions
****************************************************************************/
#define ETHER_PKT_BUF_SZ            (PKT_BUFFER_SIZE)

/* added 16 byte is for 128-bit alignment of rx buffer */
#define ETHER_RX_PKT_POOL_SZ        (ETHER_PKT_BUF_SZ * ETHER_RX_PKT_BUF_NUM) + 16
#define ETHER_TX_PKT_POOL_SZ        (ETHER_PKT_BUF_SZ * ETHER_TX_PKT_BUF_NUM) + 16

#define ETHER_PKT_MASK              0x55AA1234

#define NUM_TX_DESC                 48                      /* continuous biggest tx packet will not fill 
                                                               whole descriptors at normal operation */
#define NUM_RX_DESC                 64                      /* continuous smallest rx packet will not fill 
                                                               whole descriptors at normal operation */

#define ETHER_TX_PKT_BUF_NUM        NUM_TX_DESC
#define ETHER_RX_PKT_BUF_NUM        NUM_RX_DESC * 3         /* packet buffer count must be greater than twice of
                                                               descriptor count to ensure every buffer hooked on 
                                                               descriptor is flushed and invalidated */    

#define MAX_RECV_PKT_CNT            (UINT32)(NUM_RX_DESC)
#define MAX_TXMIT_PKT_CNT           (UINT32)(NUM_TX_DESC)
#define MAX_RECV_BYTE_CNT           (UINT32)(15000)
#define MAX_TXMIT_BYTE_CNT          (UINT32)(15000)

#define MAX_INT_LOOP_CNT            (UINT32)(2)

/****************************************************************************
** Structure and Type definitions
****************************************************************************/

/****************************************************************************
** Macro
****************************************************************************/
//#define NONCACHE(addr)				(((addr) & 0x0fffffff) | 0x60000000)
//#define NONCACHE(addr)				(((addr) & 0x0fffffff) | 0x40000000)
//#define NONCACHE(addr)				(((addr) & 0x0fffffff) | 0x80000000)

#define UTIL_Printf		Printf
/* The macro set rx own bit to indicate descriptor is owned by DMA */
#define INIT_RX_DESC(prCtrl) \
		{\
			prCtrl->Bits.u2BufSize = PKT_BUFFER_SIZE; \
			prCtrl->Bits.fgOwn = OWNED_BY_DMA; \
		}


/* The macro lock semaphore */
#define ETHER_SEMA_LOCK(hHandle)  VERIFY(OSR_OK == x_sema_lock(hHandle, X_SEMA_OPTION_WAIT))

/* The macro unlock semaphore */
#define ETHER_SEMA_UNLOCK(hHandle)  VERIFY(OSR_OK == x_sema_unlock(hHandle))

/****************************************************************************
** Function Declarations
****************************************************************************/
void  vEtherDrvStart(NET_DRV_IF_T *prDev);
void  vEtherDrvStop(NET_DRV_IF_T *prDev);
void  vEtherRecvPacket(NET_DRV_IF_T *prDev);
void  vEtherDeferTask(NET_DRV_IF_T *prDev);
void  vEtherAdapterRemove(PETHER_ADAPTER_T prMac);
void  vEtherModuleInit(NET_DRV_IF_T *prDev);
void vEtherSendComplete(NET_DRV_IF_T *prDev);
void  vEtherDrvSwReset(NET_DRV_IF_T *prDev);
void  vEtherLinkCheck(NET_DRV_IF_T *prDev);
INT32 i4EtherTxmitFrame(PETHER_ADAPTER_T prMac, PKT_BUFF_T *prPkt);
INT32 i4EtherAdapterCreat(NET_DRV_IF_T *prDev);
void vWriteHashTable(void);
void vStopSoftPollingPhy(UINT8 u1Stop);
void vEnableSupportTxCheckSumOffLoad(BOOL fgEnable);
#if 0
BOOL  GET_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
void  SET_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
void  CLR_ETHER_INFO(PETHER_ADAPTER_T prMac, UINT32 Status);
#endif
#endif // _ETHER_DRV_H_
