/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: ether_phy.h $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Ethernet PHY header file
 *
 * Author:
 * -------
 *   HsHuang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#ifndef _ETHER_PHY_H_
#define _ETHER_PHY_H_

/****************************************************************************
** MII PHY register definitions
****************************************************************************/
#define PHY_DEV_ADDR            (0x01)    // Address 1

// MII PHY register 
#define	MII_PHY_CTRL_REG        0x00					/* Basic Control */
  #define SW_RESET              ((unsigned)0x01 << 15)  
  #define LOOKBACK_EN           ((unsigned)0x01 << 14)  
  #define SPEED_100             ((unsigned)0x01 << 13)  
  #define AN_ENABLE             ((unsigned)0x01 << 12)  
  #define PWR_DOWN              ((unsigned)0x01 << 11)  
  #define RESTART_AN            ((unsigned)0x01 << 9)  
  #define DUPLEX_FULL           ((unsigned)0x01 << 8)  
  
#define	MII_PHY_STATUS_REG      0x01					/* Basic Status */
  #define AN_COMPLT             ((unsigned)0x01 << 5)  
  #define LINK                  ((unsigned)0x01 << 2)  

#define	MII_PHY_ID0_REG         0x02					/* PHY Identifier 1 */
#define	MII_PHY_ID1_REG         0x03					/* PHY Identifier 2 */
#define	MII_PHY_ANAR_REG        0x04					/* Auto Negotiation Advertisement */
#define	MII_PHY_ANLPAR_REG      0x05					/* Auto Negotiation Link Partner Ability */
  #define PARTNER_100BASE_FULL  ((unsigned)0x01 << 8)  
  #define PARTNER_100BASE       ((unsigned)0x01 << 7)  
  #define PARTNER_10BASE_FULL   ((unsigned)0x01 << 6)  
  #define PARTNER_10BASE        ((unsigned)0x01 << 5)  
  
#define	MII_PHY_ANER_REG        0x06					/* Auto Negotiation Expansion */


/****************************************************************************
** Export API
****************************************************************************/
INT32 i4MacPhyReset(UINT32 u4PhyAddr);
INT32 i4MacPhyInit(PETHER_ADAPTER_T prMac);
INT32 i4MacIntPhyInit(UINT32 u4PhyAddr);
INT32 i4MacPhyWrite(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData);
INT32 i4MacPhyRead(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 *pu4Data);
INT32 i4MacCL45Write(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 u4RegData);
INT32 i4MacCL45Read(UINT32 u4PhyAddr, UINT32 u4RegAddr, UINT32 dwLen, UINT32 *pu4Data);
INT32 i4MacPhyNegotitate(PETHER_ADAPTER_T prMac);
INT32 i4MacPhyAddrGet(PETHER_ADAPTER_T prMac);
INT32 i4MacPhyLoopback(UINT32 u4PhyAddr, BOOL fgLoopback);
#if defined(CC_MT5890) || defined(CC_MT5891)
INT32 i4MacPhyZCALInit(UINT32 u4PhyAddr);
INT32 i4MacPhyZCALRemapTable(UINT32 u4PhyAddr, UINT8 value);
#endif
#endif // _ETHER_PHY_H_
