/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef __NET_DEBUG_H__
#define __NET_DEBUG_H__

#define DEFINE_IS_LOG	NET_IsLog
#include "x_debug.h"

/****************************************************************************
** External variable
****************************************************************************/
extern UINT32 _u4EtherDbgLevel;
extern UINT32 _u4WlanDbgLevel;
extern UINT32 _u4M6s56DbgLevel;

/****************************************************************************
** Export Macro
****************************************************************************/
//#define NETLOG(lvl, fmt...)   LOG(lvl, fmt)  
#define NETLOG(lvl, fmt...)   Printf(fmt)

#define ERR_NETLOG(_Fmt...) \
        { \
            NETLOG(LOG_ERROR, "**Error[%s:%d] - ", __FILE__, __LINE__); \
            NETLOG(LOG_ERROR, _Fmt); \
        }

#define WARN_NETLOG(_Fmt...) \
        { \
            NETLOG(LOG_WARN, "**Warning[%s:%d] - ", __FILE__, __LINE__); \
            NETLOG(LOG_WARN, _Fmt); \
        }

#define INFO_NETLOG(_Fmt...) \
        { \
            NETLOG(LOG_INFO, "%s: ", __FUNCTION__); \
            NETLOG(LOG_INFO, _Fmt); \
        }

#define NOTE_NETLOG(_Fmt...) \
        { \
            NETLOG(LOG_NOTE, _Fmt); \
        }

#define DBG_NETLOG(_Fmt...) \
        { \
            NETLOG(LOG_DEBUG, "%s: ", __FUNCTION__); \
            NETLOG(LOG_DEBUG, _Fmt); \
        }


/******************************************************************************
* Ethernet debug log macro
******************************************************************************/
#define ERR_ETHER(_Fmt...) \
        { \
            if (_u4EtherDbgLevel >= LOG_ERROR) { \
                NETLOG(LOG_ERROR, "**Error[%s:%d] - ", __FILE__, __LINE__); \
                NETLOG(LOG_ERROR, _Fmt); \
            }\
        }

#define WARN_ETHER(_Fmt...) \
        { \
            if (_u4EtherDbgLevel >= LOG_WARN) { \
                NETLOG(LOG_WARN, "**Warning[%s:%d] - ", __FILE__, __LINE__); \
                NETLOG(LOG_WARN, _Fmt); \
            }\
        }

#define INFO_ETHER(_Fmt...) \
        { \
            if (_u4EtherDbgLevel >= LOG_INFO) { \
            NETLOG(LOG_INFO, "[ETHER] "); \
            NETLOG(LOG_INFO, _Fmt); \
            }\
        }

#define NOTE_ETHER(_Fmt...) \
        { \
            if (_u4EtherDbgLevel >= LOG_NOTE) { \
                NETLOG(LOG_NOTE, _Fmt); \
            }\
        }

#define DBG_ETHER(_Fmt...) \
        { \
            if (_u4EtherDbgLevel >= LOG_DEBUG) { \
                NETLOG(LOG_DEBUG, "%s: ", __FUNCTION__); \
                NETLOG(LOG_DEBUG, _Fmt); \
            }\
        }

/******************************************************************************
* Wlan debug log macro
******************************************************************************/
#define ERR_WLAN        ERR_NETLOG
#define WARN_WLAN       WARN_NETLOG
#define INFO_WLAN(_Fmt...) \
        { \
            NETLOG(LOG_INFO, "[WLAN] "); \
            NETLOG(LOG_INFO, _Fmt); \
        }
#define NOTE_WLAN       NOTE_NETLOG
#define DBG_WLAN        DBG_NETLOG

/******************************************************************************
* DTCP debug log macro
******************************************************************************/
#define ERR_DTCP        ERR_NETLOG
#define WARN_DTCP       WARN_NETLOG
#define INFO_DTCP(_Fmt...) \
        { \
            NETLOG(LOG_INFO, "[DTCP] "); \
            NETLOG(LOG_INFO, _Fmt); \
        }
#define NOTE_DTCP       NOTE_NETLOG
#define DBG_DTCP        DBG_NETLOG

#endif /* __NET_DEBUG_H__ */

