/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: net_if.c $
 *
 * Project:
 * --------
 *
 *
 * Description:
 * ------------
 *   Network driver interface source file
 *
 * Author:
 * -------
 *   Hs Huang
 *
 * Last changed:
 * -------------
 * $Author:
 *
 * $Modtime:
 *
 * $Revision:
****************************************************************************/

#include "x_lint.h"
LINT_EXT_HEADER_BEGIN

#include "x_assert.h"
#include "net_if.h"
#include "net_debug.h"

LINT_EXT_HEADER_END

/****************************************************************************
** Local definitions
****************************************************************************/

/****************************************************************************
** Local structures and enumerations
****************************************************************************/

/****************************************************************************
** Function prototypes
****************************************************************************/

/****************************************************************************
** Global variables
****************************************************************************/
UINT32 _u4EtherDbgLevel = LOG_NOTE;
UINT32 _u4WlanDbgLevel = LOG_NOTE;
UINT32 _u4M6s56DbgLevel = LOG_NOTE;


/************************************************************************
  Function : void vNetDelay1us(BYTE bValue)
  Description : delay 1 us
  Parameter   : None
  Return      : None
  Note          : RISC clock is 405MHz
************************************************************************/
void vNetDelay1us(UINT32 u4Value)
{                          
  UINT32 u4Cnt;
  
  for(u4Cnt = 0; u4Cnt < (u4Value * 25); u4Cnt++)
  {
        ;
  }
}

/*******************************************************************************
**  u4NetGetTimeTick
**
**  @brief  This function returns the number of milliseconds that have
**          elapsed since the system was booted.
**  @param
**  @result
**  @return the LSB 32-bits of the system uptime
**  @note
*******************************************************************************/
UINT32 u4NetGetTimeTick(void)
{
    UINT32 ui4_os_tick_period;
    
    ui4_os_tick_period = (UINT32)x_os_get_sys_tick_period();		
    return ((UINT32)(x_os_get_sys_tick() * ui4_os_tick_period));	
}

/*******************************************************************************
**  pvNetAlignAddr
**
**  descriptions:
**      This routine is called to align memory address
**  parameters:
**      pAddr - pointer to memory address
**      u4Align - alignment unit (bit), for example, 32 means 32-bit alignment
**  result:
**  return:
**      pAddr - pointer to aligned address
*******************************************************************************/
void *pvNetAlignAddr(void *pAddr, UINT32 u4Align)
{
    UINT32 u4AlignTmp; 
    
    ASSERT(pAddr != NULL);

    u4Align = u4Align / 8;
    if(u4Align <= 1)
    {
        return ((void *)pAddr);
    }
    else
    {
        u4AlignTmp = ((UINT32)pAddr + u4Align - 1) & ~(u4Align - 1);
        return ((void *)u4AlignTmp);
    }
}

/******************************************************************************
* Function      : MW_NET_Init
* Description   : Middleware network driver init entry point 
* Parameter     : None
* Return        : 0 if success 
*                 negative id fail 
******************************************************************************/
INT32 MW_NET_Init(void)
{
#if 0
    INT32 i4_ret = 0;
    NET_DRV_IF_T *prDev;

    vNetTestGpioInit();

    prDev = (NET_DRV_IF_T *) &_rWlanDev;
	
   	/* Init wlan driver */
 	i4_ret = i4WlanDrv_Init(prDev);			// only call it here while testing
    if (i4_ret != NET_DRV_OK)
    {
        return(i4_ret);
    }
#endif
    return (NET_DRV_OK);
}

