/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

#define BUFFER 256

int main(int argc, char *argv[])
{
    unsigned int current_name = 1;
    char buffer[BUFFER], previous;
    char buffer2[BUFFER];

    do
    {
        int current_file = -1, current_tmp_file = -1;
        int not_first = 0, line_comment = 0, multi_comment = 0, quote = 0,
            escape = 0, to_print = 0, I = 0;
        ssize_t buffered_size;

        // open each file 
        if (argc < 2) 
        {
            current_file = STDIN_FILENO;
        } 
        else 
        {
            current_file = open(argv[current_name], O_RDONLY);

            if (current_file < 0)
            {
                fprintf(stderr, "%s: error opening file '%s': %s\n", argv[0],
                  argv[current_name++], strerror(errno));
                continue;
            }

            sprintf(buffer, "%s_tmp", argv[current_name]);
            current_tmp_file = open(buffer, O_WRONLY | O_CREAT | O_TRUNC);
            if (current_tmp_file < 0)
            {
                fprintf(stderr, "%s: error opening file '%s': %s\n", argv[0],
                  buffer, strerror(errno));
                continue;
            }
        }          

        while ((buffered_size = read(current_file, buffer, BUFFER)) != 0)
        {
            if (buffered_size == (ssize_t) -1)
            {
                if (errno == EINTR) 
                {
                    continue;
                } 
                else
                {
                    break;
                } 
            }


            for (I = 0; I < (signed) buffered_size; I++)
            {
                if (line_comment) 
                {
                    line_comment = !(to_print = (buffer[I] == '\n'));
                }
                else switch (buffer[I])
                {
                case '#': 
                    if (not_first && previous == '\n')
                    {
                        to_print     = 0;
                        line_comment = 1;
                    }
                    else if(!not_first)
                    {
                        to_print     = 0;
                        line_comment = 1;
                    }
                    else
                    {
                        if (not_first && to_print)
                        {
                            if(previous != '\r')
                            {
                                write(current_tmp_file, &previous, 1);
                            }
                        } 
                        to_print = 1;
                    }
                    break;

                default: 
                    if (not_first && to_print) 
                    {
                        if(previous != '\r')
                        {
                            write(current_tmp_file, &previous, 1);
                        }
                    }
                    to_print = 1;
                    break;
                }

                not_first = 1;
                previous  = buffer[I];
            }
        }

        // the last byte
        if (not_first) 
        {
            if(previous != '\r')
            {
                write(current_tmp_file, &previous, 1);
            }
        }

        if(current_file >= 0) 
        {
            close(current_file);
        }
        if(current_tmp_file >= 0) 
        {
            close(current_tmp_file);
        }        

        sprintf(buffer2, "rm -f %s", argv[current_name]);
        system(buffer2);

        sprintf(buffer, "%s_tmp", argv[current_name]);
        sprintf(buffer2, "mv -f %s %s", buffer, argv[current_name]);
        system(buffer2);
        
        sprintf(buffer2, "chmod 666 %s", argv[current_name]);
        system(buffer2);
        
        current_name++;
    }
    while (current_name < argc);

    return 0;
}

