/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: assert.c,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

/** @file assert.c
 *  assert.c provides Assert function for assertion.
 */


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_assert.h"
#include "x_printf.h"
#include "x_hal_arm.h"
#include "x_hal_5381.h"

#ifdef CC_MTK_LOADER
#include "x_serial.h"
#else
#include "x_util.h"
#endif

#if defined(FBM_FB_LOG) || defined(FBM_CS_LOG)
#include "fbm_drvif.h"
#endif

#if defined(CC_MTK_PRELOADER) && !defined(NDEBUG)
EXTERN void CHIP_DisplayString(const CHAR *szString);
EXTERN void CHIP_DisplayHex(UINT32 u4Val);
#endif /* defined(CC_MTK_PRELOADER) && !defined(NDEBUG) */

#ifdef CC_MTK_LOADER
#define LOG(level, fmt...) Printf(fmt)
#endif

//-----------------------------------------------------------------------------
/** Assert() is the only final function when assertion happens.
 *  @param szExpress the expression of the assertion.
 *  @param szFile the file name of the assertion.
 *  @param i4Line the line number of the assertion.
 */
//-----------------------------------------------------------------------------
void Assert(const CHAR *szExpress, const CHAR *szFile, INT32 i4Line)
{
#if !defined(CC_MTK_PRELOADER) && !defined(CC_MTK_LOADER)
    // Disable interrupts first
    HalDisableIRQ();
    HalDisableFIQ();
#endif /* CC_MTK_PRELOADER */

#ifndef CC_MTK_LOADER
#ifdef FBM_FB_LOG
    //Flush FB log
    FBM_FlushFbLog(10, FBM_FB_ID_UNKNOWN);
#endif

#ifdef FBM_CS_LOG
    //Flush FB CS log
    FBM_FlushFbCsLog();
#endif

    // Flush Log data.
    UTIL_LogFlush();
#endif /* CC_MTK_LOADER */

#if defined(CC_MTK_PRELOADER) && !defined(NDEBUG)
    CHIP_DisplayString("\nAssertion fails at:\nFile: ");
    CHIP_DisplayString(szFile);
    CHIP_DisplayString(", line ");
    CHIP_DisplayHex(i4Line);
    CHIP_DisplayString("\n\t");
    CHIP_DisplayString(szExpress);
    CHIP_DisplayString("\n\nProgram terminated.\n");
#else /* defined(CC_MTK_PRELOADER) && !defined(NDEBUG) */
#ifndef __KERNEL__
    // Show assertion messages
    LOG(0, "\nAssertion fails at:\nFile: %s, line %d\n", szFile, i4Line);
    LOG(0, "\n\t%s", szExpress);
    LOG(0, "\n\nProgram terminated.\n");
#endif
#endif /* defined(CC_MTK_PRELOADER) && !defined(NDEBUG) */

    // Halt the whole system
#if defined(CC_MTK_PRELOADER) || defined(CC_MTK_PRELOADER2)
    HaltSystem();
#else
    BSP_HaltSystem();
#endif
}
