/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

//============================================================================
// NOTE:
//      In this file, all function cannot call sub-function!!!
//============================================================================
#include "rs232.h"

#include "x_ckgen.h"
#include "x_assert.h"
#include "x_timer.h"
#include "x_bim.h"
#include "x_pdwnc.h"
#include "drvcust_if.h"
#include "c_model.h"
#include "hw_pll_group.h"
#include "hw_hpcpll.h"
#include "hw_ifpll.h"
#include "hw_pdwnc.h"
//#include "hw_ifpga.h"
#include "hw_ospe.h"
//#include "hw_mempll.h"
#include "hw_tg.h"
#include "hw_lvds.h"
#include "x_hal_5381.h"
#include "chip_txt.h"
#include "hw_gpu_mfg_top.h"

extern UINT32 IS_SupportCPUHiSpeed(VOID);

void CHIP_ClockSetting(void);
void CHIP_BimInit(void);
void CHIP_1stBootRegInit(void);

void CHIP_Delay_us(UINT32 u4Micros)
{
#ifdef CC_FPGA
    volatile UINT32 i;

    for (i = 0; i < (24 * u4Micros) * 100; i++)
    {
        ;
    }
#else
    UINT32 u4Xtal = 24 * u4Micros;
    UINT32 u4Org = BIM_READ32(REG_RW_TIMER2_LOW);
    ASSERT(u4Micros < 0xAAAAAAA);   //check 32-bit
    while (u4Org - BIM_READ32(REG_RW_TIMER2_LOW) < u4Xtal);
#endif
}

void CHIP_SRAM_PowerOn(void)
{
    vIO32Write4B(IO_VIRT + 0xd180, 0x0);
    vIO32Write4B(IO_VIRT + 0xd184, 0x0);
    vIO32Write4B(IO_VIRT + 0xd188, 0x0);
    vIO32Write4B(IO_VIRT + 0xd18c, 0x0);
    vIO32Write4B(IO_VIRT + 0xd190, 0x0);
    vIO32Write4B(IO_VIRT + 0xd194, 0x0);
    vIO32Write4B(IO_VIRT + 0xd198, 0x0);
    vIO32Write4B(IO_VIRT + 0xd19c, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1a0, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1a4, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1a8, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1ac, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1b0, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1b4, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1b8, 0x0);
    vIO32Write4B(IO_VIRT + 0xd1bc, 0x0);
    vIO32Write4B(IO_VIRT + 0xd280, 0x0);
    vIO32Write4B(IO_VIRT + 0xd284, 0x0);
    vIO32Write4B(IO_VIRT + 0xd288, 0x0);
    vIO32Write4B(IO_VIRT + 0xd28c, 0x0);
    vIO32Write4B(IO_VIRT + 0xd290, 0x0);
    vIO32Write4B(IO_VIRT + 0xd294, 0x0);
}
#ifndef CC_MTK_PRELOADER2
void CM4_SRAM_Poweron(void)
{
	CHIP_DisplayString("\nCM4 power on\n");
    vIO32Write4B(PDWNC_PDSYS_CLK_SEL, 0x400004); //set PDWNC bus & cm4 clock to 600MHz
    vIO32Write4B(PDWNC_CMSYS_PWR_CTL, (u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) | 0x00000003));  //power on cmsys (bit0 -> 1, bit1 -> 1)
    // release CM4 SRAM PD signal
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG0, 0x0);
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG1, (u4IO32Read4B(PDWNC_CMSYS_SRAM_CFG1) & 0xffff0000));
    vIO32Write4B(PDWNC_PWROPT, 0x01242400); //release CM4 pwr_rst_b
    while((u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) & 0x18) != 0x18); // wait power on ack
    vIO32Write4B(PDWNC_CMSYS_PWR_CTL, (u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) & 0xfffffffb));  //power isolation disable (bit2 -> 0)
    CHIP_Delay_us(1);
}
#ifdef CC_MTK_HOTBOOT_START
void CM4_SRAM_Resume(void)
{
	CHIP_DisplayString("\nCM4 power Resume\n");

    //Power On CM4 SRAM
    vIO32WriteFldAlign(PDWNC_PDSYS_CLK_SEL, 0x1, FLD_CMSYS_CK_SEL);//set PDWNC bus & cm4 clock to 600MHz (bit[5:2] -> 1)
    vIO32Write4B(PDWNC_CMSYS_PWR_CTL, (u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) | 0x00000003));  //power on cmsys (bit0 -> 1, bit1 -> 1)
    while((u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) & 0x18) != 0x18); // wait power on ack (bit3 -> 1, bit4 -> 1)
    vIO32Write4B(PDWNC_CMSYS_PWR_CTL, (u4IO32Read4B(PDWNC_CMSYS_PWR_CTL) & 0xfffffffb));  //power isolation disable (bit2 -> 0)

    // release CM4 SRAM signal
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG1, (u4IO32Read4B(PDWNC_CMSYS_SRAM_CFG1) | 0xffff0000)); // releas SRAM sleep signal (bit[31:16] -> 0xffff)
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG2, 0xffffffff); // releas SRAM sleep signal
    CHIP_Delay_us(1);
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG3, (u4IO32Read4B(PDWNC_CMSYS_SRAM_CFG3) | 0x00003f00));//release SRAM ISOINTB(bit[13:8] -> 0x3f)
    CHIP_Delay_us(1);
    vIO32Write4B(PDWNC_CMSYS_SRAM_CFG3, (u4IO32Read4B(PDWNC_CMSYS_SRAM_CFG3) & 0xffc0ffff));//release SRAM CKISO (bit[21:16] -> 0)

    vIO32Write4B(PDWNC_PWROPT, (u4IO32Read4B(PDWNC_PWROPT) | 0x01000000)); //release CM4 pwr_rst_b (bit24 -> 1)
    CHIP_Delay_us(1);
}
#endif
void CHIP_ClockSetting(void)
{
    int reg_val = 0;
	unsigned int efuse_reg_val = 0;

    CHIP_SRAM_PowerOn();

    // 0. Select CPU/BUS/SRAM/BIST to Xtal before PLL configuration
    vIO32Write4B(IO_VIRT + REG_MP2_PLL_DIVIDER_CFG, 0x0FF); // Select CPU as XTAL
    vIO32WriteFldAlign(CKGEN_CPU_CKCFG, 0, FLD_BUS_CK_SEL); // CPU_CK_SEL is removed
    vIO32WriteFldAlign(CKGEN_REG_AXI_CKCFG, 0, FLD_AXI_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_ABIST_SRAM_CKCFG, 0x0, FLD_ABIST_SRAM_SEL);
    vIO32WriteFldAlign(CKGEN_REG_BIST_CKCFG, 0x0, FLD_BIST_SEL);
    CHIP_Delay_us(40);

    // Xtal Gm control, increase driving current to 12X
    vIO32WriteFldMulti(PDWNC_XTALCFG0, P_Fld(0xF, FLD_RG_XTAL_GM) |
                                       P_Fld(0x0, FLD_RG_XTAL_AAC_EN));

    // 1. Init SYSPLL, G3DPLL, VDECPLL, DMUXPLL, HPCPLL
    // DA_PLL_SDM_PWR_ON = 1
    vIO32WriteFldMulti(CKGEN_REG_PLL_PWR_ON, P_Fld(1, FLD_DA_DMUXPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_VDECPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_SOCPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_G3DPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_APUPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_SYSPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_CCIPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_HPCPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_VGAPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_ARMPLL_SDM_PWR_ON));

    // DA_PLL_SDM_ISO_EN = 0
    vIO32WriteFldMulti(CKGEN_REG_PLL_ISO_EN, P_Fld(0, FLD_DA_DMUXPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_VDECPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_SOCPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_G3DPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_APUPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_SYSPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_CCIPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_HPCPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_VGAPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_ARMPLL_SDM_ISO_EN));
    CHIP_Delay_us(50);

    // Power down PLL
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG0, 0x0, FLD_RG_SYSPLL_EN);
    //vIO32WriteFldAlign(REG_CCIPLL_CFG1, 0x0, FLD_RG_CCIPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG1, 0x0, FLD_RG_VDECPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG2, 0x0, FLD_RG_VDECSOCPLL_EN);
    vIO32WriteFldAlign(REG_G3DPLL_CFG1, 0x0, FLD_RG_G3DPLL_EN);
    vIO32WriteFldAlign(REG_HPCPLL_CFG1, 0x0, FLD_RG_HPCPLL_EN);
    vIO32WriteFldAlign(REG_ARMPLL_CFG1, 0x0, FLD_RG_ARMPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG3, 0x0, FLD_RG_DMUXPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG7, 0x0, FLD_RG_APUPLL_EN);

    // Set PLLGP_BIAS_EN = 0; disable if ROM code is enabled.
    //vIO32WriteFldAlign(REG_PLL_GROUP_CFG11,  0x0,  FLD_RG_PLLGP_BIAS_EN);

    // Set SYSPLL for 648 MHz
    // SYSPLL_SDM_FRA_EN = 0 for integer application
    // SYSPLL = (Xtal * SYSPLL_SDM_PCW) / SYSPLL_PREDIV / SYSPLL_POSDIV
    // SYSPLL = (Xtal * 108) / 1 / 4 (default has devider /4, POSDIV /1 is enough)
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG0, P_Fld(0x00, FLD_RG_SYSPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_SYSPLL_POSDIV) |
                                           P_Fld(0x6C, FLD_RG_SYSPLL_SDM_PCW));

    // Set VDECPLL for 588 MHz
    // VDECPLL_SDM_FRA_EN = 0 for integer application
    // VDECPLL = (Xtal * VDECPLL_SDM_PCW) / VDECPLL_PREDIV / VDECPLL_POSDIV
    // VDECPLL = (Xtal * 98) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG1, P_Fld(0x00, FLD_RG_VDECPLL_SDM_FRA_EN) |
                                           P_Fld(0x02, FLD_RG_VDECPLL_POSDIV) |
                                           P_Fld(0x62, FLD_RG_VDECPLL_SDM_PCW));


    // Set VDECSOCPLL for 456 MHz
    // VDECSOCPLL_SDM_FRA_EN = 0 for integer application
    // VDECSOCPLL = (Xtal * VDECSOCPLL_SDM_PCW) / VDECSOCPLL_PREDIV / VDECSOCPLL_POSDIV
    // VDECSOCPLL = (Xtal * 76) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG2, P_Fld(0x00, FLD_RG_VDECSOCPLL_SDM_FRA_EN) |
                                           P_Fld(0x02, FLD_RG_VDECSOCPLL_POSDIV) |
                                           P_Fld(0x4C, FLD_RG_VDECSOCPLL_SDM_PCW));

    // Set G3DPLL for 800 MHz
    // G3DPLL_SDM_FRA_EN = 0 for integer application
    // G3DCPLL = (Xtal * G3DPLL_SDM_PCW) / G3DPLL_PREDIV / G3DPLL_POSDIV
    // G3DCPLL = (Xtal * 66) / 1 / 2

    //change GPU clock source to PLL
    vIO32WriteFldMulti(CKGEN_REG_G3D_CORE_CKCFG, P_Fld(0x1, FLD_G3D_CORE_SEL));

    vIO32WriteFldMulti(REG_G3DPLL_CFG1, P_Fld(0x01, FLD_RG_G3DPLL_SDM_FRA_EN) |
                                           P_Fld(0x01, FLD_RG_G3DPLL_POSDIV));

    vIO32Write4B(REG_G3DPLL_CFG0, (0x42u << 24)|(0xAAAAAA));

    // Set DMUXPLL for 384 MHz
    // DMUXPLL_SDM_FRA_EN = 0 for integer application
    // DMUXPLL = (Xtal * DMUXPLL_SDM_PCW) / DMUXPLL_PREDIV / DMUXPLL_POSDIV
    // DMUXPLL = (Xtal * 64) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG3, P_Fld(0x00, FLD_RG_DMUXPLL_SDM_FRA_EN) |
                                           P_Fld(0x02, FLD_RG_DMUXPLL_POSDIV) |
                                           P_Fld(0x40, FLD_RG_DMUXPLL_SDM_PCW));

    // Set HPCPLL as 1200 MHz
    // HPCPLL_SDM_FRA_EN = 0 for integer application
    // HPCPLL = (Xtal * HPCPLL_SDM_PCW) / HPCPLL_PREDIV / HPCPLL_POSDIV
    // HPCPLL = (Xtal * 100) / 1 / 2
    vIO32WriteFldMulti(REG_HPCPLL_CFG1, P_Fld(0x00, FLD_RG_HPCPLL_SDM_FRA_EN) |
                                        P_Fld(0x01, FLD_RG_HPCPLL_POSDIV));
    {
        UINT32 u4HPCPLL_SDM_PCW = 0x64;

        #ifdef CC_CPU_SPEED_TEST_ENABLE
        if ((u4IO32Read4B(IO_VIRT + 0x28128) == 0) && u4IO32Read4B(IO_VIRT + 0x28100) == 0)
        {
            vIO32Write4B(IO_VIRT + 0x8150, 0xFFFFFFFF);
            vIO32Write4B(IO_VIRT + 0x8154, u4HPCPLL_SDM_PCW);
        }
        else
        {
            // Get HPCPLL setting
            if (u4IO32Read4B(IO_VIRT + 0x8150) == 0xFFFFFFFF)
            {
                u4HPCPLL_SDM_PCW = u4IO32Read4B(IO_VIRT + 0x8154);
            }
        }
        #endif // CC_CPU_SPEED_TEST_ENABLE
        vIO32WriteFldAlign(CKGEN_ANA_PLLGP_CFG1, (u4HPCPLL_SDM_PCW << 24), FLD_ANA_PLLPG_CFG1);
    }
    CHIP_Delay_us(1);  //delay 0.1us

    // Set CCIPLL(CPUPLL) as 750MHz
    // CPUPLL = (Xtal * CCIPLL_SDM_PCW) / CCIPLL_PREDIV / CCIPLL_POSDIV
    // CPUPLL = (Xtal * 74) / 1 / 2
    vIO32WriteFldAlign(REG_CCIPLL_CFG0, 0x7D000000, FLD_RG_CCIPLL_SDM_PCW);
    vIO32WriteFldAlign(REG_CCIPLL_CFG1, 0x02, FLD_RG_CCIPLL_POSDIV);
    CHIP_Delay_us(1);  //delay 0.1us

    // Set ARMPLL for 1250Mhz
    // ARMPLL_SDM_FRA_EN = 0 for integer application
    // ARMPLL = (Xtal * ARMPLL_SDM_PCW) / ARMPLL_PREDIV / ARMPLL_POSDIV
    // ARMPLL = (Xtal * 100) / 1 / 2
    vIO32WriteFldMulti(REG_ARMPLL_CFG1, P_Fld(0x01, FLD_RG_ARMPLL_SDM_FRA_EN) |
                                        P_Fld(0x01, FLD_RG_ARMPLL_POSDIV));
    vIO32WriteFldAlign(REG_ARMPLL_CFG0, (0x68 << 24)|0x2AAAAB, FLD_RG_ARMPLL_SDM_PCW);

    // Set APUPLL for 792 MHz
    // VDECPLL_SDM_FRA_EN = 0 for integer application
    // VDECPLL = (Xtal * VDECPLL_SDM_PCW) / VDECPLL_PREDIV / VDECPLL_POSDIV
    // VDECPLL = (Xtal * 65) / 1 / 2
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG7, P_Fld(0x01, FLD_RG_APUPLL_SDM_FRA_EN) |
                                           P_Fld(0x01, FLD_RG_APUPLL_POSDIV) |
                                           P_Fld(0x41, FLD_RG_APUPLL_SDM_PCW));

    // Power on G3DPLL, SYSPLL, VDECPLL, HPCPLL, ARMPLL
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG0, 0x1, FLD_RG_SYSPLL_EN);
    vIO32WriteFldAlign(REG_CCIPLL_CFG1, 0x1, FLD_RG_CCIPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG1, 0x1, FLD_RG_VDECPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG2, 0x1, FLD_RG_VDECSOCPLL_EN);
    vIO32WriteFldAlign(REG_G3DPLL_CFG1, 0x1, FLD_RG_G3DPLL_EN);
    vIO32WriteFldAlign(REG_HPCPLL_CFG1, 0x1, FLD_RG_HPCPLL_EN);
    vIO32WriteFldAlign(REG_ARMPLL_CFG1, 0x1, FLD_RG_ARMPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG3, 0x1, FLD_RG_DMUXPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG7, 0x1, FLD_RG_APUPLL_EN);

    // Enable DDS setting
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG7, 0x1, FLD_RG_HADDS1_PWDB);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG8, 0x1, FLD_RG_XDDS_PWDB);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG18, 0x1, FLD_RG_DDDS1_PWDB);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG19, 0x1, FLD_RG_DDDS2_PWDB);
    CHIP_Delay_us(50);

    // 3. Set IFPLL as 864MHz
    // DEMOD BG and V2I enable, X=don't care, w 0x10033604 0xXXXXX000
    vIO32Write4B(REG_IFTOP_CFG1, 0x0);
    // IFPLL enable and setting
    vIO32Write4B(REG_IFPLL_CFG0, 0x0001003F);
    // IFPLL output enable 864MHz
    vIO32Write4B(REG_IFPLL_CFG1, 0xFC221A24);

    // 4. Init Ethernet PLL, USB PLL
    // Set VCO to 2400 MHz, then it will output 25M, 600M, and 480M clock
    // FLD_RG_XPLL_SDM_FRA_EN = 0 for integer application
    // XPLL = (Xtal * FLD_RG_XPLL_SDM_PCW) / FLD_RG_XPLL_POSDIV
    // XPLL = (Xtal * 100) / 0
    vIO32WriteFldMulti(PDWNC_PLLGPCFG0, P_Fld(0x00, FLD_RG_XPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_XPLL_POSDIV) |
                                           P_Fld(0x64, FLD_RG_DMUXPLL_SDM_PCW));

    vIO32WriteFldAlign(PDWNC_PLLGPCFG0, 0x1, FLD_RG_XPLL_EN);

    // already do this in step0
    CHIP_Delay_us(500);
    vIO32WriteFldAlign(CKGEN_REG_USB_CKCFG, 1, FLD_USB_SEL);
    vIO32WriteFldAlign(CKGEN_USB_CLK_CKCFG, 1, FLD_USBPHY_CK_DIV9);

    //keep RG_USB20_INTR_EN=0b, use external R current
    vIO32Write4B(IO_VIRT + 0xc0300, (u4IO32Read4B(IO_VIRT + 0xc0300) | 0x00008000)); // RG_USB20_USBPLL_FORCE_ON = 1b
    vIO32Write4B(IO_VIRT + 0xc0318, (u4IO32Read4B(IO_VIRT + 0xc0318) & (~ 0x00800000)));  // RG_USB20_BC11_SW_EN =0

    //add ssusb port 2 clock settings
    vIO32Write4B(IO_VIRT + 0xd47c,(u4IO32Read4B(IO_VIRT + 0xd47c) | 0x00000003));

    // 5. Select CPU clock to HPCPLL and Bus clock
    // cpu clock = HPCPLL , bus clock = 216Mhz
    /*
    Clock source selection[3:0]
        0: xtal_ck
        1: cpupll_ck
        2: enetpll_ck (600MHZ)
        3: syspll_ck (648MHz)
        4: usbpll_ck (480MHz)
        5: sawlesspll_ck (432MHz)
        6: enetpll_d2_ck (300MHZ)
        7: usbpll_d2_ck (240MHz )
        8: sawlesspll_d2_ck (216MHz )
        9: sawlesspll_d3_ck (144MHz )
        10: usbpll_d4_ck (120MHz )
        11: sawlesspll_d4_ck (108MHZ)
        12: enetpll_d4_ck (150MHZ)
        13: MEM_CK
        14: slow_ck
        15: bist_ck
    BUS_CK selection[10:8]
        0: bus_ck = xtal_ck
        1: bus_ck = sawlesspll_d2_ck
        2: bus_ck = usbpll_d2p5_ck
        3: bus_ck = syspll_d4_ck
        4: bus_ck = sawlesspll_d3_ck
        5: bus_ck = usbpll_d4_ck/dmuxpll_d2 (EPS1 later)
      * 6: bus_ck = syspll_d3_ck
        7: bus_ck = slow_ck
    */
    vIO32WriteFldAlign(CKGEN_CPU_CKCFG, 6, FLD_BUS_CK_SEL);
    // axi_ck source selection: 0x1000_d254[1:0]
    //   0: xtal_ck (24MHz)
    //   1: usb_phy_pll_d2_ck (240MHz)
    //   2: vdecpll_d2_ck
    // * 3: syspll_d3_ck (216MHz)
    vIO32WriteFldAlign(CKGEN_REG_AXI_CKCFG, 3, FLD_AXI_CK_SEL);
    // SMI settings
    // 1: sawlesspll_d2_ck??
    if(IS_IC_5895_ES1())
    {
        vIO32WriteFldAlign(CKGEN_REG_SMI_CKCFG, 2, FLD_SMI_CK_SEL);
    }
    else
    {
        vIO32WriteFldAlign(CKGEN_REG_SMI_CKCFG, 1, FLD_SMI_CK_SEL);
    }
    // 1: syspll_d2_ck
    vIO32WriteFldAlign(CKGEN_REG_SMI_BUS_CKCFG, 1, FLD_SMI_BUS_CK_SEL);
    BIM_WRITE32(REG_MISC2,BIM_READ32(REG_MISC2)|SMI1_COMMON_CLKEN|SMI0_COMMON_CLKEN);

#ifdef CC_DRAM_96BIT
    // Enable larb19 ~ larb22 power
    // power on
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCFE_WRAP_PWR_ON);
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCFE_WRAP_PWR_ON_2ND);
    //release clock
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 0, FLD_SCFE_WRAP_CLK_DIS);
    //release ISO
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 0, FLD_SCFE_WRAP_ISO_EN);
    //release reset
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCFE_WRAP_RST_B);
    
    // Enable larb23 power
    // power on
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCBE_WRAP_PWR_ON);
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCBE_WRAP_PWR_ON_2ND);
    //release clock
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 0, FLD_SCBE_WRAP_CLK_DIS);
    //release ISO
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 0, FLD_SCBE_WRAP_ISO_EN);
    //release reset
    vIO32WriteFldAlign(CKGEN_REG_SCBE_SCFE_CFG, 1, FLD_SCBE_WRAP_RST_B);
#endif

    // Temporarily hardcode this for bringup
    // need to be disccused where is appropriate place to setup.
    // Enalbe vdec lat power
    vIO32Write4B(0x1205f0f8, 0x0);
    vIO32Write4B(0x1205f0fc, 0x0);
    if (IS_IC_5895_ES1()) {
        // enable vdec core1 sram power
        vIO32Write4B(0x1205fa98, 0x3e);
        vIO32Write4B(0x1205fa98, 0x3c);
        vIO32Write4B(0x1205fa98, 0x38);
        vIO32Write4B(0x1205fa98, 0x30);
        vIO32Write4B(0x1205fa98, 0x20);
        vIO32Write4B(0x1205fa98, 0x00);
    } else {
        // enable vdec core1 sram power
        vIO32Write4B(0x1205faa8, 0x3e);
        vIO32Write4B(0x1205faa8, 0x3c);
        vIO32Write4B(0x1205faa8, 0x38);
        vIO32Write4B(0x1205faa8, 0x30);
        vIO32Write4B(0x1205faa8, 0x20);
        vIO32Write4B(0x1205faa8, 0x00);
        // enable vdec core2 sram power
        vIO32Write4B(0x1205faac, 0x3e);
        vIO32Write4B(0x1205faac, 0x3c);
        vIO32Write4B(0x1205faac, 0x38);
        vIO32Write4B(0x1205faac, 0x30);
        vIO32Write4B(0x1205faac, 0x20);
        vIO32Write4B(0x1205faac, 0x00);
    }
    // Vdec clock
    vIO32Write4B(0x12060084, 0x2);
    vIO32Write4B(0x12068084, 0x2);
    vIO32Write4B(0x12075084, 0x2);
    vIO32Write4B(0x12085084, 0x2);
    if (!IS_IC_5895_ES1()) {
        vIO32Write4B(0x12095084, 0x2);
    }
    // vdec ckgen
    vIO32WriteFldAlign(CKGEN_REG_VDEC_SYS_CKCFG, 1, FLD_VDEC_SYS_TST);
    vIO32WriteFldAlign(CKGEN_REG_VDEC_LAE_CKCFG, 1, FLD_VDEC_LAE_CK_SEL);

    
    // MCU_BIU clock must be running at 1/2 CPUPLL clock
    IO_WRITE32(IO_VIRT, REG_MCUSYS_ACLKEN_DIV, (IO_READ32(IO_VIRT, REG_MCUSYS_ACLKEN_DIV) & 0xFFFFFFE0) | 0x12); // MCUCFG_ACLKEN_DIV[4:0] = 0x12
    CHIP_Delay_us(1);
    vIO32Write4B(IO_VIRT + REG_MP2_PLL_DIVIDER_CFG, 0x2FF);
    vIO32Write4B(IO_VIRT + REG_MCUSYS_BUS_PLL_DIV_CFG, 0x2FF);
    BIM_WRITE32(REG_CKDIV, 0x18);

    // 6. set common clk
    // C27M = sawless / 16
    vIO32WriteFldAlign(CKGEN_REG_C27M_CKCFG, 0x1, FLD_C27M_SEL);

#if 0
    // 7. setup vopll
    // TX analog setting
    vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x2, RG_LVDSA_PREDRV_VREF);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0, RG_LVDSA_BG_PWD);
    vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(1, RG_LVDS_BG_INTR_EN)|P_Fld(0x20, RG_LVDS_INTR_CAL));

    // vopll source xtal
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG18, 2, FLD_RG_DDDS1_CKSEL);

    // vopll setting
    vIO32WriteFldMulti(REG_LVDSTX_PLL_CFG0, P_Fld(1, REG_LVDSTX_PLL_CFG0_VPLL_BG_CKDIV)|P_Fld(1, REG_LVDSTX_PLL_CFG0_VPLL_BG_CKEN));
    vIO32WriteFldAlign(CKGEN_PLL_CFG1, 0, CKGEN_PLL_CFG1_REG_VPLL_PWR_ON);
    vIO32WriteFldAlign(CKGEN_PLL_CFG2, 1, CKGEN_PLL_CFG2_REG_VPLL_ISO_EN);
    vIO32Write4B(REG_LVDSTX_PLL_CFG0, 0x00000907);
    vIO32Write4B(REG_LVDSTX_PLL_CFG1, 0x94188402);
    vIO32Write4B(REG_LVDSTX_PLL_CFG2, 0x00200033);
    vIO32Write4B(REG_LVDSTX_PLL_CFG3, 0x2e840000);
    vIO32Write4B(REG_LVDSTX_PLL_CFG4, 0x83548000);

    // Power on
    CHIP_Delay_us(10);
    vIO32WriteFldAlign(CKGEN_PLL_CFG1, 1, CKGEN_PLL_CFG1_REG_VPLL_PWR_ON);
    vIO32WriteFldAlign(CKGEN_PLL_CFG2, 0, CKGEN_PLL_CFG2_REG_VPLL_ISO_EN);
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG1, 0, REG_LVDSTX_PLL_CFG1_VPLL_PWD);
    //vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_LVDS_PWD);      // Capy remove
    //vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_VB1_TX_PWD);  // Capy remove
    //vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BIAS_BYP);            // Capy remove

    // OCLK and TCLK Common
    vIO32WriteFldAlign(CKGEN_MBIST_CK_CTRL, 0, FLD_BIST_VPCLK_SEL);

    // OCLK Path
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, FLD_OCLK_SEL_LVDS, FLD_OCLK_SEL);
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0, FLD_OCLK_TSTSEL);

    // TCLK Path
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1, FLD_TCLK_SEL);
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0, FLD_TCLK_TSTSEL);

    // VB1 75MHz
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2, FLD_CLK75M2_CK_SEL);
    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2, FLD_VB1_CK_SEL);

    // VB1 reset
    vIO32WriteFldAlign(TMGR0, 0, TMGR0_CKRST_CFG_VB1);
    vIO32WriteFldAlign(TMGR2, 0, TMGR2_CKRST_CFG_VB1OSD);
    CHIP_Delay_us(10);
    vIO32WriteFldAlign(TMGR0, 1, TMGR0_CKRST_CFG_VB1);
    vIO32WriteFldAlign(TMGR2, 1, TMGR2_CKRST_CFG_VB1OSD);
#endif
    // 8. setup serial flash pll
    //   0: 3.375 MHz   1: 5.4 MHz      2: 6.75 MHz     3: 9 MHz
    //   4: 13.5 MHz    5: 27 MHz       6: 30 MHz       7: 48 MHz
    //   8: 54 MHz      9: 72 MHz       10: 108 MHz     11: 0.84375 MHz
    vIO32Write4B(IO_VIRT + 0x87B0,5);
    // 1: sample nor flash data at falling edge clock.  Delay 0.5T for timing issue at 5(24MHz).
    vIO32Write4B(IO_VIRT + 0x87B4,(u4IO32Read4B(IO_VIRT + 0x87B4) | 0x00000001));

    // 9. setup sram/BIST clock
    // abist_sram clock selection: 0x1000_d148[1:0]
    //   0: xtal_d1_ck (24MHz)
    //   1: usb_phy_pll_d2_ck (240MHz)
    //   2: sawlesspll_d2_ck (216MHz)
    // * 3: syspll_d3_ck (216MHz)
    vIO32WriteFldAlign(CKGEN_REG_ABIST_SRAM_CKCFG, 0x2, FLD_ABIST_SRAM_SEL);

#if 0 // only for ROM code
    // Speed up the ROM code clock
    // BIST clock selection
    // 0 : xtal (24Mhz)
    // 1 : sawlesspll (432Mhz)
    // 2 : SYSPLL / 2 (324Mhz)
    // 3 : sawlesspll / 3 (144Mhz)
    vIO32WriteFldAlign(CKGEN_REG_BIST_CKCFG, 0x2, FLD_BIST_SEL);
#endif

    // 10. setup nand pll (0: XTAL 24Mhz, 6: 192Mhz), and improve nand PIN driving
    vIO32WriteFldAlign(CKGEN_REG_NFI_CKCFG, 0x6, FLD_NFI_SEL);

    // 11. setup ssusb
    vIO32WriteFldAlign(CKGEN_REG_XHCI_CKCFG, 0x1, FLD_XHCI_CK_SEL); //108MHz

#if defined(CC_MT5895) //for PTPOD efuse value
    vIO32Write4B(0x10066400, 0x80030001); // enable 0x10066290 to write
    vIO32Write4B(0x10066290, u4IO32Read4B(0x10063134));
    vIO32Write4B(0x10066420, u4IO32Read4B(0x10063138));
    vIO32Write4B(0x10066424, u4IO32Read4B(0x1006313C));
    vIO32Write4B(0x10066428, u4IO32Read4B(0x10063140));
    vIO32Write4B(0x1006642c, u4IO32Read4B(0x10063144));
#elif defined(CC_MT5893)
    vIO32Write4B(0x10066290, u4IO32Read4B(0x10063090));
    vIO32Write4B(0x10066420, u4IO32Read4B(0x10063094));
    vIO32Write4B(0x10066424, u4IO32Read4B(0x10063098));
    vIO32Write4B(0x10066428, u4IO32Read4B(0x1006309c));
    vIO32Write4B(0x1006642c, u4IO32Read4B(0x100630b0));
#endif

#if 0
    // 12. Set Postscaler output panel timing to output stage
    #if 0//current MJC cannot enable MJC or disable MJC by define, need basd on 4K panel or not
    #if defined(CC_SUPPORT_MJC)
    vIO32WriteFldAlign(OSTG_01, 1, REG_MJC_IN_EN);
    #else
    vIO32WriteFldAlign(OSTG_01, 0, REG_MJC_IN_EN);
    #endif
    #endif

    vIO32WriteFldAlign(OSTG_01, 1, R_BYPASS_OD);
#endif
    // 13. Set PTP pll source
    // PTP_ck clock selection: 480/8=60MHz
    // 0: xtal_ck
    // 1: usbpll_d8
    // 2: sawlesspll_d8
    // 3: syspll_d16
    // PTP_SLOW_ck clock selection: 24MHz/512=48KHz
    // 2'b00:	xtal_ck
    // 2'b01:  xtal_d512
    // 2'b10:	xtal_d1024
    // 2'b11:  xtal_d2048
    vIO32WriteFldAlign(CKGEN_REG_PTP_CKCFG, 0x1, FLD_PTP_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_PTP_SLOW_CKCFG, 0x1, FLD_PTP_SLOW_CK_SEL);
#if 0
    //GPU setting
	vIO32WriteFldAlign(CKGEN_REG_G3D_CORE_CKCFG, 1, FLD_G3D_CORE_SEL); //g3dpll_d1_ck
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x1f, FLD_MFG_RESET); //GPU MFG reset, toggle
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x0, FLD_MFG_RESET);
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x1f, FLD_MFG_RESET);
    //socif sync  delay 2T -> 3T
    vIO32WriteFldMulti(GPU_MFG_ASYNC_CON, P_Fld(2, FLD_GPU_MFG_SOCIF_SLV_SEL) | P_Fld(2, FLD_GPU_MFG_SOCIF_MST_SEL));
    //Disable GPU ultra mode
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_PR_EN);
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_LL_EN);
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_WR_EN);
    //disable GPU DCM
    vIO32WriteFldAlign(GPU_MFG_GLOBAL_CON, 0x1, FLD_GPU_MFG_PSEL_M_GLOBAL_CON_OPT);
#endif
#ifndef CC_MTK_HOTBOOT_START
    //13. Power On CM4 SRAM for AC on and standby case
    CM4_SRAM_Poweron();
#else
	//13. Power On CM4 SRAM for suspend case
	if(IO32ReadFldAlign(PDWNC_RESRV4,FLD_RG_CM4_GA_STATUS))
    {
        CM4_SRAM_Poweron();
        vIO32WriteFldAlign(PDWNC_RESRV4, 0x0, FLD_RG_CM4_REINIT_EN);  //--> bit2 :CM4 power off : 0: no power off
    }
    else
    {
        CM4_SRAM_Resume();
        vIO32WriteFldAlign(PDWNC_RESRV4, 0x1, FLD_RG_CM4_REINIT_EN);  //--> bit2 :CM4 power off : 1 power off
    }
#endif
}

#ifdef CC_ARM_GIC
static unsigned int mt53xx_irq_pol[] = {
0xffffffff, 0xffffffff,
0x00000000, 0x02000000,
0x00040010, 0x00000000,
0xc0000000, 0xfc01003f,
0xb87fff7f, 0x0cf7ffff,
0xfffffe00, 0xffffffff,
0x01c03fff, 0x000003fc
};

void GIC_Interrupt_Polarity(void)
{
	int i = 0, size = sizeof(mt53xx_irq_pol) / sizeof(mt53xx_irq_pol[0]);

	for (i = 0; i < size; i++)
	{
			MCU_WRITE32(REG_MCU_INT_POL_CTL0 + (i * 4), mt53xx_irq_pol[i]);
	}
}
#endif

void CHIP_BimInit(void)
{
#ifdef CC_MTK_PRELOADER
    if (!CHIP_IsRemap())
    {
        // set bus access timeout to avoid register access halting CPU.
        BIM_WRITE32(REG_RW_TIMEOUT_CTRL, TO_FLASH_CNT | TO_DRAM_CNT | TO_IOBUS_CNT | TO_FLASH_EN | TO_DRAM_EN | TO_IOBUS_EN);
        //BIM_WRITE32(REG_RW_BTIME, TO_IORW | TO_IOWW);

        // Disable irq & fiq
        BIM_WRITE32(REG_RW_IRQEN, 0);
        BIM_WRITE32(REG_RW_FIQEN, 0);
        BIM_WRITE32(REG_RW_IRQCL, 0xffffffff);
        BIM_WRITE32(REG_RW_FIQCL, 0xffffffff);

        #ifndef CC_LOAD_UBOOT // only cpu0 is working in nucleus
        BIM_CpuDie(1);
        BIM_CpuDie(2);
        BIM_CpuDie(3);
        #endif

        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU1_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU1_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU2_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU2_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU3_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU3_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(BIM_BASE, BANK_SECURE_ENABLE1, (IO_READ32(BIM_BASE, BANK_SECURE_ENABLE1)) & (~(0x1 << 9)));

	#ifdef CC_ARM_GIC
	GIC_Interrupt_Polarity();
	#endif
    }
#else
    #ifdef CC_ARM_GIC
    GIC_Init();
    #endif
#endif // CC_MTK_PRELOADER
}

void CHIP_1stBootRegInit(void)
{
#ifndef CC_MTK_PRELOADER
    // Initial timer, then can use HAL_Delay_us()
    VERIFY(HAL_InitTimer());
#endif /* CC_MTK_PRELOADER */
}
#endif /* CC_MTK_PRELOADER2 */
void HaltSystem(void)
{
    CHIP_DisplayString("\nHal\n");
    while(1);
}

void CHIP_Reboot(void)
{
    HAL_WRITE32(PDWNC_WDTCTL, 0x1); // PDWNC_WRITE32(REG_RW_WATCHDOG_EN, 1);//vIO32WriteFldAlign(PDWNC_WDTCTL, 0x1, FLD_WDT0E);
    HAL_WRITE32(PDWNC_WDT0, 0xffffffff); // PDWNC_WRITE32(REG_RW_WATCHDOG_TMR, 0xffffff00);//vIO32Write4B(PDWNC_WDT0, 0xffffff00);
    ASSERT(0);
}
