/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: louis.ko $
 * $Date: 2015/07/17 $
 * $RCSfile: msdcboot.c,v $
 * $Revision: #3 $
 *
 *---------------------------------------------------------------------------*/

/** @file msdcboot.c
 *  msdcboot.c provides the low level register access function of msdc device.
 */

#if defined(CC_EMMC_BOOT) && defined(CC_MTK_PRELOADER)

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "x_timer.h"
#include "x_ldr_env.h"
#include "chip_txt.h"
#include "msdcboot.h"

#ifndef TIMER_LOW_REG
#define TIMER_LOW_REG       REG_RW_TIMER2_LOW
#endif
#define TIMEOUT_COUNT(x)    (24000*x)

EXTERN void CHIP_Delay_us(UINT32 u4Micros);

//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------
volatile static UINT32 _u4MsdcAccuVect = 0;
static UINT32 _u4CurMsdcclkFreq = 0;
static UINT32 _u4StartOffsetAlign = 0;
static UINT32 _u4EndOffsetAlign = 0;
static UINT32 _u4ActuralLen = 0;

static sdcard_t sdCard;
static UINT32 _u4MsdcFreq[] = {SRC_CLK_FREQ, BUS_CLK_FREQ, BUS_DIV_2_CLK_FREQ, BUS_DIV_4_CLK_FREQ, BUS_DIV_8_CLK_FREQ, BUS_DIV_16_CLK_FREQ, INIT_BUS_FREQ};
static UINT32 _u4MsdcFreqIdx = 0;
static UINT32 ch = 0;
static BOOL _fgMsdcIdentyCardOK = FALSE;
BOOL _fgMsdcInit;

#define ROMMSDCTAG_0 0x2F4B544D
#define ROMMSDCTAG_1 0x2F565444
#define ROMMSDCTAG_2 0x434D4F52
#define ROMMSDCTAG_3 0x2F45444F
#define ROMMSDCTAG_4 0x4344534D
#define ROMMSDCTAG_5 0x544F4F42
#define ROMMSDCTAG_6 0x3130562F
#define ROMMSDCTAG_7 0x0030302E

const static UINT32 _u4Signature[] __attribute__ ((section ("ROMNANDTAG"))) =
{
    ROMMSDCTAG_0, ROMMSDCTAG_1, ROMMSDCTAG_2, ROMMSDCTAG_3,     // MTK/DTV/ROMCODE/
    ROMMSDCTAG_4, ROMMSDCTAG_5, ROMMSDCTAG_6, ROMMSDCTAG_7,     // MSDCBOOT/V01.00
    0,            0,            0,            0,
    0,            0,            0,            0,

    ROMMSDCTAG_5, 0,            0,            0,
    0,            0,            0,            0,
    0,            0,            0,            0,
    0,            0,            0,            0
};

//-----------------------------------------------------------------------------
// static functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/** CHIP_EmmcShowHex()
 */
//-----------------------------------------------------------------------------
#if 0 //for debug
static VOID MsdcDumpRegister(VOID)
{
   #define MSDC_READ32(addr)              (*(volatile unsigned int *)(addr))

    Printf("MSDC_CFG 0x%08x \n",MSDC_READ32(MSDC_CFG));
    Printf("MSDC_IOCON 0x%08x \n",MSDC_READ32(MSDC_IOCON));
    Printf("MSDC_PS 0x%08x \n",MSDC_READ32(MSDC_PS));
    Printf("MSDC_INT 0x%08x \n",MSDC_READ32(MSDC_INT));

    Printf("MSDC_INTEN 0x%08x \n",MSDC_READ32(MSDC_INTEN));
    Printf("MSDC_FIFOCS 0x%08x \n",MSDC_READ32(MSDC_FIFOCS));
    Printf("MSDC_TXDATA 0x%08x \n",MSDC_READ32(MSDC_TXDATA));
    Printf("MSDC_RXDATA 0x%08x \n",MSDC_READ32(MSDC_RXDATA));
    Printf("SDC_CFG 0x%08x \n",MSDC_READ32(SDC_CFG));
    Printf("SDC_CMD 0x%08x \n",MSDC_READ32(SDC_CMD));
    Printf("SDC_ARG 0x%08x \n",MSDC_READ32(SDC_ARG));
    Printf("SDC_STS 0x%08x \n",MSDC_READ32(SDC_STS));
    Printf("SDC_RESP0 0x%08x \n",MSDC_READ32(SDC_RESP0));
    Printf("SDC_RESP1 0x%08x \n",MSDC_READ32(SDC_RESP1));
    Printf("SDC_RESP2 0x%08x \n",MSDC_READ32(SDC_RESP2));
    Printf("SDC_RESP3 0x%08x \n",MSDC_READ32(SDC_RESP3));
    Printf("SDC_BLK_NUM 0x%08x \n",MSDC_READ32(SDC_BLK_NUM));

    Printf("SDC_CSTS 0x%08x \n",MSDC_READ32(SDC_CSTS));
    Printf("SDC_CSTS_EN 0x%08x \n",MSDC_READ32(SDC_CSTS_EN));
    Printf("SDC_DATCRC_STS 0x%08x \n",MSDC_READ32(SDC_DATCRC_STS));
    Printf("EMMC_CFG0 0x%08x \n",MSDC_READ32(EMMC_CFG0));
    Printf("EMMC_CFG1 0x%08x \n",MSDC_READ32(EMMC_CFG1));
    Printf("EMMC_STS 0x%08x \n",MSDC_READ32(EMMC_STS));
    Printf("EMMC_IOCON 0x%08x \n",MSDC_READ32(EMMC_IOCON));

    Printf("PATCH_BIT0 0x%08x \n",MSDC_READ32(0xf006d0b0));
    Printf("PATCH_BIT1 0x%08x \n",MSDC_READ32(0xf006d0b4));
    Printf("PAD_TUNE 0x%08x \n",MSDC_READ32(PAD_TUNE));

}
#endif


static void CHIP_EmmcShowHex(const CHAR *pu1Str, UINT32 u4Hex)
{
    CHIP_DisplayString(pu1Str);
    CHIP_DisplayHex(u4Hex);
    CHIP_DisplayString("\n");
}

static void MsdcShowChipInfo(UINT32 *pCID)
{
    CHIP_EmmcShowHex("\nCID:", pCID[0]);
    CHIP_EmmcShowHex(":", pCID[1]);
    CHIP_EmmcShowHex(":", pCID[2]);
    CHIP_EmmcShowHex(":", pCID[3]);
}

static void MsdcSetMemory(void* pdestination, UINT32 u4Ch, UINT32 u4length)
{
    UINT32 i;
    UINT32 *pbDstBufPtr;

    pbDstBufPtr = (UINT32 *)pdestination;

    for (i = 0; i < u4length; i += 4) {
        *pbDstBufPtr = u4Ch;
        pbDstBufPtr++;
    }
}

static void MsdcCopyMemory(void* pdestination, void* psource, UINT32 u4length)
{
    UINT32 i;
    UINT32 *pbDstBufPtr, *pbSrcBufPtr;

    pbDstBufPtr = (UINT32 *)pdestination;
    pbSrcBufPtr = (UINT32 *)psource;

    for (i = 0; i < u4length; i += 4) {
        *pbDstBufPtr = *pbSrcBufPtr;
        pbDstBufPtr++;
        pbSrcBufPtr++;
    }
}

#define x_memset(dest, ch, len)   MsdcSetMemory((void*)(dest), (ch), (len))
#define x_memcpy(dest, src, len)  MsdcCopyMemory((void*)(dest), (void*)(src), (len))

static unsigned char MsdcReset(void)
{
    UINT32 i;

    // Reset MSDC
    MSDC_SETBIT(MSDC_CFG, MSDC_CFG_RST);

    i = BIM_READ32(TIMER_LOW_REG);
    do {
        if (0 == (MSDC_READ32(MSDC_CFG) & MSDC_CFG_RST))
            return MSDC_SUCCESS;
    } while ((i - BIM_READ32(TIMER_LOW_REG)) < TIMEOUT_COUNT(MSDC_RST_TIMEOUT_LIMIT_COUNT));
    return MSDC_FAILED;
}

static INT32 MsdcClrFifo(void)
{
    UINT32 i;

    // Reset FIFO
    MSDC_SETBIT(MSDC_FIFOCS, MSDC_FIFOCS_FIFOCLR);

    i = BIM_READ32(TIMER_LOW_REG);
    do {
        if (0 == (MSDC_READ32(MSDC_FIFOCS) & (MSDC_FIFOCS_FIFOCLR | MSDC_FIFOCS_TXFIFOCNT_MASK | MSDC_FIFOCS_RXFIFOCNT_MASK)))
            return MSDC_SUCCESS;
    } while ((i - BIM_READ32(TIMER_LOW_REG)) < TIMEOUT_COUNT(MSDC_FIFOCLR_TIMEOUT_LIMIT_COUNT));
    return MSDC_FAILED;
}

static void MsdcChkFifo(sd_cmd_t *pCmd)
{
    // Check if rx/tx fifo is zero
    if ((MSDC_READ32(MSDC_FIFOCS) & (MSDC_FIFOCS_TXFIFOCNT_MASK | MSDC_FIFOCS_RXFIFOCNT_MASK)) != 0)
    {
        MsdcClrFifo();
    }
}

static void MsdcClrIntr(void)
{
    // Check MSDC Interrupt vector register
    if (0 != MSDC_READ32(MSDC_INT)) // Clear MSDC Interrupt vector register
    {
        MSDC_WRITE32(MSDC_INT, MSDC_READ32(MSDC_INT));
    }
}

static INT32 MsdcWaitClkStable(void)
{
    UINT32 i;

    i = BIM_READ32(TIMER_LOW_REG);
    do {
        if (0 != (MSDC_READ32(MSDC_CFG) & MSDC_CFG_CARD_CK_STABLE))
            return MSDC_SUCCESS;
    } while ((i - BIM_READ32(TIMER_LOW_REG)) < TIMEOUT_COUNT(MSDC_CLK_TIMEOUT_LIMIT_COUNT));
    return MSDC_FAILED;
}

static INT32 MsdcInit(void)
{
    // Reset MSDC
    if (MSDC_SUCCESS != MsdcReset()) {
        return MSDC_INIT_FAIL_RST_TIMEOUT;
    }

    // Set PIO Mode & SD/MMC Mode
    MSDC_SETBIT(MSDC_CFG, MSDC_CFG_PIO_MODE | MSDC_CFG_SD);

    // Disable sdio & Set bus to 1 bit mode
    MSDC_CLRBIT(SDC_CFG, SDC_CFG_SDIO | SDC_CFG_BW_MASK);

    // set clock mode (DIV mode)
    MSDC_CLRBIT(MSDC_CFG, (((UINT32)0x03) << 20));

    // Wait until clock is stable
    if (MSDC_SUCCESS != MsdcWaitClkStable()) {
        return MSDC_INIT_FAIL_CLK_TIMEOUT;
    }

    // Set default RISC_SIZE for DWRD pio mode
    MSDC_WRITE32(MSDC_IOCON, (MSDC_READ32(MSDC_IOCON) & ~MSDC_IOCON_RISC_SIZE_MASK) | MSDC_IOCON_RISC_SIZE_DWRD | MSDC_IOCON_R_D_SMPL);

    // Set Data timeout setting => Maximum setting
    MSDC_WRITE32(SDC_CFG, (MSDC_READ32(SDC_CFG) & ~(((UINT32)0xFF) << SDC_CFG_DTOC_SHIFT)) | (((UINT32)0xFF) << SDC_CFG_DTOC_SHIFT));

    return MSDC_SUCCESS;
}

static void MsdcContinueClock(INT32 i4ContinueClock)
{
    if (i4ContinueClock) {
        // Set clock continuous even if no command
        MSDC_SETBIT(MSDC_CFG, (((UINT32)0x01) << 1));
    } else {
        // Set clock power down if no command
        MSDC_CLRBIT(MSDC_CFG, (((UINT32)0x01) << 1));
    }
}

static char MsdcSetClkfreq(UINT32 clkFreq)
{
    unsigned int divider, divmode = 0;
    unsigned int expFreq = 0;

    if (_u4CurMsdcclkFreq == clkFreq)
        return MSDC_SUCCESS;

    if (clkFreq >= SRC_CLK_FREQ) {
        clkFreq = SRC_CLK_FREQ;
        /*
           default use higher clock 24MHZ
         */
        divmode = 0;
    } else if (clkFreq < LOWEST_BUS_FREQ) {
        clkFreq = INIT_BUS_FREQ;
        divmode = 0;
    } else {
        divmode = 0;
    }

    for (divider = 0; divider <= 0xFFF; divider++) {
        if (1 == divmode) {
            expFreq = SRC_CLK_FREQ;
        } else if (0 == divider) {
            expFreq = SRC_CLK_FREQ / 2;
        } else {
            expFreq = SRC_CLK_FREQ / 4 / divider;
        }

        if (expFreq <= clkFreq) {
            MSDC_WRITE32(MSDC_CFG, (MSDC_READ32(MSDC_CFG) & (~0x003FFF00)) | (divider << MSDC_CFG_CK_DIV_SHIFT) | (divmode << 20));

            // Wait until clock is stable
            if (MSDC_FAILED == MsdcWaitClkStable())
                return MSDC_FAILED;

            _u4CurMsdcclkFreq = clkFreq;

            return MSDC_SUCCESS;
        }
    }

    return MSDC_FAILED;
}

// Return when any interrupt is matched or timeout
static INT32 MsdcWaitIntr(UINT32 vector, UINT32 timeoutCnt)
{
    UINT32 i;

    // Clear Vector variable
    _u4MsdcAccuVect = 0;
    i = BIM_READ32(TIMER_LOW_REG);
    do {
        // Status match any bit
        if (0 != (MSDC_READ32(MSDC_INT) & vector)) {
            _u4MsdcAccuVect |= MSDC_READ32(MSDC_INT);
            MSDC_WRITE32(MSDC_INT, _u4MsdcAccuVect);
            return MSDC_SUCCESS;
        }
    } while ((i - BIM_READ32(TIMER_LOW_REG)) < TIMEOUT_COUNT(timeoutCnt));

    // Timeout case
    return MSDC_FAILED;
}

static void MsdcErrorHandling(void)
{
    // Reset MSDC
    MsdcReset();

    // Clear FIFO
    MsdcClrFifo();

    // Check if FIFO clean, check if rx/tx fifo is zero
    if ((MSDC_READ32(MSDC_FIFOCS) & (MSDC_FIFOCS_TXFIFOCNT_MASK | MSDC_FIFOCS_RXFIFOCNT_MASK)) != 0)
        MsdcClrFifo();

    // Clear MSDC interrupt
    MsdcClrIntr();
}

static INT32 MsdcSendCmd(sd_cmd_t *pCmd)
{
    UINT32 respType = RESPONSE_NO;
    UINT32 sdcCmd = 0;
    INT32 i4Ret = MSDC_SUCCESS;
    UINT32 u4CmdDoneVect;
    UINT32 i;

    // Check if rx/tx fifo is zero
    MsdcChkFifo(pCmd);

    // Clear interrupt Vector
    MsdcClrIntr();

    i = BIM_READ32(TIMER_LOW_REG);
    while (!((0 == (MSDC_READ32(SDC_STS) & (SDC_STS_SDCBUSY | SDC_STS_CMDBUSY))) && (0 == MSDC_READ32(MSDC_INT)))) {
        if ((i - BIM_READ32(TIMER_LOW_REG)) >= TIMEOUT_COUNT(MSDC_WAIT_SDC_BUS_TIMEOUT_LIMIT_COUNT)) {
            i4Ret = MSDC_FAILED;
            goto ErrorHandling;
        }
    }

    /* Figure out the response type */
    switch (pCmd->idx)
    {
    case CMD0_GO_IDLE_STATE:
        respType = RESPONSE_NO;
        sdcCmd |= SDC_CMD_RSPTYPE_NO;
        break;

    case CMD1_MMC_SEND_OP_COND:
        respType = RESPONSE_R3;
        sdcCmd |= SDC_CMD_RSPTYPE_R3;
        break;

    case CMD2_ALL_SEND_CID:
        respType = RESPONSE_R2;
        sdcCmd |= SDC_CMD_RSPTYPE_R2;
        break;

    case CMD3_SEND_RELATIVE_ADDR:
        respType = RESPONSE_R1;
        sdcCmd |= SDC_CMD_RSPTYPE_R1;
        break;

    case CMD6_MMC_SWITCH:
        respType = RESPONSE_R1B;
        sdcCmd |= SDC_CMD_RSPTYPE_R1B;
        break;

    case CMD7_SELECT_CARD:
        respType = RESPONSE_R1;
        sdcCmd |= SDC_CMD_RSPTYPE_R1;
        break;

    case CMD9_SEND_CSD:
        respType = RESPONSE_R2;
        sdcCmd |= SDC_CMD_RSPTYPE_R2;
        break;

    case CMD12_STOP_TRANSMISSION:
        respType = RESPONSE_R1B;
        sdcCmd |= (SDC_CMD_STOP | SDC_CMD_RSPTYPE_R1B);
        break;

    case CMD17_READ_SINGLE_BLOCK:
        respType = RESPONSE_R1;
        sdcCmd |= (SDC_CMD_RSPTYPE_R1 | DTYPE_SINGLE_BLK | SDC_CMD_READ);
        MSDC_WRITE32(SDC_BLK_NUM, 1);
        break;

    case CMD18_READ_MULTIPLE_BLOCK:
        respType = RESPONSE_R1;
        sdcCmd |= (SDC_CMD_RSPTYPE_R1 | DTYPE_MULTI_BLK | SDC_CMD_READ);
        MSDC_WRITE32(SDC_BLK_NUM, pCmd->buffLen / SDHC_BLK_SIZE);
        break;

    case CMD24_WRITE_BLOCK:
        respType = RESPONSE_R1;
        sdcCmd |= (SDC_CMD_RSPTYPE_R1 | DTYPE_SINGLE_BLK | SDC_CMD_WRITE);
        MSDC_WRITE32(SDC_BLK_NUM, 1);
        break;

    case CMD25_WRITE_MULTIPLE_BLOCK:
        respType = RESPONSE_R1;
        sdcCmd |= (SDC_CMD_RSPTYPE_R1 | DTYPE_MULTI_BLK | SDC_CMD_WRITE);
        MSDC_WRITE32(SDC_BLK_NUM, pCmd->buffLen / SDHC_BLK_SIZE);
        break;
    }

    // Set Blk Length
    sdcCmd |= ((SDHC_BLK_SIZE) << SDC_CMD_LEN_SHIFT);

    // Set SDC_CMD.CMD
    sdcCmd |= (pCmd->idx & 0x3F);

    // Set SDC Argument
    MSDC_WRITE32(SDC_ARG, pCmd->arg);

    /* Send the commands to the device */
    MSDC_WRITE32(SDC_CMD, sdcCmd);

    // Wait for command and response if existed
    u4CmdDoneVect = INT_SD_CMDRDY | INT_SD_CMDTO | INT_SD_RESP_CRCERR;

    if (MSDC_SUCCESS != MsdcWaitIntr(u4CmdDoneVect, MSDC_WAIT_CMD_TIMEOUT_LIMIT_COUNT)) {
        i4Ret = MSDC_CMD_ERR_WAIT_CMD_TO;
        goto ErrorHandling;
    }

    if (_u4MsdcAccuVect & INT_SD_CMDTO) {
        i4Ret = MSDC_CMD_ERR_NO_RESP;
        goto ErrorHandling;
    } else if (_u4MsdcAccuVect & INT_SD_RESP_CRCERR) {
        i4Ret = MSDC_CMD_ERR_RESP_CRCERR;
        goto ErrorHandling;
    } else if ((_u4MsdcAccuVect & (~(INT_SD_CMDRDY))) || (0 != MSDC_READ32(MSDC_INT))) {
        i4Ret = MSDC_FAILED;
        goto ErrorHandling;
    }

    // Handle the response
    switch (respType)
    {
    case RESPONSE_NO:
        return MSDC_SUCCESS;

    case RESPONSE_R1:
    case RESPONSE_R1B:
        pCmd->resp[0] = MSDC_READ32(SDC_RESP0);
        break;

    case RESPONSE_R2:
        pCmd->resp[0] = MSDC_READ32(SDC_RESP0);
        pCmd->resp[1] = MSDC_READ32(SDC_RESP1);
        pCmd->resp[2] = MSDC_READ32(SDC_RESP2);
        pCmd->resp[3] = MSDC_READ32(SDC_RESP3);
        break;

    case RESPONSE_R3:
        pCmd->resp[0] = MSDC_READ32(SDC_RESP0);
        break;
    }

    if (PIO_DATA_MODE == pCmd->dataMode) {

        UINT32 u4RxFifoCnt, u4TxFifoCnt;
        UINT32 *pBufData = (UINT32 *)(pCmd->pBuff);
        UINT32 u4BufEnd = (UINT32)pBufData + (UINT32)(pCmd->buffLen);
        UINT32 u4ActualEnd = (UINT32)pBufData + _u4ActuralLen;
        UINT32 u4RxCnt = 0;

        u4CmdDoneVect = INT_SD_DATTO | INT_SD_DATA_CRCERR;

        // Read
        if (!(sdcCmd & SDC_CMD_WRITE))
        {
            if (_u4StartOffsetAlign) {
                do
                {
                    // wait until fifo has enough data
                    u4RxFifoCnt = (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_RXFIFOCNT_MASK);

                    while ((sizeof(INT32) <= u4RxFifoCnt) && (_u4StartOffsetAlign > 0))
                    {
                        // Read Data
                        MSDC_READ32(MSDC_RXDATA);
                        u4RxFifoCnt -= sizeof(INT32);
                        _u4StartOffsetAlign -= sizeof(INT32);
                    }

                } while (_u4StartOffsetAlign);
            }

            while (u4ActualEnd > (UINT32)pBufData)
            {
                // wait until fifo has enough data
                u4RxFifoCnt = (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_RXFIFOCNT_MASK);

                while ((u4ActualEnd > (UINT32)pBufData) && (sizeof(INT32) <= u4RxFifoCnt))
                {
                    // Read Data
                    *pBufData = MSDC_READ32(MSDC_RXDATA);
                    pBufData++;

                    u4RxFifoCnt -= sizeof(INT32);

                    u4RxCnt++;

                    if (u4RxFifoCnt == 0) // Check CRC error happens or not every 128 Byte
                    {
                        // Check if done vector occurs
                         if (u4CmdDoneVect & MSDC_READ32(MSDC_INT))
                         {
							CHIP_EmmcShowHex("crcerr:", MSDC_READ32(MSDC_INT));
							 i4Ret = MSDC_DATA_CRC_ERR;
							 goto done;
                         }
                        u4RxCnt = 0;
                    }

                }
            }

            if (_u4EndOffsetAlign)
            {
                do
                {
                    // wait until fifo has enough data
                    u4RxFifoCnt = (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_RXFIFOCNT_MASK);

                    while (sizeof(INT32) <= u4RxFifoCnt && (_u4EndOffsetAlign > 0))
                    {
                        // Read Data
                        MSDC_READ32(MSDC_RXDATA);
                        u4RxFifoCnt -= sizeof(INT32);
                        _u4EndOffsetAlign -= sizeof(INT32);
                    }

                } while (_u4EndOffsetAlign);
            }
		} else {
            while (u4BufEnd > (UINT32)pBufData)
            {
                // Check if error done vector occurs
                if (u4CmdDoneVect & (_u4MsdcAccuVect | MSDC_READ32(MSDC_INT)))
                    break;

                // wait until fifo has enough space
                u4TxFifoCnt = MSDC_FIFO_LEN - (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_TXFIFOCNT_MASK);

                if (sizeof(INT32) <= u4TxFifoCnt) {
                    while ((u4BufEnd > (UINT32)pBufData) && (sizeof(INT32) <= u4TxFifoCnt))
                    {
                        // Write Data
                        MSDC_WRITE32(MSDC_TXDATA, *pBufData);
                        pBufData++;
                        u4TxFifoCnt -= sizeof(INT32);

                        if (u4TxFifoCnt == 0)
                            u4TxFifoCnt = MSDC_FIFO_LEN - (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_TXFIFOCNT_MASK);
                    }
                } else if (sizeof(INT32) > u4TxFifoCnt) {
                    for (i = 0; i < MSDC_WAIT_DATA_TIMEOUT_LIMIT_COUNT; i++)
                    {
                        u4TxFifoCnt = MSDC_FIFO_LEN - (MSDC_READ32(MSDC_FIFOCS) & MSDC_FIFOCS_TXFIFOCNT_MASK);
                        if (sizeof(INT32) <= u4TxFifoCnt)
                            break;
                    }
					if (i == MSDC_WAIT_DATA_TIMEOUT_LIMIT_COUNT) {
                        i4Ret = MSDC_CMD_ERR_WAIT_DATATO;
                        goto ErrorHandling;
                    }
                } else if (u4CmdDoneVect & MSDC_READ32(MSDC_INT)) {
                    // Check if done vector occurs
                    break;
                }
            }
        }
    }

done:
	if (PIO_DATA_MODE == pCmd->dataMode) {
        // Wait for data complete
        if (MSDC_SUCCESS != MsdcWaitIntr((u4CmdDoneVect | INT_SD_XFER_COMPLETE), 10 * MSDC_WAIT_DATA_COMPLETE_TIMEOUT_LIMIT_COUNT)) {
            i4Ret = MSDC_CMD_ERR_WAIT_DATATO;
            goto ErrorHandling;
        }

        if (_u4MsdcAccuVect & INT_SD_DATTO) {
            i4Ret = MSDC_CMD_ERR_DATTO;
        } else if (_u4MsdcAccuVect & INT_SD_DATA_CRCERR) {
			i4Ret = MSDC_DATA_CRC_ERR;
        } else if ((_u4MsdcAccuVect & ~(INT_SD_XFER_COMPLETE)) || (0 != MSDC_READ32(MSDC_INT))) {
            i4Ret = MSDC_CMD_ERR_DATA_FAILED;
        }
    }

ErrorHandling:
    // No error handling is needed if no data transfer, so we only perform error handling for read data
        MsdcErrorHandling();

    return i4Ret;
}

static INT32 cmd0_go_idle_state(UINT32 arg)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD0_GO_IDLE_STATE;
    cmd.arg = arg;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static INT32 cmd1_mmc_send_op_cond(void)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD1_MMC_SEND_OP_COND;
    //cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset
    cmd.arg = HOST_MMC_OCR;

    i4Ret = MsdcSendCmd(&cmd);
    if (MSDC_SUCCESS == i4Ret)
        x_memcpy(&(sdCard.memOcr), &cmd.resp[0], sizeof(mem_ocr_t));
    return i4Ret;
}

static INT32 cmd2_all_send_cid(void)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD2_ALL_SEND_CID;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    i4Ret = MsdcSendCmd(&cmd);
    if (MSDC_SUCCESS == i4Ret) {
        x_memcpy(&(sdCard.cid), cmd.resp, sizeof(sd_cid_t));
        MsdcShowChipInfo(cmd.resp);
    }
    return i4Ret;
}

static INT32 cmd3_send_relative_addr(void)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD3_SEND_RELATIVE_ADDR;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    if (sdCard.cardType == CARDTYPE_MEM_MMC) {
        sdCard.memRca = MMC_DEF_RCA;
        cmd.arg = (sdCard.memRca) << 16;
    }

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static INT32 cmd6_mmc_set_bus_width(INT32 busWidth)
{
    INT32 i4Ret;
    sd_cmd_t cmd;
    UINT32 i;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD6_MMC_SWITCH;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    // determine Arguments according to bus selection
    if (1 == busWidth) {
        cmd.arg = MMC_CMD6_ARG_1BIT_BUS;
    } else if (4 == busWidth) {
        cmd.arg = MMC_CMD6_ARG_4BIT_BUS;
    } else if (8 == busWidth) {
        cmd.arg = MMC_CMD6_ARG_8BIT_BUS;
    }

    i4Ret = MsdcSendCmd(&cmd);

    MsdcContinueClock(1);
    i = BIM_READ32(TIMER_LOW_REG);
    do {
		if (0 != (MSDC_READ32(MSDC_PS) & (0x1 << 16))) {
			break;
        }
    } while ((i - BIM_READ32(TIMER_LOW_REG)) < TIMEOUT_COUNT(MSDC_WAIT_DAT0_BUSY_LIMIT_COUNT));
    MsdcContinueClock(0);

    return i4Ret;
}

static INT32 cmd7_select_card(UINT32 sel)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD7_SELECT_CARD;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    if (sel) {
        cmd.arg = (sdCard.memRca) << 16;
    } else {
        cmd.arg = 0;
    }

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static INT32 cmd9_send_csd(void)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD9_SEND_CSD;
    cmd.dataMode = NULL_DATA_MODE;
    cmd.arg = (sdCard.memRca) << 16;

    i4Ret = MsdcSendCmd(&cmd);
    if (MSDC_SUCCESS == i4Ret)
        x_memcpy(&(sdCard.csd), cmd.resp, sizeof(csd_20_t));
    return i4Ret;
}

static INT32 cmd17_read_single_block(UINT64 addr, INT8 *pBuff, UINT32 buffLen)
{
    INT32 i4Ret;
    sd_cmd_t cmd;
    UINT64 u8Tmp;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD17_READ_SINGLE_BLOCK;
    cmd.dataMode = PIO_DATA_MODE;
    cmd.pBuff = pBuff;
    cmd.buffLen = buffLen;
    _u4ActuralLen = buffLen;

    if (sdCard.blkAddrMode == 1) {
        cmd.arg = (UINT32)(addr >> 9);                 // Block addressing mode
    } else {
        cmd.arg = (UINT32)(addr & (~0x1FF));           // Byte addressing mode
    }
    _u4StartOffsetAlign = (UINT32)(addr - (addr & (~0x1FF)));

    u8Tmp = (UINT64)(addr + buffLen - 1);
    u8Tmp &= (~0x1FF);
    u8Tmp += 0x200;
    cmd.buffLen = (UINT32)(u8Tmp - (addr & (~0x1FF)));

    _u4EndOffsetAlign = (UINT32)(u8Tmp - (addr + buffLen));

    //CHIP_EmmcShowHex("\n_u4ActuralLen :", _u4ActuralLen);
    //CHIP_EmmcShowHex("\n_u4StartOffsetAlign :", _u4StartOffsetAlign);
    //CHIP_EmmcShowHex("\n_u4EndOffsetAlign :", _u4EndOffsetAlign);
    //CHIP_EmmcShowHex("\ncmd.arg :", cmd.arg);
    //CHIP_EmmcShowHex("\ncmd.buffLen :", cmd.buffLen);

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static INT32 cmd18_read_multiple_block(UINT64 addr, INT8 *pBuff, UINT32 buffLen)
{
    INT32 i4Ret;
    sd_cmd_t cmd;
    UINT64 u8Tmp;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD18_READ_MULTIPLE_BLOCK;
    cmd.dataMode = PIO_DATA_MODE;
    cmd.pBuff = pBuff;
    _u4ActuralLen = buffLen;

    if (sdCard.blkAddrMode == 1) {
        cmd.arg = (UINT32)(addr >> 9);    // Block addressing mode
    } else {
        cmd.arg = addr & (~0x1FF);        // Byte addressing mode
    }
    _u4StartOffsetAlign = (UINT32)(addr - (addr & (~0x1FF)));

    u8Tmp = (UINT64)(addr + buffLen - 1);
    u8Tmp &= (~0x1FF);
    u8Tmp += 0x200;
    cmd.buffLen = (UINT32)(u8Tmp - (addr & (~0x1FF)));

    _u4EndOffsetAlign = (UINT32)(u8Tmp - (addr + buffLen));

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static INT32 cmd12_stop_transmission(void)
{
    INT32 i4Ret;
    sd_cmd_t cmd;

    x_memset(&cmd, 0, sizeof(sd_cmd_t));
    cmd.idx = CMD12_STOP_TRANSMISSION;
    // cmd.dataMode = NULL_DATA_MODE;   // dataMode = 0 by x_memset

    i4Ret = MsdcSendCmd(&cmd);
    return i4Ret;
}

static unsigned char MsdcSendCSD(void)
{
    if (MSDC_SUCCESS != cmd9_send_csd())
        return MSDC_FAILED;

    return MSDC_SUCCESS;
}

// Final stage of card identification flow.
static unsigned char MsdcCardTypeChk(void)
{
    // CMD2
    if (MSDC_SUCCESS != cmd2_all_send_cid())
        return MSDC_CMD2_FAIL;

    // CMD3
    if (MSDC_SUCCESS != cmd3_send_relative_addr())
        return MSDC_CMD3_FAIL;

    return MSDC_SUCCESS;
}

static unsigned char MsdcCardRescan()
{
    unsigned char i;
    unsigned char uRet = 0;

    for (i = 0; i < CMD0_RESET_LIMIT; i++) {
        // CMD0 Pin1 = High
        if (MSDC_SUCCESS != cmd0_go_idle_state(0))
            return MSDC_FAILED;
    }

    if (MSDC_SUCCESS != cmd1_mmc_send_op_cond())
        return MSDC_CMD1_FAIL;

    sdCard.cardType = CARDTYPE_MEM_MMC;

    if (sdCard.memOcr.ocr & HOST_OCR)
    {
        UINT32 retry = 0;

        // Set new Voltage
        while (1) {
            uRet = cmd1_mmc_send_op_cond();

            if (MSDC_SUCCESS == uRet) {
                if (sdCard.memOcr.memRdy) {
                    sdCard.memInitDone = 1;

                    if (sdCard.memOcr.hcs)
                    {   // MMC : Sector addressing mode
                        sdCard.blkAddrMode = 1;
                    }
                    else
                    {   // MMC :  Byte addressing mode
                        sdCard.blkAddrMode = 0;
                    }

                    uRet = MsdcCardTypeChk();
                    if (MSDC_SUCCESS != uRet)
                        return uRet;

                    return MSDC_SUCCESS;
                } else {
                    retry++;
                    if (retry < MMC_CMD1_RETRY_LIMIT) {
                        CHIP_DisplayString(".");
                        CHIP_Delay_us(2000);
                        continue;
                    } else {
                        return MSDC_CMD1_RETRY_TIMEOUT;
                    }
                }
            } else {
                return MSDC_CMD1_FAIL;
            }
        }
    }
    return MSDC_SUCCESS;
}

static INT32 MsdcSetBusWidth(INT32 busWidth)
{
    if (MSDC_SUCCESS != cmd6_mmc_set_bus_width(busWidth))
        return MSDC_CMD6_SET_BUS_WIDTH_FAIL;

    /* Modify MSDC Register Settings */
    if (1 == busWidth) {
        MSDC_WRITE32(SDC_CFG, (MSDC_READ32(SDC_CFG) & ~SDC_CFG_BW_MASK) | (0 << SDC_CFG_BW_SHIFT));
    } else if (4 == busWidth) {
        MSDC_WRITE32(SDC_CFG, (MSDC_READ32(SDC_CFG) & ~SDC_CFG_BW_MASK) | (0x01 << SDC_CFG_BW_SHIFT));
    } else if (8 == busWidth) {
        MSDC_WRITE32(SDC_CFG, (MSDC_READ32(SDC_CFG) & ~SDC_CFG_BW_MASK) | (0x02 << SDC_CFG_BW_SHIFT));
    }

    return MSDC_SUCCESS;
}

static INT32 MsdcMemRead(UINT64 addr, void *pBuff, UINT32 buffLen)
{
    INT32 i4Ret, i4RetStop;
    UINT64 u8TmpAddr;

    u8TmpAddr = addr & (~(SDHC_BLK_SIZE - 1));

    //CHIP_EmmcShowHex("\nu4TmpAddr :", u4TmpAddr);
    //CHIP_EmmcShowHex("\naddr :", addr);
    //CHIP_EmmcShowHex("\nbuffLen :", buffLen);
    //CHIP_EmmcShowHex("\npBuff :", (UINT32)pBuff);

    if (((addr + buffLen) - u8TmpAddr) <= SDHC_BLK_SIZE) {
        // CMD17 Single block read
        i4Ret = cmd17_read_single_block(addr, pBuff, buffLen);

		return i4Ret;
    } else {
        // CMD18 Multiple block read
        i4Ret = cmd18_read_multiple_block(addr, pBuff, buffLen);
        if (MSDC_SUCCESS != i4Ret) {
            i4RetStop = cmd12_stop_transmission();
            // Error Handling process : sw should manually add stop command when multi-block read Failed
            if (MSDC_SUCCESS != i4RetStop)
				CHIP_EmmcShowHex("error cmd12 :", i4RetStop);
                //return MSDC_CMD12_FAIL;

			return i4Ret;
        }

        i4RetStop = cmd12_stop_transmission();
        if (MSDC_SUCCESS != i4RetStop)
            return MSDC_CMD12_FAIL;
    }

    return MSDC_SUCCESS;
}

//---------------------------------------------------------------------------
// MSDC PinMux Switch  function
//---------------------------------------------------------------------------
static void MSDC_PinMux()
{
	ch = 1;

    /*eMMC pinmux mode is 0*/
    MSDC_CLRBIT(EMMC_PAD_PMUX_CFG0, EMMC_PAD_PMUX_RST_CLK_DS_CMD_SEL);
    MSDC_CLRBIT(EMMC_PAD_PMUX_CFG1, EMMC_PAD_PMUX_DAT0_7_SEL);

    /*eMMC driving  setting*/
    MSDC_CLRBIT(EMMC_PAD_DRIVING_CFG0, EMMC_PAD_DRIVING_CLK_CMD_DAT0_1_SEL);
    MSDC_SETBIT(EMMC_PAD_DRIVING_CFG0, (0xDB6) << EMMC_PAD_DRIVING_CLK_CMD_DAT0_1_SEL_SHIFT);
    MSDC_CLRBIT(EMMC_PAD_DRIVING_CFG1, EMMC_PAD_DRIVING_DAT2_7_SEL);
    MSDC_SETBIT(EMMC_PAD_DRIVING_CFG1, (0x136DB6) << EMMC_PAD_DRIVING_DAT2_7_SEL_SHIFT);

    /*eMMC SR setting*/
    MSDC_CLRBIT(EMMC_PAD_SR_CFG, EMMC_PAD_SR_CLK_CMD_DAT_SEL);

    /*eMMC SMT setting*/
    MSDC_SETBIT(EMMC_PAD_SMT_CFG, EMMC_PAD_SMT_CLK_CMD_DAT_SEL);

    /*eMMC RDSEL setting*/
    MSDC_CLRBIT(EMMC_PAD_RDSEL_CFG0, EMMC_PAD_RDSEL_CLK_CMD_SEL);
    MSDC_CLRBIT(EMMC_PAD_RDSEL_CFG1, EMMC_PAD_RDSEL_DAT0_3_SEL);
    MSDC_CLRBIT(EMMC_PAD_RDSEL_CFG2, EMMC_PAD_RDSEL_DAT4_7_RST_SEL);

    /*eMMC TDSEL setting*/
    MSDC_CLRBIT(EMMC_PAD_TDSEL_CFG0, EMMC_PAD_TDSEL_CLK_CMD_SEL);
    MSDC_CLRBIT(EMMC_PAD_TDSEL_CFG1, EMMC_PAD_TDSEL_DAT_SEL);

    _fgMsdcIdentyCardOK = FALSE;
    _u4MsdcFreqIdx = 0;
}

//----------------------------------------------------------------------------
//MSDC PU/PD
//----------------------------------------------------------------------------
void MSDC_PUPD(void)
{
    /* PUPD cmd/dat/rst pull up, clk pull down*/
    MSDC_CLRBIT(EMMC_PAD_PUPD_CFG, EMMC_PAD_PUPD_CFG_CMD_DAT_RST_SEL);
    MSDC_SETBIT(EMMC_PAD_PUPD_CFG, EMMC_PAD_PUPD_CFG_CLK_SEL);

    //eMMC R1R0 setting
    MSDC_CLRBIT(EMMC_PAD_R0R1_CFG, EMMC_PAD_R0R1_CLK_SEL);
    MSDC_SETBIT(EMMC_PAD_R0R1_CFG, (0x2) << EMMC_PAD_R0R1_CLK_SEL_SHIFT);
    MSDC_CLRBIT(EMMC_PAD_R0R1_CFG, EMMC_PAD_R0R1_CMD_DAT_SEL);
    MSDC_SETBIT(EMMC_PAD_R0R1_CFG, (0x15555) << EMMC_PAD_R0R1_CMD_DAT_SEL_SHIFT);

    //eMMC IES setting
    MSDC_SETBIT(EMMC_PAD_IES_CFG, EMMC_PAD_IES_CLK_CMD_DAT_SEL);
}

//---------------------------------------------------------------------------
// MSDC interface function
//---------------------------------------------------------------------------
static unsigned char MsdcCardIdentify(void)
{
    unsigned char uRet = 0;
    if (MSDC_SUCCESS != MsdcSetClkfreq(INIT_BUS_FREQ))
        return MSDC_CARD_IDENT_SET_INIT_CLK_FAIL;

    uRet = MsdcCardRescan();
    if (MSDC_SUCCESS != uRet)
        return uRet;

    if (MSDC_SUCCESS != MsdcSendCSD())
    {
        //TBD
    }

    // CMD7 Select the target card
    if (MSDC_SUCCESS != cmd7_select_card(1))
        return MSDC_CMD7_SELECT_CARD_FAIL;

    // Set bus width
    if (MSDC_SUCCESS != MsdcSetBusWidth(8))
    {
        //TBD
    }

    uRet = MsdcSetClkfreq(_u4MsdcFreq[_u4MsdcFreqIdx]);
    if (MSDC_SUCCESS != uRet)
        return MSDC_CARD_IDENT_SET_HIGH_CLK_FAIL;

    return MSDC_SUCCESS;
}

//---------------------------------------------------------------------------
// MSDC interface function
//---------------------------------------------------------------------------
static unsigned char MsdcSysInit(void)
{
    // Initial variable
    unsigned char uRet = 0;

    // Init MSDC
    uRet = MsdcInit();
    if (MSDC_SUCCESS != uRet)
        return uRet;

    // Initial Msdc Freq setting flag
    _u4MsdcFreqIdx = 0;

    MsdcContinueClock(0);    // Clock power down when no SD Command

    return MSDC_SUCCESS;
}

static unsigned char MsdcIdentyCard(BOOL fgForceIdentCard)
{
    LDR_ENV_T *prLdrEnv = (LDR_ENV_T *)CC_LDR_ENV_OFFSET;

    if (_fgMsdcIdentyCardOK && !fgForceIdentCard)
        return MSDC_SUCCESS;

    // Clear sdCard structure, let card re-identify
    x_memset(&sdCard, 0, sizeof(sdcard_t));

    unsigned char uRet = 0;
    // Identify Card
    uRet = MsdcCardIdentify();
    if (MSDC_SUCCESS != uRet)
        return uRet;

    sdCard.flagHost = ch;
    x_memcpy((void *)(prLdrEnv->szMSDCenv), &sdCard, sizeof(sdcard_t));

    _fgMsdcIdentyCardOK = TRUE;
    return MSDC_SUCCESS;
}

static unsigned char MsdcReadCard(UINT64 u8Addr, UINT32 *pu4Buf, UINT32 u4Length)
{
    unsigned char uRetryCnt = 0;
    unsigned char uRet = 0;
	unsigned int  usample = 0;

    u4Length = (u4Length + 0x03) & (~0x03);

    do
    {
        uRet = MsdcMemRead(u8Addr, (void *)pu4Buf, u4Length);

        // Protect maxium retry
        if (uRetryCnt++ > MSDC_READ_CARD_RETRY_LIMIT)
            break;

        // Add error handling
        if (uRet != MSDC_SUCCESS) {
			if ((uRet == MSDC_CMD_ERR_RESP_CRCERR) || (uRet == MSDC_CMD_ERR_NO_RESP)) {
				usample = MSDC_READ32(MSDC_IOCON) & MSDC_IOCON_D_SMPL;
				if (usample == 0) {
					MSDC_WRITE32(MSDC_IOCON, MSDC_READ32(MSDC_IOCON) | MSDC_IOCON_D_SMPL);
					CHIP_EmmcShowHex("cmd1:", MSDC_READ32(MSDC_IOCON));
				} else {
					MSDC_WRITE32(MSDC_IOCON, MSDC_READ32(MSDC_IOCON) & (~MSDC_IOCON_D_SMPL));
					CHIP_EmmcShowHex("cmd2:", MSDC_READ32(MSDC_IOCON));
				}

			} else {
				//dump_register();
				usample = MSDC_READ32(MSDC_IOCON) & MSDC_IOCON_R_D_SMPL;
				if (usample == 1) {
					MSDC_WRITE32(MSDC_IOCON, MSDC_READ32(MSDC_IOCON) & (~MSDC_IOCON_R_D_SMPL));
					CHIP_EmmcShowHex("data1:", MSDC_READ32(MSDC_IOCON));
				} else {
					MSDC_WRITE32(MSDC_IOCON, MSDC_READ32(MSDC_IOCON) | MSDC_IOCON_R_D_SMPL);
					CHIP_EmmcShowHex("data2:", MSDC_READ32(MSDC_IOCON));
					if (_u4MsdcFreqIdx < (sizeof(_u4MsdcFreq) / sizeof(UINT32) - 1))
						_u4MsdcFreqIdx++;

					MsdcSetClkfreq(_u4MsdcFreq[_u4MsdcFreqIdx]);

					// If failed count too many, switch to 1 bit mode for read retry
					if (uRetryCnt > MSDC_READ_CARD_RETRY_LIMIT / 2) {
						// Set bus width
						MsdcSetBusWidth(1);
					}
				}
			}
		}
	} while (uRet != MSDC_SUCCESS);

    if (uRet != MSDC_SUCCESS)
        return uRet;

    return MSDC_SUCCESS;
}

static void MsdcToggleContClock(void)
{
    static INT32 i4ContClock = 1;

    // Continue clock & non-continue clock toggle for retry
    if (i4ContClock) {
        MsdcContinueClock(0);    // Clock power down when no SD Command
    } else {
        MsdcContinueClock(1);    // Clock continues even if no SD Command
    }

    i4ContClock = !i4ContClock;
}

static unsigned char MsdcIdentReadMMC(UINT32 *pu4Buf, UINT64 u8Addr, UINT32 u4Length)
{
    unsigned char uRetryCnt = 0;
    unsigned char uRet = 0;

    // Identify Card
    uRetryCnt = 0;
    do
    {
        if (uRetryCnt == 0) {
            uRet = MsdcIdentyCard(FALSE); // Don't need force identify card in the first time
        } else {
            uRet = MsdcIdentyCard(TRUE); // Force identify if retry
        }

        if (uRet != MSDC_SUCCESS) {
            MsdcToggleContClock(); // Toggle Continue clock setting for retry
        }

        if (uRetryCnt++ > MSDC_IDENTIFY_CARD_RETRY_LIMIT) {
            break;
        }
    } while (uRet != MSDC_SUCCESS);

    if (uRet != MSDC_SUCCESS)
        return uRet;

    // Read data
    uRetryCnt = 0;
    do
    {
        uRet = MsdcReadCard(u8Addr, pu4Buf, u4Length);

        if (uRet != MSDC_SUCCESS) {
            // Toggle Continue clock setting for retry
            MsdcToggleContClock();

            // If retry too many times, re-init the eMMC card
            if (uRetryCnt >= (MSDC_READ_BOOTLDR_DATA_RETRY_LIMIT / 2))
                MsdcIdentyCard(TRUE);
        }

        if (uRetryCnt++ > MSDC_READ_BOOTLDR_DATA_RETRY_LIMIT)
            break;
    } while (uRet != MSDC_SUCCESS);

    if (uRet != MSDC_SUCCESS)
        return uRet;

    return MSDC_SUCCESS;
}

// This function is just protect _u4Signature/ROMCODETAG not optimized by compiler.
UINT32 MsdcDummy(UINT32 i)
{
    return _u4Signature[i];
}

INT32 CHIP_eMMCCopyToDRAM(UINT32 *pu4DstBuf, UINT64 u8ByteOffset, UINT32 u4Len)
{
    unsigned char uRetryCnt = 0;
    unsigned char uRet = 0;

    if (!_fgMsdcInit) {

        // This function is just protect _u4Signature/ROMCODETAG not optimized by compiler.
        MsdcDummy(0);

        MSDC_PinMux();
		MSDC_PUPD();

        do {
            uRet = MsdcSysInit();
            if (uRetryCnt++ > MSDC_SYS_INIT_RETRY_LIMIT) {
                break;
            }
        } while (uRet != MSDC_SUCCESS);

        if (uRet != MSDC_SUCCESS) {
            return uRet;
        }
        _fgMsdcInit = TRUE;
    }
    return MsdcIdentReadMMC(pu4DstBuf, u8ByteOffset, u4Len);
}

#endif // CC_EMMC_BOOT
