/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


//-----------------------------------------------------------------------------
// Constant definitions
//-----------------------------------------------------------------------------

#define SHA1_HASH_SIZE      (20)
#define SHA224_HASH_SIZE    (28)
#define SHA256_HASH_SIZE    (32)
#define SHA_BLOCK_SIZE      (64)    //512-bit

// This structure will hold context information for the SHA-256
// hashing operation
typedef struct
{
    UINT32 Intermediate_Hash[SHA256_HASH_SIZE/4];   /* Message Digest  */

    UINT32 Length_Low;                              /* Message length in bits      */
    UINT32 Length_High;                             /* Message length in bits      */

                                                    /* Index into message block array   */
    INT32 Message_Block_Index;
    UINT8 Message_Block[64];                        /* 512-bit message blocks      */

    INT32 Computed;                                 /* Is the digest computed?         */
    INT32 Corrupted;                                /* Is the message digest corrupted? */
} SHA256_CONTEXT_T;

//-----------------------------------------------------------------------------
// Prototype  of inter-file functions
//-----------------------------------------------------------------------------

EXTERN INT32 SHA256_Reset(SHA256_CONTEXT_T *);
EXTERN INT32 SHA256_Input(SHA256_CONTEXT_T *, const UINT8 *, UINT32);
EXTERN INT32 SHA256_Result(SHA256_CONTEXT_T *, UINT8 Message_Digest[SHA256_HASH_SIZE]);
EXTERN INT32 SHA256(const UINT8 *message_array, UINT32 length, UINT8 Message_Digest[SHA256_HASH_SIZE]);
EXTERN INT32 mtk_hmac_sha256(UINT8 * key, UINT32 key_size, UINT8 * msg, UINT32 msg_size,  UINT8 result[SHA256_HASH_SIZE]);

