/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: xiaojun.zhu $
 * $Date: 2015/08/08 $
 * $RCSfile: drv_lvds.c,v $
 * $Revision: #5 $
 *
 *---------------------------------------------------------------------------*/
#ifdef CC_UP8032_ATV
#include "general_mt82.h"
#endif
#ifdef CC_COPLAT_MT82
#include "util.h"
#endif
#include "general.h"
#include "hw_lvds.h"
#include "hw_mlvds.h"
#include "hw_ospe.h"
#include "hw_tg.h"
#include "drv_lvds.h"
#include "drv_display.h"
#include "drv_vdoclk.h"
#include "x_timer.h"
#include "nptv_debug.h"
#ifndef CC_COPLAT_MT82
#include "drvcust_if.h"
#include "pmx_drvif.h"
#include "osd_drvif.h"
#include "vdp_if.h"
#endif
#include "x_ckgen.h"
#include "x_gpio.h"


// switch the source of both group b and group c from internal group b
//#define CC_LVDS_WA1


//-----------------------------------------------------------------------------
// Static variables
//-----------------------------------------------------------------------------
#ifdef DRV_SUPPORT_EXTMJC
static UINT32 _u4CurrentLvdsMode = (UINT32)eLvdsVidNormal;
static UINT32 _u4LvdsColorDomain = eColorDomainRGB;
#endif
static UINT32 _u4APadPDValue=0;
UINT32  LVDSTVO=0;

//automation
enum
{
  LVDS_PWON,
  LVDS_PWDOWN
};

UINT32 u2LVDSFifoSkwPre = 0;
#define LVDS_EFUSE_DEN  1070

//-----------------------------------------------------------------------------
// Static functions
//-----------------------------------------------------------------------------

static void _vDrvLVDSSetPortMap(void)
{
    // Lvds control : 
    UINT32 u4LvdsCtrlAEven = 0;
    UINT32 u4LvdsCtrlAOdd = 1;
    #ifndef CC_LVDS_WA1
    UINT32 u4LvdsCtrlBEven = 2;
    #endif
    UINT32 u4LvdsCtrlBOdd = 3;
    UINT32 au4PortRotte[4] = {0, 1, 2, 3};

    // panel specific port rotate
    if (LVDS_USE_INDEPENDENT_SETTING)
    {
        au4PortRotte[0] = LVDS_A_INDEPENDENT_SETTING;
        au4PortRotte[1] = LVDS_B_INDEPENDENT_SETTING;
        au4PortRotte[2] = LVDS_C_INDEPENDENT_SETTING;
        au4PortRotte[3] = LVDS_D_INDEPENDENT_SETTING;
    }

    // PCB layout rotate
    u4LvdsCtrlAEven = DRVCUST_PanelGet(eLvdsACtrlEven);
    u4LvdsCtrlAOdd = DRVCUST_PanelGet(eLvdsACtrlOdd);
    u4LvdsCtrlAEven = (u4LvdsCtrlAEven < 4)? au4PortRotte[u4LvdsCtrlAEven] : 0;
    u4LvdsCtrlAOdd = (u4LvdsCtrlAOdd < 4)? au4PortRotte[u4LvdsCtrlAOdd] : 0;   

    u4LvdsCtrlBEven = DRVCUST_PanelGet(eLvdsBCtrlEven);
    u4LvdsCtrlBOdd = DRVCUST_PanelGet(eLvdsBCtrlOdd);
    u4LvdsCtrlBEven = (u4LvdsCtrlBEven < 4)? au4PortRotte[u4LvdsCtrlBEven] : 0;
    u4LvdsCtrlBOdd = (u4LvdsCtrlBOdd < 4)? au4PortRotte[u4LvdsCtrlBOdd] : 0;   

    // A/B channel for PORTA, C/D channel for PORTB    
    vIO32WriteFldAlign(LVDSB_REG08, u4LvdsCtrlAEven, RG_A_SW);
    vIO32WriteFldAlign(LVDSB_REG08, u4LvdsCtrlAOdd,  RG_B_SW);
    vIO32WriteFldAlign(LVDSB_REG08, u4LvdsCtrlBEven, RG_C_SW);
    vIO32WriteFldAlign(LVDSB_REG08, u4LvdsCtrlBOdd,  RG_D_SW);
	
    if ((IS_PANEL_L12R12)&&(LVDS_OUTPUT_PORT == DUAL_PORT))
    {
        vIO32WriteFldAlign(LVDSB_REG08, 0, RG_A_SW);
        vIO32WriteFldAlign(LVDSB_REG08, 2, RG_B_SW);
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_ODD_SW); 
    }

    if (IS_IC_5895())
    {
    	vIO32WriteFldAlign(LVDSB_REG08, 0, RG_C_SW);
        vIO32WriteFldAlign(LVDSB_REG08, 1, RG_D_SW);
    }
}

static void _vDrvLVDSPanelInterfaceConfig(void)
{
	if ((IS_SEPARATE_OSD_AND_DATA)||(IS_DISABLE_LVDS_LINEBUFFER))
	{
		vIO32WriteFldAlign(VINTF_B0, 1,  VINTF_LINE_BUFFER_BYPASS); 
		vIO32WriteFldAlign(VINTF_P19, 3,  VINTF_DISP_R_ALLIGN_TYPE); 		
	}	
	else
	{
		vIO32WriteFldAlign(VINTF_B0, 0,  VINTF_LINE_BUFFER_BYPASS); 
		vIO32WriteFldAlign(VINTF_P19, 0,  VINTF_DISP_R_ALLIGN_TYPE); 				
	}

	vIO32WriteFldAlign(VINTF_P4, 1,  VINTF_REORDER_BYPASS); 	
	vIO32WriteFldAlign(VINTF_P4, wDISPLAY_WIDTH/2 ,  VINTF_REORDER_MIDDLE_PT);

	vIO32WriteFldAlign(VINTF_B0, 0 ,  VINTF_HV_BYPASS);
	vIO32WriteFldAlign(VINTF_B0, 1 ,  VINTF_HV_HOLD);     

	if(IS_PANEL_L12R12)
	{
		vIO32WriteFldAlign(VINTF_B0, 1,  VINTF_PDP_EN);
	}
	else
	{
		vIO32WriteFldAlign(VINTF_B0, 0,  VINTF_PDP_EN);    
	}

	// reorder setting
#ifdef CC_SCPOS_3DTV_SUPPORT 
	if ((LVDS_DISP_3D == LVDS_DISP_3D_SHUTTER) && (LVDS_DISP_SPLIT == LVDS_DISP_SPLIT_ON))
	{
		vIO32WriteFldAlign(VINTF_P4, 0,  VINTF_REORDER_BYPASS);    
		vIO32WriteFldAlign(VINTF_P4, REORDER_FORMAIN_OUT_2CH_HALF,	VINTF_REORDER_FORMAIN_OUT); 	
	}
	else
#endif	
	{
		vIO32WriteFldAlign(VINTF_P4, REORDER_FORMAIN_OUT_SEQ,  VINTF_REORDER_FORMAIN_OUT);     
	}
}

void vTXCRCStatus(void)
{
    UINT8 u1group,Group_Cnt;
    
    vIO32WriteFldAlign(LVDSB_REG10, 60,  RG_CRC_VCNT);
	if((LVDS_OUTPUT_PORT == SINGLE_PORT))
	{
     Group_Cnt = 1;
	}
	else if((LVDS_OUTPUT_PORT == DUAL_PORT))
	{
	 Group_Cnt = 2;
	}
	else // FOUR_PORT
	{
	 Group_Cnt = 4;
	}
	
    for (u1group = 0; u1group < Group_Cnt; u1group++)
    {
     vIO32WriteFldAlign(LVDSB_REG10, u1group,  RG_CRC_SEL);     
     vIO32WriteFldAlign(LVDSB_REG10, 1,  RG_CRC_CLR); 
     x_thread_delay(10);//10mS    
     vIO32WriteFldAlign(LVDSB_REG10, 0,  RG_CRC_CLR);     
     vIO32WriteFldAlign(LVDSB_REG10, 0,  RG_CRC_START);              
     x_thread_delay(10);//10mS         
     vIO32WriteFldAlign(LVDSB_REG10, 1,  RG_CRC_START);         
     while (!(IO32ReadFldAlign(LVDSB_REG12, ST_LVDS_CRC_RDY))) { }    
     if(IO32ReadFldAlign(LVDSB_REG12, ST_LVDS_CRC_ERR))
     {
      LOG(0,"Fail = [LVDS_%d] = %lx%lx\n",u1group,IO32ReadFldAlign(LVDSB_REG13, ST_LVDS_CRC_OUT_41_24),IO32ReadFldAlign(LVDSB_REG12, ST_LVDS_CRC_OUT_23_0));
	  LOG(0,"Fail = [VGA_%d] = %lx\n",u1group,IO32ReadFldAlign(LVDSB_REG11, ST_VGA_CRC_OUT));
     }
     else
     {
      LOG(0,"OK = [LVDS_%d] = %lx%lx\n",u1group,IO32ReadFldAlign(LVDSB_REG13, ST_LVDS_CRC_OUT_41_24),IO32ReadFldAlign(LVDSB_REG12, ST_LVDS_CRC_OUT_23_0));
	  LOG(0,"OK = [VGA_%d] = %lx\n",u1group,IO32ReadFldAlign(LVDSB_REG11, ST_VGA_CRC_OUT));
     } 
    }

    vIO32WriteFldAlign(LVDSB_REG20, 1,  RG_LVDSRX_FIFO_EN);

	if(wDrvGetOutputVTotal()>=1)
	{
	 if(IS_PANEL_L12R12)
	 vIO32WriteFldAlign(LVDSB_REG21, (PANEL_GetPanelHeight()/2)-1,  RG_LVDSRX_CRC_H_END);
	 else
	 vIO32WriteFldAlign(LVDSB_REG21, PANEL_GetPanelHeight()-1,  RG_LVDSRX_CRC_H_END);
	} 
	vIO32WriteFldAlign(LVDSB_REG21, 1,  RG_LVDSRX_CRC_H_START);	

	vIO32WriteFldAlign(LVDSB_REG20, 1,  RG_LVDSRX_CRC_CLR);	
	x_thread_delay(10);//10mS  
	vIO32WriteFldAlign(LVDSB_REG20, 0,  RG_LVDSRX_CRC_CLR);	
	vIO32WriteFldAlign(LVDSB_REG20, 0,  RG_LVDSRX_CRC_START);
	x_thread_delay(10);//10mS  
	vIO32WriteFldAlign(LVDSB_REG20, 1,  RG_LVDSRX_CRC_START);
	
	while (!(IO32ReadFldAlign(LVDSB_REG22,  LVDSRX_CRC_DONE))) { }    
	if(IO32ReadFldAlign(LVDSB_REG22, LVDSRX_CRC_FAIL))
	{
	 LOG(0,"Fail = [FIFO LVDS] = %lx\n",IO32ReadFldAlign(LVDSB_REG22, LVDSRX_CRC_VALUE));
	}
	else
	{
	 LOG(0,"OK = [FIFO LVDS] = %lx\n",IO32ReadFldAlign(LVDSB_REG22, LVDSRX_CRC_VALUE));
	} 

#if 0  // removed in Wukong
	vDrvTCONRXCRC(0);
#endif // removed in Wukong	
}

static void _vDrvLVDSConfigFormat(void)
{
    UINT32 u4MSBSW;
    
    u4MSBSW = LVDS_DISP_FORMAT;
    #ifdef DRV_SUPPORT_EXTMJC
    if (IS_COMPANION_CHIP_ON())
    {
        if (vDrvGetCurLvdsMode() !=  eLvdsVidNormal)
        {
            u4MSBSW = LVDS_MSB_SW_OFF;
        }
    }
    #endif

    
    // set lvds format
    if (u4MSBSW == LVDS_SPECIAL_NS) // special NS
    {
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_NS_VESA_EN);        
        vIO32WriteFldAlign(LVDSB_REG06, 1, RG_SPECIAL_NS);        
    }
    else if (u4MSBSW == LVDS_MSB_SW_ON) // NS
    {
		if (DISP_BIT == DISP_18BIT)
		{
			vIO32WriteFldAlign(LVDSB_REG06, 0, RG_NS_VESA_EN);
		}
		else
		{
        	vIO32WriteFldAlign(LVDSB_REG06, 1, RG_NS_VESA_EN);  
        }
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_SPECIAL_NS);        
    }
    else // JEIDA
    {
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_NS_VESA_EN);        
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_SPECIAL_NS);        
    }

	if (IS_SEPARATE_OSD_AND_DATA)
	{
        vIO32WriteFldAlign(LVDSB_REG26, 3, RG_OSD_SWITCH_CH);  
        vIO32WriteFldAlign(LVDSB_REG26, 1, RG_OSD_ENCODE_EN);  		
	}
	else
	{
        vIO32WriteFldAlign(LVDSB_REG26, 0, RG_OSD_SWITCH_CH);  
        vIO32WriteFldAlign(LVDSB_REG26, 0, RG_OSD_ENCODE_EN);  		
	}

    _vDrvLVDSPanelInterfaceConfig();
}

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------
void vDrvLVDSSetTermination(UINT8 bTerm)
{
    UINT32 u4TermVal;
    u4TermVal = bTerm|bTerm<<4|bTerm<<8|bTerm<<12|bTerm<<16|bTerm<<20|bTerm<<24|bTerm<<28;
    vIO32WriteFldAlign(REG_LVDS_TX_CFG27, u4TermVal, RG_LVDSA_TERM_0);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG28, u4TermVal, RG_LVDSA_TERM_1);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG29, u4TermVal, RG_LVDSA_TERM_2);
    vIO32WriteFldAlign(REG_LVDS_TX_CFG30, u4TermVal, RG_LVDSA_TERM_3);
}

#if defined(CONFIG_OPM) || defined(CC_SUPPORT_STR)
    BOOL fgLVDSInit = FALSE;
#else
    static BOOL fgLVDSInit = FALSE;
#endif
void vDrvLVDSInit(void)
{
    UINT32 u4Value;
    UINT64 *pu4PinMap,*pu4PNSwap;
    UINT32 u4INTREfuse;
    UINT32 u4NumEfuse;

	if (IS_PANEL_L12R12)
    {    
     vIO32WriteFldAlign(LVDSB_REG00, 1, RG_LVDSTX_2CH_ARCH);     
    }
    else
    {
     vIO32WriteFldAlign(LVDSB_REG00, 0, RG_LVDSTX_2CH_ARCH); 	    
    }
	
    if (IO32ReadFldAlign(DDDS_08, FLAG_LVDS_INITIAL) == 1)
    {
        fgLVDSInit = TRUE;
    }

	if (!fgLVDSInit)
	{
        // set TVO(driving current) if necessary
        u4Value = PANEL_GetDrivingCurrent();
        if (u4Value <= 15)
        {
            vDrvLVDSSetDriving((UINT8)u4Value);
        }
        else
        {
            if (!IS_IC_5895())
            {
            #ifndef LVDS_GPIO_SUPPORT
                vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x3fff,RG_LVDSB_R_TERM_EVEN_D11)| P_Fld(0x3fff,RG_LVDSB_R_TERM_ODD_D11));   
            #endif
            }
            vDrvLVDSSetDriving(7); // [Capy] 7: 9mA
        }

        // Disable 2CH architecture in MT5368/96
    	//    vIO32WriteFldAlign(VINTF_B0, 0, VINTF_2CH_EN);  // 98/80 remove

        //bypass LVDS input buffer since the clock source is the same
    	// it can avoid vsync reset makes data improper
        vIO32WriteFldAlign(LVDSB_REG01, 1, RG_IBUF_BYPASS);
    	
        // ==Pair Swap ==
        if(NULL == (pu4PinMap = (UINT64*)DRVCUST_PanelGetUptr((QUERY_TYPE_T)ePanelLvds10bitPinMap)))
        {
            // Port B Setting
            vIO32WriteFldMulti(LVDSB_REG03, P_Fld(7, RG_RLV7_SEL)|P_Fld(6, RG_RLV6_SEL)|P_Fld(5, RG_RLV5_SEL)|P_Fld(4, RG_RLV4_SEL)|P_Fld(3, RG_RLV3_SEL)|P_Fld(2, RG_RLV2_SEL)|P_Fld(1, RG_RLV1_SEL)|P_Fld(0, RG_RLV0_SEL));
            vIO32WriteFldMulti(LVDSB_REG04, P_Fld(11, RG_RLV_CK1_SEL)|P_Fld(10, RG_RLV_CK0_SEL)|P_Fld(9, RG_RLV9_SEL)|P_Fld(8, RG_RLV8_SEL));		

    	    // Port A Setting	
            vIO32WriteFldMulti(LVDSB_REG02, P_Fld(7, RG_LLV7_SEL)|P_Fld(6, RG_LLV6_SEL)|P_Fld(5, RG_LLV5_SEL)|P_Fld(4, RG_LLV4_SEL)|P_Fld(3, RG_LLV3_SEL)|P_Fld(2, RG_LLV2_SEL)|P_Fld(1, RG_LLV1_SEL)|P_Fld(0, RG_LLV0_SEL));
            vIO32WriteFldMulti(LVDSB_REG04, P_Fld(11, RG_LLV_CK1_SEL)|P_Fld(10, RG_LLV_CK0_SEL)|P_Fld(9, RG_LLV9_SEL)|P_Fld(8, RG_LLV8_SEL));
        }
        else
        {
            // remap data pairs
            vIO32WriteFldMulti(LVDSB_REG02, P_Fld(pu4PinMap[7], RG_LLV7_SEL)|P_Fld(pu4PinMap[6], RG_LLV6_SEL)|P_Fld(pu4PinMap[5], RG_LLV5_SEL)|P_Fld(pu4PinMap[4], RG_LLV4_SEL)|P_Fld(pu4PinMap[3], RG_LLV3_SEL)|P_Fld(pu4PinMap[2], RG_LLV2_SEL)|P_Fld(pu4PinMap[1], RG_LLV1_SEL)|P_Fld(pu4PinMap[0], RG_LLV0_SEL));
            vIO32WriteFldMulti(LVDSB_REG04, P_Fld(pu4PinMap[11], RG_LLV_CK1_SEL)|P_Fld(pu4PinMap[10], RG_LLV_CK0_SEL)|P_Fld(pu4PinMap[9], RG_LLV9_SEL)|P_Fld(pu4PinMap[8], RG_LLV8_SEL));
            vIO32WriteFldMulti(LVDSB_REG03, P_Fld(pu4PinMap[19], RG_RLV7_SEL)|P_Fld(pu4PinMap[18], RG_RLV6_SEL)|P_Fld(pu4PinMap[17], RG_RLV5_SEL)|P_Fld(pu4PinMap[16], RG_RLV4_SEL)|P_Fld(pu4PinMap[15], RG_RLV3_SEL)|P_Fld(pu4PinMap[14], RG_RLV2_SEL)|P_Fld(pu4PinMap[13], RG_RLV1_SEL)|P_Fld(pu4PinMap[12], RG_RLV0_SEL));
            vIO32WriteFldMulti(LVDSB_REG04, P_Fld(pu4PinMap[23], RG_RLV_CK1_SEL)|P_Fld(pu4PinMap[22], RG_RLV_CK0_SEL)|P_Fld(pu4PinMap[21], RG_RLV9_SEL)|P_Fld(pu4PinMap[20], RG_RLV8_SEL));
        }

        // == PN Swap ==
        // pn swap BE0(RLV5)/ BO2(RLV2)/ BO0(RLV0)
        if(NULL == (pu4PNSwap = (UINT64*)DRVCUST_PanelGetUptr((QUERY_TYPE_T)ePanelLvds10bitPNSwap)))
        {
            vIO32WriteFldMulti(LVDSB_REG05, 
                P_Fld(0, RG_RLVCK1_PN_SWAP)|P_Fld(0, RG_RLVCK0_PN_SWAP)|P_Fld(0, RG_RLV9_PN_SWAP)|P_Fld(0, RG_RLV8_PN_SWAP)|P_Fld(0, RG_RLV7_PN_SWAP)|P_Fld(0, RG_RLV6_PN_SWAP)|
                P_Fld(0, RG_RLV5_PN_SWAP)|P_Fld(0, RG_RLV4_PN_SWAP)|P_Fld(0, RG_RLV3_PN_SWAP)|P_Fld(0, RG_RLV2_PN_SWAP)|P_Fld(0, RG_RLV1_PN_SWAP)|P_Fld(0, RG_RLV0_PN_SWAP)|	
                P_Fld(0, RG_LLVCK1_PN_SWAP)|P_Fld(0, RG_LLVCK0_PN_SWAP)|P_Fld(0, RG_LLV9_PN_SWAP)|P_Fld(0, RG_LLV8_PN_SWAP)|P_Fld(0, RG_LLV7_PN_SWAP)|P_Fld(0, RG_LLV6_PN_SWAP)|
                P_Fld(0, RG_LLV5_PN_SWAP)|P_Fld(0, RG_LLV4_PN_SWAP)|P_Fld(0, RG_LLV3_PN_SWAP)|P_Fld(0, RG_LLV2_PN_SWAP)|P_Fld(0, RG_LLV1_PN_SWAP)|P_Fld(0, RG_LLV0_PN_SWAP)
                );
        }
        else
        {
            vIO32WriteFldMulti(LVDSB_REG05, 
                P_Fld(pu4PNSwap[11], RG_RLVCK1_PN_SWAP)|P_Fld(pu4PNSwap[10], RG_RLVCK0_PN_SWAP)|P_Fld(pu4PNSwap[9], RG_RLV9_PN_SWAP)|P_Fld(pu4PNSwap[8], RG_RLV8_PN_SWAP)|P_Fld(pu4PNSwap[7], RG_RLV7_PN_SWAP)|P_Fld(pu4PNSwap[6], RG_RLV6_PN_SWAP)|
                P_Fld(pu4PNSwap[5], RG_RLV5_PN_SWAP)|P_Fld(pu4PNSwap[4], RG_RLV4_PN_SWAP)|P_Fld(pu4PNSwap[3], RG_RLV3_PN_SWAP)|P_Fld(pu4PNSwap[2], RG_RLV2_PN_SWAP)|P_Fld(pu4PNSwap[1], RG_RLV1_PN_SWAP)|P_Fld(pu4PNSwap[0], RG_RLV0_PN_SWAP)|   
                P_Fld(pu4PNSwap[23], RG_LLVCK1_PN_SWAP)|P_Fld(pu4PNSwap[22], RG_LLVCK0_PN_SWAP)|P_Fld(pu4PNSwap[21], RG_LLV9_PN_SWAP)|P_Fld(pu4PNSwap[20], RG_LLV8_PN_SWAP)|P_Fld(pu4PNSwap[19], RG_LLV7_PN_SWAP)|P_Fld(pu4PNSwap[18], RG_LLV6_PN_SWAP)|
                P_Fld(pu4PNSwap[17], RG_LLV5_PN_SWAP)|P_Fld(pu4PNSwap[16], RG_LLV4_PN_SWAP)|P_Fld(pu4PNSwap[15], RG_LLV3_PN_SWAP)|P_Fld(pu4PNSwap[14], RG_LLV2_PN_SWAP)|P_Fld(pu4PNSwap[13], RG_LLV1_PN_SWAP)|P_Fld(pu4PNSwap[12], RG_LLV0_PN_SWAP)
                );
        }

        if (IS_IC_5895())
        {
            // 1.
            vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0, RG_LVDSA_CHBIAS_PWD);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(0, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(0, RG_LVDSA_CKCHLDO_PWD));
            if (PANEL_GetDisplayType() == PANEL_DISPLAY_TYPE_VB1)
            {
                vIO32WriteFldMulti(REG_LVDS_TX_CFG12, P_Fld(0x01, RG_LVDSA_TVCM)|P_Fld(7, RG_LVDSA_NSRC)|P_Fld(7, RG_LVDSA_PSRC));
            }
            else
            {
                vIO32WriteFldMulti(REG_LVDS_TX_CFG12, P_Fld(0x03, RG_LVDSA_TVCM)|P_Fld(1, RG_LVDSA_TVCM_TRACK)
                                                    |P_Fld(0, RG_LVDSA_NSRC)|P_Fld(1, RG_LVDSA_PSRC));
            }
            vIO32WriteFldMulti(REG_LVDS_TX_CFG12, P_Fld(1, RG_LVDSA_DIV5_EN)|P_Fld(1, RG_LVDSA_CHBIAS_PWD));
            vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0xffffffff, RG_LVDSA_EXT_EN);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0x2, RG_LVDSA_PREDRV_VREF);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0, RG_LVDSA_BG_PWD);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(1, RG_LVDS_BG_INTR_EN)|P_Fld(0x20, RG_LVDS_INTR_CAL));
            
            // 2.
            vIO32WriteFldMulti(REG_LVDS_TX_CFG19, P_Fld(0, RG_LVDSA_HSN_MODE_EN)|P_Fld(0, RG_LVDSA_HSP_MODE_EN)|P_Fld(1, RG_LVDSA_PREDRV_LVDS_EN));
            vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 1, RG_LVDSA_PREDRV_VOD);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG39, 0, RG_LVDSA_PREDRV_PWD);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG26, 0, RG_LVDSA_PREDRV_LDO_PWD);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG14, /*P_Fld(4, RG_LVDSA_BIAS_SEL)|*/P_Fld(7, RG_LVDSA_NSRC_PRE)|P_Fld(7, RG_LVDSA_PSRC_PRE));

            vIO32WriteFldAlign(REG_LVDS_TX_CFG18, 0, RG_LVDSA_LVDS_SEL);  // [Capy] 0: LVDS & VBO; 1: mLVDS(reserved)
            vIO32WriteFldAlign(REG_LVDS_TX_CFG17, 0x0, RG_LVDSA_CLK_IQ_SEL);

            // 3.
            vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x1, RG_LVDSA_RESYNC_LVDS_SEL_D11);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG24, 0x1, RG_CHOP_CLK_EN);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_RESYNC_CLK_IQ_SEL_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG12, P_Fld(0x1, RG_LVDSA_DIV5_EN)|P_Fld(0x2, RG_LVDSA_CKPATH_EN));
            vIO32WriteFldAlign(REG_LVDS_TX_CFG23, 0x0, RG_LVDSA_SLDO_LVROD);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG25, 0x0, RG_LVDSA_SLDO_PWD);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 0x0, RG_LVDSA_DRV_OPMODE_EN);

            vIO32WriteFldAlign(REG_LVDS_TX_CFG14, 1, RG_LVDSA_TERM_EN);
            vDrvLVDSSetTermination(8);  // 100ohm as default
        }
        else
        {
            //vIO32WriteFldAlign(REG_LVDS_TX_CFG8, 1, RG_LVDSB_LDO_PWD_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG0, P_Fld(0x0B, RG_LVDSB_TVCM_D11 )|P_Fld(1, RG_LVDSB_EN_D11 )|P_Fld(1, RG_LVDSB_BIAS_EN_D11));

            vIO32WriteFldMulti(REG_LVDS_TX_CFG2, P_Fld(0x0, RG_LVDSB_CLK_IQ_SEL_EVEN_D11)|P_Fld(0,RG_LVDSB_CLK_IQ_SEL_ODD_D11 ));
        }
        
        if (IS_DISABLE_PANEL_HVSYNC)
        {
            // disable vsync output for dual LVDS channels
            vIO32WriteFldAlign(LVDSB_REG06, 1, RG_VS_SEL);				  
            // disable hsync output for dual LVDS channels
            vIO32WriteFldAlign(LVDSB_REG06, 1, RG_HS_SEL);				  
        }
        else
        {
            // enable vsync output for dual LVDS channels
            vIO32WriteFldAlign(LVDSB_REG06, 0, RG_VS_SEL);                
            // enable hsync output for dual LVDS channels
            vIO32WriteFldAlign(LVDSB_REG06, 0, RG_HS_SEL);                
        }
        // enable data_en output for dual LVDS channels
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DE_SEL);                

        u4INTREfuse=(UINT32)(((IO32ReadFldAlign(IO_VIRT + 0x86DC, AC_FULLDW)) >> 16) & 0x1F);
        u4NumEfuse = 1000*(6+u4INTREfuse);
        u4INTREfuse = u4NumEfuse/LVDS_EFUSE_DEN;
        u4NumEfuse = u4NumEfuse - (u4INTREfuse*LVDS_EFUSE_DEN);
        //Printf("[LVDS Efuse]INT=%d,NUM=%d,DEN=%d\n", u4INTREfuse, u4NumEfuse, LVDS_EFUSE_DEN);
        if (u4NumEfuse >= (LVDS_EFUSE_DEN>>1))
        {
            u4INTREfuse += 1;
            //Printf("[LVDS Efuse]INT+1-->%d\n", u4INTREfuse);
        }

        vIO32WriteFldAlign(REG_LVDS_TX_CFG25, (u4INTREfuse&0x1F), RG_LVDS_INTR_CAL); 
        vIO32WriteFldAlign(REG_LVDS_TX_CFG24, ((u4INTREfuse&0x20)>>5), RG_LVDS_INTR_CAL_BIT5);
            
        //LVDS 7->4 FIFO enable 
        #ifdef CC_MTK_LOADER
        vDrvLVDS7To4FifoEnable();
        #else
        vIO32WriteFldAlign(LVDSB_REG08, 1, RG_LVDS_7TO10FIFO_EN);                
        #endif

        vIO32WriteFldAlign(DDDS_08, 1, FLAG_LVDS_INITIAL);
	} 
}

#define LVDSDrivingIndex 8
PRIVATE UINT8 bLVDSDrivingCurrent[LVDSDrivingIndex+1] =
{
//	0x00, 0x00,0x00,0x00,0x02,0x04,0x06,0x08,0x0A,
//  3mA,  3mA,  3mA,, 3mA,,   4mA,   5mA,   6mA,   7mA,    8mA,//Remy,99 will be add 4 

    0x00, 0x01,0x03,0x05,0x07,0x09,0x0B,0x0D,0x0F,
//  1.5mA, 2mA,  3mA, 4mA,  5mA,  6mA, 7mA, 8mA, 9mA, [Capy]	
//  0x00, 0x00,0x02,0x04,0x06,0x08,0x0A,0x0C,0x0E,
//  1mA,  1mA,2mA,3mA, 4mA, 5mA, 6mA,7mA,8mA, [Husky]
};


void vDrvLVDSSetDriving(UINT8 bDriving)
{    
    //mt5363/87 bit [0] 1 mA bit [1] 1 mA bit [2] 1 mA bit [3] 2 mA
    //mt5365/95 bit [0] 1 mA bit [1] 2 mA bit [2] 4 mA bit [3] 4 mA
    //mt5396      bit [0] 05mA bit [1] 1 mA bit [2] 2 mA bit [3] 4 mA
    //mt5398      bit [0] 05mA bit [1] 1 mA bit [2] 2 mA bit [3] 4 mA    
    bDriving &= 0xf;  //0~15
    if (bDriving > LVDSDrivingIndex) 
    {
        bDriving = LVDSDrivingIndex;
    }
    bDriving = bLVDSDrivingCurrent[bDriving];
    //vIO32WriteFldAlign(REG_LVDS_TX_CFG12, bDriving, RG_LVDSA_TVO);
    LVDSTVO=vDrvLVDSsettingDrivingCurrent(bDriving);
    if (IS_IC_5895())
    {
        vIO32WriteFldAlign(REG_LVDS_TX_CFG31, LVDSTVO, RG_LVDSA_TVO_0);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG32, LVDSTVO, RG_LVDSA_TVO_1);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG33, LVDSTVO, RG_LVDSA_TVO_2);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG34, LVDSTVO, RG_LVDSA_TVO_3);
    }
}
UINT32 vDrvLVDSsettingDrivingCurrent(UINT8 bCurrent)
{	
	UINT32 TVOValue;
	TVOValue = bCurrent|bCurrent<<4|bCurrent<<8|bCurrent<<12|bCurrent<<16|bCurrent<<20|bCurrent<<24|bCurrent<<28;
	return (TVOValue);
}

UINT8 vDrvLVDSGetDriving(void)
{
    UINT8 bDriving;

    if (IS_IC_5895())
    {
        //bDriving = (UINT8)(IO32ReadFldAlign(REG_LVDS_TX_CFG12, RG_LVDSA_TVO)/2+1);
        bDriving = (UINT8)(IO32ReadFldAlign(REG_LVDS_TX_CFG31, RG_LVDSA_TVO_0)&0xF);
        bDriving = (bDriving+1)/2;  // level
    }
    else
    {
        bDriving =(UINT8)(((IO32ReadFldAlign(REG_LVDS_TX_CFG3, RG_LVDSB_TVO_EVEN_D11))>>24)/2+1);
    }
    //bDriving &= 0xf;  //0~15
    return (bDriving);
}

void vDrvLVDSSetVCM(UINT8 bVCM)
{    
    bVCM &= 0xf;  //0~15
    if (IS_IC_5895())
    {
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, bVCM, RG_LVDSA_TVCM);
    }
    else
    {
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, bVCM, RG_LVDSB_TVCM_D11);	
    }
}

UINT8 vDrvLVDSGetVCM(void)
{
    UINT8 bVCM=0;

    if (IS_IC_5895())
    {
        bVCM =(UINT8)IO32ReadFldAlign(REG_LVDS_TX_CFG12,  RG_LVDSA_TVCM);
    }
    bVCM &= 0xf;  //0~15
    return (bVCM);
}

#ifdef DRV_SUPPORT_EXTMJC
void vDrvSetLvdsColorDomain(UINT32 u4ColorDomain)
{
#ifndef CC_MTK_LOADER
    	UINT32 u4BgColor;
#endif

	_u4LvdsColorDomain =u4ColorDomain;
#ifndef CC_MTK_LOADER
	// set mute color
	VDP_GetBg(SV_VP_MAIN, &u4BgColor);
	VDP_SetBg(SV_VP_MAIN, u4BgColor);
	VDP_GetBg(SV_VP_PIP, &u4BgColor);
	VDP_SetBg(SV_VP_PIP, u4BgColor);
#endif
    	if(vDrvGetLvdsColorDomain() == eColorDomainYUV)
    	{
		//PMX_SetYuvMode(TRUE);
		//OSD_BASE_SetYuvOutput(TRUE);
		PMX_SetYuvMode(FALSE); // RGB to YUV do by LVDS TX Transform
		OSD_BASE_SetYuvOutput(FALSE); // RGB to YUV do by LVDS TX Transform
    	}
    	else
    	{
    		PMX_SetYuvMode(FALSE);
    	       OSD_BASE_SetYuvOutput(FALSE);
    	}

}

UINT32 vDrvGetLvdsColorDomain(void)
{
	return _u4LvdsColorDomain;
}

void vDrvSetCurLvdsMode(UINT32 u4LvdsMode)
{
    if (_u4CurrentLvdsMode == u4LvdsMode)
    {
        return;
    }
    
    _u4CurrentLvdsMode = u4LvdsMode;
    LOG(1, "vDrvSetCurLvdsMode, _u4CurrentLvdsMode=%d\n", _u4CurrentLvdsMode);

    vDrvLVDSChgMod(_u4CurrentLvdsMode);
}

UINT32 vDrvGetCurLvdsMode(void)
{
    return _u4CurrentLvdsMode;
}

#endif

/**
 * @brief vDrvLVDSChgMod
 * Change mode for 53xx + 828x
 *
 * @param  u4Mode
 * @retval void
 */
void vDrvLVDSChgMod(UINT32 u4Mode)
{
    UINT32 u48BitMode;


    if (((LVDS_MODE_T)u4Mode == eLvdsVidYuv422TwoPort8Bit)
        || ((LVDS_MODE_T)u4Mode == eLvdsVidYuv422OnePort8Bit)
        || ((LVDS_MODE_T)u4Mode == eLvdsVidRgb444OnePort8Bit))
    {
        u48BitMode = 1;
    }
    else
    {
        u48BitMode = 0;
    }

    #ifdef CC_SCPOS_3DTV_SUPPORT
    if ((LVDS_MODE_T)u4Mode != eLvdsVidNormal) //  MLVDS_SO0_RG_RES_FLD msut be 1
    {
		UINT32 u4EnPair;
		if (DRVCUST_PanelQuery(eDISP_RLVDSEncodeLink, &u4EnPair) == 0)
		{
			u1SetDISP_REncodeEn(((UINT8)u4EnPair)|0x01); //  MLVDS_SO0_RG_RES_FLD msut be 1
		} 
		else
		{
			vIO32WriteFldAlign(LVDSB_REG08, 1, RG_OSD_3D_IN_HS);
			vIO32WriteFldAlign(LVDSB_REG08, 1, RG_OSD_3D_IN_VS);	 
		}
    }
	else
	{
	 vIO32WriteFldAlign(LVDSB_REG08, 0, RG_OSD_3D_IN_HS);
	 vIO32WriteFldAlign(LVDSB_REG08, 0, RG_OSD_3D_IN_VS);	
	}
	#endif
	
    // select one mode
    switch ((LVDS_MODE_T)u4Mode)
    {
    case eLvdsVidNormal:
        // Back to the initial setting and return
        vIO32WriteFldAlign(OSTG_01, 0, REG_YUV2YC_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_LPF_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_C_LINE_EXT);
        vIO32WriteFldMulti(LVDSB_REG06, P_Fld(0, RG_MERGE_OSD)|P_Fld(0, RG_2CH_MERGE)
                                            |P_Fld(0, RG_RGB_444_MERGE)|P_Fld(u48BitMode, RG_8BIT_DUAL));
        vDrvVOPLLSet();
        break;


    case eLvdsVidYuv422TwoPort:
    case eLvdsVidYuv422TwoPort8Bit:
        // enable Tx mode 3
        vIO32WriteFldAlign(OSTG_01, 1, REG_YUV2YC_EN);
        vIO32WriteFldAlign(OSTG_01, 1, REG_LPF_EN);
        vIO32WriteFldAlign(OSTG_01, 1, REG_C_LINE_EXT);
        vIO32WriteFldMulti(LVDSB_REG06, P_Fld(1, RG_MERGE_OSD)|P_Fld(1, RG_2CH_MERGE)
                                            |P_Fld(0, RG_RGB_444_MERGE)|P_Fld(u48BitMode, RG_8BIT_DUAL));
        #if 0
        #ifdef DRV_SUPPORT_EXTMJC_MODE7
        if (DRVCUST_InitGet(eFlagDDRQfp)) //mt5387 condition
        {
            vIO32WriteFldAlign(OUTSTG_OS_19, 1, OS_19_REG_8BIT_DUAL);
        }
        #endif
        #endif
        vDrvVOPLLSet();
        break;

    case eLvdsVidRgb444TwoPort:  
        // enable Tx mode 4
        vIO32WriteFldAlign(OSTG_01, 0, REG_YUV2YC_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_LPF_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_C_LINE_EXT);
        vIO32WriteFldMulti(LVDSB_REG06, P_Fld(1, RG_MERGE_OSD)|P_Fld(1, RG_2CH_MERGE)
                                            |P_Fld(1, RG_RGB_444_MERGE)|P_Fld(u48BitMode, RG_8BIT_DUAL));
        vDrvVOPLLSet();
        break;

    case eLvdsVidYuv422OnePort:
    case eLvdsVidYuv422OnePort8Bit:
        vIO32WriteFldAlign(OSTG_01, 1, REG_YUV2YC_EN);
        vIO32WriteFldAlign(OSTG_01, 1, REG_LPF_EN);
        vIO32WriteFldAlign(OSTG_01, 1, REG_C_LINE_EXT);
        vIO32WriteFldMulti(LVDSB_REG06, P_Fld(1, RG_MERGE_OSD)|P_Fld(0, RG_2CH_MERGE)
                                            |P_Fld(0, RG_RGB_444_MERGE)|P_Fld(u48BitMode, RG_8BIT_DUAL));
        vDrvVOPLLSet();
        break;

    case eLvdsVidRgb444OnePort:
    case eLvdsVidRgb444OnePort8Bit:
        vIO32WriteFldAlign(OSTG_01, 0, REG_YUV2YC_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_LPF_EN);
        vIO32WriteFldAlign(OSTG_01, 0, REG_C_LINE_EXT);
        vIO32WriteFldMulti(LVDSB_REG06, P_Fld(1, RG_MERGE_OSD)|P_Fld(0, RG_2CH_MERGE)
                                            |P_Fld(1, RG_RGB_444_MERGE)|P_Fld(u48BitMode, RG_8BIT_DUAL));
        vDrvVOPLLSet();
        break;
    default:
        LOG(0, "Not Support LVDS Mode (%d)\n", u4Mode);
        break;
    }

    // the clock changed, reset 74Fifo
    vDrvLVDS7To4FifoEnable();

    // set lvds format
    _vDrvLVDSConfigFormat();

    // config LVDS PD, dual port, and 10bit control bits
    vDrvLVDSOn();
    
}

void vSetLVDSPadPD(UINT32 u4PDVal)
{
    // for !MT5895
    UINT8 EVENPD;
    UINT8 ODDPD;
    UINT32 u4PDVal_tmp;
    //////////////////

    // Lvds control : 
    UINT32 u4LvdsCtrlAEven = 0;
    UINT32 u4LvdsCtrlAOdd = 1;
    UINT32 u4LvdsCtrlBEven = 2;
    UINT32 u4LvdsCtrlBOdd = 3;
    UINT32 au4PortRotte[4] = {0, 1, 2, 3};
    UINT32 au4PDCh[4];
    UINT32 u4TmpCnt;
    UINT64 *pu4PinMap;
    UINT32 u4TmpPDCh;
    UINT32 u4TmpCnt2;
    UINT8 u1PinCh;
    //const UINT32 au4PadRemapTbl[12]={0,1,2,3,4,0,1,2,3,4,5/*EvenCLK*/,5/*OddCLK*/};
    const UINT32 au4PadRemapTbl[12]={0,1,2,4,5,0,1,2,4,5,3/*EvenCLK*/,3/*OddCLK*/};

    // panel specific port rotate
    if (LVDS_USE_INDEPENDENT_SETTING)
    {
        au4PortRotte[0] = LVDS_A_INDEPENDENT_SETTING;
        au4PortRotte[1] = LVDS_B_INDEPENDENT_SETTING;
        au4PortRotte[2] = LVDS_C_INDEPENDENT_SETTING;
        au4PortRotte[3] = LVDS_D_INDEPENDENT_SETTING;
    }

    // PCB layout rotate
    u4LvdsCtrlAEven = DRVCUST_PanelGet(eLvdsACtrlEven);
    u4LvdsCtrlAOdd = DRVCUST_PanelGet(eLvdsACtrlOdd);
    u4LvdsCtrlAEven = (u4LvdsCtrlAEven < 4)? au4PortRotte[u4LvdsCtrlAEven] : 0;
    u4LvdsCtrlAOdd = (u4LvdsCtrlAOdd < 4)? au4PortRotte[u4LvdsCtrlAOdd] : 0;   

    u4LvdsCtrlBEven = DRVCUST_PanelGet(eLvdsBCtrlEven);
    u4LvdsCtrlBOdd = DRVCUST_PanelGet(eLvdsBCtrlOdd);
    u4LvdsCtrlBEven = (u4LvdsCtrlBEven < 4)? au4PortRotte[u4LvdsCtrlBEven] : 0;
    u4LvdsCtrlBOdd = (u4LvdsCtrlBOdd < 4)? au4PortRotte[u4LvdsCtrlBOdd] : 0;   

    pu4PinMap = (UINT64*)DRVCUST_PanelGetUptr((QUERY_TYPE_T)ePanelLvds10bitPinMap);

    // u4PDVal: BO/BE/AO/AE (3/2/1/0)
    for(u4TmpCnt=0;u4TmpCnt<4;u4TmpCnt++)
    {
        au4PDCh[u4TmpCnt] = ((u4PDVal >> (6*u4TmpCnt)) & 0x3f);
        if(NULL != pu4PinMap)
        {
            u4TmpPDCh = 0; //au4PDCh[u4TmpCnt]; 
            // remap the analog pad channel
            for(u4TmpCnt2=0;u4TmpCnt2<6;u4TmpCnt2++)
            {
                if(u4TmpCnt2 < 5)
                {   //data channel
                    u1PinCh =(UINT8)(u4TmpCnt2+5*(u4TmpCnt%2));
                }
                else
                {   //clk channel
                    u1PinCh =(UINT8)(10+(u4TmpCnt%2));
                }
                if(0 != (au4PDCh[u4TmpCnt] & (1<< au4PadRemapTbl[u1PinCh])))
                {
                    //power down this channel
                    u4TmpPDCh |= (1 << au4PadRemapTbl[pu4PinMap[u1PinCh]]);
                }  
            }
            au4PDCh[u4TmpCnt] = u4TmpPDCh;
        }
    }
    u4PDVal = (au4PDCh[0] << (6*u4LvdsCtrlAEven)) |
              (au4PDCh[1] << (6*u4LvdsCtrlAOdd)) |
              (au4PDCh[2] << (6*u4LvdsCtrlBEven)) |
              (au4PDCh[3] << (6*u4LvdsCtrlBOdd)) ;

    if(1)  // just for IC verification
	{
		u4PDVal = 0x0;
    }
    
    if (IS_IC_5895())
    {
        UNUSED(u4PDVal_tmp);
        UNUSED(EVENPD);
        UNUSED(ODDPD);
        u4PDVal = 0;  // all lane on, for Husky LVDS
        vIO32WriteFldAlign(REG_LVDS_TX_CFG13, (0xFFFFFFFF-u4PDVal), RG_LVDSA_EXT_EN);
        //vIO32WriteFldAlign(REG_LVDS_TX_CFG0, (0xFFF-(u4PDVal>>12)), RG_LVDSB_EXT_EN_D11);
        //vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0x1, RG_LVDSA_EXT_EN_D11_E0);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG15, u4PDVal, RG_LVDSA_DRV_PWD);
        //u4PDVal_tmp= (u4PDVal>>12&0xFFF);
    }
    else
    {
        //vIO32WriteFldAlign(REG_LVDS_TX_CFG0, (0x3FFF-u4PDVal), RG_LVDSB_EXT_EN_D11);
        //vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0x1, RG_LVDSB_EXT_EN_D11_E0);
        u4PDVal_tmp= (u4PDVal&0xFFF);
      
        EVENPD=(( u4PDVal_tmp>>6)|(((u4PDVal_tmp>>6)& 0x20)<<7))&0x5F;
        ODDPD=(( u4PDVal_tmp&0x3F)|(((u4PDVal_tmp&0x3F)& 0x20)<<7))&0x5F;
        vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld((0x5F-EVENPD), RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld((0x5F-ODDPD),RG_LVDSB_DRV_EN_ODD_D11));
    }
}

#define DEBUG_MSG_TX 0
static void vSetLVDSPadSourcePD(UINT32 u4DispBit, UINT32 u4LVDSPort)
{
	UINT8  u1EnableDataBit,u1Link;	
	UINT32 u4ClockPD, u4LinkPD, u4EnablePair, u4FinaleEnablePair;		
    UINT32 u4PadPDTbl[4]={0,1,2,3}; // {PD_A,PD_B,PD_C,PD_D}
    
    #if DEBUG_MSG_TX
    LOG(0,"[SA7]u4DispBit = 0x%8x, u4LVDSPort = 0x%8x, \n",u4DispBit,u4LVDSPort);   
    #endif

    u4LinkPD = 0x00; 
    u4EnablePair = 0;
    u4LinkPD = 0; 
    u4ClockPD = 0;  
	
	// Enable bit Control
	if (u4DispBit == DISP_18BIT) u1EnableDataBit = 0x18;
	else if (u4DispBit == DISP_24BIT) u1EnableDataBit = 0x10;	
   	else if (u4DispBit == DISP_30BIT) u1EnableDataBit = 0x00;		
   	else u1EnableDataBit = 0x00;
   
   #if DEBUG_MSG_TX
   LOG(0,"u1EnableDataBit = 0x%8x\n",u1EnableDataBit);  
   #endif
                 
    for (u1Link=0; u1Link<4; u1Link++)
    {
     u4EnablePair = u4EnablePair + (u1EnableDataBit << (u1Link*5));        
    }             
    #if DEBUG_MSG_TX                   
    LOG(0,"u4EnablePair = 0x%8x\n",u4EnablePair);
    #endif
    
   	// Power Down Link Control   	
	if (u4LVDSPort == SINGLE_PORT) 
    {
       for (u1Link=0; u1Link<4; u1Link++)
       {
         if(u4PadPDTbl[u1Link]!=0)
         {
            u4LinkPD = u4LinkPD +   (0x1F<<(u1Link*5));          
         }                             
       }                
    }
	else if (u4LVDSPort == DUAL_PORT)
	
	{
       for (u1Link=0; u1Link<4; u1Link++)
       {
         if ((u4PadPDTbl[u1Link]==2) || (u4PadPDTbl[u1Link]==3))
         {
            u4LinkPD = u4LinkPD +   (0x1F<<(u1Link*5));
         }                             
       }                  
    }             
   	else if (u4LVDSPort == FOUR_PORT) u4LinkPD = 0x00;		
   	else u4LinkPD = 0x00;   	   	

    #if DEBUG_MSG_TX
    LOG(0,"u4EnablePair = 0x%8x\n",u4EnablePair);
    LOG(0,"u4LinkPD = 0x%8x\n",u4LinkPD);    
    #endif
    
    // Power Down clock  path
	if (u4LVDSPort == SINGLE_PORT)
	{
       for (u1Link=0; u1Link<4; u1Link++)
       {
         if(u4PadPDTbl[u1Link]!=0)
         {
            u4ClockPD = u4ClockPD +   (0x1<<u1Link); 
         }                             
       }              
    }               
	else if (u4LVDSPort == DUAL_PORT)
	{
       for (u1Link=0; u1Link<4; u1Link++)
       {
         if ((u4PadPDTbl[u1Link]==2) || (u4PadPDTbl[u1Link]==3))
         {
            u4ClockPD = u4ClockPD +   (0x1<<u1Link);  
         }                             
       }             
    }     
   	else if (u4LVDSPort == FOUR_PORT) u4ClockPD = 0x00;		
   	else u4ClockPD = 0x00;   
    
    #if DEBUG_MSG_TX
    LOG(0,"u1ClockPD = 0x%8x\n",u4ClockPD);
    #endif

    if(((IS_PANEL_L12R12) && (u4LVDSPort == DUAL_PORT))||(IS_FORCE_LVDS_DIG_4CH_ON))
    {
      u4LinkPD = 0x00; 
      u4ClockPD = 0x00;
    }	

    #if 0  // Just for Remy(5881)
    u4ClockPD = 0x00;  // Remy just has one clk pair, if need to swap 0DD/EVEN link, should enable linkA and linkB's clk.
    #endif

    if (IS_IC_5895())
    {
        u4LinkPD = 0x00;  // all lane on, for Husky LVDS
        u4ClockPD = 0x00;
    }

	// Combin
    u4FinaleEnablePair = (u4LinkPD | u4EnablePair | (u4ClockPD << 20)) & 0xFFFFFF; 
	#if DEBUG_MSG_TX
    LOG(0,"u4FinaleEnablePair = 0x%8x\n",u4FinaleEnablePair); 
	#endif
	vIO32WriteFldAlign(LVDSB_REG07, u4FinaleEnablePair, RG_PD);
}     

// Power on all analog and digital PAD control
void vSetLVDSDAPadPDForceOn(void)
{
	vSetLVDSPadPD(0);
	vSetLVDSPadSourcePD(DISP_30BIT,FOUR_PORT);	
}

void u4ReadAPad(void)
{
    if (IS_IC_5895())
    {
        LOG(0,"               BE1 BE0 BO4 BO3 | BOCK BO2 BO1 BO0 | AE4 AE3 AECK AE2 | AE1 AE0 AO4 AO3 | AOCK AO2 AO1 AO0\n");
        LOG(0,"LVDSA_EXT_EN:  <0x%6lx>\n", IO32ReadFldAlign(REG_LVDS_TX_CFG13,RG_LVDSA_EXT_EN));
        LOG(0,"LVDSA_DRV_PWD: <0x%6lx>\n", IO32ReadFldAlign(REG_LVDS_TX_CFG15,RG_LVDSA_DRV_PWD));
    }
    else
    {
        LOG(0,"                    ECK E5 E4 | E3 E2 E1 E0\n");
        LOG(0,"LVDSB_DRV_EN_EVEN:  <0x%2lx>\n", IO32ReadFldAlign(REG_LVDS_TX_CFG1,RG_LVDSB_DRV_EN_EVEN_D11));
        LOG(0,"                    OCK O5 O4 | O3 O2 O1 O0\n");
        LOG(0,"LVDSB_DRV_EN_ODD:   <0x%2lx>\n", IO32ReadFldAlign(REG_LVDS_TX_CFG1,RG_LVDSB_DRV_EN_ODD_D11));
    }
}

void vSetLVDSAPad(UINT8 u1OnOff, UINT16 u4BExtEn, UINT16 u4BDrvPwd,UINT16 u4AExtEn, UINT16 u4ADrvPwd)
{
    UINT8 EVEN;
    UINT8 ODD;

    switch (u1OnOff)
    {
        case 0:  // power on
            vSetLVDSPadPD(_u4APadPDValue);
            break;
        case 1:  // power down
            vSetLVDSPadPD(0xFFFFFF);
            break;
        case 2: // user-defined
            vIO32WriteFldAlign(REG_LVDS_TX_CFG13, u4AExtEn, RG_LVDSA_EXT_EN);
            //vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0x1, RG_LVDSA_EXT_EN_D11_E0);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG15, u4ADrvPwd, RG_LVDSA_DRV_PWD);
            if (!IS_IC_5895())
            {
                EVEN=(( u4BExtEn>>6)|(((u4BExtEn>>6)& 0x20)<<7))&0x5F;
                ODD=(( u4BExtEn&0x3F)|(((u4BExtEn&0x3F)& 0x20)<<7))&0x5F;
                vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(EVEN, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(ODD,RG_LVDSB_DRV_EN_ODD_D11));
            }
            else
            {
                UNUSED(EVEN);
                UNUSED(ODD);
            }
            break;
        case 3:  // port A power down
            vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0x0, RG_LVDSA_EXT_EN);
            //vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0x1, RG_LVDSA_EXT_EN_D11_E0);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG15, 0xFFFFFFFF, RG_LVDSA_DRV_PWD);
            break;
        case 4: // port B power down
            if (!IS_IC_5895())
            {
                vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(0x0, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(0x0,RG_LVDSB_DRV_EN_ODD_D11));
            }
            break;
        default:
            break;
    }
    u4ReadAPad();
}

static UINT32 u4DrvLVDSGetDispBit(void)
{
	UINT32 u4DispBit;

	u4DispBit = DISP_BIT;

#ifdef DRV_SUPPORT_EXTMJC
    if (IS_COMPANION_CHIP_ON())
    {
        if (vDrvGetCurLvdsMode() !=  eLvdsVidNormal)        
        {
            u4DispBit = DISP_30BIT;
        }
    }
#endif

	return (u4DispBit);
}

static UINT32 u4DrvLVDSGetDispPort(void)
{
	UINT32 u4LVDSPort;

	// for force power on lvds analog pad.
	if(IS_FORCE_LVDS_ANA_4CH_ON)
	{
		u4LVDSPort = FOUR_PORT; 
	}
	else if(IS_FORCE_LVDS_ANA_2CH_ON)
	{
    	u4LVDSPort = DUAL_PORT;	
	}
	else
	{
    	u4LVDSPort = LVDS_OUTPUT_PORT;
	}

#ifdef DRV_SUPPORT_EXTMJC
    if (IS_COMPANION_CHIP_ON())
    {
        if (vDrvGetCurLvdsMode() !=  eLvdsVidNormal)        
        {
            u4LVDSPort = DUAL_PORT;
        }
    }
#endif

	return (u4LVDSPort);
}

void u4ReadDPadPD(void)
{
	LOG(0,"       CD CC CB CA | D4 D3 D2 D1 | D0 C4 C3 C2 | C1 C0 B4 B3 | B2 B1 B0 A4 | A3 A2 A1 A0\n");
	LOG(0,"RG_PD: <0x%6lx>\n", (IO32ReadFldAlign(LVDSB_REG07,RG_PD)));
}

void vSetLVDSDPadPD(UINT8 u1OnOff, UINT32 u4Value)
{
    switch (u1OnOff)
    {
        case 0:  // power on
            vSetLVDSPadSourcePD(u4DrvLVDSGetDispBit(),u4DrvLVDSGetDispPort());
            break;
        case 1:  // power down
            vIO32WriteFldAlign(LVDSB_REG07, 0xFFFFFF, RG_PD);
            break;
        case 2: // user-defined
            vIO32WriteFldAlign(LVDSB_REG07, (u4Value & 0xFFFFFF), RG_PD);
            break;
        default:
            break;
    }
    u4ReadDPadPD();
}

/**
 * @brief vSetPairSeqInverse

 * Control LVDS analog pad sequence for power on

 * @param  1: Inverse 0: Normal
 * @retval void
 */

UINT8 u1PairSeqInverse = 0;
void vSetPairSeqInverse(UINT8 u1Type)
{
// u1PairSeqInverse = 0;
// [MT5398] 		Analog related setting - 0: O0 ; 1: O1 ; 2: O2 ; 3: O3 ; 4: O4 ;		    5: OCK ; 6: E0 ; 7: E1 ; 8: E2 ;  9: E3 ;	10: E4 ;             11: ECK
// [MT5880/5860] Analog related setting - 0: O0 ; 1: O1 ; 2: O2 ; 3: O3 ; 4: O4 ; 5: O5 ; 6: OCK ; 7: E0 ; 8: E1 ; 9: E2 ; 10: E3 ; 11: E4 ; 12: E5 ; 13: ECK
// u1PairSeqInverse = 1;
// [MT5398]		 Analog related setting - 0: OCK; 1: O4 ; 2: O3 ; 3: O2 ; 4: O1 ;	     5: O0 ; 6: ECK ; 7: E4 ; 8: E3 ;  9: E3 ;	 10: E1 ;	           11: E0
// [MT5880/5860] Analog related setting - 0: OCK ; 1: O5 ; 2: O4 ; 3: O3 ; 4: O2 ; 5: O1 ; 6: O1 ; 7: ECK ; 8: E5 ; 9: E4 ; 10: E3 ; 11: E2 ; 12: E1 ; 13: E0

 u1PairSeqInverse = u1Type;
}

extern UINT8 _Lvds7To4FifoReset;
extern BOOL fgIsVsyncIsrStart;
#if defined(CONFIG_OPM) || defined(CC_SUPPORT_STR)
    UINT8 fgLVDSonoff = 0;
#else
    static UINT8 fgLVDSonoff = 0;
#endif
/**
 * @brief vDrvLVDSOn

 * Power On LVDS

 * @param  void
 * @retval void
 */
void vDrvLVDSOn(void)
{
    UINT32 u4DispBit;
    UINT32 u4LVDSPort;
    UINT32 u4PDValue = 0;
    UINT32 u4TwoChMerge;
	UINT32 u4LvdsCtrlAEven;
	  
    if ((IO32ReadFldAlign(DDDS_08, FLAG_LVDS_ONOFF) == 1)&& (!(IS_PANEL_L12R12)))
    {
        fgLVDSonoff = TRUE;
    }


	if (!fgLVDSonoff)
	{
    if (!IS_IC_5895())
    {   
    #ifndef LVDS_GPIO_SUPPORT
        vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x3fff,RG_LVDSB_R_TERM_EVEN_D11)| P_Fld(0x3fff,RG_LVDSB_R_TERM_ODD_D11));   
    #endif	
    }
    if (IS_IC_5895())
    {
        //A group power on sequence
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0, RG_LVDSA_CHBIAS_PWD);
        vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(0, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(0, RG_LVDSA_CKCHLDO_PWD));
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 1, RG_LVDSA_DIV5_EN);    
    }
    else
    {
        //B group power on sequence
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 1,RG_LVDSB_BIAS_EN_D11 );
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 1, RG_LVDSB_EN_D11 );
    }

	u4DispBit = DISP_BIT;

	// for force power on lvds analog pad.
	if(IS_FORCE_LVDS_ANA_4CH_ON)
	{
		u4LVDSPort = FOUR_PORT; 
	}
	else if(IS_FORCE_LVDS_ANA_2CH_ON)
	{
    	u4LVDSPort = DUAL_PORT;	
	}
	else
	{
    	u4LVDSPort = LVDS_OUTPUT_PORT;
	}	
	
    u4TwoChMerge = (LVDS_OUTPUT_PORT == DUAL_PORT)? 1: 0;
    #ifdef DRV_SUPPORT_EXTMJC
    if (IS_COMPANION_CHIP_ON())
    {
        if (vDrvGetCurLvdsMode() !=  eLvdsVidNormal)        
        {
            u4DispBit = DISP_30BIT;
            u4LVDSPort = DUAL_PORT;

            if ((vDrvGetCurLvdsMode() == eLvdsVidYuv422TwoPort8Bit) || (vDrvGetCurLvdsMode() == eLvdsVidYuv422TwoPort) || (vDrvGetCurLvdsMode() == eLvdsVidRgb444TwoPort))
            {
                u4TwoChMerge = 1;
            }
            else if ((vDrvGetCurLvdsMode() == eLvdsVidYuv422OnePort) || (vDrvGetCurLvdsMode() == eLvdsVidRgb444OnePort) || (vDrvGetCurLvdsMode() == eLvdsVidYuv422OnePort8Bit) || (vDrvGetCurLvdsMode() == eLvdsVidRgb444OnePort8Bit))
            {
                u4TwoChMerge = 0;
            }
        }
    }
    #endif

	if(LVDS_DISP_ODD_SW)
	{
		u4LvdsCtrlAEven = 1;
	}
	else if (LVDS_USE_INDEPENDENT_SETTING)
	{
	    if ((LVDS_A_INDEPENDENT_SETTING ==1) || (LVDS_A_INDEPENDENT_SETTING ==0))
	    {
			u4LvdsCtrlAEven = LVDS_A_INDEPENDENT_SETTING;
	    }	
		else
		{
			u4LvdsCtrlAEven = 0;
		}
	}	
	else
	{
		u4LvdsCtrlAEven = DRVCUST_PanelGet(eLvdsACtrlEven);
	}	
            
    switch (u4DispBit)
    {
        case DISP_30BIT:
            // enable LVDS 10-bit
            vIO32WriteFldAlign(LVDSB_REG06, 1, RG_10B_EN);                
            if(u4LVDSPort == FOUR_PORT)
            {
                vIO32WriteFldAlign(LVDSB_REG06, 1, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);
				u4PDValue = 0;
            }
            else if (u4LVDSPort == DUAL_PORT)
            {
				if (IS_PANEL_L12R12)
				{            
                 vIO32WriteFldAlign(LVDSB_REG06, 1, RG_4CH);                
                 vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);                
				}
				else
				{            
                 vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                 vIO32WriteFldAlign(LVDSB_REG06, u4TwoChMerge, RG_DUAL);                
				}					
                
                u4PDValue = 0x00fff000; // PD BO/BE
            }
            else  // single port
            {
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);                
                if (u4LvdsCtrlAEven == 1)  // power down Port A-Odd
                {
					if (u1PairSeqInverse)
					u4PDValue = 0x00ffffC0;
					else
                    u4PDValue = 0x00fff03f; // PD BO/BE/AO
                }
                else if (u4LvdsCtrlAEven == 0) // power down Port A-Even
                {
					if (u1PairSeqInverse)
                    u4PDValue = 0x00ffff3f;
					else	
                    u4PDValue = 0x00ffffc0; // PD BO/BE/AE
                }
            }
            break;

        case DISP_24BIT:
            // disable LVDS 10-bit
            vIO32WriteFldAlign(LVDSB_REG06, 0, RG_10B_EN);                
            if(u4LVDSPort == FOUR_PORT)
            {
                vIO32WriteFldAlign(LVDSB_REG06, 1, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);
				if (u1PairSeqInverse)
				u4PDValue = 0x00082082;
				else					
				u4PDValue = 0x00410410;  // PD O4, E4 
            }
            else       
            if (u4LVDSPort == DUAL_PORT)
            {
				if (IS_PANEL_L12R12)
				{            
                 vIO32WriteFldAlign(LVDSB_REG06, 1, RG_4CH);                
                 vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);
				}
				else
				{            
                 vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                 vIO32WriteFldAlign(LVDSB_REG06, 1, RG_DUAL);
				}					
				if (u1PairSeqInverse)
				u4PDValue = 0x00fff082;
				else
                u4PDValue = 0x00fff410;   // PD BO/BE/AE4/AO4
            }
            else  // single port
            {
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);
                if (u4LvdsCtrlAEven == 1)  // power down Port A-Odd
                {
					if (u1PairSeqInverse)
                    u4PDValue = 0x00ffffC2;
                    else
                    u4PDValue = 0x00fff43f;   // PD BO/BE/AO/AE4
                }
                else if (u4LvdsCtrlAEven == 0) // power down Port A-Even
                {
					if (u1PairSeqInverse)
                    u4PDValue = 0x00fff0bf;
                    else					
                    u4PDValue = 0x00ffffd0;   // PD BO/BE/AE/AO4
                }
            }
            break;

        case DISP_18BIT:
            // disable LVDS 10-bit
            vIO32WriteFldAlign(LVDSB_REG06, 0, RG_10B_EN);                
            if (u4LVDSPort == DUAL_PORT)
            {
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 1, RG_DUAL);
				if (u1PairSeqInverse)
				u4PDValue = 0x00fff186;
				else
				u4PDValue = 0x00fff618;					
           }
            else  // single port
            {
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_4CH);                
                vIO32WriteFldAlign(LVDSB_REG06, 0, RG_DUAL);
                if (u4LvdsCtrlAEven == 1)  // power down Port A-Odd
                {
					if (u1PairSeqInverse)
					u4PDValue = 0x00ffffc6;						
					else	
					u4PDValue = 0x00fff63f;
                }
                else if (u4LvdsCtrlAEven == 0) // power down Port A-Even
                {
					if (u1PairSeqInverse)
					u4PDValue = 0x00fff1bf;						
					else
					u4PDValue = 0x00ffffd8;
                }
            }
            break;

        default:
            break;
    }

    if (IS_IC_5895())
    {
        u4PDValue = 0;  // all lane on, for Husky LVDS
    }
    // Analog Pair Power down control
    _u4APadPDValue = u4PDValue;
	vSetLVDSPadPD(u4PDValue);
	// Some case the digital pad control is different with analog pad control by LVDS Port Contrl (ex: OSD and Data seperated)
	u4LVDSPort = LVDS_OUTPUT_PORT;
    // Digital Pair Power down control
	vSetLVDSPadSourcePD(u4DispBit,u4LVDSPort);

    #ifdef CC_COPLAT_MT82
    vUtDelay2us(5);
    #else
    HAL_Delay_us(10);
    #endif

    if (IS_IC_5895())
    {
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0, RG_LVDSA_DIV5_EN);	
        HAL_Delay_us(LVDSA_EN_REST_DELAYTIME);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 1, RG_LVDSA_DIV5_EN);
    }
    else
    {
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0,RG_LVDSB_EN_D11 );	
        HAL_Delay_us(LVDSA_EN_REST_DELAYTIME);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 1,RG_LVDSB_EN_D11 );	
    }

	#ifdef CC_MTK_LOADER    
    vDrvLVDS7To4FifoEnable();   
	#else
	if (fgIsVsyncIsrStart)
	{
		_Lvds7To4FifoReset = 2;
	}
	else
	{
		LOG(0, "[LVDS]NO output Vsync! toggle FIFO immediately!\n");
		vDrvLVDS7To4FifoEnable();
	}	
	#endif
	#ifndef NDEBUG
    LOG(0, " ======= [SA7] vDrvLVDSOn toggele RG_LVDSA_DIV5_EN ===========\n");
	#endif

	#ifdef LVDS_ESD_PROTECT
		u2LVDSFifoSkwPre = u1ChkLVDSFIFOSkewStatus();
	#endif

    #ifdef CC_EXTERNAL_LVDS_CHIP 
    IMpac3P_SetLvds(1);
    #endif
	#ifdef CC_EXTMJC_WA	
	UINT8 u1MsgErr;
	drv_extmjc_reset_rx_tx(100,&u1MsgErr);
	#endif	
    fgLVDSonoff = TRUE;
    vIO32WriteFldAlign(DDDS_08, 1, FLAG_LVDS_ONOFF);
	}
}

/**
 * @brief vDrvLVDSOff

 * Power Down LVDS

 * @param  void
 * @retval void
 */
void vDrvLVDSOff(void)
{
    // power down all digital channel
    vIO32WriteFldAlign(LVDSB_REG07, 0xffffff, RG_PD);                                           

    if (IS_IC_5895())
    {
        //A group power off sequence
        vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 0, RG_LVDSA_DIV5_EN);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0, RG_LVDSA_EXT_EN);   
        vIO32WriteFldAlign(REG_LVDS_TX_CFG15, 0xFFFFFFFF, RG_LVDSA_DRV_PWD);    
        //vIO32WriteFldMulti(REG_LVDS_TX_CFG23, P_Fld(1, RG_LVDSA_CKLDO_CKGENLDO_PWD)|P_Fld(1, RG_LVDSA_CKCHLDO_PWD));
        //vIO32WriteFldAlign(REG_LVDS_TX_CFG12, 1, RG_LVDSA_CHBIAS_PWD);
    }
    else
    {
        //B group power off sequence
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0,RG_LVDSB_EN_D11 );    
        vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(0, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(0,RG_LVDSB_DRV_EN_ODD_D11));  
        vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0, RG_LVDSB_BIAS_EN_D11 );
    }
    
#ifdef CC_EXTERNAL_LVDS_CHIP 
	IMpac3P_SetLvds(0);
#endif

	fgLVDSonoff = FALSE;
	vIO32WriteFldAlign(DDDS_08, 0, FLAG_LVDS_ONOFF);
}

/**
 * @brief vDrvLVDSConfig

 * Power Down LVDS

 * @param  void
 * @retval void
 */
void vDrvLVDSConfig(void)
{   
    // set lvds format
    _vDrvLVDSConfigFormat();

    if (LVDS_DISP_ODD_SW == LVDS_ODD_SW_ON)
    {
        if (LVDS_OUTPUT_PORT == SINGLE_PORT)
        {
         vIO32WriteFldAlign(LVDSB_REG06, 0, RG_ODD_SW); 
         vIO32WriteFldAlign(LVDSB_REG06, 1, RG_CH_SW);        
        }
        else
        {
         vIO32WriteFldAlign(LVDSB_REG06, 1, RG_ODD_SW); 
		 vIO32WriteFldAlign(LVDSB_REG06, 0, RG_CH_SW); 
        } 
    }
    else
    {
        if (LVDS_OUTPUT_PORT == SINGLE_PORT)
        {
         vIO32WriteFldAlign(LVDSB_REG06, 0, RG_CH_SW);        
        }
        else
        {    
        vIO32WriteFldAlign(LVDSB_REG06, 0, RG_ODD_SW);                
        }
    }

    if (HSYNC_POLARITY == HSYNC_LOW)
    {  
        vIO32WriteFldAlign(LVDSB_REG00, 1, RG_HSYNC_P);                
    }
    else
    {        
        vIO32WriteFldAlign(LVDSB_REG00, 0, RG_HSYNC_P);                
    }

    if (VSYNC_POLARITY == VSYNC_LOW)
    {       
        vIO32WriteFldAlign(LVDSB_REG00, 1, RG_VSYNC_P);                
        vIO32WriteFldAlign(LVDSB_REG01, 1, RG_IBUF_RST_SEL);                
    }
    else
    {
        vIO32WriteFldAlign(LVDSB_REG00, 0, RG_VSYNC_P);                
        vIO32WriteFldAlign(LVDSB_REG01, 0, RG_IBUF_RST_SEL);                
    }


    // Lvds port swap control
    _vDrvLVDSSetPortMap();
   
}

void u1TCONIFSourceSet(UINT8 val)
{
  if (val<=2)
  {
   vIO32WriteFldAlign(VINTF_P0, val, VINTF_PANEL_IN_SEL);   
  } 
}

UINT8 u1TCONIFSourceGet(void)
{
  return (IO32ReadFldAlign(VINTF_P0, VINTF_PANEL_IN_SEL));
}

void u1TCONIFLineBufferSet(UINT8 val)
{
   if (val<=1)
   {
    vIO32WriteFldAlign(VINTF_B0, val, VINTF_LINE_BUFFER_BYPASS);   
   }	
}

UINT8 u1TCONIFLineBufferGet(void)
{
  return (IO32ReadFldAlign(VINTF_B0, VINTF_LINE_BUFFER_BYPASS));
}


void vDrvLVDS7To4FifoEnable(void)
{
    UINT8 u1Unmute;
	
	#ifndef NDEBUG
	LOG(0, "[LVDS] vDrvLVDS7To4FifoEnable !\n");
	#endif

    if(IO32ReadFldAlign(MUTE_00, R_MUTE_POST_EN) ==1 )   
    {
	  u1Unmute = 0;
    }
	else
	{
      vIO32WriteFldAlign(MUTE_00, 1,  R_MUTE_POST_EN);
	  u1Unmute = 1;
	}  
    vIO32WriteFldAlign(LVDSB_REG08, 0, RG_LVDS_7TO10FIFO_EN);                
    vIO32WriteFldAlign(LVDSB_REG08, 1, RG_LVDS_7TO10FIFO_EN);                
    
    if(u1Unmute)
    	vIO32WriteFldAlign(MUTE_00, 0, R_MUTE_POST_EN);
}

//automation cli 
void LVDS_ATERM_ATVO_Setting(UINT8 mode)
{
    if(mode==1)
    {
        if (IS_IC_5895())
        {   
            vIO32WriteFldAlign(REG_LVDS_TX_CFG31, 7, RG_LVDSA_TVO_0);  // 5mA
            vIO32WriteFldAlign(REG_LVDS_TX_CFG32, 7, RG_LVDSA_TVO_1);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG33, 7, RG_LVDSA_TVO_2);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG34, 7, RG_LVDSA_TVO_3);
        }
        else
        {
            LVDSTVO=vDrvLVDSsettingDrivingCurrent(8);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG3,  LVDSTVO, RG_LVDSB_TVO_EVEN_D11);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG4, LVDSTVO, RG_LVDSB_TVO_ODD_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x3fff,RG_LVDSB_R_TERM_EVEN_D11)| P_Fld(0x3fff,RG_LVDSB_R_TERM_ODD_D11)); 
        }
    }
    else if(mode==2)
    {
        if (IS_IC_5895())
        {   
            vIO32WriteFldAlign(REG_LVDS_TX_CFG31, 3, RG_LVDSA_TVO_0);  // 3mA
            vIO32WriteFldAlign(REG_LVDS_TX_CFG32, 3, RG_LVDSA_TVO_1);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG33, 3, RG_LVDSA_TVO_2);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG34, 3, RG_LVDSA_TVO_3);
        }
        else
        {
            LVDSTVO=vDrvLVDSsettingDrivingCurrent(4);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG3,  LVDSTVO, RG_LVDSB_TVO_EVEN_D11);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG4, LVDSTVO, RG_LVDSB_TVO_ODD_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x1555,RG_LVDSB_R_TERM_EVEN_D11)| P_Fld(0x1555,RG_LVDSB_R_TERM_ODD_D11)); 
        }
    }
    else if(mode==3)
    {
        if (IS_IC_5895())
        {   
            vIO32WriteFldAlign(REG_LVDS_TX_CFG31, 7, RG_LVDSA_TVO_0);  // 5mA
            vIO32WriteFldAlign(REG_LVDS_TX_CFG32, 7, RG_LVDSA_TVO_1);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG33, 7, RG_LVDSA_TVO_2);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG34, 7, RG_LVDSA_TVO_3);
        }
        else
        {
            LVDSTVO=vDrvLVDSsettingDrivingCurrent(8);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG3,  LVDSTVO, RG_LVDSB_TVO_EVEN_D11);
            vIO32WriteFldAlign(REG_LVDS_TX_CFG4, LVDSTVO, RG_LVDSB_TVO_ODD_D11);
        }
    }   
}

void LVDS_ATERM_ATVO_Restore(UINT32 ATERN, UINT8 ATVO)
{
    UINT32 u4Tvo;
    if (IS_IC_5895())
    {
        u4Tvo = vDrvLVDSsettingDrivingCurrent(ATVO);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG31, u4Tvo, RG_LVDSA_TVO_0);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG32, u4Tvo, RG_LVDSA_TVO_1);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG33, u4Tvo, RG_LVDSA_TVO_2);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG34, u4Tvo, RG_LVDSA_TVO_3);
    }
    else
    {
        LVDSTVO=vDrvLVDSsettingDrivingCurrent(ATVO);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG3, LVDSTVO, RG_LVDSB_TVO_EVEN_D11);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG4, LVDSTVO, RG_LVDSB_TVO_ODD_D11);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG6, ATERN, RG_LVDSB_R_TERM_EVEN_D11);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG6, ATERN, RG_LVDSB_R_TERM_ODD_D11);
    }
}

void LVDS_ATVO_Set(UINT8 mode)
{
    UINT32 u4TVO;
    if (IS_IC_5895())
    {
        u4TVO = vDrvLVDSsettingDrivingCurrent(mode);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG31, u4TVO, RG_LVDSA_TVO_0);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG32, u4TVO, RG_LVDSA_TVO_1);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG33, u4TVO, RG_LVDSA_TVO_2);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG34, u4TVO, RG_LVDSA_TVO_3);
    }
    else
    {
        LVDSTVO=vDrvLVDSsettingDrivingCurrent(mode);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG3, LVDSTVO, RG_LVDSB_TVO_EVEN_D11);
        vIO32WriteFldAlign(REG_LVDS_TX_CFG4, LVDSTVO, RG_LVDSB_TVO_ODD_D11);
    }
}

void LVDS_APSRC_Set(UINT8 mode)
{
    vIO32WriteFldAlign(REG_LVDS_TX_CFG0,mode,RG_LVDSA_PSRC);
}

void LVDS_ANSRC_Set(UINT8 mode)
{
    vIO32WriteFldAlign(REG_LVDS_TX_CFG0,mode,RG_LVDSA_NSRC);
}

void LVDS_PADPD_Set(UINT8 mode)
{
    if(mode == LVDS_PWDOWN)
    {
        if (IS_IC_5895())
        {   
            vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0x0, RG_LVDSA_EXT_EN);
        }
        else
        {
            //vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0x0, RG_LVDSB_EXT_EN_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(0, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(0,RG_LVDSB_DRV_EN_ODD_D11));
        }
    }
    else
    {
        if (IS_IC_5895())
        {
            vIO32WriteFldAlign(REG_LVDS_TX_CFG13, 0xFFFFFFFF, RG_LVDSA_EXT_EN);
        }
        else
        {
            //vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0xFFF, RG_LVDSB_EXT_EN_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(0xFFF, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(0xFFF,RG_LVDSB_DRV_EN_ODD_D11));
        }
    }
}

void vDrvVCOPhase_SEL(UINT8 u1Phase)
{
	//vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG2, u1Phase, REG_LVDSTX_PLL_CFG2_VPLL_DLY_DATA);  // Capy remove
}

void SET_DDDSFreeRun(void)
{
	vIO32WriteFldAlign(DDDS_00,0,DDDS_00_DISP_EN);
	vIO32WriteFldAlign(DDDS_00,0,DDDS_00_FIX_FS_DDDS_SEL);			
	vIO32WriteFldAlign(DDDS_03,0,DDDS_03_SPREAD_INIT);
	vIO32WriteFldAlign(DDDS_03,1,DDDS_03_SPREAD_INIT);
	vIO32WriteFldAlign(DDDS_03,0,DDDS_03_SPREAD_INIT);						
}

void vSetPI_GROUP(UINT8 u1AEven, UINT8 u1AODD, UINT8 u1BEven, UINT8 u1BOdd)
{
	vIO32WriteFldAlign(LVDSB_REG08, u1AEven, RG_A_SW);
	vIO32WriteFldAlign(LVDSB_REG08, u1AODD,  RG_B_SW);
	vIO32WriteFldAlign(LVDSB_REG08, u1BEven, RG_C_SW);
	vIO32WriteFldAlign(LVDSB_REG08, u1BOdd,  RG_D_SW);
}

UINT8 vGetPI_GROUP(UINT8 u1Link)
{
	if(u1Link == 0)
		return (IO32ReadFldAlign(LVDSB_REG08, RG_A_SW));
	else if(u1Link == 1)
		return (IO32ReadFldAlign(LVDSB_REG08, RG_B_SW));
	else if(u1Link == 2) 
		return (IO32ReadFldAlign(LVDSB_REG08, RG_C_SW));
	else if(u1Link == 3)  
		return (IO32ReadFldAlign(LVDSB_REG08, RG_D_SW));
	else
		return 0;
}

void vSet_DataPair(UINT8 u1Port,UINT8 u1Pair, UINT8 u1Value)
{
    if(u1Port)  // Port B
    {
		if(u1Pair == 0)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV0_SEL);
		else if(u1Pair == 1)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV1_SEL);
		else if(u1Pair == 2)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV2_SEL);
		else if(u1Pair == 3)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV3_SEL);
		else if(u1Pair == 4)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV4_SEL);
		else if(u1Pair == 5)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV5_SEL);	
		else if(u1Pair == 6)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV6_SEL);		
		else if(u1Pair == 7)
			vIO32WriteFldAlign(LVDSB_REG03, u1Value, RG_RLV7_SEL);		
		else if(u1Pair == 8)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_RLV8_SEL);			
		else if(u1Pair == 9)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_RLV9_SEL);			
		else if(u1Pair == 0x0A)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_RLV_CK0_SEL);			
		else if(u1Pair == 0x0B)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_RLV_CK1_SEL);		
    }
	else  // Port A
	{
		if(u1Pair == 0)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV0_SEL);
		else if(u1Pair == 1)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV1_SEL);
		else if(u1Pair == 2)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV2_SEL);
		else if(u1Pair == 3)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV3_SEL);
		else if(u1Pair == 4)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV4_SEL);
		else if(u1Pair == 5)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV5_SEL);	
		else if(u1Pair == 6)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV6_SEL);		
		else if(u1Pair == 7)
			vIO32WriteFldAlign(LVDSB_REG02, u1Value, RG_LLV7_SEL);		
		else if(u1Pair == 8)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_LLV8_SEL);			
		else if(u1Pair == 9)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_LLV9_SEL);			
		else if(u1Pair == 0x0A)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_LLV_CK0_SEL);			
		else if(u1Pair == 0x0B)
			vIO32WriteFldAlign(LVDSB_REG04, u1Value, RG_LLV_CK1_SEL);		
	}
}

UINT8 u1Get_DataPair(UINT8 u1Port,UINT8 u1Pair)
{
    if(u1Port)  // Port B
    {
		if(u1Pair == 0x00)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV0_SEL));
		else if(u1Pair == 0x01)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV1_SEL));
		else if(u1Pair == 0x02)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV2_SEL));
		else if(u1Pair == 0x03)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV3_SEL));
		else if(u1Pair == 0x04)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV4_SEL));
		else if(u1Pair == 0x05)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV5_SEL));	
		else if(u1Pair == 0x06)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV6_SEL));		
		else if(u1Pair == 0x07)
			return (IO32ReadFldAlign(LVDSB_REG03, RG_RLV7_SEL));		
		else if(u1Pair == 0x08)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_RLV8_SEL));			
		else if(u1Pair == 0x09)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_RLV9_SEL));			
		else if(u1Pair == 0x0A)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_RLV_CK0_SEL));			
		else if(u1Pair == 0x0B)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_RLV_CK1_SEL));			
		else
			return 0;		
    }
	else  // Port A
	{
		if(u1Pair == 0x00)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV0_SEL));
		else if(u1Pair == 0x01)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV1_SEL));
		else if(u1Pair == 0x02)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV2_SEL));
		else if(u1Pair == 0x03)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV3_SEL));
		else if(u1Pair == 0x04)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV4_SEL));
		else if(u1Pair == 0x05)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV5_SEL));	
		else if(u1Pair == 0x06)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV6_SEL));		
		else if(u1Pair == 0x07)
			return (IO32ReadFldAlign(LVDSB_REG02, RG_LLV7_SEL));		
		else if(u1Pair == 0x08)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_LLV8_SEL));			
		else if(u1Pair == 0x09)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_LLV9_SEL));	
		else if(u1Pair == 0x0A)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_LLV_CK0_SEL));			
		else if(u1Pair == 0x0B)
			return (IO32ReadFldAlign(LVDSB_REG04, RG_LLV_CK1_SEL));			
		else
			return 0;
	}
}	

void vDrvLVDSToGPIOInit(UINT8 u1OnOff)
{
    UINT32 u4Val_tmp=0;
    UINT8 EPD=0;
    UINT8 OPD=0;
    UINT64 *pu4GPOMap;
    UINT64 *pu4GPIMap;

    if (!IS_IC_5895())  // only for Port B
    {
        if (u1OnOff)  // Set to GPO/I
        {
            pu4GPOMap = (UINT64*)DRVCUST_PanelGetUptr(ePanelLVDSToGPO);
            pu4GPIMap = (UINT64*)DRVCUST_PanelGetUptr(ePanelLVDSToGPI);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld(0x0, RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld(0x0,RG_LVDSB_DRV_EN_ODD_D11));
            vIO32WriteFldMulti(REG_LVDS_TX_CFG7,P_Fld(pu4GPOMap[27], RG_LVDSB_ECKP_GPO_EN)|P_Fld(pu4GPOMap[26], RG_LVDSB_ECKN_GPO_EN)|P_Fld(pu4GPOMap[25], RG_LVDSB_E5P_GPO_EN)|P_Fld(pu4GPOMap[24], RG_LVDSB_E5N_GPO_EN)
                                                |P_Fld(pu4GPOMap[23], RG_LVDSB_E4P_GPO_EN)|P_Fld(pu4GPOMap[22], RG_LVDSB_E4N_GPO_EN)|P_Fld(pu4GPOMap[21], RG_LVDSB_E3P_GPO_EN)|P_Fld(pu4GPOMap[20], RG_LVDSB_E3N_GPO_EN)
                                                |P_Fld(pu4GPOMap[19], RG_LVDSB_E2P_GPO_EN)|P_Fld(pu4GPOMap[18], RG_LVDSB_E2N_GPO_EN)|P_Fld(pu4GPOMap[17], RG_LVDSB_E1P_GPO_EN)|P_Fld(pu4GPOMap[16], RG_LVDSB_E1N_GPO_EN)
                                                |P_Fld(pu4GPOMap[15], RG_LVDSB_E0P_GPO_EN)|P_Fld(pu4GPOMap[14], RG_LVDSB_E0N_GPO_EN)
                                                |P_Fld(pu4GPOMap[13], RG_LVDSB_OCKP_GPO_EN)|P_Fld(pu4GPOMap[12], RG_LVDSB_OCKN_GPO_EN)|P_Fld(pu4GPOMap[11], RG_LVDSB_O5P_GPO_EN)|P_Fld(pu4GPOMap[10], RG_LVDSB_O5N_GPO_EN)
                                                |P_Fld(pu4GPOMap[9], RG_LVDSB_O4P_GPO_EN)|P_Fld(pu4GPOMap[8], RG_LVDSB_O4N_GPO_EN)|P_Fld(pu4GPOMap[7], RG_LVDSB_O3P_GPO_EN)|P_Fld(pu4GPOMap[6], RG_LVDSB_O3N_GPO_EN)
                                                |P_Fld(pu4GPOMap[5], RG_LVDSB_O2P_GPO_EN)|P_Fld(pu4GPOMap[4], RG_LVDSB_O2N_GPO_EN)|P_Fld(pu4GPOMap[3], RG_LVDSB_O1P_GPO_EN)|P_Fld(pu4GPOMap[2], RG_LVDSB_O1N_GPO_EN)
                                                |P_Fld(pu4GPOMap[1], RG_LVDSB_O0P_GPO_EN)|P_Fld(pu4GPOMap[0], RG_LVDSB_O0N_GPO_EN));
            vIO32WriteFldMulti(REG_LVDS_TX_CFG8,P_Fld(pu4GPIMap[27], RG_LVDSB_ECKP_GPI_EN)|P_Fld(pu4GPIMap[26], RG_LVDSB_ECKN_GPI_EN)|P_Fld(pu4GPIMap[25], RG_LVDSB_E5P_GPI_EN)|P_Fld(pu4GPIMap[24], RG_LVDSB_E5N_GPI_EN)
                                                |P_Fld(pu4GPIMap[23], RG_LVDSB_E4P_GPI_EN)|P_Fld(pu4GPIMap[22], RG_LVDSB_E4N_GPI_EN)|P_Fld(pu4GPIMap[21], RG_LVDSB_E3P_GPI_EN)|P_Fld(pu4GPIMap[20], RG_LVDSB_E3N_GPI_EN)
                                                |P_Fld(pu4GPIMap[19], RG_LVDSB_E2P_GPI_EN)|P_Fld(pu4GPIMap[18], RG_LVDSB_E2N_GPI_EN)|P_Fld(pu4GPIMap[17], RG_LVDSB_E1P_GPI_EN)|P_Fld(pu4GPIMap[16], RG_LVDSB_E1N_GPI_EN)
                                                |P_Fld(pu4GPIMap[15], RG_LVDSB_E0P_GPI_EN)|P_Fld(pu4GPIMap[14], RG_LVDSB_E0N_GPI_EN)
                                                |P_Fld(pu4GPIMap[13], RG_LVDSB_OCKP_GPI_EN)|P_Fld(pu4GPIMap[12], RG_LVDSB_OCKN_GPI_EN)|P_Fld(pu4GPIMap[11], RG_LVDSB_O5P_GPI_EN)|P_Fld(pu4GPIMap[10], RG_LVDSB_O5N_GPI_EN)
                                                |P_Fld(pu4GPIMap[9], RG_LVDSB_O4P_GPI_EN)|P_Fld(pu4GPIMap[8], RG_LVDSB_O4N_GPI_EN)|P_Fld(pu4GPIMap[7], RG_LVDSB_O3P_GPI_EN)|P_Fld(pu4GPIMap[6], RG_LVDSB_O3N_GPI_EN)
                                                |P_Fld(pu4GPIMap[5], RG_LVDSB_O2P_GPI_EN)|P_Fld(pu4GPIMap[4], RG_LVDSB_O2N_GPI_EN)|P_Fld(pu4GPIMap[3], RG_LVDSB_O1P_GPI_EN)|P_Fld(pu4GPIMap[2], RG_LVDSB_O1N_GPI_EN)
                                                |P_Fld(pu4GPIMap[1], RG_LVDSB_O0P_GPI_EN)|P_Fld(pu4GPIMap[0], RG_LVDSB_O0N_GPI_EN));
            vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0x1, RG_LVDSB_GPO_TURBO_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x0, RG_LVDSB_R_TERM_EVEN_D11)|P_Fld(0x0, RG_LVDSB_R_TERM_ODD_D11));
        }
        else  // resume to LVDS
        {
            u4Val_tmp= (_u4APadPDValue&0xFFF);
            
            EPD=(( u4Val_tmp>>6)|(((u4Val_tmp>>6)& 0x20)<<7))&0x5F;
            OPD=(( u4Val_tmp&0x3F)|(((u4Val_tmp&0x3F)& 0x20)<<7))&0x5F;
            vIO32WriteFldMulti(REG_LVDS_TX_CFG1, P_Fld((0x5F-EPD), RG_LVDSB_DRV_EN_EVEN_D11)|P_Fld((0x5F-OPD),RG_LVDSB_DRV_EN_ODD_D11));
            vIO32WriteFldMulti(REG_LVDS_TX_CFG7,P_Fld(0x0, RG_LVDSB_GPO_EN_EVEN_D11)|P_Fld(0x0, RG_LVDSB_GPO_EN_ODD_D11));
            vIO32WriteFldMulti(REG_LVDS_TX_CFG8,P_Fld(0x0, RG_LVDSB_GPI_EN_EVEN_D11)|P_Fld(0x0, RG_LVDSB_GPI_EN_ODD_D11));
            vIO32WriteFldAlign(REG_LVDS_TX_CFG0, 0x0, RG_LVDSB_GPO_TURBO_D11);
            vIO32WriteFldMulti(REG_LVDS_TX_CFG6, P_Fld(0x3FFF, RG_LVDSB_R_TERM_EVEN_D11)|P_Fld(0x3FFF, RG_LVDSB_R_TERM_ODD_D11));
        }
    }
    else
    {
        UNUSED(u4Val_tmp);
        UNUSED(EPD);
        UNUSED(OPD);
        UNUSED(pu4GPOMap);
        UNUSED(pu4GPIMap);
        LOG(0, "Do not have such function!!\n");
    }
}

void vDrvSetLVDSToGPI(UINT8 u1LVDSPin, UINT8 u1OnOff)
{
    UINT16 u2LVDSBO_GPI=0;
    UINT16 u2LVDSBE_GPI=0;
    UINT16 u2LVDSBO_GPO=0;
    UINT16 u2LVDSBE_GPO=0;

    if (!IS_IC_5895())  // only for Port B
    {
        u2LVDSBO_GPI = IO32ReadFldAlign(REG_LVDS_TX_CFG8, RG_LVDSB_GPI_EN_ODD_D11);
        u2LVDSBE_GPI = IO32ReadFldAlign(REG_LVDS_TX_CFG8, RG_LVDSB_GPI_EN_EVEN_D11);

        u2LVDSBO_GPO = IO32ReadFldAlign(REG_LVDS_TX_CFG7, RG_LVDSB_GPO_EN_ODD_D11);
        u2LVDSBE_GPO = IO32ReadFldAlign(REG_LVDS_TX_CFG7, RG_LVDSB_GPO_EN_EVEN_D11);

        if (u1OnOff)  // Set to GPI
        {
            if (u1LVDSPin >= 16)  // BE
            {
                u1LVDSPin -= 16;
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBE_GPI |= (1 << u1LVDSPin);

                u2LVDSBE_GPO ^= (1 << u1LVDSPin);
            }
            else  // BO
            {
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBO_GPI |= (1 << u1LVDSPin);

                u2LVDSBO_GPO ^= (1 << u1LVDSPin);
            }
        }
        else
        {
            if (u1LVDSPin >= 16)  // BE
            {
                u1LVDSPin -= 16;
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBE_GPI ^= (1 << u1LVDSPin);

                u2LVDSBE_GPO |= (1 << u1LVDSPin); 
            }
            else  // BO
            {
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBO_GPI ^= (1 << u1LVDSPin);

                u2LVDSBO_GPO |= (1 << u1LVDSPin);
            }
        }

        //LOG(0,"**[GPI]BO=0x%x, BE=0x%x\n", u2LVDSBO_GPI, u2LVDSBE_GPI);
        //LOG(0,"**[GPO]BO=0x%x, BE=0x%x\n", u2LVDSBO_GPO, u2LVDSBE_GPO);
        vIO32WriteFldMulti(REG_LVDS_TX_CFG8,P_Fld(u2LVDSBE_GPI, RG_LVDSB_GPI_EN_EVEN_D11)|P_Fld(u2LVDSBO_GPI, RG_LVDSB_GPI_EN_ODD_D11));
        vIO32WriteFldMulti(REG_LVDS_TX_CFG7,P_Fld(u2LVDSBE_GPO, RG_LVDSB_GPO_EN_EVEN_D11)|P_Fld(u2LVDSBO_GPO, RG_LVDSB_GPO_EN_ODD_D11));
    }
    else
    {
        UNUSED(u2LVDSBO_GPI);
        UNUSED(u2LVDSBE_GPI);
        UNUSED(u2LVDSBO_GPO);
        UNUSED(u2LVDSBE_GPO);
        LOG(0, "Do not have such function!!\n");
    }
}

void vDrvSetLVDSToGPO(UINT8 u1LVDSPin, UINT8 u1OnOff)
{
    UINT16 u2LVDSBO_GPI=0;
    UINT16 u2LVDSBE_GPI=0;
    UINT16 u2LVDSBO_GPO=0;
    UINT16 u2LVDSBE_GPO=0;

    if (!IS_IC_5895())  // only for Port B
    {
        u2LVDSBO_GPI = IO32ReadFldAlign(REG_LVDS_TX_CFG8, RG_LVDSB_GPI_EN_ODD_D11);
        u2LVDSBE_GPI = IO32ReadFldAlign(REG_LVDS_TX_CFG8, RG_LVDSB_GPI_EN_EVEN_D11);

        u2LVDSBO_GPO = IO32ReadFldAlign(REG_LVDS_TX_CFG7, RG_LVDSB_GPO_EN_ODD_D11);
        u2LVDSBE_GPO = IO32ReadFldAlign(REG_LVDS_TX_CFG7, RG_LVDSB_GPO_EN_EVEN_D11);

        if (u1OnOff)
        {
            if (u1LVDSPin >= 16)  // BE
            {
                u1LVDSPin -= 16;
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBE_GPO |= (1 << u1LVDSPin);

                u2LVDSBE_GPI ^= (1 << u1LVDSPin);
            }
            else  // BO
            {
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBO_GPO |= (1 << u1LVDSPin);

                u2LVDSBO_GPI ^= (1 << u1LVDSPin);
            }
        }
        else
        {
            if (u1LVDSPin >= 16)  // BE
            {
                u1LVDSPin -= 16;
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBE_GPO ^= (1 << u1LVDSPin);

                u2LVDSBE_GPI |= (1 << u1LVDSPin);
            }
            else  // BO
            {
                if (u1LVDSPin>13)
                {
                    u1LVDSPin=13;
                }
                u2LVDSBO_GPO ^= (1 << u1LVDSPin);

                u2LVDSBO_GPI |= (1 << u1LVDSPin);
            }
        }
        //LOG(0,"##[GPI]BO=0x%x, BE=0x%x\n", u2LVDSBO_GPI, u2LVDSBE_GPI);
        //LOG(0,"##[GPO]BO=0x%x, BE=0x%x\n", u2LVDSBO_GPO, u2LVDSBE_GPO);
        vIO32WriteFldMulti(REG_LVDS_TX_CFG8,P_Fld(u2LVDSBE_GPI, RG_LVDSB_GPI_EN_EVEN_D11)|P_Fld(u2LVDSBO_GPI, RG_LVDSB_GPI_EN_ODD_D11));
        vIO32WriteFldMulti(REG_LVDS_TX_CFG7,P_Fld(u2LVDSBE_GPO, RG_LVDSB_GPO_EN_EVEN_D11)|P_Fld(u2LVDSBO_GPO, RG_LVDSB_GPO_EN_ODD_D11));
    }
    else
    {
        UNUSED(u2LVDSBO_GPI);
        UNUSED(u2LVDSBE_GPI);
        UNUSED(u2LVDSBO_GPO);
        UNUSED(u2LVDSBE_GPO);
        LOG(0, "Do not have such function!!\n");
    }
}

void vDrvSetPanelIFRGBInOrder(UINT8 u1R,UINT8 u1G,UINT8 u1B)
{
	vIO32WriteFldMulti(VINTF_P0, P_Fld(u1B, VINTF_PANEL_IN_B_SWAP)|P_Fld(u1G,VINTF_PANEL_IN_G_SWAP)|P_Fld(u1R,VINTF_PANEL_IN_R_SWAP));
}

 /**

 * @brief vDrvSetVOPLLClockSchema
 *
 * Config VOPLL clock schema
 *
 * @param  u1ClockSrc: vopll clock source 
 * @retval void 
 */
void vDrvSetVOPLLClockSchema(UINT8 u1ClockSrc)
{
    LOG(3,"vDrvSetVOPLLClockSchema, u1ClockSrc=%d.\n", u1ClockSrc);

    vIO32WriteFldAlign(CKGEN_REG_XDDS_CKCFG, u1ClockSrc, FLD_XDDS_SEL);
    switch (u1ClockSrc)
    {
        case VOPLL_CLK_SRC_DDDS:
            if (IS_IC_5895())
            {
                if (IS_PANEL_L12R12)
                    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 1, FLD_OCLK_DIV2_SEL);//for SS-XGA 240P and SP-PDP panel, set /2
                else
                    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 0, FLD_OCLK_DIV2_SEL);
            }

            if (LVDS_OUTPUT_PORT == SINGLE_PORT)
            {
                if (IS_IC_5895())
                {
                    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 0, FLD_OCLK_DIV_SEL);
                }
                else
                {
                    //   vIO32WriteFldAlign(CKGEN_VPCLK_CFG, 0, FLD_OCLK_DIV_SEL);
                }
            }
            else if (LVDS_OUTPUT_PORT == DUAL_PORT)
            {
                if (IS_IC_5895())
                {
                    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 1, FLD_OCLK_DIV_SEL);
                }
                else
                {
                    //   vIO32WriteFldAlign(CKGEN_VPCLK_CFG, 1, FLD_OCLK_DIV_SEL);
                }
            }
            else // FOUR_PORT 
            {
                #if 0
                if(IS_LVDS_HIGHSPEED)
                {
                vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2, FLD_OCLK_DIV_SEL);
                }
                else
                #endif
                {
                    if (IS_IC_5895())
                    {
                        vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2, FLD_OCLK_DIV_SEL);
                    }
                }
            }
            break;

        case VOPLL_CLK_SRC_XTAL:
            break;

        default:	
            break;
    }

    if (PANEL_IsSupport4K2KDD()) // all for 4KDD
    {
        LOG(0, "@vDrvSetVOPLLClockSchema -->4KDD\n");
        vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1, FLD_OCLK_DIV3_SEL);
        vIO32WriteFldAlign(TMGR4, 1, TMGR4_1CHTO2CH_EN);
    }
    else
    {
        LOG(0, "@vDrvSetVOPLLClockSchema -->normal\n");
		if (IS_DISP_4K2K_TYPE3 && (GetPANEL4K2K_TYPE() == PANEL_TYPE_2K1K_120))  // For 4K2K60 OSD
		{
			vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1, FLD_OCLK_DIV3_SEL);  // 600/2, for 2ch clk
		}
		else
		{
	        vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0, FLD_OCLK_DIV3_SEL);
		}
        vIO32WriteFldAlign(TMGR4, 0, TMGR4_1CHTO2CH_EN);
    }
}

#ifdef LVDS_ESD_PROTECT
void vLVDSFIFOSkew_Recover(void)
{
	if(!IS_VBYONE)
	{
		#ifdef CC_MTK_LOADER	
	    vDrvLVDS7To4FifoEnable();   
		#else
		_Lvds7To4FifoReset = 2;	
		#endif
		
		#ifndef NDEBUG
	    LOG(0, " ======= [ESD] vLVDSFIFOSkew_Recover ===========\n");
		#endif
	}
}

UINT32 u1ChkLVDSFIFOSkewStatus(void)
{
	UINT16 u1WrPtr = 0;
	UINT16 u1RdPtr = 0;
	UINT32 u2FIFOSkw=0;
	u1WrPtr = IO32ReadFldAlign(LVDSB_REG22, LVDSTX_FIFO_WRPTR_ESD);
	u1RdPtr = IO32ReadFldAlign(LVDSB_REG22, LVDSTX_FIFO_RDPTR_ESD);
	
	u2FIFOSkw = (u1WrPtr<<8) | u1RdPtr ;
	return (u2FIFOSkw);
}

BOOL fgIsLVDSFIFOSkew(void)
{
	static UINT8 u1FifoSkewCnt = 0;
	UINT32 u2Skew = 0;
	BOOL fgFifoSkwRecover = FALSE;

	u2Skew = u1ChkLVDSFIFOSkewStatus();
	LOG(6, "[LVDS] FIFO skew = 0x%x\n",u2Skew);
	if(u2LVDSFifoSkwPre != u2Skew)
	{
		LOG(6, "[LVDS] FIFO pre skew: 0x%x\n", u2LVDSFifoSkwPre);
		u1FifoSkewCnt ++;
		LOG(6, "[LVDS] FIFO CNT: %d\n", u1FifoSkewCnt);
		if (u1FifoSkewCnt > 2)
		{
			u1FifoSkewCnt = 0;
			fgFifoSkwRecover = TRUE;
		}
	}
	else
	{
		u1FifoSkewCnt = 0;
		fgFifoSkwRecover = FALSE;
	}
	u2LVDSFifoSkwPre = u2Skew;
	LOG(6, "[LVDS] FIFO Recover Flag: %d\n", fgFifoSkwRecover);
	
	return (fgFifoSkwRecover);	
}
#endif

