/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/


/*****************************************************************************
 *
 * Filename:
 * ---------
 *   $Workfile: hw_vdoin.c $
 *
 * Project:
 * --------
 *   MT8205
 *
 * Description:
 * ------------
 *   VDOIN module register access function
 *
 * Author:
 * -------
 *
 * Last changed:
 * -------------
 * $Author: brianpc.huang $
 *
 * $Modtime: 04/05/17 6:55p $
 *
 * $Revision: #1 $
 ****************************************************************************/

#include "general.h"
#include "hw_vdoin.h"
#include "x_assert.h"

#ifdef CC_MTK_LOADER
#undef x_crit_start
#undef x_crit_end
#define x_crit_start() 0
#define x_crit_end(x) UNUSED(x)
#define CRIT_STATE_T UINT8
#else
#include "x_os.h"
#include "sif_if.h"
#endif


/* Reg READ/WRITE function, for 82xx & 53xx
 *    provide for VDOON(16bit,2002xxxx)register
 *    82xx only has 16 bit address space.
 */
#if REG_ALL_FUNC
UINT8 u1RegRead1B(UINT16 reg16)
{
    return (*(volatile UINT8 *)(VDOIN_BASE+reg16));
}
UINT32 u4RegRead4B(UINT16 reg16)
{
    ASSERT((reg16&3)==0);
    return (*(volatile UINT32 *)(VDOIN_BASE+reg16));
}
void vRegWrite4B(UINT16 reg16, UINT32 val32)
{
    ASSERT((reg16&3)==0);
    (*(volatile UINT32 *)(VDOIN_BASE+reg16)=(val32));
}
void vRegWrite4B_S(UINT16 reg16, UINT8 bByte3, UINT8 bByte2, UINT8 bByte1, UINT8 bByte0)
{
    ASSERT((reg16&3)==0);
    (*(volatile UINT32 *)(VDOIN_BASE+reg16)=((bByte3<<24)|(bByte2<<16)|(bByte1<<8)|(bByte0)));
}
#endif

#if  0 //REG_2LAYER_IO32
UINT16 u2RegRead2B(UINT16 reg16)
{
    return u2IO32Read2B(VDOIN_BASE+reg16);
}
void vRegWrite1B(UINT16 reg16,/*UINT8*/UINT32 val8)
{
    vIO32Write1B(VDOIN_BASE+reg16,val8);
}
void vRegWrite1BMsk(UINT16 reg16,/*UINT8*/UINT32 val8,UINT8 msk8)
{
    vIO32Write1BMsk(VDOIN_BASE+reg16,val8,msk8);
}
void vRegWrite2B(UINT16 reg16, /*UINT16*/UINT32 val16)
{
    vIO32Write2B(VDOIN_BASE+reg16,val16);
}
void vRegWrite2BMsk(UINT16 reg16, /*UINT16*/UINT32 val16, UINT16 msk16)
{
    vIO32Write2BMsk(VDOIN_BASE+reg16,val16,msk16);
}

void vRegWrite4BMsk(UINT16 reg16, UINT32 val32, UINT32 msk32)
{
    vIO32Write4BMsk(VDOIN_BASE+reg16,val32,msk32);
}
#endif


#ifndef CC_MTK_LOADER
// ======================================================================
/* MT8297 READ/WRITE function, only for 539x later */

#ifdef CC_MT5391
UINT8 u1MT8297DirRead1B(UINT8 reg8)
{
    INT16 ret;
    VERIFY((ret=SIF_MT8297_DirectRead8(reg8))>=0);
    return (UINT8)(ret&0xff);
}

UINT8 u1MT8297InDRead1B(UINT8 reg8)
{
    INT16 ret;
    VERIFY((ret=SIF_MT8297_InDirectRead8(reg8))>=0);
    return (UINT8)(ret&0xff);
}

void vMT8297DirWrite1B(UINT8 reg8, UINT8 val8)
{
    VERIFY(SIF_MT8297_DirectWrite8(reg8, val8)==0);
}

void vMT8297InDWrite1B(UINT8 reg8,UINT8 val8)
{
    VERIFY(SIF_MT8297_InDirectWrite8(reg8, val8)==0);
}

void vMT8297DirWrite1BMsk(UINT8 reg8,UINT8 val8,UINT8 msk8)
{
    VERIFY(SIF_MT8297_DirectWrite8Msk(reg8, val8,msk8)==0);
}

void vMT8297InDWrite1BMsk(UINT8 reg8,UINT8 val8,UINT8 msk8)
{
    VERIFY(SIF_MT8297_InDirectWrite8Msk(reg8, val8,msk8)==0);
}
#endif
#endif
