/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: sif_diag.c,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file sif_diag.c
 *  SIF module hareware test function.
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "sif_if.h"
#include "drvcust_if.h"

#define DEFINE_IS_LOG      SIF_IsLog
#include "x_debug.h"
#include "x_drv_cli.h"
#include "x_assert.h"
#include "x_bim.h"
#include "x_os.h"

//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------

//-------------------------------------------------------------------------
/** _TestSIFMaster
 *  SIF module master test function.
 *  @param  VOID
 *  @retval   0	    Success
 *  @retval  -1	    Fail
 */
//-------------------------------------------------------------------------
static INT32 _TestSIFMaster(VOID)
{
    UINT8 u1Value[0x20] = {0};
    UINT16 u2Ret;
    UINT32 u4Bus, u4Addr, u4Size, u4Div;

    DRVCUST_OptQuery(eSystemEepromBus, &u4Bus);
    DRVCUST_OptQuery(eSystemEepromAddress, &u4Addr);
    DRVCUST_OptQuery(eSystemEepromSize, &u4Size);
    DRVCUST_OptQuery(eSystemEepromClkDiv, &u4Div);

    if (u4Size <= 0x800)
    {
        u2Ret = SIF_X_Read((UINT8)u4Bus, (UINT16)u4Div, (UINT8)u4Addr, 1, 0, u1Value, 0x20);
    }
    else
    {
        u2Ret = SIF_X_Write((UINT8)u4Bus, (UINT16)u4Div, (UINT8)u4Addr, 0, 0, u1Value, 2);
        if (u2Ret != 2)
        {
            LOG(1, "I2C Master %d test fail.\n", u4Bus);
            return -1;
        }
        u2Ret = SIF_X_Read((UINT8)u4Bus, (UINT16)u4Div, (UINT8)u4Addr, 0, 0, u1Value, 0x20);
    }

    if (u2Ret != 0x20)
    {
        LOG(1, "I2C Master %d test fail.\n", u4Bus);
        return -1;
    }

    return 0;
}


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------
//-------------------------------------------------------------------------
/** _SIF_Diag
 *  SIF module test entry function.
 *  @param  VOID
 *  @retval   0	    Success
 *  @retval  -1		Fail
 */
//-------------------------------------------------------------------------
INT32 _SIF_Diag(VOID)
{
    SIF_Init();

    if (_TestSIFMaster() < 0)
    {
        LOG(0, "FAIL\n");
        return -1;
    }

    LOG(0, "OK\n");

    return 0;
}


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------
