/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date$
 * $RCSfile: sif_if.c,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/

/** @file sif_if.c
 *  This file implement serial interface SIF function.
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------
#include "sif_debug.h"
#include "sif_hw.h"
#include "sif_isr.h"
#include "sif_if.h"

#include "drvcust_if.h"
#include "x_assert.h"
#include "x_bim.h"
#include "x_os.h"
#include "x_pinmux.h"
#include "x_hal_arm.h"
#include "x_printf.h"
#include "x_hal_5381.h"

#include "x_pdwnc.h"

#ifdef CONFIG_I2C_MT53XX
#include <linux/i2c.h>
#include <asm/atomic.h>
#include <linux/types.h>

#endif



//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------
/// Use OSAI function option
#if defined (CC_MTK_LOADER)
#define SIF_USE_OSAI            0
#else // defined (CC_MTK_LOADER)
#define SIF_USE_OSAI            1
#endif // defined (CC_MTK_LOADER)

//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------
#define  SIF_OK    (0)


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Global Variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------
EXTERN SIF_STRUCT_T* SIF_M0_GetStruct(void);
EXTERN SIF_STRUCT_T* SIF_M1_GetStruct(void);
EXTERN SIF_STRUCT_T* SIF_M2_GetStruct(void);
EXTERN SIF_STRUCT_T* SIF_PDM0_GetStruct(void);
EXTERN SIF_STRUCT_T* SIF_PDM1_GetStruct(void);
EXTERN SIF_STRUCT_T* SIF_SECM0_GetStruct(void);
//#endif

//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------

static BOOL _fgInit = FALSE;


static PFN_SIF_CFG_PIN_AND_MAP_FUNC _pfnSifCfgPinSetFunc = NULL;


//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------



#if defined(CC_FAST_INIT)

//LH for suspend/resume
void SIF_pm_suspend(void)
{
}
void SIF_pm_resume(void)
{

}
#endif

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------
//-------------------------------------------------------------------------
/** SIF_Init
 *  Initialize SIF module.
 *  @param  VOID
 *  @return  VOID
 */
//-------------------------------------------------------------------------
VOID SIF_Init(VOID)
{ 	
	UINT32 i;

    if (_fgInit)
    {
        return;
    }
    if (DRVCUST_InitQueryUptr(eSifCfgPinSetFunc, (UPTR*)(void*)&_pfnSifCfgPinSetFunc) != 0)
    {
    		_pfnSifCfgPinSetFunc = NULL;
			LOG(0,"_pfnSifCfgPinSetFunc should not be NULL\n");
			ASSERT(0);
    }
	for(i=0;i<SIF_MAX_ALL_ICS_MASTER;i++)
	{

#ifndef CONFIG_I2C_MT53XX
		if(_pfnSifCfgPinSetFunc(i) > -1)
			UNUSED(SIF_HwInit(i));
#endif
	}
	_fgInit = TRUE;

}


//-------------------------------------------------------------------------
/** SIF_Stop
 *  terminate SIF module.
 *  @param  VOID
 *  @return  VOID
 */
//-------------------------------------------------------------------------
#ifdef CC_CLI
VOID SIF_Stop(VOID)
{

    UINT32 i;
    if (_fgInit)
	{
		for(i=0;i<SIF_MAX_ALL_ICS_MASTER;i++)
		{
			if(_pfnSifCfgPinSetFunc(i) > -1)
				UNUSED(SIF_Master_Stop(i));
		}
	}
		
}
#endif

UINT16 SIF_Read(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddr,
                UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;

    u2Ret = SIF_X_Read(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, 1, u1WordAddr,
                       pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_ReadNoSubAddr
 *  Read data without word address from SIF module from SIF_BUS_SYSTEM_I2C.
 *  @param  u2ClkDiv    Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr   Serial interface device address.
 *  @param  pu1Buf      Pointer to user's buffer.
 *  @param  u2ByteCnt   Number of byte to read. Max = 256 bytes.
 *  @retval > 0         Actual number of bytes read.
 *  @retval = 0         Error
 */
UINT16 SIF_ReadNoSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8* pu1Buf,
                         UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Read(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, 0, 0,
                       pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_TunerReadNoSubAddr
 *  NIM module api: read data without word address from SIF module from SIF_BUS_TUNER_I2C.
 *  @param  u2ClkDiv    Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr   Serial interface device address.
 *  @param  pu1Buf      Pointer to user's buffer.
 *  @param  u2ByteCnt   Number of byte to read. Max = 256 bytes.
 *  @retval > 0         Actual number of bytes read.
 *  @retval = 0         Error
 */
UINT16 SIF_TunerReadNoSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8* pu1Buf,
                              UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Read(SIF_BUS_TUNER_I2C, u2ClkDiv, u1DevAddr, 0, 0,
                       pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_Write
 *  Write data with word address to SIF module on SIF_BUS_SYSTEM_I2C.
 *  @param  u2ClkDiv    Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr   Serial interface device address.
 *  @param  u1WordAddr  Serial interface word address.
 *  @param  pu1Buf      Pointer to user's buffer.
 *  @param  u2ByteCnt   Number of byte to write. Max = 256 bytes.
 *  @retval > 0         Actual number of bytes written.
 *  @retval = 0         Error
 */
UINT16 SIF_Write(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddr,
                 UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Write(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, 1, u1WordAddr,
                        pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_WriteNoSubAddr
 *  Write data without word address to SIF module on SIF_BUS_SYSTEM_I2C.
 *  @param  u2ClkDiv    Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr   Serial interface device address.
 *  @param  pu1Buf      Pointer to user's buffer.
 *  @param  u2ByteCnt   Number of byte to write. Max = 256 bytes.
 *  @retval > 0         Actual number of bytes written.
 *  @retval = 0         Error
 */
UINT16 SIF_WriteNoSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8* pu1Buf,
                          UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Write(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, 0, 0,
                        pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_TunerWriteNoSubAddr
 *  NIM module api: write data without word address to SIF module on SIF_BUS_TUNER_I2C.
 *  @param  u2ClkDiv    Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr   Serial interface device address.
 *  @param  pu1Buf      Pointer to user's buffer.
 *  @param  u2ByteCnt   Number of byte to write. Max = 256 bytes.
 *  @retval > 0         Actual number of bytes written.
 *  @retval = 0         Error
 */
UINT16 SIF_TunerWriteNoSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8* pu1Buf,
                               UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Write(SIF_BUS_TUNER_I2C, u2ClkDiv, u1DevAddr, 0, 0,
                        pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_ReadMultipleSubAddress
 *  Read data with word address, of which length up to three, from SIF module from SIF_BUS_SYSTEM_I2C.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to read. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes read.
 *  @retval = 0             Error
 */
UINT16 SIF_ReadMultipleSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                               UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Read(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, u1WordAddrNum, u4WordAddr,
                       pu1Buf, u2ByteCnt);
    return u2Ret;
}


/** SIF_WriteMultipleSubAddress
 *  Write data with word address, of which length up to three, to SIF module on SIF_BUS_SYSTEM_I2C.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to read. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes written.
 *  @retval = 0             Error
 */
UINT16 SIF_WriteMultipleSubAddr(UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                                UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
    u2Ret = SIF_X_Write(SIF_BUS_SYSTEM_I2C, u2ClkDiv, u1DevAddr, u1WordAddrNum, u4WordAddr,
                        pu1Buf, u2ByteCnt);
    return u2Ret;
}

#ifdef CONFIG_I2C_MT53XX
/** SIF_Xfer_Read
 *  Read data with channel ID, word address, of which length up to three, from SIF module.
 *  @param  u1ChannelID     SIF bus ID.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to read. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes read.
 *  @retval = 0             Error
 */
UINT16 SIF_Xfer_Read(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                  UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    INT32 i4Ret = 0;
    UINT16 u2Ret = 0;
    UINT32 u4IsRead = 1;
    INT32 u1ChannelID = u1CHannelID;
    UINT8 buf1[sizeof(u2ClkDiv)+sizeof(u1WordAddrNum)+sizeof(u4WordAddr)];
    struct i2c_msg xfer_msg[2];
    struct i2c_adapter *i2c_adapt;
#ifdef CC_FIXED_SIF_CLK_DIV
    UINT32 u4FixClkDiv;
    u4FixClkDiv = DRVCUST_InitGet(eSifcfgFixClkDiv);
    u2ClkDiv = u4FixClkDiv;
#endif
    x_memcpy(buf1,&u2ClkDiv,sizeof(u2ClkDiv));//2B clock div
    buf1[sizeof(u2ClkDiv)]=u1WordAddrNum;//addr num
    x_memcpy(buf1+sizeof(u2ClkDiv)+sizeof(u1WordAddrNum),&u4WordAddr,sizeof(u4WordAddr));//4B addr

    xfer_msg[0].addr=u1DevAddr;
    xfer_msg[0].len=sizeof(buf1);
    xfer_msg[0].buf=buf1;
    xfer_msg[0].flags=u4IsRead;
    xfer_msg[1].addr=u1DevAddr;
    xfer_msg[1].len=u2ByteCnt;
    xfer_msg[1].buf=pu1Buf;
    xfer_msg[1].flags=u4IsRead;

    i2c_adapt = i2c_get_adapter(u1ChannelID);
    if(!i2c_adapt)
    {
        LOG(1,"can not get adapter for channel %d\n",u1CHannelID);
        goto X_READ_ERROR;
    }
    else
    {
        i4Ret = i2c_transfer(i2c_adapt,xfer_msg,sizeof(xfer_msg)/sizeof(xfer_msg[0]));
        if(i4Ret<0)
        {
            LOG(1,"i2c adap%d transfer fails, act:%s, addr:0x%x, error=%d\n",u1CHannelID, u4IsRead?"read":"write",u1DevAddr,i4Ret);
            u2Ret = 0;
        }
        else
        {
            u2Ret = i4Ret;
        }
        i2c_put_adapter(i2c_adapt);
    }



    if (u2Ret != u2ByteCnt)
    {
        LOG(1, "[SIF][Read_Error]Channle(%d)Device_Addr(0x%x) want read(%d) bytes. Actually read(%d) bytes\n", u1CHannelID, u1DevAddr, u2ByteCnt, u2Ret);
    }
	

X_READ_ERROR:
    return u2Ret;

}

/** SIF_Xfer_Write
 *  Write data with channel ID, word address, of which length up to three, to SIF module.
 *  @param  u1ChannelID     SIF bus ID.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to write. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes written.
 *  @retval = 0             Error
 */
UINT16 SIF_Xfer_Write(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                   UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    INT32 i4Ret = 0;
    UINT16 u2Ret = 0;
    UINT32 u4IsRead = 0;
    INT32 u1ChannelID = u1CHannelID;
    UINT8 buf1[sizeof(u2ClkDiv)+sizeof(u1WordAddrNum)+sizeof(u4WordAddr)];
    struct i2c_msg xfer_msg[2];
    struct i2c_adapter *i2c_adapt;
#ifdef CC_FIXED_SIF_CLK_DIV
    UINT32 u4FixClkDiv;
    u4FixClkDiv = DRVCUST_InitGet(eSifcfgFixClkDiv);
    u2ClkDiv = u4FixClkDiv;
#endif

    x_memcpy(buf1,&u2ClkDiv,sizeof(u2ClkDiv));//2B clock div
    buf1[sizeof(u2ClkDiv)]=u1WordAddrNum;//addr num
    x_memcpy(buf1+sizeof(u2ClkDiv)+sizeof(u1WordAddrNum),&u4WordAddr,sizeof(u4WordAddr));//4B addr

    xfer_msg[0].addr=u1DevAddr;
    xfer_msg[0].len=sizeof(buf1);
    xfer_msg[0].buf=buf1;
    xfer_msg[0].flags=u4IsRead;
    xfer_msg[1].addr=u1DevAddr;
    xfer_msg[1].len=u2ByteCnt;
    xfer_msg[1].buf=pu1Buf;
    xfer_msg[1].flags=u4IsRead;

    i2c_adapt = i2c_get_adapter(u1ChannelID);
    if(!i2c_adapt)
    {
        LOG(1,"can not get adapter for channel %d\n",u1CHannelID);
        goto X_WRITE_ERROR;
    }
    else
    {
        i4Ret = i2c_transfer(i2c_adapt,xfer_msg,sizeof(xfer_msg)/sizeof(xfer_msg[0]));
        if(i4Ret<0)
        {
            LOG(1,"i2c adap%d transfer fails, act:%s, addr:0x%x, error=%d\n",u1CHannelID, u4IsRead?"read":"write",u1DevAddr,i4Ret);
            u2Ret = 0;
        }
        else
        {
            u2Ret = i4Ret;
        }
        i2c_put_adapter(i2c_adapt);
    }
	
    if (u2Ret != u2ByteCnt)
    {
        LOG(1, "[SIF][Write_Error]Channle(%d)Device_Addr(0x%x) want write(%d) bytes. Actually write(%d) bytes\n", u1CHannelID, u1DevAddr, u2ByteCnt, u2Ret);
    }
X_WRITE_ERROR:
	return u2Ret;
	
}
extern bool mtk_i2c_is_suspended(struct i2c_adapter *adap);
BOOL SIF_IS_SUSPENDED(UINT8 u1CHannelID)
{
    INT32 u1ChannelID = u1CHannelID;
    struct i2c_adapter *i2c_adapt;
    i2c_adapt = i2c_get_adapter(u1ChannelID);
	if(i2c_adapt == NULL)
	{
       Printf("SIF_IS_SUSPENDED is a NULL.\n");
	   return 0;
	}
    return mtk_i2c_is_suspended(i2c_adapt);
}
#endif

/** SIF_X_Read
 *  Read data with channel ID, word address, of which length up to three, from SIF module.
 *  @param  u1ChannelID     SIF bus ID.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to read. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes read.
 *  @retval = 0             Error
 */
UINT16 SIF_X_Read(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                  UINT8* pu1Buf, UINT16 u2ByteCnt)
{
	UINT16 u2Ret = 0;
	INT8 u1TmpID = -1;
//to make iic driver common, we make a drvcust mapping for every IC  

	SIF_Init ();//make sure sif init

	if(_pfnSifCfgPinSetFunc)
	{
		u1TmpID =_pfnSifCfgPinSetFunc(u1CHannelID);
		if(u1TmpID < 0)
		{
			Printf("invaild channel id \n");
			return u2Ret;
		}
	}
	else
	{
		Printf("_pfnSifCfgPinSetFunc must not be NULL\n");
		ASSERT(0);
		return u2Ret;
	}

#ifndef CONFIG_I2C_MT53XX
	u2Ret = SIF_RW(u2ClkDiv, u1TmpID, u1DevAddr, u4WordAddr,u1WordAddrNum, SIF_R, pu1Buf, u2ByteCnt);
#else
    u2Ret = SIF_Xfer_Read(u1TmpID,u2ClkDiv,u1DevAddr,u1WordAddrNum, u4WordAddr,pu1Buf,u2ByteCnt);
#endif

    if (u2Ret != u2ByteCnt)
    {
    	LOG(1, "[SIF][Read_Error]Channle(%d)Device_Addr(0x%x) want read(%d) bytes. Actually read(%d) bytes\n", u1CHannelID, u1DevAddr, u2ByteCnt, u2Ret);
    }
    return u2Ret;

}

#if 0
/** SIF_X_SpecialRead
 *  Read data with channel ID, word address, of which length up to three, from SIF module.
 *  @param  u1ChannelID     SIF bus ID.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to read. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes read.
 *  @retval = 0             Error
 *   start + slave address + w +tnflg + target address + w +  register address
 *
 restart + slave address + w +tnflg + target address + r
 *    restart + slave address + r +data ... +stop
 */
UINT16 SIF_X_SpecialRead(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                         UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;

#ifndef CONFIG_I2C_MT53XX
		SIF_Init ();//make sure sif init
#endif

    if(_pfnSifCfgPinSetFunc)
    {
    	u1CHannelID=_pfnSifCfgPinSetFunc(u1CHannelID);
    	if(u1CHannelID < 0)
    	{
    		LOG(0,"invaild channel id \n");
    		return u2Ret;
    	}
    }
    else
    {
    	LOG(0,"_pfnSifCfgPinSetFunc must not be NULL\n");
    	ASSERT(0);
    }
	

    if ((u1CHannelID>=SIF_MAX_NUM_BUS) || (_aprSifStruct[u1CHannelID]==NULL))
    {
        goto X_READ_ERROR;
    }
    if (_aprSifStruct[u1CHannelID]->rSifOps.pfn_X_Sp_Read == NULL)
    {
        goto X_READ_ERROR;
    }

    _SIF_SetActiveMaster(_aprSifStruct[u1CHannelID]->u1SifMaster, _aprSifStruct[u1CHannelID]);
    u2Ret = _aprSifStruct[u1CHannelID]->rSifOps.pfn_X_Sp_Read(u1CHannelID, u2ClkDiv, u1DevAddr, u1WordAddrNum, u4WordAddr, pu1Buf, u2ByteCnt);
X_READ_ERROR:
    return u2Ret;
}
#endif

/** SIF_X_Write
 *  Write data with channel ID, word address, of which length up to three, to SIF module.
 *  @param  u1ChannelID     SIF bus ID.
 *  @param  u2ClkDiv        Parameter of divider to divide 27Mhz for SCL.
 *  @param  u1DevAddr       Serial interface device address.
 *  @param  u1WordAddrNum   Serial interface word address length.
 *  @param  u4WordAddr      Serial interface word address. Format XX B2 B1 B0. B2 is sent on SIF bus first.
 *  @param  pu1Buf          Pointer to user's buffer.
 *  @param  u2ByteCnt       Number of byte to write. Max = 256 bytes.
 *  @retval > 0             Actual number of bytes written.
 *  @retval = 0             Error
 */
UINT16 SIF_X_Write(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr,
                   UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT16 u2Ret = 0;
	INT8 u1TmpID = -1;
//to make iic driver common, we make a drvcust mapping for every IC  
	SIF_Init ();//make sure sif init

	if(_pfnSifCfgPinSetFunc)
	{
    	u1TmpID =_pfnSifCfgPinSetFunc(u1CHannelID);
    	if(u1TmpID < 0)
    	{
    		Printf("invaild channel id \n");
    		return u2Ret;
    	}
	}
	else
	{
		Printf("_pfnSifCfgPinSetFunc must not be NULL\n");
		ASSERT(0);
		return u2Ret;
	}

#ifndef CONFIG_I2C_MT53XX
	u2Ret = SIF_RW(u2ClkDiv, u1TmpID, u1DevAddr, u4WordAddr,u1WordAddrNum, SIF_W, pu1Buf, u2ByteCnt);
#else
    u2Ret = SIF_Xfer_Write(u1TmpID,u2ClkDiv,u1DevAddr,u1WordAddrNum, u4WordAddr,pu1Buf,u2ByteCnt);
#endif
    if (u2Ret != u2ByteCnt)
    {
    	LOG(1, "[SIF][Write_Error]Channle(%d)Device_Addr(0x%x) want write(%d) bytes. Actually write(%d) bytes\n", u1CHannelID, u1DevAddr, u2ByteCnt, u2Ret);
    }
    
    return u2Ret;
}

UINT16 SIF_X_Write_SpecialDemod(UINT8 u1CHannelID, UINT16 u2ClkDiv, UINT8 u1DevAddr, UINT8 u1WordAddrNum, UINT32 u4WordAddr, UINT8* pu1Buf, UINT16 u2ByteCnt)
{
    UINT8 au1Data[4];
    UINT16 u2Ret = 0;
	INT8 u1TmpID = -1;
//to make iic driver common, we make a drvcust mapping for every IC  
	SIF_Init ();//make sure sif init


	if(_pfnSifCfgPinSetFunc)
	{
    	u1TmpID =_pfnSifCfgPinSetFunc(u1CHannelID);
    	if(u1TmpID < 0)
    	{
    		Printf("invaild channel id \n");
    		return u2Ret;
    	}
	}
	else
	{
		Printf("_pfnSifCfgPinSetFunc must not be NULL\n");
		ASSERT(0);
		return u2Ret;
	}



    if (u1WordAddrNum == 1)
    {
    	au1Data[0] = (UINT8)(u4WordAddr & 0xFF);
    }
    if (u1WordAddrNum == 2)
    {
    	au1Data[0] = (UINT8)((u4WordAddr >> 8) & 0xFF);
    	au1Data[1] = (UINT8)(u4WordAddr & 0xFF);
    }
    else if (u1WordAddrNum == 3)
    {
    	au1Data[0] = (UINT8)((u4WordAddr >> 16) & 0xFF);
    	au1Data[1] = (UINT8)((u4WordAddr >> 8) & 0xFF);
    	au1Data[2] = (UINT8)(u4WordAddr & 0xFF);
    }
    else
    {
    	goto X_WRITE_SD_ERROR;
    }

#ifndef CONFIG_I2C_MT53XX

	u2Ret = SIF_RW(u2ClkDiv, u1TmpID, u1DevAddr, u4WordAddr,u1WordAddrNum, SIF_W, au1Data, (UINT16)u1WordAddrNum);
    if (u2Ret != (UINT16)u1WordAddrNum)
    {
        goto X_WRITE_SD_ERROR;
    }
	u2Ret = SIF_RW(u2ClkDiv, u1TmpID, u1DevAddr, 0,0, SIF_W, pu1Buf, u2ByteCnt);
#else
    u2Ret = SIF_Xfer_Write(u1TmpID, u2ClkDiv, u1DevAddr, 0, 0, au1Data, (UINT16)u1WordAddrNum);
    if(u2Ret != (UINT16)u1WordAddrNum)
    {
        goto X_WRITE_SD_ERROR;
    }
    u2Ret = SIF_Xfer_Write(u1TmpID,u2ClkDiv,u1DevAddr,0,0,pu1Buf,u2ByteCnt);
#endif

X_WRITE_SD_ERROR:
    return u2Ret;

}

#ifdef __MODEL_slt
extern INT32 _SIF_Diag(UINT8 u1Channel);
INT32 SIF_Diag(VOID)
{
    return 0;
}
#endif

#ifdef CC_SUPPORT_STR

//-----------------------------------------------------------------------------
/**SIF_pm_str_suspend
 */
//-----------------------------------------------------------------------------
void SIF_pm_str_suspend(void)
{

}


//-----------------------------------------------------------------------------
/** SIF_pm_str_resume
 */
//-----------------------------------------------------------------------------
void SIF_pm_str_resume(void)
{

}
#endif // CC_SUPPORT_STR

