/*
 * (C) Copyright 2000-2010
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <linux/stddef.h>
#include <search.h>
#include <errno.h>
#include <malloc.h>
#include "x_bim.h"
#include <version.h>
#include "eeprom_if.h"
#include "hw_pdwnc.h"

#if defined(CC_MT5893)
#include "../../drv_lib/mt5893/inc/x_hal_5381.h"
#elif defined(CC_MT5895)
#include "../../drv_lib/mt5895/inc/x_hal_5381.h"
#endif

#define CC_UBOOT_TRANSFER_USBGPIO_KERNEL
#ifdef CC_S_PLATFORM
#define CC_UBOOT_TRANSFER_UART_MODE
#endif
#define CC_UBOOT_TRANSFER_MSDCGPIO_KERNEL

#if defined(CC_KERNEL_HS200_SUPPORT) || defined(CC_KERNEL_HS400_SUPPORT)
#define CC_UBOOT_TRANSFER_MSDC_AUTOK_KERNEL 1
#else
#define CC_UBOOT_TRANSFER_MSDC_AUTOK_KERNEL 0
#endif

#if defined(CC_KERNEL_ETT_HS200_SUPPORT)
#define CC_UBOOT_TRANSFER_MSDC_ETT_KERNEL 1
#else
#define CC_UBOOT_TRANSFER_MSDC_ETT_KERNEL 0
#endif

#include "drvcust_if.h"

#ifdef CC_S_PLATFORM
#include "x_ldr_env.h"
#endif

#ifdef CC_CMDLINE_UPGRADE_STATUS
#include "x_ldr_env.h"
#endif

#ifdef CC_FBM_DYNMJC
#include "x_ldr_env.h"
#endif

DECLARE_GLOBAL_DATA_PTR;

/************************************************************************
 * Default settings to be used when no valid environment is found
 */
#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)

const uchar default_environment[] = {
#ifdef	CONFIG_BOOTARGS
	"bootargs="	CONFIG_BOOTARGS			"\0"
#endif
#ifdef	CONFIG_EMMCLOG
	"emmclog="	CONFIG_EMMCLOG			"\0"
#endif
#ifdef	CONFIG_BOOTCOMMAND
	"bootcmd="	CONFIG_BOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_RAMBOOTCOMMAND
	"ramboot="	CONFIG_RAMBOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_NFSBOOTCOMMAND
	"nfsboot="	CONFIG_NFSBOOTCOMMAND		"\0"
#endif
#if defined(CONFIG_BOOTDELAY) && (CONFIG_BOOTDELAY >= 0)
	"bootdelay="	MK_STR(CONFIG_BOOTDELAY)	"\0"
#endif
#if defined(CONFIG_BAUDRATE) && (CONFIG_BAUDRATE >= 0)
	"baudrate="	MK_STR(CONFIG_BAUDRATE)		"\0"
#endif
#ifdef	CONFIG_LOADS_ECHO
	"loads_echo="	MK_STR(CONFIG_LOADS_ECHO)	"\0"
#endif
#ifdef	CONFIG_ETHADDR
	"ethaddr="	MK_STR(CONFIG_ETHADDR)		"\0"
#endif
#ifdef	CONFIG_ETH1ADDR
	"eth1addr="	MK_STR(CONFIG_ETH1ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH2ADDR
	"eth2addr="	MK_STR(CONFIG_ETH2ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH3ADDR
	"eth3addr="	MK_STR(CONFIG_ETH3ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH4ADDR
	"eth4addr="	MK_STR(CONFIG_ETH4ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH5ADDR
	"eth5addr="	MK_STR(CONFIG_ETH5ADDR)		"\0"
#endif
#ifdef	CONFIG_IPADDR
	"ipaddr="	MK_STR(CONFIG_IPADDR)		"\0"
#endif
#ifdef	CONFIG_SERVERIP
	"serverip="	MK_STR(CONFIG_SERVERIP)		"\0"
#endif
#ifdef	CONFIG_SYS_AUTOLOAD
	"autoload="	CONFIG_SYS_AUTOLOAD		"\0"
#endif
#ifdef	CONFIG_PREBOOT
	"preboot="	CONFIG_PREBOOT			"\0"
#endif
#ifdef	CONFIG_ROOTPATH
	"rootpath="	CONFIG_ROOTPATH			"\0"
#endif
#ifdef	CONFIG_GATEWAYIP
	"gatewayip="	MK_STR(CONFIG_GATEWAYIP)	"\0"
#endif
#ifdef	CONFIG_NETMASK
	"netmask="	MK_STR(CONFIG_NETMASK)		"\0"
#endif
#ifdef	CONFIG_HOSTNAME
	"hostname="	MK_STR(CONFIG_HOSTNAME)		"\0"
#endif
#ifdef	CONFIG_BOOTFILE
	"bootfile="	CONFIG_BOOTFILE			"\0"
#endif
#ifdef	CONFIG_LOADADDR
	"loadaddr="	MK_STR(CONFIG_LOADADDR)		"\0"
#endif
#ifdef	CONFIG_CLOCKS_IN_MHZ
	"clocks_in_mhz=1\0"
#endif
#if defined(CONFIG_PCI_BOOTDELAY) && (CONFIG_PCI_BOOTDELAY > 0)
	"pcidelay="	MK_STR(CONFIG_PCI_BOOTDELAY)	"\0"
#endif
#ifdef	CONFIG_EXTRA_ENV_SETTINGS
	CONFIG_EXTRA_ENV_SETTINGS
#endif
	"\0"
};
extern char mtdparts_default[];

struct hsearch_data env_htab;
//int default_environment_size = sizeof(default_environment);
int default_environment_size = 0x2000;
static unsigned int kmemsize = 0;
static unsigned int kmemstart2_high = 0;
static unsigned int kmemstart2_low = 0;
static unsigned int kmemsize2 = 0;
static unsigned int kmemsize2_4G = 0;
static unsigned int kmemsize2_3_5G = 0;
static unsigned int kmemsize2_3G = 0;
#ifdef CC_MEM_SIZE_LARGE_THAN_4G
static unsigned int kmemsize2_5G = 0;
static unsigned int kmemsize2_6G = 0;
#endif
static unsigned int kmem2start = 0;
static unsigned int kmem2size = 0;
static unsigned int	kmem3start = 0;
static unsigned int	kmem3size = 0;
#ifdef CC_FBM_DYNMJC
static unsigned int kmemsize_nomjc = 0;
#endif

#if defined(CC_DYNAMIC_FBMSRM_CONF)
static unsigned int kmem4ksize = 0;
static unsigned int kmemfhdsize = 0;
static unsigned int kmem24ksize = 0;
static unsigned int kmem2fhdsize = 0;
static unsigned int gpu4kstart = 0;
static unsigned int gpufhdstart = 0;
static unsigned int gpu4ksize = 0;
static unsigned int gpufhdsize = 0;
static unsigned int gpu4kionsize = 0;
static unsigned int gpufhdionsize = 0;
#endif

#ifdef CC_RAMOOPS_SUPPORT
static unsigned int RamoopsMemAddress = 0;
#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
static unsigned int RamoopsMemAddress_nomjc = 0;
#endif
#endif

static uchar env_get_char_init(int index)
{
	/* if crc was bad, use the default environment */
	if (gd->env_valid)
		return env_get_char_spec(index);
	else
		return default_environment[index];
}

uchar env_get_char_memory(int index)
{
	return *env_get_addr(index);
}

uchar env_get_char(int index)
{
	/* if relocated to RAM */
	if (gd->flags & GD_FLG_RELOC)
		return env_get_char_memory(index);
	else
		return env_get_char_init(index);
}

const uchar *env_get_addr(int index)
{
	if (gd->env_valid)
		return (uchar *)(gd->env_addr + index);
	else
		return &default_environment[index];
}

extern void env_add_bootargs(char* oldstr, char* newstr);


uint8_t env_get_bootargs(const char* key, char* value)
{
    char *p1 = NULL, *p2 = NULL;
    char *bootargs = NULL;
    unsigned int value_strlen = 0;

    bootargs = getenv("bootargs");
    if(bootargs == NULL || key == NULL)
    {
        return 0; //false, get the key string fail.
    }
    p1 = strstr(bootargs, key);
    if(p1 == NULL)
    {
        return 0; //false, get the key string fail.
    }

    p1 += strlen(key);
    if(*p1 == '=')
    {
        p1++;
        if(*p1 == '\"')
        {
            p2 = strstr(p1+1,"\" ");
            if(p2 != NULL)
            {
                value_strlen = p2-p1+1;
                strncpy(value,p1,value_strlen);
                value[value_strlen] = '\0';
            }
        }
        else
        {
            p2 = strstr(p1," ");
            if(p2 != NULL)
            {
                value_strlen = p2-p1;
                strncpy(value,p1,value_strlen);
                value[value_strlen] = '\0';
            }
        }
    }
    return 1;//true, get the key string OK.
}

void print_bootargs_size(void)
{
    char *bootargs = NULL;
    bootargs = getenv("bootargs");
    if(bootargs == NULL) return;

    printf("\n## uBoot bootargs size is limited: %d Bytes, current size: %d Bytes.\n\n",
            COMMAND_LINE_SIZE, strlen(bootargs));
	if((strlen(bootargs))>= COMMAND_LINE_SIZE)
	{
		while(1)
		{
			Printf("[Sony commandline]the command line size is not enough.\n");
		}
	}
}

#ifdef CC_UBOOT_ADD_USB_ETH_MODE
static void env_add_usb_eth(void)
{
	char usbEthCtrl[16] = {0};
	char usbEthPort[] = "adb_port=0";//always using usb port 0 for vitual Ethernet port in FY17

	unsigned int u4IsUsbEth = 0;
    Printf("+[env_add_usb_eth]\n");
    // adb_enalbe  -> arch/arm/mach-mt53xx/core.c/ssusb_adb_flag to set host/device mode
	if((0==DRVCUST_InitQuery(eFlagIsUsbEthMode, &u4IsUsbEth)) && u4IsUsbEth)
	{
		snprintf(usbEthCtrl, sizeof(usbEthCtrl),"adb_enable=%d", u4IsUsbEth);
        Printf("+[env_add_usb_eth]1 %s, %s \n", usbEthCtrl, usbEthPort);
		env_add_bootargs("adb_enable=", usbEthCtrl);
		env_add_bootargs("adb_port=", usbEthPort);
	}     
    else 
    {
        snprintf(usbEthCtrl, sizeof(usbEthCtrl),"adb_enable=%d", 0);
        Printf("+[env_add_usb_eth]2 %s, %s \n", usbEthCtrl, usbEthPort);
		env_add_bootargs("adb_enable=", usbEthCtrl);
    }
}
#endif
#if defined(CC_UBOOT_ADD_USB_LOG_MODE)
#ifdef CC_S_PLATFORM
static void env_add_usb_log(void)
{
    unsigned int u4IsUsbLog = 0;

    #if defined(CC_LMP_CUST) || defined(CC_PHX_CUST)
    char usbSerialConsole[128] = "console=ttyUSB0,115200n1";
    #else
    char usbSerialConsole[128] = "console=ttyMT0,115200n1 console=ttyUSB0,115200n1";
    #endif
    char uartSerialConsole[128] = "console=ttyMT0,115200n1";


    if((0==DRVCUST_InitQuery(eFlagIsUsbLogMode, &u4IsUsbLog)) && u4IsUsbLog)
    {
        Printf("+[env_add_usb_log-usb]  = %s\n", usbSerialConsole);
        env_add_bootargs("console=", usbSerialConsole);
    }
    else
    {
        Printf("+[env_add_usb_log-uart]  = %s\n", uartSerialConsole);
        env_add_bootargs("console=", uartSerialConsole);
    }
}
#else
extern int EEPROM_Read(uint64_t u8Offset, int u4MemPtr, int u4MemLen);
static void env_add_usb_log(void)
{
    char u1FactMode = 0;
    char usbserialconsole[128] = "console=ttyUSB0,115200n1";

    if (0 == EEPROM_Read(CC_UBOOT_ADD_USB_LOG_MODE_ADDR, (int)(void*)&u1FactMode, sizeof(char)))
    {
        if (u1FactMode == 1)
        {
            env_add_bootargs(NULL, usbserialconsole);
        }
    }
}
#endif
#endif

#ifdef CC_ENABLE_MTK_MODEL_INDEX
extern unsigned int GetCurrentPanelIndex(void);
#endif
static void env_add_panel_index(void)
{
    char newstr[32] = {0};

#ifdef CC_ENABLE_MTK_MODEL_INDEX
    snprintf(newstr, sizeof(newstr), "modelindex_id=%d", GetCurrentPanelIndex());
#else
    snprintf(newstr, sizeof(newstr), "modelindex_id=%d", GetModelIndexFromEEPROM());
#endif // CC_ENABLE_MTK_MODEL_INDEX
    env_add_bootargs("modelindex_id=", newstr);
}


#ifdef CC_ENABLE_MALI_CORE_NUM
extern unsigned int Efuse_MaliMaliCoreNum(void);
static void env_add_mali_core_num(void)
{
    char newstr[32] = {0};

    snprintf(newstr, sizeof(newstr), "mali_core_num=%d", Efuse_MaliMaliCoreNum());
    env_add_bootargs("mali_core_num=", newstr);
}
#endif
#ifdef CC_SUPPORT_CHIPID

static void env_add_chipid(void)
{
    char newstr[64] = {0};
	#if defined(CC_MT5893) || defined(CC_MT5895)
	snprintf(newstr, sizeof(newstr), "androidboot.hardware=mt%x", (*((volatile u32*)(IO_VIRT+0x0000d064)))>>16);
	#else
    snprintf(newstr, sizeof(newstr), "androidboot.hardware=mt%x", (*((volatile u32*)(0xf000d064)))>>16);
	#endif
    env_add_bootargs("androidboot.hardware=", newstr);
}
#endif
#ifdef CC_S_PLATFORM
#ifdef CC_UBOOT_TRANSFER_UART_MODE
static void env_add_uart_mode(void)
{
	int val=0;
	char str[32] = {0};
	EEPROM_Read((UINT64)0x6c8, (UINT32)&val, (UINT32)sizeof(char));//val = 3 VDA 4 Locatel
	snprintf(str, sizeof(str), "uart_mode=%d",val);
	env_add_bootargs("uart_mode=", str);

}
#endif
#endif
#ifdef SYS_C4TV_SUPPORT

static void env_add_dual_partition(void)
{
    DTVCFG_T rDtvCfg;
    UBOOT_EEPDTV_GetCfg(&rDtvCfg);
    if ((rDtvCfg.u1Flags7 & DTVCFG_FLAG7_DUAL_BANK_CAST_SYSTEM) == DTVCFG_FLAG7_DUAL_BANK_CAST_SYSTEM)
    	{
			env_add_bootargs("system=", "system=1");
    	}
	else
		{
			env_add_bootargs("system=", "system=0");
		}
	
    if ((rDtvCfg.u1Flags7 & DTVCFG_FLAG7_DUAL_BANK_3RD) == DTVCFG_FLAG7_DUAL_BANK_3RD)
    	{
			env_add_bootargs("3rd_ro=", "3rd_ro=1");
    	}
	else
		{
			env_add_bootargs("3rd_ro=", "3rd_ro=0");
		}

    if ((rDtvCfg.u1Flags7 & DTVCFG_FLAG7_DUAL_BANK_CHROME) == DTVCFG_FLAG7_DUAL_BANK_CHROME)
    	{
    		env_add_bootargs("chrome=", "chrome=1");
    	}
    else
    	{
    		env_add_bootargs("chrome=", "chrome=0");
    	}
}
#endif
#ifdef CC_KERNEL_NO_LOG
static void env_add_forbid_uart(void)
{
    char newstr[16] = {0};
    snprintf(newstr, sizeof(newstr),"forbid_uart0=%d", 1);
    env_add_bootargs("forbid_uart0=", newstr);
}
#endif

#if defined(CC_MT5895)
static void env_add_uart_dma_addr_and_size()
{
    unsigned int uart_dma_addr = DRVCUST_InitGet(eFbmMemUartDMAStart);
    unsigned int uart_dma_size = DRVCUST_InitGet(eFbmMemUartDMASize);
    char new_str[64];

    memset(new_str, 0, sizeof(new_str));
    snprintf(new_str, sizeof(new_str), "uart0_dma_addr=%#x@%#x", uart_dma_size, uart_dma_addr);

    env_add_bootargs("uart0_dma_addr=", new_str);
}
#endif

static void env_add_uart_perfor(void)
{
    char new_str[32] = {0};
    unsigned int new_uint = ~0U;
    char *perfor = getenv("uart_perfor");

    if (perfor == NULL)
    {
        #ifdef CC_S_PLATFORM
        perfor = "dma_tx";
        #else
        perfor = "swfifo_tx";
        #endif
        setenv("uart_perfor", perfor);
    }

    if (!strncmp(perfor, "dma_tx", strlen("dma_tx")))
    {
        new_uint = 0x13;
    }
    else if (!strncmp(perfor, "swfifo_tx", strlen("swfifo_tx")))
    {
        new_uint = 0x12;
    }
    else if (!strncmp(perfor, "raw_tx", strlen("raw_tx")))
    {
        new_uint = 0x11;
    }
    else if (!strncmp(perfor, "swfifo_dma_tx", strlen("swfifo_dma_tx")))
    {
        new_uint = 0x12 + (0x01 << 28);
    }
    else if (!strncmp(perfor, "raw_dma_tx", strlen("raw_dma_tx")))
    {
        new_uint = 0x11 + (0x01 << 28);
    }
    else if (!strncmp(perfor, "off", strlen("off")))
    {
        new_uint = 0x0;
    }

    if (~0U != new_uint)
    {
        snprintf(new_str, sizeof(new_str), "uartp_mode=%u", new_uint);
        env_add_bootargs("uartp_mode=", new_str);
    }
}

#ifdef CC_ENABLE_CMA_SUPPORT
static void env_add_cmaCHA_endaddress(void)
{
    char newstr[32] = {0};
    snprintf(newstr, sizeof(newstr), "cmaCHA_endaddress=%d", (((PHYS_SDRAM_1+PHYS_SDRAM_1_SIZE) >>20) - 4)); // MByte
    env_add_bootargs("cmaCHA_endaddress=", newstr);
}

static void env_del_cmaCHA_endaddress(void)
{
    char newstr[32] = {0};
    snprintf(newstr, sizeof(newstr), "cmaCHA_endaddress=%d", 0);
    env_add_bootargs("cmaCHA_endaddress=", newstr);
}
#if CONFIG_NR_DRAM_BANKS>1
static void env_add_cmaCHB_endaddress(void)
{
    char newstr[32] = {0};
    snprintf(newstr, sizeof(newstr), "cmaCHB_endaddress=%d", (((PHYS_SDRAM_2+PHYS_SDRAM_2_SIZE) >> 20) - 4));//MByte
    env_add_bootargs("cmaCHB_endaddress=", newstr);
}

static void env_del_cmaCHB_endaddress(void)
{
    char newstr[32] = {0};
    snprintf(newstr, sizeof(newstr), "cmaCHB_endaddress=%d", 0);
    env_add_bootargs("cmaCHB_endaddress=", newstr);
}
#endif
#endif

#ifdef CC_MT5890
static void env_add_have_vp9(void)
{
	char newstr[16] = {0};

    UINT32 IcVersion=BSP_GetIcVersion();

	if(IcVersion >= 0x58900003)
	{
		snprintf(newstr, sizeof(newstr), "have_vp9=%d", 1);
		env_add_bootargs("have_vp9=", newstr);
	}
	else
	{
		snprintf(newstr, sizeof(newstr), "have_vp9=%d", 0);
		env_add_bootargs("have_vp9=", newstr);
	}

}

#endif

#ifdef CC_VP9_SUPPORT
static void env_add_vp9_support(void)
{
	char newstr[26] = {0};
	snprintf(newstr, sizeof(newstr), "androidboot.vp9_support=%d", 1);
	env_add_bootargs("androidboot.vp9_support=", newstr);
}
#endif

static void env_add_gpu_addr(void)
{
	char newstr[30] = {0};
#if defined(CC_DYNAMIC_FBMSRM_CONF)
	if(FBMDynIs4KMode())
		snprintf(newstr, sizeof(newstr), "gpustart=%s", getenv("gpu4kstart"));
	else
		snprintf(newstr, sizeof(newstr), "gpustart=%s", getenv("gpufhdstart"));
#else
	snprintf(newstr, sizeof(newstr), "gpustart=%lld", (long long int)PHYS_GPU_START);
#endif
	env_add_bootargs("gpustart=", newstr);

#if defined(CC_DYNAMIC_FBMSRM_CONF)
	if(FBMDynIs4KMode())
		snprintf(newstr, sizeof(newstr), "gpusize=%s", getenv("gpu4ksize"));
	else
		snprintf(newstr, sizeof(newstr), "gpusize=%s", getenv("gpufhdsize"));
#else
	snprintf(newstr, sizeof(newstr), "gpusize=%d", PHYS_GPU_SIZE);
#endif
	env_add_bootargs("gpusize=", newstr);

#if defined(CC_DYNAMIC_FBMSRM_CONF)
	if(FBMDynIs4KMode())
		snprintf(newstr, sizeof(newstr), "gpuionsize=%s", getenv("gpu4kionsize"));
	else
		snprintf(newstr, sizeof(newstr), "gpuionsize=%s", getenv("gpufhdionsize"));
#else
	snprintf(newstr, sizeof(newstr), "gpuionsize=%d", PHYS_GPU_ION_SIZE);
#endif
	env_add_bootargs("gpuionsize=", newstr);
}
#ifdef CC_S_PLATFORM
#if (defined(CC_OX_CUST) || defined(CC_WK_CUST) || defined(CC_HK_CUST) || defined(CC_CP_CUST))
extern char *UBOOT_EEP_GetSerial(void);
#endif
#endif
char serialstr[20] = {0};
char *env_add_usb_serialno(void)
{
	char newstr[50] = {0};
	#ifdef CC_S_PLATFORM
	#if (defined(CC_OX_CUST) || defined(CC_WK_CUST) || defined(CC_HK_CUST) || defined(CC_CP_CUST))
	char *serialstr1;
	serialstr1 = UBOOT_EEP_GetSerial();
	memcpy(serialstr, serialstr1, 15);
	//Printf("env_add_usb_serialno(%s)\n",serialstr);
	snprintf(newstr, sizeof(newstr), "androidboot.serialno=%s", serialstr);
	env_add_bootargs("androidboot.serialno=", newstr);
	#else
	#ifdef CC_MT5893
	#define ETHERNET_BASE1	(IO_VIRT+0x00032014)
	#define ETHERNET_BASE2	(IO_VIRT+0x00032018)
	#else
	#define ETHERNET_BASE1	(0xF0032014)
	#define ETHERNET_BASE2	(0xF0032018)
	#endif
	snprintf(serialstr, 16, "%08x%08x", (*((volatile u32*)(ETHERNET_BASE1))), (*((volatile u32*)(ETHERNET_BASE2))));
	snprintf(newstr, sizeof(newstr), "androidboot.serialno=%s", serialstr);	
	env_add_bootargs("androidboot.serialno=", newstr);
	#endif
	#else
	#ifdef CC_MT5893
	#define ETHERNET_BASE1	(IO_VIRT+0x00032014)
	#define ETHERNET_BASE2	(IO_VIRT+0x00032018)
	#else
	#define ETHERNET_BASE1	(0xF0032014)
	#define ETHERNET_BASE2	(0xF0032018)
	#endif
	snprintf(serialstr, 16, "%08x%08x", (*((volatile u32*)(ETHERNET_BASE1))), (*((volatile u32*)(ETHERNET_BASE2))));
	snprintf(newstr, sizeof(newstr), "androidboot.serialno=%s", serialstr);	
	env_add_bootargs("androidboot.serialno=", newstr);
	#endif
	return serialstr;
}

#ifdef CC_ANDROID_SECURE_BOOT
void env_add_boot_state(unsigned int state)
{
    char newstr[40] = {0};
    snprintf(newstr, sizeof(newstr), "androidboot.verifiedbootstate=%s", state == 0 ? "green" : state == 1 ? "yellow" : state == 2 ? "orange" : "red");
    env_add_bootargs("androidboot.verifiedbootstate=", newstr);
}
#endif

#ifdef CC_ANDROID_RETROFIT_DP_SUPPORT
void env_add_force_normal_boot(void)
{
    const char force_normal_boot[] = "androidboot.force_normal_boot=1";
    env_add_bootargs(force_normal_boot, force_normal_boot);
}
void env_del_force_normal_boot(void)
{
    env_del_bootargs("androidboot.force_normal_boot=1", NULL);
}
#endif

#ifdef CC_3RD_KO_INSERTION
#ifdef CC_EMMC_BOOT
extern int emmc_read_from_partition(unsigned char* buf, char* pname, u64 offset, unsigned int size);
#endif
static void env_add_beic_support(void)
{
#ifdef CC_SONY_FILE_BASED_RECIPE_SUPPORT
	UINT32 u4ModelType=0;
	UINT32 u4EnableBe=0;

#ifdef CC_COMBINE_4K_2K
	EEPROM_Read((UINT64)0x10010, (UINT32)&u4ModelType, (UINT32)sizeof(char));
	EEPROM_Read((UINT64)0x1050A, (UINT32)&u4EnableBe, (UINT32)sizeof(char));

	if ((u4ModelType&0x08)==0)
	{
		goto out;
	}
	else if (u4EnableBe == 0x1)
	{
		goto out;
	}
#else
	EEPROM_Read((UINT64)BE_IC_SUPPORT_ADDR, (UINT32)&u4ModelType, (UINT32)sizeof(char));
	if(((UINT8)u4ModelType) != 0x79)
	{
		goto out;
	}
#endif

	//Printf("env_add_beic_support exist\n");
	env_add_bootargs("beic_supported=", "beic_supported=1");

out:
	return;
#else
	char newstr[32] = {0};
	char ddri_header[4] = {0};

#ifdef CC_EMMC_BOOT
	emmc_read_from_partition(ddri_header, "ddri", 0, sizeof(ddri_header));
#endif
	if(strncmp(ddri_header, "DDRI",4)==0)
	{
	  //Printf("env_add_beic_support exist\n");
		  snprintf(newstr, sizeof(newstr), "beic_supported=%d", 1);
		  env_add_bootargs("beic_supported=", newstr);
	}
#endif
}

#endif

#ifdef CC_TUNER_DBG_LVL_BY_UBOOT
static void env_add_tuner_dbg_lvl(void)
{
    char newstr[32] = {0};

    snprintf(newstr, sizeof(newstr), "tuner_debug_lvl=0");
    printf("env_add_tuner_dbg_lvl : %s\n",newstr);
    env_add_bootargs("tuner_debug_lvl=", newstr);

    return ;
}
#endif

#ifdef CC_S_PLATFORM

static void env_add_product_model(void)
{
    char newstr[32] = {0};
    char value = ((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->szMSDCenv1[63];

    snprintf(newstr, sizeof(newstr), "product_model=%d", value);
    env_add_bootargs("product_model=", newstr);

    return ;
}

static void env_add_ic_version(void)
{
	char newstr[16] = {0};
	UINT32 IcVersion = BSP_GetIcVersion();

	#if  defined(CC_MT5893)
		if(IcVersion == IC_VER_5893_AA)
		{
			char newstrtemp[16] ={"ES1"};
			snprintf(newstr, sizeof(newstr), "m3_version=%s", newstrtemp);
			env_add_bootargs("m3_version=", newstr);
		}
		else if(IcVersion == IC_VER_5893_AB)
		{
			char newstrtemp[16] ={"ES2"};
			snprintf(newstr, sizeof(newstr), "m3_version=%s", newstrtemp);
			env_add_bootargs("m3_version=", newstr);
		}
		else if (IcVersion == IC_VER_5893_AC)
		{
			char newstrtemp[16] ={"ES3"};
			snprintf(newstr, sizeof(newstr), "m3_version=%s", newstrtemp);
			env_add_bootargs("m3_version=", newstr);
		}
		else if (IcVersion == IC_VER_5893_AD)
		{
			char newstrtemp[16] ={"ES4"};
			snprintf(newstr, sizeof(newstr), "m3_version=%s", newstrtemp);
			env_add_bootargs("m3_version=", newstr);
		}
		else
		{
			char newstrtemp[16] ={"ESX"};
			snprintf(newstr, sizeof(newstr), "m3_version=%s", newstrtemp);
			env_add_bootargs("m3_version=", newstr);
		}
	#elif defined(CC_MT5895)
		if(IcVersion == IC_VER_5895_AA)
		{
			char newstrtemp[16] ={"ES1"};
			snprintf(newstr, sizeof(newstr), "m5_version=%s", newstrtemp);
			env_add_bootargs("m5_version=", newstr);
		}
		else if(IcVersion == IC_VER_5895_AB)
		{
			char newstrtemp[16] ={"ES2"};
			snprintf(newstr, sizeof(newstr), "m5_version=%s", newstrtemp);
			env_add_bootargs("m5_version=", newstr);
		}
		else if (IcVersion == IC_VER_5895_AC)
		{
			char newstrtemp[16] ={"ES3"};
			snprintf(newstr, sizeof(newstr), "m5_version=%s", newstrtemp);
			env_add_bootargs("m5_version=", newstr);
		}
		else if (IcVersion == IC_VER_5895_AD)
		{
			char newstrtemp[16] ={"ES4"};
			snprintf(newstr, sizeof(newstr), "m5_version=%s", newstrtemp);
			env_add_bootargs("m5_version=", newstr);
		}
		else
		{
			char newstrtemp[16] ={"ESX"};
			snprintf(newstr, sizeof(newstr), "m5_version=%s", newstrtemp);
			env_add_bootargs("m5_version=", newstr);
		}
	#endif
}

static void env_add_hard_lock_detector(void)
{
	char new_str[32] = {0};
	unsigned int new_uint = ~0U;
	char *perfor = getenv("enable_hard_lock");

	if (perfor == NULL)
	{
		perfor = "on";
		setenv("enable_hard_lock", perfor);
	}

	if (!strncmp(perfor, "on", strlen("on")))
	{
		new_uint = 0x11;
	}
	else if(!strncmp(perfor, "off", strlen("off")))
	{
		new_uint = 0x2;
	}
	else
	{
        //
	}
	if (~0U != new_uint)
	{
		snprintf(new_str, sizeof(new_str), "enable_hard_lock=%u", new_uint);
		env_add_bootargs("enable_hard_lock=", new_str);
	}
}

static void env_add_decoder_spec(void)
{
       UINT32 u4DecoderSpec=0;
       char newstr[64] = {0};
       EEPROM_Read((UINT64)0x10019, (UINT32)&u4DecoderSpec, (UINT32)sizeof(char));//read decoder spec field, and set parameter
       if((u4DecoderSpec&0x01)==0x01)
       {
               snprintf(newstr, sizeof(newstr), "androidboot.svp.decorder.spec=8k");
               env_add_bootargs("androidboot.svp.decorder.spec=", newstr);
       }
}

//This is workaround solution for Uro3 mk2 models
#if defined(CC_HK_CUST)
#define SEGMENT_OFFSET 0x10089
#define UR3_MK2_1 0x4C
#define UR3_MK2_2 0x4D
#define UR3_MK2_3 0x4E
#define UR3_MK2_4 0x51
static void env_add_product_variant_bootargs(void)
{
    char newstr[64] = {0};
    UINT32 segment = 0;
    EEPROM_Read((UINT64)SEGMENT_OFFSET, (UINT32)&segment, 1);
    if(segment==UR3_MK2_1||segment==UR3_MK2_2||segment==UR3_MK2_3||segment==UR3_MK2_4)
    {
        snprintf(newstr, sizeof(newstr), "androidboot.svp.product.variant=UR3");
        env_add_bootargs("androidboot.svp.product.variant=", newstr);
    }
}
#endif

#define MODEL_YEAR_OFFSET 0x10000
static void env_add_model_year(void)
{
    char newstr[64] = {0};
    UINT32 model_year = 0;
    EEPROM_Read((UINT64)MODEL_YEAR_OFFSET, (UINT32)&model_year, 1);
    if((0 < model_year)&&(model_year < 100))
    {
        /* 0x01:FY15, 0x02:FY16, ... */
        model_year += 14;
    }
    else
    {
        /* Invalid */
        model_year = 0;
    }
    snprintf(newstr, sizeof(newstr), "androidboot.svp.modelyear=%d", model_year);
    env_add_bootargs("androidboot.svp.modelyear=", newstr);
}

#define REMOTE_VOICE_OFFSET 0x1010b
#define REMOTE_VOICE_BUNDLED 0x02
static void env_add_remoter_voice(void)
{
    char newstr[64] = {0};
    UINT32 remote_voice = 0;
    EEPROM_Read((UINT64)REMOTE_VOICE_OFFSET, (UINT32)&remote_voice, 1);
    if(remote_voice == REMOTE_VOICE_BUNDLED)
    {
        char newstrtemp[16] ={"1"};
        snprintf(newstr, sizeof(newstr), "androidboot.svp.remotevoice=%s", newstrtemp);
        env_add_bootargs("androidboot.svp.remotevoice=", newstr);
    }
    else
    {
        char newstrtemp[16] ={"0"};
        snprintf(newstr, sizeof(newstr), "androidboot.svp.remotevoice=%s", newstrtemp);
        env_add_bootargs("androidboot.svp.remotevoice=", newstr);
    }
}

#define BUILTIN_MICROPHONE_OFFSET 0x1010d
#define BUILTIN_MICROPHONE_PDM_MIC 0x02
static void env_add_builtin_microphone(void)
{
    char newstr[64] = {0};
    UINT32 builtin_microphone = 0;
    EEPROM_Read((UINT64)BUILTIN_MICROPHONE_OFFSET, (UINT32)&builtin_microphone, 1);
    if(builtin_microphone == BUILTIN_MICROPHONE_PDM_MIC)
    {
        char newstrtemp[16] ={"1"};
        sprintf(newstr, "androidboot.svp.builtinmic=%s", newstrtemp);
        env_add_bootargs("androidboot.svp.builtinmic=", newstr);
    }
    else
    {
        char newstrtemp[16] ={"0"};
        sprintf(newstr, "androidboot.svp.builtinmic=%s", newstrtemp);
        env_add_bootargs("androidboot.svp.builtinmic=", newstr);
    }
}

static void env_add_from_mvi(void)
{
	char str[128+1] = {0};
	char str2[1] = {0};
	char newstr[64] = {0};
	UBOOT_EEPDTV_GetCustRawData((UINT32)str,0,(UINT32)(sizeof(str)-1)); //this api is safe
	if (str[0]) 
	{
		env_add_bootargs(str, str);
	}
	UBOOT_EEPDTV_GetCustRawData((UINT32)str2,128,1); //this api is safe
	if((str2[0]&0x3)==0x0)
	{
		char newstrtemp[16] ={"1920x1080"};
		snprintf(newstr, sizeof(newstr), "androidboot.svp.panel.resolution=%s", newstrtemp);
		env_add_bootargs("androidboot.svp.panel.resolution=", newstr);
	}
	else if((str2[0]&0x3)==0x1)
	{
		if((DRVCUST_Get_BE_Type() == DRV_CUST_BE_TYPE_S_8K_Y18) || (DRVCUST_Get_BE_Type() == DRV_CUST_BE_TYPE_S_8K_Y20))
		{
			char newstrtemp[16] ={"7680x4320"};
			snprintf(newstr, sizeof(newstr), "androidboot.svp.panel.resolution=%s", newstrtemp);
			env_add_bootargs("androidboot.svp.panel.resolution=", newstr);
		}
		else
		{
			char newstrtemp[16] ={"3840x2160"};
			snprintf(newstr, sizeof(newstr), "androidboot.svp.panel.resolution=%s", newstrtemp);
			env_add_bootargs("androidboot.svp.panel.resolution=", newstr);
		}
	}
	else if((str2[0]&0x3)==0x2)
	{
		char newstrtemp[16] ={"7680x4320"};
		snprintf(newstr, sizeof(newstr), "androidboot.svp.panel.resolution=%s", newstrtemp);
		env_add_bootargs("androidboot.svp.panel.resolution=", newstr);
	}
	else
	{
		char newstrtemp[16] ={"15360x8640"};
		snprintf(newstr, sizeof(newstr), "androidboot.svp.panel.resolution=%s", newstrtemp);
		env_add_bootargs("androidboot.svp.panel.resolution=", newstr);
	}
	env_add_decoder_spec();//add decoder spec information
	env_add_model_year();  //add model year for GG request "ro_oem_key1"
	env_add_remoter_voice(); //add model year for GG request "ro_oem_key1"
        env_add_builtin_microphone(); //add builtin mic for Far Field Mic. model
#if defined(CC_HK_CUST)
	env_add_product_variant_bootargs(); //add workaround solution for Uro3 mk2 models
#endif
	return ;
}

static void env_add_usb_hub(void)
{
	char str2[1] = {0};
	char newstr[32] = {0};
	UBOOT_EEPDTV_GetCustRawData((UINT32)str2,129,1); //Bytes 130
    snprintf(newstr, sizeof(newstr), "usb_hub=%d", str2[0]);
    env_add_bootargs("usb_hub=", newstr);
	return ;
}




#ifdef CC_DYNAMIC_USB_ADB_CONFIG
#ifdef CC_UBOOT_AUTO_TEST_ADB
static void env_add_adb_support(void)
{
	char newstr[16] = {0};
	snprintf(newstr, sizeof(newstr), "adb_enable=%d", 1);    //default enable ADB AUTO_TEST mode.
	env_add_bootargs("adb_enable=", newstr);
	sprintf(newstr, "adb_port=%d", 1);
	env_add_bootargs("adb_port=", newstr);
}

#else // CC_UBOOT_AUTO_TEST_ADB

extern unsigned int UBOOT_EEPDTV_IsADB(void);
static void env_add_adb_support(void)
{
    char newstr[16] = {0};
    unsigned int u4Support=0;

    u4Support = UBOOT_EEPDTV_IsADB();

    Printf("+[env_add_adb_support] u4Support = %d", u4Support);

    if(u4Support)
    {
        snprintf(newstr, sizeof(newstr), "adb_enable=%d", 1);
        env_add_bootargs("adb_enable=", newstr);
    }
	else
	{
        snprintf(newstr, sizeof(newstr), "adb_enable=%d", 0);
        env_add_bootargs("adb_enable=", newstr);
	}

	snprintf(newstr, sizeof(newstr), "adb_port=%d", 1);

    env_add_bootargs("adb_port=", newstr);
    
}
#endif //CC_UBOOT_AUTO_TEST_ADB
#endif //CC_DYNAMIC_USB_ADB_CONFIG
#ifdef CC_USB3_TO_ADB_PORT_CONFIG
#ifdef CC_UBOOT_AUTO_TEST_ADB
static void env_add_u32adb_support(void)
{
    char newstr[16] = {0};
    snprintf(newstr, sizeof(newstr), "u32adb=%d", 1);    //default enable ADB AUTO_TEST mode.
    env_add_bootargs("u32adb=", newstr);
}

#else //CC_UBOOT_AUTO_TEST_ADB

extern unsigned int UBOOT_EEPDTV_IsU3ToADB(void);
static void env_add_u32adb_support(void)
{
    char newstr[16] = {0};
	unsigned int u4Support=0;
	char ddri_header[4] = {0};

    u4Support = UBOOT_EEPDTV_IsU3ToADB();
    Printf("env_add_u32adb_support read from EEPDTV u32adb ADB Support = %d\n", u4Support);

	if(u4Support)
	{
          snprintf(newstr, sizeof(newstr), "u32adb=%d", 1);
          env_add_bootargs("u32adb=", newstr);
	}
	else
	{
          snprintf(newstr, sizeof(newstr), "u32adb=%d", 0);
          env_add_bootargs("u32adb=", newstr);
	}
}
#endif //CC_UBOOT_AUTO_TEST_ADB
#endif //CC_USB3_TO_ADB_PORT_CONFIG
#else //CC_S_PLATFORM
#if 0
static void env_add_adb_support(void)
{
    char newstr[16] = {0};
    sprintf(newstr, "adb_enable=%d", 1);
    env_add_bootargs("adb_enable=", newstr);

    sprintf(newstr, "adb_port=%d", 0);
    env_add_bootargs("adb_port=", newstr);
}
#endif

#ifdef CC_WIFI_U2_SPEED_ON_U3PORT
static void env_add_wifi_u3_support(void)
{
	char newstr[16] = {0};
	snprintf(newstr, sizeof(newstr), "wifi_u3=%d", 1);
	env_add_bootargs("wifi_u3=", newstr);
}
#endif
#ifdef CC_WIFI_25G_SUPPORT
static void env_add_wifi_25g_support(void)
{
	char newstr[16] = {0};
	snprintf(newstr, sizeof(newstr), "wifi_25g=%d", 1);
	env_add_bootargs("wifi_25g=", newstr);
}
#endif

#ifdef CC_UBOOT_AUTO_TEST_ADB
static void env_add_adb_support(void)
{
	char newstr[16] = {0};
	snprintf(newstr, sizeof(newstr), "adb_enable=%d", 1);		//default enable ADB AUTO_TEST mode.
	env_add_bootargs("adb_enable=", newstr);
}

#else
extern unsigned int UBOOT_EEPDTV_IsSupportADB(void);

#ifdef CC_UBOOT_CDC_ALWAYS_ENABLE
static void env_add_adb_support(void)
{
	char newstr[16] = {0};
	unsigned int u4Support=1;

    Printf("Env_common.c/env_add_adb_support for vfy ADB Support = %d\n", u4Support);

	if(u4Support)
	{
		snprintf(newstr, sizeof(newstr), "adb_enable=%d", 1);
		env_add_bootargs("adb_enable=", newstr);
	}
}
#else
static void env_add_adb_support(void)
{
	char newstr[16] = {0};
	unsigned int u4Support=0;

    // get data from borad/mt5xxx/mt5xxx_sif.c
    // the detail layout confirm with corresponding SI
	u4Support = UBOOT_EEPDTV_IsSupportADB();
    Printf("Env_common.c/env_add_adb_support read EEPROM ADB Support = %d\n", u4Support);

	if(u4Support)
	{
		snprintf(newstr, sizeof(newstr), "adb_enable=%d", 1);
		env_add_bootargs("adb_enable=", newstr);
	}
	else
		{
		snprintf(newstr, sizeof(newstr), "adb_enable=%d", 0);
		env_add_bootargs("adb_enable=", newstr);
	}
}
#endif
#endif

#endif //CC_S_PLATFORM

#ifdef CC_UBOOT_TRANSFER_USBGPIO_KERNEL
#define MUSB_MAX_CONTROLLERS 4

static void env_add_usb_gpio(void)
{
	int i;
	int u4UsbPortConfig;
	char pstr[11];
	int _i4usbportusing[MUSB_MAX_CONTROLLERS] ={eUSB0Config,eUSB1Config,eUSB2Config,eUSB3Config};
	int _i4usbpwrgpio[MUSB_MAX_CONTROLLERS] ={eUSBVbus0Gpio,eUSBVbus1Gpio,eUSBVbus2Gpio,eUSBVbus3Gpio};
    #ifdef CC_USB_DELAY_RESUME_SUPPORT
	int _i4usbdelayresume[MUSB_MAX_CONTROLLERS] ={eUSB0DelayResume,eUSB1DelayResume,eUSB2DelayResume,eUSB3DelayResume};
    #endif
	int _i4usbpwrpolarity[MUSB_MAX_CONTROLLERS] ={eUSBVbus0EnablePolarity,eUSBVbus1EnablePolarity,eUSBVbus2EnablePolarity,eUSBVbus3EnablePolarity};
	int _i4usbocgpio[MUSB_MAX_CONTROLLERS] ={eUSB0OCGpio,eUSB1OCGpio,eUSB2OCGpio,eUSB3OCGpio};
	int _i4usbocgpiopolarity[MUSB_MAX_CONTROLLERS] ={eUSB0OCEnablePolarity,eUSB1OCEnablePolarity,eUSB2OCEnablePolarity,eUSB3OCEnablePolarity};

	char usbportusing[64] = "usbportusing=";
	char usbpwrgpio[64] = "usbpwrgpio=";
//	char usbpwrpolarity[32] = "usbpwrpolarity=";
    #ifdef CC_USB_DELAY_RESUME_SUPPORT
	char usbdelayresume[64] = "usbdelayresume=";
    #endif
	char usbocgpio[64] = "usbocgpio=";
//	char usbocgpiopolarity[32] = "usbocgpiopolarity=";
    char usbhubgpio[64] = "usbhubrstgpio=";

    for(i = 0; i < MUSB_MAX_CONTROLLERS; i++)
    	{
    		if(0 == DRVCUST_InitQuery(_i4usbportusing[i], &u4UsbPortConfig))
    			{
				snprintf(pstr, 11, "%d",u4UsbPortConfig);
				strncat(usbportusing,pstr, strlen(pstr));
    		if( 0 ==DRVCUST_InitQuery(_i4usbpwrgpio[i], &u4UsbPortConfig))
    			{
					snprintf(pstr, 11,"%d",u4UsbPortConfig);
					strncat(usbpwrgpio,pstr,strlen(pstr) );
    			}
			else
				{
					strncat(usbpwrgpio,"-1", strlen("-1"));
				}
    			}
			else
				{
					strncat(usbpwrgpio,"-1", 2);
					strncat(usbportusing,"-1", 2);
				}
            #ifdef CC_USB_DELAY_RESUME_SUPPORT
			if( 0 == DRVCUST_InitQuery(_i4usbdelayresume[i], &u4UsbPortConfig))
			    {
				    snprintf(pstr, 11,"%d",u4UsbPortConfig);
					strncat(usbdelayresume,pstr, strlen(pstr));
			    }
			else
			    {
					strncat(usbdelayresume,"-1", 2);
			    }
            #endif
			if( 0==DRVCUST_InitQuery(_i4usbpwrpolarity[i], &u4UsbPortConfig))
				{
					snprintf(pstr, 11,"%d",u4UsbPortConfig);
					strncat(usbpwrgpio,":", strlen(":"));
					strncat(usbpwrgpio,pstr, strlen(pstr));
				}
			else
				{
					strncat(usbpwrgpio,":", strlen(":"));
					strncat(usbpwrgpio,"-1", 2);
				}

			if ( 0==DRVCUST_InitQuery(_i4usbocgpio[i], &u4UsbPortConfig))
				{
					snprintf(pstr, 11,"%d",u4UsbPortConfig);
					strncat(usbocgpio,pstr, strlen(pstr));
				}
			else
				{
					strncat(usbocgpio,"-1", 2);
				}

			if(0 ==DRVCUST_InitQuery(_i4usbocgpiopolarity[i], &u4UsbPortConfig))
				{
					snprintf(pstr, 11,"%d",u4UsbPortConfig);
					strncat(usbocgpio,":", strlen(":"));
					strncat(usbocgpio,pstr, strlen(pstr));
				}

			else
				{
					strncat(usbocgpio,":", strlen(":"));
					strncat(usbocgpio,"-1", strlen("-1"));
				}
			if(i !=MUSB_MAX_CONTROLLERS-1)
				{

					strncat(usbportusing,",", strlen(","));
					strncat(usbpwrgpio,",", strlen(","));
                    #ifdef CC_USB_DELAY_RESUME_SUPPORT
                    strncat(usbdelayresume,",", strlen(","));
                    #endif
				//	strncat(usbpwrpolarity,",");
					strncat(usbocgpio,",", strlen(","));
				//	strcat(usbocgpiopolarity,",");

				}

    	}

        if(0 == DRVCUST_InitQuery(eUSBHubResetGpio, &u4UsbPortConfig))
        {
            snprintf(pstr, 11,"%d",u4UsbPortConfig);
            strncat(usbhubgpio,pstr, strlen(pstr));
	    }
		else
		{
		    strncat(usbhubgpio,"-1", strlen("-1"));
		}

		if(0 == DRVCUST_InitQuery(eUSBHubResetPolarity, &u4UsbPortConfig))
        {
            snprintf(pstr, 11,":%d",u4UsbPortConfig);
            strncat(usbhubgpio,pstr, strlen(pstr));
	    }
		else
		{
		    strncat(usbhubgpio,":-1", strlen(":-1"));
		}
	//printf("%s,\n",	usbportusing);
	//printf("%s,\n",	usbpwrgpio);
	//printf("%s,\n",	usbpwrpolarity);
	//printf("%s,\n",	usbocgpio);
	//printf("%s,\n",	usbocgpiopolarity);

    env_add_bootargs("usbportusing=", usbportusing);
    env_add_bootargs("usbpwrgpio=", usbpwrgpio);
    #ifdef CC_USB_DELAY_RESUME_SUPPORT
	env_add_bootargs("usbdelayresume=", usbdelayresume);
    #endif
   // env_add_bootargs("usbpwrpolarity=", usbpwrpolarity);
    env_add_bootargs("usbocgpio=", usbocgpio);
  //  env_add_bootargs("usbocgpiopolarity=", usbocgpiopolarity);
    /* for cdc support, default support usb port0 */
    //env_add_bootargs("usbcdcsupport=", "usbcdcsupport=1,0,0,0");
    env_add_bootargs("usbhubrstgpio=", usbhubgpio);
}
#endif

#ifdef CC_UBOOT_TRANSFER_MSDCGPIO_KERNEL
#define MSDC_GPIO_MAX_NUMBERS 6

static void env_add_msdc_gpio(void)
{
	int i;
	int u4MsdcGpioConfig;
	char pstr[11];
	int _i4MSDCgpio[MSDC_GPIO_MAX_NUMBERS]={eMSDC0DetectGpio,eMSDC0WriteProtectGpio,eMSDC0PoweronoffDetectGpio,eMSDC0VoltageSwitchGpio,eMSDCbackup1Gpio,eMSDCbackup2Gpio};

	char msdcgpio[32] = "msdcgpio=";

    for(i = 0; i < MSDC_GPIO_MAX_NUMBERS; i++)
    {
		if( 0 ==DRVCUST_InitQuery(_i4MSDCgpio[i], &u4MsdcGpioConfig))
		{
			snprintf(pstr, 11,"%d",u4MsdcGpioConfig);
			strncat(msdcgpio,pstr, strlen(pstr) );
		}
		else
		{
			strncat(msdcgpio,"-1", strlen("-1"));
		}

		if(i !=MSDC_GPIO_MAX_NUMBERS-1)
		{
			strncat(msdcgpio,",", strlen(","));
		}
    }
	printf("%s,\n",msdcgpio);
    env_add_bootargs("msdcgpio=",msdcgpio);
}
#endif

static void env_add_wlan_gpio(void)
{
    int i4ResetWLangpio;
    char pstr[11];
    char rstwlan_gpio[32] = "rstwlan_gpio=";

    if (DRVCUST_InitQuery(eResetWLanGpio, &i4ResetWLangpio) == 0)
    {
        snprintf(pstr, 11,"%d", i4ResetWLangpio);
        strncat(rstwlan_gpio, pstr, strlen(pstr));
    }
    else
    {
        strncat(rstwlan_gpio, "-1", strlen("-1"));
    }
    env_add_bootargs("rstwlan_gpio=", rstwlan_gpio);
}

#if CC_UBOOT_TRANSFER_MSDC_ETT_KERNEL
#define MSDC_ETT_NUMBERS 5
extern uint szETT_CmdLine[];

void env_add_msdc_ett(void)
{
	int i;
	int u4MsdcGpioConfig;
	char pstr[32];

	printf("...=====MSDC Add ETT Param=====...\n");

	char ett[50 + MSDC_ETT_NUMBERS * 10] = "msdcett=";

    for (i = 0; i < MSDC_ETT_NUMBERS; i++)
    {
        printf("[%d] = 0x%x\n", i, szETT_CmdLine[i]);
		snprintf(pstr, sizeof(pstr), "0x%x", szETT_CmdLine[i]);
		strncat(ett, pstr, strlen(pstr));

		if (i !=MSDC_ETT_NUMBERS - 1)
			strncat(ett, ",", strlen(","));
    }
	printf("\n %s \n", ett);
    env_add_bootargs("msdcett=", ett);
}
#endif


#if CC_UBOOT_TRANSFER_MSDC_AUTOK_KERNEL
#define MSDC_AUTOK_NUMBERS 17
extern uint szAutoK_CmdLine[];

void env_add_msdc_autok(void)
{
	int i;
	int u4MsdcGpioConfig;
	char pstr[32];

	printf("...=====MSDC Add Autok Param=====...\n");

	char autok[50+MSDC_AUTOK_NUMBERS*10] = "msdcautok=";

    for(i = 0; i < MSDC_AUTOK_NUMBERS; i++)
    {
        printf("[%d] = 0x%x\n",i,szAutoK_CmdLine[i]);
		snprintf(pstr,sizeof(pstr), "0x%x",szAutoK_CmdLine[i]);
		strncat(autok,pstr, strlen(pstr));

		if(i !=MSDC_AUTOK_NUMBERS-1)
		{
			strncat(autok,",", strlen(","));
		}
    }
	printf("\n %s \n",autok);
    env_add_bootargs("msdcautok=",autok);
}
#endif

#ifdef CC_TRUSTZONE_SUPPORT
static void env_add_tz_size(void)
{
    char *p;
    char buf[32];
    p = getenv("tzsize");
    snprintf(buf,sizeof(buf),"tzsz=%s", p);
    env_add_bootargs("tzsz=", buf);
}
#endif

#if !defined(ANDROID) && defined(CC_SONY_FILE_BASED_RECIPE_SUPPORT)
static void env_add_firmware_path(void)
{
    char buf[32];
    snprintf(buf,sizeof(buf),"firmware_class.path=%s", "/recipe/data");
    env_add_bootargs("firmware_class.path=", buf);
}
#endif

#if defined(CC_CMDLINE_SUPPORT_BOOTREASON)
#if defined(CC_MT5893) || defined(CC_MT5895)
#define PDWNC_REG_RESERVE_BOOTREASON  ((volatile UINT32 *)(PDWNC_RESRV5))
#else
#define PDWNC_REG_RESERVE_BOOTREASON  ((volatile UINT32 *)0xF0028624)
#endif
static void env_add_bootreason(void)
{
	char newstr[64] = {0};
	UINT32 wakeupReason = *(PDWNC_REG_RESERVE_BOOTREASON);

	// clear boot reason register
	*(PDWNC_REG_RESERVE_BOOTREASON) = wakeupReason & 0xFFFFFF00;
	if ((wakeupReason & 0xFF) == 0x1)
	{
		snprintf(newstr, sizeof(newstr), "androidboot.bootreason=kernel_panic");
	}
	else if ((wakeupReason & 0xFF) == 0x2)
	{
		snprintf(newstr, sizeof(newstr), "androidboot.bootreason=reboot");
	}
	else if ((wakeupReason & 0xFF) == 0x3)
	{
		snprintf(newstr, sizeof(newstr), "androidboot.bootreason=watchdog");
	}
	else
	{
		// normal boot
		return;
	}
	env_add_bootargs("androidboot.bootreason=", newstr);
}
#endif

#ifdef ANDROID
#ifdef CC_UBOOT_MINIGUNZIP_SUPPORT
static void env_add_minigzip(void)
{
    char newstr[32] = {0};

    snprintf(newstr, sizeof(newstr), "minigzip=1");
    env_add_bootargs("minigzip=", newstr);
}
#endif
#endif
#ifdef CC_SUPPORT_STR
static void env_add_no_console_suspend(void)
{
    char newstr[32] = {0};

    snprintf(newstr, sizeof(newstr),"no_console_suspend");
    env_add_bootargs("no_console_suspend", newstr);
}
#endif
#ifdef CC_ENABLE_CMA_SUPPORT
static void env_add_cma(void)
{
    char newstr[32] = {0};

    snprintf(newstr,sizeof(newstr), "cma=0");
    env_add_bootargs("cma=", newstr);
}
#endif
#ifdef CC_PDWNC_REBOOT_NOTIFY
static void env_add_recovery_mode(void)
{
    char newstr[32] = {0};

    if (BIM_IsRecoveryMode())
    {
        snprintf(newstr, sizeof(newstr),"AndroidRecovery");
        env_add_bootargs("AndroidRecovery", newstr);
    }
}
#endif

#ifdef CC_CUSTOM_PMIC_I2C_SUPPORT
#define MTK_PMIC_I2C_NUMS 18
void env_add_pmic_i2c(void)
{
	int i;
	int u4PMICConfig;
	char pstr[32];
	int _i4pmicusing[MTK_PMIC_I2C_NUMS] =
	{
	eGetVCPUdevaddr,
	eGetVCPUdataddr,
	eGetVCPUmodaddr,
	eGetVCPUmodmask,
	eGetVCPUmodepwm_en,
	eGetVCPUvmin,
	eGetVCPUvmax,
	eGetVCPUvstep,
	eGetVCPUchannelid,
	eGetVGPUdevaddr,
	eGetVGPUdataddr,
	eGetVGPUmodaddr,
	eGetVGPUmodmask,
	eGetVGPUmodepwm_en,
	eGetVGPUvmin,
	eGetVGPUvmax,
	eGetVGPUvstep,
	eGetVGPUchannelid};

	printf("...=====PMIC I2C Param=====...\n");

	char mtk_regulator[50+MTK_PMIC_I2C_NUMS*4] = "mt53xxregulator=";

	for(i=0;i<MTK_PMIC_I2C_NUMS;i++)
	{
		if(0 == DRVCUST_InitQuery(_i4pmicusing[i], &u4PMICConfig))
		{
			snprintf(pstr, sizeof(pstr), "%d",u4PMICConfig);
			strncat(mtk_regulator,pstr, strlen(pstr) );
		}
		else
		{
			strncat(mtk_regulator,"-1", strlen("-1"));
		}
		strncat(mtk_regulator,":", strlen(":"));
	}

	printf("\n %s \n",mtk_regulator);
    env_add_bootargs("mt53xxregulator=",mtk_regulator);
}
void env_add_disable_dvfs(void)
{
	char newstr[32] = {0};

	#ifdef CC_S_PLATFORM
		#if 0 //def CC_MT5895  // tem sulotion to disable dvfs due to HW is not ready
		sprintf(newstr, "disable_dvfs=1");
		#else
		//sprintf(newstr, "disable_dvfs=0");//Mark off for DTV03021096
		#endif
	#else
	snprintf(newstr, sizeof(newstr), "disable_dvfs=0");
	#endif
	//env_add_bootargs("disable_dvfs=", newstr);//Mark off for DTV03021096
}
#endif

extern int DRVCUST_QueryPart(int i, int *pData, int type);
extern int DRVCUST_QueryPart64(int i, u64 *pData, int type);

static void set_cust_env(void)
{
#ifndef CC_KERNEL_ON_NOR
    int i = 0, i4PartNum = 0, encrypt = 0;
    u64 size = 0;
    char *p1 = NULL, *p2 = NULL;
    char *name = NULL;
    char buf[64] = {0};
    char *mtdparts = NULL;
    char *bootargs = NULL;
#ifdef CC_PARTITION_WP_SUPPORT
    int wp = 0;
#endif

    mtdparts = (char *)malloc(default_environment_size);
    memset(mtdparts, 0, default_environment_size);

    bootargs = (char *)malloc(default_environment_size);
    memset(bootargs, 0, default_environment_size);

    DRVCUST_QueryPart(0, &i4PartNum, 0);

    // generate mtdparts string
    while (i <= i4PartNum)
    {
        DRVCUST_QueryPart64(i, &size, 1);
        DRVCUST_QueryPart(i, (UPTR *)&name, 2);
        DRVCUST_QueryPart(i, &encrypt, 3);
#ifdef CC_PARTITION_WP_SUPPORT
        DRVCUST_QueryPart(i, &wp, 4);
#endif
        if (!size)
        {
            break;
        }

        if (i == 0)
        {
#if defined(CC_EMMC_BOOT)
            strncat(mtdparts, "mt53xx-emmc:", strlen("mt53xx-emmc:"));
#else
            strncat(mtdparts, "mt53xx-nand:", strlen("mt53xx-emmc:"));
#endif
        }
        else
        {
            strncat(mtdparts, ",", strlen(","));
        }

		/* CAUTION: 64 bit size limition 2TB !!! */
        if ((size % (1024 * 1024)) == 0)
        {
            snprintf(buf, sizeof(buf), "%dM(%s)", (u32)(size /1024 /1024), name);
        }
        else if ((size % 1024) == 0)
        {
            snprintf(buf,  sizeof(buf), "%dk(%s)", (u32)(size /1024), name);
        }
        else
        {
            BUG();
        }

        strncat(mtdparts, buf, strlen(buf));

        if (encrypt)
        {
            strncat(mtdparts, "enc", strlen("enc"));
        }
#ifdef CC_PARTITION_WP_SUPPORT
        if (wp)
        {
            strncat(mtdparts, "wp", strlen("wp"));
        }
#endif

        i++;
    }

    strncpy(mtdparts_default, mtdparts, default_environment_size - 1 );

    // update mtdparts string
    setenv("mtdparts", mtdparts);

    // generate bootargs string
    p1 = getenv("bootargs");
    if (p1 == NULL)
    {
        free(mtdparts);
        free(bootargs);
        return;
    }

    strncpy(bootargs, p1, strlen(p1) + 1);

    // delete any existing mtdparts string in bootargs string.
    p1 = strstr(bootargs, "mtdparts=");
    if (p1)
    {
        p2 = strstr(p1, " ");
        if(p2)
        {
            while(*++p2)
            {
                *p1++ = *p2;
            }

            *p1++ = ' ';
        }

        *p1 = '\0';
    }
    else
    {
        strncat(bootargs, " ", strlen(" "));
    }

    // append new mtdparts string at the end of bootargs string.
    strncat(bootargs, "mtdparts=", strlen("mtdparts="));
    strncat(bootargs, mtdparts, strlen(mtdparts));

    //For DTV03021096
    strncat(bootargs, "  disable_dvfs=0", strlen("  disable_dvfs=0"));

    // update bootargs environment
    setenv("bootargs", bootargs);

    // set kernel memory size, kmemsize always is not equal to zero.
    if (kmemsize)
    {
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize);
        setenv("kmemsize", buf);
#ifdef CC_FBM_DYNMJC
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize_nomjc);
        setenv("kmemsize_nomjc", buf);
#endif
#ifdef CC_TWO_MEM_REGION_BANK
        snprintf(buf,  sizeof(buf), "0x%x%08x", kmemstart2_high,kmemstart2_low);
        setenv("kmemstart2", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2);
        setenv("kmemsize2", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2_4G);
        setenv("kmemsize2_4G", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2_3_5G);
        setenv("kmemsize2_3_5G", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2_3G);
        setenv("kmemsize2_3G", buf);
    #ifdef CC_MEM_SIZE_LARGE_THAN_4G
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2_5G);
        setenv("kmemsize2_5G", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmemsize2_6G);
        setenv("kmemsize2_6G", buf);
    #endif
#endif
#if defined(CC_DYNAMIC_FBMSRM_CONF)
		snprintf(buf,  sizeof(buf), "0x%x", kmem4ksize);
		setenv("kmem4ksize", buf);
		snprintf(buf,  sizeof(buf), "0x%x", kmemfhdsize);
		setenv("kmemfhdsize", buf);
#endif
#if CONFIG_NR_DRAM_BANKS>1
        snprintf(buf,  sizeof(buf), "0x%x", kmem2start);
        setenv("kmem2start", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmem2size);
        setenv("kmem2size", buf);
#if defined(CC_DYNAMIC_FBMSRM_CONF)
        snprintf(buf,  sizeof(buf), "0x%x", kmem24ksize);
        setenv("kmem24ksize", buf);
        snprintf(buf,  sizeof(buf), "0x%x", kmem2fhdsize);
        setenv("kmem2fhdsize", buf);
        snprintf(buf,  sizeof(buf), "%d", gpu4kstart);
        setenv("gpu4kstart", buf);
        snprintf(buf,  sizeof(buf), "%d", gpufhdstart);
        setenv("gpufhdstart", buf);
        snprintf(buf,  sizeof(buf), "%d", gpu4ksize);
        setenv("gpu4ksize", buf);
        snprintf(buf,  sizeof(buf), "%d", gpufhdsize);
        setenv("gpufhdsize", buf);
        snprintf(buf,  sizeof(buf), "%d", gpu4kionsize);
        setenv("gpu4kionsize", buf);
        snprintf(buf,  sizeof(buf), "%d", gpufhdionsize);
        setenv("gpufhdionsize", buf);
#endif
#endif
#ifdef CC_SUPPORT_CHANNEL_C
#ifdef CC_MT5891
	if(BSP_GetIcVersion() != IC_VER_5891_AA)
		{
			snprintf(buf,  sizeof(buf), "0x%x", kmem3start);
			setenv("kmem3start", buf);
			snprintf(buf,  sizeof(buf), "0x%x", kmem3size);
			setenv("kmem3size", buf);
		}
#else
	snprintf(buf,  sizeof(buf), "0x%x", kmem3start);
	setenv("kmem3start", buf);
	snprintf(buf,  sizeof(buf), "0x%x", kmem3size);
	setenv("kmem3size", buf);
#endif
#endif

    }
	
#ifdef CC_RAMOOPS_SUPPORT
	//RamoopsMemAddress always is not equal to zero.
#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
	if (RamoopsMemAddress&&RamoopsMemAddress_nomjc)
#else
	if (RamoopsMemAddress)
#endif
	{
#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
		snprintf(buf,  sizeof(buf), "0x%x", RamoopsMemAddress_nomjc);
		setenv("ramoops.mem_address_nomjc", buf);
		snprintf(buf,  sizeof(buf), "0x%x", RamoopsMemAddress);
		setenv("ramoops.mem_address", buf);
#else
		snprintf(buf,  sizeof(buf), "0x%x", RamoopsMemAddress);
		setenv("ramoops.mem_address", buf);
#endif

		snprintf(buf, sizeof(buf), "0x%x",RAMOOP_MEM_SIZE);
		setenv("ramoops.mem_size", buf);
		
		snprintf(buf, sizeof(buf), "0x%x",RAMOOP_CONSOLE_SIZE);
		setenv("ramoops.console_size", buf);
		
		snprintf(buf, sizeof(buf), "0x%x",RAMOOP_FTRACE_SIZE);
		setenv("ramoops.ftrace_size", buf);
		
		snprintf(buf, sizeof(buf), "0x%x",RAMOOP_PMSG_SIZE);
		setenv("ramoops.pmsg_size", buf);
		
		snprintf(buf, sizeof(buf), "0x%x",RAMOOP_RECORD_SIZE);
		setenv("ramoops.record_size", buf);
	}
#endif

#ifdef CC_TRUSTZONE_SUPPORT
    if(TRUSTZONE_MEM_SIZE%(1024*1024) == 0)
    {
        snprintf(buf, sizeof(buf), "%dm", TRUSTZONE_MEM_SIZE/(1024*1024));
    }
    else if(TRUSTZONE_MEM_SIZE%1024 == 0)
    {
        snprintf(buf, sizeof(buf), "%dk", TRUSTZONE_MEM_SIZE/(1024));
    }
    else
    {
        snprintf(buf,  sizeof(buf), "%d", TRUSTZONE_MEM_SIZE);
    }
    setenv("tzsize", buf);
#endif

#ifdef CC_ANDROID_SECURE_BOOT_CLASS_B
#ifdef CC_ANDROID_SECURE_BOOT_UNLOCK
    setenv("devicestate", "unlock");
#else
    setenv("devicestate", "lock");
#endif
#endif//CC_ANDROID_SECURE_BOOT_CLASS_B

    free(mtdparts);
    free(bootargs);
#endif
}

void set_default_env(const char *s)
{
	unsigned int* penv;

	if (sizeof(default_environment) > ENV_SIZE) {
		puts("\n*** Error - default environment is too large.\n\n");
		return;
	}

	if (s) {
		if (*s == '!') {
			printf("\n*** Warning - %s, using default environment.\n\n", s+1);
		} else {
			puts(s);
		}
	} else {
		puts("\nUsing default environment.\n\n");
	}

	penv = (unsigned int*)env_ptr;
	printf("\n #################### set_default_env penv[0] = 0x%x.\n", penv[0]);
	//get ATAG kernel memory from env.o
	if (penv[0]==0x464c457f)
	{
		int i=1;
		for (; i<0x40; i++)
		{
			// find magic key, the next work is global_kernel_dram_size
			if (penv[i] == 0x53965368)
			{
		#if defined(CC_DYNAMIC_FBMSRM_CONF)
				kmemsize = penv[i+1];
				kmem4ksize = penv[i+2];
				kmemfhdsize = penv[i+3];
			#if CONFIG_NR_DRAM_BANKS>1
				kmem2start = penv[i+4];
				kmem2size = penv[i+5];
				kmem24ksize = penv[i+6];
				kmem2fhdsize = penv[i+7];
			#endif
				gpu4kstart = penv[i+8];
				gpufhdstart = penv[i+9];
				gpu4ksize = penv[i+10];
				gpufhdsize = penv[i+11];
				gpu4kionsize = penv[i+12];
				gpufhdionsize = penv[i+13];
			#ifdef CC_SUPPORT_CHANNEL_C
				kmem3start = penv[i+14];
				kmem3size = penv[i+15];
			#endif
			#ifdef CC_RAMOOPS_SUPPORT
				RamoopsMemAddress = penv[i+16];
			#endif
		#else
				kmemsize = penv[i+1];
			#if CONFIG_NR_DRAM_BANKS>1
				kmem2start = penv[i+2];
				kmem2size = penv[i+3];

				#ifdef CC_SUPPORT_CHANNEL_C
				kmem3start = penv[i+4];
				kmem3size = penv[i+5];
				#endif

				#ifdef CC_RAMOOPS_SUPPORT
					#ifndef CC_SUPPORT_CHANNEL_C 
				RamoopsMemAddress = penv[i+4];
					#else
				RamoopsMemAddress = penv[i+6];
					#endif
				#endif
			#else
				#ifdef CC_TWO_MEM_REGION_BANK
					kmemstart2_high = penv[i+2];
					kmemstart2_low = penv[i+3];
					kmemsize2 = penv[i+4];
					kmemsize2_4G = penv[i+5];
					kmemsize2_3_5G = penv[i+6];
					kmemsize2_3G = penv[i+7];
					#ifdef CC_MEM_SIZE_LARGE_THAN_4G
						kmemsize2_5G = penv[i+8];
						kmemsize2_6G = penv[i+9];
						#ifdef CC_FBM_DYNMJC
						kmemsize_nomjc = penv[i+10];
						#endif
					#else
						#ifdef CC_FBM_DYNMJC
						kmemsize_nomjc = penv[i+8];
						#endif
					#endif
				#else
					#ifdef CC_FBM_DYNMJC
					kmemsize_nomjc = penv[i+2];
					#endif
				#endif

				#ifdef CC_RAMOOPS_SUPPORT
					#ifndef CC_TWO_MEM_REGION_BANK
						#ifdef CC_FBM_DYNMJC
						RamoopsMemAddress = penv[i+3];
						#ifdef CC_MT5895
						RamoopsMemAddress_nomjc = penv[i+4];
						#endif
						#else
						RamoopsMemAddress = penv[i+2];
						#endif
					#else
						#ifdef CC_MEM_SIZE_LARGE_THAN_4G
							#ifdef CC_FBM_DYNMJC
							RamoopsMemAddress = penv[i+11];
							#ifdef CC_MT5895
							RamoopsMemAddress_nomjc = penv[i+12];
							#endif
							#else
							RamoopsMemAddress = penv[i+10];
							#endif
						#else
							#ifdef CC_FBM_DYNMJC
							RamoopsMemAddress = penv[i+9];
							#ifdef CC_MT5895
							RamoopsMemAddress_nomjc = penv[i+10];
							#endif
							#else
							RamoopsMemAddress = penv[i+8];
							#endif
						#endif
					#endif
				#endif
			#endif
		#endif
				printf("set_default_env from env partition !!!\n\n kmemsize\t= 0x%08x\n kmemstart2\t= 0x%x%08x\n kmemsize2\t= 0x%08x\n kmem2start\t= 0x%08x\n kmem2size\t= 0x%08x\n kmem3start\t= 0x%08x\n kmem3size\t= 0x%08x\n\n", kmemsize, kmemstart2_high, kmemstart2_low, kmemsize2, kmem2start, kmem2size,kmem3start,kmem3size);

				break;
			}
		}
	}

	//if get ATAG kernel memory from env.o fail, force initial
	if(kmemsize == 0)
	{
#ifndef CC_FBM_DYNMJC
		kmemsize = PHYS_SDRAM_1_SIZE;
#else
		#ifndef CC_MT5895
		kmemsize = PHYS_SDRAM_1_SIZE;
		#else
		kmemsize = PHYS_SDRAM_1_SIZE_MJC;
		#endif
		kmemsize_nomjc = PHYS_SDRAM_1_SIZE_NOMJC;
#endif
#ifdef CC_TWO_MEM_REGION_BANK
		kmemstart2_high = (PHYS_SDRAM_1_2 >> 32);
		kmemstart2_low = (PHYS_SDRAM_1_2 & 0xffffffff);
		kmemsize2 = PHYS_SDRAM_1_2_SIZE;
		kmemsize2_4G = PHYS_SDRAM_1_2_SIZE_4G;
		kmemsize2_3_5G = PHYS_SDRAM_1_2_SIZE_3_5G;
		kmemsize2_3G = PHYS_SDRAM_1_2_SIZE_3G;
		#ifdef CC_MEM_SIZE_LARGE_THAN_4G
		kmemsize2_5G = PHYS_SDRAM_1_2_SIZE_5G;
		kmemsize2_6G = PHYS_SDRAM_1_2_SIZE_6G;
		#endif
#endif
#if defined(CC_DYNAMIC_FBMSRM_CONF)
		kmem4ksize = PHYS_SDRAM_1_4K_SIZE;
		kmemfhdsize = PHYS_SDRAM_1_FHD_SIZE;

	#if CONFIG_NR_DRAM_BANKS>1
		kmem2start = PHYS_SDRAM_2;
		kmem2size = PHYS_SDRAM_2_SIZE;
		kmem24ksize = PHYS_SDRAM_2_4K_SIZE;
		kmem2fhdsize = PHYS_SDRAM_2_FHD_SIZE;
	#endif
	#if defined(PHYS_GPU_4K_START)&&defined(PHYS_GPU_FHD_START)
		gpu4kstart = PHYS_GPU_4K_START;
		gpufhdstart = PHYS_GPU_FHD_START;
		gpu4ksize = PHYS_GPU_4K_SIZE;
		gpufhdsize = PHYS_GPU_FHD_SIZE;
		gpu4kionsize = PHYS_GPU_4K_ION_SIZE;
		gpufhdionsize = PHYS_GPU_FHD_ION_SIZE;
	#endif
#else
	#if CONFIG_NR_DRAM_BANKS>1
		kmem2start = PHYS_SDRAM_2;
		kmem2size = PHYS_SDRAM_2_SIZE;

		#ifdef CC_SUPPORT_CHANNEL_C
		kmem3start = PHYS_SDRAM_3;
		kmem3size = PHYS_SDRAM_3_SIZE;
		#endif
	#endif
#endif
	#ifdef CC_RAMOOPS_SUPPORT
		RamoopsMemAddress = RAMOOP_MEM_ADDRESS;
		#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
		RamoopsMemAddress_nomjc = RAMOOP_MEM_ADDRESS_NOMJC;
		#endif
	#endif

		printf("set_default_env from macro definition !!!\n\n kmemsize\t= 0x%08x\n kmemstart2\t= 0x%x%08x\n kmemsize2\t= 0x%08x\n kmem2start\t= 0x%08x\n kmem2size\t= 0x%08x\n kmem3start\t= 0x%08x\n kmem3size\t= 0x%08x\n\n", kmemsize, kmemstart2_high, kmemstart2_low, kmemsize2, kmem2start, kmem2size,kmem3start,kmem3size);
	}

	if (himport_r(&env_htab, (char *)default_environment,
			sizeof(default_environment), '\0', 0) == 0)
		error("Environment import failed: errno = %d\n", errno);

	gd->flags |= GD_FLG_ENV_READY;

	set_cust_env();
	saveenv();
}

/*
 * Check if CRC is valid and (if yes) import the environment.
 * Note that "buf" may or may not be aligned.
 */
int env_import(const char *buf, int check)
{
	env_t *ep = (env_t *)buf;

	if (check) {
		uint32_t crc;

		memcpy(&crc, &ep->crc, sizeof(crc));

		if (crc32(0, ep->data, ENV_SIZE) != crc) {
			set_default_env("!bad CRC");
			return 0;
		}
	}

	if (himport_r(&env_htab, (char *)ep->data, ENV_SIZE, '\0', 0)) {
		gd->flags |= GD_FLG_ENV_READY;
		return 1;
	}

	error("Cannot import environment: errno = %d\n", errno);

	set_default_env("!import failed");

	return 0;
}
#if defined(CC_SECURE_ROM_BOOT)
int IsRunOnUsb(char* uenv, int uenv_size);

static void env_add_usig(void)
{
    char *p1 = NULL;
    char uenv[125];
    char *bootargs = (char *)malloc(default_environment_size);

    if (IsRunOnUsb((char *)&uenv, sizeof(uenv)))
    {
        // generate bootargs string
        p1 = getenv("bootargs");
        if (p1 == NULL)
        {
            free(bootargs);
            return;
        }
        printf("run_on_usb, uenv=%s\n", uenv);
        strncpy(bootargs, p1, default_environment_size - 1);
        // append new string at the end of bootargs string.
        strncat(bootargs, " ", strlen(" "));
        strncat(bootargs, uenv, strlen(uenv));
        // update bootargs environment
        setenv("bootargs", bootargs);
        setenv("usigenv", uenv);
    }
    free(bootargs);

}
#endif // CC_SECURE_ROM_BOOT

#if defined(CC_RAMOOPS_SUPPORT)

static void env_add_ramoop(void)
{
	char newstr[64] = {0};
#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
    int is_support_mjc = ((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->u4SupportMJC;
#endif
#if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
	if(!is_support_mjc)
	{
		snprintf(newstr,  sizeof(newstr), "ramoops.mem_address=%s", getenv("ramoops.mem_address_nomjc"));
		env_add_bootargs("ramoops.mem_address=", newstr);
	}
	else
	{
		snprintf(newstr, sizeof(newstr), "ramoops.mem_address=%s", getenv("ramoops.mem_address"));
		env_add_bootargs("ramoops.mem_address=", newstr);
	}
#else
	snprintf(newstr, sizeof(newstr), "ramoops.mem_address=%s", getenv("ramoops.mem_address"));
	env_add_bootargs("ramoops.mem_address=", newstr);
#endif

	snprintf(newstr,sizeof(newstr), "ramoops.mem_size=%s",getenv("ramoops.mem_size"));
	env_add_bootargs("ramoops.mem_size=", newstr);

	snprintf(newstr,sizeof(newstr), "ramoops.console_size=%s",getenv("ramoops.console_size"));
	env_add_bootargs("ramoops.console_size=", newstr);

	snprintf(newstr,sizeof(newstr), "ramoops.ftrace_size=%s",getenv("ramoops.ftrace_size"));
	env_add_bootargs("ramoops.ftrace_size=", newstr);

	snprintf(newstr,sizeof(newstr), "ramoops.pmsg_size=%s",getenv("ramoops.pmsg_size"));
	env_add_bootargs("ramoops.pmsg_size=", newstr);

	snprintf(newstr,sizeof(newstr), "ramoops.record_size=%s",getenv("ramoops.record_size"));
	env_add_bootargs("ramoops.record_size=", newstr);
}

#endif // CC_SECURE_ROM_BOOT
#ifdef CC_DTB_IN_BOOTIMG
void env_add_boot_dtb_index(char* index_string)
{
    const char boot_dtb_prefix[] = "androidboot.dtb_idx=";
    char boot_dtb_args[64] = {0};

    strncpy(boot_dtb_args,boot_dtb_prefix, strlen(boot_dtb_prefix));
    strncat(boot_dtb_args,index_string, strlen(index_string));
    printf("DTB bootargs: %s\n", boot_dtb_args);
    env_add_bootargs(boot_dtb_prefix, boot_dtb_args);
}
#endif
#if defined(ANDROID) && defined(CC_SUPPORT_DTO)
void env_add_boot_dtbo_index(char* index_string)
{
    const char boot_dtbo_prefix[] = "androidboot.dtbo_idx=";
    char boot_dtbo_args[64] = {0};

    strncpy(boot_dtbo_args,boot_dtbo_prefix, strlen(boot_dtbo_prefix));
    strncat(boot_dtbo_args,index_string, strlen(index_string));
    env_add_bootargs(boot_dtbo_prefix, boot_dtbo_args);
}
#endif

#ifdef CC_SUPPORT_AB_UPDATE
void env_add_boot_slot_suffix(char* slot_suffix)
{
    const char boot_slot_prefix[] = "androidboot.slot_suffix=";
    char boot_slot_args[64] = {0};

    strncpy(boot_slot_args,boot_slot_prefix,strlen(boot_slot_prefix));
    strncat(boot_slot_args,slot_suffix, strlen(slot_suffix));
    env_add_bootargs(boot_slot_prefix, boot_slot_args);
}

void env_add_boot_skip_initramfs(void)
{
    const char boot_skip_initramfs[] = "skip_initramfs";
    env_add_bootargs(boot_skip_initramfs, boot_skip_initramfs);
}
void env_del_skip_initramfs(void)
{
    env_del_bootargs("skip_initramfs", NULL);
}
#endif

void env_add_boot_kernel_cmdline(char* kernel_cmdline)
{
    
    env_add_bootargs(NULL, kernel_cmdline);
}

#ifdef CC_SUPPORT_AVB
void env_add_boot_avb_cmdline(char* avb_cmdline)
{
    printf("\n## AVB2.0 append extra bootargs:\n%s\n\n",avb_cmdline);
    env_add_bootargs(NULL, avb_cmdline);
}
#endif

#ifdef CC_S_PLATFORM
static void env_add_dolby_vision_support(void)
{
    int val = 0;
    char str[32] = {0};
    EEPROM_Read((UINT64)0x1020C, (UINT32)&val, (UINT32)sizeof(char));
    snprintf(str, sizeof(str), "dolby_vision_support=%d", val);
    env_add_bootargs("dolby_vision_support=", str);
}
#endif

void env_relocate(void)
{
#ifdef CC_FBM_DYNMJC
    int is_support_mjc = ((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->u4SupportMJC;
#endif
#if defined(CONFIG_NEEDS_MANUAL_RELOC)
    env_reloc();
#endif

    if (gd->env_valid == 0) {
#if defined(CONFIG_ENV_IS_NOWHERE) /* Environment not changable */
        set_default_env(NULL);
#else
        show_boot_progress(-60);
        set_default_env("!bad CRC");
#endif
    } else {
        env_relocate_spec();
#ifndef CC_KERNEL_ON_NOR
        {
            char *p1 = NULL;

            p1 = getenv("mtdparts");
            if (p1 == NULL)
            {
                return;
            }
            strncpy(mtdparts_default, p1, 1023);
        }
#endif
    }

#ifdef CC_ENABLE_MTK_MODEL_INDEX
    env_add_panel_index();
#endif

#ifdef CC_ENABLE_MALI_CORE_NUM
    env_add_mali_core_num();
#endif

#ifdef CC_UBOOT_TRANSFER_USBGPIO_KERNEL
    env_add_usb_gpio();
#endif

#ifdef CC_S_PLATFORM
#ifdef CC_UBOOT_TRANSFER_UART_MODE
    env_add_uart_mode();
#endif
#endif

#ifdef CC_UBOOT_TRANSFER_MSDCGPIO_KERNEL
    env_add_msdc_gpio();
#endif

#ifdef CC_TRUSTZONE_SUPPORT
    env_add_tz_size();
#endif

#ifdef CC_FAST_INIT
    env_add_fastboot();
#endif

#ifdef CC_SUPPORT_STR
    env_add_no_console_suspend();
#endif

#ifdef CC_ENABLE_CMA_SUPPORT
    env_add_cma();
#endif

#ifdef ANDROID
#ifdef CC_UBOOT_MINIGUNZIP_SUPPORT
    env_add_minigzip();
#endif
#endif

#ifdef CC_PDWNC_REBOOT_NOTIFY
    env_add_recovery_mode();
#endif

#ifdef CC_MT53XX_SUPPORT_2G_DRAM
    //env_add_vmalloc_cust();
#endif

#ifdef CC_SECURE_ROM_BOOT
    env_add_usig();
#endif

#ifdef CONFIG_AMIGAONEG3SE
    disable_nvram();
#endif

#ifdef CC_KERNEL_NO_LOG
    char *print = getenv("print");
    if (print == NULL)
    {
        setenv("print", "off");
        print = getenv("print");
    }

    if (!strncmp(print, "off", strlen("off")))
    {
        env_add_forbid_uart();
    }
#endif

#if defined(CC_MT5895)
    env_add_uart_dma_addr_and_size();
#endif
    env_add_uart_perfor();

#ifdef CC_TUNER_DBG_LVL_BY_UBOOT
    /* set it by addboot cmd for debug feasibility */
    //env_add_tuner_dbg_lvl();
#endif

#ifdef CC_ENABLE_CMA_SUPPORT
    char *cmaCHA = getenv("cma_support_Cha");
    if (cmaCHA == NULL)
    {
        setenv("cma_support_Cha", "Y");
        cmaCHA = getenv("cma_support_Cha");
    }
    if (!strncmp(cmaCHA, "Y", strlen("Y")))
    {
        env_add_cmaCHA_endaddress();
    }
    else
    {
        env_del_cmaCHA_endaddress();
    }
#if CONFIG_NR_DRAM_BANKS > 1
    char *cmaCHB = getenv("cma_support_Chb");
    if (cmaCHB == NULL)
    {
        setenv("cma_support_Chb", "Y");
        cmaCHB = getenv("cma_support_Chb");
    }
    if (!strncmp(cmaCHB, "Y", strlen("Y")))
    {
        env_add_cmaCHB_endaddress();
    }
    else
    {
        env_del_cmaCHB_endaddress();
    }
#endif
#endif

#ifdef CC_MT5890
    env_add_have_vp9();
#endif

#ifdef CC_VP9_SUPPORT
    env_add_vp9_support();
#endif

    env_add_gpu_addr();
#ifndef CC_UBOOT_AUTO_TEST_ADB
    env_add_usb_serialno();
#endif

#ifdef CC_3RD_KO_INSERTION
    env_add_beic_support();
#endif

#ifdef CC_S_PLATFORM
    env_add_product_model();
    env_add_from_mvi();
    env_add_hard_lock_detector();
    env_add_usb_hub();
    env_add_ic_version();
#ifdef CC_DYNAMIC_USB_ADB_CONFIG
    env_add_adb_support();
#endif
#ifdef CC_USB3_TO_ADB_PORT_CONFIG
    env_add_u32adb_support();
#endif
#else
    env_add_adb_support();
#endif // CC_S_PLATFORM

#ifdef CC_SUPPORT_CHIPID
#if defined(ANDROID) || defined(CC_C4TV_SUPPORT)
    env_add_chipid();
#endif
#endif

#if defined(CC_CMDLINE_SUPPORT_BOOTREASON)
    env_add_bootreason();
#endif
#ifdef CC_CMDLINE_UPGRADE_STATUS
    if (((LDR_ENV_T *)CC_LDR_ENV_OFFSET)->u4UpgradeStatus == 1)
    {
        env_add_bootargs("upgrade_bit=", "upgrade_bit=1");
    }
#endif

#ifdef LPJ_IC_VER1
    if (BSP_GetIcVersion() == LPJ_IC_VER1)
    {
        env_add_bootargs("lpj=", LPJ_IC_VALUE1);
    }
#endif

#ifdef LPJ_IC_VER1
    if (BSP_GetIcVersion() == LPJ_IC_VER2)
    {
        env_add_bootargs("lpj=", LPJ_IC_VALUE2);
    }
#endif

#if defined(CC_UBOOT_ADD_USB_LOG_MODE)
    env_add_usb_log();
#endif

#ifdef CC_UBOOT_ADD_USB_ETH_MODE
    env_add_usb_eth();
#endif

#if CC_UBOOT_TRANSFER_MSDC_AUTOK_KERNEL
    env_add_msdc_autok();
#endif

#if CC_UBOOT_TRANSFER_MSDC_ETT_KERNEL
    env_add_msdc_ett();
#endif

#ifndef CC_UBOOT_WO_AUTOSTART
    setenv("autostart", "yes");
#endif

#ifdef CC_WIFI_25G_SUPPORT
    env_add_wifi_25g_support();
#endif

#ifdef SYS_C4TV_SUPPORT
    env_add_dual_partition();
#endif

#ifdef CC_UBOOT_AUTO_TEST_PERMISSIVE
    env_add_bootargs("androidboot.selinux=", "androidboot.selinux=permissive");
    printf("Set SELinux to permissive mode for AUTO_TEST!!\n");
#endif//CC_UBOOT_AUTO_TEST_PERMISSIVE

#if (!defined(NDEBUG)) || (!defined(CC_DISABLE_BASIC_PART))
    env_add_bootargs("androidboot.selinux=", "androidboot.selinux=permissive");
#endif

#if defined(CC_RAMOOPS_SUPPORT)
    env_add_ramoop();
#endif

#if defined(CC_CUSTOM_PMIC_I2C_SUPPORT)
    env_add_pmic_i2c();
    env_add_disable_dvfs();
#endif
    env_add_wlan_gpio();

#if !defined(ANDROID) && defined(CC_SONY_FILE_BASED_RECIPE_SUPPORT)
    env_add_firmware_path();
#endif

#ifdef CC_S_PLATFORM
    env_add_dolby_vision_support();
#endif

    {
        char str[64];
        #if defined(CC_FBM_FB_IN_CHANNELB) // fb buffer in fbm pool
            #if defined(CC_UBOOT_TWO_BANK)
                snprintf(str,sizeof(str), "fb_start=0x%llx", TOTAL_MEM_SIZE + PHYS_SDRAM_2_SIZE); // multi-channel,fb buffer in channelB
            #else
                #ifdef CC_TWO_MEM_REGION_BANK  // for husky two region case!
                    snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE + DIRECT_FB_MEM_SIZE);
                #else  //for one channel case!
                    snprintf(str,sizeof(str), "fb_start=0x%llx", TOTAL_MEM_SIZE - FBM_MEM_CFG_SIZE - TRUSTZONE_MEM_SIZE);
                #endif
            #endif
        #else
            #ifndef CC_FBM_DYNMJC
            snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE);
            #else
            #ifndef CC_MT5895
            if(!is_support_mjc)
            {
                snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE_NOMJC);
            }
            else
            {
                snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE);
            }
            #else
            if(!is_support_mjc)
            {
                snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE_NOMJC);
            }
            else
            {
                snprintf(str,sizeof(str), "fb_start=0x%llx", PHYS_SDRAM_1 + PHYS_SDRAM_1_SIZE_MJC);
            }
            #endif
            #endif
        #endif
        env_add_bootargs("fb_start=", str);
        #if defined(CC_FBM_FB_IN_CHANNELB)
        snprintf(str,sizeof(str), "fb_size=0x%x", FBM_MTFB_SIZE/2);
        #else
        #if defined(CC_MT5895)&&defined(CC_FBM_DYNMJC)
        if(!is_support_mjc)
        {
        #if defined(CC_SUPPORT_4K2K_UI_COEXIST)
            snprintf(str,sizeof(str), "fb_size=0x%x", 3840*2160*4*3);
        #else
            snprintf(str,sizeof(str), "fb_size=0x%x", 1920*1080*4*3);
        #endif
        }
        else
        {
            snprintf(str,sizeof(str), "fb_size=0x%x", 1920*1080*4*3);
        }
        #else
        snprintf(str,sizeof(str), "fb_size=0x%x", 1920*1080*4*3);
        #endif
        #endif
        env_add_bootargs("fb_size=", str);
    }
}

#ifdef CONFIG_AUTO_COMPLETE
int env_complete(char *var, int maxv, char *cmdv[], int bufsz, char *buf)
{
	ENTRY *match;
	int found, idx;

	idx = 0;
	found = 0;
	cmdv[0] = NULL;

	while ((idx = hmatch_r(var, idx, &match, &env_htab))) {
		int vallen = strlen(match->key) + 1;

		if (found >= maxv - 2 || bufsz < vallen)
			break;

		cmdv[found++] = buf;
		memcpy(buf, match->key, vallen);
		buf += vallen;
		bufsz -= vallen;
	}

	qsort(cmdv, found, sizeof(cmdv[0]), strcmp_compar);

	if (idx)
		cmdv[found++] = "...";

	cmdv[found] = NULL;
	return found;
}
#endif
