/*
 * This is from the Android Project,
 * Repository: https://android.googlesource.com/platform/bootable/bootloader/legacy
 * File: include/boot/bootimg.h
 * Commit: 4205b865141ff2e255fe1d3bd16de18e217ef06a
 *
 * Copyright (C) 2008 The Android Open Source Project
 *
 * SPDX-License-Identifier: BSD-2-Clause
 */

#ifndef _ANDROID_IMAGE_H_
#define _ANDROID_IMAGE_H_

#define ANDR_BOOT_MAGIC "ANDROID!"
#define ANDR_BOOT_MAGIC_SIZE 8
#define ANDR_BOOT_NAME_SIZE 16
#define ANDR_BOOT_ARGS_SIZE 512
#define ANDR_BOOT_EXTRA_ARGS_SIZE 1024

struct andr_img_hdr_legacy {
	char magic[ANDR_BOOT_MAGIC_SIZE];

	u32 kernel_size;	/* size in bytes */
	u32 kernel_addr;	/* physical load addr */

	u32 ramdisk_size;	/* size in bytes */
	u32 ramdisk_addr;	/* physical load addr */

	u32 second_size;	/* size in bytes */
	u32 second_addr;	/* physical load addr */

	u32 tags_addr;		/* physical addr for kernel tags */
	u32 page_size;		/* flash page size we assume */
	u32 unused;			/* future expansion: should be 0 */
	u32 os_version;		/* operating system version and security patch level */

	char name[ANDR_BOOT_NAME_SIZE]; /* asciiz product name */

	char cmdline[ANDR_BOOT_ARGS_SIZE];

	u32 id[8]; /* timestamp / checksum / sha1 / etc */
};

/*
 * +-----------------+
 * | boot header     | 1 page
 * +-----------------+
 * | kernel          | n pages
 * +-----------------+
 * | ramdisk         | m pages
 * +-----------------+
 * | second stage    | o pages
 * +-----------------+
 *
 * n = (kernel_size + page_size - 1) / page_size
 * m = (ramdisk_size + page_size - 1) / page_size
 * o = (second_size + page_size - 1) / page_size
 *
 * 0. all entities are page_size aligned in flash
 * 1. kernel and ramdisk are required (size != 0)
 * 2. second is optional (second_size == 0 -> no second)
 * 3. load each element (kernel, ramdisk, second) at
 *    the specified physical address (kernel_addr, etc)
 * 4. prepare tags at tag_addr.  kernel_args[] is
 *    appended to the kernel commandline in the tags.
 * 5. r0 = 0, r1 = MACHINE_TYPE, r2 = tags_addr
 * 6. if second_size != 0: jump to second_addr
 *    else: jump to kernel_addr
 */
struct andr_img_hdr {
	char magic[ANDR_BOOT_MAGIC_SIZE];

	u32 kernel_size;	/* size in bytes */
	u32 kernel_addr;	/* physical load addr */

	u32 ramdisk_size;	/* size in bytes */
	u32 ramdisk_addr;	/* physical load addr */

	u32 second_size;	/* size in bytes */
	u32 second_addr;	/* physical load addr */

	u32 tags_addr;		/* physical addr for kernel tags */
	u32 page_size;		/* flash page size we assume */
	u32 header_version;	/* future expansion: should be 0 */
	u32 os_version;		/* operating system version and security patch level */

	char name[ANDR_BOOT_NAME_SIZE]; /* asciiz product name */

	char cmdline[ANDR_BOOT_ARGS_SIZE];

	u32 id[8]; /* timestamp / checksum / sha1 / etc */

	char extra_cmdline[ANDR_BOOT_EXTRA_ARGS_SIZE];

	u32 recovery_dtbo_size;
	u64 recovery_dtbo_offset;
	u32 header_size;
	u32 dtb_size;
	u64 dtb_addr;
}__attribute__((packed));

/*
 * DTBO image header
 */
#define DT_TABLE_MAGIC 0xd7b7ab1e

struct dt_table_header {
    u32 magic;              //DT_TABLE_MAGIC: 0xd7b7able
    u32 total_size;         // includes dt_table_header + all dt_table_entry and all dtb/dtbo
    u32 header_size;        // sizeof(dt_table_header)

    u32 dt_entry_size;      // sizeof(dt_table_entry)
    u32 dt_entry_count;     // number of dt_table_entry
    u32 dt_entries_offset;  // offset to the first dt_table_entry

    u32 page_size;          // flash page size we assume
    u32 version;            // must be zero in android P.
};

/*
 * DTBO Entry header
 */
struct dt_table_entry {
    u32 dt_size;
    u32 dt_offset;      // offset from head of dt_table_header

    u32 id;             // optional, must be zero if unused
    u32 rev;            // optional, must be zero if unused
    u32 custom[4];      // optional, must be zero if unused
};

#endif
