/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: jiewen.zhou $
 * $Date: 2015/06/05 $
 * $RCSfile: drv_tcon.c,v $
 * $Revision: #2 $
 *
 *---------------------------------------------------------------------------*/
#include "sv_const.h"
#include "feature.h"

#include "hw_tg.h"

#include "sif_if.h"

#include "drv_tcon.h"

#ifdef DEFINE_IS_LOG
#undef DEFINE_IS_LOG
#endif
#ifdef CC_MTK_LOADER
#define DEFINE_IS_LOG(level, fmt...)   Printf(fmt)
#else /* CC_MTK_LOADER */
#define DEFINE_IS_LOG   IR_IsLog
#endif /* CC_MTK_LOADER */

//GAMMA BUS & CLK RATE
#define GAMMAIC_BUS 0
#define GAMMAIC_CLK_RATE 0x100
// for V13 gamma IC
#define GAMMAIC_ID_V13 0x40
#define GAMMAIC_DATA_CNT_V13 1


// Gamma
struct sTCONREGTABLE sRegCustGamma_V13[] = {
  // Gamma voltage
  {0x0000, 0x19, 0xffffffff},  //{0x0000, 0x2A, 0xffffffff}, for EPI V12
  {0x0001, 0x09, 0xffffffff},
  {0x0002, 0x03, 0xffffffff},
  {0x0003, 0x03, 0xffffffff},
  {0x0004, 0x0B, 0xffffffff}, 
  {0x0005, 0x0B, 0xffffffff}, 
  {0x0006, 0x20, 0xffffffff},
  {0x0007, 0x01, 0xffffffff},
  {0x0008, 0x03, 0xffffffff},
  {0x0009, 0x03, 0xffffffff},
  {0x000A, 0x01, 0xffffffff},
  {0x000B, 0x42, 0xffffffff}, //{0x000B, 0x3D, 0xffffffff},for EPI V12
  {0x000C, 0x01, 0xffffffff},
  {0x000D, 0x04, 0xffffffff}, //{0x000D, 0xFD, 0xffffffff},for EPI V12
  {0x000E, 0x00, 0xffffffff},
  {0x000F, 0x8B, 0xffffffff}, //{0x000F, 0x88, 0xffffffff},for EPI V12
  {0x0010, 0x01, 0xffffffff},
  {0x0011, 0x74, 0xffffffff},
  {0x0012, 0x00, 0xffffffff},
  {0x0013, 0xFC, 0xffffffff}, //{0x0013, 0xFD, 0xffffffff}, for EPI V12
  {0x0014, 0x00, 0xffffffff},
  {0x0015, 0xBD, 0xffffffff},
  {0x0016, 0x00, 0xffffffff},
  {0x0017, 0xAB, 0xffffffff}, //{0x0017, 0xB3, 0xffffffff}, for EPI V12
  {0x0018, 0x04, 0xffffffff}, //{0x0018, 0x02, 0xffffffff}, for EPI V12
};

// for V4 gamma IC
#define GAMMAIC_ID (0x74 << 1)
#define GAMMAIC_DATA_CNT 2
// Gamma
struct sTCONREGTABLE sRegCustGamma[] = {
  // Gamma voltage
  {0x0000, 0x6B03, 0xffffffff},
  {0x0001, 0xEB03, 0xffffffff},
  {0x0002, 0x6B03, 0xffffffff},
  {0x0003, 0x3E03, 0xffffffff},
  {0x0004, 0xFD02, 0xffffffff},
  {0x0005, 0xB602, 0xffffffff},
  {0x0006, 0x8402, 0xffffffff},
  {0x0007, 0x0902, 0xffffffff},
  {0x0008, 0xEF01, 0xffffffff},
  {0x0009, 0x7601, 0xffffffff},
  {0x000A, 0x4201, 0xffffffff},
  {0x000B, 0xFB00, 0xffffffff},
  {0x000C, 0xB900, 0xffffffff},
  {0x000D, 0x8E00, 0xffffffff},
  {0x000E, 0x1300, 0xffffffff},
  {0x000F, 0x8E00, 0xffffffff},
  {0x0012, 0xB581, 0xffffffff},
};


//-----------------------------------------------------------------------------
// mini LVDS Interface Setting
//-----------------------------------------------------------------------------

//miniLVDS LGD V4 FHD 60Hz 8Bit Non-GIP Normal
  struct sTCONREGTABLE sRegMini_MLVDS_LGDV4FHD60[] = {

  // Pixel reorder
  {IO_VIRT + 0x35c1c, 0xD4002000, 0xFFFFFFFF}, //Blue Shift Mode=5   
  {IO_VIRT + 0x35c2c, 0x00640780, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c34, 0x00A00000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c4c, 0x00000780, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c50, 0x80000780, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c54, 0x00000001, 0xFFFFFFFF},  
  {IO_VIRT + 0x35c00, 0x001147FF, 0xFFFFFFFF},  // 2 Write Count, 2 Read Conut
  //{IO_VIRT + 0x35c38, 0x00000464, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
                                
  // mini-LVDS Encode
  {IO_VIRT + 0x35000, 0x00000011, 0xFFFFFFFF},  // 2Port

  {IO_VIRT + 0x3500c, 0x0000003f, 0xFFFFFFFF},  //Reset Pluse
  {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF},  
  {IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF},  //6Pair
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF},  //enable CH0/CH1
  {IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF},  //CH0/CH1 swap

  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},  //Pair select
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF},  //PN swap disable
  };

//miniLVDS Samsung MB7 FHD 60Hz 8Bit TRD 
struct sTCONREGTABLE sRegMini_MLVDS_SAMSUNGMB7[] = {

  //Pixel Reorder
  {IO_VIRT + 0x35c00, 0x00118400, 0xFFFFFFFF},  // 3 Write Count, 2 Read Conut
  {IO_VIRT + 0x35c1c, 0xd4000000, 0xFFFFFFFF},  //Blue Shift Mode=5
  {IO_VIRT + 0x35c20, 0x28a80000, 0xFFFFFFFF},  //Enable TRD, Select TRD Mode
  
  {IO_VIRT + 0x35c28, 0x08980020, 0xFFFFFFFF},  //Set RG_HSTART_R
  {IO_VIRT + 0x35c2c, 0x00100284, 0xFFFFFFFF},  //Set RG_HWIDTH_R & RG_HACTIVE_R
  {IO_VIRT + 0x35c30, 0x005a0059, 0xFFFFFFFF},  //Set RG_DE_HSTART_R & RG_DE_HWIDTH_R
  {IO_VIRT + 0x35c34, 0x00320059, 0xFFFFFFFF},  //Set RG_PRE_DE_HSTART_R & PRE_DE_HWIDTH_R
  {IO_VIRT + 0x35c4c, 0x0000078c, 0xFFFFFFFF},  

  {IO_VIRT + 0x35c48, 0x82dd0780, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c50, 0x0000278c, 0xFFFFFFFF}, 
                                        
  {IO_VIRT + 0x35c54, 0x0284278c, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c58, 0x0508278c, 0xFFFFFFFF}, 

  {IO_VIRT + 0x35c5c, 0x50000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c60, 0x00008142, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c64, 0x028493c6, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c68, 0x0508a64a, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35cc0, 0x04800000, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

  //mini-LVDS 
  {IO_VIRT + 0x35000, 0x00000011, 0xFFFFFFFF},  // 2Port

  {IO_VIRT + 0x3500c, 0x0000003f, 0xFFFFFFFF},  //Reset Pluse
  {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF},  //6Pair
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF},  //enable CH0/CH1
  {IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF},  //CH0/CH1 swap
  
  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},  //Pair select
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF},  //PN swap disable
  };

//miniLVDS CMO 2011 FHD 60Hz 8Bit Flip-Pixel-2 
  struct sTCONREGTABLE sRegMini_MLVDS_CMO2011FHD60[] = {
  
  // Pixel reorder
  {IO_VIRT + 0x35c1c, 0xD4000000, 0xFFFFFFFF}, //Blue Shift Mode=5 
  {IO_VIRT + 0x35c2c, 0x00640782, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c34, 0x00A00000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c4c, 0x00000781, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c50, 0x80000781, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c54, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c00, 0x001107FF, 0xFFFFFFFF}, // 1 Write Count, 2 Read Conut
  //{IO_VIRT + 0x35c38, 0x00000464, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  // mini-LVDS Encode
  {IO_VIRT + 0x35000, 0x00000011, 0xFFFFFFFF},  // 2Port

  {IO_VIRT + 0x3500c, 0x0000003f, 0xFFFFFFFF},  //Reset Pluse
  {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF},  
  {IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF},  //6Pair
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF},  //enable CH0/CH1
  {IO_VIRT + 0x35020, 0x00000004, 0xFFFFFFFF}, 
	
  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},  //Pair select
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF},  //PN swap disable
  };

//miniLVDS Panasonic 2011 HD 60Hz 8Bit Normal 
struct sTCONREGTABLE sRegMini_MLVDS_PANASONIC32WXGA1P6P[] = {

  // Pixel reorder
  {IO_VIRT + 0x35c00, 0x000007FF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c1c, 0xD4000000, 0xFFFFFFFF},  //Blue Shift Mode=5
  {IO_VIRT + 0x35c04, 0xFFFFFFFF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c0c, 0x002AB2AB, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c10, 0x002AB557, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c14, 0x00557557, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c20, 0xA0000007, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c4c, 0x00000558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c50, 0x00000558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c30, 0x003C0000, 0xFFFFFFFF},
  {IO_VIRT + 0x35c2c, 0x00640558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c34, 0x00280000, 0xFFFFFFFF}, 
  //{IO_VIRT + 0x35c38, 0x00000300, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
    
  // mini-LVDS Encode
  {IO_VIRT + 0x35000, 0x00000010, 0xFFFFFFFF},

  {IO_VIRT + 0x3500c, 0x0000003F, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35010, 0x00FFFFFF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35018, 0x000003FF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35020, 0x00000004, 0xFFFFFFFF}, 
  
  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF}, 
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF}, 
}; 

//miniLVDS Panasonic 2012 HD 60Hz 8Bit Normal
struct sTCONREGTABLE sRegMini_MLVDS_PANASONIC32WXGA2P3P[] = {

  {IO_VIRT + 0x34008, 0x37F9FFFF, 0xFFFFFFFF}, 
  
  // Pixel reorder
  {IO_VIRT + 0x35c00, 0x001107FF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c1c, 0xEA000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c04, 0xFFFFFFFF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c0c, 0x002AB2AB, 0xFFFFFFFF},
  {IO_VIRT + 0x35c10, 0x002AB557, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c14, 0x00557557, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c20, 0xA0000007, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c4c, 0x00000558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c50, 0x00000558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c30, 0x003C0000, 0xFFFFFFFF},
  {IO_VIRT + 0x35c2c, 0x00640558, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c34, 0x00280000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c64, 0x02AC0000, 0xFFFFFFFF}, 
  //{IO_VIRT + 0x35c38, 0x00000300, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
		
  // mini-LVDS Encode
  {IO_VIRT + 0x35000, 0x00000011, 0xFFFFFFFF}, 

  {IO_VIRT + 0x3500c, 0x0000003F, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35014, 0x00010000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF}, 
	  
  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF}, 

}; 

//miniLVDS Panasonic FHD 120Hz
struct sTCONREGTABLE sRegMini_MLVDS_PANASONIC42FHD120[] = {

  // Pixel reorder
  {IO_VIRT + 0x35c00, 0x001147FF, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c1c, 0xE8000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c2c, 0x00640782, 0xFFFFFFFF}, 
  //xF0035c34, 0x00A00000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c4c, 0x00000781, 0xFFFFFFFF},
  {IO_VIRT + 0x35c50, 0x80000781, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35ca0, 0x843E0001, 0xFFFFFFFF},
  {IO_VIRT + 0x35ca4, 0x8007843E, 0xFFFFFFFF},
  //{IO_VIRT + 0x35c38, 0x00000464, 0xFFFFFFFF},

  {IO_VIRT + 0x35CA0, 0x04410001, 0xFFFFFFFF},
  {IO_VIRT + 0x35CA4, 0x00000464, 0xFFFFFFFF},

  {IO_VIRT + 0x34008, 0x2001FFFF, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
                                
  // mini-LVDS Encode
  {IO_VIRT + 0x35000, 0x000000D1, 0xFFFFFFFF},
  
  {IO_VIRT + 0x3500c, 0x0000C01C, 0xFFFFFFFF},
  {IO_VIRT + 0x35010, 0xE8E80000, 0xFFFFFFFF},
  {IO_VIRT + 0x35014, 0x80000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF}, 

  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF},
  {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF},

  {IO_VIRT + 0x35108, 0x0000F000, 0xFFFFFFFF},
  {IO_VIRT + 0x35114, 0x0000F000, 0xFFFFFFFF},
  {IO_VIRT + 0x35120, 0x0000F000, 0xFFFFFFFF},
  {IO_VIRT + 0x35124, 0x18C018C0, 0xFFFFFFFF},
  {IO_VIRT + 0x351AC, 0xC8000477, 0xFFFFFFFF},

  {IO_VIRT + 0x3512C, 0x20201010, 0xFFFFFFFF},
  {IO_VIRT + 0x35130, 0xF4B45420, 0xFFFFFFFF},
  {IO_VIRT + 0x35134, 0xECB4A844, 0xFFFFFFFF},
  {IO_VIRT + 0x35138, 0xF0A09020, 0xFFFFFFFF},
  {IO_VIRT + 0x3513C, 0x00000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35140, 0x000C181C, 0xFFFFFFFF},
  {IO_VIRT + 0x35144, 0x00000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35148, 0x0C2C0C0C, 0xFFFFFFFF},

  {IO_VIRT + 0x3514C, 0x10102020, 0xFFFFFFFF},
  {IO_VIRT + 0x35150, 0x2054B4F4, 0xFFFFFFFF},
  {IO_VIRT + 0x35154, 0x44A8B4EC, 0xFFFFFFFF},
  {IO_VIRT + 0x35158, 0x2898A8F8, 0xFFFFFFFF},
  {IO_VIRT + 0x3515C, 0x00000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35160, 0x1C180C00, 0xFFFFFFFF},
  {IO_VIRT + 0x35164, 0x00000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35168, 0x0C0C2C0C, 0xFFFFFFFF},  
  
};

struct sTCONREGTABLE sRegMini_MLVDS_LGD32WXGAV9[] = {

  {IO_VIRT + 0x35000, 0x20000058, 0xffffffff},
  {IO_VIRT + 0x35110, 0x00100000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

  {IO_VIRT + 0x35104, 0x00012456, 0xffffffff},
  {IO_VIRT + 0x35108, 0x00012456, 0xffffffff},
  {IO_VIRT + 0x3510C, 0xBABA9898, 0xffffffff},

  {IO_VIRT + 0x35100, 0x0000FC3F, 0xffffffff},
  {IO_VIRT + 0x35130, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35110, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35004, 0x05580628, 0xffffffff},
  {IO_VIRT + 0x35008, 0x0300033A, 0xffffffff},
  {IO_VIRT + 0x35280, 0x82AA0000, 0xffffffff},

};

//miniLVDS CSOT HD 60Hz 8Bit TRD
struct sTCONREGTABLE sRegMini_MLVDS_CHINASTARWXGA8BIT[] =  { 

  //Pixel Reorder
  {IO_VIRT + 0x35c00, 0x00008400, 0xFFFFFFFF},  // 1 port 6pairs Vsync polarity
  {IO_VIRT + 0x35c18, 0x80000000, 0xFFFFFFFF}, 	
  {IO_VIRT + 0x35c1c, 0xD4000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35c20, 0xA8A80000, 0xFFFFFFFF},
  {IO_VIRT + 0x35c24, 0x00000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35c28, 0x06180010, 0xFFFFFFFF}, 	
  {IO_VIRT + 0x35c2c, 0x001001C8, 0xFFFFFFFF},  //RG_HACTIVE_R=(hactive/3) 
  {IO_VIRT + 0x35c30, 0x00400040, 0xFFFFFFFF},  //RG_DE_HSTART_R=(htotal/3)-(hactive/3),RG_DE_HWIDTH_R=(htotal/3)-(hactive/3) 
  {IO_VIRT + 0x35c34, 0x002C0040, 0xFFFFFFFF},  //RG_PRE_DE_HSTART_R=(htotal/3)-(hactive/3)-20,RG_PRE_DE_HWIDTH_R=(htotal/3)-(hactive/3)
  {IO_VIRT + 0x35c48, 0x82070558, 0xFFFFFFFF},  //RG_SPLIT_HWIDTH_R=(htotal/3)-1
  
  {IO_VIRT + 0x35c4c, 0x00000558, 0xFFFFFFFF}, 		
  {IO_VIRT + 0x35c50, 0x00002558, 0xFFFFFFFF}, 										   
  {IO_VIRT + 0x35c54, 0x01C82558, 0xFFFFFFFF},    
  {IO_VIRT + 0x35c58, 0x03902558, 0xFFFFFFFF},    
  {IO_VIRT + 0x35c5c, 0x50000000, 0xFFFFFFFF},    
  {IO_VIRT + 0x35c60, 0x000000E4, 0xFFFFFFFF},    
  {IO_VIRT + 0x35c64, 0x01C812AC, 0xFFFFFFFF}, 	 
  {IO_VIRT + 0x35c68, 0x03902474, 0xFFFFFFFF}, 	
  {IO_VIRT + 0x35cc0, 0x04800000, 0xFFFFFFFF},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

  //mini-LVDS 
  {IO_VIRT + 0x35000, 0x00000010, 0xFFFFFFFF},
  {IO_VIRT + 0x35004, 0x00000000, 0xFFFFFFFF},   //minilvds reset pulse  
  {IO_VIRT + 0x35008, 0x00000000, 0xFFFFFFFF},   //minilvds reset pulse  
  {IO_VIRT + 0x3500c, 0x0000003f, 0xFFFFFFFF},   //minilvds reset pulse 
  {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF},   //samsung mute vsync count need set ?
  {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
  {IO_VIRT + 0x35020, 0x000000E0, 0xFFFFFFFF},   //how to set 
  
  {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},   //pairs select
  {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  
  {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
};

//miniLVDS Hisense 2011 FHD 8Bit Normal
struct sTCONREGTABLE sRegMini_MLVDS_HISENSE_HE420FF[] = {


   // Pixel reorder
   {IO_VIRT + 0x35c18, 0xBFFFFFFF, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35c00, 0x00110400, 0xFFFFFFFF},  //{IO_VIRT + 0x35c00, 0x001007FF, 0xFFFFFFFF},// set 2 port , 2 read/write counter   need to check
   {IO_VIRT + 0x35c1c, 0xC0000000, 0xFFFFFFFF},  //No Blue Shift
   {IO_VIRT + 0x35c28, 0x08980000, 0xFFFFFFFF},
   {IO_VIRT + 0x35c2c, 0x00640780, 0xFFFFFFFF},
   {IO_VIRT + 0x35c30, 0x00B40000, 0xFFFFFFFF},  //pattern's edge shift 10 lines to right 
   {IO_VIRT + 0x35c34, 0x00A00000, 0xFFFFFFFF},
   {IO_VIRT + 0x35c38, 0x00000001, 0xFFFFFFFF},
   {IO_VIRT + 0x35cA0, 0x843E0001, 0xFFFFFFFF},
   {IO_VIRT + 0x35cA4, 0x80068439, 0xFFFFFFFF},
   {IO_VIRT + 0x35c50, 0x00000780, 0xFFFFFFFF},
   {IO_VIRT + 0x35c4C, 0x00000780, 0xFFFFFFFF},
   {IO_VIRT + 0x35c54, 0x00000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35c58, 0x00000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35c5C, 0x00000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35c60, 0x00000000, 0xFFFFFFFF},  //{IO_VIRT + 0x35c60, 0x000083C0, 0xFFFFFFFF},
   {IO_VIRT + 0x35c64, 0x03C00000, 0xFFFFFFFF},
   {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

   // mini-LVDS Encode
   {IO_VIRT + 0x35000, 0x00000011, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35004, 0x00000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35008, 0x00000000, 0xFFFFFFFF},   
   {IO_VIRT + 0x3500c, 0x0000003F, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35014, 0x00000000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35020, 0x000000E4, 0xFFFFFFFF}, 
   
   {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},  //{IO_VIRT + 0x35028, 0x76543210, 0xFFFFFFFF},
   {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  //{IO_VIRT + 0x3502c, 0x76543210, 0xFFFFFFFF},  
   {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF},  //{IO_VIRT + 0x35030, 0x0A0A9898, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF},    
};

//miniLVDS Hisense 2013 HD 6Bit Normal
struct sTCONREGTABLE sRegMini_MLVDS_HISENSE32WXGA2P3P[] = {
	
    // Pixel reorder
	{IO_VIRT + 0x35c00, 0x001107FF, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c1c, 0xD4000000, 0xFFFFFFFF},
	{IO_VIRT + 0x35c04, 0xFFFFFFFF, 0xFFFFFFFF},
	{IO_VIRT + 0x35c0c, 0x002AB2AB, 0xFFFFFFFF},
	{IO_VIRT + 0x35c10, 0x002AB557, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c14, 0x00557557, 0xFFFFFFFF},
	{IO_VIRT + 0x35c20, 0xA0000007, 0xFFFFFFFF},
	{IO_VIRT + 0x35c2c, 0x00640558, 0xFFFFFFFF},
	{IO_VIRT + 0x35c30, 0x003C0000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c34, 0x00280000, 0xFFFFFFFF},
	{IO_VIRT + 0x35c4c, 0x00000558, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c50, 0x00000558, 0xFFFFFFFF},
	{IO_VIRT + 0x35c64, 0x02AC0000, 0xFFFFFFFF}, 
	//{IO_VIRT + 0x35c38, 0x00000300, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
	  
	// mini-LVDS Encode
	{IO_VIRT + 0x35000, 0x00000051, 0xFFFFFFFF}, 
  
	{IO_VIRT + 0x3500c, 0x000000E0, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35010, 0x00000000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35014, 0x00010000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF}, 
	
  	{IO_VIRT + 0x35028, 0x1034A567, 0xFFFFFFFF},
  	{IO_VIRT + 0x3502c, 0x1034A567, 0xFFFFFFFF},  
  	{IO_VIRT + 0x35030, 0x24249898, 0xFFFFFFFF},  
	{IO_VIRT + 0x35034, 0x08C008C0, 0xFFFFFFFF}, 
	
}; 

//miniLVDS Hisense 2013 HD 8Bit Normal
struct sTCONREGTABLE sRegMini_MLVDS_HISENSE32WXGA1P6P[] = {
	
    // Pixel reorder
	{IO_VIRT + 0x35c00, 0x000007FF, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c1c, 0xC0000000, 0xFFFFFFFF},
	{IO_VIRT + 0x35c04, 0xFFFFFFFF, 0xFFFFFFFF},
	{IO_VIRT + 0x35c0c, 0x002AB2AB, 0xFFFFFFFF},
	{IO_VIRT + 0x35c10, 0x002AB557, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c14, 0x00557557, 0xFFFFFFFF},
	{IO_VIRT + 0x35c20, 0xA0000007, 0xFFFFFFFF},
	{IO_VIRT + 0x35c2c, 0x00640558, 0xFFFFFFFF},
	{IO_VIRT + 0x35c30, 0x003C0000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c34, 0x00280000, 0xFFFFFFFF},
	{IO_VIRT + 0x35c4c, 0x00000558, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35c50, 0x00000558, 0xFFFFFFFF},
	//{IO_VIRT + 0x35c38, 0x00000300, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
	  
	// mini-LVDS Encode
	{IO_VIRT + 0x35000, 0x00000010, 0xFFFFFFFF}, 
  
	{IO_VIRT + 0x3500c, 0x000000E0, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35010, 0x00FFFFFF, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35014, 0x80010000, 0xFFFFFFFF}, 
	{IO_VIRT + 0x35018, 0x000003FF, 0xFFFFFFFF},  
	{IO_VIRT + 0x35020, 0x00000001, 0xFFFFFFFF}, 
	
  	{IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},
  	{IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  
  	{IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
  	{IO_VIRT + 0x35034, 0x00000000, 0xFFFFFFFF},  
	
};

//miniLVDS LGD V12 HD 60Hz 6Bit DRD 
struct sTCONREGTABLE sRegMini_MLVDS_LGD32WXGAV12[] = {

   // Pixel reorder
   {IO_VIRT + 0x35C18, 0xBFFFFFFF, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C28,	0x062A0000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C2C,	0x00640556, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C30,	0x003A0000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C34,	0x00300000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C3C,	0x00080300, 0xFFFFFFFF},	 
   {IO_VIRT + 0x35C38,	0x00000001, 0xFFFFFFFF},	

   {IO_VIRT + 0x35C00, 0x00000400, 0xFFFFFFFF},
   {IO_VIRT + 0x35C20, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C1C, 0x00FFC3FF, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C48, 0x00000780, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C80, 0x00531420, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C84, 0x00531420, 0xFFFFFFFF},	 
   {IO_VIRT + 0x35C40, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C44, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C50, 0x00000556, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C4C, 0x00000556, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C54, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C58, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C5C, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C60, 0x00000000, 0xFFFFFFFF},	
   {IO_VIRT + 0x35C64, 0x02AB0000, 0xFFFFFFFF},
   {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

   // mini-LVDS Encode
   {IO_VIRT + 0x35000, 0x00000050, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35004, 0x00000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35008, 0x00F01F84, 0xFFFFFFFF},   
   {IO_VIRT + 0x3500c, 0x00001000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35010, 0x0000FFFF, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35014, 0x10000000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF}, 
   {IO_VIRT + 0x3501C, 0x03FF03FF, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35020, 0x000000E0, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35064, 0x000000FF, 0xFFFFFFFF},
   {IO_VIRT + 0x35068, 0x0000FFFF, 0xFFFFFFFF},
   
   {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},
   {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},  
   {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF}, 
   {IO_VIRT + 0x35104, 0x00FFFFC7, 0xFFFFFFFF},
   {IO_VIRT + 0x3510C, 0x00FFFFC7, 0xFFFFFFFF},
   {IO_VIRT + 0x35110, 0x02110000, 0xFFFFFFFF},
};

struct sTCONREGTABLE sRegMini_MLVDS_LGD32WXGAV13[] = {
   // Pixel reorder	
   {IO_VIRT + 0x35C1C, 0xC0FFC3FF, 0xFFFFFFFF},  //No Blue Shfit
   {IO_VIRT + 0x35C20, 0xA4000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35C80, 0x00451302, 0xFFFFFFFF},
   {IO_VIRT + 0x35C00, 0x000047FF, 0xFFFFFFFF},
   {IO_VIRT + 0x35C4C, 0x00000556, 0xFFFFFFFF},
   {IO_VIRT + 0x35C50, 0x00001556, 0xFFFFFFFF},
   {IO_VIRT + 0x35C54, 0x02AB1556, 0xFFFFFFFF},
   {IO_VIRT + 0x35C48, 0x831F0780, 0xFFFFFFFF},  //RG_SPLIT_HWIDTH_R=(htotal/2)-1
   {IO_VIRT + 0x35C5C, 0x30000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35C60, 0x00008000, 0xFFFFFFFF},
   {IO_VIRT + 0x35C64, 0x03C092AB, 0xFFFFFFFF},
   {IO_VIRT + 0x35C2C,	0x006402AB, 0xFFFFFFFF},  //RG_HACTIVE_R=(hactive/2)
   {IO_VIRT + 0x35C30,	0x00750075, 0xFFFFFFFF},  //RG_DE_HSTART_R=(htotal/2)-(hactive/2),RG_DE_HWIDTH_R=(htotal/2)-(hactive/2)
   {IO_VIRT + 0x35C34,	0x006B0075, 0xFFFFFFFF},  //RG_PRE_DE_HSTART_R=(htotal/2)-(hactive/2)-10,RG_PRE_DE_HWIDTH_R=(htotal/2)-(hactive/2)
   {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
   
   // mini-LVDS Encode
   {IO_VIRT + 0x35000, 0x00000040, 0xFFFFFFFF},  //6bit  1port
   {IO_VIRT + 0x35004, 0x00000000, 0xFFFFFFFF},  //MLVDS_RST 8[7:2] as MLVDS Ctrl P11~P6, MLVDS_RST 7[7:2] as MLVDS Ctrl P17~P12
   {IO_VIRT + 0x35008, 0x00F01F04, 0xFFFFFFFF},  //MLVDS_RST 8[7:2] as , MLVDS_RST 7[7:2] as MLVDS Ctrl P5~P0 
   {IO_VIRT + 0x3500c, 0x0000D800, 0xFFFFFFFF},  //MLVDS_RST 8[7:2] as MLVDS Ctrl P11~P6, MLVDS_RST 9[7:2] as MLVDS Ctrl P17~P12
   {IO_VIRT + 0x35010, 0x00F8FFFF, 0xFFFFFFFF},  //Reset pulse or Ctrl bit select
   {IO_VIRT + 0x35014, 0x10010000, 0xFFFFFFFF},  //3pair, enable Embedded_En
   {IO_VIRT + 0x35018, 0x00000000, 0xFFFFFFFF},  //enable CH0/CH1
   {IO_VIRT + 0x3501C, 0x03FF03FF, 0xFFFFFFFF},  //disable CH2/CH3
   {IO_VIRT + 0x35020, 0x000000E0, 0xFFFFFFFF},  
   {IO_VIRT + 0x35064, 0x000000FF, 0xFFFFFFFF},  
   {IO_VIRT + 0x35068, 0x0000FFFF, 0xFFFFFFFF},
   
   {IO_VIRT + 0x35028, 0x1234A567, 0xFFFFFFFF},
   {IO_VIRT + 0x3502c, 0x1234A567, 0xFFFFFFFF},
   {IO_VIRT + 0x35030, 0x04049898, 0xFFFFFFFF},
   {IO_VIRT + 0x35024, 0x01000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35024, 0x81000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35024, 0x01000000, 0xFFFFFFFF},
   {IO_VIRT + 0x35000, 0x00000050, 0xFFFFFFFF},
   {IO_VIRT + 0x35104, 0x00FFFFC7, 0xFFFFFFFF},
   {IO_VIRT + 0x3510C, 0x00FFFFC7, 0xFFFFFFFF},
   {IO_VIRT + 0x35110, 0x01440000, 0xFFFFFFFF},
};

//-----------------------------------------------------------------------------
// EPI Interface Setting
//-----------------------------------------------------------------------------

  //EPI LGDV12FHD60
   struct sTCONREGTABLE sReg_PANEL_EPI_LGDV12FHD60[] = {

  //{IO_VIRT + 0x35000, 0x00000015, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x005547ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffe3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  
  //  {IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000780, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000780, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x01400000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x02800000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x03c00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x05000000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x06400000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},

  };

  //EPI LGDV12FHD120
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV12FHD120[] = {

  //{IO_VIRT + 0x35000, 0x00000015, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff},
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x005547ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffe3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  
  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000780, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000780, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x01400000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x02800000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x03c00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x05000000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x06400000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},

  };

  //EPI LGDV13FHD60
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV13FHD60[] = {

  //{IO_VIRT + 0x35000, 0x00000015, 0xffffffff}, 

  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 

  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 

  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},

  //pixel reorder
  {IO_VIRT + 0x35c00, 0x005547ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffe3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},

  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000780, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000780, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x01400000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x02800000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x03c00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x05000000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x06400000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},

  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000080, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},
  //separate CTR2 TCON setting
  {IO_VIRT + 0x35580, 0x76543211, 0xffffffff}, //channel1 CTR2
  {IO_VIRT + 0x35584, 0xfedcba08, 0xffffffff}, 
  {IO_VIRT + 0x35588, 0x76543211, 0xffffffff}, //channel2 CTR2
  {IO_VIRT + 0x3558c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35590, 0x76543211, 0xffffffff}, //channel3 CTR3
  {IO_VIRT + 0x35594, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35598, 0x76543211, 0xffffffff}, //channel4 CTR4
  {IO_VIRT + 0x3559c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a0, 0x76543211, 0xffffffff}, //channel5 CTR5
  {IO_VIRT + 0x355a4, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a8, 0x76543211, 0xffffffff}, //channel6 CTR6
  {IO_VIRT + 0x355ac, 0xfedcba08, 0xffffffff},
};

  //EPI LGDV13FHD120
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV13FHD120[] = {
  
  //{IO_VIRT + 0x35000, 0x00000015, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x005547ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffe3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  
  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000780, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000780, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x01400000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x02800000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x03c00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x05000000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x06400000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000080, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},
  //separate CTR2 TCON setting
  {IO_VIRT + 0x35580, 0x76543211, 0xffffffff}, //channel1 CTR2
  {IO_VIRT + 0x35584, 0xfedcba08, 0xffffffff}, 
  {IO_VIRT + 0x35588, 0x76543211, 0xffffffff}, //channel2 CTR2
  {IO_VIRT + 0x3558c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35590, 0x76543211, 0xffffffff}, //channel3 CTR3
  {IO_VIRT + 0x35594, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35598, 0x76543211, 0xffffffff}, //channel4 CTR4
  {IO_VIRT + 0x3559c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a0, 0x76543211, 0xffffffff}, //channel5 CTR5
  {IO_VIRT + 0x355a4, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a8, 0x76543211, 0xffffffff}, //channel6 CTR6
  {IO_VIRT + 0x355ac, 0xfedcba08, 0xffffffff},
 };

  //EPI LGDV14FHD60 8LANE
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV14FHD60_8LANE[] = {
  
  //{IO_VIRT + 0x35000, 0x00000017, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x007747ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffc3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  {IO_VIRT + 0x35c2c, 0x00640781, 0xffffffff},
  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000781, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000781, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x00F00000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x01E00000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x02D00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x03C00000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x04B00000, 0xffffffff},
  {IO_VIRT + 0x35c78, 0x05A00000, 0xffffffff},
  {IO_VIRT + 0x35c7C, 0x06900000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x35404, 0x76543210, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000090, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},
  //separate CTR2 TCON setting
  {IO_VIRT + 0x35580, 0x76543211, 0xffffffff}, //channel1 CTR2
  {IO_VIRT + 0x35584, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35588, 0x76543211, 0xffffffff}, //channel2 CTR2
  {IO_VIRT + 0x3558c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35590, 0x76543211, 0xffffffff}, //channel3 CTR2
  {IO_VIRT + 0x35594, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35598, 0x76543211, 0xffffffff}, //channel4 CTR2
  {IO_VIRT + 0x3559c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a0, 0x76543211, 0xffffffff}, //channel5 CTR2
  {IO_VIRT + 0x355a4, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a8, 0x76543211, 0xffffffff}, //channel6 CTR2
  {IO_VIRT + 0x355ac, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355cc, 0x76543211, 0xffffffff}, //channel7 CTR2
  {IO_VIRT + 0x355d0, 0xfedcba08, 0xffffffff}, 
  {IO_VIRT + 0x355d4, 0x76543211, 0xffffffff}, //channel8 CTR2
  {IO_VIRT + 0x355d8, 0xfedcba08, 0xffffffff},
  };
  
  //EPI LGDV14FHD120 8LANE
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV14FHD120_8LANE[] = {
  
  //{IO_VIRT + 0x35000, 0x00000017, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x0ffcffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x007747ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xe8ffc3ff, 0xffffffff},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  {IO_VIRT + 0x35c2c, 0x00640781, 0xffffffff},
  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000781, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x80000781, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x00000001, 0xffffffff},
  {IO_VIRT + 0x35c64, 0x00F00000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x01E00000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x02D00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x03C00000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x04B00000, 0xffffffff},
  {IO_VIRT + 0x35c78, 0x05A00000, 0xffffffff},
  {IO_VIRT + 0x35c7C, 0x06900000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x35404, 0x76543210, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000090, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},
  //separate CTR2 TCON setting
  {IO_VIRT + 0x35580, 0x76543211, 0xffffffff}, //channel1 CTR2
  {IO_VIRT + 0x35584, 0xfedcba08, 0xffffffff}, 
  {IO_VIRT + 0x35588, 0x76543211, 0xffffffff}, //channel2 CTR2
  {IO_VIRT + 0x3558c, 0xfedcba08, 0xffffffff}, 
  {IO_VIRT + 0x35590, 0x76543211, 0xffffffff}, //channel3 CTR2
  {IO_VIRT + 0x35594, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35598, 0x76543211, 0xffffffff}, //channel4 CTR2
  {IO_VIRT + 0x3559c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a0, 0x76543211, 0xffffffff}, //channel5 CTR2
  {IO_VIRT + 0x355a4, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355a8, 0x76543211, 0xffffffff}, //channel6 CTR2
  {IO_VIRT + 0x355ac, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355cc, 0x76543211, 0xffffffff}, //channel7 CTR2
  {IO_VIRT + 0x355d0, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x355d4, 0x76543211, 0xffffffff}, //channel8 CTR2
  {IO_VIRT + 0x355d8, 0xfedcba08, 0xffffffff},  
  };  
  //EPI LGDV14FHD60 4LANE DRD
  struct sTCONREGTABLE sReg_PANEL_EPI_LGDV14FHD60_4LANE[] = {
  
  //{IO_VIRT + 0x35000, 0x00000013, 0xffffffff}, 
  
  //{IO_VIRT + 0x34004, 0x00000100, 0xffffffff}, 
  //{IO_VIRT + 0x34008, 0x07f4ffff, 0xffffffff}, 
  //{IO_VIRT + 0x3400c, 0x00004000, 0xffffffff}, 
  
  //{IO_VIRT + 0x34014, 0x00210000, 0xffffffff}, 
  
  //panel interface
  {IO_VIRT + 0x34600, 0x04802359, 0xffffffff},
  //{IO_VIRT + 0x34604, 0x07800894, 0xffffffff},
  //{IO_VIRT + 0x34610, 0x001c0001, 0xffffffff},
  //{IO_VIRT + 0x34614, 0x000003ff, 0xffffffff},
  //{IO_VIRT + 0x34618, 0x02000100, 0xffffffff},
  //{IO_VIRT + 0x34710, 0x00000000, 0xffffffff},
  
  //pixel reorder
  {IO_VIRT + 0x35c00, 0x003347ff, 0xffffffff},
  {IO_VIRT + 0x35c1c, 0xc0ffc3ff, 0xffffffff},
  {IO_VIRT + 0x35c20, 0xa4000000, 0xffffffff},
  {IO_VIRT + 0x35C80, 0x00451302, 0xFFFFFFFF},
  //{IO_VIRT + 0x35c28, 0x0898001a, 0xffffffff},
  {IO_VIRT + 0x35c2c, 0x006403C0, 0xffffffff},
  {IO_VIRT + 0x35c30, 0x008A008A, 0xffffffff},
  {IO_VIRT + 0x35c34, 0x0080008A, 0xffffffff},
  //{IO_VIRT + 0x35c38, 0x00000464, 0xffffffff},
  {IO_VIRT + 0x35c48, 0x844B0780, 0xffffffff},
  {IO_VIRT + 0x35c4c, 0x00000780, 0xffffffff},
  {IO_VIRT + 0x35c50, 0x00001780, 0xffffffff},
  {IO_VIRT + 0x35c54, 0x03C01780, 0xffffffff},
  {IO_VIRT + 0x35C5C, 0x28000000, 0xFFFFFFFF},
  {IO_VIRT + 0x35c64, 0x00F00000, 0xffffffff},
  {IO_VIRT + 0x35c68, 0x01E00000, 0xffffffff},
  {IO_VIRT + 0x35c6c, 0x02D00000, 0xffffffff},
  {IO_VIRT + 0x35c70, 0x03C01000, 0xffffffff},
  {IO_VIRT + 0x35c74, 0x04B01000, 0xffffffff},
  {IO_VIRT + 0x35c78, 0x05A01000, 0xffffffff},
  {IO_VIRT + 0x35c7C, 0x06901000, 0xffffffff},
  {IO_VIRT + 0x35CD8, 0x00000000, 0xFFFFFFFF},
  
  //EPI
  {IO_VIRT + 0x35400, 0x00001601, 0xffffffff},
  {IO_VIRT + 0x35404, 0x76325410, 0xffffffff},
  {IO_VIRT + 0x3541c, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x35508, 0x00000600, 0xffffffff},
  {IO_VIRT + 0x35500, 0xfdfe0103, 0xffffffff},
  {IO_VIRT + 0x3551c, 0x04380006, 0xffffffff},
  {IO_VIRT + 0x35524, 0x00020002, 0xffffffff},
  {IO_VIRT + 0x35528, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3552c, 0x00000300, 0xffffffff},
  {IO_VIRT + 0x35530, 0x76543211, 0xffffffff},
  {IO_VIRT + 0x35534, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x3550c, 0x00000070, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000000, 0xffffffff},
  //{IO_VIRT + 0x35514, 0x00000001, 0xffffffff},
  //separate CTR2 TCON setting
  {IO_VIRT + 0x35580, 0x76543211, 0xffffffff}, //channel1 CTR2
  {IO_VIRT + 0x35584, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35588, 0x76543211, 0xffffffff}, //channel2 CTR2
  {IO_VIRT + 0x3558c, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35590, 0x76543211, 0xffffffff}, //channel3 CTR2
  {IO_VIRT + 0x35594, 0xfedcba08, 0xffffffff},
  {IO_VIRT + 0x35598, 0x76543211, 0xffffffff}, //channel4 CTR2
  {IO_VIRT + 0x3559c, 0xfedcba08, 0xffffffff},
  };

//-----------------------------------------------------------------------------
// Timing Control Signal Setting
//-----------------------------------------------------------------------------

//TCON LGD LGD V4 Non-GIP
  struct sTCONREGTABLE sRegTg_MLVDS_LGDV4FHD60[] = {

  // SOE, TG0
  {IO_VIRT + 0x34840, 0x01100035, 0xffffffff},
  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34848, 0x01100035, 0xffffffff},  
  {IO_VIRT + 0x3484c, 0x0443000A, 0xffffffff},
  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                    
  // POL, TG1,  2 Line Inversion                       
  {IO_VIRT + 0x34860, 0x076C076C, 0xffffffff},
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34868, 0x076C076C, 0xffffffff},  
  {IO_VIRT + 0x3486c, 0x04430009, 0xffffffff},
  {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
  {IO_VIRT + 0x34874, 0x80020004, 0xffffffff},
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3487c, 0x10000000, 0xffffffff},
   /*
  // POL, TG1,  1 Frame Inversion				   
  {IO_VIRT + 0x34860, 0x00010001, 0xffffffff},
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34868, 0x00010001, 0xffffffff},  
  {IO_VIRT + 0x3486c, 0x04430443, 0xffffffff},
  {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34874, 0x04430443, 0xffffffff},
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3487c, 0x80008001, 0xffffffff}, 
  */
  // GSP, TG11                      
  {IO_VIRT + 0x349A0, 0x05AB03F0, 0xffffffff},
  {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349Ac, 0x00090008, 0xffffffff},
  {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
  {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},
                                    
  // GSP_R, TG8                     
  {IO_VIRT + 0x34940, 0x05AB03F0, 0xffffffff},
  {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3494c, 0x00090008, 0xffffffff},
  {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
  {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff},
                                    
  // GOE, TG2                       
  {IO_VIRT + 0x34880, 0x08200642, 0xffffffff},
  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34888, 0x08200642, 0xffffffff},
  {IO_VIRT + 0x3488c, 0x04420009, 0xffffffff},
  {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},
                                    
  // GSC, TG12                      
  {IO_VIRT + 0x349c0, 0x079D03DB, 0xffffffff},
  {IO_VIRT + 0x349c4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349c8, 0x079D03DB, 0xffffffff},
  {IO_VIRT + 0x349cc, 0x04420009, 0xffffffff},
  {IO_VIRT + 0x349d0, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x349d4, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x349d8, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x349dc, 0x0f000000, 0xffffffff},
                                    
  // FLK, TG9                       
  {IO_VIRT + 0x34960, 0x049C0726, 0xffffffff},
  {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34968, 0x049C0726, 0xffffffff},
  {IO_VIRT + 0x3496c, 0x04420006, 0xffffffff},
  {IO_VIRT + 0x34970, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34974, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34978, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff},
  
  // DPM, TG10                      
  {IO_VIRT + 0x34820, 0x04000400, 0x04000400}, //GPO :1
                                    
  // H_CONV, TG5                    
  {IO_VIRT + 0x34820, 0x00000020, 0x00000020}, //GPO :0
                                    
  // OPT_N, TG6                     
  {IO_VIRT + 0x34820, 0x00400040, 0x00400040}, //GPO :1
                                    
  // 0PT_P , TG7                    
  {IO_VIRT + 0x34820, 0x00000080, 0x00000080}  //GPO :0

  };

  //TCON Samsung MB7 TRD 
  struct sTCONREGTABLE sRegTg_MLVDS_SAMSUNGMB7[] = {
  
  // SOE, TG0 for TP											 
  {IO_VIRT + 0x34840, 0x005a0038, 0xffffffff},    
  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x34848, 0x005a0038, 0xffffffff},    
  {IO_VIRT + 0x3484c, 0x00180019, 0xffffffff},    
  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},    
  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},    
  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},    
  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},    
														   
  // POL, TG1  for POL (1 Line Inversion) 					 
  //{IO_VIRT + 0x34860, 0x00600060, 0xffffffff},  
  //{IO_VIRT + 0x34864, 0x00000000, 0xffffffff},  
  //{IO_VIRT + 0x34868, 0x00600060, 0xffffffff},  
  //{IO_VIRT + 0x3486c, 0x0Cdf0007, 0xffffffff},  
  //{IO_VIRT + 0x34870, 0x80000202, 0xffffffff},  
  //{IO_VIRT + 0x34874, 0x80010002, 0xffffffff},  
  //{IO_VIRT + 0x34878, 0x00000007, 0xffffffff},  
  //{IO_VIRT + 0x3487c, 0x10000000, 0xffffffff},  
										   
  // POL, TG1  for POL (1 Frame Inversion)				   
  {IO_VIRT + 0x34860, 0x00010001, 0xffffffff},    
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x34868, 0x00010001, 0xffffffff},    
  {IO_VIRT + 0x3486c, 0x00050005, 0xffffffff},    
  {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x34874, 0x00050005, 0xffffffff},    
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},    
  {IO_VIRT + 0x3487c, 0x80008001, 0xffffffff}, 	 
										   
  // GSP, TG11 for STV										 
  {IO_VIRT + 0x349A0, 0x01a101a1, 0xffffffff},    
  {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x349Ac, 0x001A0016, 0xffffffff},    
  {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},    
  {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},    
															 
  // H-CONV, TG5 for CPV1					   
  {IO_VIRT + 0x348e0, 0x01900200, 0xffffffff},    
  {IO_VIRT + 0x348e4, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x348e8, 0x01900200, 0xffffffff},    
  {IO_VIRT + 0x348ec, 0x00160017, 0xffffffff},    
  {IO_VIRT + 0x348f0, 0x80000303, 0xffffffff},    
  {IO_VIRT + 0x348f4, 0x80020002, 0xffffffff},    
  {IO_VIRT + 0x348f8, 0x00000007, 0xffffffff},    
  {IO_VIRT + 0x348fc, 0x00000000, 0xffffffff},    
															 
  // GSC, TG12 for CPV2							 
  {IO_VIRT + 0x349C0, 0x01900200, 0xffffffff},    
  {IO_VIRT + 0x349C4, 0x00000000, 0xffffffff},    
  {IO_VIRT + 0x349C8, 0x01900200, 0xffffffff},    
  {IO_VIRT + 0x349Cc, 0x00160018, 0xffffffff},    
  {IO_VIRT + 0x349D0, 0x80000303, 0xffffffff},    
  {IO_VIRT + 0x349D4, 0x80010001, 0xffffffff},    
  {IO_VIRT + 0x349D8, 0x00000007, 0xffffffff},    
  {IO_VIRT + 0x349Dc, 0x0f000000, 0xffffffff}, 	 

  // GSP_R, TG8 for CPV3             
  {IO_VIRT + 0x34940, 0x01900200, 0xffffffff},
  {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34948, 0x01900200, 0xffffffff},
  {IO_VIRT + 0x3494c, 0x00160019, 0xffffffff},
  {IO_VIRT + 0x34950, 0x80000303, 0xffffffff},
  {IO_VIRT + 0x34954, 0x80030003, 0xffffffff},
  {IO_VIRT + 0x34958, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff},						
							 
  };

  //TCON CMO 2011 Flip-Pixel-2
  struct sTCONREGTABLE sRegTg_MLVDS_CMO2011FHD60[] = {
  
  // SOE, TG0
  {IO_VIRT + 0x34840, 0x01100035, 0xffffffff},
  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34848, 0x01100035, 0xffffffff},  
  {IO_VIRT + 0x3484c, 0x0443000A, 0xffffffff},
  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                    
  // POL, TG1,  2 Line Inversion                       
  {IO_VIRT + 0x34860, 0x076C076C, 0xffffffff},
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34868, 0x076C076C, 0xffffffff},  
  {IO_VIRT + 0x3486c, 0x04430009, 0xffffffff},
  {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
  {IO_VIRT + 0x34874, 0x80020004, 0xffffffff},
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3487c, 0x10000000, 0xffffffff},
   /*
  // POL, TG1,  1 Frame Inversion				   
  {IO_VIRT + 0x34860, 0x00010001, 0xffffffff},
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34868, 0x00010001, 0xffffffff},  
  {IO_VIRT + 0x3486c, 0x04430443, 0xffffffff},
  {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34874, 0x04430443, 0xffffffff},
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3487c, 0x80008001, 0xffffffff}, 
  */
  // GSP, TG11                      
  {IO_VIRT + 0x349A0, 0x05AB03F0, 0xffffffff},
  {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349Ac, 0x00090008, 0xffffffff},
  {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
  {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},
                                    
  // GSP_R, TG8                     
  {IO_VIRT + 0x34940, 0x05AB03F0, 0xffffffff},
  {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x3494c, 0x00090008, 0xffffffff},
  {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
  {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff},
                                    
  // GOE, TG2                       
  {IO_VIRT + 0x34880, 0x08200642, 0xffffffff},
  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34888, 0x08200642, 0xffffffff},
  {IO_VIRT + 0x3488c, 0x04420009, 0xffffffff},
  {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},
                                    
  // GSC, TG12                      
  {IO_VIRT + 0x349c0, 0x079D03DB, 0xffffffff},
  {IO_VIRT + 0x349c4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349c8, 0x079D03DB, 0xffffffff},
  {IO_VIRT + 0x349cc, 0x04420009, 0xffffffff},
  {IO_VIRT + 0x349d0, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x349d4, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x349d8, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x349dc, 0x0f000000, 0xffffffff},
	                                  
  };

  //TCON Panasonic 2011 WXGA
  struct sTCONREGTABLE sRegTg_MLVDS_PANASONIC32WXGA1P6P[] = {

	{IO_VIRT + 0x3481C, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34820, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34824, 0x00000000, 0xffffffff},

	// GOE, TP2 for DATA1
	{IO_VIRT + 0x34880, 0x03850321, 0xffffffff},
	{IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34888, 0x03850321, 0xffffffff},
	{IO_VIRT + 0x3488C, 0x00090008, 0xffffffff},
	{IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3489C, 0x00000000, 0xffffffff},  
                                          
	// TP14 for DATA1                 
	{IO_VIRT + 0x34A00, 0x03210321, 0xffffffff},
	{IO_VIRT + 0x34A04, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34A08, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34A0c, 0x000B000A, 0xffffffff},
	{IO_VIRT + 0x34A10, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34A14, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34A18, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x34A1c, 0x0F000000, 0xffffffff},  
                                          
	// HCONV, TPK5 for DATA2                 
	{IO_VIRT + 0x348E0, 0x03850321, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x03850321, 0xffffffff},
	{IO_VIRT + 0x348EC, 0x000A0009, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348FC, 0x00000000, 0xffffffff},  
                                          
	// TPK1 for DATA2                 
	{IO_VIRT + 0x34860, 0x03210321, 0xffffffff},
	{IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x3486C, 0x030B030A, 0xffffffff},
	{IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34878, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3487C, 0x00000000, 0xffffffff},  
                                          
	// OR merge TP2 and TP14 to DATA2 
	// OR merge TP5 and TP1   to DATA1
	{IO_VIRT + 0x34814, 0x00100E00, 0xffffffff},
	{IO_VIRT + 0x34818, 0x00000410, 0xffffffff},  
                                          
	// OPT_N, TPK6 for CPV		   
	{IO_VIRT + 0x34900, 0x03B40358, 0xffffffff},
	{IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34908, 0x03B40358, 0xffffffff},
	{IO_VIRT + 0x3490C, 0x04420008, 0xffffffff},
	{IO_VIRT + 0x34910, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34914, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3491C, 0x00000000, 0xffffffff},
                                          
	// SOE, TPK0 for LP		           
	{IO_VIRT + 0x34840, 0x00600B9D, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x00600B9D, 0xffffffff},
	{IO_VIRT + 0x3484C, 0x04420009, 0xffffffff},
	{IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3485C, 0x00000000, 0xffffffff},
                                          
	// OPT_P, TPK7 for POLL						   
	{IO_VIRT + 0x34920, 0x04550455, 0xffffffff},
	{IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34928, 0x04550455, 0xffffffff},
	{IO_VIRT + 0x3492C, 0x00080008, 0xffffffff},
	{IO_VIRT + 0x34930, 0x80000404, 0xffffffff},
	{IO_VIRT + 0x34934, 0x80040002, 0xffffffff},
	{IO_VIRT + 0x34938, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3493C, 0x10000000, 0xffffffff},
								   
	// GSC, TPK12 for POLR                 
	{IO_VIRT + 0x349C0, 0x04550455, 0xffffffff}, 																													  
	{IO_VIRT + 0x349C4, 0x00000000, 0xffffffff}, 																													  
	{IO_VIRT + 0x349C8, 0x04550455, 0xffffffff}, 																													  
	{IO_VIRT + 0x349CC, 0x00080008, 0xffffffff}, 																													  
	{IO_VIRT + 0x349D0, 0x80000404, 0xffffffff}, 																													  
	{IO_VIRT + 0x349D4, 0x80040002, 0xffffffff}, 																													  
	{IO_VIRT + 0x349D8, 0x00000087, 0xffffffff}, 																													  
	{IO_VIRT + 0x349DC, 0x1F000000, 0xffffffff},

  };
  
   struct sTCONREGTABLE sRegTg_MLVDS_PANASONIC32WXGA2P3P[] = {

	// TP2 GOE for DATA1
	{IO_VIRT + 0x34880, 0x08200765, 0xffffffff},
	{IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34888, 0x08200765, 0xffffffff},
	{IO_VIRT + 0x3488C, 0x00090008, 0xffffffff},
	{IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34894, 0x00080009, 0xffffffff},
	{IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3489C, 0x00000000, 0xffffffff},  
                                          
	 // TP14 for DATA1                
	{IO_VIRT + 0x34A00, 0x0C000765, 0xffffffff},
	{IO_VIRT + 0x34A04, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34A08, 0x0C000C00, 0xffffffff},
	{IO_VIRT + 0x34A0c, 0x000B000A, 0xffffffff},
	{IO_VIRT + 0x34A10, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34A14, 0x000A000B, 0xffffffff},
	{IO_VIRT + 0x34A18, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x34A1c, 0x0F000000, 0xffffffff},  
                                          
	 // TPK5 H-CONV for DATA2                
	{IO_VIRT + 0x348E0, 0x0C000765, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x07650765, 0xffffffff},
	{IO_VIRT + 0x348EC, 0x00090008, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x00080009, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348FC, 0x00000000, 0xffffffff},  
                                          
	 // TPK1 for DATA2                
	{IO_VIRT + 0x34860, 0x0C000765, 0xffffffff},
	{IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34868, 0x08200765, 0xffffffff},
	{IO_VIRT + 0x3486C, 0x0307000A, 0xffffffff},
	{IO_VIRT + 0x34870, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34874, 0x000A0307, 0xffffffff},
	{IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3487C, 0x00000000, 0xffffffff},  
                                   
	// OR merge TP2 and TP14 to DATA1 
	// OR merge TP5 and TP1   to DATA2
	{IO_VIRT + 0x34810, 0x000000D0, 0xffffffff},
	{IO_VIRT + 0x34814, 0x00160E00, 0xffffffff},
	{IO_VIRT + 0x34818, 0x00040510, 0xffffffff},  
                                        
	// TPK6 OPT_N for CPV		   
	{IO_VIRT + 0x34900, 0x087007D0, 0xffffffff},
	{IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34908, 0x087007D0, 0xffffffff},
	{IO_VIRT + 0x3490C, 0x00070008, 0xffffffff},
	{IO_VIRT + 0x34910, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34914, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3491C, 0x00000000, 0xffffffff},
                                          
	// TPK0 SOE for LP		           
	{IO_VIRT + 0x34840, 0x00600B9D, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x00600B9D, 0xffffffff},
	{IO_VIRT + 0x3484C, 0x00070008, 0xffffffff},
	{IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3485C, 0x00000000, 0xffffffff},
                                          
	// TPK7 OPT_P for POLL						   
	{IO_VIRT + 0x34920, 0x05950595, 0xffffffff},
	{IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34928, 0x05950595, 0xffffffff},
	{IO_VIRT + 0x3492C, 0x00070008, 0xffffffff},
	{IO_VIRT + 0x34930, 0x80000404, 0xffffffff},
	{IO_VIRT + 0x34934, 0x80040002, 0xffffffff},
	{IO_VIRT + 0x34938, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3493C, 0x10000000, 0xffffffff},

  };
   
  //TCON Panasonic FHD 120Hz
	struct sTCONREGTABLE sRegTg_MLVDS_PANASONIC42FHD120[] = {
  
	  {IO_VIRT + 0x3481C, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34820, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34824, 0x00000000, 0xffffffff},
  
	  // GOE, TP2 for DATA1
	  {IO_VIRT + 0x34880, 0x0CA00AB0, 0xffffffff},
	  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34888, 0x01A00AB0, 0xffffffff}, 
	  {IO_VIRT + 0x3488C, 0x04420441, 0xffffffff},
	  {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34894, 0x04420442, 0xffffffff},
	  {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3489C, 0x00000000, 0xffffffff},  
											
	  // TP14 for DATA1 				
	  {IO_VIRT + 0x34A00, 0x01A00AB0, 0xffffffff},
	  {IO_VIRT + 0x34A04, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34A08, 0x01A00AB0, 0xffffffff},
	  {IO_VIRT + 0x34A0c, 0x04450443, 0xffffffff},
	  {IO_VIRT + 0x34A10, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34A14, 0x04440444, 0xffffffff},
	  {IO_VIRT + 0x34A18, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x34A1c, 0x0F000000, 0xffffffff},  
											
	  // HCONV, TPK5 for DATA2				   
	  {IO_VIRT + 0x348E0, 0x01A00AB0, 0xffffffff},
	  {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348E8, 0x01A001A0, 0xffffffff},
	  {IO_VIRT + 0x348EC, 0x04420441, 0xffffffff},
	  {IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x348F4, 0x04420442, 0xffffffff},
	  {IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x348FC, 0x00000000, 0xffffffff},  
											
	  // TPK1 for DATA2 				
	  {IO_VIRT + 0x34860, 0x01A00AB0, 0xffffffff},
	  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34868, 0x0CA001A0, 0xffffffff},
	  {IO_VIRT + 0x3486C, 0x04450443, 0xffffffff},
	  {IO_VIRT + 0x34870, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34874, 0x04430445, 0xffffffff},
	  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3487C, 0x00000000, 0xffffffff},  
	  
	  // TPK3 for DATA2 				
	  {IO_VIRT + 0x348A0, 0x0CA00AB0, 0xffffffff},
	  {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348A8, 0x0CA00AB0, 0xffffffff},
	  {IO_VIRT + 0x348AC, 0x00090008, 0xffffffff},
	  {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x348B4, 0x00080007, 0xffffffff},
	  {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x348BC, 0x00000000, 0xffffffff},  
	  
	  // OR merge TP2 and TP14 to DATA2 
	  // OR merge TP5 and TP1	to DATA1
	  {IO_VIRT + 0x34814, 0x00100E00, 0xffffffff},
	  {IO_VIRT + 0x34818, 0x00000410, 0xffffffff},  

	  // OR merge TP2, TP14,and TP3 to DATA2 
	  {IO_VIRT + 0x34828, 0x76343210, 0xffffffff},	
	  
	  {IO_VIRT + 0x3483C, 0x76243510, 0xffffffff},	
	  
	  // OPT_N, TPK6 for CPV		 
	  {IO_VIRT + 0x34900, 0x00A00BC0, 0xffffffff},
	  {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34908, 0x00A00BC0, 0xffffffff},
	  {IO_VIRT + 0x3490C, 0x00080008, 0xffffffff},
	  {IO_VIRT + 0x34910, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34914, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3491C, 0x00000000, 0xffffffff},
											
	  // SOE, TPK0 for LP				 
	  {IO_VIRT + 0x34840, 0x01500040, 0xffffffff},
	  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34848, 0x01500040, 0xffffffff},
	  {IO_VIRT + 0x3484C, 0x00080009, 0xffffffff},
	  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3485C, 0x00000000, 0xffffffff},
											
	  // OPT_P, TPK7 for POLL						 
	  {IO_VIRT + 0x34920, 0x00C800C8, 0xffffffff},
	  {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34928, 0x00C800C8, 0xffffffff},
	  {IO_VIRT + 0x3492C, 0x00050005, 0xffffffff},
	  {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34934, 0x00050005, 0xffffffff},
	  {IO_VIRT + 0x34938, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3493C, 0x80008001, 0xffffffff},
									 
	  // GSC, TPK12 for POLR				 
	  {IO_VIRT + 0x349C0, 0x00C800C8, 0xffffffff}, 																													
	  {IO_VIRT + 0x349C4, 0x00000000, 0xffffffff}, 																													
	  {IO_VIRT + 0x349C8, 0x00C800C8, 0xffffffff}, 																													
	  {IO_VIRT + 0x349CC, 0x00050005, 0xffffffff}, 																													
	  {IO_VIRT + 0x349D0, 0x00000000, 0xffffffff}, 																													
	  {IO_VIRT + 0x349D4, 0x00050005, 0xffffffff}, 																													
	  {IO_VIRT + 0x349D8, 0x00000087, 0xffffffff}, 																													
	  {IO_VIRT + 0x349DC, 0x8F002001, 0xffffffff},
  
	};

struct sTCONREGTABLE sRegTg_MLVDS_LGD32WXGAV9[] = {
	
  {IO_VIRT + 0x34838, 0xFEDCBA28, 0xffffffff},

	// SOE, TG0
  {IO_VIRT + 0x34840, 0x00D00002, 0xffffffff},
  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34848, 0x00D00002, 0xffffffff},  
  {IO_VIRT + 0x3484c, 0x00060007, 0xffffffff},
  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
	
  // POL, TG1
  {IO_VIRT + 0x34860, 0x0B300B30, 0xffffffff},
  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34868, 0x0B300B30, 0xffffffff},  
  {IO_VIRT + 0x3486c, 0x04430007, 0xffffffff},
  {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
  {IO_VIRT + 0x34874, 0x80010003, 0xffffffff},
  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3487c, 0x10000000, 0xffffffff},
	
  // GSP, TG11
  {IO_VIRT + 0x349A0, 0x05200520, 0xffffffff},
  {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349Ac, 0x00080007, 0xffffffff},
  {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
  {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},
	
  // GOE, TG2
  {IO_VIRT + 0x34880, 0x003009A0, 0xffffffff},
  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x34888, 0x003009A0, 0xffffffff},
  {IO_VIRT + 0x3488c, 0x04420006, 0xffffffff},
  {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},
	  
  // GSC, TG12
  {IO_VIRT + 0x349c0, 0x05200B30, 0xffffffff},
  {IO_VIRT + 0x349c4, 0x00000000, 0xffffffff},
  {IO_VIRT + 0x349c8, 0x05200B30, 0xffffffff},
  {IO_VIRT + 0x349cc, 0x04420006, 0xffffffff},
  {IO_VIRT + 0x349d0, 0x20000000, 0xffffffff},
  {IO_VIRT + 0x349d4, 0x80000000, 0xffffffff},
  {IO_VIRT + 0x349d8, 0x00000007, 0xffffffff},
  {IO_VIRT + 0x349dc, 0x0f000000, 0xffffffff},
	  
  // H_CONV, TG5
  {IO_VIRT + 0x34820, 0x00000020, 0x00000020}, //GPO :0

};

struct sTCONREGTABLE sRegTg_MLVDS_CHINASTARWXGA8BIT[] = {
	
	// POL, TG0, OK	  
	{IO_VIRT + 0x34840, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x02F00001, 0xffffffff},  
	{IO_VIRT + 0x3484c, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x00140001, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3485c, 0x10000000, 0xffffffff},   
	  
	// TP1, TG1, OK 
	{IO_VIRT + 0x34860, 0x00AC0034, 0xffffffff},
	{IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34868, 0x00AC0034, 0xffffffff},  
	{IO_VIRT + 0x3486c, 0x09170015, 0xffffffff},
	{IO_VIRT + 0x34870, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x34874, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3487c, 0x00000000, 0xffffffff},
	  
	// GSP for STV, TG11,   OK
	{IO_VIRT + 0x349A0, 0x02300235, 0xffffffff},
	{IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x349Ac, 0x00180016, 0xffffffff},
	{IO_VIRT + 0x349B0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x349B4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},
	  
	// GOE for OE, TG2, OK
	{IO_VIRT + 0x34820,0x00040004,0x00040004}, //GPO :1
		  
	// GSC for CKV, TG12,	OK
	{IO_VIRT + 0x349C0, 0x02110016, 0xffffffff},
	{IO_VIRT + 0x349C4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x349C8, 0x02110016, 0xffffffff},
	{IO_VIRT + 0x349Cc, 0x00150016, 0xffffffff},
	{IO_VIRT + 0x349D0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x349D4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x349D8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x349Dc, 0x0F000000, 0xffffffff},
  
	// H_CONV for GVON, TG5,	OK				  
	{IO_VIRT + 0x348E0, 0x01F40040, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x01F40040, 0xffffffff},
	{IO_VIRT + 0x348Ec, 0x00150016, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},  

};

struct sTCONREGTABLE sRegTg_MLVDS_HISENSE_HE420FF[] = {

   //GR0
   //   {IO_VIRT + 0x34800, 0x00000000, 0xffffffff},
   //GR4
   {IO_VIRT + 0x34810, 0x000D0000, 0xffffffff},
   //GR5
   {IO_VIRT + 0x34814, 0x00000E0F, 0xffffffff},  //Bmux select
   //GR6
   {IO_VIRT + 0x34818, 0x03000033, 0xffffffff},  //combination
   //GR7							  
   {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff}, 

																											  
   // GSP, TG11 for YDIOU/STV
   {IO_VIRT + 0x349A0, 0x063808A0, 0xffffffff},
   {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349A8, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349Ac, 0x000A0009, 0xffffffff},
   {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
   {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},

   // GSC, TG12 //----------TG12^TG13--------//  for YCLK/CKV
   {IO_VIRT + 0x349C0, 0x00FA0001, 0xffffffff},
   {IO_VIRT + 0x349C4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349C8, 0x00FA0001, 0xffffffff},
   {IO_VIRT + 0x349Cc, 0x0008000A, 0xffffffff},
   {IO_VIRT + 0x349D0, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x349D4, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x349D8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x349Dc, 0x0F000000, 0xffffffff},

   {IO_VIRT + 0x349E0, 0x00FA0001, 0xffffffff},
   {IO_VIRT + 0x349E4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x349E8, 0x00FA0001, 0xffffffff},
   {IO_VIRT + 0x349Ec, 0x04420442, 0xffffffff},
   {IO_VIRT + 0x349F0, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x349F4, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x349F8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x349Fc, 0x0F000000, 0xffffffff},

   //GOE, TG2   //----------TG2^TG14--------//  for YOE/OE
   {IO_VIRT + 0x34880, 0x0C0A029B, 0xffffffff},
   {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34888, 0x0C0A029B, 0xffffffff},
   {IO_VIRT + 0x3488c, 0x0442000A, 0xffffffff},
   {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},

   {IO_VIRT + 0x34A00, 0x0C0A029B, 0xffffffff},
   {IO_VIRT + 0x34A04, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34A08, 0x0C0A029B, 0xffffffff},
   {IO_VIRT + 0x34A0c, 0x04440442, 0xffffffff},
   {IO_VIRT + 0x34A10, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34A14, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34A18, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x34A1c, 0x0F000000, 0xffffffff},


   // SOE, TG0 //----------TG0^TG15--------//   for STB2/TP
   {IO_VIRT + 0x34840, 0x029B0001, 0xffffffff},
   {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34848, 0x029B0001, 0xffffffff},  
   {IO_VIRT + 0x3484c, 0x0008000A, 0xffffffff},
   {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},

   {IO_VIRT + 0x34A20, 0x029B0001, 0xffffffff},
   {IO_VIRT + 0x34A24, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34A28, 0x029B0001, 0xffffffff},  
   {IO_VIRT + 0x34A2c, 0x04420442, 0xffffffff},
   {IO_VIRT + 0x34A30, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34A34, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34A38, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x34A3c, 0x0F000000, 0xffffffff},

   // POL_A, TG5   for POL_L
   {IO_VIRT + 0x348E0, 0x07E507E5, 0xffffffff},
   {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348E8, 0x07E507E5, 0xffffffff},  
   {IO_VIRT + 0x348Ec, 0x00070007, 0xffffffff},
   {IO_VIRT + 0x348F0, 0x80000202, 0xffffffff},
   {IO_VIRT + 0x348F4, 0x80010002, 0xffffffff},
   {IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x348Fc, 0x10000000, 0xffffffff},

   // POL, TG1	for POL  or POL_R
   {IO_VIRT + 0x34860, 0x07E507E5, 0xffffffff},
   {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},																													  
   {IO_VIRT + 0x34868, 0x07E507E5, 0xffffffff},
   {IO_VIRT + 0x3486c, 0x00070007, 0xffffffff},
   {IO_VIRT + 0x34870, 0x80000202, 0xffffffff},																													  
   {IO_VIRT + 0x34874, 0x80010002, 0xffffffff},
   {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},																													  
   {IO_VIRT + 0x3487c, 0x10000000, 0xffffffff},		

   //GSP_R, TG8  for YDIOD
   {IO_VIRT + 0x34940, 0x063808A0, 0xffffffff},
   {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x3494c, 0x000C000B, 0xffffffff},
   {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
   {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff},
 
   // YDIOD test out is TG6
   {IO_VIRT + 0x34900, 0x063808A0, 0xffffffff},
   {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34908, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x3490c, 0x000C000B, 0xffffffff},
   {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34914, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34918, 0x00000004, 0xffffffff},
   {IO_VIRT + 0x3491c, 0x00000000, 0xffffffff},

   // VGH_CTL, TG7   for Power IC
   {IO_VIRT + 0x34920, 0x01e50800, 0xffffffff},
   {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34928, 0x01e50800, 0xffffffff},
   {IO_VIRT + 0x3492c, 0x04420008, 0xffffffff},
   {IO_VIRT + 0x34930, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34934, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34938, 0x00000087, 0xffffffff},
   {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},
};
 
  struct sTCONREGTABLE sRegTg_MLVDS_HISENSE32WXGA2P3P[] = {
 
   // TCON1 for POL 					  
   {IO_VIRT + 0x34860, 0x03F003F0, 0xffffffff},
   {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34868, 0x03F003F0, 0xffffffff},
   {IO_VIRT + 0x3486C, 0x00080008, 0xffffffff},
   {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34874, 0x00080008, 0xffffffff},
   {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3487C, 0x80008001, 0xffffffff},
 
   // TCON0 for TP						  
   {IO_VIRT + 0x34840, 0x007D08BE, 0xffffffff}, //{IO_VIRT + 0x348E0, 0x004C08D9, 0xffffffff},
   {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34848, 0x007D08BE, 0xffffffff}, //{IO_VIRT + 0x348E8, 0x004C08D9, 0xffffffff},
   {IO_VIRT + 0x3484C, 0x00080008, 0xffffffff}, 
   {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3485C, 0x00000000, 0xffffffff},									   
 
   // TCON6 for VST 						  
   {IO_VIRT + 0x34900, 0x04C504B5, 0xffffffff}, //{IO_VIRT + 0x34860, 0x05570562, 0xffffffff},
   {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34908, 0x00000000, 0xffffffff}, 
   {IO_VIRT + 0x3490C, 0x000A0009, 0xffffffff},
   {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34914, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34918, 0x00000004, 0xffffffff},
   {IO_VIRT + 0x3491C, 0x00000000, 0xffffffff},  
 
   // TCON5 for CPV 						  
   {IO_VIRT + 0x348E0, 0x037506CC, 0xffffffff}, //{IO_VIRT + 0x348C0, 0x025706E0, 0xffffffff},
   {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348E8, 0x037506CC, 0xffffffff}, //{IO_VIRT + 0x348C8, 0x025706E0, 0xffffffff},
   {IO_VIRT + 0x348EC, 0x00090009, 0xffffffff},
   {IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x348F4, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x348FC, 0x00000000, 0xffffffff},
 
   // TCON2 for OE		  
   {IO_VIRT + 0x34820, 0x00040004, 0xffffffff}, //GPO :1
 
   // TCON9 for FLK 					  
   {IO_VIRT + 0x34960, 0x06CB0578, 0xffffffff}, //{IO_VIRT + 0x34880, 0x05AF0730, 0xffffffff}, 
   {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34968, 0x06CB0578, 0xffffffff}, //{IO_VIRT + 0x34888, 0x05AF0730, 0xffffffff},
   {IO_VIRT + 0x3496C, 0x00080009, 0xffffffff}, //{IO_VIRT + 0x3488C, 0x00090009, 0xffffffff},
   {IO_VIRT + 0x34970, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34974, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34978, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3497C, 0x00000000, 0xffffffff},										 
 };
 
  struct sTCONREGTABLE sRegTg_MLVDS_HISENSE32WXGA1P6P[] = {
 
   // TCON0 for POL 					  
   {IO_VIRT + 0x34840, 0x00E20112, 0xffffffff},
   {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34848, 0x00E20112, 0xffffffff},
   {IO_VIRT + 0x3484C, 0x043F0002, 0xffffffff},
   {IO_VIRT + 0x34850, 0x80000404, 0xffffffff},
   {IO_VIRT + 0x34854, 0x80040002, 0xffffffff},
   {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3485C, 0x20000000, 0xffffffff},
 
   // TCON5 for TP						  
   {IO_VIRT + 0x348E0, 0x007D08BE, 0xffffffff},
   {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348E8, 0x007D08BE, 0xffffffff}, 
   {IO_VIRT + 0x348EC, 0x00080008, 0xffffffff}, 
   {IO_VIRT + 0x348F0, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x348F4, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x348F8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x348FC, 0x00000000, 0xffffffff},
 
   // TCON4 for STV 						  
   {IO_VIRT + 0x348C0, 0x04C504B5, 0xffffffff}, 
   {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348C8, 0x00000000, 0xffffffff}, 
   {IO_VIRT + 0x348CC, 0x000A0009, 0xffffffff},
   {IO_VIRT + 0x348D0, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348D4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348D8, 0x00000004, 0xffffffff},
   {IO_VIRT + 0x348DC, 0x00000000, 0xffffffff},  
 
   // TCON2 for CPV 						  
   {IO_VIRT + 0x34880, 0x037506CC, 0xffffffff}, 
   {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x34888, 0x037506CC, 0xffffffff}, 
   {IO_VIRT + 0x3488C, 0x00090009, 0xffffffff},
   {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x3489C, 0x00000000, 0xffffffff},
 
   // TCON1 for OE		  
   {IO_VIRT + 0x34820, 0x00020002, 0x00020002}, //GPO :1
 
   // TCON3 for FLK 					  
   {IO_VIRT + 0x348A0, 0x06CB0578, 0xffffffff}, 
   {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
   {IO_VIRT + 0x348A8, 0x06CB0578, 0xffffffff}, 
   {IO_VIRT + 0x348AC, 0x00080009, 0xffffffff}, 
   {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
   {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
   {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
   {IO_VIRT + 0x348BC, 0x00000000, 0xffffffff},										 
 };

struct sTCONREGTABLE sRegTg_MLVDS_LGD32WXGAV12[] = {					
  
      // SOE, TCO[0]	IO_VIRT + 0x34840
	  {IO_VIRT + 0x34840, 0x00011107, 0xffffffff},
	  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34848, 0x00011107, 0xffffffff},  
	  {IO_VIRT + 0x3484c, 0x00030004, 0xffffffff},
	  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
 	  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
										
	  //POL    TCO[1]  IO_VIRT + 0x34860
	  {IO_VIRT + 0x34860, 0x10FC10FC, 0xffffffff},
	  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34868, 0x10FC10FC, 0xffffffff},
	  {IO_VIRT + 0x3486c, 0x031C0001, 0xffffffff},
	  {IO_VIRT + 0x34870, 0x80000202, 0xffffffff},
	  {IO_VIRT + 0x34874, 0x80010002, 0xffffffff},
	  {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},

	  //GSP	TCO[2] IO_VIRT + 0x34880
	  {IO_VIRT + 0x34880, 0x02A002B5, 0xffffffff},
   	  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34888, 0x02A002B5, 0xffffffff},
      {IO_VIRT + 0x3488c, 0x00050003, 0xffffffff},
      {IO_VIRT + 0x34890, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34894, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34898, 0x00000004, 0xffffffff},
      {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},

	  //GST    TCO[5]  IO_VIRT + 0x348e0
	  {IO_VIRT + 0x348E0, 0x02A002B5, 0xffffffff},
	  {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348E8, 0x02A002B5, 0xffffffff},
	  {IO_VIRT + 0x348Ec, 0x00050003, 0xffffffff},
	  {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

	  //GST_END    TCO[9]  IO_VIRT + 0x34960
	  {IO_VIRT + 0x34960, 0x004E016C, 0xffffffff},
	  {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34968, 0x004E016C, 0xffffffff},
	  {IO_VIRT + 0x3496c, 0x0310030D, 0xffffffff},
	  {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff},
	  
	  //MCLK	TCO[4]	IO_VIRT + 0x348c0
	  {IO_VIRT + 0x348C0, 0x10000C6A, 0xffffffff},
	  {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348C8, 0x10000C6A, 0xffffffff},
	  {IO_VIRT + 0x348Cc, 0x030B0007, 0xffffffff},
	  {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},
  
	  //GCLK	TCO[3]	IO_VIRT + 0x348a0
	  {IO_VIRT + 0x348A0, 0x0A8902AB, 0xffffffff},
	  {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348A8, 0x0A8902AB, 0xffffffff},
	  {IO_VIRT + 0x348Ac, 0x03090005, 0xffffffff},
	  {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},
  
	  //GCLK_END	TCO[8]	IO_VIRT + 0x34940
	  {IO_VIRT + 0x34940, 0x004E004E, 0xffffffff},
	  {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34948, 0x004E004E, 0xffffffff},
	  {IO_VIRT + 0x3494c, 0x03100309, 0xffffffff},
	  {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 
  
	  //EO	 TCO[6]  IO_VIRT + 0x34900
	  {IO_VIRT + 0x34900, 0x05000080, 0xffffffff},
	  {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34908, 0x05000080, 0xffffffff},
	  {IO_VIRT + 0x3490c, 0x03110311, 0xffffffff},
	  {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34914, 0x03110311, 0xffffffff},
	  {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3491c, 0x80200080, 0xffffffff}, 
	  
  	  //EO2    TCO[10]	IO_VIRT + 0x34980
	  {IO_VIRT + 0x34980, 0x05000080, 0xffffffff},
	  {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34988, 0x05000080, 0xffffffff},
	  {IO_VIRT + 0x3498c, 0x03110311, 0xffffffff},
	  {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34994, 0x03110311, 0xffffffff},
	  {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3499c, 0x80400100, 0xffffffff},
	  
	  //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
	  {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
	  {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
	  {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
	  {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff}, 
  
	  {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
										
      {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
	  {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
	  {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
	  {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

};

struct sTCONREGTABLE sRegTg_MLVDS_LGD32WXGAV13[] = {					
  
      // SOE, TCO[0]	IO_VIRT + 0x34840
	  {IO_VIRT + 0x34840, 0x00CB0060, 0xffffffff},
	  {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34848, 0x00CB0060, 0xffffffff},  
	  {IO_VIRT + 0x3484c, 0x000A000C, 0xffffffff},
	  {IO_VIRT + 0x34850, 0x20000000, 0xffffffff},
 	  {IO_VIRT + 0x34854, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34858, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
										
	  //POL    TCO[4]  IO_VIRT + 0x348C0
	  {IO_VIRT + 0x348C0, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348C8, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348Cc, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348D0, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348D4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x348D8, 0x00000080, 0xffffffff},
	  {IO_VIRT + 0x348Dc, 0x20000000, 0xffffffff},

	  //GSP	TCO[1] IO_VIRT + 0x34860
	  {IO_VIRT + 0x34860, 0x00010001, 0xffffffff},
   	  {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34868, 0x00010001, 0xffffffff},
      {IO_VIRT + 0x3486c, 0x00120011, 0xffffffff},
      {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
      {IO_VIRT + 0x34878, 0x00000004, 0xffffffff},
      {IO_VIRT + 0x3487c, 0x00000000, 0xffffffff},

	  //GST    TCO[11]  IO_VIRT + 0x349A0
	  {IO_VIRT + 0x349A0, 0x000D003E, 0xffffffff},
	  {IO_VIRT + 0x349A4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x349A8, 0x000D003E, 0xffffffff},
	  {IO_VIRT + 0x349Ac, 0x000D000B, 0xffffffff},
	  {IO_VIRT + 0x349B0, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x349B4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x349B8, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x349Bc, 0x00000000, 0xffffffff},

	  //GST_END    TCO[7]  IO_VIRT + 0x34920
	  {IO_VIRT + 0x34920, 0x01810001, 0xffffffff},
	  {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34928, 0x01810001, 0xffffffff},
	  {IO_VIRT + 0x3492c, 0x06160619, 0xffffffff},
	  {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34938, 0x00000084, 0xffffffff},
	  {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},
	  
	  //MCLK	TCO[2]	IO_VIRT + 0x34880
	  {IO_VIRT + 0x34880, 0x03200301, 0xffffffff},
	  {IO_VIRT + 0x34884, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34888, 0x026E0210, 0xffffffff},
	  {IO_VIRT + 0x3488c, 0x0615000E, 0xffffffff},
	  {IO_VIRT + 0x34890, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34894, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34898, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3489c, 0x00000000, 0xffffffff},
  
	  //GCLK	TCO[12]	IO_VIRT + 0x349C0
	  {IO_VIRT + 0x349C0, 0x02010075, 0xffffffff},
	  {IO_VIRT + 0x349C4, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x349C8, 0x01C70076, 0xffffffff},
	  {IO_VIRT + 0x349Cc, 0x1611000D, 0xffffffff},
	  {IO_VIRT + 0x349D0, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x349D4, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x349D8, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x349Dc, 0x0F000000, 0xffffffff},
  
	  //GCLK_END	TCO[8]	IO_VIRT + 0x34940
	  {IO_VIRT + 0x34940, 0x013E0081, 0xffffffff},
	  {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34948, 0x013E0081, 0xffffffff},
	  {IO_VIRT + 0x3494c, 0x06190612, 0xffffffff},
	  {IO_VIRT + 0x34950, 0x20000000, 0xffffffff},
	  {IO_VIRT + 0x34954, 0x80000000, 0xffffffff},
	  {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 
  
	  //EO	 TCO[6]  IO_VIRT + 0x34900
	  {IO_VIRT + 0x34900, 0x01BA0163, 0xffffffff},
	  {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34908, 0x01BA0163, 0xffffffff},
	  {IO_VIRT + 0x3490c, 0x061A061A, 0xffffffff},
	  {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34914, 0x061A061A, 0xffffffff},
	  {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3491c, 0x80200080, 0xffffffff}, 
	  
  	  //EO2    TCO[10]	IO_VIRT + 0x34980
	  {IO_VIRT + 0x34980, 0x01BA0163, 0xffffffff},
	  {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34988, 0x01BA0163, 0xffffffff},
	  {IO_VIRT + 0x3498c, 0x061A061A, 0xffffffff},
	  {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34994, 0x061A061A, 0xffffffff},
	  {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
	  {IO_VIRT + 0x3499c, 0x80400100, 0xffffffff},
	  
	  /*//VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
	  {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
	  {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
	  {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
	  {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
	  {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},*/ 
  
	  {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
										
      {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
	  {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x3480C, 0x00000000, 0xffffffff},
	  {IO_VIRT + 0x34814, 0x0A000000, 0xffffffff}, //Bmux select
	  {IO_VIRT + 0x34810, 0x00087000, 0xffffffff},
	  {IO_VIRT + 0x34818, 0x01401000, 0xffffffff}, //combination
	  {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

};

//SOE table
//         60Hz                      120Hz
//         start(T1) width(T2)  start(T1) width(T2)
//32 Inch  5            50            x            x
//42 Inch  5            43            5            24
//47 Inch  5            43            5            32
//55 Inch  5            43            5            32

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD120[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00001805, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00001805, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00001805, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00001805, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00001805, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00001805, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00001805, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04430007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410004, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04480441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04480445, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD60[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002B05, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002B05, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002B05, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002B05, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002B05, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002B05, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002B05, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x0082007E, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x0082007E, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410004, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04460441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00840074, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00840074, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04470447, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04470447, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04450442, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80200080, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };

//#define EPIV12FHD120_42_TABLE
//#define EPIV12FHD120_47_TABLE
//#define EPIV12FHD120_55_TABLE
//#define EPIV12FHD60_32_TABLE
//#define EPIV12FHD60_42_TABLE
//#define EPIV12FHD60_47_TABLE
//#define EPIV12FHD60_55_TABLE
//SOE table
//         60Hz                      120Hz
//         start(T1) width(T2)  start(T1) width(T2)
//32 Inch  5            50            x            x
//42 Inch  5            43            5            24
//47 Inch  5            43            5            32
//55 Inch  5            43            5            32

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD120_42[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00001805, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00001805, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00001805, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00001805, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00001805, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00001805, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00001805, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04430007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410004, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04480441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04480445, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD120_47[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002005, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002005, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002005, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002005, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002005, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002005, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002005, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x000B000B, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x000B000B, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00FB00F3, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00FB00F3, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x00D6003D, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x00D6003D, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04430005, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x003D003D, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x003D003D, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x003D003D, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x003D003D, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x044A0446, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD120_55[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002005, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002005, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002005, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002005, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002005, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002005, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002005, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x00100003, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00FD00F5, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04430007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x0064014F, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410004, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04480441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00380018, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04490449, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0014014F, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04480445, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD60_32[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00003205, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00003205, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00003205, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00003205, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00003205, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00003205, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00003205, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x0020000B, 0xffffffff}, //0x0013000B -> 0x0020000B
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x0020000B, 0xffffffff}, //0x0013000B -> 0x0020000B,
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00FA00F9, 0xffffffff}, //0x01050104 -> 0x00FA00F9
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00FA00F9, 0xffffffff}, //0x01050104 -> 0x00FA00F9
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x00B20162, 0xffffffff}, // 0x00B20010 -> 0x00B20162
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x00B20162, 0xffffffff}, // 0x00B20010 -> 0x00B20162
    {IO_VIRT + 0x348Ac, 0x04450004, 0xffffffff}, // 0x04430003 -> 0x04430002
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x00010001, 0xffffffff}, // 0x007000DC --> 0x00010001
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x00010001, 0xffffffff}, // 0x007000DC --> 0x00010001
    {IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff}, // 0x0444043F --> 0x04480440
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00140004, 0xffffffff}, // 0x00840074 --> 0x00140004
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00140004, 0xffffffff}, // 0x00840074 --> 0x00140004
    {IO_VIRT + 0x3490c, 0x044D044D, 0xffffffff}, // 0x04450445 --> 0x044B044B
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x044D044D, 0xffffffff}, // 0x04450445 --> 0x044B044B
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff}, // 0x00000003 --> 0x00000007
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0001016E, 0xffffffff}, // 0x007000DC --> 0x0001016E
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0001016E, 0xffffffff}, // 0x007000DC --> 0x0001016E
    {IO_VIRT + 0x3496c, 0x044A0445, 0xffffffff}, // 0x044B043e --> 0x04480443
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80200080, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD60_42[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002B05, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002B05, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002B05, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002B05, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002B05, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002B05, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002B05, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x0082007E, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x0082007E, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410004, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04460441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00840074, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00840074, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04470447, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04470447, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x007000DC, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04450442, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80200080, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };
// 20111017 by LGE jaewook.chung 
struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD60_47[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002B05, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002B05, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002B05, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002B05, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002B05, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002B05, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002B05, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x01050089, 0xffffffff}, // 0x0082007E --> 0x01050089
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x01050089, 0xffffffff}, // 0x0082007E --> 0x01050089
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x00B20010, 0xffffffff}, // 0x007000DC --> 0x00B20010
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x00B20010, 0xffffffff}, // 0x007000DC --> 0x00B20010
    {IO_VIRT + 0x348Ac, 0x04450005, 0xffffffff}, // 0x043F0002 --> 0x04430003
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x00010001, 0xffffffff}, // 0x007000DC --> 0x00010001
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x00010001, 0xffffffff}, // 0x007000DC --> 0x00010001
    {IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff}, // 0x0444043F --> 0x04480440
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00140004, 0xffffffff}, // 0x00840074 --> 0x00140004
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00140004, 0xffffffff}, // 0x00840074 --> 0x00140004
    {IO_VIRT + 0x3490c, 0x044D044D, 0xffffffff}, // 0x04450445 --> 0x044B044B
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x044D044D, 0xffffffff}, // 0x04450445 --> 0x044B044B
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff}, // 0x00000003 --> 0x00000007
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0001016E, 0xffffffff}, // 0x007000DC --> 0x0001016E
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0001016E, 0xffffffff}, // 0x007000DC --> 0x0001016E
    {IO_VIRT + 0x3496c, 0x044A0445, 0xffffffff}, // 0x04430440 ==> 0x04480443
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80200080, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };

struct sTCONREGTABLE sRegTg_EPI_LGDV12FHD60_55[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00002B05, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00002B05, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00002B05, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00002B05, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00002B05, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00002B05, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00002B05, 0xffffffff}, //channel6 SOE

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x0013000B, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x011E009E, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x011E009E, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x00B20026, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x00B20026, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410005, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x000C0001, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x000C0001, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04470447, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff}, // 0x00000003 --> 0x00000007
    {IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x000B016E, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x000B016E, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x044A0445, 0xffffffff}, // 0x04430440 ==> 0x04480443
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044D044D, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80200080, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
    };
  
  struct sTCONREGTABLE sRegTg_EPI_LGDV13FHD60_42[] = {
    //SOE
    {IO_VIRT + 0x35508, 0x00013409, 0xffffffff}, //setting model by model
//    {IO_VIRT + 0x35550, 0x00002B05, 0xffffffff}, //channel1 SOE
//    {IO_VIRT + 0x35554, 0x00002B05, 0xffffffff}, //channel2 SOE
//    {IO_VIRT + 0x35558, 0x00002B05, 0xffffffff}, //channel3 SOE
//    {IO_VIRT + 0x3555C, 0x00002B05, 0xffffffff}, //channel4 SOE
//    {IO_VIRT + 0x35560, 0x00002B05, 0xffffffff}, //channel5 SOE
//    {IO_VIRT + 0x35564, 0x00002B05, 0xffffffff}, //channel6 SOE
	//CTR2 1st Source IC setting
    {IO_VIRT + 0x35568, 0x00000090, 0xffffffff},
    //CTR2 2nd Source IC setting
    {IO_VIRT + 0x3556c, 0x00000090, 0xffffffff},
    //CTR2 3rd Source IC setting
    {IO_VIRT + 0x35570, 0x00000890, 0xffffffff},
    //CTR2 4th Source IC setting
    {IO_VIRT + 0x35574, 0x00000890, 0xffffffff},
    //CTR2 5th Source IC setting
    {IO_VIRT + 0x35578, 0x00000090, 0xffffffff},
    //CTR2 6th Source IC setting
    {IO_VIRT + 0x3557c, 0x00000090, 0xffffffff},
    //Enable CTR2 separate control
    {IO_VIRT + 0x3541c, 0x00003f00, 0xffffffff},

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00070006, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x04430002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x002B0001, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x002B0001, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x010F010B, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x010F010B, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x0051001F, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x0051001F, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04420005, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x044B044B, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x044B044B, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80200080, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x0001016C, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x0001016C, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x044A0445, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x00140004, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x044B044B, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x044B044B, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80400100, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };

  
  struct sTCONREGTABLE sRegTg_EPI_LGDV13FHD120_42[] = {
	//SOE
	//{IO_VIRT + 0x35508, 0x00012c06, 0xffffffff}, //setting model by model
	{IO_VIRT + 0x35550, 0x0001330A, 0xffffffff}, //channel1 SOE
	{IO_VIRT + 0x35554, 0x00013108, 0xffffffff}, //channel2 SOE
	{IO_VIRT + 0x35558, 0x00012F06, 0xffffffff}, //channel3 SOE
	{IO_VIRT + 0x3555C, 0x00013007, 0xffffffff}, //channel4 SOE
	{IO_VIRT + 0x35560, 0x00013209, 0xffffffff}, //channel5 SOE
	{IO_VIRT + 0x35564, 0x0001340B, 0xffffffff}, //channel6 SOE	 
	//CTR2 1st Source IC setting
	{IO_VIRT + 0x35568, 0x00000090, 0xffffffff},
	//CTR2 2nd Source IC setting
	{IO_VIRT + 0x3556c, 0x00000090, 0xffffffff},
	//CTR2 3rd Source IC setting
	{IO_VIRT + 0x35570, 0x00000890, 0xffffffff},
	//CTR2 4th Source IC setting
	{IO_VIRT + 0x35574, 0x00000090, 0xffffffff},
	//CTR2 5th Source IC setting
	{IO_VIRT + 0x35578, 0x00000090, 0xffffffff},
	//CTR2 6th Source IC setting
	{IO_VIRT + 0x3557c, 0x00000890, 0xffffffff},
	//Enable CTR2 and CTR1 separate control
	{IO_VIRT + 0x3541c, 0x00003f3f, 0xffffffff},
  
	//GSP    TCO[0]  IO_VIRT + 0x34840
	{IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x3484c, 0x00090008, 0xffffffff},
	{IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
										
	//POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
										
	//GST    TCO[5]  IO_VIRT + 0x348e0
	{IO_VIRT + 0x348E0, 0x000B000B, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x000B000B, 0xffffffff},
	{IO_VIRT + 0x348Ec, 0x00040002, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},
  
	//MCLK	TCO[4]	IO_VIRT + 0x348c0
	{IO_VIRT + 0x348C0, 0x00DD00D5, 0xffffffff},
	{IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348C8, 0x00DD00D5, 0xffffffff},
	{IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
	{IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},
  
	//GCLK	TCO[3]	IO_VIRT + 0x348a0
	{IO_VIRT + 0x348A0, 0x00CC0058, 0xffffffff},
	{IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348A8, 0x00CC0058, 0xffffffff},
	{IO_VIRT + 0x348Ac, 0x04420005, 0xffffffff},
	{IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},
  
	//GCLK_END	TCO[8]	IO_VIRT + 0x34940
	{IO_VIRT + 0x34940, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34948, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x3494c, 0x044A0442, 0xffffffff},
	{IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 
  
	//EO	 TCO[6]  IO_VIRT + 0x34900
	{IO_VIRT + 0x34900, 0x00380018, 0xffffffff},
	{IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34908, 0x00380018, 0xffffffff},
	{IO_VIRT + 0x3490c, 0x044B044B, 0xffffffff},
	{IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34914, 0x044B044B, 0xffffffff},
	{IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3491c, 0x80004001, 0xffffffff},
  
	//GST_END    TCO[9]  IO_VIRT + 0x34960
	{IO_VIRT + 0x34960, 0x0001016E, 0xffffffff},
	{IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34968, 0x0001016E, 0xffffffff},
	{IO_VIRT + 0x3496c, 0x044A0445, 0xffffffff},
	{IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 
  
	//VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
	{IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
	{IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},
  
	  
	{IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
										
    //{IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
	{IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
	{IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
	{IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
  
};
  
  struct sTCONREGTABLE sRegTg_EPI_LGDV14FHD60_8LANE[] = {
    //SOE
//    {IO_VIRT + 0x35508, 0x00020306, 0xffffffff}, //setting model by model
    {IO_VIRT + 0x35550, 0x00020306, 0xffffffff}, //channel1 SOE
    {IO_VIRT + 0x35554, 0x00020306, 0xffffffff}, //channel2 SOE
    {IO_VIRT + 0x35558, 0x00020306, 0xffffffff}, //channel3 SOE
    {IO_VIRT + 0x3555C, 0x00020306, 0xffffffff}, //channel4 SOE
    {IO_VIRT + 0x35560, 0x00020306, 0xffffffff}, //channel5 SOE
    {IO_VIRT + 0x35564, 0x00020306, 0xffffffff}, //channel6 SOE
    {IO_VIRT + 0x355BC, 0x00020306, 0xffffffff}, //channel7 SOE
    {IO_VIRT + 0x355C0, 0x00020306, 0xffffffff}, //channel8 SOE

	//CTR2 1st Source IC setting
    {IO_VIRT + 0x35568, 0x00000090, 0xffffffff},
    //CTR2 2nd Source IC setting
    {IO_VIRT + 0x3556c, 0x00000090, 0xffffffff},
    //CTR2 3rd Source IC setting
    {IO_VIRT + 0x35570, 0x00000090, 0xffffffff},
    //CTR2 4th Source IC setting
    {IO_VIRT + 0x35574, 0x00000090, 0xffffffff},
    //CTR2 5th Source IC setting
    {IO_VIRT + 0x35578, 0x00000090, 0xffffffff},
    //CTR2 6th Source IC setting
    {IO_VIRT + 0x3557c, 0x00000090, 0xffffffff},
	//CTR2 7th Source IC setting
	{IO_VIRT + 0x355C4, 0x00000090, 0xffffffff},
	//CTR2 8th Source IC setting
	{IO_VIRT + 0x355C8, 0x00000090, 0xffffffff},
	//Enable CTR2 separate control
	{IO_VIRT + 0x3541c, 0x0000ffff, 0xffffffff},	

    //GSP    TCO[0]  IO_VIRT + 0x34840
    {IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
    {IO_VIRT + 0x3484c, 0x00070006, 0xffffffff},
    {IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
                                      
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00E20112, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x043F0002, 0xffffffff},
    {IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
    {IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
                                      
    //GST    TCO[5]  IO_VIRT + 0x348e0
    {IO_VIRT + 0x348E0, 0x00120014, 0xffffffff},
    {IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348E8, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348Ec, 0x00050004, 0xffffffff},
    {IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},

    //MCLK    TCO[4]  IO_VIRT + 0x348c0
    {IO_VIRT + 0x348C0, 0x00D7009C, 0xffffffff},
    {IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348C8, 0x00D7009C, 0xffffffff},
    {IO_VIRT + 0x348Cc, 0x04430007, 0xffffffff},
    {IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},

    //GCLK    TCO[3]  IO_VIRT + 0x348a0
    {IO_VIRT + 0x348A0, 0x009C0016, 0xffffffff},
    {IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x348A8, 0x009C0016, 0xffffffff},
    {IO_VIRT + 0x348Ac, 0x04410005, 0xffffffff},
    {IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
    {IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
    {IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},

    //GCLK_END    TCO[8]  IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04470441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 

    //EO   TCO[6]  IO_VIRT + 0x34900
    {IO_VIRT + 0x34900, 0x002A0018, 0xffffffff},
    {IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34908, 0x002A0018, 0xffffffff},
    {IO_VIRT + 0x3490c, 0x04480448, 0xffffffff},
    {IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34914, 0x04480448, 0xffffffff},
    {IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3491c, 0x80200080, 0xffffffff},

    //GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04470444, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 

    //VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
    {IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
    {IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},

    //EO2    TCO[10]  IO_VIRT + 0x34980
    {IO_VIRT + 0x34980, 0x002A0018, 0xffffffff},
    {IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34988, 0x002A0018, 0xffffffff},
    {IO_VIRT + 0x3498c, 0x04480448, 0xffffffff},
    {IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34994, 0x04480448, 0xffffffff},
    {IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
    {IO_VIRT + 0x3499c, 0x80400100, 0xffffffff}, 

    {IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
                                      
//    {IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
    {IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
    {IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
    {IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},

    };
  	
  struct sTCONREGTABLE sRegTg_EPI_LGDV14FHD120_8LANE[] = {
	//SOE
//	{IO_VIRT + 0x35508, 0x00020306, 0xffffffff}, //setting model by model
 	{IO_VIRT + 0x35550, 0x0000DB0C, 0xffffffff}, //channel1 SOE
 	{IO_VIRT + 0x35554, 0x0000D90A, 0xffffffff}, //channel2 SOE
 	{IO_VIRT + 0x35558, 0x0000D708, 0xffffffff}, //channel3 SOE
 	{IO_VIRT + 0x3555C, 0x0000D506, 0xffffffff}, //channel4 SOE
 	{IO_VIRT + 0x35560, 0x0000D607, 0xffffffff}, //channel5 SOE
 	{IO_VIRT + 0x35564, 0x0000D809, 0xffffffff}, //channel6 SOE
 	{IO_VIRT + 0x355BC, 0x0000DA0B, 0xffffffff}, //channel7 SOE
 	{IO_VIRT + 0x355C0, 0x0000DC0D, 0xffffffff}, //channel8 SOE
 
	//CTR2 1st Source IC setting
    {IO_VIRT + 0x35568, 0x00000E12, 0xffffffff},
    //CTR2 2nd Source IC setting
    {IO_VIRT + 0x3556c, 0x00000E12, 0xffffffff},
    //CTR2 3rd Source IC setting
    {IO_VIRT + 0x35570, 0x00000E12, 0xffffffff},
    //CTR2 4th Source IC setting
    {IO_VIRT + 0x35574, 0x00000E12, 0xffffffff},
    //CTR2 5th Source IC setting
    {IO_VIRT + 0x35578, 0x00000E12, 0xffffffff},
    //CTR2 6th Source IC setting
    {IO_VIRT + 0x3557c, 0x00000E12, 0xffffffff},
    //CTR2 7th Source IC setting
    {IO_VIRT + 0x355C4, 0x00000E12, 0xffffffff},
    //CTR2 8th Source IC setting
    {IO_VIRT + 0x355C8, 0x00000E12, 0xffffffff},
    //Enable CTR2 separate control
    {IO_VIRT + 0x3541c, 0x0000ffff, 0xffffffff}, 

	//GSP    TCO[0]  IO_VIRT + 0x34840
	{IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x3484c, 0x00070006, 0xffffffff},
	{IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
										
    //POL    TCO[1]  IO_VIRT + 0x34860
    {IO_VIRT + 0x34860, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34868, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3486c, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34870, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34874, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34878, 0x00000080, 0xffffffff},
    {IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
										
	//GST    TCO[5]  IO_VIRT + 0x348e0
	{IO_VIRT + 0x348E0, 0x00120014, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348Ec, 0x00050004, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},
  
	//MCLK	TCO[4]	IO_VIRT + 0x348c0
	{IO_VIRT + 0x348C0, 0x00E600AD, 0xffffffff},
	{IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348C8, 0x00E600AD, 0xffffffff},
	{IO_VIRT + 0x348Cc, 0x04440007, 0xffffffff},
	{IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},
  
	//GCLK	TCO[3]	IO_VIRT + 0x348a0
	{IO_VIRT + 0x348A0, 0x009C0085, 0xffffffff},
	{IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348A8, 0x009C0085, 0xffffffff},
	{IO_VIRT + 0x348Ac, 0x04430005, 0xffffffff},
	{IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},
  
	//GCLK_END	TCO[8]	IO_VIRT + 0x34940
    {IO_VIRT + 0x34940, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3494c, 0x04470441, 0xffffffff},
    {IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 
  
	//EO	 TCO[6]  IO_VIRT + 0x34900
	{IO_VIRT + 0x34900, 0x002A0018, 0xffffffff},
	{IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34908, 0x002A0018, 0xffffffff},
	{IO_VIRT + 0x3490c, 0x04480448, 0xffffffff},
	{IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34914, 0x04480448, 0xffffffff},
	{IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3491c, 0x80200080, 0xffffffff},
  
	//GST_END    TCO[9]  IO_VIRT + 0x34960
    {IO_VIRT + 0x34960, 0x00010001, 0xffffffff},
    {IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34968, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x3496c, 0x04470444, 0xffffffff},
    {IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
    {IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
    {IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 
  
	//VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
	{IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
	{IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},
  
	{IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
										
 //	{IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
	{IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34814, 0x00908000, 0xffffffff}, //Bmux select
	{IO_VIRT + 0x34818, 0x00000440, 0xffffffff}, //combination
	{IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
  
	};
	
  struct sTCONREGTABLE sRegTg_EPI_LGDV14FHD60_4LANE[] = {
	//SOE
//	{IO_VIRT + 0x35508, 0x0000EF29, 0xffffffff}, //setting model by model
	{IO_VIRT + 0x35550, 0x0000EB0C, 0xffffffff}, //channel1 SOE
	{IO_VIRT + 0x35554, 0x0000EE0F, 0xffffffff}, //channel2 SOE
	{IO_VIRT + 0x35558, 0x0000EC0D, 0xffffffff}, //channel3 SOE
  	{IO_VIRT + 0x3555C, 0x0000E90A, 0xffffffff}, //channel4 SOE
  
	//CTR2 1st Source IC setting
	{IO_VIRT + 0x35568, 0x00000070, 0xffffffff},
    //CTR2 2nd Source IC setting
	{IO_VIRT + 0x3556c, 0x00000870, 0xffffffff},
	//CTR2 3rd Source IC setting
  	{IO_VIRT + 0x35570, 0x00000070, 0xffffffff},
	//CTR2 4th Source IC setting
  	{IO_VIRT + 0x35574, 0x00000870, 0xffffffff},
	//Enable CTR2 separate control
    {IO_VIRT + 0x3541c, 0x00000f0f, 0xffffffff}, 
  
	//GSP    TCO[0]  IO_VIRT + 0x34840
	{IO_VIRT + 0x34840, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x34844, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34848, 0x012c0001, 0xffffffff},
	{IO_VIRT + 0x3484c, 0x00070006, 0xffffffff},
	{IO_VIRT + 0x34850, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34854, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34858, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3485c, 0x00000000, 0xffffffff},
										
	//POL    TCO[1]  IO_VIRT + 0x34860
	{IO_VIRT + 0x34860, 0x00100010, 0xffffffff},
	{IO_VIRT + 0x34864, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34868, 0x00100010, 0xffffffff},
	{IO_VIRT + 0x3486c, 0x0885000D, 0xffffffff},
	{IO_VIRT + 0x34870, 0x80000404, 0xffffffff},
	{IO_VIRT + 0x34874, 0x80040002, 0xffffffff},
	{IO_VIRT + 0x34878, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3487c, 0x20000000, 0xffffffff},
										
	//GST    TCO[5]  IO_VIRT + 0x348e0
	{IO_VIRT + 0x348E0, 0x00140003, 0xffffffff},
	{IO_VIRT + 0x348E4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348E8, 0x00140003, 0xffffffff},
	{IO_VIRT + 0x348Ec, 0x000D000B, 0xffffffff},
	{IO_VIRT + 0x348F0, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348F8, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x348Fc, 0x00000000, 0xffffffff},
  
	//MCLK	TCO[4]	IO_VIRT + 0x348c0
	{IO_VIRT + 0x348C0, 0x00C700B2, 0xffffffff},
	{IO_VIRT + 0x348C4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348C8, 0x00C700B2, 0xffffffff},
	{IO_VIRT + 0x348Cc, 0x0884000F, 0xffffffff},
	{IO_VIRT + 0x348D0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348D4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348D8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Dc, 0x00000000, 0xffffffff},
  
	//GCLK	TCO[3]	IO_VIRT + 0x348a0
	{IO_VIRT + 0x348A0, 0x005A0029, 0xffffffff},
	{IO_VIRT + 0x348A4, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x348A8, 0x005A0029, 0xffffffff},
	{IO_VIRT + 0x348Ac, 0x0881000D, 0xffffffff},
	{IO_VIRT + 0x348B0, 0x20000000, 0xffffffff},
	{IO_VIRT + 0x348B4, 0x80000000, 0xffffffff},
	{IO_VIRT + 0x348B8, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x348Bc, 0x00000000, 0xffffffff},
  
	//GCLK_END	TCO[8]	IO_VIRT + 0x34940
	{IO_VIRT + 0x34940, 0x005D003F, 0xffffffff},
	{IO_VIRT + 0x34944, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34948, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x3494c, 0x08870882, 0xffffffff},
	{IO_VIRT + 0x34950, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34954, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34958, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3495c, 0x00000000, 0xffffffff}, 
  
	//EO	 TCO[6]  IO_VIRT + 0x34900
	{IO_VIRT + 0x34900, 0x00700059, 0xffffffff},
	{IO_VIRT + 0x34904, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34908, 0x00700059, 0xffffffff},
	{IO_VIRT + 0x3490c, 0x08880888, 0xffffffff},
	{IO_VIRT + 0x34910, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34914, 0x08880888, 0xffffffff},
	{IO_VIRT + 0x34918, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3491c, 0x80200080, 0xffffffff},
  
	//GST_END    TCO[9]  IO_VIRT + 0x34960
	{IO_VIRT + 0x34960, 0x00D50072, 0xffffffff},
	{IO_VIRT + 0x34964, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34968, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x3496c, 0x08870885, 0xffffffff},
	{IO_VIRT + 0x34970, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34974, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34978, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3497c, 0x00000000, 0xffffffff}, 
  
	//VCOM_DYN  TCO[7]  IO_VIRT + 0x34920
	{IO_VIRT + 0x34920, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x34924, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34928, 0x00010001, 0xffffffff},
	{IO_VIRT + 0x3492c, 0x02320001, 0xffffffff},
	{IO_VIRT + 0x34930, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34934, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34938, 0x00000004, 0xffffffff},
	{IO_VIRT + 0x3493c, 0x00000000, 0xffffffff},
  
	//EO2    TCO[10]	IO_VIRT + 0x34980
	{IO_VIRT + 0x34980, 0x00700059, 0xffffffff},
	{IO_VIRT + 0x34984, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34988, 0x00700059, 0xffffffff},
	{IO_VIRT + 0x3498c, 0x08880888, 0xffffffff},
	{IO_VIRT + 0x34990, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34994, 0x08880888, 0xffffffff},
	{IO_VIRT + 0x34998, 0x00000007, 0xffffffff},
	{IO_VIRT + 0x3499c, 0x80400100, 0xffffffff}, 
  
	{IO_VIRT + 0x34820, 0x00000000, 0xffffffff}, //GPO :1
										
 //	{IO_VIRT + 0x34800, 0x8100ffff, 0xffffffff},
	{IO_VIRT + 0x34804, 0x00000000, 0xffffffff},
	{IO_VIRT + 0x34814, 0x0A908000, 0xffffffff}, //Bmux select
	{IO_VIRT + 0x34818, 0x00001440, 0xffffffff}, //combination
	{IO_VIRT + 0x3481c, 0x00000000, 0xffffffff},
  
	};

//-----------------------------------------------------------------------------
// Static functions
//-----------------------------------------------------------------------------
static UINT32 _u4TconCustGammaWrite(UINT8 u1offset, UINT8 *pu1Buf, UINT8 u1Count)
{
	LOG(0,"0x%8x, 0x%8x, 0xffffffff,\n", u1offset, (pu1Buf[0]<<8) | pu1Buf[1]);
    if(PANEL_GetDisplayType()== PANEL_DISPLAY_TYPE_EPI)
    	return (SIF_Write(GAMMAIC_CLK_RATE, GAMMAIC_ID_V13, u1offset, (UINT8 *)pu1Buf, GAMMAIC_DATA_CNT_V13));
    else
    	return (SIF_Write(GAMMAIC_CLK_RATE, GAMMAIC_ID, u1offset, (UINT8 *)pu1Buf, GAMMAIC_DATA_CNT));
}

#if 0 // for warning free
static UINT32 _u4TconCustGammaRead(UINT8 u1offset, UINT8 *pu1Buf, UINT8 u1Count)
{
	return (SIF_Read(GAMMAIC_CLK_RATE, GAMMAIC_ID, u1offset, (UINT8 *)pu1Buf, GAMMAIC_DATA_CNT));
}
#endif

//-----------------------------------------------------------------------------
// Public functions
//-----------------------------------------------------------------------------

UINT8 u1TconCustDPMCtrlEN1(UINT8 val)
{
  vIO32WriteFldAlign(GR8, 1, GR8_TCGPOEN_A);
	
  if (val)
  {
      vIO32WriteFldAlign(GR8, 1, GR8_TCGPO_A);
  }
  else
  {
      vIO32WriteFldAlign(GR8, 0, GR8_TCGPO_A);
  }
  
  return(1);
}

UINT8 u1TconCustDPMCtrlEN2(UINT8 val)
{
    if (val)
    {
        // TODO
    }
    else
    {
        // TODO
    }

    return(1);
}

// Gamma
UINT8 u1TconCustGammaInit(void)
{
	UINT8 u1Err = 0;
	UINT8 i = 0;
	UINT8 u1count;
	UINT8 u1tmp[2];
	UINT32 u2GammaVoltage = 0;
    struct sTCONREGTABLE *pTgTable = NULL;    

	LOG(0,"Tcon Gamma init\n");

    if(PANEL_GetDisplayType()== PANEL_DISPLAY_TYPE_EPI)
    {
        pTgTable = sRegCustGamma_V13;
	    u1count = (sizeof(sRegCustGamma_V13) / sizeof(struct sTCONREGTABLE));
    }
    else
    {
        pTgTable = sRegCustGamma;
	    u1count = (sizeof(sRegCustGamma) / sizeof(struct sTCONREGTABLE));
    }

	for ( i = 0 ; i < u1count ; i++)
    {
                //#if  HISENSE_IPS_2010_PANEL 
      		//u1tmp[1] = (UINT8)((sRegCustGamma[i].u4Val) >> 8);
		//u1tmp[0] = (UINT8)((sRegCustGamma[i].u4Val) & 0xff);
		u2GammaVoltage = pTgTable[i].u4Val;
		u1tmp[1] = (UINT8)(u2GammaVoltage >> 8);
		u1tmp[0] = (UINT8)(u2GammaVoltage & 0xff);
		if (_u4TconCustGammaWrite((UINT8)(pTgTable[i].u4Addr), u1tmp, 2) == 0)
		{
		
			u1Err = 1;
			LOG(0,"!!!Tcon Gamma write fail %d, %d, %d \n", \
				              (int)pTgTable[i].u4Addr, \
				              u1tmp[0], \
				              u1tmp[1]);			
		}
		else
		{
    		LOG(0,"Tcon Gamma write ok %d , %d , %d \n", \
                              (int)pTgTable[i].u4Addr, \
                              u1tmp[0], \
                              u1tmp[1]);
		}
    }

    // check if init ok
	if (u1Err == 0)
	{
		LOG(0,"Program Tcon Gamma dacs %d counts, %d / %d\n", i, sizeof(pTgTable), sizeof(struct sTCONREGTABLE));
	}
	else
	{
	     LOG(0,"Program Tcon Gamma dacs fail !!!\n");
	}
	
	return(1);
}

// program gamma value into non-volatile memory
UINT8  u1TconCustGammaFlashing(void)
{
	UINT8 u1Err = 0;
	UINT8 i = 0;
	UINT8 u1count;
	UINT8 u1tmp[2];
	UINT8 u1Ret = 0;
	UINT16 u2GammaVoltage = 0;
    struct sTCONREGTABLE *pTgTable = NULL;    

#ifdef TCON_DBG_MSG_EN
   	LOG(0,"Tcon Gamma init\n");
#endif
    if(PANEL_GetDisplayType()== PANEL_DISPLAY_TYPE_EPI)
    {
        pTgTable = sRegCustGamma_V13;
	    u1count = (sizeof(sRegCustGamma_V13) / sizeof(struct sTCONREGTABLE));
    }
    else
    {
        pTgTable = sRegCustGamma;
	    u1count = (sizeof(sRegCustGamma) / sizeof(struct sTCONREGTABLE));
    }

	for ( i = 0 ; i < u1count ; i++)
    {
    
		//when write non-voltial memory  D15 D14 must be 0 1
		u2GammaVoltage = (((pTgTable[i].u4Val)&0x03FF)|0x4000);
		u1tmp[0] = (UINT8)((u2GammaVoltage >> 8) );  // program gamma into non-voilate
		u1tmp[1] = (UINT8)(u2GammaVoltage & 0xff);

		if (_u4TconCustGammaWrite((UINT8)(pTgTable[i].u4Addr), u1tmp, 2) == 0)
		{
		
			u1Err = 1;
#ifdef TCON_DBG_MSG_EN
 		  	LOG(0,"!!!Tcon Gamma write fail 0x%x, 0x%x, 0x%x\n",\
				              pTgTable[i].u4Addr,\
				              u1tmp[0],\
				              u1tmp[1]);
#endif
		}
		else
		{
#ifdef TCON_DBG_MSG_EN
		   	LOG(0,"Tcon Gamma write ok 0x%x, 0x%x, 0x%x\n", \
                              pTgTable[i].u4Addr, \
                              u1tmp[0], \
                              u1tmp[1]);
#endif
		}
    }

    // check if init ok
	if (u1Err == 0)
	{
#ifdef TCON_DBG_MSG_EN
    	LOG(0,"Program Tcon Gamma dacs %d counts, %d / %d\n", i, sizeof(pTgTable), sizeof(struct sTCONREGTABLE));
#endif
	}
	else
	{
#ifdef TCON_DBG_MSG_EN
   	   LOG(0,"Program Tcon Gamma dacs fail !!!\n");
#endif
	}

    return (u1Ret);
}

UINT8 u1TconCustTgInit(void)
{
	UINT8 i = 0;
	UINT8 u1count = 0;
	UINT32 u4tmp, u4tmp1;
    struct sTCONREGTABLE *pTgTable = NULL;    

	switch (GetCurrentPanelIndex())
    {                 
#ifdef PANEL_MLVDS_LGDV4FHD60
        case PANEL_MLVDS_LGDV4FHD60:             
            u1count = (sizeof(sRegTg_MLVDS_LGDV4FHD60) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_LGDV4FHD60;            
        break;
#endif

#ifdef PANEL_MLVDS_LGDV4FHD120
		case PANEL_MLVDS_LGDV4FHD120:             
            u1count = (sizeof(sRegTg_MLVDS_LGDV4FHD60) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_LGDV4FHD60;            
        break;
#endif

#ifdef PANEL_MLVDS_PANASONIC32WXGA1P6P
        case PANEL_MLVDS_PANASONIC32WXGA1P6P:             
            u1count = (sizeof(sRegTg_MLVDS_PANASONIC32WXGA1P6P) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_PANASONIC32WXGA1P6P;            
        break;
#endif

#ifdef PANEL_MLVDS_PANASONIC32WXGA2P3P
		case PANEL_MLVDS_PANASONIC32WXGA2P3P:             
            u1count = (sizeof(sRegTg_MLVDS_PANASONIC32WXGA2P3P) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_PANASONIC32WXGA2P3P;            
        break;
#endif
		
#ifdef PANEL_MLVDS_PANASONIC42FHD120
        case PANEL_MLVDS_PANASONIC42FHD120:             
            u1count = (sizeof(sRegTg_MLVDS_PANASONIC42FHD120) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_PANASONIC42FHD120;            
        break;
#endif
		
#ifdef PANEL_MLVDS_CMO2011FHD60
        case PANEL_MLVDS_CMO2011FHD60:             
            u1count = (sizeof(sRegTg_MLVDS_CMO2011FHD60) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_CMO2011FHD60;            
        break;
#endif
		
#ifdef PANEL_MLVDS_CMO2011FHD120
		case PANEL_MLVDS_CMO2011FHD120:             
            u1count = (sizeof(sRegTg_MLVDS_CMO2011FHD60) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_CMO2011FHD60;            
        break;
#endif
		
#ifdef PANEL_MLVDS_SS_MB7
        case PANEL_MLVDS_SS_MB7:             
            u1count = (sizeof(sRegTg_MLVDS_SAMSUNGMB7) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_MLVDS_SAMSUNGMB7;            
        break;
#endif

#ifdef PANEL_EPI_LGDV12GIPFHD60
        case PANEL_EPI_LGDV12GIPFHD60:             
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD60) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_EPI_LGDV12FHD60;            
        break;
#endif

#ifdef PANEL_EPI_LGDV12GIPFHD120
        case PANEL_EPI_LGDV12GIPFHD120:             
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD120) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_EPI_LGDV12FHD120;            
        break;
#endif

#ifdef PANEL_EPI_LGDV13GIPFHD60
        case PANEL_EPI_LGDV13GIPFHD60:             
            u1count = (sizeof(sRegTg_EPI_LGDV13FHD60_42) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_EPI_LGDV13FHD60_42;            
        break;
#endif

#ifdef PANEL_EPI_LGDV13GIPFHD120
        case PANEL_EPI_LGDV13GIPFHD120:             
            u1count = (sizeof(sRegTg_EPI_LGDV13FHD120_42) / sizeof(struct sTCONREGTABLE));            
            pTgTable = sRegTg_EPI_LGDV13FHD120_42;            
        break;
#endif

#ifdef PANEL_MLVDS_LGD32WXGAV9
		case PANEL_MLVDS_LGD32WXGAV9:
			u1count = (sizeof(sRegTg_MLVDS_LGD32WXGAV9) / sizeof(struct sTCONREGTABLE)); 		   
			pTgTable = sRegTg_MLVDS_LGD32WXGAV9;    

		break;	 
#endif

#ifdef PANEL_MLVDS_CHINASTARWXGA8BIT
		case PANEL_MLVDS_CHINASTARWXGA8BIT:
			u1count = (sizeof(sRegTg_MLVDS_CHINASTARWXGA8BIT) / sizeof(struct sTCONREGTABLE));		   
			pTgTable = sRegTg_MLVDS_CHINASTARWXGA8BIT;	
		break;	
#endif
#ifdef PANEL_MLVDS_HISENSE_HE420FF
		case PANEL_MLVDS_HISENSE_HE420FF:
			u1count = (sizeof(sRegTg_MLVDS_HISENSE_HE420FF) / sizeof(struct sTCONREGTABLE));		   
			pTgTable = sRegTg_MLVDS_HISENSE_HE420FF;	
		break;	
#endif

#ifdef PANEL_MLVDS_HISENSE32WXGA2P3P
		case PANEL_MLVDS_HISENSE32WXGA2P3P:			  
			u1count = (sizeof(sRegTg_MLVDS_HISENSE32WXGA2P3P) / sizeof(struct sTCONREGTABLE));			
			pTgTable = sRegTg_MLVDS_HISENSE32WXGA2P3P;			
		break;
#endif

#ifdef PANEL_MLVDS_HISENSE32WXGA1P6P
		case PANEL_MLVDS_HISENSE32WXGA1P6P:			  
			u1count = (sizeof(sRegTg_MLVDS_HISENSE32WXGA1P6P) / sizeof(struct sTCONREGTABLE));			
			pTgTable = sRegTg_MLVDS_HISENSE32WXGA1P6P;			
		break;
#endif

#ifdef PANEL_MLVDS_LGD32WXGAV12
		case PANEL_MLVDS_LGD32WXGAV12:
			u1count = (sizeof(sRegTg_MLVDS_LGD32WXGAV12) / sizeof(struct sTCONREGTABLE));		   
			pTgTable = sRegTg_MLVDS_LGD32WXGAV12;	
		break;	
#endif

#ifdef PANEL_MLVDS_LGD32WXGAV13
		case PANEL_MLVDS_LGD32WXGAV13:
			u1count = (sizeof(sRegTg_MLVDS_LGD32WXGAV13) / sizeof(struct sTCONREGTABLE));		   
			pTgTable = sRegTg_MLVDS_LGD32WXGAV13;	
		break;	
#endif

#ifdef PANEL_EPI_LGDV14GIPFHD60_8LANE
		case PANEL_EPI_LGDV14GIPFHD60_8LANE:			   
			u1count = (sizeof(sRegTg_EPI_LGDV14FHD60_8LANE) / sizeof(struct sTCONREGTABLE));			 
			pTgTable = sRegTg_EPI_LGDV14FHD60_8LANE;			 
		break;
#endif

#ifdef PANEL_EPI_LGDV14GIPFHD120_8LANE
		case PANEL_EPI_LGDV14GIPFHD120_8LANE:			   
			u1count = (sizeof(sRegTg_EPI_LGDV14FHD120_8LANE) / sizeof(struct sTCONREGTABLE));			 
			pTgTable = sRegTg_EPI_LGDV14FHD120_8LANE;			 
		break;
#endif
#ifdef PANEL_EPI_LGDV14GIPFHD60_4LANE
		case PANEL_EPI_LGDV14GIPFHD60_4LANE:			   
			u1count = (sizeof(sRegTg_EPI_LGDV14FHD60_4LANE) / sizeof(struct sTCONREGTABLE));			 
			pTgTable = sRegTg_EPI_LGDV14FHD60_4LANE;			 
		break;
#endif

        default:
          break;
    }               

    LOG(1,"cust tg init\n");
    if ((pTgTable != NULL) && u1count)    
        {        
        for ( i = 0 ; i < u1count ; i++)        
            {            
            u4tmp = pTgTable[i].u4Addr;            
            u4tmp1 = (u4IO32Read4B(u4tmp) & ~(pTgTable[i].u4Msk)) | ((pTgTable[i].u4Val) & (pTgTable[i].u4Msk));                 
            vIO32Write4B(u4tmp,u4tmp1);
            }    
        }      
    LOG(1,"Tcon reg flashing %d counts, %d / %d\n", i, sizeof(pTgTable), sizeof(struct sTCONREGTABLE));

  return(1);
}

void vDrvTCONCustSel(VIDEO_TCON_TABLE_T eTconTableSel)
{
    UINT8 i = 0;
    UINT8 u1count=0;
    UINT32 u4tmp, u4tmp1;
	struct sTCONREGTABLE *pTgTable = NULL;

    switch (eTconTableSel)
    {
         
        #ifdef EPIV12FHD120_42_TABLE
        case EPIV12FHD120_42: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD120_42) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD120_42;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD120_42\n\n\n\n");
        break;
        #endif

        #ifdef EPIV12FHD120_47_TABLE
        case EPIV12FHD120_47: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD120_47) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD120_47;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD120_47\n\n\n\n");
        break;
        #endif

        #ifdef EPIV12FHD120_55_TABLE
        case EPIV12FHD120_55: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD120_55) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD120_55;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD120_55\n\n\n\n");
        break;
        #endif
		
        #ifdef EPIV12FHD60_32_TABLE
        case EPIV12FHD60_32: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD60_32) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD60_32;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD60_32\n\n\n\n");
        break;
        #endif

        #ifdef EPIV12FHD60_42_TABLE
        case EPIV12FHD60_42: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD60_42) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD60_42;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD60_42\n\n\n\n");
        break;
        #endif

        #ifdef EPIV12FHD60_47_TABLE
        case EPIV12FHD60_47: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD60_47) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD60_47;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD60_47\n\n\n\n");
        break;
        #endif
        
        #ifdef EPIV12FHD60_55_TABLE
        case EPIV12FHD60_55: 
            u1count = (sizeof(sRegTg_EPI_LGDV12FHD60_55) / sizeof(struct sTCONREGTABLE));
            pTgTable = sRegTg_EPI_LGDV12FHD60_55;
			//printk("\n\n\nTconRegTable = sRegTg_EPI_LGDV12FHD60_55\n\n\n\n");
        break;
        #endif

        default:
        
		u1count = (sizeof(sRegTg_EPI_LGDV12FHD60_55) / sizeof(struct sTCONREGTABLE));
		pTgTable = sRegTg_EPI_LGDV12FHD60_55;
        break;
    }   

    LOG(1,"cust tg init\n");
    if ( u1count)
    {
        for ( i = 0 ; i < u1count ; i++)
        {
            u4tmp = pTgTable[i].u4Addr;
            u4tmp1 = (u4IO32Read4B(u4tmp) & ~(pTgTable[i].u4Msk)) | ((pTgTable[i].u4Val) & (pTgTable[i].u4Msk));
     
            vIO32Write4B(u4tmp,u4tmp1);
        }
    }  
    LOG(1,"Tcon reg flashing %d counts, %d / %d\n", i, sizeof(pTgTable), sizeof(struct sTCONREGTABLE));
    
}

void _vDrvTconSetPinMux(void)
{
	if(PANEL_GetDisplayType()!= PANEL_DISPLAY_TYPE_EPI)
	{
		#if defined(CC_MT5891) || defined(CC_MT5893) || defined(CC_MT5895)
		#else
    	BSP_PinSet( PIN_TCON0, PINMUX_FUNCTION1);   //SOE
    	BSP_PinSet( PIN_TCON1, PINMUX_FUNCTION1);   //POL
    	BSP_PinSet( PIN_TCON2, PINMUX_FUNCTION1);   //GOE 
    	BSP_PinSet( PIN_TCON3, PINMUX_FUNCTION1);   //Reserved 
    	BSP_PinSet( PIN_TCON4, PINMUX_FUNCTION1);   //Reserved 
    	BSP_PinSet( PIN_TCON5, PINMUX_FUNCTION1);   //H_CONV 
    	BSP_PinSet( PIN_TCON6, PINMUX_FUNCTION1);   //OPT_N
    	BSP_PinSet( PIN_TCON7, PINMUX_FUNCTION1);   //OPT_P 
    	BSP_PinSet( PIN_TCON8, PINMUX_FUNCTION1);   //GSP_R 
    	BSP_PinSet( PIN_TCON9, PINMUX_FUNCTION1);   //FLK 
    	BSP_PinSet( PIN_TCON10, PINMUX_FUNCTION1);  //DPM
    	BSP_PinSet( PIN_TCON11, PINMUX_FUNCTION1);  //GSP
    	BSP_PinSet( PIN_TCON12, PINMUX_FUNCTION1);  //GSC
    	#endif
	}
	else
	{
		#if defined(CC_MT5891) || defined(CC_MT5893) || defined(CC_MT5895)
		#else
 		BSP_PinSet( PIN_TCON3, PINMUX_FUNCTION1);   //GCLK
    	BSP_PinSet( PIN_TCON4, PINMUX_FUNCTION1);   //MCLK
    	BSP_PinSet( PIN_TCON5, PINMUX_FUNCTION1);   //GST
    	BSP_PinSet( PIN_TCON6, PINMUX_FUNCTION1);   //EO
    	BSP_PinSet( PIN_TCON7, PINMUX_FUNCTION1);   //VCOM_DYN 
    	BSP_PinSet( PIN_TCON12, PINMUX_FUNCTION3);  // LOCKN
    	#endif
	}
}


