/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
  * $RCSfile: panel_table_cust.c,v $
  * $Revision: #4 $
  * $Date: 2015/10/09 $
  * $Author: jiewen.zhou $
  * Description:
  *
  *---------------------------------------------------------------------------*/
#include "sv_const.h"
#include "feature.h"
#include "drv_tcon.h"
#include "drv_ldmspi.h"
#include "u_drv_cust.h"
#ifdef CC_MSDC_ENABLE
#include "msdc_if.h"
#endif
#include "drvcust_if.h"
// command word
#define LDM_SCANNING_ON	(0x02)
#define LDM_SCANNING_OFF (0)
#define LDM_CURRENT_BOOST_ON (0x04)
#define LDM_CURRENT_BOOST_OFF (0)
#define LDM_H1B_PANEL (0x40)
#define LDM_V2B_PANEL (0)
#define LDM_REVERSE_ON (0x80)
#define LDM_REVERSE_OFF (0)



typedef struct
{
    //--- Index Info -----------------------
    UINT8 u1PanelId;
    UINT8 au1PanelName[16];
    UINT8 u1InchSize;
}__attribute__ ((packed)) PANEL_HEADER_CUSTOM_ATTRIBUTE_T;

typedef struct
{
    UINT8 u1LUTSel;
    UINT32 u4Dither;
    UINT8 u1NVMType;

    UINT8 au1PQIndex[4];
    UINT8 au1AQIndex[4];

    UINT8 au1BLExtData[8];
    UINT8 au1DimExtData[3];
    UINT8  u1HdmiEdidTblIdx;
    UINT8  u1VgaEdidTblIdx;
    
	//new add 40bytes
	UINT32 u4ControlWord3;
	UINT32 u4ControlWord4;
	UINT32 u4ControlWord5;
	UINT8  u1VcomType;
	
	UINT16 u2BL_PWM_FREQ;
	UINT16 u2BL_CUR_PWM_FREQ;
	UINT16 u2BL_PWM_NUM_NORM;
	UINT16 u2BL_VLED_PWM_FREQ;
	UINT16 u2BL_LOGO_GAIN;
	UINT16 u2CURRENT_START;
	UINT16 u2VLED_NORMAL;
	UINT16 u2BL_WAIT_LOGO;
	UINT16 u2TM2_VLED_TGT_SLP_START;
	UINT16 u2TM2_VLED_TGT_OFT_START;
	UINT16 u2Panel_low_limit;
	UINT16 u2Panel_high_limit;
	UINT8  u1BL_ERR_NUM;
	UINT8  u1LD_ERR_NUM;
	UINT8  u1Pre_emphasis;
}__attribute__ ((packed)) PANEL_Ext2_CUSTOM_ATTRIBUTE_T;

typedef struct
{
    UINT32 u4Error_Limit;
    UINT8 u1Gam_Fol_Limit;
    UINT8 u1BL_CON_ON;
}__attribute__ ((packed)) PANEL_EXT_GAMMA_ATTRIBUTE_T;


typedef struct
{   //Total 612 bytes
    //--- Index Info -----------------------
    PANEL_HEADER_CUSTOM_ATTRIBUTE_T rHeader; //18bytes
    //--- Basic Info -----------------------
    PANEL_ATTRIBUTE_T rBasicInfo; //66bytes
    //--- Extra Info -----------------------//total 32bytes
    UINT32 u4PixelClk48Hz;
    UINT16 u2HTotal48Hz;
    UINT16 u2VTotal48Hz;

    UINT8 au1HSyncWidth[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT8 au1VSyncWidth[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT16 au2HFrontPorch[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT16 au2VbackPorch[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ

    UINT8 u1HSyncPolarity;
    UINT8 u1VSyncPolarity;

    UINT8 u1HVSyncAlignment;
    UINT16 u2DimmingFrequency48Hz;

    UINT8 u1DrivingCurrent;

    //--- Gamma Info -----------------------
    PANEL_EXT_GAMMA_ATTRIBUTE_T rGammaInfo; // 6bytes

    //--- Other info 1 --------------------// 6bytes
	UINT16 u2PanelSizeCM_Width;
	UINT16 u2PanelSizeCM_Height;
	
	UINT8 u14kbeRstPanelOnWait;
	UINT8 u1TconOnCellWait;

	//--- Reserved data -----------------------// 11bytes
	UINT8 au1Reserved[11]; 

    //--- Other info 2 --------------------// 3bytes
    UINT8 u1ShownOnMenu;
	UINT8 u1PanelType;
    UINT8 u1PanelAttr ;
    //-----mini lvds infor  --------------------// 7bypes
    UINT8  u1Port;
    UINT8  u1Ratio;
    UINT8 u1SSPermillage;
    UINT32 u4SSFreq;

    PANEL_Ext2_CUSTOM_ATTRIBUTE_T   rExt2Info;// 27+40(new add)=67bytes

	//70bytes x (8+4) = 840 bytes new add
	GRAPHMODE_ATTRIBUTE_T rVideoAttr[PNL_VDO_ATTR];
	GRAPHMODE_ATTRIBUTE_T rGraphicAttr[PNL_GFX_ATTR];

    //remove
    //UINT8 au1Pedding[4];

}__attribute__ ((packed)) PANEL_CUSTOM_FLASH_ATTRIBUTE_T;

 //#ifdef SUPPORT_PANEL_48HZ

//#define EXT_PANEL_BEGIN_INDEX PANEL_SONY_X_GY_1_WXGA

typedef struct
{
    UINT32 u4PanelIndex ;
    UINT32 u4PixelClk48Hz;
    UINT16 u2HTotal48Hz;
    UINT16 u2VTotal48Hz;
    UINT16 u2DimmingFrequency48Hz;

    #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
    UINT8 u1HSyncWidth[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT8 u1VSyncWidth[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT16 u2HFrontPorch[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT16 u2VbackPorch[3];//0:    SV_DCLK_48HZ, 1: SV_DCLK_50HZ, 2:SV_DCLK_60HZ
    UINT8 u1HSyncPolarity;
    UINT8 u1VSyncPolarity;
    #endif

    #ifdef SUPPORT_PANEL_DITHER
    UINT32 u4Dither;
    #endif
    #ifdef SUPPORT_PANEL_DRIVING
    UINT8 u1DrivingCurrent;
    #endif

    #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
    UINT32 u4ErrorLimit;  // ERR_LIM:[7:0]; ERR_LIM_LOW:[31:16]
    #endif

    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
    UINT16 u2PixelSize;
    #endif

    #ifdef SUPPORT_PANEL_SS
    UINT8 u1SSPermillage;
    UINT32 u4SSFreq;
    #endif

    #ifdef SUPPORT_PANEL_SCAN_PWM
	UINT32 u4ScanPWMStart;  // permillage, for example, 1 means 0.001
	UINT32 u4ScanPWMDuty;   // permillage, for example, 500 means 0.5
    #endif
	
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
	UINT8 u1PreEmpEn;
	UINT8 u1PreEmpVal;
	#endif
	
	#ifdef SUPPORT_PANEL_SIZE_CM
	UINT16 u2PanleWidth;
	UINT16 u2PanleHeight;
	#endif
    #ifdef SUPPORT_PANEL_4KBERST_PANELON_WAIT
    UINT8 u14kbeRstPanelOnWait;
    #endif

    #ifdef SUPPORT_PANEL_TCONON_CELL_WAIT
    UINT8 u1TconOnCellWait;
    #endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
	UINT8  u1StepEn;
	UINT8  u1StepLevel;
	#endif
} PANEL_ATTRIBUTE_EXT_T;


#if defined(SUPPORT_PANEL_CUSTOMER_SPEC) || defined(SUPPORT_PANEL_SS) || defined(SUPPORT_PANEL_DITHER) || defined(SUPPORT_PANEL_DRIVING)|| defined(SUPPORT_PANEL_ERRORLIMIT_TABLE)|| defined(SUPPORT_PANEL_3D_PIXEL_SIZE) || defined(SUPPORT_PANEL_SCAN_PWM)|| defined(SUPPORT_PANEL_PRE_EMPHASIS) || defined(SUPPORT_PANEL_SIZE_CM)
static PANEL_ATTRIBUTE_EXT_T _arPanelAttributeExt[]=
{
	#if ((PANEL_SELECT == PANEL_AUO_37_HW1) || CC_MULTI_PANEL)
		{
			PANEL_AUO_37_HW1,

			148500000,				  //u4PixelClk48Hz
			2184,					   // u2HTotal48Hz
			1416,					 // u2VTotal48Hz
			150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
			{32, 32, 32},			   //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
			{4, 4, 4},				  // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
			{136, 136, 136},			// H front porch, 1 represents 1 pixel
			{16, 16, 16},//{316, 260, 34}			 // V back porch, 1represents 1 line
			HSYNC_LOW,
			VSYNC_LOW,
        #endif
        #ifdef SUPPORT_PANEL_DITHER
			0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
			0x3,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
        0x10,
        #endif
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
			320, // Pixel Size = 0.320 mm = 320 um
	    #endif
        #ifdef SUPPORT_PANEL_SS
			20,
			0,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
			0,	// u4ScanPWMStart, permillage, 1 means 0.001
			0,	// u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
			0,  // u1PreEmpEn
			0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
			0,  // u2PanleWidth
			0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
		},
    #endif

    #if ((PANEL_SELECT == PANEL_SS_40_FHD) || CC_MULTI_PANEL)
    {
        PANEL_SS_40_FHD,

        79700000,                  //u4PixelClk48Hz
        1723,                      // u2HTotal48Hz
        962,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {32, 32, 32},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {4, 4, 4},                 // u1VSyncWidth[3]; 4bits, 1 represents 1 line
        {276, 276, 92},         // H front porch, 1 represents 1 pixel
        {19, 19, 19},//{172, 134, 72}          // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x3,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
        0x10,
        #endif
        #ifdef SUPPORT_PANEL_SS
        20,
        0,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
    },
    #endif
    #if ((PANEL_SELECT == PANEL_SS_40_FHD_10BIT) || CC_MULTI_PANEL)
    {
		PANEL_SS_40_FHD_10BIT,

        148500000,                //u4PixelClk48Hz
        2184,                      // u2HTotal48Hz
        1416,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {32, 32, 32},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {4, 4, 4},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {136, 136, 136},            // H front porch, 1 represents 1 pixel
        {16, 16, 16},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x3,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
        0x10,
        #endif
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif
        #ifdef SUPPORT_PANEL_SS
        20,
        0,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
    },
    #endif
	
    #if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_VB1) || CC_MULTI_PANEL)
    {
		PANEL_LG_42_WU5_SLB1_VB1,

        297000000,                //u4PixelClk48Hz
        2748,                      // u2HTotal48Hz
        1126,                    // u2VTotal48Hz
        150,

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {4, 4, 4},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {88, 528, 636},            // H front porch, 1 represents 1 pixel
        {32, 32, 32},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
	#endif
	#ifdef SUPPORT_PANEL_DITHER
        0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
        0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
        0xb7ff0003,  // +/-0.33%
	#endif
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	#endif
	#ifdef SUPPORT_PANEL_SS
        20,
        0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
	#endif
    },
    #endif
	
    #if ((PANEL_SELECT == PANEL_CMI_V420H2_LS1) || CC_MULTI_PANEL)
    {
		PANEL_CMI_V420H2_LS1,

        296703297,                //u4PixelClk48Hz
        2292,                      // u2HTotal48Hz
        1350,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {32, 32, 32},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {4, 4, 4},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {136, 136, 136},            // H front porch, 1 represents 1 pixel
        {16, 16, 16},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x3,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
        0x10,
        #endif
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif
        #ifdef SUPPORT_PANEL_SS
        20,
        0,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif	
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
    },
    #endif
	
	#if ((PANEL_SELECT == PANEL_VIDEO_1080P) || CC_MULTI_PANEL)
    {
		PANEL_VIDEO_1080P,

        148500000,                //u4PixelClk48Hz
        2750,                      // u2HTotal48Hz
        1125,                    // u2VTotal48Hz
        160,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {5, 5, 5},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {592, 483, 43},            // H front porch, 1 represents 1 pixel
        {35, 35, 35},              // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x4,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif	
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
    },
    #endif
	
    #if ((PANEL_SELECT == PANEL_SLT_FHD_60Hz_10BIT_AUTO) || CC_MULTI_PANEL)
    {
		PANEL_SLT_FHD_60Hz_10BIT_AUTO,

        148500000,                //u4PixelClk48Hz
        2200,                      // u2HTotal48Hz
        1406,                    // u2VTotal48Hz
        160,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x4,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL2,
		#endif
    },
    #endif
	
    #if ((PANEL_SELECT == PANEL_VB1_V420DK1_KS1) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_V420DK1_KS1,

        297047808,              // u4PixelClk48Hz
        2748,                   // u2HTotal48Hz
        2252,                   // u2VTotal48Hz
        150,                    // u2DimmingFrequency48Hz

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},           // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {10, 10, 10},           // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {636, 528, 88},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {72, 72, 72},           // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    

        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0xb7ff0003,  // +/-0.33%
        #endif          
        #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
        #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif  
	#if ((PANEL_SELECT == PANEL_4K1K_VB1) || CC_MULTI_PANEL) 
	{
		PANEL_4K1K_VB1,

		297047808,				  //u4PixelClk48Hz
		5496,					   // u2HTotal48Hz
		1126,					 // u2VTotal48Hz
		150,

		#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
		{88, 88, 88},			   //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
		{4, 4, 4},				  // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
		{1272, 1056, 176},			 // H front porch, 1 represents 1 pixel
		{32, 32, 32},//{316, 260, 34}			  // V back porch, 1represents 1 line
		HSYNC_LOW,
		VSYNC_LOW,
		#endif	  
 		#ifdef SUPPORT_PANEL_DITHER
		0xa000,
		#endif
		#ifdef SUPPORT_PANEL_DRIVING
		0x7,
		#endif
		#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
		0xb7ff0003,  // +/-0.33%
		#endif			
		#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
		320, // Pixel Size = 0.320 mm = 320 um
		#endif			  
		#ifdef SUPPORT_PANEL_SS
		0,
		0,
		#endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
        1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
	},
	#endif

	#if ((PANEL_SELECT == PANEL_FHD_BACKEND) || CC_MULTI_PANEL) 
	{
		PANEL_FHD_BACKEND,

		148500000,				// u4PixelClk48Hz
		2184,					// u2HTotal48Hz
		1416,					// u2VTotal48Hz
		150,					// u2DimmingFrequency48Hz

		#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
		{32, 32, 32},			// u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
		{4, 4, 4},			    // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
		{32, 32, 32},		    // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
		{16, 16, 16},			// u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
		HSYNC_LOW,
		VSYNC_LOW,
		#endif	  

		#ifdef SUPPORT_PANEL_DITHER
		0xa000,
		#endif
		#ifdef SUPPORT_PANEL_DRIVING
		0x7,
		#endif
		#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
		0xb7ff0003,  // +/-0.33%
		#endif			
		#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
		320,					// Pixel Size = 0.320 mm = 320 um
		#endif			  
		#ifdef SUPPORT_PANEL_SS
		0,
		0,
		#endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		89,  // u2PanleWidth
		50,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
	},
	#endif	
	
    #if ((PANEL_SELECT == PANEL_AUO_4K2K_FHD60) || CC_MULTI_PANEL) 
    {
		PANEL_AUO_4K2K_FHD60,

        148500000,                //u4PixelClk48Hz
        2200,                      // u2HTotal48Hz
        1406,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x5,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        30,
        30000,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif	
    #if ((PANEL_SELECT == PANEL_CMI_4K2K_FHD120) || CC_MULTI_PANEL) 
    {
		PANEL_CMI_4K2K_FHD120,

        297000000,                //u4PixelClk48Hz
        2292,                      // u2HTotal48Hz
        1350,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {30, 30, 30},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x4,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        30,
        30000,
        #endif
        #ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
        #endif	
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif		
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif
    #if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX) || CC_MULTI_PANEL) 
    {
		PANEL_CMO_V580DK1_LS1_INX,

        297000000,                //u4PixelClk48Hz
        2292,                      // u2HTotal48Hz
        1350,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {30, 30, 30},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x4,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        30,
        30000,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif	
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif
    #if ((PANEL_SELECT == PANEL_CSOT_4K2K_FHD60) || CC_MULTI_PANEL) 
    {
		PANEL_CSOT_4K2K_FHD60,

        148500000,                //u4PixelClk48Hz
        2292,                      // u2HTotal48Hz
        1350,                    // u2VTotal48Hz
        150,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {30, 30, 30},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x4,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0x10,
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        30,
        30000,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif	
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif	
    #if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_VB1) || CC_MULTI_PANEL) 
    {
		PANEL_CMO_V580DK1_LS1_INX_VB1,
		
		296947200,				  //u4PixelClk48Hz
		2200,					   // u2HTotal48Hz
		1406,					 // u2VTotal48Hz
		160,
		
        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
		{44, 44, 44},			   //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
		{3, 3, 3},				  // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
		{16, 16, 16},			 // H front porch, 1 represents 1 pixel
		{3, 3, 3},              // V back porch, 1represents 1 line
		HSYNC_LOW,
		VSYNC_LOW,
        #endif    
        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
		0xb7ff0003,  // +/-0.33%
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif	
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif	
    #if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_60_VB1) || CC_MULTI_PANEL) 
    {
		PANEL_CMO_V580DK1_LS1_INX_60_VB1,

        148473600,                //u4PixelClk48Hz
        2200,                      // u2HTotal48Hz
        1406,                    // u2VTotal48Hz
        160,

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    
         #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0xb7ff0003,  // +/-0.33%
        #endif          
	    #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	    #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
	},
    #endif	
	
#if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST) || CC_MULTI_PANEL) 
    {
		PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST,

        148473600,                //u4PixelClk48Hz
        2200,                      // u2HTotal48Hz
        1406,                    // u2VTotal48Hz
        160,

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},              //u1HSyncWidth[3]; 5bits, 1 represents 1 pixel in 5387
        {3, 3, 3},                // u1VSyncWidth[3]= VSyn, 4bits, 1 represents 1 line,
        {16, 16, 16},            // H front porch, 1 represents 1 pixel
        {3, 3, 3},//{316, 260, 34}            // V back porch, 1represents 1 line
        HSYNC_LOW,
        VSYNC_LOW,
	#endif	  
	 #ifdef SUPPORT_PANEL_DITHER
        0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
        0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	    320, // Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
        0,
        0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
    #endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,  // u2PanleWidth
		0,  // u2PanleHeight
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
	#endif
	},
#endif	
    #if ((PANEL_SELECT == PANEL_4K_BACKEND) || CC_MULTI_PANEL) 
    {
        PANEL_4K_BACKEND,

        297047808,              // u4PixelClk48Hz
        5496,                   // u2HTotal48Hz
        1126,                   // u2VTotal48Hz
        150,                    // u2DimmingFrequency48Hz

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88, 88, 88},           // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {4, 4, 4},              // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {1272, 1056, 176},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {32, 32, 32},           // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    

        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0xb7ff0003,  // +/-0.33%
        #endif          
        #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
        #endif            
        #ifdef SUPPORT_PANEL_SS
        0,
        0,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
    	#endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		0,  // u1PreEmpEn
		0,  // u1PreEmpVal
		#endif
		#ifdef SUPPORT_PANEL_SIZE_CM
		143,  // u2PanleWidth
		80,  // u2PanleHeight
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif  

#if ((PANEL_SELECT == PANEL_4K2KDD) || CC_MULTI_PANEL) 
{
	PANEL_4K2KDD,

	593568000,			  // u4PixelClk48Hz
	5496,					// u2HTotal48Hz
	2250,					// u2VTotal48Hz
	150,					// u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
	{88, 88, 88},		 // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
	{6, 6, 6},		  // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
	{1128, 912, 32},	  // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
	{6, 6, 6},		  // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
	HSYNC_LOW,
	VSYNC_LOW,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
	0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
	0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
	0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	320,					// Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
	0,
	0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
	0,	// u4ScanPWMStart, 0 means 0%
	0,	// u4ScanPWMDuty, 0 means 0%
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
	1,	// u1PreEmpEn
	1,	// u1PreEmpVal
	#endif		
	#ifdef SUPPORT_PANEL_SIZE_CM
	0,
	0,	
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
	1, 
	PANEL_STEP_LIMIT_LEVEL3,
	#endif
},
#endif	
#if ((PANEL_SELECT == PANEL_4K2KDD_2SEC) || CC_MULTI_PANEL) 
{
	PANEL_4K2KDD_2SEC,

	593568000,			  // u4PixelClk48Hz
	5496,					// u2HTotal48Hz
	2250,					// u2VTotal48Hz
	150,					// u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
	{88, 88, 88},		 // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
	{6, 6, 6},		  // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
	{1128, 912, 32},	  // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
	{6, 6, 6},		  // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
	HSYNC_LOW,
	VSYNC_LOW,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
	0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
	0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
	0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
	320,					// Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
	0,
	0,
	#endif	
	#ifdef SUPPORT_PANEL_SCAN_PWM
	0,	// u4ScanPWMStart, 0 means 0%
	0,	// u4ScanPWMDuty, 0 means 0%
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
	1,	// u1PreEmpEn
	1,	// u1PreEmpVal
	#endif
	#ifdef SUPPORT_PANEL_SIZE_CM
	0,
	0,	
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
	1, 
	PANEL_STEP_LIMIT_LEVEL3,
	#endif
},
#endif 
	#if ((PANEL_SELECT == PANEL_VB1_4K2KDD_GRAPH) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_4K2KDD_GRAPH,

        593568000,            // u4PixelClk48Hz
        5496,                   // u2HTotal48Hz
        2250,                   // u2VTotal48Hz
        96,                    // u2DimmingFrequency48Hz

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88, 88, 88},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {6, 6, 6},        // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {1128, 912, 32},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {6, 6, 6},        // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    

        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0xb7ff0003,  // +/-0.33%
        #endif          
        #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
        #endif            
        #ifdef SUPPORT_PANEL_SS
        10,
        30000,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif		
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,	
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif 

	#if ((PANEL_SELECT == PANEL_VB1_1080P24) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_1080P24,

        74250000,               // u4PixelClk48Hz
        2750,                   // u2HTotal48Hz
        1125,                   // u2VTotal48Hz
        24,                     // u2DimmingFrequency48Hz

        #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {44, 44, 44},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {3, 3, 3},        // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {16, 16, 16},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {3, 3, 3},        // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
        #endif    

        #ifdef SUPPORT_PANEL_DITHER
        0xa000,
        #endif
        #ifdef SUPPORT_PANEL_DRIVING
        0x7,
        #endif
        #ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE    
        0xb7ff0003,  // +/-0.33%
        #endif          
        #ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
        #endif            
        #ifdef SUPPORT_PANEL_SS
        10,
        30000,
        #endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
        #endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif		
		#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,	
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
    #endif 

	#if ((PANEL_SELECT == PANEL_VB1_576P) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_576P,

        27000000,               // u4PixelClk48Hz
        864,                   // u2HTotal48Hz
        625,                   // u2VTotal48Hz
        50,                     // u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {64, 64, 64},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {5, 5, 5},        // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {12, 12, 12},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {39, 39, 39},        // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_HIGH,
        VSYNC_HIGH,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
        0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
        0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
        0,
        0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif		
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,	
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
	    1, 
		PANEL_STEP_LIMIT_LEVEL3,
	#endif
    },
	#endif 
	
	#if ((PANEL_SELECT == PANEL_VB1_2160P) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_2160P,
			
        593568000,            // u4PixelClk48Hz
        5496,                   // u2HTotal48Hz
        2250,                   // u2VTotal48Hz
        150,                    // u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88, 88, 88},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {10, 10, 10},        // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {1272, 1056, 176},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {72, 72, 72},        // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
        0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
        0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
        0,
        0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif		
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
		1, 
		PANEL_STEP_LIMIT_LEVEL3,
	#endif
    },
	#endif	
	
	#if ((PANEL_SELECT == PANEL_VB1_2160P30) || CC_MULTI_PANEL) 
	{
		PANEL_VB1_2160P30,

		296784000,				// u4PixelClk48Hz
		5496,					// u2HTotal48Hz
		2250,					// u2VTotal48Hz
		150,					// u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
		{88, 88, 88},			// u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
		{10, 10, 10},			    // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
		{1272, 1056, 176},		    // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
		{72, 72, 72},			// u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
		HSYNC_LOW,
		VSYNC_LOW,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
		0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
		0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
		0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
		320,					// Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
		0,
		0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, permillage, 1 means 0.001
        0,  // u4ScanPWMDuty, permillage, 1 means 0.001
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,	
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
	#endif
	},
	#endif

	#if ((PANEL_SELECT == PANEL_VB1_2160P_2SEC) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_2160P_2SEC,
			
        593568000,            // u4PixelClk48Hz
        5496,                   // u2HTotal48Hz
        2250,                   // u2VTotal48Hz
        150,                    // u2DimmingFrequency48Hz

	#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88, 88, 88},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {10, 10, 10},        // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {1272, 1056, 176},      // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {72, 72, 72},        // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
	#endif	  

	#ifdef SUPPORT_PANEL_DITHER
        0xa000,
	#endif
	#ifdef SUPPORT_PANEL_DRIVING
        0x7,
	#endif
	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
	#endif			
	#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
	#endif			  
	#ifdef SUPPORT_PANEL_SS
        0,
        0,
	#endif
	#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
	#endif
	#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
	#endif		
	#ifdef SUPPORT_PANEL_SIZE_CM
		0,
		0,
	#endif
	#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
	#endif
    },
	#endif	

	#if ((PANEL_SELECT == PANEL_VB1_4K2K120) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_4K2K120,
	#if 1
        593568000*2,            // u4PixelClk48Hz
        5496,                   // u2HTotal48Hz
        2250,                   // u2VTotal48Hz
	#else
		593894400,			    // u4PixelClk48Hz
		4400,					// u2HTotal48Hz
		2812,					// u2VTotal48Hz
	#endif
        150,                    // u2DimmingFrequency48Hz

		#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88, 88, 88},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {6, 6, 6},           // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
	#if 1
        {1128, 912, 32},     // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
	#else
		{32, 32, 32},
	#endif
        {6, 6, 6},           // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
		#endif	  

		#ifdef SUPPORT_PANEL_DITHER
        0xa000,
		#endif
		#ifdef SUPPORT_PANEL_DRIVING
        0x7,
		#endif
		#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
		#endif			
		#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,         // Pixel Size = 0.320 mm = 320 um
		#endif			  
		#ifdef SUPPORT_PANEL_SS
        10,
        30000,
		#endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
		#endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif		
		#ifdef SUPPORT_PANEL_SIZE_CM
		92,
		52,
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
	#endif		
	
	#if ((PANEL_SELECT == PANEL_VB1_8K4K30) || CC_MULTI_PANEL) 
    {
        PANEL_VB1_8K4K30,

        593568000*2,            // u4PixelClk48Hz
        5496*2,                 // u2HTotal48Hz
        2250*2,                 // u2VTotal48Hz
        48,                     // u2DimmingFrequency48Hz

		#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
        {88*2, 88*2, 88*2},        // u1HSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {6*2, 6*2, 6*2},           // u1VSyncWidth[3] = {48Hz, 50Hz, 60Hz}
        {1128*2, 912*2, 32*2},     // u2HFrontPorch[3] = {48Hz, 50Hz, 60Hz}
        {6*2, 6*2, 6*2},           // u2VBackPorch[3] = {48Hz, 50Hz, 60Hz}
        HSYNC_LOW,
        VSYNC_LOW,
		#endif	  

		#ifdef SUPPORT_PANEL_DITHER
        0xa000,
		#endif
		#ifdef SUPPORT_PANEL_DRIVING
        0x7,
		#endif
		#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE	 
        0xb7ff0003,  // +/-0.33%
		#endif			
		#ifdef SUPPORT_PANEL_3D_PIXEL_SIZE
        320,                    // Pixel Size = 0.320 mm = 320 um
		#endif			  
		#ifdef SUPPORT_PANEL_SS
        10,
        30000,
		#endif
		#ifdef SUPPORT_PANEL_SCAN_PWM
        0,  // u4ScanPWMStart, 0 means 0%
        0,  // u4ScanPWMDuty, 0 means 0%
		#endif
		#ifdef SUPPORT_PANEL_PRE_EMPHASIS
		1,  // u1PreEmpEn
		1,  // u1PreEmpVal
		#endif		
		#ifdef SUPPORT_PANEL_SIZE_CM
		92,
		52,
		#endif
		#ifdef SUPPORT_PANEL_STEP_LIMIT
		1,
		PANEL_STEP_LIMIT_LEVEL3,
		#endif
    },
	#endif		
};
#endif //defined(SUPPORT_PANEL_CUSTOMER_SPEC)


// NOTE! the configuration order must follow panel index order
#ifndef CC_PANEL_FROM_FLASH
static PANEL_ATTRIBUTE_T _arPanelAttribute[] =
{
#if ((PANEL_SELECT == PANEL_XGA) || CC_MULTI_PANEL) // refer to CLAA150XP03
    {
        1024,           // u2PanelWidth
        768,            // u2PanelHeight

        74000000,       // u4PixelClkMax
        64930909,       // u4PixelClk60Hz
        54163200,       // u4PixelClk50Hz
        44000000,       // u4PixelClkMin

        1522,           // u2HTotalMax
        1344,           // u2HTotal60Hz
        1344,           // u2HTotal50Hz
        1214,           // u2HTotalMin

        888,            // u2VTotalMax
        806,            // u2VTotal60Hz
        806,            // u2VTotal50Hz
        789,            // u2VTotalMin
        62,             // u1VClkMax
        48,             // u1VClkMin

        0x87,         // u1HSyncWidth
        5,            // u1VSyncWidth
        0x818,        // u1HPosition
        0x3E1,        // u1VPosition
        SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
        PWM_HIGH_PANEL_BRIGHT,

        0xFF,           // (100%) u1BacklightHigh
        0x68,           // (84%) u1BacklightMiddle
        0x00,           // (68%) u1BacklightLow
        0,              // u1Reserved
        0,              // u2DimmingFrequency60Hz
        0,              // u2DimmingFrequency50Hz

        7,              // u1LvdsOnDalay
        80,             // u1BacklightOnDelay
        21,             // u1BacklightOffDelay
        6,              // u1LvdsOffDalay

        50,             // u2MaxOverscan
        0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SXGA) || CC_MULTI_PANEL) // no spec, not recommed
    {
    1280,           // u2PanelWidth
    1024,           // u2PanelHeight

    120000000,      // u4PixelClkMax
    107856610,      // u4PixelClk60Hz
    89970400,       // u4PixelClk50Hz
    88000000,       // u4PixelClkMin

    1801,           // u2HTotalMax
    1688,           // u2HTotal60Hz
    1688,           // u2HTotal50Hz
    1150,           // u2HTotalMin

    1101,           // u2VTotalMax
    1066,           // u2VTotal60Hz
    1066,           // u2VTotal50Hz
    901,            // u2VTotalMin
    75,             // u1VClkMax
    55,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    5,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_19_A1) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 56000000,   // u4PixelClkMax
    88807193,       // u4PixelClk60Hz
    88807193,       // u4PixelClk50Hz
    2 * 36489600,   // u4PixelClkMin

    1920,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1920,           // u2HTotal50Hz
    1500,           // u2HTotalMin

    1050,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    905,            // u2VTotalMin
    75,             // u1VClkMax
    56,             // u1VClkMin

    30,              // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x70,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    45,             // u1BacklightOnDelay
    9,              // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_27_W1) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    720,            // u2PanelHeight

    79500000,       // u4PixelClkMax
    74175815,       // u4PixelClk60Hz
    72000000,       // u4PixelClk50Hz
    70500000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1650,           // u2HTotal60Hz
    1800,           // u2HTotal50Hz
    1450,           // u2HTotalMin

    850,            // u2VTotalMax
    750,            // u2VTotal60Hz
    800,            // u2VTotal50Hz
    730,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,           // u1VPosition
    //245,          // u1HPosition
    //126,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    45,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_27_B1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678910,       // u4PixelClk60Hz
    71470500,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1798,           // u2HTotal50Hz
    1436,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    770,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT, // PWM_HIGH_PANEL_BRIGHT not verified

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_32_B1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678921,       // u4PixelClk60Hz
    75850000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1850,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    820,            // u2VTotal50Hz
    770,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xff,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_37_H1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2250,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT, // PWM_HIGH_PANEL_BRIGHT not verified

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    10,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_42_H1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_ON |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_20_XW2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    66898000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1660,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax,
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x59,           // (67%) u1BacklightMiddle
    0xB3,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    70,             // u1BacklightOnDelay
    2,              // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_32_XW1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    888,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    7,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    21,             // u1BacklightOffDelay
    6,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_HW1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 85000000,   // u4PixelClkMax
    149010989,      // u4PixelClk60Hz
    124300000,      // u4PixelClk50Hz
    2 * 65280000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2000,           // u2HTotalMin

    1200,           // u2VTotalMax
    1130,           // u2VTotal60Hz
    1130,           // u2VTotal50Hz
    1088,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#else
	DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
#endif

    0xFF,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_26_W1) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    66744000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1848,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1328,           // u2HTotalMin

    845,            // u2VTotalMax
    810,            // u2VTotal60Hz
    810,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    6,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_32_W1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1416,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // (100%) u1BacklightHigh
    0xAB,           // (67%) u1BacklightMiddle
    0xE6,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_37_WX1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1472,           // u2HTotalMin

    1063,           // u2VTotalMax
    790 ,            // u2VTotal60Hz
    890 ,            // u2VTotal50Hz
    776 ,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    4,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_37_WU1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,
    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_W2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    76300000,       // u4PixelClkMax
    72354845,       // u4PixelClk60Hz
    72580000,       // u4PixelClk50Hz
    68100000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1456,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    950,            // u2VTotal50Hz
    776,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,
    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    0,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_40_WT) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    80367632,       // u4PixelClk60Hz
    67040000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    2040,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1450,           // u2HTotalMin

    1500,           // u2VTotalMax
    838,            // u2VTotal60Hz
    838,            // u2VTotal50Hz
    773,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x68,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    110,            // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    5,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CLAA_15_XP3) || CC_MULTI_PANEL)
    {
    1024,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    64930901,       // u4PixelClk60Hz
    54163200,       // u4PixelClk50Hz
    50000000,       // u4PixelClkMin

    1370,           // u2HTotalMax
    1344,           // u2HTotal60Hz
    1344,           // u2HTotal50Hz
    1150,           // u2HTotalMin

    860,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    794,            // u2VTotalMin
    75,             // u1VClkMax
    55,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT, // PWM_LOW_PANEL_BRIGHT not verified

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_QD_32_HL1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    74984000,       // u4PixelClk50Hz
    70000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1148,           // u2VTotalMax
    810,            // u2VTotal60Hz
    910,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,
    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_26_WX2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1456,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    776,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,//180,            // u2DimmingFrequency60Hz
    0,//150,            // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_XW2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85162837,       // u4PixelClk60Hz
    71040000,       // u4PixelClk50Hz
    65130000,       // u4PixelClkMin

    1515,           // u2HTotalMax
    1480,           // u2HTotal60Hz
    1480,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    980,            // u2VTotalMax
    960,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    940,            // u2VTotalMin
    61,             // u1VClkMax
    49,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM.
    0x00,//0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,//0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    70,             // u1BacklightOnDelay
    2,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_47_WU4) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1149,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,//180,            // u2DimmingFrequency60Hz
    0,//150,            // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_22_Z1) || CC_MULTI_PANEL)
    {
    1680,           // u2PanelWidth
    1050,           // u2PanelHeight

    2 * 74500000,   // u4PixelClkMax
    119112887,      // u4PixelClk60Hz
    114460000,      // u4PixelClk50Hz
    94340000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1840,           // u2HTotal60Hz
    1940,           // u2HTotal50Hz
    1780,           // u2HTotalMin

    1251,           // u2VTotalMax
    1080,           // u2VTotal60Hz
    1180,           // u2VTotal50Hz
    1060,           // u2VTotalMin
    76,             // u1VClkMax
    50,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    12,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_19_M2) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 65700000,   // u4PixelClkMax
    103672328,      // u4PixelClk60Hz
     90250000,      // u4PixelClk50Hz
    2 * 44375000,   // u4PixelClkMin (refer to PANEL_HSD_19_MGW1)

    1920,           // u2HTotalMax
    1840,           // u2HTotal60Hz
    1900,           // u2HTotal50Hz
    1760,           // u2HTotalMin

    1063,           // u2VTotalMax
    940,            // u2VTotal60Hz
    950,            // u2VTotal50Hz
    908,            // u2VTotalMin
    76,             // u1VClkMax
    56,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh  (Need Norman to confirm)
    0x70,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    15,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    15,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_26_B1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    78626374,       // u4PixelClk60Hz
    65587500,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1650,           // u2HTotal60Hz
    1650,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
    // Increase max luminance.
    0x00,//0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,//160,            // u2DimmingFrequency50Hz
    0,//160,            // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU2) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1105,           // u2VTotalMin
    53,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh (Need Norman to confirm)
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    15,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_HSD_19_MGW1) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 68375000,   // u4PixelClkMax
    106593566,      // u4PixelClk60Hz
    88916800,       // u4PixelClk50Hz
    2 * 44375000,   // u4PixelClkMin

    1936,           // u2HTotalMax
    1904,           // u2HTotal60Hz
    1904,           // u2HTotal50Hz
    1600,           // u2HTotalMin

    1001,           // u2VTotalMax
    934,            // u2VTotal60Hz
    934,            // u2VTotal50Hz
    926,            // u2VTotalMin
    75,             // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh  (Need Norman to confirm)
    0x70,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_32_XW2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    888,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    21,             // u1BacklightOffDelay
    6,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_315_B1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678910,       // u4PixelClk60Hz
    75850000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1850,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    820,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_50_X4_PDP) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    76923000,       // u4PixelClkMax
    74170000,       // u4PixelClk60Hz
    74170000,       // u4PixelClk50Hz
    71429000,       // u4PixelClkMin

    1545,           // u2HTotalMax
    1542,           // u2HTotal60Hz
    1542,           // u2HTotal50Hz
    1540,           // u2HTotalMin

    961,            // u2VTotalMax
    802,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    801,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    96,             // u1HSyncWidth
    10,             // u1VSyncWidth
    0x810,          // u1HPosition
    0x340,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0xAB,           // (67%) u1BacklightMiddle
    0xE6,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    0,              // u1LvdsOnDalay...relay_on->vs_on
    150,            // u1BacklightOnDelay...vs_on->disp_on
    250,            // u1BacklightOffDelay...disp_off->vs_off
    30,             // u1LvdsOffDalay...vs_off->relay_off

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_26_XW3) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_V0) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    90,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    7,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_V1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    90,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    7,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_154_LCM_C01) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 60000000,   // u4PixelClkMax
    88807193,       // u4PixelClk60Hz
    74080000,       // u4PixelClk50Hz
    2 * 25000000,   // u4PixelClkMin

    1760,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1520,           // u2HTotalMin

    1500,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    910,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_23_W2_L1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    84000000,       // u4PixelClkMax
    81550130,       // u4PixelClk60Hz
    68026400,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1750,           // u2HTotalMax
    1688,           // u2HTotal60Hz
    1688,           // u2HTotal50Hz
    1420,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    780,            // u2VTotalMin
    65,             // u1VClkMax
    55,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x30,           // u1BacklightHigh
    0x59,           // u1BacklightMiddle
    0xf0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    120,            // u1BacklightOnDelay
    12,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_V6) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    164000000,      // u4PixelClkMax
    146016000,      // u4PixelClk120Hz
    149760000,      // u4PixelClk50Hz
    139400000,      // u4PixelClkMin

    1600,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1496,           // u2HTotalMin

    980,            // u2VTotalMax
    780,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    776,            // u2VTotalMin
    121,             // u1VClkMax
    99,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xa0,           // (100%) u1BacklightHigh
    0x80,           // (84%) u1BacklightMiddle
    0x60,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    35,             // u1BacklightOnDelay
    2,              // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_32_WX3_SLB1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    71200000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1456,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // (100%) u1BacklightHigh
    0xAB,           // (67%) u1BacklightMiddle
    0xE6,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_37_WX1_SLA1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    71200000,       // u4PixelClk50Hz
    63000000,       // u4PixelClkMin

    1840,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1446,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CLAA_32_WB2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    84000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    66744000,       // u4PixelClk50Hz
    62000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1575,           // u2HTotalMin

    1063,           // u2VTotalMax
    810,            // u2VTotal60Hz
    810,            // u2VTotal50Hz
    790,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x1B,           // (100%) u1BacklightHigh
    0x73,           // (67%) u1BacklightMiddle
    0xB2,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    16,             // u1BacklightOnDelay
    15,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_154_LCM_C04) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_201_TVM_C01) || CC_MULTI_PANEL)
    {
    1400,           // u2PanelWidth
    1050,           // u2PanelHeight

    2 * 67500000,   // u4PixelClkMax
    107856623,      // u4PixelClk60Hz
    89970400,       // u4PixelClk50Hz
    2 * 40000000,   // u4PixelClkMin

    1960,           // u2HTotalMax
    1688,           // u2HTotal60Hz
    1688,           // u2HTotal50Hz
    1480,           // u2HTotalMin

    1300,           // u2VTotalMax
    1066,           // u2VTotal60Hz
    1066,           // u2VTotal50Hz
    1051,           // u2VTotalMin
    75,             // u1VClkMax
    56,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_ON |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    15,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_37_WX1_SL2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1472,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    776,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_XW2_V5) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    87760000,       // u4PixelClkMax
    85162837,       // u4PixelClk60Hz
    71040000,       // u4PixelClk50Hz
    64710000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    980,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    49,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,//0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,//0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_42_XW1_V3) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    822,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    10,             // u1LvdsOffDalay

    15,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_QD_32_HL1_W) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    74984000,       // u4PixelClk50Hz
    70000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1148,           // u2VTotalMax
    810,            // u2VTotal60Hz
    910,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_XW1_V1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85162837,       // u4PixelClk60Hz
    71040000,       // u4PixelClk50Hz
    65130000,       // u4PixelClkMin

    1515,           // u2HTotalMax
    1480,           // u2HTotal60Hz
    1480,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    980,            // u2VTotalMax
    960,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    940,            // u2VTotalMin
    61,             // u1VClkMax
    49,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    8,              // u1BacklightOffDelay
    15,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_XW1_V2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85162837,       // u4PixelClk60Hz
    71040000,       // u4PixelClk50Hz
    65130000,       // u4PixelClkMin

    1515,           // u2HTotalMax
    1480,           // u2HTotal60Hz
    1480,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    980,            // u2VTotalMax
    960,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    940,            // u2VTotalMin
    61,             // u1VClkMax
    49,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    8,              // u1BacklightOffDelay
    15,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_W2_SLA2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    76300000,       // u4PixelClkMax
    72354845,       // u4PixelClk60Hz
    72580000,       // u4PixelClk50Hz
    68100000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1528,           // u2HTotal50Hz
    1456,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    950,            // u2VTotal50Hz
    776,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_V3) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    963,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    90,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    7,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_A01H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1900,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1015,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    13,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_42_HW1_V0) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 84960000,   // u4PixelClkMax
    149010989,      // u4PixelClk60Hz
    124300000,      // u4PixelClk50Hz
    2 * 67360000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2060,           // u2HTotalMin

    1200,           // u2VTotalMax
    1130,           // u2VTotal60Hz
    1130,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU3_SLA1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

#if 1
	2 * 77000000,	// u4PixelClkMax
	148500000,		// u4PixelClk60Hz
	148500000,		// u4PixelClk50Hz
	2 * 59630000,	// u4PixelClkMin
#else  // for VOPLL verification
	2 * 77000000,	// u4PixelClkMax
	148351648,		// u4PixelClk60Hz
	123750000,		// u4PixelClk50Hz
	2 * 59630000,	// u4PixelClkMin
#endif

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

#if 1
	1420,			// u2VTotalMax
	1125,			// u2VTotal60Hz
	1350,			// u2VTotal50Hz
	1091,			// u2VTotalMin
#else  // for VOPLL verification
	1201,			// u2VTotalMax
	1125,			// u2VTotal60Hz
	1125,			// u2VTotal50Hz
	1091,			// u2VTotalMin
#endif

    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_42_H1_L5) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_ON |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_154_C1_P1) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 60000000,   // u4PixelClkMax
    88807193,       // u4PixelClk60Hz
    74080000,       // u4PixelClk50Hz
    2 * 25000000,   // u4PixelClkMin

    1760,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1520,           // u2HTotalMin

    1500,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    910,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_154_I2_L2) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_40_FHD) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    130000000,      // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    155000000,      // u4PixelClkMin

    2650,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1550,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    65,             // u1VClkMax
    45,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_32_WT_L5) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80367632,       // u4PixelClk60Hz
    80364200,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1918,           // u2HTotal50Hz
    1460,           // u2HTotalMin

    1200,           // u2VTotalMax
    838,            // u2VTotal60Hz
    838,            // u2VTotal50Hz
    773,            // u2VTotalMin
    66,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x30,           // u1BacklightHigh
    0x59,           // u1BacklightMiddle
    0xf0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    101,            // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_46_HS_L03) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1139,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x20,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_42_H1_L5_DELTA) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_ON |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_154_I5_P2) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_40_XW1_V0) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1000,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xE0,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    70,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_40_WT_L17) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    80367632,       // u4PixelClk60Hz
    67040000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    2040,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1450,           // u2HTotalMin

    1500,           // u2VTotalMax
    838,            // u2VTotal60Hz
    838,            // u2VTotal50Hz
    773,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x68,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    110,            // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    5,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW1_VH) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    66744000,       // u4PixelClk50Hz
    55000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1148,           // u2VTotalMax
    810,            // u2VTotal60Hz
    910,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_C12H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678910,       // u4PixelClk60Hz
    75850000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1850,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    820,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW1_VG) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    74984000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1148,           // u2VTotalMax
    810,            // u2VTotal60Hz
    910,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_A02H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1900,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1015,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    13,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_A12H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1900,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1015,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    13,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_A03H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012787,       // u4PixelClk60Hz
    74984000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1148,           // u2VTotalMax
    810,            // u2VTotal60Hz
    910,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x68,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_A21H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1900,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1015,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0xFF,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    13,             // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_C02H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678910,       // u4PixelClk60Hz
    75850000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1850,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    820,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_260_TVM_C01H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    74909481,       // u4PixelClk60Hz
    62480000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1572,           // u2HTotal60Hz
    1572,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_260_TVM_C11H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    74909481,       // u4PixelClk60Hz
    62480000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1572,           // u2HTotal60Hz
    1572,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    11,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_260_TVM_A01H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366224,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    55000000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    2,             // u1BacklightOffDelay
    4,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_260_TVM_A11H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366224,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    55000000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    784,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    2,             // u1BacklightOffDelay
    4,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_26_B1_L04) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    78626374,       // u4PixelClk60Hz
    65587500,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1650,           // u2HTotal60Hz
    1650,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
    // Increase max luminance.
    0x00,//0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    160,            // u2DimmingFrequency60Hz
    160,            // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_260_TVM_A04H) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80012777,       // u4PixelClk60Hz
    66744000,       // u4PixelClk50Hz
    55000000,       // u4PixelClkMin

    1780,           // u2HTotalMax
    1648,           // u2HTotal60Hz
    1648,           // u2HTotal50Hz
    1512,           // u2HTotalMin

    1053,           // u2VTotalMax
    810,            // u2VTotal60Hz
    810,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
    // Increase max luminance.
    0x00,//0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xFF,           // (50%) u1BacklightLow
    0,              // u1Reserved
    160,            // u2DimmingFrequency60Hz
    160,            // u2DimmingFrequency50Hz

    6,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    4,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_42_HW1_V1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 84960000,   // u4PixelClkMax
    149010989,      // u4PixelClk60Hz
    124300000,      // u4PixelClk50Hz
    2 * 67360000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2060,           // u2HTotalMin

    1200,           // u2VTotalMax
    1130,           // u2VTotal60Hz
    1130,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_46_HS_L03_BESTBUY) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1139,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x20,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_154_LCM_C05) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_154_LCM_C03) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_154_I2_P2) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_19_A1_P2) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 56000000,   // u4PixelClkMax
    88807193,       // u4PixelClk60Hz
    74080000,       // u4PixelClk50Hz
    2 * 36489600,   // u4PixelClkMin

    1920,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1500,           // u2HTotalMin

    1050,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    905,            // u2VTotalMin
    75,             // u1VClkMax
    56,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x70,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    45,             // u1BacklightOnDelay
    9,              // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_22_Z1_P1) || CC_MULTI_PANEL)
    {
    1680,           // u2PanelWidth
    1050,           // u2PanelHeight

    2 * 74500000,   // u4PixelClkMax
    119112887,      // u4PixelClk60Hz
    114460000,      // u4PixelClk50Hz
    94340000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1840,           // u2HTotal60Hz
    1940,           // u2HTotal50Hz
    1780,           // u2HTotalMin

    1251,           // u2VTotalMax
    1080,           // u2VTotal60Hz
    1180,           // u2VTotal50Hz
    1060,           // u2VTotalMin
    76,             // u1VClkMax
    50,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    12,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_154_I2_P1) || CC_MULTI_PANEL)
    {
    1280,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    71036164,       // u4PixelClk60Hz
    59256000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1440,           // u2HTotal60Hz
    1440,           // u2HTotal50Hz
    1360,           // u2HTotalMin

    1063,           // u2VTotalMax
    823,            // u2VTotal60Hz
    823,            // u2VTotal50Hz
    810,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0x70,           // u1BacklightMiddle
    0x00,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_INNOLUX_19_AW01) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    170000000,      // u4PixelClkMax
    88807193,            // u4PixelClk60Hz
    74080000,       // u4PixelClk50Hz
    50000000,      // u4PixelClkMin...

    2000,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1504,           // u2HTotalMin

    1063,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    905,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xA0,           // (100%) u1BacklightHigh //
    0x80,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDelay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_PV_320_TVM_C01) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    85678910,       // u4PixelClk60Hz
    75850000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1798,           // u2HTotal60Hz
    1850,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,            // u2VTotalMax
    795,            // u2VTotal60Hz
    820,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
    // Inverse PWM, increase luminance.
    0x00,//0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xff,//0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    10,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_42_H1_L8) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 60000000,   // u4PixelClkMin

    2300,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2100,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1115,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_ON |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WX4_SLB1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    2 * 80000000,   // u4PixelClkMax
    144520280,      // u4PixelClk60Hz
    120554000,      // u4PixelClk50Hz
    2 * 56000000,   // u4PixelClkMin

    1558,           // u2HTotalMax
    1526,           // u2HTotal60Hz
    1526,           // u2HTotal50Hz
    1494,           // u2HTotalMin

    805,            // u2VTotalMax
    790,            // u2VTotal60Hz
    790,            // u2VTotal50Hz
    775,            // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_40_FHD_10BIT) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    130000000,      // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    155000000,      // u4PixelClkMin

    2650,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1550,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    65,             // u1VClkMax
    45,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_24BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_24BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_V6_8280) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    800,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    76675325,       // u4PixelClk120Hz
    74880000,       // u4PixelClk100Hz
    69700000,       // u4PixelClkMin

    1600,           // u2HTotalMax
    1560,           // u2HTotal120Hz
    1560,           // u2HTotal100Hz
    1496,           // u2HTotalMin

    980,            // u2VTotalMax
    820,            // u2VTotal120Hz
    960,            // u2VTotal100Hz
    776,            // u2VTotalMin
    122,            // u1VClkMax
    98,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xa0,           // (100%) u1BacklightHigh
    0x80,           // (84%) u1BacklightMiddle
    0x60,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    2,              // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_8280) || CC_MULTI_PANEL)
    {
    1952,           // u2PanelWidth
    1096,           // u2PanelHeight

    2 * 75000000,   // u4PixelClkMax
    148351648,      // u4PixelClk120Hz
    148351648,      // u4PixelClk100Hz
    2 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal120Hz
    2200,           // u2HTotal100Hz
    2080,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal120Hz
    1350,           // u2VTotal100Hz
    1090,           // u2VTotalMin
    121,            // u1VClkMax
    91,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
},
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WX4_SLB1_8280) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    800,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    75004196,       // u4PixelClk120Hz
    62566000,       // u4PixelClk100Hz
    56000000,       // u4PixelClkMin

    1558,           // u2HTotalMax
    1526,           // u2HTotal120Hz
    1526,           // u2HTotal100Hz
    1494,           // u2HTotalMin

    850,            // u2VTotalMax
    820,            // u2VTotal120Hz
    820,            // u2VTotal100Hz
    775,            // u2VTotalMin
    123,            // u1VClkMax
    97,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_42_HW1_V3) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

#ifdef CC_SCPOS_3DTV_SUPPORT
    4 * 84900000,   // u4PixelClkMax
    297000000,//297830400,      // u4PixelClk60Hz
    247500000,//248192000,      // u4PixelClk50Hz
    4 * 59400000,// 4 * 69400000,   // u4PixelClkMin

    2508,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,          // u2HTotal50Hz
    2112,           // u2HTotalMin
#else
    4 * 84900000,   // u4PixelClkMax
    297830400,      // u4PixelClk60Hz
    248192000,      // u4PixelClk50Hz
    4 * 69400000,   // u4PixelClkMin

    2508,           // u2HTotalMax
    2240,           // u2HTotal60Hz
    2240,           // u2HTotal50Hz
    2112,           // u2HTotalMin
#endif

    1201,           // u2VTotalMax
    1125,           // u2VTotal120Hz
    1125,           // u2VTotal100Hz
    1090,           // u2VTotalMin
    120,             // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,
#else
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,
#endif

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    25,             // u1BacklightOnDelay
    0,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_40_HH_LH2) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    310000000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    297000000,      // u4PixelClk50Hz
    240000000,   // u4PixelClkMin

    2701,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2140,           // u2HTotalMin

    1601,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1105,           // u2VTotalMin
    125,            // u1VClkMax
    90,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_46_HH_L01) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    310000000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    296700000,      // u4PixelClk50Hz
    260000000,   // u4PixelClkMin

    2350,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2300,           // u2HTotal50Hz
    2090,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1290,           // u2VTotal50Hz
    1092,           // u2VTotalMin
    130,             // u1VClkMax
    90,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT |
    LVDS_A_FROM_AE | LVDS_B_FROM_BE | LVDS_C_FROM_AO | LVDS_D_FROM_BO,

    0x20,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_FHD60_LVDS_CAP) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,         // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    4,             // u1HSyncWidth
    5,              // u1VSyncWidth
    0,            // u1HPosition
    1024,    // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_FHD120_LVDS_CAP) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,         // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    4,             // u1HSyncWidth
    5,              // u1VSyncWidth
    0,            // u1HPosition
    1027,    // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_WXGA60_LVDS_CAP) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,         // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    4,             // u1HSyncWidth
    5,              // u1VSyncWidth
    0,            // u1HPosition
    1027,    // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_WXGA120_LVDS_CAP) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 70000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,         // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    4,             // u1HSyncWidth
    5,              // u1VSyncWidth
    0,            // u1HPosition
    1027,    // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x23, // (100%) u1BacklightHigh     // u1BacklightHigh
    0x7D, // (67%) u1BacklightMiddle    // u1BacklightMiddle
    0xC5, // (50%) u1BacklightLow       // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SLT_FHD_60Hz_10BIT) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 82000000,      // u4PixelClMax
    148500000,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 50000000,      // u4PixelClkMin

    2650,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1550,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    65,             // u1VClkMax
    45,             // u1VClkMin

    30,              // u1HSyncWidth
    1,               // u1VSyncWidth
    0x82c,           // u1HPosition

    0x3fc,            // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SLT_FHD_120Hz_10BIT) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 84900000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 59400000,   // u4PixelClkMin

    2650,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1125,
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    130,
    90,

    30,               // u1HSyncWidth
    1,                // u1VSyncWidth
    0x810,            // u1HPosition

    0x400,            // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_INNOLUX_19_AW02_LIPS_LOW) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    170000000,      // u4PixelClkMax
    88807193,            // u4PixelClk60Hz
    74080000,       // u4PixelClk50Hz
    50000000,      // u4PixelClkMin...

    2000,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1504,           // u2HTotalMin

    1063,           // u2VTotalMax
    926,            // u2VTotal60Hz
    926,            // u2VTotal50Hz
    905,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xF0,           // (100%) u1BacklightHigh
    0x90,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDelay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_216_B1_L01_LIPS_LOW) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    78626374,       // u4PixelClk60Hz
    65587500,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1650,           // u2HTotal60Hz
    1650,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,
    // Increase max luminance.
    0xF0,           // (100%) u1BacklightHigh //
    0x90,           // (77%) u1BacklightMiddle
    0x00,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,            // u2DimmingFrequency60Hz
    0,            // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_260_B1_L11_LIPS_HIGH) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    78626374,       // u4PixelClk60Hz
    65587500,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1650,           // u2HTotal60Hz
    1650,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    795,            // u2VTotal60Hz
    795,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
    // Increase max luminance.
    0x00,//0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    160,            // u2DimmingFrequency60Hz
    160,            // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_19_PW1) || CC_MULTI_PANEL)
    {
    1440,           // u2PanelWidth
    900,            // u2PanelHeight

    2 * 64000000,   // u4PixelClkMax
    94970880,       // u4PixelClk60Hz
    79142400,       // u4PixelClk50Hz
    2 * 32500000,   // u4PixelClkMin

    2048,           // u2HTotalMax
    1728,           // u2HTotal60Hz
    1728,           // u2HTotal50Hz
    1520,           // u2HTotalMin

    2048,           // u2VTotalMax
    916,            // u2VTotal60Hz
    916,            // u2VTotal50Hz
    910,            // u2VTotalMin
    76,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x70,           // (77%) u1BacklightMiddle
    0xFF,           // (55%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    200,             // u1BacklightOnDelay
    200,              // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_216_B1_L1) || CC_MULTI_PANEL)
     {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    80415584,       // u4PixelClk60Hz //V(842) H(2376) F(60)
    67080000,       // u4PixelClk50Hz //V(842) H(2376) F(50)
    60000000,       // u4PixelClkMin...spec conflict

    1936,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    888,           // u2VTotalMax
    861,           // u2VTotal60Hz
    861,           // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    57,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // (100%) u1BacklightHigh
    0x70,           // (77%) u1BacklightMiddle//@I001 refine back light duty cycle
    0xFF,           // (55%) u1BacklightLow
    0,              // u1Reserved
    3200,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay 					//@I001
    30,             // u1BacklightOnDelay 				//@I001
    6,             // u1BacklightOffDelay 				//@I001
    1,              // u1LvdsOffDalay

    90,             // u2MaxOverscan                               //@I012 20080708
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_XW2_VC) || CC_MULTI_PANEL)  // @sammi modified 080910start
    {
    1366,           // u2PanelWidth
    768,           // u2PanelHeight

    85000000,   // u4PixelClkMax
    78115200,      // u4PixelClk60Hz
    59645000,      // u4PixelClk50Hz
    60000000,   // u4PixelClkMin

    1901,           // u2HTotalMax
    1649,           // u2HTotal60Hz
    1511,           // u2HTotal50Hz
    1415,           // u2HTotalMin

    1016,           // u2VTotalMax
    807,           // u2VTotal60Hz
    791,           // u2VTotal50Hz
    785,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition


    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // u1BacklightHigh
    0xB4,           // u1BacklightMiddle
    0x34,           // u1BacklightLow
    0,              // u1Reserved
    150,              // u2DimmingFrequency60Hz
    150,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved               // @sammi modified 080910 end
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_32_WT_1) || CC_MULTI_PANEL) // start add by stanley 080908
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    86000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    2001,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1451,           // u2HTotalMin

    1301,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,         // (100%) u1BacklightHigh 0x00
    0xB5,         // (67%) u1BacklightMiddle 0x8B
    0x34,         // (50%) u1BacklightLow    0xFF
    0,              // u1Reserved
    150,             // u2DimmingFrequency60Hz
    150,             // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_26_WT_1) || CC_MULTI_PANEL) // start add by stanley 080908
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0xB4,           // (67%) u1BacklightMiddle
    0x35,           // (50%) u1BacklightLow
    0,              // u1Reserved
    160,              // u2DimmingFrequency60Hz
    160,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_22_B1_L01) || CC_MULTI_PANEL)  // sammi modified 080912
    {
    1366,           // u2PanelWidth
    768,           // u2PanelHeight

    82000000,   // u4PixelClkMax
    76000000,      // u4PixelClk60Hz
    76000000,      // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1937,           // u2HTotalMax
    1561,           // u2HTotal60Hz
    1561,           // u2HTotal50Hz
    1443,           // u2HTotalMin

    984,           // u2VTotalMax
    807,           // u2VTotal60Hz
    807,           // u2VTotal50Hz
    779,           // u2VTotalMin
    76,             // u1VClkMax
    50,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition

    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,


    0xFF,           // u1BacklightHigh
    0xB5,           // u1BacklightMiddle
    0x34,           // u1BacklightLow
    0,              // u1Reserved
    150,              // u2DimmingFrequency60Hz
    150,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    12,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_19_XW1) || CC_MULTI_PANEL)   //sammi modifed 080912
      {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    90000000,   // u4PixelClkMax
    78000000,       // u4PixelClk60Hz
    78000000,       // u4PixelClk50Hz
    60000000,   // u4PixelClkMin

    2048,           // u2HTotalMax
    1607,           // u2HTotal60Hz
    1607,           // u2HTotal50Hz
    1417,           // u2HTotalMin

    1024,           // u2VTotalMax
    809,            // u2VTotal60Hz
    809,            // u2VTotal50Hz
    777,            // u2VTotalMin
    76,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x80,           // (77%) u1BacklightMiddle
    0x34,           // (55%) u1BacklightLow
    0,              // u1Reserved
    150,              // u2DimmingFrequency60Hz
    150,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    70,             // u1BacklightOnDelay
    2,              // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_52_FHD) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    155000000,      // u4PixelClkMax
    147029702,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    130000000,      // u4PixelClkMin

    2450,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2154,           // u2HTotalMin

    1480,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1100,           // u2VTotalMin
    62,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x20,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xa0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_32_WT_1_WXGA2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    86000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    2001,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1451,           // u2HTotalMin

    1301,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    774,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF| LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,         // (100%) u1BacklightHigh 0x00
    0xB5,         // (67%) u1BacklightMiddle 0x8B
    0x34,         // (50%) u1BacklightLow    0xFF
    0,              // u1Reserved
    150,             // u2DimmingFrequency60Hz
    150,             // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif // end add by stanley 080908
#if ((PANEL_SELECT == PANEL_AUO_315_XW2_VS_WXGA2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    164000000,      // u4PixelClkMax
    146016000,      // u4PixelClk120Hz
    149760000,      // u4PixelClk50Hz
    139400000,      // u4PixelClkMin

    1600,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1496,           // u2HTotalMin

    980,            // u2VTotalMax
    780,            // u2VTotal60Hz
    960,            // u2VTotal50Hz
    776,            // u2VTotalMin
    121,             // u1VClkMax
    99,             // u1VClkMin
    //61,             // u1VClkMax
    //49,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xa0,           // (100%) u1BacklightHigh
    0x80,           // (84%) u1BacklightMiddle
    0x60,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    35,             // u1BacklightOnDelay
    2,              // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SLT_FHD_60Hz_10BIT_AUTO_NS) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    74500000,      // u4PixelClMax
    74250000,      // u4PixelClk60Hz
    74250000,      // u4PixelClk50Hz
    74000000,      // u4PixelClkMin

    2650,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1550,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    31,             // u1VClkMax
    29,             // u1VClkMin

    30,             // u1HSyncWidth
    3, // 1,               // u1VSyncWidth
    0x812,	//0x82c,           // u1HPosition

    0x3fb,            // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    1,              // u1LvdsOnDalay
    20,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_VIDEO_1080P_JEIDA) || CC_MULTI_PANEL)
    {
	1920,			// u2PanelWidth
	1080,			// u2PanelHeight

	2 * 77500000,	// u4PixelClkMax
	148500000,		// u4PixelClk60Hz
	148500000,		// u4PixelClk50Hz
	2 * 66970000,	// u4PixelClkMin

	2800,			// u2HTotalMax
	2200,			// u2HTotal60Hz
	2640,			// u2HTotal50Hz
	2080,			// u2HTotalMin

	1406,			// u2VTotalMax (display mode)
	1125,			// u2VTotal60Hz
	1125,			// u2VTotal50Hz
	1089,			// u2VTotalMin
    63,             // u1VClkMax
	47, 			// u1VClkMin...spec not defined

	44, 			// u1HSyncWidth
	5,				// u1VSyncWidth
	0x82B,			// u1HPosition
	0x3D8,			// u1VPosition

	DUAL_PORT | DISP_24BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
	WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
    0,              // u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_T315XW02_VS_WXGA1) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    888,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    7,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    21,             // u1BacklightOffDelay
    6,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_T315XW02_VS_WXGA2) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    88000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    62868000,       // u4PixelClk50Hz
    54000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    888,            // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    62,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0xFF,           // (100%) u1BacklightHigh
    0x68,           // (84%) u1BacklightMiddle
    0x00,           // (68%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    7,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    21,             // u1BacklightOffDelay
    6,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_32NC_LBA_SESL_FHD) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    155000000,      // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    148500000,      // u4PixelClk50Hz
    130000000,      // u4PixelClkMin

    2650,           // u2HTotalMax
    2184,           // u2HTotal60Hz
    2184,           // u2HTotal50Hz
    2160,           // u2HTotalMin

    1550,           // u2VTotalMax
    1134,           // u2VTotal60Hz
    1360,           // u2VTotal50Hz
    1118,           // u2VTotalMin
    62,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    4,              // u1LvdsOnDalay
    40,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_46_HW03_V4) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | LVDS_DISP_SPLIT_ON,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_65_HW02_V1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 84900000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 59400000,   // u4PixelClkMin

    2508,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2112,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal120Hz
    1125,           // u2VTotal100Hz
    1090,           // u2VTotalMin
    120,             // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#if 0 // AUO Glasses
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,
#else // RealD Glasses
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#endif
    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    25,             // u1BacklightOnDelay
    0,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WUD_SAC1_10BIT) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    120,              // u2DimmingFrequency60Hz
    100,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_LG_42_WU5_SLB1_VB1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    297000000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2640,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_VB1,
#else
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_VB1,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    SUPPORT_VB1_GRAPH_MODE,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV4FHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_LGDV4FHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV4GIPFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_LGDV4GIPFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_ChangHong_PDP) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 76250000,   // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    148500000,      // u4PixelClk50Hz
    2 * 72250000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2000,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1089,           // u2VTotalMin
    62,             // u1VClkMax
    45,             // u1VClkMin

    20,             // u1HSyncWidth
    5,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x3FB,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV5GIPFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV5GIPFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV6GIPFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGDV6GIPFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_AUOFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_AUOFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_SHAFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_SHAFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_FUNAWXGA) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_FUNAFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_HIWXGA09) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_HIWXGA10) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_WXGA) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75366234,       // u4PixelClk60Hz
    68510000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1722,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1700,           // u2HTotal50Hz
    1414,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    806,            // u2VTotal50Hz
    789,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_CMO2011FHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC32WXGA1P6P) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    78106560,       // u4PixelClk60Hz
    78106560,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1721,           // u2HTotalMax
    1576,    //1492,           // u2HTotal60Hz
    1576,    //1492,           // u2HTotal50Hz
    1413,           // u2HTotalMin

    1062,           // u2VTotalMax, Winton for Display Mode
//    821,            // u2VTotalMax
    826,             // u2VTotal60Hz
    991,            // u2VTotal50Hz
    700,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | OD_ON  | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_47_EUH_PR) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition

	// AmTRAN : LVDS = Special NS, PR order = LRLR(RealD glasses) or RLRL(AUO glasses)
	// TPV : LVDS = Special NS, PR order = RLRL(RealD glasses) or LRLR(AUO glasses)
	// Philip : LVDS = NS, PR order = RLRL(unknown glasses)
#if 1
	FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR |OD_ON,
#else
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_M236H5_L0A) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_LG_42_WUD_SAC1_10BIT_NS) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1200,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    120,              // u2DimmingFrequency60Hz
    100,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_PR) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_SG) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_M236H3_LHH3) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    344784000,   // u4PixelClkMax
    316574400,      // u4PixelClk60Hz
    316736000,      // u4PixelClk50Hz
    220180000,   // u4PixelClkMin

    2200,           // u2HTotalMax
    2020,           // u2HTotal60Hz
    2020,           // u2HTotal50Hz
    2020,           // u2HTotalMin

    1600,           // u2VTotalMax (display mode)
    1306,           // u2VTotal60Hz
    1569,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF | LVDS_DISP_SPLIT_ON|
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF | LVDS_DISP_SPLIT_ON|
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_AS3693) || CC_MULTI_PANEL)
   {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS| LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
   },
#endif
#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_IWATT) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS| LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_SPECIAL_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_LG_42_WU5_SLB1_SG_BEX) || CC_MULTI_PANEL) // FHD-120Hz-2D also define as FHD-120Hz-SGX
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    344784000,   // u4PixelClkMax
    316574400,      // u4PixelClk60Hz
    316736000,      // u4PixelClk50Hz
    220180000,   // u4PixelClkMin

    2200,           // u2HTotalMax
    2020,           // u2HTotal60Hz
    2020,           // u2HTotal50Hz
    2020,           // u2HTotalMin

    1569,           // u2VTotalMax (display mode)
    1306,           // u2VTotal60Hz
    1569,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_AUO_46_HW03_V4_FHD240) || CC_MULTI_PANEL) // FHD-240Hz-SG
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    8*76000000,   // u4PixelClkMax
    2*302016000,      // u4PixelClk60Hz
    2*296703297,      // u4PixelClk50Hz
    8*66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1144,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    240,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF | LVDS_SPECIAL_NS | 
    LVDS_A_FROM_AE | LVDS_B_FROM_BE | LVDS_C_FROM_AO | LVDS_D_FROM_BO |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER| LVDS_DISP_SPLIT_OFF | LVDS_HIGHSPEED,
    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    0, //2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_AUO_46_HW03_V3) || CC_MULTI_PANEL) // FHD-120Hz-2D also define as FHD-120Hz-SG
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 76000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_30BIT | LVDS_NS| LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER |
    LVDS_A_FROM_AE | LVDS_B_FROM_BE | LVDS_C_FROM_AO | LVDS_D_FROM_BO,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT |
    LVDS_A_FROM_AE | LVDS_B_FROM_BE | LVDS_C_FROM_AO | LVDS_D_FROM_BO,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_LG_50_CINEMA) || CC_MULTI_PANEL)
    {
    2560,           // u2PanelWidth
    1080,           // u2PanelHeight

    4*103500000,     // u4PixelClkMax
    4*96770000,      // u4PixelClk60Hz
    4*96770000,      // u4PixelClk50Hz
    4*89900000,      // u4PixelClkMin

    4*710,           // u2HTotalMax
    4*700,           // u2HTotal60Hz
    4*700,           // u2HTotal50Hz
    4*680,           // u2HTotalMin

    1390,           // u2VTotalMax (display mode)
    1152,           // u2VTotal60Hz
    1383,           // u2VTotal50Hz
    1105,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF | LVDS_SPECIAL_NS |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR| LVDS_DISP_SPLIT_OFF |
    PANEL_ASPECT_RATIO_21_9,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    0,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_FOR_8283) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    148500000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1400,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition

#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,
#endif

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif



#if ((PANEL_SELECT == PANEL_AUO_T576DC01_V1) || CC_MULTI_PANEL)
    {
    2560,           // u2PanelWidth
    1080,           // u2PanelHeight

    4*103008000,     // u4PixelClkMax
    4*96004800,      // u4PixelClk60Hz
    4*96004800,      // u4PixelClk50Hz
    4*92064000,      // u4PixelClkMin

    4*740,           // u2HTotalMax
    4*708,           // u2HTotal60Hz
    4*708,           // u2HTotal50Hz
    4*700,           // u2HTotalMin

    1392,           // u2VTotalMax (display mode)
    1130,           // u2VTotal60Hz
    1356,           // u2VTotal50Hz
    1096,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL| LVDS_DISP_SPLIT_OFF | PANEL_ASPECT_RATIO_21_9,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    0,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMI_V420H2_LS1) || CC_MULTI_PANEL) // FHD-120Hz-SG
	{
	1920,           // u2PanelWidth
	1080,           // u2PanelHeight

	4 * 77500000,   // u4PixelClkMax
	296703297,      // u4PixelClk60Hz
	296703297,      // u4PixelClk50Hz
	4 * 66970000,   // u4PixelClkMin

	2720,           // u2HTotalMax
	2200,           // u2HTotal60Hz
	2200,           // u2HTotal50Hz
	2080,           // u2HTotalMin

	1380,           // u2VTotalMax (display mode)
	1125,           // u2VTotal60Hz
	1350,           // u2VTotal50Hz
	1090,           // u2VTotalMin
	120,            // u1VClkMax
	60,             // u1VClkMin...spec not defined

	30,             // u1HSyncWidth
	3,              // u1VSyncWidth
	0x810,          // u1HPosition
	0x400,          // u1VPosition

	FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
#if 1 // order = 1023	
	LVDS_A_FROM_AO | LVDS_B_FROM_AE| LVDS_C_FROM_BE | LVDS_D_FROM_BO |
#endif	
	WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
	//WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
	
	0x08,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0x90,           // u1BacklightLow
	0,              // u1Reserved
	0,              // u2DimmingFrequency60Hz
	0,              // u2DimmingFrequency50Hz

	2,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan
	DISP_48HZ_TO_48HZ,              // u2Reserved
	},
#endif
#if ((PANEL_SELECT == PANEL_LC420EUD_SDF1_PR) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    247500000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

//    1165,           // u2VTotalMax
    1201,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition

#if 1
	FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#else
	FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_T645HW05_V0) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 84900000,   // u4PixelClkMax
    297000000,//297830400,      // u4PixelClk60Hz
    247500000,//248192000,      // u4PixelClk50Hz
    4 * 59400000,// 4 * 69400000,   // u4PixelClkMin

    2508,           // u2HTotalMax
    2200,//2239,           // u2HTotal60Hz
    2200,//2239,           // u2HTotal50Hz
    2112,           // u2HTotalMin

    1201,           // u2VTotalMax (display mode)
    1125,           // u2VTotal120Hz
    1125,           // u2VTotal100Hz
    1090,           // u2VTotalMin
    120,             // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition

    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,

    0xf0,           // u1BacklightHigh
    0xd0,           // u1BacklightMiddle
    0xb0,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    5,              // u1LvdsOnDalay
    25,             // u1BacklightOnDelay
    0,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_SS_42_PDP) || CC_MULTI_PANEL)
   {
	 1024,			 // u2PanelWidth
	 2 * 768,			 // u2PanelHeight
	 2 * 78000000,		// u4PixelClkMax
	 2 * 74000000,		// u4PixelClk60Hz
	 2 * 74000000,		 // u4PixelClk50Hz
	 2 * 71200000,		 // u4PixelClkMin
	 1505,			// u2HTotalMax
	 1504,			// u2HTotal60Hz
	 1504,			// u2HTotal50Hz
	 1451,			// u2HTotalMin
	 2 * 830,		   // u2VTotalMax
	 2 * 820, 		   // u2VTotal60Hz
	 2 * 984, 		   // u2VTotal50Hz
	 2 * 810, 		   // u2VTotalMin
	 63,			 // u1VClkMax
	 47,			 // u1VClkMin
	 10,			 // u1HSyncWidth
	 2 * 6, 			 // u1VSyncWidth
	 0x810, 		 // u1HPosition
	 0x400, 		 // u1VPosition
	 DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |PWM_LOW_PANEL_BRIGHT | PANEL_L12R12
	 | LVDS_A_FROM_AO | LVDS_B_FROM_AE | LVDS_C_FROM_BE | LVDS_D_FROM_BO,
	 0x00,			 // u1BacklightHigh
	 0x80,			 // u1BacklightMiddle
	 0xFF,			 // u1BacklightLow
	 0, 			 // u1Reserved
	 0, 			 // u2DimmingFrequency60Hz
	 0, 			 // u2DimmingFrequency50Hz
	 1, 			 // u1LvdsOnDalay
	 20,			 // u1BacklightOnDelay
	 20,			 // u1BacklightOffDelay
	 1, 			 // u1LvdsOffDalay
	 50,			 // u2MaxOverscan
	 0, 			 // u2Reserved
   },
  #endif
#if ((PANEL_SELECT == PANEL_LVDS_TO_8282) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_SS_58_PDP) || CC_MULTI_PANEL) // FHD-60Hz-2D also define as FHD-60Hz-PR
    {
    1920,           // u2PanelWidth
    2 * 1080,       // u2PanelHeight

    4 * 76250000,   // u4PixelClkMax
    4 * 74250000,   // u4PixelClk60Hz
    4 * 74250000,   // u4PixelClk50Hz
    4 * 72250000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2000,           // u2HTotalMin

    2 * 1500,       // u2VTotalMax
    2 * 1125,       // u2VTotal60Hz
    2 * 1350,       // u2VTotal50Hz
    2 * 1089,       // u2VTotalMin
    124,            // u1VClkMax...spec not defined
    90,             // u1VClkMin...spec not defined

    40,             // u1HSyncWidth
    2 * 5,          // u1VSyncWidth
    0x858,          // u1HPosition
    (0x400 - 0x10*2),          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF | PANEL_L12R12|
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF | PANEL_L12R12|
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#endif

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV12GIPFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77500000,   // u4PixelClkMax
    296703297/2,      // u4PixelClk60Hz
    296703297/2,      // u4PixelClk50Hz
    2 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,            // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_AUO_T315HB01_V1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 82000000,   // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 50000000,   // u4PixelClkMin

    2650, // 2649,           // u2HTotalMax
    2200, // 2199,           // u2HTotal60Hz
    2200, // 2199,           // u2HTotal50Hz
    2060, // 2059,           // u2HTotalMin

    1480, // 1479,           // u2VTotalMax
    1125, // 1124,           // u2VTotal60Hz
    1125, // 1124,           // u2VTotal50Hz
    1090, // 1089,           // u2VTotalMin
    63,             // u1VClkMax...spec not defined
    47,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_RLRL,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xFF,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_CMI_42_HW03_V3) || CC_MULTI_PANEL) // FHD-120Hz-2D also define as FHD-120Hz-SG
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 76000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin...spec not defined

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_30BIT | LVDS_NS| LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER |
    LVDS_A_FROM_AO | LVDS_B_FROM_AE | LVDS_C_FROM_BE | LVDS_D_FROM_BO,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT |
    LVDS_A_FROM_AO | LVDS_B_FROM_AE | LVDS_C_FROM_BE | LVDS_D_FROM_BO,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT ==  PANEL_EPI_LGDV12GIPFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_LG_32_PDP) || CC_MULTI_PANEL)
   {

    1366,           // u2PanelWidth
    2 * 768,            // u2PanelHeight

    2 * 80000000,       // u4PixelClkMax
    2 * 72354836,       // u4PixelClk60Hz
    2 * 71200000,       // u4PixelClk50Hz
    2 * 68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1456,           // u2HTotalMin

    2 * 1063,           // u2VTotalMax
    2 * 790,            // u2VTotal60Hz
    2 * 890,            // u2VTotal50Hz
    2 * 775,            // u2VTotalMin

	 63,			 // u1VClkMax
	 47,			 // u1VClkMin
	 10,			 // u1HSyncWidth
	 2 * 6, 			 // u1VSyncWidth
	 0x810, 		 // u1HPosition
	 0x400, 		 // u1VPosition
	 DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |PWM_LOW_PANEL_BRIGHT | PANEL_L12R12,
	 0x00,			 // u1BacklightHigh
	 0x80,			 // u1BacklightMiddle
	 0xFF,			 // u1BacklightLow
	 0, 			 // u1Reserved
	 0, 			 // u2DimmingFrequency60Hz
	 0, 			 // u2DimmingFrequency50Hz
	 1, 			 // u1LvdsOnDalay
	 20,			 // u1BacklightOnDelay
	 20,			 // u1BacklightOffDelay
	 1, 			 // u1LvdsOffDalay
	 50,			 // u2MaxOverscan
	 0, 			 // u2Reserved
   },
  #endif
#if ((PANEL_SELECT == PANEL_SS_L40V8200_120SG) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    310000000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    297000000,      // u4PixelClk50Hz
    240000000,   // u4PixelClkMin

    2701,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2140,           // u2HTotalMin

    1601,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1105,           // u2VTotalMin
    125,            // u1VClkMax
    90,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER |
    LVDS_A_FROM_AO | LVDS_B_FROM_BO | LVDS_C_FROM_AE | LVDS_D_FROM_BE,

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_PDP_PD5L_XGA60) || CC_MULTI_PANEL)
	  {
	  1024, 		  // u2PanelWidth
	  768,			  // u2PanelHeight

	  90000000, 	  // u4PixelClkMax
	  81080160, 	  // u4PixelClk60Hz
	  80948000, 	  // u4PixelClk50Hz
	  70000000, 	  // u4PixelClkMin

	  2050, 		  // u2HTotalMax
	  1636, 		  // u2HTotal60Hz
	  1960, 		  // u2HTotal50Hz
	  1414, 		  // u2HTotalMin

	  1063, 		  // u2VTotalMax
	  826,			  // u2VTotal60Hz
	  826,			  // u2VTotal50Hz
	  789,			  // u2VTotalMin
	  63,			  // u1VClkMax
	  48,			  // u1VClkMin

	  16,			  // u1HSyncWidth
	  2,			  // u1VSyncWidth
	  0xA34,		  // u1HPosition
	  0x400,		  // u1VPosition
	  SINGLE_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
	  PWM_LOW_PANEL_BRIGHT,

	  0x60, 		  // (100%) u1BacklightHigh
	  0x80, 		  // (67%) u1BacklightMiddle
	  0xA0, 		  // (50%) u1BacklightLow
	  0,			  // u1Reserved
	  120,				// u2DimmingFrequency60Hz
	  100,				// u2DimmingFrequency50Hz

	  3,			  // u1LvdsOnDalay
	  60,			  // u1BacklightOnDelay
	  5,			  // u1BacklightOffDelay
	  3,			  // u1LvdsOffDalay

	  30,			  // u2MaxOverscan
	  0,			  // u2Reserved
	  },
#endif

#if ((PANEL_SELECT == PANEL_LG_42_WU3_SLA1_HISPD) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | LVDS_HIGHSPEED,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
  #if ((PANEL_SELECT == PANEL_MLVDS_CMO2011FHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_SS_MB4) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_SS_MB7) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77000000,   // u4PixelClkMax
    148351648,      // u4PixelClk60Hz
    123750000,      // u4PixelClk50Hz
    2 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1201,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC32WXGA2P3P) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    78106560,       // u4PixelClk60Hz
    78106560,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    1721,           // u2HTotalMax
    1576,    //1492,           // u2HTotal60Hz
    1576,    //1492,           // u2HTotal50Hz
    1413,           // u2HTotalMin

    1062,           // u2VTotalMax, Winton for Display Mode
//    821,            // u2VTotalMax
    826,             // u2VTotal60Hz
    991,            // u2VTotal50Hz
    700,            // u2VTotalMin
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | OD_ON  | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_VIDEO_1080P) || CC_MULTI_PANEL)
	{
	1920,			// u2PanelWidth
	1080,			// u2PanelHeight

	2 * 77500000,	// u4PixelClkMax
	148500000,		// u4PixelClk60Hz
	148500000,		// u4PixelClk50Hz
	2 * 66970000,	// u4PixelClkMin

	2800,			// u2HTotalMax
	2200,			// u2HTotal60Hz
	2640,			// u2HTotal50Hz
	2080,			// u2HTotalMin

	1406,			// u2VTotalMax (display mode)
	1125,			// u2VTotal60Hz
	1125,			// u2VTotal50Hz
	1089,			// u2VTotalMin
	63,			// u1VClkMax
	47, 			// u1VClkMin...spec not defined

	44, 			// u1HSyncWidth
	5,				// u1VSyncWidth
	0x858,			// u1HPosition
	0x3DC,			// u1VPosition

	DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
	WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	ENGINEER_DISPLAY_MODE,				// u2Reserved
	},
#endif

#if ((PANEL_SELECT == PANEL_VIDEO_720P) || CC_MULTI_PANEL)
		{
		1280,			// u2PanelWidth
		720,			// u2PanelHeight

		78700000,		// u4PixelClkMax
		74250000,		// u4PixelClk60Hz
		74250000,		// u4PixelClk50Hz
		69795000,		// u4PixelClkMin

		2000,			// u2HTotalMax
		1650,			// u2HTotal60Hz
		1980,			// u2HTotal50Hz
		1400,			// u2HTotalMin

		800,			// u2VTotalMax
		750,			// u2VTotal60Hz
		750,			// u2VTotal50Hz
		500,			// u2VTotalMin
		63, 			// u1VClkMax
		47, 			// u1VClkMin

		40, 			// u1HSyncWidth
		5,				// u1VSyncWidth
		0x86E,			// u1HPosition
		0x3ED,			// u1VPosition
		SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
		PWM_LOW_PANEL_BRIGHT,

		0x70,			// u1BacklightHigh
		0x80,			// u1BacklightMiddle
		0x90,			// u1BacklightLow
		0,				// u1Reserved
		0,				// u2DimmingFrequency60Hz
		0,				// u2DimmingFrequency50Hz

		3,				// u1LvdsOnDalay
		30, 			// u1BacklightOnDelay
		30, 			// u1BacklightOffDelay
		3,				// u1LvdsOffDalay

		30, 			// u2MaxOverscan
		0,				// u2Reserved
		},
#endif

#if ((PANEL_SELECT == PANEL_VIDEO_576P) || CC_MULTI_PANEL)
	{
	720,			// u2PanelWidth
	576,			// u2PanelHeight

	35000000,		// u4PixelClkMax
	32400000,		// u4PixelClk60Hz
	27000000,		// u4PixelClk50Hz
	13500000,		// u4PixelClkMin

	900,			// u2HTotalMax
	864,			// u2HTotal60Hz
	864,			// u2HTotal50Hz
	800,			// u2HTotalMin

	680,			// u2VTotalMax
	625,			// u2VTotal60Hz
	625,			// u2VTotal50Hz
	500,			// u2VTotalMin
	63, 			// u1VClkMax
	47, 			// u1VClkMin

	64, 			// u1HSyncWidth
	5,				// u1VSyncWidth
	0x810,			// u1HPosition
	0x3DA, 			// u1VPosition
	SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	0,				// u2Reserved
	},
#endif

#if ((PANEL_SELECT == PANEL_VIDEO_480P) || CC_MULTI_PANEL)
		{
		720,			// u2PanelWidth
		480,			// u2PanelHeight

		30000000,		// u4PixelClkMax
		27003000,		// u4PixelClk60Hz
		27003000,		// u4PixelClk50Hz
		13500000,		// u4PixelClkMin

		900,			// u2HTotalMax
		858,			// u2HTotal60Hz
		858,			// u2HTotal50Hz
		800,			// u2HTotalMin

		680,			// u2VTotalMax
		525,			// u2VTotal60Hz
		630,			// u2VTotal50Hz
		500,			// u2VTotalMin
		63, 			// u1VClkMax
		47, 			// u1VClkMin

		62, 			// u1HSyncWidth
		6,				// u1VSyncWidth
		0x810,			// u1HPosition
		0x3E2, // 0x3DC,			// u1VPosition
		SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
		PWM_LOW_PANEL_BRIGHT,

		0x70,			// u1BacklightHigh
		0x80,			// u1BacklightMiddle
		0x90,			// u1BacklightLow
		0,				// u1Reserved
		0,				// u2DimmingFrequency60Hz
		0,				// u2DimmingFrequency50Hz

		3,				// u1LvdsOnDalay
		30, 			// u1BacklightOnDelay
		30, 			// u1BacklightOffDelay
		3,				// u1LvdsOffDalay

		30, 			// u2MaxOverscan
		0,				// u2Reserved
		},
#endif

#if ((PANEL_SELECT == PANEL_LG_42_WU3_HISPD_2CH) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    2 * 1080,           // u2PanelHeight

    4 * 77000000,   // u4PixelClkMax
    2 *148351648,      // u4PixelClk60Hz
    2 *123750000,      // u4PixelClk50Hz
    4 * 59630000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    2 * 1201,           // u2VTotalMax
    2 * 1125,           // u2VTotal60Hz
    2 * 1125,           // u2VTotal50Hz
    2 * 1091,           // u2VTotalMin
    120,             // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    2 * 3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | LVDS_HIGHSPEED| PANEL_L12R12,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_AUO_37_HW2) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 85000000,   // u4PixelClkMax
    149010989,      // u4PixelClk60Hz
    124300000,      // u4PixelClk50Hz
    2 * 65280000,   // u4PixelClkMin

    2360,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2000,           // u2HTotalMin

    1200,           // u2VTotalMax
    1130,           // u2VTotal60Hz
    1130,           // u2VTotal50Hz
    1088,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_POLARIZED_LRLR,
#else
	DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT,
#endif

    0xB0,           // u1BacklightHigh
    0xBC,           // u1BacklightMiddle
    0xC4,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    50,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    1,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC42FHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 90000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 50000000,   // u4PixelClkMin

    2500,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    1900,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1000,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_MLVDS,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV9) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    70000000,       // u4PixelClk60Hz
    70000000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1700,           // u2HTotalMax
    1500,           // u2HTotal60Hz
    1500,           // u2HTotal50Hz
    1300,           // u2HTotalMin

    1100,           // u2VTotalMax, Winton for Display Mode
    778,             // u2VTotal60Hz
    934,            // u2VTotal50Hz
    700,            // u2VTotalMin
    
    75,             // u1VClkMax
    48,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | OD_ON | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_SLT_FHD_60Hz_10BIT_AUTO) || CC_MULTI_PANEL)
    {
    1 * 1920,			// u2PanelWidth
	1 * 1080,			// u2PanelHeight

	2 * 77000000,	// u4PixelClkMax
	2 * 74250000,	   // u4PixelClk60Hz
	2 * 74250000,	   // u4PixelClk50Hz
	2 * 59630000,	// u4PixelClkMin

	1 * 2560,			// u2HTotalMax
	1 * 2200,			// u2HTotal60Hz
	1 * 2200,			// u2HTotal50Hz
	1 * 2120,			// u2HTotalMin

	1 * 1450,			// u2VTotalMax
	1 * 1125,			// u2VTotal60Hz
	1 * 1350,			// u2VTotal50Hz
	1 * 1091,			// u2VTotalMin
	63,			 // u1VClkMax
	47, 			// u1VClkMin
	
	30,             // u1HSyncWidth
	3,              // u1VSyncWidth
	0x810,          // u1HPosition
	0x3FA,          // u1VPosition
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_HIGH_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan
    ENGINEER_DISPLAY_MODE,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE_HE420FF) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 82000000,   // u4PixelClkMax
    148500000,  	// u4PixelClk60Hz
    123750000,	   	// u4PixelClk50Hz
    2 * 50000000,   // u4PixelClkMin

    2650,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2060,           // u2HTotalMin

    1480,           // u2VTotalMax (display mode)
    1125,           // u2VTotal60Hz
    1125,	       // u2VTotal50Hz
    1090,           // u2VTotalMin
    
    63,	       // u1VClkMax
    47,           // u1VClkMin

    30,             // u1HSyncWidth
    5,              // u1VSyncWidth
	0x810,			// u1HPosition
	0x400,			// u1VPosition
    
    DUAL_PORT | DISP_24BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |
    PWM_HIGH_PANEL_BRIGHT|DISP_TYPE_MLVDS,
    
    255,           // u1BacklightHigh
    191,           // u1BacklightMiddle
    64,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    6,//3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved	
    
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE32WXGA2P3P) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    75441600,       // u4PixelClk60Hz
    75426000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    967,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_18BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE32WXGA1P6P) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    82000000,       // u4PixelClkMax
    75441600,       // u4PixelClk60Hz
    75426000,       // u4PixelClk50Hz
    60000000,       // u4PixelClkMin

    1936,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1442,           // u2HTotalMin

    1063,           // u2VTotalMax
    806,            // u2VTotal60Hz
    967,            // u2VTotal50Hz
    778,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    60,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_MLVDS_CHINASTARWXGA8BIT) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    85000000,       // u4PixelClkMax
    75441600,       // u4PixelClk60Hz
    75426000,       // u4PixelClk50Hz
    65000000,       // u4PixelClkMin

    2000,           // u2HTotalMax
    1560,           // u2HTotal60Hz
    1560,           // u2HTotal50Hz
    1460,           // u2HTotalMin

    1015,           // u2VTotalMax
    806,             // u2VTotal60Hz
    967,            // u2VTotal50Hz
    784,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    50,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },

#endif

#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV12) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    80000000,       // u4PixelClkMax
    72354836,       // u4PixelClk60Hz
    68530000,       // u4PixelClk50Hz
    68000000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1528,           // u2HTotal60Hz
    1540,           // u2HTotal50Hz
    1416,           // u2HTotalMin

    1063,           // u2VTotalMax
    790,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_18BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | OD_ON | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    50,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },

#endif

#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV13) || CC_MULTI_PANEL)
    {
    1366,           // u2PanelWidth
    768,            // u2PanelHeight

    113273280,       // u4PixelClkMax
    85440000,       // u4PixelClk60Hz
    71200000,       // u4PixelClk50Hz
    54870000,       // u4PixelClkMin

    1776,           // u2HTotalMax
    1600,           // u2HTotal60Hz
    1600,           // u2HTotal50Hz
    1416,           // u2HTotalMin

    1063,           // u2VTotalMax
    890,            // u2VTotal60Hz
    890,            // u2VTotal50Hz
    775,            // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    16,             // u1HSyncWidth
    1,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    SINGLE_PORT | DISP_18BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | OD_ON | DISP_TYPE_MLVDS,

    0x60,           // (100%) u1BacklightHigh
    0x80,           // (67%) u1BacklightMiddle
    0xA0,           // (50%) u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    50,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    5,              // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },

#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV13GIPFHD60) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77500000,   // u4PixelClkMax
    296703297/2,      // u4PixelClk60Hz
    296703297/2,      // u4PixelClk50Hz
    2 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,            // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV13GIPFHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV14GIPFHD60_8LANE) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77500000,   // u4PixelClkMax
    296703297/2,      // u4PixelClk60Hz
    296703297/2,      // u4PixelClk50Hz
    2 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,            // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    EPI_LANE_8,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV14GIPFHD120_8LANE) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax
    296703297,      // u4PixelClk60Hz
    296703297,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1380,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    120,            // u1VClkMax
    60,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    FOUR_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    EPI_LANE_8,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_EPI_LGDV14GIPFHD60_4LANE) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77500000,   // u4PixelClkMax
    296703297/2,      // u4PixelClk60Hz
    296703297/2,      // u4PixelClk50Hz
    2 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2196,           // u2HTotal60Hz
    2196,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    1500,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,            // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_EPI,
#else
    DUAL_PORT | DISP_24BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_EPI,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    EPI_LANE_4,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT ==  PANEL_VB1_V420DK1_KS1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    2160,           // u2PanelHeight

    4 * 75000000,   // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    297000000,      // u4PixelClk50Hz
    4 * 66970000,   // u4PixelClkMin

    2720,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2640,           // u2HTotal50Hz
    2080,           // u2HTotalMin

    2850,           // u2VTotalMax
    2250,           // u2VTotal60Hz
    2250,           // u2VTotal50Hz
    1090,           // u2VTotalMin
    63,            // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_VB1,
#else
    FOUR_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_VB1,
#endif

    0x08,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    2,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    DISP_4K2K_TYPE_3 | SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | VB1_ALL_LANE_ON | DISP_4K2K_FORMAT_YUV444 | DISP_48HZ_TO_48HZ,           // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_4K1K_VB1) || CC_MULTI_PANEL)
	{
		3840,			// u2PanelWidth
		1080,			// u2PanelHeight
	
		4 * 77000000,	// u4PixelClkMax
		4 * 74250000,	   // u4PixelClk60Hz
		4 * 74250000,	   // u4PixelClk50Hz
		4 * 59630000,	// u4PixelClkMin
	
		2 * 2800,			// u2HTotalMax
		2 * 2200,			// u2HTotal60Hz
		2 * 2640,			// u2HTotal50Hz
		2120,			    // u2HTotalMin
	
		1 * 1200,			// u2VTotalMax
		1 * 1125,			// u2VTotal60Hz
		1 * 1125,			// u2VTotal50Hz
		1 * 1090,			// u2VTotalMin
		63,			 // u1VClkMax
		47, 			// u1VClkMin
	
		2 * 44, 			// u1HSyncWidth
		4,				// u1VSyncWidth
		0x8B0,			// u1HPosition
		0x3DC,			// u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
		OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
		OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT,
#endif

		0x70,			// u1BacklightHigh
		0x80,			// u1BacklightMiddle
		0x90,			// u1BacklightLow
		0,				// u1Reserved
		0,				// u2DimmingFrequency60Hz
		0,				// u2DimmingFrequency50Hz
	
		3,				// u1LvdsOnDalay
		30, 			// u1BacklightOnDelay
		30, 			// u1BacklightOffDelay
		3,				// u1LvdsOffDalay
	
		30, 			// u2MaxOverscan
		DISP_4K2K_TYPE_3 | SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | DISP_4K2K_FORMAT_YUV444 | DISP_48HZ_TO_48HZ,				// u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_THINE_THAV226_FHD60_VB1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 75000000,   // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    148500000,      // u4PixelClk50Hz
    2 * 74000000,   // u4PixelClkMin

    2560,           // u2HTotalMax
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1420,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1091,           // u2VTotalMin
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
    DUAL_PORT | DISP_30BIT | LVDS_MSB_SW_OFF | LVDS_ODD_SW_OFF |
    PWM_LOW_PANEL_BRIGHT | DISP_TYPE_VB1,

    0x70,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0x90,           // u1BacklightLow
    0,              // u1Reserved
    0,              // u2DimmingFrequency60Hz
    0,              // u2DimmingFrequency50Hz

    3,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    0,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_4K2K30_VB1) || CC_MULTI_PANEL)
    {
    2 * 1920,                 // u2PanelWidth
    2 * 1080,                 // u2PanelHeight

    4 * 77000000, // u4PixelClkMax
    4 * 74250000,    // u4PixelClk60Hz
    4 * 74250000,    // u4PixelClk50Hz
    4 * 59630000, // u4PixelClkMin

    5760,                 // u2HTotalMax
    2 * 2200,                 // u2HTotal60Hz
    2 * 2640,                 // u2HTotal50Hz
    3960,                 // u2HTotalMin

    2450,                 // u2VTotalMax
    2 * 1125,                 // u2VTotal60Hz
    2 * 1125,                 // u2VTotal50Hz
    2200,                 // u2VTotalMin
    31,                   // u1VClkMax
    23,                  // u1VClkMin

    30,                     // u1HSyncWidth
    3,                               // u1VSyncWidth
    0x810,                     // u1HPosition
    0x400,                     // u1VPosition
    FOUR_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |DISP_TYPE_VB1 |
    PWM_LOW_PANEL_BRIGHT,

    0x70,                       // u1BacklightHigh
    0x80,                       // u1BacklightMiddle
    0x90,                       // u1BacklightLow
    0,                             // u1Reserved
    0,                             // u2DimmingFrequency60Hz
    0,                             // u2DimmingFrequency50Hz

    3,                   // u1LvdsOnDalay
    30,                  // u1BacklightOnDelay
    30,                  // u1BacklightOffDelay
    3,                   // u1LvdsOffDalay

    30,                  // u2MaxOverscan
    DISABLE_LVDS_LINEBUFFER | DISP_48HZ_TO_48HZ ,                             // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_FHD_BACKEND) || CC_MULTI_PANEL)
{
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2*74498000,      // u4PixelClkMax
    2*74250000,      // u4PixelClk60Hz
    2*74250000,      // u4PixelClk50Hz
    2*74002000,      // u4PixelClkMin

    2560,           // u2HTotalMax
    2184,           // u2HTotal60Hz
    2184,           // u2HTotal50Hz
    2120,           // u2HTotalMin

    1420,           // u2VTotalMax
    1134,           // u2VTotal60Hz
    1360,           // u2VTotal50Hz
    1091,           // u2VTotalMin

    63,             // u1VClkMax
    47,             // u1VClkMin
	
    32,             // u1HSyncWidth
    4,              // u1VSyncWidth
    0x820,          // u1HPosition
    0x3EC,          // u1VPosition

    DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
    PWM_LOW_PANEL_BRIGHT,

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xE0,           // u1BacklightLow 
    0,              // u1Reserved
    160,            // u2DimmingFrequency60Hz
    160,            // u2DimmingFrequency50Hz

	15,             // u1LvdsOnDalay
	110,            // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan

    // u4ControlWord2
    DISP_2K_TYPE_1 | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
},
#endif

#if ((PANEL_SELECT == PANEL_AUO_4K2K) || CC_MULTI_PANEL)
		{
		2 * 1920,			// u2PanelWidth
		2 * 1080,			// u2PanelHeight
	
		4 * 77000000,	// u4PixelClkMax
		4 * 74250000,	   // u4PixelClk60Hz
		4 * 74250000,	   // u4PixelClk50Hz
		4 * 59630000,	// u4PixelClkMin
	
		2 * 2560,			// u2HTotalMax
		2 * 2200,			// u2HTotal60Hz
		2 * 2200,			// u2HTotal50Hz
		2 * 2120,			// u2HTotalMin
	
		2 * 1201,			// u2VTotalMax
		2 * 1125,			// u2VTotal60Hz
		2 * 1350,			// u2VTotal50Hz
		2 * 1091,			// u2VTotalMin
		63,			 // u1VClkMax
		47, 			// u1VClkMin
	
		2 * 44, 			// u1HSyncWidth
		2 * 5,				// u1VSyncWidth
		0x8B0,			// u1HPosition
		0x3B4,			// u1VPosition
		FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
		PWM_LOW_PANEL_BRIGHT,
	
		0x70,			// u1BacklightHigh
		0x80,			// u1BacklightMiddle
		0x90,			// u1BacklightLow
		0,				// u1Reserved
		0,				// u2DimmingFrequency60Hz
		0,				// u2DimmingFrequency50Hz
	
		3,				// u1LvdsOnDalay
		30, 			// u1BacklightOnDelay
		30, 			// u1BacklightOffDelay
		3,				// u1LvdsOffDalay
	
		30, 			// u2MaxOverscan
		DISABLE_LVDS_LINEBUFFER,				// u2Reserved
		},
#endif

#if ((PANEL_SELECT == PANEL_AUO_4K2K_FHD60) || CC_MULTI_PANEL)
		{
		1 * 1920,			// u2PanelWidth
		1 * 1080,			// u2PanelHeight
	
		2 * 77000000,	// u4PixelClkMax
		2 * 74250000,	   // u4PixelClk60Hz
		2 * 74250000,	   // u4PixelClk50Hz
		2 * 59630000,	// u4PixelClkMin
	
		1 * 2560,			// u2HTotalMax
		1 * 2200,			// u2HTotal60Hz
		1 * 2200,			// u2HTotal50Hz
		1 * 2120,			// u2HTotalMin
	
		1 * 1450,			// u2VTotalMax
		1 * 1125,			// u2VTotal60Hz
		1 * 1350,			// u2VTotal50Hz
		1 * 1091,			// u2VTotalMin
		63,			 // u1VClkMax
		47, 			// u1VClkMin
	
		1 * 44, 			// u1HSyncWidth
		1 * 5,				// u1VSyncWidth
		0x858,			// u1HPosition
		0x3DD,			// u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
		DUAL_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
		PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
		DUAL_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
		PWM_LOW_PANEL_BRIGHT,
#endif
	
		0x70,			// u1BacklightHigh
		0x80,			// u1BacklightMiddle
		0x90,			// u1BacklightLow
		0,				// u1Reserved
		0,				// u2DimmingFrequency60Hz
		0,				// u2DimmingFrequency50Hz
	
		3,				// u1LvdsOnDalay
		30, 			// u1BacklightOnDelay
		30, 			// u1BacklightOffDelay
		3,				// u1LvdsOffDalay
	
		30, 			// u2MaxOverscan
		DISP_4K2K_TYPE_2 | DISP_48HZ_TO_48HZ,				// u2Reserved
		},
#endif
#if ((PANEL_SELECT == PANEL_CMI_4K2K) || CC_MULTI_PANEL)
                {
                2 * 1920,                 // u2PanelWidth
                2 * 1080,                 // u2PanelHeight
        
                4 * 77000000, // u4PixelClkMax
                4 * 74250000,    // u4PixelClk60Hz
                4 * 74250000,    // u4PixelClk50Hz
                4 * 59630000, // u4PixelClkMin
        
                2 * 2560,                 // u2HTotalMax
                2 * 2200,                 // u2HTotal60Hz
                2 * 2200,                 // u2HTotal50Hz
                2 * 2120,                 // u2HTotalMin
        
                2 * 1201,                 // u2VTotalMax
                2 * 1125,                 // u2VTotal60Hz
                2 * 1125,                 // u2VTotal50Hz
                2 * 1091,                 // u2VTotalMin
                63,                   // u1VClkMax
                47,                  // u1VClkMin
        
                2 *30,                     // u1HSyncWidth
                2 * 3,                               // u1VSyncWidth
                0x810,                     // u1HPosition
                0x400,                     // u1VPosition
                FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF |
                LVDS_A_FROM_AO | LVDS_A_FROM_AE | LVDS_C_FROM_BE | LVDS_D_FROM_BO |
                PWM_LOW_PANEL_BRIGHT,
        
                0x70,                       // u1BacklightHigh
                0x80,                       // u1BacklightMiddle
                0x90,                       // u1BacklightLow
                0,                             // u1Reserved
                0,                             // u2DimmingFrequency60Hz
                0,                             // u2DimmingFrequency50Hz
        
                3,                             // u1LvdsOnDalay
                30,                  // u1BacklightOnDelay
                30,                  // u1BacklightOffDelay
                3,                             // u1LvdsOffDalay
        
                30,                  // u2MaxOverscan
                DISABLE_LVDS_LINEBUFFER ,                             // u2Reserved
                },
#endif

#if ((PANEL_SELECT == PANEL_CMI_4K2K_FHD120) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax, 4 * 75000000
    2 * 148500000,      // u4PixelClk60Hz, 296703297
    2 * 148500000,      // u4PixelClk50Hz, 296703297
    4 * 60000000,   // u4PixelClkMin, 4 * 66970000

    2680,           // u2HTotalMax, 2720
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin, 2080

    1395,           // u2VTotalMax, 1380
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1100,           // u2VTotalMin, 1090
    123,            // u1VClkMax, 120
    97,             // u1VClkMin, 60

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_NS| LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xE0,           // u1BacklightLow
    0,              // u1Reserved
    160,              // u2DimmingFrequency60Hz
    160,              // u2DimmingFrequency50Hz

    20,              // u1LvdsOnDalay
    80,             // u1BacklightOnDelay
    20,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    DISP_4K2K_TYPE_1 | DISP_48HZ_TO_48HZ,              // u2Reserved
	},
#endif
#if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax, 4 * 75000000
    2 * 148500000,      // u4PixelClk60Hz, 296703297
    2 * 148500000,      // u4PixelClk50Hz, 296703297
    4 * 60000000,   // u4PixelClkMin, 4 * 66970000

    2680,           // u2HTotalMax, 2720
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin, 2080

    1395,           // u2VTotalMax, 1380
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1100,           // u2VTotalMin, 1090
    123,            // u1VClkMax, 120
    97,             // u1VClkMin, 60

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_NS| LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    FOUR_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
#endif

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xE0,           // u1BacklightLow 
    0,              // u1Reserved
    160,              // u2DimmingFrequency60Hz
    160,              // u2DimmingFrequency50Hz

    15,              // u1LvdsOnDalay
    110,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    DISP_4K2K_TYPE_1 | DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_CSOT_4K2K_FHD60) || CC_MULTI_PANEL)
    {
	1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    2 * 77500000,   // u4PixelClkMax
    148500000,      // u4PixelClk60Hz
    148500000,      // u4PixelClk50Hz 
    2 * 60000000,   // u4PixelClkMin

    2680,           // u2HTotalMax, 2720
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin
    
    1395,           // u2VTotalMax, 1380
    1125,           // u2VTotal50Hz
    1350,           // u2VTotal50Hz 
    1100,           // u2VTotalMin, 1090
    63,             // u1VClkMax
    47,             // u1VClkMin

    30,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x400,          // u1VPosition
#ifndef SUPPORT_NTK_DEMO_BOARD
	#ifdef CC_SCPOS_3DTV_SUPPORT
	DUAL_PORT | DISP_30BIT | LVDS_NS| LVDS_ODD_SW_OFF | 
     PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
	#else
    DUAL_PORT | DISP_30BIT | LVDS_NS | LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
	#endif
#else
	#ifdef CC_SCPOS_3DTV_SUPPORT
	DUAL_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | 
     PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
	#else
    DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT,
	#endif
#endif

    0x00,           // u1BacklightHigh (Need Norman to confirm)
    0x80,           // u1BacklightMiddle
    0xC0,           // u1BacklightLow
    0,              // u1Reserved
    160,              // u2DimmingFrequency60Hz
    160,              // u2DimmingFrequency50Hz

#ifndef SUPPORT_NTK_DEMO_BOARD
    15,              // u1LvdsOnDalay
#else
	1,               // u1LvdsOnDalay
#endif
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay 
    2,              // u1LvdsOffDalay

    50,             // u2MaxOverscan
    DISP_4K2K_TYPE_2 | DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif

#if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_VB1) || CC_MULTI_PANEL)
    {
    1920,           // u2PanelWidth
    1080,           // u2PanelHeight

    4 * 77500000,   // u4PixelClkMax, 4 * 75000000
    2 * 148500000,      // u4PixelClk60Hz, 296703297
    2 * 148500000,      // u4PixelClk50Hz, 296703297
    4 * 60000000,   // u4PixelClkMin, 4 * 66970000

    2680,           // u2HTotalMax, 2720
    2200,           // u2HTotal60Hz
    2200,           // u2HTotal50Hz
    2120,           // u2HTotalMin, 2080

    1395,           // u2VTotalMax, 1380
    1125,           // u2VTotal60Hz
    1350,           // u2VTotal50Hz
    1100,           // u2VTotalMin, 1090
    123,            // u1VClkMax, 120
    97,             // u1VClkMin, 60

    44,             // u1HSyncWidth
    3,              // u1VSyncWidth
    0x810,          // u1HPosition
    0x3FA,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
	FOUR_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER | DISP_TYPE_VB1,
#else
    FOUR_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | 
    WFB_PANEL_RESET_ON | PWM_LOW_PANEL_BRIGHT | DISP_TYPE_VB1,
#endif

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xE0,           // u1BacklightLow 
    0,              // u1Reserved
    160,              // u2DimmingFrequency60Hz
    160,              // u2DimmingFrequency50Hz

    15,              // u1LvdsOnDalay
    30,             // u1BacklightOnDelay
    30,             // u1BacklightOffDelay
    2,              // u1LvdsOffDalay

    30,             // u2MaxOverscan
    DISP_4K2K_TYPE_1 | DISPR_INVERSE | DISP_48HZ_TO_48HZ,              // u2Reserved
    },
#endif
#if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_60_VB1) || CC_MULTI_PANEL)
		{
		1 * 1920,			// u2PanelWidth
		1 * 1080,			// u2PanelHeight
	
		4 * 77000000,	   // u4PixelClkMax
		2 * 74250000,	   // u4PixelClk60Hz
		2 * 74250000,	   // u4PixelClk50Hz
		2 * 59630000,	   // u4PixelClkMin
	
		5760,			// u2HTotalMax, 1 * 2560
		1 * 2200,			// u2HTotal60Hz
		1 * 2200,			// u2HTotal50Hz
		1 * 2120,			// u2HTotalMin
	
		2450,			// u2VTotalMax, 1 * 1450
		1 * 1125,			// u2VTotal60Hz
		1 * 1350,			// u2VTotal50Hz
		1 * 1091,			// u2VTotalMin
		63,			 // u1VClkMax
		47, 			// u1VClkMin
	
		
		44,             // u1HSyncWidth
		3,              // u1VSyncWidth
		0x810,          // u1HPosition
		0x3FA,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
		DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
		DUAL_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT,
#endif
	
		0x00,           // u1BacklightHigh
		0x80,           // u1BacklightMiddle
		0xE0,           // u1BacklightLow 
		0,              // u1Reserved
		160,              // u2DimmingFrequency60Hz
		160,              // u2DimmingFrequency50Hz

		15,              // u1LvdsOnDalay
		30,             // u1BacklightOnDelay
		30,             // u1BacklightOffDelay
		2,              // u1LvdsOffDalay

		30,             // u2MaxOverscan

		DISP_4K2K_TYPE_2 | DISPR_INVERSE | DISP_48HZ_TO_48HZ,				// u2Reserved
		},
#endif
#if ((PANEL_SELECT == PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST) || CC_MULTI_PANEL)
{
		1 * 1920,			// u2PanelWidth
		1 * 1080,			// u2PanelHeight
	
		4 * 77000000,	   // u4PixelClkMax
		2 * 74250000,	   // u4PixelClk60Hz
		2 * 74250000,	   // u4PixelClk50Hz
		2 * 59630000,	   // u4PixelClkMin
	
		5760,			// u2HTotalMax, 1 * 2560
		1 * 2200,			// u2HTotal60Hz
		1 * 2200,			// u2HTotal50Hz
		1 * 2120,			// u2HTotalMin
	
		2450,			// u2VTotalMax, 1 * 1450
		1 * 1125,			// u2VTotal60Hz
		1 * 1350,			// u2VTotal50Hz
		1 * 1091,			// u2VTotalMin
		63,			 // u1VClkMax
		47, 			// u1VClkMin
	
		
		44,             // u1HSyncWidth
		3,              // u1VSyncWidth
		0x810,          // u1HPosition
		0x3FA,          // u1VPosition
#ifdef CC_SCPOS_3DTV_SUPPORT
		DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
		DUAL_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
		PWM_LOW_PANEL_BRIGHT,
#endif
	
		0x00,           // u1BacklightHigh
		0x80,           // u1BacklightMiddle
		0xE0,           // u1BacklightLow 
		0,              // u1Reserved
		160,              // u2DimmingFrequency60Hz
		160,              // u2DimmingFrequency50Hz

		15,              // u1LvdsOnDalay
		30,             // u1BacklightOnDelay
		30,             // u1BacklightOffDelay
		2,              // u1LvdsOffDalay

		30,             // u2MaxOverscan

		DISP_4K2K_TYPE_4 | DISPR_INVERSE | DISP_48HZ_TO_48HZ,				// u2Reserved
		},
#endif

#if ((PANEL_SELECT == PANEL_4K_BACKEND) || CC_MULTI_PANEL)
{
    3840,           // u2PanelWidth
    1080,           // u2PanelHeight

    297992000,      // u4PixelClkMax
    297000000,      // u4PixelClk60Hz
    297000000,      // u4PixelClk50Hz
    296008000,      // u4PixelClkMin

    5500,           // u2HTotalMax
    4400,           // u2HTotal60Hz
    5280,           // u2HTotal50Hz
    2196,           // u2HTotalMin

    2253,           // u2VTotalMax
    1125,           // u2VTotal60Hz
    1125,           // u2VTotal50Hz
    1124,           // u2VTotalMin

    63,             // u1VClkMax
    47,             // u1VClkMin
	
    88,             // u1HSyncWidth
    4,              // u1VSyncWidth
    0x8B0,          // u1HPosition
    0x3DC,          // u1VPosition

#ifdef CC_SCPOS_3DTV_SUPPORT
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
    PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
    OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
    PWM_LOW_PANEL_BRIGHT,
#endif

    0x00,           // u1BacklightHigh
    0x80,           // u1BacklightMiddle
    0xE0,           // u1BacklightLow 
    0,              // u1Reserved
    160,            // u2DimmingFrequency60Hz
    160,            // u2DimmingFrequency50Hz

	3,             // u1LvdsOnDalay
	30,            // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	3,              // u1LvdsOffDalay

    30,             // u2MaxOverscan

    // u4ControlWord2
   DISP_4K2K_TYPE_3 | SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | DISP_4K2K_FORMAT_YUV444 | DISP_48HZ_TO_48HZ,
},
#endif

#if ((PANEL_SELECT == PANEL_4K2KDD) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	8 * 77000000,	   // u4PixelClkMax
	8 * 74250000,	   // u4PixelClk60Hz
	8 * 74250000,	   // u4PixelClk50Hz
	8 * 59630000,	   // u4PixelClkMin

	5760,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin
	63,			 // u1VClkMax
	47, 			// u1VClkMin

	88,             // u1HSyncWidth
	6,              // u1VSyncWidth
	0x820,          // u1HPosition
	0x3F4,          // u1VPosition

#if 0  //#ifdef CC_SCPOS_3DTV_SUPPORT
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,
#endif

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan

#ifdef CC_SUPPORT_UHD
	DISP_4K2K_TYPE_5 |SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,				// u2Reserved
#else
	VB1_ALL_LANE_ON | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
#endif
},
#endif

#if ((PANEL_SELECT == PANEL_4K2KDD_2SEC) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	8 * 77000000,	   // u4PixelClkMax
	8 * 74250000,	   // u4PixelClk60Hz
	8 * 74250000,	   // u4PixelClk50Hz
	8 * 59630000,	   // u4PixelClkMin

	5760,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin
	63,			 // u1VClkMax
	47, 			// u1VClkMin

	
	88,             // u1HSyncWidth
	6,              // u1VSyncWidth
	0x820,          // u1HPosition
	0x3F4,          // u1VPosition

#if 0  //#ifdef CC_SCPOS_3DTV_SUPPORT
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 | VB1_2SEC_EN |
	PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 | VB1_2SEC_EN |
	PWM_LOW_PANEL_BRIGHT,
#endif

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan   

#ifdef CC_SUPPORT_UHD
	DISP_4K2K_TYPE_5 |SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,				// u2Reserved
#else
	VB1_ALL_LANE_ON | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
#endif
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_4K2KDD_GRAPH) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	8 * 77000000,	   // u4PixelClkMax
	8 * 74250000,	   // u4PixelClk60Hz
	8 * 74250000,	   // u4PixelClk50Hz
	8 * 59630000,	   // u4PixelClkMin

	5760,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin
	63,			 // u1VClkMax
	47, 			// u1VClkMin

	
	88,             // u1HSyncWidth
	6,              // u1VSyncWidth
	0x820,          // u1HPosition
	0x3F4,          // u1VPosition

#if 0  //#ifdef CC_SCPOS_3DTV_SUPPORT
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,
#endif

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	120,              // u2DimmingFrequency60Hz
	100,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan   

#ifdef CC_SUPPORT_UHD
	DISP_4K2K_TYPE_5 |SUPPORT_VB1_GRAPH_MODE | SUPPORT_UHD_OSD_SEPARATE | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ , 			// u2Reserved
#else
	VB1_ALL_LANE_ON | SUPPORT_VB1_GRAPH_MODE | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
#endif
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_480P) || CC_MULTI_PANEL)
{
	720,			// u2PanelWidth
	480,			// u2PanelHeight

	30000000,		// u4PixelClkMax
	27003000,		// u4PixelClk60Hz
	27003000,		// u4PixelClk50Hz
	13500000,		// u4PixelClkMin

	900,			// u2HTotalMax
	858,			// u2HTotal60Hz
	858,			// u2HTotal50Hz
	800,			// u2HTotalMin

	680,			// u2VTotalMax
	525,			// u2VTotal60Hz
	630,			// u2VTotal50Hz
	500,			// u2VTotalMin
	63, 			// u1VClkMax
	47, 			// u1VClkMin

	62, 			// u1HSyncWidth
	6,				// u1VSyncWidth
	0x810,			// u1HPosition
	0x3E2, // 0x3DC,			// u1VPosition
	
	SINGLE_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	FT_HALF_STEP,				// u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_720P) || CC_MULTI_PANEL)
{
	1280,			// u2PanelWidth
	720,			// u2PanelHeight

	78700000,		// u4PixelClkMax
	74250000,		// u4PixelClk60Hz
	74250000,		// u4PixelClk50Hz
	69795000,		// u4PixelClkMin

	2000,			// u2HTotalMax
	1650,			// u2HTotal60Hz
	1980,			// u2HTotal50Hz
	1400,			// u2HTotalMin

	800,			// u2VTotalMax
	750,			// u2VTotal60Hz
	750,			// u2VTotal50Hz
	500,			// u2VTotalMin
	63, 			// u1VClkMax
	47, 			// u1VClkMin

	40, 			// u1HSyncWidth
	5,				// u1VSyncWidth
	0x86E,			// u1HPosition
	0x3ED,			// u1VPosition
	
	SINGLE_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	FT_HALF_STEP,				// u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_1080P) || CC_MULTI_PANEL)
{
	1920,			// u2PanelWidth
	1080,			// u2PanelHeight

	2 * 77500000,	// u4PixelClkMax
	148500000,		// u4PixelClk60Hz
	148500000,		// u4PixelClk50Hz
	2 * 66970000,	// u4PixelClkMin

	2800,			// u2HTotalMax
	2200,			// u2HTotal60Hz
	2640,			// u2HTotal50Hz
	2080,			// u2HTotalMin

	1406,			// u2VTotalMax (display mode)
	1125,			// u2VTotal60Hz
	1125,			// u2VTotal50Hz
	1089,			// u2VTotalMin
	63,			// u1VClkMax
	47, 			// u1VClkMin...spec not defined

	44, 			// u1HSyncWidth
	3,				// u1VSyncWidth
	0x810,			// u1HPosition
	0x3FA,			// u1VPosition

	DUAL_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	0,				// u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_1080P24) || CC_MULTI_PANEL)
{
	1920,			// u2PanelWidth
	1080,			// u2PanelHeight

	118104000,	    // u4PixelClkMax
	74250000,		// u4PixelClk60Hz
	74250000,		// u4PixelClk50Hz
	54362880,	    // u4PixelClkMin

	2800,			// u2HTotalMax
	2200,			// u2HTotal60Hz
	2640,			// u2HTotal50Hz
	2080,			// u2HTotalMin

	1406,			// u2VTotalMax (display mode)
	1125,			// u2VTotal60Hz
	1125,			// u2VTotal50Hz
	1089,			// u2VTotalMin
	
	33,			    // u1VClkMax
	23, 			// u1VClkMin...spec not defined

	44, 			// u1HSyncWidth
	3,				// u1VSyncWidth
	0x810,			// u1HPosition
	0x3FA,			// u1VPosition

	SINGLE_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	FT_HALF_STEP,				// u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_576P) || CC_MULTI_PANEL)
{
	720,			// u2PanelWidth
	576,			// u2PanelHeight

	30000000,		// u4PixelClkMax
	27000000,		// u4PixelClk60Hz
	27000000,		// u4PixelClk50Hz
	13500000,		// u4PixelClkMin

	900,			// u2HTotalMax
	864,			// u2HTotal60Hz
	864,			// u2HTotal50Hz
	800,			// u2HTotalMin

	680,			// u2VTotalMax
	625,			// u2VTotal60Hz
	625,			// u2VTotal50Hz
	500,			// u2VTotalMin
	63, 			// u1VClkMax
	47, 			// u1VClkMin

	64, 			// u1HSyncWidth
	5,				// u1VSyncWidth
	0x80C,			// u1HPosition
	0x3D4, 			// u1VPosition
	
	SINGLE_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x70,			// u1BacklightHigh
	0x80,			// u1BacklightMiddle
	0x90,			// u1BacklightLow
	0,				// u1Reserved
	0,				// u2DimmingFrequency60Hz
	0,				// u2DimmingFrequency50Hz

	3,				// u1LvdsOnDalay
	30, 			// u1BacklightOnDelay
	30, 			// u1BacklightOffDelay
	3,				// u1LvdsOffDalay

	30, 			// u2MaxOverscan
	FT_HALF_STEP,				// u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_2160P) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	8 * 77000000,	   // u4PixelClkMax
	8 * 74250000,	   // u4PixelClk60Hz
	8 * 74250000,	   // u4PixelClk50Hz
	8 * 59630000,	   // u4PixelClkMin

	5760,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin

	63,			 // u1VClkMax
	47, 			// u1VClkMin

	88,             // u1HSyncWidth
	10,              // u1VSyncWidth
	0x8B0,          // u1HPosition
	0x3AE,          // u1VPosition

	OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan

	DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_2160P30) || CC_MULTI_PANEL)
{
    2 * 1920,                 // u2PanelWidth
    2 * 1080,                 // u2PanelHeight

    4 * 77000000, // u4PixelClkMax
    4 * 74250000,    // u4PixelClk60Hz
    4 * 74250000,    // u4PixelClk50Hz
    4 * 59630000, // u4PixelClkMin

    5760,                 // u2HTotalMax
    2 * 2200,                 // u2HTotal60Hz
    2 * 2640,                 // u2HTotal50Hz
    3960,                 // u2HTotalMin

    2450,                 // u2VTotalMax
    2 * 1125,                 // u2VTotal60Hz
    2 * 1125,                 // u2VTotal50Hz
    2190,                 // u2VTotalMin
    31,                   // u1VClkMax
    23,                  // u1VClkMin

    88,                     // u1HSyncWidth
    10,                               // u1VSyncWidth
    0x8B0,                     // u1HPosition
    0x3AE,                     // u1VPosition
    
    FOUR_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF |DISP_TYPE_VB1 |
    PWM_LOW_PANEL_BRIGHT,

    0x70,                       // u1BacklightHigh
    0x80,                       // u1BacklightMiddle
    0x90,                       // u1BacklightLow
    0,                             // u1Reserved
    0,                             // u2DimmingFrequency60Hz
    0,                             // u2DimmingFrequency50Hz

    3,                   // u1LvdsOnDalay
    30,                  // u1BacklightOnDelay
    30,                  // u1BacklightOffDelay
    3,                   // u1LvdsOffDalay

    30,                  // u2MaxOverscan

	DISABLE_LVDS_LINEBUFFER | DISP_48HZ_TO_48HZ ,   // u2Reserved
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_2160P_2SEC) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	8 * 77000000,	   // u4PixelClkMax
	8 * 74250000,	   // u4PixelClk60Hz
	8 * 74250000,	   // u4PixelClk50Hz
	8 * 59630000,	   // u4PixelClkMin

	5760,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin

	63,			 // u1VClkMax
	47, 			// u1VClkMin

	88,             // u1HSyncWidth
	10,              // u1VSyncWidth
	0x8B0,          // u1HPosition
	0x3AE,          // u1VPosition

	OCTA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 | VB1_2SEC_EN |
	PWM_LOW_PANEL_BRIGHT,

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan

	DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_4K2K120) || CC_MULTI_PANEL)
{
	3840,			// u2PanelWidth
	2160,			// u2PanelHeight

	16 * 77000000,	   // u4PixelClkMax
	16 * 74250000,	   // u4PixelClk60Hz
	16 * 74250000,	   // u4PixelClk50Hz
	16 * 59630000,	   // u4PixelClkMin

#if 1
	5760,			// u2HTotalMax
	4400,			// u2HTotal60Hz
	5280,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2450,			// u2VTotalMax
	2250,			// u2VTotal60Hz
	2250,			// u2VTotal50Hz
	2182,			// u2VTotalMin
#else	
	4500,			// u2HTotalMax, 1 * 2560
	4400,			// u2HTotal60Hz
	4400,			// u2HTotal50Hz
	4240,			// u2HTotalMin

	2850,			// u2VTotalMax, 1 * 1450
	2250,			// u2VTotal60Hz
	2700,			// u2VTotal50Hz
	2182,			// u2VTotalMin
#endif

	120,			// u1VClkMax
	60, 			// u1VClkMin

	88,             // u1HSyncWidth
	6,              // u1VSyncWidth
	0x820,          // u1HPosition
	0x3F4,          // u1VPosition

#if 0  //#ifdef CC_SCPOS_3DTV_SUPPORT
	HEXA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	HEXA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,
#endif

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan

	VB1_ALL_LANE_ON | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
},
#endif

#if ((PANEL_SELECT == PANEL_VB1_8K4K30) || CC_MULTI_PANEL)
{
	3840*2,			// u2PanelWidth
	2160*2,			// u2PanelHeight

	16 * 77000000,	   // u4PixelClkMax
	16 * 74250000,	   // u4PixelClk60Hz
	16 * 74250000,	   // u4PixelClk50Hz
	16 * 59630000,	   // u4PixelClkMin

	5760*2,			// u2HTotalMax
	4400*2,			// u2HTotal60Hz
	5280*2,			// u2HTotal50Hz
	4240*2,			// u2HTotalMin

	2450*2,			// u2VTotalMax
	2250*2,			// u2VTotal60Hz
	2250*2,			// u2VTotal50Hz
	2182*2,			// u2VTotalMin

	31,			    // u1VClkMax
	23, 			// u1VClkMin

	88*2,           // u1HSyncWidth
	6*2,            // u1VSyncWidth
	0x840,          // u1HPosition
	0x3EE,          // u1VPosition

#if 0  //#ifdef CC_SCPOS_3DTV_SUPPORT
	HEXA_PORT | DISP_30BIT | LVDS_JEIDA | LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT | LVDS_DISP_3D_SHUTTER,
#else
	HEXA_PORT | DISP_30BIT | LVDS_JEIDA| LVDS_ODD_SW_OFF | DISP_TYPE_VB1 |
	PWM_LOW_PANEL_BRIGHT,
#endif

	0x00,           // u1BacklightHigh
	0x80,           // u1BacklightMiddle
	0xE0,           // u1BacklightLow 
	0,              // u1Reserved
	160,              // u2DimmingFrequency60Hz
	160,              // u2DimmingFrequency50Hz

	15,              // u1LvdsOnDalay
	30,             // u1BacklightOnDelay
	30,             // u1BacklightOffDelay
	2,              // u1LvdsOffDalay

	30,             // u2MaxOverscan

	VB1_ALL_LANE_ON | DISABLE_LVDS_LINEBUFFER | DISPR_INVERSE | DISP_48HZ_TO_48HZ,
},
#endif
};
#else
static PANEL_ATTRIBUTE_T _arPanelAttribute[] =
{
#if ((PANEL_SELECT == PANEL_XGA) || CC_MULTI_PANEL) // refer to CLAA150XP03
    {
        1024,           // u2PanelWidth
        768,            // u2PanelHeight

        74000000,       // u4PixelClkMax
        64930909,       // u4PixelClk60Hz
        54163200,       // u4PixelClk50Hz
        44000000,       // u4PixelClkMin

        1522,           // u2HTotalMax
        1344,           // u2HTotal60Hz
        1344,           // u2HTotal50Hz
        1214,           // u2HTotalMin

        888,            // u2VTotalMax
        806,            // u2VTotal60Hz
        806,            // u2VTotal50Hz
        789,            // u2VTotalMin
        62,             // u1VClkMax
        48,             // u1VClkMin

        0x87,         // u1HSyncWidth
        5,            // u1VSyncWidth
        0x818,        // u1HPosition
        0x3E1,        // u1VPosition
        SINGLE_PORT | DISP_24BIT | LVDS_MSB_SW_ON | LVDS_ODD_SW_OFF |
        PWM_HIGH_PANEL_BRIGHT,

        0xFF,           // (100%) u1BacklightHigh
        0x68,           // (84%) u1BacklightMiddle
        0x00,           // (68%) u1BacklightLow
        0,              // u1Reserved
        0,              // u2DimmingFrequency60Hz
        0,              // u2DimmingFrequency50Hz

        7,              // u1LvdsOnDalay
        80,             // u1BacklightOnDelay
        21,             // u1BacklightOffDelay
        6,              // u1LvdsOffDalay

        50,             // u2MaxOverscan
        0,              // u2Reserved
    },
#endif
};
#endif

static PANEL_LOCAL_DIMMING_ATTRIBUTE_T _arPanelLDAttribute[]=
{
  #if ((PANEL_SELECT == PANEL_EPI_LGDV12GIPFHD120) || CC_MULTI_PANEL)
	  {
        PANEL_EPI_LGDV12GIPFHD120,
        INCH_42,
        BL_EDGE_LED,    			
        E_SPI_LGD_PROTOCOL,                 // UINT8  u1ProtocolIndex;
        SV_ON,                             // UINT8 u1SWModeEn;
        SV_OFF,                             // UINT8  u1ScanningOnoff;
        0,

        // clock configuration
        600000,                             // UINT32 u4OutputClock;      // SPI output clock , unit is Hz
        CPOL_OFF,                           // UINT8  u1CPOL;             // clock polarity
        CPHA_OFF,                           // UINT8  u1CPHA;             // clock phase

        // Vsync access timing control
        540,                                 // UINT8  u1VSyncHighTime;    // Vsync high time ,unit is us
        540,                                 // UINT8  u1WaitTime;         // wait time from Vsync low to CS low or data , unit is us

        // CS access timing control
        14800,                              // UINT16 u2CSHighTime;       // output CS hold time, unit is ns
        14,                                 // UINT8  u1Tcld;             // setup time from CS low to SCLK edge , unit is ns
        14,                                 // UINT8  u1Tclg;             // hold time from SCLK edge to CS high, unit is ns
        SV_OFF,                             // UINT8  u1HheadSingleEn;    // CS setting
        SV_OFF,                             // UINT8  u1HdataSingleEn;
        SV_OFF,                             // UINT8  u1HfootSingleEn;
        SV_OFF,                             // UINT8  u1VheadSingleEn;
        SV_ON,                              // UINT8  u1VdataSingleEn;
        SV_OFF,                             // UINT8  u1VfootSingleEn;

        // LED output format
        LD_FMT_DEPTH_8BIT,                  // UINT8 u1BitMode;           // bits in dimming data, 0:8bit , 1:10bit , 2:12bit ,  3:7bit

        0,     //UINT16 u2DimmingLowBound;
        995,                                // UINT16 u2HistMaxThd;         // set histogram threshold to 99.5%
        2,     //H block number
        8,     //V block number
#ifdef CC_MT5399
/*
        PANEL_LIGHT_DIR_DIRECT=0,
        PANEL_LIGHT_DIR_LR=1,
        PANEL_LIGHT_DIR_TB=2,
        PANEL_LIGHT_DIR_L=3,
        PANEL_LIGHT_DIR_R=4,    
        PANEL_LIGHT_DIR_T=5,
        PANEL_LIGHT_DIR_B=6,  
*/
        1,

       LDM_SCANNING_ON,            //bit1  LDM_SCANNING_OFF (0)            LDM_SCANNING_ON (1)
       LDM_CURRENT_BOOST_OFF,  //bit2  LDM_CURRENT_BOOST_OFF (0)   LDM_CURRENT_BOOST_ON (1)
       LDM_V2B_PANEL,              //bit6  LDM_V2B_PANEL (0)               LDM_H1B_PANEL(1)
       LDM_REVERSE_OFF,             //bit7  LDM_REVERSE_OFF (0)             LDM_REVERSE_ON(1)      
#endif
	   },
  #endif
};


static PANEL_MINILVDS_ATTRIBUTE_T _arPanelMiniLVDSAttribute[]=
{
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV4FHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV4FHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
		   TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV4FHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV4FHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV4GIPFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV4GIPFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV4GIPFHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV4GIPFHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV5GIPFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV5GIPFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV5GIPFHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV5GIPFHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV6GIPFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV6GIPFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_LGDV6GIPFHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_LGDV6GIPFHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_AUOFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_AUOFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_AUOFHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_AUOFHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_SHAFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_SHAFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_SHAFHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_SHAFHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_FUNAWXGA) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_FUNAWXGA,
		   SINGLE_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_FUNAFHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_FUNAFHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_HIWXGA09) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_HIWXGA09,
		   SINGLE_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_HIWXGA10) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_HIWXGA10,
		   SINGLE_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_WXGA) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_WXGA,
		   SINGLE_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
   #endif
  #if ((PANEL_SELECT == PANEL_MLVDS_CMO2011FHD60) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_CMO2011FHD60,
		   DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
		 },
  #endif
#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC32WXGA1P6P) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_PANASONIC32WXGA1P6P,
		   SINGLE_PORT,
		   TCLK_PCLK_RATION_2,
           TCON_TCLK_RATION_1,
		 },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_CMO2011FHD120) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_CMO2011FHD120,
		 FOUR_PORT,
		 TCLK_PCLK_RATION_1,
         TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_SS_MB4) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_SS_MB4,
		 DUAL_PORT,
		 TCLK_PCLK_RATION_1,
         TCON_TCLK_RATION_1,
	   },
#endif
  #if ((PANEL_SELECT == PANEL_MLVDS_SS_MB7) || CC_MULTI_PANEL)
	     {
	       PANEL_MLVDS_SS_MB7,
	       DUAL_PORT,
		   TCLK_PCLK_RATION_1,
           TCON_TCLK_RATION_1,
	     },
  #endif
#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC32WXGA2P3P) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_PANASONIC32WXGA2P3P,
		 SINGLE_PORT,
		 TCLK_PCLK_RATION_2,
         TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_PANASONIC42FHD120) || CC_MULTI_PANEL)
		 {
		   PANEL_MLVDS_PANASONIC42FHD120,
		   FOUR_PORT,
		   TCLK_PCLK_RATION_0P75,
           TCON_TCLK_RATION_2,
		 },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV9) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_LGD32WXGAV9,
		 SINGLE_PORT,
		 TCLK_PCLK_RATION_2,
		 TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE_HE420FF) || CC_MULTI_PANEL)
	   {
		  PANEL_MLVDS_HISENSE_HE420FF,
		  DUAL_PORT,
		  TCLK_PCLK_RATION_2,
		  TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE32WXGA2P3P) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_HISENSE32WXGA2P3P,
		 DUAL_PORT,
		 TCLK_PCLK_RATION_1P5,
         TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_HISENSE32WXGA1P6P) || CC_MULTI_PANEL)
	   {
		 PANEL_MLVDS_HISENSE32WXGA1P6P,
		 SINGLE_PORT,
		 TCLK_PCLK_RATION_2,
         TCON_TCLK_RATION_1,
	   },
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_CHINASTARWXGA8BIT) || CC_MULTI_PANEL)
	   {
		  PANEL_MLVDS_CHINASTARWXGA8BIT,
		  SINGLE_PORT,
		  TCLK_PCLK_RATION_2, 
          TCON_TCLK_RATION_1,
		},
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV12) || CC_MULTI_PANEL)
		{
		  PANEL_MLVDS_LGD32WXGAV12,
		  SINGLE_PORT,
		  TCLK_PCLK_RATION_3, 
          TCON_TCLK_RATION_1,
		},
#endif
#if ((PANEL_SELECT == PANEL_MLVDS_LGD32WXGAV13) || CC_MULTI_PANEL)
		{
		  PANEL_MLVDS_LGD32WXGAV13,
		  SINGLE_PORT,
		  TCLK_PCLK_RATION_3, 
          TCON_TCLK_RATION_1,
		},
#endif

};



#ifdef CC_PANEL_FROM_FLASH

static UINT8 au1PeakLumaData[3] = {0,0,0}; // Max/Avg/Min  offset A3/A4/A5

UINT8 au1FlashPanelTableVersion[PANEL_VERSION_LENGTH + 1];
PANEL_HEADER_CUSTOM_ATTRIBUTE_T rFlashPanelHeader;
PANEL_Ext2_CUSTOM_ATTRIBUTE_T rFlashPanelExt2;
GRAPHMODE_ATTRIBUTE_T rVideo[PNL_VDO_ATTR];
GRAPHMODE_ATTRIBUTE_T rGraphic[PNL_GFX_ATTR];
#define BL_CTRL_IF_MASK        (7 << 8)
#define BL_CTRL_OLED           (6<< 28)
#define BE_TYPE_MASK           (7 << 28)
    #define BE_TYPE_NO_FRC         (0<< 28)
    #define BE_TYPE_FRC_60HZ       (1<< 28)
    #define BE_TYPE_FRC_120HZ      (2<< 28)
    #define BE_TYPE_4KBE_120_240HZ (3<< 28)
    #define BE_TYPE_4KBE_120_120HZ (4<< 28)
    #define BE_TYPE_4KBE_60HZ      (5<< 28)
    #define BE_TYPE_4KBE_8KBE      (6<< 28)
    #define BE_TYPE_INTERNAL_FRC   (7<< 28)

#define PANEL_FLASH_HEADER_SIZE (16 + 4 + 12) // Version, Panel num, reserve.

UINT8 au1PanelAttr ;
#ifdef CC_FLIP_MIRROR_SUPPORT
#include "drv_display.h"
#endif
UINT32 FlashPanel_StringCmp(CHAR *pSrc, CHAR *pDes)
{
    UINT32 idx,srcidx=0,desidx=0;
	UINT32 ui4Ret=0;

    for (idx=0; idx<PANEL_VERSION_LENGTH+1; idx++)
    {
        //Printf("FlashPanel_StringCmp Des = 0x%x\n",*(pDes+idx));
        if ((*(pDes+idx) == 0x20) || (*(pDes+idx) == 0xff) || (*(pDes+idx) == 0x00))
        {
            desidx = idx;
			break;
        } 
    }

    for (idx=0; idx<PANEL_VERSION_LENGTH+1; idx++)
    {
        //Printf("FlashPanel_StringCmp Psrc = 0x%x\n",*(pSrc+idx));
        if ((*(pSrc+idx) == 0x20) || (*(pSrc+idx) == 0xff) || (*(pSrc+idx) == 0x00))
        {
            srcidx = idx;
			break;
        } 
    }

    //Printf("FlashPanel_StringCmp SRCIDX= %d, DESIDX= %d\n",srcidx,desidx);
	
    if(srcidx != desidx) {
		ui4Ret = 1;
    } else {
	    for (idx=0; idx<srcidx; idx++)
        {
            if((*(pSrc+idx)) != (*(pDes+idx))) 
			{         
			    ui4Ret = 1;
			    break;              
            }
	    }
	}
    return ui4Ret;
}

#ifdef CC_SONY_FILE_BASED_RECIPE_SUPPORT
#define RECIPE_FILE_LEN          256
INT32 BEBoot_CheckFile(const char *filename, UINT32 u4BufSize);
INT32 LDR_BEReadFile(const char *pPathname, UINT32 u4Offset, VOID* pvBuf, UINT32 u4BufSize);
#endif

static BOOL CustomLoadPanelData(UINT32 u4PartId, UINT32 u4Offset, void *pInfo, UINT32 u4Size)
{
#ifdef CC_SONY_FILE_BASED_RECIPE_SUPPORT
    char fw_name[RECIPE_FILE_LEN];
#endif
    if (pInfo == NULL)
    {
        return FALSE;
    }    
#ifndef CC_UBOOT	
#ifdef CC_SONY_FILE_BASED_RECIPE_SUPPORT
	strncpy(fw_name, "panel.bin", RECIPE_FILE_LEN);
        fw_name[RECIPE_FILE_LEN - 1] = '\0';
	if (BEBoot_CheckFile(fw_name, RECIPE_FILE_LEN))
	 {
		//Printf("CustomLoadPanelData: file found: %s\n", fw_name);
		if (LDR_BEReadFile(fw_name, u4Offset, pInfo, u4Size) < (INT32)u4Size)
		{
			return FALSE;
		}
	} else {
		Printf("CustomLoadPanelData: file not found: %s\n", fw_name);
	}
#else
#ifdef CC_MTK_LOADER
#ifdef CC_EMMC_BOOT
	if (Loader_ReadMsdc(u4PartId, u4Offset, pInfo, u4Size) < (INT32)u4Size)
	{
		return FALSE;
	}
#else
	if (Loader_ReadNandFlash(u4PartId, u4Offset, pInfo, u4Size) < (INT32)u4Size)
	{
		return FALSE;
	}
#endif
#else
	if (STORG_SyncRead(u4PartId, u4Offset, pInfo, u4Size) < (INT32)u4Size)
	{
		return FALSE;
	}
#endif //CC_MTK_LOADER
#endif
#endif 
    return TRUE;
}

CHAR * FlashPanel_GetVersion(void)
{
#ifdef CC_MTK_LOADER
    UINT32 u4PartId = 0;
    UINT32 u4Offset = 0;

    #ifdef PANEL_FLASH_TABLE_BASE_PARTITION
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    #endif
    #ifdef PANEL_FLASH_HEADER_ADDRESS
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
    #endif
    x_memset(au1FlashPanelTableVersion, 0, sizeof(au1FlashPanelTableVersion));
    CustomLoadPanelData(u4PartId, u4Offset, (void *)au1FlashPanelTableVersion, PANEL_VERSION_LENGTH);  // get version info
#endif
    
    return (CHAR *)(au1FlashPanelTableVersion);
}

CHAR * FlashPanel_GetIDString(void)
{
    return (CHAR *)(&rFlashPanelHeader.au1PanelName);
}

UINT32 FlashPanel_GetID(void)
{
    return (UINT32)(rFlashPanelHeader.u1PanelId);
}

UINT32 FlashPanel_GetDither(void)
{
    return 0;
}

UINT8 FlashPanel_GetDVTargetMinPQv2(void)
{
    return rFlashPanelExt2.u1LUTSel;
}
UINT8 FlashPanel_GetDVTargetMaxPQv2(void)
{
    UINT32 u4Tmp=0;

    u4Tmp = rFlashPanelExt2.u4Dither;
	u4Tmp = u4Tmp&0x000000FF;
	
    return (UINT8)(u4Tmp);
}
UINT8 FlashPanel_GetDVGx(void)
{
    UINT32 u4Tmp=0;

    u4Tmp = rFlashPanelExt2.u4Dither;
	u4Tmp = (u4Tmp&0x0000FF00)>>8;
	
    return (UINT8)(u4Tmp);
}
UINT8 FlashPanel_GetDVGy(void)
{
    UINT32 u4Tmp=0;

    u4Tmp = rFlashPanelExt2.u4Dither;
	u4Tmp = (u4Tmp&0x00FF0000)>>16;

    return (UINT8)(u4Tmp);
}
UINT8 FlashPanel_GetDVBx(void)
{
    UINT32 u4Tmp=0;

    u4Tmp = rFlashPanelExt2.u4Dither;
	u4Tmp = (u4Tmp&0xFF000000)>>24;

    return (UINT8)(u4Tmp);
}
UINT8 FlashPanel_GetDVBy(void)
{
    return rFlashPanelExt2.u1NVMType;
}
UINT8 FlashPanel_GetDVRx(void)
{
    return rFlashPanelExt2.au1PQIndex[0];
}
UINT8 FlashPanel_GetDVRy(void)
{
    return rFlashPanelExt2.au1PQIndex[1];
}

UINT32 FlashPanel_GetVRRMin(void)
{
    return (rFlashPanelExt2.au1PQIndex[2] & 0x3F);
}

UINT32 FlashPanel_GetVRRMax(void)
{
    UINT32 u4VrrMax;
    u4VrrMax = (rFlashPanelExt2.au1PQIndex[2] & 0xC0); 
    u4VrrMax = rFlashPanelExt2.au1PQIndex[3] + (u4VrrMax<<2);
    return  u4VrrMax;
}

UINT32 FlashPanel_GetAQIndex(UINT32 u4Idx)
{
    UNUSED(u4Idx);
    return (UINT32)rFlashPanelExt2.au1AQIndex[0];
}

UINT32 FlashPanel_GetPQIndex(UINT32 u4Idx)
{
    UNUSED(u4Idx);
    return (UINT32)rFlashPanelExt2.au1PQIndex[0];
}
UINT32 FlashPanel_GetHdmiEdidIndex(void)
{
    return (UINT32)rFlashPanelExt2.u1HdmiEdidTblIdx;
}

UINT32 FlashPanel_GetVgaEdidIndex(void)
{
    return (UINT32)rFlashPanelExt2.u1VgaEdidTblIdx;
}

UINT32 FlashPanel_GetInchSize(void)
{
    return (UINT32)(rFlashPanelHeader.u1InchSize);
}
UINT32 FlashPanel_GetControlWord(void)
{
    return _arPanelAttribute[0].u4ControlWord;
}

#define CUST_VB1_2SEC_EN (1<<5)
#define CUST_VB1_4SEC_EN (2<<5)

UINT32 FlashPanel_GetControlWord_2SecEn(void)
{
    //return (_arPanelAttribute[0].u4ControlWord & VB1_2SEC_EN);
    return (rFlashPanelExt2.u4ControlWord4 & CUST_VB1_2SEC_EN);
}
UINT32 FlashPanel_GetControlWord_4SecEn(void)
{
    //return (_arPanelAttribute[0].u4ControlWord & VB1_4SEC_EN);
    return (rFlashPanelExt2.u4ControlWord4 & CUST_VB1_4SEC_EN);
}
UINT32 FlashPanel_GetControlWord_8SecEn(void)
{
    //return (_arPanelAttribute[0].u4ControlWord & VB1_8SEC_EN);
    return FALSE; //sony define not contains 8 section case.
}
UINT32 FlashPanel_GetControlWord2(void)
{
    return _arPanelAttribute[0].u4ControlWord2;
}
UINT32 FlashPanel_GetControlWord2_GraphModeEn(void)
{
    return (_arPanelAttribute[0].u4ControlWord2 & SUPPORT_VB1_GRAPH_MODE);
}
UINT32 FlashPanel_GetControlWord3(void)
{
    return (UINT32)(rFlashPanelExt2.u4ControlWord3);
}
UINT32 FlashPanel_GetControlWord3_BLCtrlIf(void)
{
    return (rFlashPanelExt2.u4ControlWord3 & BL_CTRL_IF_MASK);
}
UINT32 FlashPanel_GetControlWord3_BEType(void)
{
    return (rFlashPanelExt2.u4ControlWord3 & BE_TYPE_MASK);
}

UINT32 FlashPanel_GetControlWord4(void)
{
    return (UINT32)(rFlashPanelExt2.u4ControlWord4);
}
UINT32 FlashPanel_GetControlWord5(void)
{
    return (UINT32)(rFlashPanelExt2.u4ControlWord5);
}
UINT32 FlashPanel_GetVCOMDRV(void)
{
    return (UINT32)(rFlashPanelExt2.u1VcomType);
}
UINT32 FlashPanel_GetBL_PWM_FREQ(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_PWM_FREQ);
}
UINT32 FlashPanel_GetBL_CUR_PWM_FREQ(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_CUR_PWM_FREQ);
}
UINT32 FlashPanel_GetBL_PWM_NUM_NORM(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_PWM_NUM_NORM);
}
UINT32 FlashPanel_GetBL_LOGO_GAIN(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_LOGO_GAIN);
}
UINT32 FlashPanel_GetCURRENT_START(void)
{
    return (UINT32)(rFlashPanelExt2.u2CURRENT_START);
}

UINT32 FlashPanel_GetBL_WAIT_LOGO(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_WAIT_LOGO);
}

UINT32 FlashPanel_GetVLED_NORMAL(void)
{
    return (UINT32)(rFlashPanelExt2.u2VLED_NORMAL);
}

UINT32 FlashPanel_GetVLED_PWM_FREQ(void)
{
    return (UINT32)(rFlashPanelExt2.u2BL_VLED_PWM_FREQ);

}

UINT32 FlashPanel_GetVgaEdidTblIdx(void)
{
    return (UINT32)(rFlashPanelExt2.u1VgaEdidTblIdx);
}
UINT16 FlashPanel_GetPanelLowLimit(void)
{
    return rFlashPanelExt2.u2Panel_low_limit;
}
UINT16 FlashPanel_GetPanelHighLimit(void)
{
    return rFlashPanelExt2.u2Panel_high_limit;
}
UINT8 FlashPanel_GetBLErrNum(void)
{
    return rFlashPanelExt2.u1BL_ERR_NUM;
}
UINT8 FlashPanel_GetLDErrNum(void)
{
    return rFlashPanelExt2.u1LD_ERR_NUM;
}

UINT8 FlashPanel_GetPre_Emphasis(void)
{
    return rFlashPanelExt2.u1Pre_emphasis;
}

UINT8 FlashPanel_GetPreEmphasisEn(void)
{
    return ((rFlashPanelExt2.u1Pre_emphasis&0x8)>>3);
}

UINT8 FlashPanel_GetPreEmphasisVal(void)
{
    return (rFlashPanelExt2.u1Pre_emphasis&0x7);
}

UINT8 FlashPanel_GetPanelAttr(void)
{
    return au1PanelAttr; //bit 0: flip; bit 1: mirror; bit 2: LED type; bit 3: LVDS DE Only
}

//rPanelData.au1Reserved[3];
UINT8 FlashPanel_GetMaxLuminance(void)
{
    return au1PeakLumaData[0];
}

//rPanelData.au1Reserved[4];
UINT8 FlashPanel_GetAvgLuminance(void)
{
    return au1PeakLumaData[1];
}

//rPanelData.au1Reserved[5];
UINT8 FlashPanel_GetMinLuminance(void)
{
    return au1PeakLumaData[2];
}

#if 0
UINT32 FlashPanel_IsLedPanel(void)
{
    return (UINT32)((au1PanelAttr & PANEL_LED_TYPE)== 0 ?0:1);
}
UINT8 FlashPanel_IsDEOnly(void)
{
    return (UINT8)((au1PanelAttr & PANEL_DE_ONLY)== 0 ? 0 : 1);
}
VOID FlashPanel_SetDEOnly(UINT8 u1_only)
{
    if (u1_only)
    {
        au1PanelAttr |= PANEL_DE_ONLY;
    }
    else
    {
        au1PanelAttr &= (~PANEL_DE_ONLY);
    }

    return TRUE;
}
#endif

UINT32 CustomGetNumPanelFromFlash(void)
{
    UINT32 u4PartId = 0;
    UINT32 u4Offset = 0;
    UINT32 u4PanelNum = 0;
        
#if defined(PANEL_FLASH_TABLE_BASE_PARTITION) && defined(PANEL_FLASH_HEADER_ADDRESS)
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
#else
    Printf ("PANEL_FLASH_TABLE_BASE_PARTITION or PANEL_FLASH_HEADER_ADDRESS is not defined\n");
    return 0;
#endif   
    CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number
    Printf ("CustomGetNumPanelFromFlash: %d\n", u4PanelNum);
    return u4PanelNum;
}
void CustomGetPanelMiscInfoFromFlash(MISC_TYPE_PANEL_INFO_T *pData, UINT32 u4Num)
{
    UINT32 u4PartId = 0;
    UINT32 u4Offset = 0;
    UINT32 u4PanelNum = 0;
    UINT32 u4Addr = 0;
    UINT32 u4TableSize = 0;
    UINT32 i = 0;   
    PANEL_CUSTOM_FLASH_ATTRIBUTE_T rPanelData;

    x_memset(pData, 0, sizeof(MISC_TYPE_PANEL_INFO_T)*u4Num);
    x_memset(&rPanelData, 0, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T));
    
#if defined(PANEL_FLASH_TABLE_BASE_PARTITION) && defined(PANEL_FLASH_HEADER_ADDRESS)
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
#else
    Printf ("PANEL_FLASH_TABLE_BASE_PARTITION or PANEL_FLASH_HEADER_ADDRESS is not defined\n");
    return;
#endif

    // load panel table info
    CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number
    if (u4PanelNum < u4Num)
    {
        u4Num = u4PanelNum;
    }

    u4Addr = DRVCUST_PanelGet(ePanelFlashTableAddress);
    u4TableSize = DRVCUST_PanelGet(ePanelFlashTableSize);
    
    for (i = 0; i < u4Num; i++)
    {
        // init data structure
        x_memset(&rPanelData, 0, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T));
        if (!CustomLoadPanelData(u4PartId, u4Addr+(u4TableSize*i), (void *)&rPanelData, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T)))
		{
            continue;
        }
        x_memcpy(pData[i].au1PanelName, rPanelData.rHeader.au1PanelName, sizeof(pData[i].au1PanelName));
        pData[i].u1PanelId = rPanelData.rHeader.u1PanelId;
        pData[i].u1Shown = rPanelData.u1ShownOnMenu;
        Printf("--- Panel[%3d] [%s] shown(%d) ---\n", pData[i].u1PanelId, pData[i].au1PanelName, pData[i].u1Shown);
    }
}

void CustomDumpPanelListFromFlash(void)
{
    UINT32 u4PartId = 0;
    UINT32 u4Offset = 0;
    UINT32 u4PanelNum = 0;
    UINT32 u4Addr = 0;
    UINT32 u4TableSize = 0;
    UINT32 i = 0;   
    PANEL_CUSTOM_FLASH_ATTRIBUTE_T rPanelData;
    UINT32 u4PartitionSize = 0;
    UINT32 u4PanelNumMaxInPartition = 0;
    
#if defined(PANEL_FLASH_TABLE_BASE_PARTITION) && defined(PANEL_FLASH_HEADER_ADDRESS)
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
#else
    Printf ("PANEL_FLASH_TABLE_BASE_PARTITION or PANEL_FLASH_HEADER_ADDRESS is not defined\n");
    return;
#endif   
    u4PartitionSize = DRVCUST_InitGet64((QUERY_TYPE_T)(eNANDFlashPartSize0+u4PartId));\
	u4PanelNumMaxInPartition = (u4PartitionSize - PANEL_FLASH_HEADER_SIZE) / sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T);
                              
    // init data structure
    x_memset(&rPanelData, 0, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T));

    // load panel table info
    CustomLoadPanelData(u4PartId, u4Offset, (void *)(rPanelData.rHeader.au1PanelName), 16);  // get version info, use memory of panel name to store version
    CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number

    //u4PanelNum = (u4PanelNum > 60)? 60: u4PanelNum; // add for no Flash Panel in flash
    u4PanelNum = (u4PanelNum > u4PanelNumMaxInPartition) ? u4PanelNumMaxInPartition : u4PanelNum; // add for invalid Panel num.
    if (u4PanelNum == 0) // add for no Flash Panel in flash.
    {
      Printf("Dump Panel List from Flash Failed (no panel data)\n");
	  return;
	}

    rPanelData.rHeader.au1PanelName[15] = '\0';
    Printf ("version: %s, total number %d\n", rPanelData.rHeader.au1PanelName, u4PanelNum);

    u4Addr = DRVCUST_PanelGet(ePanelFlashTableAddress);
    u4TableSize = DRVCUST_PanelGet(ePanelFlashTableSize);

    for (i = 0; i < u4PanelNum; i++)
    {
        if (!CustomLoadPanelData(u4PartId, u4Addr+(u4TableSize*i), (void *)&rPanelData, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T)))
        {
            continue;
        }
        rPanelData.rHeader.au1PanelName[15]='\0';
        Printf("---------- Panel[%3d] %s %dx%d ----------\n",
        rPanelData.rHeader.u1PanelId, rPanelData.rHeader.au1PanelName, rPanelData.rBasicInfo.u2PanelWidth, rPanelData.rBasicInfo.u2PanelHeight);
    }
}

#ifndef CC_MTK_LOADER
extern void vDrvUpdatePanelParam(UINT32 u4Type);
#endif

BOOL CustomPanelLoadTableFromFlash(UINT32 u4Type, void *pData)
{
    UINT32 u4Offset = 0;
    UINT32 u4TableSize;
    UINT32 u4Addr;
    UINT32 u4PanelNum = 0;
    UINT32 i,j;   
    PANEL_CUSTOM_FLASH_ATTRIBUTE_T rPanelData;
#ifdef CC_MTK_LOADER
    PANEL_CUSTOM_FLASH_ATTRIBUTE_T *rPanelAllData = NULL;
#endif
    UINT32 u4PartId = 0;
	CHAR DesStringCmp[PANEL_VERSION_LENGTH+1];
	CHAR SrcStringCmp[PANEL_VERSION_LENGTH+1];
	UINT32 u4PartitionSize = 0;
	UINT32 u4PanelNumMaxInPartition = 0;
    UINT32 u4PQ3DLUTDstDram = DRVCUST_InitGet(eFbmMem3DLUTStart);
    UINT8 *puSPIBuf = (UINT8 *)(u4PQ3DLUTDstDram + 0x300000);

    if (pData == NULL || (u4PQ3DLUTDstDram == 0 && DRVCUST_Get_InterMemc_type() == 1))
    {
        return FALSE;
    }

    Printf ("%s u4Type:%d pData:%x \n",__FUNCTION__,u4Type,*(UINT32 *)pData);

    #ifdef PANEL_FLASH_TABLE_BASE_PARTITION
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    #endif
    #ifdef PANEL_FLASH_HEADER_ADDRESS
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
    #endif
	u4PartitionSize = DRVCUST_InitGet64((QUERY_TYPE_T)(eNANDFlashPartSize0+u4PartId));
	u4PanelNumMaxInPartition = (u4PartitionSize - PANEL_FLASH_HEADER_SIZE) / sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T);                                  

	// init data structure
    rPanelData.rHeader.u1PanelId = 0;

    // load panel table info
    CustomLoadPanelData(u4PartId, u4Offset, (void *)au1FlashPanelTableVersion, 16);  // get version info
    au1FlashPanelTableVersion[16] = '\0';
    CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number

    u4Addr = DRVCUST_PanelGet(ePanelFlashTableAddress);
    u4TableSize = DRVCUST_PanelGet(ePanelFlashTableSize);

    //u4PanelNum = (u4PanelNum > 60)? 60: u4PanelNum; // add for no Flash Panel in flash
    u4PanelNum = (u4PanelNum > u4PanelNumMaxInPartition) ? u4PanelNumMaxInPartition : u4PanelNum; // add for invalid Panel num.
    if (u4PanelNum == 0) // add for no Flash Panel in flash.
    {
      Printf("Load Panel Table from Flash Failed (no panel data)\n");
      return FALSE;
    }
#ifdef CC_MTK_LOADER
    rPanelAllData = (PANEL_CUSTOM_FLASH_ATTRIBUTE_T *)x_mem_alloc(sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T)*u4PanelNum);
    if (rPanelAllData == NULL)
    {
      Printf("Failed to allocate memory\n");
      return FALSE;
    }
    if (!CustomLoadPanelData(u4PartId, u4Addr, (void *)rPanelAllData, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T)*u4PanelNum))
    {
      Printf("Load Panel Table from Flash Failed\n");
      x_mem_free(rPanelAllData);
      return FALSE;
    }
#endif

    for (i = 0; i < u4PanelNum; i++)
    {
#ifdef CC_MTK_LOADER	
        rPanelData = rPanelAllData[i];
#else
        if (!CustomLoadPanelData(u4PartId, u4Addr+(u4TableSize*i), (void *)&rPanelData, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T)))
        {
            continue;
        }
#endif
        if (u4Type == LOAD_PANEL_FROM_FLASH_BY_ID)
        {
            if (rPanelData.rHeader.u1PanelId == (UINT8)(*(UINT32*)pData))
            {
                Printf ("LOAD_PANEL_FROM_FLASH_BY_ID MATCH: %d\n", rPanelData.rHeader.u1PanelId);            
                Printf ("u4TableSize=%d, rPanelData.au1Reserved[0]=0x%02x\n", u4TableSize, rPanelData.au1Reserved[0]);
                Printf ("rPanelData.au1Reserved[1]=0x%02x, rPanelData.au1Reserved[2]=0x%02x\n", rPanelData.au1Reserved[1], rPanelData.au1Reserved[2]);
                Printf ("puSPIBuf=0x%p, rPanelData.rVideoAttr[2].u1FrameRateMax=%d\n", puSPIBuf, rPanelData.rVideoAttr[2].u1FrameRateMax);
                #ifdef CC_MTK_LOADER
                if(DRVCUST_Get_InterMemc_type() == 1) {
                    x_memset((void*)puSPIBuf, 0, 4);
                    x_memcpy((void*)puSPIBuf, (void*)(&rPanelData.au1Reserved[0]), 3);
                    x_memcpy((void*)(puSPIBuf+3), (void*)(&rPanelData.rVideoAttr[2].u1FrameRateMax), 1);
                }
                #endif
                break;
            }            
        }
        else // search by name
        {
#if 1       
            CHAR *pStr;
            CHAR *pFlashStr;
            pStr = (CHAR*)pData;
            pFlashStr = (CHAR*)rPanelData.rHeader.au1PanelName;
            DesStringCmp[PANEL_VERSION_LENGTH] = '\0';
            SrcStringCmp[PANEL_VERSION_LENGTH] = '\0';
			for (j=0; j<PANEL_VERSION_LENGTH; j++)
			{
			    DesStringCmp[j]= *(pFlashStr+j);
				SrcStringCmp[j]= *(pStr+j);
			}
            //Printf ("LOAD_PANEL_FROM_FLASH_BY_ID FLASH_DATA: %s\n",pFlashStr);
			//Printf ("LOAD_PANEL_FROM_FLASH_BY_ID NVM_DATA: %s\n",pStr);
            //Printf ("LOAD_PANEL_FROM_FLASH_BY_ID FLASH_DATA: %s\n",&DesStringCmp[0]);
			//Printf ("LOAD_PANEL_FROM_FLASH_BY_ID NVM_DATA: %s\n",&SrcStringCmp[0]);

            if (FlashPanel_StringCmp(&SrcStringCmp[0], &DesStringCmp[0]) == 0)
            {
                //Printf ("LOAD_PANEL_FROM_FLASH_BY_NAME MATCH\n");
                Printf ("LOAD_PANEL_FROM_FLASH_BY_NAME MATCH: %s\n", SrcStringCmp);
                break;
            }            
#endif            
        }         
    }
#ifdef CC_MTK_LOADER
    x_mem_free(rPanelAllData);
#endif
    #ifdef CC_S_PLATFORM
    printf("[loader]Load Panel Table from Flash id=%d,u4PanelNum=%d,u4Type=%d i=%d.\n",(UINT8)(*(UINT32*)pData), u4PanelNum, u4Type,i);
    #endif
    if (i >= u4PanelNum)
    {
        //printf("Load Panel Table from Flash Failed\n");
        Printf("Load Panel Table from Flash Failed (couldn't find matched panel)\n");
        return FALSE;
    }

    // convert panel data to internal format
    x_memcpy((void*)(&rFlashPanelHeader), (void*)(&rPanelData.rHeader), sizeof(PANEL_HEADER_CUSTOM_ATTRIBUTE_T));
    x_memcpy((void*)(&_arPanelAttribute[0]), (void*)(&rPanelData.rBasicInfo), sizeof(PANEL_ATTRIBUTE_T));
    x_memcpy((void*)(&rFlashPanelExt2), (void*)(&rPanelData.rExt2Info), sizeof(PANEL_Ext2_CUSTOM_ATTRIBUTE_T));

    x_memcpy((void*)(rVideo), (void*)(rPanelData.rVideoAttr), sizeof(GRAPHMODE_ATTRIBUTE_T)*PNL_VDO_ATTR);
    
    x_memcpy((void*)(rGraphic), (void*)(rPanelData.rGraphicAttr), sizeof(GRAPHMODE_ATTRIBUTE_T)*PNL_GFX_ATTR);

    au1PeakLumaData[0] = rPanelData.au1Reserved[3];
    au1PeakLumaData[1] = rPanelData.au1Reserved[4];
    au1PeakLumaData[2] = rPanelData.au1Reserved[5];

    #ifdef SUPPORT_PANEL_48HZ
    _arPanelAttributeExt[0].u4PixelClk48Hz = rPanelData.u4PixelClk48Hz;
    _arPanelAttributeExt[0].u2HTotal48Hz = rPanelData.u2HTotal48Hz;
    _arPanelAttributeExt[0].u2VTotal48Hz = rPanelData.u2VTotal48Hz;
    _arPanelAttributeExt[0].u2DimmingFrequency48Hz = rPanelData.u2DimmingFrequency48Hz;
    #endif

    #ifdef SUPPORT_PANEL_CUSTOMER_SPEC
	for(j=0;j<3;j++)
	{
      _arPanelAttributeExt[0].u1HSyncWidth[j] = rPanelData.au1HSyncWidth[j];
	  _arPanelAttributeExt[0].u1VSyncWidth[j] = rPanelData.au1VSyncWidth[j];
	  _arPanelAttributeExt[0].u2HFrontPorch[j] = rPanelData.au2HFrontPorch[j];
      _arPanelAttributeExt[0].u2VbackPorch[j] = rPanelData.au2VbackPorch[j];
	}
    _arPanelAttributeExt[0].u1HSyncPolarity = rPanelData.u1HSyncPolarity;
    _arPanelAttributeExt[0].u1VSyncPolarity = rPanelData.u1VSyncPolarity;
    #endif


    #ifdef SUPPORT_PANEL_DITHER
     _arPanelAttributeExt[0].u4Dither = rPanelData.rExt2Info.u4Dither;
    #endif

    #ifdef SUPPORT_PANEL_SS
    _arPanelAttributeExt[0].u1SSPermillage = rPanelData.u1SSPermillage;
    _arPanelAttributeExt[0].u4SSFreq = rPanelData.u4SSFreq;
    #endif

    #ifdef SUPPORT_PANEL_DRIVING
    _arPanelAttributeExt[0].u1DrivingCurrent = rPanelData.u1DrivingCurrent;
    #endif

	#ifdef SUPPORT_PANEL_ERRORLIMIT_TABLE
	_arPanelAttributeExt[0].u4ErrorLimit = rPanelData.rGammaInfo.u4Error_Limit;
	#endif

	#ifdef SUPPORT_PANEL_SIZE_CM
	_arPanelAttributeExt[0].u2PanleWidth = rPanelData.u2PanelSizeCM_Width;
	_arPanelAttributeExt[0].u2PanleHeight= rPanelData.u2PanelSizeCM_Height;
	#endif
	#ifdef SUPPORT_PANEL_4KBERST_PANELON_WAIT
	 _arPanelAttributeExt[0].u14kbeRstPanelOnWait = rPanelData.u14kbeRstPanelOnWait;
	#endif

	#ifdef SUPPORT_PANEL_TCONON_CELL_WAIT
	 _arPanelAttributeExt[0].u1TconOnCellWait= rPanelData.u1TconOnCellWait;
	#endif
    //Mini Lvds informatio
    _arPanelMiniLVDSAttribute[0].u1Port = rPanelData.u1Port ;
    _arPanelMiniLVDSAttribute[0].u1Ratio= rPanelData.u1Ratio ;
    au1PanelAttr = rPanelData.u1PanelAttr;

#if 0
    Printf("[][] flash panel debug\n");
    Printf("[][] total size %d\n", sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T));
    Printf("[][] head size %d\n", sizeof(PANEL_HEADER_CUSTOM_ATTRIBUTE_T));
    Printf("[][] basic info size %d\n", sizeof(PANEL_ATTRIBUTE_T));
    Printf("[][] gamma size %d\n", sizeof(PANEL_EXT_GAMMA_ATTRIBUTE_T));
    Printf("[][] custom size %d\n", sizeof(PANEL_Ext2_CUSTOM_ATTRIBUTE_T));
            
    Printf("[][] reserved %x %x %x %x %x %x %x %x\n",rPanelData.au1Reserved[0], rPanelData.au1Reserved[1], rPanelData.au1Reserved[2], rPanelData.au1Reserved[3]
                                                    ,rPanelData.au1Reserved[13],rPanelData.au1Reserved[14],rPanelData.au1Reserved[15],rPanelData.au1Reserved[16]);
        
    Printf("[][]au1PanelAttr 0x%x\n",au1PanelAttr);
#endif
	
    #ifdef  CC_FLIP_MIRROR_SUPPORT
    u1SetFlipMirrorConfig((au1PanelAttr & PANEL_MIRROR_TYPE) , (au1PanelAttr & PANEL_FLIP_TYPE) );
    if(au1PanelAttr & PANEL_MIRROR_TYPE)
    {
       Printf("Panel mirror enable\n");
    }
    if(au1PanelAttr & PANEL_FLIP_TYPE)
    {
       Printf("Panel flip enable\n");
    }
    #endif

    SelectPanel(0);
	#ifndef CC_MTK_LOADER
	vDrvUpdatePanelParam(GetPANEL4K2K_TYPE());
	#endif
    return TRUE;
}

UINT32 FlashPanel_GetPanelNumber(void)
{
    UINT32 u4PartId = 0;
    UINT32 u4Offset = 0;
    UINT32 u4PanelNum = 0;
	UINT32 u4PartitionSize = 0;
	UINT32 u4PanelNumMaxInPartition = 0;

    #ifdef PANEL_FLASH_TABLE_BASE_PARTITION
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
#endif
    #ifdef PANEL_FLASH_HEADER_ADDRESS
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
    #endif
	u4PartitionSize = (UINT32)DRVCUST_InitGet64((QUERY_TYPE_T)(eNANDFlashPartSize0+u4PartId));
    if (u4PartitionSize > PANEL_FLASH_HEADER_SIZE)
    {
    	u4PanelNumMaxInPartition = (u4PartitionSize - PANEL_FLASH_HEADER_SIZE) / sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T);	

    	CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number

        //u4PanelNum = (u4PanelNum > 100)? 100: u4PanelNum; // add for no Flash Panel in flash
        u4PanelNum = (u4PanelNum > u4PanelNumMaxInPartition) ? u4PanelNumMaxInPartition : u4PanelNum; // add for invalid Panel num.
        
        return (UINT32)(u4PanelNum);
    }
    else
    {
        LOG(0, "ERROR: u4PartitionSize <= PANEL_FLASH_HEADER_SIZE !!\n");
        return 0;
    }
}

BOOL FlashPanel_ListAllPanelID(UINT8 * pui1_id)
{
    UINT32 u4Offset = 0;
    UINT32 u4TableSize;
    UINT32 u4Addr;
    UINT32 u4PanelNum = 0;
    UINT32 i;
    PANEL_CUSTOM_FLASH_ATTRIBUTE_T rPanelData;
    UINT32 u4PartId = 0;
	UINT32 u4PartitionSize = 0;
	UINT32 u4PanelNumMaxInPartition = 0;

    if (pui1_id == NULL)
    {
        return FALSE;
    }

    #ifdef PANEL_FLASH_TABLE_BASE_PARTITION
    u4PartId = PANEL_FLASH_TABLE_BASE_PARTITION;
    #endif
    #ifdef PANEL_FLASH_HEADER_ADDRESS
    u4Offset = PANEL_FLASH_HEADER_ADDRESS;
#endif
    u4PartitionSize = DRVCUST_InitGet64((QUERY_TYPE_T)(eNANDFlashPartSize0+u4PartId));
    u4PanelNumMaxInPartition = (u4PartitionSize - PANEL_FLASH_HEADER_SIZE) / sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T);

    // load panel table info
    CustomLoadPanelData(u4PartId, u4Offset+16, (void *)&u4PanelNum, 4);      // get panel number

    u4Addr = DRVCUST_PanelGet(ePanelFlashTableAddress);
    u4TableSize = DRVCUST_PanelGet(ePanelFlashTableSize);

    //u4PanelNum = (u4PanelNum > 100)? 100: u4PanelNum; // add for no Flash Panel in flash
    u4PanelNum = (u4PanelNum > u4PanelNumMaxInPartition) ? u4PanelNumMaxInPartition : u4PanelNum; // add for invalid Panel num.
    if (u4PanelNum == 0) // add for no Flash Panel in flash.
    {
      Printf("List All Panel ID Failed (no panel data)\n");
	  return FALSE;
	}

    rPanelData.rHeader.u1PanelId = 0;
    for (i = 0; i < u4PanelNum; i++)
    {
        CustomLoadPanelData(u4PartId, u4Addr+(u4TableSize*i), (void *)&rPanelData, sizeof(PANEL_CUSTOM_FLASH_ATTRIBUTE_T));

        pui1_id[i] = rPanelData.rHeader.u1PanelId;
    }
    return TRUE;
}

BOOL MTK_PanelLoadTableFromFlash(UINT32 u4Type, void *pData)
{
    if (pData == NULL)
    {
        return FALSE;
    }

    return TRUE;
}
#endif

#ifdef SUPPORT_PANEL_CUSTOMER_SPEC
//return num*10; for 120Hz panel
UINT32 PANEL_GetFrameDelay(void)
{
    return 0;
}
#endif
 
static GRAPHMODE_ATTRIBUTE_T _arVideoModeAttribute[]=
{
    {
        PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST | PANEL_SetType(PANEL_TYPE_2K1K_60),
        {148473600, 148500000, 148500000},     // au4PixelClk

        {2200, 2200, 2200},                    // au2HTotal
		1920,                                  // u2GraphModeWidth
        {16, 16, 16},                          // au2HFrontPorch
        {44, 44, 44},						   // au1HSyncWidth
        {220, 220, 220},                       // au2HBackPorch

        {1406, 1350, 1125},                    // au2VTotal
        1080,                                  // u2GraphModeHeight
        {320, 264, 39},                        // au2VFrontPorch
        {3, 3, 3},                             // au1VSyncWidth
        {3, 3, 3},                             // au2VBackPorch

		DISP_4K2K_TYPE_2,                       // u4GControlWord
		63,                                     // u1FrameRateMax
		47,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
    },
    {
        PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST | PANEL_SetType(PANEL_TYPE_2K1K_120),
        {296947200, 297000000, 297000000},
    
        {2200, 2200, 2200},
        1920,
        {16, 16, 16},
        {44, 44, 44},
        {220, 220, 220},
    
        {1406, 1350, 1125},
        1080,
        {320, 264, 39},
        {3, 3, 3},
        {3, 3, 3},
    
        DISP_4K2K_TYPE_2,
        120,
        60,
        
		HSYNC_LOW,
		VSYNC_LOW, 	        
    },
    {
        PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST | PANEL_SetType(PANEL_TYPE_4K2K_30),
        {297000000, 297000000, 297000000},      // au4PixelClk

        {5500, 5280, 4400},                     // au2HTotal
		3840,                                   // u2GraphModeWidth
        {1132, 912, 32},                        // au2HFrontPorch
        {88, 88, 88},                           // au1HSyncWidth
        {440, 440, 440},                        // au2HBackPorch

        {2250, 2250, 2250},                     // au2VTotal
		2160,                                   // u2GraphModeHight
        {78, 78, 78},                           // au2VFrontPorch
        {6, 6, 6},                              // au1VSyncWidth
        {6, 6, 6},                              // au2VBackPorch

		DISP_4K2K_TYPE_2,                       // u4GControlWord
		30,                                     // u1FrameRateMax
		24,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
    },
	{
        PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST | PANEL_SetType(PANEL_TYPE_4K2K_60),
        {296784000, 297000000, 297000000},      // au4PixelClk

        {5496, 5280, 4400},                     // au2HTotal
		3840,                                   // u2GraphModeWidth
        {1128, 912, 32},                        // au2HFrontPorch
        {88, 88, 88},                           // au1HSyncWidth
        {440, 440, 440},                        // au2HBackPorch

        {2250, 2250, 2250},                     // au2VTotal
		2160,                                   // u2GraphModeHight
        {78, 78, 78},                           // au2VFrontPorch
        {6, 6, 6},                              // au1VSyncWidth
        {6, 6, 6},                              // au2VBackPorch

		DISP_4K2K_TYPE_2,                       // u4GControlWord
		60,                                     // u1FrameRateMax
		48,                                     // u1FrameRateMin
		
		HSYNC_LOW,
		VSYNC_LOW,		
    },
 
    {
		PANEL_4K1K_VB1 | PANEL_SetType(PANEL_TYPE_4K2K_60),
		{297000000, 297000000, 297000000},		// au4PixelClk

		{5496, 5280, 4400}, 					// au2HTotal
		3840,									// u2GraphModeWidth
		{1272, 1056, 176},						// au2HFrontPorch
		{88, 88, 88},							// au1HSyncWidth
		{296, 296, 296},						// au2HBackPorch

		{2252, 2250, 2250}, 					// au2VTotal
		2160,									// u2GraphModeHight
		{10, 8, 8}, 							// au2VFrontPorch
		{10, 10, 10},							// au1VSyncWidth
		{72, 72, 72},							// au2VBackPorch

		DISP_4K2K_TYPE_3,						// u4GControlWord
		60, 									// u1FrameRateMax
		48, 									// u1FrameRateMin
		
		HSYNC_LOW,
		VSYNC_LOW, 	
	},
	{
		PANEL_4K1K_VB1 | PANEL_SetType(PANEL_TYPE_4K1K_60),
		{297000000, 297000000, 297000000},

		{5496, 5280, 4400},
		3840,
		{1272, 1056, 176},
		{88, 88, 88},
		{296, 296, 296},

		{1126, 1125, 1125},
		1080,
		{10, 9, 9},
		{4, 4, 4},
		{32, 32, 32},

		DISP_4K2K_TYPE_3,
		60,
		48,
		
		HSYNC_LOW,
		VSYNC_LOW, 		
	},
	{
		PANEL_4K1K_VB1 | PANEL_SetType(PANEL_TYPE_2K1K_120),
		{297000000, 297000000, 297000000},
	
		{2748, 2640, 2200},
		1920,
		{636, 528, 88},
		{44, 44, 44},
		{148, 148, 148},
	
		{1126, 1125, 1125},
		1080,
		{10, 9, 9},
		{4, 4, 4},
		{32, 32, 32},
	
		DISP_4K2K_TYPE_3,
		120,
		96,
		
		HSYNC_LOW,
		VSYNC_LOW,			
	},
	{
		PANEL_4K1K_VB1 | PANEL_SetType(PANEL_TYPE_4K2K_30),
		{297000000, 297000000, 297000000},		// au4PixelClk

		{5496, 5280, 4400}, 					// au2HTotal
		3840,									// u2GraphModeWidth
		{1272, 1056, 176},						// au2HFrontPorch
		{88, 88, 88},							// au1HSyncWidth
		{296, 296, 296},						// au2HBackPorch

		{2252, 2250, 2250}, 					// au2VTotal
		2160,									// u2GraphModeHight
		{10, 8, 8}, 							// au2VFrontPorch
		{10, 10, 10},							// au1VSyncWidth
		{72, 72, 72},							// au2VBackPorch

		DISP_4K2K_TYPE_3,						// u4GControlWord
		30, 									// u1FrameRateMax
		24, 									// u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
	},
	{
		PANEL_4K1K_VB1 | PANEL_SetType(PANEL_TYPE_2K1K_60),
		{148500000, 148500000, 148500000},		// au4PixelClk

		{2748, 2640, 2200}, 					// au2HTotal
		1920,									// u2GraphModeWidth
		{636, 528, 88},
		{44, 44, 44},
		{148, 148, 148},					   // au2HBackPorch

		{1126, 1125, 1125},
		1080,
		{10, 9, 9},
		{4, 4, 4},
		{32, 32, 32},						   // au2VBackPorch

		DISP_4K2K_TYPE_3,						// u4GControlWord
		60, 									// u1FrameRateMax
		47, 									// u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
	},
	{
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_4K2K_60),
        {297000000, 297000000, 297000000},      // au4PixelClk

        {5496, 5280, 4400},                     // au2HTotal
		3840,                                   // u2GraphModeWidth
        {1272, 1056, 176},                      // au2HFrontPorch
        {88, 88, 88},                           // au1HSyncWidth
        {296, 296, 296},                        // au2HBackPorch

        {2252, 2250, 2250},                     // au2VTotal
		2160,                                   // u2GraphModeHight
        {10, 8, 8},                             // au2VFrontPorch
        {10, 10, 10},                           // au1VSyncWidth
        {72, 72, 72},                           // au2VBackPorch

		DISP_4K2K_TYPE_3,                       // u4GControlWord
		60,                                     // u1FrameRateMax
		48,                                     // u1FrameRateMin
		
		HSYNC_LOW,
		VSYNC_LOW,		
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_4K1K_60),
        {297000000, 297000000, 297000000},

        {5496, 5280, 4400},
		3840,
        {1272, 1056, 176},
        {88, 88, 88},
        {296, 296, 296},

        {1126, 1125, 1125},
		1080,
        {10, 9, 9},
        {4, 4, 4},
        {32, 32, 32},

		DISP_4K2K_TYPE_3,
		60,
		48,
		
		HSYNC_LOW,
		VSYNC_LOW, 		
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_2K1K_120),
        {297000000, 297000000, 297000000},
    
        {2748, 2640, 2200},
        1920,
        {636, 528, 88},
        {44, 44, 44},
        {148, 148, 148},
    
        {1126, 1125, 1125},
        1080,
        {10, 9, 9},
        {4, 4, 4},
        {32, 32, 32},
    
        DISP_4K2K_TYPE_3,
        120,
        96,
        
		HSYNC_LOW,
		VSYNC_LOW, 	        
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_4K2K_30),
        {297000000, 297000000, 297000000},      // au4PixelClk

        {5496, 5280, 4400},                     // au2HTotal
		3840,                                   // u2GraphModeWidth
        {1272, 1056, 176},                      // au2HFrontPorch
        {88, 88, 88},                           // au1HSyncWidth
        {296, 296, 296},                        // au2HBackPorch

        {2252, 2250, 2250},                     // au2VTotal
		2160,                                   // u2GraphModeHight
        {10, 8, 8},                             // au2VFrontPorch
        {10, 10, 10},                           // au1VSyncWidth
        {72, 72, 72},                           // au2VBackPorch

		DISP_4K2K_TYPE_3,                       // u4GControlWord
		30,                                     // u1FrameRateMax
		24,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_2K1K_60),
        {148500000, 148500000, 148500000},      // au4PixelClk

        {2748, 2640, 2200},                     // au2HTotal
		1920,                                   // u2GraphModeWidth
        {636, 528, 88},
        {44, 44, 44},
        {148, 148, 148},                       // au2HBackPorch

        {1126, 1125, 1125},
        1080,
        {10, 9, 9},
        {4, 4, 4},
        {32, 32, 32},                          // au2VBackPorch

		DISP_4K2K_TYPE_3,                       // u4GControlWord
		60,                                     // u1FrameRateMax
		47,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
    },

	{
        PANEL_FHD_BACKEND | PANEL_SetType(PANEL_TYPE_720P_120),
        {148500000, 148500000, 148500000},      // au4PixelClk

        {1650, 1650, 1650},                     // au2HTotal
		1280,                                   // u2GraphModeWidth
        {110, 110, 110},
        {32, 32, 32},
        {228, 228, 228},                       // au2HBackPorch

        {938, 900, 750},
        720,
        {203, 165, 15},
        {4, 4, 4},
        {11, 11, 11},                          // au2VBackPorch

		DISP_2K_TYPE_1,                          // u4GControlWord
		120,                                     // u1FrameRateMax
		96,                                     // u1FrameRateMin

		HSYNC_HIGH,
		VSYNC_HIGH,
    },
	{
        PANEL_FHD_BACKEND | PANEL_SetType(PANEL_TYPE_2K1K_60),
        {148500000, 148500000, 148500000},      // au4PixelClk

        {2184, 2184, 2184},                     // au2HTotal
		1920,                                   // u2GraphModeWidth
        {32, 32, 32},
        {32, 32, 32},
        {200, 200, 200},                       // au2HBackPorch

        {1416, 1360, 1134},
        1080,
        {316, 260, 34},
        {4, 4, 4},
        {16, 16, 16},                          // au2VBackPorch

		DISP_2K_TYPE_1,                         // u4GControlWord
		63,                                     // u1FrameRateMax
		47,                                     // u1FrameRateMin

		HSYNC_HIGH,
		VSYNC_HIGH,
    },
    {
        PANEL_FHD_BACKEND | PANEL_SetType(PANEL_TYPE_1920_540P),
        {148500000, 148500000, 148500000},      // au4PixelClk

        {2184, 2184, 2184},                     // au2HTotal
		1920,                                   // u2GraphModeWidth
        {32, 32, 32},
        {32, 32, 32},
        {200, 200, 200},                       // au2HBackPorch

        {708, 680, 567},
        540,
        {158, 130, 17},
        {4, 4, 4},
        {6, 6, 6},                          // au2VBackPorch

		DISP_2K_TYPE_1,                          // u4GControlWord
		123,                                     // u1FrameRateMax
		95,                                     // u1FrameRateMin

		HSYNC_HIGH,
		VSYNC_HIGH,
    },
};

#if defined(SUPPORT_PANEL_GRAPH_MODE)
static GRAPHMODE_ATTRIBUTE_T _arGraphModeAttribute[]=
{
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_4K2K_30),
        {297000000, 297000000, 297000000},      // au4PixelClk

        {5496, 5280, 4400},                     // au2HTotal
		3840,                                   // u2GraphModeWidth
        {1272, 1056, 176},                      // au2HFrontPorch
        {88, 88, 88},                           // au1HSyncWidth
        {296, 296, 296},                        // au2HBackPorch

        {2252, 2250, 2250},                     // au2VTotal
		2160,                                   // u2GraphModeHight
        {10, 8, 8},                             // au2VFrontPorch
        {10, 10, 10},                           // au1VSyncWidth
        {72, 72, 72},                           // au2VBackPorch

		GM_FORMAT_CASE0_SMODE1 | GM_LANE_4,     // u4GControlWord
		30,                                     // u1FrameRateMax
		24,                                     // u1FrameRateMin

		HSYNC_HIGH,
		VSYNC_HIGH,
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_4K1K_60),
        {297000000, 297000000, 297000000},

        {5496, 5280, 4400},
		3840,
        {1272, 1056, 176},
        {88, 88, 88},
        {296, 296, 296},

        {1126, 1125, 1125},
		1080,
        {10, 9, 9},
        {4, 4, 4},
        {32, 32, 32},

		GM_FORMAT_CASE0_SMODE1 | GM_LANE_4,
		60,
		48,

		HSYNC_HIGH,
		VSYNC_HIGH,
    },
    {
        PANEL_4K_BACKEND | PANEL_SetType(PANEL_TYPE_2K1K_60),
        {148500000, 148500000, 148500000},

        {2200, 2200, 2200},
		1920,
        {88, 88, 88},
        {44, 44, 44},
        {148, 148, 148},

        {1406, 1350, 1125},
		1080,
        {320, 264, 39},
        {3, 3, 3},
        {3, 3, 3},

		GM_FORMAT_CASE2_MMODE0 | GM_LANE_2,
		60,
		48,

		HSYNC_HIGH,
		VSYNC_HIGH,
    },   

	{
	    PANEL_VB1_4K2KDD_GRAPH| PANEL_SetType(PANEL_TYPE_2K1K_60),
	    {148473600, 148500000, 148500000},     // au4PixelClk

	    {2200, 2200, 2200},                    // au2HTotal
		1920,                                  // u2GraphModeWidth
	    {16, 16, 16},                          // au2HFrontPorch
	    {44, 44, 44},						   // au1HSyncWidth
	    {220, 220, 220},                       // au2HBackPorch

	    {1406, 1350, 1125},                    // au2VTotal
	    1080,                                  // u2GraphModeHeight
	    {320, 264, 39},                        // au2VFrontPorch
	    {3, 3, 3},                             // au1VSyncWidth
	    {3, 3, 3},                             // au2VBackPorch

		GM_FORMAT_CASE2_MMODE0 | GM_LANE_2,     // u4GControlWord
		63,                                     // u1FrameRateMax
		47,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
	},
	{
	    PANEL_VB1_4K2KDD_GRAPH| PANEL_SetType(PANEL_TYPE_2K1K_120),
	    {296947200, 297000000, 297000000},     // au4PixelClk

	    {2200, 2200, 2200},                    // au2HTotal
		1920,                                  // u2GraphModeWidth
	    {16, 16, 16},                          // au2HFrontPorch
	    {44, 44, 44},						   // au1HSyncWidth
	    {220, 220, 220},                       // au2HBackPorch

	    {1406, 1350, 1125},                    // au2VTotal
	    1080,                                  // u2GraphModeHeight
	    {320, 264, 39},                        // au2VFrontPorch
	    {3, 3, 3},                             // au1VSyncWidth
	    {3, 3, 3},                             // au2VBackPorch

		GM_FORMAT_CASE2_MMODE0 | GM_LANE_4,     // u4GControlWord
		63,                                     // u1FrameRateMax
		47,                                     // u1FrameRateMin

		HSYNC_LOW,
		VSYNC_LOW,
	},
};
#endif

CHAR* GetPanelName(UINT32 u4Index)
{
#if defined(CC_PANEL_FROM_FLASH)
    UNUSED(u4Index);
    return FlashPanel_GetIDString();
#else
    switch (u4Index)
    {
    case PANEL_XGA: return "PANEL_XGA(PANEL_DEFAULT)";
    case PANEL_SXGA: return "PANEL_SXGA";

    case PANEL_CMO_154_C1_P1: return "PANEL_CMO_154_C1_P1";
    case PANEL_CMO_154_I2_L2: return "PANEL_CMO_154_I2_L2";
    case PANEL_CMO_154_I2_P1: return "PANEL_CMO_154_I2_P1";
    case PANEL_CMO_154_I2_P2: return "PANEL_CMO_154_I2_P2";
    case PANEL_CMO_154_I5_P2: return "PANEL_CMO_154_I5_P2";
    case PANEL_CMO_19_A1: return "PANEL_CMO_19_A1";
    case PANEL_CMO_19_A1_P2: return "PANEL_CMO_19_A1_P2";
    case PANEL_CMO_22_Z1: return "PANEL_CMO_22_Z1";
    case PANEL_CMO_22_Z1_P1: return "PANEL_CMO_22_Z1_P1";
    case PANEL_CMO_26_B1: return "CMO_V260B1_L01";
    case PANEL_CMO_26_B1_L04: return "CMO_V260B1_L01_L04 (TEST)";
    case PANEL_CMO_27_W1: return "PANEL_CMO_27_W1";
    case PANEL_CMO_27_B1: return "PANEL_CMO_27_B1";
    case PANEL_CMO_315_B1: return "CMO_V315B1_L01";
    case PANEL_CMO_32_B1: return "CMO_V320B1_L04";
    case PANEL_CMO_37_H1: return "PANEL_CMO_37_H1";
    case PANEL_CMO_42_H1: return "PANEL_CMO_42_H1";
    case PANEL_CMO_42_H1_L5: return "CMO_V420H1_L05";
    case PANEL_CMO_42_H1_L8: return "CMO V420H1_L08";
    case PANEL_CMO_42_H1_L5_DELTA: return "CMO_V420H1_L05_DELTA (TEST)";

    case PANEL_AUO_20_XW2: return "PANEL_AUO_20_XW2";
    case PANEL_AUO_26_XW3: return "PANEL_AUO_26_XW3";
    case PANEL_AUO_315_XW1_VH: return "AUO_T315XW01_VH";
    case PANEL_AUO_315_XW1_VG: return "AUO_T315XW01_VG";
    case PANEL_AUO_315_XW2_V0: return "AUO_T315XW02_V0";
    case PANEL_AUO_315_XW2_V1: return "AUO_T315XW02_V1";
    case PANEL_AUO_315_XW2_V3: return "AUO_T315XW02_V3";
    case PANEL_AUO_315_XW2_V6: return "PANEL_AUO_315_XW2_V6";
    case PANEL_AUO_32_XW1: return "AUO_T320XW01_V5";
    case PANEL_AUO_32_XW2: return "PANEL_AUO_32_XW2";
    case PANEL_AUO_37_XW1_V1: return "AUO_T370XW01_V1";
    case PANEL_AUO_37_XW1_V2: return "AUO_T370XW01_V2";
    case PANEL_AUO_37_XW2: return "AUO_T370XW01_V0";
    case PANEL_AUO_37_XW2_V5: return "PANEL_AUO_37_XW2_V5";
    case PANEL_AUO_37_HW1: return "PANEL_AUO_37_HW1";
    case PANEL_AUO_37_HW2: return "PANEL_AUO_37_HW2";
    case PANEL_AUO_40_XW1_V0: return "AUO_T400XW01_V0 (TEST)";
    case PANEL_AUO_42_HW1_V1: return "PANEL_AUO_42_HW1_V1 (TEST)";
    case PANEL_AUO_42_XW1_V3: return "AUO_T420XW01_V3";
    case PANEL_AUO_42_HW1_V0: return "PANEL_AUO_42_HW1_V0";
    case PANEL_AUO_42_HW1_V3: return "PANEL_AUO_42_HW1_V3";

    case PANEL_LG_26_W1: return "PANEL_LG_26_W1";
    case PANEL_LG_26_WX2: return "PANEL_LG_26_WX2";
    case PANEL_LG_32_W1: return "PANEL_LG_32_W1";
    case PANEL_LG_32_WX3_SLB1: return "PANEL_LG_32_WX3_SLB1";
    case PANEL_LG_37_WX1: return "PANEL_LG_37_WX1";
    case PANEL_LG_37_WX1_SL2: return "PANEL_LG_37_WX1_SL2";
    case PANEL_LG_37_WX1_SLA1: return "PANEL_LG_37_WX1_SLA1";
    case PANEL_LG_37_WU1: return "PANEL_LG_37_WU1";
    case PANEL_LG_42_W2: return "PANEL_LG_42_W2";
    case PANEL_LG_42_W2_SLA2: return "PANEL_LG_42_W2_SLA2";
    case PANEL_LG_42_WU2: return "PANEL_LG_42_WU2";
    case PANEL_LG_42_WU3_SLA1: return "PANEL_LG_42_WU3_SLA1";
    case PANEL_LG_42_WU5_SLB1: return "PANEL_LG_42_WU5_SLB1";
    case PANEL_LG_42_WX4_SLB1: return "PANEL_LG_42_WX4_SLB1";
    case PANEL_LG_47_WU4: return "PANEL_LG_47_WU4";
    case PANEL_LG_50_X4_PDP: return "PANEL_LG_50_X4_PDP";

    case PANEL_SS_19_M2: return "PANEL_SS_19_M2";
    case PANEL_SS_23_W2_L1: return "PANEL_SS_23_W2_L1";
    case PANEL_SS_32_WT_L5: return "PANEL_SS_32_WT_L5";
    case PANEL_SS_40_WT: return "SAMSUNG_LTA400WT_L11";
    case PANEL_SS_40_FHD: return "SAMSUNG_SS_40_FHD";
    case PANEL_SS_40_FHD_10BIT: return "SAMSUNG_SS_40_FHD_10BIT";
    case PANEL_SS_40_WT_L17: return "SAMSUNG_LTA400WT_L17 (TEST)";
    case PANEL_SS_40_HH_LH2:        return "SAMSUNG_LTA400HH_LH2";
    case PANEL_SS_46_HS_L03: return "SAMSUNG_LTA460HS_L03";
    case PANEL_SS_46_HH_L01: return "SAMSUNG_LTA460HH_L01";
    case PANEL_SS_46_HS_L03_BESTBUY: return "SAMSUNG_LTA460HS_L03_BESTBUY (TEST)";

    case PANEL_CLAA_15_XP3: return "PANEL_CLAA_15_XP3";
    case PANEL_CLAA_32_WB2: return "PANEL_CLAA_32_WB2";

    case PANEL_HSD_19_MGW1: return "PANEL_HSD_19_MGW1";
    case PANEL_QD_32_HL1: return "AUO_T315XW01_VC(T)";
    case PANEL_QD_32_HL1_W: return "AUO_T315XW01_VC(W)";

    case PANEL_PV_154_LCM_C01: return "PANEL_PV_154_LCM_C01";
    case PANEL_PV_154_LCM_C03: return "PANEL_PV_154_LCM_C03";
    case PANEL_PV_154_LCM_C04: return "PANEL_PV_154_LCM_C04";
    case PANEL_PV_154_LCM_C05: return "PANEL_PV_154_LCM_C05";
    case PANEL_PV_201_TVM_C01: return "PANEL_PV_201_TVM_C01";
    case PANEL_PV_260_TVM_A01H: return "Proview_PV260TVM_A01H (TEST)";
    case PANEL_PV_260_TVM_A04H: return "Proview_PV260TVM_A04H (TEST)";
    case PANEL_PV_260_TVM_A11H: return "Proview_PV260TVM_A11H (TEST)";
    case PANEL_PV_260_TVM_C01H: return "Proview_PV260TVM_C01H (TEST)";
    case PANEL_PV_260_TVM_C11H: return "Proview_PV260TVM_C11H (TEST)";
    case PANEL_PV_320_TVM_A01H: return "Proview_PV320TVM_A01H";
    case PANEL_PV_320_TVM_A02H: return "Proview_PV320TVM_A02H";
    case PANEL_PV_320_TVM_A03H: return "Proview_PV320TVM_A03H (TEST)";
    case PANEL_PV_320_TVM_A12H: return "Proview_PV320TVM_A12H (TEST)";
    case PANEL_PV_320_TVM_A21H: return "Proview_PV320TVM_A21H (TEST)";
    case PANEL_PV_320_TVM_C01:  return "Proview_PV320TVM_C01";
    case PANEL_PV_320_TVM_C02H: return "Proview_PV320TVM_C02H (TEST)";
    case PANEL_PV_320_TVM_C12H: return "Proview_PV320TVM_C12H";

    case PANEL_INNOLUX_19_AW01: return "PANEL_INNOLUX_19_AW01";

    case PANEL_AUO_315_XW2_V6_8280: return "PANEL_AUO_315_XW2_V6_8280";
    case PANEL_LG_42_WU5_SLB1_8280: return "PANEL_LG_42_WU5_SLB1_8280";
    case PANEL_LG_42_WX4_SLB1_8280: return "PANEL_LG_42_WX4_SLB1_8280";


    case PANEL_FHD60_LVDS_CAP: 	    return "PANEL_FHD60_LVDS_CAP";
    case PANEL_FHD120_LVDS_CAP:     return "PANEL_FHD120_LVDS_CAP";
    case PANEL_WXGA60_LVDS_CAP:     return "PANEL_WXGA60_LVDS_CAP";
    case PANEL_WXGA120_LVDS_CAP:    return "PANEL_WXGA120_LVDS_CAP";
    case PANEL_SLT_FHD_60Hz_10BIT:       return "PANEL_SLT_FHD_60Hz_10BIT";
    case PANEL_SLT_FHD_120Hz_10BIT:    return "PANEL_SLT_FHD_120Hz_10BIT";

    //For SBTVD
    case PANEL_INNOLUX_19_AW02_LIPS_LOW:	return "PANEL_INNOLUX_19_AW02_LIPS_LOW";
    case PANEL_CMO_216_B1_L01_LIPS_LOW:	return "PANEL_CMO_216_B1_L01_LIPS_LOW";
    case PANEL_CMO_260_B1_L11_LIPS_HIGH:	return "PANEL_CMO_260_B1_L11_LIPS_HIGH";

    case PANEL_CMO_216_B1_L1: return "PANEL_CMO_216_B1_L1";
    case PANEL_AUO_19_PW1: return "PANEL_AUO_19_PW1";
    case PANEL_AUO_19_XW1: return "PANEL_AUO_19_XW1";                    //sammi modifed 080912
    case PANEL_CMO_22_B1_L01: return "PANEL_CMO_22_B1_L01";            //sammi modified 080912
    case PANEL_AUO_37_XW2_VC: return "PANEL_AUO_37_XW2_VC";         //sammi modified 080912
    case PANEL_AUO_26_WT_1: return "PANEL_AUO_26_WT_1"; // add by stanley 080908
    case PANEL_SS_32_WT_1: return "PANEL_SS_32_WT_1";	// add by stanley 080908
    case PANEL_SS_52_FHD: return "SAMSUNG_SS_52_FHD";
    case PANEL_SS_32_WT_1_WXGA2: return "PANEL_SS_32_WT_1_WXGA2";
    case PANEL_AUO_315_XW2_VS_WXGA2: return "PANEL_AUO_315_XW2_VS_WXGA2";
    case PANEL_SLT_FHD_60Hz_10BIT_AUTO_NS: return "PANEL_SLT_FHD_60Hz_10BIT_AUTO_NS"; //SONY 2K
    case PANEL_VIDEO_1080P_JEIDA: return "PANEL_VIDEO_1080P_JEIDA"; // SONY 2K
    case PANEL_AUO_T315XW02_VS_WXGA1: return "PANEL_AUO_T315XW02_VS_WXGA1";// WXGA1
    case PANEL_AUO_T315XW02_VS_WXGA2: return "PANEL_AUO_T315XW02_VS_WXGA2"; // WXGA2
    case PANEL_SS_32NC_LBA_SESL_FHD: return "PANEL_SS_32NC_LBA_SESL_FHD";
    case PANEL_AUO_46_HW03_V4: return "PANEL_AUO_46_HW03_V4"; // WXGA2 // 1920x1080 (240Hz for MT5395+2xMT8282)
    case PANEL_AUO_65_HW02_V1: return "PANEL_AUO_65_HW02_V1"; // WXGA2 // 1920x1080 (120Hz PR Panel)

    case PANEL_LG_42_WUD_SAC1_10BIT: return "PANEL_LG_42_WUD_SAC1_10BIT";

    case PANEL_LG_42_WU5_SLB1_VB1: return "PANEL_LG_42_WU5_SLB1_VB1";

    case PANEL_MLVDS_LGDV4FHD60: return "PANEL_MLVDS_LGDV4FHD60";
    case PANEL_MLVDS_LGDV4FHD120: return "PANEL_MLVDS_LGDV4FHD120";
    case PANEL_MLVDS_LGDV4GIPFHD60: return "PANEL_MLVDS_LGDV4GIPFHD60";
    case PANEL_MLVDS_LGDV4GIPFHD120: return "PANEL_MLVDS_LGDV4GIPFHD120";
    case PANEL_ChangHong_PDP: return "PANEL_ChangHong_PDP";
    case PANEL_MLVDS_LGDV5GIPFHD60: return "PANEL_MLVDS_LGDV5GIPFHD60";
    case PANEL_MLVDS_LGDV5GIPFHD120: return "PANEL_MLVDS_LGDV5GIPFHD120";
    case PANEL_MLVDS_LGDV6GIPFHD60: return "PANEL_MLVDS_LGDV6GIPFHD60";
    case PANEL_MLVDS_LGDV6GIPFHD120: return "PANEL_MLVDS_LGDV6GIPFHD120";
    case PANEL_MLVDS_AUOFHD60: return "PANEL_MLVDS_AUOFHD60";
    case PANEL_MLVDS_AUOFHD120: return "PANEL_MLVDS_AUOFHD120";
    case PANEL_MLVDS_SHAFHD60: return "PANEL_MLVDS_SHAFHD60";
    case PANEL_MLVDS_SHAFHD120: return "PANEL_MLVDS_SHAFHD120";
    case PANEL_MLVDS_FUNAWXGA: return "PANEL_MLVDS_FUNAWXGA";
    case PANEL_MLVDS_FUNAFHD60: return "PANEL_MLVDS_FUNAFHD60";
    case PANEL_MLVDS_HIWXGA09: return "PANEL_MLVDS_HIWXGA09";
    case PANEL_MLVDS_HIWXGA10: return "PANEL_MLVDS_HIWXGA10";
    case PANEL_MLVDS_WXGA: return "PANEL_MLVDS_WXGA";
    case PANEL_MLVDS_CMO2011FHD60: return "PANEL_MLVDS_CMO2011FHD60";
    case PANEL_MLVDS_PANASONIC32WXGA1P6P: return "PANEL_MLVDS_PANASONIC32WXGA1P6P";
    case PANEL_LG_47_EUH_PR: return "PANEL_LG_47_EUH_PR";
    case PANEL_CMO_M236H5_L0A: return "PANEL_CMO_M236H5_L0A";
    case PANEL_LG_42_WUD_SAC1_10BIT_NS: return "PANEL_LG_42_WUD_SAC1_10BIT_NS";
    case PANEL_LG_42_WU5_SLB1_PR: return "PANEL_LG_42_WU5_SLB1_PR";
    case PANEL_LG_42_WU5_SLB1_SG: return "PANEL_LG_42_WU5_SLB1_SG";
    case PANEL_CMO_M236H3_LHH3:return "PANEL_CMO_M236H3_LHH3";
    case PANEL_LG_42_WU5_SLB1_AS3693: return "PANEL_LG_42_WU5_SLB1_AS3693";
    case PANEL_LG_42_WU5_SLB1_IWATT: return"PANEL_LG_42_WU5_SLB1_IWATT";
	case PANEL_LG_42_WU5_SLB1_SG_BEX: return "PANEL_LG_42_WU5_SLB1_SG_BEX";
	case PANEL_AUO_46_HW03_V4_FHD240: return"PANEL_AUO_46_HW03_V4_FHD240";
	case PANEL_AUO_46_HW03_V3: return"PANEL_AUO_46_HW03_V3";
	case PANEL_LG_50_CINEMA: return"PANEL_LG_50_CINEMA";
	case PANEL_FOR_8283: return "PANEL_FOR_8283";
	case PANEL_AUO_T576DC01_V1: return "PANEL_AUO_T576DC01_V1";
	case PANEL_CMI_V420H2_LS1: return "PANEL_CMI_V420H2_LS1";
	case PANEL_LC420EUD_SDF1_PR: return "PANEL_LC420EUD_SDF1_PR";
	case PANEL_AUO_T645HW05_V0: return "PANEL_AUO_T645HW05_V0";
	case PANEL_SS_42_PDP: return "PANEL_SS_42_PDP";
    case PANEL_LVDS_TO_8282: return "PANEL_LVDS_TO_8282";
	case PANEL_SS_58_PDP: return "PANEL_SS_58_PDP";
	case PANEL_EPI_LGDV12GIPFHD60: return "PANEL_EPI_LGDV12GIPFHD60";
	case PANEL_AUO_T315HB01_V1: return "PANEL_AUO_T315HB01_V1";
	case PANEL_CMI_42_HW03_V3: return "PANEL_CMI_42_HW03_V3";
	case PANEL_EPI_LGDV12GIPFHD120: return "PANEL_EPI_LGDV12GIPFHD120";
	case PANEL_LG_32_PDP: return "PANEL_LG_32_PDP";
	case PANEL_SS_L40V8200_120SG: return "PANEL_SS_L40V8200_120SG";
	case PANEL_PDP_PD5L_XGA60: return "PANEL_PDP_PD5L_XGA60";
	case PANEL_LG_42_WU3_SLA1_HISPD: return "PANEL_LG_42_WU3_SLA1_HISPD";
    case PANEL_MLVDS_CMO2011FHD120: return "PANEL_MLVDS_CMO2011FHD120";
	case PANEL_MLVDS_SS_MB4: return "PANEL_MLVDS_SS_MB4";
	case PANEL_MLVDS_SS_MB7: return "PANEL_MLVDS_SS_MB7";
	case PANEL_MLVDS_PANASONIC32WXGA2P3P: return "PANEL_MLVDS_PANASONIC32WXGA2P3P";
	case PANEL_VIDEO_1080P: return "PANEL_VIDEO_1080P";
	case PANEL_VIDEO_720P: return "PANEL_VIDEO_720P";
	case PANEL_VIDEO_576P: return "PANEL_VIDEO_576P";
	case PANEL_VIDEO_480P: return "PANEL_VIDEO_480P";
	case PANEL_LG_42_WU3_HISPD_2CH: return "PANEL_LG_42_WU3_HISPD_2CH";
	case PANEL_MLVDS_PANASONIC42FHD120: return "PANEL_MLVDS_PANASONIC42FHD120";
	case PANEL_MLVDS_LGD32WXGAV9: return "PANEL_MLVDS_LGD32WXGAV9";
	case PANEL_SLT_FHD_60Hz_10BIT_AUTO:  return "PANEL_SLT_FHD_60Hz_10BIT_AUTO";
	case PANEL_MLVDS_HISENSE_HE420FF:return "PANEL_MLVDS_HISENSE_HE420FF";
	case PANEL_MLVDS_HISENSE32WXGA2P3P: return "PANEL_MLVDS_HISENSE32WXGA2P3P";	
	case PANEL_MLVDS_HISENSE32WXGA1P6P: return "PANEL_MLVDS_HISENSE32WXGA1P6P";
	case PANEL_MLVDS_CHINASTARWXGA8BIT   : return "PANEL_MLVDS_CHINASTARWXGA8BIT";
	case PANEL_MLVDS_LGD32WXGAV12   : return "PANEL_MLVDS_LGD32WXGAV12";
	case PANEL_MLVDS_LGD32WXGAV13   : return "PANEL_MLVDS_LGD32WXGAV13";
	case PANEL_EPI_LGDV13GIPFHD60: return "PANEL_EPI_LGDV13GIPFHD60";
	case PANEL_EPI_LGDV13GIPFHD120: return "PANEL_EPI_LGDV13GIPFHD120";
	case PANEL_EPI_LGDV14GIPFHD60_8LANE: return "PANEL_EPI_LGDV14GIPFHD60_8LANE";
	case PANEL_EPI_LGDV14GIPFHD120_8LANE: return "PANEL_EPI_LGDV14GIPFHD120_8LANE";
	case PANEL_EPI_LGDV14GIPFHD60_4LANE: return "PANEL_EPI_LGDV14GIPFHD60_4LANE";
	case PANEL_VB1_V420DK1_KS1: return "PANEL_VB1_V420DK1_KS1";
	case PANEL_4K1K_VB1: return "PANEL_4K1K_VB1";
	case PANEL_THINE_THAV226_FHD60_VB1: return "PANEL_THINE_THAV226_FHD60_VB1";
	case PANEL_4K2K30_VB1: return "PANEL_4K2K30_VB1";
	case PANEL_FHD_BACKEND: return "PANEL_FHD_BACKEND";
	
	case PANEL_AUO_4K2K: return "PANEL_AUO_4K2K";
	case PANEL_AUO_4K2K_FHD60: return "PANEL_AUO_4K2K_FHD60";
	case PANEL_CMI_4K2K: return "PANEL_CMI_4K2K";
	case PANEL_CMI_4K2K_FHD120: return "PANEL_CMI_4K2K_FHD120";
	case PANEL_CMO_V580DK1_LS1_INX: return "PANEL_CMO_V580DK1_LS1_INX";
	case PANEL_CSOT_4K2K_FHD60: return "PANEL_CSOT_4K2K_FHD60";
	case PANEL_CMO_V580DK1_LS1_INX_VB1: return "PANEL_CMO_V580DK1_LS1_INX_VB1";
	case PANEL_CMO_V580DK1_LS1_INX_60_VB1 : return "PANEL_CMO_V580DK1_LS1_INX_60_VB1";
	case PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST : return "PANEL_CMO_V580DK1_LS1_INX_60_VB1_TEST";

	case PANEL_4K_BACKEND: return "PANEL_4K_BACKEND";
	case PANEL_4K2KDD: return "PANEL_4K2KDD";
	case PANEL_4K2KDD_2SEC: return "PANEL_4K2KDD_2SEC";
	case PANEL_VB1_4K2KDD_GRAPH: return "PANEL_VB1_4K2KDD_GRAPH";
	case PANEL_VB1_480P: return "PANEL_VB1_480P";
	case PANEL_VB1_720P: return "PANEL_VB1_720P";
	case PANEL_VB1_1080P: return "PANEL_VB1_1080P";
	case PANEL_VB1_1080P24: return "PANEL_VB1_1080P24";
	case PANEL_VB1_576P: return "PANEL_VB1_576P";
	case PANEL_VB1_2160P: return "PANEL_VB1_2160P";
	case PANEL_VB1_2160P30: return "PANEL_VB1_2160P30";
	case PANEL_VB1_2160P_2SEC: return "PANEL_VB1_2160P_2SEC";	
	case PANEL_VB1_4K2K120: return "PANEL_VB1_4K2K120";
	case PANEL_VB1_8K4K30: return "PANEL_VB1_8K4K30";
    default: return "NO NAME";
    }
#endif
}

#ifdef SUPPORT_TX_SWAP_CONTROL
static const UINT64 _au4PanelLvds10bitPinMap[24] =
{
 // Port A (PAD-A) LLV0~LLV9, LLK_CK0,LLK_CK1
 LVDS_AO0, LVDS_AO1, LVDS_AO2, LVDS_AO3, LVDS_AO4,  // AO0,AO1,AO2,AO3,AO4,
 LVDS_BE1, LVDS_BO4, LVDS_BO3, LVDS_BO1, LVDS_AE4,  // AE0,AE1,AE2,AE3,AE4,
 LVDS_ACLK1, LVDS_BO2,                            // AOCK, AECK 
 // Port B (PAD-B) RLV0~RLV9, RLK_CK0,RLK_CK1
 LVDS_AE3, LVDS_AE2, LVDS_AE1, LVDS_BCLK1, LVDS_BO0,   // BO0,BO1,BO2,BO3,BO4,
 LVDS_BCLK2, LVDS_AE0, LVDS_BE2, LVDS_BE3, LVDS_BE4,   // BE0,BE1,BE2,BE3,BE4,
 LVDS_BE0, LVDS_BCLK2,                             // BOCK, BECK
};

static const UINT64 _au4PanelLvds10bitPNSwap[24] =
{
 // Port A (PADA) RLV0~RLV9, LLK_CK0,LLK_CK1
 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0,
 0, 0,
 // Port B (PADB) RLV0~RLV9, RLK_CK0,RLK_CK1
 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0,
 0, 0
};
static UINT64 *_pu4PanelLVDSLaneSwap = (UINT64 *)_au4PanelLvds10bitPinMap;
static UINT64 *_pu4PanelLVDSLanePNSwap = (UINT64 *)_au4PanelLvds10bitPNSwap;

// Set Lane swap table
// A + B + C + D
static const UINT64 _au4PanelVB1LaneSwap[32] =
{
	// 1st 8 lanes: 0x8064~0x8068
	VB1_DATA_LANE0, VB1_DATA_LANE1, VB1_DATA_LANE2, VB1_DATA_LANE3,
	VB1_DATA_LANE4, VB1_DATA_LANE5, VB1_DATA_LANE6, VB1_DATA_LANE7,
	
	// 3rd 8 lanes(osd 8 lanes): 0x8068~0x806C
	VB1_DATA_LANE8, VB1_DATA_LANE9, VB1_DATA_LANE10, VB1_DATA_LANE11,
	VB1_DATA_LANE12, VB1_DATA_LANE13, VB1_DATA_LANE14, VB1_DATA_LANE15,

	// 2nd 8 lanes: 0x806C~0x8070
	VB1_GRAPH_LANE0, VB1_GRAPH_LANE1, VB1_GRAPH_LANE2, VB1_GRAPH_LANE3,
	VB1_GRAPH_LANE4, VB1_GRAPH_LANE5, VB1_GRAPH_LANE6, VB1_GRAPH_LANE7,

    // 4th 8 lanes: 0x8074~0x8078
    VB1_DATA_LANE24, VB1_DATA_LANE25, VB1_DATA_LANE26, VB1_DATA_LANE27,
	VB1_DATA_LANE28, VB1_DATA_LANE29, VB1_DATA_LANE30, VB1_DATA_LANE31
};

// Set Lane P/N swap table
static const UINT64 _au4PanelVB1LanePNSwap[40] =
{
	// 1st 8 lanes: 0x9820
	0,0,0,0,
	0,0,0,0,
	
	// 2nd 8 lanes: 0x9220
	0,0,0,0,
	0,0,0,0,

    // 3rd 8 lanes: 0x9420
	0,0,0,0,
	0,0,0,0,

    // 4th 8 lanes: 0x9620
	0,0,0,0,
	0,0,0,0,

    // osd 8 lanes: 0x9A20
	0,0,0,0,
	0,0,0,0
};

static UINT64 *_pu4PanelVB1LaneSwap = (UINT64 *)_au4PanelVB1LaneSwap;
static UINT64 *_pu4PanelVB1LanePNSwap = (UINT64 *)_au4PanelVB1LanePNSwap;
#endif

#ifdef SUPPORT_LVDS_TO_GPIO
static const UINT32 _au4PanelPortB2GPO[28] = 
{
	// 0 ~ 13
	// O0N, O0P, O1N, O1P, O2N, O2P, O3N, O3P, O4N, O4P, O5N, O5P, OCKN, OCKP, 
       0,   0,  0,   0,   1,  1,   1,   0,  1,   1,   0,   0,  0,    1,
    // 14 ~ 27
	// E0N, E0P, E1N, E1P, E2N, E2P, E3N, E3P, E4N, E4P, E5N, E5P, ECKN, ECKP,
	   0,  1,   1,  1,  0,   1,  0,   0,  1,  1,   0,  0,  0,    0
};

static const UINT32 _au4PanelPortB2GPI[28] = 
{
	// 0 ~ 13
	// O0N, O0P, O1N, O1P, O2N, O2P, O3N, O3P, O4N, O4P, O5N, O5P, OCKN, OCKP, 
       0,   0,  0,   1,   0,  0,   0,   0,  0,   0,   0,   0,  1,    0,
    // 14 ~ 27
	// E0N, E0P, E1N, E1P, E2N, E2P, E3N, E3P, E4N, E4P, E5N, E5P, ECKN, ECKP,
	   1,  0,   0,  0,  1,   0,  1,   0,  0,  0,   0,  0,  1,    1
};

static UINT32 *_pu4PanelPortB2GPO = (UINT32*)_au4PanelPortB2GPO;
static UINT32 *_pu4PanelPortB2GPI = (UINT32*)_au4PanelPortB2GPI;
#endif
