/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author: sakamoto $
 * $Date: 2017/10/16 $
 * $RCSfile: aud_if.h,v $
 * $Revision: #4 $
 *
 *---------------------------------------------------------------------------*/

/** @file aud_if.h
 * Declare audio related structure and function table.
 */

#ifndef X_AUD_IF_H
#define X_AUD_IF_H

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "x_typedef.h"
#include "dmx_if.h"
#include "feeder_if.h"
#include "sif_if.h"
#include "stc_drvif.h"
#include "u_drv_cust.h"
#include "aproc_common_if.h"
#ifdef CC_AUD_M3_FEATURE
#include "hdmi_earc_if.h"
#endif

//-----------------------------------------------------------------------------
// Audio table enum and structure
//-----------------------------------------------------------------------------
#define DUAL_AUD_DEC_CONNECT 0
#define DUAL_AUD_DEC_DISCONNECT 1

#define AUD_DSP_NUM                     ((UINT8)2)

#if defined (CC_AUD_4_DECODER_SUPPORT)
#define AUD_DEC_NUM                     ((UINT8)4)
#elif defined (CC_MT5391_AUD_3_DECODER)
#define AUD_DEC_NUM                     ((UINT8)3)
#else
#define AUD_DEC_NUM                     ((UINT8)2)
#endif

#define AUD_DEC_MAX                     ((UINT8)4)

#define AUD_CPU_DEC_MAX                 ((UINT8)3)

// Decoder ID check macro
#define AUD_DEC_ID_VALIDATE(u1DecId)    if (u1DecId>=AUD_DEC_NUM)\
                                            ASSERT(0)
#define AUD_CPUDEC_ID_VALIDATE(u1DecId)    if (u1DecId >= AUD_CPU_DEC_MAX)\
                                                ASSERT(0)
#define AUD_DEC_ID_VALIDATE_2(u1DecId)  if (u1DecId>AUD_DEC_AUX)\
                                            ASSERT(0)
#define AUD_DEC_ID_VALIDATE_3(u1DecId)  if (u1DecId>AUD_DEC_THIRD)\
                                            ASSERT(0)
#define AUD_DEC_ID_VALIDATE_SUB(u1DecId)  if (u1DecId>AUD_DEC_SUB)\
                                                ASSERT(0)
#define AUD_DSP_ID_VALIDATE(u1DspId)    VERIFY((u1DspId == AUD_DSP0) || (u1DspId == AUD_DSP1))

#define AUD_ADPAN_CHANNEL_VALIDATE(eChannel)    if (eChannel>=AUD_ADPAN_CH_NUM)\
                                                    ASSERT(0)

#define AUD_OUT_PORT_VALIDATE(eOut)     if ((AUD_OUT_PORT_T)eOut>=AUD_OUT_PORT_NUM)\
                                                ASSERT(0)

#define AUD_NULL_PT_CHECK(pt)                                                                              \
                {                                                                                                               \
                    if ((void*)(pt) == NULL)                                                                           \
                    {                                                                                                           \
                        LOG(0, "[%s][%d] pointer allocate Failed\n", __FUNCTION__, __LINE__);   \
                        break;                                                                                              \
                    }                                                                                                           \
                }

#if defined(CC_MT5891) || defined(CC_MT5893) || defined(CC_MT5895)
#define AUD_AOUT_NO     ((UINT8)3)
#else
#define AUD_AOUT_NO     ((UINT8)2)
#endif

#define APLL_ID_MAIN            0
#define APLL_ID_AUX             1
#if defined(CC_MT5891) || defined(CC_MT5893) || defined(CC_MT5895)
#define APLL_ID_THIRD           2
#define APLL_ID_4TH             3

#endif

#define UNUSED_GPIO     (0xffffffff)
#define SW_NO_USED      (0xff)
#define NO_USED         (0xff)

#define MUX_SW_0        1, 1
#define MUX_SW_1        0, 1
#define MUX_SW_2        1, 0
#define MUX_SW_3        0, 0

#define DEFAULT_0_DB    0x20000

#ifdef CC_AUD_MULTI_BUFFER_MEMORY_PLAY
#define MAX_MEM_BUFFER_NUMBER       (16)
#endif

#define CH_EQ_BAND_NO   5
#define AUD_EQ_IIR_BAND_NUM     8

#define PEQ_SETTING_MAX 9
#define PEQ_GAIN_NUM_FOR_VOL    4

#define EQ_BAND_GAIN_DB_MAX             14

#ifdef CC_AUD_3_5K_TREBLE
// Bass/Treble constant
#define BASS_TREBLE_GAIN_DB_MAX         14
// Bass Treble
#define BASS_TREBLE_GAIN_MAX            ((BASS_TREBLE_GAIN_DB_MAX*4) + 1) // range -14 ~ +14 dB, 0.5dB/Step
#else
// Bass/Treble constant
#define BASS_TREBLE_GAIN_DB_MAX         14
// Bass Treble
#define BASS_TREBLE_GAIN_MAX ( (14*2) + 1) // range -14 ~ +14 dB, 1 dB/Step
#endif

#define BBE_LEVEL_MAX                   5
#define BBE_PROCESS_MIN                 (3*2)
#define BBE_PROCESS_MAX                 (12*2)
#define BBE_LO_CONTOUR_MIN              (3*2)
#define BBE_LO_CONTOUR_MAX              (12*2)
#define BBE_3D_GAIN_MAX                 100

#define CDNOTCH_Q_IDX_MIN               0
#define CDNOTCH_Q_IDX_MAX               37
#define CDNOTCH_FC_MIN                  60
#define CDNOTCH_FC_MAX                  15000

#define AUD_VOLUME_OFFSET_IDX_NUM 13
#define AUD_SW_VOLUME_OFFSET_IDX_NUM 25

#ifdef CC_SUPPORT_STR
#define RESUME_BASS_ENABLE             0
#define RESUME_TREBLE_ENABLE           1
#define RESUME_VIRSUR1_ENABLE          2
#define RESUME_VIRSUR2_ENABLE          3
#define RESUME_EQ_ENABLE               4
#define RESUME_AD_SPEAKER_ENABLE       5
#define RESUME_AD_HP_EnABLE            6
#define RESUME_AD_SPEAKER_HP_EnABLE    7
#define RESUME_INDEX_MAX               8
#endif

#if defined(CC_AUD_SM_SUPPORT)
#define INDIA_GAIN_SP_DB   4.5
#define INDIA_GAIN_LO_DB   0
#define INDIA_GAIN_HP_DB   0
#elif defined(CC_AUD_W_SUPPORT)
#define INDIA_GAIN_SP_DB   5
#define INDIA_GAIN_LO_DB   5
#define INDIA_GAIN_HP_DB   5
#else
#define INDIA_GAIN_SP_DB   5
#define INDIA_GAIN_LO_DB   5
#define INDIA_GAIN_HP_DB   5
#endif

//#ifndef ARM_SUPPORT_SRS_SS
//#define ARM_SUPPORT_SRS_SS
//#endif

#ifndef ARM_SUPPORT_SRS_TS3D
#define ARM_SUPPORT_SRS_TS3D
#endif

#ifdef CC_AUD_SUPPORT_AUTO_MUTE
// Default settings of Aout Auto Mute
#define AOUT_AUTO_MUTE_ENABLE TRUE
#define AOUT_AUTO_MUTE_THRESHOLD 100
#endif // CC_AUD_SUPPORT_AUTO_MUTE

// cpu_mask is unsigned int that 1 mean core 0
//                               2           1
//                               4           2

#define ISR_AFFINITY_CORE0  1
#define ISR_AFFINITY_CORE1  2
#define ISR_AFFINITY_CORE2  4

/* SPDIF */

// channel status
typedef struct
{
    UINT8 u1Mode; // [0] 0 = consumer mode
    UINT8 u1SampleMode; // [1] 0 = linear PCM, 1 = non-linear PCM (RAW)
    UINT8 u1Copyright; // [2] 0 = copyright is asserted, 1 = no copyright
    UINT8 u1PreEmphasis; // [3,5] pre-emphasis
    UINT8 u1Category; // [8,15] category code, MSB 1 bit = L-bit
    UINT8 u1SourceNum; // [16, 19] source number
    UINT8 u1CannelNum; // [20, 23] channel number
    UINT8 u1SampleRate; // [24, 27] sample rate
    UINT8 u1ClockAccuray; // [28, 29] clock accuracy
    UINT8 u1WordLength; // [32, 35] word length
    UINT8 u1OrgSampleRate; // [36, 39] original sample rate
}   SPDIF_CHANNEL_STATUS_T;

typedef struct {
    UPTR u4Str;// start address
    UINT32 u4End; // total block number
    UINT32 u4Size; // bytes in one block per channel
    UINT32 u4Ch; // channel number
    UINT32 u4Wcurp; // Write port, current pointer
    UINT32 u4Rcurp; // Read port, current pointer
    UINT32 u4Wp; // write pointer
    UINT32 u4Rp; // read pointer
    UINT32 u4ProCnt; //process count
    UINT32 u4SiBase; // sideinformation base address
} _IO_BUFFER_T;

// sample rate in channel status
#define SPDIF_SAMPLE_RATE_22KHZ 0x4 // 22.05 kHz
#define SPDIF_SAMPLE_RATE_44KHZ 0x0 // 44.1 kHz
#define SPDIF_SAMPLE_RATE_88KHZ 0x8 // 88.2 kHz
#define SPDIF_SAMPLE_RATE_176KHZ 0xc // 176.4 kHz
#define SPDIF_SAMPLE_RATE_24KHZ 0x6 // 24 kHz
#define SPDIF_SAMPLE_RATE_48KHZ 0x2 // 48 kHz
#define SPDIF_SAMPLE_RATE_96KHZ 0xa // 96 kHz
#define SPDIF_SAMPLE_RATE_192KHZ 0xe // 192 kHz
#define SPDIF_SAMPLE_RATE_32KHZ 0x3 // 32 kHz
#define SPDIF_SAMPLE_RATE_768KHZ 0x9 //768kHz
#define SPDIF_SAMPLE_RATE_NOT_INDICATE 0x1 // not indicated

// word length in channel status
#define SPDIF_WORDLENGTH_MAX24_NOT_INDICATE 0x1 // max 24bits, not indicate
#define SPDIF_WORDLENGTH_MAX24_20BITS 0x3 // max 24bits, 20bits
#define SPDIF_WORDLENGTH_MAX24_22BITS 0x5 // max 24bits, 22bits
#define SPDIF_WORDLENGTH_MAX24_23BITS 0x9 // max 24bits, 23bits
#define SPDIF_WORDLENGTH_MAX24_24BITS 0xb // max 24bits, 24bits
#define SPDIF_WORDLENGTH_MAX24_21BITS 0xd // max 24bits, 21bits
#define SPDIF_WORDLENGTH_MAX20_NOT_INDICATE 0x0 // max 20bits, not indicate
#define SPDIF_WORDLENGTH_MAX20_16BITS 0x2 // max 20bits, 16bits
#define SPDIF_WORDLENGTH_MAX20_18BITS 0x4 // max 20bits, 18bits
#define SPDIF_WORDLENGTH_MAX20_19BITS 0x8 // max 20bits, 19bits
#define SPDIF_WORDLENGTH_MAX20_20BITS 0xa // max 20bits, 20bits
#define SPDIF_WORDLENGTH_MAX20_17BITS 0xc // max 20bits, 17bits

#if defined(CC_AUD_UNICORN_CUST)||defined(CC_MAPLE_CUST_DRV)
//support HDMI upto 6 channel
#define CC_HDMI_PCM_MULT_CHANNEL
#define HDMI_MAX_CH_NUM 6
#endif

// IEC channel status by
#define IEC_CHANNEL_STATUS_BY_SETUP (0x0)
#define IEC_CHANNEL_STATUS_BY_SOURCE (0x1)

// Copyright bit
// 0 = copyright is asserted
// 1 = no copyright is asserted
#define IEC_COPY_NEVER                 (0x00)
#define IEC_COPY_FREELY                  (0x01)
#define IEC_COPY_NO_MORE            (0x01)
#define IEC_COPY_ONCE                    (0x00)

// IEC category code
#define IEC_ORIGINAL_GENERAL            (0x00)
#define IEC_ORIGINAL_BROAD_CAST         (0x64)
#define IEC_NON_ORIGINAL_BROAD_CAST     (0xe4)
#define IEC_ORIGINAL_BROAD_CAST_EU      (0x0c)
#define IEC_ORIGINAL_BROAD_CAST_JP      (0x04)
#define IEC_CH_STATUS_BY_SOURCE         (0xff)
#define IEC_OTHER_SEMICONDUCTOR         (0x78)

// L-bit
#define IEC_CATEGORY_LBIT_NUMBER 7
#define IEC_CATEGORY_LBIT (1 << IEC_CATEGORY_LBIT_NUMBER)
#define IEC_CATEGORY_LBIT_MASK (~IEC_CATEGORY_LBIT)

// IEC word length (defined in register map order)
#define IEC_WORD_LENGTH_24BIT           (0 << 0)
#define IEC_WORD_LENGTH_DEFAULT      (1 << 0)
#define IEC_WORD_LENGTH_20BIT           (2 << 0)
#define IEC_WORD_LENGTH_16BIT           (3 << 0)

//DTS
#define AUD_DTS_HEADER_LENGTH           (10)

typedef enum
{
    BURST_INFO_NULL_DATA = 0,
    BURST_INFO_AC3,
    BURST_INFO_AAC = 7,
    BURST_INFO_HEAAC = 19,
    BURST_INFO_DTS = 11,
    BURST_INFO_DTS_512 = 11,
    BURST_INFO_DTS_1024,
    BURST_INFO_DTS_2048,
    BURST_INFO_EAC3 = 0x15,
    BURST_INFO_MPEG4_AAC = 0x37,
    BURST_INFO_NUM
}   SPDIF_BURST_INFO_T;

#if defined(CC_AUD_ARM_SUPPORT) && defined(CC_AUD_ARM_RENDER)
typedef enum {
    APROC_RAW_UNKNOWN = 0,
    APROC_RAW_PCM,
    APROC_RAW_AC3,
    APROC_RAW_AAC,
    APROC_RAW_MPEG4_AAC,
    APROC_RAW_HEAAC,
    APROC_RAW_DTS_512,
    APROC_RAW_DTS_1024,
    APROC_RAW_DTS_2048,
    APROC_RAW_EAC3,
    APROC_RAW_NUM
} APROC_IEC_AUDFMT_T;
#endif
#define IEC_CFG0_CATEGORYCODE_BIT 0
#define IEC_CFG0_CATEGORYCODE_MASK (~0xff)

#define IEC_CFG0_PCM_RAW_BIT 8
#define IEC_CFG0_PCM_RAW_MASK (~0x300)
#define IEC_CFG0_PCM_RAW_ISRAW (0x3 << IEC_CFG0_PCM_RAW_BIT)
#define IEC_CFG0_PCM_RAW_ISPCM (0x0 << IEC_CFG0_PCM_RAW_BIT)

#define IEC_CFG0_WORDLENGTH_BIT 12
#define IEC_CFG0_WORDLENGTH_MASK (~0xf000)
#define IEC_CFG0_WORDLENGTH_24BITS (SPDIF_WORDLENGTH_MAX24_24BITS << IEC_CFG0_WORDLENGTH_BIT)
#define IEC_CFG0_WORDLENGTH_20BITS (SPDIF_WORDLENGTH_MAX24_20BITS << IEC_CFG0_WORDLENGTH_BIT)
#define IEC_CFG0_WORDLENGTH_16BITS (SPDIF_WORDLENGTH_MAX20_16BITS << IEC_CFG0_WORDLENGTH_BIT)
#define IEC_CFG0_WORDLENGTH_DEFAULT (SPDIF_WORDLENGTH_MAX20_NOT_INDICATE << IEC_CFG0_WORDLENGTH_BIT)

#define IEC_WORDLENGTH_24BITS 0x0
#define IEC_WORDLENGTH_DEFAULT 0x1
#define IEC_WORDLENGTH_20BITS 0x2
#define IEC_WORDLENGTH_16BITS 0x3

#define IEC_CFG1_DIGITAL_BIT 8
#define IEC_CFG1_DIGITAL_MASK (~(1 << IEC_CFG1_DIGITAL_BIT))
#define IEC_CFG1_DIGITAL_RAW (0x1 << IEC_CFG1_DIGITAL_BIT)
#define IEC_CFG1_DIGITAL_PCM (0x0 << IEC_CFG1_DIGITAL_BIT)

#define IEC_CFG1_CBIT_BIT 7
#define IEC_CFG1_CBIT_MASK (~(1 << IEC_CFG1_CBIT_BIT))

#define IEC_CFG1_SAMPLERATE_BIT 0
#define IEC_CFG1_SAMPLERATE_MASK (~(0xf))
#define IEC_CFG1_SAMPLERATE_32K (SPDIF_SAMPLE_RATE_32KHZ << IEC_CFG1_SAMPLERATE_BIT)
#define IEC_CFG1_SAMPLERATE_44K (SPDIF_SAMPLE_RATE_44KHZ << IEC_CFG1_SAMPLERATE_BIT)
#define IEC_CFG1_SAMPLERATE_48K (SPDIF_SAMPLE_RATE_48KHZ << IEC_CFG1_SAMPLERATE_BIT)
#define IEC_CFG1_SAMPLERATE_176K (SPDIF_SAMPLE_RATE_176KHZ << IEC_CFG1_SAMPLERATE_BIT)
#define IEC_CFG1_SAMPLERATE_192K (SPDIF_SAMPLE_RATE_192KHZ << IEC_CFG1_SAMPLERATE_BIT)
#define IEC_CFG1_SAMPLERATE_768K (SPDIF_SAMPLE_RATE_768KHZ << IEC_CFG1_SAMPLERATE_BIT)



// PROM ID
#define DspPROMTstAddr   0x39           // PROM test address
#define DspPROMRdAddr    0x3A           // PROM read port address
#define DspPROMMsk       0xFFFF         // PROM intruction mask
#define DspPROMEnTst     0x100000       // PROM test enable
#define DspPROMVerOffset 0x78           // PROM ID start address
// DROM ID
#define DspDROMRdAddr    0x4000         // DROM read port address
#define DspDROMMsk       0xFFFFFF       // DROM data mask
#define DspDROMVerOffset 0x0            // DROM ID start address

#define DSP_WMA_MWIF
#define CC_SET_VOLUME

// Downmix channel position
#define DOWNMIX_CH_DO_ALL_POST_PROC     ((UINT8)0x00)
#define DOWNMIX_CH_BEFORE_AVC           ((UINT8)0x01)
#define DOWNMIX_CH_BYPASS_POST_PROC     ((UINT8)0x03)

#ifdef CC_AUD_HDMI_PARSER_2_0_SUPPORT
#define IEC_SYNCWORD_PA_PB  0xf8724e1f    //pa must equal to 0x72f8,the really pa in spec is 0xf872
#define IEC_SYNCWORD_PA     0xf872        //pa must equal to 0x72f8,the really pa in spec is 0xf872
#define IEC_SYNCWORD_PB   0x4e1f         //pb must equal to 0x1f4e,the really pb in spec is 0x4e1f
#define IEC_SYNCWORD_DTS_14_EX 0x07F0
#define IEC_SYNCWORD_DTS_14 0x1FFFE800
#define IEC_SYNCWORD_DTS_16 0x7FFE8001
//#define IEC_SYNCWORD_CHECK_PC  0xe0e0  //pc bit 5-7 and bit 13-15 must set 0
#define IEC_MAX_TRANSFER    0x17f8  //0x1800(frame size)-4(sync word)
#define IEC_AC3_FRAME_SIZE 0x1800
#define HDMI_BUFFER_SIZE (0)
#define HDMI_PARSER_PCM_TRANSFER_SIZE 1024
#define HDMI_PARSER_DTSCD_TRANSFER_SIZE 1024
#define HDMI_PARSER_NON_DETECT_SIZE 0x8040
#define HDMI_PARSER_DETECT_PCM_SIZE 0xffff    //MAT debug 0x9080  //0x2840
#endif

#define MPEG_FRAME_SIZE 0x870 //1152 sample for 48 kHZ, in 90K Hz Clock.
#define MPEG_AAC_FRAME_SIZE 2700    //30ms, cannot exceed minimum of video set audio delay

/*For Power ON/OFF music flag in EEP*/
#define AUD_POWER_ON_MUSIC_MASK            (0x1 << 0)
#define AUD_POWER_OFF_MUSIC_MASK           (0x1 << 1)

// ASRC definition
#define ASRC_BASE_FREQ_8K           0x000a0000
#define ASRC_BASE_FREQ_16K          0x00140000
#define ASRC_BASE_FREQ_32K          0x00280000

#define ASRC_BASE_FREQ_11K          0x000dc800
#define ASRC_BASE_FREQ_22K          0x001b9000
#define ASRC_BASE_FREQ_44K          0x00372000

#define ASRC_BASE_FREQ_12K          0x000f0000
#define ASRC_BASE_FREQ_24K          0x001e0000
#define ASRC_BASE_FREQ_48K          0x003c0000
/*For Power ON/OFF music flag in EEP*/
#define AUD_POWER_ON_MUSIC_MASK            (0x1 << 0)
#define AUD_POWER_OFF_MUSIC_MASK           (0x1 << 1)
// new SDPFI control
typedef enum
{
    SPDIF_REG_TYPE_DEFAULT = 0,
    SPDIF_REG_TYPE_USER,
    SPDIF_REG_TYPE_CIPLUS,
    SPDIF_REG_TYPE_NUM
}   SPDIF_REG_TYPE_T;

#if 0 //move to drvcust_if.h
typedef enum
{
    FORMAT_RJ,
    FORMAT_LJ,
    FORMAT_I2S
}   DATA_FORMAT_T;

typedef enum
{
    MCLK_128FS,
    MCLK_192FS,
    MCLK_256FS,
    MCLK_384FS,
    MCLK_512FS,
    MCLK_768FS
}   MCLK_FREQUENCY_T;
#endif

typedef enum
{
    LRCK_CYC_16,
    LRCK_CYC_24,
    LRCK_CYC_32
}   LRCK_CYC_T;

typedef enum
{
    DAC_16_BIT,
    DAC_18_BIT,
    DAC_20_BIT,
    DAC_24_BIT
}   DAC_DATA_NUMBER_T;

typedef enum
{
    AUD_BS_SEL_PSR,
    AUD_BS_SEL_LINE_IN1,
    AUD_BS_SEL_DMX,
    AUD_BS_SEL_LINE_IN2,
    AUD_BS_SEL_USER0,
    AUD_BS_SEL_USER1,
    AUD_BS_SEL_LINE_IN3,
    AUD_BS_SEL_NUM
} AUD_BS_SEL_T;

typedef enum
{
    AUD_STREAM_FROM_OTHERS = 0,         ///< ex. Pink Noise Generator
    AUD_STREAM_FROM_DIGITAL_TUNER,      ///< Digital tuner. TS
    AUD_STREAM_FROM_ANALOG_TUNER,       ///< ex. Analog tuner. SIF/AF
    AUD_STREAM_FROM_SPDIF,              ///< SPDIF-in
    AUD_STREAM_FROM_LINE_IN,            ///< Line in
    AUD_STREAM_FROM_HDMI,               ///< HDMI in
    AUD_STREAM_FROM_MEMORY,             ///< Memory
    AUD_STREAM_FROM_BUF_AGT,            ///< Buffer agent
    AUD_STREAM_FROM_FEEDER,             ///< Feeder
    AUD_STREAM_FROM_MULTI_MEDIA,        /// < Multi-Media
    AUD_STREAM_FROM_LINE_IN_2 = 10,          ///< Line in2
    AUD_STREAM_FROM_LINE_IN_3 = 11,          ///< Line in3 for Echo Cancel & BT.
    AUD_STREAM_FROM_GST,                    /// Gstreamer
    AUD_STREAM_FROM_I2S,                ///<I2S in
    AUD_STREAM_FROM_NUM                 ///< Number of stream source
}   AUD_DEC_STREAM_FROM_T;

typedef enum
{
    AUD_MIXSND_0 = 0,
    AUD_MIXSND_1,
    AUD_MIXSND_2,//for aaudio
    AUD_MIXSND_3,
#ifdef  CC_AUD_SUPPORT_NEW_TUNNEL_MODE
    AUD_MIXSND_4,
    AUD_MIXSND_5,
#endif
#ifdef CC_AUD_APROC_HWMIX
    AUD_HW_MIXSND_0,
    AUD_HW_MIXSND_1,
#endif
    AUD_MIXSND_NUM
}   AUD_MIXSND_ID_T;

typedef struct
{
    DATA_FORMAT_T      eFormat;         ///< format of alignment
    DAC_DATA_NUMBER_T  eBits;           ///< number of bits per sample
    LRCK_CYC_T         eCycle;          ///< cycles per sample
    MCLK_FREQUENCY_T   eSampleFreq;     ///< DAC sampling frequence
    BOOL               fgDataInvert;    ///< Invert audio output for OP phase
    BOOL               fgLRInvert;      ///< Invert L/R audio output
}   AOUT_CFG_T;

typedef enum
{
    AUD_INPUT_ID_COMP_VID_0 = 0,
    AUD_INPUT_ID_COMP_VID_1,
    AUD_INPUT_ID_COMP_VID_2,
    AUD_INPUT_ID_COMP_VID_3,
    AUD_INPUT_ID_S_VID_0,
    AUD_INPUT_ID_S_VID_1,
    AUD_INPUT_ID_S_VID_2,
    AUD_INPUT_ID_YPBPR_0,
    AUD_INPUT_ID_YPBPR_1,
    AUD_INPUT_ID_YPBPR_2,
    AUD_INPUT_ID_VGA_0,
    AUD_INPUT_ID_VGA_1,
    AUD_INPUT_ID_HDMI_0,
    AUD_INPUT_ID_HDMI_1,
    AUD_INPUT_ID_HDMI_2,
    AUD_INPUT_ID_HDMI_3,
    AUD_INPUT_ID_HDMI_4,
    AUD_INPUT_ID_DVI_0,
    AUD_INPUT_ID_DVI_1,
    AUD_INPUT_ID_DVI_2,
    AUD_INPUT_ID_DVI_3,
    AUD_INPUT_ID_DVI_4,
    AUD_INPUT_ID_SCART_0,
    AUD_INPUT_ID_SCART_1,
    AUD_INPUT_ID_SCART_2,
    AUD_INPUT_ID_SCART_3,
    AUD_INPUT_ID_AUXIN_0,
    AUD_INPUT_ID_AUXIN_1,
    AUD_INPUT_ID_MAX
} AUD_INPUT_ID_T;

typedef enum
{
    AUD_CHL_L_R = 0,        //APROC_POSTBUF_SP
    AUD_CHL_LS_RS,          //APROC_POSTBUF_SP_LSRS
    AUD_CHL_C_SW,           //APROC_POSTBUF_SP_CSW
    AUD_CHL_IEC,            //APROC_POSTBUF_IEC
    AUD_CHL_DOWNMIX,        //APROC_POSTBUF_HP
    AUD_CHL_BYPASS,         //APROC_POSTBUF_MON
    AUD_CHL_EXTCH,          //APROC_POSTBUF_SCART
    AUD_CHL_AUX             //AUX_LR   ???
}   AUD_CHL_DEF_T;

typedef enum
{
    AUD_AP_COUNTRY_UNKNOWN = 0,
    AUD_AP_COUNTRY_GA,
    AUD_AP_COUNTRY_INDIA,
    AUD_AP_COUNTRY_AZERBAIJAN,
    AUD_AP_COUNTRY_CIS,
    AUD_AP_COUNTRY_CHINA,
    AUD_AP_COUNTRY_LATIN
} AUD_APP_COUNTRY_SEL;

typedef enum
{
    AUD_SPEAKER = 0,
    AUD_HEADPHONE,
    AUD_SPDIF,
    AUD_AV_OUT,
    AUD_MONITOR_SCART,
    AUD_TV_SCART,
    AUD_BLUETOOTH,
    AUD_OUT_PORT_NUM
} AUD_OUT_PORT_T;

typedef void (*CODEC_INIT)(void);
typedef void (*CODEC_MUTE)(UINT8 u1DacId, BOOL fgMute);
typedef void (*CODEC_MUTE_CLK)(UINT8 u1DacId, BOOL fgMute);
typedef void (*CODEC_SET_DAC_IN_FMT)(DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk);
typedef void (*CODEC_SET_ADC_OUT_FMT)(DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk);
typedef void (*CODEC_SET_DAC_GAIN)(UINT8 u1DacId, INT16 i2Gain);
typedef void (*CODEC_GET_DAC_GAIN)(UINT8 u1DacId, INT16* pi2Gain);
typedef void (*CODEC_SET_ADC_GAIN)(UINT8 u1DacId, INT16 i2Gain);
typedef void (*CODEC_GET_ADC_GAIN)(UINT8 u1DacId, INT16* pi2Gain);
typedef void (*CODEC_SET_HP_GAIN)(UINT8 u1HpId, INT16 i2Gain);
typedef void (*CODEC_GET_HP_GAIN)(UINT8 u1HpId, INT16* i2Gain);
typedef void (*CODEC_SET_ADC_MUX_SEL)(UINT8 u1Sel);
typedef void (*CODEC_REG_WRITE)(UINT8 u1Offset, UINT8 u1Data);
typedef UINT8 (*CODEC_REG_READ)(UINT8 u1Offset);
typedef void (*CODEC_SET_DAC_CHL)(UINT8 u1DacId, AUD_CHL_DEF_T eChl);
typedef void (*CODEC_GET_DAC_CHL)(void);
typedef void  (*CODEC_ADC_DOWN)(BOOL fgEnalbe);

typedef void (*ADAC_INIT)(void);
typedef void (*ADAC_MUTE)(BOOL fgMute);
typedef void (*ADAC_ENABLE)(BOOL fgEnable);
typedef void (*ADAC_CHL_SEL)(UINT8 u1DacId, UINT8 eChl);
typedef void (*ADAC_SET_DAC_IN_FMT)(DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk);
typedef void (*ADAC_SET_DAC_GAIN)(UINT8 u1HpId, INT16 i2Gain);
typedef void (*ADAC_REG_WRITE)(UINT8 u1Offset, UINT8 u1Data);
typedef UINT8 (*ADAC_REG_READ)(UINT8 u1Offset);

#if 0 //move to drvcust_if.h
typedef void (*AMP_INIT)(void);
typedef void (*AMP_MUTE)(BOOL fgMute);
typedef void (*AMP_SET_DAC_IN_FMT)(DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk);
typedef void (*AMP_SET_DAC_GAIN)(UINT8 u1HpId, INT16 i2Gain);
typedef void (*AMP_REG_WRITE)(UINT8 u1Offset, UINT8 u1Data);
typedef UINT8 (*AMP_REG_READ)(UINT8 u1Offset);
#endif

/// Audio codec function table.
typedef struct
{
    CODEC_INIT             pfnInit;
    CODEC_MUTE             pfnMute;
    CODEC_MUTE_CLK         pfnMuteClk;
    CODEC_SET_DAC_IN_FMT   pfnSetDacInFmt;
    CODEC_SET_ADC_OUT_FMT  pfnSetAdcOutFmt;
    CODEC_SET_DAC_GAIN     pfnSetDacGain;
    CODEC_GET_DAC_GAIN     pfnGetDacGain;
    CODEC_SET_ADC_GAIN     pfnSetAdcGain;
    CODEC_GET_ADC_GAIN     pfnGetAdcGain;
    CODEC_SET_HP_GAIN      pfnSetHpGain;
    CODEC_GET_HP_GAIN      pfnGetHpGain;
    CODEC_SET_ADC_MUX_SEL  pfnSetAdcMuxSel;
    CODEC_REG_WRITE        pfnRegWrite;
    CODEC_REG_READ         pfnRegRead;
    CODEC_SET_DAC_CHL      pfnSetDacChl;
    CODEC_GET_DAC_CHL      pfnGetDacChl;
    CODEC_ADC_DOWN         pfnADCDown;
} CODEC_FUNCTBL_T;

/// Codec configuration table.
typedef struct
{
    CHAR             szName[32];
    INT16            i2MaxHpGain;
    INT16            i2MinHpGain;
    INT16            i2MaxDacGain;
    INT16            i2MinDacGain;
    UINT8            u1AdcMuxNum;
    UINT8            u1DacNum;
    UINT8            u1HpNum;
    CODEC_FUNCTBL_T* prCodecFuncTbl;
} CODEC_T;

/// Internal dac function table.
typedef struct
{
    ADAC_INIT           pfnInit;
    ADAC_MUTE           pfnMute;
    ADAC_ENABLE         pfnEnable;
    ADAC_CHL_SEL        pfnChlSelect;
    ADAC_SET_DAC_IN_FMT pfnSetDacInFmt;
    ADAC_SET_DAC_GAIN   pfnSetDacGain;
    ADAC_REG_WRITE      pfnRegWrite;
    ADAC_REG_READ       pfnRegRead;
} ADAC_FUNCTBL_T;

/// Internal dac configuration table.
typedef struct
{
    CHAR            szName[32];
    INT16           i2MaxGain;
    INT16           i2MinGain;
    UINT8           u1DacNum;
    BOOL            fInterDac;
    ADAC_FUNCTBL_T* prDacFuncTbl;
} ADAC_T;

/// Audio codec select table.
typedef enum
{
    AUD_CODEC_MT8291 = 0,
    AUD_CODEC_WM8776,
    AUD_CODEC_MT538xInternal,
    AUD_CODEC_MT536xInternal,
    AUD_CODEC_MT539xInternal,
    AUD_CODEC_MT5387,
    AUD_CODEC_MT8292,
    AUD_CODEC_MT5365,
    AUD_CODEC_MT5398,
    AUD_CODEC_OTHERS,
    AUD_CODEC_NULL
} AUD_CODEC_T;

/// Audio codec select table2.
typedef enum
{
    AUD_ADAC_MT537X_INTERNAL = 0,
    AUD_ADAC_WM8766,
    AUD_ADAC_CSXXX,
    AUD_ADAC_OTHERS,
    AUD_ADAC_NULL
} AUD_ADAC_T;

/// Audio channel table.
typedef enum
{
    AUD_CH_FRONT_LEFT = 0,
    AUD_CH_FRONT_RIGHT,
    AUD_CH_REAR_LEFT,
    AUD_CH_REAR_RIGHT,
    AUD_CH_CENTER,
    AUD_CH_SUB_WOOFER,
    AUD_CH_IEC_LEFT,
    AUD_CH_IEC_RIGHT,
    AUD_CH_DMX_LEFT,
    AUD_CH_DMX_RIGHT,
    AUD_CH_BYPASS_LEFT,
    AUD_CH_BYPASS_RIGHT,
    AUD_CH_CH1314_LEFT,
    AUD_CH_CH1314_RIGHT,
    AUD_CH_AUX_FRONT_LEFT,
    AUD_CH_AUX_FRONT_RIGHT,
    AUD_CH_INPUT_SRC,                   ///< For input source volume control
    AUD_CH_ALL,                         ///< For master volume control
    AUD_CH_LFE,
    AUD_CH_THIRD,
    AUD_CH_INPUT_DEC,    //AUD_CH_INPUT_DEC
    AUD_CH_BT,           //BT
#if defined(CC_MT5895)
    AUD_CH_EARC,
#endif
    AUD_CH_NUM
}   AUD_CH_T;

/// Audio digital amplify table.
typedef enum
{
    AUD_MIX_CH_LR = 0x0100,
    AUD_MIX_CH_LSRS = 0x0200,
    AUD_MIX_CH_CW = 0x0400,
    AUD_MIX_CH_IEC_LR = 0x0800,
    AUD_MIX_CH_DMX_LR = 0x1000,
    AUD_MIX_CH_NUM
}AUD_MIXSND_CH_T;

/// Audio digital amplify table.
typedef enum
{
    AUD_AMP_APOGEE_DDX_2051 = 0,
    AUD_AMP_AKM_XXX,
    AUD_AMP_WOLFSON_XXX,
    AUD_AMP_PHILIPS_XXX,
    AUD_AMP_OTHERS,
    AUD_AMP_AD82581,
    AUD_AMP_AD87588, /* billtu_2015_0723, add */
    AUD_AMP_BM28723,
    AUD_AMP_TAS5707,
    AUD_AMP_STA339,
    AUD_AMP_NTP7400L,
    AUD_AMP_NTP7515,
    AUD_AMP_TAS5717,
    AUD_AMP_STA381,
    AUD_AMP_NE3201,
    AUD_AMP_DRVCUST,  //AMP_DRV_CUST
    AUD_AMP_RT9119,
    AUD_AMP_NULL
} AUD_AMP_T;

typedef enum
{
    AUD_AMP_ID_LR = 0,
    AUD_AMP_ID_LsRs,
    AUD_AMP_ID_CSw,
    AUD_AMP_ID_NUM
} AUD_AMP_ID_T;

/// Audio opctrl polarity table.
typedef enum
{
    AUD_OPCTRL_MUTE_GPIO_LOW_ENALBE = 0,
    AUD_OPCTRL_MUTE_GPIO_HIGH_ENALBE = 1
}   AUD_OPCTRL_MUTE_GPIO_POLARITY_T;


/// Audio codec mutel polarity table.
typedef enum
{
    AUD_MUTE_GPIO_LOW_ENALBE = 0,
    AUD_MUTE_GPIO_HIGH_ENALBE = 1
}   AUD_MUTE_GPIO_POLARITY_T;

/// Audio amplify enable polarity table.
typedef enum
{
    AUD_AMP_GPIO_LOW_ENALBE = 0,
    AUD_AMP_GPIO_HIGH_ENALBE = 1
}   AUD_AMP_GPIO_POLARITY_T;

/// Audio gpio enable polarity table.
typedef enum
{
    AUD_GPIO_LOW_ENALBE = 0,
    AUD_GPIO_HIGH_ENALBE = 1
}   AUD_GPIO_POLARITY_T;

/// Audio DVD gpio enable polarity table.
typedef enum
{
    AUD_DVD_GPIO_LOW_ENALBE = 0,
    AUD_DVD_GPIO_HIGH_ENALBE = 1
}   AUD_DVD_GPIO_POLARITY_T;

#ifdef CC_AUD_UNICORN_CUST
typedef enum
{
    KD_AUD_AMP_MUTE = 0,
    KD_AUD_AMP_UNMUTE = 1
}   KD_AUD_AMP_MUTE_T;
#endif //CC_AUD_UNICORN_CUST
#ifdef CC_MAPLE_CUST_DRV
typedef enum
{
    AUD_CUST_AMP_UNMUTE = 0,
    AUD_CUST_AMP_MUTE   = 1,
}   AUD_CUST_AMP_MUTE_T;
#endif //CC_MAPLE_CUST_DRV

/// Audio EQ table.
typedef enum
{
    AUD_EQ_OFF = 0,
    AUD_EQ_ROCK,
    AUD_EQ_POP,
    AUD_EQ_LIVE,
    AUD_EQ_DANCE,
    AUD_EQ_TECHNO,
    AUD_EQ_CLASSIC,
    AUD_EQ_SOFT,
    AUD_EQ_CUST1,
    AUD_EQ_CUST2,
    AUD_EQ_CUST3,
    AUD_EQ_CUST4,
    AUD_EQ_CUST5,
    AUD_EQ_CUST6,
    AUD_EQ_CUST7,
    AUD_EQ_CUST8,
    AUD_EQ_BBE_TV,
    AUD_EQ_BBE_AV,
    AUD_EQ_SBASS,
    AUD_EQ_NUM
}   AUD_EQ_TYPE_T;

/// Audio limiter flag table.
typedef enum
{
    AUD_LIMITER_VOL = 0,
    AUD_LIMITER_VSURR,
    AUD_LIMITER_SBASS,
    AUD_LIMITER_EQ,
    AUD_LIMITER_REVERB,
    AUD_LIMITER_SPEAKER,
    AUD_LIMITER_VOICE,
    AUD_LIMITER_NUM,
} AUD_LIMITER_FLAG_T;

/// Audio MixSnd table.
typedef enum
{
    AUD_MIXSND_CLIP0 = 0,
    AUD_MIXSND_CLIP1,
    AUD_MIXSND_CLIP2,
    AUD_MIXSND_CLIP3,
    AUD_MIXSND_CLIP_NUM
}   AUD_MIXSND_CLIP_T;

#ifdef CC_AUD_HDMI_SPDIF_CFG
typedef enum
{
    DEC_STOP = 0,
    DEC_PLAY = 1,
} HDMI_DEC_STATE;

typedef enum
{
    SPDIF_OFF = 0,
    SPDIF_BYPASS = 1,
    SPDIF_MUTE = 2,
} HDMI_SPDIF_STATE;

typedef enum
{
    ACP_NONE = 0,
    ACP_CD,
    ACP_DVD_AUDIO,
    ACP_SACD
} HDMI_ACPTYPE;

typedef enum
{
    PCMData = 0,
    RAWData = 1,
} HDMI_DATATYPE;

typedef enum
{
    FS_32KHZ = 0x03,
    FS_44KHZ = 0x00,
    FS_48KHZ = 0x02,
    FS_88KHZ = 0x08,
    FS_96KHZ = 0x0A,
    FS_176KHZ = 0x0C,
    FS_192KHZ = 0x0E,
    FS_UNKNOWN = 0x01,
} HDMI_AUD_FS;

typedef enum
{
    DigitalMute_AnalogMute = 0,         ///STOP_ALL = 0,
    DigitalUnMute_AnalogMute = 1,       ///STOP_SPDIF_BYPASS = 1,
    DigitalUnMute_AnalogUnMute = 2,     ///PLAY_SPDIF_BYPASS = 2,
    DigitalMute_AnalogUnMute = 3,       ///PLAY_SPDIF_MUTE = 3,
} AUD_OUTPUT_STATE;

typedef struct
{
    HDMI_ACPTYPE    eACPType;          ///0:ACP Packet is none, 1:CD, 2:DVD-Audio, 3:SACD
    HDMI_DATATYPE   eDataType;          ///0:PCM, 1:Compression (Raw).
    UINT8           u1CopyRight;        ///0: Copyright, 1: No Copyright
    UINT8           u1Category;         ///By HDMI Channel Status each 192 frames
    HDMI_AUD_FS     eSampeFreq;        ///0000:32KHz, 1100:44.1KHz, 0100:48KHz
} AUD_HDMI_CHL_STATUS;
#endif

/// Audio input mux select table.
typedef struct
{
    AUD_INPUT_ID_T  eAudInputId;
    UINT8           u1AdcMuxSel;        ///< ADC mux select, ex. WM8776(0~5)
    UINT8           u1SwGpioNum1Val;    ///< 0: Low, 1: High
    UINT8           u1SwGpioNum2Val;    ///< 0: Low, 1: High
    UINT8           u1SwGpioNum3Val;    ///< 0: Low, 1: High
    UINT8           u1SwGpioNum4Val;    ///< 0: Low, 1: High
} AUD_INPUT_MUX_SEL_T;

/// Audio input mux select table with size table.
typedef struct
{
    UINT32                     u4Size;
    const AUD_INPUT_MUX_SEL_T* prAudInMuxSel;
} AUD_INPUT_MUX_SEL_TABLE_T;

/// Audio input mux select switch table.
typedef struct
{
    UINT32      u4AudInSwGpioNum1;      ///< ex.GPIO 17
    UINT32      u4AudInSwGpioNum2;      ///< ex.GPIO 15
    UINT32      u4AudInSwGpioNum3;      ///< ex.GPIO xx
    UINT32      u4AudInSwGpioNum4;      ///< ex.GPIO yy
}   AUD_INPUT_SW_GPIO_T;

/// Audio dac volume table.
typedef struct
{
    UINT32      u4LineOutVol;
    UINT32      u4HeadphoneVol;
    UINT32      u4SpeakerVol;
} AUD_ADAC_VOL_CFG_T;

/// Audio dac usage table.
typedef struct
{
    AUD_ADAC_T  eAdac1;
    AUD_ADAC_T  eAdac2;
    AUD_ADAC_T  eAdac3;
    AUD_ADAC_T  eAdac4;
    AUD_CODEC_T eCodec1;
    AUD_CODEC_T eCodec2;
    AUD_AMP_T   eAudAmp1;
    AUD_AMP_T   eAudAmp2;
} AUD_ADAC_USAGE_T;

/// Audio aout PAD config table.
typedef struct
{
    AUD_CH_T    ePadSdata0;
    AUD_CH_T    ePadSdata1;
    AUD_CH_T    ePadSdata2;
    AUD_CH_T    ePadSdata3;
    AUD_CH_T    ePadSdata4;
    AUD_CH_T    ePadSdata5;
} AUD_AOUT_PAD_CFG_T;

/// Audio internal dac channel config table.
typedef struct
{
    AUD_CHL_DEF_T eDac0ChlSel;
    AUD_CHL_DEF_T eDac1ChlSel;
    AUD_CHL_DEF_T eDac2ChlSel;
    AUD_CHL_DEF_T eDac3ChlSel;
} AUD_INTER_DAC_CFG_T;

typedef struct
{
    UINT32 u4Size;
    const UINT8* prAudSetupMelody;
} AUD_SETUP_MELODY_TABLE_T;


/// Audio ATV decoder type table.
typedef enum
{
    AUD_ATV_DECODER_PAL,
    AUD_ATV_DECODER_A2,
    AUD_ATV_DECODER_MTS,
    AUD_ATV_DECODER_FMFM,
    AUD_ATV_DECODER_NUM
} AUD_ATV_DECODER_TYPE_T;

/// Audio factory MTS detection item table.
typedef enum
{
    AUD_FACTORY_MTS_NUM_CHECK,          ///< numbers of check
    AUD_FACTORY_MTS_NUM_PILOT,          ///< numbers of pilot
    AUD_FACTORY_MTS_PILOT_HIGH,         ///< pilot higher threshold
    AUD_FACTORY_MTS_PILOT_LOW,          ///< pilot lower threshold
    AUD_FACTORY_MTS_NUM_SAP,            ///< numbers of sap
    AUD_FACTORY_MTS_SAP_HIGH,           ///< sap higher threshold
    AUD_FACTORY_MTS_SAP_LOW             ///< sap lower threshold
} AUD_FACTORY_MTS_DETECTION_ITEM_T;

/// Audio factory FM mute table.
typedef enum
{
    AUD_FACTORY_FM_MUTE,                ///< carrier sense mute mode on/off
    AUD_FACTORY_FM_MUTE_HIGH,           ///< higher carrier sense mute threshold
    AUD_FACTORY_FM_MUTE_LOW,             ///< lower carrier sense mute threshold
    AUD_FACTORY_FM_MUTE_HIGH_HDEV, ///< higher carrier sense mute threshold for hdev mode
    AUD_FACTORY_FM_MUTE_LOW_HDEV  ///< lower carrier sense mute threshold for hdev mode
} AUD_FACTORY_FM_MUTE_ITEM_T;

/// Audio factory A2 detection table.
typedef enum
{
    AUD_FACTORY_A2_NUM_CHECK,           ///< numbers of check
    AUD_FACTORY_A2_NUM_DOUBLE,          ///< numbers of double check
    AUD_FACTORY_A2_MONO_WEIGHT,         ///< mono wieght
    AUD_FACTORY_A2_STEREO_WEIGHT,       ///< stereo wieght
    AUD_FACTORY_A2_DUAL_WEIGHT          ///< dual wieght
} AUD_FACTORY_A2_DETECTION_ITEM_T;

/// Audio factory A2 threshold table.
typedef enum
{
    AUD_FACTORY_A2_MONO2STEREO,
    AUD_FACTORY_A2_STEREO2MONO,
    AUD_FACTORY_A2_WEAK_RF_LVL_MONO2STEREO,
    AUD_FACTORY_A2_WEAK_RF_LVL_STEREO2MONO,
} AUD_FACTORY_A2_THRESHOLD_ITEM_T;

/// Audio factory PAL detection table.
typedef enum
{
    AUD_FACTORY_PAL_CORRECT,            ///< correct threshold
    AUD_FACTORY_PAL_SYNC_LOOP,          ///< total sync loop
    AUD_FACTORY_PAL_ERROR,              ///< error threshold
    AUD_FACTORY_PAL_PARITY_ERROR,       ///< parity error threshold
    AUD_FACTORY_PAL_NUM_FRAMES          ///< every number frames
} AUD_FACTORY_PAL_DETECTION_ITEM_T;

/// Audio factory PAL_AM mute table.
typedef enum
{
    AUD_FACTORY_PAL_AM_MUTE,            ///< AM mute mode on/off
    AUD_FACTORY_PAL_AM_MUTE_HIGH,       ///< higher AM mute threshold
    AUD_FACTORY_PAL_AM_MUTE_LOW         ///< lower AM mute threshold
} AUD_FACTORY_PAL_AM_MUTE_ITEM_T;

/// Audio factory FM_FM detection table.
typedef enum
{
    AUD_FACTORY_FMFM_NUM_CHECK,         ///< numbers of check
    AUD_FACTORY_FMFM_NUM_DOUBLE,        ///< numbers of double check
    AUD_FACTORY_FMFM_MONO_WEIGHT,       ///< mono wieght
    AUD_FACTORY_FMFM_STEREO_WEIGHT,     ///< stereo wieght
    AUD_FACTORY_FMFM_DUAL_WEIGHT,       ///< dual wieght
    AUD_FACTORY_FMFM_DETECT_CONFID
} AUD_FACTORY_FMFM_DETECTION_ITEM_T;

typedef enum
{
    AUD_FACTORY_SAP_MUTE_LOWER,         /* higher AM mute threshold */
    AUD_FACTORY_SAP_MUTE_HIGHER,         /* lower AM mute threshold */
    AUD_FACTORY_SAP_HP_MUTE_HIGHER,  /* higher SAP HPF mute threshold */
    AUD_FACTORY_SAP_HP_MUTE_LOWER,   /* higher SAP HPF mute threshold */
    AUD_FACTORY_SAP_FILTER_SEL,             /* SAP filter selection */
    AUD_FACTORY_SAP_RATIO                      /* SAP SNR  */
} AUD_FACTORY_SAP_MUTE_ITEM_T;

typedef enum
{
    AUD_FACTORY_SATU_MUTE_LOWER,         /* higher satutaion mute threshold */
    AUD_FACTORY_SATU_MUTE_HIGHER         /* lower satutaion mute threshold */
} AUD_FACTORY_SATU_MUTE_ITEM_T;

typedef struct
{
    UINT16 u2PeqFc;
    UINT8  u1PeqQ;
    INT16 i2PeqGain;
}  AUD_PEQ_CFG_T;

typedef enum
{
    AUD_PEQ_CASE0 = 0, //Wall Normal & Clear sound on
    AUD_PEQ_CASE1,  //Table Normal & Clear sound on
    AUD_PEQ_CASE2,  // Wall Skype
    AUD_PEQ_CASE3,  // Table Skype
    AUD_PEQ_CASE4,  // Wall Normal & Clear sound off
    AUD_PEQ_CASE5,  // Table Normal & Clear sound off
    AUD_PEQ_CASE_NUM
}AUD_PEQ_CASE_T;

typedef enum
{
    AUD_SM_CASE0 = 0,
    AUD_SM_CASE1,
    AUD_SM_CASE2,
    AUD_SM_CASE3,
    AUD_SM_CASE4,
    AUD_SM_CASE5,
    AUD_SM_CASE6,
    AUD_SM_CASE7,
    AUD_SM_CASE8,
    AUD_SM_CASE9,
    AUD_SM_CASE10,
    AUD_SM_CASE11,
    AUD_SM_CASE12,
    AUD_SM_CASE13,
    AUD_SM_CASE14,
    AUD_SM_CASE15,
    AUD_SM_NUM
}   AUD_POST_SM_T;
typedef enum
{
    AUD_SOUND_MODE_DEFAULT = 0,
    AUD_SOUND_MODE_STANDARD ,
    AUD_SOUND_MODE_DYNAMIC,
    AUD_SOUND_MODE_CLEAR_VOICE,
    AUD_SOUND_MODE_NUM
}   AUD_US_SOUND_MODE_T;

typedef enum
{
    AUD_VS_CASE1 = 0,
    AUD_VS_CASE2,
    AUD_VS_CASE3,
    AUD_VS_CASE4,
    AUD_VS_CASE5,
    AUD_VS_NUM
}   AUD_VS_MODE_T;
#if defined(CC_PHX_CUST)||defined(CC_LMP_CUST)
typedef enum
{
    AUD_CLR_VOICE_MODE_OFF = 0,
    AUD_CLR_VOICE_MODE_LOW,
    AUD_CLR_VOICE_MODE_MID,
    AUD_CLR_VOICE_MODE_HIGH,
    AUD_CLR_VOICE_MODE_NUM
}   AUD_CLR_VOICE_MODE_T;
#endif
typedef struct
{
    UINT16 u2NumCheck;
    UINT16 u2NumPilot;              // stereo detectio
    UINT16 u2PilotHigherThd;
    UINT16 u2PilotLowerThd;
    UINT16 u2NumSap;              // SAP detectio
    UINT16 u2SapHigherThd;
    UINT16 u2SapLowerThd;
    UINT8 u1HdevMode;
    BOOL fgCarrierShift;
    BOOL fgSatuMuteMode;
    UINT8 u1SatuMuteHigherThd;
    UINT8 u1SatuMuteLowerThd;
    BOOL fgCarrierMuteMode;
    UINT8 u1CarrierMuteHigherThd;
    UINT8 u1CarrierMuteLowerThd;
    UINT8 u1MonoStereoPrescale;
    UINT8 u1SapPrescale;
    BOOL fgPilotOffsetDetect;
    UINT8 u1SapNoiseMuteHigh;
    UINT8 u1SapNoiseMuteLow;
    UINT8 u1SAPFilterType;  /// [0:auto/1:manual]
    UINT8 u1SAPFilterMode;  /// [0:big/1:small]
#ifdef CC_AUD_MTS_OUTPUT_LEVEL_GAIN_SUPPORT
    UINT32 u2OutputLevelGain;
#endif
}  AUD_MTS_FAC_T;

typedef struct
{
    UINT16 u2NumCheck;
    UINT16 u2NumDoubleCheck;
    UINT16 u2MonoWeight;
    UINT16 u2StereoWeight;
    UINT16 u2DualWeight;
    BOOL fgHdevMode;
    BOOL fgCarrierShift;
    BOOL fgCarrierMuteMode;
    UINT8 u1CarrierMuteHigherThd;
    UINT8 u1CarrierMuteLowerThd;
    UINT8 u1A2Prescale;
    BOOL fgSatuMuteMode;
    BOOL fgNonEuMode;
    UINT16 u2Mono2StereoEU;
    UINT16 u2Stereo2MonoEU;
    UINT16 u2Mono2StereoM;
    UINT16 u2Stereo2MonoM;
    UINT16 u2WeakRfLvlM2SEU;
    UINT16 u2WeakRfLvlS2MEU;
}  AUD_A2_FAC_T;

typedef struct
{
    UINT16 u2CorrectThd;
    UINT16 u2TotalSyncLoop;
    UINT16 u2ErrorThd;
    UINT16 u2ParityErrorThd;
    UINT16 u2EveryNumFrames;
    BOOL fgHdevMode;
    BOOL fgAmMuteMode;
    UINT8 u1AmMuteHigherThd;
    UINT8 u1AmMuteLowerThd;
    BOOL fgCarrierShift;
    BOOL fgCarrierMuteMode;
    UINT8 u1CarrierMuteHigherThd;
    UINT8 u1CarrierMuteLowerThd;
#ifdef CC_AUD_SX1_FEATURE
    UINT8 u1CarrierMuteHdevHigherThd;
    UINT8 u1CarrierMuteHdevLowerThd;
#endif
    UINT8 u1PalPrescale;
    UINT8 u1AmPrescale;
    UINT8 u1NicamPrescale;
    BOOL fgSatuMuteMode;
    BOOL fgNonEuMode;
    BOOL fgNicamIBoost;
}  AUD_PAL_FAC_T;

typedef struct
{
    BOOL fgCLK;
    BOOL fgAOSDATA0;
    BOOL fgAOSDATA1;
    BOOL fgAOSDATA2;
    BOOL fgAOSDATA3;
    BOOL fgAOSDATA4;
} AUD_DIGITAL_DATA_T;

typedef enum
{
    AUD_PEQ_Q_0_33 = 0,
    AUD_PEQ_Q_0_43,
    AUD_PEQ_Q_0_56,
    AUD_PEQ_Q_0_75,
    AUD_PEQ_Q_1_00,
    AUD_PEQ_Q_1_20,
    AUD_PEQ_Q_1_50,
    AUD_PEQ_Q_1_80,
    AUD_PEQ_Q_2_20,
    AUD_PEQ_Q_2_27,
    AUD_PEQ_Q_3_30,
    AUD_PEQ_Q_3_39,
    AUD_PEQ_Q_4_70,
    AUD_PEQ_Q_5_60,
    AUD_PEQ_Q_6_80,
    AUD_PEQ_Q_8_20,
    AUD_PEQ_Q_MAX
} AUD_PEQ_Q_TYPE_T;

//---------------------------------------------------------------------------
// Global Variable
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


/* Note that this enumeration should be consistent with middleware x_aud_dec.h  */
typedef enum
{
    AUD_FMT_UNKNOWN = 0,
    AUD_FMT_MPEG,
    AUD_FMT_AC3,
    AUD_FMT_PCM,
    AUD_FMT_MP3,
    AUD_FMT_AAC,
    AUD_FMT_DTS,
    AUD_FMT_WMA,
    AUD_FMT_RA,
    AUD_FMT_HDCD,
    AUD_FMT_MLP,     // 10
    AUD_FMT_MTS,
    AUD_FMT_EU_CANAL_PLUS,
    AUD_FMT_PAL,
    AUD_FMT_A2,
    AUD_FMT_FMFM,
    AUD_FMT_NICAM,
    AUD_FMT_TTXAAC,
    AUD_FMT_DETECTOR,
    AUD_FMT_MINER,
    AUD_FMT_LPCM,       // 20
    AUD_FMT_FMRDO,
    AUD_FMT_FMRDO_DET,
    AUD_FMT_SBCDEC,
    AUD_FMT_SBCENC,     // 24,
    AUD_FMT_MP3ENC,     // 25, MP3ENC_SUPPORT
    AUD_FMT_G711DEC,    // 26
    AUD_FMT_G711ENC,    // 27
    AUD_FMT_DRA,        // 28
    AUD_FMT_COOK,        // 29
    AUD_FMT_G729DEC,     // 30
    AUD_FMT_VORBISDEC,    //31, CC_VORBIS_SUPPORT
    AUD_FMT_WMAPRO,    //32  please sync number with middleware\res_mngr\drv\x_aud_dec.h
    AUD_FMT_HE_AAC,     //33 Terry Added for S1 UI notification
    AUD_FMT_HE_AAC_V2,  //34
    AUD_FMT_AMR,        //35 amr-nb run in DSP
    AUD_FMT_AWB,        //36 amr-wb run in DSP
    AUD_FMT_APE,        //37 //ian APE decoder
    AUD_FMT_FLAC = 39,  //39, paul_flac
    AUD_FMT_G726 = 40,  //40, G727 decoder
    AUD_FMT_ALAC = 41,  //41, alac decoder
    AUD_FMT_AC4  = 42,
    AUD_FMT_OPUS = 43,
    AUD_FMT_TRUEHD = 44,
    AUD_FMT_MAT_PCM = 45,
    AUD_FMT_TV_SYS = 63, //63, sync from x_aud_dec.h
    AUD_FMT_OMX_MSADPCM = 0x80,
    AUD_FMT_OMX_IMAADPCM = 0x81,
    AUD_FMT_OMX_ALAW = 0x82,
    AUD_FMT_OMX_ULAW = 0x83,
}   AUD_FMT_T;

/* Note that this enumeration should be consistent with middleware x_aud_dec.h  */
typedef enum
{
    AUD_META_NON= 0,
    AUD_META_ATMOS = 1,
    AUD_META_UNKNOWN = 0xff,
}   AUD_META_T;

/* Move AUD_DEC_STREAM_FROM_T to drv_cust/drvcust_if.h */

// Macro to create bit mask
#define MAKE_BIT_MASK(_val)     (((UINT32) 1) << (UINT32)_val)

/* The following enumeration and bitmasks are used to convey */
/* TV-System type information.                               */
/* Note that this enumeration should be consistent with middleware x_aud_dec.h  */
typedef enum
{
    AUD_TV_SYS_UNKNOWN = -1, /* Must be set to '-1' else I loose an entry in the bit mask. */
    AUD_TV_SYS_A = 0,
    AUD_TV_SYS_B = 1,
    AUD_TV_SYS_C = 2,
    AUD_TV_SYS_D = 3,
    AUD_TV_SYS_E = 4,
    AUD_TV_SYS_F = 5,
    AUD_TV_SYS_G = 6,
    AUD_TV_SYS_H = 7,
    AUD_TV_SYS_I = 8,
    AUD_TV_SYS_J = 9,
    AUD_TV_SYS_K = 10,
    AUD_TV_SYS_K_PRIME = 11,
    AUD_TV_SYS_L = 12,
    AUD_TV_SYS_L_PRIME = 13,
    AUD_TV_SYS_M = 14,
    AUD_TV_SYS_N = 15,
    AUD_TV_SYS_A2 = 16,
    AUD_TV_SYS_PAL = 17,
    AUD_TV_SYS_NICAM = 18,
    AUD_TV_SYS_SECAM = 19
}   AUD_TV_SYS_T;

#define AUD_TV_SYS_MASK_A               MAKE_BIT_MASK (AUD_TV_SYS_A)
#define AUD_TV_SYS_MASK_B               MAKE_BIT_MASK(AUD_TV_SYS_B)
#define AUD_TV_SYS_MASK_C               MAKE_BIT_MASK(AUD_TV_SYS_C)
#define AUD_TV_SYS_MASK_D               MAKE_BIT_MASK(AUD_TV_SYS_D)
#define AUD_TV_SYS_MASK_E               MAKE_BIT_MASK(AUD_TV_SYS_E)
#define AUD_TV_SYS_MASK_F               MAKE_BIT_MASK(AUD_TV_SYS_F)
#define AUD_TV_SYS_MASK_G               MAKE_BIT_MASK(AUD_TV_SYS_G)
#define AUD_TV_SYS_MASK_H               MAKE_BIT_MASK(AUD_TV_SYS_H)
#define AUD_TV_SYS_MASK_I               MAKE_BIT_MASK(AUD_TV_SYS_I)
#define AUD_TV_SYS_MASK_J               MAKE_BIT_MASK(AUD_TV_SYS_J)
#define AUD_TV_SYS_MASK_K               MAKE_BIT_MASK(AUD_TV_SYS_K)
#define AUD_TV_SYS_MASK_K_PRIME         MAKE_BIT_MASK(AUD_TV_SYS_K_PRIME)
#define AUD_TV_SYS_MASK_L               MAKE_BIT_MASK(AUD_TV_SYS_L)
#define AUD_TV_SYS_MASK_L_PRIME         MAKE_BIT_MASK(AUD_TV_SYS_L_PRIME)
#define AUD_TV_SYS_MASK_M               MAKE_BIT_MASK(AUD_TV_SYS_M)
#define AUD_TV_SYS_MASK_N               MAKE_BIT_MASK(AUD_TV_SYS_N)

#define AUD_TV_SYS_MASK_A2              MAKE_BIT_MASK(AUD_TV_SYS_A2)
#define AUD_TV_SYS_MASK_PAL             MAKE_BIT_MASK(AUD_TV_SYS_PAL)
#define AUD_TV_SYS_MASK_NICAM           MAKE_BIT_MASK(AUD_TV_SYS_NICAM)
#define AUD_TV_SYS_MASK_SECAM           MAKE_BIT_MASK(AUD_TV_SYS_SECAM)

/* The following enumeration and bitmasks are used to convey */
/* Audio-System type information.                            */
/* Note that this enumeration should be consistent with middleware x_aud_dec.h */
typedef enum
{
    AUD_SYS_UNKNOWN = -1, /* Must be set to '-1' else I loose an entry in the bit mask. */
    AUD_SYS_AM,
    AUD_SYS_FM_MONO,
    AUD_SYS_FM_EIA_J,
    AUD_SYS_FM_A2,
    AUD_SYS_FM_A2_DK1,
    AUD_SYS_FM_A2_DK2,
    AUD_SYS_FM_RADIO,
    AUD_SYS_NICAM,
    AUD_SYS_BTSC
}   AUD_SYS_T;

#define AUD_SYS_MASK_AM                 MAKE_BIT_MASK (AUD_SYS_AM)
#define AUD_SYS_MASK_FM_MONO            MAKE_BIT_MASK (AUD_SYS_FM_MONO)
#define AUD_SYS_MASK_FM_EIA_J           MAKE_BIT_MASK (AUD_SYS_FM_EIA_J)
#define AUD_SYS_MASK_FM_A2              MAKE_BIT_MASK (AUD_SYS_FM_A2)
#define AUD_SYS_MASK_FM_A2_DK1          MAKE_BIT_MASK (AUD_SYS_FM_A2_DK1)
#define AUD_SYS_MASK_FM_A2_DK2          MAKE_BIT_MASK (AUD_SYS_FM_A2_DK2)
#define AUD_SYS_MASK_FM_RADIO           MAKE_BIT_MASK (AUD_SYS_FM_RADIO)
#define AUD_SYS_MASK_NICAM              MAKE_BIT_MASK (AUD_SYS_NICAM)
#define AUD_SYS_MASK_BTSC               MAKE_BIT_MASK (AUD_SYS_BTSC)

/* Note that this enumeration should be consistent with middleware x_aud_dec.h */
/// Audio middleware system table.
typedef struct _MW_TV_AUD_SYS_T
{
    UINT32  u4TvSysMask;
    UINT32  u4AudSysMask;
} MW_TV_AUD_SYS_T;

#ifndef _TV_AUD_SYS_T
#define _TV_AUD_SYS_T
typedef enum
{
    SV_NONE_DETECTED = 0x0,     //0x0
    SV_MTS           = 0x1,     //0x1
    SV_FM_FM         = 0x2,     //0x2
    SV_NTSC_M        = 0x3,     //0x3
    SV_A2_BG         = 0x4,     //0x4
    SV_A2_DK1        = 0x5,     //0x5
    SV_A2_DK2        = 0x6,     //0x6
    SV_A2_DK3        = 0x7,     //0x7
    SV_PAL_I         = 0x8,     //0x8
    SV_PAL_BG        = 0x9,     //0x09
    SV_PAL_DK        = 0xa,     //0x0a
    SV_SECAM_L       = 0xb,     //0x0b
    SV_SECAM_L_PLUM  = 0xc,     //0x0c
    SV_SECAM_BG      = 0xd,     //0x0d, 0x0c is reserved for SECAM L' setting for tuner
    SV_SECAM_DK      = 0xe,      //0x0e
    SV_AUD_SYS_NUM   = 0xf
} TV_AUD_SYS_T;
#endif

/* Note that this enumeration should be consistent with AUD_DEC_MTS_T in middleware x_aud_dec.h */
typedef enum
{
    AUD_SOUND_MODE_UNKNOWN = 0,
    AUD_SOUND_MODE_MONO,
    AUD_SOUND_MODE_STEREO,
    AUD_SOUND_MODE_SUB_LANG,
    AUD_SOUND_MODE_DUAL1,
    AUD_SOUND_MODE_DUAL2,
    AUD_SOUND_MODE_NICAM_MONO,
    AUD_SOUND_MODE_NICAM_STEREO,
    AUD_SOUND_MODE_NICAM_DUAL1,
    AUD_SOUND_MODE_NICAM_DUAL2,
    AUD_SOUND_MODE_A2_DUAL1_DUAL2,
    AUD_SOUND_MODE_NICAM_DUAL1_DUAL2,
    AUD_SOUND_MODE_FM_MONO,
    AUD_SOUND_MODE_FM_STEREO
} AUD_SOUND_MODE_T;

typedef enum
{
    AUD_MATRIX_OUT_OFF = 0,
    AUD_MATRIX_OUT_L_MONO,
    AUD_MATRIX_OUT_R_MONO,
    AUD_MATRIX_OUT_LR_MIX
} AUD_MATRIX_OUT_T;

/// Audio mem buffer information table.
typedef struct
{
    UINT8* pData;
    UINT32 u4Length;
    UINT32 u4LoopCount;
}   MEM_BUFFER_INFO_T;

#ifdef CC_AUD_MULTI_BUFFER_MEMORY_PLAY
typedef struct
{
    UINT8* pData;
    UINT32 u4Length;
    UINT32 u4BufferSize;
    UINT32 u4Bid;   // Block data id
    BOOL   fgValid;
    BOOL   fgEOB;   // End of block data
} MEM_INFO_T;
#endif

typedef enum
{
    AUD_DUAL_DEC_UNINITIALIZED = 0,
    AUD_DUAL_DEC_INIT_AUX_DECODER,
    AUD_DUAL_DEC_PLAY,
    AUD_DUAL_DEC_STOP,
    AUD_DUAL_DEC_IDLE
}   AUD_DUAL_DEC_DRV_STATE_T;

typedef enum
{
    AUD_UNINITIALIZED = 0,
    AUD_TRIGGER_ADSP,
    AUD_WAIT_POWER_ON,
    AUD_INIT,
    AUD_WAIT_CMD,
    AUD_WAIT_PLAY,
    AUD_WAIT_PAUSE,
    AUD_WAIT_RESUME,
    AUD_WAIT_STOP,
    AUD_WAIT_STOPSTREAM,
    AUD_IDLE,
    AUD_ON_PLAY,
    AUD_ON_PAUSE,
    AUD_STOPPED,
    AUD_DECODE_INIT,
    AUD_CHANGE_FORMAT,
    AUD_DSP_ERROR
}   AUD_DRV_STATE_T;

typedef enum
{
    AUD_DRV_IDLE = 0,
    AUD_DRV_WORKING,
    AUD_DRV_SUSPENDING,
    AUD_DRV_SUSPENDED,
    AUD_DRV_RESUMING,
}   AUD_DRV_STR_STATE_T;

typedef enum
{
    AUD_HEART_BEAT_INIT = 0,
    AUD_HEART_BEAT_STOPPED,
    AUD_RENDER_NOT_START,
    AUD_RENDER_STARTED,
    AUD_RENDER_STOPPED
}   AUD_HEART_BEAT_STATE_T;

typedef enum
{
    AV_SYNC_FREE_RUN = 0,       // no need to syn
    AV_SYNC_SLAVE,              // syn to STC
    AV_SYNC_AUDIO_MASTER       // update A-PTS to A-STC, may use in PCR or HardDisk playback
}   AV_SYNC_MODE_T;

typedef enum
{
    AUD_MM_MODE_NORMAL = 0,
    AUD_MM_MODE_KARAOKE,
    AUD_MM_MODE_KARAOKE_MIC_PASS,
    AUD_MM_MODE_GAME,
    AUD_MM_MODE_PVR
}   AUD_MM_MODE_T;

typedef enum
{
    AUD_PATH_NONE,
    AUD_PATH_MAIN,
    AUD_PATH_SUB
}    AUD_PATH_T;

typedef enum
{
    AV_SYNC_STC_A1 = 0,
    AV_SYNC_STC_A2,
    AV_SYNC_STC_V1,
    AV_SYNC_STC_V2
}   AV_SYNC_STC_SELECT_T;

/* Audio command types. */
typedef enum
{
    AUD_CMD_PLAY       = 0,
    AUD_CMD_STOP       = 1,
    AUD_CMD_RESET      = 2,
    AUD_CMD_PAUSE      = 3,
    AUD_CMD_AVSYNC     = 4,
    AUD_CMD_LOADCODE   = 5,
    AUD_CMD_RESUME  = 6,
    AUD_CMD_CHANGE_FORMAT = 7,
    AUD_CMD_STOPSTREAM = 8,
    AUD_CMD_UNMUTE     = 9,
    AUD_CMD_NULL       = 10
}   AUD_DEC_CMD_T;

/* Audio command state.*/
typedef enum
{
    AUD_CMD_ST_PLAY    = 0,
    AUD_CMD_ST_STOP    = 1,
    AUD_CMD_ST_NULL
}   AUD_DEC_CMD_STATE_T;

/* Audio dual decoder command types. */
typedef enum
{
    AUD_DUAL_DEC_CMD_PLAY    = 0,
    AUD_DUAL_DEC_CMD_STOP    = 1
}   AUD_DUAL_DEC_CMD_T;

/* Audio play mute command types. */
typedef enum
{
    AUD_PM_OP_SET_DELAY = 0,
    AUD_PM_OP_UNMUTE,
    AUD_PM_CMD_RESET,
    AUD_PM_CMD_VDP_MODE_CHG,
    AUD_PM_CMD_MUTE,
    AUD_PM_CMD_VDP_MODE_CHG_DONE,
    AUD_PM_CMD_DSP_PLAY,
    AUD_PM_CMD_UNMUTE_WITH_VDP,
    AUD_PM_CMD_UNMUTE_TIME,
    AUD_PM_CMD_SET_PIP_VDO_PATH,
    AUD_PM_CMD_SET_DELAY_READY,
    AUD_PM_CMD_AOUT_ENABLE,
    AUD_PM_CMD_NUM
}   AUD_PLAY_MUTE_CMD_T;

/* IEC types. */
typedef enum
{
    AUD_IEC_CFG_PCM = 0,          // used for Linear PCM
    AUD_IEC_CFG_RAW,              // used for DD Encoder, UDC Converter, DTS, AAC
    AUD_IEC_CFG_DDP_PASS_THROUGH, // used for TB24 DDP Pass Through
    AUD_IEC_CFG_DDP,              // used for DDP Encoder
    AUD_IEC_CFG_AAC,              // used for AAC pass through
}   AUD_IEC_T;

/* IEC PCM Channel */
typedef enum
{
    AUD_IEC_PCM_CH_L_R = 0,
    AUD_IEC_PCM_CH_LS_RS,
    AUD_IEC_PCM_CH_C_SW,
    AUD_IEC_PCM_CH_7_8,
    AUD_IEC_LINE_IN,
    AUD_IEC_PCM_CH_9_10,
    AUD_IEC_AUX
}   AUD_IEC_CH_T;

typedef enum
{
    IEC_RAW_SRC_OFF = 0,
    IEC_RAW_SRC_DEC1_ON = 0x10,
    IEC_RAW_SRC_DEC2_ON = 0x20,
    IEC_RAW_SRC_DEC3_ON = 0x40
} AUD_IEC_RAWSRC_T;

typedef struct
{
    UINT8 u1DecId; //spdif input decoder id
    // spdif information
    AUD_IEC_T eIecCfg; // user select
    // driver register information
    SPDIF_REG_TYPE_T eSpdifRegType;
    // control information
    AUD_DEC_STREAM_FROM_T eStreamSrc;
    AUD_FMT_T eDecFormat;
} AUD_IECINFO_T;

typedef enum
{
    FS_16K = 0x00,
    FS_22K,
    FS_24K,
    FS_32K,
    FS_44K,
    FS_48K,
    FS_64K,
    FS_88K,
    FS_96K,
    FS_176K,
    FS_192K,
    FS_8K, // appended since 09/10/2007, don't change the order
    FS_11K, // appended since 09/10/2007, don't change the order
    FS_12K, // appended since 09/10/2007, don't change the order
    FS_52K, // appended since 24/02/2010, don't change the order
    FS_56K,
    FS_62K,  // appended since 24/02/2010, don't change the order
    FS_64K_SRC,
    FS_6K,
    FS_10K,
    FS_5K,
    FS_NUM,
}   SAMPLE_FREQ_T;

/* Audio types. */
typedef enum
{
    AUD_TYPE_UNKNOWN = 0,
    AUD_TYPE_MONO,                  /* 1/0 */
    AUD_TYPE_MONO_SUB,              /* 1+sub-language */
    AUD_TYPE_DUAL_MONO,             /* 1+1 */
    AUD_TYPE_STEREO,                /* 2/0 */
    AUD_TYPE_STEREO_SUB,            /* 2+sub-language */
    AUD_TYPE_STEREO_DOLBY_SURROUND, /* 2/0, dolby surround */
    AUD_TYPE_SURROUND_2CH,          /* 2/1 */
    AUD_TYPE_SURROUND,              /* 3/1 */
    AUD_TYPE_3_0,                   /* 3/0 */
    AUD_TYPE_4_0,                   /* 2/2 */
    AUD_TYPE_5_1,                   /* 3/2 +SW */
    AUD_TYPE_7_1,                   /* 5/2 */
    AUD_TYPE_FM_MONO_NICAM_MONO,
    AUD_TYPE_FM_MONO_NICAM_STEREO,
    AUD_TYPE_FM_MONO_NICAM_DUAL,
    AUD_TYPE_OTHERS,
    AUD_TYPE_5_0,                    /* 3/2 */
    AUD_TYPE_JOINT_STEREO,
    AUD_TYPE_FMRDO_MONO,
    AUD_TYPE_FMRDO_STEREO,

    AUD_TYPE_MONO_LFE,                 /* 1/0.L */
    AUD_TYPE_DUAL_MONO_LFE,            /* 1+1.L */
    AUD_TYPE_STEREO_LFE,               /* 2/0.L */
    AUD_TYPE_SURROUND_2CH_LFE,         /* 2/1.L */
    AUD_TYPE_SURROUND_LFE,             /* 3/1.L */
    AUD_TYPE_3_1,                  /* 3/0.L */
    AUD_TYPE_4_1,                  /* 2/2.L */

    AUD_TYPE_LEFT_SIDE_STEREO,     //paul_flac
    AUD_TYPE_RIGHT_SIDE_STEREO,
    AUD_TYPE_MID_SIDE_STEREO
}   AUD_TYPE_T;

/* audio decoder types. */
typedef enum
{
    AUD_DECODE_TYPE_UNKNOWN = 0,
    AUD_DECODE_TYPE_AC3,
    AUD_DECODE_TYPE_EAC3,
    AUD_DECODE_TYPE_FLAC,//paul_flac
    AUD_DECODE_TYPE_AAC,
    AUD_DECODE_TYPE_HEAAC,
    AUD_DECODE_TYPE_HEAAC_V2,
    AUD_DECODE_TYPE_LPCM_ALAW,
    AUD_DECODE_TYPE_LPCM_ULAW,
    AUD_DECODE_TYPE_MPEG1_LAYER3,
    AUD_DECODE_TYPE_MPEG1_LAYER2,
    AUD_DECODE_TYPE_MPEG1_LAYER1,
    AUD_DECODE_TYPE_MPEG2_LAYER3,
    AUD_DECODE_TYPE_MPEG2_LAYER2,
    AUD_DECODE_TYPE_MPEG2_LAYER1,
    AUD_DECODE_TYPE_DTS_NORMAL, // DTS M6
    AUD_DECODE_TYPE_DTS_EXPRESS,
    AUD_DECODE_TYPE_DTS_HD,
    AUD_DECODE_TYPE_AC4,
    AUD_DECODE_TYPE_OPUS,
    AUD_DECODE_TYPE_MAT_TRUEHD, // 20
    AUD_DECODE_TYPE_MAT_PCM
} AUD_DECODE_TYPE_T;

/// Audio ain configuration table.
typedef struct
{
    AUD_DEC_STREAM_FROM_T eStrSrc;      ///< Stream source
    DATA_FORMAT_T      eFormat;         ///< format of alignment
    DAC_DATA_NUMBER_T  eBits;           ///< number of bits per sample
    LRCK_CYC_T         eCycle;          ///< cycles per sample
    MCLK_FREQUENCY_T   eSampleFreq;     ///< DAC sampling frequence
    BOOL               fgDataInvert;    ///< Invert audio output for OP phase
    BOOL               fgLRInvert;      ///< Invert L/R audio output
    //========================================================================
    UINT8 uFormat;                      ///< format of alignment
    UINT8 uBits;                        ///< number of bits per sample
    UINT8 uCycle;                       ///< cycles per sample
    BOOL  fgIsSPDIFin;                  ///< TRUE is slave mode
    BOOL  fgLrckInv;                    ///< TRUE while LRCK Low is Left channel
}   AIN_CFG_T;

typedef enum
{
    AVC_COMP_VIDEO_0 = 0,               ///< Composite video (CVBS) # 0
    AVC_COMP_VIDEO_1,                   ///< Composite video (CVBS) # 1
    AVC_COMP_VIDEO_2,                   ///< Composite video (CVBS) # 2
    AVC_S_VIDEO_0,                      ///< S-Video
    AVC_Y_PB_PR_0,                      ///< YPbPr # 0
    AVC_Y_PB_PR_1,                      ///< YPbPr # 1
    AVC_VGA,                            ///< VGA, via headphone jet
    AVC_HDMI,                           ///< HDMI (DVI)
    AVC_S_VIDEO_1,
    AVC_S_VIDEO_2
}   AV_COMPONENT_T;

/// Audio decoder information table.
typedef struct
{
    AUD_FMT_T   e_aud_fmt;
    AUD_TYPE_T  e_aud_type;
    UINT32      ui4_sample_rate;
    UINT32      ui4_data_rate;
    UINT8       ui1_bit_depth;
    AUD_DECODE_TYPE_T e_dec_type;
    AUD_META_T e_meta_type;
} AUD_DEC_INFO_T;

typedef enum
{
    AUD_COND_ERROR = -1,
    AUD_COND_CTRL_DONE,
    AUD_COND_AUD_CLIP_DONE,
    AUD_COND_MEM_BUFFER_DONE,
    AUD_COND_FLUSH_DONE,
    AUD_COND_FEED_ME_ON,
    AUD_COND_FEED_ME_OFF,
    AUD_COND_AUD_INFO_CHG,
    AUD_COND_TV_SYS_DETECTED,
    AUD_COND_AUD_DECODE_STATUS_CHG,
    AUD_COND_MM_EOS,
    AUD_COND_FM_RADIO_DET_DONE = 10,
    AUD_COND_PROTECT_INFO,       ///[Joel]20101011, CC_AUD_HDMI_SPDIF_CFG
    AUD_COND_UPLOAD_INFO_CHG,
    AUD_COND_AUD_CLIP_PLAY_DONE,
    AUD_COND_AUD_CODEC_CHG,      //audio codec change notify
    AUD_COND_AUD_BIT_RATE_CHG,
    AUD_COND_STREAM_FORMAT_CHG,
    AUD_COND_MUTE_STATE_CHG,
    AUD_COND_AUD_MEM_STATE_CHG,
    AUD_COND_INPUT_SOURCE_STATE_CHG,
    AUD_COND_TTS_PLAY_FINISH_DONE = 20 ,
    AUD_COND_AUD_CLIP_REQUEST_DATA,
    AUD_COND_AUD_SEEK_DONE,
    AUD_COND_DOLBY_ATMOS,
    AUD_COND_NFY_MAX, // AUD_COND_NFY_T should be the same as MTAUDEDC_NFY_T
}   AUD_COND_NFY_T;

typedef enum
{
    AUD_DECODE_UNKNOWN = -1, ///< initial value; the decode status should be initialized to this value after connect/disconnect or AUD_DEC_CTRL_STOP
    AUD_DECODE_NO_DATA,      ///< audio decoder found no audio data in stream. there will be a timer threashold value for decoder to change the decode status from NORMAL/ERROR to NO_DATA
    AUD_DECODE_NORMAL,     ///< audio decoder decode the first audio frame data successfully after AUD_DEC_CTRL_PLAY
    AUD_DECODE_ERROR,       ///< audio decoder decode the audio data abnormally (e.g. unsupported format)
    AUD_DECODE_NOT_SUPPORT, ///< unsupported format for WMA decoder
    AUD_DECODE_FMT_CONFLICT
}   AUD_DECODE_STATUS_T;

typedef enum
{
    AUD_MEM_UNKNOWN = -1,
    AUD_MEM_DECODE_DONE,
    AUD_MEM_ENDOF_DATA,
    AUD_MEM_NO_DATA,
    AUD_MEM_NORMAL,
    AUD_MEM_ERROR,
    AUD_MEM_NOT_SUPPORT
} AUD_MEM_STATE_T;

typedef enum
{
    AUD_SOURCE_UNKOWN = -1,
    AUD_SOURCE_NO_SIGNAL,
    AUD_SOURCE_STABLE,
    AUD_SOURCE_UNSTABLE,
} AUD_SOURCE_STATE_T;

typedef enum
{
    AUD_HP_COND_HEADPHONE_COND_UNPLUGED = 0,
    AUD_HP_COND_HEADPHONE_COND_PLUGED
} AUD_HP_COND_NFY_T;

typedef enum
{
    AUD_YW_COND_UPLOAD_ERROR = -1,
    AUD_YW_COND_UPLOAD_OK
} AUD_YW_COND_NFY_T;

typedef enum        //Andrew Wen : 2007/2/2 for STC differnece setting
{
    AUD_STC_LO = 0,
    AUD_STC_HI,
    AUD_STC_WS
}   AUD_STC_DIFF_SET;

typedef enum
{
    AUD_NFY_STREAM = 0,
    AUD_NFY_HP,
    AUD_NFY_YW
}   AUD_NFY_TAG_T;

/* Notify function */
typedef void (*AudDecNfyFct) (void*           pvNfyTag,
                              UINT8           u1DecId,
                              AUD_COND_NFY_T  eNfyCond,
                              UPTR          u4Data1,
                              UPTR          u4Data2);

/* Notify setting info. */
/// Audio Notify setting info table.
typedef struct
{
    void*           pvTag;

    AudDecNfyFct    pfAudDecNfy;
}   AUD_NFY_INFO_T;

/// Audio reverb mode table.
typedef struct
{
    UINT32      u4Gain;
    UINT8       u1ReverbBank0;
    UINT8       u1ReverbBank1;
    UINT8       u1ReverbBank2;
    UINT8       u1ReverbBank3;
} REVERB_MODE_T;

/// Audio PAL2 setting table.
typedef struct
{
    UINT16 u2Config;
    UINT16 u2Mode;
} PL2_SETTING_T;

/// Audio EQ flag table.
typedef struct
{
    UINT8       fgChEqOn       : 1;
    UINT8       fgBassBoostOn  : 1;
    UINT8       fgSuperBassOn  : 1;
    UINT8       fgClearBoostOn : 1;
    UINT8       Reserve        : 4;
}   EQ_FLAG_T;

typedef union
{
    UINT8       u1Eq;
    EQ_FLAG_T   rField;
} EQ_FLAG_UNION_T;

/// Audio channel band table.
typedef struct
{
    UINT8       u1Channel;
    UINT8       u1Band;
} CH_BAND_T;

/// Audio EQ band table.
typedef struct
{
    UINT8       u1BandIdx;
    INT8        i1Level;
}   EQ_BAND_CFG;

/// Audio EQ band information table.
typedef struct
{
    UINT8       ui1_band_idx;
    INT8        i1_max_lvl;
    INT8        i1_min_lvl;
    UINT32      ui4_freq_lower_bound;
    UINT32      ui4_freq_upper_bound;
} AUD_EQ_BAND_INFO_T;

// Test tone freqence definition for table selection
typedef enum
{
    TEST_TONE_1K_0dB = 0,
    TEST_TONE_1K_20dB,
    TEST_TONE_1K_30dB,
    TEST_TONE_1K_60dB,
    TEST_TONE_1K_120dB,
    TEST_TONE_2K_0dB,
    TEST_TONE_3K_0dB,
    TEST_TONE_4K_0dB,
    TEST_TONE_5K_0dB,
    TEST_TONE_8K_0dB,
    TEST_TONE_12K_20dB,  //10
    TEST_TONE_12K_30dB,  //11
    TEST_TONE_12_5K_0dB, //12
    TEST_TONE_20K_0dB,
    TEST_TONE_200_0dB,
    TEST_TONE_200_20dB,
    TEST_TONE_200_30dB,
    TEST_TONE_200_60dB,
    TEST_TONE_400_0dB,
    TEST_TONE_1K3K_0dB,
    TEST_TONE_1K_0dB_LCH_ONLY,
    TEST_TONE_1K_MIX_0dB,
    TEST_TONE_400_1K_0dB,
    TEST_TONE_1K_0dB_RCH_ONLY,
    TEST_TONE_400_0dB_LCH_ONLY,
    TEST_TONE_32K_2CH_15KHZ_0dB
} TEST_TONE_T;

/// Audio clip type table.
typedef struct
{
    BOOL        fgPlayTestTone;
    UINT32      u4OutputChlSelect;
    UINT32      u4RepeatCount;
    UINT32      u4ToneFreq;
    UINT32      u4AudCtrlTypeAddr;

    AUD_FMT_T           eAudFmt;
    MEM_BUFFER_INFO_T   eMemBuf;
} AUD_CLIP_TYPE_T;

/* BBE mode */
typedef enum
{
    AUD_BBE_MODE_OFF = 0,
    AUD_BBE_MODE_BBE,
    AUD_BBE_MODE_BBE_VIVA
}   AUD_BBE_MODE_T;

typedef enum
{
    AUD_FM_RADIO_NON_DETECTED = 0,
    AUD_FM_RADIO_DETECTED
} AUD_FM_RADIO_DET_T;

typedef enum
{
    AUD_VIDEO_IS_STABLE = 0,
    AUD_VIDEO_IS_UNSTABLE = 1
}   AUD_VIDEO_STATE_T;

#if defined(CC_AUD_UNICORN_CUST) || defined(CC_AUD_SAKURA_RENDER)
/// Audio Delay.
typedef struct
{
    /* The current mechanism / BE Video Delay */
    UINT32 u4VideoDrvDly_ms;        /* ms,   Delay from MTK video driver                     */
    UINT32 u4BeDly_ms;              /* ms,   Delay from S 4K/2K Ex-FRC                    */

    /* AQ Sheet: MM_AU_DLY */
    INT32  i4MainAdj_ms;            /* ms,   (-128..127ms) caculate from AQ(MM_MAIN_ADJ)   */
    UINT32 u4SpAdj_ms;              /* ms,   value from AQ(MM_SP_ADJ)     */
    UINT32 u4HoLoSwAdj_ms;          /* ms,   Value from AQ(MM_HOLOSW_ADJ)  */
    UINT32 u4SpdifAdj_ms;           /* ms,   Value from AQ(MM_SPDIF_ADJ).  */
    UINT32 u4WswDly_ms;             /* ms,   Value from AQ(MM_WSW_DLY)  */
    UINT32 u4BtDly_ms;              /* ms,   Value from AQ(MM_BT_DLY)  */

    /* AQ Sheet: MM_SMT */
    INT32  u4FLDly_sample;            /* sample,  Value from AQ(MM_FL_DELAY)   */
    INT32  u4FRDly_sample;            /* sample,  Value from AQ(MM_FR_DELAY)   */
    INT32  u4WLDly_sample;            /* sample,  Value from AQ(MM_WL_DELAY)   */
    INT32  u4WRDly_sample;            /* sample,  Value from AQ(MM_WR_DELAY)   */

    /* sample delay */
    UINT32 u4MainDelay_sample;
    UINT32 u4SpDiffDelay_sample;
    UINT32 u4HpDiffDelay_sample;
    UINT32 u4SpdifDiffDelay_sample; /* */
    UINT32 u4ScartMainDelay_sample;     // newly add for SCART Main Delay
#ifdef CC_AUD_SAKURA_RENDER
    UINT32 u4SpdifEncDiffDelay_sample; /* */
#endif //CC_AUD_SAKURA_RENDER

    /* Subwoofer Delay UI */
    UINT32 u4SubWooferSync;         /* 0.24, UI value 0..24 means -12..12                    */
    UINT32 u4USwDlyAdj_sample;      /* bank, from UI mapping to banks,                       */
    UINT32 u4LSwDlyAdj_sample;      /* bank, from UI mapping to banks,                       */

    /* SPDIF RAW */
    UINT32 u4SpdifRawDelay;         /* Frames, Value set to APROC SPDIF Raw Delay.           */
#ifdef CC_MT5895
    UINT32 u4EarcAdj_ms;
    UINT32 u4EarcDiffDelay_sample;
    UINT32 u4EarcRawDelay;
    UINT32 u4EarcEncDiffDelay_sample;
#endif
} AUD_DLY_SETVAL_T;
#endif /* CC_AUD_UNICORN_CUST, CC_AUD_SAKURA_RENDER */
//-----------------------------------------------------------------------------
// Begin of NVM and Profile Functions
//   for customers use EEPROM and FLASH audio parameters
//-----------------------------------------------------------------------------
// NVM Data Releated
#ifdef CC_AUD_USE_NVM
#define AUD_NVM_LENGTH (0xFF)
#define AUD_NVM_OFFSET (0x3500) /* For NVM layout issue */
#define AUD_NVM_PROFILE_OFFSET1 (0x0520) /* 120913 Isaac */
#define AUD_NVM_PROFILE_OFFSET2 (0x1050) /* 120913 Isaac */
#define AUD_NVM_PROFILE_OFFSET3 (0x1520) /* 120913 Isaac */
#define AUD_FLASH_LENGTH        (1152)    //2k13 v0.3
#define AUD_FLASH_COMMON_LENGTH (996)
//#define AUD_DD_BANNER_OFFSET (0x11D3)
#define AUD_NVM_FLAG        (0xaf)
#define CC_AUD_NVM_CHECK_VALID

typedef enum
{
    AUD_NVM_READ = 0,
    AUD_NVM_WRITE,
    AUD_NVM_INIT,
    AUD_NVM_VOL_INIT,
    AUD_NVM_PROFILE_QRY,
    AUD_NVM_VS_INIT,
    AUD_NVM_PROFILE_OFFSET
} AUD_NVM_OP_T;

typedef enum
{
    AUD_NVM_NOP = 0,
    // MTS
    AUD_NVM_MTS_OBSRV_TIME,
    AUD_NVM_MTS_STEREO_TIME,
    AUD_NVM_MTS_SAP_TIME,
    AUD_NVM_MTS_UP_THR_BTSC,
    AUD_NVM_MTS_LO_THR_BTSC,
    AUD_NVM_MTS_UP_THR_SAP,
    AUD_NVM_MTS_LO_THR_SAP,
    AUD_NVM_MTS_NMUTE_FM_UPTHR,
    AUD_NVM_MTS_NMUTE_FM_LOTHR,
    AUD_NVM_MTS_NMUTE_SAP_UPTHR,
    AUD_NVM_MTS_NMUTE_SAP_LOTHR,
    AUD_NVM_MTS_PRE_BTSC_STMONO,
    AUD_NVM_MTS_PRE_SAP,
    AUD_NVM_MTS_HIGH_DEV,
    AUD_NVM_MTS_CARRIER_SHIFT,
    AUD_NVM_MTS_FMCARR_MUTE,
    // A2
    AUD_NVM_A2_HDEV_MODE,
    AUD_NVM_A2_CR_SHIFT,
    AUD_NVM_A2_FM_MUTE,
    AUD_NVM_A2_UP_THR_FM,
    AUD_NVM_A2_LO_THR_FM,
    AUD_NVM_A2_PRE,
    AUD_NVM_A2_NON_EU,
    // PAL
    AUD_NVM_PAL_HDEV_MODE,
    AUD_NVM_PAL_CR_SHIFT,
    AUD_NVM_PAL_AM_MUTE,
    AUD_NVM_PAL_UP_THR_AM,
    AUD_NVM_PAL_LO_THR_AM,
    AUD_NVM_PAL_FM_MUTE,
    AUD_NVM_PAL_UP_THR_FM,
    AUD_NVM_PAL_LO_THR_FM,
    AUD_NVM_PAL_PRE,
    AUD_NVM_PAL_AM_PRE,
    AUD_NVM_PAL_NICAM_PRE,
    AUD_NVM_PAL_NON_EU,
    AUD_NVM_A2_PRE_DK,
    AUD_NVM_PAL_PRE_DK,
    AUD_NVM_PAL_AM_PRE_DK,
    AUD_NVM_PAL_NICAM_PRE_DK,
    // AVC
    AUD_NVM_AVC_ADJUST_RATE,
    AUD_NVM_AVC_UI_ADJUST_RATE,
    AUD_NVM_AVC_LEVEL,
    AUD_NVM_AVC_GAIN_UP,
    AUD_NVM_LIPSYNC_A,
    AUD_NVM_LIPSYNC_B,
    AUD_NVM_LIPSYNC_C,
    AUD_NVM_FLAT,
    AUD_NVM_REVERB,
    // AVL
    AUD_NVM_AVL_FC,
    AUD_NVM_AVL_BAND0_TARGET,
    AUD_NVM_AVL_BAND0_ATTACK,
    AUD_NVM_AVL_BAND0_RELEASE,
    AUD_NVM_AVL_BAND0_RATIO,
    AUD_NVM_AVL_BAND0_SILENCE,
    AUD_NVM_AVL_BAND0_EXPAND,
    AUD_NVM_AVL_BAND0_POST_GAIN,
    AUD_NVM_AVL_BAND0_HOLD,
    AUD_NVM_AVL_BAND1_TARGET,
    AUD_NVM_AVL_BAND1_ATTACK,
    AUD_NVM_AVL_BAND1_RELEASE,
    AUD_NVM_AVL_BAND1_RATIO,
    AUD_NVM_AVL_BAND1_SILENCE,
    AUD_NVM_AVL_BAND1_EXPAND,
    AUD_NVM_AVL_BAND1_POST_GAIN,
    AUD_NVM_AVL_BAND1_HOLD,
    // Vol
    AUD_NVM_VOL_MIN,
    AUD_NVM_VOL_MAX,
    AUD_NVM_VOL_STEP1,
    AUD_NVM_VOL_STEP2,
    AUD_NVM_VOL_STEP3,
    AUD_NVM_VOL_STEP4,
    AUD_NVM_VOL_STEP5,
    // HP Vol
    AUD_NVM_HP_VOL_MAX,
    AUD_NVM_HP_VOL_STEP1,
    AUD_NVM_HP_VOL_STEP2,
    AUD_NVM_HP_VOL_STEP3,
    AUD_NVM_HP_VOL_STEP4,
    AUD_NVM_HP_VOL_STEP5,
    // Input source gain
    AUD_NVM_PREHDMI,
    AUD_NVM_PREAV,
    AUD_NVM_PREUSB,
    AUD_NVM_PREATV,
    AUD_NVM_PREDTV,
    // Bass
    AUD_NVM_BASS_SUBWOOFER,
    AUD_NVM_BASS_SPKSIZE,
    // VS
    AUD_NVM_VS_CLARITY,
    AUD_NVM_VS_WIDTH,
    AUD_NVM_VS_LRGAIN,
    AUD_NVM_VS_XTALK_GIAN,
    AUD_NVM_VS_OPGAIN,
    AUD_NVM_VS_SWGAIN,
    // 3DVS
    AUD_NVM_3DVS_CLARITY,
    AUD_NVM_3DVS_WIDTH,
    AUD_NVM_3DVS_LRGAIN,
    AUD_NVM_3DVS_XTALK_GIAN,
    AUD_NVM_3DVS_OPGAIN,
    AUD_NVM_3DVS_SWGAIN,
    AUD_NVM_VS_FO,
    // BBE
    AUD_NVM_BBE_LEVEL,
    AUD_NVM_BBE_UPGAIN,
    AUD_NVM_BBE_LOGAIN,
    AUD_NVM_BBE_3DGAIN,
    AUD_NVM_PL2_ON,
    AUD_NVM_PL2_MODE,
    AUD_NVM_MTS_FMSATU_MUTE,
    AUD_NVM_MTS_FMSATU_MUTE_UPTHR,
    AUD_NVM_MTS_FMSATU_MUTE_LOTHR,
    AUD_NVM_PILOT_OFF_DETECT,
    AUD_NVM_AMP_MUTE,
    // Extra volume gain
    AUD_NVM_LR_EXTRA_LINEIN,
    AUD_NVM_LR_EXTRA_OTHERS,
    // Spdif
  //  AUD_NVM_SPDIF_NORMAL,
  //  AUD_NVM_SPDIF_LOW,
    AUD_NVM_SPDIF_MORE,
    AUD_NVM_SPDIF_MEDIUM,
    AUD_NVM_SPDIF_LESS,
    // Limiter
    AUD_NVM_LIMITER_MODE,
    AUD_NVM_LIMITER_FLAG,
    AUD_NVM_LIMITER_THRES,
    AUD_NVM_LIMITER_ATTACK_RATE,
    AUD_NVM_LIMITER_RELEASE_RATE,
    AUD_NVM_LIMITER_RATIO,
    AUD_NVM_CUTOFF_FREQ,
    // Notch
    AUD_NVM_NOTCH_FC,
    AUD_NVM_NOTCH_Q_ON,
    AUD_NVM_NOTCH_Q_OFF,
    // EQ Table & Normal & Clear sound on
    AUD_NVM_EQ_USER1_1_CS_ON,
    AUD_NVM_EQ_USER1_2_CS_ON,
    AUD_NVM_EQ_USER1_3_CS_ON,
    AUD_NVM_EQ_USER1_4_CS_ON,
    AUD_NVM_EQ_USER1_5_CS_ON,
    AUD_NVM_EQ_USER1_6_CS_ON,
    AUD_NVM_EQ_USER1_7_CS_ON,
    // EQ Wall & Normal & Clear sound on
    AUD_NVM_EQ_USER2_1_CS_ON,
    AUD_NVM_EQ_USER2_2_CS_ON,
    AUD_NVM_EQ_USER2_3_CS_ON,
    AUD_NVM_EQ_USER2_4_CS_ON,
    AUD_NVM_EQ_USER2_5_CS_ON,
    AUD_NVM_EQ_USER2_6_CS_ON,
    AUD_NVM_EQ_USER2_7_CS_ON,
    // EQ Table & Normal & Clear sound off
    AUD_NVM_EQ_USER1_1_CS_OFF,
    AUD_NVM_EQ_USER1_2_CS_OFF,
    AUD_NVM_EQ_USER1_3_CS_OFF,
    AUD_NVM_EQ_USER1_4_CS_OFF,
    AUD_NVM_EQ_USER1_5_CS_OFF,
    AUD_NVM_EQ_USER1_6_CS_OFF,
    AUD_NVM_EQ_USER1_7_CS_OFF,
    // EQ Wall & Normal & Clear sound off
    AUD_NVM_EQ_USER2_1_CS_OFF,
    AUD_NVM_EQ_USER2_2_CS_OFF,
    AUD_NVM_EQ_USER2_3_CS_OFF,
    AUD_NVM_EQ_USER2_4_CS_OFF,
    AUD_NVM_EQ_USER2_5_CS_OFF,
    AUD_NVM_EQ_USER2_6_CS_OFF,
    AUD_NVM_EQ_USER2_7_CS_OFF,
    // EQ Table & Skype
    AUD_NVM_EQ_USER3_1,
    AUD_NVM_EQ_USER3_2,
    AUD_NVM_EQ_USER3_3,
    AUD_NVM_EQ_USER3_4,
    AUD_NVM_EQ_USER3_5,
    AUD_NVM_EQ_USER3_6,
    AUD_NVM_EQ_USER3_7,
    // EQ Wall & Skype
    AUD_NVM_EQ_USER4_1,
    AUD_NVM_EQ_USER4_2,
    AUD_NVM_EQ_USER4_3,
    AUD_NVM_EQ_USER4_4,
    AUD_NVM_EQ_USER4_5,
    AUD_NVM_EQ_USER4_6,
    AUD_NVM_EQ_USER4_7,
    // VB
    AUD_NVM_VB_ONOFF,
    AUD_NVM_VB_FC,
    AUD_NVM_VB_GAIN1,
    AUD_NVM_VB_GAIN2,
    // PEQ 0/1 for Bass/Treble
    AUD_NVM_PEQ_ONOFF,
    AUD_NVM_PEQ_FC_USER1_0,
    AUD_NVM_PEQ_Q_USER1_0,
    AUD_NVM_PEQ_GAIN_USER1_0,
    AUD_NVM_PEQ_FC_USER1_1,
    AUD_NVM_PEQ_Q_USER1_1,
    AUD_NVM_PEQ_GAIN_USER1_1,
    // PEQ 2-8 Wall & Normal & Clear sound on
    AUD_NVM_PEQ_FC_USER1_2,
    AUD_NVM_PEQ_Q_USER1_2,
    AUD_NVM_PEQ_GAIN_USER1_2_CS_ON,
    AUD_NVM_PEQ_FC_USER1_3,
    AUD_NVM_PEQ_Q_USER1_3,
    AUD_NVM_PEQ_GAIN_USER1_3_CS_ON,
    AUD_NVM_PEQ_FC_USER1_4,
    AUD_NVM_PEQ_Q_USER1_4,
    AUD_NVM_PEQ_GAIN_USER1_4_CS_ON,
    AUD_NVM_PEQ_FC_USER1_5,
    AUD_NVM_PEQ_Q_USER1_5,
    AUD_NVM_PEQ_GAIN_USER1_5_CS_ON,
    AUD_NVM_PEQ_FC_USER1_6,
    AUD_NVM_PEQ_Q_USER1_6,
    AUD_NVM_PEQ_GAIN_USER1_6_CS_ON,
    AUD_NVM_PEQ_FC_USER1_7,
    AUD_NVM_PEQ_Q_USER1_7,
    AUD_NVM_PEQ_GAIN_USER1_7_CS_ON,
    AUD_NVM_PEQ_FC_USER1_8,
    AUD_NVM_PEQ_Q_USER1_8,
    AUD_NVM_PEQ_GAIN_USER1_8_CS_ON,
    // PEQ Table & Normal & Clear sound on
    AUD_NVM_PEQ_FC_USER2_0,
    AUD_NVM_PEQ_Q_USER2_0,
    AUD_NVM_PEQ_GAIN_USER2_0,
    AUD_NVM_PEQ_FC_USER2_1,
    AUD_NVM_PEQ_Q_USER2_1,
    AUD_NVM_PEQ_GAIN_USER2_1,
    AUD_NVM_PEQ_FC_USER2_2,
    AUD_NVM_PEQ_Q_USER2_2,
    AUD_NVM_PEQ_GAIN_USER2_2_CS_ON,
    AUD_NVM_PEQ_FC_USER2_3,
    AUD_NVM_PEQ_Q_USER2_3,
    AUD_NVM_PEQ_GAIN_USER2_3_CS_ON,
    AUD_NVM_PEQ_FC_USER2_4,
    AUD_NVM_PEQ_Q_USER2_4,
    AUD_NVM_PEQ_GAIN_USER2_4_CS_ON,
    AUD_NVM_PEQ_FC_USER2_5,
    AUD_NVM_PEQ_Q_USER2_5,
    AUD_NVM_PEQ_GAIN_USER2_5_CS_ON,
    AUD_NVM_PEQ_FC_USER2_6,
    AUD_NVM_PEQ_Q_USER2_6,
    AUD_NVM_PEQ_GAIN_USER2_6_CS_ON,
    AUD_NVM_PEQ_FC_USER2_7,
    AUD_NVM_PEQ_Q_USER2_7,
    AUD_NVM_PEQ_GAIN_USER2_7_CS_ON,
    AUD_NVM_PEQ_FC_USER2_8,
    AUD_NVM_PEQ_Q_USER2_8,
    AUD_NVM_PEQ_GAIN_USER2_8_CS_ON,
    // PEQ 2-8 Wall & Normal & Clear sound off
    AUD_NVM_PEQ_GAIN_USER1_2_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_3_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_4_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_5_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_6_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_7_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER1_8_CS_OFF,
    // PEQ Table & Normal & Clear sound off
    AUD_NVM_PEQ_GAIN_USER2_2_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_3_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_4_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_5_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_6_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_7_CS_OFF,
    AUD_NVM_PEQ_GAIN_USER2_8_CS_OFF,
    // PEQ Wall & Skype
    AUD_NVM_PEQ_FC_USER3_2,
    AUD_NVM_PEQ_Q_USER3_2,
    AUD_NVM_PEQ_GAIN_USER3_2,
    AUD_NVM_PEQ_FC_USER3_3,
    AUD_NVM_PEQ_Q_USER3_3,
    AUD_NVM_PEQ_GAIN_USER3_3,
    AUD_NVM_PEQ_FC_USER3_4,
    AUD_NVM_PEQ_Q_USER3_4,
    AUD_NVM_PEQ_GAIN_USER3_4,
    AUD_NVM_PEQ_FC_USER3_5,
    AUD_NVM_PEQ_Q_USER3_5,
    AUD_NVM_PEQ_GAIN_USER3_5,
    AUD_NVM_PEQ_FC_USER3_6,
    AUD_NVM_PEQ_Q_USER3_6,
    AUD_NVM_PEQ_GAIN_USER3_6,
    AUD_NVM_PEQ_FC_USER3_7,
    AUD_NVM_PEQ_Q_USER3_7,
    AUD_NVM_PEQ_GAIN_USER3_7,
    AUD_NVM_PEQ_FC_USER3_8,
    AUD_NVM_PEQ_Q_USER3_8,
    AUD_NVM_PEQ_GAIN_USER3_8,
    // PEQ Table & Skype
    AUD_NVM_PEQ_FC_USER4_0,
    AUD_NVM_PEQ_Q_USER4_0,
    AUD_NVM_PEQ_GAIN_USER4_0,
    AUD_NVM_PEQ_FC_USER4_1,
    AUD_NVM_PEQ_Q_USER4_1,
    AUD_NVM_PEQ_GAIN_USER4_1,
    AUD_NVM_PEQ_FC_USER4_2,
    AUD_NVM_PEQ_Q_USER4_2,
    AUD_NVM_PEQ_GAIN_USER4_2,
    AUD_NVM_PEQ_FC_USER4_3,
    AUD_NVM_PEQ_Q_USER4_3,
    AUD_NVM_PEQ_GAIN_USER4_3,
    AUD_NVM_PEQ_FC_USER4_4,
    AUD_NVM_PEQ_Q_USER4_4,
    AUD_NVM_PEQ_GAIN_USER4_4,
    AUD_NVM_PEQ_FC_USER4_5,
    AUD_NVM_PEQ_Q_USER4_5,
    AUD_NVM_PEQ_GAIN_USER4_5,
    AUD_NVM_PEQ_FC_USER4_6,
    AUD_NVM_PEQ_Q_USER4_6,
    AUD_NVM_PEQ_GAIN_USER4_6,
    AUD_NVM_PEQ_FC_USER4_7,
    AUD_NVM_PEQ_Q_USER4_7,
    AUD_NVM_PEQ_GAIN_USER4_7,
    AUD_NVM_PEQ_FC_USER4_8,
    AUD_NVM_PEQ_Q_USER4_8,
    AUD_NVM_PEQ_GAIN_USER4_8,
        // Volume dependent PEQ
    AUD_NVM_PEQ_GAIN_VOL1_7,
    AUD_NVM_PEQ_GAIN_VOL2_7,
    AUD_NVM_PEQ_GAIN_VOL3_7,
    AUD_NVM_PEQ_GAIN_VOL4_7,
    AUD_NVM_PEQ_GAIN_VOL1_8,
    AUD_NVM_PEQ_GAIN_VOL2_8,
    AUD_NVM_PEQ_GAIN_VOL3_8,
    AUD_NVM_PEQ_GAIN_VOL4_8,
    // Beep tone
    AUD_NVM_BEEP_FC,
    AUD_NVM_BEEP_VOLUME,
    AUD_NVM_BEEP_DURATION,
    // Bass management
    AUD_NVM_BASS_MANA_ONOFF,
    // Dual mono
    AUD_NVM_DUAL_MONO,
    #if 1
    // Class D (TAS5711)
    AUD_NVM_D_SYS_CTRL_REG2,
//  AUD_NVM_D_MASTER_VOL,
//  AUD_NVM_D_CH1_VOL,
//  AUD_NVM_D_CH2_VOL,
//  AUD_NVM_D_CH3_VOL,
    AUD_NVM_D_VOL_CONF_REG,
    AUD_NVM_D_ADBD,
    AUD_NVM_D_INPUT_MUX,
    AUD_NVM_D_CH4_SRC_SEL,
    AUD_NVM_D_PWM_MUX,

    AUD_NVM_D_CH1_BQ0_1,
    AUD_NVM_D_CH1_BQ0_2,
    AUD_NVM_D_CH1_BQ0_3,
    AUD_NVM_D_CH1_BQ0_4,
    AUD_NVM_D_CH1_BQ0_5,
    AUD_NVM_D_CH1_BQ1_1,
    AUD_NVM_D_CH1_BQ1_2,
    AUD_NVM_D_CH1_BQ1_3,
    AUD_NVM_D_CH1_BQ1_4,
    AUD_NVM_D_CH1_BQ1_5,
    AUD_NVM_D_CH1_BQ2_1,
    AUD_NVM_D_CH1_BQ2_2,
    AUD_NVM_D_CH1_BQ2_3,
    AUD_NVM_D_CH1_BQ2_4,
    AUD_NVM_D_CH1_BQ2_5,
    AUD_NVM_D_CH1_BQ3_1,
    AUD_NVM_D_CH1_BQ3_2,
    AUD_NVM_D_CH1_BQ3_3,
    AUD_NVM_D_CH1_BQ3_4,
    AUD_NVM_D_CH1_BQ3_5,
    AUD_NVM_D_CH1_BQ4_1,
    AUD_NVM_D_CH1_BQ4_2,
    AUD_NVM_D_CH1_BQ4_3,
    AUD_NVM_D_CH1_BQ4_4,
    AUD_NVM_D_CH1_BQ4_5,
    AUD_NVM_D_CH1_BQ5_1,
    AUD_NVM_D_CH1_BQ5_2,
    AUD_NVM_D_CH1_BQ5_3,
    AUD_NVM_D_CH1_BQ5_4,
    AUD_NVM_D_CH1_BQ5_5,
    AUD_NVM_D_CH1_BQ6_1,
    AUD_NVM_D_CH1_BQ6_2,
    AUD_NVM_D_CH1_BQ6_3,
    AUD_NVM_D_CH1_BQ6_4,
    AUD_NVM_D_CH1_BQ6_5,
    AUD_NVM_D_CH2_BQ0_1,
    AUD_NVM_D_CH2_BQ0_2,
    AUD_NVM_D_CH2_BQ0_3,
    AUD_NVM_D_CH2_BQ0_4,
    AUD_NVM_D_CH2_BQ0_5,
    AUD_NVM_D_CH2_BQ1_1,
    AUD_NVM_D_CH2_BQ1_2,
    AUD_NVM_D_CH2_BQ1_3,
    AUD_NVM_D_CH2_BQ1_4,
    AUD_NVM_D_CH2_BQ1_5,
    AUD_NVM_D_CH2_BQ2_1,
    AUD_NVM_D_CH2_BQ2_2,
    AUD_NVM_D_CH2_BQ2_3,
    AUD_NVM_D_CH2_BQ2_4,
    AUD_NVM_D_CH2_BQ2_5,
    AUD_NVM_D_CH2_BQ3_1,
    AUD_NVM_D_CH2_BQ3_2,
    AUD_NVM_D_CH2_BQ3_3,
    AUD_NVM_D_CH2_BQ3_4,
    AUD_NVM_D_CH2_BQ3_5,
    AUD_NVM_D_CH2_BQ4_1,
    AUD_NVM_D_CH2_BQ4_2,
    AUD_NVM_D_CH2_BQ4_3,
    AUD_NVM_D_CH2_BQ4_4,
    AUD_NVM_D_CH2_BQ4_5,
    AUD_NVM_D_CH2_BQ5_1,
    AUD_NVM_D_CH2_BQ5_2,
    AUD_NVM_D_CH2_BQ5_3,
    AUD_NVM_D_CH2_BQ5_4,
    AUD_NVM_D_CH2_BQ5_5,
    AUD_NVM_D_CH2_BQ6_1,
    AUD_NVM_D_CH2_BQ6_2,
    AUD_NVM_D_CH2_BQ6_3,
    AUD_NVM_D_CH2_BQ6_4,
    AUD_NVM_D_CH2_BQ6_5,

    AUD_NVM_D_DRC1_AE_1,
    AUD_NVM_D_DRC1_AE_2,
    AUD_NVM_D_DRC1_AA_1,
    AUD_NVM_D_DRC1_AA_2,
    AUD_NVM_D_DRC1_AD_1,
    AUD_NVM_D_DRC1_AD_2,
    AUD_NVM_D_DRC2_AE_1,
    AUD_NVM_D_DRC2_AE_2,
    AUD_NVM_D_DRC2_AA_1,
    AUD_NVM_D_DRC2_AA_2,
    AUD_NVM_D_DRC2_AD_1,
    AUD_NVM_D_DRC2_AD_2,

    AUD_NVM_D_DRC1_T,
    AUD_NVM_D_DRC1_K,
    AUD_NVM_D_DRC1_O,
    AUD_NVM_D_DRC2_T,
    AUD_NVM_D_DRC2_K,
    AUD_NVM_D_DRC2_O,

    AUD_NVM_D_DRC_CTRL,
    AUD_NVM_D_BANK_SWITCH,

    AUD_NVM_D_CH1_OUT_MIX_1,
    AUD_NVM_D_CH1_OUT_MIX_2,
    AUD_NVM_D_CH1_OUT_MIX_3,
    AUD_NVM_D_CH2_OUT_MIX_1,
    AUD_NVM_D_CH2_OUT_MIX_2,
    AUD_NVM_D_CH2_OUT_MIX_3,

    AUD_NVM_D_CH1_IN_MIX_1,
    AUD_NVM_D_CH1_IN_MIX_2,
    AUD_NVM_D_CH1_IN_MIX_3,
    AUD_NVM_D_CH1_IN_MIX_4,
    AUD_NVM_D_CH2_IN_MIX_1,
    AUD_NVM_D_CH2_IN_MIX_2,
    AUD_NVM_D_CH2_IN_MIX_3,
    AUD_NVM_D_CH2_IN_MIX_4,
    AUD_NVM_D_CH3_IN_MIX_1,
    AUD_NVM_D_CH3_IN_MIX_2,
    AUD_NVM_D_CH3_IN_MIX_3,

    AUD_NVM_D_OUT_POST_SCALE_1,
    AUD_NVM_D_OUT_POST_SCALE_2,

    AUD_NVM_D_CH1_BQ7_1,
    AUD_NVM_D_CH1_BQ7_2,
    AUD_NVM_D_CH1_BQ7_3,
    AUD_NVM_D_CH1_BQ7_4,
    AUD_NVM_D_CH1_BQ7_5,
    AUD_NVM_D_CH1_BQ8_1,
    AUD_NVM_D_CH1_BQ8_2,
    AUD_NVM_D_CH1_BQ8_3,
    AUD_NVM_D_CH1_BQ8_4,
    AUD_NVM_D_CH1_BQ8_5,
    AUD_NVM_D_SUB_BQ0_1,
    AUD_NVM_D_SUB_BQ0_2,
    AUD_NVM_D_SUB_BQ0_3,
    AUD_NVM_D_SUB_BQ0_4,
    AUD_NVM_D_SUB_BQ0_5,
    AUD_NVM_D_SUB_BQ1_1,
    AUD_NVM_D_SUB_BQ1_2,
    AUD_NVM_D_SUB_BQ1_3,
    AUD_NVM_D_SUB_BQ1_4,
    AUD_NVM_D_SUB_BQ1_5,
    AUD_NVM_D_CH2_BQ7_1,
    AUD_NVM_D_CH2_BQ7_2,
    AUD_NVM_D_CH2_BQ7_3,
    AUD_NVM_D_CH2_BQ7_4,
    AUD_NVM_D_CH2_BQ7_5,
    AUD_NVM_D_CH2_BQ8_1,
    AUD_NVM_D_CH2_BQ8_2,
    AUD_NVM_D_CH2_BQ8_3,
    AUD_NVM_D_CH2_BQ8_4,
    AUD_NVM_D_CH2_BQ8_5,

    AUD_NVM_D_PSEUDO_CH2_BQ_1,
    AUD_NVM_D_PSEUDO_CH2_BQ_2,
    AUD_NVM_D_PSEUDO_CH2_BQ_3,
    AUD_NVM_D_PSEUDO_CH2_BQ_4,
    AUD_NVM_D_PSEUDO_CH2_BQ_5,

    AUD_NVM_D_CH4_OUT_MIX_1,
    AUD_NVM_D_CH4_OUT_MIX_2,
    AUD_NVM_D_CH4_IN_MIX_1,
    AUD_NVM_D_CH4_IN_MIX_2,

    AUD_NVM_D_IDF_POST_SCALE,
    AUD_NVM_D_2ND_LEFT_CH_MIX,
    AUD_NVM_D_2ND_RIGHT_CH_MIX,

    AUD_NVM_D_MOD_LIMIT_REG,
    #endif
    //Class D (STA381BW)
    AUD_NVM_AMP_START,
    AUD_NVM_AMP_END,
    AUD_NVM_D_CONF_REG_A,
    AUD_NVM_D_CONF_REG_B,
    AUD_NVM_D_CONF_REG_C,
    AUD_NVM_D_CONF_REG_D,
    AUD_NVM_D_CONF_REG_E,
    AUD_NVM_D_CONF_REG_F,

    AUD_NVM_D_MUTE_LINE_OUT_CONF_REG,
    AUD_NVM_D_MASTER_VOL,
    AUD_NVM_D_CH1_VOL,
    AUD_NVM_D_CH2_VOL,
    AUD_NVM_D_CH3_VOL,
    AUD_NVM_D_CH1_CONF_REG,
    AUD_NVM_D_CH2_CONF_REG,
    AUD_NVM_D_CH3_CONF_REG,

    AUD_NVM_D_TONE_CONL_REG,

    AUD_NVM_D_DRC_LIMITER_1_RATE,
    AUD_NVM_D_DRC_LIMITER_1_THRES,
    AUD_NVM_D_DRC_LIMITER_2_RATE,
    AUD_NVM_D_DRC_LIMITER_2_THRES,

    AUD_NVM_D_DRC_LIMITER_1_EXT_ATTACK_THRES,
    AUD_NVM_D_DRC_LIMITER_1_EXT_RELEASE_THRES,
    AUD_NVM_D_DRC_LIMITER_2_EXT_ATTACK_THRES,
    AUD_NVM_D_DRC_LIMITER_2_EXT_RELEASE_THRES,

    AUD_NVM_D_EXT_CONF_REG,
    AUD_NVM_D_EXT_VOL_CONF_REG_A,
    AUD_NVM_D_EXT_VOL_CONF_REG_B,

    AUD_NVM_D_ZERO_DETECT_CONF_REG,
    AUD_NVM_D_HP_LINE_OUT_CONF_REG,

    AUD_NVM_D_EXT_COEFF_RANGE_UP_A, /*Rico 20120907*/
    AUD_NVM_D_EXT_COEFF_RANGE_UP_B,

    AUD_NVM_D_CH1_BQ1_H10,
    AUD_NVM_D_CH1_BQ1_H11,
    AUD_NVM_D_CH1_BQ1_H12,
    AUD_NVM_D_CH1_BQ1_H13,
    AUD_NVM_D_CH1_BQ1_H14,

    AUD_NVM_D_CH1_BQ2_H20,
    AUD_NVM_D_CH1_BQ2_H21,
    AUD_NVM_D_CH1_BQ2_H22,
    AUD_NVM_D_CH1_BQ2_H23,
    AUD_NVM_D_CH1_BQ2_H24,

    AUD_NVM_D_CH1_BQ3_H30,
    AUD_NVM_D_CH1_BQ3_H31,
    AUD_NVM_D_CH1_BQ3_H32,
    AUD_NVM_D_CH1_BQ3_H33,
    AUD_NVM_D_CH1_BQ3_H34,

    AUD_NVM_D_CH1_BQ4_H40,
    AUD_NVM_D_CH1_BQ4_H41,
    AUD_NVM_D_CH1_BQ4_H42,
    AUD_NVM_D_CH1_BQ4_H43,
    AUD_NVM_D_CH1_BQ4_H44,

    AUD_NVM_D_CH2_BQ1_H10,
    AUD_NVM_D_CH2_BQ1_H11,
    AUD_NVM_D_CH2_BQ1_H12,
    AUD_NVM_D_CH2_BQ1_H13,
    AUD_NVM_D_CH2_BQ1_H14,

    AUD_NVM_D_CH2_BQ2_H20,
    AUD_NVM_D_CH2_BQ2_H21,
    AUD_NVM_D_CH2_BQ2_H22,
    AUD_NVM_D_CH2_BQ2_H23,
    AUD_NVM_D_CH2_BQ2_H24,

    AUD_NVM_D_CH2_BQ3_H30,
    AUD_NVM_D_CH2_BQ3_H31,
    AUD_NVM_D_CH2_BQ3_H32,
    AUD_NVM_D_CH2_BQ3_H33,
    AUD_NVM_D_CH2_BQ3_H34,

    AUD_NVM_D_CH2_BQ4_H40,
    AUD_NVM_D_CH2_BQ4_H41,
    AUD_NVM_D_CH2_BQ4_H42,
    AUD_NVM_D_CH2_BQ4_H43,
    AUD_NVM_D_CH2_BQ4_H44,

    AUD_NVM_D_CH12_BQ5_H0,
    AUD_NVM_D_CH12_BQ5_H1,
    AUD_NVM_D_CH12_BQ5_H2,
    AUD_NVM_D_CH12_BQ5_H3,
    AUD_NVM_D_CH12_BQ5_H4,

    AUD_NVM_D_CH3_BQ1_H0,
    AUD_NVM_D_CH3_BQ1_H1,
    AUD_NVM_D_CH3_BQ1_H2,
    AUD_NVM_D_CH3_BQ1_H3,
    AUD_NVM_D_CH3_BQ1_H4,

    AUD_NVM_D_CH1_PRE_SCALE,
    AUD_NVM_D_CH2_PRE_SCALE,
    AUD_NVM_D_CH1_POST_SCALE,
    AUD_NVM_D_CH2_POST_SCALE,
    AUD_NVM_D_CH3_POST_SCALE,

    AUD_NVM_D_RAM_RESERVED,

    AUD_NVM_D_CH1_MIX_1,
    AUD_NVM_D_CH1_MIX_2,
    AUD_NVM_D_CH2_MIX_1,
    AUD_NVM_D_CH2_MIX_2,
    AUD_NVM_D_CH3_MIX_1,
    AUD_NVM_D_CH3_MIX_2,

} AUD_NVM_TYPE_T;

typedef struct
{
    UINT16              u2Offset;
    UINT8               u1Length;
    UINT32              u4DefaultValue;
    AUD_NVM_TYPE_T      eDataType;
} AUD_NVM_DATA_T;

typedef struct
{
    INT16 i2NvmInputSrcHDMI;
    INT16 i2NvmInputSrcAV;
    INT16 i2NvmInputSrcUSB;
    INT16 i2NvmInputSrcATV;
    INT16 i2NvmInputSrcDTV;
} AUD_IPT_SRC_GAIN_T;

typedef enum
{
    AUD_NVM_DK = 0,
    AUD_NVM_NON_DK,
    AUD_NVM_TOTAL
} AUD_NVM_ATV_FMT_T;

typedef enum
{
    AUD_NVM_DEFAULT_M = 0,
    AUD_NVM_DEFAULT_BG = 1,
    AUD_NVM_DEFAULT_DK = 2,
    AUD_NVM_DEFAULT_I = 3,
    AUD_NVM_DEFAULT_TOTAL
} AUD_NVM_DEFAULT_ATV_SYS_T;

typedef enum
{
    AUD_NVM_QRY_A2_GAIN = 0,
    AUD_NVM_QRY_PAL_GAIN,
    AUD_NVM_QRY_AM_GAIN,
    AUD_NVM_QRY_NICAM_GAIN
} AUD_NVM_ATV_QRY_T;

typedef enum
{
    AUD_NVM_L_VOL_TBL = 0,
    AUD_NVM_R_VOL_TBL,
    AUD_NVM_L_VOL_TBL_SKYPE,
    AUD_NVM_R_VOL_TBL_SKYPE,
    AUD_NVM_COMMON_VOL_TBL,
    AUD_NVM_VOL_TBL_NUM
} AUD_NVM_VOL_TBL_T;

typedef struct _AUD_NVM_ATV_PREGAIN
{
    UINT32 u4A2_FineGain[AUD_NVM_TOTAL];
    UINT8  u1A2_FineGain[AUD_NVM_TOTAL];
    UINT32 u4PAL_FineGain[AUD_NVM_TOTAL];
    UINT8  u1PAL_FineGain[AUD_NVM_TOTAL];
    UINT32 u4AM_FineGain[AUD_NVM_TOTAL];
    UINT8  u1AM_FineGain[AUD_NVM_TOTAL];
    UINT32 u4NICAM_FineGain[AUD_NVM_TOTAL];
    UINT8  u1NICAM_FineGain[AUD_NVM_TOTAL];
}AUD_NVM_ATV_PREGAIN;
#endif

typedef enum
{
    AUD_DEC_OVM_OFF = 0,
    AUD_DEC_OVM_LOW,
    AUD_DEC_OVM_HIGH
} AUD_DEC_OVM_MODE_T;

typedef enum
{
    AUD_DEC_OUTPORT_MODE_UNKNOW = 0,
    AUD_DEC_OUTPORT_MODE_HEADPHONE,
    AUD_DEC_OUTPORT_MODE_LINEOUT,
    AUD_DEC_OUTPORT_MODE_SUB_WOOFER
}   AUD_DEC_OUTPORT_MODE_T;

/* new OVM mode */
#define NEW_PAL_SHIFT (0)
#define NEW_A2_SHIFT (2)
#define NEW_MTS_SHIFT (4)
#define NEW_FMFM_SHIFT (6)
#define NEW_PAL_MASK (0x3 << NEW_PAL_SHIFT)
#define NEW_A2_MASK (0x3<< NEW_A2_SHIFT)
#define NEW_MTS_MASK (0x3 << NEW_MTS_SHIFT)
#define NEW_FMFM_MASK (0x3 << NEW_FMFM_SHIFT)

//==========  Flash AQ Related Definitions ===================================
#ifndef CC_AUD_UNICORN_CUST
#define AUD_FLASH_AQ_SIZE (512*1024) //Total Flash AQ size on NAND Flash partition.
#else
#define AUD_FLASH_AQ_SIZE (5*1024*1024) //Total Flash AQ size on NAND Flash partition.
#endif
#define AUD_FLASH_AQ_BUF_SIZE (10*1024)   // Flash AQ buffer size for temp usage.
#define FLASH_AQ_VER_LENGTH 8
#define FLASH_AQ_VER_OFFSET 0
#define FLASH_AQ_SCENE_NUM   5
#define FLASH_AQ_COMM_TABLE_ITEMS 4   // The number of tables in Common Sheet.(including version)

#define FLASH_AQ_ID_NUM 28            // The number of AQ IDs. e.x. AQ1~11 mean 11 AQ IDs.
#define FLASH_AQ_TYPE3_TABLE_SIZE 186 // The size in bytes of TYPE3 only tables.

#define AQ_LEVEL_ITEMS 3
#define AQ_BASS_ITEMS 2
#define AQ_BASS_MANAGE_ITEMS 8
#define AQ_EQ_ITEMS 7
#define AQ_DRC_ITEMS 8
#define AQ_AVC_ITEMS 5
#define AQ_SPH_ITEMS 1
#define AQ_MVS_ITEMS 7
#define AQ_POSTSCL_ITEMS 2
#define AQ_PEQ_ITEMS 45
#define AQ_EXT_PEQ_ITEMS 35
#define AQ_STC_FILTER_ITEMS 20
#define AQ_EXT_PEQ_CSW_ITEMS 40
#define AQ_BACKLIGHT_ITEMS 12
#define AQ_VBASS_ITEMS 7
#define AQ_FIR_ITEMS 256
#define AQ_LOUDNESS_ITEMS 25    ///Change
#define AQ_MTK_DRC_ITEMS 33 ///Change
#define AQ_STC381Coeff_ITEMS 75


#define FLASH_AQ_COMM_SRC_NUM 5
#define FLASH_AQ_COMM_CH_NUM  12
#define FLASH_AQ_SRC_NUM  4
#define FLASH_AQ_SM_NUM   4
#define AQ_LIMITER_ITEMS 7
#define AQ_BM_CUTOFF_NUM 17
#define AQ_SW_CUTOFF_NUM 7

#define AQ_MODEL_FUNC_WOOFER   (1<<0)
#define AQ_MODEL_FUNC_DVD      (1<<1)

typedef enum
{
    AQ_TYPE_VERSION =0,
    AQ_TYPE_CH_VOLUME,
    AQ_TYPE_SRC_VOLUME,
    AQ_TYPE_VOL_TABLE,
    AQ_TYPE_LO_VOL_TABLE,
    AQ_TYPE_PEQ,
    AQ_TYPE_LEVEL,
    AQ_TYPE_BASS_TREBLE,
    AQ_TYPE_BASS_MANAGEMENT,
    AQ_TYPE_EQ,
    AQ_TYPE_DRC,
    AQ_TYPE_AVC,
    AQ_TYPE_LIMITER,
    AQ_TYPE_SPKHEIGHT,
    AQ_TYPE_MVS,
    AQ_TYPE_POST,
    AQ_TYPE_SP_VOL,
    AQ_TYPE_LO_VOL,
    AQ_TYPE_EXT_PEQ,
    AQ_TYPE_STC381FILTER,
    AQ_TYPE_EXT_PEQ_LSRS,
    AQ_TYPE_EXT_PEQ_CSW,
    AQ_TYPE_TYPE3_TBL,
    AQ_TYPE_BACKLIGHT,
    AQ_TYPE_VBASS,
    AQ_TYPE_FIR,
    AQ_TYPE_HP_VOL_TABLE,
    AQ_TYPE_LOUDNESS,
    AQ_TYPE_MTKDRC,
    AQ_TYPE_STC381COEFF,
    AQ_TYPE_MIC_GAIN,
    AQ_TYPE_RESERVED,
    AQ_TYPE_NUM
}AUD_AQ_TYPE_T;


typedef enum
{
    AUD_OPL_BBE = 0,
    AUD_OPL_VIRTUAL_BASS,
    AUD_OPL_PEQ,
    AUD_OPL_M2S,
    AUD_OPL_CLEAR_VOICE,
    AUD_OPL_SPK_HEIGHT,
    AUD_OPL_CLIPPER,
    AUD_OPL_NOISE_REDUCTION,
    AUD_OPTL_KEY_SHIFT,
    AUD_OPL_VIQ,
    AUD_OPL_LXP,
    AUD_OPL_AUDYSSEY_ADV,
    AUD_OPL_AUDYSSEY_ABX,
    AUD_OPL_AUDYSSEY_AEQ,
    AUD_OPL_CDNOTCH,
    AUD_OPL_SRS_PEQ,
    AUD_OPL_SRS_LMT,
    AUD_OPL_SRS_VIQ,
    AUD_OPL_FIR,                //CC_AUD_FIR_SUPPORT
    AUD_OPL_SRS_AEQ,//19
    AUD_OPL_SPK_HEIGHT_V2,
    AUD_OPL_END,                // the end of all elements
    AUD_OPL_NUM
} AUD_OPL_FUNC_TYPE_T;

typedef struct
{
    AUD_AQ_TYPE_T       eDataType;
    CHAR                pChar[8];
    UINT8               u1X;
    UINT8               u1Y;
    UINT8               u1Bytes;
} AUD_AQ_DSCRPT_T;

typedef struct
{
    UINT8               u1Cutoff;
    UINT8               u1LfeGain;
    UINT8               u1AttackHi;
    UINT8               u1ReleaseHi;
    UINT8               u1RateHi;
    UINT8               u1AttackLo;
    UINT8               u1ReleaseLo;
    UINT8               u1RateLo;
} AUD_AQ_AMP_DRC_T;

typedef struct
{
    UINT32               u4DRC1FiltAlpha;
    UINT32               u4DRC1FiltOmega;
    UINT32               u4DRC1AttackRate;
    UINT32               u4DRC1ReleaseRate;
    UINT32               u4DRC1AttackTh;
    UINT32               u4DRC2FiltAlpha;
    UINT32               u4DRC2FiltOmega;
    UINT32               u4DRC2AttackRate;
    UINT32               u4DRC2ReleaseRate;
    UINT32               u4DRC2AttackTh;
} AUD_AQ_AMP_DRC2_T;     //for TAS5171

typedef struct
{
    UINT32 u4Data[130];
} AUD_AQ_EXTPEQ_DATA2_T; // for TAS5171

typedef struct
{
    UINT8               u1ReleaseRate0Hi;        //band 0, DRC0
    UINT8               u1ReleaseRate0Mid;
    UINT8               u1ReleaseRate0Lo;
    UINT8               u1AttackRate0Hi;
    UINT8               u1AttackRate0Mid;
    UINT8               u1AttackRate0Lo;
    UINT8               u1LimiterTH0Hi;
    UINT8               u1LimiterTH0Mid;
    UINT8               u1LimiterTH0Lo;
    UINT8               u1CompressorRate0Hi;
    UINT8               u1CompressorRate0Mid;
    UINT8               u1CompressorRate0Lo;
    UINT8               u1CompressorTH0Hi;
    UINT8               u1CompressorTH0Mid;
    UINT8               u1CompressorTH0Lo;

    UINT8               u1ReleaseRate1Hi;       //band 0, DRC2
    UINT8               u1ReleaseRate1Mid;
    UINT8               u1ReleaseRate1Lo;
    UINT8               u1AttackRate1Hi;
    UINT8               u1AttackRate1Mid;
    UINT8               u1AttackRate1Lo;
    UINT8               u1LimiterTH1Hi;
    UINT8               u1LimiterTH1Mid;
    UINT8               u1LimiterTH1Lo;
    UINT8               u1CompressorRate1Hi;
    UINT8               u1CompressorRate1Mid;
    UINT8               u1CompressorRate1Lo;
    UINT8               u1CompressorTH1Hi;
    UINT8               u1CompressorTH1Mid;
    UINT8               u1CompressorTH1Lo;

    UINT8               u1ReleaseRate2Hi;      //band 1, DRC1
    UINT8               u1ReleaseRate2Mid;
    UINT8               u1ReleaseRate2Lo;
    UINT8               u1AttackRate2Hi;
    UINT8               u1AttackRate2Mid;
    UINT8               u1AttackRate2Lo;
    UINT8               u1LimiterTH2Hi;
    UINT8               u1LimiterTH2Mid;
    UINT8               u1LimiterTH2Lo;
    UINT8               u1CompressorRate2Hi;
    UINT8               u1CompressorRate2Mid;
    UINT8               u1CompressorRate2Lo;
    UINT8               u1CompressorTH2Hi;
    UINT8               u1CompressorTH2Mid;
    UINT8               u1CompressorTH2Lo;

    UINT8               u1ReleaseRate3Hi;      //band 1, DRC3
    UINT8               u1ReleaseRate3Mid;
    UINT8               u1ReleaseRate3Lo;
    UINT8               u1AttackRate3Hi;
    UINT8               u1AttackRate3Mid;
    UINT8               u1AttackRate3Lo;
    UINT8               u1LimiterTH3Hi;
    UINT8               u1LimiterTH3Mid;
    UINT8               u1LimiterTH3Lo;
    UINT8               u1CompressorRate3Hi;
    UINT8               u1CompressorRate3Mid;
    UINT8               u1CompressorRate3Lo;
    UINT8               u1CompressorTH3Hi;
    UINT8               u1CompressorTH3Mid;
    UINT8               u1CompressorTH3Lo;

    UINT8               u1OffsetDRC0Hi;
    UINT8               u1OffsetDRC0Mid;
    //    UINT8               u1OffsetDRC0Lo;
    UINT8               u1OffsetDRC1Hi;
    UINT8               u1OffsetDRC1Mid;
    //    UINT8               u1OffsetDRC1Lo;
    UINT8               u1OffsetDRC2Hi;
    UINT8               u1OffsetDRC2Mid;
    //    UINT8               u1OffsetDRC2Lo;
    UINT8               u1OffsetDRC3Hi;
    UINT8               u1OffsetDRC3Mid;
    //    UINT8               u1OffsetDRC3Lo;
    UINT8               u1STCCFG0;
    UINT8               u1STCCFG1;
    UINT8               u1Reserved0;
    UINT8               u1Reserved1;

    UINT8               u1DynamicAttackRateHi;
    UINT8               u1DynamicAttackRateMid;
    UINT8               u1DynamicAttackRateLo;

} AUD_AQ_AMP_STC_COEFF_T;

typedef struct
{
    UINT8 u1Data[15];
}
AUD_AQ_AMP_RAM_DATA_T;

typedef enum
{
    AQ_OP_EXE = 0,
    AQ_OP_INIT
} AUD_AQ_OP_TYPE_T;

typedef enum
{
    AQ_EXT_PEQ_HP = 0,
    AQ_EXT_PEQ_LP,
    AQ_EXT_PEQ_BP,
    AQ_EXT_PEQ_FLAT
} AUD_AQ_EXTPEQ_TYPE_T;

typedef struct
{
    UINT32 u4Data[AQ_EXT_PEQ_ITEMS];
} AUD_AQ_EXTPEQ_DATA_T;

typedef enum
{
    AQ_AMP_FS_32K = 0,
    AQ_AMP_FS_44K,
    AQ_AMP_FS_48K,
    AQ_AMP_FS_NUM
} AUD_AQ_AMP_FS_T;

typedef struct
{
    UINT32 u4SpkCfg;
    UINT32 u4Cutoff;
    UINT8 u1Panorama;
    UINT8 u1Dimesion;
    UINT8 u1CWidth;
} AUD_AQ_TYPE3_DATA_T;

typedef enum
{
    AQ_BL_ITEM_START,
    TEMP_TH_LOW = 0,
    TEMP_TH_HIGH,
    TEMP_TH_LIMIT,
    BL_TEMP_CNTTH,
    BL_TEMP_INC,
    TEMP_BL_BOTTOM,
    OPT_BL_DEADB,
    OPT_BL_DEADB_CIN,
    BL_CRI_TEMP_L,
    BL_CRI_TEMP_H,
    TH_SHTDNLIMIT,
    AQ_BL_ITEM_END
} AUD_AQ_BL_REG;
//===================================================

//----------- End of NVM and Profile Data Types -------------------------------

typedef enum
{
    AUD_SUR_CLARITY = 0,
    AUD_SUR_WIDTH,
    AUD_SUR_LRGAIN,
    AUD_SUR_XTALK,
    AUD_SUR_OUTPUT_GAIN,
    AUD_SUR_BASS_GAIN,
    AUD_SUR_FO,
    AUD_SUR_PRAR_NUM
}   AUD_VSURR_CFG_TYPE_T;

typedef struct
{
    UINT32 u4Clarity;
    UINT32 u4Width;
    UINT32 u4LRGain;
    UINT32 u4Xtalk;
    UINT32 u4OutputGain;
    UINT32 u4BassGain;
    UINT32 u4Fo;
} AUD_VSURR_CFG_T;

typedef struct
{
    UINT16 u2Pktno;        //W_WMA_PACKET_NO
    UINT16 u2HdrSrc;       //W_WMA_HEADER_SOURCE
    UINT32 u4Pktcnt;       //D_WMA_PACKET_COUNT
    UINT32 u4Pktsz;        //D_WMA_PACKET_SIZE
    UINT16 u2Wmaver;       //W_WMA_VERSION
    UINT16 u2Numch;        //W_WMA_NUM_CH
    UINT32 u4Sampersec;    //D_WMA_SAMPLE_PER_SEC
    UINT32 u4Bpersec;      //D_WMA_BYTE_PER_SEC
    UINT32 u4Blocksz;      //D_WMA_BLOCK_SIZE
    UINT16 u2Encoderopt;   //W_WMA_ENCODER_OPTION
    UINT32 u4ChannelMask;   //W_WMA_CHANNEL_MASK
    UINT16 u2BitsPerSample;   //W_WMA_BITS_SAMPLE
    UINT16 u2AdvEncoderopt;   //W_WMA_ADV_OPTION
    UINT32 u4AdvEncoderopt2;   //W_WMA_ADV_OPTION2
    UINT32 u4VoiceEncodeOptions1;
    UINT32 u4VoiceEncodeOptions2[2];
} AUD_WMA_CFG_T;
#if 1 //CC_APE_SUPPORT
//ian APE decoder
typedef struct
{
    UINT16 u2ParseAudOnly;          //W_APE_PARSE_AUDIO_ONLY
    UINT16 u2FileVersion;              //W_APE_FILE_VESION
    UINT32 u4CompressLevel;        //D_APE_COMPRESS_LEVEL
    UINT32 u4BlockPerFrm;           //D_APE_BLOCK_PER_FRAME
    UINT32 u4FinalFrmBlock;        //D_APE_FINAL_FRAME_BLOCK
    UINT16 u2TotalFrmNumLow;    //W_APE_TOTAL_FRAME_NUM_LOW
    UINT16 u2TotalFrmNumHigh;   //W_APE_TOTAL_FRAME_NUM_HIGH
    UINT16 u2Bpersample;            //W_APE_BITS_PER_SAMPLE
    UINT16 u2CHNum;                   //W_APE_CHANNEL_NUM
    UINT32 u4SamplingRate;        //D_APE_INPUT_SAMPLING_RATE
    UINT16 u2MuteBankNum;        //W_APE_MUTE_BANK_NUMBERS
    UINT16 u2InvalidBytes;           //W_APE_INVALID_BYTES
} AUD_APE_CFG_T;
#endif
typedef struct
{
    UINT16 u2Numch;
    UINT32 u4Sampersec;
    UINT16 u2BitsPerSample;
} AUD_AMR_CFG_T;

typedef struct
{
    UINT16 u2SampPerFrame;        //W_COOK_SAMP_PER_FRAME
    UINT16 u2ChanNum;       //W_COOK_CHANNEL_NUM
    UINT16 u2SampRate;       //W_COOK_INPUT_SAMPLING_RATE
    UINT16 u2FrameSzInBytes;        //W_COOK_FRM_SZ_IN_BYTES
    UINT16 u2RegnNum;       //W_COOK_REGION_NUM
    UINT16 u2StartRegn;        //W_COOK_COUPLING_START_REGN
    UINT16 u2CplQBits;    //W_COOK_COUPLING_QBITS

} AUD_COOK_CFG_T;

typedef struct//paul_flac
{
    UINT16 u2MinBlkSz;//Minimum block size
    UINT16 u2MaxBlkSz;//Maximum block size
    UINT16 u2ChannelNum;//channel num
    UINT16 u2BPS;//bits per sample
    UINT32 u4MinFrmSz;//Minimum frame size
    UINT32 u4MaxFrmSz;//Maximum frame size
    UINT32 u4SmpRate;//Sample rate
} AUD_FLAC_INFO_T;

typedef struct//alac
{
    UINT32 u4FrameLength;//
    UINT8 u1ComoVer;//
    UINT8 u1BitDepth;//
    UINT8 u1KukiPb;//
    UINT8 u1KukiMb;//
    UINT8 u1KukiKb;//
    UINT8 u1NumChan;//
    UINT16 u2MaxRun;//
    UINT32 u4MaxFraByte;//
    UINT32 u4AvgBitRate;//
    UINT32 u4SmpRate;//
} AUD_ALAC_CFG_T;

typedef struct
{
    UINT8   u1SmartRev[16];
} AUD_SMART_AUD_INFO_T;

typedef enum
{
    PCM_24_BITS = 0,
    PCM_IMA_ADPCM_BIG_ENDIAN = 1,
    PCM_16_BITS_BIG_ENDIAN  = 2,
    PCM_MS_ADPCM_BIG_ENDIAN  = 3,
    PCM_IMA_ADPCM_LITTLE_ENDIAN = 1 | (1 << 15),
    PCM_16_BITS_LITTLE_ENDIAN = 2 | (1 << 15),
    PCM_MS_ADPCM_LITTLE_ENDIAN = 3 | (1 << 15),
    PCM_A_LAW_LITTLE_ENDIAN = (1 << 15) | (1 << 14),
    PCM_U_LAW_LITTLE_ENDIAN = (1 << 15) | (1 << 13)
} PCM_INPUT_TYPE_T;

/// Audio PCM type
typedef struct
{
    SAMPLE_FREQ_T      eSampleFreq;  ///< PCM Sampling Rate.
    PCM_INPUT_TYPE_T   ePcmDataInfo; ///< PCM Sampling Rate.
    UINT8  ePCMChNum;   /// PCM Input Channel Number
} AUD_PCM_SETTING_T;

typedef enum
{
    DEC_LITTLE_ENDIAN = 0,
    DEC_BIG_ENDIAN
} DATA_ENDIAN_T;

/// Audio AC3 type
typedef struct
{
    DATA_ENDIAN_T      eEndian;  /// input data endian
} AUD_AC3_SETTING_T;

//Dynamic volume curve setting  //Andrew Wen 07/8/20
#define VOL_TABLE_CHK_PNT 7

/// Callbalck function if clip playback is done
typedef INT32 (*AUD_CLIP_NOTIFY)(UINT8 u1DecId, UINT32 u1Datat, UINT32 u4Datat2);

#ifdef CC_53XX_SWDMX_V2
typedef void (*AUD_WMAErr_NOTIFY)(VOID* prInst, UINT32 u4WMAErrCode);  // megaa 20090828
#else
typedef void (*AUD_WMAErr_NOTIFY)(UINT32 u4WMAErrCode);
#endif
//static AUD_WMAErr_NOTIFY _hWMAErrNotifyFunc = NULL;

//----- LPCM decoder -----
typedef enum
{
    LPCM_TYPE_NORAML = 0,
    LPCM_TYPE_MS_ADPCM = 1,
    LPCM_TYPE_IMA_ADPCM = 2,
    LPCM_TYPE_A_LAW = 3,
    LPCM_TYPE_U_LAW = 4
} LPCM_INPUT_TYPE_T;
typedef struct
{
    UINT8 isUpdate;
    VOID* pParams;
    UINT16 noParam;
    UINT8 dataOption;
    UINT8 varOption;
} LGSE_FN_PARAM_T;

typedef struct
{
    SAMPLE_FREQ_T      eSampleFreq;
    LPCM_INPUT_TYPE_T  eLPcmType;
    UINT16             ui2_num_ch;
    UINT16             ui2_block_align;
    UINT16             ui2_bits_per_sample;
    BOOL               b_big_endian;
    BOOL               b_info_not_specific; /*pcm info from dmx*/
    BOOL               b_signed_data;
} AUD_LPCM_INFO_T;

typedef struct
{
    UINT8   u1LpcmChAssignment;
    UINT8   u1LpcmFreqGrp1;
    UINT8   u1LpcmFreqGrp2;
    UINT8   u1LpcmBitShift;
    UINT8   u1LpcmDrcValue;
    UINT8   u1LpcmBitResGrp1;
    UINT8   u1LpcmBitResGrp2;
    UINT8   u1LpcmDrcFlag;
    UINT16  u2LpcmDecodingType;
    UINT16  u2ADPCMDecodingType;
    UINT16  u2ADPCMBlockAlign;
} AUD_LPCM_SETTING_T;
//----- end of LPCM -----


#if 1 //CC_AUD_QUERY_VDO_INFO
typedef enum
{
    AUD_SRM_VDO_480 = 0,
    AUD_SRM_VDO_576,
    AUD_SRM_VDO_720,
    AUD_SRM_VDO_1080,
    AUD_SRM_VDO_HEIGHT_NUM
} AUD_SRM_VDO_HEIGHT_T;

typedef enum
{
    AUD_SRM_VDO_P = 0,
    AUD_SRM_VDO_I,
    AUD_SRM_VDO_IP_NUM
} AUD_SRM_VDO_IP_T;

typedef enum
{
    AUD_SRM_VDO_50HZ = 0,
    AUD_SRM_VDO_60HZ,
    AUD_SRM_VDO_RATE_NUM
} AUD_SRM_VDO_RATE_T;

typedef enum
{
    AUD_SRM_VDO_NO_CUST = 0,
    AUD_SRM_VDO_MIXED_CUST,
    AUD_SRM_VDO_ALL_CUST
} AUD_SRM_VDO_DELAY_CUST_TYPE_T;

typedef struct
{
    INT16 i2Delay[AUD_SRM_VDO_HEIGHT_NUM][AUD_SRM_VDO_IP_NUM][AUD_SRM_VDO_RATE_NUM];
} AUD_SRM_VDO_DELAY_TBL;
#endif

//#define VM_FILTER1_NUM   6  ///CC_VOCAL_MUTE
//#define VM_FILTER2_NUM   6  ///CC_VOCAL_MUTE

//----- HDMI Pre_parser setting -----
#ifdef CC_AUD_HDMI_PARSER_2_0_SUPPORT
typedef enum
{
    AUD_HDMI_PARSER_INIT = 0,
    //AUD_HDMI_PARSER_BUFFERING, //remove buffering,it may cause some output underrun
    AUD_HDMI_PARSER_FIND_VALID_FRAME,
    AUD_HDMI_PARSER_WAIT_VALID_FRAME,
    AUD_HDMI_PARSER_TANSFER_FRAME,

}   AUD_HDMI_PARSER_STATE_T;

/*
typedef enum
{
    AUD_HDMI_NOT_FIND_PA,
    AUD_HDMI_FIND_PA,
    AUD_HDMI_FIND_PB,
}   AUD_HDMI_CHECK_SYNC_WORD_T;
*/

typedef enum
{
    AUD_RAW_AC3 = 0,    //AC3
    AUD_RAW_PAUSE,      // Pause frame
    AUD_RAW_MPEG_1_LY_1, //mpeg1 layer1
    AUD_RAW_MPEG_2_W_O,  //mpeg 1 layer 2/3,mpeg2 w/0 extension
    AUD_RAW_MPEG_2_W,    //mpeg 2 w extension
    AUD_RAW_AAC,
    AUD_RAW_MPEG_2_LY_1,  //mpeg 2 layer 1 LSF
    AUD_RAW_MPEG_2_LY_2_3, //mpeg-2 layer 2/3 LSF
    AUD_RAW_DTS_1,            //DTS type I
    AUD_RAW_DTS_2,            //DTS type II
    AUD_RAW_DTS_3,            //DTS type III
    AUD_RAW_DTS_HD,         //DTS HD
    AUD_RAW_E_AC3,          //E-AC3
    AUD_RAW_AC3_HD,         //Dolby Tru HD
    AUD_RAW_AAC_LOAS,       //mpeg4 AAC_loas
    AUD_RAW_NUM,
} AUD_RAW_FMT_T;

typedef struct
{
    UPTR u4FifoStartAddr;
    UPTR u4FifoEndAddr;
    UINT32 u4FifoSize;
    UPTR u4FifoPtr;
}  AUD_FIFO_INFO_T;


typedef struct
{
    UINT16 u2format;  //pc
    UINT32 u4Size;    //pd
    UPTR u4Wp; //bit-stream start
}  AUD_HDMI_RAW_DATA_INFO_T;

typedef struct
{
    AUD_FIFO_INFO_T eHDMIBuffer;
    AUD_FIFO_INFO_T eDSPBuffer;
    HANDLE_T _hSemaHDMIParserThreadWait;
    BOOL _fgHDMIParserThreadGo;
    AUD_FMT_T e_aud_det_fmt;
    BOOL _fgFmtChg;
    AUD_HDMI_RAW_DATA_INFO_T eSpdifInfo;
    //AUD_HDMI_CHECK_SYNC_WORD_T eCheckSta;
    UINT32 u4PaPb;
    AUD_HDMI_PARSER_STATE_T eHDMIParserSta;
    UPTR u4LastTransferPtr;  //It's used for PCM transfer
    UINT32 u4NoRawDataSize;
    UINT32 u4NoTransferSize;
    UINT32 u4TFrameCount;      //Tranfer frame count, for debug
    BOOL _fgDTSCD;   //_fgDTSCD and _fgDTSSW is for DTS CD,DTS CD don't have papbpcpd,we should reference DTS header
    BOOL _fgDTSSW;    //find DTS Sync word
    BOOL _fgRaw;
    BOOL _fgReset;
} AUD_HDMI_PARSER_INFO_T;


#endif

typedef enum
{
    AUD_HDMI_HDCP = 0,
    AUD_HDMI_INFO_NUM,
}   AUD_HDMI_INFO_T;
//----- end of HDMI Pre_parser-----

typedef enum
{
    AUD_DEC_CMD_SET_STATE,
    AUD_DEC_CMD_TERMINATE,
    AUD_DEC_CMD_MAX
} AUD_DEC_CMD;

typedef enum
{
    AUD_DEC_STATE_INIT,
    AUD_DEC_STATE_PLAY,
    AUD_DEC_STATE_STOP,
    AUD_DEC_STATE_IDLE,
    AUD_DET_STATE_MAX
} AUD_DEC_STATE;

typedef struct
{
    AUD_DEC_CMD       eCmd;
    AUD_DEC_STATE     eStateTo; // for AUD_OMX_CMD_SET_STATE
    //for the future extension
} AUD_DEC_MSG_T;

typedef enum
{
    AUD_STC_AUDIO = 0,     // for some legacy heritage, STC_A and demux STC1(0x1723C)
    AUD_STC_DMX,             // for some legacy heritage, demux_STC2(0x1728C)
    AUD_STC_DMX2            // currently cannot be used
} AUD_STC_ID; // 2_STC_Task

typedef enum
{
#ifdef CC_MT5895
    AUD_AOUT3_BYPASS_OFF,       //normal path3: decoder->encoder..
    AUD_AOUT3_BYPASS_HDMI_HW,   //path1 HW bypass: hdmi rx ->earc(ES2)
    AUD_AOUT3_BYPASS_HDMI_SW,   //path2 SW bypass: hdmi pre-buffer to Aout3 buffer
    AUD_AOUT3_DMIC_M5,
#else
    AUD_AOUT3_BYPASS_OFF,
    AUD_AOUT3_BYPASS_HDMI,
    AUD_AOUT3_BYPASS_DSD,
#endif
} AUD_AOUT3_BYPASS;

#if 0
typedef enum
{
    AUD_AOUT1_BYPASS_OFF,  //path3 that means IEC ->earc
    AUD_AOUT1_BYPASS_HDMI, //path2 SW bypass:hdmi pre-buffer to Aout1 buffer
} AUD_AOUT1_BYPASS;
#endif

typedef struct      //PTSQueue_V2
{
    UINT32       u4Wp;
    UINT32       u4Pts;
    UINT32       u4Refresh;
    UINT32       u4Expired;
    UINT32       u4DmxWp;
    UINT32       u4PtsHi;
    UINT32       u4Reserved5;
    UINT32       u4Reserved6;
    UINT32       u4Reserved7;
    UINT32       u4Reserved8;
    UINT32       u4Reserved9;
    UINT32       u4Reserved10;
    UINT32       u4Reserved11;
    UINT32       u4Reserved12;
    UINT32       u4Reserved13;
    UINT32       u4Reserved14;
} AUD_PES_INFO_T;

typedef enum
{
    AUD_ENC_MODE_OFF = 0,
    AUD_ENC_MODE_INPUT_BEFORE_POST_PROCESSING,
    AUD_ENC_MODE_INPUT_AFTER_POST_PROCESSING,
    AUD_ENC_MODE_NUM
}   AUD_ENC_MODE_T;

#ifdef CC_ENABLE_AOMX
typedef struct {
    UINT8 u1DecId;
    UPTR u4Wp;                        ///< Write pointer
    UINT64 u8Pts;                       ///< PTS
    BOOL fgEOS;                         ///< File playback: End of stream
}GST_AUDIO_PES_T;
#endif


#define AC3_KARA_DISABLE        ((UINT16)0x0001)    ///< disable karaoke mode
#define AC3_KARA_AWARE          ((UINT16)0x0002)    ///< karaoke aware mode
#define AC3_KARA_NONE           ((UINT16)0x0004)    ///< none (in capable mode)
#define AC3_KARA_V1             ((UINT16)0x0008)    ///< V1 only (in capable mode)
#define AC3_KARA_V2             ((UINT16)0x0010)    ///< V2 only (in capable mode)
#define AC3_KARA_BOTH           ((UINT16)0x0020)    ///< V1+V2   (in capable mode)
#define AC3_KARA_VOC_ASIST      ((UINT16)0x0040)    ///< vocal assist mode
#define AC3_KARA_NO_MELODY      ((UINT16)0x0080)    ///< melody off

#define AC3_DM_STEREO           0   ///< AC3 dual mono mode stereo
#define AC3_DM_L_MONO           1   ///< AC3 dual mono mode left mono
#define AC3_DM_R_MONO           2   ///< AC3 dual mono mode right mono
#define AC3_DM_M_MONO           3   ///< AC3 dual mono mode mix mono

#define AC3_COMP_CUSTOM_0       0   ///< AC3 compression mode custom 0
#define AC3_COMP_CUSTOM_1       1   ///< AC3 compression mode custom 1
#define AC3_COMP_LINE           2   ///< AC3 compression line mode P.S. default set as line mode
#define AC3_COMP_RF             3   ///< AC3 compression RF mode
#define AC3_COMP_WLINE          4   ///< AC3 compression wide range line mode

// *********************************************************************
// Constant definitions
// *********************************************************************
#define AUD_OK                  (INT32)(0)
#define AUD_FAIL                (INT32)(-1)

#define AUD_A_DSP               ((UINT8)0)
#define AUD_D_DSP               ((UINT8)1)

#define AUD_DSP0                ((UINT8)0)
#define AUD_DSP1                ((UINT8)1)

#define AUD_DEC_MAIN            ((UINT8)0)
#define AUD_DEC_AUX             ((UINT8)1)
#define AUD_DEC_THIRD           ((UINT8)2)
#define AUD_DEC_4TH             ((UINT8)3)
#define AUD_DEC_SUB             ((UINT8)4)

#define AUD_LINEIN_MAIN         ((UINT8)0)
#define AUD_LINEIN_AUX          ((UINT8)1)
#define AUD_LINEIN_THIRD        ((UINT8)2)

#define INTERNAL_DAC_ID_MAIN    ((UINT8)0)
#define INTERNAL_DAC_ID_AUX     ((UINT8)1)
#define INTERNAL_DAC_ID_THIRD   ((UINT8)2)
#define INTERNAL_DAC_ID_4TH     ((UINT8)3)


/* Move AUD_DEC_NUM to drv_cust/drvcust_if.h */

#define AUD_CMD_FLAG_PLAY       ((UINT32) (1<<(UINT32)AUD_CMD_PLAY))
#define AUD_CMD_FLAG_STOP       ((UINT32) (1<<(UINT32)AUD_CMD_STOP))
#define AUD_CMD_FLAG_RESET      ((UINT32) (1<<(UINT32)AUD_CMD_RESET))
#define AUD_CMD_FLAG_PAUSE      ((UINT32) (1<<(UINT32)AUD_CMD_PAUSE))
#define AUD_CMD_FLAG_AVSYNC     ((UINT32) (1<<(UINT32)AUD_CMD_AVSYNC))
#define AUD_CMD_FLAG_LOADCODE   ((UINT32) (1<<(UINT32)AUD_CMD_LOADCODE))
#define AUD_CMD_FLAG_RESUME     ((UINT32) (1<<(UINT32)AUD_CMD_RESUME))
#define AUD_CMD_FLAG_STOPSTREAM ((UINT32) (1<<(UINT32)AUD_CMD_STOPSTREAM))
#define AUD_CMD_FLAG_CHANGE_FORMAT     ((UINT32) (1<<(UINT32)AUD_CMD_CHANGE_FORMAT))

#define AUD_DUAL_DEC_CMD_FLAG_PLAY  ((UINT32) (1<<(UINT32)AUD_DUAL_DEC_CMD_PLAY))
#define AUD_DUAL_DEC_CMD_FLAG_STOP  ((UINT32) (1<<(UINT32)AUD_DUAL_DEC_CMD_STOP))

/* AV Sync Debug Log Switch */
#define AUD_DBG_SYNC_SEND_PTS       ((UINT16) (1<<(UINT16)0))
#define AUD_DBG_SYNC_STC_DIFF       ((UINT16) (1<<(UINT16)1))
#define AUD_DBG_SYNC_PTS_FRESH      ((UINT16) (1<<(UINT16)2))

#define AUD_POST_DISABLE ((UINT8)0)
#define AUD_POST_ENABLE ((UINT8)1)
#define AUD_POST_ENABLE_FROM_FALLBACK ((UINT8)2)

#ifdef CC_AUD_ARM_SUPPORT  //#A0008
/* Audio to Main signals
*/
// Signal
#define APROC_A2M_SIG_REG APROC_A2MSR0
#define APROC_A2M_DATA0_REG APROC_A2MSR1
#define APROC_A2M_DATA1_REG APROC_A2MSR2
#define APROC_A2M_READ_REG APROC_A2MSR3
#define APROC_A2M_READ_REG1 APROC_A2MSR4
#define APROC_A2M_READ_REG2 APROC_A2MSR5
#define APROC_A2M_READ_REG3 APROC_A2MSR6
#define APROC_A2M_READ_REG4 APROC_A2MSR7
#define APROC_A2M_READ_REG5 APROC_A2MSR8
#define APROC_A2M_READ_REG6 APROC_A2MSR9
#define APROC_A2M_READ_REG7 APROC_A2MSR10
#define APROC_A2M_READ_REG8 APROC_A2MSR11

// Log
#define APROC_A2M_LOG_ADDR APROC_A2MSR31
#define APROC_A2M_LOG_DATA0 APROC_A2MSR30
#define APROC_A2M_LOG_DATA1 APROC_A2MSR29
#define APROC_A2M_LOG_DATA2 APROC_A2MSR28
#define APROC_A2M_LOG_DATA3 APROC_A2MSR27
// Isr Log
#define APROC_A2M_ISR_LOG_ADDR APROC_A2MSR26
#define APROC_A2M_ISR_LOG_DATA0 APROC_A2MSR25
#define APROC_A2M_ISR_LOG_DATA1 APROC_A2MSR24
#define APROC_A2M_ISR_LOG_DATA2 APROC_A2MSR23
#define APROC_A2M_ISR_LOG_DATA3 APROC_A2MSR22
#define APROC_A2M_ISR_LOG_DATA4 APROC_A2MSR21
/* Main to Audio signals
*/
#define APROC_M2A_SIG_REG APROC_M2ASR0
#define APROC_M2A_DATA0_REG APROC_M2ASR1
#define APROC_M2A_DATA1_REG APROC_M2ASR2

#define APROC_M2A_SIG_REG_CMD APROC_M2ASR4
#define APROC_M2A_DATA0_REG_CMD APROC_M2ASR5
#define APROC_M2A_DATA1_REG_CMD APROC_M2ASR6
#define AUD_APROC_CHAR_NUM 2000 //2048
#define AUD_APROC_CHAR_LEN (AUD_APROC_CHAR_NUM+3+2)

/* Defines
*/
//---------------------------------------------------------------------------
// Signal define (Sync. with APROC)
//
// Signal structure:
// u4Sig, u4Data0, u4Data1
//

/* Signal = APROC_SIGNAL(Idx,Cmd) = [23,8] Idx, [7,0] Cmd
*/
#define APROC_SIG_CMD_MSK           0xFF
#define APROC_SIG_CMD(a)            ((a) & APROC_SIG_CMD_MSK)
#define APROC_SIG_IDX_MSK           0xFFFF
#define APROC_SIG_IDX_SHIFT 8
#define APROC_SIG_IDX(a)            ((a) & APROC_SIG_IDX_MSK)
#define APROC_SIGNAL(Idx,Cmd)       ((APROC_SIG_IDX(Idx) << APROC_SIG_IDX_SHIFT) | APROC_SIG_CMD(Cmd))

/* u4Sig
*/
// Cmd
#define APROC_SIG_CMD_EXIT_CODE     APROC_SIG_CMD(0x0) // kernel usage
#define APROC_SIG_CMD_READ          APROC_SIG_CMD(0x1)
#define APROC_SIG_CMD_WRITE         APROC_SIG_CMD(0x2)
#define APROC_SIG_CMD_OPEN          APROC_SIG_CMD(0x3)
#define APROC_SIG_CMD_CLOSE         APROC_SIG_CMD(0x4)
#define APROC_SIG_CMD_INIT          APROC_SIG_CMD(0x5)
#define APROC_SIG_CMD_SET           APROC_SIG_CMD(0x6)
#define APROC_SIG_CMD_GET           APROC_SIG_CMD(0x7)
#define APROC_SIG_CMD_NOTIFY        APROC_SIG_CMD(0x8)
#ifdef CC_AUD_FIX_A2M_INT_LOST
#define APROC_SIG_CMD_OPEN_AGAIN    APROC_SIG_CMD(0x9)
#define APROC_SIG_CMD_CLOSE_AGAIN   APROC_SIG_CMD(0xa)
#endif // CC_AUD_FIX_A2M_INT_LOST
// Idx
#define APROC_SIG_IDX_KERNEL        APROC_SIG_IDX(0x1)
#define APROC_SIG_IDX_POSTPROC      APROC_SIG_IDX(0x2)
#define APROC_SIG_IDX_ADSP          APROC_SIG_IDX(0x3)
#define APROC_SIG_IDX_IOBUF         APROC_SIG_IDX(0x4)
#define APROC_SIG_IDX_APROC         APROC_SIG_IDX(0x5)
#define APROC_SIG_IDX_DEC0          APROC_SIG_IDX(0x6)
#define APROC_SIG_IDX_DEC1          APROC_SIG_IDX(0x7)
#define APROC_SIG_IDX_DEC2          APROC_SIG_IDX(0x8)
#define APROC_SIG_IDX_DEC3          APROC_SIG_IDX(0x9)
#define APROC_SIG_IDX_MIXSND0       APROC_SIG_IDX(0xA) //parson mixsnd
#define APROC_SIG_IDX_MIXSND1       APROC_SIG_IDX(0xB)
#define APROC_SIG_IDX_MIXSND2       APROC_SIG_IDX(0xC) //parson mixsnd
#define APROC_SIG_IDX_MIXSND3       APROC_SIG_IDX(0xD)

#define APROC_SIG_IDX_UPLOAD0       APROC_SIG_IDX(0xE) //parson upload
#define APROC_SIG_IDX_UPLOAD1       APROC_SIG_IDX(0xF) //parson upload
#define APROC_SIG_IDX_UPLOAD2       APROC_SIG_IDX(0x10) //parson upload
#define APROC_SIG_IDX_UPLOAD3       APROC_SIG_IDX(0x11) //parson upload

#define APROC_SIG_IDX_SRC           APROC_SIG_IDX(0x12)
#define APROC_SIG_IDX_RISCPOST      APROC_SIG_IDX(0x13)  //parson riscpost
#define APROC_SIG_IDX_HW_MIXSND0    APROC_SIG_IDX(0x14)
#define APROC_SIG_IDX_HW_MIXSND1    APROC_SIG_IDX(0x15)
#define APROC_SIG_IDX_DEC2_AD       APROC_SIG_IDX(0x16)
#define APROC_SIG_IDX_A2M_ISR       APROC_SIG_IDX(0x17)  //HW Mixsnd
#ifdef CC_AUD_DOLBY_SUPPORT_MS12_RISCENC
#define APROC_SIG_IDX_RISCENC_UP    APROC_SIG_IDX(0x18)
#define APROC_SIG_IDX_RISCENC_DOWN  APROC_SIG_IDX(0x19)
#endif // CC_AUD_DOLBY_SUPPORT_MS12_RISCENC
#ifdef CC_AUD_CPU_DEC
#define APROC_SIG_IDX_CPUDEC0_ISR   APROC_SIG_IDX(0x1A)  //
#define APROC_SIG_IDX_CPUDEC1_ISR   APROC_SIG_IDX(0x1B)  //
#define APROC_SIG_IDX_CPUDEC2_ISR   APROC_SIG_IDX(0x1C)  //
#define APROC_SIG_IDX_CPUIEC0_ISR   APROC_SIG_IDX(0x1D)  //
#define APROC_SIG_IDX_CPUIEC1_ISR   APROC_SIG_IDX(0x1E)  //
#define APROC_SIG_IDX_CPUIEC2_ISR   APROC_SIG_IDX(0x1F)  //
#endif // CC_AUD_CPU_DEC
#ifdef CC_AUD_RISCIEC_SUPPORT
#define APROC_SIG_IDX_IEC_ISR    APROC_SIG_IDX(0x20)
#endif//CC_AUD_RISCIEC_SUPPORT
#ifdef CC_EARC_SW_BYPASS
#define APROC_SIG_IDX_AOUT3    APROC_SIG_IDX(0x25)
#endif // CC_EARC_SW_BYPASS
//#define CC_AUD_APROC_HWMIX
#endif //CC_AUD_ARM_SUPPORT  //#A0008

#if defined(CC_AUD_ARM_SUPPORT)//#A0002
/* APROC_SIG_CMD_SET u4Data0
*/
#define APROC_SIG_SET_UPDATE_RP  APROC_SIG_CMD(0x1)
#define APROC_SIG_SET_UPDATE_WP  APROC_SIG_CMD(0x2)
#define APROC_SIG_SET_RESET      APROC_SIG_CMD(0x3) //parson mixsnd
#define APROC_SIG_SET_DELAY0_ROUTINE     APROC_SIG_CMD(0x4) //parson delay
#define APROC_SIG_SET_DELAY1_ROUTINE     APROC_SIG_CMD(0x5)
#define APROC_SIG_SET_DELAY2_ROUTINE     APROC_SIG_CMD(0x6)
#define APROC_SIG_SET_SP_PATH_ROUTINE     APROC_SIG_CMD(0x7)
#define APROC_SIG_SET_HP_PATH_ROUTINE     APROC_SIG_CMD(0x8)
#define APROC_SIG_SET_DMIX0_ROUTINE       APROC_SIG_CMD(0x9)
#define APROC_SIG_SET_DMIX1_ROUTINE       APROC_SIG_CMD(0xA)
#define APROC_SIG_SET_DMIX2_ROUTINE       APROC_SIG_CMD(0xB)
#define APROC_SIG_SET_AMIXER0_ROUTINE       APROC_SIG_CMD(0xC)
#define APROC_SIG_SET_AMIXER1_ROUTINE       APROC_SIG_CMD(0xD)
#define APROC_SIG_SET_AMIXER2_ROUTINE       APROC_SIG_CMD(0xE)
#define APROC_SIG_SET_SP_LSRS_PATH_ROUTINE     APROC_SIG_CMD(0xF)
#define APROC_SIG_SET_DEV_WORKING       APROC_SIG_CMD(0x10)
#define APROC_SIG_SET_DEV_SLEEP         APROC_SIG_CMD(0x11)
#define APROC_SIG_SET_MON_PATH_ROUTINE     APROC_SIG_CMD(0x12)
#define APROC_SIG_SET_IEC_PATH_ROUTINE     APROC_SIG_CMD(0x13)
#define APROC_SIG_SET_DELAY3_ROUTINE     APROC_SIG_CMD(0x14)
#define APROC_SIG_SET_DELAY4_ROUTINE     APROC_SIG_CMD(0x15)
#define APROC_SIG_SET_DELAY5_ROUTINE     APROC_SIG_CMD(0x16)
#define APROC_SIG_SET_DELAY6_ROUTINE     APROC_SIG_CMD(0x17)
#define APROC_SIG_SET_DELAY_RAW_ENC_ROUTINE      APROC_SIG_CMD(0x18)

/* APROC_SIG_CMD_OPEN u4Data0
*/
#define APROC_COMMAND_DONE          0x1

/* APROC_SIG_CMD_NOTIFY u4Data0
*/
// Cmd
#define APROC_SIG_NOTIFY_DONE        APROC_SIG_CMD(0x1)
#define APROC_SIG_NOTIFY_WRITE      APROC_SIG_CMD(0x2)
#define APROC_SIG_NOTIFY_UPDATE_RP  APROC_SIG_CMD(0x3)
#define APROC_SIG_NOTIFY_UPDATE_WP  APROC_SIG_CMD(0x4)
#define APROC_SIG_NOTIFY_INIT_AOUT2 APROC_SIG_CMD(0x5)
#define APROC_SIG_NOTIFY_UPDATE_AOUT2_NSADR     APROC_SIG_CMD(0x6)
#define APROC_SIG_NOTIFY_APTS                   APROC_SIG_CMD(0x7)
#define APROC_SIG_NOTIFY_HOLD_COMPLETE          APROC_SIG_CMD(0x8)
#define APROC_SIG_NOTIFY_SRC                    APROC_SIG_CMD(0x9)
#define APROC_SIG_NOTIFY_MCPU_UPDATE_ROUNTIE    APROC_SIG_CMD(0xA)
#define APROC_SIG_NOTIFY_SKIP_DONE      APROC_SIG_CMD(0xB)
#define APROC_SIG_NOTIFY_CUR_PTR_QRY    APROC_SIG_CMD(0xC)
#define APROC_SIG_NOTIFY_UPDATE_RP_OR_WP  APROC_SIG_CMD(0xD)
#ifdef CC_AUD_CPU_DEC
#define APROC_SIG_NOTIFY_DELAY_ROUNTIE_DONE  APROC_SIG_CMD(0xE)
#endif // CC_AUD_CPU_DEC
#ifdef CC_AUD_RISCIEC_SUPPORT
#define APROC_SIG_NOTIFY_UPDATE_IEC_INFO  APROC_SIG_CMD(0xF)
#endif //CC_AUD_RISCIEC_SUPPORT
// Idx
#define APROC_SIG_NOTIFY_IDX_KERNEL APROC_SIG_IDX(0x1)
#define APROC_SIG_NOTIFY_IDX_DEC0  APROC_SIG_IDX(0x2)
#define APROC_SIG_NOTIFY_IDX_DEC1 APROC_SIG_IDX(0x3)
#define APROC_SIG_NOTIFY_IDX_DEC2 APROC_SIG_IDX(0x4)
#define APROC_SIG_NOTIFY_IDX_DEC3 APROC_SIG_IDX(0x5) // reserved
#define APROC_SIG_NOTIFY_IDX_ALL APROC_SIG_IDX(0x6)

//---------------------------------------------------------------------------
// Register Map (Sync. with APROC)
//
/* _to_extend_ */
#ifdef CC_AUD_ARM_RENDER
 #if defined (CC_AUD_DISABLE_2ND_DSP) && defined (CC_AUD_ENLARGE_APROC_DRAM)
    #ifndef CC_AUD_DOLBY_SUPPORT_MS12
#define APROC_DATAPORT_BASE 0x39D000 // hard code, refer to aarm.scat
#define APROC_REGMAP_BASE   0x39E000 // hard code, refer to aarm.scat
#define APROC_AQDATA_BASE   0x3A8000
    #endif // CC_AUD_DOLBY_SUPPORT_MS12
 #else
    #ifndef CC_AUD_DOLBY_SUPPORT_MS12
#define APROC_DATAPORT_BASE 0x170000  //0xFA000 // hard code, refer to aarm.scat
#define APROC_REGMAP_BASE 0x171000   //0xFB000 // hard code, refer to aarm.scat
#define APROC_AQDATA_BASE 0x178000  //0x104000
    #endif // CC_AUD_DOLBY_SUPPORT_MS12
 #endif //CC_AUD_DISABLE_2ND_DSP && CC_AUD_ENLARGE_APROC_DRAM
#else
#define APROC_REGMAP_BASE 0xFB000
#endif // CC_AUD_ARM_RENDER

#ifdef CC_AUD_SUPPORT_DBXTV
/** Total Sonics modes for DBXTV_SetMode() */
typedef enum _DBXTV_TotSonMode
{
    TOTSON_ON       = 0,
    TOTSON_OFF
} DBXTV_TotSonMode;

/** Total Volume modes for DBXTV_SetMode() */
typedef enum _DBXTV_TotVolMode
{
    TOTVOL_NORMAL   = 0,
    TOTVOL_NIGHT,
    TOTVOL_OFF
} DBXTV_TotVolMode;

/** Total Surround modes for DBXTV_SetMode() */
typedef enum _DBXTV_TotSurMode
{
    TOTSUR_ON       = 0,
    TOTSUR_OFF
} DBXTV_TotSurMode;


/** Total Sonics modes for DBXTV_SetMode() */
typedef enum _DBXTV_TotSonTestMode
{
    TOTSON_NORMAL       = 0,
    TOTSON_MOVIE,
    TOTSON_NIGHT,
    TOTSON_DRC,
    TOTSON_NUM
} DBXTV_TotSonTestMode;
/** Total Sonics preset length in DM */
#define DBXTV_Param_LENGTH 95
/** Total Sonics preset length in PM */
#define DBXTV_Coeff_LENGTH 100
/* Lengths of presets (in words) */
/* Note if the DM or PM structure is changed, these values will need to be updated */
#define DBX_Param_LENGTH_DVC (54)
#define DBX_Coeff_LENGTH_DVC (10)
#define DBX_Param_LENGTH_DPP (20)
#define DBX_Coeff_LENGTH_DPP (25)
#define DBX_Param_LENGTH_IW (3)
#define DBX_Coeff_LENGTH_IW (5)
#define DBX_Param_LENGTH_EQ (2)
#define DBX_Coeff_LENGTH_EQ (35)
#define DBX_Param_LENGTH_BE (16)
#define DBX_Coeff_LENGTH_BE (25)

/* Offsets of presets (in words) */
/* Note if the DM or PM structure is changed, these values will need to be updated */
#define DBX_Param_OFFSET_DVC (0)
#define DBX_Coeff_OFFSET_DVC (0)
#define DBX_Param_OFFSET_DPP (54)
#define DBX_Coeff_OFFSET_DPP (10)
#define DBX_Param_OFFSET_IW (74)
#define DBX_Coeff_OFFSET_IW (35)
#define DBX_Param_OFFSET_EQ (77)
#define DBX_Coeff_OFFSET_EQ (40)
#define DBX_Param_OFFSET_BE (79)
#define DBX_Coeff_OFFSET_BE (75)

/* Offsets for DVCOnOff, DPPOnOff, IWBypass, EQBypass, BEOnOff (in words) */
/* Note if the DM structure is changed, these values will need to be updated */
#define DBX_Param_OFFSET_DVCOnOff (0)
#define DBX_Param_OFFSET_DPPOnOff (54)
#define DBX_Param_OFFSET_IWBypass (74)
#define DBX_Param_OFFSET_EQBypass (77)
#define DBX_Param_OFFSET_BEOnOff (79)
#define DBX_Param_OFFSET_DVCHiLevel (2)
#define DBX_Param_OFFSET_DVCLoLevel (23)

#define Enable_DVC (1<<0)
#define Enable_DPP (1<<1)
#define Enable_IW (1<<2)
#define Enable_EQ (1<<3)
#define Enable_BE (1<<4)
#define Enable_All (Enable_DVC | Enable_DPP | Enable_IW | Enable_EQ | Enable_BE)

extern void AUD_DBXTV_Enable(BOOL fgEnable);
extern UINT32 AUD_DBXTV_GetDbxPara(UINT32 u4Offset, UINT32 u4Type);
extern BOOL AUD_DBXTV_GetEnable(void);
extern void AUD_DBXTV_ReInit(void);
extern void AUD_DBXTV_SetMode(DBXTV_TotSonMode TotSonMode,
    DBXTV_TotVolMode TotVolMode, DBXTV_TotSurMode TotSurMode, UINT32 Enable);
extern void AUD_DBXTV_GetMode(DBXTV_TotSonMode *pTotSonMode,
    DBXTV_TotVolMode *pTotVolMode, DBXTV_TotSurMode *pTotSurMode, UINT32 *pEnable);
extern void AUD_DBXTV_SetTestEnable(BOOL fgEnable);
extern BOOL AUD_DBXTV_GetTestEnable(void);
extern DBXTV_TotSonTestMode AUD_DBXTV_GetTestMode(void);
extern void AUD_DBXTV_SetTestMode(DBXTV_TotSonTestMode TotSonMode, UINT32 Enable);

extern void AUD_DBXTV_SetSonMode(DBXTV_TotSonMode TotSonModeCust);
extern void AUD_DBXTV_SetVolMode(DBXTV_TotVolMode TotVolModeCust);
extern void AUD_DBXTV_SetSurMode(DBXTV_TotSurMode TotSurModeCust);
extern void AUD_DBXTV_GetSonMode(DBXTV_TotSonMode *pTotSonMode);
extern void AUD_DBXTV_GetVolMode(DBXTV_TotVolMode *pTotVolMode);
extern void AUD_DBXTV_GetSurMode(DBXTV_TotSurMode *pTotSurMode);

#endif//CC_AUD_SUPPORT_DBXTV

#if 1
// for Upload post usage
typedef enum
{
    /* Common, 0x0 ~ 0x100
    */
    //APROC_REG_VERSION = 0, // 0x0 ~ 0x100

    /* IO Input buffer, 0x10 ~ 0x110, 8 elements, total 32 buffers     */
    // APROC_REG_IOBUF_IN = 0x10,
    APROC_REG_IOBUF_IN_START = 0x10,
    APROC_REG_IOBUF_IN_END,
    APROC_REG_IOBUF_IN_SIZE, // in bytes
    APROC_REG_IOBUF_IN_SI_BASE,

    // APROC_REG_IOBUF_OUTPUT = 0x18,
    APROC_REG_IOBUF_OUT_START = 0x18,
    APROC_REG_IOBUF_OUT_END,
    APROC_REG_IOBUF_OUT_SIZE, // in bytes
    APROC_REG_IOBUF_OUT_SI_BASE,

    /* Working register, 0x110~ 0x120    */
    APROC_REG_POSTPROC_WORKING_VOL0 = 0x110,
    APROC_REG_POSTPROC_WORKING_VOL1,

    /* Setting register, 0x120~0x130     */
    APROC_REG_POSTPROC_SETTING_VOL0 = 0x120,
    APROC_REG_POSTPROC_SETTING_VOL1,

    #ifdef CC_AUD_SUPPORT_DBXTV
    /* DBXTV Setting register, 0x130 ~ 0x1F3*/
    APROC_REG_DBX_PARAM_BASE = 0x130,
    APROC_REG_DBX_COEFF_BASE = APROC_REG_DBX_PARAM_BASE + DBXTV_Param_LENGTH,//0x18F,
    APROC_REG_DBX_END = APROC_REG_DBX_COEFF_BASE + DBXTV_Coeff_LENGTH, //0x1F3,
    APROC_REG_DBX_ENABLE = APROC_REG_DBX_END + 1, //0x1f4
    APROC_REG_DBX_REINIT = APROC_REG_DBX_ENABLE + 1, //0x1f5
    #endif


//    APROC_REGMAP_SIZE = 0x800
} _APROC_REGMAP_T;
#endif

#ifdef ARM_SUPPORT_SRS_SS
#define APROC_ASM_ID_SRS_SS3D   APROC_ASM_ID_3RD_POST_23
#define APROC_ASM_ID_SRS_TVOL   APROC_ASM_ID_3RD_POST_24
#define APROC_ASM_ID_SRS_AEQ    APROC_ASM_ID_3RD_POST_24
#define APROC_ASM_ID_SRS_TRUD   APROC_ASM_ID_3RD_POST_24

#define APROC_ASM_ID_SRS_CC3D   APROC_ASM_ID_3RD_POST_25

#define APROC_ASM_ID_SRS_TSHD   APROC_ASM_ID_3RD_POST_26
#define APROC_ASM_ID_SRS_SPHPF  APROC_ASM_ID_3RD_POST_26

#define APROC_ASM_ID_SRS_TBHD   APROC_ASM_ID_3RD_POST_27
#define APROC_ASM_ID_SRS_SRS3D  APROC_ASM_ID_3RD_POST_27

#define APROC_ASM_ID_SRS_CSHP   APROC_ASM_ID_3RD_POST_28

#define APROC_ASM_ID_SRS_HPF    APROC_ASM_ID_3RD_POST_29
#define APROC_ASM_ID_SRS_GEQ    APROC_ASM_ID_3RD_POST_29

#define APROC_ASM_ID_SRS_BASSTREB   APROC_ASM_ID_3RD_POST_2A
#define APROC_ASM_ID_SRS_MVOL   APROC_ASM_ID_3RD_POST_2A
#define APROC_ASM_ID_SRS_HL     APROC_ASM_ID_3RD_POST_2A
#endif

#if 1//def SUPPORT_SAPOST
#define APROC_ASM_ID_SAP_MISC_23   APROC_ASM_ID_3RD_POST_23    //APROC_ASM_SPOST_MISC23_T
#endif

#ifdef ARM_SUPPORT_SRS_TS3D
#define APROC_ASM_ID_SRS_TS3D        APROC_ASM_ID_3RD_POST_2B

#define APROC_ASM_ID_SRS_TRUSHD      APROC_ASM_ID_3RD_POST_24
#define APROC_ASM_ID_SRS_LEVEL       APROC_ASM_ID_3RD_POST_24

#define APROC_ASM_ID_SRS_TVOLHD      APROC_ASM_ID_3RD_POST_25
#define APROC_ASM_ID_SRS_TRUD        APROC_ASM_ID_3RD_POST_25

#define APROC_ASM_ID_SRS_CC3D        APROC_ASM_ID_3RD_POST_26
#define APROC_ASM_ID_SRS_TSHD        APROC_ASM_ID_3RD_POST_26

#define APROC_ASM_ID_SRS_CSD         APROC_ASM_ID_3RD_POST_27
#define APROC_ASM_ID_SRS_CSHP        APROC_ASM_ID_3RD_POST_27
#define APROC_ASM_ID_SRS_SRS3D       APROC_ASM_ID_3RD_POST_27

#define APROC_ASM_ID_SRS_SPHPF       APROC_ASM_ID_3RD_POST_28
#define APROC_ASM_ID_SRS_TBHDX       APROC_ASM_ID_3RD_POST_28
#define APROC_ASM_ID_SRS_COMMON      APROC_ASM_ID_3RD_POST_28
#define APROC_ASM_ID_SRS_AEQ         APROC_ASM_ID_3RD_POST_28

#define APROC_ASM_ID_SRS_GEQ         APROC_ASM_ID_3RD_POST_29                    //5B
#define APROC_ASM_ID_SRS_HDLMT       APROC_ASM_ID_3RD_POST_29
#define APROC_ASM_ID_SRS_WOWHD       APROC_ASM_ID_3RD_POST_29

#define APROC_ASM_ID_SRS_MCDYNAMIC   APROC_ASM_ID_3RD_POST_2A
#define APROC_ASM_ID_SRS_DWNUP       APROC_ASM_ID_3RD_POST_2A
#endif





typedef struct {
    BOOL fgMVSFlag;
    INT32 i4CrossTalkGain;
    INT32 i4ClarityGain;
    INT32 i4LRGain;
    INT32 i4WidthGain;
    INT32 i4BassGain;
    INT32 i4OutputGain;
    INT32 i4Fc;
} APROC_DSP_MVS_SET_T;

typedef struct {
    BOOL fgMVBFlag;
    UINT32 u4Fc;
    UINT32 u4ExpanderTh;
    UINT32 u4AttackStep;
    UINT32 u4ReleaseStep;
    INT32 i4BassGain;
    INT32 i4DetGain;
    INT32 i4OutputGain;
} APROC_DSP_MVB_SET_T;

typedef struct {
    UINT32 u4Fc;
    UINT32 u4Q;
    UINT32 u4Gain;
} APROC_DSP_PEQ_SET_T;

typedef struct {
    UINT32 u4Type;
    UINT32 u4Fc;
    UINT32 u4Q;
    UINT32 u4Gain;
} APROC_DSP_HPF_SET_T;

typedef struct {
    UINT32 u4Type;
    UINT32 u4Fc;
    UINT32 u4Q;
    UINT32 u4Gain;
} APROC_DSP_LPF_SET_T;

typedef struct {
    UINT32 u4Mode;
    UINT32 u4Threshold;
    UINT32 u4AttackWaitNum;
    UINT32 u4ReleaseStep;
    UINT32 u4AttackStep;
    UINT32 u4AttackGain;
} APROC_DSP_SILENCE_SET_T;

typedef struct {
    UINT32 u4Flag;
    UINT32 u4Value;
} APROC_DSP_BALANCE_SET_T;


typedef struct {
    UINT32 u4Type;
    UINT32 u4Fc;
    UINT32 u4Q;
    UINT32 u4Gain;
} APROC_DSP_EQ_SET_T;

typedef struct {
    BOOL  fgLimtFlag;
    INT32 i4SetTargetLevel;
    INT32 i4SetAttackStepDB;
    INT32 i4SetReleaseStepDB;
    INT32 i4SetRatio;
    INT32 i4SetSilenceLevel;
    INT32 i4SetMaxExpandGainDB;
    INT32 i4SetPostGain;
    INT32 i4SetHoldPeriod;
    UINT32 u4SetProcMode; // DSP_DRC_SET_PROC_MODE_T
    INT32 i4SetDetPeriod;
    INT32 i4SetSgainPeriod;
} APROC_DSP_LIMT_SET_T;

typedef enum {
    APROC_LIMITER_SP,
    APROC_LIMITER_SP_SW,
    APROC_LIMITER_SP_LSRS,
    APROC_LIMITER_HP,
    APROC_LIMITER_IDX_NUM
} APROC_DSP_LIMT_IDX_T;

#if 0

typedef enum {
#ifdef CC_AUD_UNICORN_CUST
    APROC_SPK_OUTPUT_2K_2_2 = 0, // L+R+2Assist
    APROC_SPK_OUTPUT_2K_2_1,     //L+R+1Assist
    APROC_SPK_OUTPUT_4K,
    APROC_AUDIO_SYS_OUTPUT_2K,
    APROC_AUDIO_SYS_OUTPUT_4K,
    APROC_AUDIO_SYS_OUTPUT_2K_AUTO1,
    APROC_AUDIO_SYS_OUTPUT_4K_AUTO1,
    APROC_AUDIO_SYS_OUTPUT_2K_AUTO2,
    APROC_AUDIO_SYS_OUTPUT_4K_AUTO2,
    APROC_SPK_OUTPUT_4_0 = 0xA, // L+R+LS+RS
#else
    APROC_SPK_OUTPUT_4_0 = 0x0,
#endif
    APROC_SPK_OUTPUT_4_1,       //L+R+LS+RS+SW
    APROC_SPK_OUTPUT_2_2,       //L+R+SW_L+SW_R
    APROC_SPK_OUTPUT_2_1,       //L+R+SW
    APROC_SPK_OUTPUT_2_0,       //L+R
    APROC_SPK_OUTPUT_BT,
    APROC_SPK_OUTPUT_5_1,       //L+R+C+LS+RS+SW
    APROC_SPK_OUTPUT_7_1,       //L+R+C+LS+RS+SW+CH7+CH8
    APROC_SPK_OUTPUT_TEST = 0x100,
    #ifdef CC_S_PLATFORM
    APROC_SPK_OUTPUT_DTS     //DTS certification
    #endif
} APROC_SPK_OUTPUT_T;
#endif



#if (defined(CC_AUD_UNICORN_CUST) || defined(CC_MAPLE_CUST_DRV))

/* APROC Data Port (ADP)
 <TABLE>         <offset>  <end>  <Length>
 DOWNMIX_TBL           0      9        10
 DOWNMIX_TBL1         10     19        10
 LSRS_FIR             20    147       128
 STR_INFO            290    295         6
 SAPOST_DBG          296    807       512
 -------------------------------------------
 DATAPORT_MAX              1023
*/

#define APROC_DATAPORT_SIZE    1024 // maximum dataport size = 1024 (dataport.s)
#define APROC_DATAPORT_REGSIZE    4 // bytes

#define APROC_DATAPORT_DOWNMIX_TBL_BASE 0x0
#define APROC_DATAPORT_DOWNMIX_TBL_SIZE 10
#define APROC_DATAPORT_DOWNMIX_TBL_1_BASE (APROC_DATAPORT_DOWNMIX_TBL_BASE + APROC_DATAPORT_DOWNMIX_TBL_SIZE)

#define APROC_DATAPORT_FIR_SCALE_SIZE 1
#define APROC_DATAPORT_SP_LSRS_FIR_TAP 128
#define APROC_DATAPORT_SP_LSRS_128FIR_BASE (APROC_DATAPORT_DOWNMIX_TBL_1_BASE + APROC_DATAPORT_DOWNMIX_TBL_SIZE)
#define APROC_DATAPORT_SP_LSRS_128FIR_SCALE_BASE (APROC_DATAPORT_SP_LSRS_128FIR_BASE+APROC_DATAPORT_SP_LSRS_FIR_TAP)
#define APROC_DATAPORT_SP_LSRS_128FIR_SIZE (APROC_DATAPORT_SP_LSRS_FIR_TAP+APROC_DATAPORT_FIR_SCALE_SIZE)
#define APROC_DATAPORT_SP_FIR_TAP 128
#define APROC_DATAPORT_SP_FIR_BASE (APROC_DATAPORT_SP_LSRS_128FIR_SCALE_BASE + APROC_DATAPORT_FIR_SCALE_SIZE)
#define APROC_DATAPORT_SP_FIR_SCALE_BASE (APROC_DATAPORT_SP_FIR_BASE+APROC_DATAPORT_SP_LSRS_FIR_TAP)
#define APROC_DATAPORT_SP_FIR_SIZE (APROC_DATAPORT_SP_FIR_TAP+APROC_DATAPORT_FIR_SCALE_SIZE)

#define APROC_DATAPORT_SAPOST_GEN_CONFIG_BASE (APROC_DATAPORT_SP_FIR_SCALE_BASE+APROC_DATAPORT_FIR_SCALE_SIZE)
#define APROC_DATAPORT_SAPOST_GEN_CONFIG_SIZE (6)
#define APROC_DATAPORT_SAPOST_POST_CONFIG_BASE (APROC_DATAPORT_SAPOST_GEN_CONFIG_BASE+APROC_DATAPORT_SAPOST_GEN_CONFIG_SIZE)
#define APROC_DATAPORT_SAPOST_POST_CONFIG_SIZE (6)
#define APROC_DATAPORT_SAPOST_STR_INFO_BASE (APROC_DATAPORT_SAPOST_POST_CONFIG_BASE+APROC_DATAPORT_SAPOST_POST_CONFIG_SIZE)
#define APROC_DATAPORT_SAPOST_STR_INFO_SIZE (6)

#define APROC_DATAPORT_SAPOST_DEBUG_BASE (APROC_DATAPORT_SAPOST_STR_INFO_BASE+APROC_DATAPORT_SAPOST_STR_INFO_SIZE)
#define APROC_DATAPORT_SAPOST_DEBUG_SIZE (512)
#define SAPOST_DEBUG_SZ (APROC_DATAPORT_SAPOST_DEBUG_SIZE*4)

#define APROC_DATAPORT_SAPOST_SP_MODE_BASE (APROC_DATAPORT_SAPOST_DEBUG_BASE+APROC_DATAPORT_SAPOST_DEBUG_SIZE)
#define APROC_DATAPORT_SAPOST_SP_MODE_SIZE (6)
#define APROC_DATAPORT_SAPOST_BT_INFO_BASE (APROC_DATAPORT_SAPOST_SP_MODE_BASE+APROC_DATAPORT_SAPOST_SP_MODE_SIZE)
#define APROC_DATAPORT_SAPOST_BT_INFO_SIZE (6)
#define APROC_DATAPORT_SAPOST_AUD_ITEM_CMD_BASE (APROC_DATAPORT_SAPOST_BT_INFO_BASE+APROC_DATAPORT_SAPOST_BT_INFO_SIZE)
#define APROC_DATAPORT_SAPOST_AUD_ITEM_CMD_SIZE (68)
#else
/* APROC Data Port (ADP)
    */
#define APROC_DATAPORT_SIZE 1024
#define APROC_DATAPORT_REGSIZE    4 // bytes

#define APROC_DATAPORT_DOWNMIX_TBL_BASE 0x0
#define APROC_DATAPORT_DOWNMIX_TBL_SIZE 10
#define APROC_DATAPORT_DOWNMIX_TBL_1_BASE (APROC_DATAPORT_DOWNMIX_TBL_BASE + APROC_DATAPORT_DOWNMIX_TBL_SIZE)

#define APROC_DATAPORT_FIR_SCALE_SIZE 1

#define APROC_DATAPORT_SP_LSRS_FIR_TAP 128
#define APROC_DATAPORT_SP_LSRS_128FIR_BASE (APROC_DATAPORT_DOWNMIX_TBL_1_BASE + APROC_DATAPORT_DOWNMIX_TBL_SIZE)
#define APROC_DATAPORT_SP_LSRS_128FIR_SCALE_BASE (APROC_DATAPORT_SP_LSRS_128FIR_BASE+APROC_DATAPORT_SP_LSRS_FIR_TAP)
#define APROC_DATAPORT_SP_LSRS_128FIR_SIZE (APROC_DATAPORT_SP_LSRS_FIR_TAP+APROC_DATAPORT_FIR_SCALE_SIZE)

#define APROC_DATAPORT_SP_FIR_TAP 128
#define APROC_DATAPORT_SP_FIR_BASE (APROC_DATAPORT_SP_LSRS_128FIR_SCALE_BASE + APROC_DATAPORT_FIR_SCALE_SIZE)
#define APROC_DATAPORT_SP_FIR_SCALE_BASE (APROC_DATAPORT_SP_FIR_BASE+APROC_DATAPORT_SP_LSRS_FIR_TAP)
#define APROC_DATAPORT_SP_FIR_SIZE (APROC_DATAPORT_SP_FIR_TAP+APROC_DATAPORT_FIR_SCALE_SIZE)

#define APROC_DATAPORT_SAPOST_DEBUG_BASE (APROC_DATAPORT_SP_FIR_SCALE_BASE+APROC_DATAPORT_FIR_SCALE_SIZE)
#define APROC_DATAPORT_SAPOST_DEBUG_SIZE (64)
#define SAPOST_DEBUG_SZ (APROC_DATAPORT_SAPOST_DEBUG_SIZE*4)
#endif
#define APROC_DATAPORT_DBXTV_PARAM_BASE (APROC_DATAPORT_SAPOST_DEBUG_BASE+SAPOST_DEBUG_SZ)
#define APROC_DATAPORT_DBXTV_PARAM_SIZE (95)
#define APROC_DATAPORT_DBXTV_COEFF_BASE (APROC_DATAPORT_DBXTV_PARAM_BASE+APROC_DATAPORT_DBXTV_PARAM_SIZE)
#define APROC_DATAPORT_DBXTV_COEFF_SIZE (100)


//#define APROC_DATAPORT_DBXTV_ENABLE_BASE (APROC_DATAPORT_DBXTV_COEFF_BASE+APROC_DATAPORT_DBXTV_COEFF_SIZE)
//#define APROC_DATAPORT_DBXTV_ENABLE_SIZE (1)

#define APROC_DATAPORT_TS3D_PARAM_BASE (APROC_DATAPORT_DBXTV_COEFF_BASE+APROC_DATAPORT_DBXTV_COEFF_SIZE)
#define APROC_DATAPORT_TS3D_PARAM_SIZE (85)

typedef enum {
    APROC_ROUTINE_ID_ER_SP_PATH = 0x0,
    APROC_ROUTINE_ID_ER_SP_LSRS_PATH,
    APROC_ROUTINE_ID_ER_HP_PATH,
    APROC_ROUTINE_ID_DR_SP_PATH,
    APROC_ROUTINE_ID_DR_HP_PATH,
    APROC_ROUTINE_ID_DR_SPDIF_PCM_PATH,
    APROC_ROUTINE_ID_DR_SPDIF_SOUNDBAR_PATH,
    APROC_ROUTINE_ID_CR_DMIX0_PATH,
    APROC_ROUTINE_ID_CR_DMIX1_PATH,
    APROC_ROUTINE_ID_CR_DMIX2_PATH,
    APROC_ROUTINE_ID_AER_AMIXER0_PATH,
    APROC_ROUTINE_ID_AER_AMIXER1_PATH,
    APROC_ROUTINE_ID_AER_AMIXER2_PATH,
#if defined(CC_AUD_UNICORN_CUST) || defined(CC_AUD_SAKURA_RENDER)
    APROC_ROUTINE_ID_DR_SP_LSRS_PATH,
    APROC_ROUTINE_ID_DR_MAIN_SNPT_PATH,
    APROC_ROUTINE_ID_DR_AUDIO_OUT_PATH,
    APROC_ROUTINE_ID_DR_MAIN_SPDIF_PCM_PATH,
    APROC_ROUTINE_ID_DR_MAIN_SCART_PATH,     /* Newly add for SCART path */
#ifdef CC_MT5895
    APROC_ROUTINE_ID_DR_EARC_PCM_PATH,
#endif
#endif // CC_AUD_UNICORN_CUST
    APROC_ROUTINE_ID_DR_RAW_ENC_PATH,
    APROC_ROUTINE_ID_NUM
} APROC_ROUTINE_ID_T;


#define Aproc_COMM_SIG_INT           0x1
#ifdef CC_MT5895
#define Aproc_Comm_Siganl() ACPU_MCUSYS_WRITE32(APROC_M2AINT, ACPU_MCUSYS_READ32(APROC_M2AINT) \
        | Aproc_COMM_SIG_INT)
#else
#define Aproc_Comm_Siganl() vAprocBIM_write(APROC_M2AINT, u4AprocBIM_read(APROC_M2AINT) | Aproc_COMM_SIG_INT)
#endif // CC_MT5895
#define u4Aproc_Comm_Signal_Is_Buzy() (u4AprocBIM_read(APROC_M2AINT) & Aproc_COMM_SIG_INT)

#define Aproc_COMM_SIG_INT1           0x2
#ifdef CC_MT5895
#define Aproc_Comm_Siganl_Cmd() ACPU_MCUSYS_WRITE32(APROC_M2AINT, ACPU_MCUSYS_READ32(APROC_M2AINT) \
        | Aproc_COMM_SIG_INT1)
#else
#define Aproc_Comm_Siganl_Cmd() vAprocBIM_write(APROC_M2AINT, u4AprocBIM_read(APROC_M2AINT) | Aproc_COMM_SIG_INT1)
#endif // CC_MT5895
#define u4Aproc_Comm_Signal_Is_Buzy_Cmd() (u4AprocBIM_read(APROC_M2AINT) & Aproc_COMM_SIG_INT1)
typedef void (*AUD_APROC_FUNC_T)(void);
typedef void (*AUD_APROC_FUNC_PARAM_T)(UINT32);

typedef struct {
    UINT32 u4Idx;
    UINT32 u4Cmd;
    UINT32 u4Data0;
    UINT32 u4Data1;
    AUD_APROC_FUNC_T u4Func;
} _AUD_APROC_COMM_SIG_T;

#define Aproc_Sig_A2M_Is_Hit(a,b) (((a) && (a == b)) || (a == 0))
#define u4AudAprocSig_GetCMD(a) ((a) & APROC_SIG_CMD_MSK)
#define u4AudAprocSig_GetIdx(a) (((a) >> APROC_SIG_IDX_SHIFT) & APROC_SIG_IDX_MSK)

typedef enum {
    APROC_CONTROL_TYPE_SGAIN = 0,
    APROC_CONTROL_TYPE_VOL,
    APROC_CONTROL_TYPE_MUTE,
    APROC_CONTROL_TYPE_TRIM,
    APROC_CONTROL_TYPE_BALANCE,
    APROC_CONTROL_TYPE_BASSTREBLE,
    APROC_CONTROL_TYPE_MVS,
    APROC_CONTROL_TYPE_MVB,
    APROC_CONTROL_TYPE_DELAY,
    APROC_CONTROL_TYPE_DOWNMIX,
    APROC_CONTROL_TYPE_IEC,
    APROC_CONTROL_TYPE_DUALMONO,
    APROC_CONTROL_TYPE_VS,
    APROC_CONTROL_TYPE_AD,
    APROC_CONTROL_TYPE_MONOMIX,
    APROC_CONTROL_TYPE_FIR,
    APROC_CONTROL_TYPE_SEL,
    APROC_CONTROL_TYPE_CHSEL,
    APROC_CONTROL_TYPE_DRC,
    APROC_CONTROL_TYPE_AVC,
    APROC_CONTROL_TYPE_HPF,
    APROC_CONTROL_TYPE_SW, /*Aproc SW Channel*/
    APROC_CONTROL_TYPE_SILENCE,
    APROC_CONTROL_TYPE_EQ,
    APROC_CONTROL_TYPE_LIMITER,
    APROC_CONTROL_TYPE_BMANG, /*Aproc Bmang*/
    APROC_CONTROL_TYPE_PEQ, /*Aproc PEQ*/
    APROC_CONTROL_TYPE_SPEED,
    APROC_CONTROL_TYPE_DBG,
    APRIC_CONTROL_TYPE_BYPASS_SE,
    APROC_CONTROL_TYPE_INPUT_MIX,
    APROC_CONTROL_TYPE_32EQ,
#ifdef CC_MT5895
    APROC_CONTROL_TYPE_EARC,
#endif
    APROC_CONTROL_TYPE_NUM
} APROC_CONTROL_TYPE_T;

typedef enum {
    APROC_CONTROL_CH_L = 0,
    APROC_CONTROL_CH_R,
    APROC_CONTROL_CH_NUM
} APROC_CONTROL_CH_T;

typedef enum {
    APROC_DSP_DRCV20_ID_AVC0 = 0,
    APROC_DSP_DRCV20_ID_AVC1,
    APROC_DSP_DRCV20_ID_NUM
} APROC_DSP_DRCV20_ID_T;

typedef enum {
    DSP_DRCV20_ID_SP_AVC_0 = 0x0,
    DSP_DRCV20_ID_SP_AVC_1,
    DSP_DRCV20_ID_SP_AVC_2,
    DSP_DRCV20_ID_SP_LSRS_AVC_0,
    DSP_DRCV20_ID_SP_LSRS_AVC_1,
    DSP_DRCV20_ID_SP_LSRS_AVC_2,
    DSP_DRCV20_ID_HP_AVC_0,
    DSP_DRCV20_ID_HP_AVC_1,
    DSP_DRCV20_ID_HP_AVC_2,
    DSP_DRCV20_ID_SP_AVC_LIMITER,
    DSP_DRCV20_ID_HP_AVC_LIMITER,
    DSP_DRCV20_ID_SP_LIMITER,
    DSP_DRCV20_ID_SP_LSRS_LIMITER,
    DSP_DRCV20_ID_SPSW_LIMITER,
    DSP_DRCV20_ID_HP_LIMITER,
    DSP_DRCV20_ID_IEC_LIMITER,
    DSP_DRCV20_ID_NUM
} DSP_DRCV20_ID_T;


typedef enum {
    APROC_DSP_BIQ_ID_BASS_SP = 0,
    APROC_DSP_BIQ_ID_BASS_SP_LSRS,
    APROC_DSP_BIQ_ID_BASS_HP,
    APROC_DSP_BIQ_ID_TREBLE_SP,
    APROC_DSP_BIQ_ID_TREBLE_SP_LSRS,
    APROC_DSP_BIQ_ID_TREBLE_HP,
    APROC_DSP_BIQ_ID_BASS_TREBLE_FLAG_SP,
    APROC_DSP_BIQ_ID_BASS_TREBLE_FLAG_SP_LSRS,
    APROC_DSP_BIQ_ID_BASS_TREBLE_FLAG_HP,
    APROC_DSP_BIQ_ID_BASSTREBLE_NUM
} APROC_DSP_BIQ_ID_BASSTREBLE_T;

typedef enum {
    APROC_DSP_BIQ_ID_HPF_SP = 0,
    APROC_DSP_BIQ_ID_LPF_SP_LSRS,
    APROC_DSP_BIQ_ID_CHSEL_MODE,
    APROC_DSP_BIQ_ID_CHSEL_NUM
} APROC_DSP_BIQ_ID_CHSEL_T;

typedef enum {
    APROC_DSP_BIQ_ID_3DRC_0 = 0,
    APROC_DSP_BIQ_ID_3DRC_1,
    APROC_DSP_BIQ_ID_3DRC_2_1,
    APROC_DSP_BIQ_ID_3DRC_2_2,
    APROC_DSP_BIQ_ID_3DRC_TARGET0,
    APROC_DSP_BIQ_ID_3DRC_TARGET1,
    APROC_DSP_BIQ_ID_3DRC_TARGET2,
    APROC_DSP_BIQ_ID_3DRC_FLAG,
    APROC_DSP_BIQ_ID_1DRC_0,
    APROC_DSP_BIQ_ID_1DRC_TARGET0,
    APROC_DSP_BIQ_ID_1DRC_FLAG,
    APROC_DSP_BIQ_ID_DRC_NUM
} APROC_DSP_BIQ_ID_DRC_T;

/* IO control types
*/

#define APROC_IOCTRL_RET_OK 0
#define APROC_IOCTRL_RET_FAIL 1


#define APROC_IOCTRL_NO_ROUTINE 0xFFFFFFFF

// Smooth gain
typedef enum {
    APROC_IOCTRL_SGAIN_DSP_PERIOD = 0,
    APROC_IOCTRL_SGAIN_AMIXER_PERIOD,
    APROC_IOCTRL_SGAIN_MUTE_AMIXER0_PERIOD,
    APROC_IOCTRL_SGAIN_MUTE_AMIXER1_PERIOD,
    APROC_IOCTRL_SGAIN_MUTE_AMIXER2_PERIOD,
    APROC_IOCTRL_SGAIN_NUM
} APROC_IOCTRL_SGAIN_T;

// Volume
typedef enum {
    APROC_IOCTRL_VOL_SP = 0,
    APROC_IOCTRL_VOL_SP_LSRS,
    APROC_IOCTRL_VOL_SPSW,
    APROC_IOCTRL_VOL_HP,
    APROC_IOCTRL_VOL_MON,
    APROC_IOCTRL_VOL_IEC,
    APROC_IOCTRL_VOL_SCART,
    APROC_IOCTRL_VOL_BT,
    APROC_IOCTRL_VOL_AMIXER0,
    APROC_IOCTRL_VOL_AMIXER1,
    APROC_IOCTRL_VOL_AMIXER2, // 10
    APROC_IOCTRL_VOL_AMIXER2_AD,// 11
    APROC_IOCTRL_VOL_NUM
} APROC_IOCTRL_VOL_T;

// Trim
typedef enum {
    APROC_IOCTR_TRIM_INPUT0 = 0,
    APROC_IOCTR_TRIM_INPUT1,
    APROC_IOCTR_TRIM_INPUT2,
    APROC_IOCTR_TRIM_INPUT2_AD,
    APROC_IOCTR_TRIM_SP,
    APROC_IOCTR_TRIM_SPLSRS,
    APROC_IOCTR_TRIM_SPSW,
    APROC_IOCTR_TRIM_HP,
    APROC_IOCTR_TRIM_MON,
    APROC_IOCTR_TRIM_IEC,
    APROC_IOCTR_TRIM_SCART,
    APROC_IOCTR_TRIM_BT,
    APROC_IOCTR_TRIM_AMIXER0,
    APROC_IOCTR_TRIM_AMIXER1,
    APROC_IOCTR_TRIM_AMIXER2,
    APROC_IOCTR_TRIM_AMIXER2_AD,
    APROC_IOCTR_TRIM_DELAY0,
    APROC_IOCTR_TRIM_DELAY1,
    APROC_IOCTR_TRIM_MIXSND0,
    APROC_IOCTR_TRIM_MIXSND1,
    APROC_IOCTR_TRIM_MIXSND2,
    APROC_IOCTR_TRIM_MIXSND3,
    APROC_IOCTR_TRIM_NUM
} APROC_IOCTR_TRIM_T;

// Mute
typedef enum {
    APROC_IOCTR_MUTE_SRC0 = 0,
    APROC_IOCTR_MUTE_SRC1,
    APROC_IOCTR_MUTE_SRC2,
    APROC_IOCTR_MUTE_PCMR,
    APROC_IOCTR_MUTE_NUM
} APROC_IOCTR_MUTE_T;

// Mute
typedef enum {
    APROC_IOCTR_BALANCE_SP = 0,
    APROC_IOCTR_BALANCE_HP,
    APROC_IOCTR_BALANCE_IEC,
    APROC_IOCTR_BALANCE_NUM
} APROC_IOCTR_BALANCE_T;


// Dual mono
typedef enum {
    APROC_IOCTRL_DUALMONO_AMXIER0_FLAG = 0,
    APROC_IOCTRL_DUALMONO_AMXIER1_FLAG,
    APROC_IOCTRL_DUALMONO_AMXIER2_FLAG,
    APROC_IOCTRL_DUALMONO_AMXIER0_MODE,
    APROC_IOCTRL_DUALMONO_AMXIER1_MODE,
    APROC_IOCTRL_DUALMONO_AMXIER2_MODE,
    APROC_IOCTRL_DUALMONO_NUM
} APROC_IOCTRL_DUALMONO_T;

// TSM
typedef enum {
    APROC_IOCTRL_TSM_AMIXER0_FLAG = 0,
    APROC_IOCTRL_TSM_AMIXER1_FLAG,
    APROC_IOCTRL_TSM_AMIXER0_SPEED,
    APROC_IOCTRL_TSM_AMIXER1_SPEED,
    APROC_IOCTRL_TSM_NUM
} APROC_IOCTRL_TSM_T;

// IEC
typedef enum {
    APROC_IOCTRL_IEC_FLAG = 0,
    APROC_IOCTRL_IEC_MODE,
    APROC_IOCTRL_IEC_COPYPROTECT,
    APROC_IOCTRL_IEC_MUTE,
    APROC_IOCTRL_IEC_CHANNELSTATUS,
    APROC_IOCTRL_IEC_RAWINFO,
    APROC_IOCTRL_IEC_RAWDELAY,
    APROC_IOCTRL_IEC_PCMCHSEL,
    APROC_IOCTRL_IEC_PCMBITS,
    APROC_IOCTRL_IEC_OUTPUT,
    APROC_IOCTRL_IEC_RAWAUDFMT,
    APROC_IOCTRL_IEC_NUM
} APROC_IOCTRL_IEC_T;

#ifdef CC_MT5895
// eARC
typedef enum {
    APROC_IOCTRL_EARC_AOUT_PCM = 0,
    APROC_IOCTRL_EARC_AOUT_RAW,
    APROC_IOCTRL_EARC_NUM
} APROC_IOCTRL_EARC_T;
#endif

 typedef enum {
    APROC_IOCTRL_SP = 0,
    APROC_IOCTRL_LSRS,
    APROC_IOCTRL_SW,
    APROC_IOCTRL_HP,
    APROC_IOCTRL_IEC,
    APROC_IOCTRL_SCART,
} APROC_IOCTRL_T;

typedef enum {
    APROC_MEMMAP_WORKBUF = 0,
    APROC_MEMMAP_WORKBUF_PHY,
    APROC_MEMMAP_REGMAP,
    APROC_MEMMAP_DATAPORT,
    APROC_MEMMAP_ADSP_DRAM,
    APROC_MEMMAP_ADSP_CMPT,
    APROC_MEMMAP_AQDATA,
    APROC_MEMMAP_SIZE
} _APROC_MEMMAP_T;

// Speed
typedef enum {
    APROC_IOCTR_SPEED_AMIXER0 = 0,
    APROC_IOCTR_SPEED_AMIXER1,
    APROC_IOCTR_SPEED_AMIXER2,
    APROC_IOCTR_SPEED_NUM
} APROC_IOCTR_SPEED_T;

typedef enum {
    APROC_IOCTR_INPUT0 = 0,
    APROC_IOCTR_INPUT1,
    APROC_IOCTR_INPUT2,
    APROC_IOCTR_INPUT_NUM
} APROC_IOCTR_INPUT_T;

typedef enum {
    AVSYNC_CMD_NON = 0,
    AVSYNC_CMD_WAIT_ON,
    AVSYNC_CMD_WAIT_OFF,
    AVSYNC_CMD_WAIT_OFFON,
    AVSYNC_CMD_PAUSE,//user pause command
    AVSYNC_CMD_SKIP
} AVSYNC_CMD_T;


typedef enum
{
    APROC_AVSYNC_CTRL_WORK,
    APROC_AVSYNC_CTRL_HOLD,
    APROC_AVSYNC_CTRL_HOLD_COMPLETE,
}   APROC_AVSYNC_CTRL_MODE_T;

typedef enum
{
    APROC_AMX0_ONLY_DO_SKIP_IN_AMX = 1,
    APROC_AMX1_ONLY_DO_SKIP_IN_AMX = 2,
    APROC_AMX2_ONLY_DO_SKIP_IN_AMX = 4,
}   APROC_AMX_SKIP_CTRL_MODE_T;

typedef struct {
    UINT32 u4Type;
    UINT32 u4Fc;
    UINT32 u4Q;
    INT32 i4Gain;
} APROC_DSP_BIQ_INFO_T;

typedef struct {
    UINT32 u4TargetLevel;
    UINT32 u4Attack;
    UINT32 u4Release;
    UINT32 u4Max_Expand;
    UINT32 u4Flag;
} APROC_DSP_AVC_INFO_T;
typedef enum {
    APROC_BIQUAD_TYPE_LPF = 0,
    APROC_BIQUAD_TYPE_HPF,
    APROC_BIQUAD_TYPE_PEAKINGEQ,
    APROC_BIQUAD_TYPE_LOWSHELF,
    APROC_BIQUAD_TYPE_HIGHSHELF,
    APROC_BIQUAD_TYPE_BPF,
    APROC_BIQUAD_TYPE_APF,
    APROC_BIQUAD_TYPE_LPF_HI = 0x100,
    APROC_BIQUAD_TYPE_HPF_HI,
    APROC_BIQUAD_TYPE_PEAKINGEQ_HI,
    APROC_BIQUAD_TYPE_LOWSHELF_HI,
    APROC_BIQUAD_TYPE_HIGHSHELF_HI,
    APROC_BIQUAD_TYPE_BPF_HI,
    APROC_BIQUAD_TYPE_APF_HI,
    APROC_BIQUAD_TYPE_NUM
} APROC_BIQUAD_TYPE_T;

#define ADSP_BIQ_DB(dB) ((18 + (dB)) * 20)
#define APROC_BIQUAD_ONE 0x3fffff
#define APROC_BIQUAD_FS 48000

#define APROC_BIQUAD_BIT_SHIFT 7
#define APROC_BIQUAD_BIT_SHIFT1 7

#define ADSP_BIQ_DIV(a,b) ((INT32) ((INT64)((INT64) (a) << 23) / (b)))

#define APROC_ADSP_IOBUF_DEC_END (8)
#define APROC_ADSP_IOBUF_DEC_SI_END (8)

/* ADSP Decoder IO buffer define
*/
//#if 0
#if !defined(CC_AUD_ARM_RENDER)
#define APROC_ADSP_IOBUF_DEC0_START (4*0x64000) // bytes offset
#define APROC_ADSP_IOBUF_DEC0_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC0_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC0_CH (8)
#define APROC_ADSP_IOBUF_DEC0_SI_BASE (4*0x2000) // bytes offset

#define APROC_ADSP_IOBUF_DEC1_START (4*0x70000) // bytes offset
#define APROC_ADSP_IOBUF_DEC1_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC1_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC1_CH (8)
#define APROC_ADSP_IOBUF_DEC1_SI_BASE (4*0x2400) // bytes offset

#define APROC_ADSP_IOBUF_DEC2_START (4*0x7C000) // bytes offset
#define APROC_ADSP_IOBUF_DEC2_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC2_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC2_CH (8)
#define APROC_ADSP_IOBUF_DEC2_SI_BASE (4*0x2800) // bytes offset
#else
#define APROC_ADSP_IOBUF_DEC0_START (4*0x49C00)//Shrink DSP(4*0x80C00) // bytes offset
#define APROC_ADSP_IOBUF_DEC0_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC0_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC0_CH (8)
#define APROC_ADSP_IOBUF_DEC0_SI_BASE (4*0x3000) // bytes offset
#ifdef CC_AUD_CPU_DEC
#define APROC_ADSP_IOBUF_DEC0_SI_END APROC_ADSP_IOBUF_DEC_SI_END
//#define APROC_ADSP_IOBUF_DEC0_SI_SIZE (32*4)
#define APROC_ADSP_IOBUF_DEC0_SI_SIZE (128*4)
//eov related
#define IO_CPU_DEC_EVO_BLOCK_SIZE 3070 // byte in one block
#define IO_CPU_DEC_EVO_BLOCK_NUM 12 // blocks in buffer
#define IO_CPU_DEC_EVO_SIZE (IO_CPU_DEC_EVO_BLOCK_SIZE * IO_CPU_DEC_EVO_BLOCK_NUM) // total sample number
#endif // CC_AUD_CPU_DEC

#define APROC_ADSP_IOBUF_DEC1_START (4*0x55C00)//Shrink DSP (4*0x8CC00) // bytes offset
#define APROC_ADSP_IOBUF_DEC1_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC1_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC1_CH (8)
#define APROC_ADSP_IOBUF_DEC1_SI_BASE (4*0x3400) // bytes offset
#ifdef CC_AUD_CPU_DEC
#define APROC_ADSP_IOBUF_DEC1_SI_END APROC_ADSP_IOBUF_DEC_SI_END
//#define APROC_ADSP_IOBUF_DEC1_SI_SIZE (32*4)
#define APROC_ADSP_IOBUF_DEC1_SI_SIZE (128*4)
#endif // CC_AUD_CPU_DEC

#define APROC_ADSP_IOBUF_DEC2_START (4*0x61C00)//Shrink DSP (4*0x98C00) // bytes offset
#define APROC_ADSP_IOBUF_DEC2_END APROC_ADSP_IOBUF_DEC_END
#define APROC_ADSP_IOBUF_DEC2_SIZE (4*1024) // bytes
#define APROC_ADSP_IOBUF_DEC2_CH (8)
#define APROC_ADSP_IOBUF_DEC2_SI_BASE (4*0x3800) // bytes offset
#ifdef CC_AUD_CPU_DEC
#define APROC_ADSP_IOBUF_DEC2_SI_END APROC_ADSP_IOBUF_DEC_SI_END
//#define APROC_ADSP_IOBUF_DEC2_SI_SIZE (32*4)
#define APROC_ADSP_IOBUF_DEC2_SI_SIZE (128*4)
#endif // CC_AUD_CPU_DEC

#define APROC_ADSP_IOBUF_INTBUF4_START (4*0x38A80)//Shrink DSP (4*0x5e520) // bytes offset, paul record
#define APROC_ADSP_IOBUF_INTBUF4_END (36) // 0x6d0/0x30 = 0x24
#define APROC_ADSP_IOBUF_INTBUF4_CH (2)
#define APROC_ADSP_IOBUF_INTBUF4_SIZE (4 * 64 * 3 / 4) // bytes

#endif

/* IEC
*/
typedef struct {
    UINT32 u4CategoryCode; // category code
    UINT32 u4PcmRawInfo; // PCM/Raw Info
    UINT32 u4WordLength; // word length
    UINT32 u4SampleRate; // sample rate
    UINT32 u4AdditionalInfo; // additional information
    UINT32 u4CopyRight; // copyright
    UINT32 u4DigitalInfo; // digital
    UINT32 u4SourceNum; // source number
    UINT32 u4ChannelNum0; // channel number 0
    UINT32 u4ChannelNum1; // channel number 1
    UINT32 u4ClockAccuracy; // clock accuracy
} APROC_IEC_CHANNELSTATUS_T;

#ifdef CC_AUD_ARM_RENDER
typedef struct {
    AUD_DEC_STREAM_FROM_T _aeInSrc[AUD_DEC_NUM];
    AUD_DEC_CMD_T eCmd;
} APROC_MW_CTRL_T;
#endif

#define AUD_DBG_APROC_PTS           ((UINT16) (1<<(UINT16)0))
#define AUD_DBG_APROC_UPDATE_STC    ((UINT16) (1<<(UINT16)1))

#endif // defined(CC_AUD_ARM_SUPPORT) //#A0002
/*Bass Management*/
typedef enum{
    BMANG_SPKCFG_DISABLE=0,
    BMANG_SPKCFG_ENABLE=1,
}BMANG_SPKCFG_T;

typedef enum{
    BMANG_CHCFG_DISABLE=0,
    BMANG_CHCFG_ENABLE=1,
}BMANG_CHCFG_T;

typedef enum{
    BMANG_MODE_DISABLE=0,
    BMANG_MODE_ENABLE=1,
    BMANG_MODE_BYPASS_IIR=2,
}BMANG_MODE_T;

typedef enum{
    BMANG_SPKSZ_SMALL=0,
    BMANG_SPKSZ_LARGE=1,
    BMANG_SPKSZ_UNKNOWN,
}BMANG_SPKSZ_T;

typedef enum{
    BMANG_CHID_DUALMONO,
    BMANG_CHID_100,
    BMANG_CHID_200,
    BMANG_CHID_201,
    BMANG_CHID_210,
    BMANG_CHID_220,
    BMANG_CHID_221,
    BMANG_CHID_300,
    BMANG_CHID_301,
    BMANG_CHID_310,
    BMANG_CHID_320,
    BMANG_CHID_321,
    BMANG_CHID_UNKNOWN,
}BMANG_CHID_T;

typedef enum {
    BMANG_L_SPK = 0x0,
    BMANG_R_SPK,
    BMANG_C_SPK,
    BMANG_LS_SPK,
    BMANG_RS_SPK,
    BMANG_LFE_SPK,
    BMANG_SPK_NUM
} APROC_BMANG_SPK_T;

typedef struct {
    BMANG_MODE_T CtrlFlag;
    BMANG_CHID_T ChCfg;
    UINT32 u4LFETrim;
    UINT32 u4Volume;
    BMANG_SPKSZ_T sSpkSize[BMANG_SPK_NUM];
    UINT32 u4CutFc;
    UINT32 u4LFE2CH;
} APROC_DSP_BMANG_SET_T;
/*Bass Management End*/


typedef enum
{
    ADSP_BIN_MAIN,
//#ifndef CC_AUD_DOLBY_SUPPORT_MS12B
    ADSP_BIN_DYNLD_AAC,
    ADSP_BIN_DYNLD_DD,
    ADSP_BIN_DYNLD_DDC,
    ADSP_BIN_DYNLD_DDT,
    ADSP_BIN_DYNLD_DCV,
    ADSP_BIN_DYNLD_DDCO,
//#endif // CC_AUD_DOLBY_SUPPORT_MS12B
    ADSP2_BIN_MAIN,
    ADSP_BIN_AARM,
    ADSP_BIN_NUM
}   ADSP_BIN_T;

#ifdef ADSP_BIN_SUPPORT
typedef struct
{
    UPTR u4BinAddr;//Virtual address of dsp binary without header
    UINT32 u4BinSize;
    CHAR    acName[32];
} ADSP_BIN_CFG_T;
#endif
#ifdef CC_AUD_UNICORN_MS11_SUPPORT
#ifdef CC_AUD_DOLBY_SUPPORT_MS12
typedef enum
{
    APROC_MS12_BUF_MC1 = 0,
    APROC_MS12_BUF_MC2,
    APROC_MS12_BUF_DMX,
    APROC_MS12_BUF_RAWMC,
} APROC_MS12_BUF_SELECT_T;
#endif
#endif

typedef enum{
    E_CUST_AUD_NO_EVEMT=0,
    E_CUST_AUD_SOURCE_CHANGE,            //bPath
    E_CUST_AUD_FORMAT_CHANGE,            //bPath
    E_CUST_AUD_DELAY_CHANGE,
    E_CUST_AUD_VOLUME_CHANGE,            //It must be issued once the UI value is ready after the boot up.
    E_CUST_AUD_OUT_FORMAT_PRE_CHANGE,
    E_CUST_AUD_OUT_FORMAT_CHANGE,
    E_CUST_AUD_SOUND_SETTING_CHANGE,     //It must be issued once the UI value is ready after the boot up.
    E_CUST_AUD_CH_MUTE_STATE_CHANGE,     //bPath
    E_CUST_AUD_PLAY_STATE_CHANGE,        //bPath
    E_CUST_AUD_MIX_STATE_CHANGE,
    E_CUST_AUD_SERVICE_STATE_CHANGE,
    E_CUST_AUD_SUBWOOFER_STATE_CHANGE,
    E_CUST_AUD_MCK_STATE_CHANGE,
    //Request event
    E_CUST_AUD_SPEAKER_MUTE_REQUEST,
    E_CUST_AUD_SPEAKER_UNMUTE_REQUEST,
    E_CUST_AUD_INITIALIZE,
    E_CUST_AUD_UPDATE,
}E_CUST_AUD_EVENT;

typedef enum{
    E_CUST_AUD_SAPOST_GEN_CONFIG=0,
    E_CUST_AUD_SAPOST_POST_CONFIG,
    E_CUST_AUD_SAPOST_STR_INFO,
    E_CUST_AUD_SAPOST_SP_MODE,
    E_CUST_AUD_SAPOST_BT_INFO,
    E_CUST_AUD_SAPOST_AUD_ITEM_CMD,
}E_CUST_AUD_SAPOST_EVENT;

//-----------------------------------------------------------------------------
// DRVCUST_SendAudEvent
//
/** unused E_CUST_AUD_EVENT Variable eCustEvent
 *
 *  @param      eCustEvent  (E_CUST_AUD_EVENT)
 *                    u1Option     (UNIT8,not used in this function)
 *
 */
//-----------------------------------------------------------------------------
extern void DRVCUST_SendSAPostEvent(E_CUST_AUD_SAPOST_EVENT eCustEvent, UINT8 u1Option);
extern int DRVCUST_SendAudEvent(E_CUST_AUD_EVENT eCustEvent, UINT8 u1Option);
#if (defined(CC_MAPLE_CUST_DRV)||defined(CC_AUD_UNICORN_CUST))
extern CUST_AUD_SOUND_SETTING_T gSoundSetting;
#if (defined(CC_AUD_SAKURA_RENDER)||defined(CC_AUD_UNICORN_CUST))
extern CUST_AUD_SOUND_SIGNAL_T gSoundSignal[AUD_DSP1+1][AUD_DEC_AUX+1];
extern CUST_AUD_FMT_T gSignalFormat;
#endif // CC_AUD_SAKURA_RENDER
extern SAPostStreamInfo gSAPostInpStrInfo;
extern SAPostConfig gSAPostConfig;
extern SAPostGeneralConfig gSAPostGenConfig;
extern SAPostSpecialModeInfo gSAPostSpModeInfo;
extern SAPostBtInfo gSAPostBtInfo;
extern SAPostAudioItemCommandRequestParam gSAPostAudItemCmd;
extern CUST_AUD_DDP_SETTING_T gDDPSetting;
extern CUST_AUD_AAC_SETTING_T gAACSetting;
#ifdef CC_MT5895
extern CUST_AUD_HRTF_SETTING_T gHRTFSetting;
#endif
#endif

typedef enum
{
    AUD_DrvCust_AQ_Set_Init = 0,
    #ifdef CC_S_PLATFORM
    AUD_DrvCust_AQ_Set_Val,
    #endif
    AUD_DrvCust_AQ_Get_VolTal = 50,
    AUD_DrvCust_AQ_Get_List,
    AUD_DrvCust_AQ_Gen_Key,
#ifdef CC_AUD_SAKURA_RENDER
    AUD_DrvCust_AQ_Get_SVT_SP,
    AUD_DrvCust_AQ_Get_SVT_SPW,
    AUD_DrvCust_AQ_Get_SVT_HP,
    AUD_DrvCust_AQ_Get_SVT_SPDIF,
    AUD_DrvCust_AQ_Get_SVT_BT,
    AUD_DrvCust_AQ_Get_SVT_EC,
    AUD_DrvCust_AQ_Get_FIR,
    AUD_DrvCust_AQ_Get_SMT,
    AUD_DrvCust_AQ_Get_LOUDT,
    AUD_DrvCust_AQ_Get_DRCT,
    AUD_DrvCust_AQ_Get_SLT,
    AUD_DrvCust_AQ_Get_LIMT,
    AUD_DrvCust_AQ_Get_DLY,
    AUD_DrvCust_AQ_Get_ACT,
    AUD_DrvCust_AQ_Get_ADCT,
    AUD_DrvCust_AQ_Get_RSV,
    AUD_DrvCust_AQ_Get_DD_OPTIMIZER,
    AUD_DrvCust_AQ_Get_DD_REGU,
    AUD_DrvCust_AQ_Get_DD_GENERAL,
    AUD_DrvCust_AQ_Get_DD_GENERAL2,
    AUD_DrvCust_AQ_Get_DD_IEQ,
    AUD_DrvCust_AQ_Get_DD_GEQ,
    AUD_DrvCust_AQ_Get_DD_BASS_EN,
    AUD_DrvCust_AQ_Get_DD_SURR,
    AUD_DrvCust_AQ_Get_DD_VB,
    AUD_DrvCust_AQ_Get_DD_VOL_LV,
    AUD_DrvCust_AQ_Get_DD_DIALOG,
    AUD_DrvCust_AQ_Get_DA_GEN_CFG,
    AUD_DrvCust_AQ_Get_DA1_5782_VOL,
    AUD_DrvCust_AQ_Get_DA2_5782_VOL,
    AUD_DrvCust_AQ_Get_DA3_5782_VOL,
    AUD_DrvCust_AQ_Get_DA4_5782_VOL,
    AUD_DrvCust_AQ_Get_DA1_5782_BQ,
    AUD_DrvCust_AQ_Get_DA2_5782_BQ,
    AUD_DrvCust_AQ_Get_DA3_5782_BQ,
    AUD_DrvCust_AQ_Get_DA4_5782_BQ,
    AUD_DrvCust_AQ_Get_DA1_5782_DPEQ,
    AUD_DrvCust_AQ_Get_DA2_5782_DPEQ,
    AUD_DrvCust_AQ_Get_DA3_5782_DPEQ,
    AUD_DrvCust_AQ_Get_DA4_5782_DPEQ,
    AUD_DrvCust_AQ_Get_DA1_5782_DRC,
    AUD_DrvCust_AQ_Get_DA2_5782_DRC,
    AUD_DrvCust_AQ_Get_DA3_5782_DRC,
    AUD_DrvCust_AQ_Get_DA4_5782_DRC,
    AUD_DrvCust_AQ_Get_DA1_5782_GENERAL,
    AUD_DrvCust_AQ_Get_DA2_5782_GENERAL,
    AUD_DrvCust_AQ_Get_DA3_5782_GENERAL,
    AUD_DrvCust_AQ_Get_DA4_5782_GENERAL,
    AUD_DrvCust_AQ_Get_DLY2,
    AUD_DrvCust_AQ_Get_MS_RSV,

    // Driver use
    AUD_DrvCust_AQ_Get_DA_5760_SET,
#else
    AUD_DrvCust_AQ_Get_SVT_SP,
    AUD_DrvCust_AQ_Get_SVT_SPW,
    AUD_DrvCust_AQ_Get_SVT_HP,
    AUD_DrvCust_AQ_Get_SVT_LO,
    AUD_DrvCust_AQ_Get_SVT_LOH,
    AUD_DrvCust_AQ_Get_SVT_ASW,
    AUD_DrvCust_AQ_Get_SVT_DSW,
    AUD_DrvCust_AQ_Get_SVT_BT,
    AUD_DrvCust_AQ_Get_DRC,
    AUD_DrvCust_AQ_Get_LIMT,
    AUD_DrvCust_AQ_Get_SMT,
    AUD_DrvCust_AQ_Get_SPEQ,
    AUD_DrvCust_AQ_Get_SLT,
    AUD_DrvCust_AQ_Get_SWS,
    AUD_DrvCust_AQ_Get_LOUDT,
    AUD_DrvCust_AQ_Get_FIR,
    #ifdef CC_S_PLATFORM
    AUD_DrvCust_AQ_Get_DLY,
    #endif
    AUD_DrvCust_AQ_Get_ADCT,
    AUD_DrvCust_AQ_Get_ACT,
    AUD_DrvCust_AQ_Get_ADT,
    AUD_DrvCust_AQ_Get_RSV,
    AUD_DrvCust_AQ_Get_DA_GEN_CFG,
    AUD_DrvCust_AQ_Get_DA_5760_CFG1,
    AUD_DrvCust_AQ_Get_DA_5760_CFG2,
    AUD_DrvCust_AQ_Get_DA_5538_CFG,
    AUD_DrvCust_AQ_Get_DA_5538_BQ,
    AUD_DrvCust_AQ_Get_DA_5538_BT,
    AUD_DrvCust_AQ_Get_DA_5538_LOUD,
    AUD_DrvCust_AQ_Get_DA_5538_DRC,

    // Driver use
    AUD_DrvCust_AQ_Get_DA_5760_SET,
#endif //CC_AUD_SAKURA_RENDER

    AUD_DrvCust_AQ_OP_Num
} AUD_DrvCust_AQ_OP_T;  //CC_AUD_AQ_IN_DRVCUST

#if (defined(CC_AUD_SAKURA_RENDER)||defined(CC_AUD_UNICORN_CUST))
typedef struct
{
    UINT8 u1Fs;
    UINT8 u1MmMode;
    UINT8 u1TvPos;
    UINT8 u1Input;
    UINT8 u1SoundMode;
    UINT8 u1SoundBooster;
    UINT8 u1WirelessWoofer;
    UINT8 u1SurroundOnOff;
    UINT8 u1AutoOnOff;
    UINT8 u1BtConnect;
    UINT8 u1PictureMode;
    UINT8 u1ChannelConf;
} AUD_DrvCust_AQ_Key_T;
#else
typedef struct
{
    UINT8 u1Fs;
    UINT8 u1MmMode;
    UINT8 u1TvPos;
    UINT8 u1Input;
    UINT8 u1SoundMode;
    UINT8 u1SoundBooster;
    UINT8 u1WirelessWoofer;
    UINT8 u1SurroundOnOff;
    UINT8 u1AutoOnOff;
} AUD_DrvCust_AQ_Key_T;
#endif // CC_AUD_SAKURA_RENDER

#ifdef CC_AUD_DOLBY_SUPPORT_MS12
/**
 * @brief Structure holding DD(+) DMX, DRC and COMPR values of one frame.
 */
typedef struct ddp_metadata_s
{
    unsigned int b_dmx_present;                            /**< boolean indicating the presence of DMX coefficent */
    unsigned int cmixlev;                                  /**< Center mix level */
    unsigned int surmixlev;                                /**< Surround mix level */
    unsigned int b_dsurmod_present;                        /**< boolean indicating the presence of Dolby surround mode */
    unsigned int dsurmod;                                  /**< Dolby surround mode */
    unsigned int b_ext1_present;                           /**< Extended bitstream information 1 present */
    unsigned int dmixmod;                                  /**< Preferred downmix mode */
    unsigned int ltrtcmixlev;                              /**< Lt/Rt center mix level */
    unsigned int ltrtsurmixlev;                            /**< Lt/Rt surround mix level */
    unsigned int lorocmixlev;                              /**< Lo/Ro center mix level */
    unsigned int lorosurmixlev;                            /**< Lo/Ro surround mix level */
    unsigned int b_dialnorm_present[2];    /**< Program reference level (dialnorm) exists */
    int          dialnorm[2];              /**< Program reference level (dialnorm) */
    unsigned int b_dynrng_present[2][6];    /**< boolean indicating the presence of dynamic range information */
    int          dynrng[2][6];              /**< dynamic range values for each block */
    unsigned int dynrng_start_offset[2][6]; /**< start offsets of dynamic range in samples */
    unsigned int b_compr_present[2];                                   /**< boolean indicating the presence of heavy compression (RF mode) */
    int          compr[2];                                             /**< heavy compression values valid a the complete frame */
} ddp_metadata;


/**
 * @brief Structure holding (HE-)AAC DMX, DRC and COMPR values of one frame.
 *
 */
typedef struct aac_metadata_s
{
    /* DMX MPEG */
    unsigned int b_mpeg_dmx_present;                       /**< boolean indicating the presence of DMX coefficent in MPEG style */
    unsigned int mpeg_dmx_coef;                            /**< index; 2 valid bits */
    /* DVB dolbySurroundMode */
    unsigned int b_dolby_surround_present;                 /**< boolean indicating the presence of dolby surround mode indication */
    unsigned int dolby_surround_mode;                      /**< 2 valid bits: see ETSI TS 101154 Annex C5  */
    /* DMX DVB */
    unsigned int b_dvb_dmx_center_present;                 /**< boolean indicating the presence of center DMX coefficent in DVB style */
    unsigned int dvb_dmx_coef_center;                      /**< index; 3 valid bits: see ETSI TS 101 154 Annex C5 */
    unsigned int b_dvb_dmx_surround_present;               /**< boolean indicating the presence of surround DMX coefficent in DVB style */
    unsigned int dvb_dmx_coef_surround;                    /**< index; 3 valid bits: see ETSI TS 101 154 Annex C5 */
    /* program reference level */
    unsigned int b_prl_present[2];                      /**< Program reference level (dialnorm) exists */
    long         prl[2][8]; /**< Program reference level (dialnorm), given as factor to be applied, Q7.24 format */
    /* dyn range */
    unsigned int b_drc_present[2];                                   /**< boolean indicating the presence of compression information in MPEG style */
    long         drc[2][8];              /**< dynamic range values for each block, Q7.24 format */
    unsigned int drc_start_offset[2][8]; /**< start offsets of dynamic range in samples */
    /* heavy compression in DD compr bitstream format */
    unsigned int b_compr_present[2];    /**< boolean indicating the presence of compression information in DVB style */
    int          compr[2];              /**< heavy compression values valid a the complete frame, see ETSI TS 101 154 Annex C4 */
    unsigned int b_drc_presentation_mode_present[2];            /**< boolean indicating the presence of the DRC presentation mode extension: see ETSI TS 101 154 Annex C5 */
    unsigned int drc_presentation_mode[2];                      /**< coefficient indicating whether to use drc or comp in RF mode */
} aac_metadata;


/**
 * @brief Structure holding DMX, PRL and DRC values of one frame.
 *
 * The meta data sets must be available at the same time as
 * the assosiated PCM data arrives.
 * If a meta data set is still valid for the next frame it must
 * be passed again with the next frame. Due to the missmatch of frame length (AAC vs. DD) or
 * sample rate (PCMR internal vs. bit stream) more meta data sets may be passed at once.
 * The worst case in terms of number of metadata sets occurs for DD+ frames with only one audio block.
 * There up to seven metadata sets (6 frames with one audioblock each) may apply to the current output frame.
 */
typedef struct pcmr_metadata_s
{
    unsigned int metadata_type;       /**< Source of meta data: DD+, AAC, TrueHD, none (generate new meta data) */
    int          start_offset;        /**< Start sample of this meta data set */
    unsigned int acmod;               /**< DD channel mode */
    unsigned int lfeon;               /**< LFE available */
    union
    {
        ddp_metadata ddp_md;          /**< DD(+) DRC and COMPR data */
        aac_metadata aac_md;          /**< (HE-)AAC DRC and COMPR data */
    } md;
} pcmr_metadata;

//#ifdef EVO_METADATA_SUPPORT
typedef struct pcmr_metadata2_s
{
    int pcm_mixing_indicator_out;
    int n_samples_to_process;
} pcmr_metadata2;

typedef struct msf_pcm_trust_indication_s
{
    int is_leveled;
    int is_dialogue_enhanced;
    int n_samples_per_frame;
} msf_pcm_trust_indication_t;

typedef struct dap_pcm_leveled_indication_s
{
    int b_leveling_enabled;
    int b_de_enabled;
    int n_samples_to_process;
} dap_pcm_leveled_indication_t;
//#endif //EVO_METADATA_SUPPORT
/**
 * @brief Meta data types
 */
typedef enum {
    PCMR_MD_DDP  = 1,
    PCMR_MD_AAC  = 2,
} PCMR_METADATA_TYPE;
/**
 *  @brief Channel ordering equates
 */
typedef enum {
    PCMR_LEFT  =  0,    /**< left channel */
    PCMR_RGHT  =  1,    /**< right channel */
    PCMR_CNTR  =  2,    /**< center channel */
    PCMR_LFE   =  3,    /**< low frequency effects channel */
    PCMR_LSUR  =  4,    /**< left surround channel */
    PCMR_RSUR  =  5,    /**< right surround channel */
    PCMR_LSIDE =  6,    /**< side surround channel */
    PCMR_RSIDE =  7,    /**< side surround channel */
    PCMR_NONE  = -1     /**< channel not in use */
} PCMR_CHANNEL_POSITION;
typedef struct MS12_DDPENC_T
{
    // DDP Encoder Version
    UINT32 version_major;
    UINT32 version_minor;
    UINT32 version_update;
    UINT32 version_build;
    UINT32 copyright_addr;
    UINT32 scratch_mem_ptr0;                     // default: 0 (64-bit:low 32 bits)
    UINT32 scratch_mem_ptr1;                     // default: 0 (64-bit:high 32 bits)
    UINT32 scratch_mem_size;                     // default: 0
    UINT32 init_encodermode;                     // -md Encoding mode
                                                 //  5: DDP encoding(BSID 12)(default)
                                                 //  6: DD encoding(BSID 4)
                                                 //  8: DDP transcoding(BSID 12)
                                                 //  9: DD transcoding(BSID 4)
                                                 // 12: DDP low-latency encoding(BSID 16)
                                                 // 13: DD low-latency encoding(BSID 4)
    UINT32 max_chans;                            // default: DDPI_ENC_MAXCHANS
    // Mode 9: static (init-time) parameters
    UINT32 latency_offset;                       // latency offset (in samples) between metadata and PCM
    UINT32 lfe_filter_enable;                    // if 1, LFE filter enabled
    UINT32 ph90_filter_enable;                   // if 1, Phase 90 filter enabled
    UINT32 evolution_enable;                     // if 0, disable Evolution/RTLL (default enabled)
    UINT32 inpcm_alignment_enable;               // if 0, disable input PCM alignment to frame doundary
    INT32 supported_evolution_payloads;          // 32-bit bit-mask denoting payload IDs that will be passed-through
    UINT32 evo_in_offset;                        // input offset for evolution payload
    // Mode 9: dynamic (run-time) parameters
    UINT32 datarate;                             // -+d Data rate(in kbps)
                                                 // DDP and DD: 128,192,256,320,384,448,640
                                                 // DDP: 768,1024,1280,1536
                                                 // Encoding mode 8:
                                                 // acmod=0  =>         256
                                                 // acmod=1  => 128,192,256
                                                 // acmod=2  =>     192,256,320,384,448
                                                 // acmod=3  =>             320,384,448,640,768,1024
                                                 // acmod=4  =>             320,384,448,640,768,1024
                                                 // acmod=5  =>             320,384,448,640,768,1024
                                                 // acmod=6  =>             320,384,448,640,768,1024
                                                 // acmod=7  =>                 384,448,640,768,1024
                                                 // acmod=19 =>                             768,1024,1152,1280,1536
                                                 // acmod=21 =>                             768,1024,1152,1280,1536
                                                 // Encoding mode 9:
                                                 // acmod=0 =>         256
                                                 // acmod=1 => 128,192,256,384,448,640
                                                 // acmod=2 =>     192,256,384,448,640
                                                 // acmod=3 =>             384,448,640
                                                 // acmod=4 =>             384,448,640
                                                 // acmod=5 =>             384,448,640
                                                 // acmod=6 =>             384,448,640
                                                 // acmod=7 =>             384,448,640
    UINT32 acmod;                                // -a Audio coding mode
                                                 // Encoding mode 8, 9:
                                                 //  0: 1+1(L,R)
                                                 //  1: 1/0(C)
                                                 //  2: 2/0(L,R)
                                                 //  3: 3/0(L,R,C,LFE)
                                                 //  4: 2/1(L,R,LFE,S)
                                                 //  5: 3/1(L,R,C,LFE,S)
                                                 //  6: 2/2(L,R,LFE,Ls,Rs)
                                                 //  7: 3/2(L,R,CLs,Rs)
                                                 // Encoding mode 8:
                                                 // 19: 3/2+2(L,R,C,LFE,Ls,Rs,Lvh,Rvh)
                                                 // 21: 3/4(L,R,C,LFE,Ls,Rs,Lrs,Rrs)
    UINT32 lfe;                                  // -l LFE channel
                                                 // 0: Off; 1: On(default)
    UINT32 ph90_filt;                            // -9 90-degree phase shit
                                                 // 0: Off; 1: On(default)
    UINT32 dialnorm;                             // -dn Dialogue normalization in dual mono mode
                                                 // 1-31:-1dB(loudest input) through -31dB(quietest input)
    UINT32 dialnorm2;                            // -dn Dialogue normalization for second program in dual mono mode
                                                 // 1-31:-1dB(loudest input) through -31dB(quietest input)
    UINT32 dmixmod;                              // -#a Preferred stereo downmix mode
                                                 // 0: not indicated
                                                 // 1: Pro Logic downmix preferred
                                                 // 2: Stereo downmix preferred
                                                 // 3: Pro Logic II downmix preferred
    UINT32 cmixlev;                              // -cm Center mix level
                                                 // 0: 0.707(-3dB)(default)/0.707(-3dB)(default)
                                                 // 1: 0.595(-4.5dB)/0.5(-6dB)
                                                 // 2: 0.5(-6dB)/0(-infdB)
    UINT32 surmixlev;                            // -sm Surround mix level
                                                 // 0: 0.707(-3dB)(default)/0.707(-3dB)(default)
                                                 // 1: 0.595(-4.5dB)/0.5(-6dB)
                                                 // 2: 0.5(-6dB)/0(-infdB)
    UINT32 ltrtcmixlev;                          // -#d LtRt Center mix level
                                                 // 0:1.414(+3dB)/NA
                                                 // 1:1.189(+1.5dB)/NA
                                                 // 2:1(0dB)/NA
                                                 // 3:0.841(-1.5dB)
                                                 // 4:0.707(-3dB)(default)
                                                 // 5:0.595(-4.5dB)
                                                 // 6:0.5(-6dB)
                                                 // 7:0(-infdB)
    UINT32 ltrtsurmixlev;                        // -#d LtRt Surround mix level
                                                 // 0:1.414(+3dB)/NA
                                                 // 1:1.189(+1.5dB)/NA
                                                 // 2:1(0dB)/NA
                                                 // 3:0.841(-1.5dB)
                                                 // 4:0.707(-3dB)(default)
                                                 // 5:0.595(-4.5dB)
                                                 // 6:0.5(-6dB)
                                                 // 7:0(-infdB)
    UINT32 lorocmixlev;                          // -#d LoRo Center mix level
                                                 // 0:1.414(+3dB)/NA
                                                 // 1:1.189(+1.5dB)/NA
                                                 // 2:1(0dB)/NA
                                                 // 3:0.841(-1.5dB)
                                                 // 4:0.707(-3dB)(default)
                                                 // 5:0.595(-4.5dB)
                                                 // 6:0.5(-6dB)
                                                 // 7:0(-infdB)
    UINT32 lorosurmixlev;                        // -#d LoRo Surround mix level
                                                 // 0:1.414(+3dB)/NA
                                                 // 1:1.189(+1.5dB)/NA
                                                 // 2:1(0dB)/NA
                                                 // 3:0.841(-1.5dB)
                                                 // 4:0.707(-3dB)(default)
                                                 // 5:0.595(-4.5dB)
                                                 // 6:0.5(-6dB)
                                                 // 7:0(-infdB)
    UINT32 dsurexmod;                            // Surround EX mode flag
    UINT32 dheadphonmod;                         // Dolby Headphone encode flag
    UINT32 adconvtyp;                            // A/D converter type
    UINT32 mixlev;                               // -#m Audio production mixing level
                                                 // 80: 80dB SPL
                                                 // 105: 105dB SPL(default)
                                                 // 111: 111dB SPL
    UINT32 mixlev2;                              // -#m Audio production mixing level 2
                                                 // 80: 80dB SPL
                                                 // 105: 105dB SPL(default)
                                                 // 111: 111dB SPL
    UINT32 copyeightb;                           // -#n Copyright flag
                                                 // 0:Not copyrighted material; 1:Copyrighted material(default)
    UINT32 origbs;                               // -#o Original Bitstream flag
                                                 // 0:Copied; 1:Not copied(default)
    UINT32 bsmod;                                // -#p Bitstream mode
                                                 // 0: Main audio service: complete main(CM)(default)
                                                 // 1: Main audio service: music and effects(ME)
                                                 // 2: Associated audio service: vusually impaired(VI)
                                                 // 3: Associated audio service: hearing impaired(HI)
                                                 // 4: Associated audio service: dialogue(D)
                                                 // 5: Associated audio service: commentary(C)
                                                 // 6: Associated audio service: emergency(E)
                                                 // 7: Associated audio service: voice over(VO)
    UINT32 roomtyp;                              // -#r Audio production room type
                                                 // 0: Not indicated
                                                 // 1: Large room, X-curve monitor
                                                 // 2: Small room, flat monitor(default)
    UINT32 roomtyp2;                              // -#rs Audio production room type 2
                                                 // 0: Not indicated
                                                 // 1: Large room, X-curve monitor
                                                 // 2: Small room, flat monitor(default)
    UINT32 dsurmod;                              // -ds Dolby surround mode
                                                 // 0:Not indicated
                                                 // 1:Disabled
                                                 // 2:Enabled
    UINT32 loudness_rt_state;                    // default: DDPI_ENC_LOUDNESS_OFF
    UINT32 loudness_reset;                       // default: 0
    UINT32 noctets;                              // length of encoded bitstream
} APROC_MS12_DDPENC_T;
#endif // CC_AUD_DOLBY_SUPPORT_MS12
#ifdef CC_AUD_RISCAOUT_SUPPORT
typedef enum dap_cmd_type_t{
    AUD_MS12_DAP_TYPE_POSTGAIN      = 0,  /**< Invalid channel mode. */
    AUD_MS12_DAP_TYPE_VOL_BOOST,
    AUD_MS12_DAP_TYPE_CALI_BOOST,
    AUD_MS12_DAP_TYPE_LEVELER,
    AUD_MS12_DAP_TYPE_MODEN,
    AUD_MS12_DAP_TYPE_MODCALI,
    AUD_MS12_DAP_TYPE_IEQ,
    AUD_MS12_DAP_TYPE_DEENHANCER,
    AUD_MS12_DAP_TYPE_MISTEERING,
    AUD_MS12_DAP_TYPE_GEQ,
    AUD_MS12_DAP_TYPE_OPT,
    AUD_MS12_DAP_TYPE_BASSENHANCER,
    AUD_MS12_DAP_TYPE_REG,
    AUD_MS12_DAP_TYPE_VIRTUALBASS,
    AUD_MS12_DAP_TYPE_VIRTUALIZER,
    AUD_MS12_DAP_TYPE_SURDECEN,
    AUD_MS12_DAP_TYPE_MIDISABLE,
    AUD_MS12_DAP_TYPE_VBEN,
    AUD_MS12_DAP_TYPE_MODE,
    AUD_MS12_DAP_TYPE_DRC_MODE,
    AUD_MS12_DAP_TYPE_DMX_MODE,
    AUD_MS12_DAP_TYPE_OUTMODE,
    AUD_MS12_DAP_TYPE_CONNECTOR,
    AUD_MS12_DAP_TYPE_SPKOUT,
    AUD_MS12_DAP_TYPE_HPOUT,
    AUD_MS12_DAP_TYPE_SPKCFG,
    AUD_MS12_DAP_TYPE_LFEBOOSTEN,
    AUD_MS12_DAP_TYPE_LFE2LREN,
    AUD_MS12_DAP_TYPE_LFE2LRGAIN,
    AUD_MS12_DAP_TYPE_BASSEXTRAEN,
    AUD_MS12_DAP_TYPE_BASSEXTRAFREQ,
    AUD_MS12_DAP_TYPE_HEIGTHFILTERMODE,
} dap_cmd_type_t;
#endif
#ifdef CC_S_PLATFORM
extern UINT32 _gu4CurrentAqKey;
extern AUD_DrvCust_AQ_Key_T gAqKey;
#endif

#define AQ_PARAM_ANY 0xF
typedef enum {
    AQ_FS_DONT   = 0,
    AQ_FS_48KHZ  = 1,
    AQ_FS_96KHZ  = 2,
} AQ_FS;

typedef enum {
    AQ_MM_MEASURE_MODE_DONT  =0,
    AQ_MM_MEASURE_MODE_0     =1,
    AQ_MM_MEASURE_MODE_1     =2,
} AQ_MM_MEASURE_MODE;

typedef enum {
    AQ_TV_POSITION_DONT        =0,
    AQ_TV_POSITION_TABLE_STAND =1,
    AQ_TV_POSITION_WALL_MOUNT  =2,
} AQ_TV_POSITION;

typedef enum {
    AQ_INPUT_DONT=0,
    AQ_INPUT_ANALOG_RF=1,
    AQ_INPUT_DIGITAL_RF=2,
    AQ_INPUT_HDMI=3,
    AQ_INPUT_COMPOSITE=4,
    AQ_INPUT_COMPONENT=5,
    AQ_INPUT_PC=6,
    AQ_INPUT_VIDEO_APP=7,
    AQ_INPUT_PHOTO=8,
    AQ_INPUT_JELLYFISH=9,
    AQ_INPUT_SKYPE=10,
    AQ_INPUT_MIRACAST=11,
    AQ_INPUT_GENERAL_APP=12,
    AQ_INPUT_RESERVED1=13,
    AQ_INPUT_RESERVED2=14,
    AQ_INPUT_RESERVED3=15,
} AQ_INPUT;

typedef enum {
    AQ_SOUND_MODE_DONT=0,
    AQ_SOUND_MODE_STANDARD=1,
    AQ_SOUND_MODE_CINEMA=2,
    AQ_SOUND_MODE_MUSIC=3,
    AQ_SOUND_MODE_DEMO=4,
    AQ_SOUND_MODE_FOOTBALL=5,
    AQ_SOUND_MODE_NEWS=6,

#if (defined(CC_AUD_SAKURA_RENDER)||defined(CC_AUD_UNICORN_CUST))
    AQ_SOUND_MODE_STANDARD_SUR_OFF, // Surround = "Off", "Auto"( Off )
    AQ_SOUND_MODE_STANDARD_SUR_ON,  // Surround = "On", "Auto"( On )
    AQ_SOUND_MODE_DEMO_SUR_OFF,     // Surround = "Off", "Auto"( Off )
    AQ_SOUND_MODE_DEMO_SUR_ON,      // Surround = "On", "Auto"( On )
    AQ_SOUND_MODE_DEMO_S1A1,
#ifdef CC_AUD_SAKURA_RENDER
    AQ_SOUND_MODE_DOLBY_SUR_OFF,    // Surround = "Off", "Auto"( Off )
    AQ_SOUND_MODE_DOLBY_SUR_ON,     // Surround = "On", "Auto"( On )
#endif // CC_AUD_SAKURA_RENDER
#else
    AQ_SOUND_MODE_STANDARD_S0A0,
    AQ_SOUND_MODE_STANDARD_S0A1,
    AQ_SOUND_MODE_STANDARD_S1A0,
    AQ_SOUND_MODE_STANDARD_S1A1,
    AQ_SOUND_MODE_DEMO_S0A0, //10
    AQ_SOUND_MODE_DEMO_S0A1,
    AQ_SOUND_MODE_DEMO_S1A0,
    AQ_SOUND_MODE_DEMO_S1A1, //13
#endif // CC_AUD_SAKURA_RENDER
} AQ_SOUND_MODE;
#ifdef CC_AUD_SAKURA_RENDER
#define AQ_SOUND_MODE_SPORTS AQ_SOUND_MODE_FOOTBALL
#endif //CC_AUD_SAKURA_RENDER

typedef enum {
    AQ_WIRELESS_SUBWOOFER_MODE_DONT=0,
    AQ_WIRELESS_SUBWOOFER_MODE_OFF=1,//Headphone/AudioOut=(Headphone OR Line-out OR (Subwoofer AND WirelessSW dongle not connected))
    AQ_WIRELESS_SUBWOOFER_MODE_ON=2,//Headphone/AudioOut=Subwoofer AND WirelessSW dongle connected
} AQ_WIRELESS_SUBWOOFER_MODE;

typedef enum {
    AQ_SURROUND_DONT=0,
    AQ_SURROUND_OFF=1,
    AQ_SURROUND_ON=2,
    AQ_SURROUND_AUTO=3,
} AQ_SURROUND;

typedef enum {
    AQ_AUTO_VOL_DONT=0,
    AQ_AUTO_VOL_OFF=1,
    AQ_AUTO_VOL_ON=2,
} AQ_AUTO_VOL;

typedef enum {
    AQ_CHANNEL_CONF_MONO=0,
    AQ_CHANNEL_CONF_DUAL_MONO=1,
    AQ_CHANNEL_CONF_STEREO=2,
    AQ_CHANNEL_CONF_2_0_1=3,
    AQ_CHANNEL_CONF_2_1_1=4,
    AQ_CHANNEL_CONF_2_1_0=5,
    AQ_CHANNEL_CONF_2_2_0=6,
    AQ_CHANNEL_CONF_2_2_1=7,
    AQ_CHANNEL_CONF_3_0_0=8,
    AQ_CHANNEL_CONF_3_0_1=9,
    AQ_CHANNEL_CONF_3_1_0=10,
    AQ_CHANNEL_CONF_3_1_1=11,
    AQ_CHANNEL_CONF_3_2_0=12,
    AQ_CHANNEL_CONF_3_2_1=13,
} AQ_CHANNEL_CONF;

typedef enum {
    AQ_BT_AUDIO_DONT=0,
    AQ_BT_AUDIO_NOT_CONNECTED=1,
    AQ_BT_AUDIO_CONNECTED=2,
    AQ_BT_TYPE_SPEAKER,
    AQ_BT_TYPE_HEADPHONE,
} AQ_BT_AUDIO;

typedef enum {
    AQ_PICTURE_MODE_DONT=0,
    AQ_PICTURE_MODE_GAME_GRAPHICS=1,
    AQ_PICTURE_MODE_OTHER=2,
} AQ_PICTURE_MODE;

typedef enum {
    AQ_FOOTBALL_NARRATION_CANCEL_DONT=0,
    AQ_FOOTBALL_NARRATION_CANCEL_OFF=1,
    AQ_FOOTBALL_NARRATION_CANCEL_ON=2,
} AQ_FOOTBALL_NARRATION_CANCEL;

typedef enum {
    AQ_BOOSTER_ON =0,
    AQ_BOOSTER_OFF=1,
} AQ_BOOSTER_SET;

typedef enum {
    AQ_CLEAR_AUDIO_PLUS_DONT=0,
    AQ_CLEAR_AUDIO_PLUS_OFF=1,//ClearAudio+ Off
    AQ_CLEAR_AUDIO_PLUS_ON=2,//normal mode
} AQ_CLEAR_AUDIO_PLUS;

#define AQ_QRY_ANY_IDX  0x0
#define AQ_NOT_SPECIFIC 0xFFFFFFFF
extern INT32 DrvCust_AQ_SetGet(AUD_DrvCust_AQ_OP_T eType, void* arg1, void* arg2, UINT32 arg3, UINT32 arg4);
#ifdef CC_AUD_SAKURA_RENDER
extern INT32 DrvCust_AQ_SetData(UINT32 u4Group, UINT32 u4Index, UINT32 u4Offset, UINT32 u4Length, UINT8 *p1Value);
extern INT32 DrvCust_AQ_GetData(UINT32 u4Group, UINT32 u4Index, UINT32 u4Offset, UINT32 u4Length);
extern INT32 DrvCust_AQ_SetDataWord(UINT32 u4Group, UINT32 u4Index, UINT32 u4Offset, UINT32 u4Length, UINT32 *p4Value);
extern INT32 DrvCust_AQ_GetDataWord(UINT32 u4Group, UINT32 u4Index, UINT32 u4Offset, UINT32 u4Length);
extern INT32 DrvCust_AQ_Query(void);
extern UINT32 DrvCust_AQ_GetAqDebugIndex(UINT8 id);
#endif //CC_AUD_SAKURA_RENDER
#if 1//CC_AUD_UNICORN_CUST // for S  Downmix
#define SAPOST_API_VERSION_016 //added from v0.16

typedef enum {
    SAPOST_SPEAKERS_TV_SPEAKERS_TMP,
    SAPOST_SPEAKERS_AUDIO_SYSTEM_TMP,
    SAPOST_SPEAKERS_SPDIF_SOUND_BAR_TMP,
    SAPOST_SPEAKERS_BLUETOOTH_TMP,
    SAPOST_SPEAKERS_UNKNOWN_TMP,
} SAPostSpeakers_T;

typedef enum {
    SAPOST_SOUND_MODE_STANDARD_TMP,
    SAPOST_SOUND_MODE_CINEMA_TMP,
#ifdef SAPOST_API_VERSION_015
    SAPOST_SOUND_MODE_SPORTS_TMP,
    SAPOST_SOUND_MODE_LIVE_SURROUND_TMP,
#endif //SAPOST_API_VERSION_015
#ifdef SAPOST_API_VERSION_016
    SAPOST_SOUND_MODE_LIVE_FOOTBALL_TMP,
    SAPOST_SOUND_MODE_LIVE_MUSIC_TMP,
#endif //SAPOST_API_VERSION_016
#ifdef SAPOST_API_VERSION_015
    SAPOST_SOUND_MODE_GAME_TMP,
    SAPOST_SOUND_MODE_COMPRESSED_AUDIO_TMP,
#endif //SAPOST_API_VERSION_015
    SAPOST_SOUND_MODE_DEMO_TMP,
#ifdef SAPOST_API_VERSION_015
    SAPOST_SOUND_MODE_FOOTBALL_TMP,
#endif //SAPOST_API_VERSION_015
    SAPOST_SOUND_MODE_NEWS_TMP,
    SAPOST_SOUND_MODE_UNKNOWN_TMP,
} SAPostSoundMode_T;


#ifdef SAPOST_API_VERSION_016
typedef enum {
    SAPOST_SURROUND_OFF_TMP,
    SAPOST_SURROUND_ON_TMP,
    SAPOST_SURROUND_AUTO_TMP,
    SAPOST_SURROUND_UNKNOWN_TMP,
} SAPostSurround_T;
#endif //SAPOST_API_VERSION_016


#ifdef SAPOST_API_VERSION_016
typedef enum {
    SAPOST_NIGHT_MODE_OFF_TMP,
    SAPOST_NIGHT_MODE_ON_TMP,
    SAPOST_NIGHT_MODE_UNKNOWN_TMP,
} SAPostNightMode_T;
#endif //SAPOST_API_VERSION_016


typedef enum {
    SAPOST_CODEC_LPCM_TMP,
    SAPOST_CODEC_ATRAC3_TMP,
    SAPOST_CODEC_ATRACX_TMP,
    SAPOST_CODEC_WAV_TMP,
    SAPOST_CODEC_AIFF_TMP,
    SAPOST_CODEC_MPEG1_L1_TMP,
    SAPOST_CODEC_MPEG1_L2_TMP,
    SAPOST_CODEC_MPEG1_L3_TMP,
    SAPOST_CODEC_MPEG2_BC_TMP,
    SAPOST_CODEC_MPEG2_BC_MC_TMP,
    SAPOST_CODEC_MPEG2_AAC_TMP,
    SAPOST_CODEC_MPEG4_HE_AAC_TMP,
    SAPOST_CODEC_AC3_TMP,
    SAPOST_CODEC_AC3_PLUS_TMP,
    SAPOST_CODEC_WMA_TMP,
    SAPOST_CODEC_WMA_PRO_TMP,
    SAPOST_CODEC_DRA_TMP,
    SAPOST_CODEC_VORBIS_TMP,
    SAPOST_CODEC_DTS_TMP,
    SAPOST_CODEC_COOK_TMP,
    SAPOST_CODEC_SIPRO_TMP,
    SAPOST_CODEC_MU_LAW_TMP,
    SAPOST_CODEC_G711_TMP,
    SAPOST_CODEC_FLAC_TMP,
    SAPOST_CODEC_SBC_TMP,
    SAPOST_CODEC_ALAC_TMP,
    SAPOST_CODEC_DSD_TMP,
    SAPOST_CODEC_OPUS_TMP,
    SAPOST_CODEC_UNKNOWN_TMP
} SAPostCodec_T;

typedef enum {
    SAPOST_CHANNELS_MONO_TMP,
    SAPOST_CHANNELS_MONO_1_TMP,
    SAPOST_CHANNELS_DUAL_MONO_TMP,
    SAPOST_CHANNELS_DUAL_MONO_1_TMP,
    SAPOST_CHANNELS_STEREO_TMP,
    /* front + front-side + rear-side + sub-woofer */
    SAPOST_CHANNELS_2_0_0_1_TMP,
    SAPOST_CHANNELS_2_0_1_1_TMP,
    SAPOST_CHANNELS_2_0_1_0_TMP,
    SAPOST_CHANNELS_2_0_2_0_TMP,
    SAPOST_CHANNELS_2_0_2_1_TMP,
    SAPOST_CHANNELS_2_1_0_0_TMP,    // Not used
    SAPOST_CHANNELS_2_1_0_1_TMP,    // Not used
    SAPOST_CHANNELS_2_1_1_0_TMP,    // Not used
    SAPOST_CHANNELS_2_1_1_1_TMP,    // Not used
    SAPOST_CHANNELS_2_1_2_0_TMP,    // Not used
    SAPOST_CHANNELS_2_1_2_1_TMP,    // Not used
    SAPOST_CHANNELS_2_2_0_0_TMP,    // Not used
    SAPOST_CHANNELS_2_2_0_1_TMP,    // Not used
    SAPOST_CHANNELS_2_2_1_0_TMP,    // Not used
    SAPOST_CHANNELS_2_2_1_1_TMP,    // Not used
    SAPOST_CHANNELS_2_2_2_0_TMP,    // Not used
    SAPOST_CHANNELS_2_2_2_1_TMP,    // Not used
    SAPOST_CHANNELS_3_0_0_0_TMP,
    SAPOST_CHANNELS_3_0_0_1_TMP,
    SAPOST_CHANNELS_3_0_1_0_TMP,
    SAPOST_CHANNELS_3_0_1_1_TMP,
    SAPOST_CHANNELS_3_0_2_0_TMP,
    SAPOST_CHANNELS_3_0_2_1_TMP,
    SAPOST_CHANNELS_3_1_0_0_TMP,    // Not used
    SAPOST_CHANNELS_3_1_0_1_TMP,    // Not used
    SAPOST_CHANNELS_3_1_1_0_TMP,    // Not used
    SAPOST_CHANNELS_3_1_1_1_TMP,    // Not used
    SAPOST_CHANNELS_3_1_2_0_TMP,    // Not used
    SAPOST_CHANNELS_3_1_2_1_TMP,    // Not used
    SAPOST_CHANNELS_3_2_0_0_TMP,    // Not used
    SAPOST_CHANNELS_3_2_0_1_TMP,    // Not used
    SAPOST_CHANNELS_3_2_1_0_TMP,    // Not used
    SAPOST_CHANNELS_3_2_1_1_TMP,    // Not used
    SAPOST_CHANNELS_3_2_2_0_TMP,    // Not used
    SAPOST_CHANNELS_3_2_2_1_TMP,    // Not used
    SAPOST_CHANNELS_UNKNOWN_TMP
} SAPostChannels_T;


#endif
typedef enum {
    E_AUD_OMX_PATH_OFF,
    E_AUD_OMX_PATH_APROC,
    E_AUD_OMX_PATH_TRACK
}E_AUD_OMX_PATH;
#ifdef CC_AUD_CPU_DEC
// CPU decoder SI structure
typedef struct {
    // PTS
    /* DSP - "pes_info_control", APROC - "u4PtsHead", Both align@bit0
    ; bit 0 : Dec0 - The bank has PTS info.
    ; bit 1 : Dec1 - The bank has PTS info.
    ; bit 2 : Dec2 - The bank has PTS info.
    ; bit 3 :
    ; bit 4 : Dec0 - The bank is a frame head without PTS info.
    ; bit 5 : Dec1 - The bank is a frame head without PTS info.
    ; bit 6 : Dec2 - The bank is a frame head without PTS info.
    ; bit 7 :
    */
    UINT32 u4PtsHead;             // (0)
    UINT32 u4PtsLo;               // (1)
    UINT32 u4PtsHi;               // (2)
    UINT32 u4PtsFactor;           // (3)
    // Interpolation APTS
    UINT32 u4BankId;              // (4) parson to add si_info
    UINT32 u4PtsSeed;             // (5)
    UINT32 u4PtsSeedId;           // (6)
    UINT32 u4AputBank;            // (7)
	// Dec info
    UINT32 u4Fs;                  // (8)
    UINT32 speaker_config;        // (9)
    UINT32 u4BankSampleNum;       // (10)
    UINT32 u4DecChNum;            // (11) It's Channel Num (DSP send num_1, but APROC will add 1)
    UINT32 u4DecChCfg0;           // (12)
    UINT32 u4DecChCfg1;           // (13)
    // Interpolation APTS
    UINT32 u4SeedAputBank;        // (14)
    // IEC RAW
    UINT32 u4IecRawHead;          // (15)
    UINT32 u4IecRawDTS;           // (16)
    UINT32 u4IecframeSample;      // (17)
    UINT32 u4IecRawAdr;           // (18)
    UINT32 u4IecRawPd;            // (19)
    UINT32 u4IecRawPc;            // (20)
    // Decoder data
    UINT32 u4DualDec;             // (21)
#ifndef CC_AUD_DOLBY_SUPPORT_MS12
    UINT32 u4DecData[8];          // (22-29)
#else
    UINT32 u4MainAcmod;           // (22)
    UINT32 u4MainLfeOn;           // (23)
    UINT32 u4AssocAcmod;          // (24)
    UINT32 u4AssocLfeOn;          // (25)
    // MS Mixer
    INT32 i4MmdType;              // (26)
    UINT32 u4MmdData[11];         // (27-37)
    // PCM Renderer
    UINT32 u4BlockIndex;          // (38)
    UINT32 u4MdType;              // (39)
    UINT32 u4MdStartOffset;       // (40)
    UINT32 u4MdData[26];          // (41-66)
    UINT32 u4FrameIndex;          // (67) = MD_DATA_26
    UINT32 u4Is_leveled;          // (68)
    UINT32 u4Is_diaenhanced;      // (69)
    UINT32 u4Compressor_profile;  // (70)
    UINT32 u4Phase90_status;      // (71)
    UINT32 u4Compressor_profile_present;  //(72)
#endif // CC_AUD_DOLBY_SUPPORT_MS12
    UINT32 u4Errframe; // 30 // only for Main CPU
    UINT32 u4frameSize;// only for Main CPU
    UINT32 u4RawframeSize;// only for Main CPU
} AUD_CPU_DEC_SIDEINFO_T;

//#ifdef EVO_METADATA_SUPPORT
typedef struct {
    UINT32 u4evoframesize; //evo frame size
    unsigned char u4evodata[3066];
}AUD_CPU_DEC_EVOFRAM_T;

typedef struct {
    UINT32 u4evoBufsize;
    char *p4evoBuf;
}AUD_CPU_DEC_EVOBUF_T;

typedef struct {
    UINT32 u4Str;// start address
    UINT32 u4End; // total block number
    UINT32 u4Wp; // write pointer
    UINT32 u4Wcp;
    UINT32 u4Rp; // read pointer
    UINT32 u4Rcp;
    UINT32 u4EvoEnable;
    UINT32 u4Offset;  //offset of iecbuf
    UINT32 u4EvoBufsize;
    UINT32 u4Decid;
} AUD_EVO_BUFFER_T;
//#endif //EVO_METADATA_SUPPORT

typedef enum {
    AUD_CPU_DEC_IN_PLAY,
    AUD_CPU_DEC_IN_STOP,
    AUD_CPU_DEC_STATE_MAX
}AUD_CPU_DEC_STATE_T;

typedef enum {
    AUD_CPU_DEC_SET_PLAY,
    AUD_CPU_DEC_SET_STOP,
    AUD_CPU_DEC_CTRL_MAX
}AUD_CPU_DEC_CTRL_T;

typedef struct {
    UINT32 u4Str;// start address
    UINT32 u4End; // total block number
    UINT32 u4Size; // bytes in one block per channel
    UINT32 u4Ch; // channel number
    UINT32 u4Wcp; // Write port, current pointer
    UINT32 u4Rcp; // Read port, current pointer
    UINT32 u4Wp; // write pointer
    UINT32 u4Rp; // read pointer
} AUD_IO_BUFFER_T;

typedef enum {
    DTS_NATIVE = 0,
    DTS_2_0_LORO,
    DTS_2_0_LTRT,
    DTS_5_1,
    DTS_7_1,
    DTS_1_0_MONO,
    DTS_2_1,
    DTS_3_0,
    DTS_3_1,
    DTS_2_2,
    DTS_3_2,
} DTS_SPKCFG;

typedef enum {
    DTS_IEC_TRANS,
    DTS_IEC_PASS
}DTS_SPDIF_OUT;

typedef enum {
    AUD_UNINIT = 0,
    AUD_INITDONE,
    APROC_UNINIT,
    APROC_INIT_DONE,
    AUD_POST_RESUME,
    AUD_POST_AVC_DONE,
    AUD_POST_DAC_DONE,
    AUD_POST_SPKCFG_DONE,
    AUD_POST_MS12_DONE,
    AUD_POST_RESUME_DONE,
    AUD_MIXSND_RESUME,
    AUD_MIXSND_RESUME_DONE,
    AUD_UPDATE_DSP_CONFIG_DONE,
    AUD_SPOST_DONE,
} RESUME_STATE;

typedef struct CPUDEC_DRV_INFO
{
   UINT8            u1DecId;        // Decoder ID
   AUD_FMT_T        eDecFmt;       // codec format
   UINT32           u4AfifoOffSet;
   UINT32           u4PCMBufOffSet;
   UINT32           u4IECBufOffSet;
   UINT32           u4SIBufOffSet;
   UINT32           u4PCMNextChOffset;
   UINT32           u4DecParaUpdate;
}CPUDEC_DRV_INFO;


typedef struct CPUDEC_DTS_PARAMETER
{
   DTS_SPKCFG       eSpkCfg;        // Decoder ID
   UINT8            u1Drc;          // codec format
   DTS_SPDIF_OUT    eSpdifOut;
   BOOL             fgNoDiag;
   BOOL             fgLfeMix;
   BOOL                fgDecodeOnlyCore;
   BOOL                fgSpdifoutput;
   BOOL                fgPrimarydeconly;
   INT8               u1Transencode;
   INT8               u1Multdmixoutput;
}CPUDEC_DTS_PARAMETER;

#ifdef CC_AUD_DOLBY_SUPPORT_AC4
typedef struct CPUDEC_AC4_PARAMETER
{
    INT8 ac4mode;   //AC4 decoding mode:
                            //0: Single input stream, single decode
                            //1: Single input stream, dual decode, single decoder instance
                            //2: Single input stream, dual decode, dual decoder instance
                            //3: Dual input stream, dual decode (default for dual input)
    INT8 ac4de;       //Dialogue Enhancement gain that will be applied in the decoder
                            //Range: 0 to 12 dB (in 1 dB steps, default is 0 dB)
    CHAR lang[4];         //1st Preferred Language code (3 Letter ISO 639)
    CHAR lang2[4];         //2nd Preferred Language code (3 Letter ISO 639)
    INT8 at;                       //Preferred associated type of service:
                                      //1: Visually Impaired (VI) (Default)
                                      //2: Hearing Impaired (HI)
                                      //3: Commentary
    INT8 pat;                     //Prefer Presentation Selection by associated type over language:
                                      //0: Prefer selection by language
                                      //1: Prefer selection by associated type (default)
    INT32 main_pres_idx;     //Presentation index to be decoded. Overrides the presentation selection by preferred language and associated type.
                                 //0...63: Presentation index
                                 //-1:      Switch back to automatic selection by language and associated type (default)
    INT32 assoc_pres_idx;     //Presentation index to be decoded by the associated instance.
                                 //0...63: Presentation index
                                 //-1:      None. Decoder will decode the default presentation (default)
    INT32 prog_id_type;     //Determines whether the program_identifier is present or not and whether it's the short or the long version.
    CHAR program_identifier_string[2*16+1];  //Either the short program identifier(16bit) or the UUID(128bit) as a string copied from the command line.

    INT32 xa;     //MD_ID_AD_MIXING_ENABLED: AD mixing on/off, to set b_user_selected_ad_mixing :
    INT32 xu;     //MD_ID_AD_MIXING_USER_PREF: AD mixing user preference (single instance only), to set outstage_config.main_assoc_mix_pref:
}CPUDEC_AC4_PARAMETER;
#endif

typedef enum
{
    AUD_DTS_TYPE_NORMAL = 0, // DTS M6
    AUD_DTS_TYPE_EXPRESS,
    AUD_DTS_TYPE_HD
} AUD_DTS_TYPE_T;

typedef struct CPUDEC_COMMON_DEC_INFO
{
   UINT32  	ui4_sampling_rate; 	   		// sampling_rate
   UINT32	ui4_out_ch_cfg;
   UINT8	ui1_bit_depth;
   UINT32   u4atmos_present;     /* b_atmos_present */
   UINT32   u4BitRate;
   UINT32   u4SampePerFrame;
   UINT32   u4SyncFrameSizeByte;
}CPUDEC_COMMON_DEC_INFO;


typedef struct CPUDEC_AAC_DEC_INFO
{
   UINT32  	ui4_sampling_rate; 	   		// sampling_rate
   UINT32	ui4_out_ch_cfg;
   UINT32	ui4_bit_depth;
   UINT32   ui4_acmod;
   UINT32   ui4_out_ch_num;
   UINT32   ui4_sbr_enable;
   UINT32   ui4_ps_enable;
   UINT32   ui4_dualmono;
   UINT32   u4transmissionformat;// ADTS=0x0, LOAS/LATM =0x1
}CPUDEC_AAC_DEC_INFO;


typedef struct CPUDEC_DTS_DEC_INFO
{
   AUD_DTS_TYPE_T   e_dts_type;             // codec format
   UINT32  	ui4_sampling_rate; 	   		// sampling_rate
   UINT32	           ui4_out_ch_cfg;               //channel config
}CPUDEC_DTS_DEC_INFO;

typedef struct CPUDEC_AC3_DEC_INFO
{
   UINT32   u4Format;             // codec format
   UINT32   u4Acmode; 	   		// acmode
}CPUDEC_AC3_DEC_INFO;
typedef struct CPUDEC_AC4_TRUST_INDICATION
{
    int is_leveled;
    int is_dialogue_enhanced;
}CPUDEC_AC4_TRUST_INDICATION;

typedef struct CPUDEC_INFO
{
    UINT32           u4CommVer;
    UINT32           u4DTSVer;
}CPUDEC_INFO;

typedef struct DTS_CERTPARAM_MEM_T
{
    BOOL ddp; // 1. digitaldeliverypaser
    BOOL mpd; // 2. multipleassetdecode
    BOOL mdo; // 3. multidmixoutput
    BOOL ac;  // 4. AnalogComp
    BOOL l1;  // 5. l1
    BOOL nds; // 6. NoDecDefaultSpkrRemap
    BOOL doc; // 7. DecoderOnlyCore
    BOOL pd;  // 8. primarydeconly
    BOOL sdo; // 9. spdifoutput
    INT16 tc;  //10.transencode
    INT32 spo; //11. Spkout
    INT16 drc; // 12. DRCPercent
    INT16 aam; // 13. activeassetmask
    INT16 asam; // 14. ActivesecondaryAssetMask
    INT16 api; //15.AudioPresentIndex
    INT16 rs1; // 16. ReplacSet
    INT16 rs2; // 17. ReplacSet
    INT16 us; // 18. upsamplerate.
    INT16 x; // 19. audiodescpan/audiodesccenter......
}DTS_CERTPARAM_MEM_T;

typedef struct CPUDEC_CERT_INFO
{
   CHAR  Mountpath[256];
   CHAR  Filepath[256];
   CHAR  Filename[128];
   DTS_CERTPARAM_MEM_T DTSCertParam;
}CPUDEC_CERT_INFO;


typedef enum {
    DTS_CH_STATUS_CHANGE =0,
    DTS_SAMPLE_RATE_CHANGE
} CPUDEC_DTS_InfoChange;

typedef struct CPUDEC_OPUS_DEC_INFO
{
   UINT32 ui4_sample_rate;
   UINT32 ui4_data_rate;
   UINT8 ui1_bit_depth;
   UINT8 ui1_channel_count;
   UINT8 ui1_mapping_family;
}CPUDEC_OPUS_DEC_INFO;

typedef struct CPUDEC_MAT_DEC_INFO
{
   UINT32 ui4_sample_rate;
   UINT32 ui4_out_ch_cfg;
   UINT8 ui1_bit_depth;;
   UINT8 ui1_IsTrueHd;
}CPUDEC_MAT_DEC_INFO;

#endif // CC_AUD_CPU_DEC

/*Audio focus types */
typedef enum
{
    E_AUD_DEC_FOCUS_MAIN = 0,
    E_AUD_DEC_FOCUS_SUB
}   E_AUD_DEC_FOCUS_TYPE;

#ifdef CC_AUD_DOLBY_SUPPORT_DAPv2
typedef struct {
    INT32 ieq_enable;
    INT32 ieq_amount;
    INT32 ieq_nb_bands;
    INT32 a_ieq_band_center[20];
    INT32 a_ieq_band_target[20];
} DAP_IEQ_PARAM;

typedef struct {
    INT32 geq_enable;
    INT32 geq_nb_bands;
    UINT32 a_geq_band_center[20];
    INT32 a_geq_band_target[20];
} DAP_GEQ_PARAM;

typedef struct {
    INT32 optimizer_enable;
    INT32 optimizer_nb_bands;
    INT32 a_opt_band_center_freq[20];
    INT32 a_opt_band_gain[8][20];
} DAP_OPT_PARAM;

typedef struct {
    INT32 regulator_enable;
    INT32 regulator_overdrive;
    INT32 regulator_timbre;
    INT32 regulator_distortion;
    INT32 regulator_mode;
    INT32 reg_nb_bands;
    INT32 a_reg_band_center[20];
    INT32 a_reg_low_thresholds[20];
    INT32 a_reg_high_thresholds[20];
    INT32 a_reg_isolated_bands[20];
} DAP_REG_PARAM;
#endif //CC_AUD_DOLBY_SUPPORT_DAPv2

#ifdef CC_AUD_DOLBY_SUPPORT_MS12v2
typedef enum
{
    DUAL_MODE_LEFT,
    DUAL_MODE_RIGHT,
    DUAL_MODE_STEREO,
}AAC_DUAL_MODE_E;
#endif

//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

#define DEC_NUM_VALIDATE(NUM)   \
    if (NUM > AUD_DEC_NUM)      \
    {                           \
        ASSERT(0);              \
    }                           \

//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// AUD_IsDecoderPlay
//
/** Brief of AUD_IsDecoderPlay.
 *  Details of AUD_IsDecoderPlay (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval     TRUE
 *  @retval     FALSE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_IsDecoderPlay(UINT8 u1DspId, UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_IsDecoderPlay
//
/** Brief of AUD_IsDecoderRealPlay.
 *  Details of AUD_IsDecoderRealPlay (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval     TRUE
 *  @retval     FALSE
 */
//-----------------------------------------------------------------------------
BOOL AUD_IsDecoderRealPlay(UINT8 u1DspId, UINT8 u1DecId);


//-----------------------------------------------------------------------------
// AUD_IsDSPRealPlay
//
/** Brief of AUD_IsDSPRealPlay.
 *  Details of AUD_IsDSPRealPlay (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval     TRUE
 *  @retval     FALSE
 */
//-----------------------------------------------------------------------------
BOOL AUD_IsDSPRealPlay(UINT8 u1DspId, UINT8 u1DecId);

#ifdef CC_AUD_CPU_DEC
//-----------------------------------------------------------------------------
// AUD_IsCPUDecRealPlay
//
/** Brief of AUD_IsCPUDecRealPlay.
 *  Details of AUD_IsCPUDecRealPlay (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval     TRUE
 *  @retval     FALSE
 */
//-----------------------------------------------------------------------------

BOOL AUD_IsCPUDecRealPlay(UINT8 u1DecId);
#endif


//-----------------------------------------------------------------------------
// AUD_IsDecoderStop
//
/** Brief of AUD_IsDecoderStop.
 *  Details of AUD_IsDecoderStop (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval     TRUE
 *  @retval     FALSE
*/
//-----------------------------------------------------------------------------
BOOL AUD_IsDecoderStop(UINT8 u1DspId, UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_CommandDone
//
/** Audio notify command check.
 *
 *  @param  ucDecId         Decoder Id (AUD_DEC_MAIN or AUD_DEC_AUX).
 *  @param  eAudDecCmd      audio command.
 *  @param  u4Result        audio status result.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_CommandDone(UINT8 u1DspId, UINT8 u1DecId,  AUD_DEC_CMD_T eAudDecCmd, UINT32 u4Result);

//-----------------------------------------------------------------------------
// AUD_WaitCommandDone
//
/** Audio wait notify command check.
 *
 *  @param  ucDecId         Decoder Id (AUD_DEC_MAIN or AUD_DEC_AUX).
 *  @param  eAudDecCmd      audio command.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_WaitCommandDone(UINT8 u1DspId, UINT8 u1DecId, AUD_DEC_CMD_T eAudDecCmd);
//not find the definition of function AUD_DualDecWaitCommandDone
extern void AUD_DualDecWaitCommandDone(AUD_DUAL_DEC_CMD_T eDualDecCmd, BOOL fgTimeOut);
//not find the definition of function AUD_DualDecCommandDone
extern void AUD_DualDecCommandDone(AUD_DUAL_DEC_CMD_T eAudDecCmd, UINT32 u4Result);

//-----------------------------------------------------------------------------
// AUD_Init
//
/** Aud driver initialization. This function will create driver thread and let
 *  DSP go on.
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_Init(void);

//-----------------------------------------------------------------------------
// AUD_IsAudInit
//
/** Query audio initialization status.
 *
 *  @retval TRUE    Audio is initialized
 *  @retval FALSE   Audio is not initialized
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_IsAudInit(void);
//-----------------------------------------------------------------------------
// AUD_IsAudInitOK
//
/** Query if audio initialization ok.
 *
 *  @retval TRUE    Audio is initialized ok
 *  @retval FALSE   Audio is not initialized ok
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_IsAudInitOK(void);

#if 0   // Unused
//-----------------------------------------------------------------------------
// AUD_GetDecType
//
/** Get audio decoder type.
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      peStreamFrom   Stream source
 *  @param      prDecType      Stream format
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_GetDecType(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T* peStreamFrom, AUD_FMT_T* prDecType);
#endif

//-----------------------------------------------------------------------------
// AUD_GetHDMIDecodeType
//
/** Brief of AUD_GetHDMIDecodeType.
 *  Details of AUD_GetHDMIDecodeType (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      peStreamFrom     Stream source
 *  @param      peDecType        Stream format
 */
//-----------------------------------------------------------------------------
extern void AUD_GetHDMIDecodeType(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T* peStreamFrom, AUD_FMT_T* peDecType);


//-----------------------------------------------------------------------------
// AUD_SetDecMemBuffer
//
/** Brief of AUD_SetDecMemBuffer.
 *  Details of AUD_SetDecMemBuffer (optional).
 *
 *  @param      u1DecId           Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      prDecMemBuffer    Memory buffer
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_SetDecMemBuffer(UINT8 u1DspId, UINT8 u1DecId, const MEM_BUFFER_INFO_T * prDecMemBuffer);


//-----------------------------------------------------------------------------
// AUD_SetDecType
//
/** Set audio decoder type.
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      eStreamFrom    Stream source
 *  @param      eDecFmt        Stream format
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_SetDecType(UINT8 u1DspId, UINT8 u1DecId,  AUD_DEC_STREAM_FROM_T eStreamFrom, AUD_FMT_T eDecFmt);
//-----------------------------------------------------------------------------
// AUD_SetDecType2CPUDEC function seem like AUD_SetDecType()
//SetDecType To CPU DECODER
#ifdef CC_AUD_CPU_DEC
  #ifdef CC_AUD_DOLBY_SUPPORT_AC4
INT32 AUD_SetAC4Lang(char lang[4]);
extern void AUD_SetAc4DialogueEnhancerGain(UINT32 u4Gain);
extern void AUD_SetAc4PreferAssociatedType(UINT32 u4Type);
extern void AUD_SetAc4PreselectionId(UINT8 u1DecId, INT32 i4Value);
extern void AUD_SetAC4MixingUserPref(UINT8 u1DecId, INT32 u4Value);
extern void AUD_SetAc4Pat(UINT8 u1DecId, INT32 i4Value);
  #endif //CC_AUD_DOLBY_SUPPORT_AC4
extern INT32 AUD_SetDecType2CPUDEC(UINT8 u1DspId, UINT8 u1DecId,  AUD_DEC_STREAM_FROM_T eStreamFrom, AUD_FMT_T eDecFmt);
extern BOOL AUD_IsCpuDecSupportFormat(UINT8 u1DspId,UINT8 u1DecId);
extern void AUD_SetAc4AdMixEnable(UINT32 u4Value);
#endif

//-----------------------------------------------------------------------------
// AUD_SetSourceFrom
//
/** save eStreamFrom status to _arAudDecoder struct
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      eStreamFrom    Stream source
 *
 *  @retval AUD_OK    Success
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_SetSourceFrom(UINT8 u1DecId,  AUD_DEC_STREAM_FROM_T eStreamFrom);
extern INT32 AUD_SetOriDecType(UINT8 u1DecId,  AUD_DEC_STREAM_FROM_T eStreamFrom, AUD_FMT_T eDecFmt);

//-----------------------------------------------------------------------------
// _AudDumpAudioData
//
/** write data from buf to fileName,the size is count
 *
 *  @param      fileName     const *
 *  @param      buf              const *
 *  @param      count          size_t
 *
 *  @retval -1  error
 *  @retval   0  right
 */
//-----------------------------------------------------------------------------
extern INT32 _AudDumpAudioData (const INT8 *fileName, const INT8 *buf, size_t count);
//-----------------------------------------------------------------------------
// AUD_DSPCmdPlay
//
/** Brief of AUD_DSPCmdPlay.
 *  Issue play command to audio decoder, note that this is a synchronous play
 *  function
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdPlay(UINT8 u1DspId, UINT8 u1DecId);
#if defined(CC_MT5863)
//-----------------------------------------------------------------------------
// AUD_SetRegister_ADAC_A
//
/** set the ADAC Register
 *
 *  @param      void
 *
 *  @retval      void
 */
//-----------------------------------------------------------------------------
extern  void AUD_SetRegister_ADAC_A(void);
#if defined(CC_MT5863)
//-----------------------------------------------------------------------------
// AUD_SetUserMode
//
/** set the _eAudHpDePopTaskStatus
 *
 *  @param      u1UserMode
 *                    0:   AUD_HP_DEPOP_CASE_Q
 *                    1:   AUD_HP_DEPOP_CASE_O
 *                    2:   AUD_HP_DEPOP_CASE_G
 *
 *  @retval       1
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_SetUserMode(UINT32 u1UserMode);
extern  BOOL _fgHPmodeOnPre;
extern  BOOL _fgLOmodeOnPre;
#endif

#if defined(CC_MT5863)
extern  UINT32 _u4Hpvos;
extern  UINT32 _u4Sctvos;
extern  UINT32 _u4Lovos;
#endif
#endif

//-----------------------------------------------------------------------------
// AUD_DSPCmdPlayAsyn
//
/** Brief of AUD_DSPCmdPlayAsyn.
 *  Issue play command to audio decoder, note that this is a asynchronous play
 *  function. There will be a audio decoder notification when the command was done
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u1DspId        Dsp ID
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdPlayAsyn(UINT8 u1DspId, UINT8 u1DecId);


//-----------------------------------------------------------------------------
// AUD_DSPCmdResume
//
/** Brief of AUD_DSPCmdResume.
 *  Details of AUD_DSPCmdResume (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u1DspId        Dsp ID
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdResume(UINT8 u1DspId, UINT8 u1DecId);



//-----------------------------------------------------------------------------
// AUD_DSPCmdPause
//
/** Brief of AUD_DSPCmdPause.
 *  Details of AUD_DSPCmdPause (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdPause(UINT8 u1DspId, UINT8 u1DecId);

#if 0   // Unused
//-----------------------------------------------------------------------------
// AUD_DSPCmdPauseAsyn
//
/** Brief of AUD_DSPCmdPauseAsyn.
 *  Details of AUD_DSPCmdPauseAsyn (optional).
 *
 *  @param[in]        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdPauseAsyn(UINT8 u1DspId, UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
// AUD_DSPCmdStop
//
/** Brief of AUD_DSPCmdStop.
 *  Details of AUD_DSPCmdStop (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdStop(UINT8 u1DspId, UINT8 u1DecId);


//-----------------------------------------------------------------------------
// AUD_DSPCmdStopAsyn
//
/** Brief of AUD_DSPCmdStopAsyn.
 *  Details of AUD_DSPCmdStopAsyn (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdStopAsyn(UINT8 u1DspId, UINT8 u1DecId);

#if 0   // Unused
//-----------------------------------------------------------------------------
// AUD_DSPCmdReset
//
/** Brief of AUD_DSPCmdReset.
 *  Details of AUD_DSPCmdReset (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdReset(UINT8 u1DspId, UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
// AUD_DSPCmdResetAsyn
//
/** Brief of AUD_DSPCmdResetAsyn.
 *  Details of AUD_DSPCmdResetAsyn (optional).
 *
 *  @param        u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DSPCmdResetAsyn(UINT8 u1DspId, UINT8 u1DecId);


//-----------------------------------------------------------------------------
// AUD_RegNotifyFunc
//
/** Register audio notify function.
 *
 *  @param     pu4Handler   Notify function handler
 *  @param     pfNfyFunc    Notify function pointer
 *
 *  @retval TRUE            Succeed
 *  @retval FALSE           Fail
 */
extern BOOL AUD_RegNotifyFunc(UINT32* pu4Handler, AudDecNfyFct pfNfyFunc);



//-----------------------------------------------------------------------------
// AUD_DeRegNotifyFunc
//
/** Deregister audio notify function.
 *
 *  @param  u4Handler   Handler
 *
 *  @retval TRUE            Succeed
 *  @retval FALSE           Fail
 */
extern BOOL AUD_DeRegNotifyFunc(UINT32 u4Handler);

//-----------------------------------------------------------------------------
// AUD_GetDecodeType
//
/** Brief of AUD_GetDecodeType.
 *  Details of AUD_GetDecodeType (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      peStreamFrom     Stream source
 *  @param      peDecType        Stream format
 */
//-----------------------------------------------------------------------------
extern void AUD_GetDecodeType(UINT8 u1DspId, UINT8 u1DecId, AUD_DEC_STREAM_FROM_T* peStreamFrom, AUD_FMT_T* peDecType);
extern void AUD_GetOriDecType(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T* peStreamFrom, AUD_FMT_T* peDecType);

//-----------------------------------------------------------------------------
// AUD_GetStreamFrom
//
/** Brief of AUD_GetStreamFrom.
 *  Details of AUD_GetStreamFrom (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      peStreamFrom     Stream source
 */
//-----------------------------------------------------------------------------
extern void AUD_GetStreamFrom(UINT8 u1DspId, UINT8 u1DecId, AUD_DEC_STREAM_FROM_T* peStreamFrom);
extern void AUD_SetAudClipGain(UINT32 u4Gain);

//-----------------------------------------------------------------------------
// AUD_SetAudClip
//
/** Brief of AUD_SetAudClip.
 *  Details of AUD_SetAudClip (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      peAudClip
 */
//-----------------------------------------------------------------------------
extern void AUD_SetAudClip(UINT8 u1DspId, UINT8 u1DecId, AUD_CLIP_TYPE_T * peAudClip);

//-----------------------------------------------------------------------------
// AUD_GetAudioPTS
//
/** Brief of AUD_GetAudioPTS.
 *  Details of AUD_GetAudioPTS (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      prAudioPts       AudioPts
 */
//-----------------------------------------------------------------------------
extern void AUD_GetAudioPTS(UINT8 u1DecId, UINT64* rAudioPts);

extern void AUD_GetResumePTS(UINT8 u1DecId, UINT64* rAudioPts );

//-----------------------------------------------------------------------------
// AUD_PlayAudClip
//
/** Brief of AUD_PlayAudClip.
 *  Details of AUD_PlayAudClip (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u4Channel
 *  @param      fgPlayTestTone
 */
//-----------------------------------------------------------------------------
extern void AUD_PlayAudClip(UINT8 u1DspId, UINT8 u1DecId, UINT32 u4Channel, BOOL fgPlayTestTone);

//-----------------------------------------------------------------------------
// AUD_PlayAudClipWithRPN
//
/** Brief of AUD_PlayAudClipWithRPN.
 *  Details of AUD_PlayAudClipWithRPN (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u4Channel
 *  @param      ReaptNumber
 *  @param      fgPlayTestTone
 */
extern void AUD_PlayAudClipWithRPN(UINT8 u1DspId, UINT8 u1DecId, UINT32 u4Channel, UINT32 ReaptNumber, BOOL fgPlayTestTone);

//-----------------------------------------------------------------------------
// AUD_StopAudClip
//
/** Brief of AUD_StopAudClip.
 *  Details of AUD_StopAudClip (optional).
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 */
//-----------------------------------------------------------------------------
extern void AUD_StopAudClip(UINT8 u1DspId, UINT8 u1DecId);
//-----------------------------------------------------------------------------
// Aud_SetAudClipVolume
//
/** Brief of Aud_SetAudClipVolume.
 *  Details of Aud_SetAudClipVolume (optional).
 *
 *  @param      u1DecId        Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *              u1Volume       Clip Vol  0 ~ 30 mapping to -20 ~ +10dB
 */
//-----------------------------------------------------------------------------
extern void Aud_SetAudClipVolume(UINT8 u1DspId, UINT8 u1DecId, UINT32 u4Channel, UINT8 u1Volume);
//Aud_if.c ->AUD_DeleteAudioClip,free mem u1Clip_buf
extern void AUD_DeleteAudClip(void);

//-----------------------------------------------------------------------------
// AUD_AvInputMuxSel
//
/** Brief of AUD_AvInputMuxSel.
 *  Details of AUD_AvInputMuxSel (optional).
 *
 *  @param  eInputId   AV component id
 *
 */
//-----------------------------------------------------------------------------
extern void AUD_AvInputMuxSel(AUD_INPUT_ID_T eInputId);

//-----------------------------------------------------------------------------
// AUD_AvInputMuxSelByAD
//
/** Brief of AUD_AvInputMuxSelByAD.
 *  Details of AUD_AvInputMuxSelByAD(optional).
 *
 *  @param  UINT8 eADNum
 *
 */
//-----------------------------------------------------------------------------
extern void AUD_AvInputMuxSelByAD(UINT8 eADNum);


//-----------------------------------------------------------------------------
// AUD_SetCtrlType
//
/** Brief of AUD_SetCtrlType.
 *  Details of AUD_SetCtrlType (optional).
 *
 *  @param  u4MWSetCtrlType
 *
 */
//-----------------------------------------------------------------------------
extern void AUD_SetCtrlType(UINT8 u1DspId,  UINT8 u1DecId, UINT32 u4MWSetCtrlType);

//-----------------------------------------------------------------------------
// AUD_SetCtrlCmd
//
/** Brief of AUD_SetCtrlCmd.
 *  Details of AUD_SetCtrlCmd (optional).
 *
 *  @param  eMWSetCtrlCmd
 *
 */
//-----------------------------------------------------------------------------
extern void AUD_SetCtrlCmd(UINT8 u1DspId,  UINT8 u1DecId, AUD_DEC_CMD_T eMWSetCtrlCmd);

//-----------------------------------------------------------------------------
// AUD_GetCtrlCmd
//
/** Brief of AUD_GetCtrlCmd.
 *  Details of AUD_GetCtrlCmd (optional).
 *
 *  @param  eMWSetCtrlCmd
 *
 */
//-----------------------------------------------------------------------------
extern AUD_DEC_CMD_T AUD_GetCtrlCmd(UINT8 u1DspId,  UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_Mute
//
/** Brief of AUD_Mute.
 *  Details of AUD_Mute (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_Mute(UINT8 u1DecId);

#if 0   // Unused
//-----------------------------------------------------------------------------
// AUD_DtvLock
//
/** DTV audio lock check
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DtvLock(UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
// AUD_SetAvSynMode
//
/** DTV audio sync check
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      eSynMode       Sync mode, 0:Freerun 1: Slave, 2:Master
 */
//-----------------------------------------------------------------------------
extern void AUD_SetAvSynMode(UINT8 u1DecId, AV_SYNC_MODE_T eSynMode);
//-----------------------------------------------------------------------------
// AUD_GetMultimediaMode
//
/**Get Audio Multimedia Mode
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 *  @retval      Multimedia mode, 0:Normal 1: Karaoke
 */
//-----------------------------------------------------------------------------
extern AUD_MM_MODE_T AUD_GetMultimediaMode(UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_SetMultimediaMode
//
/** Set Audio Multimedia Mode
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 *  @param      eMultimediaMode  Multimedia mode, 0: Normal 1: Karaoke
 *                                                2: Karaoke mic pass
 */
//-----------------------------------------------------------------------------
extern void AUD_SetMultimediaMode(UINT8 u1DecId, AUD_MM_MODE_T eMultimediaMode);

//-----------------------------------------------------------------------------
// AUD_SetAdMode
//
/** Set AD Mode
 *
 *  @param      fgAdMode  0: AD will not be turned on in this connection, 1: AD will be turned on
 */
//-----------------------------------------------------------------------------
extern void AUD_SetAdMode(BOOL fgAdMode);

//-----------------------------------------------------------------------------
// AUD_UnMute
//
/** Brief of AUD_UnMute.
 *  Details of AUD_UnMute (optional).
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_UnMute(UINT8 u1DecId);
//-----------------------------------------------------------------------------
// AUD_DrvSetMMAudioOnly
//
/** Brief of AUD_DrvSetMMAudioOnly.
 *  set multi media audio only.
 *
 *  @param      fgMMAudioOnly         true    MM only
 *                                                     false   not only
 *
 */
//-----------------------------------------------------------------------------
extern void AUD_DrvSetMMAudioOnly(UINT8 u1DecId, BOOL fgMMAudioOnly);
#ifdef CC_ENABLE_AOMX
//-----------------------------------------------------------------------------
// AUD_DrvGetMMAudioOnly
//
/** Brief of AUD_DrvGetMMAudioOnly.
 *  get multi media only.
 *
 *  @retval       true    MMAudio only
 *                    false
 *
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DrvGetMMAudioOnly(UINT8 u1DecId);
#endif
#ifdef CC_AUD_DDI
// AUD_DrvGetMMAudioOnly
extern BOOL AUD_GetMMAudioOnly(UINT8 u1DecId);
#endif
//set Audio Description Fade enable or not
extern void AUD_SetAdFadeEnable(BOOL bEnable);
#ifdef CC_AUD_DOLBY_SUPPORT_MS12
extern void AUD_SetADGainByteEnable(BOOL bEnable);
#endif // CC_AUD_DOLBY_SUPPORT_MS12
//Pan is another format of Ad
extern void AUD_SetAdPanEnable(BOOL bEnable);
//control the Pand & Fade Pes Update.
extern void AUD_PanFadePesUpdateEn(BOOL bEnable);

//-----------------------------------------------------------------------------
// AUD_SendAudioPes
//
/** Brief of AUD_SendAudioPes.
 *  This is the callback function for dmx inform audio driver when retrieving
 *  audio PES
 *
 *  @param      prPes          PES information
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_SendAudioPes(const DMX_AUDIO_PES_T* prPes);
#ifdef CC_ENABLE_AOMX
//-----------------------------------------------------------------------------
// GST_SendAudioPes
//
/** Brief of GST_SendAudioPes.
 *  for GST inform audio driver when retrieving audio PES
 *
 *  @param      rPes          PES information
 *
 *  @retval true    Success
 *  @retval false   Fail
 */
//-----------------------------------------------------------------------------
extern BOOL GST_SendAudioPes(const GST_AUDIO_PES_T rPes);     //koro gst
//-----------------------------------------------------------------------------
// GST_StartSTC
//
/** Brief of GST_StartSTC.
 *  enable _afgStcStart[ucStcId]
 *
 *  @param      stcId         Id of STC
 *
 */
//-----------------------------------------------------------------------------
extern void GST_StartSTC(UINT8 stcId);
//-----------------------------------------------------------------------------
// GST_StopSTC
//
/** Brief of GST_StopSTC.
 *  disable _afgStcStart[ucStcId]
 *
 *  @param      stcId         Id of STC
 *
 */
//-----------------------------------------------------------------------------
extern void GST_StopSTC(UINT8 stcId);
//-----------------------------------------------------------------------------
// GST_SetSTCVal
//
/** Brief of GST_SetSTCVal.
 *  write the low 32bits value of u8Stc to stcId STC
 *
 *  @param      stcId         Id of STC
 *  @param      u8Stc        64bits stc value
 */
//-----------------------------------------------------------------------------
extern void GST_SetSTCVal(UINT8 stcId, UINT64 u8Stc);
//-----------------------------------------------------------------------------
// GST_GetSTCVal
//
/** Brief of GST_GetSTCVal.
 *  get STC value
 *
 *  @param      stcId         Id of STC
 */
//-----------------------------------------------------------------------------
extern UINT64 GST_GetSTCVal(UINT8 stcId);
//-----------------------------------------------------------------------------
// GST_AOMX_DumpAudioData
//
/** Brief of GST_AOMX_DumpAudioData.
 *  dump the data in addr to the determined route.
 *
 *  @param      u1DecId         Id of decoder
 *  @param      addr               data source address
 *  @param      size                data size
 *  @param      fgnew            ture: log the destination route to be dump
 */
//-----------------------------------------------------------------------------
extern void GST_AOMX_DumpAudioData(UINT8 u1DecId, const char *addr, UINT32 size, BOOL fgnew);
//-----------------------------------------------------------------------------
// GST_SetAprocAVSyncCtrl
//
/** Brief of GST_SetAprocAVSyncCtrl.
 *  dump the data in addr to the determined route.
 *
 *  @param      u1DecId         Id of decoder
 *  @param      eMode           AV sync mode
 */
//-----------------------------------------------------------------------------
extern VOID GST_SetAprocAVSyncCtrl(UINT8 u1DecId, APROC_AVSYNC_CTRL_MODE_T eMode);
#endif

//-----------------------------------------------------------------------------
// AUD_PcmSetting
//
/** PCM format setting
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      prPcmSetting
 */
//-----------------------------------------------------------------------------
extern void AUD_PcmSetting(UINT8 u1DspId, UINT8 u1DecId, const AUD_PCM_SETTING_T* prPcmSetting);

#ifdef CC_HDMI_PCM_MULT_CHANNEL
//-----------------------------------------------------------------------------
// AUD_LineInChNumSetting
//
/** Line-In Channel Number Setting
 *
 *  @param      u1LineInChNum    Line-In Channel Number Setting
 *  @param      *u4Format        Default Line Module Configure
 */
//-----------------------------------------------------------------------------
extern void AUD_LineInChNumSetting(UINT8 u1DecId, UINT8 u1LineInChNum);
//-----------------------------------------------------------------------------
// AUD_LineInChNumFormat
//
/** Line-In Channel Number Format
 *
 *  @param      u1LineInChNum    Line-In Channel Number Setting
 *  @param      pu4Format
 */
//-----------------------------------------------------------------------------
extern void AUD_LineInChNumFormat(UINT8 u1LineInChNum, UINT32* pu4Format);
///Drv_hdmi.h
extern UINT8 AUD_HDMI_Get_Aud_ChannelNum(void);
extern UINT8 AUD_HDMI_Get_Aud_CA(void);
#endif

//-----------------------------------------------------------------------------
// AUD_FlacSetting
//
/** FLAC format setting
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 *  @param      prFlacSetting
 *  paul_flac
 */
//-----------------------------------------------------------------------------
extern void AUD_FlacSetting(UINT8 u1DecID, AUD_FLAC_INFO_T* prFlacSetting);

//-----------------------------------------------------------------------------
/** AUD_DspFineTuneVolume
 *  Fine tune master volume/trim, and set it to DSP
 *  @param  u1DecId          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  u4ShmValue       Volume (0x20000 is equal to 0 dBFs, max value: 0x7fffff)
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspFineTuneVolume(UINT8 u1DecId, AUD_CH_T eChannel, UINT32 u4ShmValue);

//-----------------------------------------------------------------------------
/** AUD_DspGetFineTuneVolume
 *  Fine tune master volume/trim, and set it to DSP
 *  @param  u1DecId          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  u4ShmValue       Volume (0x20000 is equal to 0 dBFs, max value: 0x7fffff)
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetFineTuneVolume(UINT8 u1DecId, AUD_CH_T eChannel);

//-----------------------------------------------------------------------------
/** AUD_DspVolumeInit
 *  Initialize all variables for volume control
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspVolumeInit(void);

#ifdef CC_AUD_ARM_RENDER//CC_AUD_APROC_POST
//-----------------------------------------------------------------------------
/** AUD_AprocBMangInit
 *  Initialize Aproc BMang
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_AprocBMangInit(void);
#endif
//-----------------------------------------------------------------------------
/** AUD_DspEqCfgInit
 *  Initialize variables for equalizer preset configuration
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspEqCfgInit(void);

#ifdef CC_AUD_VOLUME_OFFSET
//-----------------------------------------------------------------------------
/** AUD_DspVolumeOffset
 *  set volume offset to master volume.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspVolumeOffset(UINT8 u1DecId, UINT8 u1Index);
#endif
//to be done,2015.10.22
extern void AUD_DspClearSoundEnable(UINT8 u1DecId, BOOL fgEnable);
#if (defined(CC_MAPLE_CUST_DRV) || defined(CC_AUD_UNICORN_CUST) || defined(CC_AUD_SAKURA_RENDER))
extern void AUD_DspSetTvPosition(UINT8 u1DecId,UINT8 u1Position);
#endif
extern void AUD_DspPostSoundMode(UINT8 u1DecId, UINT8 u1ModeIdx);
extern void AUD_DspSetPEQMode(UINT8 u1DecId,UINT8 u1ModeIdx);

//-----------------------------------------------------------------------------
/** AUD_DspPostSoundModeStart
 *  Indicate sound mode start/end
 *
 *  @param  u1DecId          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgStart          start(TRUE), end(FALSE).
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspPostSoundModeStart(UINT8 u1DecId, BOOL fgStart);

//-----------------------------------------------------------------------------
/** AUD_DspChDelayInit
 *  Initialize table for channel delay configuration
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChDelayInit(void);

#ifdef CC_AUD_SPIDF_LVL
//-----------------------------------------------------------------------------
/** AUD_DspSpdifLevel
 *  Change spdif level
 *
 *  @param u1Level
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpdifLevel(UINT8 u1Level);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspSpdifInit
 *  Initialize SPDIF configuration
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpdifInit(void);

//-----------------------------------------------------------------------------
/** AUD_DspSpdifInit
 *  Initialize SPDIF-in configuration
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_SpdifinInit(void);

//-----------------------------------------------------------------------------
/** AUD_DspAsrcInit
 *  Initialize ASRC configuration
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAsrcInit(void);


//-----------------------------------------------------------------------------
// AUD_DspReverbEnable
//
/** turn on / off reverb control.
 *
 *  @param  u1DecId          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable          "TRUE" for enable.
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspReverbEnable(UINT8 u1DecId, BOOL fgEnable);


//-----------------------------------------------------------------------------
/** AUD_DspReverbMode
 *  load the predefined reverberation parameters sets.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  bMode From mode 0 to mode 7. Mode 0 will turn the reverberation effect off.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspReverbMode(UINT8 u1DecId, UINT8 u1Mode);


//-----------------------------------------------------------------------------
// AUD_DspChannelEqEnable
//
/** turn on /off the bass boost control
 *
 *  @param  u1DecId         Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable          "TRUE" for enable.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelEqEnable(UINT8 u1DecId, BOOL fgEnable);
extern BOOL AUD_DspGetChannelEqEnable(UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_DspSpkSizeCfg
//
/** Set the bass management cut-off frequency
 *
 *  @param  u1DecId         Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u4SpkSize         u2SpkSize in xx Hz.
 *  @retval void
 */
extern void AUD_DspSpkSizeCfg(UINT8 u1DecId, UINT32 u4SpkSize);

#ifdef CC_S_SUBWOFFER_SUPPORT  ///CC_BASS_MANGER_INDEP_LPF
//-----------------------------------------------------------------------------
// AUD_DspSpkSizeLpfCfg
//
/** Set the bass management cut-off frequency
 *
 *  @param  u1DecId         Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u4SpkSize         u2SpkSize in xx Hz.
 *  @retval void
 */
extern void AUD_DspSpkSizeLpfCfg(UINT8 u1DecId, UINT32 u4SpkSize);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspBassBoostEnable
 *  turn on /off the bass boost control.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable ADSP_ON / ADSP_OFF.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspBassBoostEnable(UINT8 u1DecId, BOOL fgEnable) ;
extern BOOL AUD_DspGetBassBoostEnable(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspBassBoostGain
 *   set the bass boost gain value.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u1Level the setting value is from 0 to 28, level 14 means 0db. A large
 *                         number will boost the bass and a smaller number will attenuate the bass.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspBassBoostGain(UINT8 u1DecId, UINT8 u1Level);


//-----------------------------------------------------------------------------
/** AUD_DspGetBassBoostGain
 *   get the bass boost gain value.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *
 *  @retval bass gain value
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetBassBoostGain(UINT8 u1DecId);


//-----------------------------------------------------------------------------
/** AUD_DspTrebleBoostCtrl
 *   turn on / off treble boost control.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable  ADSP_ON / ADSP_OFF.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSuperBassEnable(UINT8 u1DecId, BOOL fgEnable);


//-----------------------------------------------------------------------------
// AUD_DspClearBoostEnable
//
/** turn on / off treble boost control
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable  ADSP_ON / ADSP_OFF.
 *  @retval void
 */
extern void AUD_DspClearBoostEnable(UINT8 u1DecId, BOOL fgEnable);
extern BOOL AUD_DspGetClearBoostEnable(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspClearBoostGain
 *   set the clear boost gain value.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u1Level  range from 0 to 28, level 14 means 0db.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspClearBoostGain(UINT8 u1DecId, UINT8 u1Level);


//-----------------------------------------------------------------------------
/** AUD_DspGetClearBoostGain
 *   get the treble boost gain value.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *
 *  @retval treble gain value
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetClearBoostGain(UINT8 u1DecId);



//-----------------------------------------------------------------------------
/** AUD_DspEqEnable
 *  turn on / off equalizer control.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable  TRUE / FALSE.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspEqEnable(UINT8 u1DecId, BOOL fgEnable);


//-----------------------------------------------------------------------------
// AUD_DspChEqBand
//
/** used to adjusting the equalizer's parameter for each channel
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u1BandIdx  EQ band index
 *  @param  u1Level range is from 0 to 28
 *  @param  fgNotify  to indicate changing the parameter immediately or not
 *
 *  @retval void
 */
extern void AUD_DspChEqBand(UINT8 u1DecId, UINT8 u1BandIdx, UINT8 u1Level, BOOL fgNotify);



//-----------------------------------------------------------------------------
/** AUD_DspChEqLoadPreset
 *   load the predefined channel equalizer setting.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eEqType  the valid type number is from 0 to type 7.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChEqLoadPreset(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType) ;


//-----------------------------------------------------------------------------
// AUD_DspChEqBandMode
//
/** used to adjusting the equalizer's parameter for each channel
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u1BandIdx  EQ band index
 *  @param  u1Level range is from 0 to 28
 *  @param  fgNotify  to indicate changing the parameter immediately or not
 *
 *  @retval void
 */

extern void AUD_DspChEqBandMode(UINT8 u1DecId, UINT8 u1BandIdx, INT8 i1Level);

//-----------------------------------------------------------------------------
/** AUD_DspChEqLoadPresetChgMode
 *   load the predefined channel equalizer setting for mode changing.
 *
 *  @param  u1DecId Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eEqType  the valid type number is from 0 to type 7.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChEqLoadPresetChgMode(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType) ;


//-----------------------------------------------------------------------------
/** AUD_DspChSetEqTable
 *   set the band gain of a specific equalizer.
 *
 *  @param  u1DecId  Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eEqType  the valid type number is from 0 to type 7.
 *  @param  eqValue  band gain, valid value from -20 ~ 20
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChSetEqTable(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType, INT8 eqValue[]);


//-----------------------------------------------------------------------------
/** AUD_DspChQryEqTable
 *   query the band gain of a specific equalizer.
 *
 *  @param  u1DecId  Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eEqType  the valid type number is from 0 to type 7.
 *  @param  eqValue  band gain, valid value from -20 ~ 20
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChQryEqTable(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType, INT8 eqValue[]);

//-----------------------------------------------------------------------------
/** AUD_DspGetEqType
 *   get the type of current equalizer.
 *
 *  @param  u1DecId  Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  eEqType
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetEqType(UINT8 u1DecId, AUD_EQ_TYPE_T* eEqType);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterMode
 *   set limiter mode
 *
 *  @param  u1Mode  limiter mode (0: off, 1: adaptive mode, 2: fixed mode).
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterMode(UINT8 u1Mode);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterFlag
 *   set limiter flag
 *
 *  @param  u1Flag  limiter flag (0: off, 1: on)
 *                         bit 0: vol
 *                         bit 1: vsurr
 *                         bit 2: sbass
 *                         bit 3: EQ
 *                         bit 4: reverb
 *                         bit 5: speaker
 *                         bit 6: voice
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterFlag(UINT8 u1Flag);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterThreshold
 *   set limiter threshold
 *
 *  @param  u4Thre  limiter threshold (0x0 ~ 0x7fffff)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterThreshold(UINT32 u4Thre);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterAttackRate
 *   set limiter attack rate
 *
 *  @param  u4AttackRate  limiter attack rate (0x0 ~ 0x7fffff)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterAttackRate(UINT32 u4AttackRate);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterRleaseRate
 *   set limiter attack rate
 *
 *  @param  u4ReleaseRate  limiter release rate (0x0 ~ 0x7fffff)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterRelaseRate(UINT32 u4ReleaseRate);

//-----------------------------------------------------------------------------
/** AUD_DspSetLimiterGainRatio
 *   set limiter gain ratio
 *
 *  @param  u4Ratio  limiter gain ratio (0x0 ~ 0x7fffff)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetLimiterGainRatio(UINT32 u4Ratio);


//-----------------------------------------------------------------------------
/** AUD_DspGetLimiterConfig
 *   get limiter configuration
 *
 *  @param  *pu1Mode  mode
 *  @param  *pu2Flag  flag
 *  @param  *pu4Thre  threshold
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetLimiterConfig(UINT8* pu1Mode, UINT16* pu2Flag, UINT32* pu4Thre);

//-----------------------------------------------------------------------------
/** AUD_DspGetLimiterConfig1
 *   get limiter configuration1
 *
 *  @param
 *  @param
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetLimiterConfig1(UINT32* pu4Attack, UINT32* pu4Release, UINT32* pu4Ratio);

//-----------------------------------------------------------------------------
/** AUD_DspAC3KaraMode
 *  setup for AC3 karaoke mode.
 *
 *  @param  u2KaraMode Karaoke mode
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAC3KaraMode(UINT16 u2KaraMode, UINT8 uDecIndx) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetAC3KaraMode
 *  setup for AC3 karaoke mode.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval u2KaraMode
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetAC3KaraMode(UINT8 uDecIndx) ;

//-----------------------------------------------------------------------------
/** AUD_DspAC3DualMono
 *  setup for AC3 dual mono mode.
 *
 *  @param  u2DMMode dual mono mode
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAC3DualMono(UINT16 u2DMMode, UINT8 uDecIndx) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetAC3DualMono
 *  setup for AC3 dual mono mode.
 *
 *  @param  u2DMMode dual mono mode
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetAC3DualMono(UINT8 uDecIndx) ;

//-----------------------------------------------------------------------------
/** AUD_DspAC3CompMode
 *  setup for compression mode.
 *
 *  @param  u2CompMode compression mode.
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAC3CompMode(UINT16 u2CompMode, UINT8 uDecIndx) ;


//-----------------------------------------------------------------------------
/** AUD_DspGetAC3CompMode
 *  setup for compression mode.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval u2CompMode compression mode.
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetAC3CompMode(UINT8 uDecIndx) ;


//-----------------------------------------------------------------------------
/** AUD_DspAC3DRCRange
 *  setup range for dynamix range compression.
 *
 *  @param  uDRCLevel  DRC range.
 *                               0x00000000-->0.0
 *                               0x007FFFFF-->1.0
 *                               0.0 ~ 1.0 step 0.125
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAC3DRCRange(UINT8 uDRCLevel, UINT8 u1DecId);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspGetAC3DRCRange
 *  setup range for dynamix range compression.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval uDRCLevel  DRC range.
 *                               0x00000000-->0.0
 *                               0x007FFFFF-->1.0
 *                               0.0 ~ 1.0 step 0.125
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetAC3DRCRange(UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspGetAC3DRCRange100
 *  setup range for dynamix range compression.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval UINT32  uDRCLevel  DRC range.
 *                               0x00000000-->0.0
 *                               0x007FFFFF-->1.0
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetAC3DRCRange100(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspAC3DRCRange100
 *  setup range for dynamix range compression.
 *
 *  @param  uDRCLevel  DRC range 0~100.
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAC3DRCRange100(UINT8 uDRCLevel, UINT8 u1DecId);

extern void Aud_G726DecCfg(UINT8 u1DecId, UINT8 u1rate);

#ifdef CC_AUD_SUPPORT_MS10
//-----------------------------------------------------------------------------
/** AUD_DspSetDDCStrmId
 *  setup range for associated stream id.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDDCControl (UINT16 u2Control, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCStrmId
 *  setup range for associated stream id.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetDDCControl (UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCStrmId
 *  setup range for associated stream id.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDDCStrmId (UINT8 u1StrmId, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCStrmId
 *  setup range for associated stream id.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetDDCStrmId (UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCAdEnable
 *  setup for DDC AD.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDDCAdEnable (UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCMixer
 *  setup for DDC mixer.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDDCMixer (UINT8 u1Control, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetDDCMixer
 *  setup for DDC mixer.
 *
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetDDCMixer (UINT8 u1DecId);

/** AUD_DspDdtDmxMode
 *  setup for DDT dmx mode.
 *
 *  @param  u1Mode: dmxmode/RF mode, u1DecId 0: first decoder 1: seconder decoder. 2: third decoder, bSetDmx 0: set dmx, 1: set RF
 *  @retval void
 *  u1Mode:
 *  00b: Line, Lt/Rt
 *  01b: Line, Lo/Ro
 *  10b: RF, Lt/Rt
 *  11b: RF, Lo/Ro
 */
//-----------------------------------------------------------------------------
extern void AUD_DspDdtDmxRfMode (UINT8 u1Mode, UINT8 u1DecId, BOOL bSetDmx);

//-----------------------------------------------------------------------------
/** AUD_DspDdtDualMode
 *  setup for DDT dual mode.
 *
 *  @param  u1Mode: dual mode, u1DecId 0: first decoder 1: seconder decoder. 2: third decoder
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspDdtDualMode (UINT8 u1Mode, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspDdtCompVal
 *  setup for DDT DRC cut/boost factor.
 *
 *  @param  u1Mode: cut/boost factor 0~100, u1DecId 0: first decoder 1: seconder decoder. 2: third decoder
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspDdtCompVal (UINT8 u1Mode, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspGetDdtDmxRfMode
 *  setup for DDT dmx mode.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder. 2: third decoder, bGetDmx 0: get dmx, 1: get RF
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetDdtDmxRfMode (UINT8 u1DecId, BOOL bGetDmx);

//-----------------------------------------------------------------------------
/** AUD_DspDdtDualMode
 *  setup for DDT dual mode.
 *
 *  @param  u1Mode: dual mode, u1DecId 0: first decoder 1: seconder decoder. 2: third decoder
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetDdtDualMode (UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspDdtCompVal
 *  setup for DDT DRC cut/boost factor.
 *
 *  @param  u1Mode: cut/boost factor 0~100, u1DecId 0: first decoder 1: seconder decoder. 2: third decoder
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetDdtDrcRange (UINT8 u1DecId);

//-----------------------------------------------------------------------------

#endif  // CC_AUD_SUPPORT_MS10

/** AUD_DspSpeakerConfig
 *  speaker configuration setup.
 *
 *  @param  u4SpkCfg  Speaker config
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpeakerConfig(UINT32 u4SpkCfg, UINT8 uDecIndex);
extern UINT32 AUD_DspGetSpeakerConfig(UINT8 uDecIndex);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSpeakerSizeConfig
 *  speaker configuration setup.
 *
 *  @param  uDecIndex  Speaker config
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpeakerSizeConfig(UINT8 uDecIndex, BOOL fgFrontLarge, BOOL fgSurroundLarge, BOOL fgCenterLarge);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspSpeakerLargeSmallConfig
 *  Speaker size configuration of each channel pair.
 *  @param  u1DecIndex          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  u1FrontPair          Speaker size configuration for front left and right channels. (1: large, 0: small)
 *  @param  u2Center             Speaker size configuration for center channel. (1: large, 0: small)
 *  @param  u1Surround          Speaker size configuration for left and right surround channels. (1: large, 0: small)
 *  @param  u1CH910              Speaker size configuration for CH9/10 channels. (1: large, 0: small)
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
// large(1)/small(0)
extern void AUD_DspSpeakerLargeSmallConfig(UINT8 u1DecIndex, UINT8 u1FrontPair, UINT8 u2Center, UINT8 u1Surround, UINT8 u1CH910);
extern void AUD_DspGetSpeakerLargeSmallConfig(UINT8 u1DecIndex, UINT8* u1FrontPair, UINT8* u1Center, UINT8* u1Surround, UINT8* u1CH910);

//-----------------------------------------------------------------------------
/** AUD_DspSpeakerLsConfig
 *  setup speaker size of each channel.
 *
 *  @param  u1DecIndex 0: first decoder 1: seconder decoder.
 *  @param  u1L  large(1)/small(0).
 *  @param  u1R  large(1)/small(0).
 *  @param  u1Ls   large(1)/small(0).
 *  @param  u1Rs   large(1)/small(0).
 *  @param  u1C  large(1)/small(0).
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpeakerLsConfig(UINT8 u1DecIndex, UINT8 u1L, UINT8 u1R, UINT8 u1Ls, UINT8 u1Rs, UINT8 u1C);


//-----------------------------------------------------------------------------
/** AUD_DspSpeakerOutputConfig
 *  set speaker output config.
 *
 *  @param  u1DecIndex 0: first decoder 1: seconder decoder.
 *  @param  u1SpkCfg.
 *                              bit 0 ~ 2:.
 *                              b000: 2/0 (LT/RT downmix: prologic compatible).
 *                              b001: 1/0.
 *                              b010: 2/0 (LO/RO).
 *                              b011: 3/0.
 *                              b100: 2/1.
 *                              b101: 3/1.
 *                              b110: 2/2.
 *                              b111: 3/2.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpeakerOutputConfig(UINT8 u1DecIndex, UINT8 u1SpkCfg);
extern UINT8 AUD_DspGetSpeakerOutputConfig(UINT8 u1DecIndex);

//-----------------------------------------------------------------------------
/** AUD_DspSpeakerSubwooferEnable
 *  Turn On/Off subwoofer
 *  @param  u1DecIndex          Audio decoder id (0: first decoder, 1: second decoder).
 *  @param  fgEnable              Subwoofer channel is turned on or off. (1: on, 0: off)
 *
 *  @return void
 */
//-----------------------------------------------------------------------------
// bit 5: Subwoofer Channel present(1)/absent(0)
extern void AUD_DspSpeakerSubwooferEnable(UINT8 u1DecIndex, BOOL fgEnable);
extern BOOL AUD_DspGetSpeakerSubwooferEnable(UINT8 u1DecIndex);

//-----------------------------------------------------------------------------
/** AUD_DspChannelDelay
 *  setup channel delay for individual channel.
 *
 *  @param  u2Delay 0.05 m/unit.
*  @param  eChIndex 0(L), 1(R), 2(LS), 3(RS), 4(C), 5(SUB), 6(Bypass L), 7(Bypass R)
 *                   8(Downmix L), 9(Downmix R), 10(AUD L), 11(AUD R), 13(All), 14(LFE)
 *  @param  u1DecIndex 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelDelayValue(INT16 u2Delay, AUD_CH_T eChIndex, UINT8 uDecIndex);
extern void AUD_DspChannelDelayAP(AUD_CH_T eChIndex, UINT8 uDecIndex);
extern void AUD_DspChannelDelay(UINT8 u1DspId, UINT16 u2Delay, AUD_CH_T eChIndex, UINT8 uDecIndex);
extern void AUD_DspChannelDelay_initial(UINT16 u2Delay, AUD_CH_T eChIndex, UINT8 uDecIndex);
#ifdef CC_AUD_DDI
extern BOOL AUD_PlayMuteSetChannelDelayReady(UINT16 u2ChannId);
#endif
#ifdef CC_ENABLE_AOMX
extern BOOL AUD_PlayMuteSetAoutEnableReady(UINT16 u2Timeout);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspGetChannelDelay
 *  Get channel delay for individual channel.
 *
 *  @param  eChIndex 0(L), 1(R), 2(LS), 3(RS), 4(C), 5(SUB), 6(Bypass L), 7(Bypass R)
 *                   8(Downmix L), 9(Downmix R), 10(AUD L), 11(AUD R), 13(All), 14(LFE)
 *  @retval channel delay
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetChannelDelay(UINT8 u1DspId, AUD_CH_T eChIndex);
extern UINT32 AUD_DspGetChannelMainDelaySp(void);

#ifndef CC_AUD_ENABLE_PLAY_MUTE_CONTROL
#ifndef CC_ENABLE_AV_SYNC
//-----------------------------------------------------------------------------
/** AUD_DspInputChannelDelay
 *  setup channel delay by input source.
 *
 *  @param  eStreamFrom
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspInputChannelDelay(AUD_DEC_STREAM_FROM_T eStreamFrom);
#endif
#endif


//-----------------------------------------------------------------------------
/** AUD_DspInputStreamVolumeValue
 *  setup input stream valume index
 *
 *  @param  eStreamFrom
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_InputDecVolumeValue(UINT8 u1DecId, UINT32 u4Value);

//-----------------------------------------------------------------------------
/** AUD_DspChannelVolume
 *  Get channel volume of individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetChannelVolume(UINT8 u1DecId, AUD_CH_T eChannel);

//-----------------------------------------------------------------------------
/** AUD_DspPanFadePesUpdateEn
 *  setup ad fade pan values update by pes.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspPanFadePesUpdateEn(BOOL bEnable);

//-----------------------------------------------------------------------------
/** AUD_DspAdFadeVolumeEnable
 *  setup ad fade volume for individual channel.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdPanFadeEnable(BOOL bEnable);
extern UINT8 AUD_DspGetAdPanFadeSetting(void);

#if 1
//-----------------------------------------------------------------------------
// AudLoadAudioClip
//
/** Brief of AudLoadAudioClip.
 *  Details of AudLoadAudioClip (optional).
 *
 *  @param      u1DecId                   Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u1BufferSize             ClipBufferSize
 *  @param      u1ClipBufferPointer   u1ClipBufferPointer
 *  @param      u1DecFmt                 DecFmt
 *  @param      u1ReaptNumber        ReaptNumber
 */
//-----------------------------------------------------------------------------

void AudLoadAudioClip(UINT8 u1DspId, UINT8 u1DecId, UINT32 u1BufferSize, UPTR u1ClipBufferPointer, AUD_FMT_T u1DecFmt, UINT32 u1ReaptNumber );
#else
//-----------------------------------------------------------------------------
// AudLoadAudioClip
//
/** Brief of AudLoadAudioClip.
 *  Details of AudLoadAudioClip (optional).
 *
 *  @param      u1DecId                   Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      u1BufferSize             ClipBufferSize
 *  @param      u1ClipBufferPointer   u1ClipBufferPointer
 */
//-----------------------------------------------------------------------------

extern void AudLoadAudioClip(UINT8 u1DecId, UINT32 u1BufferSize, UINT32 u1ClipBufferPointer);
#endif



//-----------------------------------------------------------------------------
/** AUD_DspAdFadeVolumeSet
 *  setup ad fade volume for individual channel.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdFadeVolumeSet(UINT8 u1Value);

//-----------------------------------------------------------------------------
/** AUD_DspAdFadeVolumeUpdate
 *  setup ad fade volume for individual channel.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdFadeVolumeUpdate(UINT8 u1DecId, AUD_CH_T eChannel);
void AUD_GetEncData(UPTR pDest,UPTR pBufAddr,UINT32 datasize,UPTR pRStart,UPTR pREnd);

//-----------------------------------------------------------------------------
/** AUD_DspAdPanVolumeSet
 *  setup ad pan volume for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  u1Value 0~100.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdPanVolumeSet(UINT8 u1DecId, AUD_CH_T eChannel, UINT8 u1Value);

//-----------------------------------------------------------------------------
/** AUD_DspAdPanVolume
 *  setup ad pan volume for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  u1Value 0~100.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdPanVolumeUpdate(UINT8 u1DecId, AUD_CH_T eChannel);

//-----------------------------------------------------------------------------
/** _AUD_DspChannelVolShmRawValue
 *  Set master volume use share memory raw data .
 *
 *  Note that AUD_DspChannelVolume & _AUD_DspChannelVolShmRawValue can only select
 *  one to use
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u4VolShm : 0 ~ 0x20000
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelVolShmRawValue(UINT8 u1DecId, AUD_CH_T eChannel, UINT32 u4VolShm);

//-----------------------------------------------------------------------------
/** AUD_DspChannelVolume
 *  setup channel volume for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  u1Value 0~100.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelVolume(UINT8 u1DspId, UINT8 u1DecId, AUD_CH_T eChannel, UINT8 u1Value);
#ifdef CC_AUD_LINEOUT_VOL_OFFSET
//-----------------------------------------------------------------------------
/** AUD_DspLineOutVolOffset
 *  setup bypass channel volume offset
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Value 0~100.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspLineOutVolOffset(UINT8 u1DecId, UINT8 u1Value);
#endif
#ifdef CC_AUD_VOLUME_OFFSET
//-----------------------------------------------------------------------------
/** AUD_DspVolumeOffset
 *  set volume offset to master volume.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspVolumeOffset(UINT8 u1DecId, UINT8 u1Index);
extern UINT8 AUD_GetDspVolumeOffset(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspHpVolumeOffset
 *  set HP volume offset.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspHpVolumeOffset(UINT8 u1DecId, UINT8 u1Index);

//-----------------------------------------------------------------------------
/** AUD_DspLineOutVolOffset
 *  setup bypass channel volume offset
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *  @retval void
 */
//-----------------------------------------------------------------------------
//extern void AUD_DspLineOutVolOffset(UINT8 u1DecId,UINT8 u1Index);

#ifdef CC_S_SUBWOFFER_SUPPORT
//-----------------------------------------------------------------------------
/** AUD_DspSWVolumeOffset
 *  set HP volume offset.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSWVolumeOffset(UINT8 u1DecId,UINT8 u1Index);
#endif

#endif

#ifdef CC_AUD_SX1_FEATURE
//-----------------------------------------------------------------------------
/** AUD_DspAdVolOffset
 *  setup audio description volume offset
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Index  : offset map index.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAdVolOffset(UINT8 u1Value);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspWMAHeader
 *  setup WMA information
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  prWmaInfo  : pointer of WMA header information
 *  @retval void
 */
//-----------------------------------------------------------------------------
#ifdef DSP_WMA_MWIF
extern void AUD_DspWMAHeader(UINT8 u1DecId, AUD_WMA_CFG_T* prWmaInfo);
#endif
#if 1 //CC_APE_SUPPORT
// APE decoder
//-----------------------------------------------------------------------------
/** AUD_DspAPEHeader
 *  setup APE information
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  prApeInfo  : pointer of APE header information
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAPEHeader(UINT8 u1DecId, AUD_APE_CFG_T* prApeInfo);
extern void AUD_DspAPEMuteBankNum(UINT8 u1DecId, UINT16 u2MuteBankNum);
#endif
extern void AUD_DspWMAProHeader(UINT8 u1DecId, AUD_WMA_CFG_T* prWmaInfo);
extern void AUD_DspFLACHeader(UINT8 u1DecId, AUD_FLAC_INFO_T* prFlacInfo);
extern void AUD_DspALACHeader(UINT8 u1DecId, AUD_ALAC_CFG_T* rALACInfo);

//-----------------------------------------------------------------------------
/** AUD_DspChannelMute
 *  setup mute for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  fgMute TRUE, FALSE
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelMute(UINT8 u1DspId, UINT8 u1DecId, AUD_CH_T eChannel, BOOL fgMute);
extern BOOL AUD_GetDspChannelMute(UINT8 u1DecId, AUD_CH_T eChannel);

//-----------------------------------------------------------------------------
/** AUD_DspChannelVolGain
 *  setup channel volume extra gain for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  i4Value     -256~96.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelVolGain(UINT8 u1DecId, AUD_CH_T eChannel, INT32 i4Value);

//-----------------------------------------------------------------------------
/** AUD_DspChannelVolDolbyGainTB11
 *  setup channel volume Dolby gain TB11 for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  i4Value     -256~96.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelVolDolbyGainTB11(UINT8 u1DecId, AUD_CH_T eChannel, INT32 i4Value);

/***************************************************************************
     Function : AUD_DspDolbyGainTB11
     Description: Dolby Technical Bulletin 11.5 (addtional 3dB attenuation for EU)
     @param u1DecId
                AUD_DEC_MAIN: 1st dec
                AUD_DEC_AUX: 2nd dec
     @retval    void
***************************************************************************/
extern void AUD_DspDolbyGainTB11(UINT8 u1DecId);
extern void _AUD_DspSetSpeakerMode(UINT8 eMode);

//-----------------------------------------------------------------------------
/** AUD_DspGetChannelVolGain
 *  Get channel volume extra gain for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *
 *  @retval extra gain (-256~96)
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_DspGetChannelVolGain(UINT8 u1DecId, AUD_CH_T eChannel);

//-----------------------------------------------------------------------------
/** AUD_DspChannelOutPort
 *  setup output port for individual channel.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eChannel  : 0: front left channel.
 *                               1: front right channel.
 *                               2: rear left channel.
 *                               3: rear right channel.
 *                               4: center channel.
 *                               5: sub woofer channel.
 *                               6: bypass left channel.
 *                               7: bypass right channel.
 *                               8: downmix left channel.
 *                               9: downmix right channel.
 *                             10: aux front left channel.
 *                             11: aux front right channel.
 *                             12: input source gain.
 *                             13: all channels    ( For master volume control ).
 *  @param  fgEnable TRUE, FALSE
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspChannelOutPort(UINT8 u1DecId, AUD_CH_T eChannel, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspBalance
 *  setup balance.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Balance  : balance value (0~100).
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspBalance(UINT8 u1DecId, UINT8 u1Balance);

//-----------------------------------------------------------------------------
/** AUD_DspMasterVolShmRawValue
 *  Set master volume use share memory raw data .
 *
 *  Note that AUD_DspMasterVolume & _AUD_DspMasterVolShmRawValue can only select
 *  one to use
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u4VolShm : 0 ~ 0x20000
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMasterVolShmRawValue(UINT8 u1DecId, UINT32 u4VolShm);

//-----------------------------------------------------------------------------
/** AUD_DspMasterVolume
 *  set master volume.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Volume  : volume value (0~100).
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMasterVolume(UINT8 u1DecId, UINT8 u1Volume);
extern UINT8 AUD_DspGetMasterVolume(UINT8 u1DecId);
extern void AUD_DspSetMasterVolumeRatio(UINT8 u1DecId, UINT8 u1Ratio);
extern UINT8 AUD_DspGetMasterVolumeRatio(UINT8 u1DecId);
extern void AUD_DspMixSndVolume(UINT8 u1DecId, UINT8 u1Volume);
//-----------------------------------------------------------------------------
/** AUD_DspAVCEnable
 *  enable AVC.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  fgEnable  : on/off.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAVCEnable(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspMuteEnable
 *  audio mute control.
 *
 *  @param  u1DecId    0: first decoder 1: seconder decoder.
 *  @param  fgEnable   Mute/unmute. (1: mute audio, 0: unmute audio)
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMuteEnable(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspMuteAllEnable
 *  audio mute control.
 *
 *  @param  u1DecId    0: first decoder 1: seconder decoder.
 *  @param  fgEnable   Mute/unmute. (1: mute audio, 0: unmute audio)
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
void AUD_DspMuteAllEnable(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspHdmiModeMuteEnable
 *  audio mute control in HDMI mode.
 *
 *  @param  u1DecId    0: first decoder 1: seconder decoder.
 *  @param  fgEnable   Mute/unmute. (1: mute audio, 0: unmute audio)
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspHdmiModeMuteEnable(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspBassMngEnalbe
 *  Bass management control.
 *
 *  @param  u1DecId    0: first decoder 1: seconder decoder.
 *  @param  fgEnable   Bass management enable. (1: enable, 0: disable)
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspBassMngEnalbe(UINT8 u1DspId, UINT8 u1DecId, BOOL fgEnable);
extern BOOL AUD_DspGetBassMngEnable(UINT8 u1DspId, UINT8 u1DecId);
extern BOOL AUD_DspGetChannelVolumeEnable(UINT8 u1DspId, UINT8 u1DecId);
extern BOOL AUD_DspGetDelayEnable(UINT8 u1DspId, UINT8 u1DecId);
extern BOOL AUD_DspGetPostProcessingEnable(UINT8 u1DspId, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspProcMode
 *  setup for compression mode.
 *
 *  @param  u2CompMode compression mode.
 *  @param  uDecIndx 0: first decoder 1: seconder decoder.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspProcMode(UINT8 u1DspId, UINT8 u1DecId, UINT16 u2Mode);


//-----------------------------------------------------------------------------
/** AUD_DspVirtualSurroundEnalbe
 *  enable virtual surround.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  fgEnable  : on/off.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspVirtualSurroundEnalbe(UINT8 u1DecId, BOOL fgEnable);
extern BOOL AUD_DspGetVirtualSurroundEnalbe(UINT8 u1DecId);

/** AUD_DspBbeMode
 *  Set BBE mode.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1BbeMode  : 0: OFF, 1: BBE, 2: BBE ViVA4.
 *  @retval void
 */
#ifdef CC_SET_VOLUME
extern void AUD_DspSetVolTable(UINT8 u1Idx, UINT32  u4Volume);
#endif
#ifdef CC_AUD_NEW_CV_TABLE
extern void AUD_DspSetCVTable(UINT8 u1Idx, UINT32  u4Volume);
extern UINT32 AUD_DspGetCVTable(UINT8 u1VolTabIdx);
extern void AUD_DspSetStepCVTable(UINT32* u4VolTabChkPnt);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspMonoDownMix
 *  enable mono downmix.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  fgEnable  : on/off.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMonoDownMix(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspSetKaraokeMixRatio
 *  Set karaoke mix ratio
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u4MixRatio  : 0x0 ~ 0x800000 (for R channel)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetKaraokeMixRatio(UINT8 u1DecId, UINT32 u4MixRatio);


//-----------------------------------------------------------------------------
/** AUD_DspSetMatrixOutMode
 *  setup for matrix output mode.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  eMode 0: stereo (disable), 1: L mono, 2: R mono, 3: mix mono
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetMatrixOutMode(UINT8 u1DecId, AUD_MATRIX_OUT_T eMode) ;
extern UINT8 AUD_DspGetMatrixOutMode(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspAutoDmx
 *  Setup auto downmix mode for AC3
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  fgEnable  : 0: disable AC3 auto downmix, 1: enable AC3 auto downmix
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspAutoDmx(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_GetDspAutoDmx
 *  Get auto downmix mode for AC3
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  fgEnable  : 0: disable AC3 auto downmix, 1: enable AC3 auto downmix
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_GetDspAutoDmx(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspBypassPostProcess
 *  Bypass audio post-processing.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u2BypassMode  : 1: bypass post-processing.
 *                          0x8000: bypass delay
 *                          0x4000: bypass trim
 *                          0x2000: bypass post-processing
 *                          0x1000: bypass bass management
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspBypassPostProcess(UINT8 u1DspId, UINT8 u1DecId, UINT16 u2BypassMode);

//-----------------------------------------------------------------------------
/** AUD_DspSoundEffectFlag
 *  Enable all audio sound effect. (EQ, Bass/Treble, Reverb, Surround, AVC)
 *
 *  @param  u1DecId  0: first decoder 1: seconder decoder.
 *  @param  fgFlag     0: disable, 1: enable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSoundEffectFlag(UINT8 u1DecId, BOOL fgFlag);



//-----------------------------------------------------------------------------
/** AUD_DspIECConfig
 *  Routine handling IEC Configuration.
 *
 *  @param  eIecCfg  0: PCM; 1:RAW.
 *  @param  fgEnable  on/off.
 *  @retval  TRUE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspIECConfig(AUD_IEC_T eIecCfg, BOOL fgEnable);
extern UINT8 AudGetIecFlag(void);
//-----------------------------------------------------------------------------
/** AUD_DspSetIecRawFlag
 *  Enable/disable sync IEC RAW delay to channel delay automatically.
 *
 *  @param  u1Flag
 *  @retval  N/A
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspSetIecRawFlag(UINT8 u1Flag);

//-----------------------------------------------------------------------------
/** AUD_DspSetIecRawDelay
 *  Set IEC RAW delay.
 *
 *  @param  i2Delay (ms)
 *  @retval  N/A
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspSetIecRawDelay(INT16 i2Delay);

//-----------------------------------------------------------------------------
/** AUD_DspIecChannel
 *  Routine handling IEC Configuration.
 *
 *  @param  eIecChannel  select audio channel,ex L/R.
 *  @retval  TRUE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspIecChannel(AUD_IEC_CH_T eIecChannel);

//-----------------------------------------------------------------------------
/** AUD_DspIecCopyright
 *  Routine handling IEC Configuration.
 *
 *  @param  u1Enable 0: disable 1: enable
 *  @param  u1CategoryCode
 *  @retval  TRUE
 */
//-----------------------------------------------------------------------------
BOOL AUD_DspIecCopyright(UINT8 u1Enable, UINT8 u1CategoryCode);

//-----------------------------------------------------------------------------
/** AUD_DspGetIecCopyright
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetIecCopyright(AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspGetIecCategoryCode
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  category code value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetIecCategoryCode(AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspResetSpdifReg
 *   reset SPDIF register.
 *
 *  @param  void
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspResetSpdifReg (AUD_DEC_STREAM_FROM_T eStreamFrom);


//-----------------------------------------------------------------------------
/** AUD_DspSetSpdifCopyright
 *  Routine handling SPDIF Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSpdifCopyright(AUD_DEC_STREAM_FROM_T eStreamFrom, UINT8 copyright);

//-----------------------------------------------------------------------------
/** AUD_DspSetSpdifCategoryCode
 *  Routine handling SPDIF Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSpdifCategoryCode(AUD_DEC_STREAM_FROM_T eStreamFrom, UINT8 categoryCode);

//-----------------------------------------------------------------------------
/** AUD_DspSetSpdifWordLength
 *  Routine handling SPDIF Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSpdifWordLength(AUD_DEC_STREAM_FROM_T eStreamFrom, UINT8 wordLength);

//-----------------------------------------------------------------------------
/** AUD_DspSpdifSetInfo
 *  Routine handling IEC Configuration.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Enable 0: disable 1: enable
 *  @retval  TRUE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspSpdifSetInfo(AUD_DEC_STREAM_FROM_T eStreamFrom, UINT8 u1CopyRight, UINT8 u1CategoryCode);

//-----------------------------------------------------------------------------
/** AUD_DspSpdifSetCopyProtect
 *  Routine handling IEC Configuration.
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Enable 0: disable 1: enable
 *  @retval  TRUE
 */
//-----------------------------------------------------------------------------
BOOL AUD_DspSpdifSetCopyProtect(AUD_DEC_STREAM_FROM_T eStreamFrom, BOOL bCpBit, BOOL bLBit);

//-----------------------------------------------------------------------------
/** AUD_DspSetSpdifUpdateMode
 *  Routine handling SPDIF Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSpdifUpdateMode(AUD_DEC_STREAM_FROM_T eStreamFrom, UINT8 eUpdate);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifCopyright
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifCopyright(SPDIF_REG_TYPE_T type, AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifCategoryCode
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  copyright value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifCategoryCode(SPDIF_REG_TYPE_T type, AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifWordLength
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  word length value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifWordLength(SPDIF_REG_TYPE_T type, AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifUpdateMode
 *  Routine handling IEC Configuration.
 *
 *  @param  eStreamFrom     input source
 *
 *  @retval  by source? value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifUpdateMode(SPDIF_REG_TYPE_T type, AUD_DEC_STREAM_FROM_T eStreamFrom);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifUpdateMode
 *  Routine handling IEC Configuration.
 *
 *  @param
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifChannelStatus(SPDIF_CHANNEL_STATUS_T* rInfo);


//-----------------------------------------------------------------------------
/** AUD_DspGetSpdifOutputMode
 *  Routine handling IEC Configuration.
 *
 *  @param  u1DecId     decoder ID
 *
 *  @retval  value
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSpdifOutputMode(UINT8 u1DecId);

#ifdef CC_AUD_MIXSOUND_SUPPORT
//-----------------------------------------------------------------------------
/** AUD_DspMixSndEnable
 *  Mix sound enable.
 *
 *  @param  fgEnable TRUE(enable) FALSE(disable).
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMixSndEnable(BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspMixSndMute
 *  Mix sound mute.
 *
 *  @param  fgMutre TRUE(mute) FALSE(un-mute).
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMixSndMute(BOOL fgMute);

//-----------------------------------------------------------------------------
/** AUD_DspMixSndControl
 *  Mix sound control.
 *
 *  @param  u1Mode play/stop.
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMixSndControl(UINT16 u2Mode);
extern void AUD_DspMixSndControl2(UINT16 u2Mode); //NEW_MIXSOUND

extern void AUD_DspMixSndDec3(BOOL fgEnable);
//-----------------------------------------------------------------------------
/** AUD_DspMixSndData
 *  Select mix sound data source.
 *
 *  @param  u1DataIdx 0~3
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMixSndClip(UINT8 u1ClipIdx);

//-----------------------------------------------------------------------------
/** AUD_DspGetMixSndEnable
 *  Get mix sound enable.
 *
 *  @param  void.
 *  @retval  TRUE(enable) FALSE(disable).
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetMixSndEnable(void);

//-----------------------------------------------------------------------------
/** AUD_DspGetMixSndClipIdx
 *  Get mix sound clip index.
 *
 *  @param  void.
 *  @retval  TRUE(enable) FALSE(disable).
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetMixSndClipIdx(void);

extern void AUD_DspSetMixSndLoop(UINT32 u4Loop);
extern void AUD_DspSetMixSndVol(UINT32 u4Vol);
extern UINT32 AUD_DspGetMixSndLoop(void);
extern UINT32 AUD_DspGetMixSndVol(void);
#endif

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSetDemodOutputMode
 *  Set MTS output mode.
 *
 *  @param  u1Output    0: Output (Mono,Mono)
 *                               1: If stereo exists, Output (L,R) otherwise (Mono,Mono)
 *                               2: If SAP exists, Output (S,S) otherwise (L,R) otherwise (Mono,Mono)
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDemodOutputMode(UINT8 u1Output);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspSetDetectDefaultMode
 *   call this function before issue detection command.
 *
 *  @param  u1Mode 0: PAL_DK, 4: SECAM-L.
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDetectDefaultMode(UINT8 u1Mode);


//-----------------------------------------------------------------------------
/** AUD_GetVideoScrambleStatus
 *  Get VideoScrambleStatus
 *
 *  @param  void.
 *  @retval  TRUE(enable) FALSE(disable).
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_GetVideoScrambleStatus(void);


//-----------------------------------------------------------------------------
/** AUD_VideoScrambled
 *   Set Video Scramble Status
 *
 *  @param  fgEnable     0: Normal Video, 1: Video scrambled .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_VideoScrambled(BOOL fgEnable);


//-----------------------------------------------------------------------------
/** AUD_DspSetUserDefinedEqCfg
 *   setup user difined EQ config.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eEqType  0: off   others: 1~12.
 *  @param  u1BandIdx  0~4.
 *  @param  i1Value -100 ~ +100
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetUserDefinedEqCfg(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType, UINT8 u1BandIdx, INT8 i1Value);
#ifdef CC_AUD_UNICORN_CUST
//-----------------------------------------------------------------------------
/** AUD_DspSetSoDefinedEqCfg
 *   setup user difined EQ config.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eEqType  0: off   others: 1~12.
 *  @param  u1BandIdx  0~6.
 *  @param  i1Value -100 ~ +100
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSoDefinedEqCfg(UINT8 u1DecId, AUD_EQ_TYPE_T eEqType, UINT8 u1BandIdx, INT8 i1Value);
#endif
//-----------------------------------------------------------------------------
/** AUD_DspSetUserDefinedEqCfg
 *   Get user difined EQ config.
 *
 *  @param  eEqType  0: off   others: 1~12.
 *  @param  u1BandIdx  0~4.
 *  @param  i1Value (-20 ~ +20)
 *
 *  @retval  TRUE: success
 *              FALSE: failed
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetUserDefineEqCfg(AUD_EQ_TYPE_T eEqType, UINT8 u1BandIdx, INT8* pi1Value);


//-----------------------------------------------------------------------------
/** AUD_DspGetUserEqBandNum
 *   Get user defined band number.
 *
 *  @retval  CH_EQ_BAND_NO = 5.
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetUserEqBandNum(void);


//-----------------------------------------------------------------------------
/** AUD_DspGetEqBandInfo
 *   Get user defined band info.
 *
 *  @param  prEqBandInfo  structure of eq band info (AUD_EQ_BAND_INFO_T).
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetEqBandInfo(AUD_EQ_BAND_INFO_T* prEqBandInfo);


//-----------------------------------------------------------------------------
/** AUD_DspGetSrcVol
 *   get source volume.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eStrmSrc      AUD_STREAM_FROM_OTHERS = 0,         // ex. Pink Noise Generator.
                                   AUD_STREAM_FROM_DIGITAL_TUNER,      // Digital tuner.
                                   AUD_STREAM_FROM_ANALOG_TUNER,       // ex. Analog tuner.
                                   AUD_STREAM_FROM_SPDIF,              // SPDIF-in.
                                   AUD_STREAM_FROM_LINE_IN,            // Line in.
                                   AUD_STREAM_FROM_HDMI,               // HDMI in.
                                   AUD_STREAM_FROM_MEMORY,             // Memory.
                                   AUD_STREAM_FROM_BUF_AGT,            //Buffer Agent.
                                   AUD_STREAM_FROM_MULTI_MEDIA,             // Multi-media

 *  @retval  input source volume
 */
//-----------------------------------------------------------------------------
extern INT16 AUD_DspGetSrcVol(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T eStrmSrc);


//-----------------------------------------------------------------------------
/** AUD_DspSetSrcVol
 *   set source volume.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eStrmSrc      AUD_STREAM_FROM_OTHERS = 0,         // ex. Pink Noise Generator.
                                   AUD_STREAM_FROM_DIGITAL_TUNER,      // Digital tuner.
                                   AUD_STREAM_FROM_ANALOG_TUNER,       // ex. Analog tuner.
                                   AUD_STREAM_FROM_SPDIF,              // SPDIF-in.
                                   AUD_STREAM_FROM_LINE_IN,            // Line in.
                                   AUD_STREAM_FROM_HDMI,               // HDMI in.
                                   AUD_STREAM_FROM_MEMORY,             // Memory.
                                   AUD_STREAM_FROM_BUF_AGT,            //Buffer Agent.
                                   AUD_STREAM_FROM_MULTI_MEDIA,             // Multi-media

 *  @param  i2Vol  -128 ~ 96
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSrcVol(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T eStrmSrc, INT16 i2Vol);

#ifdef CC_AUD_INPUT_SRC_VOL_OFFSET
//-----------------------------------------------------------------------------
/** AUD_DspGetSrcVolOffset
 *   get source volume offset.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eStrmSrc      AUD_STREAM_FROM_OTHERS = 0,         // ex. Pink Noise Generator.
                                   AUD_STREAM_FROM_DIGITAL_TUNER,      // Digital tuner.
                                   AUD_STREAM_FROM_ANALOG_TUNER,       // ex. Analog tuner.
                                   AUD_STREAM_FROM_SPDIF,              // SPDIF-in.
                                   AUD_STREAM_FROM_LINE_IN,            // Line in.
                                   AUD_STREAM_FROM_HDMI,               // HDMI in.
                                   AUD_STREAM_FROM_MEMORY,             // Memory.
                                   AUD_STREAM_FROM_BUF_AGT,            //Buffer Agent.
                                   AUD_STREAM_FROM_MULTI_MEDIA,             // Multi-media

 *  @retval  input source volume offset
 */
//-----------------------------------------------------------------------------
extern INT16 AUD_DspGetSrcVolOffset(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T eStrmSrc);

//-----------------------------------------------------------------------------
/** AUD_DspSetSrcVolOffset
 *   set source volume offset.
 *
 *  @param  u1DecId  0:first decoder 1: seconder decoder.
 *  @param  eStrmSrc      AUD_STREAM_FROM_OTHERS = 0,         // ex. Pink Noise Generator.
                                   AUD_STREAM_FROM_DIGITAL_TUNER,      // Digital tuner.
                                   AUD_STREAM_FROM_ANALOG_TUNER,       // ex. Analog tuner.
                                   AUD_STREAM_FROM_SPDIF,              // SPDIF-in.
                                   AUD_STREAM_FROM_LINE_IN,            // Line in.
                                   AUD_STREAM_FROM_HDMI,               // HDMI in.
                                   AUD_STREAM_FROM_MEMORY,             // Memory.
                                   AUD_STREAM_FROM_BUF_AGT,            //Buffer Agent.
                                   AUD_STREAM_FROM_MULTI_MEDIA,             // Multi-media

 *  @param  i2Vol  -128 ~ 96
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSrcVolOffset(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T eStrmSrc, INT16 i2Vol);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspVolumeChange
 *   Setup master volume, or input source gain, or channel trim to DSP.
 *
 *  @param  u1DecId    0: first decoder 1: seconder decoder.
 *  @param  eAudChl  : 0: front left channel.
 *                              1: front right channel.
 *                              2: rear left channel.
 *                              3: rear right channel.
 *                              4: center channel.
 *                              5: sub woofer channel.
 *                              6: bypass left channel.
 *                              7: bypass right channel.
 *                              8: downmix left channel.
 *                              9: downmix right channel.
 *                            10: aux front left channel.
 *                            11: aux front right channel.
 *                            12: input source gain.
 *                            13: all channels    ( For master volume control ).
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspVolumeChange(UINT8 u1DspId, UINT8 u1DecId, AUD_CH_T eAudChl);

//-----------------------------------------------------------------------------
/** AUD_DspSetAvSrcVol
 *   set AV source volume.
 *
 *  @param  eInputSrcId         AUD_INPUT_ID_COMP_VID_0 = 0,
                                          AUD_INPUT_ID_COMP_VID_1,
                                          AUD_INPUT_ID_COMP_VID_2,
                                          AUD_INPUT_ID_COMP_VID_3,
                                          AUD_INPUT_ID_S_VID_0,
                                          AUD_INPUT_ID_S_VID_1,
                                          AUD_INPUT_ID_S_VID_2,
                                          AUD_INPUT_ID_YPBPR_0,
                                          AUD_INPUT_ID_YPBPR_1,
                                          AUD_INPUT_ID_YPBPR_2,
                                          AUD_INPUT_ID_VGA_0,
                                          AUD_INPUT_ID_VGA_1,
                                          AUD_INPUT_ID_HDMI_0,
                                          AUD_INPUT_ID_HDMI_1,
                                          AUD_INPUT_ID_HDMI_2,
                                          AUD_INPUT_ID_HDMI_3,
                                          AUD_INPUT_ID_DVI_0,
                                          AUD_INPUT_ID_DVI_1,
                                          AUD_INPUT_ID_DVI_2,
                                          AUD_INPUT_ID_DVI_3,
                                          AUD_INPUT_ID_SCART_0,
                                          AUD_INPUT_ID_SCART_1,
                                          AUD_INPUT_ID_SCART_2,
                                          AUD_INPUT_ID_SCART_3,
                                          AUD_INPUT_ID_AUXIN_0,
                                          AUD_INPUT_ID_AUXIN_1,
 *  @param  i2Vol  -256 ~ 256.
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetAvSrcVol(AUD_INPUT_ID_T eInputSrcId , INT16 i2Vol);

extern AUD_INPUT_ID_T AUD_DspGetAvInputSrcId(void);

//-----------------------------------------------------------------------------
/** AUD_DspGetAvInputSrcDvdId
 *
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern AUD_INPUT_ID_T AUD_DspGetAvInputSrcDvdId(void);

//-----------------------------------------------------------------------------
/** AUD_DspGetAtvTvSys
 *
 *  @param  prTvAudSys  tv audio system structure.
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetAtvTvSys(MW_TV_AUD_SYS_T* prTvAudSys);

//-----------------------------------------------------------------------------
/** AUD_DspGetAtvTvSystem
 *
 *  @param void
 *  @retval  TV_AUD_SYS_T
 */
//-----------------------------------------------------------------------------
extern TV_AUD_SYS_T AUD_DspGetAtvTvSystem(void);

/** AUD_DspGetAtvTvSystemStrength
 *
 *  @param TV_AUD_SYS_T
 *  @retval  UINT32
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetAtvTvSystemStrength(TV_AUD_SYS_T);


//-----------------------------------------------------------------------------
/** AUD_DspSetTvSrcMode
 *   set tv source mode.
 *
 *  @param  eAudSys      SV_NONE_DETECTED = 0x0,
                                   SV_MTS           = 0x1,
                                   SV_FM_FM         = 0x2,
                                   SV_NTSC_M        = 0x3,
                                   SV_A2_BG         = 0x4,
                                   SV_A2_DK1        = 0x5,
                                   SV_A2_DK2        = 0x6,
                                   SV_A2_DK3        = 0x7,
                                   SV_PAL_I         = 0x8,
                                   SV_PAL_BG        = 0x9,
                                   SV_PAL_DK        = 0xa,
                                   SV_SECAM_L       = 0xb,
                                   SV_SECAM_L_PLUM  = 0xc,
                                   SV_SECAM_BG      = 0xd,
                                   SV_SECAM_DK      = 0xe
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetTvSrcMode(TV_AUD_SYS_T eAudSys);
extern TV_AUD_SYS_T AUD_DspGetTvSrcMode(void);
#ifdef CC_AUD_TV_SYS_FINE_TUNE_VOL
extern  void AUD_SetTVSysFineTuneVol(TV_AUD_SYS_T eAudSys);
#endif
//-----------------------------------------------------------------------------
/** AUD_DspSetAtvOutputMode
 *   set tv source mode.
 *
 *  @param  u1DecId 0:first decoder 1: seconder decoder.
 *  @param  eSoundMode         UNKNOWN = 0,
                                           MONO,
                                           STEREO,
                                           SUB_LANG,
                                           DUAL1,
                                           DUAL2,
                                           NICAM_MONO,
                                           NICAM_STEREO,
                                           NICAM_DUAL1,
                                           NICAM_DUAL2,
                                           A2_DUAL1,
                                           A2_DUAL2,
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetAtvOutputMode(UINT8 u1DecId, AUD_SOUND_MODE_T eSoundMode);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspHdmiSpdifOutEnable
 *   SPDIF output control in HDMI mode.
 *
 *  @param  fgEnable    0: disable SPDIF output, 1: enable SPDIF output
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspHdmiSpdifOutEnable(BOOL fgEnable);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspGetSoundMode
 *   Get ATV sound mode
 *
 *  @param  u1DecId 0:first decoder 1: seconder decoder.
 *
 *  @retval AUD_SOUND_MODE_T
 */
//-----------------------------------------------------------------------------
extern AUD_SOUND_MODE_T AUD_DspGetSoundMode(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspGetSpectrumInfo
 *   Get audio spectrum information.
 *
 *  @param  u1DecId 0:first decoder 1: seconder decoder.
 *  @param  u1BandNum      1~15 .
 *  @retval  u4Spectrum
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetSpectrumInfo(UINT8 u1DecId, UINT8 u1BandNum);


//-----------------------------------------------------------------------------
/** AUD_DspSpectrumEnable
 *   Enable audio spectrum calculation.
 *
 *  @param  u1DecId    0:first decoder 1: seconder decoder.
 *  @param  fgEnable    0: disable calculation, 1: enable calculation.
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpectrumEnable(UINT8 u1DecId, BOOL fgEnable);


//-----------------------------------------------------------------------------
/** AUD_AudioSetStcDiffBound
 *   Set STC PTS difference bound. Bound would be (x MS * (1+uStcDiff)).
 *
 *  @param  u1DecId      0:first decoder 1: seconder decoder.
 *  @param  uStcDiffLo    for low boundary of stc diff (0~255) .
 *  @param  uStcDiffHi    for high boundary of stc diff (0~255) .
 *  @param  uStcDiffWs   for worst boundary of stc diff (0~255) .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_AudioSetStcDiffBound(UINT8 u1DecId, UINT8 uStcDiffLo, UINT8 uStcDiffHi, UINT8 uStcDiffWs);


//-----------------------------------------------------------------------------
/** AUD_DDBannerEnable
 *   Turn on this funtion to close some effect and post processing,  for DD test only.
 *
 *  @param  u1DecId      0:first decoder 1: seconder decoder.
 *  @param  fgEnable     0: disable, 1: enable .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DDBannerEnable(UINT8 u1DecId, BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_GetDDBannerEnable
 *   Get Dolby banner information.
 *
 *  @param  u1DecId      0:first decoder 1: seconder decoder.
 *  @param  fgEnable     0: disable, 1: enable .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_GetDDBannerEnable (void);

//-----------------------------------------------------------------------------
/** AUD_DspSetPalDetection
 *   demodulation related configuration.
 *
 *  @param  eChangedItem
 *  @param  u2Value
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetPalDetection ( AUD_FACTORY_PAL_DETECTION_ITEM_T eChangedItem, UINT16 u2Value);



//-----------------------------------------------------------------------------
/** AUD_DspSetPalDetection
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetPalDetection ( AUD_FACTORY_PAL_DETECTION_ITEM_T eItem);


//-----------------------------------------------------------------------------
/** AUD_DspSetA2Detection
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetA2Detection ( AUD_FACTORY_A2_DETECTION_ITEM_T eItem, UINT16 u2Value);



//-----------------------------------------------------------------------------
/** AUD_DspGetA2Detection
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetA2Detection ( AUD_FACTORY_A2_DETECTION_ITEM_T eItem);



//-----------------------------------------------------------------------------
/** AUD_DspSetA2Threshold
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetA2Threshold ( AUD_FACTORY_A2_THRESHOLD_ITEM_T eItem, UINT16 u2Value);



//-----------------------------------------------------------------------------
/** AUD_DspGetA2Threshold
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetA2Threshold ( AUD_FACTORY_A2_THRESHOLD_ITEM_T  eItem);



//-----------------------------------------------------------------------------
/** AUD_DspSetMtsDetection
 *   Set NPTV MTS Detection Configurations.
 *
 *  @param  eChangedItem
 *  @param  u2Value
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetMtsDetection ( AUD_FACTORY_MTS_DETECTION_ITEM_T eChangedItem, UINT16 u2Value)  ;



//-----------------------------------------------------------------------------
/** AUD_DspGetMtsDetection
 *   Set NPTV MTS Detection Configurations.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetMtsDetection ( AUD_FACTORY_MTS_DETECTION_ITEM_T eItem);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSetFmfmDetection
 *   demodulation related configuration.
 *
 *  @param  eChangedItem
 *  @param  u2Value
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
void AUD_DspSetFmfmDetection ( AUD_FACTORY_FMFM_DETECTION_ITEM_T eChangedItem, UINT16 u2Value);


//-----------------------------------------------------------------------------
/** AUD_DspGetFmfmDetection
 *   demodulation related configuration.
 *
 *  @param  eItem
 *
 *  @retval
 */
//-----------------------------------------------------------------------------
extern UINT16 AUD_DspGetFmfmDetection ( AUD_FACTORY_FMFM_DETECTION_ITEM_T eItem) ;
#endif

#ifndef CC_AUD_WAFFLE_OVM_SUPPORT
//-----------------------------------------------------------------------------
/** AUD_DspSetHdevMode
 *   set high deveation mode on/off.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  fgEnable
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetHdevMode ( AUD_ATV_DECODER_TYPE_T eDecType, UINT16 u2Mode);

//-----------------------------------------------------------------------------
/** AUD_DspGetHdevMode
 *   set high deveation mode on/off.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  fgEnable
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetHdevMode ( AUD_ATV_DECODER_TYPE_T eDecType);
#else
//-----------------------------------------------------------------------------
/** AUD_DspSetHdevMode
 *   set high deveation mode on/off or off/low/high.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  u1Mode
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetHdevMode ( AUD_ATV_DECODER_TYPE_T eDecType, UINT8 u1Mode);

//-----------------------------------------------------------------------------
/** AUD_DspGetHdevMode
 *   set high deveation mode on/off.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  fgEnable
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetHdevMode ( AUD_ATV_DECODER_TYPE_T eDecType);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspSetAmMute
 *   set AM mute mode on/off and thresholds.
 *
 *  @param  eChangedItem
 *  @param  u1Value
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetAmMute( AUD_FACTORY_PAL_AM_MUTE_ITEM_T eChangedItem, UINT8 u1Value);


//-----------------------------------------------------------------------------
/** AUD_DspGetAmMute
 *   set AM mute mode on/off and thresholds.
 *
 *  @param  eItem
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetAmMute( AUD_FACTORY_PAL_AM_MUTE_ITEM_T eItem);

//-----------------------------------------------------------------------------
/** AUD_DspSetFmMute
 *   set FM mute mode on/off and thresholds.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  eChangedItem
 *  @param  u1Value
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFmMute ( AUD_ATV_DECODER_TYPE_T eDecType,
                               AUD_FACTORY_FM_MUTE_ITEM_T eChangedItem,
                               UINT8 u1Value) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetFmMute
 *   set FM mute mode on/off and thresholds.
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  eChangedItem
 *  @param  u1Value
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetFmMute ( AUD_ATV_DECODER_TYPE_T eDecType,
                               AUD_FACTORY_FM_MUTE_ITEM_T eChangedItem) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetCarrierShiftMode
 *   set carrier shift mode on/off .
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *  @param  fgEnable
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetCarrierShiftMode ( AUD_ATV_DECODER_TYPE_T eDecType, BOOL fgEnable);


//-----------------------------------------------------------------------------
/** AUD_DspGetCarrierShiftMode
 *   set carrier shift mode on/off .
 *
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *
 *  @retval  TRUE/FAUSE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetCarrierShiftMode (AUD_ATV_DECODER_TYPE_T eDecType);

//-----------------------------------------------------------------------------
/** AUD_DspSetNonEUMode
 *   set Non-EU mode on/off.
 *
 *  @param  fgEnable    TRUE/FALSE
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetNonEUMode (BOOL fgEnable) ;


//-----------------------------------------------------------------------------
/** AUD_DspGetNonEUMode
 *   Get Non-EU mode on/off.
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetNonEUMode (void);


//-----------------------------------------------------------------------------
/** AUD_DspSetFMSatuMuteMode
 *   demodulation related configuration.
 *  @param  fgEnable    TRUE/FALSE
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFMSatuMuteMode (AUD_ATV_DECODER_TYPE_T eDecType, BOOL fgEnable) ;



//-----------------------------------------------------------------------------
/** AUD_DspGetFMSatuMuteMode
 *   demodulation related configuration.
 *  @param  eDecType
 *          bit0 : PAL
 *                  bit1 : A2
 *                  bit2 : MTS
 *                  bit3 : FMFM
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetFMSatuMuteMode (AUD_ATV_DECODER_TYPE_T eDecType) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetPALFineVolume
 *   Set fine tune volume for PAL.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetPALFineVolume (UINT8 u1Index) ;


//-----------------------------------------------------------------------------
/** AUD_DspGetPALFineVolume
 *   Get fine tune volume for PAL.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetPALFineVolume (void) ;


//-----------------------------------------------------------------------------
/** AUD_DspSetNicamFineVolume
 *   Set fine tune volume for NICAM.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetNicamFineVolume (UINT8 u1Index) ;


//-----------------------------------------------------------------------------
/** AUD_DspGetNicamFineVolume
 *   Get fine tune volume for NICAM.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetNicamFineVolume (void) ;


//-----------------------------------------------------------------------------
/** AUD_DspSetAMFineVolume
 *   Set fine tune volume for AM.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetAMFineVolume (UINT8 u1Index);

//-----------------------------------------------------------------------------
/** AUD_DspGetAMFineVolume
 *   Get fine tune volume for AM.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetAMFineVolume (void) ;


//-----------------------------------------------------------------------------
/** AUD_DspSetA2FineVolume
 *   Set fine tune volume for A2.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetA2FineVolume (UINT8 u1Index) ;


//-----------------------------------------------------------------------------
/** AUD_DspGetA2FineVolume
 *   Get fine tune volume for A2.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetA2FineVolume (void) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetMtsMonoFineVolume
 *   Set fine tune volume for MTS mono.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetMtsMonoFineVolume (UINT8 u1Index) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetMtsMonoFineVolume
 *   Get fine tune volume for MTS mono.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetMtsMonoFineVolume (void) ;


//-----------------------------------------------------------------------------
/** AUD_DspSetSAPFineVolume
 *   Set fine tune volume for SAP.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSAPFineVolume(UINT8 u1Index);


//-----------------------------------------------------------------------------
/** AUD_DspGetSapFineVolume
 *   Get fine tune volume for SAP.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSapFineVolume (void) ;

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSetFmfmMonoFineVolume
 *   Set fine tune volume for FMFM mono.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFmfmMonoFineVolume (UINT8 u1Index) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetFmfmMonoFineVolume
 *   Get fine tune volume for FMFM mono.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetFmfmMonoFineVolume (void) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetFmfmDualFineVolume
 *   Set fine tune volume for FMFM dual.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFmfmDualFineVolume(UINT8 u1Index);

//-----------------------------------------------------------------------------
/** AUD_DspGetFmfmDualFineVolume
 *   Get fine tune volume for FMFM dual.
 *
 *  @retval  u1Index (0~40)
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetFmfmDualFineVolume (void) ;
#endif

//-----------------------------------------------------------------------------
/** AUD_DspSetFmRadioFineVolume
 *   Set fine tune volume for FM radio.
 *
 *  @param  u1Index      0~40 .
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFmRadioFineVolume (UINT8 u1Index) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetMtsPilotDetection
 *   set MTS pilot offset detection on/off.
 *  @param  fgEnable    TRUE/FALSE
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetMtsPilotDetection (BOOL fgEnable) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetMtsPilotDetection
 *   get MTS pilot offset detection on/off.
 *  @param  void
 *  @retval  BOOL
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetMtsPilotDetection (void) ;

//-----------------------------------------------------------------------------
/** AUD_DspSetSapMute
 *   set SAP noise mute thresholds
 *  @param  eChangedItem    lower threshold or higher threshold
 *  @param  u1Value
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetSapMute( AUD_FACTORY_SAP_MUTE_ITEM_T eChangedItem, UINT8 u1Value);

//-----------------------------------------------------------------------------
/** AUD_DspGetSapMute
 *   get SAP noise mute thresholds
 *  @param  eItem lower threshold or higher threshold
 *  @retval  UINT8
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetSapMute( AUD_FACTORY_SAP_MUTE_ITEM_T eItem);

//-----------------------------------------------------------------------------
/** AUD_DspSetFMSatuMuteMode
 *   Set saturation mute thresholds
 *  @param  eItem lower threshold or higher threshold
 *  @retval  UINT8
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetFMSatuMuteTh ( AUD_FACTORY_SATU_MUTE_ITEM_T eChangedItem,
                                     UINT8 u1Value) ;

//-----------------------------------------------------------------------------
/** AUD_DspGetFMSatuMuteTh
 *   get saturation mute thresholds
 *  @param  eItem lower threshold or higher threshold
 *  @retval  UINT8
 */
//-----------------------------------------------------------------------------
extern UINT8 AUD_DspGetFMSatuMuteTh( AUD_FACTORY_SATU_MUTE_ITEM_T eItem);

//-----------------------------------------------------------------------------
// ADAC_Init
//
/** Initial audio codec and digital amp.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_Init(void);

#ifdef DIGITAL_I2S_AMP_REINIT
//-----------------------------------------------------------------------------
// I2S_AMP_Init
//
/** Initial Audio I2S digital amplifier.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void I2S_AMP_Init(void);
#endif

//-----------------------------------------------------------------------------
// ADAC_Enable
//
/** Audio codec and amp function table init.
 *
 *  @param[in]     fgEnable   TRUE for initial.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_Enable(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_Mute
//
/** Audio codec and speaker mute.
 *
 *  @param[in]     fgEnable   "TRUE" for mute,"FALSE" for unmute.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_Mute(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_MuteCLK
//
/** Audio codec Mute PWM CLK - PWM Fade-in / Fade-out
 *
 *  @param[in]     fgEnable   "TRUE" for mute,"FALSE" for unmute.
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_MuteCLK(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_DacFormatCfg
//
/** Setting DAC Data Format.
 *
 *  @param[in]     u1DecId   Decoder ID.
 *  @param[in]     eFormat   Audio data format,ex:LJ,RJ,I2S.
 *  @param[in]     eMclk   MCLK frequency.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_DacFormatCfg(UINT8 u1DecId, DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk)  ;


//-----------------------------------------------------------------------------
// ADAC_AdcFormatCfg
//
/** Setting Codec Adc Data Format.
 *
 *  @param[in]     u1DecId   Decoder ID.
 *  @param[in]     eFormat   Audio data format,ex:LJ,RJ,I2S.
 *  @param[in]     eMclk   MCLK frequency.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_AdcFormatCfg(UINT8 u1DecId, DATA_FORMAT_T eFormat, MCLK_FREQUENCY_T eMclk)  ;

//-----------------------------------------------------------------------------
// ADAC_HeadphoneVolCtl
//
/** Setting Codec Adc Data Format.
 *
 *  @param[in]     i2VolInHalfDb   volume value.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_HeadphoneVolCtl(INT16 i2VolInHalfDb);

//-----------------------------------------------------------------------------
// ADAC_GetHeadphoneVolMaxMin
/**  Setting the max and min value of headphone volume.
*
*  @param[in]     pi2Max   volume max value.
*  @param[in]     pi2Min   volume min value.
*
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_GetHeadphoneVolMaxMin(INT16* pi2Max, INT16* pi2Min);

//-----------------------------------------------------------------------------
// ADAC_DacChlSel
//
/** Configure DAC output channel
 *
 *  @param[in]     u1DacId: Dac ID, eChl: output channel
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_DacSetChl(UINT8 u1DacId, AUD_CHL_DEF_T eChl);

//-----------------------------------------------------------------------------
// ADAC_DacChlSel
//
/** Get Dac output channel  config
 *
 *  @param[in]     void
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_DacGetChl(void);

//-----------------------------------------------------------------------------
// ADAC_ADCDown
//
/** Power on/down ADC
 *
 *  @param[in]     fgEnable: True: power on, FALSE: power off
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_ADCDown(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_VolumeMuteSpeakerEnable
//
/** Mute speaker when volume value equal to zero.
 *
 *  @param[in]     fgFlag   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_VolumeMuteSpeakerEnable(BOOL fgFlag);

//-----------------------------------------------------------------------------
// ADAC_DcDetectSpkEnable
//
/** Detect the DC and mute speaker.
 *
 *  @param[in]     fgFlag   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_DcDetectSpkEnable(BOOL fgFlag);

#if defined(CC_AUD_SX1_FEATURE) || defined (CC_MAPLE_CUST_DRV)
//-----------------------------------------------------------------------------
// ADAC_SpkHpLinkEnable
//
/** Speaker & headphone link enable.
 *
 *  @param[in]     fgFlag   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_SpkHpLinkEnable(BOOL fgFlag);
extern BOOL ADAC_GetSpkHpLinkFlag(void);
#endif

//-----------------------------------------------------------------------------
// ADAC_HpDetectSpkEnable
//
/** Detect the headphone plug in and mute speaker.
 *
 *  @param[in]     fgFlag   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_HpDetectSpkEnable(BOOL fgFlag);

//-----------------------------------------------------------------------------
// ADAC_HpDetectCodecMute
//
/** Detect the headphone plug in and mute codec.
 *
 *  @param[in]     fgMute   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_HpDetectCodecMute(BOOL fgMute);


//-----------------------------------------------------------------------------
// ADAC_HpDetectDacMute
//
/** Detect the headphone plug in and mute internaldac.
 *
 *  @param[in]     fgMute   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_HpDetectDacMute(BOOL fgMute);

//-----------------------------------------------------------------------------
// ADAC_ClkChangeCodecMute
//
/** Some codec have to mute before clock source change
 *
 *  @param[in]     fgMute   "TRUE" for enable this function.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_ClkChangeCodecMute(BOOL fgMute);

//-----------------------------------------------------------------------------
// ADAC_SpeakerEnable
//
/** Speaker enable routine.
 *
 *  @param[in]     fgEnable   "TRUE" for enable, "FALSE" for mute speaker.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_SpeakerEnable(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_BtSpeakerEnable
//
/** Speaker enable routine.
 *
 *  @param[in]     fgEnable   "TRUE" for enable, "FALSE" for mute speaker.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_BtSpeakerEnable(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_SpeakerState_Reinit
//
/** Speaker State Reinit
 *
 *  Call at _SetOutPort for pwmdac de-pop noise flowchart which make
 *  - System boot & headphone plug-in, speaker un-mute (I2S non-mute)
 *  - UI Audio Speaker:Off & system re-boot speaker un-mute (I2S non-mute)
 *
 */
//-----------------------------------------------------------------------------
extern void ADAC_SpeakerState_Reinit(void);

//-----------------------------------------------------------------------------
// ADAC_SpeakerStateQuery
//
/** Speaker enable query function.
 *
 *  @param[in]     void.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_SpeakerStateQuery(void);

//-----------------------------------------------------------------------------
// ADAC_CodecMute
//
/** Codec enable routine.
 *
 *  @param[in]     fgMute   "TRUE" for enable, "FALSE" for mute codec.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_CodecMute(BOOL fgMute);


//-----------------------------------------------------------------------------
// ADAC_GpioCodecMute
//
/** Codec GPIO control for mute/unmute.
 *
 *  @param[in]     fgEnable   "TRUE" for enable, "FALSE" for mute codec from GPIO.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_GpioCodecMute(BOOL fgEnable);


//-----------------------------------------------------------------------------
// ADAC_GpioAmpEnable
//
/** Speaker GPIO control for mute/unmute.
 *
 *  @param[in]     fgEnable   "TRUE" for enable, "FALSE" for mute speaker from GPIO.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_GpioAmpEnable(BOOL fgEnable);
extern void ADAC_GpioAmpEnable1(BOOL fgEnable);

//-----------------------------------------------------------------------------
// ADAC_CodecAdChlSel
//
/** Codec ad channel select.
 *
 *  @param[in]     u1Chl   Input data channel.
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void ADAC_CodecAdChlSel(UINT8 u1Chl);

//-----------------------------------------------------------------------------
// ADAC_CodecRegRead
//
/** Read register of external codec.
 *
 *  @param[in]     u1Offset   Address.
 *
 *  @retval     UINT8   Data.
 */
//-----------------------------------------------------------------------------
extern UINT8 ADAC_CodecRegRead(UINT8 u1Offset);

//-----------------------------------------------------------------------------
// ADAC_CodecRegWrite
//
/** Write register of external codec.
 *
 *  @param[in]     u1Offset     Address.
 *  @param[in]     u1Data       Data.
 *
 *  @retval     void.
 */
//-----------------------------------------------------------------------------
extern void ADAC_CodecRegWrite(UINT8 u1Offset, UINT8 u1Data);


//-----------------------------------------------------------------------------
// ADAC_AmpRegRead
//
/** Read register of external digital amp.
 *
 *  @param[in]     u1Offset   Address.
 *
 *  @retval     UINT8   Data.
 */
//-----------------------------------------------------------------------------
extern UINT8 ADAC_AmpRegRead(UINT8 u1Offset);

//-----------------------------------------------------------------------------
// ADAC_AmpRegWrite
//
/** Write register of external digital amplify.
 *
 *  @param[in]     u1Offset     Address.
 *  @param[in]     u1Data       Data.
 *
 *  @retval     void.
 */
//-----------------------------------------------------------------------------
extern void ADAC_AmpRegWrite(UINT8 u1Dev_Id,UINT8 u1Offset, UINT8 u1Data);

extern void ADAC_AmpInit(void);
extern void ADAC_AmpMute(UINT8 u1Data);

extern BOOL AUD_SetSPDIFEnable(BOOL fgEnable);

#ifdef CC_FAST_INIT
//-----------------------------------------------------------------------------
/** AUD_pm_resume
 *  Resume function.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_pm_resume(void);

//-----------------------------------------------------------------------------
/** AUD_pm_suspend
 *  suspend function.
 *
 *  @param
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_pm_suspend(void);
#endif


//-----------------------------------------------------------------------------
// AUD_FeederSetRequestFunc
//
/** For feeder to set request function.
 *
 *  @param     prTransmit   Notify function handler
 *
 *  @retval    TRUE   ok
 *  @retval    FALSE  fail
 */
//-----------------------------------------------------------------------------
//extern BOOL AUD_FeederSetRequestInfo(FEEDER_TRANSMIT *prTransmit);
extern void AUD_GetRWPtr(UINT8 u1DspId, UINT8 u1DecId, UPTR * pu4Rp, UPTR * pu4Wp, UINT32 *pu4Size);
extern void AUD_GetStreamInfo(UINT8 u1DspId, UINT8 u1DecId, UINT8 *pu1Acmode, UINT8 *pu1SmpRate, UINT32 * pu4DataRate);
extern void AUD_GetStreamInfo_AC3(UINT8 u1DecId, UINT8* pu1Acmode, UINT8* pu1Ac3DecType, UINT8* pu1SmpRate, UINT32* pu4DataRate);
extern void AUD_StrSelect(UINT8 u1DspId, UINT8 u1DecId, UINT8 u1Str, MEM_BUFFER_INFO_T *prMemBuf);
extern void AUD_ToneGen(UINT8 u1DspId, UINT8 u1DecId, INT32 i4Fc[2], MEM_BUFFER_INFO_T *prMemBuf);
extern void AUD_UniversalTonePlay(INT32 i4LeftFreq, INT32 i4RightFreq);
extern void AUD_DspCfgSetDecType(UINT8 u1DecId, AUD_DEC_STREAM_FROM_T eStreamFrom, AUD_FMT_T eDecFmt);
extern void _AUD_DspDolbyGainTB11(UINT8 u1DecId);

extern void _AUD_OutPadEnable(UINT8 u1DecId, BOOL fgEnable);
#if defined(CC_PHX_CUST)||defined(CC_LMP_CUST)
extern void _AUD_OutPadDataEnable(BOOL fgEnable);
#endif
extern BOOL AUD_SetStartPts(UINT8 u1DecId, UINT64 u8Pts);

extern void AudShowDspStatus(UINT8 u1DspId);
extern void AudShowDolbyInfo(void);
extern void AudShowMtsStatus(UINT8 u1DecId);
#if 1///def CC_VORBIS_SUPPORT    ///CC_VORBIS_SUPPORT_Q
extern void AudShowVorbisStatus(UINT8 u1DecId);
#endif
extern void AudShowStatus(UINT8 u1DspId, UINT8 u1DecId);
extern void AUD_ChlOutDump(UINT8 u1DspId, UINT8 u1Chl);
extern UINT32 AUD_GetUnderFlowCnt(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetVolumeTable
 *   Set volume table.
 *
 *  @param  u4VolTabChkPnt[]
 *
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetVolumeTable(UINT32* u4VolTabChkPnt);
extern void AUD_SetOutPortMode(UINT8 u1DecId, UINT8 fgMode);
#if defined(CC_PHX_CUST)||defined(CC_LMP_CUST)
extern AUD_DEC_OUTPORT_MODE_T AUD_GetOutPortMode(void);
#endif
#ifdef CC_AUD_LINEOUT_VOL_MODE
extern void AUD_DspSetLineOutVolumeMode(UINT8 u1DecId, UINT8 fgMode);
extern UINT8 AUD_DspGetLineOutFixVol(void);
extern BOOL AUD_GetLineOutVolumeMode(UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspGetVolumeTable
 *   Query volume table.
 *
 *  @param  u1VolTabIdx
 *
 *  @retval  volume gain
 */
//-----------------------------------------------------------------------------
#ifdef CC_AUD_NEW_HP_VOL_TABLE
extern void AUD_DspSetHpVolumeTable(UINT8 u1Idx, UINT32 u4Vol);
extern UINT32 AUD_DspGetHpVolumeTable(UINT8 u1VolTabIdx);
#endif
extern UINT32 AUD_DspGetVolumeTable(UINT8 u1VolTabIdx);
extern UINT32 AUD_DspSpkSizeQry(UINT8 u1DecId);
#ifdef CC_S_SUBWOFFER_SUPPORT  ///CC_BASS_MANGER_INDEP_LPF
extern UINT32 AUD_DspSpkSizeLpfQry(UINT8 u1DecId);
#endif

//-----------------------------------------------------------------------------
// AUD_DrvSetDDBanner
//
/** Turn on DDBanner.
 *
 *  @param     fgEnable     TRUE/FALSE
 *
 *  @retval    void
 */
//-----------------------------------------------------------------------------
extern void AUD_DrvSetDDBanner(BOOL fgEnable);

extern void Aud_SetTvSystemMask(UINT8 u1DecId, UINT32 u4Mask);
extern UINT32 Aud_GetTvSystemMask(UINT8 u1DecId);
extern void AUD_DspADModeSet(UINT8 u1DecId, UINT16 u2Value);
extern void AUD_DspADEnable(UINT8 u1DecId, BOOL fgEnable);

#ifdef CC_AUD_PCM_LINE_IN_LPF
extern void AUD_DSPSetPCMLineInLPF(BOOL fgEnable);
#endif

extern void AUD_DspSetPcmPreBuffer (UINT8 u1DspId, UINT8 u1DecId, UINT16 u2Length);
extern UINT16 AUD_DspGetPcmPreBuffer(UINT8 u1DspId, UINT8 u1DecId);
#ifdef CC_SUPPORT_AUTO_DETECT_AUD_SYSTEM
extern void Aud_SetTvAudSystemMask(UINT8 u1DecId, UINT32 u4TvMask, UINT32 u4TvAudMask, ISO_3166_COUNT_T country_code);
#ifdef CC_AUD_DDI
extern AUD_FMT_T AudAtvFmtDetection(UINT8 u1DecId);
#else
extern AUD_FMT_T AudAtvFmtDetection(void);
#endif
#endif
extern void AUD_PCM_Detect_Raw_Mute(BOOL fgFlag);
extern BOOL AUD_Is_PCMDetectRaw(void);

//-----------------------------------------------------------------------------
/** AUD_Surround_Get_Config
 *   Set new mtk vsurr config
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_Surround_Get_Config(AUD_VSURR_CFG_T* prVsurrCfg);

//-----------------------------------------------------------------------------
/** AUD_Surround_Set_Config
 *   Set new mtk vsurr config
 *  @retval  void
 */
//-----------------------------------------------------------------------------
extern void AUD_Surround_Set_Config(AUD_VSURR_CFG_TYPE_T eVSCfgType, UINT32 u4VSParameter);

//-----------------------------------------------------------------------------
/** AUD_SendApllAdjustCmd
 *  Send APLL MODIN adjust command
 *
 *  @param  u4Level    video calculated level (0 ~ 255. 128 as center)
 *                     bit 8: 0 (slower), 1 (faster)
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_SendApllAdjustCmd(UINT32 u4Level);

//-----------------------------------------------------------------------------
/** AUD_GetNicamStatus
 *  Get Nicam exists or not
 *
 *  @param  void
 *
 *  @retval TRUE/FALSE
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_GetNicamStatus(void);

//-----------------------------------------------------------------------------
/** AUD_GetDtvAudInfo
 *  Get DTV info
 *
 *  @param  u1DecId
 *  @param  prAudInfo
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
void AUD_GetDtvAudInfo(UINT8 u1DecId, AUD_DEC_INFO_T* prAudInfo);
void AUD_GetDTSAudInfo(UINT8 u1DecId, AUD_DEC_INFO_T* prAudInfo);


//-----------------------------------------------------------------------------
/** AUD_DspSpeed
 *  Set dsp speed (non-fractional)
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  u1Speed  : 0:normal play
 *                            : 2:FF 2-times
 *                            : 4:FF 4-times
 *                            : 8:FF 8-times
 *                            : 16:FF 16-times
 *                            : 32:FF 32-times
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSpeed(UINT8 u1DecId, UINT16 u2Speed);

//-----------------------------------------------------------------------------
/** AUD_DspDownmixPosition
 *  Set Downmix Channel (CH9/10) Position
 *
 *  @param  u1DMPosition   : 0: do ALL post processing
 *                         : 1: do downmix after input source gain and before AVC
 *                         : 2: Skip post-proc & do manual output matrix
 *                         : 3: Skip post-proc & skip manual output matrix
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspDownmixPosition(UINT8 u1DspId, UINT8 u1DMPosition);
extern UINT8 AUD_DspGetDownmixPosition(UINT8 u1DspId);

//-----------------------------------------------------------------------------
/** AUD_DspMultiPairOutput
 *  Copy CH9/10 to other channel(LS/RS/C/SUB)
 *
 *  @param  u1DecId 0       : first decoder 1: seconder decoder.
 *  @param  u1MultiPair   : 0: OFF
 *                                   : 1: ON

 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspMultiPairOutput(UINT8 u1DspId, UINT8 u1DecId, UINT16 u1MultiPair);

//-----------------------------------------------------------------------------
/** _AUD_DspLRDownmix
 *  Downmix to L/R channel after Reverbation and before BassManagement
 *
 *  @param  u1DecId 0       : first decoder 1: seconder decoder.
 *  @param  u1MultiPair   : 0: OFF
 *                                   : 1: ON

 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspLRDownmix(UINT8 u1DspId, UINT8 u1DecId, BOOL fgEnable);
extern UINT16 AUD_DspGetSystemSetup(UINT8 u1DspId, UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_SetLpcmTable
 *  Set Lpcm decoder parameters
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  AUD_LPCM_SETTING_T
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_SetLpcmTable(UINT8 u1DecId, AUD_LPCM_SETTING_T rLpcmSetting);

extern UINT32 AUD_GetLPCMType(void);

//-----------------------------------------------------------------------------
// AUD_DrvAc3Setting
//
/** AC3 format setting
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      prPcmSetting
 */
//-----------------------------------------------------------------------------
extern void AUD_DrvAc3Setting(UINT8 u1DecId, const AUD_AC3_SETTING_T* prAc3Setting);

//-----------------------------------------------------------------------------
// AUD_DrvFlacSetting
//
/** FLAC format setting
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 *  @param      prPcmSetting
 *  paul_flac
 */
//-----------------------------------------------------------------------------
extern void AUD_DrvFlacSetting(UINT8 u1DecID, AUD_FLAC_INFO_T* prFlacInfo);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSetDualDecMode
 *
 *  @param  enable/disable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetDualDecMode(BOOL fgEnable);
#endif

#ifdef CC_AUD_LINEIN_USE_DEC3
extern void AUD_DspSetTriOnlyDecMode(BOOL fgEnable);
#endif
extern BOOL AUD_DspIsTriOnlyDecMode(void);


#ifdef CC_AUD_CLIPPER_SUPPORT
//-----------------------------------------------------------------------------
/** _AUD_DspSetClipperEnable
 *
 *  @param  enable/disable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetClipperEnable(BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspSetClipperValue
 *
 *  @param  vaeChannel
 *  @param  u4Value
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetClipperValue(AUD_CH_T eChannel, UINT32 u4Value);
#endif

//-----------------------------------------------------------------------------
/** _AUD_DspSetPEQEnable
 *
 *  @param  enable/disable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspGetPEQEnable(void);
extern void AUD_DspSetPEQEnable(BOOL fgEnable);


//-----------------------------------------------------------------------------
/** AUD_DspSetPEQCfg
 *
 *  @param u1DecId                    Decoder Id (AUD_DEC_MAIN or AUD_DEC_AUX).
 *  @param u1Set                      PEQ set (0 ~ 7)
 *  @param u4Frequency                Frequency (20 ~ 20K)
 *  @param eQValue                    Q
 *  @param i2Gain                     Gain (+-12dB) with 0.5dB/step
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetPEQCfg(UINT8 u1DecId, UINT8 u1PEQSet, UINT32 u4Frequency,
                              UINT32 eQValue, INT16 i2Gain);
extern void AUD_DspGetPEQCfg(UINT8 u1DecId, UINT8 u1PEQSet, UINT32* u4Frequency,
                              UINT32* eQValue, UINT32* u4Gain);

#ifdef CC_AUD_VBASS_SUPPORT
//-----------------------------------------------------------------------------
/** AUD_DspSetVirBassEnable
 *
 *  @param  enable/disable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetVirBassEnable(BOOL fgEnable);

extern void AUD_DspSetVirBassCfg(UINT8 u1DecId, UINT32 u4Frequency, UINT32 u4Gain,
                                 UINT32 u4Gain2);
extern VOID AUD_SetVBassCutOff(UINT8 u1Idx);
extern UINT8 AUD_GetVBassCutOff(void);
extern void AUD_DspGetVirBassCfg(UINT8 u1DecId, BOOL* fgEnable, UINT32* u4Gain, UINT32* u4Gain2);
#endif


extern void AUD_SetHeadphoneCond(UINT8 u1DecId, UINT8 u1HPCond);

extern UINT8 AUD_GetHeadphoneCond(UINT8 u1DecId);

#ifdef CC_AUD_USE_NVM
extern UINT32 AUD_NVM_Get_Profile(SMART_AUD_GROUP_INFO_T* prSmartAudioInfo);
extern UINT8 AUD_NVM_Get_Profile_Id(void);
extern void AUD_NVM_Get_FlashAQ_List(UINT8 u1Profile, UINT8 *pu1Data);
extern UINT32 AUD_NVM_Read(void);
#endif


extern void AUD_DspDDCOEnable(BOOL fgEnable);

extern void AUD_DspDDCOAgcEnable(BOOL fgEnable);

extern void AUD_DspDDCOLFEEnable(BOOL fgEnable);

//#ifdef ENABLE_AUDIO_RECORD_SUPPORT
extern void AUD_DspUploadDataEnable(void);
extern void AUD_DspSetUploadMode(UINT8 u1Mode);
extern void AUD_DataUpload_Init (void);
extern void AUD_DataUpload_start(void);
extern void AUD_GetUploadInfo(UINT32* u4Pts, UINT32* u4Addr, UINT32* u4Size);
extern void AUD_GetYWUploadInfo(UINT32* u4base, UINT32* u4Size);

extern void AUD_GetUploadPCMInfo(UINT32* uPCMBits, UINT32* uPCMChannels, UINT32* uPCMSampleRate);

extern void AUD_DspSetEncMode(UINT8 u1DecId, AUD_ENC_MODE_T eEncMode);
extern void AUD_DspSetEncInfo(UINT8 u1DecId, UINT32 u4Bitrates);
#ifdef CC_MT5895
extern UINT32 AUD_GetEarcDataRate(UINT8 u1DecId);
#if 0
extern void AUD_HDMIRxBypassAout1(AUD_AOUT1_BYPASS eType);
extern AUD_AOUT1_BYPASS AUD_GetHDMIRxBypassAout1(void);
#endif
#endif
//-----------------------------------------------------------------------------
// AUD_ReadDspDROM
//
/** Read DROM 24-bits data.
 *
 *  @param[in]        u4Addr           read DROM offset
 *
 *  @retval DROM value
 */
//-----------------------------------------------------------------------------
UINT16 AUD_ReadDROM (UINT8 u1DspId, UINT32 u4Addr);

//-----------------------------------------------------------------------------
// AUD_Read32ByteID
//
/** Read ROM 32 bytes ID.
 *
 *  @param[in]        u1ID_p           ROM ID pointer
 *                    u4Num            ROM ID byte number
 *
 *  @retval AUD_OK    Success
 *  @retval AUD_FAIL  Fail
 */
//-----------------------------------------------------------------------------
extern INT32 AUD_Read32ByteID (UINT8 u1DspId, UINT8 *u1ID_p, UINT32 u4Num);
extern void AUD_MWCtrl_Set(UINT8 u1DecId, BOOL fgPlay);
extern BOOL AUD_MWCtrl_Get(UINT8 u1DecId);


extern void AUD_GetDspPtsInfo (UINT8 u1DecId, UINT32* pu4DspPts, UINT32* pu4PtsPrsPnt);
extern void AUD_GetAudioTrickPts(UINT8 u1DecId, UINT64* prAudioPts, BOOL fgOldPts);
#ifdef CC_AUD_HDMI_SPDIF_CFG    ///[Joel] 20101011
extern void AUD_HdmiCfgAnalogOutput(AUD_OUTPUT_STATE u1OutputState);
extern void AUD_CfgSpdifByHDMI(AUD_HDMI_CHL_STATUS* uChlStatus);
extern void AUD_SpdifByHDMINtfy(void);
#endif

///For MTA mt_aud_mw.c use.
extern void AUD_SPDIF_Output_HDMI_Rx(BOOL fgEnable);
extern void AUD_HDMIRxBypassAout3(AUD_AOUT3_BYPASS eType);
extern AUD_AOUT3_BYPASS AUD_GetHDMIRxBypassAout3(void);
extern INT16 AUD_GetAdjustDelayResult(void);
extern void AUD_AinSetHBRMode(UINT8 u1Mode);

#ifdef CC_AUD_SETUP_MELODY
extern void Aud_PlaySetupMelody(UINT8 u1DecId, UINT32 u4Channel, UINT32 u4RepeatCount);
void Aud_GetSetupMelodyInfo(AUD_CLIP_TYPE_T* prAudClip);
#endif

extern void AUD_DspSetLogoMelody(UINT8 u1VolLevel, UINT8 u1HpVolLevel, BOOL fgEnable);

//-----------------------------------------------------------------------------
// AUD_DrvDmxIntThreshold
//
/** Set DMX interrupt threshold.
 *
 *  @param[in]          u1DecId         Decoder ID
 *                      u1Factor        threshold = 188 * u1Factor
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DrvDmxIntThreshold(UINT8 u1DecId, UINT8 u1Factor);

//-----------------------------------------------------------------------------
// AUD_SetLPCMFmt
//
/** Set LPCM parameter
 *
 *  @param[in]          u4PriData         private data about lpcm format in VOB
 *
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_SetLPCMFmt(UINT8 u1DecId, UINT32 u4PriData);
#if 0   // Unused
extern void AUD_SetLPCMFmtPS(UINT8 u1DecId, UINT32 u4PriData);
extern void AUD_SetLPCMFmtTS(UINT8 u1DecId, UINT32 u4PriData);
extern void AUD_SetLPCMFmtWIFI(UINT8 u1DecId, UINT32 u4PriData);
#endif
//-----------------------------------------------------------------------------
// AUD_IsLpcmFmtPS
//
/** Query LPCM from AVI or Program Stream.
 *
 *  @param[in]          void
 *
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_IsLpcmFmtPS(void);
//-----------------------------------------------------------------------------
// AUD_LPcmSetting
//
/** Set LPCM parameter
 *
 *  @param[in]          u1DecId
 *                      prLPcmInfo
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_LPcmSetting(UINT8 u1DecId, AUD_LPCM_INFO_T* prLPcmInfo);

//-----------------------------------------------------------------------------
// AUD_Ac3Setting
//
/** AC3 format setting
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN or AUD_DEC_AUX)
 *  @param      prPcmSetting
 */
//-----------------------------------------------------------------------------
extern void AUD_Ac3Setting(UINT8 u1DecId, const AUD_AC3_SETTING_T* prAC3Setting);


//-----------------------------------------------------------------------------
// AUD_MM_Set_Dec_Fmt_Conflict
//
/** For SWDMX callback AUD_DRV to notify that audio fomat is conflict.
 *
 *  @param[in]        u1DecId          Decoder ID
 *                    fgFlag           Enable Flag
 *
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_MM_Set_Dec_Fmt_Conflict(UINT8 u1DecId, BOOL fgFlag);
extern BOOL AUD_Is_MMDecFmtConflict(UINT8 u1DecId);
extern void AUD_MM_Set_Dec_Fmt_Scramble(UINT8 u1DecId, BOOL fgFlag);
extern void AUD_MM_Set_Avsync_event(UINT8 u1DecId);

extern BOOL AUD_Is_MMDecFmtScramble(UINT8 u1DecId);
extern INT32 AUD_MTS_CFO_IDX(void);
extern INT32 AudCmdAtvHdevDetQuery(void);
#if defined(CC_PHX_CUST) || defined(CC_LMP_CUST) || defined(CC_MAPLE_CUST_DRV)
extern INT16 AudAtvMainCarrierLvlQuery(void);
extern INT16 AudAtvSubCarrierLvlQuery(void);
extern INT16 AudAtvPilotLvlQuery(void);
extern INT16 AudAtvMainNoiseLvlQuery(void);
extern INT16 AudAtvSubNoiseLvlQuery(void);
extern UINT32 AudAtvNicamErrorQuery(void);
#endif
//-----------------------------------------------------------------------------
// AUD_MMClearEOSFlag
//
/** Clear Audio Multimedia EOS Flag
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 */
//-----------------------------------------------------------------------------
extern void AUD_MMClearEOSFlag(UINT8 u1DecId);

//-----------------------------------------------------------------------------
// AUD_MMClearPesCnt
//
/** Clear Audio Multimedia Pes Count
 *
 *  @param      u1DecId          Decoder ID (AUD_DEC_MAIN)
 */
//-----------------------------------------------------------------------------
extern void AUD_MMClearPesCnt(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspGetMpegAudInfo
 *
 *  @param u1DecId                    Decoder Id (AUD_DEC_MAIN or AUD_DEC_AUX).
 *  @param *prAudInfo                audio info structure pointer
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspGetMpegAudInfo(UINT8 u1DecId, AUD_DEC_INFO_T* prAudInfo);

#if 0   // Unused
//-----------------------------------------------------------------------------
/** AUD_DspSetMp3PreparsingMode
 *
 *  @param  enable/disable
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetMp3PreparsingMode(BOOL fgEnable);

//-----------------------------------------------------------------------------
/** AUD_DspIsMp3Vbr
 *
 *  @param  void
 *  @retval true (VBR)  false(CBR)
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspIsMp3Vbr(void);
#endif

#ifdef CC_AUD_DSP_SUPPORT_MIXSOUND_MEMPLAY
//-----------------------------------------------------------------------------
/** AUD_StopMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_StopMixSndClip(UINT8 u1StreamID);

//-----------------------------------------------------------------------------
/** AUD_PauseMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_PauseMixSndClip(UINT8 u1StreamID);

//-----------------------------------------------------------------------------
/** AUD_ResumeMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_ResumeMixSndClip(UINT8 u1StreamID);

//-----------------------------------------------------------------------------
/** AUD_RepeatMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_RepeatMixSndClip(UINT8 u1StreamID, UINT32 u1RepeatNum);

//-----------------------------------------------------------------------------
/** AUD_SetMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @param u4Addr               MixSound Clip Memory Physical Address
 *  @param u4Size               MixSound Clip Size (in byte)
 *  @param u4Gain               MixSound Clip Gain (0db => 0x20000)
 *  @param u1StereoOnOff        MixSound Clip Mono or Stereo (FALSE: mono, TRUE: stereo)
 *  @param u4SampleRate         MixSound Clip Upsample Ratio
 *  @param u1BitDepth           MixSound Clip Bit Depth (8/16)
 *  @retval BOOL                TRUE - set succ, FALSE - set fail
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_SetMixSndClip(UINT8 u1StreamID, UPTR u4Addr, UINT32 u4Size, UINT32 u4Gain, UINT8 u1StereoOnOff, UINT32 u4SampleRate, UINT8 u1BitDepth, UINT8 u1Endian);

//-----------------------------------------------------------------------------
/** AUD_PlayMixSndClip
 *
 *  @param u1StreamID           MixSound Stream ID (0~MAX_AUD_MIXSOUND_STREAM_NUM)
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_PlayMixSndClip(UINT8 u1StreamID);
extern BOOL AUD_IsMixSndClipPlay(UINT8 u1StreamID);

extern void AUD_EnableMixSndClip(void);
extern void AUD_DisableMixSndClip(void);
extern void AUD_ToneGen_MemAlloc(void);
#ifdef CC_SUPPORT_STR
extern void AUD_MixsndResume(void);
#endif
extern UINT32 AUD_GetMixsndSkipPtsNum(UINT8 u1StreamID);
extern UINT32 AUD_GetMixsndAFifoAvailableSize(UINT8 u1DecId);

extern void AUD_ClearMixSndClip(void);
extern void AUD_DspSetAudClipGain(UINT32 u4Gain);
//extern void AUD_EnableAlsaMixSnd(void);

//-----------------------------------------------------------------------------
/** AUD_QueryMixSndClip
 *
 *  @param void
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_QueryMixSndClip(void);
#ifndef DISABLE_BUILTIN_FUNCTION

extern void AUD_PlayMixSndRingFifo(UINT8 u1StreamId, UINT32 u4SampleRate, UINT8 u1StereoOnOff, UINT8 u1BitDepth, UINT32 u4BufferSize); //ALSA_MIXSND_PATH
#endif
extern void AUD_StopMixSndRingFifo(UINT8 u1StreamId); //ALSA_MIXSND_PATH

extern void AUD_DspSetMixSndLength(UINT32 u4Len);
#ifndef NEW_MIXSOUND
extern void AUD_DspSetMixSndGain(UINT32 u4Gain);
extern void AUD_DspSetMixSndStereo(UINT8 bOnOff);
extern void AUD_DspSetMixSndUpSampleRatio(UINT8 bRatio);
#endif
extern UINT32 AUD_DspGetMixSndGain(void);
extern BOOL AUD_DspGetMixSndStereo(void);
extern void AUD_DspSetMixSndMemPlay(UINT8 bOnOff); //!NEW_MIXSOUND
extern BOOL AUD_DspIsMixSndPlay(void);
extern void AUD_DspSetMixSndChannel(UINT16 wCh);
extern void AUD_DspSetMixSndChannelMute(UINT16 wCh, BOOL fgMute);
extern BOOL AUD_DspGetMixSndChannelMute(UINT16 wCh);
extern UPTR AUD_DspGetMixSndReadPtr(UINT8 u1StreamID);
extern void AUD_DspSetMixSndWritePtr(UINT8 u1StreamID, UPTR u4WritePtr);
extern void AUD_DspShowMixSndStatus(UINT8 u1StreamID);
#endif

extern void AUD_DspSetWithVideo(UINT8 u1DecId, BOOL fgIsWithVideo);
extern BOOL AUD_DspIsWithVideo(UINT8 u1DecId);

//-----------------------------------------------------------------------------
/** AUD_DspSetCapability
 *   Set Capability
 *
 *  @param      void
 *  @retval     void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspSetCapability(void);

//-----------------------------------------------------------------------------
/** AUD_DspGetCapability
 *   Get Capability
 *
 *  @param      void
 *  @retval     format capability
 */
//-----------------------------------------------------------------------------
extern UINT32 AUD_DspGetCapability(UINT8 u1Idx);

//-----------------------------------------------------------------------------
/** AUD_DspIsFormatValid
 *  Is format valid
 *
 *  @param      u4FormatInfo: format index
 *  @retval     TRUE(valid). FALSE(invalid)
 */
//-----------------------------------------------------------------------------
extern BOOL AUD_DspIsFormatValid(UINT32 u4FormatInfo);

extern BOOL AUD_DspIsSupportAC3(void);

extern BOOL AUD_DspIsSupportEAC3(void);

#if 0   // Unused
extern BOOL AUD_DspIsSupportDDCO(void);
#endif

//-----------------------------------------------------------------------------
/** AUD_DspCOOKParameter
 *  setup WMA information
 *
 *  @param  u1DecId 0: first decoder 1: seconder decoder.
 *  @param  prCookInfo  : pointer of COOK header information
 *  @retval void
 */
//-----------------------------------------------------------------------------
extern void AUD_DspCOOKParameter(UINT8 u1DecId, AUD_COOK_CFG_T* prCookInfo);

//sunman_aout
extern UINT32 AudGetAoutStatus(void);
extern void AudResetAoutStatus(void);

//sunman cfo
extern INT32 AUD_MTS_CFO_IDX(void);

extern UINT32 AUD_GetTargetCountry(void);

#ifdef CC_AUD_SETUP_MELODY
extern void AUD_PlaySetupMelody(UINT8 u1DecId, UINT32 u4Channel, UINT32 u4RepeatCount);
extern void AUD_SetSetupPlayStatue(BOOL fgIsFinish);
extern BOOL AUD_IsSetupMelodyFinish(void);
extern void AUD_StopSetupMelody(UINT8 u1DecId);
extern void AUD_DspSetMelodyVol(UINT8 u1Vol);
extern UINT8 AUD_GetSetupMelodyVol(void);
#endif

extern INT32 AUD_FlashAQReadEx(UINT8 u1AQPartition, UINT32 u4Offset, UINT8 *u1Data, UINT32 u4Length);
#ifdef SUPPORT_FLASH_AQ
extern void AUD_FlashAQInitEx(void);
extern void AUD_GetFlashAQVer(CHAR* str);
extern void AUD_FlashAQInitEx_LZMA(void);
#else
extern void AUD_GetFlashAQVer(CHAR* str);
#endif

#ifdef CC_AUD_SONY_SALVIA_OVM_GAIN
extern void _AUD_DspSetOVMGain(AUD_DEC_OVM_MODE_T eMode);
extern void APSetCountryValue(AUD_APP_COUNTRY_SEL u4CountryType);
#endif
extern AUD_APP_COUNTRY_SEL GetCountryValue(void);
extern BOOL ADAC_DigiAmp_ReadRam(AUD_AMP_ID_T u1AmpID, UINT8 u1Bank, UINT8 u1Addr, UINT8* u1Data, BOOL fgAllSet);
extern BOOL ADAC_DigiAmp_WriteRam(AUD_AMP_ID_T u1AmpID, UINT8 u1Bank, UINT8 u1Addr, AUD_AQ_AMP_RAM_DATA_T rAmpRamData, BOOL fgAllSet);

extern BOOL ADAC_STA381_ReadRam(AUD_AMP_ID_T u1AmpID, UINT8 u1Bank, UINT8 u1Addr, UINT8* u1Data, BOOL fgAllSet);
extern BOOL ADAC_STA381_WriteRam(AUD_AMP_ID_T u1AmpID, UINT8 u1Bank, UINT8 u1Addr, AUD_AQ_AMP_RAM_DATA_T rAmpRamData, BOOL fgAllSet);
extern void ADAC_STA381_SetChannelVol(AUD_AMP_ID_T u1AmpID, UINT8 u1ChId, UINT8 u1Value);
extern void ADAC_STA381_SetDualDRC(AUD_AMP_ID_T u1AmpID, AUD_AQ_AMP_DRC_T eDrcData);
extern void ADAC_STA381_SetPostScale(AUD_AMP_ID_T u1AmpID, UINT32 u4PostSclCh1, UINT32 u4PostSclCh2);
extern void ADAC_STA381_SetPEQ(AUD_AMP_ID_T u1AmpID, AUD_AQ_EXTPEQ_DATA_T* rAQExtPeqData);
extern void ADAC_STA381_SetSTCCoeff(AUD_AMP_ID_T u1AmpID, AUD_AQ_AMP_STC_COEFF_T* rAQAmpStcCoeff);
extern void ADAC_STA381_SetSTCFilter(AUD_AMP_ID_T u1AmpID, AUD_AQ_EXTPEQ_DATA_T* rAQExtPeqData);

extern void _TAS5717_SetPostScale(AUD_AMP_ID_T u1AmpID, UINT32 u4Value);
extern void _TAS5717_SetPreScale(AUD_AMP_ID_T u1AmpID, UINT32 u4Value);
extern void _TAS5717_SetChannelVol(AUD_AMP_ID_T u1AmpID, UINT8 u1ChId, UINT16 u2Value);

extern UINT8 AUD_GetPIPVideoPath(void);
extern UINT32 AUD_DspQueryChannelDelay(UINT8 u1ChIdx); //QUERY_DELAY
extern void AUD_SetSyncDbgLvl(UINT16 u2Flag);
extern UINT16 AUD_GetSyncDbgLvl(void);

extern DATA_ENDIAN_T AUD_GetDTSInfo (UINT8 u1DecId);
extern void AUD_SetDTSInfo (UINT8 u1DecId, DATA_ENDIAN_T endian);
extern void AUD_SetDTSFrameSize(UINT8 u1DecId, UINT32 u4Size);
extern void AUD_SetDTSPcVal(UINT8 u1DecId, UINT32 u4Pc);
extern UINT32 AUD_GetDTSFrameSize (UINT8 u1DecId);

#ifdef  CC_AUD_HDMI_PARSER_2_0_SUPPORT
extern void AUD_DrvGetParserDetect(UINT8 u1DecId, BOOL* pfgDeteced);
extern void AUD_DrvGetParserFormat(UINT8 u1DecId, AUD_FMT_T* peDecFmt);
extern AUD_FMT_T AUD_DrvPcToFormat(UINT8 u1DecId, UINT16 u2Pc);
#endif
#ifdef CC_MPEG_LEVEL_ADJ
extern void AUD_MpegLevelAdjust (UINT8 u1Level);
extern void AUD_DrvMpegLevelAdjust (UINT8 u1Level);
extern INT32 AUD_GetDrvMpegLevelAdjust (void);
#endif
#ifdef CC_HEAAC_LEVEL_ADJ
extern void AUD_HEAACLevelAdjust (UINT8 u1Level);
extern void AUD_DrvHEAACLevelAdjust (UINT8 u1Level);
extern INT32 AUD_GetDrvHEAACLevelAdjust (void);
#endif

extern UINT32 AUD_GetSilence(UINT8 u1DecId);
extern void AUD_SetSilence(UINT8 u1DecId, UINT32 u4Threshold);

extern void AUD_SetVdoPictureOffStatus(BOOL fgEnable);
extern BOOL AUD_GetVdoPictureOffStatus(void);


extern BOOL Aud_IsHeadphoneExist(void);

extern void AUD_DspSetStcId(UINT8 u1DecId, UINT16 u2StcId); // 2_STC_Task
extern UINT8 AUD_DspQryStcID(UINT8 u1DecId);
extern void AUD_DspGetStcId(UINT8 u1DecId, UINT16* u2StcId);
#ifdef KARAOKE_MIC_LINE_IN2
extern void AUD_SetAout2MixAout1(UINT32 u4ChSetting);
#endif

#ifdef CC_ENABLE_AV_SYNC
extern void Aud_AddLogDump(CHAR acMsg[], INT32 i4Value); //CC_AUD_DUMP_LOG
extern void Aud_ListLogDump(void);
#endif

#if defined(CC_AUD_ARM_SUPPORT)
extern void vAprocReg_Write (UINT32 u4Offset, UINT32 u4Val);
extern void vAprocReg_Write_With_Virtual (UINT32 u4Offset, UINT32 u4Val);


extern UINT32 u4AprocCommSignal_send (UINT32 u4Idx, UINT32 u4Cmd, UINT32 u4Data0, UINT32 u4Data1);
extern void vReadProc(UINT32 u4Addr, UINT32 u4Size, UINT32* p4Value);
extern void vWriteProc(UINT32 u4Addr, UINT32 u4Data);
//extern void vAprocReg_Write (UINT32 u4Offset, UINT32 u4Val);
extern void vAprocReg_Write_Mask (UINT32 u4Offset, UINT32 u4Val, UINT32 u4Mask);
extern void vAprocReg_Write_Multi (UINT32 u4Offset, UINT32* u4Val, UINT32 u4Count);
extern UINT32 u4AprocReg_Read (UINT32 u4Offset);
extern UINT32 u4AprocReg_Read_With_Virtual (UINT32 u4Offset);

extern void vAproc_NotifyAdsp(UINT32 u4Idx, UINT32 u4Cmd, UINT32 u4Data);

extern void vAprocDec_Open (UINT32 u4MixerId);
extern void vAprocDec_Close (UINT32 u4MixerId);
#ifndef CC_FPGA
extern void vAproc_EnableMidProtect(BOOL fgEnable);
#endif
#ifdef CC_AUD_ARM_RENDER
extern void vAprocDataPort_Write (UINT32 u4Offset, UINT32 *pu4Data, UINT32 u4Num);
extern void vAprocDataPort_Read (UINT32 u4Offset, UINT32 *pu4Data, UINT32 u4Num);
extern void vAprocDataPort_WriteByte (UINT32 u4Offset, UINT8 *pu1Data, UINT32 u4Num);
extern void vAprocDataPort_ReadByte (UINT32 u4Offset, UINT8 *pu1Data, UINT32 u4Num);
extern void _vAUD_Aproc_Set (UINT32 u4Type, UINT32 u4Item, UINT32 *pu4Addr, UINT32 u4Num);
extern void _vAUD_Aproc_Get (UINT32 u4Type, UINT32 u4Item, UINT32 *pu4Addr, UINT32 u4Num);
extern BOOL _AUD_DspGetAprocEqEnable(APROC_IOCTRL_T ePort, BOOL isFir);
extern void _AUD_DspSetAprocEqEnable(APROC_IOCTRL_T ePort, BOOL isFir, BOOL fgEnable);
extern BOOL bIsSupportAproc (void);
#ifdef CC_MT5895
extern void AUD_AprocIceConnect(BOOL bEnable);
#endif // CC_MT5895
extern void vAproc_Open (void);
extern void vAprocMain_Open(void);
extern void vAprocMain_Close (void);
extern void AUD_AprocEnableMixSnd(UINT8 u1MixID); //parson mixsnd
extern void AUD_AprocDisableMixSnd(UINT8 u1MixID);
extern void AUD_AprocEnableUpload(void);
extern void AUD_AprocDisableUpload(void);
extern void AUD_AprocEnableRiscPost(void); //parson riscpost
extern void AUD_AprocDisableRiscPost(void);
extern void vAprocAqData_Write (UINT32 u4Offset, UINT8 *pu1Data, UINT32 u4Num);
extern void vAprocAqData_Read (UINT32 u4Offset, UINT8 *pu1Data, UINT32 u4Num);
#ifdef CC_AUD_SAKURA_RENDER
extern void vAprocAqData_WriteWord(UINT32 u4Offset, UINT32 *pu4Data, UINT32 u4Num);
extern void vAprocAqData_ReadWord(UINT32 u4Offset, UINT32 *pu4Data, UINT32 u4Num);
#endif //CC_AUD_SAKURA_RENDER
extern void _vAUD_Aproc_SetMp3Enc(BOOL fgEnable);
#ifdef ARM_SUPPORT_SRS_SS
extern void _vAproc_PostProcSRSAEQ_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSAEQ_Get(void);
extern void _vAproc_PostProcSRSBT_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSBT_Get(void);
extern void _vAproc_PostProcSRSCC3D_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCC3D_Get(void);
extern void _vAproc_PostProcSRSCSHP_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCSHP_Get(void);
extern void _vAproc_PostProcSRSGEQ_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSGEQ_Get(void);
extern void _vAproc_PostProcSRSHL_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSHL_Get(void);
extern void _vAproc_PostProcSRSHPF_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSHPF_Get(void);
extern void _vAproc_PostProcSRSMVOL_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSMVOL_Get(void);
extern void _vAproc_PostProcSRSSPHPF_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSSPHPF_Get(void);
extern void _vAproc_PostProcSRSSRS3D_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSSRS3D_Get(void);
extern void _vAproc_PostProcSRSSS3D_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSSS3D_Get(void);
extern void _vAproc_PostProcSRSTSHD_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTSHD_Get(void);
extern void _vAproc_PostProcSRSTBHD_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTBHD_Get(void);
extern void _vAproc_PostProcSRSTRUD_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTRUD_Get(void);
extern void _vAproc_PostProcSRSTVOL_Set(UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTVOL_Get(void);
#endif

#ifdef ARM_SUPPORT_SRS_TS3D
extern void _AUD_SRS_Enable(BOOL fgEnable);
extern BOOL _AUD_SRS_GetEnable(void);
extern void _vAproc_PostProcSRSTS3D_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTS3D_Get(void);
extern void _vAproc_PostProcSRSLEVEL_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSLEVEL_Get(void);
extern void _vAproc_PostProcSRSTVOLHD_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTVOLHD_Get(void);
extern void _vAproc_PostProcSRSTRUD_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSTRUD_Get(void);
extern void _vAproc_PostProcSRSCC3D_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCC3D_Get(void);
extern void _vAproc_PostProcSRSCSD_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCSD_Get(void);
extern void _vAproc_PostProcSRSCSHP_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCSHP_Get(void);
extern void _vAproc_PostProcSRSCommon_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSCommon_Get(void);
extern void _vAproc_PostProcSRSHPF_Set (UINT32 u4type,UINT32 u4v0,UINT32 u4v1);
extern void _vAproc_PostProcSRSHPF_Get(void);
extern void _vAproc_PostProcSRSGEQ_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSGEQ_Get(void);
extern void _vAproc_PostProcSRSAEQ_Set (UINT32 u4type,UINT32 u4v0,UINT32 u4v1);
extern void _vAproc_PostProcSRSAEQ_Get(void);
extern void _vAproc_PostProcSRSMCDynamic_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSMCDynamic_Get(void);
extern void _vAproc_PostProcSRSDownup_Set (UINT32 u4type,UINT32 u4v0);
extern void _vAproc_PostProcSRSDownup_Get(void);

extern void _vAproc_PostProcSRS_DataPort(void);

#endif


extern void _vAproc_PostProcEqSpec_Get(void);
extern void _vAproc_PostProcEqSpec_Set(UINT32 u4v0);

#endif //CC_AUD_ARM_RENDER

#endif //CC_AUD_ARM_SUPPORT

void AUD_MMSetSwdmxSrcId(UINT8 u1DecId, UINT8 u1SwdmxSrcId);

#ifdef KARAOKE_MIC_LINE_IN2
extern void AUD_MicInput_Enable(BOOL fgEnable);
#endif
extern void ADAC_SIFSrcSel(UINT32 isUseSIF);
extern void AUD_PlayMuteEnable(BOOL fgEnable);
extern UINT32 vGetFBMCMMSize(void);
#ifdef CC_SUPPORT_STR
extern void AUD_PostProc_Resume(void);
extern void AUD_PostProc_Suspend(void);
extern void AUD_Mixsnd_Suspend(void);
extern void AUD_Mixsnd_Resume(void);
extern void AUD_SetUiSpkOnOff(BOOL fgEnable);
#endif
extern void AUD_QueryPacketCount(UINT8 u1DecId,  UINT32 * pPacketCount);
extern void AUD_QueryFrameCount(UINT8 u1DecId,  UINT32 * pFrameCount);
extern void AUD_QueryDecoderStatus(UINT8 u1DecId,  AUD_DEC_STATE * pDecoderStatus);
extern void AUD_QueryStreamType(UINT8 u1DecId,  AUD_FMT_T * pStreamType);
extern void AUD_QuerySampleRate(UINT8 u1DecId,  UINT32 * pSampleRate);
extern UINT32 AUD_QueryStcID(UINT8 u1DecId);
extern void AUD_QueryStc(UINT8 u1DecId,  UINT32 * pStc);
extern void AUD_QueryPts(UINT8 u1DecId,  UINT32 * pPts);
extern void AUD_QueryFirstPts(UINT8 u1DecId,  UINT32 * pPts);

#if defined(CC_AUD_ARM_SUPPORT) && defined(CC_AUD_ARM_RENDER)
extern BOOL AUD_Aproc_Chk_Notify(UINT8 *u1DecId, UINT32 u4NfyTag, UINT32 eNfyCond);
extern BOOL  AUD_Aproc_Chk_Cmd(UINT8 u1DecId);
extern void AUD_Aproc_Chk_DecId(UINT8* u1DecId);
extern void AUD_Aproc_OutCtrl(UINT8 u1DecId, APROC_MW_CTRL_T *arMwCtrl);
extern void AUD_Aproc_ChkTVDecInfoChgNfy(UINT8 u1DspId, UINT8 u1DecId);
extern void AUD_OmxPathSel(E_AUD_OMX_PATH ePath);
#ifdef CC_DUAL_AUD_DEC_SUPPORT
extern void AUD_Aproc_SetFocus(E_AUD_DEC_FOCUS_TYPE u1Focus);
#endif  //CC_DUAL_AUD_DEC_SUPPORT
#if defined(CC_AUD_APROC_KEYSOUND)
extern void AUD_AprocEnableKeySnd(void);
extern void AUD_AprocDisableKeySnd(void);
#endif
#endif //CC_AUD_ARM_RENDER

extern UINT32 AUD_GetMixSndBandNum(UINT8 u1MixSndId);
extern UINT32 AUD_GetAoutBankNum(UINT8 u1DecId);
extern UINT32 AUD_GetAoutBankSampleNum(UINT8 u1DecId);
extern UINT32 AUD_GetSkipPtsNum(UINT8 u1DecId);
extern UINT32 AUD_GetAFifoAvailableSize(UINT8 u1DecId);

extern UINT32 AUD_GetSleepMs(UINT32 u4WriteInterval, UINT32 u4Size);
extern USIZE _AfifoBufferFilledSize(void);
extern void AUD_FlushAudioFifo(UINT8 u1DecId);
extern BOOL AUD_MixsndClipFlushFifo(void);

#if (defined(CC_AUD_SAKURA_RENDER_KDAMP)||defined(CC_AUD_UNICORN_CUST))
//-----------------------------------------------------------------------------
/*   _MODIFY_
 * Add functions for DAMP
 */
//-----------------------------------------------------------------------------
extern void KD_AQ_AMP_INIT(void);
extern void KD_AQ_AMP_Refresh(void);
extern void KD_AQ_AMP_5760_FS_Change(AQ_FS fs);
extern void KD_AQ_AMP_5538_Mute(KD_AUD_AMP_MUTE_T mute);
extern void KD_AQ_AMP_5760_Mute(KD_AUD_AMP_MUTE_T mute);
#endif

#if defined(CC_AUD_ARM_SUPPORT) && defined(CC_AUD_ARM_RENDER)
extern void AUD_Aproc_SetMp3Enc(BOOL fgEnable);
extern BOOL AUD_Aproc_GetMp3Enc(void);
extern UINT32 AudDecId2MixerId(UINT8 u1DecId);
extern void AUD_AprocSpeed(UINT8 u1DecId, UINT32 u4Speed);
#endif
#if defined(CC_PHX_CUST)||defined(CC_LMP_CUST)
extern void AUD_PlayTestTone(UINT8 u1DecId, UINT8 u1Sel);
extern void AUD_StopTestTone(UINT8 u1DecId);
#endif
extern AUD_TYPE_T DSP_GetAudChlInfo(UINT8 u1DecId);
//#ifdef  ALSA_PCMDEC_PATH
#ifndef DISABLE_BUILTIN_FUNCTION
extern void AUD_SetDSPDecoderType(AUD_FMT_T u4DspDecType);
#endif
//#endif
#if defined(CC_PHX_CUST)||defined(CC_LMP_CUST)
extern void AUD_SetIsAmpSupportDDP(BOOL fgEnable);
extern BOOL AUD_GetIsAmpSupportDDP(void);
extern void vAudApSetUiDdpAutoOff(UINT8 onoff);
extern UINT8 vAudApGetUiDdpAutoOff(void);
extern void vAudSetARCEnable(BOOL fgEnable);
extern BOOL vAudGetARCEnable(void);
extern void vAudARC_SetDDPEnable(BOOL fgEnable);
#endif
#ifdef CC_AUD_TTS_SUPPORT
extern BOOL AUD_TTS_PlayPCM_open (INT32* pi4StreamId);
extern BOOL AUD_TTS_PlayPCM_close (INT32* pi4StreamId);
extern BOOL AUD_TTS_PlayPCM_play (INT32 i4StreamId, UINT32 u4SampleRate, UINT8 u1StereoOnOff, UINT8 u1BitDepth, UINT32 u4BufferSize);
extern BOOL AUD_TTS_PlayPCM_write(INT32 i4StreamId, UPTR u4BufPtr, UINT32 u4BufSize,UINT32 fgExtendInfo);
extern BOOL AUD_TTS_PlayPCM_Set_Pause(INT32 i4StreamId);
extern BOOL AUD_TTS_PlayPCM_Set_Stop(INT32 i4StreamId);
extern BOOL AUD_TTS_PlayPCM_Set_Resume(INT32 i4StreamId);
extern BOOL AUD_TTS_PlayPCM_Set_DecMainVol(void);
extern BOOL AUD_TTS_PlayPCM_Set_VolumeRestore(void);

#endif


#ifdef CC_AUD_DOLBY_SUPPORT_MS12
#ifdef CC_AUD_ITAF_SUPPORT
extern void AUD_SET_ITAF_CMDLINE(const CHAR * strCmdLine);
#endif // CC_AUD_ITAF_SUPPORT
extern void AUD_AAC_PCE_SET_FILEFMT(UINT8 u1DspId, UINT8 u1DecId, UINT8 filefmt/*0:ADTS,1:MP4,2:LOAS,3:ADIF*/);
extern void AUD_AAC_PCE_SET_CHNCONFIG(UINT8 u1DspId, UINT8 u1DecId, UINT8 u1ChnConfig);
extern void AUD_AAC_PCE_SET_NUM_VALID_CC_ELEMENT(UINT8 u1DspId, UINT8 u1DecId, UINT8 u1NumValidCCElement);
extern void AUD_AAC_PCE_SET_MONO_MIXDOWN_PRESENT(UINT8 u1DspId, UINT8 u1DecId, UINT8 u1MonoMixdownPresent);
extern void AUD_AAC_PCE_SET_ELEMENT(UINT8 u1DspId, UINT8 u1DecId, UINT8 u1ChnId /*0:FRONT, 1:SIDE, 2:BACK, 3:LFE*/,
                                             UINT8 u1NumChnElements, UINT8 *pu1ElementIsCpe, UINT8 *pu1ElementTagSel);
#endif // CC_AUD_DOLBY_SUPPORT_MS12

#ifdef CC_AUD_DOLBY_SUPPORT_MS12
// for MS12 controls
extern void AUD_MS12_ENABLE (UINT32 u4Sel /* 0:Mixer,1:Pcmr,2:DdpEnc */, BOOL bEnable);
// MIXER
extern void AUD_MS12_MIXER_ASSOC_PRESENT(BOOL bEnable);
extern UINT32 AUD_MS12_QUERY_MIXER_ASSOC_PRESENT(void);
extern void AUD_MS12_MIXER_DMX(BOOL bEnable);
extern UINT32 AUD_MS12_QUERY_MIXER_DMX(void);
extern void AUD_MS12_MIXER_USERPREF(INT32 i4Value);
extern INT32 AUD_MS12_QUERY_MIXER_USERPREF(void);
// PCMR
extern void AUD_MS12_PCMR_DISCARD_DRC(BOOL bEnable);
extern UINT32 AUD_MS12_QUERY_PCMR_DISCARD_DRC(void);
extern void AUD_MS12_PCMR_DRC_CUTBOOST (UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX */, UINT32 u4CutBoostSel /* 0:Cut,1:Boost */, UINT32 u4Value /* 0-100 */);
extern UINT32 AUD_MS12_QUERY_PCMR_DRC_CUTBOOST(UINT32 u4Sel,UINT32 u4CutBoostSel);
extern void AUD_MS12_PCMR_DRC_MODE (UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX */, UINT32 u4Value /* 0:Line,1:Rf */);
extern UINT32 AUD_MS12_QUERY_PCMR_DRC_MODE(UINT32 u4Sel);
extern void AUD_MS12_PCMR_DMX_MODE (UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX */, UINT32 u4Value /* 0:Lt/Rt,1:Lo/Ro,2:Arib */);
extern UINT32 AUD_MS12_QUERY_PCMR_DMX_MODE(UINT32 u4Sel);
extern void AUD_MS12_PCMR_DUAL_MODE (UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX */, UINT32 u4Value /* 0:stereo,1:left ch,2:right ch */);
extern UINT32 AUD_MS12_QUERY_PCMR_DUAL_MODE(UINT32 u4Sel);
extern void AUD_MS12_PCMR_OUTPUT_ACTIVE(UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX */, BOOL bEnable);
extern UINT32 AUD_MS12_QUERY_PCMR_OUTPUT_ACTIVE(UINT32 u4Sel);
extern void AUD_MS12_PCMR_ENABLE_PROCESSING(UINT32 u4Sel /* 0:MC1,1:MC2,2:DMX,3:RAWMC */, BOOL bEnable);
extern UINT32 AUD_MS12_QUERY_PCMR_ENABLE_PROCESSING(UINT32 u4Sel);
extern void AUD_MS12_PCMR_COMPRESSOR_PROFILE(UINT32 u4Cprof /* 0:No Compressor,1:Std. Film,2:Light Film,3:Std. Music,4:Light Music,5:Speech */);
extern UINT32 AUD_MS12_QUERY_PCMR_COMPRESSOR_PROFILE(void);
extern void AUD_MS12_PCMR_DELAY (UINT32 u4Delay);
extern UINT32 AUD_MS12_QUERY_PCMR_DELAY(void);
#ifdef CC_AUD_DOLBY_SUPPORT_MS12v2
extern void AUD_MS12_PCMR_MC2_DMX(BOOL bEnable);
extern BOOL AUD_MS12_QUERY_PCMR_MC2_DMX (void);
extern void AUD_MS12_PCMR_RAWMC_DMX(BOOL bEnable);
extern BOOL AUD_MS12_QUERY_PCMR_RAWMC_DMX(void);
extern UINT32 AUD_GetHeaacDualMode(UINT8 u1DecId);
extern void AUD_SetHeaacDualMode(UINT8 u1DecId, AAC_DUAL_MODE_E eDualMode);
#endif
// DDPENC
extern void AUD_MS12_DDPENC_DISPLAYBANNER(void);
#ifndef CC_AUD_CPUENC_SUPPORT
extern void AUD_MS12_DDPENC_DATARATE(UINT32 u4DataRate);
extern UINT32 AUD_MS12_QUERY_DDPENC_DATARATE(void);
extern void AUD_MS12_DDPENC_ENABLE_FILTER(UINT32 u4Sel /*0:lfe filter, 1:ph90 filter */, UINT32 u4Val /* 0:disable, 1:enable */);
extern UINT32 AUD_MS12_DDPENC_QUERY_ENABLE_FILTER(UINT32 u4Sel /*0:lfe filter, 1:ph90 filter */);
extern void AUD_MS12_DDPENC_QUERY_INFO(void);
#endif // CC_AUD_CPUENC_SUPPORT
// AAC
extern void AUD_MS12_AAC_SET_DEFAULT_DIALNORM(UINT8 u1DspId, UINT8 u1DecId, UINT32 u4DialNorm);
extern UINT32 AUD_MS12_QUERY_AAC_SET_DEFAULT_DIALNORM(UINT8 u1DspId,UINT8 u1DecId);
// AC3
extern void AUD_DspAC3MDCTBandLimitMode(UINT8 u1DspId,UINT8 u1DecId,UINT8 u1Mode);
extern UINT8 AUD_QUERY_DspAC3MDCTBandLimitMode(UINT8 u1DspId,UINT8 u1DecId);
extern void AUD_MS12_LOUDNESS_MATCHING_LEVEL_ADJUST(UINT32 u4Sel,UINT32 u4dB);
extern void AUD_MS12_INDICATE_EXTERNAL_DECODED_PCM(UINT8 u1DecId, BOOL bEnable);
extern void AUD_DolbyMS12_Loudness (UINT8 u1DecId);
extern void AUD_DolbyMS12_Init(UINT32 u4MixerId);
extern UINT32 AUD_DolbyMS12_DbToShm (INT32 i4Level);
#ifdef CC_AUD_DOLBY_SUPPORT_DAPv2
extern void AUD_MS12_DAP_DeEnhancer(BOOL bDe_en, INT32 i4De_amount, INT32 i4De_ducking);
extern void AUD_MS12_DAP_MiSteering(BOOL bMi_ieqen, BOOL bMi_dven, BOOL bMi_deen, BOOL bMi_surren);
extern void AUD_MS12_DAP_BassEnhancer(BOOL bBass_en, INT32 i4Bass_boost,
        INT32 i4Bass_cutoff, INT32 i4Bass_width);
extern void AUD_MS12_DAP_VirtualBass(INT32 *pi4VbParamSet);
extern void AUD_MS12_DAP_SurdecEn(BOOL bEnable);
#ifdef CC_AUD_DOLBY_SUPPORT_MS12v2
extern void AUD_MS12_DAP_Virtualizer(INT32 i4VirMode, INT32 i4SpkAngle, INT32 i4Surboost);
#else
extern void AUD_MS12_DAP_Virtualizer(BOOL bVirEn, INT32 i4HpReverb, INT32 i4SpkAngle,
        INT32 i4SpkStart, INT32 i4Surboost);
#endif // CC_AUD_DOLBY_SUPPORT_MS12v2
extern void AUD_MS12_DAP_MiDisable(BOOL bDisable);
extern void AUD_MS12_DAP_VirBassEn(BOOL bEnable);
extern void AUD_MS12_DAP_Mode(INT32 i4Mode);
extern void AUD_MS12_DAP_DrcMode(INT32 i4DrcMode);
extern void AUD_MS12_DAP_DmxMode(INT32 i4DmxMode);
extern void AUD_MS12_DAP_Outputmode(INT32 i4Outmode);
extern void AUD_MS12_DAP_Connector(INT32 i4Connector);
extern void AUD_MS12_DAP_SpkOut(INT32 i4Spkout);
extern void AUD_MS12_DAP_HpOut(INT32 i4Hpout);
extern void AUD_MS12_DAP_Ieq(DAP_IEQ_PARAM *prIeq_param_t);
extern void AUD_MS12_DAP_Geq(DAP_GEQ_PARAM *prGeq_param_t);
extern void AUD_MS12_DAP_Opt(DAP_OPT_PARAM *prOpt_param_t);
extern void AUD_MS12_DAP_Reg(DAP_REG_PARAM *prReg_param_t);
#ifdef CC_AUD_DOLBY_SUPPORT_MS12v2
extern void AUD_MS12_DAP_SpkCfg(INT32 i4SpkCfg);
extern void AUD_MS12_DAP_LFEBOOSTEN(INT32 i4LfeBoostEn);
extern void AUD_MS12_DAP_LFE2LREN(INT32 i4Lfe2LrEn);
extern void AUD_MS12_DAP_LFE2LRGAIN(INT32 i4Lfe2LrGain);
extern void AUD_MS12_DAP_BASSEXTRAEN(INT32 i4BassExtraEn);
extern void AUD_MS12_DAP_BASSEXTRAFREQ(INT32 i4BassExtraFreq);
extern void AUD_MS12_DAP_HEIGTHFILTERMODE(INT32 i4HeigthFilterMode);
#endif // CC_AUD_DOLBY_SUPPORT_MS12v2
extern void AUD_MS12_DAP_PREGAIN_SET(INT32 i4Pregain);
extern void AUD_MS12_DAP_POSTGAIN_SET(INT32 i4Postgain);
extern void AUD_MS12_DAP_SYSGAIN_SET(INT32 i4Sysgain);
extern void AUD_MS12_DAP_Volboost(INT32 i4VolBoost);
extern void AUD_MS12_DAP_CaliBoost(INT32 i4CaliBoost);
extern void AUD_MS12_DAP_Leveler(BOOL lev_en, INT32 lev_amount, BOOL lev_ignore_il);
extern void AUD_MS12_DAP_ModEn(BOOL bEnable);
extern void AUD_MS12_DAP_ModCali(INT32 i4ModCali);
#endif //CC_AUD_DOLBY_SUPPORT_DAPv2
#endif // CC_AUD_DOLBY_SUPPORT_MS12
#ifdef CC_AUD_M3_FEATURE
extern void AUD_PostTypeSet (INT8 PostType);
extern void AUD_Update_MS12_PCMR_DRC_MODE(BOOL forceUpdateFlag);
extern void AUD_SetEarcAudioStatus(EARCAudioStatus_t eARCStatus);
extern void AUD_SetNetflixStatus(UINT8 isNetflix);
extern INT32 AUD_GetDamSupport(void);
extern BOOL AUD_GetAribDownmixFlag(void);
extern BOOL AUD_GetAacPassThroughFlag(void);
#endif
#ifdef CC_MAPLE_CUST_DRV
extern void AUD_CUST_AMP_Init( void );
extern void AUD_CUST_AMP_Mute(AUD_CUST_AMP_MUTE_T mute);
extern void AUD_CUST_AMP_Suspend( void );
extern void AUD_CUST_AMP_Resume( void );
extern void AUD_CUST_AMP_Update( void );
extern void AUD_CUST_AMP_StartRefresh(void);
extern void AUD_CUST_AMP_StopRefresh(void);
#endif

#ifdef CC_AUD_ARM_SUPPORT
extern void AUD_Set1PIDADEnable(BOOL flag);
extern BOOL AUD_Get1PIDADStatus(void);
#endif
#ifdef CC_AUD_ARM_RENDER
extern void AUD_PostProc_FadeInFadeOut(UINT32 u4target,UINT32 u4duration,UINT32 u4type);
extern void AUD_PostProc_FadeInFadeOut_Pause(UINT32 u4Enable);
extern INT32 AUD_PostProc_GetFadeInFadeOutVolume(void);
extern INT32 _AUD_GetFadeInFadeOutVolume(void);
extern void AUD_Ms12DualMainFader(UINT32 u4Sel, INT32 i4Attenuation,
        UINT32 u4Duration, UINT32 u4Type);
#endif
extern BOOL AUD_IsAudSuspend(void);
extern void AUD_SetSuspendFlag(BOOL fgOnOff);
//aud_util.c
extern void AUD_DumpLogToFile(const CHAR *cpFileName, const CHAR *cpFormat,...);
extern void AUD_DumpLogEnable(BOOL fgEnable);
extern void AUD_DumpLogQuery(void);
extern void AUD_PRINTF(UINT32 u4LogBits, const CHAR *cpFormat,...);
extern void AUD_PrintfSetBits(UINT32 u4LogBits);
extern void AUD_PrintfQuery(void);
extern BOOL AUD_GetFastBootStatus(void);
extern UINT32 _AudGetAudioDebugLevel(void);
extern void _AudSetAudioDebugLevel(UINT32 u4Val,UINT32 u4value);


//-----------------------------------------------------------------------------
/*   _MODIFY_
 * Add functions for post implementation.
 */
//-----------------------------------------------------------------------------
#ifdef CC_AUD_UNICORN_CUST
extern void AUD_DspSoSetInput(UINT8 u1DecId, UINT8 u1Input);
extern void AUD_DspSoSetCodec(UINT8 u1DecId, UINT8 u1CodecType);
extern void AUD_DspSoSetOutputSamplingrate(UINT8 u1DecId, UINT32 u4Samplingrate);
extern void AUD_DspSoSetMpegAudioLevel(UINT8 u1DecId, INT8 i1MpegAudioLevel);
extern void AUD_DspSoSetHeaacAudioLevel(UINT8 u1DecId, INT8 i1HeaacAudioLevel);
extern void AUD_DspSoSetSoundMode(UINT8 u1SoundMode);
extern void AUD_DspSoSetMeasureMode(UINT8 u1MeasureMode);
extern void AUD_DspSoSetWirelessSubWoofer(BOOL fgSubWooferEnable);
extern void AUD_DspSoSetNarrationCancel(BOOL narrationCancel);
extern void AUD_DspSoSetClearAudioPlus(BOOL narrationCancel);
extern void AUD_DspSoSetDolbyDigitalRange(UINT8 u1DecId, UINT8 u1Range);
extern void AUD_DspSoSetHeAacDynamicRange(UINT8 u1DecId, UINT8 u1Range);
extern void AUD_DspSoHpAudioOut(UINT8 u1HpAudOut);
extern void AUD_DspSoVolumeOffset(UINT8 u1VoumeOffset);
extern void AUD_DspSoSetSwVolume(UINT8 u1Voume);
extern void AUD_DspSoSetSurround(UINT8 u1Surround);
extern void AUD_DspSoSetInputChConf(UINT8 u1ChConf);
extern void AUD_DspSoSetDigitalAudioOut(UINT8 u1AudioOut);
#endif /* CC_AUD_UNICORN_CUST */
#ifdef CC_AUD_SAKURA_RENDER
extern void AUD_DspSoSetBtConnected(UINT8 fgBtConnected);
extern void AUD_DspSoSetPictureModeSetting(UINT8 u1PictureMode);
#endif //CC_AUD_SAKURA_RENDER

#ifdef CC_AUD_CPUENC_SUPPORT
extern void AUD_CPUENC_GET_CTRL_STATUS (UINT32 *pu4Status);
extern void AUD_CPUENC_SET_CTRL_STATUS(UINT32 u4Status);
extern void AUDCPUEncBufInit (void);
extern UINT32 AUD_CPUENC_GET_WORK_BUF_ADDR (void);
extern UINT32 AUD_CPUENC_GET_WORK_BUF_SIZE (void);
extern UINT32 AUD_CPUENC_GET_UPBUF (UINT32 u4Sel);
extern UINT32 AUD_CPUENC_GET_DOWNBUF (UINT32 u4Sel);
//evo related
extern UINT32 AUD_CPUENC_GET_EVO_ENABLE (void);
extern UINT32 AUD_CPUENC_GET_MIXSNDPLAYING (void);
extern UINT32 AUD_CPUENC_CHECK_SRC_HOLD_OFF(void);
extern UINT32 AUD_CPUENC_GET_APROC_IEC_BUF_STR (void);
extern UINT32 AUD_CPUENC_GET_EVO_BUF_RCP (void);
extern UINT32 AUD_CPUENC_GET_EVO_BUF_WP (void);
extern UINT32 AUD_CPUENC_GET_DDPENC_MODE (void);
extern void AUD_CPUENC_SET_EVO_BUF_RP (void);
//end of evo related code
#ifdef CC_AUD_DOLBY_SUPPORT_MS12v2
extern UINT32 AUD_CPUENC_GET_CHMOD_LOCK (void);
extern UINT32 AUD_CPUENC_GET_ATMOS_LOCK (void);
extern UINT32 AUD_CPUENC_GET_MIX_TYPE (void);
extern UINT32 AUD_CPUENC_GET_PCM_TRUST (void);
extern UINT32 AUD_CPUENC_GET_PH90_SHIFT (void);
#endif //CC_AUD_DOLBY_SUPPORT_MS12v2
#ifdef CC_AUD_RISCIEC_SUPPORT
extern UINT32 AUD_CPUENC_GET_IEC_PD(void);
extern UINT32 AUD_CPUENC_GET_IEC_NSADR(void);
extern void AUD_CPUENC_SET_IECBUF (UINT32 u4Sel, UINT32 u4Ptr);
extern void _AUD_CPUIEC_UPDATE_RP(UINT32 u4Ptr, UINT32 u4BurstInfo);
extern void AUD_CPUENC_SET_IEC_INFO (UINT32 u4Addr, UINT32 u4Size);
extern void AUD_CPUENC_SET_IECBUF (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUDEC_SET_CPU_IEC_INFO (UINT8 u1DecId, UINT32 u4IecRawHead, UINT32 u4IecRawAdr, UINT32 u4IecRawPd, UINT32 u4IecFrameSample, UINT32 u4Fs, UINT32 u4AputBank);
#endif //CC_AUD_RISCIEC_SUPPORT
extern void AUD_CPUENC_SET_ENABLE(UINT32 u4Enable);
extern void AUD_CPUENC_SET_FLUSH_INVALIDATE(UINT32 u4Addr,UINT32 u4Size);
extern void AUD_CPUENC_SET_UPBUF (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUENC_SET_DOWNBUF (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUENC_SET_ASM(UINT32 u4Grp,UINT32 u4Id,UINT32 u4Value);
extern UINT32 AUD_CPUENC_GET_ASM(UINT32 u4Grp,UINT32 u4Id);
#endif // CC_AUD_CPUENC_SUPPORT
#ifdef CC_AUD_DOLBY_SUPPORT_MS12B
extern void AUD_CPUDEC_1Pid2Dec_Enable(BOOL fg1PidEnable);
#endif // CC_AUD_DOLBY_SUPPORT_MS12B
#ifdef CC_AUD_RISCAOUT_SUPPORT
extern void AUD_NotifyCpuPostDebugLevel(UINT32 u4DebugLevel);
extern void AUD_CPUPOST_SET_UPLOAD_WP (UINT32 u4Sel, UINT32 u4Ptr);
extern UINT32 AUD_CPUPOST_GET_APROC_BASE(void);
extern UINT32 AUD_CPUPOST_GET_WORK_BUF_ADDR (void);
extern UINT32 AUD_CPUPOST_GET_WORK_BUF_SIZE (void);
extern void AUD_CPUPOST_GET_CTRL_STATUS (UINT32 *pu4Status);
extern void AUD_CPUPOST_GET_CPUPOST_STATUS (UINT32 *pu4Status);
extern UINT32 AUD_CPUPOST_GET_WORK_BUF_ADDR (void);
extern UINT32 AUD_CPUPOST_GET_WORK_BUF_SIZE (void);
extern UINT32 AUD_CPUPOST_GET_UPBUF0 (UINT32 u4Sel);
extern UINT32 AUD_CPUPOST_GET_UPBUF1 (UINT32 u4Sel);
extern UINT32 AUD_CPUPOST_GET_UPBUF2 (UINT32 u4Sel);
extern UINT32 AUD_GetCpuPostUpbuf3(UINT32 u4Sel);
extern UINT32 AUD_CPUPOST_GET_OUTBUF (UINT32 u4Sel);
extern void AUD_CPUPOST_SET_CTRL_STATUS (UINT32 u4Status);
extern void AUD_CPUPOST_SET_AOUT_INIT (void);
extern void AUD_CPUPOST_SET_UPBUF0 (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUPOST_SET_UPBUF1 (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUPOST_SET_UPBUF2 (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_SetCpuPostUpbuf3(UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUPOST_SET_FLUSH_INVALIDATE (UINT32 u4Addr, UINT32 u4Size);
extern void AUD_CPUPOST_SET_OUTBUF (UINT32 u4Sel, UINT32 u4Ptr);
extern void AUD_CPUPOST_SET_ASM (UINT32 u4Grp, UINT32 u4Id, UINT32 u4Value);
extern UINT32 AUD_CPUPOST_GET_APROC_REGMAP_ADDR (void);
extern UINT32 AUD_CPUPOST_GET_DAPINFO_ADDR(void);
extern UINT32 AUD_CPUPOST_GET_DECCFG_ADDR(void);
extern UINT32 AUD_CPUPOST_GET_PCMRINFO_ADDR(void);
extern void AUD_CPUPOST_SET_MIXSND_RP(UINT8 u1MixsndId);
#endif
extern void AUD_SetAcVolumeOffset(INT8 i1VolumeOffset);
extern void AUD_PostProc32EQGain_Set(INT32 i4BandGain[32]);
extern void AUD_MixSndPrintHistory(void);
extern void AUD_DumpInformation(void);
extern void AUD_SetDamSupport(INT32 i4SetDamSupported);
#ifdef CC_AUD_IEC_PBUF_SUPPORT_V2
extern BOOL AUD_GetIecPbufEn(void);
extern void AUD_SetIecPbufEn(BOOL fgIecPbufEn);
#endif // CC_AUD_IEC_PBUF_SUPPORT
#endif /* _AUD_IF_H_ */

extern INT32 AUD_SetPTPID(UINT8 ptpid);
extern UINT8 AUD_GetPTPID(void);
extern void AUD_SetVadI2sSlaveModeEnable(BOOL fgEnable);

#ifdef AAUDIO_MMAP_MIXSND_SUPPORT
//Declare fuction
void AUD_EnableMixSndMMAPThread(void);
void AUD_DisableMixSndMMAPThread(void);
void AUD_StopMixSndMMAP(UINT8 u1StreamID);
INT32 AUD_PlayMixSndMMAP(UINT32 u4MixsndId, UINT32 u4BufferSize);
UPTR u4GetMixSndMMAPRingFifoRP(UINT32 u4MixsndId);
BOOL AUD_isNetflixPlay(void);
#endif

